

	.global	___lshrhi15
___lshrhi15:
	lsr	a

	.global	___lshrhi14
___lshrhi14:		
	lsr	a	
	.global	___lshrhi13
___lshrhi13:	
	lsr	a	
	.global	___lshrhi12
___lshrhi12:	
	lsr	a	
	.global	___lshrhi11
___lshrhi11:	
	lsr	a	
	.global	___lshrhi10
___lshrhi10:	
	lsr	a	
	.global	___lshrhi9
___lshrhi9:	
	lsr	a	
	.global	___lshrhi8
___lshrhi8:	
	lsr	a	
	.global	___lshrhi7
___lshrhi7:	
	lsr	a	
	.global	___lshrhi6
___lshrhi6:	
	lsr	a	
	.global	___lshrhi5
___lshrhi5:	
	lsr	a	
	.global	___lshrhi4
___lshrhi4:	
	lsr	a	
	.global	___lshrhi3
___lshrhi3:	
	lsr	a	
	.global	___lshrhi2
___lshrhi2:
	lsr	a
	lsr	a
	rtl

