#ifndef _SYS_SYSCALL_H_
#define _SYS_SYSCALL_H_

/* Note: This file may be included by assembler source.  */

#define SYS_exit        1
#define SYS_fork        2

#define SYS_read        3
#define SYS_write       4
#define SYS_open        5
#define SYS_close       6
#define SYS_wait4       7
#define SYS_creat       8
#define SYS_link        9
#define SYS_unlink      10
#define SYS_execv       11
#define SYS_chdir       12
#define SYS_mknod       14
#define SYS_chmod       15
#define SYS_chown       16
#define SYS_lseek       19
#define SYS_getpid      20
#define SYS_isatty      21
#define SYS_fstat       22
#define SYS_time        23


#define SYS_ARG         24
#define SYS_stat        38


#define SYS_pipe        42
#define SYS_execve      59
#define   SYS_times       43
#define SYS_gettimeofday 116
#define SYS_rename	134

#define SYS_utime       201 /* not really a system call */
#define SYS_wait        202 /* nor is this */

#endif
