/*
 * This file was automatically generated mkdeps.pl script. Don't edit.
 */

#ifndef __CCSNAMES_H__
#define __CCSNAMES_H__

#include "../lib/encnames.h"

/*
 * CCS tables names macros.
 */
#define ICONV_CCS_BIG5 ICONV_ENCODING_BIG5
#define ICONV_CCS_CNS11643_PLANE1 "cns11643_plane1"
#define ICONV_CCS_CNS11643_PLANE14 "cns11643_plane14"
#define ICONV_CCS_CNS11643_PLANE2 "cns11643_plane2"
#define ICONV_CCS_CP775 ICONV_ENCODING_CP775
#define ICONV_CCS_CP850 ICONV_ENCODING_CP850
#define ICONV_CCS_CP852 ICONV_ENCODING_CP852
#define ICONV_CCS_CP855 ICONV_ENCODING_CP855
#define ICONV_CCS_CP866 ICONV_ENCODING_CP866
#define ICONV_CCS_ISO_8859_1 ICONV_ENCODING_ISO_8859_1
#define ICONV_CCS_ISO_8859_10 ICONV_ENCODING_ISO_8859_10
#define ICONV_CCS_ISO_8859_11 ICONV_ENCODING_ISO_8859_11
#define ICONV_CCS_ISO_8859_13 ICONV_ENCODING_ISO_8859_13
#define ICONV_CCS_ISO_8859_14 ICONV_ENCODING_ISO_8859_14
#define ICONV_CCS_ISO_8859_15 ICONV_ENCODING_ISO_8859_15
#define ICONV_CCS_ISO_8859_2 ICONV_ENCODING_ISO_8859_2
#define ICONV_CCS_ISO_8859_3 ICONV_ENCODING_ISO_8859_3
#define ICONV_CCS_ISO_8859_4 ICONV_ENCODING_ISO_8859_4
#define ICONV_CCS_ISO_8859_5 ICONV_ENCODING_ISO_8859_5
#define ICONV_CCS_ISO_8859_6 ICONV_ENCODING_ISO_8859_6
#define ICONV_CCS_ISO_8859_7 ICONV_ENCODING_ISO_8859_7
#define ICONV_CCS_ISO_8859_8 ICONV_ENCODING_ISO_8859_8
#define ICONV_CCS_ISO_8859_9 ICONV_ENCODING_ISO_8859_9
#define ICONV_CCS_ISO_IR_111 ICONV_ENCODING_ISO_IR_111
#define ICONV_CCS_JIS_X0201_1976 "jis_x0201_1976"
#define ICONV_CCS_JIS_X0208_1990 "jis_x0208_1990"
#define ICONV_CCS_JIS_X0212_1990 "jis_x0212_1990"
#define ICONV_CCS_KOI8_R ICONV_ENCODING_KOI8_R
#define ICONV_CCS_KOI8_RU ICONV_ENCODING_KOI8_RU
#define ICONV_CCS_KOI8_U ICONV_ENCODING_KOI8_U
#define ICONV_CCS_KOI8_UNI ICONV_ENCODING_KOI8_UNI
#define ICONV_CCS_KSX1001 "ksx1001"
#define ICONV_CCS_WIN_1250 ICONV_ENCODING_WIN_1250
#define ICONV_CCS_WIN_1251 ICONV_ENCODING_WIN_1251
#define ICONV_CCS_WIN_1252 ICONV_ENCODING_WIN_1252
#define ICONV_CCS_WIN_1253 ICONV_ENCODING_WIN_1253
#define ICONV_CCS_WIN_1254 ICONV_ENCODING_WIN_1254
#define ICONV_CCS_WIN_1255 ICONV_ENCODING_WIN_1255
#define ICONV_CCS_WIN_1256 ICONV_ENCODING_WIN_1256
#define ICONV_CCS_WIN_1257 ICONV_ENCODING_WIN_1257
#define ICONV_CCS_WIN_1258 ICONV_ENCODING_WIN_1258

#endif /* !__CCSNAMES_H__ */

