/*
 * This file was automatically generated mkdeps.pl script. Don't edit.
 */

#ifndef __ENCNAMES_H__
#define __ENCNAMES_H__

/*
 * Encodings name macros.
 */
#define ICONV_ENCODING_BIG5 "big5"
#define ICONV_ENCODING_CP775 "cp775"
#define ICONV_ENCODING_CP850 "cp850"
#define ICONV_ENCODING_CP852 "cp852"
#define ICONV_ENCODING_CP855 "cp855"
#define ICONV_ENCODING_CP866 "cp866"
#define ICONV_ENCODING_EUC_JP "euc_jp"
#define ICONV_ENCODING_EUC_KR "euc_kr"
#define ICONV_ENCODING_EUC_TW "euc_tw"
#define ICONV_ENCODING_ISO_8859_1 "iso_8859_1"
#define ICONV_ENCODING_ISO_8859_10 "iso_8859_10"
#define ICONV_ENCODING_ISO_8859_11 "iso_8859_11"
#define ICONV_ENCODING_ISO_8859_13 "iso_8859_13"
#define ICONV_ENCODING_ISO_8859_14 "iso_8859_14"
#define ICONV_ENCODING_ISO_8859_15 "iso_8859_15"
#define ICONV_ENCODING_ISO_8859_2 "iso_8859_2"
#define ICONV_ENCODING_ISO_8859_3 "iso_8859_3"
#define ICONV_ENCODING_ISO_8859_4 "iso_8859_4"
#define ICONV_ENCODING_ISO_8859_5 "iso_8859_5"
#define ICONV_ENCODING_ISO_8859_6 "iso_8859_6"
#define ICONV_ENCODING_ISO_8859_7 "iso_8859_7"
#define ICONV_ENCODING_ISO_8859_8 "iso_8859_8"
#define ICONV_ENCODING_ISO_8859_9 "iso_8859_9"
#define ICONV_ENCODING_ISO_IR_111 "iso_ir_111"
#define ICONV_ENCODING_KOI8_R "koi8_r"
#define ICONV_ENCODING_KOI8_RU "koi8_ru"
#define ICONV_ENCODING_KOI8_U "koi8_u"
#define ICONV_ENCODING_KOI8_UNI "koi8_uni"
#define ICONV_ENCODING_UCS_2 "ucs_2"
#define ICONV_ENCODING_UCS_2_INTERNAL "ucs_2_internal"
#define ICONV_ENCODING_UCS_2BE "ucs_2be"
#define ICONV_ENCODING_UCS_2LE "ucs_2le"
#define ICONV_ENCODING_UCS_4 "ucs_4"
#define ICONV_ENCODING_UCS_4_INTERNAL "ucs_4_internal"
#define ICONV_ENCODING_UCS_4BE "ucs_4be"
#define ICONV_ENCODING_UCS_4LE "ucs_4le"
#define ICONV_ENCODING_US_ASCII "us_ascii"
#define ICONV_ENCODING_UTF_16 "utf_16"
#define ICONV_ENCODING_UTF_16BE "utf_16be"
#define ICONV_ENCODING_UTF_16LE "utf_16le"
#define ICONV_ENCODING_UTF_8 "utf_8"
#define ICONV_ENCODING_WIN_1250 "win_1250"
#define ICONV_ENCODING_WIN_1251 "win_1251"
#define ICONV_ENCODING_WIN_1252 "win_1252"
#define ICONV_ENCODING_WIN_1253 "win_1253"
#define ICONV_ENCODING_WIN_1254 "win_1254"
#define ICONV_ENCODING_WIN_1255 "win_1255"
#define ICONV_ENCODING_WIN_1256 "win_1256"
#define ICONV_ENCODING_WIN_1257 "win_1257"
#define ICONV_ENCODING_WIN_1258 "win_1258"

#endif /* !__ENCNAMES_H__ */

