/*
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

/*
** This include file contains a list of macro "defines" to enable programs
** to use symbolic names for register-access and bit-manipulation for the
** ADSP-BF504 processor.
**
** Copyright (C) 2009 Analog Devices Inc., All Rights Reserved.
*/

#ifndef _DEF_BF504_H
#define _DEF_BF504_H

/* Include all Core registers and bit definitions */
#include <def_LPBlackfin.h>

/* Include defBF50x_base.h for the set of #defines that are
** common to all ADSP-BF50x processors
*/
#include <defBF50x_base.h>

/* Define the set of macros that are specific to the ADSP-BF504 processor */

#endif /* _DEF_BF504_H */
