/*
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

/*
** This include file contains a list of macro "defines" to enable the programmer
** to use symbolic names for the ADSP-BF592-A peripherals.
**
** Copyright (C) 2009 Analog Devices Inc., All Rights Reserved.
*/

#ifndef _CDEF_BF592A_H
#define _CDEF_BF592A_H

/* include parts registers and bit definitions */
#include <defBF592-A.h>

/* include the core specific definitions */
#include <cdef_LPBlackfin.h>

/* include the family common definitions */
#include <cdefBF59x_base.h>

#endif /* _CDEF_BF592A_H */
