/*
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

/*
** cdefBF549M.h
**
** Copyright (C) 2008-2009 Analog Devices Inc., All Rights Reserved.
**
************************************************************************************
**
** This file just maps to the cdef for BF549 unless anything
** is required to change.
**
************************************************************************************/

#ifndef _CDEF_BF549M_H
#define _CDEF_BF549M_H

#include <cdefBF549.h>

#endif /* _CDEF_BF549M_H */
