/*
 * stpcpy - copy a string returning pointer to end.
 *
 * Copyright (c) 2020, Arm Limited.
 * SPDX-License-Identifier: MIT OR Apache-2.0 WITH LLVM-exception
 */

#define BUILD_STPCPY 1

#include "strcpy.S"
