/*
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 */

/*
** Copyright (C) 2005 Analog Devices Inc., All Rights Reserved.
**
************************************************************************************
**
** This include file contains a list of macro "defines" to enable the programmer
** to use symbolic names for register-access.
**
**/
/**********************************************************************************
** System MMR Register Map
***********************************************************************************/

#ifndef _CDEF_BF536_H
#define _CDEF_BF536_H

/* MMR Space Identical to BF537 Processor 								*/
#include <cdefBF537.h>

#endif /* _CDEF_BF536_H */

