/*
 * This file was generated automatically - don't edit it.
 * File contains iconv CCS tables for ksx1001 encoding.
 */

#include "ccsbi.h"

#if defined (ICONV_TO_UCS_CCS_KSX1001) \
 || defined (ICONV_FROM_UCS_CCS_KSX1001)

#include <_ansi.h>
#include <sys/types.h>
#include <sys/param.h>
#include "ccs.h"
#include "ccsnames.h"

/*
 * 16-bit ksx1001 -> UCS speed-optimized table (46080 bytes).
 * ======================================================================
 */
#if defined (ICONV_TO_UCS_CCS_KSX1001) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
to_ucs_speed_ksx1001[] =
{
	/* Heading Block */
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,0x0100,0x0200,0x0300,0x0400,0x0500,0x0600,0x0700,
	0x0800,0x0900,0x0A00,0x0B00,0x0C00,INVBLK,INVBLK,INVBLK,
	0x0D00,0x0E00,0x0F00,0x1000,0x1100,0x1200,0x1300,0x1400,
	0x1500,0x1600,0x1700,0x1800,0x1900,0x1A00,0x1B00,0x1C00,
	0x1D00,0x1E00,0x1F00,0x2000,0x2100,0x2200,0x2300,0x2400,
	0x2500,INVBLK,0x2600,0x2700,0x2800,0x2900,0x2A00,0x2B00,
	0x2C00,0x2D00,0x2E00,0x2F00,0x3000,0x3100,0x3200,0x3300,
	0x3400,0x3500,0x3600,0x3700,0x3800,0x3900,0x3A00,0x3B00,
	0x3C00,0x3D00,0x3E00,0x3F00,0x4000,0x4100,0x4200,0x4300,
	0x4400,0x4500,0x4600,0x4700,0x4800,0x4900,0x4A00,0x4B00,
	0x4C00,0x4D00,0x4E00,0x4F00,0x5000,0x5100,0x5200,0x5300,
	0x5400,0x5500,0x5600,0x5700,0x5800,0x5900,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	/* Block 34, Array index 0x0100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3000,0x3001,0x3002,0x00B7,0x2025,0x2026,0x00A8,
	0x3003,0x00AD,0x2015,0x2225,0xFF3C,0x223C,0x2018,0x2019,
	0x201C,0x201D,0x3014,0x3015,0x3008,0x3009,0x300A,0x300B,
	0x300C,0x300D,0x300E,0x300F,0x3010,0x3011,0x00B1,0x00D7,
	0x00F7,0x2260,0x2264,0x2265,0x221E,0x2234,0x00B0,0x2032,
	0x2033,0x2103,0x212B,0xFFE0,0xFFE1,0xFFE5,0x2642,0x2640,
	0x2220,0x22A5,0x2312,0x2202,0x2207,0x2261,0x2252,0x00A7,
	0x203B,0x2606,0x2605,0x25CB,0x25CF,0x25CE,0x25C7,0x25C6,
	0x25A1,0x25A0,0x25B3,0x25B2,0x25BD,0x25BC,0x2192,0x2190,
	0x2191,0x2193,0x2194,0x3013,0x226A,0x226B,0x221A,0x223D,
	0x221D,0x2235,0x222B,0x222C,0x2208,0x220B,0x2286,0x2287,
	0x2282,0x2283,0x222A,0x2229,0x2227,0x2228,0xFFE2,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 35, Array index 0x0200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x21D2,0x21D4,0x2200,0x2203,0x00B4,0xFF5E,0x02C7,
	0x02D8,0x02DD,0x02DA,0x02D9,0x00B8,0x02DB,0x00A1,0x00BF,
	0x02D0,0x222E,0x2211,0x220F,0x00A4,0x2109,0x2030,0x25C1,
	0x25C0,0x25B7,0x25B6,0x2664,0x2660,0x2661,0x2665,0x2667,
	0x2663,0x2299,0x25C8,0x25A3,0x25D0,0x25D1,0x2592,0x25A4,
	0x25A5,0x25A8,0x25A7,0x25A6,0x25A9,0x2668,0x260F,0x260E,
	0x261C,0x261E,0x00B6,0x2020,0x2021,0x2195,0x2197,0x2199,
	0x2196,0x2198,0x266D,0x2669,0x266A,0x266C,0x327F,0x321C,
	0x2116,0x33C7,0x2122,0x33C2,0x33D8,0x2121,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 36, Array index 0x0300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xFF01,0xFF02,0xFF03,0xFF04,0xFF05,0xFF06,0xFF07,
	0xFF08,0xFF09,0xFF0A,0xFF0B,0xFF0C,0xFF0D,0xFF0E,0xFF0F,
	0xFF10,0xFF11,0xFF12,0xFF13,0xFF14,0xFF15,0xFF16,0xFF17,
	0xFF18,0xFF19,0xFF1A,0xFF1B,0xFF1C,0xFF1D,0xFF1E,0xFF1F,
	0xFF20,0xFF21,0xFF22,0xFF23,0xFF24,0xFF25,0xFF26,0xFF27,
	0xFF28,0xFF29,0xFF2A,0xFF2B,0xFF2C,0xFF2D,0xFF2E,0xFF2F,
	0xFF30,0xFF31,0xFF32,0xFF33,0xFF34,0xFF35,0xFF36,0xFF37,
	0xFF38,0xFF39,0xFF3A,0xFF3B,0xFFE6,0xFF3D,0xFF3E,0xFF3F,
	0xFF40,0xFF41,0xFF42,0xFF43,0xFF44,0xFF45,0xFF46,0xFF47,
	0xFF48,0xFF49,0xFF4A,0xFF4B,0xFF4C,0xFF4D,0xFF4E,0xFF4F,
	0xFF50,0xFF51,0xFF52,0xFF53,0xFF54,0xFF55,0xFF56,0xFF57,
	0xFF58,0xFF59,0xFF5A,0xFF5B,0xFF5C,0xFF5D,0xFFE3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 37, Array index 0x0400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3131,0x3132,0x3133,0x3134,0x3135,0x3136,0x3137,
	0x3138,0x3139,0x313A,0x313B,0x313C,0x313D,0x313E,0x313F,
	0x3140,0x3141,0x3142,0x3143,0x3144,0x3145,0x3146,0x3147,
	0x3148,0x3149,0x314A,0x314B,0x314C,0x314D,0x314E,0x314F,
	0x3150,0x3151,0x3152,0x3153,0x3154,0x3155,0x3156,0x3157,
	0x3158,0x3159,0x315A,0x315B,0x315C,0x315D,0x315E,0x315F,
	0x3160,0x3161,0x3162,0x3163,0x3164,0x3165,0x3166,0x3167,
	0x3168,0x3169,0x316A,0x316B,0x316C,0x316D,0x316E,0x316F,
	0x3170,0x3171,0x3172,0x3173,0x3174,0x3175,0x3176,0x3177,
	0x3178,0x3179,0x317A,0x317B,0x317C,0x317D,0x317E,0x317F,
	0x3180,0x3181,0x3182,0x3183,0x3184,0x3185,0x3186,0x3187,
	0x3188,0x3189,0x318A,0x318B,0x318C,0x318D,0x318E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 38, Array index 0x0500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2170,0x2171,0x2172,0x2173,0x2174,0x2175,0x2176,
	0x2177,0x2178,0x2179,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2160,0x2161,0x2162,0x2163,0x2164,0x2165,0x2166,0x2167,
	0x2168,0x2169,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x0391,0x0392,0x0393,0x0394,0x0395,0x0396,0x0397,
	0x0398,0x0399,0x039A,0x039B,0x039C,0x039D,0x039E,0x039F,
	0x03A0,0x03A1,0x03A3,0x03A4,0x03A5,0x03A6,0x03A7,0x03A8,
	0x03A9,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x03B1,0x03B2,0x03B3,0x03B4,0x03B5,0x03B6,0x03B7,
	0x03B8,0x03B9,0x03BA,0x03BB,0x03BC,0x03BD,0x03BE,0x03BF,
	0x03C0,0x03C1,0x03C3,0x03C4,0x03C5,0x03C6,0x03C7,0x03C8,
	0x03C9,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 39, Array index 0x0600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2500,0x2502,0x250C,0x2510,0x2518,0x2514,0x251C,
	0x252C,0x2524,0x2534,0x253C,0x2501,0x2503,0x250F,0x2513,
	0x251B,0x2517,0x2523,0x2533,0x252B,0x253B,0x254B,0x2520,
	0x252F,0x2528,0x2537,0x253F,0x251D,0x2530,0x2525,0x2538,
	0x2542,0x2512,0x2511,0x251A,0x2519,0x2516,0x2515,0x250E,
	0x250D,0x251E,0x251F,0x2521,0x2522,0x2526,0x2527,0x2529,
	0x252A,0x252D,0x252E,0x2531,0x2532,0x2535,0x2536,0x2539,
	0x253A,0x253D,0x253E,0x2540,0x2541,0x2543,0x2544,0x2545,
	0x2546,0x2547,0x2548,0x2549,0x254A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 40, Array index 0x0700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3395,0x3396,0x3397,0x2113,0x3398,0x33C4,0x33A3,
	0x33A4,0x33A5,0x33A6,0x3399,0x339A,0x339B,0x339C,0x339D,
	0x339E,0x339F,0x33A0,0x33A1,0x33A2,0x33CA,0x338D,0x338E,
	0x338F,0x33CF,0x3388,0x3389,0x33C8,0x33A7,0x33A8,0x33B0,
	0x33B1,0x33B2,0x33B3,0x33B4,0x33B5,0x33B6,0x33B7,0x33B8,
	0x33B9,0x3380,0x3381,0x3382,0x3383,0x3384,0x33BA,0x33BB,
	0x33BC,0x33BD,0x33BE,0x33BF,0x3390,0x3391,0x3392,0x3393,
	0x3394,0x2126,0x33C0,0x33C1,0x338A,0x338B,0x338C,0x33D6,
	0x33C5,0x33AD,0x33AE,0x33AF,0x33DB,0x33A9,0x33AA,0x33AB,
	0x33AC,0x33DD,0x33D0,0x33D3,0x33C3,0x33C9,0x33DC,0x33C6,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 41, Array index 0x0800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x00C6,0x00D0,0x00AA,0x0126,INVALC,0x0132,INVALC,
	0x013F,0x0141,0x00D8,0x0152,0x00BA,0x00DE,0x0166,0x014A,
	INVALC,0x3260,0x3261,0x3262,0x3263,0x3264,0x3265,0x3266,
	0x3267,0x3268,0x3269,0x326A,0x326B,0x326C,0x326D,0x326E,
	0x326F,0x3270,0x3271,0x3272,0x3273,0x3274,0x3275,0x3276,
	0x3277,0x3278,0x3279,0x327A,0x327B,0x24D0,0x24D1,0x24D2,
	0x24D3,0x24D4,0x24D5,0x24D6,0x24D7,0x24D8,0x24D9,0x24DA,
	0x24DB,0x24DC,0x24DD,0x24DE,0x24DF,0x24E0,0x24E1,0x24E2,
	0x24E3,0x24E4,0x24E5,0x24E6,0x24E7,0x24E8,0x24E9,0x2460,
	0x2461,0x2462,0x2463,0x2464,0x2465,0x2466,0x2467,0x2468,
	0x2469,0x246A,0x246B,0x246C,0x246D,0x246E,0x00BD,0x2153,
	0x2154,0x00BC,0x00BE,0x215B,0x215C,0x215D,0x215E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 42, Array index 0x0900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x00E6,0x0111,0x00F0,0x0127,0x0131,0x0133,0x0138,
	0x0140,0x0142,0x00F8,0x0153,0x00DF,0x00FE,0x0167,0x014B,
	0x0149,0x3200,0x3201,0x3202,0x3203,0x3204,0x3205,0x3206,
	0x3207,0x3208,0x3209,0x320A,0x320B,0x320C,0x320D,0x320E,
	0x320F,0x3210,0x3211,0x3212,0x3213,0x3214,0x3215,0x3216,
	0x3217,0x3218,0x3219,0x321A,0x321B,0x249C,0x249D,0x249E,
	0x249F,0x24A0,0x24A1,0x24A2,0x24A3,0x24A4,0x24A5,0x24A6,
	0x24A7,0x24A8,0x24A9,0x24AA,0x24AB,0x24AC,0x24AD,0x24AE,
	0x24AF,0x24B0,0x24B1,0x24B2,0x24B3,0x24B4,0x24B5,0x2474,
	0x2475,0x2476,0x2477,0x2478,0x2479,0x247A,0x247B,0x247C,
	0x247D,0x247E,0x247F,0x2480,0x2481,0x2482,0x00B9,0x00B2,
	0x00B3,0x2074,0x207F,0x2081,0x2082,0x2083,0x2084,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 43, Array index 0x0A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3041,0x3042,0x3043,0x3044,0x3045,0x3046,0x3047,
	0x3048,0x3049,0x304A,0x304B,0x304C,0x304D,0x304E,0x304F,
	0x3050,0x3051,0x3052,0x3053,0x3054,0x3055,0x3056,0x3057,
	0x3058,0x3059,0x305A,0x305B,0x305C,0x305D,0x305E,0x305F,
	0x3060,0x3061,0x3062,0x3063,0x3064,0x3065,0x3066,0x3067,
	0x3068,0x3069,0x306A,0x306B,0x306C,0x306D,0x306E,0x306F,
	0x3070,0x3071,0x3072,0x3073,0x3074,0x3075,0x3076,0x3077,
	0x3078,0x3079,0x307A,0x307B,0x307C,0x307D,0x307E,0x307F,
	0x3080,0x3081,0x3082,0x3083,0x3084,0x3085,0x3086,0x3087,
	0x3088,0x3089,0x308A,0x308B,0x308C,0x308D,0x308E,0x308F,
	0x3090,0x3091,0x3092,0x3093,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 44, Array index 0x0B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x30A1,0x30A2,0x30A3,0x30A4,0x30A5,0x30A6,0x30A7,
	0x30A8,0x30A9,0x30AA,0x30AB,0x30AC,0x30AD,0x30AE,0x30AF,
	0x30B0,0x30B1,0x30B2,0x30B3,0x30B4,0x30B5,0x30B6,0x30B7,
	0x30B8,0x30B9,0x30BA,0x30BB,0x30BC,0x30BD,0x30BE,0x30BF,
	0x30C0,0x30C1,0x30C2,0x30C3,0x30C4,0x30C5,0x30C6,0x30C7,
	0x30C8,0x30C9,0x30CA,0x30CB,0x30CC,0x30CD,0x30CE,0x30CF,
	0x30D0,0x30D1,0x30D2,0x30D3,0x30D4,0x30D5,0x30D6,0x30D7,
	0x30D8,0x30D9,0x30DA,0x30DB,0x30DC,0x30DD,0x30DE,0x30DF,
	0x30E0,0x30E1,0x30E2,0x30E3,0x30E4,0x30E5,0x30E6,0x30E7,
	0x30E8,0x30E9,0x30EA,0x30EB,0x30EC,0x30ED,0x30EE,0x30EF,
	0x30F0,0x30F1,0x30F2,0x30F3,0x30F4,0x30F5,0x30F6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 45, Array index 0x0C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x0410,0x0411,0x0412,0x0413,0x0414,0x0415,0x0401,
	0x0416,0x0417,0x0418,0x0419,0x041A,0x041B,0x041C,0x041D,
	0x041E,0x041F,0x0420,0x0421,0x0422,0x0423,0x0424,0x0425,
	0x0426,0x0427,0x0428,0x0429,0x042A,0x042B,0x042C,0x042D,
	0x042E,0x042F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x0430,0x0431,0x0432,0x0433,0x0434,0x0435,0x0451,
	0x0436,0x0437,0x0438,0x0439,0x043A,0x043B,0x043C,0x043D,
	0x043E,0x043F,0x0440,0x0441,0x0442,0x0443,0x0444,0x0445,
	0x0446,0x0447,0x0448,0x0449,0x044A,0x044B,0x044C,0x044D,
	0x044E,0x044F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 49, Array index 0x0D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xAC00,0xAC01,0xAC04,0xAC07,0xAC08,0xAC09,0xAC0A,
	0xAC10,0xAC11,0xAC12,0xAC13,0xAC14,0xAC15,0xAC16,0xAC17,
	0xAC19,0xAC1A,0xAC1B,0xAC1C,0xAC1D,0xAC20,0xAC24,0xAC2C,
	0xAC2D,0xAC2F,0xAC30,0xAC31,0xAC38,0xAC39,0xAC3C,0xAC40,
	0xAC4B,0xAC4D,0xAC54,0xAC58,0xAC5C,0xAC70,0xAC71,0xAC74,
	0xAC77,0xAC78,0xAC7A,0xAC80,0xAC81,0xAC83,0xAC84,0xAC85,
	0xAC86,0xAC89,0xAC8A,0xAC8B,0xAC8C,0xAC90,0xAC94,0xAC9C,
	0xAC9D,0xAC9F,0xACA0,0xACA1,0xACA8,0xACA9,0xACAA,0xACAC,
	0xACAF,0xACB0,0xACB8,0xACB9,0xACBB,0xACBC,0xACBD,0xACC1,
	0xACC4,0xACC8,0xACCC,0xACD5,0xACD7,0xACE0,0xACE1,0xACE4,
	0xACE7,0xACE8,0xACEA,0xACEC,0xACEF,0xACF0,0xACF1,0xACF3,
	0xACF5,0xACF6,0xACFC,0xACFD,0xAD00,0xAD04,0xAD06,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 50, Array index 0x0E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xAD0C,0xAD0D,0xAD0F,0xAD11,0xAD18,0xAD1C,0xAD20,
	0xAD29,0xAD2C,0xAD2D,0xAD34,0xAD35,0xAD38,0xAD3C,0xAD44,
	0xAD45,0xAD47,0xAD49,0xAD50,0xAD54,0xAD58,0xAD61,0xAD63,
	0xAD6C,0xAD6D,0xAD70,0xAD73,0xAD74,0xAD75,0xAD76,0xAD7B,
	0xAD7C,0xAD7D,0xAD7F,0xAD81,0xAD82,0xAD88,0xAD89,0xAD8C,
	0xAD90,0xAD9C,0xAD9D,0xADA4,0xADB7,0xADC0,0xADC1,0xADC4,
	0xADC8,0xADD0,0xADD1,0xADD3,0xADDC,0xADE0,0xADE4,0xADF8,
	0xADF9,0xADFC,0xADFF,0xAE00,0xAE01,0xAE08,0xAE09,0xAE0B,
	0xAE0D,0xAE14,0xAE30,0xAE31,0xAE34,0xAE37,0xAE38,0xAE3A,
	0xAE40,0xAE41,0xAE43,0xAE45,0xAE46,0xAE4A,0xAE4C,0xAE4D,
	0xAE4E,0xAE50,0xAE54,0xAE56,0xAE5C,0xAE5D,0xAE5F,0xAE60,
	0xAE61,0xAE65,0xAE68,0xAE69,0xAE6C,0xAE70,0xAE78,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 51, Array index 0x0F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xAE79,0xAE7B,0xAE7C,0xAE7D,0xAE84,0xAE85,0xAE8C,
	0xAEBC,0xAEBD,0xAEBE,0xAEC0,0xAEC4,0xAECC,0xAECD,0xAECF,
	0xAED0,0xAED1,0xAED8,0xAED9,0xAEDC,0xAEE8,0xAEEB,0xAEED,
	0xAEF4,0xAEF8,0xAEFC,0xAF07,0xAF08,0xAF0D,0xAF10,0xAF2C,
	0xAF2D,0xAF30,0xAF32,0xAF34,0xAF3C,0xAF3D,0xAF3F,0xAF41,
	0xAF42,0xAF43,0xAF48,0xAF49,0xAF50,0xAF5C,0xAF5D,0xAF64,
	0xAF65,0xAF79,0xAF80,0xAF84,0xAF88,0xAF90,0xAF91,0xAF95,
	0xAF9C,0xAFB8,0xAFB9,0xAFBC,0xAFC0,0xAFC7,0xAFC8,0xAFC9,
	0xAFCB,0xAFCD,0xAFCE,0xAFD4,0xAFDC,0xAFE8,0xAFE9,0xAFF0,
	0xAFF1,0xAFF4,0xAFF8,0xB000,0xB001,0xB004,0xB00C,0xB010,
	0xB014,0xB01C,0xB01D,0xB028,0xB044,0xB045,0xB048,0xB04A,
	0xB04C,0xB04E,0xB053,0xB054,0xB055,0xB057,0xB059,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 52, Array index 0x1000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xB05D,0xB07C,0xB07D,0xB080,0xB084,0xB08C,0xB08D,
	0xB08F,0xB091,0xB098,0xB099,0xB09A,0xB09C,0xB09F,0xB0A0,
	0xB0A1,0xB0A2,0xB0A8,0xB0A9,0xB0AB,0xB0AC,0xB0AD,0xB0AE,
	0xB0AF,0xB0B1,0xB0B3,0xB0B4,0xB0B5,0xB0B8,0xB0BC,0xB0C4,
	0xB0C5,0xB0C7,0xB0C8,0xB0C9,0xB0D0,0xB0D1,0xB0D4,0xB0D8,
	0xB0E0,0xB0E5,0xB108,0xB109,0xB10B,0xB10C,0xB110,0xB112,
	0xB113,0xB118,0xB119,0xB11B,0xB11C,0xB11D,0xB123,0xB124,
	0xB125,0xB128,0xB12C,0xB134,0xB135,0xB137,0xB138,0xB139,
	0xB140,0xB141,0xB144,0xB148,0xB150,0xB151,0xB154,0xB155,
	0xB158,0xB15C,0xB160,0xB178,0xB179,0xB17C,0xB180,0xB182,
	0xB188,0xB189,0xB18B,0xB18D,0xB192,0xB193,0xB194,0xB198,
	0xB19C,0xB1A8,0xB1CC,0xB1D0,0xB1D4,0xB1DC,0xB1DD,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 53, Array index 0x1100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xB1DF,0xB1E8,0xB1E9,0xB1EC,0xB1F0,0xB1F9,0xB1FB,
	0xB1FD,0xB204,0xB205,0xB208,0xB20B,0xB20C,0xB214,0xB215,
	0xB217,0xB219,0xB220,0xB234,0xB23C,0xB258,0xB25C,0xB260,
	0xB268,0xB269,0xB274,0xB275,0xB27C,0xB284,0xB285,0xB289,
	0xB290,0xB291,0xB294,0xB298,0xB299,0xB29A,0xB2A0,0xB2A1,
	0xB2A3,0xB2A5,0xB2A6,0xB2AA,0xB2AC,0xB2B0,0xB2B4,0xB2C8,
	0xB2C9,0xB2CC,0xB2D0,0xB2D2,0xB2D8,0xB2D9,0xB2DB,0xB2DD,
	0xB2E2,0xB2E4,0xB2E5,0xB2E6,0xB2E8,0xB2EB,0xB2EC,0xB2ED,
	0xB2EE,0xB2EF,0xB2F3,0xB2F4,0xB2F5,0xB2F7,0xB2F8,0xB2F9,
	0xB2FA,0xB2FB,0xB2FF,0xB300,0xB301,0xB304,0xB308,0xB310,
	0xB311,0xB313,0xB314,0xB315,0xB31C,0xB354,0xB355,0xB356,
	0xB358,0xB35B,0xB35C,0xB35E,0xB35F,0xB364,0xB365,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 54, Array index 0x1200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xB367,0xB369,0xB36B,0xB36E,0xB370,0xB371,0xB374,
	0xB378,0xB380,0xB381,0xB383,0xB384,0xB385,0xB38C,0xB390,
	0xB394,0xB3A0,0xB3A1,0xB3A8,0xB3AC,0xB3C4,0xB3C5,0xB3C8,
	0xB3CB,0xB3CC,0xB3CE,0xB3D0,0xB3D4,0xB3D5,0xB3D7,0xB3D9,
	0xB3DB,0xB3DD,0xB3E0,0xB3E4,0xB3E8,0xB3FC,0xB410,0xB418,
	0xB41C,0xB420,0xB428,0xB429,0xB42B,0xB434,0xB450,0xB451,
	0xB454,0xB458,0xB460,0xB461,0xB463,0xB465,0xB46C,0xB480,
	0xB488,0xB49D,0xB4A4,0xB4A8,0xB4AC,0xB4B5,0xB4B7,0xB4B9,
	0xB4C0,0xB4C4,0xB4C8,0xB4D0,0xB4D5,0xB4DC,0xB4DD,0xB4E0,
	0xB4E3,0xB4E4,0xB4E6,0xB4EC,0xB4ED,0xB4EF,0xB4F1,0xB4F8,
	0xB514,0xB515,0xB518,0xB51B,0xB51C,0xB524,0xB525,0xB527,
	0xB528,0xB529,0xB52A,0xB530,0xB531,0xB534,0xB538,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 55, Array index 0x1300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xB540,0xB541,0xB543,0xB544,0xB545,0xB54B,0xB54C,
	0xB54D,0xB550,0xB554,0xB55C,0xB55D,0xB55F,0xB560,0xB561,
	0xB5A0,0xB5A1,0xB5A4,0xB5A8,0xB5AA,0xB5AB,0xB5B0,0xB5B1,
	0xB5B3,0xB5B4,0xB5B5,0xB5BB,0xB5BC,0xB5BD,0xB5C0,0xB5C4,
	0xB5CC,0xB5CD,0xB5CF,0xB5D0,0xB5D1,0xB5D8,0xB5EC,0xB610,
	0xB611,0xB614,0xB618,0xB625,0xB62C,0xB634,0xB648,0xB664,
	0xB668,0xB69C,0xB69D,0xB6A0,0xB6A4,0xB6AB,0xB6AC,0xB6B1,
	0xB6D4,0xB6F0,0xB6F4,0xB6F8,0xB700,0xB701,0xB705,0xB728,
	0xB729,0xB72C,0xB72F,0xB730,0xB738,0xB739,0xB73B,0xB744,
	0xB748,0xB74C,0xB754,0xB755,0xB760,0xB764,0xB768,0xB770,
	0xB771,0xB773,0xB775,0xB77C,0xB77D,0xB780,0xB784,0xB78C,
	0xB78D,0xB78F,0xB790,0xB791,0xB792,0xB796,0xB797,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 56, Array index 0x1400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xB798,0xB799,0xB79C,0xB7A0,0xB7A8,0xB7A9,0xB7AB,
	0xB7AC,0xB7AD,0xB7B4,0xB7B5,0xB7B8,0xB7C7,0xB7C9,0xB7EC,
	0xB7ED,0xB7F0,0xB7F4,0xB7FC,0xB7FD,0xB7FF,0xB800,0xB801,
	0xB807,0xB808,0xB809,0xB80C,0xB810,0xB818,0xB819,0xB81B,
	0xB81D,0xB824,0xB825,0xB828,0xB82C,0xB834,0xB835,0xB837,
	0xB838,0xB839,0xB840,0xB844,0xB851,0xB853,0xB85C,0xB85D,
	0xB860,0xB864,0xB86C,0xB86D,0xB86F,0xB871,0xB878,0xB87C,
	0xB88D,0xB8A8,0xB8B0,0xB8B4,0xB8B8,0xB8C0,0xB8C1,0xB8C3,
	0xB8C5,0xB8CC,0xB8D0,0xB8D4,0xB8DD,0xB8DF,0xB8E1,0xB8E8,
	0xB8E9,0xB8EC,0xB8F0,0xB8F8,0xB8F9,0xB8FB,0xB8FD,0xB904,
	0xB918,0xB920,0xB93C,0xB93D,0xB940,0xB944,0xB94C,0xB94F,
	0xB951,0xB958,0xB959,0xB95C,0xB960,0xB968,0xB969,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 57, Array index 0x1500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xB96B,0xB96D,0xB974,0xB975,0xB978,0xB97C,0xB984,
	0xB985,0xB987,0xB989,0xB98A,0xB98D,0xB98E,0xB9AC,0xB9AD,
	0xB9B0,0xB9B4,0xB9BC,0xB9BD,0xB9BF,0xB9C1,0xB9C8,0xB9C9,
	0xB9CC,0xB9CE,0xB9CF,0xB9D0,0xB9D1,0xB9D2,0xB9D8,0xB9D9,
	0xB9DB,0xB9DD,0xB9DE,0xB9E1,0xB9E3,0xB9E4,0xB9E5,0xB9E8,
	0xB9EC,0xB9F4,0xB9F5,0xB9F7,0xB9F8,0xB9F9,0xB9FA,0xBA00,
	0xBA01,0xBA08,0xBA15,0xBA38,0xBA39,0xBA3C,0xBA40,0xBA42,
	0xBA48,0xBA49,0xBA4B,0xBA4D,0xBA4E,0xBA53,0xBA54,0xBA55,
	0xBA58,0xBA5C,0xBA64,0xBA65,0xBA67,0xBA68,0xBA69,0xBA70,
	0xBA71,0xBA74,0xBA78,0xBA83,0xBA84,0xBA85,0xBA87,0xBA8C,
	0xBAA8,0xBAA9,0xBAAB,0xBAAC,0xBAB0,0xBAB2,0xBAB8,0xBAB9,
	0xBABB,0xBABD,0xBAC4,0xBAC8,0xBAD8,0xBAD9,0xBAFC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 58, Array index 0x1600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xBB00,0xBB04,0xBB0D,0xBB0F,0xBB11,0xBB18,0xBB1C,
	0xBB20,0xBB29,0xBB2B,0xBB34,0xBB35,0xBB36,0xBB38,0xBB3B,
	0xBB3C,0xBB3D,0xBB3E,0xBB44,0xBB45,0xBB47,0xBB49,0xBB4D,
	0xBB4F,0xBB50,0xBB54,0xBB58,0xBB61,0xBB63,0xBB6C,0xBB88,
	0xBB8C,0xBB90,0xBBA4,0xBBA8,0xBBAC,0xBBB4,0xBBB7,0xBBC0,
	0xBBC4,0xBBC8,0xBBD0,0xBBD3,0xBBF8,0xBBF9,0xBBFC,0xBBFF,
	0xBC00,0xBC02,0xBC08,0xBC09,0xBC0B,0xBC0C,0xBC0D,0xBC0F,
	0xBC11,0xBC14,0xBC15,0xBC16,0xBC17,0xBC18,0xBC1B,0xBC1C,
	0xBC1D,0xBC1E,0xBC1F,0xBC24,0xBC25,0xBC27,0xBC29,0xBC2D,
	0xBC30,0xBC31,0xBC34,0xBC38,0xBC40,0xBC41,0xBC43,0xBC44,
	0xBC45,0xBC49,0xBC4C,0xBC4D,0xBC50,0xBC5D,0xBC84,0xBC85,
	0xBC88,0xBC8B,0xBC8C,0xBC8E,0xBC94,0xBC95,0xBC97,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 59, Array index 0x1700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xBC99,0xBC9A,0xBCA0,0xBCA1,0xBCA4,0xBCA7,0xBCA8,
	0xBCB0,0xBCB1,0xBCB3,0xBCB4,0xBCB5,0xBCBC,0xBCBD,0xBCC0,
	0xBCC4,0xBCCD,0xBCCF,0xBCD0,0xBCD1,0xBCD5,0xBCD8,0xBCDC,
	0xBCF4,0xBCF5,0xBCF6,0xBCF8,0xBCFC,0xBD04,0xBD05,0xBD07,
	0xBD09,0xBD10,0xBD14,0xBD24,0xBD2C,0xBD40,0xBD48,0xBD49,
	0xBD4C,0xBD50,0xBD58,0xBD59,0xBD64,0xBD68,0xBD80,0xBD81,
	0xBD84,0xBD87,0xBD88,0xBD89,0xBD8A,0xBD90,0xBD91,0xBD93,
	0xBD95,0xBD99,0xBD9A,0xBD9C,0xBDA4,0xBDB0,0xBDB8,0xBDD4,
	0xBDD5,0xBDD8,0xBDDC,0xBDE9,0xBDF0,0xBDF4,0xBDF8,0xBE00,
	0xBE03,0xBE05,0xBE0C,0xBE0D,0xBE10,0xBE14,0xBE1C,0xBE1D,
	0xBE1F,0xBE44,0xBE45,0xBE48,0xBE4C,0xBE4E,0xBE54,0xBE55,
	0xBE57,0xBE59,0xBE5A,0xBE5B,0xBE60,0xBE61,0xBE64,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 60, Array index 0x1800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xBE68,0xBE6A,0xBE70,0xBE71,0xBE73,0xBE74,0xBE75,
	0xBE7B,0xBE7C,0xBE7D,0xBE80,0xBE84,0xBE8C,0xBE8D,0xBE8F,
	0xBE90,0xBE91,0xBE98,0xBE99,0xBEA8,0xBED0,0xBED1,0xBED4,
	0xBED7,0xBED8,0xBEE0,0xBEE3,0xBEE4,0xBEE5,0xBEEC,0xBF01,
	0xBF08,0xBF09,0xBF18,0xBF19,0xBF1B,0xBF1C,0xBF1D,0xBF40,
	0xBF41,0xBF44,0xBF48,0xBF50,0xBF51,0xBF55,0xBF94,0xBFB0,
	0xBFC5,0xBFCC,0xBFCD,0xBFD0,0xBFD4,0xBFDC,0xBFDF,0xBFE1,
	0xC03C,0xC051,0xC058,0xC05C,0xC060,0xC068,0xC069,0xC090,
	0xC091,0xC094,0xC098,0xC0A0,0xC0A1,0xC0A3,0xC0A5,0xC0AC,
	0xC0AD,0xC0AF,0xC0B0,0xC0B3,0xC0B4,0xC0B5,0xC0B6,0xC0BC,
	0xC0BD,0xC0BF,0xC0C0,0xC0C1,0xC0C5,0xC0C8,0xC0C9,0xC0CC,
	0xC0D0,0xC0D8,0xC0D9,0xC0DB,0xC0DC,0xC0DD,0xC0E4,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 61, Array index 0x1900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xC0E5,0xC0E8,0xC0EC,0xC0F4,0xC0F5,0xC0F7,0xC0F9,
	0xC100,0xC104,0xC108,0xC110,0xC115,0xC11C,0xC11D,0xC11E,
	0xC11F,0xC120,0xC123,0xC124,0xC126,0xC127,0xC12C,0xC12D,
	0xC12F,0xC130,0xC131,0xC136,0xC138,0xC139,0xC13C,0xC140,
	0xC148,0xC149,0xC14B,0xC14C,0xC14D,0xC154,0xC155,0xC158,
	0xC15C,0xC164,0xC165,0xC167,0xC168,0xC169,0xC170,0xC174,
	0xC178,0xC185,0xC18C,0xC18D,0xC18E,0xC190,0xC194,0xC196,
	0xC19C,0xC19D,0xC19F,0xC1A1,0xC1A5,0xC1A8,0xC1A9,0xC1AC,
	0xC1B0,0xC1BD,0xC1C4,0xC1C8,0xC1CC,0xC1D4,0xC1D7,0xC1D8,
	0xC1E0,0xC1E4,0xC1E8,0xC1F0,0xC1F1,0xC1F3,0xC1FC,0xC1FD,
	0xC200,0xC204,0xC20C,0xC20D,0xC20F,0xC211,0xC218,0xC219,
	0xC21C,0xC21F,0xC220,0xC228,0xC229,0xC22B,0xC22D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 62, Array index 0x1A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xC22F,0xC231,0xC232,0xC234,0xC248,0xC250,0xC251,
	0xC254,0xC258,0xC260,0xC265,0xC26C,0xC26D,0xC270,0xC274,
	0xC27C,0xC27D,0xC27F,0xC281,0xC288,0xC289,0xC290,0xC298,
	0xC29B,0xC29D,0xC2A4,0xC2A5,0xC2A8,0xC2AC,0xC2AD,0xC2B4,
	0xC2B5,0xC2B7,0xC2B9,0xC2DC,0xC2DD,0xC2E0,0xC2E3,0xC2E4,
	0xC2EB,0xC2EC,0xC2ED,0xC2EF,0xC2F1,0xC2F6,0xC2F8,0xC2F9,
	0xC2FB,0xC2FC,0xC300,0xC308,0xC309,0xC30C,0xC30D,0xC313,
	0xC314,0xC315,0xC318,0xC31C,0xC324,0xC325,0xC328,0xC329,
	0xC345,0xC368,0xC369,0xC36C,0xC370,0xC372,0xC378,0xC379,
	0xC37C,0xC37D,0xC384,0xC388,0xC38C,0xC3C0,0xC3D8,0xC3D9,
	0xC3DC,0xC3DF,0xC3E0,0xC3E2,0xC3E8,0xC3E9,0xC3ED,0xC3F4,
	0xC3F5,0xC3F8,0xC408,0xC410,0xC424,0xC42C,0xC430,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 63, Array index 0x1B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xC434,0xC43C,0xC43D,0xC448,0xC464,0xC465,0xC468,
	0xC46C,0xC474,0xC475,0xC479,0xC480,0xC494,0xC49C,0xC4B8,
	0xC4BC,0xC4E9,0xC4F0,0xC4F1,0xC4F4,0xC4F8,0xC4FA,0xC4FF,
	0xC500,0xC501,0xC50C,0xC510,0xC514,0xC51C,0xC528,0xC529,
	0xC52C,0xC530,0xC538,0xC539,0xC53B,0xC53D,0xC544,0xC545,
	0xC548,0xC549,0xC54A,0xC54C,0xC54D,0xC54E,0xC553,0xC554,
	0xC555,0xC557,0xC558,0xC559,0xC55D,0xC55E,0xC560,0xC561,
	0xC564,0xC568,0xC570,0xC571,0xC573,0xC574,0xC575,0xC57C,
	0xC57D,0xC580,0xC584,0xC587,0xC58C,0xC58D,0xC58F,0xC591,
	0xC595,0xC597,0xC598,0xC59C,0xC5A0,0xC5A9,0xC5B4,0xC5B5,
	0xC5B8,0xC5B9,0xC5BB,0xC5BC,0xC5BD,0xC5BE,0xC5C4,0xC5C5,
	0xC5C6,0xC5C7,0xC5C8,0xC5C9,0xC5CA,0xC5CC,0xC5CE,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 64, Array index 0x1C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xC5D0,0xC5D1,0xC5D4,0xC5D8,0xC5E0,0xC5E1,0xC5E3,
	0xC5E5,0xC5EC,0xC5ED,0xC5EE,0xC5F0,0xC5F4,0xC5F6,0xC5F7,
	0xC5FC,0xC5FD,0xC5FE,0xC5FF,0xC600,0xC601,0xC605,0xC606,
	0xC607,0xC608,0xC60C,0xC610,0xC618,0xC619,0xC61B,0xC61C,
	0xC624,0xC625,0xC628,0xC62C,0xC62D,0xC62E,0xC630,0xC633,
	0xC634,0xC635,0xC637,0xC639,0xC63B,0xC640,0xC641,0xC644,
	0xC648,0xC650,0xC651,0xC653,0xC654,0xC655,0xC65C,0xC65D,
	0xC660,0xC66C,0xC66F,0xC671,0xC678,0xC679,0xC67C,0xC680,
	0xC688,0xC689,0xC68B,0xC68D,0xC694,0xC695,0xC698,0xC69C,
	0xC6A4,0xC6A5,0xC6A7,0xC6A9,0xC6B0,0xC6B1,0xC6B4,0xC6B8,
	0xC6B9,0xC6BA,0xC6C0,0xC6C1,0xC6C3,0xC6C5,0xC6CC,0xC6CD,
	0xC6D0,0xC6D4,0xC6DC,0xC6DD,0xC6E0,0xC6E1,0xC6E8,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 65, Array index 0x1D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xC6E9,0xC6EC,0xC6F0,0xC6F8,0xC6F9,0xC6FD,0xC704,
	0xC705,0xC708,0xC70C,0xC714,0xC715,0xC717,0xC719,0xC720,
	0xC721,0xC724,0xC728,0xC730,0xC731,0xC733,0xC735,0xC737,
	0xC73C,0xC73D,0xC740,0xC744,0xC74A,0xC74C,0xC74D,0xC74F,
	0xC751,0xC752,0xC753,0xC754,0xC755,0xC756,0xC757,0xC758,
	0xC75C,0xC760,0xC768,0xC76B,0xC774,0xC775,0xC778,0xC77C,
	0xC77D,0xC77E,0xC783,0xC784,0xC785,0xC787,0xC788,0xC789,
	0xC78A,0xC78E,0xC790,0xC791,0xC794,0xC796,0xC797,0xC798,
	0xC79A,0xC7A0,0xC7A1,0xC7A3,0xC7A4,0xC7A5,0xC7A6,0xC7AC,
	0xC7AD,0xC7B0,0xC7B4,0xC7BC,0xC7BD,0xC7BF,0xC7C0,0xC7C1,
	0xC7C8,0xC7C9,0xC7CC,0xC7CE,0xC7D0,0xC7D8,0xC7DD,0xC7E4,
	0xC7E8,0xC7EC,0xC800,0xC801,0xC804,0xC808,0xC80A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 66, Array index 0x1E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xC810,0xC811,0xC813,0xC815,0xC816,0xC81C,0xC81D,
	0xC820,0xC824,0xC82C,0xC82D,0xC82F,0xC831,0xC838,0xC83C,
	0xC840,0xC848,0xC849,0xC84C,0xC84D,0xC854,0xC870,0xC871,
	0xC874,0xC878,0xC87A,0xC880,0xC881,0xC883,0xC885,0xC886,
	0xC887,0xC88B,0xC88C,0xC88D,0xC894,0xC89D,0xC89F,0xC8A1,
	0xC8A8,0xC8BC,0xC8BD,0xC8C4,0xC8C8,0xC8CC,0xC8D4,0xC8D5,
	0xC8D7,0xC8D9,0xC8E0,0xC8E1,0xC8E4,0xC8F5,0xC8FC,0xC8FD,
	0xC900,0xC904,0xC905,0xC906,0xC90C,0xC90D,0xC90F,0xC911,
	0xC918,0xC92C,0xC934,0xC950,0xC951,0xC954,0xC958,0xC960,
	0xC961,0xC963,0xC96C,0xC970,0xC974,0xC97C,0xC988,0xC989,
	0xC98C,0xC990,0xC998,0xC999,0xC99B,0xC99D,0xC9C0,0xC9C1,
	0xC9C4,0xC9C7,0xC9C8,0xC9CA,0xC9D0,0xC9D1,0xC9D3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 67, Array index 0x1F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xC9D5,0xC9D6,0xC9D9,0xC9DA,0xC9DC,0xC9DD,0xC9E0,
	0xC9E2,0xC9E4,0xC9E7,0xC9EC,0xC9ED,0xC9EF,0xC9F0,0xC9F1,
	0xC9F8,0xC9F9,0xC9FC,0xCA00,0xCA08,0xCA09,0xCA0B,0xCA0C,
	0xCA0D,0xCA14,0xCA18,0xCA29,0xCA4C,0xCA4D,0xCA50,0xCA54,
	0xCA5C,0xCA5D,0xCA5F,0xCA60,0xCA61,0xCA68,0xCA7D,0xCA84,
	0xCA98,0xCABC,0xCABD,0xCAC0,0xCAC4,0xCACC,0xCACD,0xCACF,
	0xCAD1,0xCAD3,0xCAD8,0xCAD9,0xCAE0,0xCAEC,0xCAF4,0xCB08,
	0xCB10,0xCB14,0xCB18,0xCB20,0xCB21,0xCB41,0xCB48,0xCB49,
	0xCB4C,0xCB50,0xCB58,0xCB59,0xCB5D,0xCB64,0xCB78,0xCB79,
	0xCB9C,0xCBB8,0xCBD4,0xCBE4,0xCBE7,0xCBE9,0xCC0C,0xCC0D,
	0xCC10,0xCC14,0xCC1C,0xCC1D,0xCC21,0xCC22,0xCC27,0xCC28,
	0xCC29,0xCC2C,0xCC2E,0xCC30,0xCC38,0xCC39,0xCC3B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 68, Array index 0x2000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xCC3C,0xCC3D,0xCC3E,0xCC44,0xCC45,0xCC48,0xCC4C,
	0xCC54,0xCC55,0xCC57,0xCC58,0xCC59,0xCC60,0xCC64,0xCC66,
	0xCC68,0xCC70,0xCC75,0xCC98,0xCC99,0xCC9C,0xCCA0,0xCCA8,
	0xCCA9,0xCCAB,0xCCAC,0xCCAD,0xCCB4,0xCCB5,0xCCB8,0xCCBC,
	0xCCC4,0xCCC5,0xCCC7,0xCCC9,0xCCD0,0xCCD4,0xCCE4,0xCCEC,
	0xCCF0,0xCD01,0xCD08,0xCD09,0xCD0C,0xCD10,0xCD18,0xCD19,
	0xCD1B,0xCD1D,0xCD24,0xCD28,0xCD2C,0xCD39,0xCD5C,0xCD60,
	0xCD64,0xCD6C,0xCD6D,0xCD6F,0xCD71,0xCD78,0xCD88,0xCD94,
	0xCD95,0xCD98,0xCD9C,0xCDA4,0xCDA5,0xCDA7,0xCDA9,0xCDB0,
	0xCDC4,0xCDCC,0xCDD0,0xCDE8,0xCDEC,0xCDF0,0xCDF8,0xCDF9,
	0xCDFB,0xCDFD,0xCE04,0xCE08,0xCE0C,0xCE14,0xCE19,0xCE20,
	0xCE21,0xCE24,0xCE28,0xCE30,0xCE31,0xCE33,0xCE35,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 69, Array index 0x2100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xCE58,0xCE59,0xCE5C,0xCE5F,0xCE60,0xCE61,0xCE68,
	0xCE69,0xCE6B,0xCE6D,0xCE74,0xCE75,0xCE78,0xCE7C,0xCE84,
	0xCE85,0xCE87,0xCE89,0xCE90,0xCE91,0xCE94,0xCE98,0xCEA0,
	0xCEA1,0xCEA3,0xCEA4,0xCEA5,0xCEAC,0xCEAD,0xCEC1,0xCEE4,
	0xCEE5,0xCEE8,0xCEEB,0xCEEC,0xCEF4,0xCEF5,0xCEF7,0xCEF8,
	0xCEF9,0xCF00,0xCF01,0xCF04,0xCF08,0xCF10,0xCF11,0xCF13,
	0xCF15,0xCF1C,0xCF20,0xCF24,0xCF2C,0xCF2D,0xCF2F,0xCF30,
	0xCF31,0xCF38,0xCF54,0xCF55,0xCF58,0xCF5C,0xCF64,0xCF65,
	0xCF67,0xCF69,0xCF70,0xCF71,0xCF74,0xCF78,0xCF80,0xCF85,
	0xCF8C,0xCFA1,0xCFA8,0xCFB0,0xCFC4,0xCFE0,0xCFE1,0xCFE4,
	0xCFE8,0xCFF0,0xCFF1,0xCFF3,0xCFF5,0xCFFC,0xD000,0xD004,
	0xD011,0xD018,0xD02D,0xD034,0xD035,0xD038,0xD03C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 70, Array index 0x2200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xD044,0xD045,0xD047,0xD049,0xD050,0xD054,0xD058,
	0xD060,0xD06C,0xD06D,0xD070,0xD074,0xD07C,0xD07D,0xD081,
	0xD0A4,0xD0A5,0xD0A8,0xD0AC,0xD0B4,0xD0B5,0xD0B7,0xD0B9,
	0xD0C0,0xD0C1,0xD0C4,0xD0C8,0xD0C9,0xD0D0,0xD0D1,0xD0D3,
	0xD0D4,0xD0D5,0xD0DC,0xD0DD,0xD0E0,0xD0E4,0xD0EC,0xD0ED,
	0xD0EF,0xD0F0,0xD0F1,0xD0F8,0xD10D,0xD130,0xD131,0xD134,
	0xD138,0xD13A,0xD140,0xD141,0xD143,0xD144,0xD145,0xD14C,
	0xD14D,0xD150,0xD154,0xD15C,0xD15D,0xD15F,0xD161,0xD168,
	0xD16C,0xD17C,0xD184,0xD188,0xD1A0,0xD1A1,0xD1A4,0xD1A8,
	0xD1B0,0xD1B1,0xD1B3,0xD1B5,0xD1BA,0xD1BC,0xD1C0,0xD1D8,
	0xD1F4,0xD1F8,0xD207,0xD209,0xD210,0xD22C,0xD22D,0xD230,
	0xD234,0xD23C,0xD23D,0xD23F,0xD241,0xD248,0xD25C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 71, Array index 0x2300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xD264,0xD280,0xD281,0xD284,0xD288,0xD290,0xD291,
	0xD295,0xD29C,0xD2A0,0xD2A4,0xD2AC,0xD2B1,0xD2B8,0xD2B9,
	0xD2BC,0xD2BF,0xD2C0,0xD2C2,0xD2C8,0xD2C9,0xD2CB,0xD2D4,
	0xD2D8,0xD2DC,0xD2E4,0xD2E5,0xD2F0,0xD2F1,0xD2F4,0xD2F8,
	0xD300,0xD301,0xD303,0xD305,0xD30C,0xD30D,0xD30E,0xD310,
	0xD314,0xD316,0xD31C,0xD31D,0xD31F,0xD320,0xD321,0xD325,
	0xD328,0xD329,0xD32C,0xD330,0xD338,0xD339,0xD33B,0xD33C,
	0xD33D,0xD344,0xD345,0xD37C,0xD37D,0xD380,0xD384,0xD38C,
	0xD38D,0xD38F,0xD390,0xD391,0xD398,0xD399,0xD39C,0xD3A0,
	0xD3A8,0xD3A9,0xD3AB,0xD3AD,0xD3B4,0xD3B8,0xD3BC,0xD3C4,
	0xD3C5,0xD3C8,0xD3C9,0xD3D0,0xD3D8,0xD3E1,0xD3E3,0xD3EC,
	0xD3ED,0xD3F0,0xD3F4,0xD3FC,0xD3FD,0xD3FF,0xD401,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 72, Array index 0x2400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xD408,0xD41D,0xD440,0xD444,0xD45C,0xD460,0xD464,
	0xD46D,0xD46F,0xD478,0xD479,0xD47C,0xD47F,0xD480,0xD482,
	0xD488,0xD489,0xD48B,0xD48D,0xD494,0xD4A9,0xD4CC,0xD4D0,
	0xD4D4,0xD4DC,0xD4DF,0xD4E8,0xD4EC,0xD4F0,0xD4F8,0xD4FB,
	0xD4FD,0xD504,0xD508,0xD50C,0xD514,0xD515,0xD517,0xD53C,
	0xD53D,0xD540,0xD544,0xD54C,0xD54D,0xD54F,0xD551,0xD558,
	0xD559,0xD55C,0xD560,0xD565,0xD568,0xD569,0xD56B,0xD56D,
	0xD574,0xD575,0xD578,0xD57C,0xD584,0xD585,0xD587,0xD588,
	0xD589,0xD590,0xD5A5,0xD5C8,0xD5C9,0xD5CC,0xD5D0,0xD5D2,
	0xD5D8,0xD5D9,0xD5DB,0xD5DD,0xD5E4,0xD5E5,0xD5E8,0xD5EC,
	0xD5F4,0xD5F5,0xD5F7,0xD5F9,0xD600,0xD601,0xD604,0xD608,
	0xD610,0xD611,0xD613,0xD614,0xD615,0xD61C,0xD620,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 73, Array index 0x2500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xD624,0xD62D,0xD638,0xD639,0xD63C,0xD640,0xD645,
	0xD648,0xD649,0xD64B,0xD64D,0xD651,0xD654,0xD655,0xD658,
	0xD65C,0xD667,0xD669,0xD670,0xD671,0xD674,0xD683,0xD685,
	0xD68C,0xD68D,0xD690,0xD694,0xD69D,0xD69F,0xD6A1,0xD6A8,
	0xD6AC,0xD6B0,0xD6B9,0xD6BB,0xD6C4,0xD6C5,0xD6C8,0xD6CC,
	0xD6D1,0xD6D4,0xD6D7,0xD6D9,0xD6E0,0xD6E4,0xD6E8,0xD6F0,
	0xD6F5,0xD6FC,0xD6FD,0xD700,0xD704,0xD711,0xD718,0xD719,
	0xD71C,0xD720,0xD728,0xD729,0xD72B,0xD72D,0xD734,0xD735,
	0xD738,0xD73C,0xD744,0xD747,0xD749,0xD750,0xD751,0xD754,
	0xD756,0xD757,0xD758,0xD759,0xD760,0xD761,0xD763,0xD765,
	0xD769,0xD76C,0xD770,0xD774,0xD77C,0xD77D,0xD781,0xD788,
	0xD789,0xD78C,0xD790,0xD798,0xD799,0xD79B,0xD79D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 75, Array index 0x2600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F3D,0x4F73,0x5047,0x50F9,0x52A0,0x53EF,0x5475,
	0x54E5,0x5609,0x5AC1,0x5BB6,0x6687,0x67B6,0x67B7,0x67EF,
	0x6B4C,0x73C2,0x75C2,0x7A3C,0x82DB,0x8304,0x8857,0x8888,
	0x8A36,0x8CC8,0x8DCF,0x8EFB,0x8FE6,0x99D5,0x523B,0x5374,
	0x5404,0x606A,0x6164,0x6BBC,0x73CF,0x811A,0x89BA,0x89D2,
	0x95A3,0x4F83,0x520A,0x58BE,0x5978,0x59E6,0x5E72,0x5E79,
	0x61C7,0x63C0,0x6746,0x67EC,0x687F,0x6F97,0x764E,0x770B,
	0x78F5,0x7A08,0x7AFF,0x7C21,0x809D,0x826E,0x8271,0x8AEB,
	0x9593,0x4E6B,0x559D,0x66F7,0x6E34,0x78A3,0x7AED,0x845B,
	0x8910,0x874E,0x97A8,0x52D8,0x574E,0x582A,0x5D4C,0x611F,
	0x61BE,0x6221,0x6562,0x67D1,0x6A44,0x6E1B,0x7518,0x75B3,
	0x76E3,0x77B0,0x7D3A,0x90AF,0x9451,0x9452,0x9F95,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 76, Array index 0x2700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5323,0x5CAC,0x7532,0x80DB,0x9240,0x9598,0x525B,
	0x5808,0x59DC,0x5CA1,0x5D17,0x5EB7,0x5F3A,0x5F4A,0x6177,
	0x6C5F,0x757A,0x7586,0x7CE0,0x7D73,0x7DB1,0x7F8C,0x8154,
	0x8221,0x8591,0x8941,0x8B1B,0x92FC,0x964D,0x9C47,0x4ECB,
	0x4EF7,0x500B,0x51F1,0x584F,0x6137,0x613E,0x6168,0x6539,
	0x69EA,0x6F11,0x75A5,0x7686,0x76D6,0x7B87,0x82A5,0x84CB,
	0xF900,0x93A7,0x958B,0x5580,0x5BA2,0x5751,0xF901,0x7CB3,
	0x7FB9,0x91B5,0x5028,0x53BB,0x5C45,0x5DE8,0x62D2,0x636E,
	0x64DA,0x64E7,0x6E20,0x70AC,0x795B,0x8DDD,0x8E1E,0xF902,
	0x907D,0x9245,0x92F8,0x4E7E,0x4EF6,0x5065,0x5DFE,0x5EFA,
	0x6106,0x6957,0x8171,0x8654,0x8E47,0x9375,0x9A2B,0x4E5E,
	0x5091,0x6770,0x6840,0x5109,0x528D,0x5292,0x6AA2,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 77, Array index 0x2800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x77BC,0x9210,0x9ED4,0x52AB,0x602F,0x8FF2,0x5048,
	0x61A9,0x63ED,0x64CA,0x683C,0x6A84,0x6FC0,0x8188,0x89A1,
	0x9694,0x5805,0x727D,0x72AC,0x7504,0x7D79,0x7E6D,0x80A9,
	0x898B,0x8B74,0x9063,0x9D51,0x6289,0x6C7A,0x6F54,0x7D50,
	0x7F3A,0x8A23,0x517C,0x614A,0x7B9D,0x8B19,0x9257,0x938C,
	0x4EAC,0x4FD3,0x501E,0x50BE,0x5106,0x52C1,0x52CD,0x537F,
	0x5770,0x5883,0x5E9A,0x5F91,0x6176,0x61AC,0x64CE,0x656C,
	0x666F,0x66BB,0x66F4,0x6897,0x6D87,0x7085,0x70F1,0x749F,
	0x74A5,0x74CA,0x75D9,0x786C,0x78EC,0x7ADF,0x7AF6,0x7D45,
	0x7D93,0x8015,0x803F,0x811B,0x8396,0x8B66,0x8F15,0x9015,
	0x93E1,0x9803,0x9838,0x9A5A,0x9BE8,0x4FC2,0x5553,0x583A,
	0x5951,0x5B63,0x5C46,0x60B8,0x6212,0x6842,0x68B0,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 78, Array index 0x2900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x68E8,0x6EAA,0x754C,0x7678,0x78CE,0x7A3D,0x7CFB,
	0x7E6B,0x7E7C,0x8A08,0x8AA1,0x8C3F,0x968E,0x9DC4,0x53E4,
	0x53E9,0x544A,0x5471,0x56FA,0x59D1,0x5B64,0x5C3B,0x5EAB,
	0x62F7,0x6537,0x6545,0x6572,0x66A0,0x67AF,0x69C1,0x6CBD,
	0x75FC,0x7690,0x777E,0x7A3F,0x7F94,0x8003,0x80A1,0x818F,
	0x82E6,0x82FD,0x83F0,0x85C1,0x8831,0x88B4,0x8AA5,0xF903,
	0x8F9C,0x932E,0x96C7,0x9867,0x9AD8,0x9F13,0x54ED,0x659B,
	0x66F2,0x688F,0x7A40,0x8C37,0x9D60,0x56F0,0x5764,0x5D11,
	0x6606,0x68B1,0x68CD,0x6EFE,0x7428,0x889E,0x9BE4,0x6C68,
	0xF904,0x9AA8,0x4F9B,0x516C,0x5171,0x529F,0x5B54,0x5DE5,
	0x6050,0x606D,0x62F1,0x63A7,0x653B,0x73D9,0x7A7A,0x86A3,
	0x8CA2,0x978F,0x4E32,0x5BE1,0x6208,0x679C,0x74DC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 79, Array index 0x2A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x79D1,0x83D3,0x8A87,0x8AB2,0x8DE8,0x904E,0x934B,
	0x9846,0x5ED3,0x69E8,0x85FF,0x90ED,0xF905,0x51A0,0x5B98,
	0x5BEC,0x6163,0x68FA,0x6B3E,0x704C,0x742F,0x74D8,0x7BA1,
	0x7F50,0x83C5,0x89C0,0x8CAB,0x95DC,0x9928,0x522E,0x605D,
	0x62EC,0x9002,0x4F8A,0x5149,0x5321,0x58D9,0x5EE3,0x66E0,
	0x6D38,0x709A,0x72C2,0x73D6,0x7B50,0x80F1,0x945B,0x5366,
	0x639B,0x7F6B,0x4E56,0x5080,0x584A,0x58DE,0x602A,0x6127,
	0x62D0,0x69D0,0x9B41,0x5B8F,0x7D18,0x80B1,0x8F5F,0x4EA4,
	0x50D1,0x54AC,0x55AC,0x5B0C,0x5DA0,0x5DE7,0x652A,0x654E,
	0x6821,0x6A4B,0x72E1,0x768E,0x77EF,0x7D5E,0x7FF9,0x81A0,
	0x854E,0x86DF,0x8F03,0x8F4E,0x90CA,0x9903,0x9A55,0x9BAB,
	0x4E18,0x4E45,0x4E5D,0x4EC7,0x4FF1,0x5177,0x52FE,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 80, Array index 0x2B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5340,0x53E3,0x53E5,0x548E,0x5614,0x5775,0x57A2,
	0x5BC7,0x5D87,0x5ED0,0x61FC,0x62D8,0x6551,0x67B8,0x67E9,
	0x69CB,0x6B50,0x6BC6,0x6BEC,0x6C42,0x6E9D,0x7078,0x72D7,
	0x7396,0x7403,0x77BF,0x77E9,0x7A76,0x7D7F,0x8009,0x81FC,
	0x8205,0x820A,0x82DF,0x8862,0x8B33,0x8CFC,0x8EC0,0x9011,
	0x90B1,0x9264,0x92B6,0x99D2,0x9A45,0x9CE9,0x9DD7,0x9F9C,
	0x570B,0x5C40,0x83CA,0x97A0,0x97AB,0x9EB4,0x541B,0x7A98,
	0x7FA4,0x88D9,0x8ECD,0x90E1,0x5800,0x5C48,0x6398,0x7A9F,
	0x5BAE,0x5F13,0x7A79,0x7AAE,0x828E,0x8EAC,0x5026,0x5238,
	0x52F8,0x5377,0x5708,0x62F3,0x6372,0x6B0A,0x6DC3,0x7737,
	0x53A5,0x7357,0x8568,0x8E76,0x95D5,0x673A,0x6AC3,0x6F70,
	0x8A6D,0x8ECC,0x994B,0xF906,0x6677,0x6B78,0x8CB4,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 81, Array index 0x2C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9B3C,0xF907,0x53EB,0x572D,0x594E,0x63C6,0x69FB,
	0x73EA,0x7845,0x7ABA,0x7AC5,0x7CFE,0x8475,0x898F,0x8D73,
	0x9035,0x95A8,0x52FB,0x5747,0x7547,0x7B60,0x83CC,0x921E,
	0xF908,0x6A58,0x514B,0x524B,0x5287,0x621F,0x68D8,0x6975,
	0x9699,0x50C5,0x52A4,0x52E4,0x61C3,0x65A4,0x6839,0x69FF,
	0x747E,0x7B4B,0x82B9,0x83EB,0x89B2,0x8B39,0x8FD1,0x9949,
	0xF909,0x4ECA,0x5997,0x64D2,0x6611,0x6A8E,0x7434,0x7981,
	0x79BD,0x82A9,0x887E,0x887F,0x895F,0xF90A,0x9326,0x4F0B,
	0x53CA,0x6025,0x6271,0x6C72,0x7D1A,0x7D66,0x4E98,0x5162,
	0x77DC,0x80AF,0x4F01,0x4F0E,0x5176,0x5180,0x55DC,0x5668,
	0x573B,0x57FA,0x57FC,0x5914,0x5947,0x5993,0x5BC4,0x5C90,
	0x5D0E,0x5DF1,0x5E7E,0x5FCC,0x6280,0x65D7,0x65E3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 82, Array index 0x2D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x671E,0x671F,0x675E,0x68CB,0x68C4,0x6A5F,0x6B3A,
	0x6C23,0x6C7D,0x6C82,0x6DC7,0x7398,0x7426,0x742A,0x7482,
	0x74A3,0x7578,0x757F,0x7881,0x78EF,0x7941,0x7947,0x7948,
	0x797A,0x7B95,0x7D00,0x7DBA,0x7F88,0x8006,0x802D,0x808C,
	0x8A18,0x8B4F,0x8C48,0x8D77,0x9321,0x9324,0x98E2,0x9951,
	0x9A0E,0x9A0F,0x9A65,0x9E92,0x7DCA,0x4F76,0x5409,0x62EE,
	0x6854,0x91D1,0x55AB,0x513A,0xF90B,0xF90C,0x5A1C,0x61E6,
	0xF90D,0x62CF,0x62FF,0xF90E,0xF90F,0xF910,0xF911,0xF912,
	0xF913,0x90A3,0xF914,0xF915,0xF916,0xF917,0xF918,0x8AFE,
	0xF919,0xF91A,0xF91B,0xF91C,0x6696,0xF91D,0x7156,0xF91E,
	0xF91F,0x96E3,0xF920,0x634F,0x637A,0x5357,0xF921,0x678F,
	0x6960,0x6E73,0xF922,0x7537,0xF923,0xF924,0xF925,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 83, Array index 0x2E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7D0D,0xF926,0xF927,0x8872,0x56CA,0x5A18,0xF928,
	0xF929,0xF92A,0xF92B,0xF92C,0x4E43,0xF92D,0x5167,0x5948,
	0x67F0,0x8010,0xF92E,0x5973,0x5E74,0x649A,0x79CA,0x5FF5,
	0x606C,0x62C8,0x637B,0x5BE7,0x5BD7,0x52AA,0xF92F,0x5974,
	0x5F29,0x6012,0xF930,0xF931,0xF932,0x7459,0xF933,0xF934,
	0xF935,0xF936,0xF937,0xF938,0x99D1,0xF939,0xF93A,0xF93B,
	0xF93C,0xF93D,0xF93E,0xF93F,0xF940,0xF941,0xF942,0xF943,
	0x6FC3,0xF944,0xF945,0x81BF,0x8FB2,0x60F1,0xF946,0xF947,
	0x8166,0xF948,0xF949,0x5C3F,0xF94A,0xF94B,0xF94C,0xF94D,
	0xF94E,0xF94F,0xF950,0xF951,0x5AE9,0x8A25,0x677B,0x7D10,
	0xF952,0xF953,0xF954,0xF955,0xF956,0xF957,0x80FD,0xF958,
	0xF959,0x5C3C,0x6CE5,0x533F,0x6EBA,0x591A,0x8336,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 84, Array index 0x2F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E39,0x4EB6,0x4F46,0x55AE,0x5718,0x58C7,0x5F56,
	0x65B7,0x65E6,0x6A80,0x6BB5,0x6E4D,0x77ED,0x7AEF,0x7C1E,
	0x7DDE,0x86CB,0x8892,0x9132,0x935B,0x64BB,0x6FBE,0x737A,
	0x75B8,0x9054,0x5556,0x574D,0x61BA,0x64D4,0x66C7,0x6DE1,
	0x6E5B,0x6F6D,0x6FB9,0x75F0,0x8043,0x81BD,0x8541,0x8983,
	0x8AC7,0x8B5A,0x931F,0x6C93,0x7553,0x7B54,0x8E0F,0x905D,
	0x5510,0x5802,0x5858,0x5E62,0x6207,0x649E,0x68E0,0x7576,
	0x7CD6,0x87B3,0x9EE8,0x4EE3,0x5788,0x576E,0x5927,0x5C0D,
	0x5CB1,0x5E36,0x5F85,0x6234,0x64E1,0x73B3,0x81FA,0x888B,
	0x8CB8,0x968A,0x9EDB,0x5B85,0x5FB7,0x60B3,0x5012,0x5200,
	0x5230,0x5716,0x5835,0x5857,0x5C0E,0x5C60,0x5CF6,0x5D8B,
	0x5EA6,0x5F92,0x60BC,0x6311,0x6389,0x6417,0x6843,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 85, Array index 0x3000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x68F9,0x6AC2,0x6DD8,0x6E21,0x6ED4,0x6FE4,0x71FE,
	0x76DC,0x7779,0x79B1,0x7A3B,0x8404,0x89A9,0x8CED,0x8DF3,
	0x8E48,0x9003,0x9014,0x9053,0x90FD,0x934D,0x9676,0x97DC,
	0x6BD2,0x7006,0x7258,0x72A2,0x7368,0x7763,0x79BF,0x7BE4,
	0x7E9B,0x8B80,0x58A9,0x60C7,0x6566,0x65FD,0x66BE,0x6C8C,
	0x711E,0x71C9,0x8C5A,0x9813,0x4E6D,0x7A81,0x4EDD,0x51AC,
	0x51CD,0x52D5,0x540C,0x61A7,0x6771,0x6850,0x68DF,0x6D1E,
	0x6F7C,0x75BC,0x77B3,0x7AE5,0x80F4,0x8463,0x9285,0x515C,
	0x6597,0x675C,0x6793,0x75D8,0x7AC7,0x8373,0xF95A,0x8C46,
	0x9017,0x982D,0x5C6F,0x81C0,0x829A,0x9041,0x906F,0x920D,
	0x5F97,0x5D9D,0x6A59,0x71C8,0x767B,0x7B49,0x85E4,0x8B04,
	0x9127,0x9A30,0x5587,0x61F6,0xF95B,0x7669,0x7F85,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 86, Array index 0x3100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x863F,0x87BA,0x88F8,0x908F,0xF95C,0x6D1B,0x70D9,
	0x73DE,0x7D61,0x843D,0xF95D,0x916A,0x99F1,0xF95E,0x4E82,
	0x5375,0x6B04,0x6B12,0x703E,0x721B,0x862D,0x9E1E,0x524C,
	0x8FA3,0x5D50,0x64E5,0x652C,0x6B16,0x6FEB,0x7C43,0x7E9C,
	0x85CD,0x8964,0x89BD,0x62C9,0x81D8,0x881F,0x5ECA,0x6717,
	0x6D6A,0x72FC,0x7405,0x746F,0x8782,0x90DE,0x4F86,0x5D0D,
	0x5FA0,0x840A,0x51B7,0x63A0,0x7565,0x4EAE,0x5006,0x5169,
	0x51C9,0x6881,0x6A11,0x7CAE,0x7CB1,0x7CE7,0x826F,0x8AD2,
	0x8F1B,0x91CF,0x4FB6,0x5137,0x52F5,0x5442,0x5EEC,0x616E,
	0x623E,0x65C5,0x6ADA,0x6FFE,0x792A,0x85DC,0x8823,0x95AD,
	0x9A62,0x9A6A,0x9E97,0x9ECE,0x529B,0x66C6,0x6B77,0x701D,
	0x792B,0x8F62,0x9742,0x6190,0x6200,0x6523,0x6F23,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 87, Array index 0x3200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7149,0x7489,0x7DF4,0x806F,0x84EE,0x8F26,0x9023,
	0x934A,0x51BD,0x5217,0x52A3,0x6D0C,0x70C8,0x88C2,0x5EC9,
	0x6582,0x6BAE,0x6FC2,0x7C3E,0x7375,0x4EE4,0x4F36,0x56F9,
	0xF95F,0x5CBA,0x5DBA,0x601C,0x73B2,0x7B2D,0x7F9A,0x7FCE,
	0x8046,0x901E,0x9234,0x96F6,0x9748,0x9818,0x9F61,0x4F8B,
	0x6FA7,0x79AE,0x91B4,0x96B7,0x52DE,0xF960,0x6488,0x64C4,
	0x6AD3,0x6F5E,0x7018,0x7210,0x76E7,0x8001,0x8606,0x865C,
	0x8DEF,0x8F05,0x9732,0x9B6F,0x9DFA,0x9E75,0x788C,0x797F,
	0x7DA0,0x83C9,0x9304,0x9E7F,0x9E93,0x8AD6,0x58DF,0x5F04,
	0x6727,0x7027,0x74CF,0x7C60,0x807E,0x5121,0x7028,0x7262,
	0x78CA,0x8CC2,0x8CDA,0x8CF4,0x96F7,0x4E86,0x50DA,0x5BEE,
	0x5ED6,0x6599,0x71CE,0x7642,0x77AD,0x804A,0x84FC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 88, Array index 0x3300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x907C,0x9B27,0x9F8D,0x58D8,0x5A41,0x5C62,0x6A13,
	0x6DDA,0x6F0F,0x763B,0x7D2F,0x7E37,0x851E,0x8938,0x93E4,
	0x964B,0x5289,0x65D2,0x67F3,0x69B4,0x6D41,0x6E9C,0x700F,
	0x7409,0x7460,0x7559,0x7624,0x786B,0x8B2C,0x985E,0x516D,
	0x622E,0x9678,0x4F96,0x502B,0x5D19,0x6DEA,0x7DB8,0x8F2A,
	0x5F8B,0x6144,0x6817,0xF961,0x9686,0x52D2,0x808B,0x51DC,
	0x51CC,0x695E,0x7A1C,0x7DBE,0x83F1,0x9675,0x4FDA,0x5229,
	0x5398,0x540F,0x550E,0x5C65,0x60A7,0x674E,0x68A8,0x6D6C,
	0x7281,0x72F8,0x7406,0x7483,0xF962,0x75E2,0x7C6C,0x7F79,
	0x7FB8,0x8389,0x88CF,0x88E1,0x91CC,0x91D0,0x96E2,0x9BC9,
	0x541D,0x6F7E,0x71D0,0x7498,0x85FA,0x8EAA,0x96A3,0x9C57,
	0x9E9F,0x6797,0x6DCB,0x7433,0x81E8,0x9716,0x782C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 89, Array index 0x3400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7ACB,0x7B20,0x7C92,0x6469,0x746A,0x75F2,0x78BC,
	0x78E8,0x99AC,0x9B54,0x9EBB,0x5BDE,0x5E55,0x6F20,0x819C,
	0x83AB,0x9088,0x4E07,0x534D,0x5A29,0x5DD2,0x5F4E,0x6162,
	0x633D,0x6669,0x66FC,0x6EFF,0x6F2B,0x7063,0x779E,0x842C,
	0x8513,0x883B,0x8F13,0x9945,0x9C3B,0x551C,0x62B9,0x672B,
	0x6CAB,0x8309,0x896A,0x977A,0x4EA1,0x5984,0x5FD8,0x5FD9,
	0x671B,0x7DB2,0x7F54,0x8292,0x832B,0x83BD,0x8F1E,0x9099,
	0x57CB,0x59B9,0x5A92,0x5BD0,0x6627,0x679A,0x6885,0x6BCF,
	0x7164,0x7F75,0x8CB7,0x8CE3,0x9081,0x9B45,0x8108,0x8C8A,
	0x964C,0x9A40,0x9EA5,0x5B5F,0x6C13,0x731B,0x76F2,0x76DF,
	0x840C,0x51AA,0x8993,0x514D,0x5195,0x52C9,0x68C9,0x6C94,
	0x7704,0x7720,0x7DBF,0x7DEC,0x9762,0x9EB5,0x6EC5,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 90, Array index 0x3500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8511,0x51A5,0x540D,0x547D,0x660E,0x669D,0x6927,
	0x6E9F,0x76BF,0x7791,0x8317,0x84C2,0x879F,0x9169,0x9298,
	0x9CF4,0x8882,0x4FAE,0x5192,0x52DF,0x59C6,0x5E3D,0x6155,
	0x6478,0x6479,0x66AE,0x67D0,0x6A21,0x6BCD,0x6BDB,0x725F,
	0x7261,0x7441,0x7738,0x77DB,0x8017,0x82BC,0x8305,0x8B00,
	0x8B28,0x8C8C,0x6728,0x6C90,0x7267,0x76EE,0x7766,0x7A46,
	0x9DA9,0x6B7F,0x6C92,0x5922,0x6726,0x8499,0x536F,0x5893,
	0x5999,0x5EDF,0x63CF,0x6634,0x6773,0x6E3A,0x732B,0x7AD7,
	0x82D7,0x9328,0x52D9,0x5DEB,0x61AE,0x61CB,0x620A,0x62C7,
	0x64AB,0x65E0,0x6959,0x6B66,0x6BCB,0x7121,0x73F7,0x755D,
	0x7E46,0x821E,0x8302,0x856A,0x8AA3,0x8CBF,0x9727,0x9D61,
	0x58A8,0x9ED8,0x5011,0x520E,0x543B,0x554F,0x6587,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 91, Array index 0x3600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6C76,0x7D0A,0x7D0B,0x805E,0x868A,0x9580,0x96EF,
	0x52FF,0x6C95,0x7269,0x5473,0x5A9A,0x5C3E,0x5D4B,0x5F4C,
	0x5FAE,0x672A,0x68B6,0x6963,0x6E3C,0x6E44,0x7709,0x7C73,
	0x7F8E,0x8587,0x8B0E,0x8FF7,0x9761,0x9EF4,0x5CB7,0x60B6,
	0x610D,0x61AB,0x654F,0x65FB,0x65FC,0x6C11,0x6CEF,0x739F,
	0x73C9,0x7DE1,0x9594,0x5BC6,0x871C,0x8B10,0x525D,0x535A,
	0x62CD,0x640F,0x64B2,0x6734,0x6A38,0x6CCA,0x73C0,0x749E,
	0x7B94,0x7C95,0x7E1B,0x818A,0x8236,0x8584,0x8FEB,0x96F9,
	0x99C1,0x4F34,0x534A,0x53CD,0x53DB,0x62CC,0x642C,0x6500,
	0x6591,0x69C3,0x6CEE,0x6F58,0x73ED,0x7554,0x7622,0x76E4,
	0x76FC,0x78D0,0x78FB,0x792C,0x7D46,0x822C,0x87E0,0x8FD4,
	0x9812,0x98EF,0x52C3,0x62D4,0x64A5,0x6E24,0x6F51,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 92, Array index 0x3700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x767C,0x8DCB,0x91B1,0x9262,0x9AEE,0x9B43,0x5023,
	0x508D,0x574A,0x59A8,0x5C28,0x5E47,0x5F77,0x623F,0x653E,
	0x65B9,0x65C1,0x6609,0x678B,0x699C,0x6EC2,0x78C5,0x7D21,
	0x80AA,0x8180,0x822B,0x82B3,0x84A1,0x868C,0x8A2A,0x8B17,
	0x90A6,0x9632,0x9F90,0x500D,0x4FF3,0xF963,0x57F9,0x5F98,
	0x62DC,0x6392,0x676F,0x6E43,0x7119,0x76C3,0x80CC,0x80DA,
	0x88F4,0x88F5,0x8919,0x8CE0,0x8F29,0x914D,0x966A,0x4F2F,
	0x4F70,0x5E1B,0x67CF,0x6822,0x767D,0x767E,0x9B44,0x5E61,
	0x6A0A,0x7169,0x71D4,0x756A,0xF964,0x7E41,0x8543,0x85E9,
	0x98DC,0x4F10,0x7B4F,0x7F70,0x95A5,0x51E1,0x5E06,0x68B5,
	0x6C3E,0x6C4E,0x6CDB,0x72AF,0x7BC4,0x8303,0x6CD5,0x743A,
	0x50FB,0x5288,0x58C1,0x64D8,0x6A97,0x74A7,0x7656,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 93, Array index 0x3800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x78A7,0x8617,0x95E2,0x9739,0xF965,0x535E,0x5F01,
	0x8B8A,0x8FA8,0x8FAF,0x908A,0x5225,0x77A5,0x9C49,0x9F08,
	0x4E19,0x5002,0x5175,0x5C5B,0x5E77,0x661E,0x663A,0x67C4,
	0x68C5,0x70B3,0x7501,0x75C5,0x79C9,0x7ADD,0x8F27,0x9920,
	0x9A08,0x4FDD,0x5821,0x5831,0x5BF6,0x666E,0x6B65,0x6D11,
	0x6E7A,0x6F7D,0x73E4,0x752B,0x83E9,0x88DC,0x8913,0x8B5C,
	0x8F14,0x4F0F,0x50D5,0x5310,0x535C,0x5B93,0x5FA9,0x670D,
	0x798F,0x8179,0x832F,0x8514,0x8907,0x8986,0x8F39,0x8F3B,
	0x99A5,0x9C12,0x672C,0x4E76,0x4FF8,0x5949,0x5C01,0x5CEF,
	0x5CF0,0x6367,0x68D2,0x70FD,0x71A2,0x742B,0x7E2B,0x84EC,
	0x8702,0x9022,0x92D2,0x9CF3,0x4E0D,0x4ED8,0x4FEF,0x5085,
	0x5256,0x526F,0x5426,0x5490,0x57E0,0x592B,0x5A66,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 94, Array index 0x3900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5B5A,0x5B75,0x5BCC,0x5E9C,0xF966,0x6276,0x6577,
	0x65A7,0x6D6E,0x6EA5,0x7236,0x7B26,0x7C3F,0x7F36,0x8150,
	0x8151,0x819A,0x8240,0x8299,0x83A9,0x8A03,0x8CA0,0x8CE6,
	0x8CFB,0x8D74,0x8DBA,0x90E8,0x91DC,0x961C,0x9644,0x99D9,
	0x9CE7,0x5317,0x5206,0x5429,0x5674,0x58B3,0x5954,0x596E,
	0x5FFF,0x61A4,0x626E,0x6610,0x6C7E,0x711A,0x76C6,0x7C89,
	0x7CDE,0x7D1B,0x82AC,0x8CC1,0x96F0,0xF967,0x4F5B,0x5F17,
	0x5F7F,0x62C2,0x5D29,0x670B,0x68DA,0x787C,0x7E43,0x9D6C,
	0x4E15,0x5099,0x5315,0x532A,0x5351,0x5983,0x5A62,0x5E87,
	0x60B2,0x618A,0x6249,0x6279,0x6590,0x6787,0x69A7,0x6BD4,
	0x6BD6,0x6BD7,0x6BD8,0x6CB8,0xF968,0x7435,0x75FA,0x7812,
	0x7891,0x79D5,0x79D8,0x7C83,0x7DCB,0x7FE1,0x80A5,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 95, Array index 0x3A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x813E,0x81C2,0x83F2,0x871A,0x88E8,0x8AB9,0x8B6C,
	0x8CBB,0x9119,0x975E,0x98DB,0x9F3B,0x56AC,0x5B2A,0x5F6C,
	0x658C,0x6AB3,0x6BAF,0x6D5C,0x6FF1,0x7015,0x725D,0x73AD,
	0x8CA7,0x8CD3,0x983B,0x6191,0x6C37,0x8058,0x9A01,0x4E4D,
	0x4E8B,0x4E9B,0x4ED5,0x4F3A,0x4F3C,0x4F7F,0x4FDF,0x50FF,
	0x53F2,0x53F8,0x5506,0x55E3,0x56DB,0x58EB,0x5962,0x5A11,
	0x5BEB,0x5BFA,0x5C04,0x5DF3,0x5E2B,0x5F99,0x601D,0x6368,
	0x659C,0x65AF,0x67F6,0x67FB,0x68AD,0x6B7B,0x6C99,0x6CD7,
	0x6E23,0x7009,0x7345,0x7802,0x793E,0x7940,0x7960,0x79C1,
	0x7BE9,0x7D17,0x7D72,0x8086,0x820D,0x838E,0x84D1,0x86C7,
	0x88DF,0x8A50,0x8A5E,0x8B1D,0x8CDC,0x8D66,0x8FAD,0x90AA,
	0x98FC,0x99DF,0x9E9D,0x524A,0xF969,0x6714,0xF96A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 96, Array index 0x3B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5098,0x522A,0x5C71,0x6563,0x6C55,0x73CA,0x7523,
	0x759D,0x7B97,0x849C,0x9178,0x9730,0x4E77,0x6492,0x6BBA,
	0x715E,0x85A9,0x4E09,0xF96B,0x6749,0x68EE,0x6E17,0x829F,
	0x8518,0x886B,0x63F7,0x6F81,0x9212,0x98AF,0x4E0A,0x50B7,
	0x50CF,0x511F,0x5546,0x55AA,0x5617,0x5B40,0x5C19,0x5CE0,
	0x5E38,0x5E8A,0x5EA0,0x5EC2,0x60F3,0x6851,0x6A61,0x6E58,
	0x723D,0x7240,0x72C0,0x76F8,0x7965,0x7BB1,0x7FD4,0x88F3,
	0x89F4,0x8A73,0x8C61,0x8CDE,0x971C,0x585E,0x74BD,0x8CFD,
	0x55C7,0xF96C,0x7A61,0x7D22,0x8272,0x7272,0x751F,0x7525,
	0xF96D,0x7B19,0x5885,0x58FB,0x5DBC,0x5E8F,0x5EB6,0x5F90,
	0x6055,0x6292,0x637F,0x654D,0x6691,0x66D9,0x66F8,0x6816,
	0x68F2,0x7280,0x745E,0x7B6E,0x7D6E,0x7DD6,0x7F72,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 97, Array index 0x3C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x80E5,0x8212,0x85AF,0x897F,0x8A93,0x901D,0x92E4,
	0x9ECD,0x9F20,0x5915,0x596D,0x5E2D,0x60DC,0x6614,0x6673,
	0x6790,0x6C50,0x6DC5,0x6F5F,0x77F3,0x78A9,0x84C6,0x91CB,
	0x932B,0x4ED9,0x50CA,0x5148,0x5584,0x5B0B,0x5BA3,0x6247,
	0x657E,0x65CB,0x6E32,0x717D,0x7401,0x7444,0x7487,0x74BF,
	0x766C,0x79AA,0x7DDA,0x7E55,0x7FA8,0x817A,0x81B3,0x8239,
	0x861A,0x87EC,0x8A75,0x8DE3,0x9078,0x9291,0x9425,0x994D,
	0x9BAE,0x5368,0x5C51,0x6954,0x6CC4,0x6D29,0x6E2B,0x820C,
	0x859B,0x893B,0x8A2D,0x8AAA,0x96EA,0x9F67,0x5261,0x66B9,
	0x6BB2,0x7E96,0x87FE,0x8D0D,0x9583,0x965D,0x651D,0x6D89,
	0x71EE,0xF96E,0x57CE,0x59D3,0x5BAC,0x6027,0x60FA,0x6210,
	0x661F,0x665F,0x7329,0x73F9,0x76DB,0x7701,0x7B6C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 98, Array index 0x3D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8056,0x8072,0x8165,0x8AA0,0x9192,0x4E16,0x52E2,
	0x6B72,0x6D17,0x7A05,0x7B39,0x7D30,0xF96F,0x8CB0,0x53EC,
	0x562F,0x5851,0x5BB5,0x5C0F,0x5C11,0x5DE2,0x6240,0x6383,
	0x6414,0x662D,0x68B3,0x6CBC,0x6D88,0x6EAF,0x701F,0x70A4,
	0x71D2,0x7526,0x758F,0x758E,0x7619,0x7B11,0x7BE0,0x7C2B,
	0x7D20,0x7D39,0x852C,0x856D,0x8607,0x8A34,0x900D,0x9061,
	0x90B5,0x92B7,0x97F6,0x9A37,0x4FD7,0x5C6C,0x675F,0x6D91,
	0x7C9F,0x7E8C,0x8B16,0x8D16,0x901F,0x5B6B,0x5DFD,0x640D,
	0x84C0,0x905C,0x98E1,0x7387,0x5B8B,0x609A,0x677E,0x6DDE,
	0x8A1F,0x8AA6,0x9001,0x980C,0x5237,0xF970,0x7051,0x788E,
	0x9396,0x8870,0x91D7,0x4FEE,0x53D7,0x55FD,0x56DA,0x5782,
	0x58FD,0x5AC2,0x5B88,0x5CAB,0x5CC0,0x5E25,0x6101,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 99, Array index 0x3E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x620D,0x624B,0x6388,0x641C,0x6536,0x6578,0x6A39,
	0x6B8A,0x6C34,0x6D19,0x6F31,0x71E7,0x72E9,0x7378,0x7407,
	0x74B2,0x7626,0x7761,0x79C0,0x7A57,0x7AEA,0x7CB9,0x7D8F,
	0x7DAC,0x7E61,0x7F9E,0x8129,0x8331,0x8490,0x84DA,0x85EA,
	0x8896,0x8AB0,0x8B90,0x8F38,0x9042,0x9083,0x916C,0x9296,
	0x92B9,0x968B,0x96A7,0x96A8,0x96D6,0x9700,0x9808,0x9996,
	0x9AD3,0x9B1A,0x53D4,0x587E,0x5919,0x5B70,0x5BBF,0x6DD1,
	0x6F5A,0x719F,0x7421,0x74B9,0x8085,0x83FD,0x5DE1,0x5F87,
	0x5FAA,0x6042,0x65EC,0x6812,0x696F,0x6A53,0x6B89,0x6D35,
	0x6DF3,0x73E3,0x76FE,0x77AC,0x7B4D,0x7D14,0x8123,0x821C,
	0x8340,0x84F4,0x8563,0x8A62,0x8AC4,0x9187,0x931E,0x9806,
	0x99B4,0x620C,0x8853,0x8FF0,0x9265,0x5D07,0x5D27,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 100, Array index 0x3F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D69,0x745F,0x819D,0x8768,0x6FD5,0x62FE,0x7FD2,
	0x8936,0x8972,0x4E1E,0x4E58,0x50E7,0x52DD,0x5347,0x627F,
	0x6607,0x7E69,0x8805,0x965E,0x4F8D,0x5319,0x5636,0x59CB,
	0x5AA4,0x5C38,0x5C4E,0x5C4D,0x5E02,0x5F11,0x6043,0x65BD,
	0x662F,0x6642,0x67BE,0x67F4,0x731C,0x77E2,0x793A,0x7FC5,
	0x8494,0x84CD,0x8996,0x8A66,0x8A69,0x8AE1,0x8C55,0x8C7A,
	0x57F4,0x5BD4,0x5F0F,0x606F,0x62ED,0x690D,0x6B96,0x6E5C,
	0x7184,0x7BD2,0x8755,0x8B58,0x8EFE,0x98DF,0x98FE,0x4F38,
	0x4F81,0x4FE1,0x547B,0x5A20,0x5BB8,0x613C,0x65B0,0x6668,
	0x71FC,0x7533,0x795E,0x7D33,0x814E,0x81E3,0x8398,0x85AA,
	0x85CE,0x8703,0x8A0A,0x8EAB,0x8F9B,0xF971,0x8FC5,0x5931,
	0x5BA4,0x5BE6,0x6089,0x5BE9,0x5C0B,0x5FC3,0x6C81,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 101, Array index 0x4000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xF972,0x6DF1,0x700B,0x751A,0x82AF,0x8AF6,0x4EC0,
	0x5341,0xF973,0x96D9,0x6C0F,0x4E9E,0x4FC4,0x5152,0x555E,
	0x5A25,0x5CE8,0x6211,0x7259,0x82BD,0x83AA,0x86FE,0x8859,
	0x8A1D,0x963F,0x96C5,0x9913,0x9D09,0x9D5D,0x580A,0x5CB3,
	0x5DBD,0x5E44,0x60E1,0x6115,0x63E1,0x6A02,0x6E25,0x9102,
	0x9354,0x984E,0x9C10,0x9F77,0x5B89,0x5CB8,0x6309,0x664F,
	0x6848,0x773C,0x96C1,0x978D,0x9854,0x9B9F,0x65A1,0x8B01,
	0x8ECB,0x95BC,0x5535,0x5CA9,0x5DD6,0x5EB5,0x6697,0x764C,
	0x83F4,0x95C7,0x58D3,0x62BC,0x72CE,0x9D28,0x4EF0,0x592E,
	0x600F,0x663B,0x6B83,0x79E7,0x9D26,0x5393,0x54C0,0x57C3,
	0x5D16,0x611B,0x66D6,0x6DAF,0x788D,0x827E,0x9698,0x9744,
	0x5384,0x627C,0x6396,0x6DB2,0x7E0A,0x814B,0x984D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 102, Array index 0x4100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6AFB,0x7F4C,0x9DAF,0x9E1A,0x4E5F,0x503B,0x51B6,
	0x591C,0x60F9,0x63F6,0x6930,0x723A,0x8036,0xF974,0x91CE,
	0x5F31,0xF975,0xF976,0x7D04,0x82E5,0x846F,0x84BB,0x85E5,
	0x8E8D,0xF977,0x4F6F,0xF978,0xF979,0x58E4,0x5B43,0x6059,
	0x63DA,0x6518,0x656D,0x6698,0xF97A,0x694A,0x6A23,0x6D0B,
	0x7001,0x716C,0x75D2,0x760D,0x79B3,0x7A70,0xF97B,0x7F8A,
	0xF97C,0x8944,0xF97D,0x8B93,0x91C0,0x967D,0xF97E,0x990A,
	0x5704,0x5FA1,0x65BC,0x6F01,0x7600,0x79A6,0x8A9E,0x99AD,
	0x9B5A,0x9F6C,0x5104,0x61B6,0x6291,0x6A8D,0x81C6,0x5043,
	0x5830,0x5F66,0x7109,0x8A00,0x8AFA,0x5B7C,0x8616,0x4FFA,
	0x513C,0x56B4,0x5944,0x63A9,0x6DF9,0x5DAA,0x696D,0x5186,
	0x4E88,0x4F59,0xF97F,0xF980,0xF981,0x5982,0xF982,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 103, Array index 0x4200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xF983,0x6B5F,0x6C5D,0xF984,0x74B5,0x7916,0xF985,
	0x8207,0x8245,0x8339,0x8F3F,0x8F5D,0xF986,0x9918,0xF987,
	0xF988,0xF989,0x4EA6,0xF98A,0x57DF,0x5F79,0x6613,0xF98B,
	0xF98C,0x75AB,0x7E79,0x8B6F,0xF98D,0x9006,0x9A5B,0x56A5,
	0x5827,0x59F8,0x5A1F,0x5BB4,0xF98E,0x5EF6,0xF98F,0xF990,
	0x6350,0x633B,0xF991,0x693D,0x6C87,0x6CBF,0x6D8E,0x6D93,
	0x6DF5,0x6F14,0xF992,0x70DF,0x7136,0x7159,0xF993,0x71C3,
	0x71D5,0xF994,0x784F,0x786F,0xF995,0x7B75,0x7DE3,0xF996,
	0x7E2F,0xF997,0x884D,0x8EDF,0xF998,0xF999,0xF99A,0x925B,
	0xF99B,0x9CF6,0xF99C,0xF99D,0xF99E,0x6085,0x6D85,0xF99F,
	0x71B1,0xF9A0,0xF9A1,0x95B1,0x53AD,0xF9A2,0xF9A3,0xF9A4,
	0x67D3,0xF9A5,0x708E,0x7130,0x7430,0x8276,0x82D2,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 104, Array index 0x4300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xF9A6,0x95BB,0x9AE5,0x9E7D,0x66C4,0xF9A7,0x71C1,
	0x8449,0xF9A8,0xF9A9,0x584B,0xF9AA,0xF9AB,0x5DB8,0x5F71,
	0xF9AC,0x6620,0x668E,0x6979,0x69AE,0x6C38,0x6CF3,0x6E36,
	0x6F41,0x6FDA,0x701B,0x702F,0x7150,0x71DF,0x7370,0xF9AD,
	0x745B,0xF9AE,0x74D4,0x76C8,0x7A4E,0x7E93,0xF9AF,0xF9B0,
	0x82F1,0x8A60,0x8FCE,0xF9B1,0x9348,0xF9B2,0x9719,0xF9B3,
	0xF9B4,0x4E42,0x502A,0xF9B5,0x5208,0x53E1,0x66F3,0x6C6D,
	0x6FCA,0x730A,0x777F,0x7A62,0x82AE,0x85DD,0x8602,0xF9B6,
	0x88D4,0x8A63,0x8B7D,0x8C6B,0xF9B7,0x92B3,0xF9B8,0x9713,
	0x9810,0x4E94,0x4F0D,0x4FC9,0x50B2,0x5348,0x543E,0x5433,
	0x55DA,0x5862,0x58BA,0x5967,0x5A1B,0x5BE4,0x609F,0xF9B9,
	0x61CA,0x6556,0x65FF,0x6664,0x68A7,0x6C5A,0x6FB3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 105, Array index 0x4400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x70CF,0x71AC,0x7352,0x7B7D,0x8708,0x8AA4,0x9C32,
	0x9F07,0x5C4B,0x6C83,0x7344,0x7389,0x923A,0x6EAB,0x7465,
	0x761F,0x7A69,0x7E15,0x860A,0x5140,0x58C5,0x64C1,0x74EE,
	0x7515,0x7670,0x7FC1,0x9095,0x96CD,0x9954,0x6E26,0x74E6,
	0x7AA9,0x7AAA,0x81E5,0x86D9,0x8778,0x8A1B,0x5A49,0x5B8C,
	0x5B9B,0x68A1,0x6900,0x6D63,0x73A9,0x7413,0x742C,0x7897,
	0x7DE9,0x7FEB,0x8118,0x8155,0x839E,0x8C4C,0x962E,0x9811,
	0x66F0,0x5F80,0x65FA,0x6789,0x6C6A,0x738B,0x502D,0x5A03,
	0x6B6A,0x77EE,0x5916,0x5D6C,0x5DCD,0x7325,0x754F,0xF9BA,
	0xF9BB,0x50E5,0x51F9,0x582F,0x592D,0x5996,0x59DA,0x5BE5,
	0xF9BC,0xF9BD,0x5DA2,0x62D7,0x6416,0x6493,0x64FE,0xF9BE,
	0x66DC,0xF9BF,0x6A48,0xF9C0,0x71FF,0x7464,0xF9C1,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 106, Array index 0x4500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7A88,0x7AAF,0x7E47,0x7E5E,0x8000,0x8170,0xF9C2,
	0x87EF,0x8981,0x8B20,0x9059,0xF9C3,0x9080,0x9952,0x617E,
	0x6B32,0x6D74,0x7E1F,0x8925,0x8FB1,0x4FD1,0x50AD,0x5197,
	0x52C7,0x57C7,0x5889,0x5BB9,0x5EB8,0x6142,0x6995,0x6D8C,
	0x6E67,0x6EB6,0x7194,0x7462,0x7528,0x752C,0x8073,0x8338,
	0x84C9,0x8E0A,0x9394,0x93DE,0xF9C4,0x4E8E,0x4F51,0x5076,
	0x512A,0x53C8,0x53CB,0x53F3,0x5B87,0x5BD3,0x5C24,0x611A,
	0x6182,0x65F4,0x725B,0x7397,0x7440,0x76C2,0x7950,0x7991,
	0x79B9,0x7D06,0x7FBD,0x828B,0x85D5,0x865E,0x8FC2,0x9047,
	0x90F5,0x91EA,0x9685,0x96E8,0x96E9,0x52D6,0x5F67,0x65ED,
	0x6631,0x682F,0x715C,0x7A36,0x90C1,0x980A,0x4E91,0xF9C5,
	0x6A52,0x6B9E,0x6F90,0x7189,0x8018,0x82B8,0x8553,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 107, Array index 0x4600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x904B,0x9695,0x96F2,0x97FB,0x851A,0x9B31,0x4E90,
	0x718A,0x96C4,0x5143,0x539F,0x54E1,0x5713,0x5712,0x57A3,
	0x5A9B,0x5AC4,0x5BC3,0x6028,0x613F,0x63F4,0x6C85,0x6D39,
	0x6E72,0x6E90,0x7230,0x733F,0x7457,0x82D1,0x8881,0x8F45,
	0x9060,0xF9C6,0x9662,0x9858,0x9D1B,0x6708,0x8D8A,0x925E,
	0x4F4D,0x5049,0x50DE,0x5371,0x570D,0x59D4,0x5A01,0x5C09,
	0x6170,0x6690,0x6E2D,0x7232,0x744B,0x7DEF,0x80C3,0x840E,
	0x8466,0x853F,0x875F,0x885B,0x8918,0x8B02,0x9055,0x97CB,
	0x9B4F,0x4E73,0x4F91,0x5112,0x516A,0xF9C7,0x552F,0x55A9,
	0x5B7A,0x5BA5,0x5E7C,0x5E7D,0x5EBE,0x60A0,0x60DF,0x6108,
	0x6109,0x63C4,0x6538,0x6709,0xF9C8,0x67D4,0x67DA,0xF9C9,
	0x6961,0x6962,0x6CB9,0x6D27,0xF9CA,0x6E38,0xF9CB,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 108, Array index 0x4700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6FE1,0x7336,0x7337,0xF9CC,0x745C,0x7531,0xF9CD,
	0x7652,0xF9CE,0xF9CF,0x7DAD,0x81FE,0x8438,0x88D5,0x8A98,
	0x8ADB,0x8AED,0x8E30,0x8E42,0x904A,0x903E,0x907A,0x9149,
	0x91C9,0x936E,0xF9D0,0xF9D1,0x5809,0xF9D2,0x6BD3,0x8089,
	0x80B2,0xF9D3,0xF9D4,0x5141,0x596B,0x5C39,0xF9D5,0xF9D6,
	0x6F64,0x73A7,0x80E4,0x8D07,0xF9D7,0x9217,0x958F,0xF9D8,
	0xF9D9,0xF9DA,0xF9DB,0x807F,0x620E,0x701C,0x7D68,0x878D,
	0xF9DC,0x57A0,0x6069,0x6147,0x6BB7,0x8ABE,0x9280,0x96B1,
	0x4E59,0x541F,0x6DEB,0x852D,0x9670,0x97F3,0x98EE,0x63D6,
	0x6CE3,0x9091,0x51DD,0x61C9,0x81BA,0x9DF9,0x4F9D,0x501A,
	0x5100,0x5B9C,0x610F,0x61FF,0x64EC,0x6905,0x6BC5,0x7591,
	0x77E3,0x7FA9,0x8264,0x858F,0x87FB,0x8863,0x8ABC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 109, Array index 0x4800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8B70,0x91AB,0x4E8C,0x4EE5,0x4F0A,0xF9DD,0xF9DE,
	0x5937,0x59E8,0xF9DF,0x5DF2,0x5F1B,0x5F5B,0x6021,0xF9E0,
	0xF9E1,0xF9E2,0xF9E3,0x723E,0x73E5,0xF9E4,0x7570,0x75CD,
	0xF9E5,0x79FB,0xF9E6,0x800C,0x8033,0x8084,0x82E1,0x8351,
	0xF9E7,0xF9E8,0x8CBD,0x8CB3,0x9087,0xF9E9,0xF9EA,0x98F4,
	0x990C,0xF9EB,0xF9EC,0x7037,0x76CA,0x7FCA,0x7FCC,0x7FFC,
	0x8B1A,0x4EBA,0x4EC1,0x5203,0x5370,0xF9ED,0x54BD,0x56E0,
	0x59FB,0x5BC5,0x5F15,0x5FCD,0x6E6E,0xF9EE,0xF9EF,0x7D6A,
	0x8335,0xF9F0,0x8693,0x8A8D,0xF9F1,0x976D,0x9777,0xF9F2,
	0xF9F3,0x4E00,0x4F5A,0x4F7E,0x58F9,0x65E5,0x6EA2,0x9038,
	0x93B0,0x99B9,0x4EFB,0x58EC,0x598A,0x59D9,0x6041,0xF9F4,
	0xF9F5,0x7A14,0xF9F6,0x834F,0x8CC3,0x5165,0x5344,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 110, Array index 0x4900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xF9F7,0xF9F8,0xF9F9,0x4ECD,0x5269,0x5B55,0x82BF,
	0x4ED4,0x523A,0x54A8,0x59C9,0x59FF,0x5B50,0x5B57,0x5B5C,
	0x6063,0x6148,0x6ECB,0x7099,0x716E,0x7386,0x74F7,0x75B5,
	0x78C1,0x7D2B,0x8005,0x81EA,0x8328,0x8517,0x85C9,0x8AEE,
	0x8CC7,0x96CC,0x4F5C,0x52FA,0x56BC,0x65AB,0x6628,0x707C,
	0x70B8,0x7235,0x7DBD,0x828D,0x914C,0x96C0,0x9D72,0x5B71,
	0x68E7,0x6B98,0x6F7A,0x76DE,0x5C91,0x66AB,0x6F5B,0x7BB4,
	0x7C2A,0x8836,0x96DC,0x4E08,0x4ED7,0x5320,0x5834,0x58BB,
	0x58EF,0x596C,0x5C07,0x5E33,0x5E84,0x5F35,0x638C,0x66B2,
	0x6756,0x6A1F,0x6AA3,0x6B0C,0x6F3F,0x7246,0xF9FA,0x7350,
	0x748B,0x7AE0,0x7CA7,0x8178,0x81DF,0x81E7,0x838A,0x846C,
	0x8523,0x8594,0x85CF,0x88DD,0x8D13,0x91AC,0x9577,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 111, Array index 0x4A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x969C,0x518D,0x54C9,0x5728,0x5BB0,0x624D,0x6750,
	0x683D,0x6893,0x6E3D,0x6ED3,0x707D,0x7E21,0x88C1,0x8CA1,
	0x8F09,0x9F4B,0x9F4E,0x722D,0x7B8F,0x8ACD,0x931A,0x4F47,
	0x4F4E,0x5132,0x5480,0x59D0,0x5E95,0x62B5,0x6775,0x696E,
	0x6A17,0x6CAE,0x6E1A,0x72D9,0x732A,0x75BD,0x7BB8,0x7D35,
	0x82E7,0x83F9,0x8457,0x85F7,0x8A5B,0x8CAF,0x8E87,0x9019,
	0x90B8,0x96CE,0x9F5F,0x52E3,0x540A,0x5AE1,0x5BC2,0x6458,
	0x6575,0x6EF4,0x72C4,0xF9FB,0x7684,0x7A4D,0x7B1B,0x7C4D,
	0x7E3E,0x7FDF,0x837B,0x8B2B,0x8CCA,0x8D64,0x8DE1,0x8E5F,
	0x8FEA,0x8FF9,0x9069,0x93D1,0x4F43,0x4F7A,0x50B3,0x5168,
	0x5178,0x524D,0x526A,0x5861,0x587C,0x5960,0x5C08,0x5C55,
	0x5EDB,0x609B,0x6230,0x6813,0x6BBF,0x6C08,0x6FB1,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 112, Array index 0x4B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x714E,0x7420,0x7530,0x7538,0x7551,0x7672,0x7B4C,
	0x7B8B,0x7BAD,0x7BC6,0x7E8F,0x8A6E,0x8F3E,0x8F49,0x923F,
	0x9293,0x9322,0x942B,0x96FB,0x985A,0x986B,0x991E,0x5207,
	0x622A,0x6298,0x6D59,0x7664,0x7ACA,0x7BC0,0x7D76,0x5360,
	0x5CBE,0x5E97,0x6F38,0x70B9,0x7C98,0x9711,0x9B8E,0x9EDE,
	0x63A5,0x647A,0x8776,0x4E01,0x4E95,0x4EAD,0x505C,0x5075,
	0x5448,0x59C3,0x5B9A,0x5E40,0x5EAD,0x5EF7,0x5F81,0x60C5,
	0x633A,0x653F,0x6574,0x65CC,0x6676,0x6678,0x67FE,0x6968,
	0x6A89,0x6B63,0x6C40,0x6DC0,0x6DE8,0x6E1F,0x6E5E,0x701E,
	0x70A1,0x738E,0x73FD,0x753A,0x775B,0x7887,0x798E,0x7A0B,
	0x7A7D,0x7CBE,0x7D8E,0x8247,0x8A02,0x8AEA,0x8C9E,0x912D,
	0x914A,0x91D8,0x9266,0x92CC,0x9320,0x9706,0x9756,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 113, Array index 0x4C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x975C,0x9802,0x9F0E,0x5236,0x5291,0x557C,0x5824,
	0x5E1D,0x5F1F,0x608C,0x63D0,0x68AF,0x6FDF,0x796D,0x7B2C,
	0x81CD,0x85BA,0x88FD,0x8AF8,0x8E44,0x918D,0x9664,0x969B,
	0x973D,0x984C,0x9F4A,0x4FCE,0x5146,0x51CB,0x52A9,0x5632,
	0x5F14,0x5F6B,0x63AA,0x64CD,0x65E9,0x6641,0x66FA,0x66F9,
	0x671D,0x689D,0x68D7,0x69FD,0x6F15,0x6F6E,0x7167,0x71E5,
	0x722A,0x74AA,0x773A,0x7956,0x795A,0x79DF,0x7A20,0x7A95,
	0x7C97,0x7CDF,0x7D44,0x7E70,0x8087,0x85FB,0x86A4,0x8A54,
	0x8ABF,0x8D99,0x8E81,0x9020,0x906D,0x91E3,0x963B,0x96D5,
	0x9CE5,0x65CF,0x7C07,0x8DB3,0x93C3,0x5B58,0x5C0A,0x5352,
	0x62D9,0x731D,0x5027,0x5B97,0x5F9E,0x60B0,0x616B,0x68D5,
	0x6DD9,0x742E,0x7A2E,0x7D42,0x7D9C,0x7E31,0x816B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 114, Array index 0x4D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8E2A,0x8E35,0x937E,0x9418,0x4F50,0x5750,0x5DE6,
	0x5EA7,0x632B,0x7F6A,0x4E3B,0x4F4F,0x4F8F,0x505A,0x59DD,
	0x80C4,0x546A,0x5468,0x55FE,0x594F,0x5B99,0x5DDE,0x5EDA,
	0x665D,0x6731,0x67F1,0x682A,0x6CE8,0x6D32,0x6E4A,0x6F8D,
	0x70B7,0x73E0,0x7587,0x7C4C,0x7D02,0x7D2C,0x7DA2,0x821F,
	0x86DB,0x8A3B,0x8A85,0x8D70,0x8E8A,0x8F33,0x9031,0x914E,
	0x9152,0x9444,0x99D0,0x7AF9,0x7CA5,0x4FCA,0x5101,0x51C6,
	0x57C8,0x5BEF,0x5CFB,0x6659,0x6A3D,0x6D5A,0x6E96,0x6FEC,
	0x710C,0x756F,0x7AE3,0x8822,0x9021,0x9075,0x96CB,0x99FF,
	0x8301,0x4E2D,0x4EF2,0x8846,0x91CD,0x537D,0x6ADB,0x696B,
	0x6C41,0x847A,0x589E,0x618E,0x66FE,0x62EF,0x70DD,0x7511,
	0x75C7,0x7E52,0x84B8,0x8B49,0x8D08,0x4E4B,0x53EA,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 115, Array index 0x4E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x54AB,0x5730,0x5740,0x5FD7,0x6301,0x6307,0x646F,
	0x652F,0x65E8,0x667A,0x679D,0x67B3,0x6B62,0x6C60,0x6C9A,
	0x6F2C,0x77E5,0x7825,0x7949,0x7957,0x7D19,0x80A2,0x8102,
	0x81F3,0x829D,0x82B7,0x8718,0x8A8C,0xF9FC,0x8D04,0x8DBE,
	0x9072,0x76F4,0x7A19,0x7A37,0x7E54,0x8077,0x5507,0x55D4,
	0x5875,0x632F,0x6422,0x6649,0x664B,0x686D,0x699B,0x6B84,
	0x6D25,0x6EB1,0x73CD,0x7468,0x74A1,0x755B,0x75B9,0x76E1,
	0x771E,0x778B,0x79E6,0x7E09,0x7E1D,0x81FB,0x852F,0x8897,
	0x8A3A,0x8CD1,0x8EEB,0x8FB0,0x9032,0x93AD,0x9663,0x9673,
	0x9707,0x4F84,0x53F1,0x59EA,0x5AC9,0x5E19,0x684E,0x74C6,
	0x75BE,0x79E9,0x7A92,0x81A3,0x86ED,0x8CEA,0x8DCC,0x8FED,
	0x659F,0x6715,0xF9FD,0x57F7,0x6F57,0x7DDD,0x8F2F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 116, Array index 0x4F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x93F6,0x96C6,0x5FB5,0x61F2,0x6F84,0x4E14,0x4F98,
	0x501F,0x53C9,0x55DF,0x5D6F,0x5DEE,0x6B21,0x6B64,0x78CB,
	0x7B9A,0xF9FE,0x8E49,0x8ECA,0x906E,0x6349,0x643E,0x7740,
	0x7A84,0x932F,0x947F,0x9F6A,0x64B0,0x6FAF,0x71E6,0x74A8,
	0x74DA,0x7AC4,0x7C12,0x7E82,0x7CB2,0x7E98,0x8B9A,0x8D0A,
	0x947D,0x9910,0x994C,0x5239,0x5BDF,0x64E6,0x672D,0x7D2E,
	0x50ED,0x53C3,0x5879,0x6158,0x6159,0x61FA,0x65AC,0x7AD9,
	0x8B92,0x8B96,0x5009,0x5021,0x5275,0x5531,0x5A3C,0x5EE0,
	0x5F70,0x6134,0x655E,0x660C,0x6636,0x66A2,0x69CD,0x6EC4,
	0x6F32,0x7316,0x7621,0x7A93,0x8139,0x8259,0x83D6,0x84BC,
	0x50B5,0x57F0,0x5BC0,0x5BE8,0x5F69,0x63A1,0x7826,0x7DB5,
	0x83DC,0x8521,0x91C7,0x91F5,0x518A,0x67F5,0x7B56,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 117, Array index 0x5000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8CAC,0x51C4,0x59BB,0x60BD,0x8655,0x501C,0xF9FF,
	0x5254,0x5C3A,0x617D,0x621A,0x62D3,0x64F2,0x65A5,0x6ECC,
	0x7620,0x810A,0x8E60,0x965F,0x96BB,0x4EDF,0x5343,0x5598,
	0x5929,0x5DDD,0x64C5,0x6CC9,0x6DFA,0x7394,0x7A7F,0x821B,
	0x85A6,0x8CE4,0x8E10,0x9077,0x91E7,0x95E1,0x9621,0x97C6,
	0x51F8,0x54F2,0x5586,0x5FB9,0x64A4,0x6F88,0x7DB4,0x8F1F,
	0x8F4D,0x9435,0x50C9,0x5C16,0x6CBE,0x6DFB,0x751B,0x77BB,
	0x7C3D,0x7C64,0x8A79,0x8AC2,0x581E,0x59BE,0x5E16,0x6377,
	0x7252,0x758A,0x776B,0x8ADC,0x8CBC,0x8F12,0x5EF3,0x6674,
	0x6DF8,0x807D,0x83C1,0x8ACB,0x9751,0x9BD6,0xFA00,0x5243,
	0x66FF,0x6D95,0x6EEF,0x7DE0,0x8AE6,0x902E,0x905E,0x9AD4,
	0x521D,0x527F,0x54E8,0x6194,0x6284,0x62DB,0x68A2,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 118, Array index 0x5100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6912,0x695A,0x6A35,0x7092,0x7126,0x785D,0x7901,
	0x790E,0x79D2,0x7A0D,0x8096,0x8278,0x82D5,0x8349,0x8549,
	0x8C82,0x8D85,0x9162,0x918B,0x91AE,0x4FC3,0x56D1,0x71ED,
	0x77D7,0x8700,0x89F8,0x5BF8,0x5FD6,0x6751,0x90A8,0x53E2,
	0x585A,0x5BF5,0x60A4,0x6181,0x6460,0x7E3D,0x8070,0x8525,
	0x9283,0x64AE,0x50AC,0x5D14,0x6700,0x589C,0x62BD,0x63A8,
	0x690E,0x6978,0x6A1E,0x6E6B,0x76BA,0x79CB,0x82BB,0x8429,
	0x8ACF,0x8DA8,0x8FFD,0x9112,0x914B,0x919C,0x9310,0x9318,
	0x939A,0x96DB,0x9A36,0x9C0D,0x4E11,0x755C,0x795D,0x7AFA,
	0x7B51,0x7BC9,0x7E2E,0x84C4,0x8E59,0x8E74,0x8EF8,0x9010,
	0x6625,0x693F,0x7443,0x51FA,0x672E,0x9EDC,0x5145,0x5FE0,
	0x6C96,0x87F2,0x885D,0x8877,0x60B4,0x81B5,0x8403,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 119, Array index 0x5200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8D05,0x53D6,0x5439,0x5634,0x5A36,0x5C31,0x708A,
	0x7FE0,0x805A,0x8106,0x81ED,0x8DA3,0x9189,0x9A5F,0x9DF2,
	0x5074,0x4EC4,0x53A0,0x60FB,0x6E2C,0x5C64,0x4F88,0x5024,
	0x55E4,0x5CD9,0x5E5F,0x6065,0x6894,0x6CBB,0x6DC4,0x71BE,
	0x75D4,0x75F4,0x7661,0x7A1A,0x7A49,0x7DC7,0x7DFB,0x7F6E,
	0x81F4,0x86A9,0x8F1C,0x96C9,0x99B3,0x9F52,0x5247,0x52C5,
	0x98ED,0x89AA,0x4E03,0x67D2,0x6F06,0x4FB5,0x5BE2,0x6795,
	0x6C88,0x6D78,0x741B,0x7827,0x91DD,0x937C,0x87C4,0x79E4,
	0x7A31,0x5FEB,0x4ED6,0x54A4,0x553E,0x58AE,0x59A5,0x60F0,
	0x6253,0x62D6,0x6736,0x6955,0x8235,0x9640,0x99B1,0x99DD,
	0x502C,0x5353,0x5544,0x577C,0xFA01,0x6258,0xFA02,0x64E2,
	0x666B,0x67DD,0x6FC1,0x6FEF,0x7422,0x7438,0x8A17,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 120, Array index 0x5300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9438,0x5451,0x5606,0x5766,0x5F48,0x619A,0x6B4E,
	0x7058,0x70AD,0x7DBB,0x8A95,0x596A,0x812B,0x63A2,0x7708,
	0x803D,0x8CAA,0x5854,0x642D,0x69BB,0x5B95,0x5E11,0x6E6F,
	0xFA03,0x8569,0x514C,0x53F0,0x592A,0x6020,0x614B,0x6B86,
	0x6C70,0x6CF0,0x7B1E,0x80CE,0x82D4,0x8DC6,0x90B0,0x98B1,
	0xFA04,0x64C7,0x6FA4,0x6491,0x6504,0x514E,0x5410,0x571F,
	0x8A0E,0x615F,0x6876,0xFA05,0x75DB,0x7B52,0x7D71,0x901A,
	0x5806,0x69CC,0x817F,0x892A,0x9000,0x9839,0x5078,0x5957,
	0x59AC,0x6295,0x900F,0x9B2A,0x615D,0x7279,0x95D6,0x5761,
	0x5A46,0x5DF4,0x628A,0x64AD,0x64FA,0x6777,0x6CE2,0x6D3E,
	0x722C,0x7436,0x7834,0x7F77,0x82AD,0x8DDB,0x9817,0x5224,
	0x5742,0x677F,0x7248,0x74E3,0x8CA9,0x8FA6,0x9211,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 121, Array index 0x5400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x962A,0x516B,0x53ED,0x634C,0x4F69,0x5504,0x6096,
	0x6557,0x6C9B,0x6D7F,0x724C,0x72FD,0x7A17,0x8987,0x8C9D,
	0x5F6D,0x6F8E,0x70F9,0x81A8,0x610E,0x4FBF,0x504F,0x6241,
	0x7247,0x7BC7,0x7DE8,0x7FE9,0x904D,0x97AD,0x9A19,0x8CB6,
	0x576A,0x5E73,0x67B0,0x840D,0x8A55,0x5420,0x5B16,0x5E63,
	0x5EE2,0x5F0A,0x6583,0x80BA,0x853D,0x9589,0x965B,0x4F48,
	0x5305,0x530D,0x530F,0x5486,0x54FA,0x5703,0x5E03,0x6016,
	0x629B,0x62B1,0x6355,0xFA06,0x6CE1,0x6D66,0x75B1,0x7832,
	0x80DE,0x812F,0x82DE,0x8461,0x84B2,0x888D,0x8912,0x900B,
	0x92EA,0x98FD,0x9B91,0x5E45,0x66B4,0x66DD,0x7011,0x7206,
	0xFA07,0x4FF5,0x527D,0x5F6A,0x6153,0x6753,0x6A19,0x6F02,
	0x74E2,0x7968,0x8868,0x8C79,0x98C7,0x98C4,0x9A43,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 122, Array index 0x5500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x54C1,0x7A1F,0x6953,0x8AF7,0x8C4A,0x98A8,0x99AE,
	0x5F7C,0x62AB,0x75B2,0x76AE,0x88AB,0x907F,0x9642,0x5339,
	0x5F3C,0x5FC5,0x6CCC,0x73CC,0x7562,0x758B,0x7B46,0x82FE,
	0x999D,0x4E4F,0x903C,0x4E0B,0x4F55,0x53A6,0x590F,0x5EC8,
	0x6630,0x6CB3,0x7455,0x8377,0x8766,0x8CC0,0x9050,0x971E,
	0x9C15,0x58D1,0x5B78,0x8650,0x8B14,0x9DB4,0x5BD2,0x6068,
	0x608D,0x65F1,0x6C57,0x6F22,0x6FA3,0x701A,0x7F55,0x7FF0,
	0x9591,0x9592,0x9650,0x97D3,0x5272,0x8F44,0x51FD,0x542B,
	0x54B8,0x5563,0x558A,0x6ABB,0x6DB5,0x7DD8,0x8266,0x929C,
	0x9677,0x9E79,0x5408,0x54C8,0x76D2,0x86E4,0x95A4,0x95D4,
	0x965C,0x4EA2,0x4F09,0x59EE,0x5AE6,0x5DF7,0x6052,0x6297,
	0x676D,0x6841,0x6C86,0x6E2F,0x7F38,0x809B,0x822A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 123, Array index 0x5600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xFA08,0xFA09,0x9805,0x4EA5,0x5055,0x54B3,0x5793,
	0x595A,0x5B69,0x5BB3,0x61C8,0x6977,0x6D77,0x7023,0x87F9,
	0x89E3,0x8A72,0x8AE7,0x9082,0x99ED,0x9AB8,0x52BE,0x6838,
	0x5016,0x5E78,0x674F,0x8347,0x884C,0x4EAB,0x5411,0x56AE,
	0x73E6,0x9115,0x97FF,0x9909,0x9957,0x9999,0x5653,0x589F,
	0x865B,0x8A31,0x61B2,0x6AF6,0x737B,0x8ED2,0x6B47,0x96AA,
	0x9A57,0x5955,0x7200,0x8D6B,0x9769,0x4FD4,0x5CF4,0x5F26,
	0x61F8,0x665B,0x6CEB,0x70AB,0x7384,0x73B9,0x73FE,0x7729,
	0x774D,0x7D43,0x7D62,0x7E23,0x8237,0x8852,0xFA0A,0x8CE2,
	0x9249,0x986F,0x5B51,0x7A74,0x8840,0x9801,0x5ACC,0x4FE0,
	0x5354,0x593E,0x5CFD,0x633E,0x6D79,0x72F9,0x8105,0x8107,
	0x83A2,0x92CF,0x9830,0x4EA8,0x5144,0x5211,0x578B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 124, Array index 0x5700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F62,0x6CC2,0x6ECE,0x7005,0x7050,0x70AF,0x7192,
	0x73E9,0x7469,0x834A,0x87A2,0x8861,0x9008,0x90A2,0x93A3,
	0x99A8,0x516E,0x5F57,0x60E0,0x6167,0x66B3,0x8559,0x8E4A,
	0x91AF,0x978B,0x4E4E,0x4E92,0x547C,0x58D5,0x58FA,0x597D,
	0x5CB5,0x5F27,0x6236,0x6248,0x660A,0x6667,0x6BEB,0x6D69,
	0x6DCF,0x6E56,0x6EF8,0x6F94,0x6FE0,0x6FE9,0x705D,0x72D0,
	0x7425,0x745A,0x74E0,0x7693,0x795C,0x7CCA,0x7E1E,0x80E1,
	0x82A6,0x846B,0x84BF,0x864E,0x865F,0x8774,0x8B77,0x8C6A,
	0x93AC,0x9800,0x9865,0x60D1,0x6216,0x9177,0x5A5A,0x660F,
	0x6DF7,0x6E3E,0x743F,0x9B42,0x5FFD,0x60DA,0x7B0F,0x54C4,
	0x5F18,0x6C5E,0x6CD3,0x6D2A,0x70D8,0x7D05,0x8679,0x8A0C,
	0x9D3B,0x5316,0x548C,0x5B05,0x6A3A,0x706B,0x7575,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 125, Array index 0x5800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x798D,0x79BE,0x82B1,0x83EF,0x8A71,0x8B41,0x8CA8,
	0x9774,0xFA0B,0x64F4,0x652B,0x78BA,0x78BB,0x7A6B,0x4E38,
	0x559A,0x5950,0x5BA6,0x5E7B,0x60A3,0x63DB,0x6B61,0x6665,
	0x6853,0x6E19,0x7165,0x74B0,0x7D08,0x9084,0x9A69,0x9C25,
	0x6D3B,0x6ED1,0x733E,0x8C41,0x95CA,0x51F0,0x5E4C,0x5FA8,
	0x604D,0x60F6,0x6130,0x614C,0x6643,0x6644,0x69A5,0x6CC1,
	0x6E5F,0x6EC9,0x6F62,0x714C,0x749C,0x7687,0x7BC1,0x7C27,
	0x8352,0x8757,0x9051,0x968D,0x9EC3,0x532F,0x56DE,0x5EFB,
	0x5F8A,0x6062,0x6094,0x61F7,0x6666,0x6703,0x6A9C,0x6DEE,
	0x6FAE,0x7070,0x736A,0x7E6A,0x81BE,0x8334,0x86D4,0x8AA8,
	0x8CC4,0x5283,0x7372,0x5B96,0x6A6B,0x9404,0x54EE,0x5686,
	0x5B5D,0x6548,0x6585,0x66C9,0x689F,0x6D8D,0x6DC6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 126, Array index 0x5900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x723B,0x80B4,0x9175,0x9A4D,0x4FAF,0x5019,0x539A,
	0x540E,0x543C,0x5589,0x55C5,0x5E3F,0x5F8C,0x673D,0x7166,
	0x73DD,0x9005,0x52DB,0x52F3,0x5864,0x58CE,0x7104,0x718F,
	0x71FB,0x85B0,0x8A13,0x6688,0x85A8,0x55A7,0x6684,0x714A,
	0x8431,0x5349,0x5599,0x6BC1,0x5F59,0x5FBD,0x63EE,0x6689,
	0x7147,0x8AF1,0x8F1D,0x9EBE,0x4F11,0x643A,0x70CB,0x7566,
	0x8667,0x6064,0x8B4E,0x9DF8,0x5147,0x51F6,0x5308,0x6D36,
	0x80F8,0x9ED1,0x6615,0x6B23,0x7098,0x75D5,0x5403,0x5C79,
	0x7D07,0x8A16,0x6B20,0x6B3D,0x6B46,0x5438,0x6070,0x6D3D,
	0x7FD5,0x8208,0x50D6,0x51DE,0x559C,0x566B,0x56CD,0x59EC,
	0x5B09,0x5E0C,0x6199,0x6198,0x6231,0x665E,0x66E6,0x7199,
	0x71B9,0x71BA,0x72A7,0x79A7,0x7A00,0x7FB2,0x8A70,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
};

#endif /* ICONV_TO_UCS_CCS_KSX1001 && !defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit ksx1001 -> UCS size-optimized table (17018 bytes).
 * ======================================================================
 */
#if defined (ICONV_TO_UCS_CCS_KSX1001) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
to_ucs_size_ksx1001[] =
{
	0x005D, /* Ranges number */
	0x0000, /* Unranged codes number */
	0x213D, /* First unranged code index */
	/* Ranges list: first code, last Code, array index. */
	/* Array index: 0x0003 */ 0x2121, 0x217E, 0x011A,
	/* Array index: 0x0006 */ 0x2221, 0x2265, 0x0178,
	/* Array index: 0x0009 */ 0x2321, 0x237E, 0x01BD,
	/* Array index: 0x000C */ 0x2421, 0x247E, 0x021B,
	/* Array index: 0x000F */ 0x2521, 0x252A, 0x0279,
	/* Array index: 0x0012 */ 0x2530, 0x2539, 0x0283,
	/* Array index: 0x0015 */ 0x2541, 0x2558, 0x028D,
	/* Array index: 0x0018 */ 0x2561, 0x2578, 0x02A5,
	/* Array index: 0x001B */ 0x2621, 0x2664, 0x02BD,
	/* Array index: 0x001E */ 0x2721, 0x276F, 0x0301,
	/* Array index: 0x0021 */ 0x2821, 0x287E, 0x0350,
	/* Array index: 0x0024 */ 0x2921, 0x297E, 0x03AE,
	/* Array index: 0x0027 */ 0x2A21, 0x2A73, 0x040C,
	/* Array index: 0x002A */ 0x2B21, 0x2B76, 0x045F,
	/* Array index: 0x002D */ 0x2C21, 0x2C41, 0x04B5,
	/* Array index: 0x0030 */ 0x2C51, 0x2C71, 0x04D6,
	/* Array index: 0x0033 */ 0x3021, 0x307E, 0x04F7,
	/* Array index: 0x0036 */ 0x3121, 0x317E, 0x0555,
	/* Array index: 0x0039 */ 0x3221, 0x327E, 0x05B3,
	/* Array index: 0x003C */ 0x3321, 0x337E, 0x0611,
	/* Array index: 0x003F */ 0x3421, 0x347E, 0x066F,
	/* Array index: 0x0042 */ 0x3521, 0x357E, 0x06CD,
	/* Array index: 0x0045 */ 0x3621, 0x367E, 0x072B,
	/* Array index: 0x0048 */ 0x3721, 0x377E, 0x0789,
	/* Array index: 0x004B */ 0x3821, 0x387E, 0x07E7,
	/* Array index: 0x004E */ 0x3921, 0x397E, 0x0845,
	/* Array index: 0x0051 */ 0x3A21, 0x3A7E, 0x08A3,
	/* Array index: 0x0054 */ 0x3B21, 0x3B7E, 0x0901,
	/* Array index: 0x0057 */ 0x3C21, 0x3C7E, 0x095F,
	/* Array index: 0x005A */ 0x3D21, 0x3D7E, 0x09BD,
	/* Array index: 0x005D */ 0x3E21, 0x3E7E, 0x0A1B,
	/* Array index: 0x0060 */ 0x3F21, 0x3F7E, 0x0A79,
	/* Array index: 0x0063 */ 0x4021, 0x407E, 0x0AD7,
	/* Array index: 0x0066 */ 0x4121, 0x417E, 0x0B35,
	/* Array index: 0x0069 */ 0x4221, 0x427E, 0x0B93,
	/* Array index: 0x006C */ 0x4321, 0x437E, 0x0BF1,
	/* Array index: 0x006F */ 0x4421, 0x447E, 0x0C4F,
	/* Array index: 0x0072 */ 0x4521, 0x457E, 0x0CAD,
	/* Array index: 0x0075 */ 0x4621, 0x467E, 0x0D0B,
	/* Array index: 0x0078 */ 0x4721, 0x477E, 0x0D69,
	/* Array index: 0x007B */ 0x4821, 0x487E, 0x0DC7,
	/* Array index: 0x007E */ 0x4A21, 0x4A7E, 0x0E25,
	/* Array index: 0x0081 */ 0x4B21, 0x4B7E, 0x0E83,
	/* Array index: 0x0084 */ 0x4C21, 0x4C7E, 0x0EE1,
	/* Array index: 0x0087 */ 0x4D21, 0x4D7E, 0x0F3F,
	/* Array index: 0x008A */ 0x4E21, 0x4E7E, 0x0F9D,
	/* Array index: 0x008D */ 0x4F21, 0x4F7E, 0x0FFB,
	/* Array index: 0x0090 */ 0x5021, 0x507E, 0x1059,
	/* Array index: 0x0093 */ 0x5121, 0x517E, 0x10B7,
	/* Array index: 0x0096 */ 0x5221, 0x527E, 0x1115,
	/* Array index: 0x0099 */ 0x5321, 0x537E, 0x1173,
	/* Array index: 0x009C */ 0x5421, 0x547E, 0x11D1,
	/* Array index: 0x009F */ 0x5521, 0x557E, 0x122F,
	/* Array index: 0x00A2 */ 0x5621, 0x567E, 0x128D,
	/* Array index: 0x00A5 */ 0x5721, 0x577E, 0x12EB,
	/* Array index: 0x00A8 */ 0x5821, 0x587E, 0x1349,
	/* Array index: 0x00AB */ 0x5921, 0x597E, 0x13A7,
	/* Array index: 0x00AE */ 0x5A21, 0x5A7E, 0x1405,
	/* Array index: 0x00B1 */ 0x5B21, 0x5B7E, 0x1463,
	/* Array index: 0x00B4 */ 0x5C21, 0x5C7E, 0x14C1,
	/* Array index: 0x00B7 */ 0x5D21, 0x5D7E, 0x151F,
	/* Array index: 0x00BA */ 0x5E21, 0x5E7E, 0x157D,
	/* Array index: 0x00BD */ 0x5F21, 0x5F7E, 0x15DB,
	/* Array index: 0x00C0 */ 0x6021, 0x607E, 0x1639,
	/* Array index: 0x00C3 */ 0x6121, 0x617E, 0x1697,
	/* Array index: 0x00C6 */ 0x6221, 0x627E, 0x16F5,
	/* Array index: 0x00C9 */ 0x6321, 0x637E, 0x1753,
	/* Array index: 0x00CC */ 0x6421, 0x647E, 0x17B1,
	/* Array index: 0x00CF */ 0x6521, 0x657E, 0x180F,
	/* Array index: 0x00D2 */ 0x6621, 0x667E, 0x186D,
	/* Array index: 0x00D5 */ 0x6721, 0x677E, 0x18CB,
	/* Array index: 0x00D8 */ 0x6821, 0x687E, 0x1929,
	/* Array index: 0x00DB */ 0x6921, 0x697E, 0x1987,
	/* Array index: 0x00DE */ 0x6A21, 0x6A7E, 0x19E5,
	/* Array index: 0x00E1 */ 0x6B21, 0x6B7E, 0x1A43,
	/* Array index: 0x00E4 */ 0x6C21, 0x6C7E, 0x1AA1,
	/* Array index: 0x00E7 */ 0x6D21, 0x6D7E, 0x1AFF,
	/* Array index: 0x00EA */ 0x6E21, 0x6E7E, 0x1B5D,
	/* Array index: 0x00ED */ 0x6F21, 0x6F7E, 0x1BBB,
	/* Array index: 0x00F0 */ 0x7021, 0x707E, 0x1C19,
	/* Array index: 0x00F3 */ 0x7121, 0x717E, 0x1C77,
	/* Array index: 0x00F6 */ 0x7221, 0x727E, 0x1CD5,
	/* Array index: 0x00F9 */ 0x7321, 0x737E, 0x1D33,
	/* Array index: 0x00FC */ 0x7421, 0x747E, 0x1D91,
	/* Array index: 0x00FF */ 0x7521, 0x757E, 0x1DEF,
	/* Array index: 0x0102 */ 0x7621, 0x767E, 0x1E4D,
	/* Array index: 0x0105 */ 0x7721, 0x777E, 0x1EAB,
	/* Array index: 0x0108 */ 0x7821, 0x787E, 0x1F09,
	/* Array index: 0x010B */ 0x7921, 0x797E, 0x1F67,
	/* Array index: 0x010E */ 0x7A21, 0x7A7E, 0x1FC5,
	/* Array index: 0x0111 */ 0x7B21, 0x7B7E, 0x2023,
	/* Array index: 0x0114 */ 0x7C21, 0x7C7E, 0x2081,
	/* Array index: 0x0117 */ 0x7D21, 0x7D7E, 0x20DF,
	/* Ranges content */
	/* Range 0x2121 - 0x217E, array index: 0x011A */
	0x3000,0x3001,0x3002,0x00B7,0x2025,0x2026,0x00A8,0x3003,
	0x00AD,0x2015,0x2225,0xFF3C,0x223C,0x2018,0x2019,0x201C,
	0x201D,0x3014,0x3015,0x3008,0x3009,0x300A,0x300B,0x300C,
	0x300D,0x300E,0x300F,0x3010,0x3011,0x00B1,0x00D7,0x00F7,
	0x2260,0x2264,0x2265,0x221E,0x2234,0x00B0,0x2032,0x2033,
	0x2103,0x212B,0xFFE0,0xFFE1,0xFFE5,0x2642,0x2640,0x2220,
	0x22A5,0x2312,0x2202,0x2207,0x2261,0x2252,0x00A7,0x203B,
	0x2606,0x2605,0x25CB,0x25CF,0x25CE,0x25C7,0x25C6,0x25A1,
	0x25A0,0x25B3,0x25B2,0x25BD,0x25BC,0x2192,0x2190,0x2191,
	0x2193,0x2194,0x3013,0x226A,0x226B,0x221A,0x223D,0x221D,
	0x2235,0x222B,0x222C,0x2208,0x220B,0x2286,0x2287,0x2282,
	0x2283,0x222A,0x2229,0x2227,0x2228,0xFFE2,
	/* Range 0x2221 - 0x2265, array index: 0x0178 */
	0x21D2,0x21D4,0x2200,0x2203,0x00B4,0xFF5E,0x02C7,0x02D8,
	0x02DD,0x02DA,0x02D9,0x00B8,0x02DB,0x00A1,0x00BF,0x02D0,
	0x222E,0x2211,0x220F,0x00A4,0x2109,0x2030,0x25C1,0x25C0,
	0x25B7,0x25B6,0x2664,0x2660,0x2661,0x2665,0x2667,0x2663,
	0x2299,0x25C8,0x25A3,0x25D0,0x25D1,0x2592,0x25A4,0x25A5,
	0x25A8,0x25A7,0x25A6,0x25A9,0x2668,0x260F,0x260E,0x261C,
	0x261E,0x00B6,0x2020,0x2021,0x2195,0x2197,0x2199,0x2196,
	0x2198,0x266D,0x2669,0x266A,0x266C,0x327F,0x321C,0x2116,
	0x33C7,0x2122,0x33C2,0x33D8,0x2121,
	/* Range 0x2321 - 0x237E, array index: 0x01BD */
	0xFF01,0xFF02,0xFF03,0xFF04,0xFF05,0xFF06,0xFF07,0xFF08,
	0xFF09,0xFF0A,0xFF0B,0xFF0C,0xFF0D,0xFF0E,0xFF0F,0xFF10,
	0xFF11,0xFF12,0xFF13,0xFF14,0xFF15,0xFF16,0xFF17,0xFF18,
	0xFF19,0xFF1A,0xFF1B,0xFF1C,0xFF1D,0xFF1E,0xFF1F,0xFF20,
	0xFF21,0xFF22,0xFF23,0xFF24,0xFF25,0xFF26,0xFF27,0xFF28,
	0xFF29,0xFF2A,0xFF2B,0xFF2C,0xFF2D,0xFF2E,0xFF2F,0xFF30,
	0xFF31,0xFF32,0xFF33,0xFF34,0xFF35,0xFF36,0xFF37,0xFF38,
	0xFF39,0xFF3A,0xFF3B,0xFFE6,0xFF3D,0xFF3E,0xFF3F,0xFF40,
	0xFF41,0xFF42,0xFF43,0xFF44,0xFF45,0xFF46,0xFF47,0xFF48,
	0xFF49,0xFF4A,0xFF4B,0xFF4C,0xFF4D,0xFF4E,0xFF4F,0xFF50,
	0xFF51,0xFF52,0xFF53,0xFF54,0xFF55,0xFF56,0xFF57,0xFF58,
	0xFF59,0xFF5A,0xFF5B,0xFF5C,0xFF5D,0xFFE3,
	/* Range 0x2421 - 0x247E, array index: 0x021B */
	0x3131,0x3132,0x3133,0x3134,0x3135,0x3136,0x3137,0x3138,
	0x3139,0x313A,0x313B,0x313C,0x313D,0x313E,0x313F,0x3140,
	0x3141,0x3142,0x3143,0x3144,0x3145,0x3146,0x3147,0x3148,
	0x3149,0x314A,0x314B,0x314C,0x314D,0x314E,0x314F,0x3150,
	0x3151,0x3152,0x3153,0x3154,0x3155,0x3156,0x3157,0x3158,
	0x3159,0x315A,0x315B,0x315C,0x315D,0x315E,0x315F,0x3160,
	0x3161,0x3162,0x3163,0x3164,0x3165,0x3166,0x3167,0x3168,
	0x3169,0x316A,0x316B,0x316C,0x316D,0x316E,0x316F,0x3170,
	0x3171,0x3172,0x3173,0x3174,0x3175,0x3176,0x3177,0x3178,
	0x3179,0x317A,0x317B,0x317C,0x317D,0x317E,0x317F,0x3180,
	0x3181,0x3182,0x3183,0x3184,0x3185,0x3186,0x3187,0x3188,
	0x3189,0x318A,0x318B,0x318C,0x318D,0x318E,
	/* Range 0x2521 - 0x252A, array index: 0x0279 */
	0x2170,0x2171,0x2172,0x2173,0x2174,0x2175,0x2176,0x2177,
	0x2178,0x2179,
	/* Range 0x2530 - 0x2539, array index: 0x0283 */
	0x2160,0x2161,0x2162,0x2163,0x2164,0x2165,0x2166,0x2167,
	0x2168,0x2169,
	/* Range 0x2541 - 0x2558, array index: 0x028D */
	0x0391,0x0392,0x0393,0x0394,0x0395,0x0396,0x0397,0x0398,
	0x0399,0x039A,0x039B,0x039C,0x039D,0x039E,0x039F,0x03A0,
	0x03A1,0x03A3,0x03A4,0x03A5,0x03A6,0x03A7,0x03A8,0x03A9,
	/* Range 0x2561 - 0x2578, array index: 0x02A5 */
	0x03B1,0x03B2,0x03B3,0x03B4,0x03B5,0x03B6,0x03B7,0x03B8,
	0x03B9,0x03BA,0x03BB,0x03BC,0x03BD,0x03BE,0x03BF,0x03C0,
	0x03C1,0x03C3,0x03C4,0x03C5,0x03C6,0x03C7,0x03C8,0x03C9,
	/* Range 0x2621 - 0x2664, array index: 0x02BD */
	0x2500,0x2502,0x250C,0x2510,0x2518,0x2514,0x251C,0x252C,
	0x2524,0x2534,0x253C,0x2501,0x2503,0x250F,0x2513,0x251B,
	0x2517,0x2523,0x2533,0x252B,0x253B,0x254B,0x2520,0x252F,
	0x2528,0x2537,0x253F,0x251D,0x2530,0x2525,0x2538,0x2542,
	0x2512,0x2511,0x251A,0x2519,0x2516,0x2515,0x250E,0x250D,
	0x251E,0x251F,0x2521,0x2522,0x2526,0x2527,0x2529,0x252A,
	0x252D,0x252E,0x2531,0x2532,0x2535,0x2536,0x2539,0x253A,
	0x253D,0x253E,0x2540,0x2541,0x2543,0x2544,0x2545,0x2546,
	0x2547,0x2548,0x2549,0x254A,
	/* Range 0x2721 - 0x276F, array index: 0x0301 */
	0x3395,0x3396,0x3397,0x2113,0x3398,0x33C4,0x33A3,0x33A4,
	0x33A5,0x33A6,0x3399,0x339A,0x339B,0x339C,0x339D,0x339E,
	0x339F,0x33A0,0x33A1,0x33A2,0x33CA,0x338D,0x338E,0x338F,
	0x33CF,0x3388,0x3389,0x33C8,0x33A7,0x33A8,0x33B0,0x33B1,
	0x33B2,0x33B3,0x33B4,0x33B5,0x33B6,0x33B7,0x33B8,0x33B9,
	0x3380,0x3381,0x3382,0x3383,0x3384,0x33BA,0x33BB,0x33BC,
	0x33BD,0x33BE,0x33BF,0x3390,0x3391,0x3392,0x3393,0x3394,
	0x2126,0x33C0,0x33C1,0x338A,0x338B,0x338C,0x33D6,0x33C5,
	0x33AD,0x33AE,0x33AF,0x33DB,0x33A9,0x33AA,0x33AB,0x33AC,
	0x33DD,0x33D0,0x33D3,0x33C3,0x33C9,0x33DC,0x33C6,
	/* Range 0x2821 - 0x287E, array index: 0x0350 */
	0x00C6,0x00D0,0x00AA,0x0126,INVALC,0x0132,INVALC,0x013F,
	0x0141,0x00D8,0x0152,0x00BA,0x00DE,0x0166,0x014A,INVALC,
	0x3260,0x3261,0x3262,0x3263,0x3264,0x3265,0x3266,0x3267,
	0x3268,0x3269,0x326A,0x326B,0x326C,0x326D,0x326E,0x326F,
	0x3270,0x3271,0x3272,0x3273,0x3274,0x3275,0x3276,0x3277,
	0x3278,0x3279,0x327A,0x327B,0x24D0,0x24D1,0x24D2,0x24D3,
	0x24D4,0x24D5,0x24D6,0x24D7,0x24D8,0x24D9,0x24DA,0x24DB,
	0x24DC,0x24DD,0x24DE,0x24DF,0x24E0,0x24E1,0x24E2,0x24E3,
	0x24E4,0x24E5,0x24E6,0x24E7,0x24E8,0x24E9,0x2460,0x2461,
	0x2462,0x2463,0x2464,0x2465,0x2466,0x2467,0x2468,0x2469,
	0x246A,0x246B,0x246C,0x246D,0x246E,0x00BD,0x2153,0x2154,
	0x00BC,0x00BE,0x215B,0x215C,0x215D,0x215E,
	/* Range 0x2921 - 0x297E, array index: 0x03AE */
	0x00E6,0x0111,0x00F0,0x0127,0x0131,0x0133,0x0138,0x0140,
	0x0142,0x00F8,0x0153,0x00DF,0x00FE,0x0167,0x014B,0x0149,
	0x3200,0x3201,0x3202,0x3203,0x3204,0x3205,0x3206,0x3207,
	0x3208,0x3209,0x320A,0x320B,0x320C,0x320D,0x320E,0x320F,
	0x3210,0x3211,0x3212,0x3213,0x3214,0x3215,0x3216,0x3217,
	0x3218,0x3219,0x321A,0x321B,0x249C,0x249D,0x249E,0x249F,
	0x24A0,0x24A1,0x24A2,0x24A3,0x24A4,0x24A5,0x24A6,0x24A7,
	0x24A8,0x24A9,0x24AA,0x24AB,0x24AC,0x24AD,0x24AE,0x24AF,
	0x24B0,0x24B1,0x24B2,0x24B3,0x24B4,0x24B5,0x2474,0x2475,
	0x2476,0x2477,0x2478,0x2479,0x247A,0x247B,0x247C,0x247D,
	0x247E,0x247F,0x2480,0x2481,0x2482,0x00B9,0x00B2,0x00B3,
	0x2074,0x207F,0x2081,0x2082,0x2083,0x2084,
	/* Range 0x2A21 - 0x2A73, array index: 0x040C */
	0x3041,0x3042,0x3043,0x3044,0x3045,0x3046,0x3047,0x3048,
	0x3049,0x304A,0x304B,0x304C,0x304D,0x304E,0x304F,0x3050,
	0x3051,0x3052,0x3053,0x3054,0x3055,0x3056,0x3057,0x3058,
	0x3059,0x305A,0x305B,0x305C,0x305D,0x305E,0x305F,0x3060,
	0x3061,0x3062,0x3063,0x3064,0x3065,0x3066,0x3067,0x3068,
	0x3069,0x306A,0x306B,0x306C,0x306D,0x306E,0x306F,0x3070,
	0x3071,0x3072,0x3073,0x3074,0x3075,0x3076,0x3077,0x3078,
	0x3079,0x307A,0x307B,0x307C,0x307D,0x307E,0x307F,0x3080,
	0x3081,0x3082,0x3083,0x3084,0x3085,0x3086,0x3087,0x3088,
	0x3089,0x308A,0x308B,0x308C,0x308D,0x308E,0x308F,0x3090,
	0x3091,0x3092,0x3093,
	/* Range 0x2B21 - 0x2B76, array index: 0x045F */
	0x30A1,0x30A2,0x30A3,0x30A4,0x30A5,0x30A6,0x30A7,0x30A8,
	0x30A9,0x30AA,0x30AB,0x30AC,0x30AD,0x30AE,0x30AF,0x30B0,
	0x30B1,0x30B2,0x30B3,0x30B4,0x30B5,0x30B6,0x30B7,0x30B8,
	0x30B9,0x30BA,0x30BB,0x30BC,0x30BD,0x30BE,0x30BF,0x30C0,
	0x30C1,0x30C2,0x30C3,0x30C4,0x30C5,0x30C6,0x30C7,0x30C8,
	0x30C9,0x30CA,0x30CB,0x30CC,0x30CD,0x30CE,0x30CF,0x30D0,
	0x30D1,0x30D2,0x30D3,0x30D4,0x30D5,0x30D6,0x30D7,0x30D8,
	0x30D9,0x30DA,0x30DB,0x30DC,0x30DD,0x30DE,0x30DF,0x30E0,
	0x30E1,0x30E2,0x30E3,0x30E4,0x30E5,0x30E6,0x30E7,0x30E8,
	0x30E9,0x30EA,0x30EB,0x30EC,0x30ED,0x30EE,0x30EF,0x30F0,
	0x30F1,0x30F2,0x30F3,0x30F4,0x30F5,0x30F6,
	/* Range 0x2C21 - 0x2C41, array index: 0x04B5 */
	0x0410,0x0411,0x0412,0x0413,0x0414,0x0415,0x0401,0x0416,
	0x0417,0x0418,0x0419,0x041A,0x041B,0x041C,0x041D,0x041E,
	0x041F,0x0420,0x0421,0x0422,0x0423,0x0424,0x0425,0x0426,
	0x0427,0x0428,0x0429,0x042A,0x042B,0x042C,0x042D,0x042E,
	0x042F,
	/* Range 0x2C51 - 0x2C71, array index: 0x04D6 */
	0x0430,0x0431,0x0432,0x0433,0x0434,0x0435,0x0451,0x0436,
	0x0437,0x0438,0x0439,0x043A,0x043B,0x043C,0x043D,0x043E,
	0x043F,0x0440,0x0441,0x0442,0x0443,0x0444,0x0445,0x0446,
	0x0447,0x0448,0x0449,0x044A,0x044B,0x044C,0x044D,0x044E,
	0x044F,
	/* Range 0x3021 - 0x307E, array index: 0x04F7 */
	0xAC00,0xAC01,0xAC04,0xAC07,0xAC08,0xAC09,0xAC0A,0xAC10,
	0xAC11,0xAC12,0xAC13,0xAC14,0xAC15,0xAC16,0xAC17,0xAC19,
	0xAC1A,0xAC1B,0xAC1C,0xAC1D,0xAC20,0xAC24,0xAC2C,0xAC2D,
	0xAC2F,0xAC30,0xAC31,0xAC38,0xAC39,0xAC3C,0xAC40,0xAC4B,
	0xAC4D,0xAC54,0xAC58,0xAC5C,0xAC70,0xAC71,0xAC74,0xAC77,
	0xAC78,0xAC7A,0xAC80,0xAC81,0xAC83,0xAC84,0xAC85,0xAC86,
	0xAC89,0xAC8A,0xAC8B,0xAC8C,0xAC90,0xAC94,0xAC9C,0xAC9D,
	0xAC9F,0xACA0,0xACA1,0xACA8,0xACA9,0xACAA,0xACAC,0xACAF,
	0xACB0,0xACB8,0xACB9,0xACBB,0xACBC,0xACBD,0xACC1,0xACC4,
	0xACC8,0xACCC,0xACD5,0xACD7,0xACE0,0xACE1,0xACE4,0xACE7,
	0xACE8,0xACEA,0xACEC,0xACEF,0xACF0,0xACF1,0xACF3,0xACF5,
	0xACF6,0xACFC,0xACFD,0xAD00,0xAD04,0xAD06,
	/* Range 0x3121 - 0x317E, array index: 0x0555 */
	0xAD0C,0xAD0D,0xAD0F,0xAD11,0xAD18,0xAD1C,0xAD20,0xAD29,
	0xAD2C,0xAD2D,0xAD34,0xAD35,0xAD38,0xAD3C,0xAD44,0xAD45,
	0xAD47,0xAD49,0xAD50,0xAD54,0xAD58,0xAD61,0xAD63,0xAD6C,
	0xAD6D,0xAD70,0xAD73,0xAD74,0xAD75,0xAD76,0xAD7B,0xAD7C,
	0xAD7D,0xAD7F,0xAD81,0xAD82,0xAD88,0xAD89,0xAD8C,0xAD90,
	0xAD9C,0xAD9D,0xADA4,0xADB7,0xADC0,0xADC1,0xADC4,0xADC8,
	0xADD0,0xADD1,0xADD3,0xADDC,0xADE0,0xADE4,0xADF8,0xADF9,
	0xADFC,0xADFF,0xAE00,0xAE01,0xAE08,0xAE09,0xAE0B,0xAE0D,
	0xAE14,0xAE30,0xAE31,0xAE34,0xAE37,0xAE38,0xAE3A,0xAE40,
	0xAE41,0xAE43,0xAE45,0xAE46,0xAE4A,0xAE4C,0xAE4D,0xAE4E,
	0xAE50,0xAE54,0xAE56,0xAE5C,0xAE5D,0xAE5F,0xAE60,0xAE61,
	0xAE65,0xAE68,0xAE69,0xAE6C,0xAE70,0xAE78,
	/* Range 0x3221 - 0x327E, array index: 0x05B3 */
	0xAE79,0xAE7B,0xAE7C,0xAE7D,0xAE84,0xAE85,0xAE8C,0xAEBC,
	0xAEBD,0xAEBE,0xAEC0,0xAEC4,0xAECC,0xAECD,0xAECF,0xAED0,
	0xAED1,0xAED8,0xAED9,0xAEDC,0xAEE8,0xAEEB,0xAEED,0xAEF4,
	0xAEF8,0xAEFC,0xAF07,0xAF08,0xAF0D,0xAF10,0xAF2C,0xAF2D,
	0xAF30,0xAF32,0xAF34,0xAF3C,0xAF3D,0xAF3F,0xAF41,0xAF42,
	0xAF43,0xAF48,0xAF49,0xAF50,0xAF5C,0xAF5D,0xAF64,0xAF65,
	0xAF79,0xAF80,0xAF84,0xAF88,0xAF90,0xAF91,0xAF95,0xAF9C,
	0xAFB8,0xAFB9,0xAFBC,0xAFC0,0xAFC7,0xAFC8,0xAFC9,0xAFCB,
	0xAFCD,0xAFCE,0xAFD4,0xAFDC,0xAFE8,0xAFE9,0xAFF0,0xAFF1,
	0xAFF4,0xAFF8,0xB000,0xB001,0xB004,0xB00C,0xB010,0xB014,
	0xB01C,0xB01D,0xB028,0xB044,0xB045,0xB048,0xB04A,0xB04C,
	0xB04E,0xB053,0xB054,0xB055,0xB057,0xB059,
	/* Range 0x3321 - 0x337E, array index: 0x0611 */
	0xB05D,0xB07C,0xB07D,0xB080,0xB084,0xB08C,0xB08D,0xB08F,
	0xB091,0xB098,0xB099,0xB09A,0xB09C,0xB09F,0xB0A0,0xB0A1,
	0xB0A2,0xB0A8,0xB0A9,0xB0AB,0xB0AC,0xB0AD,0xB0AE,0xB0AF,
	0xB0B1,0xB0B3,0xB0B4,0xB0B5,0xB0B8,0xB0BC,0xB0C4,0xB0C5,
	0xB0C7,0xB0C8,0xB0C9,0xB0D0,0xB0D1,0xB0D4,0xB0D8,0xB0E0,
	0xB0E5,0xB108,0xB109,0xB10B,0xB10C,0xB110,0xB112,0xB113,
	0xB118,0xB119,0xB11B,0xB11C,0xB11D,0xB123,0xB124,0xB125,
	0xB128,0xB12C,0xB134,0xB135,0xB137,0xB138,0xB139,0xB140,
	0xB141,0xB144,0xB148,0xB150,0xB151,0xB154,0xB155,0xB158,
	0xB15C,0xB160,0xB178,0xB179,0xB17C,0xB180,0xB182,0xB188,
	0xB189,0xB18B,0xB18D,0xB192,0xB193,0xB194,0xB198,0xB19C,
	0xB1A8,0xB1CC,0xB1D0,0xB1D4,0xB1DC,0xB1DD,
	/* Range 0x3421 - 0x347E, array index: 0x066F */
	0xB1DF,0xB1E8,0xB1E9,0xB1EC,0xB1F0,0xB1F9,0xB1FB,0xB1FD,
	0xB204,0xB205,0xB208,0xB20B,0xB20C,0xB214,0xB215,0xB217,
	0xB219,0xB220,0xB234,0xB23C,0xB258,0xB25C,0xB260,0xB268,
	0xB269,0xB274,0xB275,0xB27C,0xB284,0xB285,0xB289,0xB290,
	0xB291,0xB294,0xB298,0xB299,0xB29A,0xB2A0,0xB2A1,0xB2A3,
	0xB2A5,0xB2A6,0xB2AA,0xB2AC,0xB2B0,0xB2B4,0xB2C8,0xB2C9,
	0xB2CC,0xB2D0,0xB2D2,0xB2D8,0xB2D9,0xB2DB,0xB2DD,0xB2E2,
	0xB2E4,0xB2E5,0xB2E6,0xB2E8,0xB2EB,0xB2EC,0xB2ED,0xB2EE,
	0xB2EF,0xB2F3,0xB2F4,0xB2F5,0xB2F7,0xB2F8,0xB2F9,0xB2FA,
	0xB2FB,0xB2FF,0xB300,0xB301,0xB304,0xB308,0xB310,0xB311,
	0xB313,0xB314,0xB315,0xB31C,0xB354,0xB355,0xB356,0xB358,
	0xB35B,0xB35C,0xB35E,0xB35F,0xB364,0xB365,
	/* Range 0x3521 - 0x357E, array index: 0x06CD */
	0xB367,0xB369,0xB36B,0xB36E,0xB370,0xB371,0xB374,0xB378,
	0xB380,0xB381,0xB383,0xB384,0xB385,0xB38C,0xB390,0xB394,
	0xB3A0,0xB3A1,0xB3A8,0xB3AC,0xB3C4,0xB3C5,0xB3C8,0xB3CB,
	0xB3CC,0xB3CE,0xB3D0,0xB3D4,0xB3D5,0xB3D7,0xB3D9,0xB3DB,
	0xB3DD,0xB3E0,0xB3E4,0xB3E8,0xB3FC,0xB410,0xB418,0xB41C,
	0xB420,0xB428,0xB429,0xB42B,0xB434,0xB450,0xB451,0xB454,
	0xB458,0xB460,0xB461,0xB463,0xB465,0xB46C,0xB480,0xB488,
	0xB49D,0xB4A4,0xB4A8,0xB4AC,0xB4B5,0xB4B7,0xB4B9,0xB4C0,
	0xB4C4,0xB4C8,0xB4D0,0xB4D5,0xB4DC,0xB4DD,0xB4E0,0xB4E3,
	0xB4E4,0xB4E6,0xB4EC,0xB4ED,0xB4EF,0xB4F1,0xB4F8,0xB514,
	0xB515,0xB518,0xB51B,0xB51C,0xB524,0xB525,0xB527,0xB528,
	0xB529,0xB52A,0xB530,0xB531,0xB534,0xB538,
	/* Range 0x3621 - 0x367E, array index: 0x072B */
	0xB540,0xB541,0xB543,0xB544,0xB545,0xB54B,0xB54C,0xB54D,
	0xB550,0xB554,0xB55C,0xB55D,0xB55F,0xB560,0xB561,0xB5A0,
	0xB5A1,0xB5A4,0xB5A8,0xB5AA,0xB5AB,0xB5B0,0xB5B1,0xB5B3,
	0xB5B4,0xB5B5,0xB5BB,0xB5BC,0xB5BD,0xB5C0,0xB5C4,0xB5CC,
	0xB5CD,0xB5CF,0xB5D0,0xB5D1,0xB5D8,0xB5EC,0xB610,0xB611,
	0xB614,0xB618,0xB625,0xB62C,0xB634,0xB648,0xB664,0xB668,
	0xB69C,0xB69D,0xB6A0,0xB6A4,0xB6AB,0xB6AC,0xB6B1,0xB6D4,
	0xB6F0,0xB6F4,0xB6F8,0xB700,0xB701,0xB705,0xB728,0xB729,
	0xB72C,0xB72F,0xB730,0xB738,0xB739,0xB73B,0xB744,0xB748,
	0xB74C,0xB754,0xB755,0xB760,0xB764,0xB768,0xB770,0xB771,
	0xB773,0xB775,0xB77C,0xB77D,0xB780,0xB784,0xB78C,0xB78D,
	0xB78F,0xB790,0xB791,0xB792,0xB796,0xB797,
	/* Range 0x3721 - 0x377E, array index: 0x0789 */
	0xB798,0xB799,0xB79C,0xB7A0,0xB7A8,0xB7A9,0xB7AB,0xB7AC,
	0xB7AD,0xB7B4,0xB7B5,0xB7B8,0xB7C7,0xB7C9,0xB7EC,0xB7ED,
	0xB7F0,0xB7F4,0xB7FC,0xB7FD,0xB7FF,0xB800,0xB801,0xB807,
	0xB808,0xB809,0xB80C,0xB810,0xB818,0xB819,0xB81B,0xB81D,
	0xB824,0xB825,0xB828,0xB82C,0xB834,0xB835,0xB837,0xB838,
	0xB839,0xB840,0xB844,0xB851,0xB853,0xB85C,0xB85D,0xB860,
	0xB864,0xB86C,0xB86D,0xB86F,0xB871,0xB878,0xB87C,0xB88D,
	0xB8A8,0xB8B0,0xB8B4,0xB8B8,0xB8C0,0xB8C1,0xB8C3,0xB8C5,
	0xB8CC,0xB8D0,0xB8D4,0xB8DD,0xB8DF,0xB8E1,0xB8E8,0xB8E9,
	0xB8EC,0xB8F0,0xB8F8,0xB8F9,0xB8FB,0xB8FD,0xB904,0xB918,
	0xB920,0xB93C,0xB93D,0xB940,0xB944,0xB94C,0xB94F,0xB951,
	0xB958,0xB959,0xB95C,0xB960,0xB968,0xB969,
	/* Range 0x3821 - 0x387E, array index: 0x07E7 */
	0xB96B,0xB96D,0xB974,0xB975,0xB978,0xB97C,0xB984,0xB985,
	0xB987,0xB989,0xB98A,0xB98D,0xB98E,0xB9AC,0xB9AD,0xB9B0,
	0xB9B4,0xB9BC,0xB9BD,0xB9BF,0xB9C1,0xB9C8,0xB9C9,0xB9CC,
	0xB9CE,0xB9CF,0xB9D0,0xB9D1,0xB9D2,0xB9D8,0xB9D9,0xB9DB,
	0xB9DD,0xB9DE,0xB9E1,0xB9E3,0xB9E4,0xB9E5,0xB9E8,0xB9EC,
	0xB9F4,0xB9F5,0xB9F7,0xB9F8,0xB9F9,0xB9FA,0xBA00,0xBA01,
	0xBA08,0xBA15,0xBA38,0xBA39,0xBA3C,0xBA40,0xBA42,0xBA48,
	0xBA49,0xBA4B,0xBA4D,0xBA4E,0xBA53,0xBA54,0xBA55,0xBA58,
	0xBA5C,0xBA64,0xBA65,0xBA67,0xBA68,0xBA69,0xBA70,0xBA71,
	0xBA74,0xBA78,0xBA83,0xBA84,0xBA85,0xBA87,0xBA8C,0xBAA8,
	0xBAA9,0xBAAB,0xBAAC,0xBAB0,0xBAB2,0xBAB8,0xBAB9,0xBABB,
	0xBABD,0xBAC4,0xBAC8,0xBAD8,0xBAD9,0xBAFC,
	/* Range 0x3921 - 0x397E, array index: 0x0845 */
	0xBB00,0xBB04,0xBB0D,0xBB0F,0xBB11,0xBB18,0xBB1C,0xBB20,
	0xBB29,0xBB2B,0xBB34,0xBB35,0xBB36,0xBB38,0xBB3B,0xBB3C,
	0xBB3D,0xBB3E,0xBB44,0xBB45,0xBB47,0xBB49,0xBB4D,0xBB4F,
	0xBB50,0xBB54,0xBB58,0xBB61,0xBB63,0xBB6C,0xBB88,0xBB8C,
	0xBB90,0xBBA4,0xBBA8,0xBBAC,0xBBB4,0xBBB7,0xBBC0,0xBBC4,
	0xBBC8,0xBBD0,0xBBD3,0xBBF8,0xBBF9,0xBBFC,0xBBFF,0xBC00,
	0xBC02,0xBC08,0xBC09,0xBC0B,0xBC0C,0xBC0D,0xBC0F,0xBC11,
	0xBC14,0xBC15,0xBC16,0xBC17,0xBC18,0xBC1B,0xBC1C,0xBC1D,
	0xBC1E,0xBC1F,0xBC24,0xBC25,0xBC27,0xBC29,0xBC2D,0xBC30,
	0xBC31,0xBC34,0xBC38,0xBC40,0xBC41,0xBC43,0xBC44,0xBC45,
	0xBC49,0xBC4C,0xBC4D,0xBC50,0xBC5D,0xBC84,0xBC85,0xBC88,
	0xBC8B,0xBC8C,0xBC8E,0xBC94,0xBC95,0xBC97,
	/* Range 0x3A21 - 0x3A7E, array index: 0x08A3 */
	0xBC99,0xBC9A,0xBCA0,0xBCA1,0xBCA4,0xBCA7,0xBCA8,0xBCB0,
	0xBCB1,0xBCB3,0xBCB4,0xBCB5,0xBCBC,0xBCBD,0xBCC0,0xBCC4,
	0xBCCD,0xBCCF,0xBCD0,0xBCD1,0xBCD5,0xBCD8,0xBCDC,0xBCF4,
	0xBCF5,0xBCF6,0xBCF8,0xBCFC,0xBD04,0xBD05,0xBD07,0xBD09,
	0xBD10,0xBD14,0xBD24,0xBD2C,0xBD40,0xBD48,0xBD49,0xBD4C,
	0xBD50,0xBD58,0xBD59,0xBD64,0xBD68,0xBD80,0xBD81,0xBD84,
	0xBD87,0xBD88,0xBD89,0xBD8A,0xBD90,0xBD91,0xBD93,0xBD95,
	0xBD99,0xBD9A,0xBD9C,0xBDA4,0xBDB0,0xBDB8,0xBDD4,0xBDD5,
	0xBDD8,0xBDDC,0xBDE9,0xBDF0,0xBDF4,0xBDF8,0xBE00,0xBE03,
	0xBE05,0xBE0C,0xBE0D,0xBE10,0xBE14,0xBE1C,0xBE1D,0xBE1F,
	0xBE44,0xBE45,0xBE48,0xBE4C,0xBE4E,0xBE54,0xBE55,0xBE57,
	0xBE59,0xBE5A,0xBE5B,0xBE60,0xBE61,0xBE64,
	/* Range 0x3B21 - 0x3B7E, array index: 0x0901 */
	0xBE68,0xBE6A,0xBE70,0xBE71,0xBE73,0xBE74,0xBE75,0xBE7B,
	0xBE7C,0xBE7D,0xBE80,0xBE84,0xBE8C,0xBE8D,0xBE8F,0xBE90,
	0xBE91,0xBE98,0xBE99,0xBEA8,0xBED0,0xBED1,0xBED4,0xBED7,
	0xBED8,0xBEE0,0xBEE3,0xBEE4,0xBEE5,0xBEEC,0xBF01,0xBF08,
	0xBF09,0xBF18,0xBF19,0xBF1B,0xBF1C,0xBF1D,0xBF40,0xBF41,
	0xBF44,0xBF48,0xBF50,0xBF51,0xBF55,0xBF94,0xBFB0,0xBFC5,
	0xBFCC,0xBFCD,0xBFD0,0xBFD4,0xBFDC,0xBFDF,0xBFE1,0xC03C,
	0xC051,0xC058,0xC05C,0xC060,0xC068,0xC069,0xC090,0xC091,
	0xC094,0xC098,0xC0A0,0xC0A1,0xC0A3,0xC0A5,0xC0AC,0xC0AD,
	0xC0AF,0xC0B0,0xC0B3,0xC0B4,0xC0B5,0xC0B6,0xC0BC,0xC0BD,
	0xC0BF,0xC0C0,0xC0C1,0xC0C5,0xC0C8,0xC0C9,0xC0CC,0xC0D0,
	0xC0D8,0xC0D9,0xC0DB,0xC0DC,0xC0DD,0xC0E4,
	/* Range 0x3C21 - 0x3C7E, array index: 0x095F */
	0xC0E5,0xC0E8,0xC0EC,0xC0F4,0xC0F5,0xC0F7,0xC0F9,0xC100,
	0xC104,0xC108,0xC110,0xC115,0xC11C,0xC11D,0xC11E,0xC11F,
	0xC120,0xC123,0xC124,0xC126,0xC127,0xC12C,0xC12D,0xC12F,
	0xC130,0xC131,0xC136,0xC138,0xC139,0xC13C,0xC140,0xC148,
	0xC149,0xC14B,0xC14C,0xC14D,0xC154,0xC155,0xC158,0xC15C,
	0xC164,0xC165,0xC167,0xC168,0xC169,0xC170,0xC174,0xC178,
	0xC185,0xC18C,0xC18D,0xC18E,0xC190,0xC194,0xC196,0xC19C,
	0xC19D,0xC19F,0xC1A1,0xC1A5,0xC1A8,0xC1A9,0xC1AC,0xC1B0,
	0xC1BD,0xC1C4,0xC1C8,0xC1CC,0xC1D4,0xC1D7,0xC1D8,0xC1E0,
	0xC1E4,0xC1E8,0xC1F0,0xC1F1,0xC1F3,0xC1FC,0xC1FD,0xC200,
	0xC204,0xC20C,0xC20D,0xC20F,0xC211,0xC218,0xC219,0xC21C,
	0xC21F,0xC220,0xC228,0xC229,0xC22B,0xC22D,
	/* Range 0x3D21 - 0x3D7E, array index: 0x09BD */
	0xC22F,0xC231,0xC232,0xC234,0xC248,0xC250,0xC251,0xC254,
	0xC258,0xC260,0xC265,0xC26C,0xC26D,0xC270,0xC274,0xC27C,
	0xC27D,0xC27F,0xC281,0xC288,0xC289,0xC290,0xC298,0xC29B,
	0xC29D,0xC2A4,0xC2A5,0xC2A8,0xC2AC,0xC2AD,0xC2B4,0xC2B5,
	0xC2B7,0xC2B9,0xC2DC,0xC2DD,0xC2E0,0xC2E3,0xC2E4,0xC2EB,
	0xC2EC,0xC2ED,0xC2EF,0xC2F1,0xC2F6,0xC2F8,0xC2F9,0xC2FB,
	0xC2FC,0xC300,0xC308,0xC309,0xC30C,0xC30D,0xC313,0xC314,
	0xC315,0xC318,0xC31C,0xC324,0xC325,0xC328,0xC329,0xC345,
	0xC368,0xC369,0xC36C,0xC370,0xC372,0xC378,0xC379,0xC37C,
	0xC37D,0xC384,0xC388,0xC38C,0xC3C0,0xC3D8,0xC3D9,0xC3DC,
	0xC3DF,0xC3E0,0xC3E2,0xC3E8,0xC3E9,0xC3ED,0xC3F4,0xC3F5,
	0xC3F8,0xC408,0xC410,0xC424,0xC42C,0xC430,
	/* Range 0x3E21 - 0x3E7E, array index: 0x0A1B */
	0xC434,0xC43C,0xC43D,0xC448,0xC464,0xC465,0xC468,0xC46C,
	0xC474,0xC475,0xC479,0xC480,0xC494,0xC49C,0xC4B8,0xC4BC,
	0xC4E9,0xC4F0,0xC4F1,0xC4F4,0xC4F8,0xC4FA,0xC4FF,0xC500,
	0xC501,0xC50C,0xC510,0xC514,0xC51C,0xC528,0xC529,0xC52C,
	0xC530,0xC538,0xC539,0xC53B,0xC53D,0xC544,0xC545,0xC548,
	0xC549,0xC54A,0xC54C,0xC54D,0xC54E,0xC553,0xC554,0xC555,
	0xC557,0xC558,0xC559,0xC55D,0xC55E,0xC560,0xC561,0xC564,
	0xC568,0xC570,0xC571,0xC573,0xC574,0xC575,0xC57C,0xC57D,
	0xC580,0xC584,0xC587,0xC58C,0xC58D,0xC58F,0xC591,0xC595,
	0xC597,0xC598,0xC59C,0xC5A0,0xC5A9,0xC5B4,0xC5B5,0xC5B8,
	0xC5B9,0xC5BB,0xC5BC,0xC5BD,0xC5BE,0xC5C4,0xC5C5,0xC5C6,
	0xC5C7,0xC5C8,0xC5C9,0xC5CA,0xC5CC,0xC5CE,
	/* Range 0x3F21 - 0x3F7E, array index: 0x0A79 */
	0xC5D0,0xC5D1,0xC5D4,0xC5D8,0xC5E0,0xC5E1,0xC5E3,0xC5E5,
	0xC5EC,0xC5ED,0xC5EE,0xC5F0,0xC5F4,0xC5F6,0xC5F7,0xC5FC,
	0xC5FD,0xC5FE,0xC5FF,0xC600,0xC601,0xC605,0xC606,0xC607,
	0xC608,0xC60C,0xC610,0xC618,0xC619,0xC61B,0xC61C,0xC624,
	0xC625,0xC628,0xC62C,0xC62D,0xC62E,0xC630,0xC633,0xC634,
	0xC635,0xC637,0xC639,0xC63B,0xC640,0xC641,0xC644,0xC648,
	0xC650,0xC651,0xC653,0xC654,0xC655,0xC65C,0xC65D,0xC660,
	0xC66C,0xC66F,0xC671,0xC678,0xC679,0xC67C,0xC680,0xC688,
	0xC689,0xC68B,0xC68D,0xC694,0xC695,0xC698,0xC69C,0xC6A4,
	0xC6A5,0xC6A7,0xC6A9,0xC6B0,0xC6B1,0xC6B4,0xC6B8,0xC6B9,
	0xC6BA,0xC6C0,0xC6C1,0xC6C3,0xC6C5,0xC6CC,0xC6CD,0xC6D0,
	0xC6D4,0xC6DC,0xC6DD,0xC6E0,0xC6E1,0xC6E8,
	/* Range 0x4021 - 0x407E, array index: 0x0AD7 */
	0xC6E9,0xC6EC,0xC6F0,0xC6F8,0xC6F9,0xC6FD,0xC704,0xC705,
	0xC708,0xC70C,0xC714,0xC715,0xC717,0xC719,0xC720,0xC721,
	0xC724,0xC728,0xC730,0xC731,0xC733,0xC735,0xC737,0xC73C,
	0xC73D,0xC740,0xC744,0xC74A,0xC74C,0xC74D,0xC74F,0xC751,
	0xC752,0xC753,0xC754,0xC755,0xC756,0xC757,0xC758,0xC75C,
	0xC760,0xC768,0xC76B,0xC774,0xC775,0xC778,0xC77C,0xC77D,
	0xC77E,0xC783,0xC784,0xC785,0xC787,0xC788,0xC789,0xC78A,
	0xC78E,0xC790,0xC791,0xC794,0xC796,0xC797,0xC798,0xC79A,
	0xC7A0,0xC7A1,0xC7A3,0xC7A4,0xC7A5,0xC7A6,0xC7AC,0xC7AD,
	0xC7B0,0xC7B4,0xC7BC,0xC7BD,0xC7BF,0xC7C0,0xC7C1,0xC7C8,
	0xC7C9,0xC7CC,0xC7CE,0xC7D0,0xC7D8,0xC7DD,0xC7E4,0xC7E8,
	0xC7EC,0xC800,0xC801,0xC804,0xC808,0xC80A,
	/* Range 0x4121 - 0x417E, array index: 0x0B35 */
	0xC810,0xC811,0xC813,0xC815,0xC816,0xC81C,0xC81D,0xC820,
	0xC824,0xC82C,0xC82D,0xC82F,0xC831,0xC838,0xC83C,0xC840,
	0xC848,0xC849,0xC84C,0xC84D,0xC854,0xC870,0xC871,0xC874,
	0xC878,0xC87A,0xC880,0xC881,0xC883,0xC885,0xC886,0xC887,
	0xC88B,0xC88C,0xC88D,0xC894,0xC89D,0xC89F,0xC8A1,0xC8A8,
	0xC8BC,0xC8BD,0xC8C4,0xC8C8,0xC8CC,0xC8D4,0xC8D5,0xC8D7,
	0xC8D9,0xC8E0,0xC8E1,0xC8E4,0xC8F5,0xC8FC,0xC8FD,0xC900,
	0xC904,0xC905,0xC906,0xC90C,0xC90D,0xC90F,0xC911,0xC918,
	0xC92C,0xC934,0xC950,0xC951,0xC954,0xC958,0xC960,0xC961,
	0xC963,0xC96C,0xC970,0xC974,0xC97C,0xC988,0xC989,0xC98C,
	0xC990,0xC998,0xC999,0xC99B,0xC99D,0xC9C0,0xC9C1,0xC9C4,
	0xC9C7,0xC9C8,0xC9CA,0xC9D0,0xC9D1,0xC9D3,
	/* Range 0x4221 - 0x427E, array index: 0x0B93 */
	0xC9D5,0xC9D6,0xC9D9,0xC9DA,0xC9DC,0xC9DD,0xC9E0,0xC9E2,
	0xC9E4,0xC9E7,0xC9EC,0xC9ED,0xC9EF,0xC9F0,0xC9F1,0xC9F8,
	0xC9F9,0xC9FC,0xCA00,0xCA08,0xCA09,0xCA0B,0xCA0C,0xCA0D,
	0xCA14,0xCA18,0xCA29,0xCA4C,0xCA4D,0xCA50,0xCA54,0xCA5C,
	0xCA5D,0xCA5F,0xCA60,0xCA61,0xCA68,0xCA7D,0xCA84,0xCA98,
	0xCABC,0xCABD,0xCAC0,0xCAC4,0xCACC,0xCACD,0xCACF,0xCAD1,
	0xCAD3,0xCAD8,0xCAD9,0xCAE0,0xCAEC,0xCAF4,0xCB08,0xCB10,
	0xCB14,0xCB18,0xCB20,0xCB21,0xCB41,0xCB48,0xCB49,0xCB4C,
	0xCB50,0xCB58,0xCB59,0xCB5D,0xCB64,0xCB78,0xCB79,0xCB9C,
	0xCBB8,0xCBD4,0xCBE4,0xCBE7,0xCBE9,0xCC0C,0xCC0D,0xCC10,
	0xCC14,0xCC1C,0xCC1D,0xCC21,0xCC22,0xCC27,0xCC28,0xCC29,
	0xCC2C,0xCC2E,0xCC30,0xCC38,0xCC39,0xCC3B,
	/* Range 0x4321 - 0x437E, array index: 0x0BF1 */
	0xCC3C,0xCC3D,0xCC3E,0xCC44,0xCC45,0xCC48,0xCC4C,0xCC54,
	0xCC55,0xCC57,0xCC58,0xCC59,0xCC60,0xCC64,0xCC66,0xCC68,
	0xCC70,0xCC75,0xCC98,0xCC99,0xCC9C,0xCCA0,0xCCA8,0xCCA9,
	0xCCAB,0xCCAC,0xCCAD,0xCCB4,0xCCB5,0xCCB8,0xCCBC,0xCCC4,
	0xCCC5,0xCCC7,0xCCC9,0xCCD0,0xCCD4,0xCCE4,0xCCEC,0xCCF0,
	0xCD01,0xCD08,0xCD09,0xCD0C,0xCD10,0xCD18,0xCD19,0xCD1B,
	0xCD1D,0xCD24,0xCD28,0xCD2C,0xCD39,0xCD5C,0xCD60,0xCD64,
	0xCD6C,0xCD6D,0xCD6F,0xCD71,0xCD78,0xCD88,0xCD94,0xCD95,
	0xCD98,0xCD9C,0xCDA4,0xCDA5,0xCDA7,0xCDA9,0xCDB0,0xCDC4,
	0xCDCC,0xCDD0,0xCDE8,0xCDEC,0xCDF0,0xCDF8,0xCDF9,0xCDFB,
	0xCDFD,0xCE04,0xCE08,0xCE0C,0xCE14,0xCE19,0xCE20,0xCE21,
	0xCE24,0xCE28,0xCE30,0xCE31,0xCE33,0xCE35,
	/* Range 0x4421 - 0x447E, array index: 0x0C4F */
	0xCE58,0xCE59,0xCE5C,0xCE5F,0xCE60,0xCE61,0xCE68,0xCE69,
	0xCE6B,0xCE6D,0xCE74,0xCE75,0xCE78,0xCE7C,0xCE84,0xCE85,
	0xCE87,0xCE89,0xCE90,0xCE91,0xCE94,0xCE98,0xCEA0,0xCEA1,
	0xCEA3,0xCEA4,0xCEA5,0xCEAC,0xCEAD,0xCEC1,0xCEE4,0xCEE5,
	0xCEE8,0xCEEB,0xCEEC,0xCEF4,0xCEF5,0xCEF7,0xCEF8,0xCEF9,
	0xCF00,0xCF01,0xCF04,0xCF08,0xCF10,0xCF11,0xCF13,0xCF15,
	0xCF1C,0xCF20,0xCF24,0xCF2C,0xCF2D,0xCF2F,0xCF30,0xCF31,
	0xCF38,0xCF54,0xCF55,0xCF58,0xCF5C,0xCF64,0xCF65,0xCF67,
	0xCF69,0xCF70,0xCF71,0xCF74,0xCF78,0xCF80,0xCF85,0xCF8C,
	0xCFA1,0xCFA8,0xCFB0,0xCFC4,0xCFE0,0xCFE1,0xCFE4,0xCFE8,
	0xCFF0,0xCFF1,0xCFF3,0xCFF5,0xCFFC,0xD000,0xD004,0xD011,
	0xD018,0xD02D,0xD034,0xD035,0xD038,0xD03C,
	/* Range 0x4521 - 0x457E, array index: 0x0CAD */
	0xD044,0xD045,0xD047,0xD049,0xD050,0xD054,0xD058,0xD060,
	0xD06C,0xD06D,0xD070,0xD074,0xD07C,0xD07D,0xD081,0xD0A4,
	0xD0A5,0xD0A8,0xD0AC,0xD0B4,0xD0B5,0xD0B7,0xD0B9,0xD0C0,
	0xD0C1,0xD0C4,0xD0C8,0xD0C9,0xD0D0,0xD0D1,0xD0D3,0xD0D4,
	0xD0D5,0xD0DC,0xD0DD,0xD0E0,0xD0E4,0xD0EC,0xD0ED,0xD0EF,
	0xD0F0,0xD0F1,0xD0F8,0xD10D,0xD130,0xD131,0xD134,0xD138,
	0xD13A,0xD140,0xD141,0xD143,0xD144,0xD145,0xD14C,0xD14D,
	0xD150,0xD154,0xD15C,0xD15D,0xD15F,0xD161,0xD168,0xD16C,
	0xD17C,0xD184,0xD188,0xD1A0,0xD1A1,0xD1A4,0xD1A8,0xD1B0,
	0xD1B1,0xD1B3,0xD1B5,0xD1BA,0xD1BC,0xD1C0,0xD1D8,0xD1F4,
	0xD1F8,0xD207,0xD209,0xD210,0xD22C,0xD22D,0xD230,0xD234,
	0xD23C,0xD23D,0xD23F,0xD241,0xD248,0xD25C,
	/* Range 0x4621 - 0x467E, array index: 0x0D0B */
	0xD264,0xD280,0xD281,0xD284,0xD288,0xD290,0xD291,0xD295,
	0xD29C,0xD2A0,0xD2A4,0xD2AC,0xD2B1,0xD2B8,0xD2B9,0xD2BC,
	0xD2BF,0xD2C0,0xD2C2,0xD2C8,0xD2C9,0xD2CB,0xD2D4,0xD2D8,
	0xD2DC,0xD2E4,0xD2E5,0xD2F0,0xD2F1,0xD2F4,0xD2F8,0xD300,
	0xD301,0xD303,0xD305,0xD30C,0xD30D,0xD30E,0xD310,0xD314,
	0xD316,0xD31C,0xD31D,0xD31F,0xD320,0xD321,0xD325,0xD328,
	0xD329,0xD32C,0xD330,0xD338,0xD339,0xD33B,0xD33C,0xD33D,
	0xD344,0xD345,0xD37C,0xD37D,0xD380,0xD384,0xD38C,0xD38D,
	0xD38F,0xD390,0xD391,0xD398,0xD399,0xD39C,0xD3A0,0xD3A8,
	0xD3A9,0xD3AB,0xD3AD,0xD3B4,0xD3B8,0xD3BC,0xD3C4,0xD3C5,
	0xD3C8,0xD3C9,0xD3D0,0xD3D8,0xD3E1,0xD3E3,0xD3EC,0xD3ED,
	0xD3F0,0xD3F4,0xD3FC,0xD3FD,0xD3FF,0xD401,
	/* Range 0x4721 - 0x477E, array index: 0x0D69 */
	0xD408,0xD41D,0xD440,0xD444,0xD45C,0xD460,0xD464,0xD46D,
	0xD46F,0xD478,0xD479,0xD47C,0xD47F,0xD480,0xD482,0xD488,
	0xD489,0xD48B,0xD48D,0xD494,0xD4A9,0xD4CC,0xD4D0,0xD4D4,
	0xD4DC,0xD4DF,0xD4E8,0xD4EC,0xD4F0,0xD4F8,0xD4FB,0xD4FD,
	0xD504,0xD508,0xD50C,0xD514,0xD515,0xD517,0xD53C,0xD53D,
	0xD540,0xD544,0xD54C,0xD54D,0xD54F,0xD551,0xD558,0xD559,
	0xD55C,0xD560,0xD565,0xD568,0xD569,0xD56B,0xD56D,0xD574,
	0xD575,0xD578,0xD57C,0xD584,0xD585,0xD587,0xD588,0xD589,
	0xD590,0xD5A5,0xD5C8,0xD5C9,0xD5CC,0xD5D0,0xD5D2,0xD5D8,
	0xD5D9,0xD5DB,0xD5DD,0xD5E4,0xD5E5,0xD5E8,0xD5EC,0xD5F4,
	0xD5F5,0xD5F7,0xD5F9,0xD600,0xD601,0xD604,0xD608,0xD610,
	0xD611,0xD613,0xD614,0xD615,0xD61C,0xD620,
	/* Range 0x4821 - 0x487E, array index: 0x0DC7 */
	0xD624,0xD62D,0xD638,0xD639,0xD63C,0xD640,0xD645,0xD648,
	0xD649,0xD64B,0xD64D,0xD651,0xD654,0xD655,0xD658,0xD65C,
	0xD667,0xD669,0xD670,0xD671,0xD674,0xD683,0xD685,0xD68C,
	0xD68D,0xD690,0xD694,0xD69D,0xD69F,0xD6A1,0xD6A8,0xD6AC,
	0xD6B0,0xD6B9,0xD6BB,0xD6C4,0xD6C5,0xD6C8,0xD6CC,0xD6D1,
	0xD6D4,0xD6D7,0xD6D9,0xD6E0,0xD6E4,0xD6E8,0xD6F0,0xD6F5,
	0xD6FC,0xD6FD,0xD700,0xD704,0xD711,0xD718,0xD719,0xD71C,
	0xD720,0xD728,0xD729,0xD72B,0xD72D,0xD734,0xD735,0xD738,
	0xD73C,0xD744,0xD747,0xD749,0xD750,0xD751,0xD754,0xD756,
	0xD757,0xD758,0xD759,0xD760,0xD761,0xD763,0xD765,0xD769,
	0xD76C,0xD770,0xD774,0xD77C,0xD77D,0xD781,0xD788,0xD789,
	0xD78C,0xD790,0xD798,0xD799,0xD79B,0xD79D,
	/* Range 0x4A21 - 0x4A7E, array index: 0x0E25 */
	0x4F3D,0x4F73,0x5047,0x50F9,0x52A0,0x53EF,0x5475,0x54E5,
	0x5609,0x5AC1,0x5BB6,0x6687,0x67B6,0x67B7,0x67EF,0x6B4C,
	0x73C2,0x75C2,0x7A3C,0x82DB,0x8304,0x8857,0x8888,0x8A36,
	0x8CC8,0x8DCF,0x8EFB,0x8FE6,0x99D5,0x523B,0x5374,0x5404,
	0x606A,0x6164,0x6BBC,0x73CF,0x811A,0x89BA,0x89D2,0x95A3,
	0x4F83,0x520A,0x58BE,0x5978,0x59E6,0x5E72,0x5E79,0x61C7,
	0x63C0,0x6746,0x67EC,0x687F,0x6F97,0x764E,0x770B,0x78F5,
	0x7A08,0x7AFF,0x7C21,0x809D,0x826E,0x8271,0x8AEB,0x9593,
	0x4E6B,0x559D,0x66F7,0x6E34,0x78A3,0x7AED,0x845B,0x8910,
	0x874E,0x97A8,0x52D8,0x574E,0x582A,0x5D4C,0x611F,0x61BE,
	0x6221,0x6562,0x67D1,0x6A44,0x6E1B,0x7518,0x75B3,0x76E3,
	0x77B0,0x7D3A,0x90AF,0x9451,0x9452,0x9F95,
	/* Range 0x4B21 - 0x4B7E, array index: 0x0E83 */
	0x5323,0x5CAC,0x7532,0x80DB,0x9240,0x9598,0x525B,0x5808,
	0x59DC,0x5CA1,0x5D17,0x5EB7,0x5F3A,0x5F4A,0x6177,0x6C5F,
	0x757A,0x7586,0x7CE0,0x7D73,0x7DB1,0x7F8C,0x8154,0x8221,
	0x8591,0x8941,0x8B1B,0x92FC,0x964D,0x9C47,0x4ECB,0x4EF7,
	0x500B,0x51F1,0x584F,0x6137,0x613E,0x6168,0x6539,0x69EA,
	0x6F11,0x75A5,0x7686,0x76D6,0x7B87,0x82A5,0x84CB,0xF900,
	0x93A7,0x958B,0x5580,0x5BA2,0x5751,0xF901,0x7CB3,0x7FB9,
	0x91B5,0x5028,0x53BB,0x5C45,0x5DE8,0x62D2,0x636E,0x64DA,
	0x64E7,0x6E20,0x70AC,0x795B,0x8DDD,0x8E1E,0xF902,0x907D,
	0x9245,0x92F8,0x4E7E,0x4EF6,0x5065,0x5DFE,0x5EFA,0x6106,
	0x6957,0x8171,0x8654,0x8E47,0x9375,0x9A2B,0x4E5E,0x5091,
	0x6770,0x6840,0x5109,0x528D,0x5292,0x6AA2,
	/* Range 0x4C21 - 0x4C7E, array index: 0x0EE1 */
	0x77BC,0x9210,0x9ED4,0x52AB,0x602F,0x8FF2,0x5048,0x61A9,
	0x63ED,0x64CA,0x683C,0x6A84,0x6FC0,0x8188,0x89A1,0x9694,
	0x5805,0x727D,0x72AC,0x7504,0x7D79,0x7E6D,0x80A9,0x898B,
	0x8B74,0x9063,0x9D51,0x6289,0x6C7A,0x6F54,0x7D50,0x7F3A,
	0x8A23,0x517C,0x614A,0x7B9D,0x8B19,0x9257,0x938C,0x4EAC,
	0x4FD3,0x501E,0x50BE,0x5106,0x52C1,0x52CD,0x537F,0x5770,
	0x5883,0x5E9A,0x5F91,0x6176,0x61AC,0x64CE,0x656C,0x666F,
	0x66BB,0x66F4,0x6897,0x6D87,0x7085,0x70F1,0x749F,0x74A5,
	0x74CA,0x75D9,0x786C,0x78EC,0x7ADF,0x7AF6,0x7D45,0x7D93,
	0x8015,0x803F,0x811B,0x8396,0x8B66,0x8F15,0x9015,0x93E1,
	0x9803,0x9838,0x9A5A,0x9BE8,0x4FC2,0x5553,0x583A,0x5951,
	0x5B63,0x5C46,0x60B8,0x6212,0x6842,0x68B0,
	/* Range 0x4D21 - 0x4D7E, array index: 0x0F3F */
	0x68E8,0x6EAA,0x754C,0x7678,0x78CE,0x7A3D,0x7CFB,0x7E6B,
	0x7E7C,0x8A08,0x8AA1,0x8C3F,0x968E,0x9DC4,0x53E4,0x53E9,
	0x544A,0x5471,0x56FA,0x59D1,0x5B64,0x5C3B,0x5EAB,0x62F7,
	0x6537,0x6545,0x6572,0x66A0,0x67AF,0x69C1,0x6CBD,0x75FC,
	0x7690,0x777E,0x7A3F,0x7F94,0x8003,0x80A1,0x818F,0x82E6,
	0x82FD,0x83F0,0x85C1,0x8831,0x88B4,0x8AA5,0xF903,0x8F9C,
	0x932E,0x96C7,0x9867,0x9AD8,0x9F13,0x54ED,0x659B,0x66F2,
	0x688F,0x7A40,0x8C37,0x9D60,0x56F0,0x5764,0x5D11,0x6606,
	0x68B1,0x68CD,0x6EFE,0x7428,0x889E,0x9BE4,0x6C68,0xF904,
	0x9AA8,0x4F9B,0x516C,0x5171,0x529F,0x5B54,0x5DE5,0x6050,
	0x606D,0x62F1,0x63A7,0x653B,0x73D9,0x7A7A,0x86A3,0x8CA2,
	0x978F,0x4E32,0x5BE1,0x6208,0x679C,0x74DC,
	/* Range 0x4E21 - 0x4E7E, array index: 0x0F9D */
	0x79D1,0x83D3,0x8A87,0x8AB2,0x8DE8,0x904E,0x934B,0x9846,
	0x5ED3,0x69E8,0x85FF,0x90ED,0xF905,0x51A0,0x5B98,0x5BEC,
	0x6163,0x68FA,0x6B3E,0x704C,0x742F,0x74D8,0x7BA1,0x7F50,
	0x83C5,0x89C0,0x8CAB,0x95DC,0x9928,0x522E,0x605D,0x62EC,
	0x9002,0x4F8A,0x5149,0x5321,0x58D9,0x5EE3,0x66E0,0x6D38,
	0x709A,0x72C2,0x73D6,0x7B50,0x80F1,0x945B,0x5366,0x639B,
	0x7F6B,0x4E56,0x5080,0x584A,0x58DE,0x602A,0x6127,0x62D0,
	0x69D0,0x9B41,0x5B8F,0x7D18,0x80B1,0x8F5F,0x4EA4,0x50D1,
	0x54AC,0x55AC,0x5B0C,0x5DA0,0x5DE7,0x652A,0x654E,0x6821,
	0x6A4B,0x72E1,0x768E,0x77EF,0x7D5E,0x7FF9,0x81A0,0x854E,
	0x86DF,0x8F03,0x8F4E,0x90CA,0x9903,0x9A55,0x9BAB,0x4E18,
	0x4E45,0x4E5D,0x4EC7,0x4FF1,0x5177,0x52FE,
	/* Range 0x4F21 - 0x4F7E, array index: 0x0FFB */
	0x5340,0x53E3,0x53E5,0x548E,0x5614,0x5775,0x57A2,0x5BC7,
	0x5D87,0x5ED0,0x61FC,0x62D8,0x6551,0x67B8,0x67E9,0x69CB,
	0x6B50,0x6BC6,0x6BEC,0x6C42,0x6E9D,0x7078,0x72D7,0x7396,
	0x7403,0x77BF,0x77E9,0x7A76,0x7D7F,0x8009,0x81FC,0x8205,
	0x820A,0x82DF,0x8862,0x8B33,0x8CFC,0x8EC0,0x9011,0x90B1,
	0x9264,0x92B6,0x99D2,0x9A45,0x9CE9,0x9DD7,0x9F9C,0x570B,
	0x5C40,0x83CA,0x97A0,0x97AB,0x9EB4,0x541B,0x7A98,0x7FA4,
	0x88D9,0x8ECD,0x90E1,0x5800,0x5C48,0x6398,0x7A9F,0x5BAE,
	0x5F13,0x7A79,0x7AAE,0x828E,0x8EAC,0x5026,0x5238,0x52F8,
	0x5377,0x5708,0x62F3,0x6372,0x6B0A,0x6DC3,0x7737,0x53A5,
	0x7357,0x8568,0x8E76,0x95D5,0x673A,0x6AC3,0x6F70,0x8A6D,
	0x8ECC,0x994B,0xF906,0x6677,0x6B78,0x8CB4,
	/* Range 0x5021 - 0x507E, array index: 0x1059 */
	0x9B3C,0xF907,0x53EB,0x572D,0x594E,0x63C6,0x69FB,0x73EA,
	0x7845,0x7ABA,0x7AC5,0x7CFE,0x8475,0x898F,0x8D73,0x9035,
	0x95A8,0x52FB,0x5747,0x7547,0x7B60,0x83CC,0x921E,0xF908,
	0x6A58,0x514B,0x524B,0x5287,0x621F,0x68D8,0x6975,0x9699,
	0x50C5,0x52A4,0x52E4,0x61C3,0x65A4,0x6839,0x69FF,0x747E,
	0x7B4B,0x82B9,0x83EB,0x89B2,0x8B39,0x8FD1,0x9949,0xF909,
	0x4ECA,0x5997,0x64D2,0x6611,0x6A8E,0x7434,0x7981,0x79BD,
	0x82A9,0x887E,0x887F,0x895F,0xF90A,0x9326,0x4F0B,0x53CA,
	0x6025,0x6271,0x6C72,0x7D1A,0x7D66,0x4E98,0x5162,0x77DC,
	0x80AF,0x4F01,0x4F0E,0x5176,0x5180,0x55DC,0x5668,0x573B,
	0x57FA,0x57FC,0x5914,0x5947,0x5993,0x5BC4,0x5C90,0x5D0E,
	0x5DF1,0x5E7E,0x5FCC,0x6280,0x65D7,0x65E3,
	/* Range 0x5121 - 0x517E, array index: 0x10B7 */
	0x671E,0x671F,0x675E,0x68CB,0x68C4,0x6A5F,0x6B3A,0x6C23,
	0x6C7D,0x6C82,0x6DC7,0x7398,0x7426,0x742A,0x7482,0x74A3,
	0x7578,0x757F,0x7881,0x78EF,0x7941,0x7947,0x7948,0x797A,
	0x7B95,0x7D00,0x7DBA,0x7F88,0x8006,0x802D,0x808C,0x8A18,
	0x8B4F,0x8C48,0x8D77,0x9321,0x9324,0x98E2,0x9951,0x9A0E,
	0x9A0F,0x9A65,0x9E92,0x7DCA,0x4F76,0x5409,0x62EE,0x6854,
	0x91D1,0x55AB,0x513A,0xF90B,0xF90C,0x5A1C,0x61E6,0xF90D,
	0x62CF,0x62FF,0xF90E,0xF90F,0xF910,0xF911,0xF912,0xF913,
	0x90A3,0xF914,0xF915,0xF916,0xF917,0xF918,0x8AFE,0xF919,
	0xF91A,0xF91B,0xF91C,0x6696,0xF91D,0x7156,0xF91E,0xF91F,
	0x96E3,0xF920,0x634F,0x637A,0x5357,0xF921,0x678F,0x6960,
	0x6E73,0xF922,0x7537,0xF923,0xF924,0xF925,
	/* Range 0x5221 - 0x527E, array index: 0x1115 */
	0x7D0D,0xF926,0xF927,0x8872,0x56CA,0x5A18,0xF928,0xF929,
	0xF92A,0xF92B,0xF92C,0x4E43,0xF92D,0x5167,0x5948,0x67F0,
	0x8010,0xF92E,0x5973,0x5E74,0x649A,0x79CA,0x5FF5,0x606C,
	0x62C8,0x637B,0x5BE7,0x5BD7,0x52AA,0xF92F,0x5974,0x5F29,
	0x6012,0xF930,0xF931,0xF932,0x7459,0xF933,0xF934,0xF935,
	0xF936,0xF937,0xF938,0x99D1,0xF939,0xF93A,0xF93B,0xF93C,
	0xF93D,0xF93E,0xF93F,0xF940,0xF941,0xF942,0xF943,0x6FC3,
	0xF944,0xF945,0x81BF,0x8FB2,0x60F1,0xF946,0xF947,0x8166,
	0xF948,0xF949,0x5C3F,0xF94A,0xF94B,0xF94C,0xF94D,0xF94E,
	0xF94F,0xF950,0xF951,0x5AE9,0x8A25,0x677B,0x7D10,0xF952,
	0xF953,0xF954,0xF955,0xF956,0xF957,0x80FD,0xF958,0xF959,
	0x5C3C,0x6CE5,0x533F,0x6EBA,0x591A,0x8336,
	/* Range 0x5321 - 0x537E, array index: 0x1173 */
	0x4E39,0x4EB6,0x4F46,0x55AE,0x5718,0x58C7,0x5F56,0x65B7,
	0x65E6,0x6A80,0x6BB5,0x6E4D,0x77ED,0x7AEF,0x7C1E,0x7DDE,
	0x86CB,0x8892,0x9132,0x935B,0x64BB,0x6FBE,0x737A,0x75B8,
	0x9054,0x5556,0x574D,0x61BA,0x64D4,0x66C7,0x6DE1,0x6E5B,
	0x6F6D,0x6FB9,0x75F0,0x8043,0x81BD,0x8541,0x8983,0x8AC7,
	0x8B5A,0x931F,0x6C93,0x7553,0x7B54,0x8E0F,0x905D,0x5510,
	0x5802,0x5858,0x5E62,0x6207,0x649E,0x68E0,0x7576,0x7CD6,
	0x87B3,0x9EE8,0x4EE3,0x5788,0x576E,0x5927,0x5C0D,0x5CB1,
	0x5E36,0x5F85,0x6234,0x64E1,0x73B3,0x81FA,0x888B,0x8CB8,
	0x968A,0x9EDB,0x5B85,0x5FB7,0x60B3,0x5012,0x5200,0x5230,
	0x5716,0x5835,0x5857,0x5C0E,0x5C60,0x5CF6,0x5D8B,0x5EA6,
	0x5F92,0x60BC,0x6311,0x6389,0x6417,0x6843,
	/* Range 0x5421 - 0x547E, array index: 0x11D1 */
	0x68F9,0x6AC2,0x6DD8,0x6E21,0x6ED4,0x6FE4,0x71FE,0x76DC,
	0x7779,0x79B1,0x7A3B,0x8404,0x89A9,0x8CED,0x8DF3,0x8E48,
	0x9003,0x9014,0x9053,0x90FD,0x934D,0x9676,0x97DC,0x6BD2,
	0x7006,0x7258,0x72A2,0x7368,0x7763,0x79BF,0x7BE4,0x7E9B,
	0x8B80,0x58A9,0x60C7,0x6566,0x65FD,0x66BE,0x6C8C,0x711E,
	0x71C9,0x8C5A,0x9813,0x4E6D,0x7A81,0x4EDD,0x51AC,0x51CD,
	0x52D5,0x540C,0x61A7,0x6771,0x6850,0x68DF,0x6D1E,0x6F7C,
	0x75BC,0x77B3,0x7AE5,0x80F4,0x8463,0x9285,0x515C,0x6597,
	0x675C,0x6793,0x75D8,0x7AC7,0x8373,0xF95A,0x8C46,0x9017,
	0x982D,0x5C6F,0x81C0,0x829A,0x9041,0x906F,0x920D,0x5F97,
	0x5D9D,0x6A59,0x71C8,0x767B,0x7B49,0x85E4,0x8B04,0x9127,
	0x9A30,0x5587,0x61F6,0xF95B,0x7669,0x7F85,
	/* Range 0x5521 - 0x557E, array index: 0x122F */
	0x863F,0x87BA,0x88F8,0x908F,0xF95C,0x6D1B,0x70D9,0x73DE,
	0x7D61,0x843D,0xF95D,0x916A,0x99F1,0xF95E,0x4E82,0x5375,
	0x6B04,0x6B12,0x703E,0x721B,0x862D,0x9E1E,0x524C,0x8FA3,
	0x5D50,0x64E5,0x652C,0x6B16,0x6FEB,0x7C43,0x7E9C,0x85CD,
	0x8964,0x89BD,0x62C9,0x81D8,0x881F,0x5ECA,0x6717,0x6D6A,
	0x72FC,0x7405,0x746F,0x8782,0x90DE,0x4F86,0x5D0D,0x5FA0,
	0x840A,0x51B7,0x63A0,0x7565,0x4EAE,0x5006,0x5169,0x51C9,
	0x6881,0x6A11,0x7CAE,0x7CB1,0x7CE7,0x826F,0x8AD2,0x8F1B,
	0x91CF,0x4FB6,0x5137,0x52F5,0x5442,0x5EEC,0x616E,0x623E,
	0x65C5,0x6ADA,0x6FFE,0x792A,0x85DC,0x8823,0x95AD,0x9A62,
	0x9A6A,0x9E97,0x9ECE,0x529B,0x66C6,0x6B77,0x701D,0x792B,
	0x8F62,0x9742,0x6190,0x6200,0x6523,0x6F23,
	/* Range 0x5621 - 0x567E, array index: 0x128D */
	0x7149,0x7489,0x7DF4,0x806F,0x84EE,0x8F26,0x9023,0x934A,
	0x51BD,0x5217,0x52A3,0x6D0C,0x70C8,0x88C2,0x5EC9,0x6582,
	0x6BAE,0x6FC2,0x7C3E,0x7375,0x4EE4,0x4F36,0x56F9,0xF95F,
	0x5CBA,0x5DBA,0x601C,0x73B2,0x7B2D,0x7F9A,0x7FCE,0x8046,
	0x901E,0x9234,0x96F6,0x9748,0x9818,0x9F61,0x4F8B,0x6FA7,
	0x79AE,0x91B4,0x96B7,0x52DE,0xF960,0x6488,0x64C4,0x6AD3,
	0x6F5E,0x7018,0x7210,0x76E7,0x8001,0x8606,0x865C,0x8DEF,
	0x8F05,0x9732,0x9B6F,0x9DFA,0x9E75,0x788C,0x797F,0x7DA0,
	0x83C9,0x9304,0x9E7F,0x9E93,0x8AD6,0x58DF,0x5F04,0x6727,
	0x7027,0x74CF,0x7C60,0x807E,0x5121,0x7028,0x7262,0x78CA,
	0x8CC2,0x8CDA,0x8CF4,0x96F7,0x4E86,0x50DA,0x5BEE,0x5ED6,
	0x6599,0x71CE,0x7642,0x77AD,0x804A,0x84FC,
	/* Range 0x5721 - 0x577E, array index: 0x12EB */
	0x907C,0x9B27,0x9F8D,0x58D8,0x5A41,0x5C62,0x6A13,0x6DDA,
	0x6F0F,0x763B,0x7D2F,0x7E37,0x851E,0x8938,0x93E4,0x964B,
	0x5289,0x65D2,0x67F3,0x69B4,0x6D41,0x6E9C,0x700F,0x7409,
	0x7460,0x7559,0x7624,0x786B,0x8B2C,0x985E,0x516D,0x622E,
	0x9678,0x4F96,0x502B,0x5D19,0x6DEA,0x7DB8,0x8F2A,0x5F8B,
	0x6144,0x6817,0xF961,0x9686,0x52D2,0x808B,0x51DC,0x51CC,
	0x695E,0x7A1C,0x7DBE,0x83F1,0x9675,0x4FDA,0x5229,0x5398,
	0x540F,0x550E,0x5C65,0x60A7,0x674E,0x68A8,0x6D6C,0x7281,
	0x72F8,0x7406,0x7483,0xF962,0x75E2,0x7C6C,0x7F79,0x7FB8,
	0x8389,0x88CF,0x88E1,0x91CC,0x91D0,0x96E2,0x9BC9,0x541D,
	0x6F7E,0x71D0,0x7498,0x85FA,0x8EAA,0x96A3,0x9C57,0x9E9F,
	0x6797,0x6DCB,0x7433,0x81E8,0x9716,0x782C,
	/* Range 0x5821 - 0x587E, array index: 0x1349 */
	0x7ACB,0x7B20,0x7C92,0x6469,0x746A,0x75F2,0x78BC,0x78E8,
	0x99AC,0x9B54,0x9EBB,0x5BDE,0x5E55,0x6F20,0x819C,0x83AB,
	0x9088,0x4E07,0x534D,0x5A29,0x5DD2,0x5F4E,0x6162,0x633D,
	0x6669,0x66FC,0x6EFF,0x6F2B,0x7063,0x779E,0x842C,0x8513,
	0x883B,0x8F13,0x9945,0x9C3B,0x551C,0x62B9,0x672B,0x6CAB,
	0x8309,0x896A,0x977A,0x4EA1,0x5984,0x5FD8,0x5FD9,0x671B,
	0x7DB2,0x7F54,0x8292,0x832B,0x83BD,0x8F1E,0x9099,0x57CB,
	0x59B9,0x5A92,0x5BD0,0x6627,0x679A,0x6885,0x6BCF,0x7164,
	0x7F75,0x8CB7,0x8CE3,0x9081,0x9B45,0x8108,0x8C8A,0x964C,
	0x9A40,0x9EA5,0x5B5F,0x6C13,0x731B,0x76F2,0x76DF,0x840C,
	0x51AA,0x8993,0x514D,0x5195,0x52C9,0x68C9,0x6C94,0x7704,
	0x7720,0x7DBF,0x7DEC,0x9762,0x9EB5,0x6EC5,
	/* Range 0x5921 - 0x597E, array index: 0x13A7 */
	0x8511,0x51A5,0x540D,0x547D,0x660E,0x669D,0x6927,0x6E9F,
	0x76BF,0x7791,0x8317,0x84C2,0x879F,0x9169,0x9298,0x9CF4,
	0x8882,0x4FAE,0x5192,0x52DF,0x59C6,0x5E3D,0x6155,0x6478,
	0x6479,0x66AE,0x67D0,0x6A21,0x6BCD,0x6BDB,0x725F,0x7261,
	0x7441,0x7738,0x77DB,0x8017,0x82BC,0x8305,0x8B00,0x8B28,
	0x8C8C,0x6728,0x6C90,0x7267,0x76EE,0x7766,0x7A46,0x9DA9,
	0x6B7F,0x6C92,0x5922,0x6726,0x8499,0x536F,0x5893,0x5999,
	0x5EDF,0x63CF,0x6634,0x6773,0x6E3A,0x732B,0x7AD7,0x82D7,
	0x9328,0x52D9,0x5DEB,0x61AE,0x61CB,0x620A,0x62C7,0x64AB,
	0x65E0,0x6959,0x6B66,0x6BCB,0x7121,0x73F7,0x755D,0x7E46,
	0x821E,0x8302,0x856A,0x8AA3,0x8CBF,0x9727,0x9D61,0x58A8,
	0x9ED8,0x5011,0x520E,0x543B,0x554F,0x6587,
	/* Range 0x5A21 - 0x5A7E, array index: 0x1405 */
	0x6C76,0x7D0A,0x7D0B,0x805E,0x868A,0x9580,0x96EF,0x52FF,
	0x6C95,0x7269,0x5473,0x5A9A,0x5C3E,0x5D4B,0x5F4C,0x5FAE,
	0x672A,0x68B6,0x6963,0x6E3C,0x6E44,0x7709,0x7C73,0x7F8E,
	0x8587,0x8B0E,0x8FF7,0x9761,0x9EF4,0x5CB7,0x60B6,0x610D,
	0x61AB,0x654F,0x65FB,0x65FC,0x6C11,0x6CEF,0x739F,0x73C9,
	0x7DE1,0x9594,0x5BC6,0x871C,0x8B10,0x525D,0x535A,0x62CD,
	0x640F,0x64B2,0x6734,0x6A38,0x6CCA,0x73C0,0x749E,0x7B94,
	0x7C95,0x7E1B,0x818A,0x8236,0x8584,0x8FEB,0x96F9,0x99C1,
	0x4F34,0x534A,0x53CD,0x53DB,0x62CC,0x642C,0x6500,0x6591,
	0x69C3,0x6CEE,0x6F58,0x73ED,0x7554,0x7622,0x76E4,0x76FC,
	0x78D0,0x78FB,0x792C,0x7D46,0x822C,0x87E0,0x8FD4,0x9812,
	0x98EF,0x52C3,0x62D4,0x64A5,0x6E24,0x6F51,
	/* Range 0x5B21 - 0x5B7E, array index: 0x1463 */
	0x767C,0x8DCB,0x91B1,0x9262,0x9AEE,0x9B43,0x5023,0x508D,
	0x574A,0x59A8,0x5C28,0x5E47,0x5F77,0x623F,0x653E,0x65B9,
	0x65C1,0x6609,0x678B,0x699C,0x6EC2,0x78C5,0x7D21,0x80AA,
	0x8180,0x822B,0x82B3,0x84A1,0x868C,0x8A2A,0x8B17,0x90A6,
	0x9632,0x9F90,0x500D,0x4FF3,0xF963,0x57F9,0x5F98,0x62DC,
	0x6392,0x676F,0x6E43,0x7119,0x76C3,0x80CC,0x80DA,0x88F4,
	0x88F5,0x8919,0x8CE0,0x8F29,0x914D,0x966A,0x4F2F,0x4F70,
	0x5E1B,0x67CF,0x6822,0x767D,0x767E,0x9B44,0x5E61,0x6A0A,
	0x7169,0x71D4,0x756A,0xF964,0x7E41,0x8543,0x85E9,0x98DC,
	0x4F10,0x7B4F,0x7F70,0x95A5,0x51E1,0x5E06,0x68B5,0x6C3E,
	0x6C4E,0x6CDB,0x72AF,0x7BC4,0x8303,0x6CD5,0x743A,0x50FB,
	0x5288,0x58C1,0x64D8,0x6A97,0x74A7,0x7656,
	/* Range 0x5C21 - 0x5C7E, array index: 0x14C1 */
	0x78A7,0x8617,0x95E2,0x9739,0xF965,0x535E,0x5F01,0x8B8A,
	0x8FA8,0x8FAF,0x908A,0x5225,0x77A5,0x9C49,0x9F08,0x4E19,
	0x5002,0x5175,0x5C5B,0x5E77,0x661E,0x663A,0x67C4,0x68C5,
	0x70B3,0x7501,0x75C5,0x79C9,0x7ADD,0x8F27,0x9920,0x9A08,
	0x4FDD,0x5821,0x5831,0x5BF6,0x666E,0x6B65,0x6D11,0x6E7A,
	0x6F7D,0x73E4,0x752B,0x83E9,0x88DC,0x8913,0x8B5C,0x8F14,
	0x4F0F,0x50D5,0x5310,0x535C,0x5B93,0x5FA9,0x670D,0x798F,
	0x8179,0x832F,0x8514,0x8907,0x8986,0x8F39,0x8F3B,0x99A5,
	0x9C12,0x672C,0x4E76,0x4FF8,0x5949,0x5C01,0x5CEF,0x5CF0,
	0x6367,0x68D2,0x70FD,0x71A2,0x742B,0x7E2B,0x84EC,0x8702,
	0x9022,0x92D2,0x9CF3,0x4E0D,0x4ED8,0x4FEF,0x5085,0x5256,
	0x526F,0x5426,0x5490,0x57E0,0x592B,0x5A66,
	/* Range 0x5D21 - 0x5D7E, array index: 0x151F */
	0x5B5A,0x5B75,0x5BCC,0x5E9C,0xF966,0x6276,0x6577,0x65A7,
	0x6D6E,0x6EA5,0x7236,0x7B26,0x7C3F,0x7F36,0x8150,0x8151,
	0x819A,0x8240,0x8299,0x83A9,0x8A03,0x8CA0,0x8CE6,0x8CFB,
	0x8D74,0x8DBA,0x90E8,0x91DC,0x961C,0x9644,0x99D9,0x9CE7,
	0x5317,0x5206,0x5429,0x5674,0x58B3,0x5954,0x596E,0x5FFF,
	0x61A4,0x626E,0x6610,0x6C7E,0x711A,0x76C6,0x7C89,0x7CDE,
	0x7D1B,0x82AC,0x8CC1,0x96F0,0xF967,0x4F5B,0x5F17,0x5F7F,
	0x62C2,0x5D29,0x670B,0x68DA,0x787C,0x7E43,0x9D6C,0x4E15,
	0x5099,0x5315,0x532A,0x5351,0x5983,0x5A62,0x5E87,0x60B2,
	0x618A,0x6249,0x6279,0x6590,0x6787,0x69A7,0x6BD4,0x6BD6,
	0x6BD7,0x6BD8,0x6CB8,0xF968,0x7435,0x75FA,0x7812,0x7891,
	0x79D5,0x79D8,0x7C83,0x7DCB,0x7FE1,0x80A5,
	/* Range 0x5E21 - 0x5E7E, array index: 0x157D */
	0x813E,0x81C2,0x83F2,0x871A,0x88E8,0x8AB9,0x8B6C,0x8CBB,
	0x9119,0x975E,0x98DB,0x9F3B,0x56AC,0x5B2A,0x5F6C,0x658C,
	0x6AB3,0x6BAF,0x6D5C,0x6FF1,0x7015,0x725D,0x73AD,0x8CA7,
	0x8CD3,0x983B,0x6191,0x6C37,0x8058,0x9A01,0x4E4D,0x4E8B,
	0x4E9B,0x4ED5,0x4F3A,0x4F3C,0x4F7F,0x4FDF,0x50FF,0x53F2,
	0x53F8,0x5506,0x55E3,0x56DB,0x58EB,0x5962,0x5A11,0x5BEB,
	0x5BFA,0x5C04,0x5DF3,0x5E2B,0x5F99,0x601D,0x6368,0x659C,
	0x65AF,0x67F6,0x67FB,0x68AD,0x6B7B,0x6C99,0x6CD7,0x6E23,
	0x7009,0x7345,0x7802,0x793E,0x7940,0x7960,0x79C1,0x7BE9,
	0x7D17,0x7D72,0x8086,0x820D,0x838E,0x84D1,0x86C7,0x88DF,
	0x8A50,0x8A5E,0x8B1D,0x8CDC,0x8D66,0x8FAD,0x90AA,0x98FC,
	0x99DF,0x9E9D,0x524A,0xF969,0x6714,0xF96A,
	/* Range 0x5F21 - 0x5F7E, array index: 0x15DB */
	0x5098,0x522A,0x5C71,0x6563,0x6C55,0x73CA,0x7523,0x759D,
	0x7B97,0x849C,0x9178,0x9730,0x4E77,0x6492,0x6BBA,0x715E,
	0x85A9,0x4E09,0xF96B,0x6749,0x68EE,0x6E17,0x829F,0x8518,
	0x886B,0x63F7,0x6F81,0x9212,0x98AF,0x4E0A,0x50B7,0x50CF,
	0x511F,0x5546,0x55AA,0x5617,0x5B40,0x5C19,0x5CE0,0x5E38,
	0x5E8A,0x5EA0,0x5EC2,0x60F3,0x6851,0x6A61,0x6E58,0x723D,
	0x7240,0x72C0,0x76F8,0x7965,0x7BB1,0x7FD4,0x88F3,0x89F4,
	0x8A73,0x8C61,0x8CDE,0x971C,0x585E,0x74BD,0x8CFD,0x55C7,
	0xF96C,0x7A61,0x7D22,0x8272,0x7272,0x751F,0x7525,0xF96D,
	0x7B19,0x5885,0x58FB,0x5DBC,0x5E8F,0x5EB6,0x5F90,0x6055,
	0x6292,0x637F,0x654D,0x6691,0x66D9,0x66F8,0x6816,0x68F2,
	0x7280,0x745E,0x7B6E,0x7D6E,0x7DD6,0x7F72,
	/* Range 0x6021 - 0x607E, array index: 0x1639 */
	0x80E5,0x8212,0x85AF,0x897F,0x8A93,0x901D,0x92E4,0x9ECD,
	0x9F20,0x5915,0x596D,0x5E2D,0x60DC,0x6614,0x6673,0x6790,
	0x6C50,0x6DC5,0x6F5F,0x77F3,0x78A9,0x84C6,0x91CB,0x932B,
	0x4ED9,0x50CA,0x5148,0x5584,0x5B0B,0x5BA3,0x6247,0x657E,
	0x65CB,0x6E32,0x717D,0x7401,0x7444,0x7487,0x74BF,0x766C,
	0x79AA,0x7DDA,0x7E55,0x7FA8,0x817A,0x81B3,0x8239,0x861A,
	0x87EC,0x8A75,0x8DE3,0x9078,0x9291,0x9425,0x994D,0x9BAE,
	0x5368,0x5C51,0x6954,0x6CC4,0x6D29,0x6E2B,0x820C,0x859B,
	0x893B,0x8A2D,0x8AAA,0x96EA,0x9F67,0x5261,0x66B9,0x6BB2,
	0x7E96,0x87FE,0x8D0D,0x9583,0x965D,0x651D,0x6D89,0x71EE,
	0xF96E,0x57CE,0x59D3,0x5BAC,0x6027,0x60FA,0x6210,0x661F,
	0x665F,0x7329,0x73F9,0x76DB,0x7701,0x7B6C,
	/* Range 0x6121 - 0x617E, array index: 0x1697 */
	0x8056,0x8072,0x8165,0x8AA0,0x9192,0x4E16,0x52E2,0x6B72,
	0x6D17,0x7A05,0x7B39,0x7D30,0xF96F,0x8CB0,0x53EC,0x562F,
	0x5851,0x5BB5,0x5C0F,0x5C11,0x5DE2,0x6240,0x6383,0x6414,
	0x662D,0x68B3,0x6CBC,0x6D88,0x6EAF,0x701F,0x70A4,0x71D2,
	0x7526,0x758F,0x758E,0x7619,0x7B11,0x7BE0,0x7C2B,0x7D20,
	0x7D39,0x852C,0x856D,0x8607,0x8A34,0x900D,0x9061,0x90B5,
	0x92B7,0x97F6,0x9A37,0x4FD7,0x5C6C,0x675F,0x6D91,0x7C9F,
	0x7E8C,0x8B16,0x8D16,0x901F,0x5B6B,0x5DFD,0x640D,0x84C0,
	0x905C,0x98E1,0x7387,0x5B8B,0x609A,0x677E,0x6DDE,0x8A1F,
	0x8AA6,0x9001,0x980C,0x5237,0xF970,0x7051,0x788E,0x9396,
	0x8870,0x91D7,0x4FEE,0x53D7,0x55FD,0x56DA,0x5782,0x58FD,
	0x5AC2,0x5B88,0x5CAB,0x5CC0,0x5E25,0x6101,
	/* Range 0x6221 - 0x627E, array index: 0x16F5 */
	0x620D,0x624B,0x6388,0x641C,0x6536,0x6578,0x6A39,0x6B8A,
	0x6C34,0x6D19,0x6F31,0x71E7,0x72E9,0x7378,0x7407,0x74B2,
	0x7626,0x7761,0x79C0,0x7A57,0x7AEA,0x7CB9,0x7D8F,0x7DAC,
	0x7E61,0x7F9E,0x8129,0x8331,0x8490,0x84DA,0x85EA,0x8896,
	0x8AB0,0x8B90,0x8F38,0x9042,0x9083,0x916C,0x9296,0x92B9,
	0x968B,0x96A7,0x96A8,0x96D6,0x9700,0x9808,0x9996,0x9AD3,
	0x9B1A,0x53D4,0x587E,0x5919,0x5B70,0x5BBF,0x6DD1,0x6F5A,
	0x719F,0x7421,0x74B9,0x8085,0x83FD,0x5DE1,0x5F87,0x5FAA,
	0x6042,0x65EC,0x6812,0x696F,0x6A53,0x6B89,0x6D35,0x6DF3,
	0x73E3,0x76FE,0x77AC,0x7B4D,0x7D14,0x8123,0x821C,0x8340,
	0x84F4,0x8563,0x8A62,0x8AC4,0x9187,0x931E,0x9806,0x99B4,
	0x620C,0x8853,0x8FF0,0x9265,0x5D07,0x5D27,
	/* Range 0x6321 - 0x637E, array index: 0x1753 */
	0x5D69,0x745F,0x819D,0x8768,0x6FD5,0x62FE,0x7FD2,0x8936,
	0x8972,0x4E1E,0x4E58,0x50E7,0x52DD,0x5347,0x627F,0x6607,
	0x7E69,0x8805,0x965E,0x4F8D,0x5319,0x5636,0x59CB,0x5AA4,
	0x5C38,0x5C4E,0x5C4D,0x5E02,0x5F11,0x6043,0x65BD,0x662F,
	0x6642,0x67BE,0x67F4,0x731C,0x77E2,0x793A,0x7FC5,0x8494,
	0x84CD,0x8996,0x8A66,0x8A69,0x8AE1,0x8C55,0x8C7A,0x57F4,
	0x5BD4,0x5F0F,0x606F,0x62ED,0x690D,0x6B96,0x6E5C,0x7184,
	0x7BD2,0x8755,0x8B58,0x8EFE,0x98DF,0x98FE,0x4F38,0x4F81,
	0x4FE1,0x547B,0x5A20,0x5BB8,0x613C,0x65B0,0x6668,0x71FC,
	0x7533,0x795E,0x7D33,0x814E,0x81E3,0x8398,0x85AA,0x85CE,
	0x8703,0x8A0A,0x8EAB,0x8F9B,0xF971,0x8FC5,0x5931,0x5BA4,
	0x5BE6,0x6089,0x5BE9,0x5C0B,0x5FC3,0x6C81,
	/* Range 0x6421 - 0x647E, array index: 0x17B1 */
	0xF972,0x6DF1,0x700B,0x751A,0x82AF,0x8AF6,0x4EC0,0x5341,
	0xF973,0x96D9,0x6C0F,0x4E9E,0x4FC4,0x5152,0x555E,0x5A25,
	0x5CE8,0x6211,0x7259,0x82BD,0x83AA,0x86FE,0x8859,0x8A1D,
	0x963F,0x96C5,0x9913,0x9D09,0x9D5D,0x580A,0x5CB3,0x5DBD,
	0x5E44,0x60E1,0x6115,0x63E1,0x6A02,0x6E25,0x9102,0x9354,
	0x984E,0x9C10,0x9F77,0x5B89,0x5CB8,0x6309,0x664F,0x6848,
	0x773C,0x96C1,0x978D,0x9854,0x9B9F,0x65A1,0x8B01,0x8ECB,
	0x95BC,0x5535,0x5CA9,0x5DD6,0x5EB5,0x6697,0x764C,0x83F4,
	0x95C7,0x58D3,0x62BC,0x72CE,0x9D28,0x4EF0,0x592E,0x600F,
	0x663B,0x6B83,0x79E7,0x9D26,0x5393,0x54C0,0x57C3,0x5D16,
	0x611B,0x66D6,0x6DAF,0x788D,0x827E,0x9698,0x9744,0x5384,
	0x627C,0x6396,0x6DB2,0x7E0A,0x814B,0x984D,
	/* Range 0x6521 - 0x657E, array index: 0x180F */
	0x6AFB,0x7F4C,0x9DAF,0x9E1A,0x4E5F,0x503B,0x51B6,0x591C,
	0x60F9,0x63F6,0x6930,0x723A,0x8036,0xF974,0x91CE,0x5F31,
	0xF975,0xF976,0x7D04,0x82E5,0x846F,0x84BB,0x85E5,0x8E8D,
	0xF977,0x4F6F,0xF978,0xF979,0x58E4,0x5B43,0x6059,0x63DA,
	0x6518,0x656D,0x6698,0xF97A,0x694A,0x6A23,0x6D0B,0x7001,
	0x716C,0x75D2,0x760D,0x79B3,0x7A70,0xF97B,0x7F8A,0xF97C,
	0x8944,0xF97D,0x8B93,0x91C0,0x967D,0xF97E,0x990A,0x5704,
	0x5FA1,0x65BC,0x6F01,0x7600,0x79A6,0x8A9E,0x99AD,0x9B5A,
	0x9F6C,0x5104,0x61B6,0x6291,0x6A8D,0x81C6,0x5043,0x5830,
	0x5F66,0x7109,0x8A00,0x8AFA,0x5B7C,0x8616,0x4FFA,0x513C,
	0x56B4,0x5944,0x63A9,0x6DF9,0x5DAA,0x696D,0x5186,0x4E88,
	0x4F59,0xF97F,0xF980,0xF981,0x5982,0xF982,
	/* Range 0x6621 - 0x667E, array index: 0x186D */
	0xF983,0x6B5F,0x6C5D,0xF984,0x74B5,0x7916,0xF985,0x8207,
	0x8245,0x8339,0x8F3F,0x8F5D,0xF986,0x9918,0xF987,0xF988,
	0xF989,0x4EA6,0xF98A,0x57DF,0x5F79,0x6613,0xF98B,0xF98C,
	0x75AB,0x7E79,0x8B6F,0xF98D,0x9006,0x9A5B,0x56A5,0x5827,
	0x59F8,0x5A1F,0x5BB4,0xF98E,0x5EF6,0xF98F,0xF990,0x6350,
	0x633B,0xF991,0x693D,0x6C87,0x6CBF,0x6D8E,0x6D93,0x6DF5,
	0x6F14,0xF992,0x70DF,0x7136,0x7159,0xF993,0x71C3,0x71D5,
	0xF994,0x784F,0x786F,0xF995,0x7B75,0x7DE3,0xF996,0x7E2F,
	0xF997,0x884D,0x8EDF,0xF998,0xF999,0xF99A,0x925B,0xF99B,
	0x9CF6,0xF99C,0xF99D,0xF99E,0x6085,0x6D85,0xF99F,0x71B1,
	0xF9A0,0xF9A1,0x95B1,0x53AD,0xF9A2,0xF9A3,0xF9A4,0x67D3,
	0xF9A5,0x708E,0x7130,0x7430,0x8276,0x82D2,
	/* Range 0x6721 - 0x677E, array index: 0x18CB */
	0xF9A6,0x95BB,0x9AE5,0x9E7D,0x66C4,0xF9A7,0x71C1,0x8449,
	0xF9A8,0xF9A9,0x584B,0xF9AA,0xF9AB,0x5DB8,0x5F71,0xF9AC,
	0x6620,0x668E,0x6979,0x69AE,0x6C38,0x6CF3,0x6E36,0x6F41,
	0x6FDA,0x701B,0x702F,0x7150,0x71DF,0x7370,0xF9AD,0x745B,
	0xF9AE,0x74D4,0x76C8,0x7A4E,0x7E93,0xF9AF,0xF9B0,0x82F1,
	0x8A60,0x8FCE,0xF9B1,0x9348,0xF9B2,0x9719,0xF9B3,0xF9B4,
	0x4E42,0x502A,0xF9B5,0x5208,0x53E1,0x66F3,0x6C6D,0x6FCA,
	0x730A,0x777F,0x7A62,0x82AE,0x85DD,0x8602,0xF9B6,0x88D4,
	0x8A63,0x8B7D,0x8C6B,0xF9B7,0x92B3,0xF9B8,0x9713,0x9810,
	0x4E94,0x4F0D,0x4FC9,0x50B2,0x5348,0x543E,0x5433,0x55DA,
	0x5862,0x58BA,0x5967,0x5A1B,0x5BE4,0x609F,0xF9B9,0x61CA,
	0x6556,0x65FF,0x6664,0x68A7,0x6C5A,0x6FB3,
	/* Range 0x6821 - 0x687E, array index: 0x1929 */
	0x70CF,0x71AC,0x7352,0x7B7D,0x8708,0x8AA4,0x9C32,0x9F07,
	0x5C4B,0x6C83,0x7344,0x7389,0x923A,0x6EAB,0x7465,0x761F,
	0x7A69,0x7E15,0x860A,0x5140,0x58C5,0x64C1,0x74EE,0x7515,
	0x7670,0x7FC1,0x9095,0x96CD,0x9954,0x6E26,0x74E6,0x7AA9,
	0x7AAA,0x81E5,0x86D9,0x8778,0x8A1B,0x5A49,0x5B8C,0x5B9B,
	0x68A1,0x6900,0x6D63,0x73A9,0x7413,0x742C,0x7897,0x7DE9,
	0x7FEB,0x8118,0x8155,0x839E,0x8C4C,0x962E,0x9811,0x66F0,
	0x5F80,0x65FA,0x6789,0x6C6A,0x738B,0x502D,0x5A03,0x6B6A,
	0x77EE,0x5916,0x5D6C,0x5DCD,0x7325,0x754F,0xF9BA,0xF9BB,
	0x50E5,0x51F9,0x582F,0x592D,0x5996,0x59DA,0x5BE5,0xF9BC,
	0xF9BD,0x5DA2,0x62D7,0x6416,0x6493,0x64FE,0xF9BE,0x66DC,
	0xF9BF,0x6A48,0xF9C0,0x71FF,0x7464,0xF9C1,
	/* Range 0x6921 - 0x697E, array index: 0x1987 */
	0x7A88,0x7AAF,0x7E47,0x7E5E,0x8000,0x8170,0xF9C2,0x87EF,
	0x8981,0x8B20,0x9059,0xF9C3,0x9080,0x9952,0x617E,0x6B32,
	0x6D74,0x7E1F,0x8925,0x8FB1,0x4FD1,0x50AD,0x5197,0x52C7,
	0x57C7,0x5889,0x5BB9,0x5EB8,0x6142,0x6995,0x6D8C,0x6E67,
	0x6EB6,0x7194,0x7462,0x7528,0x752C,0x8073,0x8338,0x84C9,
	0x8E0A,0x9394,0x93DE,0xF9C4,0x4E8E,0x4F51,0x5076,0x512A,
	0x53C8,0x53CB,0x53F3,0x5B87,0x5BD3,0x5C24,0x611A,0x6182,
	0x65F4,0x725B,0x7397,0x7440,0x76C2,0x7950,0x7991,0x79B9,
	0x7D06,0x7FBD,0x828B,0x85D5,0x865E,0x8FC2,0x9047,0x90F5,
	0x91EA,0x9685,0x96E8,0x96E9,0x52D6,0x5F67,0x65ED,0x6631,
	0x682F,0x715C,0x7A36,0x90C1,0x980A,0x4E91,0xF9C5,0x6A52,
	0x6B9E,0x6F90,0x7189,0x8018,0x82B8,0x8553,
	/* Range 0x6A21 - 0x6A7E, array index: 0x19E5 */
	0x904B,0x9695,0x96F2,0x97FB,0x851A,0x9B31,0x4E90,0x718A,
	0x96C4,0x5143,0x539F,0x54E1,0x5713,0x5712,0x57A3,0x5A9B,
	0x5AC4,0x5BC3,0x6028,0x613F,0x63F4,0x6C85,0x6D39,0x6E72,
	0x6E90,0x7230,0x733F,0x7457,0x82D1,0x8881,0x8F45,0x9060,
	0xF9C6,0x9662,0x9858,0x9D1B,0x6708,0x8D8A,0x925E,0x4F4D,
	0x5049,0x50DE,0x5371,0x570D,0x59D4,0x5A01,0x5C09,0x6170,
	0x6690,0x6E2D,0x7232,0x744B,0x7DEF,0x80C3,0x840E,0x8466,
	0x853F,0x875F,0x885B,0x8918,0x8B02,0x9055,0x97CB,0x9B4F,
	0x4E73,0x4F91,0x5112,0x516A,0xF9C7,0x552F,0x55A9,0x5B7A,
	0x5BA5,0x5E7C,0x5E7D,0x5EBE,0x60A0,0x60DF,0x6108,0x6109,
	0x63C4,0x6538,0x6709,0xF9C8,0x67D4,0x67DA,0xF9C9,0x6961,
	0x6962,0x6CB9,0x6D27,0xF9CA,0x6E38,0xF9CB,
	/* Range 0x6B21 - 0x6B7E, array index: 0x1A43 */
	0x6FE1,0x7336,0x7337,0xF9CC,0x745C,0x7531,0xF9CD,0x7652,
	0xF9CE,0xF9CF,0x7DAD,0x81FE,0x8438,0x88D5,0x8A98,0x8ADB,
	0x8AED,0x8E30,0x8E42,0x904A,0x903E,0x907A,0x9149,0x91C9,
	0x936E,0xF9D0,0xF9D1,0x5809,0xF9D2,0x6BD3,0x8089,0x80B2,
	0xF9D3,0xF9D4,0x5141,0x596B,0x5C39,0xF9D5,0xF9D6,0x6F64,
	0x73A7,0x80E4,0x8D07,0xF9D7,0x9217,0x958F,0xF9D8,0xF9D9,
	0xF9DA,0xF9DB,0x807F,0x620E,0x701C,0x7D68,0x878D,0xF9DC,
	0x57A0,0x6069,0x6147,0x6BB7,0x8ABE,0x9280,0x96B1,0x4E59,
	0x541F,0x6DEB,0x852D,0x9670,0x97F3,0x98EE,0x63D6,0x6CE3,
	0x9091,0x51DD,0x61C9,0x81BA,0x9DF9,0x4F9D,0x501A,0x5100,
	0x5B9C,0x610F,0x61FF,0x64EC,0x6905,0x6BC5,0x7591,0x77E3,
	0x7FA9,0x8264,0x858F,0x87FB,0x8863,0x8ABC,
	/* Range 0x6C21 - 0x6C7E, array index: 0x1AA1 */
	0x8B70,0x91AB,0x4E8C,0x4EE5,0x4F0A,0xF9DD,0xF9DE,0x5937,
	0x59E8,0xF9DF,0x5DF2,0x5F1B,0x5F5B,0x6021,0xF9E0,0xF9E1,
	0xF9E2,0xF9E3,0x723E,0x73E5,0xF9E4,0x7570,0x75CD,0xF9E5,
	0x79FB,0xF9E6,0x800C,0x8033,0x8084,0x82E1,0x8351,0xF9E7,
	0xF9E8,0x8CBD,0x8CB3,0x9087,0xF9E9,0xF9EA,0x98F4,0x990C,
	0xF9EB,0xF9EC,0x7037,0x76CA,0x7FCA,0x7FCC,0x7FFC,0x8B1A,
	0x4EBA,0x4EC1,0x5203,0x5370,0xF9ED,0x54BD,0x56E0,0x59FB,
	0x5BC5,0x5F15,0x5FCD,0x6E6E,0xF9EE,0xF9EF,0x7D6A,0x8335,
	0xF9F0,0x8693,0x8A8D,0xF9F1,0x976D,0x9777,0xF9F2,0xF9F3,
	0x4E00,0x4F5A,0x4F7E,0x58F9,0x65E5,0x6EA2,0x9038,0x93B0,
	0x99B9,0x4EFB,0x58EC,0x598A,0x59D9,0x6041,0xF9F4,0xF9F5,
	0x7A14,0xF9F6,0x834F,0x8CC3,0x5165,0x5344,
	/* Range 0x6D21 - 0x6D7E, array index: 0x1AFF */
	0xF9F7,0xF9F8,0xF9F9,0x4ECD,0x5269,0x5B55,0x82BF,0x4ED4,
	0x523A,0x54A8,0x59C9,0x59FF,0x5B50,0x5B57,0x5B5C,0x6063,
	0x6148,0x6ECB,0x7099,0x716E,0x7386,0x74F7,0x75B5,0x78C1,
	0x7D2B,0x8005,0x81EA,0x8328,0x8517,0x85C9,0x8AEE,0x8CC7,
	0x96CC,0x4F5C,0x52FA,0x56BC,0x65AB,0x6628,0x707C,0x70B8,
	0x7235,0x7DBD,0x828D,0x914C,0x96C0,0x9D72,0x5B71,0x68E7,
	0x6B98,0x6F7A,0x76DE,0x5C91,0x66AB,0x6F5B,0x7BB4,0x7C2A,
	0x8836,0x96DC,0x4E08,0x4ED7,0x5320,0x5834,0x58BB,0x58EF,
	0x596C,0x5C07,0x5E33,0x5E84,0x5F35,0x638C,0x66B2,0x6756,
	0x6A1F,0x6AA3,0x6B0C,0x6F3F,0x7246,0xF9FA,0x7350,0x748B,
	0x7AE0,0x7CA7,0x8178,0x81DF,0x81E7,0x838A,0x846C,0x8523,
	0x8594,0x85CF,0x88DD,0x8D13,0x91AC,0x9577,
	/* Range 0x6E21 - 0x6E7E, array index: 0x1B5D */
	0x969C,0x518D,0x54C9,0x5728,0x5BB0,0x624D,0x6750,0x683D,
	0x6893,0x6E3D,0x6ED3,0x707D,0x7E21,0x88C1,0x8CA1,0x8F09,
	0x9F4B,0x9F4E,0x722D,0x7B8F,0x8ACD,0x931A,0x4F47,0x4F4E,
	0x5132,0x5480,0x59D0,0x5E95,0x62B5,0x6775,0x696E,0x6A17,
	0x6CAE,0x6E1A,0x72D9,0x732A,0x75BD,0x7BB8,0x7D35,0x82E7,
	0x83F9,0x8457,0x85F7,0x8A5B,0x8CAF,0x8E87,0x9019,0x90B8,
	0x96CE,0x9F5F,0x52E3,0x540A,0x5AE1,0x5BC2,0x6458,0x6575,
	0x6EF4,0x72C4,0xF9FB,0x7684,0x7A4D,0x7B1B,0x7C4D,0x7E3E,
	0x7FDF,0x837B,0x8B2B,0x8CCA,0x8D64,0x8DE1,0x8E5F,0x8FEA,
	0x8FF9,0x9069,0x93D1,0x4F43,0x4F7A,0x50B3,0x5168,0x5178,
	0x524D,0x526A,0x5861,0x587C,0x5960,0x5C08,0x5C55,0x5EDB,
	0x609B,0x6230,0x6813,0x6BBF,0x6C08,0x6FB1,
	/* Range 0x6F21 - 0x6F7E, array index: 0x1BBB */
	0x714E,0x7420,0x7530,0x7538,0x7551,0x7672,0x7B4C,0x7B8B,
	0x7BAD,0x7BC6,0x7E8F,0x8A6E,0x8F3E,0x8F49,0x923F,0x9293,
	0x9322,0x942B,0x96FB,0x985A,0x986B,0x991E,0x5207,0x622A,
	0x6298,0x6D59,0x7664,0x7ACA,0x7BC0,0x7D76,0x5360,0x5CBE,
	0x5E97,0x6F38,0x70B9,0x7C98,0x9711,0x9B8E,0x9EDE,0x63A5,
	0x647A,0x8776,0x4E01,0x4E95,0x4EAD,0x505C,0x5075,0x5448,
	0x59C3,0x5B9A,0x5E40,0x5EAD,0x5EF7,0x5F81,0x60C5,0x633A,
	0x653F,0x6574,0x65CC,0x6676,0x6678,0x67FE,0x6968,0x6A89,
	0x6B63,0x6C40,0x6DC0,0x6DE8,0x6E1F,0x6E5E,0x701E,0x70A1,
	0x738E,0x73FD,0x753A,0x775B,0x7887,0x798E,0x7A0B,0x7A7D,
	0x7CBE,0x7D8E,0x8247,0x8A02,0x8AEA,0x8C9E,0x912D,0x914A,
	0x91D8,0x9266,0x92CC,0x9320,0x9706,0x9756,
	/* Range 0x7021 - 0x707E, array index: 0x1C19 */
	0x975C,0x9802,0x9F0E,0x5236,0x5291,0x557C,0x5824,0x5E1D,
	0x5F1F,0x608C,0x63D0,0x68AF,0x6FDF,0x796D,0x7B2C,0x81CD,
	0x85BA,0x88FD,0x8AF8,0x8E44,0x918D,0x9664,0x969B,0x973D,
	0x984C,0x9F4A,0x4FCE,0x5146,0x51CB,0x52A9,0x5632,0x5F14,
	0x5F6B,0x63AA,0x64CD,0x65E9,0x6641,0x66FA,0x66F9,0x671D,
	0x689D,0x68D7,0x69FD,0x6F15,0x6F6E,0x7167,0x71E5,0x722A,
	0x74AA,0x773A,0x7956,0x795A,0x79DF,0x7A20,0x7A95,0x7C97,
	0x7CDF,0x7D44,0x7E70,0x8087,0x85FB,0x86A4,0x8A54,0x8ABF,
	0x8D99,0x8E81,0x9020,0x906D,0x91E3,0x963B,0x96D5,0x9CE5,
	0x65CF,0x7C07,0x8DB3,0x93C3,0x5B58,0x5C0A,0x5352,0x62D9,
	0x731D,0x5027,0x5B97,0x5F9E,0x60B0,0x616B,0x68D5,0x6DD9,
	0x742E,0x7A2E,0x7D42,0x7D9C,0x7E31,0x816B,
	/* Range 0x7121 - 0x717E, array index: 0x1C77 */
	0x8E2A,0x8E35,0x937E,0x9418,0x4F50,0x5750,0x5DE6,0x5EA7,
	0x632B,0x7F6A,0x4E3B,0x4F4F,0x4F8F,0x505A,0x59DD,0x80C4,
	0x546A,0x5468,0x55FE,0x594F,0x5B99,0x5DDE,0x5EDA,0x665D,
	0x6731,0x67F1,0x682A,0x6CE8,0x6D32,0x6E4A,0x6F8D,0x70B7,
	0x73E0,0x7587,0x7C4C,0x7D02,0x7D2C,0x7DA2,0x821F,0x86DB,
	0x8A3B,0x8A85,0x8D70,0x8E8A,0x8F33,0x9031,0x914E,0x9152,
	0x9444,0x99D0,0x7AF9,0x7CA5,0x4FCA,0x5101,0x51C6,0x57C8,
	0x5BEF,0x5CFB,0x6659,0x6A3D,0x6D5A,0x6E96,0x6FEC,0x710C,
	0x756F,0x7AE3,0x8822,0x9021,0x9075,0x96CB,0x99FF,0x8301,
	0x4E2D,0x4EF2,0x8846,0x91CD,0x537D,0x6ADB,0x696B,0x6C41,
	0x847A,0x589E,0x618E,0x66FE,0x62EF,0x70DD,0x7511,0x75C7,
	0x7E52,0x84B8,0x8B49,0x8D08,0x4E4B,0x53EA,
	/* Range 0x7221 - 0x727E, array index: 0x1CD5 */
	0x54AB,0x5730,0x5740,0x5FD7,0x6301,0x6307,0x646F,0x652F,
	0x65E8,0x667A,0x679D,0x67B3,0x6B62,0x6C60,0x6C9A,0x6F2C,
	0x77E5,0x7825,0x7949,0x7957,0x7D19,0x80A2,0x8102,0x81F3,
	0x829D,0x82B7,0x8718,0x8A8C,0xF9FC,0x8D04,0x8DBE,0x9072,
	0x76F4,0x7A19,0x7A37,0x7E54,0x8077,0x5507,0x55D4,0x5875,
	0x632F,0x6422,0x6649,0x664B,0x686D,0x699B,0x6B84,0x6D25,
	0x6EB1,0x73CD,0x7468,0x74A1,0x755B,0x75B9,0x76E1,0x771E,
	0x778B,0x79E6,0x7E09,0x7E1D,0x81FB,0x852F,0x8897,0x8A3A,
	0x8CD1,0x8EEB,0x8FB0,0x9032,0x93AD,0x9663,0x9673,0x9707,
	0x4F84,0x53F1,0x59EA,0x5AC9,0x5E19,0x684E,0x74C6,0x75BE,
	0x79E9,0x7A92,0x81A3,0x86ED,0x8CEA,0x8DCC,0x8FED,0x659F,
	0x6715,0xF9FD,0x57F7,0x6F57,0x7DDD,0x8F2F,
	/* Range 0x7321 - 0x737E, array index: 0x1D33 */
	0x93F6,0x96C6,0x5FB5,0x61F2,0x6F84,0x4E14,0x4F98,0x501F,
	0x53C9,0x55DF,0x5D6F,0x5DEE,0x6B21,0x6B64,0x78CB,0x7B9A,
	0xF9FE,0x8E49,0x8ECA,0x906E,0x6349,0x643E,0x7740,0x7A84,
	0x932F,0x947F,0x9F6A,0x64B0,0x6FAF,0x71E6,0x74A8,0x74DA,
	0x7AC4,0x7C12,0x7E82,0x7CB2,0x7E98,0x8B9A,0x8D0A,0x947D,
	0x9910,0x994C,0x5239,0x5BDF,0x64E6,0x672D,0x7D2E,0x50ED,
	0x53C3,0x5879,0x6158,0x6159,0x61FA,0x65AC,0x7AD9,0x8B92,
	0x8B96,0x5009,0x5021,0x5275,0x5531,0x5A3C,0x5EE0,0x5F70,
	0x6134,0x655E,0x660C,0x6636,0x66A2,0x69CD,0x6EC4,0x6F32,
	0x7316,0x7621,0x7A93,0x8139,0x8259,0x83D6,0x84BC,0x50B5,
	0x57F0,0x5BC0,0x5BE8,0x5F69,0x63A1,0x7826,0x7DB5,0x83DC,
	0x8521,0x91C7,0x91F5,0x518A,0x67F5,0x7B56,
	/* Range 0x7421 - 0x747E, array index: 0x1D91 */
	0x8CAC,0x51C4,0x59BB,0x60BD,0x8655,0x501C,0xF9FF,0x5254,
	0x5C3A,0x617D,0x621A,0x62D3,0x64F2,0x65A5,0x6ECC,0x7620,
	0x810A,0x8E60,0x965F,0x96BB,0x4EDF,0x5343,0x5598,0x5929,
	0x5DDD,0x64C5,0x6CC9,0x6DFA,0x7394,0x7A7F,0x821B,0x85A6,
	0x8CE4,0x8E10,0x9077,0x91E7,0x95E1,0x9621,0x97C6,0x51F8,
	0x54F2,0x5586,0x5FB9,0x64A4,0x6F88,0x7DB4,0x8F1F,0x8F4D,
	0x9435,0x50C9,0x5C16,0x6CBE,0x6DFB,0x751B,0x77BB,0x7C3D,
	0x7C64,0x8A79,0x8AC2,0x581E,0x59BE,0x5E16,0x6377,0x7252,
	0x758A,0x776B,0x8ADC,0x8CBC,0x8F12,0x5EF3,0x6674,0x6DF8,
	0x807D,0x83C1,0x8ACB,0x9751,0x9BD6,0xFA00,0x5243,0x66FF,
	0x6D95,0x6EEF,0x7DE0,0x8AE6,0x902E,0x905E,0x9AD4,0x521D,
	0x527F,0x54E8,0x6194,0x6284,0x62DB,0x68A2,
	/* Range 0x7521 - 0x757E, array index: 0x1DEF */
	0x6912,0x695A,0x6A35,0x7092,0x7126,0x785D,0x7901,0x790E,
	0x79D2,0x7A0D,0x8096,0x8278,0x82D5,0x8349,0x8549,0x8C82,
	0x8D85,0x9162,0x918B,0x91AE,0x4FC3,0x56D1,0x71ED,0x77D7,
	0x8700,0x89F8,0x5BF8,0x5FD6,0x6751,0x90A8,0x53E2,0x585A,
	0x5BF5,0x60A4,0x6181,0x6460,0x7E3D,0x8070,0x8525,0x9283,
	0x64AE,0x50AC,0x5D14,0x6700,0x589C,0x62BD,0x63A8,0x690E,
	0x6978,0x6A1E,0x6E6B,0x76BA,0x79CB,0x82BB,0x8429,0x8ACF,
	0x8DA8,0x8FFD,0x9112,0x914B,0x919C,0x9310,0x9318,0x939A,
	0x96DB,0x9A36,0x9C0D,0x4E11,0x755C,0x795D,0x7AFA,0x7B51,
	0x7BC9,0x7E2E,0x84C4,0x8E59,0x8E74,0x8EF8,0x9010,0x6625,
	0x693F,0x7443,0x51FA,0x672E,0x9EDC,0x5145,0x5FE0,0x6C96,
	0x87F2,0x885D,0x8877,0x60B4,0x81B5,0x8403,
	/* Range 0x7621 - 0x767E, array index: 0x1E4D */
	0x8D05,0x53D6,0x5439,0x5634,0x5A36,0x5C31,0x708A,0x7FE0,
	0x805A,0x8106,0x81ED,0x8DA3,0x9189,0x9A5F,0x9DF2,0x5074,
	0x4EC4,0x53A0,0x60FB,0x6E2C,0x5C64,0x4F88,0x5024,0x55E4,
	0x5CD9,0x5E5F,0x6065,0x6894,0x6CBB,0x6DC4,0x71BE,0x75D4,
	0x75F4,0x7661,0x7A1A,0x7A49,0x7DC7,0x7DFB,0x7F6E,0x81F4,
	0x86A9,0x8F1C,0x96C9,0x99B3,0x9F52,0x5247,0x52C5,0x98ED,
	0x89AA,0x4E03,0x67D2,0x6F06,0x4FB5,0x5BE2,0x6795,0x6C88,
	0x6D78,0x741B,0x7827,0x91DD,0x937C,0x87C4,0x79E4,0x7A31,
	0x5FEB,0x4ED6,0x54A4,0x553E,0x58AE,0x59A5,0x60F0,0x6253,
	0x62D6,0x6736,0x6955,0x8235,0x9640,0x99B1,0x99DD,0x502C,
	0x5353,0x5544,0x577C,0xFA01,0x6258,0xFA02,0x64E2,0x666B,
	0x67DD,0x6FC1,0x6FEF,0x7422,0x7438,0x8A17,
	/* Range 0x7721 - 0x777E, array index: 0x1EAB */
	0x9438,0x5451,0x5606,0x5766,0x5F48,0x619A,0x6B4E,0x7058,
	0x70AD,0x7DBB,0x8A95,0x596A,0x812B,0x63A2,0x7708,0x803D,
	0x8CAA,0x5854,0x642D,0x69BB,0x5B95,0x5E11,0x6E6F,0xFA03,
	0x8569,0x514C,0x53F0,0x592A,0x6020,0x614B,0x6B86,0x6C70,
	0x6CF0,0x7B1E,0x80CE,0x82D4,0x8DC6,0x90B0,0x98B1,0xFA04,
	0x64C7,0x6FA4,0x6491,0x6504,0x514E,0x5410,0x571F,0x8A0E,
	0x615F,0x6876,0xFA05,0x75DB,0x7B52,0x7D71,0x901A,0x5806,
	0x69CC,0x817F,0x892A,0x9000,0x9839,0x5078,0x5957,0x59AC,
	0x6295,0x900F,0x9B2A,0x615D,0x7279,0x95D6,0x5761,0x5A46,
	0x5DF4,0x628A,0x64AD,0x64FA,0x6777,0x6CE2,0x6D3E,0x722C,
	0x7436,0x7834,0x7F77,0x82AD,0x8DDB,0x9817,0x5224,0x5742,
	0x677F,0x7248,0x74E3,0x8CA9,0x8FA6,0x9211,
	/* Range 0x7821 - 0x787E, array index: 0x1F09 */
	0x962A,0x516B,0x53ED,0x634C,0x4F69,0x5504,0x6096,0x6557,
	0x6C9B,0x6D7F,0x724C,0x72FD,0x7A17,0x8987,0x8C9D,0x5F6D,
	0x6F8E,0x70F9,0x81A8,0x610E,0x4FBF,0x504F,0x6241,0x7247,
	0x7BC7,0x7DE8,0x7FE9,0x904D,0x97AD,0x9A19,0x8CB6,0x576A,
	0x5E73,0x67B0,0x840D,0x8A55,0x5420,0x5B16,0x5E63,0x5EE2,
	0x5F0A,0x6583,0x80BA,0x853D,0x9589,0x965B,0x4F48,0x5305,
	0x530D,0x530F,0x5486,0x54FA,0x5703,0x5E03,0x6016,0x629B,
	0x62B1,0x6355,0xFA06,0x6CE1,0x6D66,0x75B1,0x7832,0x80DE,
	0x812F,0x82DE,0x8461,0x84B2,0x888D,0x8912,0x900B,0x92EA,
	0x98FD,0x9B91,0x5E45,0x66B4,0x66DD,0x7011,0x7206,0xFA07,
	0x4FF5,0x527D,0x5F6A,0x6153,0x6753,0x6A19,0x6F02,0x74E2,
	0x7968,0x8868,0x8C79,0x98C7,0x98C4,0x9A43,
	/* Range 0x7921 - 0x797E, array index: 0x1F67 */
	0x54C1,0x7A1F,0x6953,0x8AF7,0x8C4A,0x98A8,0x99AE,0x5F7C,
	0x62AB,0x75B2,0x76AE,0x88AB,0x907F,0x9642,0x5339,0x5F3C,
	0x5FC5,0x6CCC,0x73CC,0x7562,0x758B,0x7B46,0x82FE,0x999D,
	0x4E4F,0x903C,0x4E0B,0x4F55,0x53A6,0x590F,0x5EC8,0x6630,
	0x6CB3,0x7455,0x8377,0x8766,0x8CC0,0x9050,0x971E,0x9C15,
	0x58D1,0x5B78,0x8650,0x8B14,0x9DB4,0x5BD2,0x6068,0x608D,
	0x65F1,0x6C57,0x6F22,0x6FA3,0x701A,0x7F55,0x7FF0,0x9591,
	0x9592,0x9650,0x97D3,0x5272,0x8F44,0x51FD,0x542B,0x54B8,
	0x5563,0x558A,0x6ABB,0x6DB5,0x7DD8,0x8266,0x929C,0x9677,
	0x9E79,0x5408,0x54C8,0x76D2,0x86E4,0x95A4,0x95D4,0x965C,
	0x4EA2,0x4F09,0x59EE,0x5AE6,0x5DF7,0x6052,0x6297,0x676D,
	0x6841,0x6C86,0x6E2F,0x7F38,0x809B,0x822A,
	/* Range 0x7A21 - 0x7A7E, array index: 0x1FC5 */
	0xFA08,0xFA09,0x9805,0x4EA5,0x5055,0x54B3,0x5793,0x595A,
	0x5B69,0x5BB3,0x61C8,0x6977,0x6D77,0x7023,0x87F9,0x89E3,
	0x8A72,0x8AE7,0x9082,0x99ED,0x9AB8,0x52BE,0x6838,0x5016,
	0x5E78,0x674F,0x8347,0x884C,0x4EAB,0x5411,0x56AE,0x73E6,
	0x9115,0x97FF,0x9909,0x9957,0x9999,0x5653,0x589F,0x865B,
	0x8A31,0x61B2,0x6AF6,0x737B,0x8ED2,0x6B47,0x96AA,0x9A57,
	0x5955,0x7200,0x8D6B,0x9769,0x4FD4,0x5CF4,0x5F26,0x61F8,
	0x665B,0x6CEB,0x70AB,0x7384,0x73B9,0x73FE,0x7729,0x774D,
	0x7D43,0x7D62,0x7E23,0x8237,0x8852,0xFA0A,0x8CE2,0x9249,
	0x986F,0x5B51,0x7A74,0x8840,0x9801,0x5ACC,0x4FE0,0x5354,
	0x593E,0x5CFD,0x633E,0x6D79,0x72F9,0x8105,0x8107,0x83A2,
	0x92CF,0x9830,0x4EA8,0x5144,0x5211,0x578B,
	/* Range 0x7B21 - 0x7B7E, array index: 0x2023 */
	0x5F62,0x6CC2,0x6ECE,0x7005,0x7050,0x70AF,0x7192,0x73E9,
	0x7469,0x834A,0x87A2,0x8861,0x9008,0x90A2,0x93A3,0x99A8,
	0x516E,0x5F57,0x60E0,0x6167,0x66B3,0x8559,0x8E4A,0x91AF,
	0x978B,0x4E4E,0x4E92,0x547C,0x58D5,0x58FA,0x597D,0x5CB5,
	0x5F27,0x6236,0x6248,0x660A,0x6667,0x6BEB,0x6D69,0x6DCF,
	0x6E56,0x6EF8,0x6F94,0x6FE0,0x6FE9,0x705D,0x72D0,0x7425,
	0x745A,0x74E0,0x7693,0x795C,0x7CCA,0x7E1E,0x80E1,0x82A6,
	0x846B,0x84BF,0x864E,0x865F,0x8774,0x8B77,0x8C6A,0x93AC,
	0x9800,0x9865,0x60D1,0x6216,0x9177,0x5A5A,0x660F,0x6DF7,
	0x6E3E,0x743F,0x9B42,0x5FFD,0x60DA,0x7B0F,0x54C4,0x5F18,
	0x6C5E,0x6CD3,0x6D2A,0x70D8,0x7D05,0x8679,0x8A0C,0x9D3B,
	0x5316,0x548C,0x5B05,0x6A3A,0x706B,0x7575,
	/* Range 0x7C21 - 0x7C7E, array index: 0x2081 */
	0x798D,0x79BE,0x82B1,0x83EF,0x8A71,0x8B41,0x8CA8,0x9774,
	0xFA0B,0x64F4,0x652B,0x78BA,0x78BB,0x7A6B,0x4E38,0x559A,
	0x5950,0x5BA6,0x5E7B,0x60A3,0x63DB,0x6B61,0x6665,0x6853,
	0x6E19,0x7165,0x74B0,0x7D08,0x9084,0x9A69,0x9C25,0x6D3B,
	0x6ED1,0x733E,0x8C41,0x95CA,0x51F0,0x5E4C,0x5FA8,0x604D,
	0x60F6,0x6130,0x614C,0x6643,0x6644,0x69A5,0x6CC1,0x6E5F,
	0x6EC9,0x6F62,0x714C,0x749C,0x7687,0x7BC1,0x7C27,0x8352,
	0x8757,0x9051,0x968D,0x9EC3,0x532F,0x56DE,0x5EFB,0x5F8A,
	0x6062,0x6094,0x61F7,0x6666,0x6703,0x6A9C,0x6DEE,0x6FAE,
	0x7070,0x736A,0x7E6A,0x81BE,0x8334,0x86D4,0x8AA8,0x8CC4,
	0x5283,0x7372,0x5B96,0x6A6B,0x9404,0x54EE,0x5686,0x5B5D,
	0x6548,0x6585,0x66C9,0x689F,0x6D8D,0x6DC6,
	/* Range 0x7D21 - 0x7D7E, array index: 0x20DF */
	0x723B,0x80B4,0x9175,0x9A4D,0x4FAF,0x5019,0x539A,0x540E,
	0x543C,0x5589,0x55C5,0x5E3F,0x5F8C,0x673D,0x7166,0x73DD,
	0x9005,0x52DB,0x52F3,0x5864,0x58CE,0x7104,0x718F,0x71FB,
	0x85B0,0x8A13,0x6688,0x85A8,0x55A7,0x6684,0x714A,0x8431,
	0x5349,0x5599,0x6BC1,0x5F59,0x5FBD,0x63EE,0x6689,0x7147,
	0x8AF1,0x8F1D,0x9EBE,0x4F11,0x643A,0x70CB,0x7566,0x8667,
	0x6064,0x8B4E,0x9DF8,0x5147,0x51F6,0x5308,0x6D36,0x80F8,
	0x9ED1,0x6615,0x6B23,0x7098,0x75D5,0x5403,0x5C79,0x7D07,
	0x8A16,0x6B20,0x6B3D,0x6B46,0x5438,0x6070,0x6D3D,0x7FD5,
	0x8208,0x50D6,0x51DE,0x559C,0x566B,0x56CD,0x59EC,0x5B09,
	0x5E0C,0x6199,0x6198,0x6231,0x665E,0x66E6,0x7199,0x71B9,
	0x71BA,0x72A7,0x79A7,0x7A00,0x7FB2,0x8A70,
	/* Unranged codes (93 codes) */
};

#endif /* ICONV_TO_UCS_CCS_KSX1001 && defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit UCS -> ksx1001 speed-optimized table (74752 bytes).
 * ======================================================================
 */

#if defined (ICONV_FROM_UCS_CCS_KSX1001) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
from_ucs_speed_ksx1001[] =
{
	/* Heading Block */
	0x0100,0x0200,0x0300,0x0400,0x0500,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	0x0600,0x0700,0x0800,0x0900,0x0A00,0x0B00,0x0C00,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	0x0D00,0x0E00,0x0F00,0x1000,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,0x1100,0x1200,
	0x1300,0x1400,0x1500,0x1600,0x1700,0x1800,0x1900,0x1A00,
	0x1B00,0x1C00,0x1D00,0x1E00,0x1F00,0x2000,0x2100,0x2200,
	0x2300,0x2400,0x2500,0x2600,0x2700,0x2800,0x2900,0x2A00,
	0x2B00,0x2C00,0x2D00,0x2E00,0x2F00,0x3000,0x3100,0x3200,
	0x3300,0x3400,0x3500,0x3600,0x3700,0x3800,0x3900,0x3A00,
	0x3B00,0x3C00,0x3D00,0x3E00,0x3F00,0x4000,0x4100,0x4200,
	0x4300,0x4400,0x4500,0x4600,0x4700,0x4800,0x4900,0x4A00,
	0x4B00,0x4C00,0x4D00,0x4E00,0x4F00,0x5000,0x5100,0x5200,
	0x5300,0x5400,0x5500,0x5600,0x5700,0x5800,0x5900,0x5A00,
	0x5B00,0x5C00,0x5D00,0x5E00,0x5F00,0x6000,0x6100,0x6200,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,0x6300,0x6400,0x6500,0x6600,
	0x6700,0x6800,0x6900,0x6A00,0x6B00,0x6C00,0x6D00,0x6E00,
	0x6F00,0x7000,0x7100,0x7200,0x7300,0x7400,0x7500,0x7600,
	0x7700,0x7800,0x7900,0x7A00,0x7B00,0x7C00,0x7D00,0x7E00,
	0x7F00,0x8000,0x8100,0x8200,0x8300,0x8400,0x8500,0x8600,
	0x8700,0x8800,0x8900,0x8A00,0x8B00,0x8C00,0x8D00,0x8E00,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,0x8F00,0x9000,INVBLK,INVBLK,INVBLK,INVBLK,0x9100,
	/* Block 1, Array index 0x0100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x222E,INVALC,INVALC,0x2234,INVALC,INVALC,0x2157,
	0x2127,INVALC,0x2823,INVALC,INVALC,0x2129,INVALC,INVALC,
	0x2146,0x213E,0x2977,0x2978,0x2225,INVALC,0x2252,0x2124,
	0x222C,0x2976,0x282C,INVALC,0x2879,0x2876,0x287A,0x222F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2821,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2822,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x213F,
	0x282A,INVALC,INVALC,INVALC,INVALC,INVALC,0x282D,0x292C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2921,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2923,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2140,
	0x292A,INVALC,INVALC,INVALC,INVALC,INVALC,0x292D,INVALC,
	/* Block 2, Array index 0x0200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2922,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2824,0x2924,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2925,0x2826,0x2926,INVALC,INVALC,INVALC,INVALC,
	0x2927,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2828,
	0x2928,0x2829,0x2929,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2930,0x282F,0x292F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x282B,0x292B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x282E,0x292E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 3, Array index 0x0300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2227,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2230,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2228,0x222B,0x222A,0x222D,INVALC,0x2229,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 4, Array index 0x0400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2541,0x2542,0x2543,0x2544,0x2545,0x2546,0x2547,
	0x2548,0x2549,0x254A,0x254B,0x254C,0x254D,0x254E,0x254F,
	0x2550,0x2551,INVALC,0x2552,0x2553,0x2554,0x2555,0x2556,
	0x2557,0x2558,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2561,0x2562,0x2563,0x2564,0x2565,0x2566,0x2567,
	0x2568,0x2569,0x256A,0x256B,0x256C,0x256D,0x256E,0x256F,
	0x2570,0x2571,INVALC,0x2572,0x2573,0x2574,0x2575,0x2576,
	0x2577,0x2578,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 5, Array index 0x0500 */
	INVALC,0x2C27,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2C21,0x2C22,0x2C23,0x2C24,0x2C25,0x2C26,0x2C28,0x2C29,
	0x2C2A,0x2C2B,0x2C2C,0x2C2D,0x2C2E,0x2C2F,0x2C30,0x2C31,
	0x2C32,0x2C33,0x2C34,0x2C35,0x2C36,0x2C37,0x2C38,0x2C39,
	0x2C3A,0x2C3B,0x2C3C,0x2C3D,0x2C3E,0x2C3F,0x2C40,0x2C41,
	0x2C51,0x2C52,0x2C53,0x2C54,0x2C55,0x2C56,0x2C58,0x2C59,
	0x2C5A,0x2C5B,0x2C5C,0x2C5D,0x2C5E,0x2C5F,0x2C60,0x2C61,
	0x2C62,0x2C63,0x2C64,0x2C65,0x2C66,0x2C67,0x2C68,0x2C69,
	0x2C6A,0x2C6B,0x2C6C,0x2C6D,0x2C6E,0x2C6F,0x2C70,0x2C71,
	INVALC,0x2C57,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 33, Array index 0x0600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x212A,INVALC,INVALC,
	0x212E,0x212F,INVALC,INVALC,0x2130,0x2131,INVALC,INVALC,
	0x2253,0x2254,INVALC,INVALC,INVALC,0x2125,0x2126,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2236,INVALC,0x2147,0x2148,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2158,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2979,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x297A,
	INVALC,0x297B,0x297C,0x297D,0x297E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 34, Array index 0x0700 */
	INVALC,INVALC,INVALC,0x2149,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2235,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2724,INVALC,INVALC,0x2260,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2265,0x2262,INVALC,INVALC,INVALC,0x2759,INVALC,
	INVALC,INVALC,INVALC,0x214A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2877,0x2878,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x287B,0x287C,0x287D,0x287E,INVALC,
	0x2530,0x2531,0x2532,0x2533,0x2534,0x2535,0x2536,0x2537,
	0x2538,0x2539,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2521,0x2522,0x2523,0x2524,0x2525,0x2526,0x2527,0x2528,
	0x2529,0x252A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2167,0x2168,0x2166,0x2169,0x216A,0x2255,0x2258,0x2256,
	0x2259,0x2257,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2221,INVALC,0x2222,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 35, Array index 0x0800 */
	0x2223,INVALC,0x2153,0x2224,INVALC,INVALC,INVALC,0x2154,
	0x2174,INVALC,INVALC,0x2175,INVALC,INVALC,INVALC,0x2233,
	INVALC,0x2232,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x216E,INVALC,INVALC,0x2170,0x2144,INVALC,
	0x2150,INVALC,INVALC,INVALC,INVALC,0x212B,INVALC,0x217C,
	0x217D,0x217B,0x217A,0x2172,0x2173,INVALC,0x2231,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2145,0x2171,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x212D,0x216F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2156,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2141,0x2155,INVALC,INVALC,0x2142,0x2143,INVALC,INVALC,
	INVALC,INVALC,0x216C,0x216D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2178,0x2179,INVALC,INVALC,0x2176,0x2177,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2241,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2151,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 36, Array index 0x0900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2152,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 37, Array index 0x0A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2867,0x2868,0x2869,0x286A,0x286B,0x286C,0x286D,0x286E,
	0x286F,0x2870,0x2871,0x2872,0x2873,0x2874,0x2875,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2967,0x2968,0x2969,0x296A,
	0x296B,0x296C,0x296D,0x296E,0x296F,0x2970,0x2971,0x2972,
	0x2973,0x2974,0x2975,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x294D,0x294E,0x294F,0x2950,
	0x2951,0x2952,0x2953,0x2954,0x2955,0x2956,0x2957,0x2958,
	0x2959,0x295A,0x295B,0x295C,0x295D,0x295E,0x295F,0x2960,
	0x2961,0x2962,0x2963,0x2964,0x2965,0x2966,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x284D,0x284E,0x284F,0x2850,0x2851,0x2852,0x2853,0x2854,
	0x2855,0x2856,0x2857,0x2858,0x2859,0x285A,0x285B,0x285C,
	0x285D,0x285E,0x285F,0x2860,0x2861,0x2862,0x2863,0x2864,
	0x2865,0x2866,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 38, Array index 0x0B00 */
	0x2621,0x262C,0x2622,0x262D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2623,0x2648,0x2647,0x262E,
	0x2624,0x2642,0x2641,0x262F,0x2626,0x2646,0x2645,0x2631,
	0x2625,0x2644,0x2643,0x2630,0x2627,0x263C,0x2649,0x264A,
	0x2637,0x264B,0x264C,0x2632,0x2629,0x263E,0x264D,0x264E,
	0x2639,0x264F,0x2650,0x2634,0x2628,0x2651,0x2652,0x2638,
	0x263D,0x2653,0x2654,0x2633,0x262A,0x2655,0x2656,0x263A,
	0x263F,0x2657,0x2658,0x2635,0x262B,0x2659,0x265A,0x263B,
	0x265B,0x265C,0x2640,0x265D,0x265E,0x265F,0x2660,0x2661,
	0x2662,0x2663,0x2664,0x2636,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2246,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2161,0x2160,INVALC,0x2243,0x2247,0x2248,0x224B,0x224A,
	0x2249,0x224C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2163,0x2162,INVALC,INVALC,0x223A,0x2239,
	INVALC,INVALC,INVALC,INVALC,0x2165,0x2164,INVALC,INVALC,
	0x2238,0x2237,INVALC,INVALC,INVALC,INVALC,0x215F,0x215E,
	0x2242,INVALC,INVALC,0x215B,INVALC,INVALC,0x215D,0x215C,
	0x2244,0x2245,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 39, Array index 0x0C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x215A,0x2159,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x224F,0x224E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2250,INVALC,0x2251,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x214F,INVALC,0x214E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x223C,0x223D,INVALC,0x2240,0x223B,0x223E,INVALC,0x223F,
	0x224D,0x225B,0x225C,INVALC,0x225D,0x225A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 49, Array index 0x0D00 */
	0x2121,0x2122,0x2123,0x2128,INVALC,INVALC,INVALC,INVALC,
	0x2134,0x2135,0x2136,0x2137,0x2138,0x2139,0x213A,0x213B,
	0x213C,0x213D,INVALC,0x216B,0x2132,0x2133,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2A21,0x2A22,0x2A23,0x2A24,0x2A25,0x2A26,0x2A27,
	0x2A28,0x2A29,0x2A2A,0x2A2B,0x2A2C,0x2A2D,0x2A2E,0x2A2F,
	0x2A30,0x2A31,0x2A32,0x2A33,0x2A34,0x2A35,0x2A36,0x2A37,
	0x2A38,0x2A39,0x2A3A,0x2A3B,0x2A3C,0x2A3D,0x2A3E,0x2A3F,
	0x2A40,0x2A41,0x2A42,0x2A43,0x2A44,0x2A45,0x2A46,0x2A47,
	0x2A48,0x2A49,0x2A4A,0x2A4B,0x2A4C,0x2A4D,0x2A4E,0x2A4F,
	0x2A50,0x2A51,0x2A52,0x2A53,0x2A54,0x2A55,0x2A56,0x2A57,
	0x2A58,0x2A59,0x2A5A,0x2A5B,0x2A5C,0x2A5D,0x2A5E,0x2A5F,
	0x2A60,0x2A61,0x2A62,0x2A63,0x2A64,0x2A65,0x2A66,0x2A67,
	0x2A68,0x2A69,0x2A6A,0x2A6B,0x2A6C,0x2A6D,0x2A6E,0x2A6F,
	0x2A70,0x2A71,0x2A72,0x2A73,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2B21,0x2B22,0x2B23,0x2B24,0x2B25,0x2B26,0x2B27,
	0x2B28,0x2B29,0x2B2A,0x2B2B,0x2B2C,0x2B2D,0x2B2E,0x2B2F,
	0x2B30,0x2B31,0x2B32,0x2B33,0x2B34,0x2B35,0x2B36,0x2B37,
	0x2B38,0x2B39,0x2B3A,0x2B3B,0x2B3C,0x2B3D,0x2B3E,0x2B3F,
	0x2B40,0x2B41,0x2B42,0x2B43,0x2B44,0x2B45,0x2B46,0x2B47,
	0x2B48,0x2B49,0x2B4A,0x2B4B,0x2B4C,0x2B4D,0x2B4E,0x2B4F,
	0x2B50,0x2B51,0x2B52,0x2B53,0x2B54,0x2B55,0x2B56,0x2B57,
	0x2B58,0x2B59,0x2B5A,0x2B5B,0x2B5C,0x2B5D,0x2B5E,0x2B5F,
	0x2B60,0x2B61,0x2B62,0x2B63,0x2B64,0x2B65,0x2B66,0x2B67,
	0x2B68,0x2B69,0x2B6A,0x2B6B,0x2B6C,0x2B6D,0x2B6E,0x2B6F,
	0x2B70,0x2B71,0x2B72,0x2B73,0x2B74,0x2B75,0x2B76,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 50, Array index 0x0E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2421,0x2422,0x2423,0x2424,0x2425,0x2426,0x2427,
	0x2428,0x2429,0x242A,0x242B,0x242C,0x242D,0x242E,0x242F,
	0x2430,0x2431,0x2432,0x2433,0x2434,0x2435,0x2436,0x2437,
	0x2438,0x2439,0x243A,0x243B,0x243C,0x243D,0x243E,0x243F,
	0x2440,0x2441,0x2442,0x2443,0x2444,0x2445,0x2446,0x2447,
	0x2448,0x2449,0x244A,0x244B,0x244C,0x244D,0x244E,0x244F,
	0x2450,0x2451,0x2452,0x2453,0x2454,0x2455,0x2456,0x2457,
	0x2458,0x2459,0x245A,0x245B,0x245C,0x245D,0x245E,0x245F,
	0x2460,0x2461,0x2462,0x2463,0x2464,0x2465,0x2466,0x2467,
	0x2468,0x2469,0x246A,0x246B,0x246C,0x246D,0x246E,0x246F,
	0x2470,0x2471,0x2472,0x2473,0x2474,0x2475,0x2476,0x2477,
	0x2478,0x2479,0x247A,0x247B,0x247C,0x247D,0x247E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 51, Array index 0x0F00 */
	0x2931,0x2932,0x2933,0x2934,0x2935,0x2936,0x2937,0x2938,
	0x2939,0x293A,0x293B,0x293C,0x293D,0x293E,0x293F,0x2940,
	0x2941,0x2942,0x2943,0x2944,0x2945,0x2946,0x2947,0x2948,
	0x2949,0x294A,0x294B,0x294C,0x225F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2831,0x2832,0x2833,0x2834,0x2835,0x2836,0x2837,0x2838,
	0x2839,0x283A,0x283B,0x283C,0x283D,0x283E,0x283F,0x2840,
	0x2841,0x2842,0x2843,0x2844,0x2845,0x2846,0x2847,0x2848,
	0x2849,0x284A,0x284B,0x284C,INVALC,INVALC,INVALC,0x225E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 52, Array index 0x1000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2749,0x274A,0x274B,0x274C,0x274D,INVALC,INVALC,INVALC,
	0x273A,0x273B,0x275C,0x275D,0x275E,0x2736,0x2737,0x2738,
	0x2754,0x2755,0x2756,0x2757,0x2758,0x2721,0x2722,0x2723,
	0x2725,0x272B,0x272C,0x272D,0x272E,0x272F,0x2730,0x2731,
	0x2732,0x2733,0x2734,0x2727,0x2728,0x2729,0x272A,0x273D,
	0x273E,0x2765,0x2766,0x2767,0x2768,0x2761,0x2762,0x2763,
	0x273F,0x2740,0x2741,0x2742,0x2743,0x2744,0x2745,0x2746,
	0x2747,0x2748,0x274E,0x274F,0x2750,0x2751,0x2752,0x2753,
	0x275A,0x275B,0x2263,0x276C,0x2726,0x2760,0x276F,0x2261,
	0x273C,0x276D,0x2735,INVALC,INVALC,INVALC,INVALC,0x2739,
	0x276A,INVALC,INVALC,0x276B,INVALC,INVALC,0x275F,INVALC,
	0x2264,INVALC,INVALC,0x2764,0x276E,0x2769,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 79, Array index 0x1100 */
	0x6C69,0x6F4B,INVALC,0x7652,INVALC,INVALC,INVALC,0x5832,
	0x6D5B,0x5F32,0x5F3E,0x793B,INVALC,0x5C74,INVALC,INVALC,
	INVALC,0x7564,INVALC,INVALC,0x7326,0x5D60,0x6126,INVALC,
	0x4E78,0x5C30,INVALC,INVALC,INVALC,INVALC,0x632A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7169,INVALC,INVALC,
	INVALC,INVALC,0x4D7A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7C2F,0x5321,INVALC,0x712B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6751,0x522C,INVALC,0x4E79,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x717D,INVALC,0x5E3F,0x7B3A,0x7939,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E52,INVALC,
	0x632B,0x6B60,INVALC,INVALC,INVALC,0x4E7A,0x4B77,0x6525,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A61,INVALC,0x544C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6A61,INVALC,INVALC,0x5C63,0x5F2D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B6B,INVALC,
	INVALC,INVALC,0x552F,INVALC,INVALC,INVALC,0x5675,INVALC,
	0x6578,INVALC,INVALC,0x5E40,0x6C23,INVALC,0x694D,INVALC,
	0x6A27,0x6976,0x7B3B,INVALC,0x6769,0x6F4C,INVALC,INVALC,
	0x5066,INVALC,INVALC,0x5E41,INVALC,INVALC,0x642C,INVALC,
	INVALC,0x584C,0x7971,INVALC,0x4E5F,0x7A24,0x6632,INVALC,
	0x7A7B,INVALC,INVALC,0x7A3D,0x4C48,0x6F4D,0x5555,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5322,INVALC,
	INVALC,INVALC,0x6C51,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6427,0x6C52,INVALC,INVALC,0x7631,INVALC,INVALC,0x4E7B,
	INVALC,INVALC,0x5051,0x4B3F,INVALC,0x6D24,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6D28,0x5E42,0x7662,0x6D5C,
	0x5C75,0x6039,INVALC,INVALC,INVALC,0x544E,INVALC,0x7435,
	INVALC,INVALC,INVALC,0x535B,0x5635,0x6C24,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6466,INVALC,0x716A,INVALC,INVALC,INVALC,0x4B6C,0x4B40,
	INVALC,INVALC,INVALC,0x6C72,INVALC,INVALC,INVALC,INVALC,
	/* Block 80, Array index 0x1200 */
	INVALC,0x506A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7972,0x6C25,0x505F,INVALC,0x676A,0x506B,0x5C51,
	0x5B69,0x7D4C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B57,
	INVALC,INVALC,INVALC,INVALC,0x5A61,INVALC,0x5636,INVALC,
	0x635F,INVALC,0x5E43,INVALC,0x5E44,0x4A21,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6E6C,INVALC,INVALC,0x5323,0x6E37,
	0x784F,INVALC,INVALC,INVALC,INVALC,0x6A48,0x6E38,0x712C,
	0x7125,0x694E,INVALC,INVALC,INVALC,0x793C,INVALC,INVALC,
	INVALC,0x6579,0x6C6A,0x5D56,0x6D42,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7825,INVALC,INVALC,INVALC,INVALC,INVALC,0x653A,
	0x5B58,INVALC,INVALC,0x4A22,INVALC,INVALC,0x514D,INVALC,
	INVALC,INVALC,0x6E6D,INVALC,INVALC,INVALC,0x6C6B,0x5E45,
	INVALC,0x6360,INVALC,0x4A49,0x7269,INVALC,0x554E,INVALC,
	0x7636,INVALC,0x4E42,0x5647,INVALC,0x6334,INVALC,0x712D,
	INVALC,0x6A62,INVALC,INVALC,INVALC,INVALC,0x5742,INVALC,
	0x7327,INVALC,INVALC,0x4D6A,INVALC,0x6B6E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5932,0x7D25,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7655,0x5562,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7835,
	INVALC,INVALC,0x4C75,0x7535,0x642D,INVALC,INVALC,INVALC,
	INVALC,0x676B,0x7155,INVALC,INVALC,INVALC,0x703B,INVALC,
	INVALC,0x6935,INVALC,0x4C49,0x7A55,INVALC,INVALC,0x6154,
	INVALC,INVALC,0x5756,INVALC,INVALC,0x5C41,INVALC,0x5E46,
	0x7A6F,0x6361,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6173,0x5C76,
	INVALC,0x4E7C,INVALC,0x5B44,INVALC,0x7871,INVALC,INVALC,
	0x5C64,INVALC,0x656F,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 81, Array index 0x1300 */
	INVALC,INVALC,0x5C31,INVALC,INVALC,INVALC,0x5556,INVALC,
	INVALC,0x735A,INVALC,0x4B41,INVALC,0x5B43,INVALC,INVALC,
	INVALC,0x597A,0x536E,INVALC,INVALC,INVALC,0x7A38,INVALC,
	INVALC,0x7D26,0x6B6F,INVALC,0x7426,INVALC,0x4C4A,0x7328,
	INVALC,0x735B,INVALC,0x5B27,0x7637,INVALC,0x4F66,0x7072,
	0x4B5A,INVALC,0x6752,0x5743,0x7670,0x685E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6526,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6567,INVALC,INVALC,INVALC,0x4A23,
	0x4C27,0x6A49,INVALC,INVALC,INVALC,INVALC,INVALC,0x7836,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7A25,INVALC,INVALC,
	INVALC,INVALC,0x712E,INVALC,0x6F4E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B6D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7630,0x6F4F,0x694F,INVALC,
	0x775E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4E53,INVALC,INVALC,INVALC,INVALC,0x5C77,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5B28,INVALC,INVALC,
	INVALC,0x4B78,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5F21,0x5D61,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x754A,0x6936,INVALC,INVALC,
	INVALC,INVALC,0x676C,0x6E6E,INVALC,0x7370,INVALC,0x5F3F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C4B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5041,INVALC,INVALC,
	INVALC,0x7452,0x603A,INVALC,INVALC,INVALC,INVALC,0x5F40,
	INVALC,0x4E60,INVALC,INVALC,INVALC,0x5C52,0x7D6A,INVALC,
	INVALC,INVALC,0x5676,INVALC,INVALC,INVALC,0x6A4A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6869,INVALC,0x632C,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7350,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A24,INVALC,0x5B78,INVALC,INVALC,INVALC,0x5E47,
	/* Block 82, Array index 0x1400 */
	0x6B70,0x7156,INVALC,INVALC,0x6562,INVALC,0x4C4C,INVALC,
	INVALC,0x4B7B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6A63,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F41,
	INVALC,0x566D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6950,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6E39,INVALC,INVALC,INVALC,INVALC,0x5563,
	INVALC,INVALC,0x5153,INVALC,0x6570,INVALC,INVALC,INVALC,
	0x6834,0x6B43,INVALC,0x6A2A,0x7A7C,0x7576,0x703C,0x7D54,
	0x603B,0x4E43,INVALC,0x503A,0x773A,0x5873,0x774D,INVALC,
	INVALC,INVALC,0x642E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x545F,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5067,INVALC,INVALC,0x6C7D,INVALC,0x522E,
	0x6E6F,0x5557,0x6A64,0x7822,0x4D6B,0x573F,0x7B31,INVALC,
	INVALC,0x4D6C,INVALC,INVALC,INVALC,0x5C32,0x506C,0x4E7D,
	0x6E70,INVALC,INVALC,INVALC,0x4C42,INVALC,INVALC,INVALC,
	0x506D,INVALC,INVALC,INVALC,INVALC,INVALC,0x6577,INVALC,
	INVALC,INVALC,0x737C,INVALC,INVALC,0x6E22,INVALC,INVALC,
	INVALC,INVALC,0x5933,INVALC,INVALC,0x5874,INVALC,0x6937,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4E2E,INVALC,INVALC,INVALC,INVALC,0x5922,INVALC,INVALC,
	INVALC,INVALC,0x5871,INVALC,0x544F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6527,0x5552,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5629,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7422,INVALC,0x7157,INVALC,
	INVALC,0x5558,INVALC,0x703D,0x5750,0x5450,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x574F,0x6B6A,0x7D6B,INVALC,
	INVALC,0x5B6D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7C45,0x4B42,INVALC,INVALC,INVALC,INVALC,0x7D55,INVALC,
	0x7448,0x686A,0x7573,INVALC,INVALC,0x795E,INVALC,INVALC,
	/* Block 83, Array index 0x1500 */
	0x536F,INVALC,INVALC,0x6C53,INVALC,INVALC,0x5D42,0x6F37,
	0x6754,INVALC,0x4A4A,INVALC,INVALC,INVALC,0x597B,INVALC,
	INVALC,0x7A7D,INVALC,INVALC,INVALC,INVALC,INVALC,0x562A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7478,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7777,0x5C2C,INVALC,INVALC,
	INVALC,0x5757,0x5F22,INVALC,INVALC,INVALC,0x4E3E,INVALC,
	0x5370,INVALC,INVALC,INVALC,INVALC,INVALC,0x7024,0x616C,
	0x4F67,0x734B,0x6D29,0x4A3E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x746F,INVALC,INVALC,INVALC,0x764E,
	INVALC,INVALC,0x5E7B,0x503B,0x5537,0x6E71,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7428,INVALC,0x5C78,INVALC,
	INVALC,INVALC,INVALC,0x4B27,INVALC,0x5A4E,INVALC,INVALC,
	INVALC,0x6066,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6D25,0x6E72,INVALC,INVALC,INVALC,INVALC,0x5C79,
	INVALC,INVALC,0x795C,INVALC,INVALC,0x735C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7872,INVALC,0x7479,
	INVALC,INVALC,INVALC,0x7C71,INVALC,INVALC,INVALC,0x503C,
	0x5B79,0x5731,INVALC,INVALC,INVALC,0x4B7C,INVALC,INVALC,
	INVALC,0x7025,0x4B7D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5574,INVALC,INVALC,INVALC,0x4D6D,
	0x4A25,INVALC,INVALC,0x562B,0x5042,INVALC,INVALC,INVALC,
	INVALC,0x703E,0x523D,0x4C24,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7A36,INVALC,
	INVALC,0x4C4D,INVALC,0x5A7A,INVALC,0x764F,INVALC,0x6938,
	INVALC,0x5875,INVALC,INVALC,INVALC,0x4C4E,INVALC,INVALC,
	INVALC,INVALC,0x574D,INVALC,INVALC,0x5451,0x696D,INVALC,
	0x4A6B,0x5962,INVALC,0x7D32,INVALC,0x632D,0x564C,0x5934,
	INVALC,INVALC,0x6127,0x6E53,0x5043,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7D33,INVALC,0x5564,INVALC,INVALC,
	0x4F68,INVALC,0x6D43,0x5032,INVALC,INVALC,0x4E7E,0x5A28,
	/* Block 84, Array index 0x1600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7850,INVALC,INVALC,
	0x7D56,INVALC,INVALC,INVALC,INVALC,0x7851,INVALC,0x7852,
	0x5C53,INVALC,INVALC,INVALC,INVALC,0x5D62,0x7B79,0x5D41,
	INVALC,0x6335,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6D5D,0x4E44,INVALC,0x4B21,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D63,INVALC,INVALC,INVALC,INVALC,0x7C5D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x792F,INVALC,INVALC,INVALC,INVALC,INVALC,0x527B,
	0x4F21,0x6428,INVALC,0x7436,0x6C7E,INVALC,INVALC,0x632E,
	0x676D,0x7D41,0x5A62,INVALC,INVALC,0x5833,INVALC,INVALC,
	INVALC,0x5D64,0x706F,0x7671,0x7A70,INVALC,INVALC,0x5175,
	INVALC,INVALC,0x5A4F,INVALC,0x5C54,INVALC,0x5C26,INVALC,
	0x6F3F,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E4F,INVALC,
	0x6059,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5956,
	0x6C54,0x6A4B,INVALC,INVALC,0x4A3F,0x5530,INVALC,0x4F69,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x716D,INVALC,0x4C4F,
	INVALC,INVALC,INVALC,INVALC,0x6478,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x646D,INVALC,INVALC,INVALC,INVALC,
	0x5758,INVALC,0x7D27,INVALC,INVALC,INVALC,INVALC,0x6A2B,
	0x7632,INVALC,INVALC,INVALC,INVALC,0x4F70,0x793D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6674,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4B5B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7351,INVALC,INVALC,INVALC,INVALC,
	0x6951,0x7329,0x5060,0x6952,INVALC,0x5A63,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6252,INVALC,0x7622,0x6174,
	INVALC,INVALC,INVALC,0x5A64,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6755,0x753F,0x4F22,0x4D2F,0x4F23,INVALC,INVALC,
	INVALC,0x4D30,0x717E,0x5023,0x612F,0x7823,INVALC,0x4A26,
	0x773B,0x726A,0x5E48,0x6953,INVALC,INVALC,INVALC,INVALC,
	0x5E49,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 85, Array index 0x1700 */
	INVALC,INVALC,INVALC,0x7D5E,0x4A40,INVALC,INVALC,INVALC,
	0x796A,0x514E,0x6E54,INVALC,0x5452,0x5923,0x7D28,0x5759,
	0x774E,0x7A3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4F56,INVALC,0x5770,INVALC,0x6B61,
	0x7845,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C7A,INVALC,
	INVALC,0x5D43,INVALC,0x795F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x676F,INVALC,INVALC,INVALC,INVALC,
	0x7D65,0x7623,INVALC,0x597C,0x7D29,INVALC,0x676E,INVALC,
	INVALC,INVALC,0x5565,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F50,INVALC,0x4D31,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7722,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7132,INVALC,0x7131,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D32,INVALC,0x5A2B,INVALC,0x4A27,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6362,0x7B3C,0x5924,INVALC,INVALC,
	0x6E3A,INVALC,INVALC,INVALC,INVALC,INVALC,0x7853,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7B7A,INVALC,0x4F24,INVALC,
	0x5C7B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7663,INVALC,INVALC,INVALC,
	0x6D2A,INVALC,INVALC,0x7221,0x4E61,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7A26,INVALC,INVALC,INVALC,INVALC,
	0x7960,INVALC,INVALC,INVALC,INVALC,0x6C56,INVALC,INVALC,
	0x646E,0x7921,INVALC,INVALC,0x7B6F,INVALC,INVALC,INVALC,
	0x796B,0x6E23,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A2C,INVALC,INVALC,INVALC,0x4A28,INVALC,INVALC,
	0x747A,INVALC,INVALC,INVALC,INVALC,0x4D56,0x7C76,INVALC,
	INVALC,INVALC,0x7449,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7854,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 86, Array index 0x1800 */
	INVALC,INVALC,INVALC,INVALC,0x7826,INVALC,0x5E4A,0x7246,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x575A,INVALC,
	0x5350,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5845,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A66,
	INVALC,0x735D,INVALC,INVALC,INVALC,0x645A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7664,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7672,INVALC,0x5F42,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x597D,
	INVALC,INVALC,INVALC,0x4C76,INVALC,INVALC,0x533A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x642F,INVALC,
	INVALC,INVALC,INVALC,0x7961,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7026,INVALC,INVALC,INVALC,
	0x4B53,INVALC,INVALC,INVALC,0x603C,INVALC,0x744A,0x547A,
	INVALC,0x7D2A,0x7962,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7437,0x7D42,0x7C30,INVALC,0x7D6C,0x4A62,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7D3D,
	INVALC,0x6A67,0x5F43,0x5152,0x4E62,INVALC,0x5324,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7D2B,INVALC,0x5F60,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7247,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6770,INVALC,0x506E,INVALC,INVALC,0x732A,
	INVALC,INVALC,INVALC,0x5E4B,0x7638,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6175,0x7133,INVALC,
	/* Block 87, Array index 0x1900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7723,INVALC,
	INVALC,0x4A29,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F25,INVALC,INVALC,0x5F44,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6130,
	INVALC,INVALC,0x703F,INVALC,0x7624,INVALC,0x6336,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7A46,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x506F,INVALC,INVALC,0x7D6D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D44,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7C77,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x663F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E2D,INVALC,0x7A3F,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6571,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6D44,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5225,INVALC,INVALC,0x7D6E,INVALC,INVALC,
	INVALC,0x7536,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6176,0x5E4C,INVALC,INVALC,0x7C5E,INVALC,
	0x6C57,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D5D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5637,0x4D33,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 88, Array index 0x1A00 */
	INVALC,INVALC,INVALC,0x7855,0x6558,INVALC,INVALC,INVALC,
	0x4F6A,INVALC,INVALC,0x4F50,INVALC,0x6A4C,INVALC,INVALC,
	INVALC,INVALC,0x6A2E,0x6A2D,INVALC,INVALC,0x5371,INVALC,
	0x5325,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x774F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6E24,INVALC,INVALC,INVALC,INVALC,0x5024,INVALC,INVALC,
	0x7222,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5070,INVALC,INVALC,INVALC,INVALC,
	0x7223,INVALC,0x7778,INVALC,INVALC,INVALC,INVALC,0x5033,
	INVALC,INVALC,0x5B29,INVALC,INVALC,0x533B,0x4A6C,INVALC,
	0x7126,0x4B55,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7767,INVALC,INVALC,0x4D5E,INVALC,0x7724,INVALC,
	INVALC,INVALC,0x7840,INVALC,INVALC,INVALC,0x535D,INVALC,
	0x4C50,INVALC,INVALC,INVALC,INVALC,0x4F26,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7673,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6177,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x535C,INVALC,INVALC,0x7A7E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7A27,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6B59,INVALC,0x4F27,0x6A2F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x646F,INVALC,INVALC,INVALC,0x6939,
	0x7158,INVALC,INVALC,0x5858,INVALC,INVALC,0x6072,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6634,
	0x5C7C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7371,INVALC,INVALC,INVALC,0x6350,INVALC,INVALC,0x727B,
	INVALC,0x5B46,0x5071,INVALC,0x5072,INVALC,INVALC,INVALC,
	/* Block 89, Array index 0x1B00 */
	0x4F5C,INVALC,0x5351,INVALC,INVALC,0x4C31,0x7758,INVALC,
	0x4B28,0x6B3C,0x643E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x745C,INVALC,
	INVALC,0x5C42,INVALC,INVALC,0x7027,INVALC,INVALC,0x6640,
	INVALC,INVALC,0x4A6D,INVALC,INVALC,INVALC,INVALC,0x686B,
	0x6568,0x5C43,INVALC,INVALC,0x6D5E,0x5372,INVALC,INVALC,
	INVALC,INVALC,0x4C77,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E54,0x672B,INVALC,INVALC,INVALC,0x4B43,
	INVALC,0x6131,INVALC,INVALC,0x7732,INVALC,INVALC,0x5373,
	0x5352,INVALC,0x7540,INVALC,INVALC,INVALC,0x5F5D,INVALC,
	INVALC,0x6E73,0x6771,INVALC,0x7D34,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7248,INVALC,INVALC,
	INVALC,0x7352,INVALC,INVALC,0x6E74,INVALC,0x6253,INVALC,
	INVALC,INVALC,INVALC,0x4C51,INVALC,0x5F6A,INVALC,INVALC,
	INVALC,0x693A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5957,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x754D,INVALC,0x7172,0x7A47,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5978,0x5442,INVALC,INVALC,INVALC,INVALC,0x7665,INVALC,
	INVALC,INVALC,INVALC,0x5D45,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6772,0x6D5F,INVALC,INVALC,0x4A4B,INVALC,
	INVALC,0x5B7A,INVALC,INVALC,INVALC,0x6835,INVALC,0x5326,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7D35,INVALC,
	INVALC,0x7949,INVALC,0x6462,INVALC,0x7B3D,INVALC,INVALC,
	0x5724,0x4E45,INVALC,INVALC,INVALC,INVALC,0x4E55,0x5666,
	INVALC,INVALC,INVALC,INVALC,0x653D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E4D,0x6C73,INVALC,INVALC,0x6D60,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6C6C,0x7B3E,0x5F6B,INVALC,0x6178,INVALC,INVALC,
	/* Block 90, Array index 0x1C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x793E,
	INVALC,INVALC,INVALC,INVALC,0x5073,0x602A,0x6862,INVALC,
	INVALC,0x6254,0x527D,INVALC,0x6528,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5953,INVALC,INVALC,INVALC,INVALC,0x535E,
	INVALC,0x7438,0x773C,0x5C7D,INVALC,0x686C,0x6467,INVALC,
	INVALC,0x6377,INVALC,INVALC,INVALC,INVALC,INVALC,0x6C28,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7A71,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6572,INVALC,INVALC,0x5074,
	0x522F,0x5C65,INVALC,INVALC,INVALC,INVALC,0x5025,0x7134,
	0x7C31,0x4C78,INVALC,INVALC,0x5D46,0x7A51,INVALC,0x775F,
	INVALC,INVALC,0x7A28,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6E75,INVALC,0x5E4E,INVALC,INVALC,INVALC,INVALC,0x6773,
	INVALC,INVALC,0x772C,0x6B44,0x6D61,0x602B,0x5D47,INVALC,
	INVALC,INVALC,INVALC,0x5233,0x523F,INVALC,INVALC,INVALC,
	0x4A4C,INVALC,INVALC,INVALC,INVALC,0x7B3F,INVALC,INVALC,
	INVALC,INVALC,0x657D,0x5D65,0x584D,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6C74,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5075,INVALC,INVALC,0x686D,0x5052,
	INVALC,0x5958,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7666,INVALC,INVALC,
	0x5B2A,INVALC,INVALC,INVALC,0x7760,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5859,INVALC,0x7423,INVALC,INVALC,0x745D,INVALC,
	INVALC,INVALC,INVALC,0x6F51,INVALC,INVALC,0x5935,INVALC,
	INVALC,0x6D2B,INVALC,0x6337,INVALC,INVALC,INVALC,INVALC,
	0x6E3B,0x4D34,INVALC,0x6073,0x6A4D,INVALC,INVALC,INVALC,
	INVALC,0x6C75,0x686E,INVALC,0x4B29,0x712F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A4D,INVALC,
	0x6C29,INVALC,0x726B,INVALC,0x7D6F,INVALC,0x7973,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6641,INVALC,INVALC,0x6C58,INVALC,INVALC,INVALC,0x6D2C,
	/* Block 91, Array index 0x1D00 */
	INVALC,0x6A4E,INVALC,0x685F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5E4F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5226,INVALC,INVALC,0x6774,0x5156,INVALC,INVALC,0x6642,
	0x6363,INVALC,INVALC,INVALC,INVALC,0x6430,INVALC,INVALC,
	INVALC,0x5834,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7625,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x735E,INVALC,INVALC,INVALC,
	INVALC,0x5725,INVALC,INVALC,INVALC,INVALC,0x7768,INVALC,
	INVALC,0x6846,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7B66,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D66,INVALC,INVALC,INVALC,0x5C7E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x585A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A2C,0x6A30,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6338,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A2A,0x6179,INVALC,0x6A31,INVALC,INVALC,INVALC,
	INVALC,0x726C,INVALC,INVALC,0x7A6E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6E55,INVALC,INVALC,INVALC,INVALC,0x7974,INVALC,
	INVALC,0x526C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 92, Array index 0x1E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7B7B,INVALC,INVALC,
	INVALC,0x7D70,INVALC,0x603D,0x4E63,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7846,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5E2E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5F45,INVALC,INVALC,0x653E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6D2D,0x7A6A,INVALC,INVALC,0x4D6E,0x6D26,INVALC,0x6D2E,
	0x706D,INVALC,0x5D21,INVALC,0x6D2F,0x7C78,INVALC,0x586B,
	INVALC,INVALC,INVALC,0x4C79,0x4D35,INVALC,INVALC,INVALC,
	INVALC,0x7A29,INVALC,0x615D,INVALC,INVALC,INVALC,INVALC,
	0x6255,0x6D4F,INVALC,INVALC,INVALC,0x5D22,INVALC,INVALC,
	0x794A,INVALC,0x6A68,INVALC,0x656D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x536B,INVALC,0x6954,
	0x617A,0x644C,INVALC,0x6164,0x6847,INVALC,INVALC,0x4E5B,
	INVALC,INVALC,INVALC,0x5C55,INVALC,0x7735,0x7C73,0x7073,
	0x4E2F,0x7135,0x6F52,0x6848,0x6B71,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4B54,0x603E,0x6378,0x6A69,0x7C32,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6074,INVALC,0x4F60,INVALC,
	0x6E25,INVALC,INVALC,0x7A2A,0x6643,0x6132,0x4A2B,INVALC,
	0x6364,0x693B,INVALC,INVALC,INVALC,INVALC,INVALC,0x6256,
	0x7372,INVALC,0x6E56,0x6A32,0x5076,0x6C59,0x5A4B,0x4F28,
	INVALC,INVALC,INVALC,INVALC,0x5D23,INVALC,INVALC,INVALC,
	0x585B,INVALC,0x794E,0x6955,0x6351,INVALC,INVALC,0x523C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x582C,0x734C,
	INVALC,0x4D7B,0x7656,INVALC,0x6775,0x686F,0x6379,0x523B,
	0x7373,0x637B,INVALC,0x5E50,0x4E30,INVALC,0x5677,0x7159,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7541,0x5C44,INVALC,
	0x753B,INVALC,0x5E51,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 93, Array index 0x1F00 */
	INVALC,0x5C66,INVALC,INVALC,0x5E52,INVALC,INVALC,0x6D62,
	0x6E76,0x6A4F,0x706E,0x637C,INVALC,0x535F,0x5374,0x6133,
	INVALC,0x6134,INVALC,INVALC,INVALC,INVALC,0x7453,INVALC,
	INVALC,0x5F46,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6956,INVALC,INVALC,INVALC,
	0x5B2B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7626,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6339,0x6B45,0x7429,0x4D36,0x5279,INVALC,0x5A2D,0x5263,
	0x4F51,INVALC,INVALC,INVALC,INVALC,0x4B5C,0x4C7A,INVALC,
	0x4F5D,INVALC,INVALC,0x6829,INVALC,0x633B,0x633A,INVALC,
	INVALC,0x605A,INVALC,INVALC,INVALC,0x6E77,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5C33,INVALC,INVALC,INVALC,INVALC,
	0x5375,INVALC,0x5726,INVALC,0x7635,0x575B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6155,INVALC,INVALC,0x546A,
	INVALC,0x5F23,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7D5F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5077,0x6D54,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4B2A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x645B,INVALC,0x617B,0x4B22,INVALC,INVALC,INVALC,
	INVALC,0x5360,INVALC,0x643F,INVALC,0x7B40,INVALC,0x5A3E,
	0x644D,INVALC,0x5639,INVALC,INVALC,INVALC,0x6F40,INVALC,
	0x617C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7639,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5F47,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6431,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C67,
	0x5C68,INVALC,INVALC,INVALC,0x7A56,INVALC,0x5376,INVALC,
	INVALC,INVALC,INVALC,0x715A,INVALC,0x7A72,INVALC,INVALC,
	/* Block 94, Array index 0x2000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x627D,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x554F,0x5078,INVALC,
	INVALC,0x4D5F,INVALC,INVALC,0x754B,INVALC,0x6470,0x4B2B,
	INVALC,0x5744,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x627E,
	INVALC,0x5D5A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5A2E,0x4A6E,INVALC,INVALC,INVALC,
	0x5539,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6321,INVALC,INVALC,0x6863,INVALC,INVALC,0x732B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F29,
	INVALC,INVALC,INVALC,0x5377,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5471,INVALC,INVALC,
	0x4E64,INVALC,0x6872,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6575,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x672E,INVALC,0x563A,INVALC,0x5F6C,0x6440,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6864,INVALC,INVALC,
	INVALC,INVALC,0x5835,INVALC,INVALC,INVALC,0x645C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7439,0x7136,INVALC,
	INVALC,0x625E,0x6135,INVALC,INVALC,0x4D6F,0x7127,0x4E65,
	0x4B5D,INVALC,INVALC,0x5963,INVALC,INVALC,0x732C,INVALC,
	INVALC,0x5079,0x6C2B,0x5E53,0x7769,INVALC,INVALC,0x7975,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x615E,0x4B6E,INVALC,
	/* Block 95, Array index 0x2100 */
	INVALC,INVALC,0x633C,0x7856,INVALC,INVALC,0x5B6E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7D71,INVALC,INVALC,INVALC,
	INVALC,0x7736,INVALC,INVALC,INVALC,INVALC,0x745E,INVALC,
	INVALC,0x726D,INVALC,0x5B59,INVALC,0x7028,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x617D,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E54,INVALC,0x602C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6D63,INVALC,INVALC,0x5361,INVALC,
	0x5F48,INVALC,INVALC,INVALC,INVALC,0x5936,INVALC,0x7D2C,
	0x6F53,INVALC,INVALC,INVALC,0x6441,0x786B,INVALC,0x5B2C,
	INVALC,INVALC,INVALC,INVALC,0x7C46,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x582D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x763A,
	INVALC,0x5B5F,0x5353,0x7847,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A4E,0x7841,0x5234,INVALC,INVALC,0x5C34,
	0x7A39,0x4A4F,INVALC,0x7C33,0x6A6A,0x6A6B,0x507A,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6D64,INVALC,INVALC,0x5D67,
	INVALC,INVALC,0x5F49,INVALC,INVALC,INVALC,INVALC,0x5F6D,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6E3C,INVALC,0x6F41,
	INVALC,INVALC,0x4C52,INVALC,0x5D24,INVALC,INVALC,INVALC,
	0x5F4A,INVALC,INVALC,INVALC,INVALC,INVALC,0x5378,0x7128,
	INVALC,INVALC,INVALC,0x4D37,INVALC,0x6F54,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x645D,0x5F6E,0x4B2C,
	0x693C,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A6C,INVALC,
	INVALC,INVALC,0x5F4B,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x793F,0x562F,0x5546,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4F2A,INVALC,INVALC,0x4E29,INVALC,INVALC,0x5678,INVALC,
	INVALC,INVALC,0x7137,0x6E78,INVALC,INVALC,INVALC,0x5959,
	0x735F,INVALC,0x7848,0x4E46,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5566,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7466,INVALC,INVALC,0x6645,0x6F55,
	INVALC,INVALC,0x4B6F,0x7C5F,INVALC,INVALC,INVALC,INVALC,
	/* Block 96, Array index 0x2200 */
	INVALC,0x5C27,INVALC,INVALC,0x5667,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7849,INVALC,INVALC,INVALC,INVALC,0x6352,
	INVALC,0x633D,INVALC,0x4F61,0x7040,0x6C5A,INVALC,0x5D57,
	0x7B70,INVALC,INVALC,0x6C2C,INVALC,INVALC,INVALC,0x7029,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7A57,0x7B41,
	INVALC,0x5240,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6530,INVALC,INVALC,INVALC,0x6D65,INVALC,INVALC,
	INVALC,INVALC,0x4B2D,INVALC,0x7930,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7725,INVALC,0x4B2E,INVALC,0x5A2F,INVALC,0x5836,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5327,0x7B32,
	INVALC,0x7D44,INVALC,0x6C2D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7B21,INVALC,INVALC,INVALC,0x6569,0x696E,
	INVALC,0x7374,0x7873,0x7041,0x5E2F,0x7830,INVALC,INVALC,
	0x7360,0x672F,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B2D,
	INVALC,0x6635,INVALC,INVALC,0x7928,INVALC,INVALC,0x5D58,
	0x6859,0x6F56,INVALC,INVALC,INVALC,0x5362,INVALC,0x625F,
	INVALC,INVALC,0x7C60,0x5748,0x7D2D,INVALC,INVALC,INVALC,
	0x5F6F,0x4C53,0x5379,INVALC,INVALC,INVALC,INVALC,0x5470,
	0x5B47,0x5E55,INVALC,INVALC,INVALC,INVALC,0x7074,INVALC,
	0x5550,0x6559,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7C47,0x5C56,0x6260,INVALC,INVALC,INVALC,0x5A30,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7323,INVALC,0x536C,
	INVALC,0x744B,INVALC,INVALC,INVALC,0x7D45,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x637D,INVALC,0x7931,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x507B,0x6C5B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x753C,0x7224,
	0x584E,0x584F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7577,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7661,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5237,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7B6C,INVALC,0x5D48,
	/* Block 97, Array index 0x2300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6468,
	INVALC,INVALC,0x5241,INVALC,INVALC,INVALC,0x7857,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x563B,0x5E56,INVALC,INVALC,
	0x773D,0x6C2E,INVALC,INVALC,INVALC,0x5061,INVALC,0x6075,
	0x6A33,INVALC,0x4E56,INVALC,INVALC,INVALC,INVALC,0x4C25,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6C76,0x6261,0x633E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7C48,INVALC,INVALC,
	0x4D70,INVALC,0x7976,INVALC,INVALC,0x5F70,INVALC,INVALC,
	INVALC,0x653F,INVALC,INVALC,INVALC,0x4E3F,INVALC,INVALC,
	INVALC,INVALC,0x7C61,0x6D30,0x7D51,0x763B,INVALC,INVALC,
	0x794F,0x6B5A,0x4A41,INVALC,0x5238,0x4D71,INVALC,0x6353,
	0x7D66,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x666D,INVALC,INVALC,
	INVALC,0x637A,INVALC,INVALC,0x702A,0x7950,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7C62,INVALC,0x7827,INVALC,
	INVALC,INVALC,0x6165,0x6E79,INVALC,INVALC,INVALC,0x6776,
	0x6A6D,INVALC,INVALC,0x7C34,0x7542,INVALC,INVALC,0x575C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7075,INVALC,0x5D68,0x536D,0x757C,INVALC,0x5A3F,INVALC,
	0x4C7B,INVALC,INVALC,INVALC,0x537A,0x7424,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6F57,INVALC,0x5443,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7B63,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7B6D,INVALC,0x602D,INVALC,INVALC,0x6A6E,
	0x7B33,0x6442,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7667,0x525D,INVALC,0x5F4C,INVALC,INVALC,0x7C49,INVALC,
	INVALC,0x6529,0x6076,0x7633,INVALC,INVALC,INVALC,INVALC,
	/* Block 98, Array index 0x2400 */
	INVALC,0x617E,INVALC,INVALC,INVALC,INVALC,0x4B70,INVALC,
	0x6A6F,0x6A70,INVALC,INVALC,INVALC,0x5A40,0x7834,0x6B72,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6443,INVALC,INVALC,
	INVALC,INVALC,0x6957,0x6471,INVALC,INVALC,INVALC,0x4A6F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E57,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7C4A,INVALC,INVALC,INVALC,0x7361,INVALC,INVALC,0x4B44,
	INVALC,INVALC,INVALC,INVALC,0x6365,INVALC,0x4B45,0x6A34,
	INVALC,INVALC,0x693D,INVALC,0x5749,INVALC,INVALC,0x6B5B,
	0x6D31,INVALC,0x4C43,0x773E,0x7C4B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7874,INVALC,0x5937,INVALC,INVALC,
	0x7353,0x7354,INVALC,INVALC,INVALC,0x7764,INVALC,0x7751,
	INVALC,INVALC,0x5837,0x4E31,0x4A42,INVALC,INVALC,0x7B34,
	0x4B46,INVALC,INVALC,0x7076,INVALC,INVALC,0x5567,INVALC,
	0x6A50,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C54,0x4B2F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x742A,0x692F,INVALC,
	INVALC,0x7543,0x6958,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D69,INVALC,INVALC,INVALC,0x7173,INVALC,
	0x557B,0x5E3B,INVALC,INVALC,0x747B,INVALC,INVALC,INVALC,
	0x7D73,0x7D72,0x7726,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D49,INVALC,INVALC,0x5453,
	INVALC,0x4C28,INVALC,0x5A41,0x4C55,INVALC,0x5964,INVALC,
	INVALC,INVALC,0x7A4A,INVALC,INVALC,INVALC,0x6563,INVALC,
	INVALC,INVALC,0x533C,INVALC,INVALC,INVALC,0x4A70,INVALC,
	INVALC,INVALC,INVALC,0x5044,INVALC,INVALC,INVALC,0x4A50,
	0x7A2B,0x6B6B,0x6778,0x5965,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5157,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7324,INVALC,INVALC,INVALC,0x547B,0x7C63,
	0x7A58,INVALC,0x7355,INVALC,0x4F2B,INVALC,INVALC,0x6B73,
	/* Block 99, Array index 0x2500 */
	0x557C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5354,
	0x4D7C,INVALC,0x5966,INVALC,0x6279,0x6221,0x6B54,INVALC,
	0x6077,0x6432,0x4C7C,INVALC,INVALC,INVALC,0x7B64,INVALC,
	INVALC,INVALC,0x742B,INVALC,INVALC,INVALC,INVALC,0x503D,
	INVALC,0x4A71,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6F38,INVALC,INVALC,INVALC,0x5740,INVALC,
	0x6E7A,0x7D74,INVALC,INVALC,0x5363,INVALC,0x7B42,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5568,0x5B2E,
	0x6136,0x7837,INVALC,INVALC,INVALC,INVALC,INVALC,0x603F,
	0x7B43,0x5D6A,INVALC,0x6222,INVALC,0x6E26,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7668,INVALC,INVALC,INVALC,INVALC,
	0x7675,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D4A,INVALC,
	INVALC,0x5062,INVALC,INVALC,INVALC,INVALC,0x5D26,INVALC,
	INVALC,0x5D6B,INVALC,INVALC,0x6479,INVALC,INVALC,0x632F,
	0x507C,INVALC,INVALC,INVALC,0x747C,INVALC,INVALC,INVALC,
	INVALC,0x4C3C,0x776A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6564,0x5F71,INVALC,INVALC,0x7761,INVALC,0x7977,
	0x6F39,INVALC,INVALC,0x7858,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7929,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7859,INVALC,INVALC,INVALC,0x6E3D,INVALC,INVALC,
	INVALC,0x5846,INVALC,INVALC,0x6463,0x754E,INVALC,INVALC,
	INVALC,INVALC,0x5D59,INVALC,INVALC,INVALC,INVALC,0x5967,
	0x5239,0x5543,INVALC,INVALC,0x5A65,0x5A50,INVALC,0x5159,
	0x4E58,INVALC,0x4B5E,0x742C,0x5A7B,INVALC,0x7669,0x6873,
	0x4F2C,0x7070,INVALC,0x747D,0x5B48,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4E40,0x6354,0x514F,0x7175,
	INVALC,0x4D72,INVALC,0x4F6B,INVALC,INVALC,INVALC,0x4D38,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6326,0x515A,
	/* Block 100, Array index 0x2600 */
	INVALC,0x7225,INVALC,INVALC,INVALC,INVALC,INVALC,0x7226,
	INVALC,0x644E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x537B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7129,INVALC,INVALC,INVALC,0x7249,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6F58,0x6649,INVALC,0x5838,0x7A73,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7335,INVALC,INVALC,0x7824,INVALC,INVALC,0x5173,
	0x6648,INVALC,INVALC,INVALC,INVALC,0x785A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C69,
	0x5E57,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B5F,INVALC,
	INVALC,INVALC,0x4F6C,INVALC,INVALC,INVALC,INVALC,0x745F,
	INVALC,INVALC,0x5174,0x523A,INVALC,INVALC,INVALC,0x5F72,
	INVALC,INVALC,INVALC,0x6137,INVALC,INVALC,INVALC,INVALC,
	0x6223,0x537C,INVALC,INVALC,0x6D66,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5B49,INVALC,INVALC,INVALC,0x647A,INVALC,
	0x4F5E,INVALC,INVALC,0x4E50,INVALC,INVALC,INVALC,INVALC,
	0x5553,0x7375,0x772E,INVALC,INVALC,0x6F48,INVALC,0x4D73,
	0x754F,0x6573,0x7042,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A51,INVALC,INVALC,INVALC,0x6A71,INVALC,0x5026,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x595A,
	0x702B,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B67,INVALC,
	INVALC,INVALC,0x6540,0x7C35,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6444,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4C29,0x7D46,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6A35,INVALC,0x652A,0x5F3A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 101, Array index 0x2700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x615F,INVALC,0x5A51,
	INVALC,INVALC,INVALC,INVALC,0x6138,INVALC,0x6874,0x537D,
	INVALC,INVALC,INVALC,INVALC,0x6224,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x724A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A66,0x7733,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7D4D,INVALC,INVALC,INVALC,0x7336,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6E57,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7544,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5824,INVALC,INVALC,INVALC,INVALC,INVALC,0x7227,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5938,0x5939,0x6F49,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x564E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x774B,0x5F2E,0x6875,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5235,INVALC,INVALC,INVALC,0x5355,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x744C,0x5A7C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5968,INVALC,0x776B,0x7549,INVALC,
	0x733C,INVALC,0x5A52,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5335,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6836,INVALC,INVALC,0x564F,0x743A,INVALC,0x7749,
	INVALC,INVALC,0x4C2A,INVALC,INVALC,0x7043,0x4C56,INVALC,
	INVALC,INVALC,0x5053,INVALC,0x533D,INVALC,INVALC,INVALC,
	0x5B7B,INVALC,0x4B60,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5364,0x7677,INVALC,INVALC,0x553A,0x734D,0x4B61,
	INVALC,INVALC,INVALC,INVALC,0x6B74,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x742D,INVALC,0x7C2A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x776C,INVALC,INVALC,INVALC,0x6876,INVALC,
	/* Block 102, Array index 0x2800 */
	0x5A67,INVALC,INVALC,INVALC,0x774C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6541,INVALC,INVALC,INVALC,INVALC,0x606E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x557D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E66,0x7C2B,0x553B,INVALC,INVALC,0x7228,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6225,0x4D39,
	0x6A72,0x4B47,INVALC,0x4D74,INVALC,INVALC,0x5B2F,0x6F59,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4D3A,INVALC,INVALC,
	0x7C79,INVALC,INVALC,INVALC,INVALC,0x5F73,0x4E67,0x5A42,
	INVALC,0x4F2D,INVALC,INVALC,INVALC,INVALC,0x6779,0x7828,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7362,INVALC,
	INVALC,INVALC,0x4A72,0x5F24,INVALC,INVALC,0x5444,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4C57,0x6542,INVALC,INVALC,
	INVALC,INVALC,0x4D3B,INVALC,0x6F5A,0x6E58,INVALC,0x5D27,
	0x6226,INVALC,INVALC,INVALC,INVALC,INVALC,0x6040,INVALC,
	INVALC,INVALC,0x5630,0x784A,INVALC,0x7C7A,INVALC,0x597E,
	INVALC,INVALC,INVALC,INVALC,0x5E30,INVALC,INVALC,INVALC,
	0x5D6C,0x5A68,INVALC,INVALC,INVALC,INVALC,INVALC,0x5460,
	INVALC,0x5679,INVALC,0x4D57,0x5E58,INVALC,INVALC,0x7278,
	INVALC,0x6456,INVALC,INVALC,0x5045,0x742E,INVALC,0x5D28,
	INVALC,INVALC,INVALC,0x6D45,0x7356,INVALC,INVALC,0x5E59,
	0x6366,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5328,
	INVALC,0x5B30,INVALC,INVALC,0x655A,0x633F,INVALC,INVALC,
	INVALC,0x5B31,INVALC,INVALC,INVALC,0x5569,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6041,0x6F5B,INVALC,INVALC,0x7069,
	INVALC,INVALC,0x5732,INVALC,INVALC,INVALC,INVALC,0x507D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5969,INVALC,INVALC,0x507E,INVALC,0x6C6D,0x5329,INVALC,
	0x7229,0x7044,INVALC,INVALC,0x6262,0x696F,INVALC,INVALC,
	INVALC,0x7951,INVALC,INVALC,0x6959,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x685A,0x5A43,0x5A44,0x5445,INVALC,0x677A,
	/* Block 103, Array index 0x2900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D60,0x6330,
	INVALC,0x5B32,0x7B44,INVALC,0x7363,INVALC,0x5925,0x7B67,
	0x5D4B,0x5054,INVALC,0x6636,0x602E,0x7D5A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C35,0x6078,
	0x6731,INVALC,INVALC,INVALC,INVALC,0x7570,INVALC,0x585C,
	0x6D46,INVALC,INVALC,INVALC,INVALC,0x6139,INVALC,0x6340,
	0x7940,0x6970,INVALC,INVALC,0x595B,INVALC,0x7364,INVALC,
	INVALC,INVALC,0x5C36,0x6469,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7045,0x6341,0x7C4C,0x7C4D,INVALC,INVALC,INVALC,
	INVALC,0x724B,INVALC,0x724C,INVALC,INVALC,INVALC,0x644F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x715B,INVALC,0x7A59,INVALC,0x7138,0x7D75,0x6079,
	INVALC,INVALC,INVALC,INVALC,0x677B,0x7C37,0x7C64,0x7B45,
	0x6367,0x5839,INVALC,0x7678,INVALC,INVALC,0x5C45,0x4C58,
	INVALC,INVALC,INVALC,0x602F,0x7467,INVALC,0x6F5C,0x4F7C,
	0x6F5D,INVALC,0x722A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7D3E,INVALC,INVALC,0x4A2C,
	0x7D3B,0x7D47,INVALC,INVALC,INVALC,INVALC,0x6732,INVALC,
	0x6A51,0x5F74,INVALC,INVALC,INVALC,INVALC,0x516C,0x645E,
	0x6543,INVALC,INVALC,INVALC,INVALC,0x5926,INVALC,INVALC,
	0x4D3C,INVALC,0x7365,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6D55,INVALC,INVALC,0x593A,INVALC,
	INVALC,INVALC,0x6D67,0x7B35,0x786C,INVALC,INVALC,INVALC,
	INVALC,0x6067,INVALC,0x4C59,INVALC,INVALC,0x5446,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6725,INVALC,0x5575,0x533E,
	INVALC,0x7C7B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6472,INVALC,
	INVALC,0x5F75,INVALC,INVALC,0x6878,0x786D,INVALC,INVALC,
	0x4E47,INVALC,INVALC,INVALC,INVALC,INVALC,0x7D76,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6858,INVALC,0x4D58,0x6756,0x4C5A,INVALC,INVALC,0x4A63,
	0x5F76,0x7047,0x7046,INVALC,0x583A,INVALC,0x7174,0x7470,
	/* Block 104, Array index 0x2A00 */
	0x754C,INVALC,INVALC,0x7C65,INVALC,INVALC,INVALC,INVALC,
	0x6A45,0x6A73,INVALC,0x5D5B,INVALC,0x5C57,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E7D,0x7279,INVALC,0x5547,
	INVALC,INVALC,INVALC,0x5850,INVALC,0x7048,0x5121,0x5122,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5954,0x5668,
	0x594A,INVALC,0x5A31,0x5847,0x5C62,0x734E,0x7574,INVALC,
	INVALC,0x7139,INVALC,INVALC,0x5A53,INVALC,0x766A,INVALC,
	INVALC,INVALC,0x4F75,INVALC,INVALC,0x7D2E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A52,INVALC,
	INVALC,0x5F34,INVALC,INVALC,INVALC,INVALC,0x575D,0x7A3A,
	0x6E27,0x753D,INVALC,0x7875,INVALC,INVALC,0x6D68,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5461,INVALC,0x5123,0x6156,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7978,INVALC,0x5B4A,
	0x4B79,0x5454,INVALC,0x595C,INVALC,0x6E3E,INVALC,0x776D,
	INVALC,INVALC,INVALC,0x526E,INVALC,INVALC,0x6166,0x7779,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D6D,
	INVALC,0x685B,INVALC,0x5B33,INVALC,INVALC,INVALC,0x5177,
	0x6030,INVALC,INVALC,0x5462,INVALC,0x7657,INVALC,0x5779,
	INVALC,INVALC,0x585D,INVALC,0x4D7D,0x722B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D3D,
	0x7842,INVALC,INVALC,0x722C,INVALC,INVALC,0x4A2D,0x4A2E,
	0x4F2E,INVALC,INVALC,INVALC,INVALC,INVALC,0x6342,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5C37,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B5A,
	0x593B,0x4A73,0x7653,0x6678,0x6A75,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6A76,INVALC,INVALC,0x7679,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F2F,INVALC,INVALC,0x4A53,INVALC,INVALC,0x4A2F,
	0x5230,0x713A,INVALC,0x5733,0x6343,0x737D,0x5E5A,INVALC,
	INVALC,INVALC,INVALC,0x5E5B,INVALC,INVALC,0x6F5E,INVALC,
	/* Block 105, Array index 0x2B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6263,0x6E7B,INVALC,INVALC,0x5F77,0x574A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E68,0x5B5B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x713B,INVALC,INVALC,INVALC,INVALC,0x6971,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7A37,0x5046,INVALC,INVALC,0x4C2B,0x6E28,INVALC,INVALC,
	0x4B7A,0x7979,0x4C7D,0x537E,INVALC,INVALC,INVALC,INVALC,
	0x6450,INVALC,INVALC,INVALC,INVALC,INVALC,0x726E,INVALC,
	0x5455,0x5F4D,INVALC,0x7C38,0x5150,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x724D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7752,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A54,
	INVALC,0x5559,INVALC,INVALC,INVALC,0x585E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D59,
	INVALC,INVALC,INVALC,0x6E29,0x763C,INVALC,INVALC,0x4C5B,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7049,INVALC,0x7C7C,
	INVALC,0x6849,0x747E,INVALC,INVALC,INVALC,INVALC,0x677C,
	0x575E,INVALC,INVALC,INVALC,INVALC,0x5E5C,INVALC,0x702C,
	0x4C7E,0x4D61,INVALC,0x613A,INVALC,0x5B6F,0x5A32,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5125,0x5C38,INVALC,INVALC,
	INVALC,0x5876,INVALC,0x5124,INVALC,0x4D62,INVALC,INVALC,
	INVALC,INVALC,0x5C6A,INVALC,INVALC,0x7077,INVALC,0x704A,
	0x503E,INVALC,0x5D5C,INVALC,INVALC,INVALC,INVALC,0x5456,
	0x5356,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D50,
	0x4D21,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F35,INVALC,
	INVALC,INVALC,0x5F78,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5421,0x4E32,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 106, Array index 0x2C00 */
	0x684A,INVALC,INVALC,INVALC,INVALC,0x6B75,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6355,0x7550,INVALC,
	INVALC,INVALC,0x7521,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5927,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x652B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x664B,INVALC,0x7571,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6545,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7923,0x605B,0x766B,INVALC,0x4B71,
	INVALC,0x596A,0x7522,INVALC,INVALC,INVALC,0x5751,INVALC,
	0x5178,0x6A78,0x6A79,0x5A33,INVALC,INVALC,INVALC,INVALC,
	0x6F5F,INVALC,INVALC,0x716F,INVALC,0x6576,0x6E3F,0x6264,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x503F,INVALC,0x7A2C,
	0x7551,0x6733,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x693E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x724E,0x5B34,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7C4E,INVALC,0x5D6E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6734,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5734,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7734,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D3E,INVALC,0x5A69,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4F30,0x7759,0x7366,INVALC,INVALC,
	0x4E59,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4E2A,INVALC,0x4B48,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5027,INVALC,0x704B,INVALC,0x5047,
	/* Block 107, Array index 0x2D00 */
	INVALC,INVALC,0x6445,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5B60,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x555A,INVALC,0x5727,INVALC,INVALC,INVALC,0x6E40,
	INVALC,0x7876,INVALC,INVALC,INVALC,INVALC,0x7552,0x6D69,
	INVALC,0x593C,INVALC,0x6546,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7523,INVALC,INVALC,
	0x5A54,0x6227,0x7B7C,INVALC,INVALC,0x715C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4A74,INVALC,INVALC,INVALC,
	0x687A,INVALC,INVALC,0x4E69,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6978,0x6265,INVALC,INVALC,INVALC,INVALC,
	0x5039,0x5472,INVALC,INVALC,INVALC,INVALC,INVALC,0x5126,
	INVALC,0x5F4E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7C74,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x532A,INVALC,INVALC,INVALC,0x4C2C,INVALC,INVALC,INVALC,
	INVALC,0x6F60,INVALC,INVALC,INVALC,0x6565,0x5055,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B7C,
	INVALC,INVALC,INVALC,INVALC,0x7C66,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4B7E,0x6D6A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E31,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7963,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5422,0x4F76,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5650,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x556A,0x716E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7A4B,INVALC,
	INVALC,INVALC,INVALC,0x6521,INVALC,INVALC,INVALC,INVALC,
	/* Block 108, Array index 0x2E00 */
	INVALC,INVALC,INVALC,INVALC,0x5531,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F6D,INVALC,0x6D6B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5532,INVALC,INVALC,INVALC,0x553C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7D62,0x732D,INVALC,0x7D5B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6930,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5127,INVALC,INVALC,0x7D63,0x4E33,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7D64,0x7A4E,
	INVALC,INVALC,INVALC,INVALC,0x4A30,INVALC,0x7727,INVALC,
	0x4F31,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6622,
	INVALC,0x7C36,0x722D,0x6F61,0x732E,0x5C46,0x596B,INVALC,
	INVALC,INVALC,0x6860,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6128,INVALC,INVALC,INVALC,INVALC,0x5576,
	0x4F7D,INVALC,INVALC,0x5E5D,INVALC,INVALC,INVALC,0x5951,
	INVALC,INVALC,INVALC,0x646A,0x724F,INVALC,0x773F,INVALC,
	INVALC,0x6266,0x6228,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6356,INVALC,
	0x6D51,INVALC,INVALC,INVALC,INVALC,INVALC,0x6979,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5631,0x5E32,
	INVALC,INVALC,0x6068,INVALC,INVALC,0x532B,INVALC,0x6B5C,
	INVALC,INVALC,0x5F2F,INVALC,0x4A43,INVALC,INVALC,0x6E7C,
	INVALC,0x7D43,INVALC,INVALC,INVALC,0x6B76,0x4F32,INVALC,
	INVALC,INVALC,INVALC,0x596C,INVALC,0x593D,INVALC,0x585F,
	INVALC,INVALC,0x5438,0x6B3E,0x5D6F,INVALC,0x5D70,0x5D71,
	0x5D72,INVALC,INVALC,0x593E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7B46,0x4F33,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 109, Array index 0x2F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6E7D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x642B,
	INVALC,0x5A45,INVALC,0x586C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5128,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6229,INVALC,INVALC,0x5E3C,
	0x6735,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B70,INVALC,
	0x6F62,0x7170,0x4F34,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B71,INVALC,
	0x6031,INVALC,INVALC,INVALC,INVALC,0x5F25,INVALC,0x7952,
	INVALC,INVALC,0x677D,INVALC,INVALC,0x6623,0x7B71,0x4B30,
	0x722E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D67,INVALC,0x685C,INVALC,INVALC,0x6757,INVALC,INVALC,
	0x7740,INVALC,0x5063,INVALC,INVALC,INVALC,0x5A21,INVALC,
	INVALC,INVALC,0x4C3D,INVALC,INVALC,0x5129,0x5D4C,INVALC,
	INVALC,0x637E,0x512A,0x682A,INVALC,0x6A36,0x797A,0x664C,
	0x7658,INVALC,INVALC,INVALC,0x5447,INVALC,INVALC,INVALC,
	0x594B,INVALC,0x5952,0x534B,0x5877,0x5A29,0x7578,INVALC,
	INVALC,0x5E5E,0x722F,0x7829,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5848,INVALC,INVALC,0x6E41,INVALC,
	INVALC,INVALC,INVALC,0x7941,INVALC,INVALC,INVALC,INVALC,
	0x5D73,0x6A7A,INVALC,0x763D,0x613B,0x4D3F,0x7454,0x664D,
	INVALC,0x7C4F,0x7B22,INVALC,0x605C,INVALC,INVALC,INVALC,
	INVALC,0x743B,0x5A55,INVALC,0x7932,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7B72,INVALC,0x5B76,INVALC,0x5E5F,
	INVALC,INVALC,INVALC,0x5B72,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x785C,0x776E,0x6B68,INVALC,0x527A,INVALC,INVALC,
	0x713C,INVALC,INVALC,0x7A5A,INVALC,INVALC,0x5A6A,0x5A46,
	0x7741,INVALC,INVALC,0x6736,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 110, Array index 0x3000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6547,0x562C,INVALC,INVALC,INVALC,
	INVALC,0x5C47,INVALC,INVALC,INVALC,INVALC,INVALC,0x6129,
	INVALC,0x622A,INVALC,0x5526,INVALC,INVALC,0x5457,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7250,INVALC,0x6A7B,
	INVALC,0x605D,0x7B73,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x713D,INVALC,INVALC,0x6267,0x7D57,INVALC,
	0x4E48,0x6A37,INVALC,0x7C40,INVALC,0x7D67,0x776F,INVALC,
	INVALC,0x5735,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F3A,0x715D,INVALC,0x5E33,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x684B,INVALC,INVALC,0x785D,INVALC,
	INVALC,0x7B47,0x5548,INVALC,0x575F,INVALC,0x5D29,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6931,INVALC,INVALC,0x7A2D,
	0x7659,0x7A74,INVALC,INVALC,INVALC,INVALC,INVALC,0x782A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x666E,INVALC,0x4C5C,
	0x613C,0x606F,INVALC,INVALC,0x693F,0x7C7D,0x664E,INVALC,
	INVALC,0x6157,INVALC,0x664F,INVALC,0x7471,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6473,
	INVALC,INVALC,0x647B,INVALC,INVALC,0x7964,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F63,INVALC,INVALC,0x4F6E,0x763E,0x6032,0x7C7E,0x512B,
	INVALC,INVALC,INVALC,0x577A,INVALC,INVALC,INVALC,0x7B48,
	INVALC,0x6257,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5423,0x7078,0x5728,INVALC,INVALC,INVALC,0x6167,INVALC,
	INVALC,0x533F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F64,INVALC,0x5745,0x6B62,INVALC,INVALC,0x7C67,INVALC,
	INVALC,0x6422,INVALC,0x6268,INVALC,0x6650,INVALC,0x7B68,
	0x7468,0x6574,0x743C,0x7455,INVALC,INVALC,INVALC,INVALC,
	/* Block 111, Array index 0x3100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F36,
	INVALC,0x7C39,0x6E42,0x4A75,INVALC,INVALC,INVALC,0x6F65,
	0x4B62,0x5424,INVALC,0x5E60,0x5A7D,0x6446,0x683E,INVALC,
	INVALC,INVALC,INVALC,0x605E,0x7634,0x6A52,INVALC,0x797B,
	INVALC,INVALC,0x6042,INVALC,0x4A64,INVALC,0x6737,INVALC,
	0x6A7D,INVALC,0x595D,INVALC,0x5A34,0x6E2A,0x7B69,INVALC,
	INVALC,INVALC,INVALC,0x5B4B,0x5A35,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x713E,INVALC,INVALC,0x532C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7B49,INVALC,
	0x5F4F,INVALC,INVALC,0x5340,0x6357,INVALC,0x6F66,0x7C50,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6940,
	INVALC,INVALC,INVALC,0x7553,INVALC,INVALC,0x6C5C,0x7737,
	INVALC,INVALC,0x6A38,0x5179,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C48,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6A39,INVALC,INVALC,INVALC,INVALC,INVALC,0x715E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5736,0x4F35,INVALC,0x5928,
	INVALC,INVALC,0x6C6E,INVALC,INVALC,0x5D2A,INVALC,INVALC,
	INVALC,INVALC,0x4D22,0x682E,INVALC,INVALC,INVALC,0x613D,
	INVALC,0x7251,INVALC,INVALC,INVALC,INVALC,0x6941,INVALC,
	INVALC,INVALC,0x527C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5B35,INVALC,0x7367,0x587E,INVALC,INVALC,
	INVALC,0x7C51,INVALC,0x6D32,0x742F,INVALC,0x7B23,INVALC,
	INVALC,0x7C41,INVALC,0x6E2B,0x5425,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7472,
	INVALC,INVALC,INVALC,INVALC,0x6E59,INVALC,INVALC,INVALC,
	0x7B4A,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D63,0x583B,
	/* Block 112, Array index 0x3200 */
	INVALC,0x655B,0x7877,INVALC,INVALC,INVALC,0x7654,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5729,
	INVALC,0x4B49,INVALC,INVALC,0x6651,0x704C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x582E,INVALC,0x7953,0x557E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x583C,0x7230,INVALC,INVALC,INVALC,
	INVALC,0x622B,0x7368,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F42,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D6C,
	INVALC,0x6738,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5A7E,INVALC,INVALC,0x4C3E,INVALC,INVALC,0x727C,
	0x5A6B,INVALC,0x6258,0x6D56,INVALC,INVALC,0x5651,0x6033,
	INVALC,INVALC,0x7C52,INVALC,0x6B48,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5341,0x704D,INVALC,
	0x4F77,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6D52,INVALC,0x5458,0x5C49,0x5771,INVALC,
	INVALC,0x5F3B,INVALC,INVALC,0x7325,INVALC,INVALC,INVALC,
	0x744D,INVALC,INVALC,INVALC,INVALC,0x713F,0x7831,INVALC,
	0x697A,INVALC,INVALC,INVALC,0x7B4B,INVALC,INVALC,0x4A55,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7954,0x774A,INVALC,INVALC,0x5648,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7C68,0x733D,
	INVALC,0x6E7E,INVALC,0x677E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5342,INVALC,INVALC,INVALC,INVALC,0x5336,INVALC,
	0x4C2D,0x767A,0x5632,0x5258,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6758,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6325,INVALC,INVALC,
	INVALC,INVALC,0x6739,INVALC,INVALC,INVALC,INVALC,0x702D,
	0x7B4C,0x6B21,INVALC,INVALC,0x5426,INVALC,INVALC,INVALC,
	INVALC,0x7B4D,INVALC,0x553D,0x715F,INVALC,INVALC,0x767B,
	INVALC,0x5E34,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x556B,INVALC,
	/* Block 113, Array index 0x3300 */
	INVALC,0x6548,INVALC,INVALC,INVALC,0x7B24,0x5439,INVALC,
	INVALC,0x5E61,INVALC,0x6423,INVALC,INVALC,INVALC,0x5737,
	INVALC,0x786E,INVALC,INVALC,INVALC,0x5E35,INVALC,INVALC,
	0x5652,INVALC,0x7955,0x673A,0x6B55,0x5577,0x6F67,0x613E,
	INVALC,INVALC,INVALC,0x7A2E,INVALC,INVALC,INVALC,0x5669,
	0x566E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x673B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6C4B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5533,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4E34,INVALC,INVALC,INVALC,
	0x7B25,0x616E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7728,INVALC,INVALC,INVALC,INVALC,0x7B4E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x583D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7B7D,INVALC,INVALC,INVALC,INVALC,
	0x7C69,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4F36,INVALC,INVALC,INVALC,0x6D47,0x6E2C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4C5D,INVALC,INVALC,
	INVALC,INVALC,0x7627,INVALC,INVALC,INVALC,0x667A,INVALC,
	INVALC,INVALC,0x7524,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7D5C,0x6D33,0x4E49,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F68,INVALC,INVALC,0x613F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7A5B,0x4B63,0x7729,INVALC,0x7B26,
	INVALC,INVALC,INVALC,0x5C39,INVALC,INVALC,INVALC,0x7140,
	0x6D48,0x6F43,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x562D,INVALC,INVALC,0x7D4E,INVALC,INVALC,INVALC,0x6821,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7B74,0x5527,INVALC,INVALC,INVALC,0x7176,INVALC,0x6653,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C5E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7832,INVALC,INVALC,INVALC,0x5C6B,INVALC,INVALC,
	/* Block 114, Array index 0x3400 */
	INVALC,INVALC,INVALC,INVALC,0x7D36,INVALC,INVALC,INVALC,
	INVALC,0x656A,INVALC,INVALC,0x7160,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5B4C,0x5D4D,INVALC,INVALC,INVALC,0x5448,INVALC,
	INVALC,0x596D,INVALC,INVALC,INVALC,INVALC,0x7525,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x667B,INVALC,INVALC,INVALC,INVALC,INVALC,0x6654,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7D48,
	INVALC,0x5621,0x7D3F,INVALC,0x7C53,INVALC,0x6F21,INVALC,
	0x673C,INVALC,INVALC,INVALC,INVALC,INVALC,0x516E,INVALC,
	INVALC,0x6655,INVALC,INVALC,0x6972,INVALC,0x5F30,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5860,0x7C3A,0x7D2F,0x704E,
	INVALC,0x5B61,INVALC,INVALC,0x6549,INVALC,0x6D34,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6043,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6358,INVALC,INVALC,INVALC,
	INVALC,0x697B,0x6A28,INVALC,INVALC,INVALC,INVALC,0x7D37,
	INVALC,INVALC,0x7B27,INVALC,0x6942,INVALC,INVALC,INVALC,
	INVALC,0x7D77,INVALC,INVALC,INVALC,INVALC,INVALC,0x6259,
	INVALC,INVALC,0x5C6C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6822,INVALC,INVALC,INVALC,
	INVALC,0x6670,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7D78,0x7D79,INVALC,INVALC,INVALC,0x763F,INVALC,
	INVALC,0x6727,INVALC,0x6657,INVALC,INVALC,INVALC,INVALC,
	0x5473,0x5449,INVALC,INVALC,INVALC,INVALC,0x567A,INVALC,
	0x5772,INVALC,0x6140,INVALC,0x5B62,0x6658,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x673D,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x704F,0x733E,0x622C,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7537,0x6070,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7D38,0x6368,INVALC,0x5427,0x687C,
	/* Block 115, Array index 0x3500 */
	0x7A52,INVALC,INVALC,INVALC,INVALC,INVALC,0x786F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5653,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5534,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7050,INVALC,0x7770,0x6E33,INVALC,INVALC,
	0x6A3A,INVALC,0x6A53,INVALC,INVALC,0x6D49,0x5D2B,INVALC,
	INVALC,INVALC,0x652C,0x7D21,INVALC,0x5F50,0x6C33,INVALC,
	0x5F51,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D6D,0x7838,
	0x777A,INVALC,INVALC,INVALC,0x782B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7460,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x543A,0x6433,INVALC,0x695A,INVALC,0x5E36,INVALC,0x593F,
	INVALC,0x5940,0x566F,INVALC,INVALC,INVALC,INVALC,0x594C,
	INVALC,0x5A2A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5F65,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7765,INVALC,INVALC,INVALC,0x4C32,INVALC,INVALC,
	0x5F79,0x5760,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x543B,INVALC,INVALC,INVALC,INVALC,0x7D7A,
	INVALC,INVALC,INVALC,INVALC,0x4C33,INVALC,INVALC,0x5B73,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5F52,INVALC,0x4E4A,INVALC,0x6E5A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6464,INVALC,
	0x7B4F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F37,
	INVALC,0x6E43,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E6A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x622D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5761,0x7A75,INVALC,INVALC,0x5549,0x782C,INVALC,INVALC,
	/* Block 116, Array index 0x3600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6759,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7369,INVALC,
	INVALC,INVALC,INVALC,0x586D,0x6344,0x7071,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6865,INVALC,INVALC,
	INVALC,0x607A,0x6E44,0x595E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B22,0x6B23,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7C42,0x6A3B,
	INVALC,INVALC,INVALC,INVALC,0x682B,0x5E62,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6D6F,INVALC,0x6823,INVALC,INVALC,INVALC,INVALC,0x4F71,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x543C,INVALC,0x7C6A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x673E,INVALC,0x7C72,INVALC,INVALC,0x5634,INVALC,INVALC,
	0x622E,INVALC,0x5337,0x7A4C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7A5C,INVALC,0x6D35,0x6163,
	INVALC,0x682C,INVALC,0x685D,INVALC,INVALC,0x6F69,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x743D,INVALC,0x4F38,0x695B,
	0x512C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A47,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B49,
	INVALC,0x684C,INVALC,INVALC,INVALC,0x5E37,INVALC,INVALC,
	INVALC,INVALC,0x563C,0x5365,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7A5D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A56,INVALC,0x4A31,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5A48,0x5F26,INVALC,0x7933,0x7252,INVALC,0x4A44,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E4B,INVALC,
	INVALC,0x4D75,INVALC,INVALC,INVALC,0x7D30,0x5528,INVALC,
	0x7141,INVALC,INVALC,0x6269,0x5C4A,0x6C34,0x7A40,INVALC,
	INVALC,0x7B28,0x5028,INVALC,INVALC,0x5A6C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x596E,
	INVALC,0x607B,INVALC,INVALC,INVALC,0x6F6A,0x7A5E,INVALC,
	/* Block 117, Array index 0x3700 */
	INVALC,0x6044,INVALC,0x4F39,INVALC,0x554A,0x5762,0x622F,
	INVALC,0x5738,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x684D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x765A,INVALC,INVALC,INVALC,INVALC,
	0x6F22,0x625A,0x767C,INVALC,INVALC,0x7B50,0x512D,INVALC,
	0x4D64,INVALC,0x512E,0x5C6D,0x684E,INVALC,0x7079,0x4E35,
	0x667C,INVALC,INVALC,0x577B,0x5056,0x5D75,0x7771,INVALC,
	0x767D,INVALC,0x5B77,INVALC,INVALC,INVALC,INVALC,0x7B6A,
	0x695C,0x5941,INVALC,0x7572,0x6045,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6A54,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7942,INVALC,0x6A3C,
	INVALC,0x5245,0x7B51,0x6740,0x6B25,INVALC,0x5F7A,0x6322,
	0x5739,INVALC,0x6943,INVALC,0x687D,0x682F,INVALC,INVALC,
	0x7253,0x7B29,0x5825,INVALC,INVALC,INVALC,INVALC,0x554B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5048,INVALC,
	INVALC,INVALC,0x512F,0x5763,INVALC,INVALC,INVALC,0x6046,
	INVALC,0x5622,INVALC,0x6D70,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5773,INVALC,INVALC,INVALC,0x7C54,INVALC,0x5A57,0x4C5F,
	INVALC,0x7254,INVALC,0x5130,INVALC,0x4C60,INVALC,0x5B7D,
	0x733F,INVALC,0x7051,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7C3B,INVALC,0x6230,INVALC,INVALC,0x6625,INVALC,INVALC,
	INVALC,0x625B,INVALC,INVALC,INVALC,0x5F5E,INVALC,0x6047,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x726F,INVALC,
	INVALC,INVALC,0x4C61,INVALC,INVALC,INVALC,INVALC,0x566A,
	INVALC,INVALC,INVALC,INVALC,0x6742,INVALC,INVALC,INVALC,
	0x4E36,INVALC,0x7340,INVALC,0x4D7E,INVALC,INVALC,INVALC,
	0x7B52,INVALC,0x7878,0x777B,INVALC,INVALC,0x683F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6837,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D36,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 118, Array index 0x3800 */
	INVALC,0x5C3A,INVALC,INVALC,0x4C34,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7177,INVALC,INVALC,INVALC,0x6838,INVALC,INVALC,
	0x4A76,INVALC,0x6424,0x7456,INVALC,INVALC,INVALC,0x5F66,
	INVALC,INVALC,INVALC,0x5F27,INVALC,0x5F67,0x6141,INVALC,
	0x6944,INVALC,INVALC,0x5C4B,0x6945,INVALC,INVALC,INVALC,
	0x6F23,0x6B26,0x4B23,0x6369,INVALC,INVALC,INVALC,0x517B,
	0x6F24,INVALC,0x6F6B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5034,
	INVALC,INVALC,INVALC,INVALC,0x4D23,INVALC,INVALC,0x6866,
	INVALC,0x6F25,INVALC,0x534C,0x5A6D,INVALC,INVALC,INVALC,
	INVALC,0x573A,INVALC,0x7255,0x7565,0x596F,INVALC,INVALC,
	INVALC,INVALC,0x7934,INVALC,INVALC,0x5554,0x7D4F,INVALC,
	INVALC,INVALC,0x5B63,INVALC,INVALC,INVALC,INVALC,0x7161,
	0x6C36,INVALC,INVALC,INVALC,INVALC,0x7B7E,0x5357,INVALC,
	0x5131,INVALC,0x4B31,INVALC,INVALC,INVALC,INVALC,0x5132,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B32,0x7142,
	INVALC,INVALC,0x7461,0x7935,INVALC,INVALC,0x6143,0x6142,
	INVALC,0x6B77,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F28,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B4A,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6639,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x785E,0x792A,0x4A77,INVALC,0x6D37,INVALC,INVALC,
	0x5338,0x7256,INVALC,INVALC,0x5459,0x6E45,0x7270,INVALC,
	INVALC,INVALC,0x4A32,INVALC,INVALC,0x5C3B,INVALC,0x7178,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6C37,INVALC,INVALC,
	INVALC,INVALC,0x654A,INVALC,0x7640,0x7D5D,INVALC,INVALC,
	0x5463,0x4C62,INVALC,0x7754,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5765,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5343,INVALC,0x5826,INVALC,0x7641,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D76,INVALC,0x4D40,INVALC,INVALC,INVALC,
	/* Block 119, Array index 0x3900 */
	0x655C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x654B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6144,INVALC,INVALC,INVALC,INVALC,INVALC,0x6830,
	0x7430,0x736A,0x5A6E,INVALC,0x573B,INVALC,0x6231,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x572A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x567B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x645F,INVALC,0x4A56,INVALC,
	INVALC,INVALC,0x6B28,INVALC,INVALC,INVALC,0x5B7E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7642,INVALC,INVALC,0x6F3B,INVALC,INVALC,INVALC,
	INVALC,0x547D,INVALC,INVALC,0x6048,INVALC,INVALC,INVALC,
	0x6839,INVALC,0x6F26,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D24,INVALC,INVALC,0x5474,0x5B21,0x5B5C,0x5B5D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6E5C,INVALC,0x4B4B,0x7C55,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E6B,INVALC,
	0x4D41,INVALC,INVALC,0x7B53,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x792B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7554,INVALC,INVALC,INVALC,INVALC,0x5929,
	INVALC,INVALC,0x695D,0x5B4D,INVALC,INVALC,0x5D4E,INVALC,
	0x6743,INVALC,0x6C4C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x796C,INVALC,INVALC,INVALC,0x4B4C,INVALC,
	INVALC,INVALC,INVALC,0x607C,0x5428,INVALC,0x6D53,0x586F,
	INVALC,0x7257,INVALC,0x4A78,0x5A6F,INVALC,INVALC,0x5654,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x594D,INVALC,
	INVALC,INVALC,0x586E,INVALC,0x7241,INVALC,INVALC,INVALC,
	0x5F53,INVALC,INVALC,INVALC,0x5A70,INVALC,0x626A,INVALC,
	/* Block 120, Array index 0x3A00 */
	INVALC,0x607D,INVALC,INVALC,0x5878,INVALC,INVALC,INVALC,
	0x772F,0x5A36,INVALC,0x4A57,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7258,INVALC,
	0x5879,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7A5F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F6F,
	0x5942,INVALC,0x7052,INVALC,0x6451,INVALC,INVALC,INVALC,
	0x7337,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7A60,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6F6C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6232,INVALC,0x543D,INVALC,INVALC,0x594E,INVALC,
	INVALC,INVALC,INVALC,0x7462,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5429,INVALC,INVALC,INVALC,INVALC,0x4D42,0x675A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7259,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x592A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x583E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5C2D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x626B,0x567C,INVALC,INVALC,
	0x4A79,INVALC,INVALC,0x545A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7457,0x4C21,INVALC,INVALC,0x4F3A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7538,
	INVALC,INVALC,INVALC,0x5943,0x5068,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6345,0x6B78,INVALC,0x7231,INVALC,INVALC,
	INVALC,0x4F3B,INVALC,INVALC,INVALC,0x532D,0x6861,0x4E6C,
	INVALC,INVALC,INVALC,0x6034,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 121, Array index 0x3B00 */
	INVALC,INVALC,0x5E63,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D77,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7232,0x7376,0x765B,
	INVALC,INVALC,INVALC,INVALC,0x577E,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x785F,INVALC,0x7772,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5029,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x665A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7526,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x573C,0x4C63,INVALC,INVALC,0x665B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D5D,INVALC,INVALC,INVALC,
	INVALC,0x5133,INVALC,INVALC,INVALC,INVALC,INVALC,0x6F6D,
	INVALC,INVALC,INVALC,INVALC,0x565E,0x6474,0x616F,INVALC,
	INVALC,0x5D78,INVALC,INVALC,INVALC,INVALC,INVALC,0x684F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A65,INVALC,INVALC,INVALC,0x5C21,
	INVALC,0x6035,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7C2C,0x7C2D,0x5827,INVALC,INVALC,INVALC,
	INVALC,0x6D38,INVALC,INVALC,INVALC,0x5B36,INVALC,INVALC,
	INVALC,INVALC,0x5670,0x732F,INVALC,INVALC,0x4D25,INVALC,
	0x5A71,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5828,INVALC,INVALC,INVALC,0x4C64,INVALC,INVALC,0x5134,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4A58,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5A72,INVALC,INVALC,INVALC,INVALC,
	/* Block 122, Array index 0x3C00 */
	INVALC,0x7527,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7528,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6626,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x556C,0x5578,0x5A73,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6346,INVALC,INVALC,INVALC,0x5E64,INVALC,
	0x5E65,0x5135,INVALC,INVALC,INVALC,INVALC,INVALC,0x5136,
	0x5137,0x7233,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x695E,INVALC,INVALC,INVALC,INVALC,INVALC,0x7053,0x7234,
	INVALC,INVALC,0x7054,0x4B64,0x7B54,0x7566,0x636A,INVALC,
	0x5E66,INVALC,INVALC,INVALC,INVALC,0x5F54,INVALC,INVALC,
	0x7879,INVALC,INVALC,INVALC,INVALC,0x702E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5138,INVALC,INVALC,INVALC,INVALC,0x565F,
	INVALC,0x5057,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7C21,0x6F6E,0x5C58,
	INVALC,0x695F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x655D,0x7D7B,
	INVALC,INVALC,0x6049,INVALC,INVALC,INVALC,0x5649,INVALC,
	INVALC,0x542A,INVALC,0x654C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6960,INVALC,INVALC,INVALC,0x5058,0x7C22,0x543E,
	0x6233,0x5E67,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C3C,0x5236,0x7555,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E21,0x7529,INVALC,INVALC,0x5D79,INVALC,INVALC,
	0x5D7A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7055,
	INVALC,INVALC,INVALC,INVALC,0x765F,INVALC,0x725A,0x646B,
	INVALC,0x7271,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6C39,INVALC,INVALC,INVALC,INVALC,
	/* Block 123, Array index 0x3D00 */
	0x7D7C,INVALC,INVALC,INVALC,INVALC,0x612A,INVALC,INVALC,
	0x4A59,INVALC,INVALC,0x6F6F,INVALC,0x752A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6C79,INVALC,INVALC,0x782D,
	INVALC,0x7242,0x7643,INVALC,0x5752,INVALC,INVALC,0x7922,
	0x7056,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x707A,INVALC,
	INVALC,0x7660,INVALC,INVALC,INVALC,INVALC,0x6973,0x7243,
	INVALC,INVALC,INVALC,0x542B,0x4A33,0x4D26,INVALC,0x4D43,
	0x4D5A,INVALC,INVALC,INVALC,INVALC,INVALC,0x594F,INVALC,
	INVALC,0x7644,INVALC,INVALC,INVALC,0x6E5D,0x6744,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6234,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F62,0x675B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6831,INVALC,0x7C2E,INVALC,INVALC,INVALC,INVALC,
	0x654D,INVALC,INVALC,INVALC,0x7A6B,INVALC,0x4F3C,INVALC,
	INVALC,0x4F62,0x4D76,INVALC,INVALC,0x6F70,INVALC,0x743E,
	INVALC,0x544D,INVALC,INVALC,0x7338,INVALC,INVALC,INVALC,
	0x6921,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7272,0x736B,INVALC,0x7057,INVALC,INVALC,
	0x4F57,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F5F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6840,0x6841,INVALC,INVALC,INVALC,0x4F63,0x6922,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x502A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7341,0x502B,INVALC,0x5464,
	INVALC,INVALC,0x6F3C,0x5821,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x595F,
	INVALC,0x7357,INVALC,INVALC,INVALC,0x5C3D,INVALC,0x4C65,
	0x6D71,INVALC,INVALC,0x7162,INVALC,0x545B,INVALC,INVALC,
	INVALC,INVALC,0x6235,INVALC,INVALC,0x4A66,INVALC,0x532E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C66,INVALC,
	INVALC,0x7153,0x7567,INVALC,INVALC,INVALC,INVALC,0x4A5A,
	/* Block 124, Array index 0x3E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7B6E,
	INVALC,0x6145,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F69,INVALC,0x6E5E,INVALC,INVALC,0x7742,INVALC,
	0x5822,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D2C,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x702F,0x563D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x612B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7936,INVALC,
	INVALC,0x5475,INVALC,0x5049,0x6F27,0x626C,INVALC,0x5B6A,
	0x4E4C,0x7568,0x7755,INVALC,0x534D,INVALC,0x737E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5035,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x607E,INVALC,0x5F7B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x665D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6824,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B4D,
	INVALC,INVALC,INVALC,0x6F28,INVALC,INVALC,INVALC,0x6E34,
	INVALC,INVALC,INVALC,INVALC,0x5A58,0x5139,INVALC,0x5F29,
	INVALC,INVALC,0x7330,INVALC,INVALC,0x4C44,INVALC,INVALC,
	INVALC,0x4E37,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6F29,INVALC,INVALC,
	INVALC,0x5F55,INVALC,INVALC,0x6D57,INVALC,INVALC,INVALC,
	0x6E46,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F3D,0x7C56,INVALC,INVALC,0x5B74,INVALC,0x6F2A,0x7839,
	INVALC,0x7569,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6359,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6146,INVALC,INVALC,INVALC,0x543F,INVALC,INVALC,INVALC,
	INVALC,0x5E68,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 125, Array index 0x3F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x706A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7342,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x532F,INVALC,
	INVALC,0x4A5B,INVALC,INVALC,INVALC,INVALC,INVALC,0x7C57,
	INVALC,INVALC,0x6D58,0x6147,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7458,0x5633,0x5D2D,
	INVALC,INVALC,INVALC,0x553E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7143,0x6E5F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x566B,INVALC,INVALC,INVALC,0x7459,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5766,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5A37,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D7B,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D4F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5823,INVALC,INVALC,0x5A59,INVALC,0x7058,
	0x6F44,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6158,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7154,INVALC,0x6D72,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x555B,INVALC,
	INVALC,0x555C,0x7344,0x4B57,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6236,INVALC,INVALC,INVALC,INVALC,0x6F71,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7B55,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5358,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D50,0x7059,
	0x4B33,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x555D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4D27,INVALC,INVALC,0x502C,INVALC,
	/* Block 126, Array index 0x4000 */
	0x513A,INVALC,0x7144,INVALC,0x6533,0x7B75,0x6961,0x7D60,
	0x7C3C,INVALC,0x5A22,0x5A23,INVALC,0x5221,INVALC,INVALC,
	0x526F,INVALC,INVALC,INVALC,0x626D,INVALC,INVALC,0x5E69,
	0x4E5C,0x7235,0x5064,0x5D51,INVALC,INVALC,INVALC,INVALC,
	0x6148,0x5B37,0x5F63,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6D39,0x7145,INVALC,0x734F,0x572B,
	0x612C,INVALC,INVALC,0x636B,INVALC,0x6E47,INVALC,INVALC,
	INVALC,0x6149,0x4A7A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x707B,0x7A61,0x705A,0x4C67,0x5A74,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C3F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E6D,INVALC,
	INVALC,0x5529,0x7A62,INVALC,INVALC,INVALC,0x5065,INVALC,
	0x6B56,INVALC,0x6C5F,INVALC,INVALC,INVALC,0x5F7C,INVALC,
	INVALC,0x7756,0x5E6A,0x4B34,INVALC,INVALC,0x6F3E,INVALC,
	INVALC,0x4C35,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F3D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6F72,0x6237,
	INVALC,INVALC,INVALC,0x4C68,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x707C,INVALC,INVALC,INVALC,
	0x5660,INVALC,0x7146,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6238,0x6B2B,INVALC,INVALC,
	INVALC,0x4B35,0x5851,INVALC,0x744E,0x7377,INVALC,INVALC,
	0x5746,INVALC,0x513B,0x772A,INVALC,0x6D4A,0x5753,0x587A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7645,
	INVALC,INVALC,0x514C,0x5D7C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F7D,INVALC,
	0x7965,INVALC,0x604A,INVALC,INVALC,0x727D,0x5330,INVALC,
	0x7473,0x5A49,INVALC,0x665E,INVALC,INVALC,INVALC,INVALC,
	0x783A,0x6850,INVALC,INVALC,0x587B,INVALC,INVALC,0x6A55,
	INVALC,INVALC,INVALC,INVALC,0x5623,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7646,INVALC,INVALC,INVALC,INVALC,
	/* Block 127, Array index 0x4100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x725B,0x647C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6832,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5A5A,INVALC,0x725C,0x7B56,0x6932,
	INVALC,0x6E2D,INVALC,0x7A63,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5C6E,INVALC,INVALC,0x756A,0x6660,
	INVALC,0x707D,INVALC,INVALC,INVALC,INVALC,INVALC,0x572C,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7545,0x6E60,INVALC,
	INVALC,0x5B65,INVALC,0x5D5E,INVALC,INVALC,0x5970,0x6923,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7179,INVALC,0x7244,0x604B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6924,INVALC,
	INVALC,0x6239,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6331,0x7C6B,0x4D28,INVALC,0x4C36,INVALC,INVALC,
	0x705B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x663A,INVALC,INVALC,0x4D29,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7343,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6159,INVALC,INVALC,0x6F2B,
	INVALC,INVALC,INVALC,0x6745,INVALC,INVALC,0x6069,INVALC,
	0x7345,INVALC,INVALC,0x5440,0x553F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 128, Array index 0x4200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D2E,INVALC,
	0x797C,INVALC,0x4C40,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6522,INVALC,INVALC,INVALC,
	0x4E38,INVALC,INVALC,INVALC,0x5852,0x7956,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x712A,0x4E51,INVALC,INVALC,0x7647,INVALC,
	0x5B6B,INVALC,0x5F7E,INVALC,INVALC,0x5861,INVALC,0x7773,
	INVALC,0x5767,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x547E,INVALC,INVALC,
	0x513C,INVALC,0x654F,INVALC,0x4B36,INVALC,0x5A38,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D44,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x563E,INVALC,INVALC,INVALC,0x623A,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F58,INVALC,INVALC,INVALC,
	0x604C,0x6B79,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7D7D,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5768,0x4B58,INVALC,INVALC,INVALC,0x6962,INVALC,INVALC,
	INVALC,0x683A,INVALC,INVALC,INVALC,0x6347,INVALC,INVALC,
	INVALC,INVALC,0x6C4D,INVALC,0x6C4E,INVALC,0x563F,INVALC,
	INVALC,INVALC,0x6327,INVALC,0x5F56,0x7D68,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E61,
	0x7628,0x5D7D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x783B,INVALC,0x6851,INVALC,INVALC,INVALC,INVALC,
	0x7957,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E6E,INVALC,INVALC,0x6C4F,INVALC,INVALC,INVALC,
	/* Block 129, Array index 0x4300 */
	0x6925,0x5655,INVALC,0x4D45,INVALC,0x6D3A,0x513D,INVALC,
	INVALC,0x4F3E,INVALC,INVALC,0x6C3B,INVALC,INVALC,INVALC,
	0x5231,INVALC,INVALC,INVALC,INVALC,0x4C69,INVALC,0x5944,
	0x697C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x513E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6C3C,INVALC,INVALC,0x652D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7730,INVALC,0x4C6A,
	INVALC,INVALC,INVALC,0x5344,INVALC,INVALC,0x5640,INVALC,
	INVALC,INVALC,0x567D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6121,INVALC,
	0x5E3D,INVALC,0x7629,INVALC,INVALC,INVALC,0x5A24,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5624,
	0x7546,INVALC,0x6122,0x6946,INVALC,INVALC,INVALC,0x7245,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7469,0x566C,0x6B53,
	INVALC,INVALC,INVALC,INVALC,0x6C3D,0x625C,0x5E6B,0x705C,
	INVALC,0x6B3F,INVALC,0x574E,0x513F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x752B,INVALC,
	INVALC,INVALC,INVALC,0x797D,INVALC,0x4A5C,INVALC,INVALC,
	INVALC,0x4D46,0x7236,INVALC,INVALC,0x5D7E,INVALC,INVALC,
	INVALC,0x4C37,0x5B38,INVALC,INVALC,INVALC,INVALC,0x5069,
	INVALC,0x4E5D,0x6B40,INVALC,0x7D22,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x784B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6A56,0x7130,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5B4E,INVALC,0x7743,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5B4F,0x4B24,INVALC,INVALC,0x7860,INVALC,
	INVALC,0x7B57,INVALC,INVALC,0x6B4A,0x6021,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E4D,INVALC,INVALC,0x545C,INVALC,INVALC,INVALC,
	0x7D58,INVALC,INVALC,INVALC,INVALC,0x5276,INVALC,INVALC,
	/* Block 130, Array index 0x4400 */
	INVALC,INVALC,0x7237,INVALC,INVALC,0x7A76,0x762A,0x7A77,
	0x5866,INVALC,0x7431,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6852,INVALC,0x4A45,0x4C6B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x626E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x623B,INVALC,0x772D,INVALC,INVALC,INVALC,0x7861,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x736C,INVALC,INVALC,INVALC,INVALC,0x5E21,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x647D,INVALC,INVALC,0x636C,INVALC,
	0x5D2F,0x5D30,INVALC,INVALC,0x4B37,0x6853,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6123,0x5260,INVALC,
	INVALC,INVALC,INVALC,0x707E,INVALC,INVALC,INVALC,INVALC,
	0x6926,0x4B72,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6D73,0x5C59,0x604D,INVALC,INVALC,INVALC,INVALC,0x775A,
	0x5B39,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C2E,INVALC,0x5A5B,INVALC,INVALC,INVALC,INVALC,0x4D47,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D31,INVALC,0x582F,0x6323,INVALC,INVALC,
	0x4E6F,INVALC,INVALC,0x7273,INVALC,INVALC,INVALC,INVALC,
	0x7833,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x604E,INVALC,0x757D,INVALC,INVALC,
	INVALC,INVALC,0x6B6C,INVALC,INVALC,0x5345,0x7C6C,0x525B,
	0x546B,INVALC,0x5E22,INVALC,INVALC,INVALC,0x6566,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7030,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5544,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D74,
	INVALC,INVALC,INVALC,0x636D,INVALC,0x6842,INVALC,0x6D75,
	0x577C,INVALC,0x6D3B,INVALC,INVALC,0x762B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7238,0x7648,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5366,0x725D,0x4F3F,INVALC,0x6B2C,INVALC,
	/* Block 131, Array index 0x4500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4F40,INVALC,0x6628,
	0x7D69,INVALC,0x4F41,INVALC,0x605F,0x5E6C,INVALC,INVALC,
	INVALC,INVALC,0x6022,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x743F,0x626F,INVALC,0x5971,0x7147,
	INVALC,0x4B38,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x797E,0x5B3A,0x5A75,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x766C,0x5A5C,0x7A64,
	INVALC,0x604F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5D32,INVALC,INVALC,INVALC,INVALC,0x6629,INVALC,0x6F73,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x736D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6B7A,INVALC,0x7966,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A5D,0x555E,
	INVALC,0x4A5E,0x5F64,INVALC,INVALC,INVALC,0x667D,INVALC,
	0x752C,INVALC,INVALC,INVALC,INVALC,INVALC,0x6475,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6963,INVALC,0x6D4B,0x4F64,INVALC,
	INVALC,INVALC,0x5853,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D33,0x546C,INVALC,INVALC,0x7239,INVALC,0x5F37,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B4E,0x7B58,INVALC,
	INVALC,0x5059,INVALC,INVALC,0x5D52,0x7774,0x675C,0x6425,
	INVALC,0x7C23,INVALC,0x5B3B,INVALC,INVALC,INVALC,0x723A,
	0x697D,0x504A,INVALC,0x7556,0x5945,0x6434,INVALC,0x6D27,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A3D,0x667E,INVALC,0x7744,0x752D,INVALC,0x5960,
	INVALC,INVALC,INVALC,0x4A34,INVALC,INVALC,0x7862,0x4F42,
	INVALC,0x6C3E,INVALC,INVALC,INVALC,0x6534,0x4D48,0x6E48,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6748,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4D49,0x7937,INVALC,
	/* Block 132, Array index 0x4600 */
	INVALC,0x7168,0x5972,0x5B75,0x4A35,0x5946,INVALC,INVALC,
	INVALC,0x5849,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x592B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6D3C,INVALC,INVALC,0x5854,INVALC,INVALC,INVALC,0x5C5A,
	INVALC,0x623C,INVALC,INVALC,0x7C6D,0x6C60,0x527E,INVALC,
	0x6947,0x662A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6270,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7A3B,
	INVALC,0x752E,0x7B2A,INVALC,INVALC,INVALC,INVALC,0x6C7B,
	INVALC,0x6C3F,0x7C58,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5465,INVALC,INVALC,INVALC,0x7943,
	INVALC,INVALC,INVALC,0x6E62,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5769,0x6D76,INVALC,INVALC,INVALC,0x5E6D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C6C,INVALC,
	0x636E,INVALC,INVALC,INVALC,INVALC,INVALC,0x6854,INVALC,
	INVALC,INVALC,0x7A78,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D34,0x6435,0x5830,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5855,INVALC,INVALC,
	INVALC,0x746A,INVALC,INVALC,INVALC,0x4E39,INVALC,INVALC,
	INVALC,0x5661,0x4F52,INVALC,0x5036,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E22,INVALC,INVALC,0x736E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7378,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C4C,INVALC,0x504B,INVALC,INVALC,INVALC,0x7C24,
	0x4D4A,0x5754,0x5E23,INVALC,0x6460,INVALC,INVALC,INVALC,
	INVALC,0x6E49,INVALC,INVALC,INVALC,0x625D,INVALC,INVALC,
	/* Block 133, Array index 0x4700 */
	INVALC,INVALC,INVALC,0x757E,0x542C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5551,INVALC,0x5870,0x7843,0x6A57,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7557,INVALC,INVALC,0x583F,INVALC,INVALC,INVALC,
	INVALC,0x7D40,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6B2D,INVALC,INVALC,INVALC,INVALC,0x552A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6728,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E4A,
	INVALC,INVALC,INVALC,0x4A67,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7863,INVALC,0x545D,INVALC,INVALC,0x6A58,INVALC,
	INVALC,INVALC,INVALC,0x7B59,0x6D77,INVALC,INVALC,0x6535,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x502D,INVALC,INVALC,
	INVALC,INVALC,0x7171,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x623D,INVALC,INVALC,INVALC,0x6348,INVALC,INVALC,INVALC,
	INVALC,0x5955,INVALC,INVALC,0x5F2A,INVALC,INVALC,INVALC,
	INVALC,0x5B3C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7864,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x717A,INVALC,INVALC,0x6536,0x736F,INVALC,INVALC,0x7B5A,
	0x6160,INVALC,0x592C,INVALC,0x756B,INVALC,0x6036,INVALC,
	INVALC,0x6948,INVALC,0x4B4F,INVALC,0x6349,INVALC,INVALC,
	INVALC,0x5E6E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x623E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5C6F,INVALC,0x5625,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6271,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x567E,INVALC,INVALC,INVALC,
	/* Block 134, Array index 0x4800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5921,INVALC,0x5840,0x5C5B,INVALC,INVALC,0x6D3D,
	0x5F38,INVALC,0x6A25,INVALC,INVALC,INVALC,0x572D,INVALC,
	INVALC,0x7379,INVALC,0x6D78,INVALC,0x7547,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x614A,0x6B63,INVALC,0x725E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x784C,INVALC,0x6A59,
	INVALC,0x5346,INVALC,0x5B66,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x752F,INVALC,INVALC,INVALC,INVALC,0x4E70,INVALC,
	INVALC,INVALC,INVALC,0x697E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7B36,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6272,INVALC,INVALC,INVALC,INVALC,
	0x4F72,0x7739,0x5973,INVALC,INVALC,0x614B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A5D,INVALC,INVALC,0x5A39,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B7B,
	INVALC,0x4B39,INVALC,INVALC,0x6D79,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6060,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7440,INVALC,
	0x7D3C,0x5F31,0x636F,INVALC,INVALC,INVALC,INVALC,0x6023,
	0x7D39,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7031,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D4B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6D3E,INVALC,INVALC,INVALC,0x5540,0x6370,0x6D7A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6964,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x556D,0x675D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5476,0x6537,INVALC,INVALC,
	INVALC,0x5B67,0x623F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E4B,
	INVALC,INVALC,0x5774,0x705D,INVALC,INVALC,INVALC,0x4E2B,
	/* Block 135, Array index 0x4900 */
	INVALC,INVALC,0x675E,INVALC,INVALC,INVALC,0x5656,0x614C,
	INVALC,INVALC,0x6833,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x656E,0x5C22,
	INVALC,INVALC,0x6050,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5535,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5521,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7B5B,INVALC,
	0x794B,INVALC,INVALC,INVALC,0x4B73,0x7425,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7A48,0x5657,INVALC,0x6965,0x7B5C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7D50,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7B76,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A25,INVALC,0x5B3D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6C62,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4D77,0x705E,INVALC,INVALC,INVALC,
	INVALC,0x7649,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E6F,
	INVALC,INVALC,INVALC,0x5331,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7C6E,INVALC,INVALC,INVALC,
	INVALC,0x6843,INVALC,0x7148,INVALC,INVALC,INVALC,0x4E71,
	INVALC,INVALC,INVALC,INVALC,0x796D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7274,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6436,INVALC,
	/* Block 136, Array index 0x4A00 */
	0x7539,INVALC,0x5C70,0x6371,INVALC,INVALC,INVALC,INVALC,
	0x6825,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x723B,INVALC,0x5E24,INVALC,0x5A4C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A69,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x635A,INVALC,0x7C59,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A5A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7944,INVALC,
	0x6324,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7B5D,INVALC,0x6F4A,INVALC,
	0x6844,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x554C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6B57,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x592D,
	INVALC,INVALC,0x7B2B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5359,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5522,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x765E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A76,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6051,INVALC,INVALC,0x6928,
	INVALC,INVALC,0x7579,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7A2F,INVALC,0x6B7C,INVALC,INVALC,0x606A,INVALC,
	/* Block 137, Array index 0x4B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6332,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5545,
	INVALC,INVALC,0x7163,0x556E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D4C,INVALC,INVALC,INVALC,INVALC,0x6D59,INVALC,
	INVALC,INVALC,INVALC,0x5841,INVALC,INVALC,INVALC,INVALC,
	0x7A6C,INVALC,INVALC,INVALC,INVALC,INVALC,0x716B,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7A3C,0x6662,INVALC,INVALC,
	INVALC,INVALC,0x7A65,0x627A,INVALC,INVALC,INVALC,0x4A36,
	INVALC,0x6437,INVALC,0x6A5B,INVALC,0x757A,INVALC,INVALC,
	INVALC,0x7B2C,0x4F43,0x6B7D,INVALC,INVALC,INVALC,INVALC,
	0x787A,INVALC,INVALC,0x5F39,INVALC,INVALC,INVALC,INVALC,
	0x6171,INVALC,0x5224,INVALC,INVALC,INVALC,INVALC,0x757B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x505A,0x505B,
	INVALC,0x6A3E,0x5931,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A37,INVALC,INVALC,0x5367,INVALC,0x7865,INVALC,INVALC,
	INVALC,INVALC,0x5332,INVALC,INVALC,INVALC,0x6240,0x725F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D65,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x792C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D4D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6E2E,0x562E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x576A,
	INVALC,INVALC,INVALC,INVALC,0x6760,0x6B2E,INVALC,INVALC,
	INVALC,0x4F59,INVALC,INVALC,0x5C4D,0x6D7B,INVALC,0x5E70,
	INVALC,0x576B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5E25,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5F57,0x5B50,0x5B51,INVALC,INVALC,
	0x5523,INVALC,INVALC,INVALC,INVALC,0x7032,INVALC,INVALC,
	/* Block 138, Array index 0x4C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C5C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A68,INVALC,0x7866,0x5C4E,INVALC,INVALC,INVALC,INVALC,
	0x6A5C,0x5B52,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6933,INVALC,INVALC,
	INVALC,INVALC,0x775B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6328,INVALC,
	0x572E,INVALC,INVALC,0x6061,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4B3A,INVALC,INVALC,0x6551,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x505C,
	INVALC,INVALC,INVALC,INVALC,0x5541,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x584A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6329,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6024,
	INVALC,0x6929,INVALC,0x5347,INVALC,INVALC,0x5C5D,0x782E,
	INVALC,INVALC,INVALC,0x4C38,INVALC,INVALC,INVALC,0x502E,
	INVALC,INVALC,INVALC,0x5872,INVALC,INVALC,0x634A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C2F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x542D,0x7651,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x504C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A46,INVALC,INVALC,0x5542,INVALC,INVALC,
	0x4E3A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A47,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7A30,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F58,INVALC,INVALC,INVALC,
	0x753A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 139, Array index 0x4D00 */
	0x656B,INVALC,0x6F74,0x5D35,INVALC,INVALC,INVALC,INVALC,
	0x4D2A,INVALC,0x6372,INVALC,0x7B77,INVALC,0x7750,INVALC,
	INVALC,INVALC,INVALC,0x7D3A,INVALC,INVALC,0x7D61,0x767E,
	0x5140,INVALC,INVALC,0x6845,INVALC,0x6438,INVALC,0x6168,
	INVALC,INVALC,INVALC,0x4C41,INVALC,0x526D,INVALC,INVALC,
	INVALC,INVALC,0x5B3E,INVALC,INVALC,0x6062,INVALC,INVALC,
	INVALC,0x7A49,INVALC,INVALC,0x614D,INVALC,0x4A38,INVALC,
	INVALC,INVALC,0x7260,0x7149,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5E71,INVALC,INVALC,INVALC,0x705F,0x7844,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6E4C,INVALC,INVALC,0x5E72,INVALC,
	0x6749,INVALC,0x6273,0x6761,INVALC,INVALC,0x634B,INVALC,
	INVALC,0x634C,INVALC,INVALC,INVALC,0x4F78,0x6F2C,INVALC,
	0x7D7E,0x7C25,0x7A31,0x5F59,INVALC,0x6052,INVALC,INVALC,
	INVALC,0x745A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x714A,INVALC,0x4E23,
	INVALC,INVALC,INVALC,INVALC,0x723C,0x6C63,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6025,INVALC,0x772B,INVALC,INVALC,
	0x6B2F,INVALC,INVALC,INVALC,INVALC,INVALC,0x655E,INVALC,
	0x6124,0x4D2B,INVALC,0x5974,0x6826,0x4D4E,0x6169,INVALC,
	0x7C6F,INVALC,0x6063,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6241,INVALC,0x4E24,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5E26,INVALC,INVALC,0x6B7E,INVALC,0x6B5D,0x7060,
	INVALC,INVALC,0x745B,INVALC,0x6274,INVALC,INVALC,0x5348,
	INVALC,INVALC,INVALC,0x746B,INVALC,0x6E35,INVALC,0x7558,
	INVALC,INVALC,0x555F,INVALC,INVALC,INVALC,0x5665,INVALC,
	INVALC,INVALC,INVALC,0x6B30,0x7463,INVALC,INVALC,INVALC,
	INVALC,0x634D,INVALC,INVALC,INVALC,INVALC,0x7474,0x7A32,
	INVALC,INVALC,0x6F75,0x4A5F,INVALC,0x6B31,0x6D3F,INVALC,
	INVALC,0x7D49,INVALC,INVALC,INVALC,INVALC,0x6426,0x7924,
	0x7033,INVALC,0x656C,INVALC,INVALC,INVALC,0x5167,INVALC,
	/* Block 140, Array index 0x4E00 */
	0x5947,0x6457,0x6A5D,INVALC,0x5477,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A3A,INVALC,
	0x5A4D,INVALC,INVALC,INVALC,0x794C,INVALC,0x615A,0x5B3F,
	INVALC,0x4C45,0x6C50,0x4B3B,INVALC,0x5E73,INVALC,INVALC,
	0x692A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5948,INVALC,INVALC,0x6E63,0x573D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4F44,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x504D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7C26,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x717B,INVALC,INVALC,INVALC,INVALC,0x7D52,0x5141,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x635B,INVALC,0x5349,INVALC,0x5C4F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C6D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E27,INVALC,INVALC,0x663B,
	0x6C21,INVALC,INVALC,INVALC,0x4C39,INVALC,INVALC,0x7B5E,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6762,INVALC,INVALC,
	0x5441,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C28,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6242,INVALC,0x7358,0x6553,INVALC,INVALC,0x7359,INVALC,
	INVALC,INVALC,0x7346,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 141, Array index 0x4F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D5B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D2C,
	INVALC,0x7C43,INVALC,INVALC,INVALC,INVALC,0x5467,INVALC,
	0x5142,INVALC,0x7925,INVALC,0x6855,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x634E,INVALC,INVALC,
	INVALC,INVALC,0x544A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F5A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7B5F,0x6763,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x787B,0x634F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7530,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5867,INVALC,0x5949,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x782F,0x6F76,INVALC,
	0x5D36,0x6E2F,0x4D78,INVALC,INVALC,INVALC,INVALC,0x5E38,
	0x7C27,0x777C,0x7731,0x4E3B,0x7421,INVALC,INVALC,0x6E4D,
	0x612E,INVALC,INVALC,0x6C43,0x4F7E,INVALC,0x783F,0x5862,
	0x5368,INVALC,INVALC,0x5E28,0x7464,0x6C42,INVALC,0x5975,
	0x7945,0x5D53,0x5671,0x6C7C,0x7C70,INVALC,INVALC,0x6D40,
	0x4A39,INVALC,0x6E64,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7261,INVALC,0x5E39,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5672,INVALC,0x5E74,INVALC,0x5F5B,INVALC,
	0x5B53,INVALC,0x7A67,0x5863,0x7441,INVALC,0x5D37,INVALC,
	INVALC,INVALC,0x7275,INVALC,INVALC,0x542E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5673,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D38,0x4F45,0x5F5F,INVALC,INVALC,
	/* Block 142, Array index 0x5000 */
	INVALC,INVALC,INVALC,INVALC,0x723E,0x7621,INVALC,0x6B4B,
	0x717C,INVALC,0x7347,INVALC,INVALC,0x606B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6D7C,INVALC,INVALC,0x615B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6E65,INVALC,0x5E75,INVALC,
	INVALC,INVALC,INVALC,0x7A53,INVALC,INVALC,INVALC,INVALC,
	0x714B,INVALC,INVALC,0x502F,0x5D39,INVALC,INVALC,0x5143,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7531,INVALC,INVALC,
	INVALC,INVALC,0x6A46,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7061,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x762C,INVALC,INVALC,INVALC,INVALC,
	0x7559,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x706B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D3A,INVALC,INVALC,INVALC,0x723F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7745,INVALC,
	INVALC,INVALC,INVALC,0x5B22,0x7276,INVALC,INVALC,0x4A3A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7775,INVALC,0x4B65,INVALC,INVALC,
	INVALC,0x6E66,INVALC,0x6053,INVALC,INVALC,INVALC,INVALC,
	0x4E25,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5658,
	INVALC,INVALC,INVALC,0x542F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 143, Array index 0x5100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6949,INVALC,INVALC,INVALC,INVALC,0x534E,
	0x7442,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B66,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7121,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6B32,INVALC,INVALC,INVALC,INVALC,0x7122,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6B33,INVALC,0x7034,INVALC,INVALC,0x4B74,
	0x5430,0x7332,0x7B37,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x756C,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E67,
	0x7432,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x756D,INVALC,0x4F73,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7062,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E4E,
	INVALC,INVALC,0x714C,INVALC,INVALC,0x6538,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5775,0x6373,0x4F65,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4F46,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7333,0x6458,0x4F79,0x4F5A,INVALC,INVALC,
	INVALC,INVALC,0x7A4D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6663,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7262,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x756E,INVALC,INVALC,0x4A3B,INVALC,INVALC,0x635C,INVALC,
	/* Block 144, Array index 0x5200 */
	INVALC,INVALC,INVALC,0x4E72,INVALC,0x5659,INVALC,INVALC,
	INVALC,0x6E30,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7465,0x5842,0x5C50,0x4C6E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5560,0x764A,0x7D4A,0x5856,0x744F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5626,0x5C3E,
	INVALC,0x5B54,0x5747,INVALC,INVALC,INVALC,INVALC,0x727E,
	INVALC,INVALC,INVALC,0x714D,INVALC,INVALC,INVALC,INVALC,
	0x6243,0x5C5E,INVALC,0x5C5F,INVALC,INVALC,0x6F2D,0x662B,
	INVALC,INVALC,INVALC,INVALC,0x795D,0x6A3F,INVALC,INVALC,
	INVALC,0x6F2E,INVALC,INVALC,INVALC,0x7450,0x4E73,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x662C,INVALC,0x4E5E,
	INVALC,INVALC,0x5579,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6374,0x4D50,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5538,INVALC,INVALC,0x777D,INVALC,
	0x5C29,INVALC,INVALC,INVALC,INVALC,0x5E76,INVALC,0x5C2A,
	0x7263,0x6934,0x525C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6966,INVALC,INVALC,0x6376,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x674A,INVALC,
	INVALC,0x504E,INVALC,INVALC,0x5A77,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A3C,INVALC,
	INVALC,INVALC,0x6E68,0x5A5E,INVALC,0x7277,INVALC,INVALC,
	0x627B,INVALC,0x4C26,INVALC,INVALC,INVALC,INVALC,0x5A3B,
	INVALC,0x6E69,INVALC,INVALC,INVALC,0x755A,INVALC,INVALC,
	/* Block 145, Array index 0x5300 */
	0x775C,0x616A,0x4E41,0x5431,INVALC,0x7D31,0x663D,INVALC,
	0x7B2D,INVALC,INVALC,0x7867,INVALC,0x614E,INVALC,0x7762,
	0x756F,0x4F47,INVALC,INVALC,0x5432,0x4C6F,INVALC,0x5468,
	INVALC,0x6E4F,0x7757,INVALC,INVALC,0x6026,0x5641,0x615C,
	0x7063,0x7164,0x5C71,0x5627,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7475,INVALC,
	INVALC,0x714E,0x7264,INVALC,INVALC,0x5030,INVALC,INVALC,
	0x6C6F,INVALC,INVALC,INVALC,0x793A,INVALC,0x6B35,INVALC,
	INVALC,0x546D,0x6244,INVALC,INVALC,INVALC,INVALC,0x6967,
	INVALC,INVALC,0x6B34,0x6A21,INVALC,0x783C,0x4E26,INVALC,
	0x7946,0x7C5A,INVALC,0x5433,0x5339,0x6A5E,INVALC,INVALC,
	INVALC,0x692B,INVALC,INVALC,0x6161,0x534F,0x7476,INVALC,
	0x6A40,0x614F,INVALC,0x4C3A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6E6A,INVALC,INVALC,INVALC,0x7064,0x7334,0x546E,
	INVALC,INVALC,0x7240,INVALC,INVALC,0x7165,INVALC,0x7443,
	0x6054,INVALC,0x6B36,INVALC,0x5721,0x4B68,INVALC,0x792D,
	0x692D,0x5864,0x7A33,0x6245,0x7C3D,INVALC,INVALC,0x6C44,
	0x5831,INVALC,0x5C2B,INVALC,INVALC,INVALC,INVALC,0x5524,
	INVALC,0x6B69,INVALC,INVALC,INVALC,0x683B,INVALC,INVALC,
	INVALC,0x5857,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7B2E,0x5161,INVALC,INVALC,0x5B40,INVALC,
	0x753E,INVALC,0x5E77,INVALC,INVALC,INVALC,INVALC,0x4A7B,
	0x7746,0x4F48,INVALC,INVALC,INVALC,0x6150,INVALC,INVALC,
	0x6E50,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6974,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E74,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x554D,INVALC,
	INVALC,0x4F5B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5D3B,INVALC,INVALC,INVALC,INVALC,0x4E2C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6968,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5434,INVALC,INVALC,
	/* Block 146, Array index 0x5400 */
	INVALC,INVALC,0x6447,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x755B,INVALC,INVALC,0x7A41,INVALC,INVALC,
	INVALC,0x5E29,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5478,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6F77,INVALC,INVALC,
	INVALC,INVALC,0x5333,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B37,0x6F78,0x755C,0x6D4C,0x5B55,0x714F,INVALC,
	INVALC,INVALC,0x7150,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7532,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x592E,0x552C,INVALC,0x6246,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7D23,INVALC,0x7B65,
	0x5F2B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6275,
	INVALC,0x762D,INVALC,0x7533,INVALC,0x7035,INVALC,INVALC,
	INVALC,INVALC,0x6125,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x755D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6C22,0x6D7D,INVALC,0x7534,0x7B38,
	INVALC,0x5B23,INVALC,INVALC,0x564A,0x4B59,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6554,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x737A,
	INVALC,0x6B38,INVALC,0x6037,0x576C,0x716C,0x652F,0x5561,
	0x576D,0x5151,INVALC,INVALC,INVALC,INVALC,INVALC,0x6172,
	0x6F79,INVALC,INVALC,INVALC,0x5D3C,0x765C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7065,INVALC,INVALC,INVALC,0x7444,
	INVALC,INVALC,0x6969,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x737B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 147, Array index 0x5500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x546F,INVALC,INVALC,
	0x4C22,0x777E,0x5F3C,INVALC,INVALC,INVALC,INVALC,0x6B4D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5037,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5642,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x682D,INVALC,INVALC,INVALC,INVALC,0x6F2F,
	0x4B25,INVALC,INVALC,INVALC,INVALC,0x4B69,INVALC,INVALC,
	INVALC,0x7A68,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C46,
	INVALC,INVALC,INVALC,0x6667,INVALC,INVALC,0x6A47,INVALC,
	INVALC,INVALC,0x5B24,INVALC,0x4F49,0x627C,0x6F7A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6B5E,INVALC,INVALC,0x7548,INVALC,0x545E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6055,INVALC,0x6F30,INVALC,INVALC,0x6247,INVALC,
	0x592F,INVALC,INVALC,INVALC,0x7967,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6765,INVALC,INVALC,0x4F4A,0x6151,
	INVALC,0x6248,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6F7B,INVALC,INVALC,0x7A79,
	INVALC,INVALC,0x5C72,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6027,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7868,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4B6A,INVALC,INVALC,INVALC,0x4B3C,INVALC,INVALC,INVALC,
	/* Block 148, Array index 0x5600 */
	INVALC,INVALC,INVALC,INVALC,0x5662,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x755E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x755F,INVALC,0x6E36,INVALC,INVALC,INVALC,0x6276,0x534A,
	0x6F7C,0x5144,0x6F31,INVALC,0x5145,INVALC,0x505E,INVALC,
	0x5961,INVALC,INVALC,0x6038,INVALC,INVALC,0x4D51,0x7339,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x674C,INVALC,0x5628,0x4E27,INVALC,0x5435,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6448,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5334,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B39,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B75,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x765D,INVALC,0x7123,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4C47,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x694A,INVALC,0x6170,INVALC,
	INVALC,INVALC,0x7560,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7B2F,INVALC,INVALC,INVALC,0x4B51,
	INVALC,INVALC,INVALC,INVALC,0x7B60,0x7265,INVALC,INVALC,
	0x6C70,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x706C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6E6B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x694B,INVALC,
	INVALC,0x4C70,INVALC,INVALC,0x572F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7321,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 149, Array index 0x5700 */
	INVALC,INVALC,INVALC,INVALC,0x7C75,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7124,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6056,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6F32,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7451,INVALC,INVALC,
	0x7721,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7151,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A7C,0x4A7D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E4E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7348,INVALC,0x733A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 150, Array index 0x5800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D7E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A26,INVALC,INVALC,0x606C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x784D,INVALC,0x4B52,INVALC,INVALC,INVALC,0x6B4E,
	INVALC,0x7958,0x7959,0x4A60,0x5A4A,INVALC,INVALC,INVALC,
	0x4B26,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A48,0x796E,0x5B6C,INVALC,INVALC,
	0x5031,INVALC,INVALC,INVALC,INVALC,0x556F,INVALC,INVALC,
	INVALC,0x6673,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6722,0x6459,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6461,
	INVALC,INVALC,0x7C44,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x796F,0x4F74,0x7766,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4E3C,INVALC,INVALC,INVALC,
	INVALC,0x7445,0x5C23,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 151, Array index 0x5900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D3D,INVALC,INVALC,INVALC,
	INVALC,0x7446,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7821,INVALC,INVALC,INVALC,0x6856,INVALC,
	INVALC,INVALC,0x5B41,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7066,INVALC,INVALC,INVALC,0x6439,
	0x766D,INVALC,0x792E,INVALC,0x5D3E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5730,0x5868,0x4B3D,INVALC,INVALC,
	0x795A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x784E,0x7970,0x606D,0x6333,0x7433,
	INVALC,INVALC,0x6A42,0x7266,0x7036,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5B56,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6B64,INVALC,INVALC,0x7267,INVALC,0x5755,0x5436,0x7968,
	0x5741,INVALC,INVALC,INVALC,INVALC,0x6555,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x696A,0x574C,INVALC,
	INVALC,INVALC,0x5369,0x6249,INVALC,0x7C5B,0x4D2D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4C30,0x6A22,INVALC,INVALC,
	0x6476,0x5040,INVALC,0x7037,0x6E21,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5776,INVALC,INVALC,INVALC,0x624A,
	0x624B,INVALC,0x7A4F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B5F,INVALC,INVALC,INVALC,INVALC,INVALC,0x564B,
	INVALC,INVALC,INVALC,0x7434,INVALC,INVALC,INVALC,INVALC,
	0x6D4D,0x6452,INVALC,INVALC,0x6A29,0x643A,0x7322,0x4D52,
	INVALC,0x764B,INVALC,0x7166,0x6D41,0x683C,0x6E51,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7067,0x624C,INVALC,
	INVALC,0x642A,INVALC,0x7561,0x6D5A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x576E,0x5171,INVALC,INVALC,INVALC,INVALC,
	0x696B,0x696C,0x6064,INVALC,INVALC,INVALC,INVALC,0x5A27,
	0x5D54,INVALC,0x6A23,INVALC,INVALC,INVALC,0x5643,0x5674,
	INVALC,0x5A5F,INVALC,0x6F33,INVALC,INVALC,INVALC,INVALC,
	/* Block 152, Array index 0x5A00 */
	0x624D,INVALC,INVALC,INVALC,INVALC,INVALC,0x6F7D,0x7268,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F45,INVALC,0x6767,INVALC,INVALC,0x577D,INVALC,
	INVALC,0x674E,INVALC,INVALC,0x5F5C,INVALC,0x7947,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5976,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5F2C,INVALC,0x565A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C24,INVALC,INVALC,INVALC,0x7038,INVALC,INVALC,
	INVALC,INVALC,0x557A,INVALC,0x6477,INVALC,INVALC,INVALC,
	0x5644,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x746C,INVALC,INVALC,INVALC,INVALC,0x6F7E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7021,INVALC,0x5E2A,INVALC,
	INVALC,0x5A3C,0x587C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7A54,INVALC,INVALC,INVALC,0x6C65,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7C28,INVALC,INVALC,0x6C66,
	INVALC,INVALC,0x584B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7B39,INVALC,0x6453,INVALC,0x4D79,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4F53,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A6A,INVALC,INVALC,0x4F54,INVALC,0x783D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7447,INVALC,
	INVALC,INVALC,INVALC,0x6A5F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x795B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5437,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6B65,INVALC,INVALC,0x6152,INVALC,
	INVALC,INVALC,INVALC,0x6A24,INVALC,INVALC,INVALC,0x7A42,
	/* Block 153, Array index 0x5B00 */
	0x7B61,0x7A6D,0x7022,0x4C71,INVALC,0x7A23,0x6277,INVALC,
	0x624E,INVALC,0x6975,INVALC,0x616B,INVALC,INVALC,INVALC,
	0x6768,0x6857,0x5A78,0x544B,INVALC,INVALC,INVALC,0x7776,
	0x5645,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5469,INVALC,INVALC,
	0x7A7A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C72,0x775D,INVALC,0x5E3A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E28,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7039,0x647E,0x6449,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6454,INVALC,INVALC,INVALC,
	0x6A43,INVALC,0x6F34,INVALC,INVALC,INVALC,0x573E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7B62,INVALC,0x4D53,
	INVALC,INVALC,INVALC,0x6F35,INVALC,INVALC,INVALC,0x7A69,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7926,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F3D,
	INVALC,0x7747,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x787D,INVALC,INVALC,0x787C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E2B,0x5B68,INVALC,INVALC,0x635D,
	INVALC,0x6162,0x5146,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7650,0x6B66,0x5A79,
	INVALC,INVALC,INVALC,INVALC,0x6C47,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E78,0x7869,0x635E,INVALC,
	/* Block 154, Array index 0x5C00 */
	INVALC,INVALC,INVALC,0x4E75,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7A43,0x6557,INVALC,0x6C48,INVALC,INVALC,INVALC,
	0x7349,INVALC,INVALC,0x643B,INVALC,INVALC,INVALC,INVALC,
	0x662E,INVALC,INVALC,INVALC,INVALC,INVALC,0x6F36,INVALC,
	0x5C3F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4E3D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5843,INVALC,INVALC,
	INVALC,0x504F,INVALC,0x4F7A,0x734A,0x6057,INVALC,INVALC,
	INVALC,0x5147,0x692E,INVALC,0x683D,INVALC,INVALC,0x7A44,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x624F,INVALC,
	INVALC,0x7A45,INVALC,INVALC,INVALC,0x7938,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5C60,INVALC,INVALC,
	0x7B30,INVALC,INVALC,INVALC,0x5829,0x655F,0x7927,INVALC,
	INVALC,0x766E,INVALC,0x764C,0x6278,INVALC,INVALC,INVALC,
	INVALC,0x6C71,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5A60,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7152,0x524C,0x4F4B,INVALC,INVALC,0x4A3D,INVALC,INVALC,
	INVALC,0x5D3F,INVALC,INVALC,INVALC,0x766F,INVALC,0x5E79,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7A34,INVALC,INVALC,
	INVALC,0x552D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7167,
	/* Block 155, Array index 0x5D00 */
	INVALC,0x5E3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5C40,INVALC,INVALC,INVALC,INVALC,INVALC,0x5148,0x5149,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x783E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4B76,INVALC,INVALC,INVALC,INVALC,
	0x5479,INVALC,INVALC,INVALC,INVALC,INVALC,0x7562,0x6153,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5869,INVALC,INVALC,0x787E,INVALC,0x4F4C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7D24,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E76,INVALC,0x7A50,
	INVALC,INVALC,0x4C73,0x663E,INVALC,INVALC,INVALC,0x762E,
	INVALC,INVALC,0x5570,INVALC,INVALC,0x514A,INVALC,INVALC,
	INVALC,0x7C3E,0x5571,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D69,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7A35,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6250,0x7477,INVALC,INVALC,INVALC,
	0x4D54,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6723,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B25,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 156, Array index 0x5E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6251,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5722,
	INVALC,INVALC,0x7763,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A26,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5021,INVALC,INVALC,INVALC,
	INVALC,0x4E5A,0x7B6B,0x5B26,0x5B5E,0x5865,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A60,
	INVALC,INVALC,INVALC,INVALC,0x582A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6560,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x565B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6F46,INVALC,
	INVALC,0x786A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6455,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E77,INVALC,INVALC,0x6058,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x576F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x746D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D66,INVALC,INVALC,INVALC,
	0x4C74,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 157, Array index 0x5F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7563,INVALC,INVALC,
	0x644A,INVALC,0x5C61,INVALC,INVALC,0x7948,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7C3F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6827,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5844,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B3E,
	INVALC,0x5C2E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5777,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7068,INVALC,0x5D40,
	INVALC,0x4F4D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5C73,0x5930,INVALC,0x6669,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 158, Array index 0x6000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x643C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6A44,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x646C,INVALC,
	0x6465,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7B78,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C3B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x643D,INVALC,INVALC,
	0x4D5C,0x5977,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D5F,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6D4E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5950,INVALC,INVALC,INVALC,INVALC,INVALC,0x6523,
	INVALC,INVALC,INVALC,INVALC,0x794D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D2E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F4E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x762F,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7D53,0x6B6D,0x565C,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 159, Array index 0x6100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6524,INVALC,INVALC,INVALC,0x5536,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x565D,INVALC,INVALC,
	INVALC,0x7969,INVALC,INVALC,INVALC,0x6724,INVALC,0x5663,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x514B,0x5664,INVALC,INVALC,INVALC,0x5572,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E7A,INVALC,0x5778,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x586A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F55,0x587D,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x582B,INVALC,INVALC,0x7D4B,INVALC,
	INVALC,INVALC,INVALC,0x7C5C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6028,0x5573,INVALC,
	INVALC,0x7D59,INVALC,INVALC,0x4C23,INVALC,INVALC,INVALC,
	0x5979,INVALC,INVALC,0x536A,0x7575,INVALC,0x6F47,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x535A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A3D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 160, Array index 0x6200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6828,
	0x5C2F,INVALC,INVALC,INVALC,INVALC,INVALC,0x7023,INVALC,
	INVALC,INVALC,INVALC,0x4D55,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6029,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E2C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x703A,0x6E31,INVALC,INVALC,0x6E32,INVALC,
	INVALC,INVALC,0x764D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E52,
	INVALC,0x5646,INVALC,INVALC,INVALC,INVALC,INVALC,0x6065,
	INVALC,INVALC,0x733B,INVALC,0x6561,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x644B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5723,INVALC,INVALC,
	0x5B42,INVALC,INVALC,INVALC,INVALC,0x4A7E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F4F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 173, Array index 0x6300 */
	0x3021,0x3022,INVALC,INVALC,0x3023,INVALC,INVALC,0x3024,
	0x3025,0x3026,0x3027,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3028,0x3029,0x302A,0x302B,0x302C,0x302D,0x302E,0x302F,
	INVALC,0x3030,0x3031,0x3032,0x3033,0x3034,INVALC,INVALC,
	0x3035,INVALC,INVALC,INVALC,0x3036,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3037,0x3038,INVALC,0x3039,
	0x303A,0x303B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x303C,0x303D,INVALC,INVALC,0x303E,INVALC,INVALC,INVALC,
	0x303F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3040,INVALC,0x3041,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3042,INVALC,INVALC,INVALC,
	0x3043,INVALC,INVALC,INVALC,0x3044,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3045,0x3046,INVALC,INVALC,0x3047,INVALC,INVALC,0x3048,
	0x3049,INVALC,0x304A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x304B,0x304C,INVALC,0x304D,0x304E,0x304F,0x3050,INVALC,
	INVALC,0x3051,0x3052,0x3053,0x3054,INVALC,INVALC,INVALC,
	0x3055,INVALC,INVALC,INVALC,0x3056,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3057,0x3058,INVALC,0x3059,
	0x305A,0x305B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x305C,0x305D,0x305E,INVALC,0x305F,INVALC,INVALC,0x3060,
	0x3061,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3062,0x3063,INVALC,0x3064,0x3065,0x3066,INVALC,INVALC,
	INVALC,0x3067,INVALC,INVALC,0x3068,INVALC,INVALC,INVALC,
	0x3069,INVALC,INVALC,INVALC,0x306A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x306B,INVALC,0x306C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x306D,0x306E,INVALC,INVALC,0x306F,INVALC,INVALC,0x3070,
	0x3071,INVALC,0x3072,INVALC,0x3073,INVALC,INVALC,0x3074,
	0x3075,0x3076,INVALC,0x3077,INVALC,0x3078,0x3079,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x307A,0x307B,INVALC,INVALC,
	/* Block 174, Array index 0x6400 */
	0x307C,INVALC,INVALC,INVALC,0x307D,INVALC,0x307E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3121,0x3122,INVALC,0x3123,
	INVALC,0x3124,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3125,INVALC,INVALC,INVALC,0x3126,INVALC,INVALC,INVALC,
	0x3127,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3128,INVALC,INVALC,0x3129,0x312A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x312B,0x312C,INVALC,INVALC,
	0x312D,INVALC,INVALC,INVALC,0x312E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x312F,0x3130,INVALC,0x3131,
	INVALC,0x3132,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3133,INVALC,INVALC,INVALC,0x3134,INVALC,INVALC,INVALC,
	0x3135,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3136,INVALC,0x3137,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3138,0x3139,INVALC,INVALC,
	0x313A,INVALC,INVALC,0x313B,0x313C,0x313D,0x313E,INVALC,
	INVALC,INVALC,INVALC,0x313F,0x3140,0x3141,INVALC,0x3142,
	INVALC,0x3143,0x3144,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3145,0x3146,INVALC,INVALC,0x3147,INVALC,INVALC,INVALC,
	0x3148,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3149,0x314A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x314B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x314C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x314D,0x314E,INVALC,INVALC,0x314F,INVALC,INVALC,INVALC,
	0x3150,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3151,0x3152,INVALC,0x3153,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3154,INVALC,INVALC,INVALC,
	0x3155,INVALC,INVALC,INVALC,0x3156,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3157,0x3158,INVALC,INVALC,0x3159,INVALC,INVALC,0x315A,
	/* Block 175, Array index 0x6500 */
	0x315B,0x315C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x315D,0x315E,INVALC,0x315F,INVALC,0x3160,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3161,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3162,0x3163,INVALC,INVALC,0x3164,INVALC,INVALC,0x3165,
	0x3166,INVALC,0x3167,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3168,0x3169,INVALC,0x316A,INVALC,0x316B,0x316C,INVALC,
	INVALC,INVALC,0x316D,INVALC,0x316E,0x316F,0x3170,INVALC,
	0x3171,INVALC,INVALC,INVALC,0x3172,INVALC,0x3173,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3174,0x3175,INVALC,0x3176,
	0x3177,0x3178,INVALC,INVALC,INVALC,0x3179,INVALC,INVALC,
	0x317A,0x317B,INVALC,INVALC,0x317C,INVALC,INVALC,INVALC,
	0x317D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x317E,0x3221,INVALC,0x3222,0x3223,0x3224,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3225,0x3226,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3227,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3228,0x3229,0x322A,INVALC,
	0x322B,INVALC,INVALC,INVALC,0x322C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x322D,0x322E,INVALC,0x322F,
	0x3230,0x3231,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3232,0x3233,INVALC,INVALC,0x3234,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3235,INVALC,INVALC,0x3236,INVALC,0x3237,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3238,INVALC,INVALC,INVALC,
	0x3239,INVALC,INVALC,INVALC,0x323A,INVALC,INVALC,INVALC,
	/* Block 176, Array index 0x6600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x323B,
	0x323C,INVALC,INVALC,INVALC,INVALC,0x323D,INVALC,INVALC,
	0x323E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x323F,0x3240,INVALC,INVALC,
	0x3241,INVALC,0x3242,INVALC,0x3243,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3244,0x3245,INVALC,0x3246,
	INVALC,0x3247,0x3248,0x3249,INVALC,INVALC,INVALC,INVALC,
	0x324A,0x324B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x324C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x324D,0x324E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x324F,0x3250,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3251,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3252,INVALC,INVALC,INVALC,0x3253,INVALC,INVALC,INVALC,
	0x3254,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3255,0x3256,INVALC,INVALC,INVALC,0x3257,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3258,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3259,0x325A,INVALC,INVALC,0x325B,INVALC,INVALC,INVALC,
	0x325C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x325D,
	0x325E,0x325F,INVALC,0x3260,INVALC,0x3261,0x3262,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3263,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3264,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3265,0x3266,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3267,0x3268,INVALC,INVALC,0x3269,INVALC,INVALC,INVALC,
	0x326A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 177, Array index 0x6700 */
	0x326B,0x326C,INVALC,INVALC,0x326D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x326E,INVALC,INVALC,INVALC,
	0x326F,INVALC,INVALC,INVALC,0x3270,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3271,0x3272,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3273,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3274,0x3275,INVALC,INVALC,
	0x3276,INVALC,0x3277,INVALC,0x3278,INVALC,0x3279,INVALC,
	INVALC,INVALC,INVALC,0x327A,0x327B,0x327C,INVALC,0x327D,
	INVALC,0x327E,INVALC,INVALC,INVALC,0x3321,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3322,0x3323,INVALC,INVALC,
	0x3324,INVALC,INVALC,INVALC,0x3325,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3326,0x3327,INVALC,0x3328,
	INVALC,0x3329,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x332A,0x332B,0x332C,INVALC,0x332D,INVALC,INVALC,0x332E,
	0x332F,0x3330,0x3331,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3332,0x3333,INVALC,0x3334,0x3335,0x3336,0x3337,0x3338,
	INVALC,0x3339,INVALC,0x333A,0x333B,0x333C,INVALC,INVALC,
	0x333D,INVALC,INVALC,INVALC,0x333E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x333F,0x3340,INVALC,0x3341,
	0x3342,0x3343,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3344,0x3345,INVALC,INVALC,0x3346,INVALC,INVALC,INVALC,
	0x3347,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3348,INVALC,INVALC,INVALC,INVALC,0x3349,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 178, Array index 0x6800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x334A,0x334B,INVALC,0x334C,0x334D,INVALC,INVALC,INVALC,
	0x334E,INVALC,0x334F,0x3350,INVALC,INVALC,INVALC,INVALC,
	0x3351,0x3352,INVALC,0x3353,0x3354,0x3355,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3356,0x3357,0x3358,INVALC,INVALC,
	0x3359,INVALC,INVALC,INVALC,0x335A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x335B,0x335C,INVALC,0x335D,
	0x335E,0x335F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3360,0x3361,INVALC,INVALC,0x3362,INVALC,INVALC,INVALC,
	0x3363,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3364,0x3365,INVALC,INVALC,0x3366,0x3367,INVALC,INVALC,
	0x3368,INVALC,INVALC,INVALC,0x3369,INVALC,INVALC,INVALC,
	0x336A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x336B,0x336C,INVALC,INVALC,0x336D,INVALC,INVALC,INVALC,
	0x336E,INVALC,0x336F,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3370,0x3371,INVALC,0x3372,INVALC,0x3373,INVALC,INVALC,
	INVALC,INVALC,0x3374,0x3375,0x3376,INVALC,INVALC,INVALC,
	0x3377,INVALC,INVALC,INVALC,0x3378,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3379,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x337A,INVALC,INVALC,INVALC,
	0x337B,INVALC,INVALC,INVALC,0x337C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x337D,0x337E,INVALC,0x3421,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3422,0x3423,INVALC,INVALC,0x3424,INVALC,INVALC,INVALC,
	0x3425,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3426,INVALC,0x3427,INVALC,0x3428,INVALC,INVALC,
	/* Block 179, Array index 0x6900 */
	INVALC,INVALC,INVALC,INVALC,0x3429,0x342A,INVALC,INVALC,
	0x342B,INVALC,INVALC,0x342C,0x342D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x342E,0x342F,INVALC,0x3430,
	INVALC,0x3431,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3432,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3433,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3434,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3435,INVALC,INVALC,INVALC,0x3436,INVALC,INVALC,INVALC,
	0x3437,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3438,0x3439,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x343A,0x343B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x343C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x343D,0x343E,INVALC,INVALC,
	INVALC,0x343F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3440,0x3441,INVALC,INVALC,0x3442,INVALC,INVALC,INVALC,
	0x3443,0x3444,0x3445,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3446,0x3447,INVALC,0x3448,INVALC,0x3449,0x344A,INVALC,
	INVALC,INVALC,0x344B,INVALC,0x344C,INVALC,INVALC,INVALC,
	0x344D,INVALC,INVALC,INVALC,0x344E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x344F,0x3450,INVALC,INVALC,0x3451,INVALC,INVALC,INVALC,
	0x3452,INVALC,0x3453,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3454,0x3455,INVALC,0x3456,INVALC,0x3457,INVALC,INVALC,
	INVALC,INVALC,0x3458,INVALC,0x3459,0x345A,0x345B,INVALC,
	0x345C,INVALC,INVALC,0x345D,0x345E,0x345F,0x3460,0x3461,
	INVALC,INVALC,INVALC,0x3462,0x3463,0x3464,INVALC,0x3465,
	0x3466,0x3467,0x3468,0x3469,INVALC,INVALC,INVALC,0x346A,
	/* Block 180, Array index 0x6A00 */
	0x346B,0x346C,INVALC,INVALC,0x346D,INVALC,INVALC,INVALC,
	0x346E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x346F,0x3470,INVALC,0x3471,0x3472,0x3473,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3474,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3475,0x3476,0x3477,INVALC,
	0x3478,INVALC,INVALC,0x3479,0x347A,INVALC,0x347B,0x347C,
	INVALC,INVALC,INVALC,INVALC,0x347D,0x347E,INVALC,0x3521,
	INVALC,0x3522,INVALC,0x3523,INVALC,INVALC,0x3524,INVALC,
	0x3525,0x3526,INVALC,INVALC,0x3527,INVALC,INVALC,INVALC,
	0x3528,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3529,0x352A,INVALC,0x352B,0x352C,0x352D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x352E,INVALC,INVALC,INVALC,
	0x352F,INVALC,INVALC,INVALC,0x3530,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3531,0x3532,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3533,INVALC,INVALC,INVALC,0x3534,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3535,0x3536,INVALC,INVALC,
	0x3537,INVALC,INVALC,0x3538,0x3539,INVALC,0x353A,INVALC,
	0x353B,INVALC,INVALC,INVALC,0x353C,0x353D,INVALC,0x353E,
	INVALC,0x353F,INVALC,0x3540,INVALC,0x3541,INVALC,INVALC,
	0x3542,INVALC,INVALC,INVALC,0x3543,INVALC,INVALC,INVALC,
	0x3544,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3545,INVALC,INVALC,INVALC,
	/* Block 181, Array index 0x6B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3546,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3547,INVALC,INVALC,INVALC,0x3548,INVALC,INVALC,INVALC,
	0x3549,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x354A,0x354B,INVALC,0x354C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x354D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x354E,0x354F,INVALC,INVALC,0x3550,INVALC,INVALC,INVALC,
	0x3551,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3552,0x3553,INVALC,0x3554,INVALC,0x3555,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3556,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3557,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3558,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3559,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x355A,INVALC,INVALC,INVALC,
	0x355B,INVALC,INVALC,INVALC,0x355C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x355D,INVALC,0x355E,
	INVALC,0x355F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3560,INVALC,INVALC,INVALC,0x3561,INVALC,INVALC,INVALC,
	0x3562,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3563,INVALC,INVALC,INVALC,INVALC,0x3564,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3565,0x3566,INVALC,INVALC,
	0x3567,INVALC,INVALC,0x3568,0x3569,INVALC,0x356A,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x356B,0x356C,INVALC,0x356D,
	INVALC,0x356E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x356F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 182, Array index 0x6C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3570,0x3571,INVALC,INVALC,
	0x3572,INVALC,INVALC,0x3573,0x3574,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3575,0x3576,INVALC,0x3577,
	0x3578,0x3579,0x357A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x357B,0x357C,INVALC,INVALC,0x357D,INVALC,INVALC,INVALC,
	0x357E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3621,0x3622,INVALC,0x3623,0x3624,0x3625,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3626,0x3627,0x3628,INVALC,INVALC,
	0x3629,INVALC,INVALC,INVALC,0x362A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x362B,0x362C,INVALC,0x362D,
	0x362E,0x362F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3630,0x3631,INVALC,INVALC,0x3632,INVALC,INVALC,INVALC,
	0x3633,INVALC,0x3634,0x3635,INVALC,INVALC,INVALC,INVALC,
	0x3636,0x3637,INVALC,0x3638,0x3639,0x363A,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x363B,0x363C,0x363D,INVALC,INVALC,
	0x363E,INVALC,INVALC,INVALC,0x363F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3640,0x3641,INVALC,0x3642,
	0x3643,0x3644,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3645,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3646,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 183, Array index 0x6D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3647,0x3648,INVALC,INVALC,0x3649,INVALC,INVALC,INVALC,
	0x364A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x364B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x364C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x364D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x364E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x364F,INVALC,INVALC,INVALC,
	0x3650,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3651,0x3652,INVALC,INVALC,
	0x3653,INVALC,INVALC,INVALC,0x3654,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3655,0x3656,INVALC,INVALC,INVALC,
	INVALC,0x3657,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3658,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3659,INVALC,INVALC,INVALC,0x365A,INVALC,INVALC,INVALC,
	0x365B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 184, Array index 0x6E00 */
	0x365C,0x365D,INVALC,INVALC,INVALC,0x365E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x365F,0x3660,INVALC,INVALC,0x3661,INVALC,INVALC,0x3662,
	0x3663,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3664,0x3665,INVALC,0x3666,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3667,INVALC,INVALC,INVALC,
	0x3668,INVALC,INVALC,INVALC,0x3669,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x366A,0x366B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x366C,INVALC,INVALC,INVALC,0x366D,INVALC,INVALC,INVALC,
	0x366E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x366F,0x3670,INVALC,0x3671,INVALC,0x3672,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3673,0x3674,INVALC,INVALC,
	0x3675,INVALC,INVALC,INVALC,0x3676,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3677,0x3678,INVALC,0x3679,
	0x367A,0x367B,0x367C,INVALC,INVALC,INVALC,0x367D,0x367E,
	0x3721,0x3722,INVALC,INVALC,0x3723,INVALC,INVALC,INVALC,
	0x3724,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3725,0x3726,INVALC,0x3727,0x3728,0x3729,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x372A,0x372B,INVALC,INVALC,
	0x372C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x372D,
	INVALC,0x372E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x372F,0x3730,INVALC,INVALC,
	0x3731,INVALC,INVALC,INVALC,0x3732,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3733,0x3734,INVALC,0x3735,
	/* Block 185, Array index 0x6F00 */
	0x3736,0x3737,INVALC,INVALC,INVALC,INVALC,INVALC,0x3738,
	0x3739,0x373A,INVALC,INVALC,0x373B,INVALC,INVALC,INVALC,
	0x373C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x373D,0x373E,INVALC,0x373F,INVALC,0x3740,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3741,0x3742,INVALC,INVALC,
	0x3743,INVALC,INVALC,INVALC,0x3744,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3745,0x3746,INVALC,0x3747,
	0x3748,0x3749,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x374A,INVALC,INVALC,INVALC,0x374B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x374C,INVALC,0x374D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x374E,0x374F,INVALC,INVALC,
	0x3750,INVALC,INVALC,INVALC,0x3751,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3752,0x3753,INVALC,0x3754,
	INVALC,0x3755,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3756,INVALC,INVALC,INVALC,0x3757,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3758,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3759,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x375A,INVALC,INVALC,INVALC,0x375B,INVALC,INVALC,INVALC,
	0x375C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x375D,0x375E,INVALC,0x375F,INVALC,0x3760,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3761,INVALC,INVALC,INVALC,
	0x3762,INVALC,INVALC,INVALC,0x3763,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3764,INVALC,0x3765,
	INVALC,0x3766,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3767,0x3768,INVALC,INVALC,0x3769,INVALC,INVALC,INVALC,
	0x376A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x376B,0x376C,INVALC,0x376D,INVALC,0x376E,INVALC,INVALC,
	/* Block 186, Array index 0x7000 */
	INVALC,INVALC,INVALC,INVALC,0x376F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3770,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3771,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3772,0x3773,INVALC,INVALC,
	0x3774,INVALC,INVALC,INVALC,0x3775,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3776,INVALC,INVALC,0x3777,
	INVALC,0x3778,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3779,0x377A,INVALC,INVALC,0x377B,INVALC,INVALC,INVALC,
	0x377C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x377D,0x377E,INVALC,0x3821,INVALC,0x3822,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3823,0x3824,INVALC,INVALC,
	0x3825,INVALC,INVALC,INVALC,0x3826,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3827,0x3828,INVALC,0x3829,
	INVALC,0x382A,0x382B,INVALC,INVALC,0x382C,0x382D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x382E,0x382F,INVALC,INVALC,
	0x3830,INVALC,INVALC,INVALC,0x3831,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3832,0x3833,INVALC,0x3834,
	INVALC,0x3835,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3836,0x3837,INVALC,INVALC,0x3838,INVALC,0x3839,0x383A,
	0x383B,0x383C,0x383D,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x383E,0x383F,INVALC,0x3840,INVALC,0x3841,0x3842,INVALC,
	INVALC,0x3843,INVALC,0x3844,0x3845,0x3846,INVALC,INVALC,
	0x3847,INVALC,INVALC,INVALC,0x3848,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3849,0x384A,INVALC,0x384B,
	0x384C,0x384D,0x384E,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 187, Array index 0x7100 */
	0x384F,0x3850,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3851,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3852,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3853,0x3854,INVALC,INVALC,0x3855,INVALC,INVALC,INVALC,
	0x3856,INVALC,0x3857,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3858,0x3859,INVALC,0x385A,INVALC,0x385B,0x385C,INVALC,
	INVALC,INVALC,INVALC,0x385D,0x385E,0x385F,INVALC,INVALC,
	0x3860,INVALC,INVALC,INVALC,0x3861,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3862,0x3863,INVALC,0x3864,
	0x3865,0x3866,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3867,0x3868,INVALC,INVALC,0x3869,INVALC,INVALC,INVALC,
	0x386A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x386B,0x386C,0x386D,INVALC,0x386E,
	INVALC,INVALC,INVALC,INVALC,0x386F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3870,0x3871,INVALC,0x3872,0x3873,INVALC,INVALC,INVALC,
	0x3874,INVALC,0x3875,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3876,0x3877,INVALC,0x3878,INVALC,0x3879,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x387A,INVALC,INVALC,INVALC,
	0x387B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x387C,0x387D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x387E,INVALC,INVALC,INVALC,
	/* Block 188, Array index 0x7200 */
	0x3921,INVALC,INVALC,INVALC,0x3922,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3923,INVALC,0x3924,
	INVALC,0x3925,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3926,INVALC,INVALC,INVALC,0x3927,INVALC,INVALC,INVALC,
	0x3928,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3929,INVALC,0x392A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x392B,0x392C,0x392D,INVALC,
	0x392E,INVALC,INVALC,0x392F,0x3930,0x3931,0x3932,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3933,0x3934,INVALC,0x3935,
	INVALC,0x3936,INVALC,INVALC,INVALC,0x3937,INVALC,0x3938,
	0x3939,INVALC,INVALC,INVALC,0x393A,INVALC,INVALC,INVALC,
	0x393B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x393C,INVALC,0x393D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x393E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x393F,INVALC,INVALC,INVALC,0x3940,INVALC,INVALC,INVALC,
	0x3941,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3942,INVALC,INVALC,INVALC,
	0x3943,INVALC,INVALC,INVALC,0x3944,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3945,INVALC,INVALC,0x3946,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3947,INVALC,INVALC,INVALC,0x3948,INVALC,INVALC,INVALC,
	0x3949,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x394A,INVALC,INVALC,0x394B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x394C,0x394D,INVALC,INVALC,0x394E,INVALC,INVALC,0x394F,
	/* Block 189, Array index 0x7300 */
	0x3950,INVALC,0x3951,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3952,0x3953,INVALC,0x3954,0x3955,0x3956,INVALC,0x3957,
	INVALC,0x3958,INVALC,INVALC,0x3959,0x395A,0x395B,0x395C,
	0x395D,INVALC,INVALC,0x395E,0x395F,0x3960,0x3961,0x3962,
	INVALC,INVALC,INVALC,INVALC,0x3963,0x3964,INVALC,0x3965,
	INVALC,0x3966,INVALC,INVALC,INVALC,0x3967,INVALC,INVALC,
	0x3968,0x3969,INVALC,INVALC,0x396A,INVALC,INVALC,INVALC,
	0x396B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x396C,0x396D,INVALC,0x396E,0x396F,0x3970,INVALC,INVALC,
	INVALC,0x3971,INVALC,INVALC,0x3972,0x3973,INVALC,INVALC,
	0x3974,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3975,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3976,0x3977,INVALC,INVALC,
	0x3978,INVALC,INVALC,0x3979,0x397A,INVALC,0x397B,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x397C,0x397D,INVALC,0x397E,
	INVALC,0x3A21,0x3A22,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A23,0x3A24,INVALC,INVALC,0x3A25,INVALC,INVALC,0x3A26,
	0x3A27,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A28,0x3A29,INVALC,0x3A2A,0x3A2B,0x3A2C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A2D,0x3A2E,INVALC,INVALC,
	0x3A2F,INVALC,INVALC,INVALC,0x3A30,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3A31,INVALC,0x3A32,
	0x3A33,0x3A34,INVALC,INVALC,INVALC,0x3A35,INVALC,INVALC,
	0x3A36,INVALC,INVALC,INVALC,0x3A37,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A38,0x3A39,0x3A3A,INVALC,
	0x3A3B,INVALC,INVALC,INVALC,0x3A3C,INVALC,INVALC,INVALC,
	/* Block 190, Array index 0x7400 */
	INVALC,INVALC,INVALC,INVALC,0x3A3D,0x3A3E,INVALC,0x3A3F,
	INVALC,0x3A40,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A41,INVALC,INVALC,INVALC,0x3A42,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A43,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A44,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A45,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A46,0x3A47,INVALC,INVALC,0x3A48,INVALC,INVALC,INVALC,
	0x3A49,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A4A,0x3A4B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A4C,INVALC,INVALC,INVALC,
	0x3A4D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A4E,0x3A4F,INVALC,INVALC,0x3A50,INVALC,INVALC,0x3A51,
	0x3A52,0x3A53,0x3A54,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A55,0x3A56,INVALC,0x3A57,INVALC,0x3A58,INVALC,INVALC,
	INVALC,0x3A59,0x3A5A,INVALC,0x3A5B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A5C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A5D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A5E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A5F,0x3A60,INVALC,INVALC,
	0x3A61,INVALC,INVALC,INVALC,0x3A62,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3A63,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A64,INVALC,INVALC,INVALC,0x3A65,INVALC,INVALC,INVALC,
	0x3A66,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 191, Array index 0x7500 */
	0x3A67,INVALC,INVALC,0x3A68,INVALC,0x3A69,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A6A,0x3A6B,INVALC,INVALC,
	0x3A6C,INVALC,INVALC,INVALC,0x3A6D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A6E,0x3A6F,INVALC,0x3A70,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A71,0x3A72,INVALC,INVALC,
	0x3A73,INVALC,INVALC,INVALC,0x3A74,INVALC,0x3A75,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A76,0x3A77,INVALC,0x3A78,
	INVALC,0x3A79,0x3A7A,0x3A7B,INVALC,INVALC,INVALC,INVALC,
	0x3A7C,0x3A7D,INVALC,INVALC,0x3A7E,INVALC,INVALC,INVALC,
	0x3B21,INVALC,0x3B22,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B23,0x3B24,INVALC,0x3B25,0x3B26,0x3B27,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3B28,0x3B29,0x3B2A,INVALC,INVALC,
	0x3B2B,INVALC,INVALC,INVALC,0x3B2C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B2D,0x3B2E,INVALC,0x3B2F,
	0x3B30,0x3B31,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B32,0x3B33,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B34,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B35,0x3B36,INVALC,INVALC,0x3B37,INVALC,INVALC,0x3B38,
	0x3B39,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B3A,INVALC,INVALC,0x3B3B,0x3B3C,0x3B3D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B3E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 192, Array index 0x7600 */
	INVALC,0x3B3F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B40,0x3B41,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B42,0x3B43,INVALC,0x3B44,0x3B45,0x3B46,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B47,0x3B48,INVALC,INVALC,0x3B49,INVALC,INVALC,INVALC,
	0x3B4A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B4B,0x3B4C,INVALC,INVALC,INVALC,0x3B4D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B4E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B4F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3B50,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B51,0x3B52,INVALC,INVALC,
	0x3B53,INVALC,INVALC,INVALC,0x3B54,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B55,INVALC,INVALC,0x3B56,
	INVALC,0x3B57,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 193, Array index 0x7700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B58,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3B59,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B5A,INVALC,INVALC,INVALC,0x3B5B,INVALC,INVALC,INVALC,
	0x3B5C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B5D,0x3B5E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B5F,0x3B60,INVALC,INVALC,0x3B61,INVALC,INVALC,INVALC,
	0x3B62,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B63,0x3B64,INVALC,0x3B65,INVALC,0x3B66,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B67,0x3B68,INVALC,0x3B69,
	0x3B6A,INVALC,INVALC,0x3B6B,0x3B6C,0x3B6D,0x3B6E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B6F,0x3B70,INVALC,0x3B71,
	0x3B72,0x3B73,INVALC,INVALC,INVALC,0x3B74,INVALC,INVALC,
	0x3B75,0x3B76,INVALC,INVALC,0x3B77,INVALC,INVALC,INVALC,
	0x3B78,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B79,0x3B7A,INVALC,0x3B7B,0x3B7C,0x3B7D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B7E,0x3C21,INVALC,INVALC,
	0x3C22,INVALC,INVALC,INVALC,0x3C23,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C24,0x3C25,INVALC,0x3C26,
	INVALC,0x3C27,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 194, Array index 0x7800 */
	0x3C28,INVALC,INVALC,INVALC,0x3C29,INVALC,INVALC,INVALC,
	0x3C2A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C2B,INVALC,INVALC,INVALC,INVALC,0x3C2C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C2D,0x3C2E,0x3C2F,0x3C30,
	0x3C31,INVALC,INVALC,0x3C32,0x3C33,INVALC,0x3C34,0x3C35,
	INVALC,INVALC,INVALC,INVALC,0x3C36,0x3C37,INVALC,0x3C38,
	0x3C39,0x3C3A,INVALC,INVALC,INVALC,INVALC,0x3C3B,INVALC,
	0x3C3C,0x3C3D,INVALC,INVALC,0x3C3E,INVALC,INVALC,INVALC,
	0x3C3F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C40,0x3C41,INVALC,0x3C42,0x3C43,0x3C44,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C45,0x3C46,INVALC,INVALC,
	0x3C47,INVALC,INVALC,INVALC,0x3C48,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C49,0x3C4A,INVALC,0x3C4B,
	0x3C4C,0x3C4D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C4E,INVALC,INVALC,INVALC,0x3C4F,INVALC,INVALC,INVALC,
	0x3C50,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C51,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C52,0x3C53,0x3C54,INVALC,
	0x3C55,INVALC,INVALC,INVALC,0x3C56,INVALC,0x3C57,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C58,0x3C59,INVALC,0x3C5A,
	INVALC,0x3C5B,INVALC,INVALC,INVALC,0x3C5C,INVALC,INVALC,
	0x3C5D,0x3C5E,INVALC,INVALC,0x3C5F,INVALC,INVALC,INVALC,
	0x3C60,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C61,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C62,INVALC,INVALC,INVALC,
	0x3C63,INVALC,INVALC,INVALC,0x3C64,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C65,INVALC,INVALC,0x3C66,
	0x3C67,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C68,INVALC,INVALC,INVALC,0x3C69,INVALC,INVALC,INVALC,
	0x3C6A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C6B,0x3C6C,INVALC,0x3C6D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C6E,0x3C6F,INVALC,INVALC,
	/* Block 195, Array index 0x7900 */
	0x3C70,INVALC,INVALC,INVALC,0x3C71,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C72,0x3C73,INVALC,0x3C74,
	INVALC,0x3C75,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C76,0x3C77,INVALC,INVALC,0x3C78,INVALC,INVALC,0x3C79,
	0x3C7A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C7B,0x3C7C,INVALC,0x3C7D,INVALC,0x3C7E,INVALC,0x3D21,
	INVALC,0x3D22,0x3D23,INVALC,0x3D24,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D25,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D26,0x3D27,INVALC,INVALC,0x3D28,INVALC,INVALC,INVALC,
	0x3D29,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D2A,INVALC,INVALC,INVALC,INVALC,0x3D2B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D2C,0x3D2D,INVALC,INVALC,
	0x3D2E,INVALC,INVALC,INVALC,0x3D2F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D30,0x3D31,INVALC,0x3D32,
	INVALC,0x3D33,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D34,0x3D35,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D36,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D37,INVALC,INVALC,0x3D38,INVALC,0x3D39,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D3A,0x3D3B,INVALC,INVALC,
	0x3D3C,INVALC,INVALC,INVALC,0x3D3D,0x3D3E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D3F,0x3D40,INVALC,0x3D41,
	INVALC,0x3D42,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D43,0x3D44,INVALC,INVALC,
	0x3D45,INVALC,INVALC,0x3D46,0x3D47,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3D48,0x3D49,0x3D4A,INVALC,0x3D4B,
	INVALC,0x3D4C,INVALC,INVALC,INVALC,INVALC,0x3D4D,INVALC,
	0x3D4E,0x3D4F,INVALC,0x3D50,0x3D51,INVALC,INVALC,INVALC,
	/* Block 196, Array index 0x7A00 */
	0x3D52,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D53,0x3D54,INVALC,INVALC,0x3D55,0x3D56,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3D57,0x3D58,0x3D59,INVALC,INVALC,
	0x3D5A,INVALC,INVALC,INVALC,0x3D5B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D5C,0x3D5D,INVALC,INVALC,
	0x3D5E,0x3D5F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3D60,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D61,0x3D62,INVALC,INVALC,0x3D63,INVALC,INVALC,INVALC,
	0x3D64,INVALC,0x3D65,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D66,0x3D67,INVALC,INVALC,0x3D68,0x3D69,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D6A,INVALC,INVALC,INVALC,
	0x3D6B,INVALC,INVALC,INVALC,0x3D6C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D6D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D6E,0x3D6F,INVALC,INVALC,0x3D70,INVALC,INVALC,0x3D71,
	0x3D72,INVALC,0x3D73,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D74,0x3D75,INVALC,INVALC,INVALC,0x3D76,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D77,0x3D78,INVALC,INVALC,
	0x3D79,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 197, Array index 0x7B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D7A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D7B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D7C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D7D,INVALC,INVALC,INVALC,
	0x3D7E,INVALC,INVALC,INVALC,0x3E21,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E22,0x3E23,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E24,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E25,0x3E26,INVALC,INVALC,
	0x3E27,INVALC,INVALC,INVALC,0x3E28,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E29,0x3E2A,INVALC,INVALC,
	INVALC,0x3E2B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E2C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E2D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E2E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E2F,INVALC,INVALC,INVALC,0x3E30,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3E31,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E32,0x3E33,INVALC,INVALC,0x3E34,INVALC,INVALC,INVALC,
	0x3E35,INVALC,0x3E36,INVALC,INVALC,INVALC,INVALC,0x3E37,
	/* Block 198, Array index 0x7C00 */
	0x3E38,0x3E39,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E3A,INVALC,INVALC,INVALC,
	0x3E3B,INVALC,INVALC,INVALC,0x3E3C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E3D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E3E,0x3E3F,INVALC,INVALC,0x3E40,INVALC,INVALC,INVALC,
	0x3E41,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E42,0x3E43,INVALC,0x3E44,INVALC,0x3E45,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E46,0x3E47,INVALC,INVALC,
	0x3E48,0x3E49,0x3E4A,INVALC,0x3E4B,0x3E4C,0x3E4D,INVALC,
	INVALC,INVALC,INVALC,0x3E4E,0x3E4F,0x3E50,INVALC,0x3E51,
	0x3E52,0x3E53,INVALC,INVALC,INVALC,0x3E54,0x3E55,INVALC,
	0x3E56,0x3E57,INVALC,INVALC,0x3E58,INVALC,INVALC,INVALC,
	0x3E59,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E5A,0x3E5B,INVALC,0x3E5C,0x3E5D,0x3E5E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E5F,0x3E60,INVALC,INVALC,
	0x3E61,INVALC,INVALC,INVALC,0x3E62,INVALC,INVALC,0x3E63,
	INVALC,INVALC,INVALC,INVALC,0x3E64,0x3E65,INVALC,0x3E66,
	INVALC,0x3E67,INVALC,INVALC,INVALC,0x3E68,INVALC,0x3E69,
	0x3E6A,INVALC,INVALC,INVALC,0x3E6B,INVALC,INVALC,INVALC,
	0x3E6C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3E6D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E6E,0x3E6F,INVALC,INVALC,
	0x3E70,0x3E71,INVALC,0x3E72,0x3E73,0x3E74,0x3E75,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E76,0x3E77,0x3E78,0x3E79,
	0x3E7A,0x3E7B,0x3E7C,INVALC,0x3E7D,INVALC,0x3E7E,INVALC,
	0x3F21,0x3F22,INVALC,INVALC,0x3F23,INVALC,INVALC,INVALC,
	0x3F24,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F25,0x3F26,INVALC,0x3F27,INVALC,0x3F28,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F29,0x3F2A,0x3F2B,INVALC,
	0x3F2C,INVALC,INVALC,INVALC,0x3F2D,INVALC,0x3F2E,0x3F2F,
	INVALC,INVALC,INVALC,INVALC,0x3F30,0x3F31,0x3F32,0x3F33,
	/* Block 199, Array index 0x7D00 */
	0x3F34,0x3F35,INVALC,INVALC,INVALC,0x3F36,0x3F37,0x3F38,
	0x3F39,INVALC,INVALC,INVALC,0x3F3A,INVALC,INVALC,INVALC,
	0x3F3B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F3C,0x3F3D,INVALC,0x3F3E,0x3F3F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F40,0x3F41,INVALC,INVALC,
	0x3F42,INVALC,INVALC,INVALC,0x3F43,0x3F44,0x3F45,INVALC,
	0x3F46,INVALC,INVALC,0x3F47,0x3F48,0x3F49,INVALC,0x3F4A,
	INVALC,0x3F4B,INVALC,0x3F4C,INVALC,INVALC,INVALC,INVALC,
	0x3F4D,0x3F4E,INVALC,INVALC,0x3F4F,INVALC,INVALC,INVALC,
	0x3F50,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F51,0x3F52,INVALC,0x3F53,0x3F54,0x3F55,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F56,0x3F57,INVALC,INVALC,
	0x3F58,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F59,INVALC,INVALC,0x3F5A,
	INVALC,0x3F5B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F5C,0x3F5D,INVALC,INVALC,0x3F5E,INVALC,INVALC,INVALC,
	0x3F5F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F60,0x3F61,INVALC,0x3F62,INVALC,0x3F63,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F64,0x3F65,INVALC,INVALC,
	0x3F66,INVALC,INVALC,INVALC,0x3F67,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F68,0x3F69,INVALC,0x3F6A,
	INVALC,0x3F6B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F6C,0x3F6D,INVALC,INVALC,0x3F6E,INVALC,INVALC,INVALC,
	0x3F6F,0x3F70,0x3F71,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F72,0x3F73,INVALC,0x3F74,INVALC,0x3F75,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F76,0x3F77,INVALC,INVALC,
	0x3F78,INVALC,INVALC,INVALC,0x3F79,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F7A,0x3F7B,INVALC,INVALC,
	0x3F7C,0x3F7D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F7E,0x4021,INVALC,INVALC,0x4022,INVALC,INVALC,INVALC,
	0x4023,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4024,0x4025,INVALC,INVALC,INVALC,0x4026,INVALC,INVALC,
	/* Block 200, Array index 0x7E00 */
	INVALC,INVALC,INVALC,INVALC,0x4027,0x4028,INVALC,INVALC,
	0x4029,INVALC,INVALC,INVALC,0x402A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x402B,0x402C,INVALC,0x402D,
	INVALC,0x402E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x402F,0x4030,INVALC,INVALC,0x4031,INVALC,INVALC,INVALC,
	0x4032,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4033,0x4034,INVALC,0x4035,INVALC,0x4036,INVALC,0x4037,
	INVALC,INVALC,INVALC,INVALC,0x4038,0x4039,INVALC,INVALC,
	0x403A,INVALC,INVALC,INVALC,0x403B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x403C,INVALC,0x403D,0x403E,INVALC,0x403F,
	INVALC,0x4040,0x4041,0x4042,0x4043,0x4044,0x4045,0x4046,
	0x4047,INVALC,INVALC,INVALC,0x4048,INVALC,INVALC,INVALC,
	0x4049,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x404A,INVALC,INVALC,0x404B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x404C,0x404D,INVALC,INVALC,
	0x404E,INVALC,INVALC,INVALC,0x404F,0x4050,0x4051,INVALC,
	INVALC,INVALC,INVALC,0x4052,0x4053,0x4054,INVALC,0x4055,
	0x4056,0x4057,0x4058,INVALC,INVALC,INVALC,0x4059,INVALC,
	0x405A,0x405B,INVALC,INVALC,0x405C,INVALC,0x405D,0x405E,
	0x405F,INVALC,0x4060,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4061,0x4062,INVALC,0x4063,0x4064,0x4065,0x4066,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4067,0x4068,INVALC,INVALC,
	0x4069,INVALC,INVALC,INVALC,0x406A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x406B,0x406C,INVALC,0x406D,
	0x406E,0x406F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4070,0x4071,INVALC,INVALC,0x4072,INVALC,0x4073,INVALC,
	0x4074,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4075,INVALC,INVALC,INVALC,INVALC,0x4076,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4077,INVALC,INVALC,INVALC,
	0x4078,INVALC,INVALC,INVALC,0x4079,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 201, Array index 0x7F00 */
	0x407A,0x407B,INVALC,INVALC,0x407C,INVALC,INVALC,INVALC,
	0x407D,INVALC,0x407E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4121,0x4122,INVALC,0x4123,INVALC,0x4124,0x4125,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4126,0x4127,INVALC,INVALC,
	0x4128,INVALC,INVALC,INVALC,0x4129,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x412A,0x412B,INVALC,0x412C,
	INVALC,0x412D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x412E,INVALC,INVALC,INVALC,0x412F,INVALC,INVALC,INVALC,
	0x4130,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4131,0x4132,INVALC,INVALC,0x4133,0x4134,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4135,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4136,0x4137,INVALC,INVALC,0x4138,INVALC,INVALC,INVALC,
	0x4139,INVALC,0x413A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x413B,0x413C,INVALC,0x413D,INVALC,0x413E,0x413F,0x4140,
	INVALC,INVALC,INVALC,0x4141,0x4142,0x4143,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4144,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4145,INVALC,0x4146,
	INVALC,0x4147,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4148,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4149,0x414A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x414B,INVALC,INVALC,INVALC,
	0x414C,INVALC,INVALC,INVALC,0x414D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x414E,0x414F,INVALC,0x4150,
	INVALC,0x4151,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4152,0x4153,INVALC,INVALC,0x4154,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4155,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4156,0x4157,INVALC,INVALC,
	/* Block 202, Array index 0x8000 */
	0x4158,INVALC,INVALC,INVALC,0x4159,0x415A,0x415B,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x415C,0x415D,INVALC,0x415E,
	INVALC,0x415F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4160,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4161,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4162,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4163,0x4164,INVALC,INVALC,0x4165,INVALC,INVALC,INVALC,
	0x4166,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4167,0x4168,INVALC,0x4169,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x416A,INVALC,INVALC,INVALC,
	0x416B,INVALC,INVALC,INVALC,0x416C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x416D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x416E,0x416F,INVALC,INVALC,0x4170,INVALC,INVALC,INVALC,
	0x4171,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4172,0x4173,INVALC,0x4174,INVALC,0x4175,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4176,0x4177,INVALC,INVALC,0x4178,INVALC,INVALC,0x4179,
	0x417A,INVALC,0x417B,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x417C,0x417D,INVALC,0x417E,INVALC,0x4221,0x4222,INVALC,
	INVALC,0x4223,0x4224,INVALC,0x4225,0x4226,INVALC,INVALC,
	0x4227,INVALC,0x4228,INVALC,0x4229,INVALC,INVALC,0x422A,
	INVALC,INVALC,INVALC,INVALC,0x422B,0x422C,INVALC,0x422D,
	0x422E,0x422F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4230,0x4231,INVALC,INVALC,0x4232,INVALC,INVALC,INVALC,
	/* Block 203, Array index 0x8100 */
	0x4233,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4234,0x4235,INVALC,0x4236,0x4237,0x4238,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4239,INVALC,INVALC,INVALC,
	0x423A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x423B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x423C,0x423D,INVALC,INVALC,
	0x423E,INVALC,INVALC,INVALC,0x423F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4240,0x4241,INVALC,0x4242,
	0x4243,0x4244,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4245,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4246,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4247,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4248,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4249,0x424A,INVALC,INVALC,
	0x424B,INVALC,INVALC,INVALC,0x424C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x424D,0x424E,INVALC,0x424F,
	INVALC,0x4250,INVALC,0x4251,INVALC,INVALC,INVALC,INVALC,
	0x4252,0x4253,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4254,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4255,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4256,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 204, Array index 0x8200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4257,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4258,INVALC,INVALC,INVALC,0x4259,INVALC,INVALC,INVALC,
	0x425A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x425B,0x425C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x425D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x425E,0x425F,INVALC,INVALC,0x4260,INVALC,INVALC,INVALC,
	0x4261,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4262,0x4263,INVALC,INVALC,INVALC,0x4264,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4265,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4266,0x4267,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4268,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4269,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x426A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x426B,INVALC,INVALC,0x426C,
	INVALC,0x426D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 205, Array index 0x8300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x426E,0x426F,INVALC,INVALC,
	0x4270,INVALC,INVALC,INVALC,0x4271,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4272,0x4273,INVALC,INVALC,
	INVALC,0x4274,0x4275,INVALC,INVALC,INVALC,INVALC,0x4276,
	0x4277,0x4278,INVALC,INVALC,0x4279,INVALC,0x427A,INVALC,
	0x427B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x427C,0x427D,INVALC,0x427E,0x4321,0x4322,0x4323,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4324,0x4325,INVALC,INVALC,
	0x4326,INVALC,INVALC,INVALC,0x4327,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4328,0x4329,INVALC,0x432A,
	0x432B,0x432C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x432D,INVALC,INVALC,INVALC,0x432E,INVALC,0x432F,INVALC,
	0x4330,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4331,INVALC,INVALC,INVALC,INVALC,0x4332,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4333,0x4334,INVALC,INVALC,0x4335,INVALC,INVALC,INVALC,
	0x4336,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4337,0x4338,INVALC,0x4339,0x433A,0x433B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x433C,0x433D,INVALC,INVALC,
	0x433E,INVALC,INVALC,INVALC,0x433F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4340,0x4341,INVALC,0x4342,
	INVALC,0x4343,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4344,INVALC,INVALC,INVALC,0x4345,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4346,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4347,INVALC,INVALC,INVALC,
	0x4348,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 206, Array index 0x8400 */
	INVALC,0x4349,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x434A,0x434B,INVALC,INVALC,0x434C,INVALC,INVALC,INVALC,
	0x434D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x434E,0x434F,INVALC,0x4350,INVALC,0x4351,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4352,INVALC,INVALC,INVALC,
	0x4353,INVALC,INVALC,INVALC,0x4354,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4355,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4356,INVALC,INVALC,INVALC,
	0x4357,INVALC,INVALC,INVALC,0x4358,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4359,0x435A,INVALC,0x435B,
	INVALC,0x435C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x435D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x435E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x435F,0x4360,INVALC,INVALC,
	0x4361,INVALC,INVALC,INVALC,0x4362,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4363,0x4364,INVALC,0x4365,
	INVALC,0x4366,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4367,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4368,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4369,INVALC,INVALC,INVALC,
	0x436A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x436B,INVALC,INVALC,INVALC,0x436C,INVALC,INVALC,INVALC,
	0x436D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x436E,0x436F,INVALC,0x4370,INVALC,0x4371,INVALC,INVALC,
	/* Block 207, Array index 0x8500 */
	INVALC,INVALC,INVALC,INVALC,0x4372,INVALC,INVALC,INVALC,
	0x4373,INVALC,INVALC,INVALC,0x4374,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4375,INVALC,INVALC,INVALC,
	INVALC,0x4376,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4377,0x4378,INVALC,INVALC,0x4379,INVALC,INVALC,INVALC,
	0x437A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x437B,0x437C,INVALC,0x437D,INVALC,0x437E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4421,0x4422,INVALC,INVALC,0x4423,INVALC,INVALC,0x4424,
	0x4425,0x4426,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4427,0x4428,INVALC,0x4429,INVALC,0x442A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x442B,0x442C,INVALC,INVALC,
	0x442D,INVALC,INVALC,INVALC,0x442E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x442F,0x4430,INVALC,0x4431,
	INVALC,0x4432,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4433,0x4434,INVALC,INVALC,0x4435,INVALC,INVALC,INVALC,
	0x4436,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4437,0x4438,INVALC,0x4439,0x443A,0x443B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x443C,0x443D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x443E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x443F,0x4440,INVALC,INVALC,
	0x4441,INVALC,INVALC,0x4442,0x4443,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4444,0x4445,INVALC,0x4446,
	0x4447,0x4448,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 208, Array index 0x8600 */
	0x4449,0x444A,INVALC,INVALC,0x444B,INVALC,INVALC,INVALC,
	0x444C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x444D,0x444E,INVALC,0x444F,INVALC,0x4450,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4451,INVALC,INVALC,INVALC,
	0x4452,INVALC,INVALC,INVALC,0x4453,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4454,0x4455,INVALC,0x4456,
	0x4457,0x4458,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4459,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x445A,0x445B,INVALC,INVALC,
	0x445C,INVALC,INVALC,INVALC,0x445D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x445E,0x445F,INVALC,0x4460,
	INVALC,0x4461,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4462,0x4463,INVALC,INVALC,0x4464,INVALC,INVALC,INVALC,
	0x4465,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4466,INVALC,INVALC,INVALC,INVALC,0x4467,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4468,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4469,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x446A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x446B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x446C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x446D,0x446E,INVALC,INVALC,0x446F,INVALC,INVALC,INVALC,
	0x4470,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4471,0x4472,INVALC,0x4473,INVALC,0x4474,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4475,INVALC,INVALC,INVALC,
	/* Block 209, Array index 0x8700 */
	0x4476,INVALC,INVALC,INVALC,0x4477,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4478,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4479,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x447A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x447B,0x447C,INVALC,INVALC,
	0x447D,INVALC,INVALC,INVALC,0x447E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4521,0x4522,INVALC,0x4523,
	INVALC,0x4524,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4525,INVALC,INVALC,INVALC,0x4526,INVALC,INVALC,INVALC,
	0x4527,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4528,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4529,0x452A,INVALC,INVALC,
	0x452B,INVALC,INVALC,INVALC,0x452C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x452D,0x452E,INVALC,INVALC,
	INVALC,0x452F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4530,0x4531,INVALC,INVALC,
	0x4532,INVALC,INVALC,INVALC,0x4533,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4534,0x4535,INVALC,0x4536,
	INVALC,0x4537,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4538,0x4539,INVALC,INVALC,0x453A,INVALC,INVALC,INVALC,
	0x453B,0x453C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x453D,0x453E,INVALC,0x453F,0x4540,0x4541,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4542,0x4543,INVALC,INVALC,
	0x4544,INVALC,INVALC,INVALC,0x4545,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4546,0x4547,INVALC,0x4548,
	0x4549,0x454A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x454B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 210, Array index 0x8800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x454C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x454D,0x454E,INVALC,INVALC,0x454F,INVALC,INVALC,INVALC,
	0x4550,INVALC,0x4551,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4552,0x4553,INVALC,0x4554,0x4555,0x4556,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4557,0x4558,INVALC,INVALC,
	0x4559,INVALC,INVALC,INVALC,0x455A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x455B,0x455C,INVALC,0x455D,
	INVALC,0x455E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x455F,INVALC,INVALC,INVALC,0x4560,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4561,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4562,INVALC,INVALC,INVALC,
	0x4563,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4564,0x4565,INVALC,INVALC,0x4566,INVALC,INVALC,INVALC,
	0x4567,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4568,0x4569,INVALC,0x456A,INVALC,0x456B,INVALC,INVALC,
	INVALC,INVALC,0x456C,INVALC,0x456D,INVALC,INVALC,INVALC,
	0x456E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x456F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4570,INVALC,INVALC,INVALC,
	0x4571,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 211, Array index 0x8900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4572,
	INVALC,0x4573,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4574,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4575,0x4576,INVALC,INVALC,
	0x4577,INVALC,INVALC,INVALC,0x4578,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4579,0x457A,INVALC,0x457B,
	INVALC,0x457C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x457D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x457E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4621,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4622,0x4623,INVALC,INVALC,0x4624,INVALC,INVALC,INVALC,
	0x4625,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4626,0x4627,INVALC,INVALC,INVALC,0x4628,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4629,INVALC,INVALC,INVALC,
	0x462A,INVALC,INVALC,INVALC,0x462B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x462C,INVALC,INVALC,INVALC,
	INVALC,0x462D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x462E,0x462F,INVALC,INVALC,0x4630,INVALC,INVALC,0x4631,
	0x4632,INVALC,0x4633,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4634,0x4635,INVALC,0x4636,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4637,INVALC,INVALC,INVALC,
	0x4638,INVALC,INVALC,INVALC,0x4639,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x463A,0x463B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x463C,0x463D,INVALC,INVALC,0x463E,INVALC,INVALC,INVALC,
	0x463F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 212, Array index 0x8A00 */
	0x4640,0x4641,INVALC,0x4642,INVALC,0x4643,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4644,0x4645,0x4646,INVALC,
	0x4647,INVALC,INVALC,INVALC,0x4648,INVALC,0x4649,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x464A,0x464B,INVALC,0x464C,
	0x464D,0x464E,INVALC,INVALC,INVALC,0x464F,INVALC,INVALC,
	0x4650,0x4651,INVALC,INVALC,0x4652,INVALC,INVALC,INVALC,
	0x4653,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4654,0x4655,INVALC,0x4656,0x4657,0x4658,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4659,0x465A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x465B,0x465C,INVALC,INVALC,
	0x465D,INVALC,INVALC,INVALC,0x465E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x465F,0x4660,INVALC,0x4661,
	0x4662,0x4663,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4664,0x4665,INVALC,INVALC,0x4666,INVALC,INVALC,INVALC,
	0x4667,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4668,0x4669,INVALC,0x466A,INVALC,0x466B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x466C,INVALC,INVALC,INVALC,
	0x466D,INVALC,INVALC,INVALC,0x466E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x466F,0x4670,INVALC,INVALC,
	0x4671,0x4672,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4673,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4674,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4675,INVALC,0x4676,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4677,0x4678,INVALC,INVALC,
	0x4679,INVALC,INVALC,INVALC,0x467A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x467B,0x467C,INVALC,0x467D,
	/* Block 213, Array index 0x8B00 */
	INVALC,0x467E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4721,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4722,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4723,INVALC,INVALC,INVALC,0x4724,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4725,INVALC,INVALC,INVALC,
	0x4726,INVALC,INVALC,INVALC,0x4727,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4728,INVALC,0x4729,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x472A,0x472B,INVALC,INVALC,0x472C,INVALC,INVALC,0x472D,
	0x472E,INVALC,0x472F,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4730,0x4731,INVALC,0x4732,INVALC,0x4733,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4734,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4735,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4736,INVALC,INVALC,INVALC,
	0x4737,INVALC,INVALC,INVALC,0x4738,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4739,INVALC,INVALC,0x473A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x473B,INVALC,INVALC,INVALC,0x473C,INVALC,INVALC,INVALC,
	0x473D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x473E,INVALC,INVALC,0x473F,INVALC,0x4740,INVALC,INVALC,
	/* Block 214, Array index 0x8C00 */
	INVALC,INVALC,INVALC,INVALC,0x4741,INVALC,INVALC,INVALC,
	0x4742,INVALC,INVALC,INVALC,0x4743,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4744,0x4745,INVALC,0x4746,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4747,0x4748,INVALC,INVALC,
	0x4749,INVALC,INVALC,INVALC,0x474A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x474B,0x474C,INVALC,0x474D,
	INVALC,0x474E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x474F,0x4750,INVALC,INVALC,0x4751,INVALC,INVALC,INVALC,
	0x4752,INVALC,INVALC,INVALC,INVALC,0x4753,INVALC,INVALC,
	0x4754,0x4755,INVALC,0x4756,INVALC,0x4757,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4758,0x4759,INVALC,INVALC,
	0x475A,INVALC,INVALC,INVALC,0x475B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x475C,0x475D,INVALC,0x475E,
	0x475F,0x4760,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4761,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4762,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4763,0x4764,INVALC,INVALC,0x4765,INVALC,INVALC,INVALC,
	0x4766,INVALC,0x4767,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4768,0x4769,INVALC,0x476A,INVALC,0x476B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x476C,0x476D,INVALC,INVALC,
	0x476E,INVALC,INVALC,INVALC,0x476F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4770,0x4771,INVALC,0x4772,
	INVALC,0x4773,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 215, Array index 0x8D00 */
	0x4774,0x4775,INVALC,INVALC,0x4776,INVALC,INVALC,INVALC,
	0x4777,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4778,0x4779,INVALC,0x477A,0x477B,0x477C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x477D,INVALC,INVALC,INVALC,
	0x477E,INVALC,INVALC,INVALC,0x4821,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4822,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4823,0x4824,INVALC,INVALC,0x4825,INVALC,INVALC,INVALC,
	0x4826,INVALC,INVALC,INVALC,INVALC,0x4827,INVALC,INVALC,
	0x4828,0x4829,INVALC,0x482A,INVALC,0x482B,INVALC,INVALC,
	INVALC,0x482C,INVALC,INVALC,0x482D,0x482E,INVALC,INVALC,
	0x482F,INVALC,INVALC,INVALC,0x4830,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4831,
	INVALC,0x4832,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4833,0x4834,INVALC,INVALC,0x4835,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4836,INVALC,0x4837,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4838,0x4839,INVALC,INVALC,
	0x483A,INVALC,INVALC,INVALC,0x483B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x483C,INVALC,0x483D,
	INVALC,0x483E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x483F,INVALC,INVALC,INVALC,0x4840,INVALC,INVALC,INVALC,
	0x4841,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4842,INVALC,0x4843,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4844,0x4845,INVALC,INVALC,
	0x4846,INVALC,INVALC,INVALC,0x4847,INVALC,INVALC,INVALC,
	INVALC,0x4848,INVALC,INVALC,0x4849,INVALC,INVALC,0x484A,
	INVALC,0x484B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x484C,INVALC,INVALC,INVALC,0x484D,INVALC,INVALC,INVALC,
	0x484E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x484F,INVALC,INVALC,INVALC,INVALC,0x4850,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4851,0x4852,INVALC,INVALC,
	/* Block 216, Array index 0x8E00 */
	0x4853,INVALC,INVALC,INVALC,0x4854,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4855,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4856,0x4857,INVALC,INVALC,0x4858,INVALC,INVALC,INVALC,
	0x4859,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x485A,0x485B,INVALC,0x485C,INVALC,0x485D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x485E,0x485F,INVALC,INVALC,
	0x4860,INVALC,INVALC,INVALC,0x4861,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4862,INVALC,INVALC,0x4863,
	INVALC,0x4864,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4865,0x4866,INVALC,INVALC,0x4867,INVALC,0x4868,0x4869,
	0x486A,0x486B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x486C,0x486D,INVALC,0x486E,INVALC,0x486F,INVALC,INVALC,
	INVALC,0x4870,INVALC,INVALC,0x4871,INVALC,INVALC,INVALC,
	0x4872,INVALC,INVALC,INVALC,0x4873,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4874,0x4875,INVALC,INVALC,
	INVALC,0x4876,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4877,0x4878,INVALC,INVALC,0x4879,INVALC,INVALC,INVALC,
	0x487A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x487B,0x487C,INVALC,0x487D,INVALC,0x487E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 250, Array index 0x8F00 */
	0x4B50,0x4B56,0x4B67,0x4D4F,0x4D68,0x4E2D,0x4F7B,0x5022,
	0x5038,0x5050,0x505D,0x5154,0x5155,0x5158,0x515B,0x515C,
	0x515D,0x515E,0x515F,0x5160,0x5162,0x5163,0x5164,0x5165,
	0x5166,0x5168,0x5169,0x516A,0x516B,0x516D,0x516F,0x5170,
	0x5172,0x5176,0x517A,0x517C,0x517D,0x517E,0x5222,0x5223,
	0x5227,0x5228,0x5229,0x522A,0x522B,0x522D,0x5232,0x523E,
	0x5242,0x5243,0x5244,0x5246,0x5247,0x5248,0x5249,0x524A,
	0x524B,0x524D,0x524E,0x524F,0x5250,0x5251,0x5252,0x5253,
	0x5254,0x5255,0x5256,0x5257,0x5259,0x525A,0x525E,0x525F,
	0x5261,0x5262,0x5264,0x5265,0x5266,0x5267,0x5268,0x5269,
	0x526A,0x526B,0x5270,0x5271,0x5272,0x5273,0x5274,0x5275,
	0x5277,0x5278,0x5466,0x547C,0x5525,0x552B,0x552E,0x5638,
	0x564D,0x574B,0x5764,0x5B45,0x5B64,0x5C25,0x5D25,0x5D55,
	0x5D74,0x5E7C,0x5E7E,0x5F33,0x5F61,0x5F68,0x6071,0x612D,
	0x616D,0x6375,0x6421,0x6429,0x652E,0x6531,0x6532,0x6539,
	0x653B,0x653C,0x6544,0x654E,0x6550,0x6552,0x6556,0x657A,
	0x657B,0x657C,0x657E,0x6621,0x6624,0x6627,0x662D,0x662F,
	0x6630,0x6631,0x6633,0x6637,0x6638,0x663C,0x6644,0x6646,
	0x6647,0x664A,0x6652,0x6656,0x6659,0x665C,0x665F,0x6661,
	0x6664,0x6665,0x6666,0x6668,0x666A,0x666B,0x666C,0x666F,
	0x6671,0x6672,0x6675,0x6676,0x6677,0x6679,0x6721,0x6726,
	0x6729,0x672A,0x672C,0x672D,0x6730,0x673F,0x6741,0x6746,
	0x6747,0x674B,0x674D,0x674F,0x6750,0x6753,0x675F,0x6764,
	0x6766,0x6777,0x6867,0x6868,0x6870,0x6871,0x6877,0x6879,
	0x687B,0x687E,0x6927,0x692C,0x694C,0x6977,0x6A41,0x6A65,
	0x6A74,0x6A77,0x6A7C,0x6A7E,0x6B24,0x6B27,0x6B29,0x6B2A,
	0x6B3A,0x6B3B,0x6B3D,0x6B41,0x6B42,0x6B46,0x6B47,0x6B4C,
	0x6B4F,0x6B50,0x6B51,0x6B52,0x6B58,0x6C26,0x6C27,0x6C2A,
	0x6C2F,0x6C30,0x6C31,0x6C32,0x6C35,0x6C38,0x6C3A,0x6C40,
	0x6C41,0x6C45,0x6C46,0x6C49,0x6C4A,0x6C55,0x6C5D,0x6C5E,
	0x6C61,0x6C64,0x6C67,0x6C68,0x6C77,0x6C78,0x6C7A,0x6D21,
	0x6D22,0x6D23,0x6D6E,0x6E5B,0x723D,0x727A,0x7331,0x7427,
	/* Block 251, Array index 0x9000 */
	0x746E,0x7674,0x7676,0x7738,0x7748,0x7753,0x785B,0x7870,
	0x7A21,0x7A22,0x7A66,0x7C29,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 256, Array index 0x9100 */
	INVALC,0x2321,0x2322,0x2323,0x2324,0x2325,0x2326,0x2327,
	0x2328,0x2329,0x232A,0x232B,0x232C,0x232D,0x232E,0x232F,
	0x2330,0x2331,0x2332,0x2333,0x2334,0x2335,0x2336,0x2337,
	0x2338,0x2339,0x233A,0x233B,0x233C,0x233D,0x233E,0x233F,
	0x2340,0x2341,0x2342,0x2343,0x2344,0x2345,0x2346,0x2347,
	0x2348,0x2349,0x234A,0x234B,0x234C,0x234D,0x234E,0x234F,
	0x2350,0x2351,0x2352,0x2353,0x2354,0x2355,0x2356,0x2357,
	0x2358,0x2359,0x235A,0x235B,0x212C,0x235D,0x235E,0x235F,
	0x2360,0x2361,0x2362,0x2363,0x2364,0x2365,0x2366,0x2367,
	0x2368,0x2369,0x236A,0x236B,0x236C,0x236D,0x236E,0x236F,
	0x2370,0x2371,0x2372,0x2373,0x2374,0x2375,0x2376,0x2377,
	0x2378,0x2379,0x237A,0x237B,0x237C,0x237D,0x2226,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x214B,0x214C,0x217E,0x237E,INVALC,0x214D,0x235C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
};

#endif /* ICONV_FROM_UCS_CCS_KSX1001 && !defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit UCS -> ksx1001 size-optimized table (29992 bytes).
 * ======================================================================
 */

#if defined (ICONV_FROM_UCS_CCS_KSX1001) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
from_ucs_size_ksx1001[] =
{
	0x01B6, /* Ranges number */
	0x1338, /* Unranged codes number */
	0x1424, /* First unranged code index */
	/* Ranges list: first code, last Code, array index. */
	/* Array index: 0x0003 */ 0x00B0, 0x00BF, 0x0525,
	/* Array index: 0x0006 */ 0x0131, 0x0133, 0x0535,
	/* Array index: 0x0009 */ 0x013F, 0x0142, 0x0538,
	/* Array index: 0x000C */ 0x0149, 0x014B, 0x053C,
	/* Array index: 0x000F */ 0x02D8, 0x02DD, 0x053F,
	/* Array index: 0x0012 */ 0x0391, 0x03A9, 0x0545,
	/* Array index: 0x0015 */ 0x03B1, 0x03C9, 0x055E,
	/* Array index: 0x0018 */ 0x0410, 0x0451, 0x0577,
	/* Array index: 0x001B */ 0x207F, 0x2084, 0x05B9,
	/* Array index: 0x001E */ 0x215B, 0x2169, 0x05BF,
	/* Array index: 0x0021 */ 0x2170, 0x2179, 0x05CE,
	/* Array index: 0x0024 */ 0x2190, 0x2199, 0x05D8,
	/* Array index: 0x0027 */ 0x2225, 0x222E, 0x05E2,
	/* Array index: 0x002A */ 0x2460, 0x246E, 0x05EC,
	/* Array index: 0x002D */ 0x2474, 0x2482, 0x05FB,
	/* Array index: 0x0030 */ 0x249C, 0x24B5, 0x060A,
	/* Array index: 0x0033 */ 0x24D0, 0x24E9, 0x0624,
	/* Array index: 0x0036 */ 0x2500, 0x2503, 0x063E,
	/* Array index: 0x0039 */ 0x250C, 0x254B, 0x0642,
	/* Array index: 0x003C */ 0x25A0, 0x25A9, 0x0682,
	/* Array index: 0x003F */ 0x25C6, 0x25C8, 0x068C,
	/* Array index: 0x0042 */ 0x25CE, 0x25D1, 0x068F,
	/* Array index: 0x0045 */ 0x2660, 0x266D, 0x0693,
	/* Array index: 0x0048 */ 0x3000, 0x3003, 0x06A1,
	/* Array index: 0x004B */ 0x3008, 0x3015, 0x06A5,
	/* Array index: 0x004E */ 0x3041, 0x3093, 0x06B3,
	/* Array index: 0x0051 */ 0x30A1, 0x30F6, 0x0706,
	/* Array index: 0x0054 */ 0x3131, 0x318E, 0x075C,
	/* Array index: 0x0057 */ 0x3200, 0x321C, 0x07BA,
	/* Array index: 0x005A */ 0x3260, 0x327B, 0x07D7,
	/* Array index: 0x005D */ 0x3380, 0x33CA, 0x07F3,
	/* Array index: 0x0060 */ 0x33DB, 0x33DD, 0x083E,
	/* Array index: 0x0063 */ 0x4E07, 0x4E0D, 0x0841,
	/* Array index: 0x0066 */ 0x4E14, 0x4E19, 0x0848,
	/* Array index: 0x0069 */ 0x4E4B, 0x4E4F, 0x084E,
	/* Array index: 0x006C */ 0x4E5D, 0x4E5F, 0x0853,
	/* Array index: 0x006F */ 0x4E8E, 0x4E95, 0x0856,
	/* Array index: 0x0072 */ 0x4EA1, 0x4EAE, 0x085E,
	/* Array index: 0x0075 */ 0x4ED4, 0x4ED9, 0x086C,
	/* Array index: 0x0078 */ 0x4EE3, 0x4EE5, 0x0872,
	/* Array index: 0x007B */ 0x4F09, 0x4F11, 0x0875,
	/* Array index: 0x007E */ 0x4F46, 0x4F48, 0x087E,
	/* Array index: 0x0081 */ 0x4F4D, 0x4F51, 0x0881,
	/* Array index: 0x0084 */ 0x4F59, 0x4F5C, 0x0886,
	/* Array index: 0x0087 */ 0x4FC2, 0x4FC4, 0x088A,
	/* Array index: 0x008A */ 0x4FDD, 0x4FE1, 0x088D,
	/* Array index: 0x008D */ 0x5023, 0x502D, 0x0892,
	/* Array index: 0x0090 */ 0x5047, 0x5049, 0x089D,
	/* Array index: 0x0093 */ 0x5074, 0x5078, 0x08A0,
	/* Array index: 0x0096 */ 0x50FF, 0x5101, 0x08A5,
	/* Array index: 0x0099 */ 0x5140, 0x514E, 0x08A8,
	/* Array index: 0x009C */ 0x5165, 0x516E, 0x08B7,
	/* Array index: 0x009F */ 0x5175, 0x5178, 0x08C1,
	/* Array index: 0x00A2 */ 0x51C9, 0x51CD, 0x08C5,
	/* Array index: 0x00A5 */ 0x51DC, 0x51DE, 0x08CA,
	/* Array index: 0x00A8 */ 0x51F6, 0x51FA, 0x08CD,
	/* Array index: 0x00AB */ 0x5206, 0x520A, 0x08D2,
	/* Array index: 0x00AE */ 0x5236, 0x523B, 0x08D7,
	/* Array index: 0x00B1 */ 0x524A, 0x524D, 0x08DD,
	/* Array index: 0x00B4 */ 0x5287, 0x5289, 0x08E1,
	/* Array index: 0x00B7 */ 0x52A9, 0x52AB, 0x08E4,
	/* Array index: 0x00BA */ 0x52D5, 0x52E4, 0x08E7,
	/* Array index: 0x00BD */ 0x5315, 0x5319, 0x08F7,
	/* Array index: 0x00C0 */ 0x533F, 0x534A, 0x08FC,
	/* Array index: 0x00C3 */ 0x5351, 0x5354, 0x0908,
	/* Array index: 0x00C6 */ 0x536F, 0x5377, 0x090C,
	/* Array index: 0x00C9 */ 0x53C8, 0x53CD, 0x0915,
	/* Array index: 0x00CC */ 0x53E1, 0x53F3, 0x091B,
	/* Array index: 0x00CF */ 0x5408, 0x5411, 0x092E,
	/* Array index: 0x00D2 */ 0x5438, 0x543E, 0x0938,
	/* Array index: 0x00D5 */ 0x547B, 0x547D, 0x093F,
	/* Array index: 0x00D8 */ 0x5586, 0x558A, 0x0942,
	/* Array index: 0x00DB */ 0x5598, 0x559D, 0x0947,
	/* Array index: 0x00DE */ 0x55A7, 0x55AE, 0x094D,
	/* Array index: 0x00E1 */ 0x574D, 0x5751, 0x0955,
	/* Array index: 0x00E4 */ 0x5805, 0x580A, 0x095A,
	/* Array index: 0x00E7 */ 0x582F, 0x5835, 0x0960,
	/* Array index: 0x00EA */ 0x58F9, 0x58FD, 0x0967,
	/* Array index: 0x00ED */ 0x5914, 0x591C, 0x096C,
	/* Array index: 0x00F0 */ 0x5927, 0x592E, 0x0975,
	/* Array index: 0x00F3 */ 0x5947, 0x5949, 0x097D,
	/* Array index: 0x00F6 */ 0x594E, 0x5957, 0x0980,
	/* Array index: 0x00F9 */ 0x596A, 0x596E, 0x098A,
	/* Array index: 0x00FC */ 0x5982, 0x5984, 0x098F,
	/* Array index: 0x00FF */ 0x59D0, 0x59D4, 0x0992,
	/* Array index: 0x0102 */ 0x59D9, 0x59DD, 0x0997,
	/* Array index: 0x0105 */ 0x5B54, 0x5B5F, 0x099C,
	/* Array index: 0x0108 */ 0x5B85, 0x5B8C, 0x09A8,
	/* Array index: 0x010B */ 0x5B93, 0x5B9C, 0x09B0,
	/* Array index: 0x010E */ 0x5BA2, 0x5BA6, 0x09BA,
	/* Array index: 0x0111 */ 0x5BB3, 0x5BB9, 0x09BF,
	/* Array index: 0x0114 */ 0x5BBF, 0x5BC7, 0x09C6,
	/* Array index: 0x0117 */ 0x5BD0, 0x5BD4, 0x09CF,
	/* Array index: 0x011A */ 0x5BDE, 0x5BEF, 0x09D4,
	/* Array index: 0x011D */ 0x5C07, 0x5C11, 0x09E6,
	/* Array index: 0x0120 */ 0x5C38, 0x5C40, 0x09F1,
	/* Array index: 0x0123 */ 0x5DE1, 0x5DE8, 0x09FA,
	/* Array index: 0x0126 */ 0x5DF1, 0x5DF4, 0x0A02,
	/* Array index: 0x0129 */ 0x5E5F, 0x5E63, 0x0A06,
	/* Array index: 0x012C */ 0x5E72, 0x5E7E, 0x0A0B,
	/* Array index: 0x012F */ 0x5EB5, 0x5EB8, 0x0A18,
	/* Array index: 0x0132 */ 0x5EC8, 0x5ECA, 0x0A1C,
	/* Array index: 0x0135 */ 0x5EDF, 0x5EE3, 0x0A1F,
	/* Array index: 0x0138 */ 0x5F11, 0x5F18, 0x0A24,
	/* Array index: 0x013B */ 0x5F66, 0x5F71, 0x0A2C,
	/* Array index: 0x013E */ 0x5F7F, 0x5F81, 0x0A38,
	/* Array index: 0x0141 */ 0x5F8A, 0x5F92, 0x0A3B,
	/* Array index: 0x0144 */ 0x5F97, 0x5F99, 0x0A44,
	/* Array index: 0x0147 */ 0x5FA8, 0x5FAA, 0x0A47,
	/* Array index: 0x014A */ 0x5FD6, 0x5FD9, 0x0A4A,
	/* Array index: 0x014D */ 0x6041, 0x6043, 0x0A4E,
	/* Array index: 0x0150 */ 0x6062, 0x6070, 0x0A51,
	/* Array index: 0x0153 */ 0x60B0, 0x60B8, 0x0A60,
	/* Array index: 0x0156 */ 0x60DF, 0x60E1, 0x0A69,
	/* Array index: 0x0159 */ 0x60F9, 0x60FB, 0x0A6C,
	/* Array index: 0x015C */ 0x610D, 0x610F, 0x0A6F,
	/* Array index: 0x015F */ 0x6147, 0x614C, 0x0A72,
	/* Array index: 0x0162 */ 0x6162, 0x6168, 0x0A78,
	/* Array index: 0x0165 */ 0x6198, 0x619A, 0x0A7F,
	/* Array index: 0x0168 */ 0x61C7, 0x61CB, 0x0A82,
	/* Array index: 0x016B */ 0x61F6, 0x6200, 0x0A87,
	/* Array index: 0x016E */ 0x620A, 0x6212, 0x0A92,
	/* Array index: 0x0171 */ 0x623E, 0x6241, 0x0A9B,
	/* Array index: 0x0174 */ 0x6247, 0x624D, 0x0A9F,
	/* Array index: 0x0177 */ 0x62C7, 0x62DC, 0x0AA6,
	/* Array index: 0x017A */ 0x62EC, 0x62F3, 0x0ABC,
	/* Array index: 0x017D */ 0x633A, 0x633E, 0x0AC4,
	/* Array index: 0x0180 */ 0x63A0, 0x63A2, 0x0AC9,
	/* Array index: 0x0183 */ 0x63A5, 0x63AA, 0x0ACC,
	/* Array index: 0x0186 */ 0x6478, 0x647A, 0x0AD2,
	/* Array index: 0x0189 */ 0x6491, 0x6493, 0x0AD5,
	/* Array index: 0x018C */ 0x64E1, 0x64E7, 0x0AD8,
	/* Array index: 0x018F */ 0x652A, 0x652C, 0x0ADF,
	/* Array index: 0x0192 */ 0x6536, 0x653F, 0x0AE2,
	/* Array index: 0x0195 */ 0x654D, 0x6551, 0x0AEC,
	/* Array index: 0x0198 */ 0x6574, 0x6578, 0x0AF1,
	/* Array index: 0x019B */ 0x65E5, 0x65ED, 0x0AF6,
	/* Array index: 0x019E */ 0x65FA, 0x65FF, 0x0AFF,
	/* Array index: 0x01A1 */ 0x6606, 0x6615, 0x0B05,
	/* Array index: 0x01A4 */ 0x661E, 0x6620, 0x0B15,
	/* Array index: 0x01A7 */ 0x662D, 0x6631, 0x0B18,
	/* Array index: 0x01AA */ 0x6641, 0x6644, 0x0B1D,
	/* Array index: 0x01AD */ 0x665B, 0x665F, 0x0B21,
	/* Array index: 0x01B0 */ 0x6664, 0x666F, 0x0B26,
	/* Array index: 0x01B3 */ 0x6673, 0x667A, 0x0B32,
	/* Array index: 0x01B6 */ 0x6687, 0x6689, 0x0B3A,
	/* Array index: 0x01B9 */ 0x6696, 0x6698, 0x0B3D,
	/* Array index: 0x01BC */ 0x66B2, 0x66B4, 0x0B40,
	/* Array index: 0x01BF */ 0x66F0, 0x6700, 0x0B43,
	/* Array index: 0x01C2 */ 0x671B, 0x671F, 0x0B54,
	/* Array index: 0x01C5 */ 0x6726, 0x672E, 0x0B59,
	/* Array index: 0x01C8 */ 0x674E, 0x6753, 0x0B62,
	/* Array index: 0x01CB */ 0x676D, 0x6777, 0x0B68,
	/* Array index: 0x01CE */ 0x67B6, 0x67B8, 0x0B73,
	/* Array index: 0x01D1 */ 0x67CF, 0x67D4, 0x0B76,
	/* Array index: 0x01D4 */ 0x67EF, 0x67F6, 0x0B7C,
	/* Array index: 0x01D7 */ 0x683C, 0x6843, 0x0B84,
	/* Array index: 0x01DA */ 0x6850, 0x6854, 0x0B8C,
	/* Array index: 0x01DD */ 0x68AD, 0x68B6, 0x0B91,
	/* Array index: 0x01E0 */ 0x6953, 0x695A, 0x0B9B,
	/* Array index: 0x01E3 */ 0x695E, 0x6963, 0x0BA3,
	/* Array index: 0x01E6 */ 0x696B, 0x696F, 0x0BA9,
	/* Array index: 0x01E9 */ 0x6975, 0x6979, 0x0BAE,
	/* Array index: 0x01EC */ 0x69CB, 0x69CD, 0x0BB3,
	/* Array index: 0x01EF */ 0x6A38, 0x6A3A, 0x0BB6,
	/* Array index: 0x01F2 */ 0x6B5F, 0x6B66, 0x0BB9,
	/* Array index: 0x01F5 */ 0x6BD2, 0x6BD8, 0x0BC1,
	/* Array index: 0x01F8 */ 0x6C3E, 0x6C42, 0x0BC8,
	/* Array index: 0x01FB */ 0x6C5D, 0x6C60, 0x0BCD,
	/* Array index: 0x01FE */ 0x6C7D, 0x6C88, 0x0BD1,
	/* Array index: 0x0201 */ 0x6C90, 0x6C9B, 0x0BDD,
	/* Array index: 0x0204 */ 0x6CB8, 0x6CC4, 0x0BE9,
	/* Array index: 0x0207 */ 0x6CE1, 0x6CE5, 0x0BF6,
	/* Array index: 0x020A */ 0x6CEE, 0x6CF0, 0x0BFB,
	/* Array index: 0x020D */ 0x6D35, 0x6D3E, 0x0BFE,
	/* Array index: 0x0210 */ 0x6D77, 0x6D79, 0x0C08,
	/* Array index: 0x0213 */ 0x6D85, 0x6D8E, 0x0C0B,
	/* Array index: 0x0216 */ 0x6DC3, 0x6DC7, 0x0C15,
	/* Array index: 0x0219 */ 0x6DD8, 0x6DDA, 0x0C1A,
	/* Array index: 0x021C */ 0x6DF5, 0x6DFB, 0x0C1D,
	/* Array index: 0x021F */ 0x6E17, 0x6E26, 0x0C24,
	/* Array index: 0x0222 */ 0x6E2B, 0x6E2F, 0x0C34,
	/* Array index: 0x0225 */ 0x6E3A, 0x6E3E, 0x0C39,
	/* Array index: 0x0228 */ 0x6E5B, 0x6E5F, 0x0C3E,
	/* Array index: 0x022B */ 0x6EFE, 0x6F02, 0x0C43,
	/* Array index: 0x022E */ 0x6F57, 0x6F5F, 0x0C48,
	/* Array index: 0x0231 */ 0x6F7A, 0x6F7E, 0x0C51,
	/* Array index: 0x0234 */ 0x6FBE, 0x6FC3, 0x0C56,
	/* Array index: 0x0237 */ 0x6FDF, 0x6FE1, 0x0C5C,
	/* Array index: 0x023A */ 0x7018, 0x701F, 0x0C5F,
	/* Array index: 0x023D */ 0x7098, 0x709A, 0x0C67,
	/* Array index: 0x0240 */ 0x70AB, 0x70AF, 0x0C6A,
	/* Array index: 0x0243 */ 0x70B7, 0x70B9, 0x0C6F,
	/* Array index: 0x0246 */ 0x7164, 0x7169, 0x0C72,
	/* Array index: 0x0249 */ 0x71E5, 0x71E7, 0x0C78,
	/* Array index: 0x024C */ 0x71FB, 0x7200, 0x0C7B,
	/* Array index: 0x024F */ 0x723A, 0x7240, 0x0C81,
	/* Array index: 0x0252 */ 0x7246, 0x7248, 0x0C88,
	/* Array index: 0x0255 */ 0x731B, 0x731D, 0x0C8B,
	/* Array index: 0x0258 */ 0x7329, 0x732B, 0x0C8E,
	/* Array index: 0x025B */ 0x7394, 0x7398, 0x0C91,
	/* Array index: 0x025E */ 0x73C9, 0x73CF, 0x0C96,
	/* Array index: 0x0261 */ 0x73E3, 0x73EA, 0x0C9D,
	/* Array index: 0x0264 */ 0x7403, 0x7409, 0x0CA5,
	/* Array index: 0x0267 */ 0x7420, 0x743A, 0x0CAC,
	/* Array index: 0x026A */ 0x743F, 0x7444, 0x0CC7,
	/* Array index: 0x026D */ 0x7457, 0x746A, 0x0CCD,
	/* Array index: 0x0270 */ 0x7530, 0x7533, 0x0CE1,
	/* Array index: 0x0273 */ 0x7559, 0x755D, 0x0CE5,
	/* Array index: 0x0276 */ 0x75B1, 0x75BE, 0x0CEA,
	/* Array index: 0x0279 */ 0x761F, 0x7626, 0x0CF8,
	/* Array index: 0x027C */ 0x767B, 0x767E, 0x0D00,
	/* Array index: 0x027F */ 0x76DB, 0x76E4, 0x0D04,
	/* Array index: 0x0282 */ 0x77ED, 0x77EF, 0x0D0E,
	/* Array index: 0x0285 */ 0x7825, 0x7827, 0x0D11,
	/* Array index: 0x0288 */ 0x788C, 0x788E, 0x0D14,
	/* Array index: 0x028B */ 0x78BA, 0x78BC, 0x0D17,
	/* Array index: 0x028E */ 0x792A, 0x792C, 0x0D1A,
	/* Array index: 0x0291 */ 0x7947, 0x7949, 0x0D1D,
	/* Array index: 0x0294 */ 0x7956, 0x7960, 0x0D20,
	/* Array index: 0x0297 */ 0x798D, 0x7991, 0x0D2B,
	/* Array index: 0x029A */ 0x79BD, 0x79C1, 0x0D30,
	/* Array index: 0x029D */ 0x79C9, 0x79CB, 0x0D35,
	/* Array index: 0x02A0 */ 0x7A3B, 0x7A40, 0x0D38,
	/* Array index: 0x02A3 */ 0x7B49, 0x7B56, 0x0D3E,
	/* Array index: 0x02A6 */ 0x7C3D, 0x7C3F, 0x0D4C,
	/* Array index: 0x02A9 */ 0x7CB1, 0x7CB3, 0x0D4F,
	/* Array index: 0x02AC */ 0x7CDE, 0x7CE0, 0x0D52,
	/* Array index: 0x02AF */ 0x7D02, 0x7D0D, 0x0D55,
	/* Array index: 0x02B2 */ 0x7D17, 0x7D1B, 0x0D61,
	/* Array index: 0x02B5 */ 0x7D20, 0x7D22, 0x0D66,
	/* Array index: 0x02B8 */ 0x7D2B, 0x7D30, 0x0D69,
	/* Array index: 0x02BB */ 0x7D42, 0x7D46, 0x0D6F,
	/* Array index: 0x02BE */ 0x7D71, 0x7D73, 0x0D74,
	/* Array index: 0x02C1 */ 0x7DB1, 0x7DB5, 0x0D77,
	/* Array index: 0x02C4 */ 0x7DBA, 0x7DBF, 0x0D7C,
	/* Array index: 0x02C7 */ 0x7DDD, 0x7DE3, 0x0D82,
	/* Array index: 0x02CA */ 0x7E1B, 0x7E23, 0x0D89,
	/* Array index: 0x02CD */ 0x7E69, 0x7E6D, 0x0D92,
	/* Array index: 0x02D0 */ 0x7FDF, 0x7FE1, 0x0D97,
	/* Array index: 0x02D3 */ 0x806F, 0x8073, 0x0D9A,
	/* Array index: 0x02D6 */ 0x807D, 0x807F, 0x0D9F,
	/* Array index: 0x02D9 */ 0x8084, 0x808C, 0x0DA2,
	/* Array index: 0x02DC */ 0x8105, 0x810A, 0x0DAB,
	/* Array index: 0x02DF */ 0x8178, 0x817A, 0x0DB1,
	/* Array index: 0x02E2 */ 0x81BD, 0x81C2, 0x0DB4,
	/* Array index: 0x02E5 */ 0x81FA, 0x81FE, 0x0DBA,
	/* Array index: 0x02E8 */ 0x821B, 0x8221, 0x0DBF,
	/* Array index: 0x02EB */ 0x822A, 0x822C, 0x0DC6,
	/* Array index: 0x02EE */ 0x8235, 0x8239, 0x0DC9,
	/* Array index: 0x02F1 */ 0x826E, 0x8272, 0x0DCE,
	/* Array index: 0x02F4 */ 0x82AC, 0x82BF, 0x0DD3,
	/* Array index: 0x02F7 */ 0x82D1, 0x82D7, 0x0DE7,
	/* Array index: 0x02FA */ 0x82E5, 0x82E7, 0x0DEE,
	/* Array index: 0x02FD */ 0x82FD, 0x8305, 0x0DF1,
	/* Array index: 0x0300 */ 0x8334, 0x8339, 0x0DFA,
	/* Array index: 0x0303 */ 0x83A9, 0x83AB, 0x0E00,
	/* Array index: 0x0306 */ 0x83EF, 0x83F4, 0x0E03,
	/* Array index: 0x0309 */ 0x840A, 0x840E, 0x0E09,
	/* Array index: 0x030C */ 0x8568, 0x856A, 0x0E0E,
	/* Array index: 0x030F */ 0x85A6, 0x85AA, 0x0E11,
	/* Array index: 0x0312 */ 0x85CD, 0x85CF, 0x0E16,
	/* Array index: 0x0315 */ 0x865B, 0x865F, 0x0E19,
	/* Array index: 0x0318 */ 0x8861, 0x8863, 0x0E1E,
	/* Array index: 0x031B */ 0x887E, 0x8882, 0x0E21,
	/* Array index: 0x031E */ 0x88F3, 0x88F5, 0x0E26,
	/* Array index: 0x0321 */ 0x8A16, 0x8A18, 0x0E29,
	/* Array index: 0x0324 */ 0x8A6D, 0x8A75, 0x0E2C,
	/* Array index: 0x0327 */ 0x8AA0, 0x8AAA, 0x0E35,
	/* Array index: 0x032A */ 0x8AEA, 0x8AEE, 0x0E40,
	/* Array index: 0x032D */ 0x8AF6, 0x8AFA, 0x0E45,
	/* Array index: 0x0330 */ 0x8AFE, 0x8B04, 0x0E4A,
	/* Array index: 0x0333 */ 0x8B16, 0x8B1D, 0x0E51,
	/* Array index: 0x0336 */ 0x8C9D, 0x8CA2, 0x0E59,
	/* Array index: 0x0339 */ 0x8CA7, 0x8CCA, 0x0E5F,
	/* Array index: 0x033C */ 0x8CE0, 0x8CE6, 0x0E83,
	/* Array index: 0x033F */ 0x8CFB, 0x8CFD, 0x0E8A,
	/* Array index: 0x0342 */ 0x8D04, 0x8D0A, 0x0E8D,
	/* Array index: 0x0345 */ 0x8E47, 0x8E4A, 0x0E94,
	/* Array index: 0x0348 */ 0x8EAA, 0x8EAC, 0x0E98,
	/* Array index: 0x034B */ 0x8ECA, 0x8ECD, 0x0E9B,
	/* Array index: 0x034E */ 0x8F12, 0x8F15, 0x0E9F,
	/* Array index: 0x0351 */ 0x8F1B, 0x8F1F, 0x0EA3,
	/* Array index: 0x0354 */ 0x8F26, 0x8F2A, 0x0EA8,
	/* Array index: 0x0357 */ 0x8FAD, 0x8FB2, 0x0EAD,
	/* Array index: 0x035A */ 0x9000, 0x9008, 0x0EB3,
	/* Array index: 0x035D */ 0x900D, 0x9023, 0x0EBC,
	/* Array index: 0x0360 */ 0x904A, 0x9055, 0x0ED3,
	/* Array index: 0x0363 */ 0x905C, 0x9063, 0x0EDF,
	/* Array index: 0x0366 */ 0x906D, 0x906F, 0x0EE7,
	/* Array index: 0x0369 */ 0x907C, 0x908A, 0x0EEA,
	/* Array index: 0x036C */ 0x90AF, 0x90B1, 0x0EF9,
	/* Array index: 0x036F */ 0x9149, 0x914E, 0x0EFC,
	/* Array index: 0x0372 */ 0x91AB, 0x91B5, 0x0F02,
	/* Array index: 0x0375 */ 0x91C9, 0x91D1, 0x0F0D,
	/* Array index: 0x0378 */ 0x9210, 0x9212, 0x0F16,
	/* Array index: 0x037B */ 0x9262, 0x9266, 0x0F19,
	/* Array index: 0x037E */ 0x931E, 0x9328, 0x0F1E,
	/* Array index: 0x0381 */ 0x958F, 0x9594, 0x0F29,
	/* Array index: 0x0384 */ 0x95A3, 0x95A5, 0x0F2F,
	/* Array index: 0x0387 */ 0x95D4, 0x95D6, 0x0F32,
	/* Array index: 0x038A */ 0x964B, 0x964D, 0x0F35,
	/* Array index: 0x038D */ 0x965B, 0x9664, 0x0F38,
	/* Array index: 0x0390 */ 0x9673, 0x9678, 0x0F42,
	/* Array index: 0x0393 */ 0x968A, 0x968E, 0x0F48,
	/* Array index: 0x0396 */ 0x9698, 0x969C, 0x0F4D,
	/* Array index: 0x0399 */ 0x96C0, 0x96CE, 0x0F52,
	/* Array index: 0x039C */ 0x96E8, 0x96EA, 0x0F61,
	/* Array index: 0x039F */ 0x97FF, 0x9813, 0x0F64,
	/* Array index: 0x03A2 */ 0x984C, 0x984E, 0x0F79,
	/* Array index: 0x03A5 */ 0x98ED, 0x98EF, 0x0F7C,
	/* Array index: 0x03A8 */ 0x98FC, 0x98FE, 0x0F7F,
	/* Array index: 0x03AB */ 0x9949, 0x994D, 0x0F82,
	/* Array index: 0x03AE */ 0x99AC, 0x99AE, 0x0F87,
	/* Array index: 0x03B1 */ 0x99D0, 0x99D2, 0x0F8A,
	/* Array index: 0x03B4 */ 0x9B41, 0x9B45, 0x0F8D,
	/* Array index: 0x03B7 */ 0x9DF8, 0x9DFA, 0x0F92,
	/* Array index: 0x03BA */ 0xAC07, 0xAC0A, 0x0F95,
	/* Array index: 0x03BD */ 0xAC10, 0xAC1D, 0x0F99,
	/* Array index: 0x03C0 */ 0xAC2C, 0xAC31, 0x0FA7,
	/* Array index: 0x03C3 */ 0xAC80, 0xAC8C, 0x0FAD,
	/* Array index: 0x03C6 */ 0xAC9C, 0xACA1, 0x0FBA,
	/* Array index: 0x03C9 */ 0xACA8, 0xACB0, 0x0FC0,
	/* Array index: 0x03CC */ 0xACB8, 0xACBD, 0x0FC9,
	/* Array index: 0x03CF */ 0xACEF, 0xACF6, 0x0FCF,
	/* Array index: 0x03D2 */ 0xAD73, 0xAD76, 0x0FD7,
	/* Array index: 0x03D5 */ 0xAD7B, 0xAD82, 0x0FDB,
	/* Array index: 0x03D8 */ 0xADFF, 0xAE01, 0x0FE3,
	/* Array index: 0x03DB */ 0xAE4A, 0xAE50, 0x0FE6,
	/* Array index: 0x03DE */ 0xAE5C, 0xAE61, 0x0FED,
	/* Array index: 0x03E1 */ 0xAE78, 0xAE7D, 0x0FF3,
	/* Array index: 0x03E4 */ 0xAEBC, 0xAEC0, 0x0FF9,
	/* Array index: 0x03E7 */ 0xAECC, 0xAED1, 0x0FFE,
	/* Array index: 0x03EA */ 0xAF3F, 0xAF43, 0x1004,
	/* Array index: 0x03ED */ 0xAFC7, 0xAFCE, 0x1009,
	/* Array index: 0x03F0 */ 0xB053, 0xB059, 0x1011,
	/* Array index: 0x03F3 */ 0xB098, 0xB0A2, 0x1018,
	/* Array index: 0x03F6 */ 0xB0A8, 0xB0B5, 0x1023,
	/* Array index: 0x03F9 */ 0xB0C4, 0xB0C9, 0x1031,
	/* Array index: 0x03FC */ 0xB108, 0xB10C, 0x1037,
	/* Array index: 0x03FF */ 0xB118, 0xB11D, 0x103C,
	/* Array index: 0x0402 */ 0xB123, 0xB125, 0x1042,
	/* Array index: 0x0405 */ 0xB134, 0xB139, 0x1045,
	/* Array index: 0x0408 */ 0xB192, 0xB194, 0x104B,
	/* Array index: 0x040B */ 0xB298, 0xB29A, 0x104E,
	/* Array index: 0x040E */ 0xB2E2, 0xB301, 0x1051,
	/* Array index: 0x0411 */ 0xB310, 0xB315, 0x1071,
	/* Array index: 0x0414 */ 0xB354, 0xB35F, 0x1077,
	/* Array index: 0x0417 */ 0xB380, 0xB385, 0x1083,
	/* Array index: 0x041A */ 0xB524, 0xB52A, 0x1089,
	/* Array index: 0x041D */ 0xB540, 0xB545, 0x1090,
	/* Array index: 0x0420 */ 0xB54B, 0xB54D, 0x1096,
	/* Array index: 0x0423 */ 0xB55C, 0xB561, 0x1099,
	/* Array index: 0x0426 */ 0xB5B0, 0xB5B5, 0x109F,
	/* Array index: 0x0429 */ 0xB5BB, 0xB5BD, 0x10A5,
	/* Array index: 0x042C */ 0xB5CC, 0xB5D1, 0x10A8,
	/* Array index: 0x042F */ 0xB78C, 0xB799, 0x10AE,
	/* Array index: 0x0432 */ 0xB7A8, 0xB7AD, 0x10BC,
	/* Array index: 0x0435 */ 0xB7FC, 0xB801, 0x10C2,
	/* Array index: 0x0438 */ 0xB807, 0xB809, 0x10C8,
	/* Array index: 0x043B */ 0xB834, 0xB839, 0x10CB,
	/* Array index: 0x043E */ 0xB9CC, 0xB9D2, 0x10D1,
	/* Array index: 0x0441 */ 0xB9E1, 0xB9E5, 0x10D8,
	/* Array index: 0x0444 */ 0xB9F4, 0xB9FA, 0x10DD,
	/* Array index: 0x0447 */ 0xBA53, 0xBA55, 0x10E4,
	/* Array index: 0x044A */ 0xBA64, 0xBA69, 0x10E7,
	/* Array index: 0x044D */ 0xBA83, 0xBA87, 0x10ED,
	/* Array index: 0x0450 */ 0xBAA8, 0xBAAC, 0x10F2,
	/* Array index: 0x0453 */ 0xBB34, 0xBB3E, 0x10F7,
	/* Array index: 0x0456 */ 0xBC08, 0xBC1F, 0x1102,
	/* Array index: 0x0459 */ 0xBC40, 0xBC45, 0x111A,
	/* Array index: 0x045C */ 0xBCB0, 0xBCB5, 0x1120,
	/* Array index: 0x045F */ 0xBCCD, 0xBCD1, 0x1126,
	/* Array index: 0x0462 */ 0xBCF4, 0xBCF8, 0x112B,
	/* Array index: 0x0465 */ 0xBD87, 0xBD8A, 0x1130,
	/* Array index: 0x0468 */ 0xBE57, 0xBE5B, 0x1134,
	/* Array index: 0x046B */ 0xBE70, 0xBE75, 0x1139,
	/* Array index: 0x046E */ 0xBE7B, 0xBE7D, 0x113F,
	/* Array index: 0x0471 */ 0xBE8C, 0xBE91, 0x1142,
	/* Array index: 0x0474 */ 0xBEE3, 0xBEE5, 0x1148,
	/* Array index: 0x0477 */ 0xBF18, 0xBF1D, 0x114B,
	/* Array index: 0x047A */ 0xC0AC, 0xC0B6, 0x1151,
	/* Array index: 0x047D */ 0xC0BC, 0xC0C1, 0x115C,
	/* Array index: 0x0480 */ 0xC0D8, 0xC0DD, 0x1162,
	/* Array index: 0x0483 */ 0xC11C, 0xC127, 0x1168,
	/* Array index: 0x0486 */ 0xC12C, 0xC131, 0x1174,
	/* Array index: 0x0489 */ 0xC148, 0xC14D, 0x117A,
	/* Array index: 0x048C */ 0xC164, 0xC169, 0x1180,
	/* Array index: 0x048F */ 0xC18C, 0xC190, 0x1186,
	/* Array index: 0x0492 */ 0xC2EB, 0xC2F1, 0x118B,
	/* Array index: 0x0495 */ 0xC2F8, 0xC2FC, 0x1192,
	/* Array index: 0x0498 */ 0xC313, 0xC315, 0x1197,
	/* Array index: 0x049B */ 0xC4FF, 0xC501, 0x119A,
	/* Array index: 0x049E */ 0xC544, 0xC54E, 0x119D,
	/* Array index: 0x04A1 */ 0xC553, 0xC559, 0x11A8,
	/* Array index: 0x04A4 */ 0xC55D, 0xC561, 0x11AF,
	/* Array index: 0x04A7 */ 0xC570, 0xC575, 0x11B4,
	/* Array index: 0x04AA */ 0xC5B8, 0xC5BE, 0x11BA,
	/* Array index: 0x04AD */ 0xC5C4, 0xC5D1, 0x11C1,
	/* Array index: 0x04B0 */ 0xC5EC, 0xC5F0, 0x11CF,
	/* Array index: 0x04B3 */ 0xC5FC, 0xC608, 0x11D4,
	/* Array index: 0x04B6 */ 0xC618, 0xC61C, 0x11E1,
	/* Array index: 0x04B9 */ 0xC62C, 0xC63B, 0x11E6,
	/* Array index: 0x04BC */ 0xC650, 0xC655, 0x11F6,
	/* Array index: 0x04BF */ 0xC6B8, 0xC6BA, 0x11FC,
	/* Array index: 0x04C2 */ 0xC74F, 0xC758, 0x11FF,
	/* Array index: 0x04C5 */ 0xC77C, 0xC77E, 0x1209,
	/* Array index: 0x04C8 */ 0xC783, 0xC78A, 0x120C,
	/* Array index: 0x04CB */ 0xC794, 0xC79A, 0x1214,
	/* Array index: 0x04CE */ 0xC7A0, 0xC7A6, 0x121B,
	/* Array index: 0x04D1 */ 0xC7BC, 0xC7C1, 0x1222,
	/* Array index: 0x04D4 */ 0xC883, 0xC88D, 0x1228,
	/* Array index: 0x04D7 */ 0xC904, 0xC906, 0x1233,
	/* Array index: 0x04DA */ 0xC9D9, 0xC9DD, 0x1236,
	/* Array index: 0x04DD */ 0xC9EC, 0xC9F1, 0x123B,
	/* Array index: 0x04E0 */ 0xCA08, 0xCA0D, 0x1241,
	/* Array index: 0x04E3 */ 0xCA5C, 0xCA61, 0x1247,
	/* Array index: 0x04E6 */ 0xCC27, 0xCC29, 0x124D,
	/* Array index: 0x04E9 */ 0xCC38, 0xCC3E, 0x1250,
	/* Array index: 0x04EC */ 0xCC54, 0xCC59, 0x1257,
	/* Array index: 0x04EF */ 0xCCA8, 0xCCAD, 0x125D,
	/* Array index: 0x04F2 */ 0xCE5F, 0xCE61, 0x1263,
	/* Array index: 0x04F5 */ 0xCEA0, 0xCEA5, 0x1266,
	/* Array index: 0x04F8 */ 0xCEF4, 0xCEF9, 0x126C,
	/* Array index: 0x04FB */ 0xCF2C, 0xCF31, 0x1272,
	/* Array index: 0x04FE */ 0xD0D0, 0xD0D5, 0x1278,
	/* Array index: 0x0501 */ 0xD0EC, 0xD0F1, 0x127E,
	/* Array index: 0x0504 */ 0xD140, 0xD145, 0x1284,
	/* Array index: 0x0507 */ 0xD30C, 0xD310, 0x128A,
	/* Array index: 0x050A */ 0xD31C, 0xD321, 0x128F,
	/* Array index: 0x050D */ 0xD338, 0xD33D, 0x1295,
	/* Array index: 0x0510 */ 0xD38C, 0xD391, 0x129B,
	/* Array index: 0x0513 */ 0xD584, 0xD589, 0x12A1,
	/* Array index: 0x0516 */ 0xD610, 0xD615, 0x12A7,
	/* Array index: 0x0519 */ 0xD754, 0xD759, 0x12AD,
	/* Array index: 0x051C */ 0xF900, 0xFA0B, 0x12B3,
	/* Array index: 0x051F */ 0xFF01, 0xFF5E, 0x13BF,
	/* Array index: 0x0522 */ 0xFFE0, 0xFFE6, 0x141D,
	/* Ranges content */
	/* Range 0x00B0 - 0x00BF, array index: 0x0525 */
	0x2146,0x213E,0x2977,0x2978,0x2225,INVALC,0x2252,0x2124,
	0x222C,0x2976,0x282C,INVALC,0x2879,0x2876,0x287A,0x222F,
	/* Range 0x0131 - 0x0133, array index: 0x0535 */
	0x2925,0x2826,0x2926,
	/* Range 0x013F - 0x0142, array index: 0x0538 */
	0x2828,0x2928,0x2829,0x2929,
	/* Range 0x0149 - 0x014B, array index: 0x053C */
	0x2930,0x282F,0x292F,
	/* Range 0x02D8 - 0x02DD, array index: 0x053F */
	0x2228,0x222B,0x222A,0x222D,INVALC,0x2229,
	/* Range 0x0391 - 0x03A9, array index: 0x0545 */
	0x2541,0x2542,0x2543,0x2544,0x2545,0x2546,0x2547,0x2548,
	0x2549,0x254A,0x254B,0x254C,0x254D,0x254E,0x254F,0x2550,
	0x2551,INVALC,0x2552,0x2553,0x2554,0x2555,0x2556,0x2557,
	0x2558,
	/* Range 0x03B1 - 0x03C9, array index: 0x055E */
	0x2561,0x2562,0x2563,0x2564,0x2565,0x2566,0x2567,0x2568,
	0x2569,0x256A,0x256B,0x256C,0x256D,0x256E,0x256F,0x2570,
	0x2571,INVALC,0x2572,0x2573,0x2574,0x2575,0x2576,0x2577,
	0x2578,
	/* Range 0x0410 - 0x0451, array index: 0x0577 */
	0x2C21,0x2C22,0x2C23,0x2C24,0x2C25,0x2C26,0x2C28,0x2C29,
	0x2C2A,0x2C2B,0x2C2C,0x2C2D,0x2C2E,0x2C2F,0x2C30,0x2C31,
	0x2C32,0x2C33,0x2C34,0x2C35,0x2C36,0x2C37,0x2C38,0x2C39,
	0x2C3A,0x2C3B,0x2C3C,0x2C3D,0x2C3E,0x2C3F,0x2C40,0x2C41,
	0x2C51,0x2C52,0x2C53,0x2C54,0x2C55,0x2C56,0x2C58,0x2C59,
	0x2C5A,0x2C5B,0x2C5C,0x2C5D,0x2C5E,0x2C5F,0x2C60,0x2C61,
	0x2C62,0x2C63,0x2C64,0x2C65,0x2C66,0x2C67,0x2C68,0x2C69,
	0x2C6A,0x2C6B,0x2C6C,0x2C6D,0x2C6E,0x2C6F,0x2C70,0x2C71,
	INVALC,0x2C57,
	/* Range 0x207F - 0x2084, array index: 0x05B9 */
	0x297A,INVALC,0x297B,0x297C,0x297D,0x297E,
	/* Range 0x215B - 0x2169, array index: 0x05BF */
	0x287B,0x287C,0x287D,0x287E,INVALC,0x2530,0x2531,0x2532,
	0x2533,0x2534,0x2535,0x2536,0x2537,0x2538,0x2539,
	/* Range 0x2170 - 0x2179, array index: 0x05CE */
	0x2521,0x2522,0x2523,0x2524,0x2525,0x2526,0x2527,0x2528,
	0x2529,0x252A,
	/* Range 0x2190 - 0x2199, array index: 0x05D8 */
	0x2167,0x2168,0x2166,0x2169,0x216A,0x2255,0x2258,0x2256,
	0x2259,0x2257,
	/* Range 0x2225 - 0x222E, array index: 0x05E2 */
	0x212B,INVALC,0x217C,0x217D,0x217B,0x217A,0x2172,0x2173,
	INVALC,0x2231,
	/* Range 0x2460 - 0x246E, array index: 0x05EC */
	0x2867,0x2868,0x2869,0x286A,0x286B,0x286C,0x286D,0x286E,
	0x286F,0x2870,0x2871,0x2872,0x2873,0x2874,0x2875,
	/* Range 0x2474 - 0x2482, array index: 0x05FB */
	0x2967,0x2968,0x2969,0x296A,0x296B,0x296C,0x296D,0x296E,
	0x296F,0x2970,0x2971,0x2972,0x2973,0x2974,0x2975,
	/* Range 0x249C - 0x24B5, array index: 0x060A */
	0x294D,0x294E,0x294F,0x2950,0x2951,0x2952,0x2953,0x2954,
	0x2955,0x2956,0x2957,0x2958,0x2959,0x295A,0x295B,0x295C,
	0x295D,0x295E,0x295F,0x2960,0x2961,0x2962,0x2963,0x2964,
	0x2965,0x2966,
	/* Range 0x24D0 - 0x24E9, array index: 0x0624 */
	0x284D,0x284E,0x284F,0x2850,0x2851,0x2852,0x2853,0x2854,
	0x2855,0x2856,0x2857,0x2858,0x2859,0x285A,0x285B,0x285C,
	0x285D,0x285E,0x285F,0x2860,0x2861,0x2862,0x2863,0x2864,
	0x2865,0x2866,
	/* Range 0x2500 - 0x2503, array index: 0x063E */
	0x2621,0x262C,0x2622,0x262D,
	/* Range 0x250C - 0x254B, array index: 0x0642 */
	0x2623,0x2648,0x2647,0x262E,0x2624,0x2642,0x2641,0x262F,
	0x2626,0x2646,0x2645,0x2631,0x2625,0x2644,0x2643,0x2630,
	0x2627,0x263C,0x2649,0x264A,0x2637,0x264B,0x264C,0x2632,
	0x2629,0x263E,0x264D,0x264E,0x2639,0x264F,0x2650,0x2634,
	0x2628,0x2651,0x2652,0x2638,0x263D,0x2653,0x2654,0x2633,
	0x262A,0x2655,0x2656,0x263A,0x263F,0x2657,0x2658,0x2635,
	0x262B,0x2659,0x265A,0x263B,0x265B,0x265C,0x2640,0x265D,
	0x265E,0x265F,0x2660,0x2661,0x2662,0x2663,0x2664,0x2636,
	/* Range 0x25A0 - 0x25A9, array index: 0x0682 */
	0x2161,0x2160,INVALC,0x2243,0x2247,0x2248,0x224B,0x224A,
	0x2249,0x224C,
	/* Range 0x25C6 - 0x25C8, array index: 0x068C */
	0x215F,0x215E,0x2242,
	/* Range 0x25CE - 0x25D1, array index: 0x068F */
	0x215D,0x215C,0x2244,0x2245,
	/* Range 0x2660 - 0x266D, array index: 0x0693 */
	0x223C,0x223D,INVALC,0x2240,0x223B,0x223E,INVALC,0x223F,
	0x224D,0x225B,0x225C,INVALC,0x225D,0x225A,
	/* Range 0x3000 - 0x3003, array index: 0x06A1 */
	0x2121,0x2122,0x2123,0x2128,
	/* Range 0x3008 - 0x3015, array index: 0x06A5 */
	0x2134,0x2135,0x2136,0x2137,0x2138,0x2139,0x213A,0x213B,
	0x213C,0x213D,INVALC,0x216B,0x2132,0x2133,
	/* Range 0x3041 - 0x3093, array index: 0x06B3 */
	0x2A21,0x2A22,0x2A23,0x2A24,0x2A25,0x2A26,0x2A27,0x2A28,
	0x2A29,0x2A2A,0x2A2B,0x2A2C,0x2A2D,0x2A2E,0x2A2F,0x2A30,
	0x2A31,0x2A32,0x2A33,0x2A34,0x2A35,0x2A36,0x2A37,0x2A38,
	0x2A39,0x2A3A,0x2A3B,0x2A3C,0x2A3D,0x2A3E,0x2A3F,0x2A40,
	0x2A41,0x2A42,0x2A43,0x2A44,0x2A45,0x2A46,0x2A47,0x2A48,
	0x2A49,0x2A4A,0x2A4B,0x2A4C,0x2A4D,0x2A4E,0x2A4F,0x2A50,
	0x2A51,0x2A52,0x2A53,0x2A54,0x2A55,0x2A56,0x2A57,0x2A58,
	0x2A59,0x2A5A,0x2A5B,0x2A5C,0x2A5D,0x2A5E,0x2A5F,0x2A60,
	0x2A61,0x2A62,0x2A63,0x2A64,0x2A65,0x2A66,0x2A67,0x2A68,
	0x2A69,0x2A6A,0x2A6B,0x2A6C,0x2A6D,0x2A6E,0x2A6F,0x2A70,
	0x2A71,0x2A72,0x2A73,
	/* Range 0x30A1 - 0x30F6, array index: 0x0706 */
	0x2B21,0x2B22,0x2B23,0x2B24,0x2B25,0x2B26,0x2B27,0x2B28,
	0x2B29,0x2B2A,0x2B2B,0x2B2C,0x2B2D,0x2B2E,0x2B2F,0x2B30,
	0x2B31,0x2B32,0x2B33,0x2B34,0x2B35,0x2B36,0x2B37,0x2B38,
	0x2B39,0x2B3A,0x2B3B,0x2B3C,0x2B3D,0x2B3E,0x2B3F,0x2B40,
	0x2B41,0x2B42,0x2B43,0x2B44,0x2B45,0x2B46,0x2B47,0x2B48,
	0x2B49,0x2B4A,0x2B4B,0x2B4C,0x2B4D,0x2B4E,0x2B4F,0x2B50,
	0x2B51,0x2B52,0x2B53,0x2B54,0x2B55,0x2B56,0x2B57,0x2B58,
	0x2B59,0x2B5A,0x2B5B,0x2B5C,0x2B5D,0x2B5E,0x2B5F,0x2B60,
	0x2B61,0x2B62,0x2B63,0x2B64,0x2B65,0x2B66,0x2B67,0x2B68,
	0x2B69,0x2B6A,0x2B6B,0x2B6C,0x2B6D,0x2B6E,0x2B6F,0x2B70,
	0x2B71,0x2B72,0x2B73,0x2B74,0x2B75,0x2B76,
	/* Range 0x3131 - 0x318E, array index: 0x075C */
	0x2421,0x2422,0x2423,0x2424,0x2425,0x2426,0x2427,0x2428,
	0x2429,0x242A,0x242B,0x242C,0x242D,0x242E,0x242F,0x2430,
	0x2431,0x2432,0x2433,0x2434,0x2435,0x2436,0x2437,0x2438,
	0x2439,0x243A,0x243B,0x243C,0x243D,0x243E,0x243F,0x2440,
	0x2441,0x2442,0x2443,0x2444,0x2445,0x2446,0x2447,0x2448,
	0x2449,0x244A,0x244B,0x244C,0x244D,0x244E,0x244F,0x2450,
	0x2451,0x2452,0x2453,0x2454,0x2455,0x2456,0x2457,0x2458,
	0x2459,0x245A,0x245B,0x245C,0x245D,0x245E,0x245F,0x2460,
	0x2461,0x2462,0x2463,0x2464,0x2465,0x2466,0x2467,0x2468,
	0x2469,0x246A,0x246B,0x246C,0x246D,0x246E,0x246F,0x2470,
	0x2471,0x2472,0x2473,0x2474,0x2475,0x2476,0x2477,0x2478,
	0x2479,0x247A,0x247B,0x247C,0x247D,0x247E,
	/* Range 0x3200 - 0x321C, array index: 0x07BA */
	0x2931,0x2932,0x2933,0x2934,0x2935,0x2936,0x2937,0x2938,
	0x2939,0x293A,0x293B,0x293C,0x293D,0x293E,0x293F,0x2940,
	0x2941,0x2942,0x2943,0x2944,0x2945,0x2946,0x2947,0x2948,
	0x2949,0x294A,0x294B,0x294C,0x225F,
	/* Range 0x3260 - 0x327B, array index: 0x07D7 */
	0x2831,0x2832,0x2833,0x2834,0x2835,0x2836,0x2837,0x2838,
	0x2839,0x283A,0x283B,0x283C,0x283D,0x283E,0x283F,0x2840,
	0x2841,0x2842,0x2843,0x2844,0x2845,0x2846,0x2847,0x2848,
	0x2849,0x284A,0x284B,0x284C,
	/* Range 0x3380 - 0x33CA, array index: 0x07F3 */
	0x2749,0x274A,0x274B,0x274C,0x274D,INVALC,INVALC,INVALC,
	0x273A,0x273B,0x275C,0x275D,0x275E,0x2736,0x2737,0x2738,
	0x2754,0x2755,0x2756,0x2757,0x2758,0x2721,0x2722,0x2723,
	0x2725,0x272B,0x272C,0x272D,0x272E,0x272F,0x2730,0x2731,
	0x2732,0x2733,0x2734,0x2727,0x2728,0x2729,0x272A,0x273D,
	0x273E,0x2765,0x2766,0x2767,0x2768,0x2761,0x2762,0x2763,
	0x273F,0x2740,0x2741,0x2742,0x2743,0x2744,0x2745,0x2746,
	0x2747,0x2748,0x274E,0x274F,0x2750,0x2751,0x2752,0x2753,
	0x275A,0x275B,0x2263,0x276C,0x2726,0x2760,0x276F,0x2261,
	0x273C,0x276D,0x2735,
	/* Range 0x33DB - 0x33DD, array index: 0x083E */
	0x2764,0x276E,0x2769,
	/* Range 0x4E07 - 0x4E0D, array index: 0x0841 */
	0x5832,0x6D5B,0x5F32,0x5F3E,0x793B,INVALC,0x5C74,
	/* Range 0x4E14 - 0x4E19, array index: 0x0848 */
	0x7326,0x5D60,0x6126,INVALC,0x4E78,0x5C30,
	/* Range 0x4E4B - 0x4E4F, array index: 0x084E */
	0x717D,INVALC,0x5E3F,0x7B3A,0x7939,
	/* Range 0x4E5D - 0x4E5F, array index: 0x0853 */
	0x4E7A,0x4B77,0x6525,
	/* Range 0x4E8E - 0x4E95, array index: 0x0856 */
	0x694D,INVALC,0x6A27,0x6976,0x7B3B,INVALC,0x6769,0x6F4C,
	/* Range 0x4EA1 - 0x4EAE, array index: 0x085E */
	0x584C,0x7971,INVALC,0x4E5F,0x7A24,0x6632,INVALC,0x7A7B,
	INVALC,INVALC,0x7A3D,0x4C48,0x6F4D,0x5555,
	/* Range 0x4ED4 - 0x4ED9, array index: 0x086C */
	0x6D28,0x5E42,0x7662,0x6D5C,0x5C75,0x6039,
	/* Range 0x4EE3 - 0x4EE5, array index: 0x0872 */
	0x535B,0x5635,0x6C24,
	/* Range 0x4F09 - 0x4F11, array index: 0x0875 */
	0x7972,0x6C25,0x505F,INVALC,0x676A,0x506B,0x5C51,0x5B69,
	0x7D4C,
	/* Range 0x4F46 - 0x4F48, array index: 0x087E */
	0x5323,0x6E37,0x784F,
	/* Range 0x4F4D - 0x4F51, array index: 0x0881 */
	0x6A48,0x6E38,0x712C,0x7125,0x694E,
	/* Range 0x4F59 - 0x4F5C, array index: 0x0886 */
	0x6579,0x6C6A,0x5D56,0x6D42,
	/* Range 0x4FC2 - 0x4FC4, array index: 0x088A */
	0x4C75,0x7535,0x642D,
	/* Range 0x4FDD - 0x4FE1, array index: 0x088D */
	0x5C41,INVALC,0x5E46,0x7A6F,0x6361,
	/* Range 0x5023 - 0x502D, array index: 0x0892 */
	0x5B27,0x7637,INVALC,0x4F66,0x7072,0x4B5A,INVALC,0x6752,
	0x5743,0x7670,0x685E,
	/* Range 0x5047 - 0x5049, array index: 0x089D */
	0x4A23,0x4C27,0x6A49,
	/* Range 0x5074 - 0x5078, array index: 0x08A0 */
	0x7630,0x6F4F,0x694F,INVALC,0x775E,
	/* Range 0x50FF - 0x5101, array index: 0x08A5 */
	0x5E47,0x6B70,0x7156,
	/* Range 0x5140 - 0x514E, array index: 0x08A8 */
	0x6834,0x6B43,INVALC,0x6A2A,0x7A7C,0x7576,0x703C,0x7D54,
	0x603B,0x4E43,INVALC,0x503A,0x773A,0x5873,0x774D,
	/* Range 0x5165 - 0x516E, array index: 0x08B7 */
	0x6C7D,INVALC,0x522E,0x6E6F,0x5557,0x6A64,0x7822,0x4D6B,
	0x573F,0x7B31,
	/* Range 0x5175 - 0x5178, array index: 0x08C1 */
	0x5C32,0x506C,0x4E7D,0x6E70,
	/* Range 0x51C9 - 0x51CD, array index: 0x08C5 */
	0x5558,INVALC,0x703D,0x5750,0x5450,
	/* Range 0x51DC - 0x51DE, array index: 0x08CA */
	0x574F,0x6B6A,0x7D6B,
	/* Range 0x51F6 - 0x51FA, array index: 0x08CD */
	0x7D55,INVALC,0x7448,0x686A,0x7573,
	/* Range 0x5206 - 0x520A, array index: 0x08D2 */
	0x5D42,0x6F37,0x6754,INVALC,0x4A4A,
	/* Range 0x5236 - 0x523B, array index: 0x08D7 */
	0x7024,0x616C,0x4F67,0x734B,0x6D29,0x4A3E,
	/* Range 0x524A - 0x524D, array index: 0x08DD */
	0x5E7B,0x503B,0x5537,0x6E71,
	/* Range 0x5287 - 0x5289, array index: 0x08E1 */
	0x503C,0x5B79,0x5731,
	/* Range 0x52A9 - 0x52AB, array index: 0x08E4 */
	0x703E,0x523D,0x4C24,
	/* Range 0x52D5 - 0x52E4, array index: 0x08E7 */
	0x5451,0x696D,INVALC,0x4A6B,0x5962,INVALC,0x7D32,INVALC,
	0x632D,0x564C,0x5934,INVALC,INVALC,0x6127,0x6E53,0x5043,
	/* Range 0x5315 - 0x5319, array index: 0x08F7 */
	0x5D62,0x7B79,0x5D41,INVALC,0x6335,
	/* Range 0x533F - 0x534A, array index: 0x08FC */
	0x527B,0x4F21,0x6428,INVALC,0x7436,0x6C7E,INVALC,INVALC,
	0x632E,0x676D,0x7D41,0x5A62,
	/* Range 0x5351 - 0x5354, array index: 0x0908 */
	0x5D64,0x706F,0x7671,0x7A70,
	/* Range 0x536F - 0x5377, array index: 0x090C */
	0x5956,0x6C54,0x6A4B,INVALC,INVALC,0x4A3F,0x5530,INVALC,
	0x4F69,
	/* Range 0x53C8 - 0x53CD, array index: 0x0915 */
	0x6951,0x7329,0x5060,0x6952,INVALC,0x5A63,
	/* Range 0x53E1 - 0x53F3, array index: 0x091B */
	0x6755,0x753F,0x4F22,0x4D2F,0x4F23,INVALC,INVALC,INVALC,
	0x4D30,0x717E,0x5023,0x612F,0x7823,INVALC,0x4A26,0x773B,
	0x726A,0x5E48,0x6953,
	/* Range 0x5408 - 0x5411, array index: 0x092E */
	0x796A,0x514E,0x6E54,INVALC,0x5452,0x5923,0x7D28,0x5759,
	0x774E,0x7A3E,
	/* Range 0x5438 - 0x543E, array index: 0x0938 */
	0x7D65,0x7623,INVALC,0x597C,0x7D29,INVALC,0x676E,
	/* Range 0x547B - 0x547D, array index: 0x093F */
	0x6362,0x7B3C,0x5924,
	/* Range 0x5586 - 0x558A, array index: 0x0942 */
	0x744A,0x547A,INVALC,0x7D2A,0x7962,
	/* Range 0x5598 - 0x559D, array index: 0x0947 */
	0x7437,0x7D42,0x7C30,INVALC,0x7D6C,0x4A62,
	/* Range 0x55A7 - 0x55AE, array index: 0x094D */
	0x7D3D,INVALC,0x6A67,0x5F43,0x5152,0x4E62,INVALC,0x5324,
	/* Range 0x574D - 0x5751, array index: 0x0955 */
	0x533B,0x4A6C,INVALC,0x7126,0x4B55,
	/* Range 0x5805 - 0x580A, array index: 0x095A */
	0x4C31,0x7758,INVALC,0x4B28,0x6B3C,0x643E,
	/* Range 0x582F - 0x5835, array index: 0x0960 */
	0x686B,0x6568,0x5C43,INVALC,INVALC,0x6D5E,0x5372,
	/* Range 0x58F9 - 0x58FD, array index: 0x0967 */
	0x6C6C,0x7B3E,0x5F6B,INVALC,0x6178,
	/* Range 0x5914 - 0x591C, array index: 0x096C */
	0x5073,0x602A,0x6862,INVALC,INVALC,0x6254,0x527D,INVALC,
	0x6528,
	/* Range 0x5927 - 0x592E, array index: 0x0975 */
	0x535E,INVALC,0x7438,0x773C,0x5C7D,INVALC,0x686C,0x6467,
	/* Range 0x5947 - 0x5949, array index: 0x097D */
	0x5074,0x522F,0x5C65,
	/* Range 0x594E - 0x5957, array index: 0x0980 */
	0x5025,0x7134,0x7C31,0x4C78,INVALC,INVALC,0x5D46,0x7A51,
	INVALC,0x775F,
	/* Range 0x596A - 0x596E, array index: 0x098A */
	0x772C,0x6B44,0x6D61,0x602B,0x5D47,
	/* Range 0x5982 - 0x5984, array index: 0x098F */
	0x657D,0x5D65,0x584D,
	/* Range 0x59D0 - 0x59D4, array index: 0x0992 */
	0x6E3B,0x4D34,INVALC,0x6073,0x6A4D,
	/* Range 0x59D9 - 0x59DD, array index: 0x0997 */
	0x6C75,0x686E,INVALC,0x4B29,0x712F,
	/* Range 0x5B54 - 0x5B5F, array index: 0x099C */
	0x4D6E,0x6D26,INVALC,0x6D2E,0x706D,INVALC,0x5D21,INVALC,
	0x6D2F,0x7C78,INVALC,0x586B,
	/* Range 0x5B85 - 0x5B8C, array index: 0x09A8 */
	0x536B,INVALC,0x6954,0x617A,0x644C,INVALC,0x6164,0x6847,
	/* Range 0x5B93 - 0x5B9C, array index: 0x09B0 */
	0x5C55,INVALC,0x7735,0x7C73,0x7073,0x4E2F,0x7135,0x6F52,
	0x6848,0x6B71,
	/* Range 0x5BA2 - 0x5BA6, array index: 0x09BA */
	0x4B54,0x603E,0x6378,0x6A69,0x7C32,
	/* Range 0x5BB3 - 0x5BB9, array index: 0x09BF */
	0x7A2A,0x6643,0x6132,0x4A2B,INVALC,0x6364,0x693B,
	/* Range 0x5BBF - 0x5BC7, array index: 0x09C6 */
	0x6256,0x7372,INVALC,0x6E56,0x6A32,0x5076,0x6C59,0x5A4B,
	0x4F28,
	/* Range 0x5BD0 - 0x5BD4, array index: 0x09CF */
	0x585B,INVALC,0x794E,0x6955,0x6351,
	/* Range 0x5BDE - 0x5BEF, array index: 0x09D4 */
	0x582C,0x734C,INVALC,0x4D7B,0x7656,INVALC,0x6775,0x686F,
	0x6379,0x523B,0x7373,0x637B,INVALC,0x5E50,0x4E30,INVALC,
	0x5677,0x7159,
	/* Range 0x5C07 - 0x5C11, array index: 0x09E6 */
	0x6D62,0x6E76,0x6A4F,0x706E,0x637C,INVALC,0x535F,0x5374,
	0x6133,INVALC,0x6134,
	/* Range 0x5C38 - 0x5C40, array index: 0x09F1 */
	0x6339,0x6B45,0x7429,0x4D36,0x5279,INVALC,0x5A2D,0x5263,
	0x4F51,
	/* Range 0x5DE1 - 0x5DE8, array index: 0x09FA */
	0x625E,0x6135,INVALC,INVALC,0x4D6F,0x7127,0x4E65,0x4B5D,
	/* Range 0x5DF1 - 0x5DF4, array index: 0x0A02 */
	0x5079,0x6C2B,0x5E53,0x7769,
	/* Range 0x5E5F - 0x5E63, array index: 0x0A06 */
	0x763A,INVALC,0x5B5F,0x5353,0x7847,
	/* Range 0x5E72 - 0x5E7E, array index: 0x0A0B */
	0x4A4E,0x7841,0x5234,INVALC,INVALC,0x5C34,0x7A39,0x4A4F,
	INVALC,0x7C33,0x6A6A,0x6A6B,0x507A,
	/* Range 0x5EB5 - 0x5EB8, array index: 0x0A18 */
	0x645D,0x5F6E,0x4B2C,0x693C,
	/* Range 0x5EC8 - 0x5ECA, array index: 0x0A1C */
	0x793F,0x562F,0x5546,
	/* Range 0x5EDF - 0x5EE3, array index: 0x0A1F */
	0x5959,0x735F,INVALC,0x7848,0x4E46,
	/* Range 0x5F11 - 0x5F18, array index: 0x0A24 */
	0x633D,INVALC,0x4F61,0x7040,0x6C5A,INVALC,0x5D57,0x7B70,
	/* Range 0x5F66 - 0x5F71, array index: 0x0A2C */
	0x6569,0x696E,INVALC,0x7374,0x7873,0x7041,0x5E2F,0x7830,
	INVALC,INVALC,0x7360,0x672F,
	/* Range 0x5F7F - 0x5F81, array index: 0x0A38 */
	0x5D58,0x6859,0x6F56,
	/* Range 0x5F8A - 0x5F92, array index: 0x0A3B */
	0x7C60,0x5748,0x7D2D,INVALC,INVALC,INVALC,0x5F6F,0x4C53,
	0x5379,
	/* Range 0x5F97 - 0x5F99, array index: 0x0A44 */
	0x5470,0x5B47,0x5E55,
	/* Range 0x5FA8 - 0x5FAA, array index: 0x0A47 */
	0x7C47,0x5C56,0x6260,
	/* Range 0x5FD6 - 0x5FD9, array index: 0x0A4A */
	0x753C,0x7224,0x584E,0x584F,
	/* Range 0x6041 - 0x6043, array index: 0x0A4E */
	0x6C76,0x6261,0x633E,
	/* Range 0x6062 - 0x6070, array index: 0x0A51 */
	0x7C61,0x6D30,0x7D51,0x763B,INVALC,INVALC,0x794F,0x6B5A,
	0x4A41,INVALC,0x5238,0x4D71,INVALC,0x6353,0x7D66,
	/* Range 0x60B0 - 0x60B8, array index: 0x0A60 */
	0x7075,INVALC,0x5D68,0x536D,0x757C,INVALC,0x5A3F,INVALC,
	0x4C7B,
	/* Range 0x60DF - 0x60E1, array index: 0x0A69 */
	0x6A6E,0x7B33,0x6442,
	/* Range 0x60F9 - 0x60FB, array index: 0x0A6C */
	0x6529,0x6076,0x7633,
	/* Range 0x610D - 0x610F, array index: 0x0A6F */
	0x5A40,0x7834,0x6B72,
	/* Range 0x6147 - 0x614C, array index: 0x0A72 */
	0x6B5B,0x6D31,INVALC,0x4C43,0x773E,0x7C4B,
	/* Range 0x6162 - 0x6168, array index: 0x0A78 */
	0x5837,0x4E31,0x4A42,INVALC,INVALC,0x7B34,0x4B46,
	/* Range 0x6198 - 0x619A, array index: 0x0A7F */
	0x7D73,0x7D72,0x7726,
	/* Range 0x61C7 - 0x61CB, array index: 0x0A82 */
	0x4A50,0x7A2B,0x6B6B,0x6778,0x5965,
	/* Range 0x61F6 - 0x6200, array index: 0x0A87 */
	0x547B,0x7C63,0x7A58,INVALC,0x7355,INVALC,0x4F2B,INVALC,
	INVALC,0x6B73,0x557C,
	/* Range 0x620A - 0x6212, array index: 0x0A92 */
	0x5966,INVALC,0x6279,0x6221,0x6B54,INVALC,0x6077,0x6432,
	0x4C7C,
	/* Range 0x623E - 0x6241, array index: 0x0A9B */
	0x5568,0x5B2E,0x6136,0x7837,
	/* Range 0x6247 - 0x624D, array index: 0x0A9F */
	0x603F,0x7B43,0x5D6A,INVALC,0x6222,INVALC,0x6E26,
	/* Range 0x62C7 - 0x62DC, array index: 0x0AA6 */
	0x5967,0x5239,0x5543,INVALC,INVALC,0x5A65,0x5A50,INVALC,
	0x5159,0x4E58,INVALC,0x4B5E,0x742C,0x5A7B,INVALC,0x7669,
	0x6873,0x4F2C,0x7070,INVALC,0x747D,0x5B48,
	/* Range 0x62EC - 0x62F3, array index: 0x0ABC */
	0x4E40,0x6354,0x514F,0x7175,INVALC,0x4D72,INVALC,0x4F6B,
	/* Range 0x633A - 0x633E, array index: 0x0AC4 */
	0x6F58,0x6649,INVALC,0x5838,0x7A73,
	/* Range 0x63A0 - 0x63A2, array index: 0x0AC9 */
	0x5553,0x7375,0x772E,
	/* Range 0x63A5 - 0x63AA, array index: 0x0ACC */
	0x6F48,INVALC,0x4D73,0x754F,0x6573,0x7042,
	/* Range 0x6478 - 0x647A, array index: 0x0AD2 */
	0x5938,0x5939,0x6F49,
	/* Range 0x6491 - 0x6493, array index: 0x0AD5 */
	0x774B,0x5F2E,0x6875,
	/* Range 0x64E1 - 0x64E7, array index: 0x0AD8 */
	0x5364,0x7677,INVALC,INVALC,0x553A,0x734D,0x4B61,
	/* Range 0x652A - 0x652C, array index: 0x0ADF */
	0x4E66,0x7C2B,0x553B,
	/* Range 0x6536 - 0x653F, array index: 0x0AE2 */
	0x6225,0x4D39,0x6A72,0x4B47,INVALC,0x4D74,INVALC,INVALC,
	0x5B2F,0x6F59,
	/* Range 0x654D - 0x6551, array index: 0x0AEC */
	0x5F73,0x4E67,0x5A42,INVALC,0x4F2D,
	/* Range 0x6574 - 0x6578, array index: 0x0AF1 */
	0x6F5A,0x6E58,INVALC,0x5D27,0x6226,
	/* Range 0x65E5 - 0x65ED, array index: 0x0AF6 */
	0x6C6D,0x5329,INVALC,0x7229,0x7044,INVALC,INVALC,0x6262,
	0x696F,
	/* Range 0x65FA - 0x65FF, array index: 0x0AFF */
	0x685A,0x5A43,0x5A44,0x5445,INVALC,0x677A,
	/* Range 0x6606 - 0x6615, array index: 0x0B05 */
	0x4D60,0x6330,INVALC,0x5B32,0x7B44,INVALC,0x7363,INVALC,
	0x5925,0x7B67,0x5D4B,0x5054,INVALC,0x6636,0x602E,0x7D5A,
	/* Range 0x661E - 0x6620, array index: 0x0B15 */
	0x5C35,0x6078,0x6731,
	/* Range 0x662D - 0x6631, array index: 0x0B18 */
	0x6139,INVALC,0x6340,0x7940,0x6970,
	/* Range 0x6641 - 0x6644, array index: 0x0B1D */
	0x7045,0x6341,0x7C4C,0x7C4D,
	/* Range 0x665B - 0x665F, array index: 0x0B21 */
	0x7A59,INVALC,0x7138,0x7D75,0x6079,
	/* Range 0x6664 - 0x666F, array index: 0x0B26 */
	0x677B,0x7C37,0x7C64,0x7B45,0x6367,0x5839,INVALC,0x7678,
	INVALC,INVALC,0x5C45,0x4C58,
	/* Range 0x6673 - 0x667A, array index: 0x0B32 */
	0x602F,0x7467,INVALC,0x6F5C,0x4F7C,0x6F5D,INVALC,0x722A,
	/* Range 0x6687 - 0x6689, array index: 0x0B3A */
	0x4A2C,0x7D3B,0x7D47,
	/* Range 0x6696 - 0x6698, array index: 0x0B3D */
	0x516C,0x645E,0x6543,
	/* Range 0x66B2 - 0x66B4, array index: 0x0B40 */
	0x6D67,0x7B35,0x786C,
	/* Range 0x66F0 - 0x6700, array index: 0x0B43 */
	0x6858,INVALC,0x4D58,0x6756,0x4C5A,INVALC,INVALC,0x4A63,
	0x5F76,0x7047,0x7046,INVALC,0x583A,INVALC,0x7174,0x7470,
	0x754C,
	/* Range 0x671B - 0x671F, array index: 0x0B54 */
	0x5850,INVALC,0x7048,0x5121,0x5122,
	/* Range 0x6726 - 0x672E, array index: 0x0B59 */
	0x5954,0x5668,0x594A,INVALC,0x5A31,0x5847,0x5C62,0x734E,
	0x7574,
	/* Range 0x674E - 0x6753, array index: 0x0B62 */
	0x575D,0x7A3A,0x6E27,0x753D,INVALC,0x7875,
	/* Range 0x676D - 0x6777, array index: 0x0B68 */
	0x7978,INVALC,0x5B4A,0x4B79,0x5454,INVALC,0x595C,INVALC,
	0x6E3E,INVALC,0x776D,
	/* Range 0x67B6 - 0x67B8, array index: 0x0B73 */
	0x4A2D,0x4A2E,0x4F2E,
	/* Range 0x67CF - 0x67D4, array index: 0x0B76 */
	0x5B5A,0x593B,0x4A73,0x7653,0x6678,0x6A75,
	/* Range 0x67EF - 0x67F6, array index: 0x0B7C */
	0x4A2F,0x5230,0x713A,INVALC,0x5733,0x6343,0x737D,0x5E5A,
	/* Range 0x683C - 0x6843, array index: 0x0B84 */
	0x4C2B,0x6E28,INVALC,INVALC,0x4B7A,0x7979,0x4C7D,0x537E,
	/* Range 0x6850 - 0x6854, array index: 0x0B8C */
	0x5455,0x5F4D,INVALC,0x7C38,0x5150,
	/* Range 0x68AD - 0x68B6, array index: 0x0B91 */
	0x5E5C,INVALC,0x702C,0x4C7E,0x4D61,INVALC,0x613A,INVALC,
	0x5B6F,0x5A32,
	/* Range 0x6953 - 0x695A, array index: 0x0B9B */
	0x7923,0x605B,0x766B,INVALC,0x4B71,INVALC,0x596A,0x7522,
	/* Range 0x695E - 0x6963, array index: 0x0BA3 */
	0x5751,INVALC,0x5178,0x6A78,0x6A79,0x5A33,
	/* Range 0x696B - 0x696F, array index: 0x0BA9 */
	0x716F,INVALC,0x6576,0x6E3F,0x6264,
	/* Range 0x6975 - 0x6979, array index: 0x0BAE */
	0x503F,INVALC,0x7A2C,0x7551,0x6733,
	/* Range 0x69CB - 0x69CD, array index: 0x0BB3 */
	0x4F30,0x7759,0x7366,
	/* Range 0x6A38 - 0x6A3A, array index: 0x0BB6 */
	0x5A54,0x6227,0x7B7C,
	/* Range 0x6B5F - 0x6B66, array index: 0x0BB9 */
	0x6622,INVALC,0x7C36,0x722D,0x6F61,0x732E,0x5C46,0x596B,
	/* Range 0x6BD2 - 0x6BD8, array index: 0x0BC1 */
	0x5438,0x6B3E,0x5D6F,INVALC,0x5D70,0x5D71,0x5D72,
	/* Range 0x6C3E - 0x6C42, array index: 0x0BC8 */
	0x5B70,INVALC,0x6F62,0x7170,0x4F34,
	/* Range 0x6C5D - 0x6C60, array index: 0x0BCD */
	0x6623,0x7B71,0x4B30,0x722E,
	/* Range 0x6C7D - 0x6C88, array index: 0x0BD1 */
	0x5129,0x5D4C,INVALC,INVALC,0x637E,0x512A,0x682A,INVALC,
	0x6A36,0x797A,0x664C,0x7658,
	/* Range 0x6C90 - 0x6C9B, array index: 0x0BDD */
	0x594B,INVALC,0x5952,0x534B,0x5877,0x5A29,0x7578,INVALC,
	INVALC,0x5E5E,0x722F,0x7829,
	/* Range 0x6CB8 - 0x6CC4, array index: 0x0BE9 */
	0x5D73,0x6A7A,INVALC,0x763D,0x613B,0x4D3F,0x7454,0x664D,
	INVALC,0x7C4F,0x7B22,INVALC,0x605C,
	/* Range 0x6CE1 - 0x6CE5, array index: 0x0BF6 */
	0x785C,0x776E,0x6B68,INVALC,0x527A,
	/* Range 0x6CEE - 0x6CF0, array index: 0x0BFB */
	0x5A6A,0x5A46,0x7741,
	/* Range 0x6D35 - 0x6D3E, array index: 0x0BFE */
	0x6267,0x7D57,INVALC,0x4E48,0x6A37,INVALC,0x7C40,INVALC,
	0x7D67,0x776F,
	/* Range 0x6D77 - 0x6D79, array index: 0x0C08 */
	0x7A2D,0x7659,0x7A74,
	/* Range 0x6D85 - 0x6D8E, array index: 0x0C0B */
	0x666E,INVALC,0x4C5C,0x613C,0x606F,INVALC,INVALC,0x693F,
	0x7C7D,0x664E,
	/* Range 0x6DC3 - 0x6DC7, array index: 0x0C15 */
	0x4F6E,0x763E,0x6032,0x7C7E,0x512B,
	/* Range 0x6DD8 - 0x6DDA, array index: 0x0C1A */
	0x5423,0x7078,0x5728,
	/* Range 0x6DF5 - 0x6DFB, array index: 0x0C1D */
	0x6650,INVALC,0x7B68,0x7468,0x6574,0x743C,0x7455,
	/* Range 0x6E17 - 0x6E26, array index: 0x0C24 */
	0x5F36,INVALC,0x7C39,0x6E42,0x4A75,INVALC,INVALC,INVALC,
	0x6F65,0x4B62,0x5424,INVALC,0x5E60,0x5A7D,0x6446,0x683E,
	/* Range 0x6E2B - 0x6E2F, array index: 0x0C34 */
	0x605E,0x7634,0x6A52,INVALC,0x797B,
	/* Range 0x6E3A - 0x6E3E, array index: 0x0C39 */
	0x595D,INVALC,0x5A34,0x6E2A,0x7B69,
	/* Range 0x6E5B - 0x6E5F, array index: 0x0C3E */
	0x5340,0x6357,INVALC,0x6F66,0x7C50,
	/* Range 0x6EFE - 0x6F02, array index: 0x0C43 */
	0x4D63,0x583B,INVALC,0x655B,0x7877,
	/* Range 0x6F57 - 0x6F5F, array index: 0x0C48 */
	0x727C,0x5A6B,INVALC,0x6258,0x6D56,INVALC,INVALC,0x5651,
	0x6033,
	/* Range 0x6F7A - 0x6F7E, array index: 0x0C51 */
	0x6D52,INVALC,0x5458,0x5C49,0x5771,
	/* Range 0x6FBE - 0x6FC3, array index: 0x0C56 */
	0x5336,INVALC,0x4C2D,0x767A,0x5632,0x5258,
	/* Range 0x6FDF - 0x6FE1, array index: 0x0C5C */
	0x702D,0x7B4C,0x6B21,
	/* Range 0x7018 - 0x701F, array index: 0x0C5F */
	0x5652,INVALC,0x7955,0x673A,0x6B55,0x5577,0x6F67,0x613E,
	/* Range 0x7098 - 0x709A, array index: 0x0C67 */
	0x7D5C,0x6D33,0x4E49,
	/* Range 0x70AB - 0x70AF, array index: 0x0C6A */
	0x7A5B,0x4B63,0x7729,INVALC,0x7B26,
	/* Range 0x70B7 - 0x70B9, array index: 0x0C6F */
	0x7140,0x6D48,0x6F43,
	/* Range 0x7164 - 0x7169, array index: 0x0C72 */
	0x5860,0x7C3A,0x7D2F,0x704E,INVALC,0x5B61,
	/* Range 0x71E5 - 0x71E7, array index: 0x0C78 */
	0x704F,0x733E,0x622C,
	/* Range 0x71FB - 0x7200, array index: 0x0C7B */
	0x7D38,0x6368,INVALC,0x5427,0x687C,0x7A52,
	/* Range 0x723A - 0x7240, array index: 0x0C81 */
	0x652C,0x7D21,INVALC,0x5F50,0x6C33,INVALC,0x5F51,
	/* Range 0x7246 - 0x7248, array index: 0x0C88 */
	0x6D6D,0x7838,0x777A,
	/* Range 0x731B - 0x731D, array index: 0x0C8B */
	0x586D,0x6344,0x7071,
	/* Range 0x7329 - 0x732B, array index: 0x0C8E */
	0x607A,0x6E44,0x595E,
	/* Range 0x7394 - 0x7398, array index: 0x0C91 */
	0x743D,INVALC,0x4F38,0x695B,0x512C,
	/* Range 0x73C9 - 0x73CF, array index: 0x0C96 */
	0x5A48,0x5F26,INVALC,0x7933,0x7252,INVALC,0x4A44,
	/* Range 0x73E3 - 0x73EA, array index: 0x0C9D */
	0x6269,0x5C4A,0x6C34,0x7A40,INVALC,INVALC,0x7B28,0x5028,
	/* Range 0x7403 - 0x7409, array index: 0x0CA5 */
	0x4F39,INVALC,0x554A,0x5762,0x622F,INVALC,0x5738,
	/* Range 0x7420 - 0x743A, array index: 0x0CAC */
	0x6F22,0x625A,0x767C,INVALC,INVALC,0x7B50,0x512D,INVALC,
	0x4D64,INVALC,0x512E,0x5C6D,0x684E,INVALC,0x7079,0x4E35,
	0x667C,INVALC,INVALC,0x577B,0x5056,0x5D75,0x7771,INVALC,
	0x767D,INVALC,0x5B77,
	/* Range 0x743F - 0x7444, array index: 0x0CC7 */
	0x7B6A,0x695C,0x5941,INVALC,0x7572,0x6045,
	/* Range 0x7457 - 0x746A, array index: 0x0CCD */
	0x6A3C,INVALC,0x5245,0x7B51,0x6740,0x6B25,INVALC,0x5F7A,
	0x6322,0x5739,INVALC,0x6943,INVALC,0x687D,0x682F,INVALC,
	INVALC,0x7253,0x7B29,0x5825,
	/* Range 0x7530 - 0x7533, array index: 0x0CE1 */
	0x6F23,0x6B26,0x4B23,0x6369,
	/* Range 0x7559 - 0x755D, array index: 0x0CE5 */
	0x573A,INVALC,0x7255,0x7565,0x596F,
	/* Range 0x75B1 - 0x75BE, array index: 0x0CEA */
	0x785E,0x792A,0x4A77,INVALC,0x6D37,INVALC,INVALC,0x5338,
	0x7256,INVALC,INVALC,0x5459,0x6E45,0x7270,
	/* Range 0x761F - 0x7626, array index: 0x0CF8 */
	0x6830,0x7430,0x736A,0x5A6E,INVALC,0x573B,INVALC,0x6231,
	/* Range 0x767B - 0x767E, array index: 0x0D00 */
	0x5474,0x5B21,0x5B5C,0x5B5D,
	/* Range 0x76DB - 0x76E4, array index: 0x0D04 */
	0x607C,0x5428,INVALC,0x6D53,0x586F,INVALC,0x7257,INVALC,
	0x4A78,0x5A6F,
	/* Range 0x77ED - 0x77EF, array index: 0x0D0E */
	0x532D,0x6861,0x4E6C,
	/* Range 0x7825 - 0x7827, array index: 0x0D11 */
	0x7232,0x7376,0x765B,
	/* Range 0x788C - 0x788E, array index: 0x0D14 */
	0x565E,0x6474,0x616F,
	/* Range 0x78BA - 0x78BC, array index: 0x0D17 */
	0x7C2C,0x7C2D,0x5827,
	/* Range 0x792A - 0x792C, array index: 0x0D1A */
	0x556C,0x5578,0x5A73,
	/* Range 0x7947 - 0x7949, array index: 0x0D1D */
	0x5136,0x5137,0x7233,
	/* Range 0x7956 - 0x7960, array index: 0x0D20 */
	0x7053,0x7234,INVALC,INVALC,0x7054,0x4B64,0x7B54,0x7566,
	0x636A,INVALC,0x5E66,
	/* Range 0x798D - 0x7991, array index: 0x0D2B */
	0x7C21,0x6F6E,0x5C58,INVALC,0x695F,
	/* Range 0x79BD - 0x79C1, array index: 0x0D30 */
	0x5058,0x7C22,0x543E,0x6233,0x5E67,
	/* Range 0x79C9 - 0x79CB, array index: 0x0D35 */
	0x5C3C,0x5236,0x7555,
	/* Range 0x7A3B - 0x7A40, array index: 0x0D38 */
	0x542B,0x4A33,0x4D26,INVALC,0x4D43,0x4D5A,
	/* Range 0x7B49 - 0x7B56, array index: 0x0D3E */
	0x5475,INVALC,0x5049,0x6F27,0x626C,INVALC,0x5B6A,0x4E4C,
	0x7568,0x7755,INVALC,0x534D,INVALC,0x737E,
	/* Range 0x7C3D - 0x7C3F, array index: 0x0D4C */
	0x7458,0x5633,0x5D2D,
	/* Range 0x7CB1 - 0x7CB3, array index: 0x0D4F */
	0x555C,0x7344,0x4B57,
	/* Range 0x7CDE - 0x7CE0, array index: 0x0D52 */
	0x5D50,0x7059,0x4B33,
	/* Range 0x7D02 - 0x7D0D, array index: 0x0D55 */
	0x7144,INVALC,0x6533,0x7B75,0x6961,0x7D60,0x7C3C,INVALC,
	0x5A22,0x5A23,INVALC,0x5221,
	/* Range 0x7D17 - 0x7D1B, array index: 0x0D61 */
	0x5E69,0x4E5C,0x7235,0x5064,0x5D51,
	/* Range 0x7D20 - 0x7D22, array index: 0x0D66 */
	0x6148,0x5B37,0x5F63,
	/* Range 0x7D2B - 0x7D30, array index: 0x0D69 */
	0x6D39,0x7145,INVALC,0x734F,0x572B,0x612C,
	/* Range 0x7D42 - 0x7D46, array index: 0x0D6F */
	0x707B,0x7A61,0x705A,0x4C67,0x5A74,
	/* Range 0x7D71 - 0x7D73, array index: 0x0D74 */
	0x7756,0x5E6A,0x4B34,
	/* Range 0x7DB1 - 0x7DB5, array index: 0x0D77 */
	0x4B35,0x5851,INVALC,0x744E,0x7377,
	/* Range 0x7DBA - 0x7DBF, array index: 0x0D7C */
	0x513B,0x772A,INVALC,0x6D4A,0x5753,0x587A,
	/* Range 0x7DDD - 0x7DE3, array index: 0x0D82 */
	0x727D,0x5330,INVALC,0x7473,0x5A49,INVALC,0x665E,
	/* Range 0x7E1B - 0x7E23, array index: 0x0D89 */
	0x5A5A,INVALC,0x725C,0x7B56,0x6932,INVALC,0x6E2D,INVALC,
	0x7A63,
	/* Range 0x7E69 - 0x7E6D, array index: 0x0D92 */
	0x6331,0x7C6B,0x4D28,INVALC,0x4C36,
	/* Range 0x7FDF - 0x7FE1, array index: 0x0D97 */
	0x6E61,0x7628,0x5D7D,
	/* Range 0x806F - 0x8073, array index: 0x0D9A */
	0x5624,0x7546,INVALC,0x6122,0x6946,
	/* Range 0x807D - 0x807F, array index: 0x0D9F */
	0x7469,0x566C,0x6B53,
	/* Range 0x8084 - 0x808C, array index: 0x0DA2 */
	0x6C3D,0x625C,0x5E6B,0x705C,INVALC,0x6B3F,INVALC,0x574E,
	0x513F,
	/* Range 0x8105 - 0x810A, array index: 0x0DAB */
	0x7A76,0x762A,0x7A77,0x5866,INVALC,0x7431,
	/* Range 0x8178 - 0x817A, array index: 0x0DB1 */
	0x6D73,0x5C59,0x604D,
	/* Range 0x81BD - 0x81C2, array index: 0x0DB4 */
	0x5345,0x7C6C,0x525B,0x546B,INVALC,0x5E22,
	/* Range 0x81FA - 0x81FE, array index: 0x0DBA */
	0x5366,0x725D,0x4F3F,INVALC,0x6B2C,
	/* Range 0x821B - 0x8221, array index: 0x0DBF */
	0x743F,0x626F,INVALC,0x5971,0x7147,INVALC,0x4B38,
	/* Range 0x822A - 0x822C, array index: 0x0DC6 */
	0x797E,0x5B3A,0x5A75,
	/* Range 0x8235 - 0x8239, array index: 0x0DC9 */
	0x766C,0x5A5C,0x7A64,INVALC,0x604F,
	/* Range 0x826E - 0x8272, array index: 0x0DCE */
	0x4A5D,0x555E,INVALC,0x4A5E,0x5F64,
	/* Range 0x82AC - 0x82BF, array index: 0x0DD3 */
	0x5D52,0x7774,0x675C,0x6425,INVALC,0x7C23,INVALC,0x5B3B,
	INVALC,INVALC,INVALC,0x723A,0x697D,0x504A,INVALC,0x7556,
	0x5945,0x6434,INVALC,0x6D27,
	/* Range 0x82D1 - 0x82D7, array index: 0x0DE7 */
	0x6A3D,0x667E,INVALC,0x7744,0x752D,INVALC,0x5960,
	/* Range 0x82E5 - 0x82E7, array index: 0x0DEE */
	0x6534,0x4D48,0x6E48,
	/* Range 0x82FD - 0x8305, array index: 0x0DF1 */
	0x4D49,0x7937,INVALC,INVALC,0x7168,0x5972,0x5B75,0x4A35,
	0x5946,
	/* Range 0x8334 - 0x8339, array index: 0x0DFA */
	0x7C6D,0x6C60,0x527E,INVALC,0x6947,0x662A,
	/* Range 0x83A9 - 0x83AB, array index: 0x0E00 */
	0x5D34,0x6435,0x5830,
	/* Range 0x83EF - 0x83F4, array index: 0x0E03 */
	0x7C24,0x4D4A,0x5754,0x5E23,INVALC,0x6460,
	/* Range 0x840A - 0x840E, array index: 0x0E09 */
	0x5551,INVALC,0x5870,0x7843,0x6A57,
	/* Range 0x8568 - 0x856A, array index: 0x0E0E */
	0x4F72,0x7739,0x5973,
	/* Range 0x85A6 - 0x85AA, array index: 0x0E11 */
	0x7440,INVALC,0x7D3C,0x5F31,0x636F,
	/* Range 0x85CD - 0x85CF, array index: 0x0E16 */
	0x5540,0x6370,0x6D7A,
	/* Range 0x865B - 0x865F, array index: 0x0E19 */
	0x7A48,0x5657,INVALC,0x6965,0x7B5C,
	/* Range 0x8861 - 0x8863, array index: 0x0E1E */
	0x7B2C,0x4F43,0x6B7D,
	/* Range 0x887E - 0x8882, array index: 0x0E21 */
	0x505A,0x505B,INVALC,0x6A3E,0x5931,
	/* Range 0x88F3 - 0x88F5, array index: 0x0E26 */
	0x5F57,0x5B50,0x5B51,
	/* Range 0x8A16 - 0x8A18, array index: 0x0E29 */
	0x7D61,0x767E,0x5140,
	/* Range 0x8A6D - 0x8A75, array index: 0x0E2C */
	0x4F78,0x6F2C,INVALC,0x7D7E,0x7C25,0x7A31,0x5F59,INVALC,
	0x6052,
	/* Range 0x8AA0 - 0x8AAA, array index: 0x0E35 */
	0x6124,0x4D2B,INVALC,0x5974,0x6826,0x4D4E,0x6169,INVALC,
	0x7C6F,INVALC,0x6063,
	/* Range 0x8AEA - 0x8AEE, array index: 0x0E40 */
	0x6F75,0x4A5F,INVALC,0x6B31,0x6D3F,
	/* Range 0x8AF6 - 0x8AFA, array index: 0x0E45 */
	0x6426,0x7924,0x7033,INVALC,0x656C,
	/* Range 0x8AFE - 0x8B04, array index: 0x0E4A */
	0x5167,INVALC,0x5947,0x6457,0x6A5D,INVALC,0x5477,
	/* Range 0x8B16 - 0x8B1D, array index: 0x0E51 */
	0x615A,0x5B3F,INVALC,0x4C45,0x6C50,0x4B3B,INVALC,0x5E73,
	/* Range 0x8C9D - 0x8CA2, array index: 0x0E59 */
	0x782F,0x6F76,INVALC,0x5D36,0x6E2F,0x4D78,
	/* Range 0x8CA7 - 0x8CCA, array index: 0x0E5F */
	0x5E38,0x7C27,0x777C,0x7731,0x4E3B,0x7421,INVALC,INVALC,
	0x6E4D,0x612E,INVALC,INVALC,0x6C43,0x4F7E,INVALC,0x783F,
	0x5862,0x5368,INVALC,INVALC,0x5E28,0x7464,0x6C42,INVALC,
	0x5975,0x7945,0x5D53,0x5671,0x6C7C,0x7C70,INVALC,INVALC,
	0x6D40,0x4A39,INVALC,0x6E64,
	/* Range 0x8CE0 - 0x8CE6, array index: 0x0E83 */
	0x5B53,INVALC,0x7A67,0x5863,0x7441,INVALC,0x5D37,
	/* Range 0x8CFB - 0x8CFD, array index: 0x0E8A */
	0x5D38,0x4F45,0x5F5F,
	/* Range 0x8D04 - 0x8D0A, array index: 0x0E8D */
	0x723E,0x7621,INVALC,0x6B4B,0x717C,INVALC,0x7347,
	/* Range 0x8E47 - 0x8E4A, array index: 0x0E94 */
	0x4B74,0x5430,0x7332,0x7B37,
	/* Range 0x8EAA - 0x8EAC, array index: 0x0E98 */
	0x5775,0x6373,0x4F65,
	/* Range 0x8ECA - 0x8ECD, array index: 0x0E9B */
	0x7333,0x6458,0x4F79,0x4F5A,
	/* Range 0x8F12 - 0x8F15, array index: 0x0E9F */
	0x7465,0x5842,0x5C50,0x4C6E,
	/* Range 0x8F1B - 0x8F1F, array index: 0x0EA3 */
	0x5560,0x764A,0x7D4A,0x5856,0x744F,
	/* Range 0x8F26 - 0x8F2A, array index: 0x0EA8 */
	0x5626,0x5C3E,INVALC,0x5B54,0x5747,
	/* Range 0x8FAD - 0x8FB2, array index: 0x0EAD */
	0x5E76,INVALC,0x5C2A,0x7263,0x6934,0x525C,
	/* Range 0x9000 - 0x9008, array index: 0x0EB3 */
	0x775C,0x616A,0x4E41,0x5431,INVALC,0x7D31,0x663D,INVALC,
	0x7B2D,
	/* Range 0x900D - 0x9023, array index: 0x0EBC */
	0x614E,INVALC,0x7762,0x756F,0x4F47,INVALC,INVALC,0x5432,
	0x4C6F,INVALC,0x5468,INVALC,0x6E4F,0x7757,INVALC,INVALC,
	0x6026,0x5641,0x615C,0x7063,0x7164,0x5C71,0x5627,
	/* Range 0x904A - 0x9055, array index: 0x0ED3 */
	0x6B34,0x6A21,INVALC,0x783C,0x4E26,INVALC,0x7946,0x7C5A,
	INVALC,0x5433,0x5339,0x6A5E,
	/* Range 0x905C - 0x9063, array index: 0x0EDF */
	0x6161,0x534F,0x7476,INVALC,0x6A40,0x614F,INVALC,0x4C3A,
	/* Range 0x906D - 0x906F, array index: 0x0EE7 */
	0x7064,0x7334,0x546E,
	/* Range 0x907C - 0x908A, array index: 0x0EEA */
	0x5721,0x4B68,INVALC,0x792D,0x692D,0x5864,0x7A33,0x6245,
	0x7C3D,INVALC,INVALC,0x6C44,0x5831,INVALC,0x5C2B,
	/* Range 0x90AF - 0x90B1, array index: 0x0EF9 */
	0x4A7B,0x7746,0x4F48,
	/* Range 0x9149 - 0x914E, array index: 0x0EFC */
	0x6B37,0x6F78,0x755C,0x6D4C,0x5B55,0x714F,
	/* Range 0x91AB - 0x91B5, array index: 0x0F02 */
	0x6C22,0x6D7D,INVALC,0x7534,0x7B38,INVALC,0x5B23,INVALC,
	INVALC,0x564A,0x4B59,
	/* Range 0x91C9 - 0x91D1, array index: 0x0F0D */
	0x6B38,INVALC,0x6037,0x576C,0x716C,0x652F,0x5561,0x576D,
	0x5151,
	/* Range 0x9210 - 0x9212, array index: 0x0F16 */
	0x4C22,0x777E,0x5F3C,
	/* Range 0x9262 - 0x9266, array index: 0x0F19 */
	0x5B24,INVALC,0x4F49,0x627C,0x6F7A,
	/* Range 0x931E - 0x9328, array index: 0x0F1E */
	0x6276,0x534A,0x6F7C,0x5144,0x6F31,INVALC,0x5145,INVALC,
	0x505E,INVALC,0x5961,
	/* Range 0x958F - 0x9594, array index: 0x0F29 */
	0x6B4E,INVALC,0x7958,0x7959,0x4A60,0x5A4A,
	/* Range 0x95A3 - 0x95A5, array index: 0x0F2F */
	0x4A48,0x796E,0x5B6C,
	/* Range 0x95D4 - 0x95D6, array index: 0x0F32 */
	0x796F,0x4F74,0x7766,
	/* Range 0x964B - 0x964D, array index: 0x0F35 */
	0x5730,0x5868,0x4B3D,
	/* Range 0x965B - 0x9664, array index: 0x0F38 */
	0x784E,0x7970,0x606D,0x6333,0x7433,INVALC,INVALC,0x6A42,
	0x7266,0x7036,
	/* Range 0x9673 - 0x9678, array index: 0x0F42 */
	0x7267,INVALC,0x5755,0x5436,0x7968,0x5741,
	/* Range 0x968A - 0x968E, array index: 0x0F48 */
	0x5369,0x6249,INVALC,0x7C5B,0x4D2D,
	/* Range 0x9698 - 0x969C, array index: 0x0F4D */
	0x6476,0x5040,INVALC,0x7037,0x6E21,
	/* Range 0x96C0 - 0x96CE, array index: 0x0F52 */
	0x6D4D,0x6452,INVALC,INVALC,0x6A29,0x643A,0x7322,0x4D52,
	INVALC,0x764B,INVALC,0x7166,0x6D41,0x683C,0x6E51,
	/* Range 0x96E8 - 0x96EA, array index: 0x0F61 */
	0x696B,0x696C,0x6064,
	/* Range 0x97FF - 0x9813, array index: 0x0F64 */
	0x7A42,0x7B61,0x7A6D,0x7022,0x4C71,INVALC,0x7A23,0x6277,
	INVALC,0x624E,INVALC,0x6975,INVALC,0x616B,INVALC,INVALC,
	INVALC,0x6768,0x6857,0x5A78,0x544B,
	/* Range 0x984C - 0x984E, array index: 0x0F79 */
	0x7039,0x647E,0x6449,
	/* Range 0x98ED - 0x98EF, array index: 0x0F7C */
	0x7650,0x6B66,0x5A79,
	/* Range 0x98FC - 0x98FE, array index: 0x0F7F */
	0x5E78,0x7869,0x635E,
	/* Range 0x9949 - 0x994D, array index: 0x0F82 */
	0x504F,INVALC,0x4F7A,0x734A,0x6057,
	/* Range 0x99AC - 0x99AE, array index: 0x0F87 */
	0x5829,0x655F,0x7927,
	/* Range 0x99D0 - 0x99D2, array index: 0x0F8A */
	0x7152,0x524C,0x4F4B,
	/* Range 0x9B41 - 0x9B45, array index: 0x0F8D */
	0x4E5A,0x7B6B,0x5B26,0x5B5E,0x5865,
	/* Range 0x9DF8 - 0x9DFA, array index: 0x0F92 */
	0x7D53,0x6B6D,0x565C,
	/* Range 0xAC07 - 0xAC0A, array index: 0x0F95 */
	0x3024,0x3025,0x3026,0x3027,
	/* Range 0xAC10 - 0xAC1D, array index: 0x0F99 */
	0x3028,0x3029,0x302A,0x302B,0x302C,0x302D,0x302E,0x302F,
	INVALC,0x3030,0x3031,0x3032,0x3033,0x3034,
	/* Range 0xAC2C - 0xAC31, array index: 0x0FA7 */
	0x3037,0x3038,INVALC,0x3039,0x303A,0x303B,
	/* Range 0xAC80 - 0xAC8C, array index: 0x0FAD */
	0x304B,0x304C,INVALC,0x304D,0x304E,0x304F,0x3050,INVALC,
	INVALC,0x3051,0x3052,0x3053,0x3054,
	/* Range 0xAC9C - 0xACA1, array index: 0x0FBA */
	0x3057,0x3058,INVALC,0x3059,0x305A,0x305B,
	/* Range 0xACA8 - 0xACB0, array index: 0x0FC0 */
	0x305C,0x305D,0x305E,INVALC,0x305F,INVALC,INVALC,0x3060,
	0x3061,
	/* Range 0xACB8 - 0xACBD, array index: 0x0FC9 */
	0x3062,0x3063,INVALC,0x3064,0x3065,0x3066,
	/* Range 0xACEF - 0xACF6, array index: 0x0FCF */
	0x3074,0x3075,0x3076,INVALC,0x3077,INVALC,0x3078,0x3079,
	/* Range 0xAD73 - 0xAD76, array index: 0x0FD7 */
	0x313B,0x313C,0x313D,0x313E,
	/* Range 0xAD7B - 0xAD82, array index: 0x0FDB */
	0x313F,0x3140,0x3141,INVALC,0x3142,INVALC,0x3143,0x3144,
	/* Range 0xADFF - 0xAE01, array index: 0x0FE3 */
	0x315A,0x315B,0x315C,
	/* Range 0xAE4A - 0xAE50, array index: 0x0FE6 */
	0x316D,INVALC,0x316E,0x316F,0x3170,INVALC,0x3171,
	/* Range 0xAE5C - 0xAE61, array index: 0x0FED */
	0x3174,0x3175,INVALC,0x3176,0x3177,0x3178,
	/* Range 0xAE78 - 0xAE7D, array index: 0x0FF3 */
	0x317E,0x3221,INVALC,0x3222,0x3223,0x3224,
	/* Range 0xAEBC - 0xAEC0, array index: 0x0FF9 */
	0x3228,0x3229,0x322A,INVALC,0x322B,
	/* Range 0xAECC - 0xAED1, array index: 0x0FFE */
	0x322D,0x322E,INVALC,0x322F,0x3230,0x3231,
	/* Range 0xAF3F - 0xAF43, array index: 0x1004 */
	0x3246,INVALC,0x3247,0x3248,0x3249,
	/* Range 0xAFC7 - 0xAFCE, array index: 0x1009 */
	0x325D,0x325E,0x325F,INVALC,0x3260,INVALC,0x3261,0x3262,
	/* Range 0xB053 - 0xB059, array index: 0x1011 */
	0x327A,0x327B,0x327C,INVALC,0x327D,INVALC,0x327E,
	/* Range 0xB098 - 0xB0A2, array index: 0x1018 */
	0x332A,0x332B,0x332C,INVALC,0x332D,INVALC,INVALC,0x332E,
	0x332F,0x3330,0x3331,
	/* Range 0xB0A8 - 0xB0B5, array index: 0x1023 */
	0x3332,0x3333,INVALC,0x3334,0x3335,0x3336,0x3337,0x3338,
	INVALC,0x3339,INVALC,0x333A,0x333B,0x333C,
	/* Range 0xB0C4 - 0xB0C9, array index: 0x1031 */
	0x333F,0x3340,INVALC,0x3341,0x3342,0x3343,
	/* Range 0xB108 - 0xB10C, array index: 0x1037 */
	0x334A,0x334B,INVALC,0x334C,0x334D,
	/* Range 0xB118 - 0xB11D, array index: 0x103C */
	0x3351,0x3352,INVALC,0x3353,0x3354,0x3355,
	/* Range 0xB123 - 0xB125, array index: 0x1042 */
	0x3356,0x3357,0x3358,
	/* Range 0xB134 - 0xB139, array index: 0x1045 */
	0x335B,0x335C,INVALC,0x335D,0x335E,0x335F,
	/* Range 0xB192 - 0xB194, array index: 0x104B */
	0x3374,0x3375,0x3376,
	/* Range 0xB298 - 0xB29A, array index: 0x104E */
	0x3443,0x3444,0x3445,
	/* Range 0xB2E2 - 0xB301, array index: 0x1051 */
	0x3458,INVALC,0x3459,0x345A,0x345B,INVALC,0x345C,INVALC,
	INVALC,0x345D,0x345E,0x345F,0x3460,0x3461,INVALC,INVALC,
	INVALC,0x3462,0x3463,0x3464,INVALC,0x3465,0x3466,0x3467,
	0x3468,0x3469,INVALC,INVALC,INVALC,0x346A,0x346B,0x346C,
	/* Range 0xB310 - 0xB315, array index: 0x1071 */
	0x346F,0x3470,INVALC,0x3471,0x3472,0x3473,
	/* Range 0xB354 - 0xB35F, array index: 0x1077 */
	0x3475,0x3476,0x3477,INVALC,0x3478,INVALC,INVALC,0x3479,
	0x347A,INVALC,0x347B,0x347C,
	/* Range 0xB380 - 0xB385, array index: 0x1083 */
	0x3529,0x352A,INVALC,0x352B,0x352C,0x352D,
	/* Range 0xB524 - 0xB52A, array index: 0x1089 */
	0x3575,0x3576,INVALC,0x3577,0x3578,0x3579,0x357A,
	/* Range 0xB540 - 0xB545, array index: 0x1090 */
	0x3621,0x3622,INVALC,0x3623,0x3624,0x3625,
	/* Range 0xB54B - 0xB54D, array index: 0x1096 */
	0x3626,0x3627,0x3628,
	/* Range 0xB55C - 0xB561, array index: 0x1099 */
	0x362B,0x362C,INVALC,0x362D,0x362E,0x362F,
	/* Range 0xB5B0 - 0xB5B5, array index: 0x109F */
	0x3636,0x3637,INVALC,0x3638,0x3639,0x363A,
	/* Range 0xB5BB - 0xB5BD, array index: 0x10A5 */
	0x363B,0x363C,0x363D,
	/* Range 0xB5CC - 0xB5D1, array index: 0x10A8 */
	0x3640,0x3641,INVALC,0x3642,0x3643,0x3644,
	/* Range 0xB78C - 0xB799, array index: 0x10AE */
	0x3677,0x3678,INVALC,0x3679,0x367A,0x367B,0x367C,INVALC,
	INVALC,INVALC,0x367D,0x367E,0x3721,0x3722,
	/* Range 0xB7A8 - 0xB7AD, array index: 0x10BC */
	0x3725,0x3726,INVALC,0x3727,0x3728,0x3729,
	/* Range 0xB7FC - 0xB801, array index: 0x10C2 */
	0x3733,0x3734,INVALC,0x3735,0x3736,0x3737,
	/* Range 0xB807 - 0xB809, array index: 0x10C8 */
	0x3738,0x3739,0x373A,
	/* Range 0xB834 - 0xB839, array index: 0x10CB */
	0x3745,0x3746,INVALC,0x3747,0x3748,0x3749,
	/* Range 0xB9CC - 0xB9D2, array index: 0x10D1 */
	0x3838,INVALC,0x3839,0x383A,0x383B,0x383C,0x383D,
	/* Range 0xB9E1 - 0xB9E5, array index: 0x10D8 */
	0x3843,INVALC,0x3844,0x3845,0x3846,
	/* Range 0xB9F4 - 0xB9FA, array index: 0x10DD */
	0x3849,0x384A,INVALC,0x384B,0x384C,0x384D,0x384E,
	/* Range 0xBA53 - 0xBA55, array index: 0x10E4 */
	0x385D,0x385E,0x385F,
	/* Range 0xBA64 - 0xBA69, array index: 0x10E7 */
	0x3862,0x3863,INVALC,0x3864,0x3865,0x3866,
	/* Range 0xBA83 - 0xBA87, array index: 0x10ED */
	0x386B,0x386C,0x386D,INVALC,0x386E,
	/* Range 0xBAA8 - 0xBAAC, array index: 0x10F2 */
	0x3870,0x3871,INVALC,0x3872,0x3873,
	/* Range 0xBB34 - 0xBB3E, array index: 0x10F7 */
	0x392B,0x392C,0x392D,INVALC,0x392E,INVALC,INVALC,0x392F,
	0x3930,0x3931,0x3932,
	/* Range 0xBC08 - 0xBC1F, array index: 0x1102 */
	0x3952,0x3953,INVALC,0x3954,0x3955,0x3956,INVALC,0x3957,
	INVALC,0x3958,INVALC,INVALC,0x3959,0x395A,0x395B,0x395C,
	0x395D,INVALC,INVALC,0x395E,0x395F,0x3960,0x3961,0x3962,
	/* Range 0xBC40 - 0xBC45, array index: 0x111A */
	0x396C,0x396D,INVALC,0x396E,0x396F,0x3970,
	/* Range 0xBCB0 - 0xBCB5, array index: 0x1120 */
	0x3A28,0x3A29,INVALC,0x3A2A,0x3A2B,0x3A2C,
	/* Range 0xBCCD - 0xBCD1, array index: 0x1126 */
	0x3A31,INVALC,0x3A32,0x3A33,0x3A34,
	/* Range 0xBCF4 - 0xBCF8, array index: 0x112B */
	0x3A38,0x3A39,0x3A3A,INVALC,0x3A3B,
	/* Range 0xBD87 - 0xBD8A, array index: 0x1130 */
	0x3A51,0x3A52,0x3A53,0x3A54,
	/* Range 0xBE57 - 0xBE5B, array index: 0x1134 */
	0x3A78,INVALC,0x3A79,0x3A7A,0x3A7B,
	/* Range 0xBE70 - 0xBE75, array index: 0x1139 */
	0x3B23,0x3B24,INVALC,0x3B25,0x3B26,0x3B27,
	/* Range 0xBE7B - 0xBE7D, array index: 0x113F */
	0x3B28,0x3B29,0x3B2A,
	/* Range 0xBE8C - 0xBE91, array index: 0x1142 */
	0x3B2D,0x3B2E,INVALC,0x3B2F,0x3B30,0x3B31,
	/* Range 0xBEE3 - 0xBEE5, array index: 0x1148 */
	0x3B3B,0x3B3C,0x3B3D,
	/* Range 0xBF18 - 0xBF1D, array index: 0x114B */
	0x3B42,0x3B43,INVALC,0x3B44,0x3B45,0x3B46,
	/* Range 0xC0AC - 0xC0B6, array index: 0x1151 */
	0x3B67,0x3B68,INVALC,0x3B69,0x3B6A,INVALC,INVALC,0x3B6B,
	0x3B6C,0x3B6D,0x3B6E,
	/* Range 0xC0BC - 0xC0C1, array index: 0x115C */
	0x3B6F,0x3B70,INVALC,0x3B71,0x3B72,0x3B73,
	/* Range 0xC0D8 - 0xC0DD, array index: 0x1162 */
	0x3B79,0x3B7A,INVALC,0x3B7B,0x3B7C,0x3B7D,
	/* Range 0xC11C - 0xC127, array index: 0x1168 */
	0x3C2D,0x3C2E,0x3C2F,0x3C30,0x3C31,INVALC,INVALC,0x3C32,
	0x3C33,INVALC,0x3C34,0x3C35,
	/* Range 0xC12C - 0xC131, array index: 0x1174 */
	0x3C36,0x3C37,INVALC,0x3C38,0x3C39,0x3C3A,
	/* Range 0xC148 - 0xC14D, array index: 0x117A */
	0x3C40,0x3C41,INVALC,0x3C42,0x3C43,0x3C44,
	/* Range 0xC164 - 0xC169, array index: 0x1180 */
	0x3C49,0x3C4A,INVALC,0x3C4B,0x3C4C,0x3C4D,
	/* Range 0xC18C - 0xC190, array index: 0x1186 */
	0x3C52,0x3C53,0x3C54,INVALC,0x3C55,
	/* Range 0xC2EB - 0xC2F1, array index: 0x118B */
	0x3D48,0x3D49,0x3D4A,INVALC,0x3D4B,INVALC,0x3D4C,
	/* Range 0xC2F8 - 0xC2FC, array index: 0x1192 */
	0x3D4E,0x3D4F,INVALC,0x3D50,0x3D51,
	/* Range 0xC313 - 0xC315, array index: 0x1197 */
	0x3D57,0x3D58,0x3D59,
	/* Range 0xC4FF - 0xC501, array index: 0x119A */
	0x3E37,0x3E38,0x3E39,
	/* Range 0xC544 - 0xC54E, array index: 0x119D */
	0x3E46,0x3E47,INVALC,INVALC,0x3E48,0x3E49,0x3E4A,INVALC,
	0x3E4B,0x3E4C,0x3E4D,
	/* Range 0xC553 - 0xC559, array index: 0x11A8 */
	0x3E4E,0x3E4F,0x3E50,INVALC,0x3E51,0x3E52,0x3E53,
	/* Range 0xC55D - 0xC561, array index: 0x11AF */
	0x3E54,0x3E55,INVALC,0x3E56,0x3E57,
	/* Range 0xC570 - 0xC575, array index: 0x11B4 */
	0x3E5A,0x3E5B,INVALC,0x3E5C,0x3E5D,0x3E5E,
	/* Range 0xC5B8 - 0xC5BE, array index: 0x11BA */
	0x3E70,0x3E71,INVALC,0x3E72,0x3E73,0x3E74,0x3E75,
	/* Range 0xC5C4 - 0xC5D1, array index: 0x11C1 */
	0x3E76,0x3E77,0x3E78,0x3E79,0x3E7A,0x3E7B,0x3E7C,INVALC,
	0x3E7D,INVALC,0x3E7E,INVALC,0x3F21,0x3F22,
	/* Range 0xC5EC - 0xC5F0, array index: 0x11CF */
	0x3F29,0x3F2A,0x3F2B,INVALC,0x3F2C,
	/* Range 0xC5FC - 0xC608, array index: 0x11D4 */
	0x3F30,0x3F31,0x3F32,0x3F33,0x3F34,0x3F35,INVALC,INVALC,
	INVALC,0x3F36,0x3F37,0x3F38,0x3F39,
	/* Range 0xC618 - 0xC61C, array index: 0x11E1 */
	0x3F3C,0x3F3D,INVALC,0x3F3E,0x3F3F,
	/* Range 0xC62C - 0xC63B, array index: 0x11E6 */
	0x3F43,0x3F44,0x3F45,INVALC,0x3F46,INVALC,INVALC,0x3F47,
	0x3F48,0x3F49,INVALC,0x3F4A,INVALC,0x3F4B,INVALC,0x3F4C,
	/* Range 0xC650 - 0xC655, array index: 0x11F6 */
	0x3F51,0x3F52,INVALC,0x3F53,0x3F54,0x3F55,
	/* Range 0xC6B8 - 0xC6BA, array index: 0x11FC */
	0x3F6F,0x3F70,0x3F71,
	/* Range 0xC74F - 0xC758, array index: 0x11FF */
	0x403F,INVALC,0x4040,0x4041,0x4042,0x4043,0x4044,0x4045,
	0x4046,0x4047,
	/* Range 0xC77C - 0xC77E, array index: 0x1209 */
	0x404F,0x4050,0x4051,
	/* Range 0xC783 - 0xC78A, array index: 0x120C */
	0x4052,0x4053,0x4054,INVALC,0x4055,0x4056,0x4057,0x4058,
	/* Range 0xC794 - 0xC79A, array index: 0x1214 */
	0x405C,INVALC,0x405D,0x405E,0x405F,INVALC,0x4060,
	/* Range 0xC7A0 - 0xC7A6, array index: 0x121B */
	0x4061,0x4062,INVALC,0x4063,0x4064,0x4065,0x4066,
	/* Range 0xC7BC - 0xC7C1, array index: 0x1222 */
	0x406B,0x406C,INVALC,0x406D,0x406E,0x406F,
	/* Range 0xC883 - 0xC88D, array index: 0x1228 */
	0x413D,INVALC,0x413E,0x413F,0x4140,INVALC,INVALC,INVALC,
	0x4141,0x4142,0x4143,
	/* Range 0xC904 - 0xC906, array index: 0x1233 */
	0x4159,0x415A,0x415B,
	/* Range 0xC9D9 - 0xC9DD, array index: 0x1236 */
	0x4223,0x4224,INVALC,0x4225,0x4226,
	/* Range 0xC9EC - 0xC9F1, array index: 0x123B */
	0x422B,0x422C,INVALC,0x422D,0x422E,0x422F,
	/* Range 0xCA08 - 0xCA0D, array index: 0x1241 */
	0x4234,0x4235,INVALC,0x4236,0x4237,0x4238,
	/* Range 0xCA5C - 0xCA61, array index: 0x1247 */
	0x4240,0x4241,INVALC,0x4242,0x4243,0x4244,
	/* Range 0xCC27 - 0xCC29, array index: 0x124D */
	0x4276,0x4277,0x4278,
	/* Range 0xCC38 - 0xCC3E, array index: 0x1250 */
	0x427C,0x427D,INVALC,0x427E,0x4321,0x4322,0x4323,
	/* Range 0xCC54 - 0xCC59, array index: 0x1257 */
	0x4328,0x4329,INVALC,0x432A,0x432B,0x432C,
	/* Range 0xCCA8 - 0xCCAD, array index: 0x125D */
	0x4337,0x4338,INVALC,0x4339,0x433A,0x433B,
	/* Range 0xCE5F - 0xCE61, array index: 0x1263 */
	0x4424,0x4425,0x4426,
	/* Range 0xCEA0 - 0xCEA5, array index: 0x1266 */
	0x4437,0x4438,INVALC,0x4439,0x443A,0x443B,
	/* Range 0xCEF4 - 0xCEF9, array index: 0x126C */
	0x4444,0x4445,INVALC,0x4446,0x4447,0x4448,
	/* Range 0xCF2C - 0xCF31, array index: 0x1272 */
	0x4454,0x4455,INVALC,0x4456,0x4457,0x4458,
	/* Range 0xD0D0 - 0xD0D5, array index: 0x1278 */
	0x453D,0x453E,INVALC,0x453F,0x4540,0x4541,
	/* Range 0xD0EC - 0xD0F1, array index: 0x127E */
	0x4546,0x4547,INVALC,0x4548,0x4549,0x454A,
	/* Range 0xD140 - 0xD145, array index: 0x1284 */
	0x4552,0x4553,INVALC,0x4554,0x4555,0x4556,
	/* Range 0xD30C - 0xD310, array index: 0x128A */
	0x4644,0x4645,0x4646,INVALC,0x4647,
	/* Range 0xD31C - 0xD321, array index: 0x128F */
	0x464A,0x464B,INVALC,0x464C,0x464D,0x464E,
	/* Range 0xD338 - 0xD33D, array index: 0x1295 */
	0x4654,0x4655,INVALC,0x4656,0x4657,0x4658,
	/* Range 0xD38C - 0xD391, array index: 0x129B */
	0x465F,0x4660,INVALC,0x4661,0x4662,0x4663,
	/* Range 0xD584 - 0xD589, array index: 0x12A1 */
	0x475C,0x475D,INVALC,0x475E,0x475F,0x4760,
	/* Range 0xD610 - 0xD615, array index: 0x12A7 */
	0x4778,0x4779,INVALC,0x477A,0x477B,0x477C,
	/* Range 0xD754 - 0xD759, array index: 0x12AD */
	0x4867,INVALC,0x4868,0x4869,0x486A,0x486B,
	/* Range 0xF900 - 0xFA0B, array index: 0x12B3 */
	0x4B50,0x4B56,0x4B67,0x4D4F,0x4D68,0x4E2D,0x4F7B,0x5022,
	0x5038,0x5050,0x505D,0x5154,0x5155,0x5158,0x515B,0x515C,
	0x515D,0x515E,0x515F,0x5160,0x5162,0x5163,0x5164,0x5165,
	0x5166,0x5168,0x5169,0x516A,0x516B,0x516D,0x516F,0x5170,
	0x5172,0x5176,0x517A,0x517C,0x517D,0x517E,0x5222,0x5223,
	0x5227,0x5228,0x5229,0x522A,0x522B,0x522D,0x5232,0x523E,
	0x5242,0x5243,0x5244,0x5246,0x5247,0x5248,0x5249,0x524A,
	0x524B,0x524D,0x524E,0x524F,0x5250,0x5251,0x5252,0x5253,
	0x5254,0x5255,0x5256,0x5257,0x5259,0x525A,0x525E,0x525F,
	0x5261,0x5262,0x5264,0x5265,0x5266,0x5267,0x5268,0x5269,
	0x526A,0x526B,0x5270,0x5271,0x5272,0x5273,0x5274,0x5275,
	0x5277,0x5278,0x5466,0x547C,0x5525,0x552B,0x552E,0x5638,
	0x564D,0x574B,0x5764,0x5B45,0x5B64,0x5C25,0x5D25,0x5D55,
	0x5D74,0x5E7C,0x5E7E,0x5F33,0x5F61,0x5F68,0x6071,0x612D,
	0x616D,0x6375,0x6421,0x6429,0x652E,0x6531,0x6532,0x6539,
	0x653B,0x653C,0x6544,0x654E,0x6550,0x6552,0x6556,0x657A,
	0x657B,0x657C,0x657E,0x6621,0x6624,0x6627,0x662D,0x662F,
	0x6630,0x6631,0x6633,0x6637,0x6638,0x663C,0x6644,0x6646,
	0x6647,0x664A,0x6652,0x6656,0x6659,0x665C,0x665F,0x6661,
	0x6664,0x6665,0x6666,0x6668,0x666A,0x666B,0x666C,0x666F,
	0x6671,0x6672,0x6675,0x6676,0x6677,0x6679,0x6721,0x6726,
	0x6729,0x672A,0x672C,0x672D,0x6730,0x673F,0x6741,0x6746,
	0x6747,0x674B,0x674D,0x674F,0x6750,0x6753,0x675F,0x6764,
	0x6766,0x6777,0x6867,0x6868,0x6870,0x6871,0x6877,0x6879,
	0x687B,0x687E,0x6927,0x692C,0x694C,0x6977,0x6A41,0x6A65,
	0x6A74,0x6A77,0x6A7C,0x6A7E,0x6B24,0x6B27,0x6B29,0x6B2A,
	0x6B3A,0x6B3B,0x6B3D,0x6B41,0x6B42,0x6B46,0x6B47,0x6B4C,
	0x6B4F,0x6B50,0x6B51,0x6B52,0x6B58,0x6C26,0x6C27,0x6C2A,
	0x6C2F,0x6C30,0x6C31,0x6C32,0x6C35,0x6C38,0x6C3A,0x6C40,
	0x6C41,0x6C45,0x6C46,0x6C49,0x6C4A,0x6C55,0x6C5D,0x6C5E,
	0x6C61,0x6C64,0x6C67,0x6C68,0x6C77,0x6C78,0x6C7A,0x6D21,
	0x6D22,0x6D23,0x6D6E,0x6E5B,0x723D,0x727A,0x7331,0x7427,
	0x746E,0x7674,0x7676,0x7738,0x7748,0x7753,0x785B,0x7870,
	0x7A21,0x7A22,0x7A66,0x7C29,
	/* Range 0xFF01 - 0xFF5E, array index: 0x13BF */
	0x2321,0x2322,0x2323,0x2324,0x2325,0x2326,0x2327,0x2328,
	0x2329,0x232A,0x232B,0x232C,0x232D,0x232E,0x232F,0x2330,
	0x2331,0x2332,0x2333,0x2334,0x2335,0x2336,0x2337,0x2338,
	0x2339,0x233A,0x233B,0x233C,0x233D,0x233E,0x233F,0x2340,
	0x2341,0x2342,0x2343,0x2344,0x2345,0x2346,0x2347,0x2348,
	0x2349,0x234A,0x234B,0x234C,0x234D,0x234E,0x234F,0x2350,
	0x2351,0x2352,0x2353,0x2354,0x2355,0x2356,0x2357,0x2358,
	0x2359,0x235A,0x235B,0x212C,0x235D,0x235E,0x235F,0x2360,
	0x2361,0x2362,0x2363,0x2364,0x2365,0x2366,0x2367,0x2368,
	0x2369,0x236A,0x236B,0x236C,0x236D,0x236E,0x236F,0x2370,
	0x2371,0x2372,0x2373,0x2374,0x2375,0x2376,0x2377,0x2378,
	0x2379,0x237A,0x237B,0x237C,0x237D,0x2226,
	/* Range 0xFFE0 - 0xFFE6, array index: 0x141D */
	0x214B,0x214C,0x217E,0x237E,INVALC,0x214D,0x235C,
	/* Unranged codes (438 codes) */
	/* Array index: 0x1424 */ 0x00A1,0x222E,
	/* Array index: 0x1424 */ 0x00A4,0x2234,
	/* Array index: 0x1424 */ 0x00A7,0x2157,
	/* Array index: 0x1424 */ 0x00A8,0x2127,
	/* Array index: 0x1424 */ 0x00AA,0x2823,
	/* Array index: 0x1424 */ 0x00AD,0x2129,
	/* Array index: 0x1424 */ 0x00C6,0x2821,
	/* Array index: 0x1424 */ 0x00D0,0x2822,
	/* Array index: 0x1424 */ 0x00D7,0x213F,
	/* Array index: 0x1424 */ 0x00D8,0x282A,
	/* Array index: 0x1424 */ 0x00DE,0x282D,
	/* Array index: 0x1424 */ 0x00DF,0x292C,
	/* Array index: 0x1424 */ 0x00E6,0x2921,
	/* Array index: 0x1424 */ 0x00F0,0x2923,
	/* Array index: 0x1424 */ 0x00F7,0x2140,
	/* Array index: 0x1424 */ 0x00F8,0x292A,
	/* Array index: 0x1424 */ 0x00FE,0x292D,
	/* Array index: 0x1424 */ 0x0111,0x2922,
	/* Array index: 0x1424 */ 0x0126,0x2824,
	/* Array index: 0x1424 */ 0x0127,0x2924,
	/* Array index: 0x1424 */ 0x0138,0x2927,
	/* Array index: 0x1424 */ 0x0152,0x282B,
	/* Array index: 0x1424 */ 0x0153,0x292B,
	/* Array index: 0x1424 */ 0x0166,0x282E,
	/* Array index: 0x1424 */ 0x0167,0x292E,
	/* Array index: 0x1424 */ 0x02C7,0x2227,
	/* Array index: 0x1424 */ 0x02D0,0x2230,
	/* Array index: 0x1424 */ 0x0401,0x2C27,
	/* Array index: 0x1424 */ 0x2015,0x212A,
	/* Array index: 0x1424 */ 0x2018,0x212E,
	/* Array index: 0x1424 */ 0x2019,0x212F,
	/* Array index: 0x1424 */ 0x201C,0x2130,
	/* Array index: 0x1424 */ 0x201D,0x2131,
	/* Array index: 0x1424 */ 0x2020,0x2253,
	/* Array index: 0x1424 */ 0x2021,0x2254,
	/* Array index: 0x1424 */ 0x2025,0x2125,
	/* Array index: 0x1424 */ 0x2026,0x2126,
	/* Array index: 0x1424 */ 0x2030,0x2236,
	/* Array index: 0x1424 */ 0x2032,0x2147,
	/* Array index: 0x1424 */ 0x2033,0x2148,
	/* Array index: 0x1424 */ 0x203B,0x2158,
	/* Array index: 0x1424 */ 0x2074,0x2979,
	/* Array index: 0x1424 */ 0x2103,0x2149,
	/* Array index: 0x1424 */ 0x2109,0x2235,
	/* Array index: 0x1424 */ 0x2113,0x2724,
	/* Array index: 0x1424 */ 0x2116,0x2260,
	/* Array index: 0x1424 */ 0x2121,0x2265,
	/* Array index: 0x1424 */ 0x2122,0x2262,
	/* Array index: 0x1424 */ 0x2126,0x2759,
	/* Array index: 0x1424 */ 0x212B,0x214A,
	/* Array index: 0x1424 */ 0x2153,0x2877,
	/* Array index: 0x1424 */ 0x2154,0x2878,
	/* Array index: 0x1424 */ 0x21D2,0x2221,
	/* Array index: 0x1424 */ 0x21D4,0x2222,
	/* Array index: 0x1424 */ 0x2200,0x2223,
	/* Array index: 0x1424 */ 0x2202,0x2153,
	/* Array index: 0x1424 */ 0x2203,0x2224,
	/* Array index: 0x1424 */ 0x2207,0x2154,
	/* Array index: 0x1424 */ 0x2208,0x2174,
	/* Array index: 0x1424 */ 0x220B,0x2175,
	/* Array index: 0x1424 */ 0x220F,0x2233,
	/* Array index: 0x1424 */ 0x2211,0x2232,
	/* Array index: 0x1424 */ 0x221A,0x216E,
	/* Array index: 0x1424 */ 0x221D,0x2170,
	/* Array index: 0x1424 */ 0x221E,0x2144,
	/* Array index: 0x1424 */ 0x2220,0x2150,
	/* Array index: 0x1424 */ 0x2234,0x2145,
	/* Array index: 0x1424 */ 0x2235,0x2171,
	/* Array index: 0x1424 */ 0x223C,0x212D,
	/* Array index: 0x1424 */ 0x223D,0x216F,
	/* Array index: 0x1424 */ 0x2252,0x2156,
	/* Array index: 0x1424 */ 0x2260,0x2141,
	/* Array index: 0x1424 */ 0x2261,0x2155,
	/* Array index: 0x1424 */ 0x2264,0x2142,
	/* Array index: 0x1424 */ 0x2265,0x2143,
	/* Array index: 0x1424 */ 0x226A,0x216C,
	/* Array index: 0x1424 */ 0x226B,0x216D,
	/* Array index: 0x1424 */ 0x2282,0x2178,
	/* Array index: 0x1424 */ 0x2283,0x2179,
	/* Array index: 0x1424 */ 0x2286,0x2176,
	/* Array index: 0x1424 */ 0x2287,0x2177,
	/* Array index: 0x1424 */ 0x2299,0x2241,
	/* Array index: 0x1424 */ 0x22A5,0x2151,
	/* Array index: 0x1424 */ 0x2312,0x2152,
	/* Array index: 0x1424 */ 0x2592,0x2246,
	/* Array index: 0x1424 */ 0x25B2,0x2163,
	/* Array index: 0x1424 */ 0x25B3,0x2162,
	/* Array index: 0x1424 */ 0x25B6,0x223A,
	/* Array index: 0x1424 */ 0x25B7,0x2239,
	/* Array index: 0x1424 */ 0x25BC,0x2165,
	/* Array index: 0x1424 */ 0x25BD,0x2164,
	/* Array index: 0x1424 */ 0x25C0,0x2238,
	/* Array index: 0x1424 */ 0x25C1,0x2237,
	/* Array index: 0x1424 */ 0x25CB,0x215B,
	/* Array index: 0x1424 */ 0x2605,0x215A,
	/* Array index: 0x1424 */ 0x2606,0x2159,
	/* Array index: 0x1424 */ 0x260E,0x224F,
	/* Array index: 0x1424 */ 0x260F,0x224E,
	/* Array index: 0x1424 */ 0x261C,0x2250,
	/* Array index: 0x1424 */ 0x261E,0x2251,
	/* Array index: 0x1424 */ 0x2640,0x214F,
	/* Array index: 0x1424 */ 0x2642,0x214E,
	/* Array index: 0x1424 */ 0x327F,0x225E,
	/* Array index: 0x1424 */ 0x33CF,0x2739,
	/* Array index: 0x1424 */ 0x33D0,0x276A,
	/* Array index: 0x1424 */ 0x33D3,0x276B,
	/* Array index: 0x1424 */ 0x33D6,0x275F,
	/* Array index: 0x1424 */ 0x33D8,0x2264,
	/* Array index: 0x1424 */ 0x4E00,0x6C69,
	/* Array index: 0x1424 */ 0x4E01,0x6F4B,
	/* Array index: 0x1424 */ 0x4E03,0x7652,
	/* Array index: 0x1424 */ 0x4E11,0x7564,
	/* Array index: 0x1424 */ 0x4E1E,0x632A,
	/* Array index: 0x1424 */ 0x4E2D,0x7169,
	/* Array index: 0x1424 */ 0x4E32,0x4D7A,
	/* Array index: 0x1424 */ 0x4E38,0x7C2F,
	/* Array index: 0x1424 */ 0x4E39,0x5321,
	/* Array index: 0x1424 */ 0x4E3B,0x712B,
	/* Array index: 0x1424 */ 0x4E42,0x6751,
	/* Array index: 0x1424 */ 0x4E43,0x522C,
	/* Array index: 0x1424 */ 0x4E45,0x4E79,
	/* Array index: 0x1424 */ 0x4E56,0x4E52,
	/* Array index: 0x1424 */ 0x4E58,0x632B,
	/* Array index: 0x1424 */ 0x4E59,0x6B60,
	/* Array index: 0x1424 */ 0x4E6B,0x4A61,
	/* Array index: 0x1424 */ 0x4E6D,0x544C,
	/* Array index: 0x1424 */ 0x4E73,0x6A61,
	/* Array index: 0x1424 */ 0x4E76,0x5C63,
	/* Array index: 0x1424 */ 0x4E77,0x5F2D,
	/* Array index: 0x1424 */ 0x4E7E,0x4B6B,
	/* Array index: 0x1424 */ 0x4E82,0x552F,
	/* Array index: 0x1424 */ 0x4E86,0x5675,
	/* Array index: 0x1424 */ 0x4E88,0x6578,
	/* Array index: 0x1424 */ 0x4E8B,0x5E40,
	/* Array index: 0x1424 */ 0x4E8C,0x6C23,
	/* Array index: 0x1424 */ 0x4E98,0x5066,
	/* Array index: 0x1424 */ 0x4E9B,0x5E41,
	/* Array index: 0x1424 */ 0x4E9E,0x642C,
	/* Array index: 0x1424 */ 0x4EB6,0x5322,
	/* Array index: 0x1424 */ 0x4EBA,0x6C51,
	/* Array index: 0x1424 */ 0x4EC0,0x6427,
	/* Array index: 0x1424 */ 0x4EC1,0x6C52,
	/* Array index: 0x1424 */ 0x4EC4,0x7631,
	/* Array index: 0x1424 */ 0x4EC7,0x4E7B,
	/* Array index: 0x1424 */ 0x4ECA,0x5051,
	/* Array index: 0x1424 */ 0x4ECB,0x4B3F,
	/* Array index: 0x1424 */ 0x4ECD,0x6D24,
	/* Array index: 0x1424 */ 0x4EDD,0x544E,
	/* Array index: 0x1424 */ 0x4EDF,0x7435,
	/* Array index: 0x1424 */ 0x4EF0,0x6466,
	/* Array index: 0x1424 */ 0x4EF2,0x716A,
	/* Array index: 0x1424 */ 0x4EF6,0x4B6C,
	/* Array index: 0x1424 */ 0x4EF7,0x4B40,
	/* Array index: 0x1424 */ 0x4EFB,0x6C72,
	/* Array index: 0x1424 */ 0x4F01,0x506A,
	/* Array index: 0x1424 */ 0x4F2F,0x5B57,
	/* Array index: 0x1424 */ 0x4F34,0x5A61,
	/* Array index: 0x1424 */ 0x4F36,0x5636,
	/* Array index: 0x1424 */ 0x4F38,0x635F,
	/* Array index: 0x1424 */ 0x4F3A,0x5E43,
	/* Array index: 0x1424 */ 0x4F3C,0x5E44,
	/* Array index: 0x1424 */ 0x4F3D,0x4A21,
	/* Array index: 0x1424 */ 0x4F43,0x6E6C,
	/* Array index: 0x1424 */ 0x4F55,0x793C,
	/* Array index: 0x1424 */ 0x4F69,0x7825,
	/* Array index: 0x1424 */ 0x4F6F,0x653A,
	/* Array index: 0x1424 */ 0x4F70,0x5B58,
	/* Array index: 0x1424 */ 0x4F73,0x4A22,
	/* Array index: 0x1424 */ 0x4F76,0x514D,
	/* Array index: 0x1424 */ 0x4F7A,0x6E6D,
	/* Array index: 0x1424 */ 0x4F7E,0x6C6B,
	/* Array index: 0x1424 */ 0x4F7F,0x5E45,
	/* Array index: 0x1424 */ 0x4F81,0x6360,
	/* Array index: 0x1424 */ 0x4F83,0x4A49,
	/* Array index: 0x1424 */ 0x4F84,0x7269,
	/* Array index: 0x1424 */ 0x4F86,0x554E,
	/* Array index: 0x1424 */ 0x4F88,0x7636,
	/* Array index: 0x1424 */ 0x4F8A,0x4E42,
	/* Array index: 0x1424 */ 0x4F8B,0x5647,
	/* Array index: 0x1424 */ 0x4F8D,0x6334,
	/* Array index: 0x1424 */ 0x4F8F,0x712D,
	/* Array index: 0x1424 */ 0x4F91,0x6A62,
	/* Array index: 0x1424 */ 0x4F96,0x5742,
	/* Array index: 0x1424 */ 0x4F98,0x7327,
	/* Array index: 0x1424 */ 0x4F9B,0x4D6A,
	/* Array index: 0x1424 */ 0x4F9D,0x6B6E,
	/* Array index: 0x1424 */ 0x4FAE,0x5932,
	/* Array index: 0x1424 */ 0x4FAF,0x7D25,
	/* Array index: 0x1424 */ 0x4FB5,0x7655,
	/* Array index: 0x1424 */ 0x4FB6,0x5562,
	/* Array index: 0x1424 */ 0x4FBF,0x7835,
	/* Array index: 0x1424 */ 0x4FC9,0x676B,
	/* Array index: 0x1424 */ 0x4FCA,0x7155,
	/* Array index: 0x1424 */ 0x4FCE,0x703B,
	/* Array index: 0x1424 */ 0x4FD1,0x6935,
	/* Array index: 0x1424 */ 0x4FD3,0x4C49,
	/* Array index: 0x1424 */ 0x4FD4,0x7A55,
	/* Array index: 0x1424 */ 0x4FD7,0x6154,
	/* Array index: 0x1424 */ 0x4FDA,0x5756,
	/* Array index: 0x1424 */ 0x4FEE,0x6173,
	/* Array index: 0x1424 */ 0x4FEF,0x5C76,
	/* Array index: 0x1424 */ 0x4FF1,0x4E7C,
	/* Array index: 0x1424 */ 0x4FF3,0x5B44,
	/* Array index: 0x1424 */ 0x4FF5,0x7871,
	/* Array index: 0x1424 */ 0x4FF8,0x5C64,
	/* Array index: 0x1424 */ 0x4FFA,0x656F,
	/* Array index: 0x1424 */ 0x5002,0x5C31,
	/* Array index: 0x1424 */ 0x5006,0x5556,
	/* Array index: 0x1424 */ 0x5009,0x735A,
	/* Array index: 0x1424 */ 0x500B,0x4B41,
	/* Array index: 0x1424 */ 0x500D,0x5B43,
	/* Array index: 0x1424 */ 0x5011,0x597A,
	/* Array index: 0x1424 */ 0x5012,0x536E,
	/* Array index: 0x1424 */ 0x5016,0x7A38,
	/* Array index: 0x1424 */ 0x5019,0x7D26,
	/* Array index: 0x1424 */ 0x501A,0x6B6F,
	/* Array index: 0x1424 */ 0x501C,0x7426,
	/* Array index: 0x1424 */ 0x501E,0x4C4A,
	/* Array index: 0x1424 */ 0x501F,0x7328,
	/* Array index: 0x1424 */ 0x5021,0x735B,
	/* Array index: 0x1424 */ 0x503B,0x6526,
	/* Array index: 0x1424 */ 0x5043,0x6567,
	/* Array index: 0x1424 */ 0x504F,0x7836,
	/* Array index: 0x1424 */ 0x5055,0x7A25,
	/* Array index: 0x1424 */ 0x505A,0x712E,
	/* Array index: 0x1424 */ 0x505C,0x6F4E,
	/* Array index: 0x1424 */ 0x5065,0x4B6D,
	/* Array index: 0x1424 */ 0x5080,0x4E53,
	/* Array index: 0x1424 */ 0x5085,0x5C77,
	/* Array index: 0x1424 */ 0x508D,0x5B28,
	/* Array index: 0x1424 */ 0x5091,0x4B78,
	/* Array index: 0x1424 */ 0x5098,0x5F21,
	/* Array index: 0x1424 */ 0x5099,0x5D61,
	/* Array index: 0x1424 */ 0x50AC,0x754A,
	/* Array index: 0x1424 */ 0x50AD,0x6936,
	/* Array index: 0x1424 */ 0x50B2,0x676C,
	/* Array index: 0x1424 */ 0x50B3,0x6E6E,
	/* Array index: 0x1424 */ 0x50B5,0x7370,
	/* Array index: 0x1424 */ 0x50B7,0x5F3F,
	/* Array index: 0x1424 */ 0x50BE,0x4C4B,
	/* Array index: 0x1424 */ 0x50C5,0x5041,
	/* Array index: 0x1424 */ 0x50C9,0x7452,
	/* Array index: 0x1424 */ 0x50CA,0x603A,
	/* Array index: 0x1424 */ 0x50CF,0x5F40,
	/* Array index: 0x1424 */ 0x50D1,0x4E60,
	/* Array index: 0x1424 */ 0x50D5,0x5C52,
	/* Array index: 0x1424 */ 0x50D6,0x7D6A,
	/* Array index: 0x1424 */ 0x50DA,0x5676,
	/* Array index: 0x1424 */ 0x50DE,0x6A4A,
	/* Array index: 0x1424 */ 0x50E5,0x6869,
	/* Array index: 0x1424 */ 0x50E7,0x632C,
	/* Array index: 0x1424 */ 0x50ED,0x7350,
	/* Array index: 0x1424 */ 0x50F9,0x4A24,
	/* Array index: 0x1424 */ 0x50FB,0x5B78,
	/* Array index: 0x1424 */ 0x5104,0x6562,
	/* Array index: 0x1424 */ 0x5106,0x4C4C,
	/* Array index: 0x1424 */ 0x5109,0x4B7B,
	/* Array index: 0x1424 */ 0x5112,0x6A63,
	/* Array index: 0x1424 */ 0x511F,0x5F41,
	/* Array index: 0x1424 */ 0x5121,0x566D,
	/* Array index: 0x1424 */ 0x512A,0x6950,
	/* Array index: 0x1424 */ 0x5132,0x6E39,
	/* Array index: 0x1424 */ 0x5137,0x5563,
	/* Array index: 0x1424 */ 0x513A,0x5153,
	/* Array index: 0x1424 */ 0x513C,0x6570,
	/* Array index: 0x1424 */ 0x5152,0x642E,
	/* Array index: 0x1424 */ 0x515C,0x545F,
	/* Array index: 0x1424 */ 0x5162,0x5067,
	/* Array index: 0x1424 */ 0x5171,0x4D6C,
	/* Array index: 0x1424 */ 0x517C,0x4C42,
	/* Array index: 0x1424 */ 0x5180,0x506D,
	/* Array index: 0x1424 */ 0x5186,0x6577,
	/* Array index: 0x1424 */ 0x518A,0x737C,
	/* Array index: 0x1424 */ 0x518D,0x6E22,
	/* Array index: 0x1424 */ 0x5192,0x5933,
	/* Array index: 0x1424 */ 0x5195,0x5874,
	/* Array index: 0x1424 */ 0x5197,0x6937,
	/* Array index: 0x1424 */ 0x51A0,0x4E2E,
	/* Array index: 0x1424 */ 0x51A5,0x5922,
	/* Array index: 0x1424 */ 0x51AA,0x5871,
	/* Array index: 0x1424 */ 0x51AC,0x544F,
	/* Array index: 0x1424 */ 0x51B6,0x6527,
	/* Array index: 0x1424 */ 0x51B7,0x5552,
	/* Array index: 0x1424 */ 0x51BD,0x5629,
	/* Array index: 0x1424 */ 0x51C4,0x7422,
	/* Array index: 0x1424 */ 0x51C6,0x7157,
	/* Array index: 0x1424 */ 0x51E1,0x5B6D,
	/* Array index: 0x1424 */ 0x51F0,0x7C45,
	/* Array index: 0x1424 */ 0x51F1,0x4B42,
	/* Array index: 0x1424 */ 0x51FD,0x795E,
	/* Array index: 0x1424 */ 0x5200,0x536F,
	/* Array index: 0x1424 */ 0x5203,0x6C53,
	/* Array index: 0x1424 */ 0x520E,0x597B,
	/* Array index: 0x1424 */ 0x5211,0x7A7D,
	/* Array index: 0x1424 */ 0x5217,0x562A,
	/* Array index: 0x1424 */ 0x521D,0x7478,
	/* Array index: 0x1424 */ 0x5224,0x7777,
	/* Array index: 0x1424 */ 0x5225,0x5C2C,
	/* Array index: 0x1424 */ 0x5229,0x5757,
	/* Array index: 0x1424 */ 0x522A,0x5F22,
	/* Array index: 0x1424 */ 0x522E,0x4E3E,
	/* Array index: 0x1424 */ 0x5230,0x5370,
	/* Array index: 0x1424 */ 0x5243,0x746F,
	/* Array index: 0x1424 */ 0x5247,0x764E,
	/* Array index: 0x1424 */ 0x5254,0x7428,
	/* Array index: 0x1424 */ 0x5256,0x5C78,
	/* Array index: 0x1424 */ 0x525B,0x4B27,
	/* Array index: 0x1424 */ 0x525D,0x5A4E,
	/* Array index: 0x1424 */ 0x5261,0x6066,
	/* Array index: 0x1424 */ 0x5269,0x6D25,
	/* Array index: 0x1424 */ 0x526A,0x6E72,
	/* Array index: 0x1424 */ 0x526F,0x5C79,
	/* Array index: 0x1424 */ 0x5272,0x795C,
	/* Array index: 0x1424 */ 0x5275,0x735C,
	/* Array index: 0x1424 */ 0x527D,0x7872,
	/* Array index: 0x1424 */ 0x527F,0x7479,
	/* Array index: 0x1424 */ 0x5283,0x7C71,
	/* Array index: 0x1424 */ 0x528D,0x4B7C,
	/* Array index: 0x1424 */ 0x5291,0x7025,
	/* Array index: 0x1424 */ 0x5292,0x4B7D,
	/* Array index: 0x1424 */ 0x529B,0x5574,
	/* Array index: 0x1424 */ 0x529F,0x4D6D,
	/* Array index: 0x1424 */ 0x52A0,0x4A25,
	/* Array index: 0x1424 */ 0x52A3,0x562B,
	/* Array index: 0x1424 */ 0x52A4,0x5042,
	/* Array index: 0x1424 */ 0x52BE,0x7A36,
	/* Array index: 0x1424 */ 0x52C1,0x4C4D,
	/* Array index: 0x1424 */ 0x52C3,0x5A7A,
	/* Array index: 0x1424 */ 0x52C5,0x764F,
	/* Array index: 0x1424 */ 0x52C7,0x6938,
	/* Array index: 0x1424 */ 0x52C9,0x5875,
	/* Array index: 0x1424 */ 0x52CD,0x4C4E,
	/* Array index: 0x1424 */ 0x52D2,0x574D,
	/* Array index: 0x1424 */ 0x52F3,0x7D33,
	/* Array index: 0x1424 */ 0x52F5,0x5564,
	/* Array index: 0x1424 */ 0x52F8,0x4F68,
	/* Array index: 0x1424 */ 0x52FA,0x6D43,
	/* Array index: 0x1424 */ 0x52FB,0x5032,
	/* Array index: 0x1424 */ 0x52FE,0x4E7E,
	/* Array index: 0x1424 */ 0x52FF,0x5A28,
	/* Array index: 0x1424 */ 0x5305,0x7850,
	/* Array index: 0x1424 */ 0x5308,0x7D56,
	/* Array index: 0x1424 */ 0x530D,0x7851,
	/* Array index: 0x1424 */ 0x530F,0x7852,
	/* Array index: 0x1424 */ 0x5310,0x5C53,
	/* Array index: 0x1424 */ 0x5320,0x6D5D,
	/* Array index: 0x1424 */ 0x5321,0x4E44,
	/* Array index: 0x1424 */ 0x5323,0x4B21,
	/* Array index: 0x1424 */ 0x532A,0x5D63,
	/* Array index: 0x1424 */ 0x532F,0x7C5D,
	/* Array index: 0x1424 */ 0x5339,0x792F,
	/* Array index: 0x1424 */ 0x534D,0x5833,
	/* Array index: 0x1424 */ 0x5357,0x5175,
	/* Array index: 0x1424 */ 0x535A,0x5A4F,
	/* Array index: 0x1424 */ 0x535C,0x5C54,
	/* Array index: 0x1424 */ 0x535E,0x5C26,
	/* Array index: 0x1424 */ 0x5360,0x6F3F,
	/* Array index: 0x1424 */ 0x5366,0x4E4F,
	/* Array index: 0x1424 */ 0x5368,0x6059,
	/* Array index: 0x1424 */ 0x537D,0x716D,
	/* Array index: 0x1424 */ 0x537F,0x4C4F,
	/* Array index: 0x1424 */ 0x5384,0x6478,
	/* Array index: 0x1424 */ 0x5393,0x646D,
	/* Array index: 0x1424 */ 0x5398,0x5758,
	/* Array index: 0x1424 */ 0x539A,0x7D27,
	/* Array index: 0x1424 */ 0x539F,0x6A2B,
	/* Array index: 0x1424 */ 0x53A0,0x7632,
	/* Array index: 0x1424 */ 0x53A5,0x4F70,
	/* Array index: 0x1424 */ 0x53A6,0x793D,
	/* Array index: 0x1424 */ 0x53AD,0x6674,
	/* Array index: 0x1424 */ 0x53BB,0x4B5B,
	/* Array index: 0x1424 */ 0x53C3,0x7351,
	/* Array index: 0x1424 */ 0x53D4,0x6252,
	/* Array index: 0x1424 */ 0x53D6,0x7622,
	/* Array index: 0x1424 */ 0x53D7,0x6174,
	/* Array index: 0x1424 */ 0x53DB,0x5A64,
	/* Array index: 0x1424 */ 0x53F8,0x5E49,
	/* Array index: 0x1424 */ 0x5403,0x7D5E,
	/* Array index: 0x1424 */ 0x5404,0x4A40,
	/* Array index: 0x1424 */ 0x541B,0x4F56,
	/* Array index: 0x1424 */ 0x541D,0x5770,
	/* Array index: 0x1424 */ 0x541F,0x6B61,
	/* Array index: 0x1424 */ 0x5420,0x7845,
	/* Array index: 0x1424 */ 0x5426,0x5C7A,
	/* Array index: 0x1424 */ 0x5429,0x5D43,
	/* Array index: 0x1424 */ 0x542B,0x795F,
	/* Array index: 0x1424 */ 0x5433,0x676F,
	/* Array index: 0x1424 */ 0x5442,0x5565,
	/* Array index: 0x1424 */ 0x5448,0x6F50,
	/* Array index: 0x1424 */ 0x544A,0x4D31,
	/* Array index: 0x1424 */ 0x5451,0x7722,
	/* Array index: 0x1424 */ 0x5468,0x7132,
	/* Array index: 0x1424 */ 0x546A,0x7131,
	/* Array index: 0x1424 */ 0x5471,0x4D32,
	/* Array index: 0x1424 */ 0x5473,0x5A2B,
	/* Array index: 0x1424 */ 0x5475,0x4A27,
	/* Array index: 0x1424 */ 0x5480,0x6E3A,
	/* Array index: 0x1424 */ 0x5486,0x7853,
	/* Array index: 0x1424 */ 0x548C,0x7B7A,
	/* Array index: 0x1424 */ 0x548E,0x4F24,
	/* Array index: 0x1424 */ 0x5490,0x5C7B,
	/* Array index: 0x1424 */ 0x54A4,0x7663,
	/* Array index: 0x1424 */ 0x54A8,0x6D2A,
	/* Array index: 0x1424 */ 0x54AB,0x7221,
	/* Array index: 0x1424 */ 0x54AC,0x4E61,
	/* Array index: 0x1424 */ 0x54B3,0x7A26,
	/* Array index: 0x1424 */ 0x54B8,0x7960,
	/* Array index: 0x1424 */ 0x54BD,0x6C56,
	/* Array index: 0x1424 */ 0x54C0,0x646E,
	/* Array index: 0x1424 */ 0x54C1,0x7921,
	/* Array index: 0x1424 */ 0x54C4,0x7B6F,
	/* Array index: 0x1424 */ 0x54C8,0x796B,
	/* Array index: 0x1424 */ 0x54C9,0x6E23,
	/* Array index: 0x1424 */ 0x54E1,0x6A2C,
	/* Array index: 0x1424 */ 0x54E5,0x4A28,
	/* Array index: 0x1424 */ 0x54E8,0x747A,
	/* Array index: 0x1424 */ 0x54ED,0x4D56,
	/* Array index: 0x1424 */ 0x54EE,0x7C76,
	/* Array index: 0x1424 */ 0x54F2,0x7449,
	/* Array index: 0x1424 */ 0x54FA,0x7854,
	/* Array index: 0x1424 */ 0x5504,0x7826,
	/* Array index: 0x1424 */ 0x5506,0x5E4A,
	/* Array index: 0x1424 */ 0x5507,0x7246,
	/* Array index: 0x1424 */ 0x550E,0x575A,
	/* Array index: 0x1424 */ 0x5510,0x5350,
	/* Array index: 0x1424 */ 0x551C,0x5845,
	/* Array index: 0x1424 */ 0x552F,0x6A66,
	/* Array index: 0x1424 */ 0x5531,0x735D,
	/* Array index: 0x1424 */ 0x5535,0x645A,
	/* Array index: 0x1424 */ 0x553E,0x7664,
	/* Array index: 0x1424 */ 0x5544,0x7672,
	/* Array index: 0x1424 */ 0x5546,0x5F42,
	/* Array index: 0x1424 */ 0x554F,0x597D,
	/* Array index: 0x1424 */ 0x5553,0x4C76,
	/* Array index: 0x1424 */ 0x5556,0x533A,
	/* Array index: 0x1424 */ 0x555E,0x642F,
	/* Array index: 0x1424 */ 0x5563,0x7961,
	/* Array index: 0x1424 */ 0x557C,0x7026,
	/* Array index: 0x1424 */ 0x5580,0x4B53,
	/* Array index: 0x1424 */ 0x5584,0x603C,
	/* Array index: 0x1424 */ 0x55C5,0x7D2B,
	/* Array index: 0x1424 */ 0x55C7,0x5F60,
	/* Array index: 0x1424 */ 0x55D4,0x7247,
	/* Array index: 0x1424 */ 0x55DA,0x6770,
	/* Array index: 0x1424 */ 0x55DC,0x506E,
	/* Array index: 0x1424 */ 0x55DF,0x732A,
	/* Array index: 0x1424 */ 0x55E3,0x5E4B,
	/* Array index: 0x1424 */ 0x55E4,0x7638,
	/* Array index: 0x1424 */ 0x55FD,0x6175,
	/* Array index: 0x1424 */ 0x55FE,0x7133,
	/* Array index: 0x1424 */ 0x5606,0x7723,
	/* Array index: 0x1424 */ 0x5609,0x4A29,
	/* Array index: 0x1424 */ 0x5614,0x4F25,
	/* Array index: 0x1424 */ 0x5617,0x5F44,
	/* Array index: 0x1424 */ 0x562F,0x6130,
	/* Array index: 0x1424 */ 0x5632,0x703F,
	/* Array index: 0x1424 */ 0x5634,0x7624,
	/* Array index: 0x1424 */ 0x5636,0x6336,
	/* Array index: 0x1424 */ 0x5653,0x7A46,
	/* Array index: 0x1424 */ 0x5668,0x506F,
	/* Array index: 0x1424 */ 0x566B,0x7D6D,
	/* Array index: 0x1424 */ 0x5674,0x5D44,
	/* Array index: 0x1424 */ 0x5686,0x7C77,
	/* Array index: 0x1424 */ 0x56A5,0x663F,
	/* Array index: 0x1424 */ 0x56AC,0x5E2D,
	/* Array index: 0x1424 */ 0x56AE,0x7A3F,
	/* Array index: 0x1424 */ 0x56B4,0x6571,
	/* Array index: 0x1424 */ 0x56BC,0x6D44,
	/* Array index: 0x1424 */ 0x56CA,0x5225,
	/* Array index: 0x1424 */ 0x56CD,0x7D6E,
	/* Array index: 0x1424 */ 0x56D1,0x7536,
	/* Array index: 0x1424 */ 0x56DA,0x6176,
	/* Array index: 0x1424 */ 0x56DB,0x5E4C,
	/* Array index: 0x1424 */ 0x56DE,0x7C5E,
	/* Array index: 0x1424 */ 0x56E0,0x6C57,
	/* Array index: 0x1424 */ 0x56F0,0x4D5D,
	/* Array index: 0x1424 */ 0x56F9,0x5637,
	/* Array index: 0x1424 */ 0x56FA,0x4D33,
	/* Array index: 0x1424 */ 0x5703,0x7855,
	/* Array index: 0x1424 */ 0x5704,0x6558,
	/* Array index: 0x1424 */ 0x5708,0x4F6A,
	/* Array index: 0x1424 */ 0x570B,0x4F50,
	/* Array index: 0x1424 */ 0x570D,0x6A4C,
	/* Array index: 0x1424 */ 0x5712,0x6A2E,
	/* Array index: 0x1424 */ 0x5713,0x6A2D,
	/* Array index: 0x1424 */ 0x5716,0x5371,
	/* Array index: 0x1424 */ 0x5718,0x5325,
	/* Array index: 0x1424 */ 0x571F,0x774F,
	/* Array index: 0x1424 */ 0x5728,0x6E24,
	/* Array index: 0x1424 */ 0x572D,0x5024,
	/* Array index: 0x1424 */ 0x5730,0x7222,
	/* Array index: 0x1424 */ 0x573B,0x5070,
	/* Array index: 0x1424 */ 0x5740,0x7223,
	/* Array index: 0x1424 */ 0x5742,0x7778,
	/* Array index: 0x1424 */ 0x5747,0x5033,
	/* Array index: 0x1424 */ 0x574A,0x5B29,
	/* Array index: 0x1424 */ 0x5761,0x7767,
	/* Array index: 0x1424 */ 0x5764,0x4D5E,
	/* Array index: 0x1424 */ 0x5766,0x7724,
	/* Array index: 0x1424 */ 0x576A,0x7840,
	/* Array index: 0x1424 */ 0x576E,0x535D,
	/* Array index: 0x1424 */ 0x5770,0x4C50,
	/* Array index: 0x1424 */ 0x5775,0x4F26,
	/* Array index: 0x1424 */ 0x577C,0x7673,
	/* Array index: 0x1424 */ 0x5782,0x6177,
	/* Array index: 0x1424 */ 0x5788,0x535C,
	/* Array index: 0x1424 */ 0x578B,0x7A7E,
	/* Array index: 0x1424 */ 0x5793,0x7A27,
	/* Array index: 0x1424 */ 0x57A0,0x6B59,
	/* Array index: 0x1424 */ 0x57A2,0x4F27,
	/* Array index: 0x1424 */ 0x57A3,0x6A2F,
	/* Array index: 0x1424 */ 0x57C3,0x646F,
	/* Array index: 0x1424 */ 0x57C7,0x6939,
	/* Array index: 0x1424 */ 0x57C8,0x7158,
	/* Array index: 0x1424 */ 0x57CB,0x5858,
	/* Array index: 0x1424 */ 0x57CE,0x6072,
	/* Array index: 0x1424 */ 0x57DF,0x6634,
	/* Array index: 0x1424 */ 0x57E0,0x5C7C,
	/* Array index: 0x1424 */ 0x57F0,0x7371,
	/* Array index: 0x1424 */ 0x57F4,0x6350,
	/* Array index: 0x1424 */ 0x57F7,0x727B,
	/* Array index: 0x1424 */ 0x57F9,0x5B46,
	/* Array index: 0x1424 */ 0x57FA,0x5071,
	/* Array index: 0x1424 */ 0x57FC,0x5072,
	/* Array index: 0x1424 */ 0x5800,0x4F5C,
	/* Array index: 0x1424 */ 0x5802,0x5351,
	/* Array index: 0x1424 */ 0x581E,0x745C,
	/* Array index: 0x1424 */ 0x5821,0x5C42,
	/* Array index: 0x1424 */ 0x5824,0x7027,
	/* Array index: 0x1424 */ 0x5827,0x6640,
	/* Array index: 0x1424 */ 0x582A,0x4A6D,
	/* Array index: 0x1424 */ 0x583A,0x4C77,
	/* Array index: 0x1424 */ 0x584A,0x4E54,
	/* Array index: 0x1424 */ 0x584B,0x672B,
	/* Array index: 0x1424 */ 0x584F,0x4B43,
	/* Array index: 0x1424 */ 0x5851,0x6131,
	/* Array index: 0x1424 */ 0x5854,0x7732,
	/* Array index: 0x1424 */ 0x5857,0x5373,
	/* Array index: 0x1424 */ 0x5858,0x5352,
	/* Array index: 0x1424 */ 0x585A,0x7540,
	/* Array index: 0x1424 */ 0x585E,0x5F5D,
	/* Array index: 0x1424 */ 0x5861,0x6E73,
	/* Array index: 0x1424 */ 0x5862,0x6771,
	/* Array index: 0x1424 */ 0x5864,0x7D34,
	/* Array index: 0x1424 */ 0x5875,0x7248,
	/* Array index: 0x1424 */ 0x5879,0x7352,
	/* Array index: 0x1424 */ 0x587C,0x6E74,
	/* Array index: 0x1424 */ 0x587E,0x6253,
	/* Array index: 0x1424 */ 0x5883,0x4C51,
	/* Array index: 0x1424 */ 0x5885,0x5F6A,
	/* Array index: 0x1424 */ 0x5889,0x693A,
	/* Array index: 0x1424 */ 0x5893,0x5957,
	/* Array index: 0x1424 */ 0x589C,0x754D,
	/* Array index: 0x1424 */ 0x589E,0x7172,
	/* Array index: 0x1424 */ 0x589F,0x7A47,
	/* Array index: 0x1424 */ 0x58A8,0x5978,
	/* Array index: 0x1424 */ 0x58A9,0x5442,
	/* Array index: 0x1424 */ 0x58AE,0x7665,
	/* Array index: 0x1424 */ 0x58B3,0x5D45,
	/* Array index: 0x1424 */ 0x58BA,0x6772,
	/* Array index: 0x1424 */ 0x58BB,0x6D5F,
	/* Array index: 0x1424 */ 0x58BE,0x4A4B,
	/* Array index: 0x1424 */ 0x58C1,0x5B7A,
	/* Array index: 0x1424 */ 0x58C5,0x6835,
	/* Array index: 0x1424 */ 0x58C7,0x5326,
	/* Array index: 0x1424 */ 0x58CE,0x7D35,
	/* Array index: 0x1424 */ 0x58D1,0x7949,
	/* Array index: 0x1424 */ 0x58D3,0x6462,
	/* Array index: 0x1424 */ 0x58D5,0x7B3D,
	/* Array index: 0x1424 */ 0x58D8,0x5724,
	/* Array index: 0x1424 */ 0x58D9,0x4E45,
	/* Array index: 0x1424 */ 0x58DE,0x4E55,
	/* Array index: 0x1424 */ 0x58DF,0x5666,
	/* Array index: 0x1424 */ 0x58E4,0x653D,
	/* Array index: 0x1424 */ 0x58EB,0x5E4D,
	/* Array index: 0x1424 */ 0x58EC,0x6C73,
	/* Array index: 0x1424 */ 0x58EF,0x6D60,
	/* Array index: 0x1424 */ 0x590F,0x793E,
	/* Array index: 0x1424 */ 0x5922,0x5953,
	/* Array index: 0x1424 */ 0x5931,0x6377,
	/* Array index: 0x1424 */ 0x5937,0x6C28,
	/* Array index: 0x1424 */ 0x593E,0x7A71,
	/* Array index: 0x1424 */ 0x5944,0x6572,
	/* Array index: 0x1424 */ 0x595A,0x7A28,
	/* Array index: 0x1424 */ 0x5960,0x6E75,
	/* Array index: 0x1424 */ 0x5962,0x5E4E,
	/* Array index: 0x1424 */ 0x5967,0x6773,
	/* Array index: 0x1424 */ 0x5973,0x5233,
	/* Array index: 0x1424 */ 0x5974,0x523F,
	/* Array index: 0x1424 */ 0x5978,0x4A4C,
	/* Array index: 0x1424 */ 0x597D,0x7B3F,
	/* Array index: 0x1424 */ 0x598A,0x6C74,
	/* Array index: 0x1424 */ 0x5993,0x5075,
	/* Array index: 0x1424 */ 0x5996,0x686D,
	/* Array index: 0x1424 */ 0x5997,0x5052,
	/* Array index: 0x1424 */ 0x5999,0x5958,
	/* Array index: 0x1424 */ 0x59A5,0x7666,
	/* Array index: 0x1424 */ 0x59A8,0x5B2A,
	/* Array index: 0x1424 */ 0x59AC,0x7760,
	/* Array index: 0x1424 */ 0x59B9,0x5859,
	/* Array index: 0x1424 */ 0x59BB,0x7423,
	/* Array index: 0x1424 */ 0x59BE,0x745D,
	/* Array index: 0x1424 */ 0x59C3,0x6F51,
	/* Array index: 0x1424 */ 0x59C6,0x5935,
	/* Array index: 0x1424 */ 0x59C9,0x6D2B,
	/* Array index: 0x1424 */ 0x59CB,0x6337,
	/* Array index: 0x1424 */ 0x59E6,0x4A4D,
	/* Array index: 0x1424 */ 0x59E8,0x6C29,
	/* Array index: 0x1424 */ 0x59EA,0x726B,
	/* Array index: 0x1424 */ 0x59EC,0x7D6F,
	/* Array index: 0x1424 */ 0x59EE,0x7973,
	/* Array index: 0x1424 */ 0x59F8,0x6641,
	/* Array index: 0x1424 */ 0x59FB,0x6C58,
	/* Array index: 0x1424 */ 0x59FF,0x6D2C,
	/* Array index: 0x1424 */ 0x5A01,0x6A4E,
	/* Array index: 0x1424 */ 0x5A03,0x685F,
	/* Array index: 0x1424 */ 0x5A11,0x5E4F,
	/* Array index: 0x1424 */ 0x5A18,0x5226,
	/* Array index: 0x1424 */ 0x5A1B,0x6774,
	/* Array index: 0x1424 */ 0x5A1C,0x5156,
	/* Array index: 0x1424 */ 0x5A1F,0x6642,
	/* Array index: 0x1424 */ 0x5A20,0x6363,
	/* Array index: 0x1424 */ 0x5A25,0x6430,
	/* Array index: 0x1424 */ 0x5A29,0x5834,
	/* Array index: 0x1424 */ 0x5A36,0x7625,
	/* Array index: 0x1424 */ 0x5A3C,0x735E,
	/* Array index: 0x1424 */ 0x5A41,0x5725,
	/* Array index: 0x1424 */ 0x5A46,0x7768,
	/* Array index: 0x1424 */ 0x5A49,0x6846,
	/* Array index: 0x1424 */ 0x5A5A,0x7B66,
	/* Array index: 0x1424 */ 0x5A62,0x5D66,
	/* Array index: 0x1424 */ 0x5A66,0x5C7E,
	/* Array index: 0x1424 */ 0x5A92,0x585A,
	/* Array index: 0x1424 */ 0x5A9A,0x5A2C,
	/* Array index: 0x1424 */ 0x5A9B,0x6A30,
	/* Array index: 0x1424 */ 0x5AA4,0x6338,
	/* Array index: 0x1424 */ 0x5AC1,0x4A2A,
	/* Array index: 0x1424 */ 0x5AC2,0x6179,
	/* Array index: 0x1424 */ 0x5AC4,0x6A31,
	/* Array index: 0x1424 */ 0x5AC9,0x726C,
	/* Array index: 0x1424 */ 0x5ACC,0x7A6E,
	/* Array index: 0x1424 */ 0x5AE1,0x6E55,
	/* Array index: 0x1424 */ 0x5AE6,0x7974,
	/* Array index: 0x1424 */ 0x5AE9,0x526C,
	/* Array index: 0x1424 */ 0x5B05,0x7B7B,
	/* Array index: 0x1424 */ 0x5B09,0x7D70,
	/* Array index: 0x1424 */ 0x5B0B,0x603D,
	/* Array index: 0x1424 */ 0x5B0C,0x4E63,
	/* Array index: 0x1424 */ 0x5B16,0x7846,
	/* Array index: 0x1424 */ 0x5B2A,0x5E2E,
	/* Array index: 0x1424 */ 0x5B40,0x5F45,
	/* Array index: 0x1424 */ 0x5B43,0x653E,
	/* Array index: 0x1424 */ 0x5B50,0x6D2D,
	/* Array index: 0x1424 */ 0x5B51,0x7A6A,
	/* Array index: 0x1424 */ 0x5B63,0x4C79,
	/* Array index: 0x1424 */ 0x5B64,0x4D35,
	/* Array index: 0x1424 */ 0x5B69,0x7A29,
	/* Array index: 0x1424 */ 0x5B6B,0x615D,
	/* Array index: 0x1424 */ 0x5B70,0x6255,
	/* Array index: 0x1424 */ 0x5B71,0x6D4F,
	/* Array index: 0x1424 */ 0x5B75,0x5D22,
	/* Array index: 0x1424 */ 0x5B78,0x794A,
	/* Array index: 0x1424 */ 0x5B7A,0x6A68,
	/* Array index: 0x1424 */ 0x5B7C,0x656D,
	/* Array index: 0x1424 */ 0x5B8F,0x4E5B,
	/* Array index: 0x1424 */ 0x5BAC,0x6074,
	/* Array index: 0x1424 */ 0x5BAE,0x4F60,
	/* Array index: 0x1424 */ 0x5BB0,0x6E25,
	/* Array index: 0x1424 */ 0x5BCC,0x5D23,
	/* Array index: 0x1424 */ 0x5BD7,0x523C,
	/* Array index: 0x1424 */ 0x5BF5,0x7541,
	/* Array index: 0x1424 */ 0x5BF6,0x5C44,
	/* Array index: 0x1424 */ 0x5BF8,0x753B,
	/* Array index: 0x1424 */ 0x5BFA,0x5E51,
	/* Array index: 0x1424 */ 0x5C01,0x5C66,
	/* Array index: 0x1424 */ 0x5C04,0x5E52,
	/* Array index: 0x1424 */ 0x5C16,0x7453,
	/* Array index: 0x1424 */ 0x5C19,0x5F46,
	/* Array index: 0x1424 */ 0x5C24,0x6956,
	/* Array index: 0x1424 */ 0x5C28,0x5B2B,
	/* Array index: 0x1424 */ 0x5C31,0x7626,
	/* Array index: 0x1424 */ 0x5C45,0x4B5C,
	/* Array index: 0x1424 */ 0x5C46,0x4C7A,
	/* Array index: 0x1424 */ 0x5C48,0x4F5D,
	/* Array index: 0x1424 */ 0x5C4B,0x6829,
	/* Array index: 0x1424 */ 0x5C4D,0x633B,
	/* Array index: 0x1424 */ 0x5C4E,0x633A,
	/* Array index: 0x1424 */ 0x5C51,0x605A,
	/* Array index: 0x1424 */ 0x5C55,0x6E77,
	/* Array index: 0x1424 */ 0x5C5B,0x5C33,
	/* Array index: 0x1424 */ 0x5C60,0x5375,
	/* Array index: 0x1424 */ 0x5C62,0x5726,
	/* Array index: 0x1424 */ 0x5C64,0x7635,
	/* Array index: 0x1424 */ 0x5C65,0x575B,
	/* Array index: 0x1424 */ 0x5C6C,0x6155,
	/* Array index: 0x1424 */ 0x5C6F,0x546A,
	/* Array index: 0x1424 */ 0x5C71,0x5F23,
	/* Array index: 0x1424 */ 0x5C79,0x7D5F,
	/* Array index: 0x1424 */ 0x5C90,0x5077,
	/* Array index: 0x1424 */ 0x5C91,0x6D54,
	/* Array index: 0x1424 */ 0x5CA1,0x4B2A,
	/* Array index: 0x1424 */ 0x5CA9,0x645B,
	/* Array index: 0x1424 */ 0x5CAB,0x617B,
	/* Array index: 0x1424 */ 0x5CAC,0x4B22,
	/* Array index: 0x1424 */ 0x5CB1,0x5360,
	/* Array index: 0x1424 */ 0x5CB3,0x643F,
	/* Array index: 0x1424 */ 0x5CB5,0x7B40,
	/* Array index: 0x1424 */ 0x5CB7,0x5A3E,
	/* Array index: 0x1424 */ 0x5CB8,0x644D,
	/* Array index: 0x1424 */ 0x5CBA,0x5639,
	/* Array index: 0x1424 */ 0x5CBE,0x6F40,
	/* Array index: 0x1424 */ 0x5CC0,0x617C,
	/* Array index: 0x1424 */ 0x5CD9,0x7639,
	/* Array index: 0x1424 */ 0x5CE0,0x5F47,
	/* Array index: 0x1424 */ 0x5CE8,0x6431,
	/* Array index: 0x1424 */ 0x5CEF,0x5C67,
	/* Array index: 0x1424 */ 0x5CF0,0x5C68,
	/* Array index: 0x1424 */ 0x5CF4,0x7A56,
	/* Array index: 0x1424 */ 0x5CF6,0x5376,
	/* Array index: 0x1424 */ 0x5CFB,0x715A,
	/* Array index: 0x1424 */ 0x5CFD,0x7A72,
	/* Array index: 0x1424 */ 0x5D07,0x627D,
	/* Array index: 0x1424 */ 0x5D0D,0x554F,
	/* Array index: 0x1424 */ 0x5D0E,0x5078,
	/* Array index: 0x1424 */ 0x5D11,0x4D5F,
	/* Array index: 0x1424 */ 0x5D14,0x754B,
	/* Array index: 0x1424 */ 0x5D16,0x6470,
	/* Array index: 0x1424 */ 0x5D17,0x4B2B,
	/* Array index: 0x1424 */ 0x5D19,0x5744,
	/* Array index: 0x1424 */ 0x5D27,0x627E,
	/* Array index: 0x1424 */ 0x5D29,0x5D5A,
	/* Array index: 0x1424 */ 0x5D4B,0x5A2E,
	/* Array index: 0x1424 */ 0x5D4C,0x4A6E,
	/* Array index: 0x1424 */ 0x5D50,0x5539,
	/* Array index: 0x1424 */ 0x5D69,0x6321,
	/* Array index: 0x1424 */ 0x5D6C,0x6863,
	/* Array index: 0x1424 */ 0x5D6F,0x732B,
	/* Array index: 0x1424 */ 0x5D87,0x4F29,
	/* Array index: 0x1424 */ 0x5D8B,0x5377,
	/* Array index: 0x1424 */ 0x5D9D,0x5471,
	/* Array index: 0x1424 */ 0x5DA0,0x4E64,
	/* Array index: 0x1424 */ 0x5DA2,0x6872,
	/* Array index: 0x1424 */ 0x5DAA,0x6575,
	/* Array index: 0x1424 */ 0x5DB8,0x672E,
	/* Array index: 0x1424 */ 0x5DBA,0x563A,
	/* Array index: 0x1424 */ 0x5DBC,0x5F6C,
	/* Array index: 0x1424 */ 0x5DBD,0x6440,
	/* Array index: 0x1424 */ 0x5DCD,0x6864,
	/* Array index: 0x1424 */ 0x5DD2,0x5835,
	/* Array index: 0x1424 */ 0x5DD6,0x645C,
	/* Array index: 0x1424 */ 0x5DDD,0x7439,
	/* Array index: 0x1424 */ 0x5DDE,0x7136,
	/* Array index: 0x1424 */ 0x5DEB,0x5963,
	/* Array index: 0x1424 */ 0x5DEE,0x732C,
	/* Array index: 0x1424 */ 0x5DF7,0x7975,
	/* Array index: 0x1424 */ 0x5DFD,0x615E,
	/* Array index: 0x1424 */ 0x5DFE,0x4B6E,
	/* Array index: 0x1424 */ 0x5E02,0x633C,
	/* Array index: 0x1424 */ 0x5E03,0x7856,
	/* Array index: 0x1424 */ 0x5E06,0x5B6E,
	/* Array index: 0x1424 */ 0x5E0C,0x7D71,
	/* Array index: 0x1424 */ 0x5E11,0x7736,
	/* Array index: 0x1424 */ 0x5E16,0x745E,
	/* Array index: 0x1424 */ 0x5E19,0x726D,
	/* Array index: 0x1424 */ 0x5E1B,0x5B59,
	/* Array index: 0x1424 */ 0x5E1D,0x7028,
	/* Array index: 0x1424 */ 0x5E25,0x617D,
	/* Array index: 0x1424 */ 0x5E2B,0x5E54,
	/* Array index: 0x1424 */ 0x5E2D,0x602C,
	/* Array index: 0x1424 */ 0x5E33,0x6D63,
	/* Array index: 0x1424 */ 0x5E36,0x5361,
	/* Array index: 0x1424 */ 0x5E38,0x5F48,
	/* Array index: 0x1424 */ 0x5E3D,0x5936,
	/* Array index: 0x1424 */ 0x5E3F,0x7D2C,
	/* Array index: 0x1424 */ 0x5E40,0x6F53,
	/* Array index: 0x1424 */ 0x5E44,0x6441,
	/* Array index: 0x1424 */ 0x5E45,0x786B,
	/* Array index: 0x1424 */ 0x5E47,0x5B2C,
	/* Array index: 0x1424 */ 0x5E4C,0x7C46,
	/* Array index: 0x1424 */ 0x5E55,0x582D,
	/* Array index: 0x1424 */ 0x5E84,0x6D64,
	/* Array index: 0x1424 */ 0x5E87,0x5D67,
	/* Array index: 0x1424 */ 0x5E8A,0x5F49,
	/* Array index: 0x1424 */ 0x5E8F,0x5F6D,
	/* Array index: 0x1424 */ 0x5E95,0x6E3C,
	/* Array index: 0x1424 */ 0x5E97,0x6F41,
	/* Array index: 0x1424 */ 0x5E9A,0x4C52,
	/* Array index: 0x1424 */ 0x5E9C,0x5D24,
	/* Array index: 0x1424 */ 0x5EA0,0x5F4A,
	/* Array index: 0x1424 */ 0x5EA6,0x5378,
	/* Array index: 0x1424 */ 0x5EA7,0x7128,
	/* Array index: 0x1424 */ 0x5EAB,0x4D37,
	/* Array index: 0x1424 */ 0x5EAD,0x6F54,
	/* Array index: 0x1424 */ 0x5EBE,0x6A6C,
	/* Array index: 0x1424 */ 0x5EC2,0x5F4B,
	/* Array index: 0x1424 */ 0x5ED0,0x4F2A,
	/* Array index: 0x1424 */ 0x5ED3,0x4E29,
	/* Array index: 0x1424 */ 0x5ED6,0x5678,
	/* Array index: 0x1424 */ 0x5EDA,0x7137,
	/* Array index: 0x1424 */ 0x5EDB,0x6E78,
	/* Array index: 0x1424 */ 0x5EEC,0x5566,
	/* Array index: 0x1424 */ 0x5EF3,0x7466,
	/* Array index: 0x1424 */ 0x5EF6,0x6645,
	/* Array index: 0x1424 */ 0x5EF7,0x6F55,
	/* Array index: 0x1424 */ 0x5EFA,0x4B6F,
	/* Array index: 0x1424 */ 0x5EFB,0x7C5F,
	/* Array index: 0x1424 */ 0x5F01,0x5C27,
	/* Array index: 0x1424 */ 0x5F04,0x5667,
	/* Array index: 0x1424 */ 0x5F0A,0x7849,
	/* Array index: 0x1424 */ 0x5F0F,0x6352,
	/* Array index: 0x1424 */ 0x5F1B,0x6C2C,
	/* Array index: 0x1424 */ 0x5F1F,0x7029,
	/* Array index: 0x1424 */ 0x5F26,0x7A57,
	/* Array index: 0x1424 */ 0x5F27,0x7B41,
	/* Array index: 0x1424 */ 0x5F29,0x5240,
	/* Array index: 0x1424 */ 0x5F31,0x6530,
	/* Array index: 0x1424 */ 0x5F35,0x6D65,
	/* Array index: 0x1424 */ 0x5F3A,0x4B2D,
	/* Array index: 0x1424 */ 0x5F3C,0x7930,
	/* Array index: 0x1424 */ 0x5F48,0x7725,
	/* Array index: 0x1424 */ 0x5F4A,0x4B2E,
	/* Array index: 0x1424 */ 0x5F4C,0x5A2F,
	/* Array index: 0x1424 */ 0x5F4E,0x5836,
	/* Array index: 0x1424 */ 0x5F56,0x5327,
	/* Array index: 0x1424 */ 0x5F57,0x7B32,
	/* Array index: 0x1424 */ 0x5F59,0x7D44,
	/* Array index: 0x1424 */ 0x5F5B,0x6C2D,
	/* Array index: 0x1424 */ 0x5F62,0x7B21,
	/* Array index: 0x1424 */ 0x5F77,0x5B2D,
	/* Array index: 0x1424 */ 0x5F79,0x6635,
	/* Array index: 0x1424 */ 0x5F7C,0x7928,
	/* Array index: 0x1424 */ 0x5F85,0x5362,
	/* Array index: 0x1424 */ 0x5F87,0x625F,
	/* Array index: 0x1424 */ 0x5F9E,0x7074,
	/* Array index: 0x1424 */ 0x5FA0,0x5550,
	/* Array index: 0x1424 */ 0x5FA1,0x6559,
	/* Array index: 0x1424 */ 0x5FAE,0x5A30,
	/* Array index: 0x1424 */ 0x5FB5,0x7323,
	/* Array index: 0x1424 */ 0x5FB7,0x536C,
	/* Array index: 0x1424 */ 0x5FB9,0x744B,
	/* Array index: 0x1424 */ 0x5FBD,0x7D45,
	/* Array index: 0x1424 */ 0x5FC3,0x637D,
	/* Array index: 0x1424 */ 0x5FC5,0x7931,
	/* Array index: 0x1424 */ 0x5FCC,0x507B,
	/* Array index: 0x1424 */ 0x5FCD,0x6C5B,
	/* Array index: 0x1424 */ 0x5FE0,0x7577,
	/* Array index: 0x1424 */ 0x5FEB,0x7661,
	/* Array index: 0x1424 */ 0x5FF5,0x5237,
	/* Array index: 0x1424 */ 0x5FFD,0x7B6C,
	/* Array index: 0x1424 */ 0x5FFF,0x5D48,
	/* Array index: 0x1424 */ 0x600F,0x6468,
	/* Array index: 0x1424 */ 0x6012,0x5241,
	/* Array index: 0x1424 */ 0x6016,0x7857,
	/* Array index: 0x1424 */ 0x601C,0x563B,
	/* Array index: 0x1424 */ 0x601D,0x5E56,
	/* Array index: 0x1424 */ 0x6020,0x773D,
	/* Array index: 0x1424 */ 0x6021,0x6C2E,
	/* Array index: 0x1424 */ 0x6025,0x5061,
	/* Array index: 0x1424 */ 0x6027,0x6075,
	/* Array index: 0x1424 */ 0x6028,0x6A33,
	/* Array index: 0x1424 */ 0x602A,0x4E56,
	/* Array index: 0x1424 */ 0x602F,0x4C25,
	/* Array index: 0x1424 */ 0x604D,0x7C48,
	/* Array index: 0x1424 */ 0x6050,0x4D70,
	/* Array index: 0x1424 */ 0x6052,0x7976,
	/* Array index: 0x1424 */ 0x6055,0x5F70,
	/* Array index: 0x1424 */ 0x6059,0x653F,
	/* Array index: 0x1424 */ 0x605D,0x4E3F,
	/* Array index: 0x1424 */ 0x6085,0x666D,
	/* Array index: 0x1424 */ 0x6089,0x637A,
	/* Array index: 0x1424 */ 0x608C,0x702A,
	/* Array index: 0x1424 */ 0x608D,0x7950,
	/* Array index: 0x1424 */ 0x6094,0x7C62,
	/* Array index: 0x1424 */ 0x6096,0x7827,
	/* Array index: 0x1424 */ 0x609A,0x6165,
	/* Array index: 0x1424 */ 0x609B,0x6E79,
	/* Array index: 0x1424 */ 0x609F,0x6776,
	/* Array index: 0x1424 */ 0x60A0,0x6A6D,
	/* Array index: 0x1424 */ 0x60A3,0x7C34,
	/* Array index: 0x1424 */ 0x60A4,0x7542,
	/* Array index: 0x1424 */ 0x60A7,0x575C,
	/* Array index: 0x1424 */ 0x60BC,0x537A,
	/* Array index: 0x1424 */ 0x60BD,0x7424,
	/* Array index: 0x1424 */ 0x60C5,0x6F57,
	/* Array index: 0x1424 */ 0x60C7,0x5443,
	/* Array index: 0x1424 */ 0x60D1,0x7B63,
	/* Array index: 0x1424 */ 0x60DA,0x7B6D,
	/* Array index: 0x1424 */ 0x60DC,0x602D,
	/* Array index: 0x1424 */ 0x60F0,0x7667,
	/* Array index: 0x1424 */ 0x60F1,0x525D,
	/* Array index: 0x1424 */ 0x60F3,0x5F4C,
	/* Array index: 0x1424 */ 0x60F6,0x7C49,
	/* Array index: 0x1424 */ 0x6101,0x617E,
	/* Array index: 0x1424 */ 0x6106,0x4B70,
	/* Array index: 0x1424 */ 0x6108,0x6A6F,
	/* Array index: 0x1424 */ 0x6109,0x6A70,
	/* Array index: 0x1424 */ 0x6115,0x6443,
	/* Array index: 0x1424 */ 0x611A,0x6957,
	/* Array index: 0x1424 */ 0x611B,0x6471,
	/* Array index: 0x1424 */ 0x611F,0x4A6F,
	/* Array index: 0x1424 */ 0x6127,0x4E57,
	/* Array index: 0x1424 */ 0x6130,0x7C4A,
	/* Array index: 0x1424 */ 0x6134,0x7361,
	/* Array index: 0x1424 */ 0x6137,0x4B44,
	/* Array index: 0x1424 */ 0x613C,0x6365,
	/* Array index: 0x1424 */ 0x613E,0x4B45,
	/* Array index: 0x1424 */ 0x613F,0x6A34,
	/* Array index: 0x1424 */ 0x6142,0x693D,
	/* Array index: 0x1424 */ 0x6144,0x5749,
	/* Array index: 0x1424 */ 0x6153,0x7874,
	/* Array index: 0x1424 */ 0x6155,0x5937,
	/* Array index: 0x1424 */ 0x6158,0x7353,
	/* Array index: 0x1424 */ 0x6159,0x7354,
	/* Array index: 0x1424 */ 0x615D,0x7764,
	/* Array index: 0x1424 */ 0x615F,0x7751,
	/* Array index: 0x1424 */ 0x616B,0x7076,
	/* Array index: 0x1424 */ 0x616E,0x5567,
	/* Array index: 0x1424 */ 0x6170,0x6A50,
	/* Array index: 0x1424 */ 0x6176,0x4C54,
	/* Array index: 0x1424 */ 0x6177,0x4B2F,
	/* Array index: 0x1424 */ 0x617D,0x742A,
	/* Array index: 0x1424 */ 0x617E,0x692F,
	/* Array index: 0x1424 */ 0x6181,0x7543,
	/* Array index: 0x1424 */ 0x6182,0x6958,
	/* Array index: 0x1424 */ 0x618A,0x5D69,
	/* Array index: 0x1424 */ 0x618E,0x7173,
	/* Array index: 0x1424 */ 0x6190,0x557B,
	/* Array index: 0x1424 */ 0x6191,0x5E3B,
	/* Array index: 0x1424 */ 0x6194,0x747B,
	/* Array index: 0x1424 */ 0x61A4,0x5D49,
	/* Array index: 0x1424 */ 0x61A7,0x5453,
	/* Array index: 0x1424 */ 0x61A9,0x4C28,
	/* Array index: 0x1424 */ 0x61AB,0x5A41,
	/* Array index: 0x1424 */ 0x61AC,0x4C55,
	/* Array index: 0x1424 */ 0x61AE,0x5964,
	/* Array index: 0x1424 */ 0x61B2,0x7A4A,
	/* Array index: 0x1424 */ 0x61B6,0x6563,
	/* Array index: 0x1424 */ 0x61BA,0x533C,
	/* Array index: 0x1424 */ 0x61BE,0x4A70,
	/* Array index: 0x1424 */ 0x61C3,0x5044,
	/* Array index: 0x1424 */ 0x61E6,0x5157,
	/* Array index: 0x1424 */ 0x61F2,0x7324,
	/* Array index: 0x1424 */ 0x6207,0x5354,
	/* Array index: 0x1424 */ 0x6208,0x4D7C,
	/* Array index: 0x1424 */ 0x6216,0x7B64,
	/* Array index: 0x1424 */ 0x621A,0x742B,
	/* Array index: 0x1424 */ 0x621F,0x503D,
	/* Array index: 0x1424 */ 0x6221,0x4A71,
	/* Array index: 0x1424 */ 0x622A,0x6F38,
	/* Array index: 0x1424 */ 0x622E,0x5740,
	/* Array index: 0x1424 */ 0x6230,0x6E7A,
	/* Array index: 0x1424 */ 0x6231,0x7D74,
	/* Array index: 0x1424 */ 0x6234,0x5363,
	/* Array index: 0x1424 */ 0x6236,0x7B42,
	/* Array index: 0x1424 */ 0x6253,0x7668,
	/* Array index: 0x1424 */ 0x6258,0x7675,
	/* Array index: 0x1424 */ 0x626E,0x5D4A,
	/* Array index: 0x1424 */ 0x6271,0x5062,
	/* Array index: 0x1424 */ 0x6276,0x5D26,
	/* Array index: 0x1424 */ 0x6279,0x5D6B,
	/* Array index: 0x1424 */ 0x627C,0x6479,
	/* Array index: 0x1424 */ 0x627F,0x632F,
	/* Array index: 0x1424 */ 0x6280,0x507C,
	/* Array index: 0x1424 */ 0x6284,0x747C,
	/* Array index: 0x1424 */ 0x6289,0x4C3C,
	/* Array index: 0x1424 */ 0x628A,0x776A,
	/* Array index: 0x1424 */ 0x6291,0x6564,
	/* Array index: 0x1424 */ 0x6292,0x5F71,
	/* Array index: 0x1424 */ 0x6295,0x7761,
	/* Array index: 0x1424 */ 0x6297,0x7977,
	/* Array index: 0x1424 */ 0x6298,0x6F39,
	/* Array index: 0x1424 */ 0x629B,0x7858,
	/* Array index: 0x1424 */ 0x62AB,0x7929,
	/* Array index: 0x1424 */ 0x62B1,0x7859,
	/* Array index: 0x1424 */ 0x62B5,0x6E3D,
	/* Array index: 0x1424 */ 0x62B9,0x5846,
	/* Array index: 0x1424 */ 0x62BC,0x6463,
	/* Array index: 0x1424 */ 0x62BD,0x754E,
	/* Array index: 0x1424 */ 0x62C2,0x5D59,
	/* Array index: 0x1424 */ 0x62F7,0x4D38,
	/* Array index: 0x1424 */ 0x62FE,0x6326,
	/* Array index: 0x1424 */ 0x62FF,0x515A,
	/* Array index: 0x1424 */ 0x6301,0x7225,
	/* Array index: 0x1424 */ 0x6307,0x7226,
	/* Array index: 0x1424 */ 0x6309,0x644E,
	/* Array index: 0x1424 */ 0x6311,0x537B,
	/* Array index: 0x1424 */ 0x632B,0x7129,
	/* Array index: 0x1424 */ 0x632F,0x7249,
	/* Array index: 0x1424 */ 0x6349,0x7335,
	/* Array index: 0x1424 */ 0x634C,0x7824,
	/* Array index: 0x1424 */ 0x634F,0x5173,
	/* Array index: 0x1424 */ 0x6350,0x6648,
	/* Array index: 0x1424 */ 0x6355,0x785A,
	/* Array index: 0x1424 */ 0x6367,0x5C69,
	/* Array index: 0x1424 */ 0x6368,0x5E57,
	/* Array index: 0x1424 */ 0x636E,0x4B5F,
	/* Array index: 0x1424 */ 0x6372,0x4F6C,
	/* Array index: 0x1424 */ 0x6377,0x745F,
	/* Array index: 0x1424 */ 0x637A,0x5174,
	/* Array index: 0x1424 */ 0x637B,0x523A,
	/* Array index: 0x1424 */ 0x637F,0x5F72,
	/* Array index: 0x1424 */ 0x6383,0x6137,
	/* Array index: 0x1424 */ 0x6388,0x6223,
	/* Array index: 0x1424 */ 0x6389,0x537C,
	/* Array index: 0x1424 */ 0x638C,0x6D66,
	/* Array index: 0x1424 */ 0x6392,0x5B49,
	/* Array index: 0x1424 */ 0x6396,0x647A,
	/* Array index: 0x1424 */ 0x6398,0x4F5E,
	/* Array index: 0x1424 */ 0x639B,0x4E50,
	/* Array index: 0x1424 */ 0x63C0,0x4A51,
	/* Array index: 0x1424 */ 0x63C4,0x6A71,
	/* Array index: 0x1424 */ 0x63C6,0x5026,
	/* Array index: 0x1424 */ 0x63CF,0x595A,
	/* Array index: 0x1424 */ 0x63D0,0x702B,
	/* Array index: 0x1424 */ 0x63D6,0x6B67,
	/* Array index: 0x1424 */ 0x63DA,0x6540,
	/* Array index: 0x1424 */ 0x63DB,0x7C35,
	/* Array index: 0x1424 */ 0x63E1,0x6444,
	/* Array index: 0x1424 */ 0x63ED,0x4C29,
	/* Array index: 0x1424 */ 0x63EE,0x7D46,
	/* Array index: 0x1424 */ 0x63F4,0x6A35,
	/* Array index: 0x1424 */ 0x63F6,0x652A,
	/* Array index: 0x1424 */ 0x63F7,0x5F3A,
	/* Array index: 0x1424 */ 0x640D,0x615F,
	/* Array index: 0x1424 */ 0x640F,0x5A51,
	/* Array index: 0x1424 */ 0x6414,0x6138,
	/* Array index: 0x1424 */ 0x6416,0x6874,
	/* Array index: 0x1424 */ 0x6417,0x537D,
	/* Array index: 0x1424 */ 0x641C,0x6224,
	/* Array index: 0x1424 */ 0x6422,0x724A,
	/* Array index: 0x1424 */ 0x642C,0x5A66,
	/* Array index: 0x1424 */ 0x642D,0x7733,
	/* Array index: 0x1424 */ 0x643A,0x7D4D,
	/* Array index: 0x1424 */ 0x643E,0x7336,
	/* Array index: 0x1424 */ 0x6458,0x6E57,
	/* Array index: 0x1424 */ 0x6460,0x7544,
	/* Array index: 0x1424 */ 0x6469,0x5824,
	/* Array index: 0x1424 */ 0x646F,0x7227,
	/* Array index: 0x1424 */ 0x6488,0x564E,
	/* Array index: 0x1424 */ 0x649A,0x5235,
	/* Array index: 0x1424 */ 0x649E,0x5355,
	/* Array index: 0x1424 */ 0x64A4,0x744C,
	/* Array index: 0x1424 */ 0x64A5,0x5A7C,
	/* Array index: 0x1424 */ 0x64AB,0x5968,
	/* Array index: 0x1424 */ 0x64AD,0x776B,
	/* Array index: 0x1424 */ 0x64AE,0x7549,
	/* Array index: 0x1424 */ 0x64B0,0x733C,
	/* Array index: 0x1424 */ 0x64B2,0x5A52,
	/* Array index: 0x1424 */ 0x64BB,0x5335,
	/* Array index: 0x1424 */ 0x64C1,0x6836,
	/* Array index: 0x1424 */ 0x64C4,0x564F,
	/* Array index: 0x1424 */ 0x64C5,0x743A,
	/* Array index: 0x1424 */ 0x64C7,0x7749,
	/* Array index: 0x1424 */ 0x64CA,0x4C2A,
	/* Array index: 0x1424 */ 0x64CD,0x7043,
	/* Array index: 0x1424 */ 0x64CE,0x4C56,
	/* Array index: 0x1424 */ 0x64D2,0x5053,
	/* Array index: 0x1424 */ 0x64D4,0x533D,
	/* Array index: 0x1424 */ 0x64D8,0x5B7B,
	/* Array index: 0x1424 */ 0x64DA,0x4B60,
	/* Array index: 0x1424 */ 0x64EC,0x6B74,
	/* Array index: 0x1424 */ 0x64F2,0x742D,
	/* Array index: 0x1424 */ 0x64F4,0x7C2A,
	/* Array index: 0x1424 */ 0x64FA,0x776C,
	/* Array index: 0x1424 */ 0x64FE,0x6876,
	/* Array index: 0x1424 */ 0x6500,0x5A67,
	/* Array index: 0x1424 */ 0x6504,0x774C,
	/* Array index: 0x1424 */ 0x6518,0x6541,
	/* Array index: 0x1424 */ 0x651D,0x606E,
	/* Array index: 0x1424 */ 0x6523,0x557D,
	/* Array index: 0x1424 */ 0x652F,0x7228,
	/* Array index: 0x1424 */ 0x6545,0x4D3A,
	/* Array index: 0x1424 */ 0x6548,0x7C79,
	/* Array index: 0x1424 */ 0x6556,0x6779,
	/* Array index: 0x1424 */ 0x6557,0x7828,
	/* Array index: 0x1424 */ 0x655E,0x7362,
	/* Array index: 0x1424 */ 0x6562,0x4A72,
	/* Array index: 0x1424 */ 0x6563,0x5F24,
	/* Array index: 0x1424 */ 0x6566,0x5444,
	/* Array index: 0x1424 */ 0x656C,0x4C57,
	/* Array index: 0x1424 */ 0x656D,0x6542,
	/* Array index: 0x1424 */ 0x6572,0x4D3B,
	/* Array index: 0x1424 */ 0x657E,0x6040,
	/* Array index: 0x1424 */ 0x6582,0x5630,
	/* Array index: 0x1424 */ 0x6583,0x784A,
	/* Array index: 0x1424 */ 0x6585,0x7C7A,
	/* Array index: 0x1424 */ 0x6587,0x597E,
	/* Array index: 0x1424 */ 0x658C,0x5E30,
	/* Array index: 0x1424 */ 0x6590,0x5D6C,
	/* Array index: 0x1424 */ 0x6591,0x5A68,
	/* Array index: 0x1424 */ 0x6597,0x5460,
	/* Array index: 0x1424 */ 0x6599,0x5679,
	/* Array index: 0x1424 */ 0x659B,0x4D57,
	/* Array index: 0x1424 */ 0x659C,0x5E58,
	/* Array index: 0x1424 */ 0x659F,0x7278,
	/* Array index: 0x1424 */ 0x65A1,0x6456,
	/* Array index: 0x1424 */ 0x65A4,0x5045,
	/* Array index: 0x1424 */ 0x65A5,0x742E,
	/* Array index: 0x1424 */ 0x65A7,0x5D28,
	/* Array index: 0x1424 */ 0x65AB,0x6D45,
	/* Array index: 0x1424 */ 0x65AC,0x7356,
	/* Array index: 0x1424 */ 0x65AF,0x5E59,
	/* Array index: 0x1424 */ 0x65B0,0x6366,
	/* Array index: 0x1424 */ 0x65B7,0x5328,
	/* Array index: 0x1424 */ 0x65B9,0x5B30,
	/* Array index: 0x1424 */ 0x65BC,0x655A,
	/* Array index: 0x1424 */ 0x65BD,0x633F,
	/* Array index: 0x1424 */ 0x65C1,0x5B31,
	/* Array index: 0x1424 */ 0x65C5,0x5569,
	/* Array index: 0x1424 */ 0x65CB,0x6041,
	/* Array index: 0x1424 */ 0x65CC,0x6F5B,
	/* Array index: 0x1424 */ 0x65CF,0x7069,
	/* Array index: 0x1424 */ 0x65D2,0x5732,
	/* Array index: 0x1424 */ 0x65D7,0x507D,
	/* Array index: 0x1424 */ 0x65E0,0x5969,
	/* Array index: 0x1424 */ 0x65E3,0x507E,
	/* Array index: 0x1424 */ 0x65F1,0x7951,
	/* Array index: 0x1424 */ 0x65F4,0x6959,
	/* Array index: 0x1424 */ 0x6625,0x7570,
	/* Array index: 0x1424 */ 0x6627,0x585C,
	/* Array index: 0x1424 */ 0x6628,0x6D46,
	/* Array index: 0x1424 */ 0x6634,0x595B,
	/* Array index: 0x1424 */ 0x6636,0x7364,
	/* Array index: 0x1424 */ 0x663A,0x5C36,
	/* Array index: 0x1424 */ 0x663B,0x6469,
	/* Array index: 0x1424 */ 0x6649,0x724B,
	/* Array index: 0x1424 */ 0x664B,0x724C,
	/* Array index: 0x1424 */ 0x664F,0x644F,
	/* Array index: 0x1424 */ 0x6659,0x715B,
	/* Array index: 0x1424 */ 0x6684,0x7D3E,
	/* Array index: 0x1424 */ 0x668E,0x6732,
	/* Array index: 0x1424 */ 0x6690,0x6A51,
	/* Array index: 0x1424 */ 0x6691,0x5F74,
	/* Array index: 0x1424 */ 0x669D,0x5926,
	/* Array index: 0x1424 */ 0x66A0,0x4D3C,
	/* Array index: 0x1424 */ 0x66A2,0x7365,
	/* Array index: 0x1424 */ 0x66AB,0x6D55,
	/* Array index: 0x1424 */ 0x66AE,0x593A,
	/* Array index: 0x1424 */ 0x66B9,0x6067,
	/* Array index: 0x1424 */ 0x66BB,0x4C59,
	/* Array index: 0x1424 */ 0x66BE,0x5446,
	/* Array index: 0x1424 */ 0x66C4,0x6725,
	/* Array index: 0x1424 */ 0x66C6,0x5575,
	/* Array index: 0x1424 */ 0x66C7,0x533E,
	/* Array index: 0x1424 */ 0x66C9,0x7C7B,
	/* Array index: 0x1424 */ 0x66D6,0x6472,
	/* Array index: 0x1424 */ 0x66D9,0x5F75,
	/* Array index: 0x1424 */ 0x66DC,0x6878,
	/* Array index: 0x1424 */ 0x66DD,0x786D,
	/* Array index: 0x1424 */ 0x66E0,0x4E47,
	/* Array index: 0x1424 */ 0x66E6,0x7D76,
	/* Array index: 0x1424 */ 0x6703,0x7C65,
	/* Array index: 0x1424 */ 0x6708,0x6A45,
	/* Array index: 0x1424 */ 0x6709,0x6A73,
	/* Array index: 0x1424 */ 0x670B,0x5D5B,
	/* Array index: 0x1424 */ 0x670D,0x5C57,
	/* Array index: 0x1424 */ 0x6714,0x5E7D,
	/* Array index: 0x1424 */ 0x6715,0x7279,
	/* Array index: 0x1424 */ 0x6717,0x5547,
	/* Array index: 0x1424 */ 0x6731,0x7139,
	/* Array index: 0x1424 */ 0x6734,0x5A53,
	/* Array index: 0x1424 */ 0x6736,0x766A,
	/* Array index: 0x1424 */ 0x673A,0x4F75,
	/* Array index: 0x1424 */ 0x673D,0x7D2E,
	/* Array index: 0x1424 */ 0x6746,0x4A52,
	/* Array index: 0x1424 */ 0x6749,0x5F34,
	/* Array index: 0x1424 */ 0x6756,0x6D68,
	/* Array index: 0x1424 */ 0x675C,0x5461,
	/* Array index: 0x1424 */ 0x675E,0x5123,
	/* Array index: 0x1424 */ 0x675F,0x6156,
	/* Array index: 0x1424 */ 0x677B,0x526E,
	/* Array index: 0x1424 */ 0x677E,0x6166,
	/* Array index: 0x1424 */ 0x677F,0x7779,
	/* Array index: 0x1424 */ 0x6787,0x5D6D,
	/* Array index: 0x1424 */ 0x6789,0x685B,
	/* Array index: 0x1424 */ 0x678B,0x5B33,
	/* Array index: 0x1424 */ 0x678F,0x5177,
	/* Array index: 0x1424 */ 0x6790,0x6030,
	/* Array index: 0x1424 */ 0x6793,0x5462,
	/* Array index: 0x1424 */ 0x6795,0x7657,
	/* Array index: 0x1424 */ 0x6797,0x5779,
	/* Array index: 0x1424 */ 0x679A,0x585D,
	/* Array index: 0x1424 */ 0x679C,0x4D7D,
	/* Array index: 0x1424 */ 0x679D,0x722B,
	/* Array index: 0x1424 */ 0x67AF,0x4D3D,
	/* Array index: 0x1424 */ 0x67B0,0x7842,
	/* Array index: 0x1424 */ 0x67B3,0x722C,
	/* Array index: 0x1424 */ 0x67BE,0x6342,
	/* Array index: 0x1424 */ 0x67C4,0x5C37,
	/* Array index: 0x1424 */ 0x67DA,0x6A76,
	/* Array index: 0x1424 */ 0x67DD,0x7679,
	/* Array index: 0x1424 */ 0x67E9,0x4F2F,
	/* Array index: 0x1424 */ 0x67EC,0x4A53,
	/* Array index: 0x1424 */ 0x67FB,0x5E5B,
	/* Array index: 0x1424 */ 0x67FE,0x6F5E,
	/* Array index: 0x1424 */ 0x6812,0x6263,
	/* Array index: 0x1424 */ 0x6813,0x6E7B,
	/* Array index: 0x1424 */ 0x6816,0x5F77,
	/* Array index: 0x1424 */ 0x6817,0x574A,
	/* Array index: 0x1424 */ 0x6821,0x4E68,
	/* Array index: 0x1424 */ 0x6822,0x5B5B,
	/* Array index: 0x1424 */ 0x682A,0x713B,
	/* Array index: 0x1424 */ 0x682F,0x6971,
	/* Array index: 0x1424 */ 0x6838,0x7A37,
	/* Array index: 0x1424 */ 0x6839,0x5046,
	/* Array index: 0x1424 */ 0x6848,0x6450,
	/* Array index: 0x1424 */ 0x684E,0x726E,
	/* Array index: 0x1424 */ 0x686D,0x724D,
	/* Array index: 0x1424 */ 0x6876,0x7752,
	/* Array index: 0x1424 */ 0x687F,0x4A54,
	/* Array index: 0x1424 */ 0x6881,0x5559,
	/* Array index: 0x1424 */ 0x6885,0x585E,
	/* Array index: 0x1424 */ 0x688F,0x4D59,
	/* Array index: 0x1424 */ 0x6893,0x6E29,
	/* Array index: 0x1424 */ 0x6894,0x763C,
	/* Array index: 0x1424 */ 0x6897,0x4C5B,
	/* Array index: 0x1424 */ 0x689D,0x7049,
	/* Array index: 0x1424 */ 0x689F,0x7C7C,
	/* Array index: 0x1424 */ 0x68A1,0x6849,
	/* Array index: 0x1424 */ 0x68A2,0x747E,
	/* Array index: 0x1424 */ 0x68A7,0x677C,
	/* Array index: 0x1424 */ 0x68A8,0x575E,
	/* Array index: 0x1424 */ 0x68C4,0x5125,
	/* Array index: 0x1424 */ 0x68C5,0x5C38,
	/* Array index: 0x1424 */ 0x68C9,0x5876,
	/* Array index: 0x1424 */ 0x68CB,0x5124,
	/* Array index: 0x1424 */ 0x68CD,0x4D62,
	/* Array index: 0x1424 */ 0x68D2,0x5C6A,
	/* Array index: 0x1424 */ 0x68D5,0x7077,
	/* Array index: 0x1424 */ 0x68D7,0x704A,
	/* Array index: 0x1424 */ 0x68D8,0x503E,
	/* Array index: 0x1424 */ 0x68DA,0x5D5C,
	/* Array index: 0x1424 */ 0x68DF,0x5456,
	/* Array index: 0x1424 */ 0x68E0,0x5356,
	/* Array index: 0x1424 */ 0x68E7,0x6D50,
	/* Array index: 0x1424 */ 0x68E8,0x4D21,
	/* Array index: 0x1424 */ 0x68EE,0x5F35,
	/* Array index: 0x1424 */ 0x68F2,0x5F78,
	/* Array index: 0x1424 */ 0x68F9,0x5421,
	/* Array index: 0x1424 */ 0x68FA,0x4E32,
	/* Array index: 0x1424 */ 0x6900,0x684A,
	/* Array index: 0x1424 */ 0x6905,0x6B75,
	/* Array index: 0x1424 */ 0x690D,0x6355,
	/* Array index: 0x1424 */ 0x690E,0x7550,
	/* Array index: 0x1424 */ 0x6912,0x7521,
	/* Array index: 0x1424 */ 0x6927,0x5927,
	/* Array index: 0x1424 */ 0x6930,0x652B,
	/* Array index: 0x1424 */ 0x693D,0x664B,
	/* Array index: 0x1424 */ 0x693F,0x7571,
	/* Array index: 0x1424 */ 0x694A,0x6545,
	/* Array index: 0x1424 */ 0x6968,0x6F5F,
	/* Array index: 0x1424 */ 0x6995,0x693E,
	/* Array index: 0x1424 */ 0x699B,0x724E,
	/* Array index: 0x1424 */ 0x699C,0x5B34,
	/* Array index: 0x1424 */ 0x69A5,0x7C4E,
	/* Array index: 0x1424 */ 0x69A7,0x5D6E,
	/* Array index: 0x1424 */ 0x69AE,0x6734,
	/* Array index: 0x1424 */ 0x69B4,0x5734,
	/* Array index: 0x1424 */ 0x69BB,0x7734,
	/* Array index: 0x1424 */ 0x69C1,0x4D3E,
	/* Array index: 0x1424 */ 0x69C3,0x5A69,
	/* Array index: 0x1424 */ 0x69D0,0x4E59,
	/* Array index: 0x1424 */ 0x69E8,0x4E2A,
	/* Array index: 0x1424 */ 0x69EA,0x4B48,
	/* Array index: 0x1424 */ 0x69FB,0x5027,
	/* Array index: 0x1424 */ 0x69FD,0x704B,
	/* Array index: 0x1424 */ 0x69FF,0x5047,
	/* Array index: 0x1424 */ 0x6A02,0x6445,
	/* Array index: 0x1424 */ 0x6A0A,0x5B60,
	/* Array index: 0x1424 */ 0x6A11,0x555A,
	/* Array index: 0x1424 */ 0x6A13,0x5727,
	/* Array index: 0x1424 */ 0x6A17,0x6E40,
	/* Array index: 0x1424 */ 0x6A19,0x7876,
	/* Array index: 0x1424 */ 0x6A1E,0x7552,
	/* Array index: 0x1424 */ 0x6A1F,0x6D69,
	/* Array index: 0x1424 */ 0x6A21,0x593C,
	/* Array index: 0x1424 */ 0x6A23,0x6546,
	/* Array index: 0x1424 */ 0x6A35,0x7523,
	/* Array index: 0x1424 */ 0x6A3D,0x715C,
	/* Array index: 0x1424 */ 0x6A44,0x4A74,
	/* Array index: 0x1424 */ 0x6A48,0x687A,
	/* Array index: 0x1424 */ 0x6A4B,0x4E69,
	/* Array index: 0x1424 */ 0x6A52,0x6978,
	/* Array index: 0x1424 */ 0x6A53,0x6265,
	/* Array index: 0x1424 */ 0x6A58,0x5039,
	/* Array index: 0x1424 */ 0x6A59,0x5472,
	/* Array index: 0x1424 */ 0x6A5F,0x5126,
	/* Array index: 0x1424 */ 0x6A61,0x5F4E,
	/* Array index: 0x1424 */ 0x6A6B,0x7C74,
	/* Array index: 0x1424 */ 0x6A80,0x532A,
	/* Array index: 0x1424 */ 0x6A84,0x4C2C,
	/* Array index: 0x1424 */ 0x6A89,0x6F60,
	/* Array index: 0x1424 */ 0x6A8D,0x6565,
	/* Array index: 0x1424 */ 0x6A8E,0x5055,
	/* Array index: 0x1424 */ 0x6A97,0x5B7C,
	/* Array index: 0x1424 */ 0x6A9C,0x7C66,
	/* Array index: 0x1424 */ 0x6AA2,0x4B7E,
	/* Array index: 0x1424 */ 0x6AA3,0x6D6A,
	/* Array index: 0x1424 */ 0x6AB3,0x5E31,
	/* Array index: 0x1424 */ 0x6ABB,0x7963,
	/* Array index: 0x1424 */ 0x6AC2,0x5422,
	/* Array index: 0x1424 */ 0x6AC3,0x4F76,
	/* Array index: 0x1424 */ 0x6AD3,0x5650,
	/* Array index: 0x1424 */ 0x6ADA,0x556A,
	/* Array index: 0x1424 */ 0x6ADB,0x716E,
	/* Array index: 0x1424 */ 0x6AF6,0x7A4B,
	/* Array index: 0x1424 */ 0x6AFB,0x6521,
	/* Array index: 0x1424 */ 0x6B04,0x5531,
	/* Array index: 0x1424 */ 0x6B0A,0x4F6D,
	/* Array index: 0x1424 */ 0x6B0C,0x6D6B,
	/* Array index: 0x1424 */ 0x6B12,0x5532,
	/* Array index: 0x1424 */ 0x6B16,0x553C,
	/* Array index: 0x1424 */ 0x6B20,0x7D62,
	/* Array index: 0x1424 */ 0x6B21,0x732D,
	/* Array index: 0x1424 */ 0x6B23,0x7D5B,
	/* Array index: 0x1424 */ 0x6B32,0x6930,
	/* Array index: 0x1424 */ 0x6B3A,0x5127,
	/* Array index: 0x1424 */ 0x6B3D,0x7D63,
	/* Array index: 0x1424 */ 0x6B3E,0x4E33,
	/* Array index: 0x1424 */ 0x6B46,0x7D64,
	/* Array index: 0x1424 */ 0x6B47,0x7A4E,
	/* Array index: 0x1424 */ 0x6B4C,0x4A30,
	/* Array index: 0x1424 */ 0x6B4E,0x7727,
	/* Array index: 0x1424 */ 0x6B50,0x4F31,
	/* Array index: 0x1424 */ 0x6B6A,0x6860,
	/* Array index: 0x1424 */ 0x6B72,0x6128,
	/* Array index: 0x1424 */ 0x6B77,0x5576,
	/* Array index: 0x1424 */ 0x6B78,0x4F7D,
	/* Array index: 0x1424 */ 0x6B7B,0x5E5D,
	/* Array index: 0x1424 */ 0x6B7F,0x5951,
	/* Array index: 0x1424 */ 0x6B83,0x646A,
	/* Array index: 0x1424 */ 0x6B84,0x724F,
	/* Array index: 0x1424 */ 0x6B86,0x773F,
	/* Array index: 0x1424 */ 0x6B89,0x6266,
	/* Array index: 0x1424 */ 0x6B8A,0x6228,
	/* Array index: 0x1424 */ 0x6B96,0x6356,
	/* Array index: 0x1424 */ 0x6B98,0x6D51,
	/* Array index: 0x1424 */ 0x6B9E,0x6979,
	/* Array index: 0x1424 */ 0x6BAE,0x5631,
	/* Array index: 0x1424 */ 0x6BAF,0x5E32,
	/* Array index: 0x1424 */ 0x6BB2,0x6068,
	/* Array index: 0x1424 */ 0x6BB5,0x532B,
	/* Array index: 0x1424 */ 0x6BB7,0x6B5C,
	/* Array index: 0x1424 */ 0x6BBA,0x5F2F,
	/* Array index: 0x1424 */ 0x6BBC,0x4A43,
	/* Array index: 0x1424 */ 0x6BBF,0x6E7C,
	/* Array index: 0x1424 */ 0x6BC1,0x7D43,
	/* Array index: 0x1424 */ 0x6BC5,0x6B76,
	/* Array index: 0x1424 */ 0x6BC6,0x4F32,
	/* Array index: 0x1424 */ 0x6BCB,0x596C,
	/* Array index: 0x1424 */ 0x6BCD,0x593D,
	/* Array index: 0x1424 */ 0x6BCF,0x585F,
	/* Array index: 0x1424 */ 0x6BDB,0x593E,
	/* Array index: 0x1424 */ 0x6BEB,0x7B46,
	/* Array index: 0x1424 */ 0x6BEC,0x4F33,
	/* Array index: 0x1424 */ 0x6C08,0x6E7D,
	/* Array index: 0x1424 */ 0x6C0F,0x642B,
	/* Array index: 0x1424 */ 0x6C11,0x5A45,
	/* Array index: 0x1424 */ 0x6C13,0x586C,
	/* Array index: 0x1424 */ 0x6C23,0x5128,
	/* Array index: 0x1424 */ 0x6C34,0x6229,
	/* Array index: 0x1424 */ 0x6C37,0x5E3C,
	/* Array index: 0x1424 */ 0x6C38,0x6735,
	/* Array index: 0x1424 */ 0x6C4E,0x5B71,
	/* Array index: 0x1424 */ 0x6C50,0x6031,
	/* Array index: 0x1424 */ 0x6C55,0x5F25,
	/* Array index: 0x1424 */ 0x6C57,0x7952,
	/* Array index: 0x1424 */ 0x6C5A,0x677D,
	/* Array index: 0x1424 */ 0x6C68,0x4D67,
	/* Array index: 0x1424 */ 0x6C6A,0x685C,
	/* Array index: 0x1424 */ 0x6C6D,0x6757,
	/* Array index: 0x1424 */ 0x6C70,0x7740,
	/* Array index: 0x1424 */ 0x6C72,0x5063,
	/* Array index: 0x1424 */ 0x6C76,0x5A21,
	/* Array index: 0x1424 */ 0x6C7A,0x4C3D,
	/* Array index: 0x1424 */ 0x6C8C,0x5447,
	/* Array index: 0x1424 */ 0x6CAB,0x5848,
	/* Array index: 0x1424 */ 0x6CAE,0x6E41,
	/* Array index: 0x1424 */ 0x6CB3,0x7941,
	/* Array index: 0x1424 */ 0x6CC9,0x743B,
	/* Array index: 0x1424 */ 0x6CCA,0x5A55,
	/* Array index: 0x1424 */ 0x6CCC,0x7932,
	/* Array index: 0x1424 */ 0x6CD3,0x7B72,
	/* Array index: 0x1424 */ 0x6CD5,0x5B76,
	/* Array index: 0x1424 */ 0x6CD7,0x5E5F,
	/* Array index: 0x1424 */ 0x6CDB,0x5B72,
	/* Array index: 0x1424 */ 0x6CE8,0x713C,
	/* Array index: 0x1424 */ 0x6CEB,0x7A5A,
	/* Array index: 0x1424 */ 0x6CF3,0x6736,
	/* Array index: 0x1424 */ 0x6D0B,0x6547,
	/* Array index: 0x1424 */ 0x6D0C,0x562C,
	/* Array index: 0x1424 */ 0x6D11,0x5C47,
	/* Array index: 0x1424 */ 0x6D17,0x6129,
	/* Array index: 0x1424 */ 0x6D19,0x622A,
	/* Array index: 0x1424 */ 0x6D1B,0x5526,
	/* Array index: 0x1424 */ 0x6D1E,0x5457,
	/* Array index: 0x1424 */ 0x6D25,0x7250,
	/* Array index: 0x1424 */ 0x6D27,0x6A7B,
	/* Array index: 0x1424 */ 0x6D29,0x605D,
	/* Array index: 0x1424 */ 0x6D2A,0x7B73,
	/* Array index: 0x1424 */ 0x6D32,0x713D,
	/* Array index: 0x1424 */ 0x6D41,0x5735,
	/* Array index: 0x1424 */ 0x6D59,0x6F3A,
	/* Array index: 0x1424 */ 0x6D5A,0x715D,
	/* Array index: 0x1424 */ 0x6D5C,0x5E33,
	/* Array index: 0x1424 */ 0x6D63,0x684B,
	/* Array index: 0x1424 */ 0x6D66,0x785D,
	/* Array index: 0x1424 */ 0x6D69,0x7B47,
	/* Array index: 0x1424 */ 0x6D6A,0x5548,
	/* Array index: 0x1424 */ 0x6D6C,0x575F,
	/* Array index: 0x1424 */ 0x6D6E,0x5D29,
	/* Array index: 0x1424 */ 0x6D74,0x6931,
	/* Array index: 0x1424 */ 0x6D7F,0x782A,
	/* Array index: 0x1424 */ 0x6D91,0x6157,
	/* Array index: 0x1424 */ 0x6D93,0x664F,
	/* Array index: 0x1424 */ 0x6D95,0x7471,
	/* Array index: 0x1424 */ 0x6DAF,0x6473,
	/* Array index: 0x1424 */ 0x6DB2,0x647B,
	/* Array index: 0x1424 */ 0x6DB5,0x7964,
	/* Array index: 0x1424 */ 0x6DC0,0x6F63,
	/* Array index: 0x1424 */ 0x6DCB,0x577A,
	/* Array index: 0x1424 */ 0x6DCF,0x7B48,
	/* Array index: 0x1424 */ 0x6DD1,0x6257,
	/* Array index: 0x1424 */ 0x6DDE,0x6167,
	/* Array index: 0x1424 */ 0x6DE1,0x533F,
	/* Array index: 0x1424 */ 0x6DE8,0x6F64,
	/* Array index: 0x1424 */ 0x6DEA,0x5745,
	/* Array index: 0x1424 */ 0x6DEB,0x6B62,
	/* Array index: 0x1424 */ 0x6DEE,0x7C67,
	/* Array index: 0x1424 */ 0x6DF1,0x6422,
	/* Array index: 0x1424 */ 0x6DF3,0x6268,
	/* Array index: 0x1424 */ 0x6E32,0x6042,
	/* Array index: 0x1424 */ 0x6E34,0x4A64,
	/* Array index: 0x1424 */ 0x6E36,0x6737,
	/* Array index: 0x1424 */ 0x6E38,0x6A7D,
	/* Array index: 0x1424 */ 0x6E43,0x5B4B,
	/* Array index: 0x1424 */ 0x6E44,0x5A35,
	/* Array index: 0x1424 */ 0x6E4A,0x713E,
	/* Array index: 0x1424 */ 0x6E4D,0x532C,
	/* Array index: 0x1424 */ 0x6E56,0x7B49,
	/* Array index: 0x1424 */ 0x6E58,0x5F4F,
	/* Array index: 0x1424 */ 0x6E67,0x6940,
	/* Array index: 0x1424 */ 0x6E6B,0x7553,
	/* Array index: 0x1424 */ 0x6E6E,0x6C5C,
	/* Array index: 0x1424 */ 0x6E6F,0x7737,
	/* Array index: 0x1424 */ 0x6E72,0x6A38,
	/* Array index: 0x1424 */ 0x6E73,0x5179,
	/* Array index: 0x1424 */ 0x6E7A,0x5C48,
	/* Array index: 0x1424 */ 0x6E90,0x6A39,
	/* Array index: 0x1424 */ 0x6E96,0x715E,
	/* Array index: 0x1424 */ 0x6E9C,0x5736,
	/* Array index: 0x1424 */ 0x6E9D,0x4F35,
	/* Array index: 0x1424 */ 0x6E9F,0x5928,
	/* Array index: 0x1424 */ 0x6EA2,0x6C6E,
	/* Array index: 0x1424 */ 0x6EA5,0x5D2A,
	/* Array index: 0x1424 */ 0x6EAA,0x4D22,
	/* Array index: 0x1424 */ 0x6EAB,0x682E,
	/* Array index: 0x1424 */ 0x6EAF,0x613D,
	/* Array index: 0x1424 */ 0x6EB1,0x7251,
	/* Array index: 0x1424 */ 0x6EB6,0x6941,
	/* Array index: 0x1424 */ 0x6EBA,0x527C,
	/* Array index: 0x1424 */ 0x6EC2,0x5B35,
	/* Array index: 0x1424 */ 0x6EC4,0x7367,
	/* Array index: 0x1424 */ 0x6EC5,0x587E,
	/* Array index: 0x1424 */ 0x6EC9,0x7C51,
	/* Array index: 0x1424 */ 0x6ECB,0x6D32,
	/* Array index: 0x1424 */ 0x6ECC,0x742F,
	/* Array index: 0x1424 */ 0x6ECE,0x7B23,
	/* Array index: 0x1424 */ 0x6ED1,0x7C41,
	/* Array index: 0x1424 */ 0x6ED3,0x6E2B,
	/* Array index: 0x1424 */ 0x6ED4,0x5425,
	/* Array index: 0x1424 */ 0x6EEF,0x7472,
	/* Array index: 0x1424 */ 0x6EF4,0x6E59,
	/* Array index: 0x1424 */ 0x6EF8,0x7B4A,
	/* Array index: 0x1424 */ 0x6F06,0x7654,
	/* Array index: 0x1424 */ 0x6F0F,0x5729,
	/* Array index: 0x1424 */ 0x6F11,0x4B49,
	/* Array index: 0x1424 */ 0x6F14,0x6651,
	/* Array index: 0x1424 */ 0x6F15,0x704C,
	/* Array index: 0x1424 */ 0x6F20,0x582E,
	/* Array index: 0x1424 */ 0x6F22,0x7953,
	/* Array index: 0x1424 */ 0x6F23,0x557E,
	/* Array index: 0x1424 */ 0x6F2B,0x583C,
	/* Array index: 0x1424 */ 0x6F2C,0x7230,
	/* Array index: 0x1424 */ 0x6F31,0x622B,
	/* Array index: 0x1424 */ 0x6F32,0x7368,
	/* Array index: 0x1424 */ 0x6F38,0x6F42,
	/* Array index: 0x1424 */ 0x6F3F,0x6D6C,
	/* Array index: 0x1424 */ 0x6F41,0x6738,
	/* Array index: 0x1424 */ 0x6F51,0x5A7E,
	/* Array index: 0x1424 */ 0x6F54,0x4C3E,
	/* Array index: 0x1424 */ 0x6F62,0x7C52,
	/* Array index: 0x1424 */ 0x6F64,0x6B48,
	/* Array index: 0x1424 */ 0x6F6D,0x5341,
	/* Array index: 0x1424 */ 0x6F6E,0x704D,
	/* Array index: 0x1424 */ 0x6F70,0x4F77,
	/* Array index: 0x1424 */ 0x6F81,0x5F3B,
	/* Array index: 0x1424 */ 0x6F84,0x7325,
	/* Array index: 0x1424 */ 0x6F88,0x744D,
	/* Array index: 0x1424 */ 0x6F8D,0x713F,
	/* Array index: 0x1424 */ 0x6F8E,0x7831,
	/* Array index: 0x1424 */ 0x6F90,0x697A,
	/* Array index: 0x1424 */ 0x6F94,0x7B4B,
	/* Array index: 0x1424 */ 0x6F97,0x4A55,
	/* Array index: 0x1424 */ 0x6FA3,0x7954,
	/* Array index: 0x1424 */ 0x6FA4,0x774A,
	/* Array index: 0x1424 */ 0x6FA7,0x5648,
	/* Array index: 0x1424 */ 0x6FAE,0x7C68,
	/* Array index: 0x1424 */ 0x6FAF,0x733D,
	/* Array index: 0x1424 */ 0x6FB1,0x6E7E,
	/* Array index: 0x1424 */ 0x6FB3,0x677E,
	/* Array index: 0x1424 */ 0x6FB9,0x5342,
	/* Array index: 0x1424 */ 0x6FCA,0x6758,
	/* Array index: 0x1424 */ 0x6FD5,0x6325,
	/* Array index: 0x1424 */ 0x6FDA,0x6739,
	/* Array index: 0x1424 */ 0x6FE4,0x5426,
	/* Array index: 0x1424 */ 0x6FE9,0x7B4D,
	/* Array index: 0x1424 */ 0x6FEB,0x553D,
	/* Array index: 0x1424 */ 0x6FEC,0x715F,
	/* Array index: 0x1424 */ 0x6FEF,0x767B,
	/* Array index: 0x1424 */ 0x6FF1,0x5E34,
	/* Array index: 0x1424 */ 0x6FFE,0x556B,
	/* Array index: 0x1424 */ 0x7001,0x6548,
	/* Array index: 0x1424 */ 0x7005,0x7B24,
	/* Array index: 0x1424 */ 0x7006,0x5439,
	/* Array index: 0x1424 */ 0x7009,0x5E61,
	/* Array index: 0x1424 */ 0x700B,0x6423,
	/* Array index: 0x1424 */ 0x700F,0x5737,
	/* Array index: 0x1424 */ 0x7011,0x786E,
	/* Array index: 0x1424 */ 0x7015,0x5E35,
	/* Array index: 0x1424 */ 0x7023,0x7A2E,
	/* Array index: 0x1424 */ 0x7027,0x5669,
	/* Array index: 0x1424 */ 0x7028,0x566E,
	/* Array index: 0x1424 */ 0x702F,0x673B,
	/* Array index: 0x1424 */ 0x7037,0x6C4B,
	/* Array index: 0x1424 */ 0x703E,0x5533,
	/* Array index: 0x1424 */ 0x704C,0x4E34,
	/* Array index: 0x1424 */ 0x7050,0x7B25,
	/* Array index: 0x1424 */ 0x7051,0x616E,
	/* Array index: 0x1424 */ 0x7058,0x7728,
	/* Array index: 0x1424 */ 0x705D,0x7B4E,
	/* Array index: 0x1424 */ 0x7063,0x583D,
	/* Array index: 0x1424 */ 0x706B,0x7B7D,
	/* Array index: 0x1424 */ 0x7070,0x7C69,
	/* Array index: 0x1424 */ 0x7078,0x4F36,
	/* Array index: 0x1424 */ 0x707C,0x6D47,
	/* Array index: 0x1424 */ 0x707D,0x6E2C,
	/* Array index: 0x1424 */ 0x7085,0x4C5D,
	/* Array index: 0x1424 */ 0x708A,0x7627,
	/* Array index: 0x1424 */ 0x708E,0x667A,
	/* Array index: 0x1424 */ 0x7092,0x7524,
	/* Array index: 0x1424 */ 0x70A1,0x6F68,
	/* Array index: 0x1424 */ 0x70A4,0x613F,
	/* Array index: 0x1424 */ 0x70B3,0x5C39,
	/* Array index: 0x1424 */ 0x70C8,0x562D,
	/* Array index: 0x1424 */ 0x70CB,0x7D4E,
	/* Array index: 0x1424 */ 0x70CF,0x6821,
	/* Array index: 0x1424 */ 0x70D8,0x7B74,
	/* Array index: 0x1424 */ 0x70D9,0x5527,
	/* Array index: 0x1424 */ 0x70DD,0x7176,
	/* Array index: 0x1424 */ 0x70DF,0x6653,
	/* Array index: 0x1424 */ 0x70F1,0x4C5E,
	/* Array index: 0x1424 */ 0x70F9,0x7832,
	/* Array index: 0x1424 */ 0x70FD,0x5C6B,
	/* Array index: 0x1424 */ 0x7104,0x7D36,
	/* Array index: 0x1424 */ 0x7109,0x656A,
	/* Array index: 0x1424 */ 0x710C,0x7160,
	/* Array index: 0x1424 */ 0x7119,0x5B4C,
	/* Array index: 0x1424 */ 0x711A,0x5D4D,
	/* Array index: 0x1424 */ 0x711E,0x5448,
	/* Array index: 0x1424 */ 0x7121,0x596D,
	/* Array index: 0x1424 */ 0x7126,0x7525,
	/* Array index: 0x1424 */ 0x7130,0x667B,
	/* Array index: 0x1424 */ 0x7136,0x6654,
	/* Array index: 0x1424 */ 0x7147,0x7D48,
	/* Array index: 0x1424 */ 0x7149,0x5621,
	/* Array index: 0x1424 */ 0x714A,0x7D3F,
	/* Array index: 0x1424 */ 0x714C,0x7C53,
	/* Array index: 0x1424 */ 0x714E,0x6F21,
	/* Array index: 0x1424 */ 0x7150,0x673C,
	/* Array index: 0x1424 */ 0x7156,0x516E,
	/* Array index: 0x1424 */ 0x7159,0x6655,
	/* Array index: 0x1424 */ 0x715C,0x6972,
	/* Array index: 0x1424 */ 0x715E,0x5F30,
	/* Array index: 0x1424 */ 0x716C,0x6549,
	/* Array index: 0x1424 */ 0x716E,0x6D34,
	/* Array index: 0x1424 */ 0x717D,0x6043,
	/* Array index: 0x1424 */ 0x7184,0x6358,
	/* Array index: 0x1424 */ 0x7189,0x697B,
	/* Array index: 0x1424 */ 0x718A,0x6A28,
	/* Array index: 0x1424 */ 0x718F,0x7D37,
	/* Array index: 0x1424 */ 0x7192,0x7B27,
	/* Array index: 0x1424 */ 0x7194,0x6942,
	/* Array index: 0x1424 */ 0x7199,0x7D77,
	/* Array index: 0x1424 */ 0x719F,0x6259,
	/* Array index: 0x1424 */ 0x71A2,0x5C6C,
	/* Array index: 0x1424 */ 0x71AC,0x6822,
	/* Array index: 0x1424 */ 0x71B1,0x6670,
	/* Array index: 0x1424 */ 0x71B9,0x7D78,
	/* Array index: 0x1424 */ 0x71BA,0x7D79,
	/* Array index: 0x1424 */ 0x71BE,0x763F,
	/* Array index: 0x1424 */ 0x71C1,0x6727,
	/* Array index: 0x1424 */ 0x71C3,0x6657,
	/* Array index: 0x1424 */ 0x71C8,0x5473,
	/* Array index: 0x1424 */ 0x71C9,0x5449,
	/* Array index: 0x1424 */ 0x71CE,0x567A,
	/* Array index: 0x1424 */ 0x71D0,0x5772,
	/* Array index: 0x1424 */ 0x71D2,0x6140,
	/* Array index: 0x1424 */ 0x71D4,0x5B62,
	/* Array index: 0x1424 */ 0x71D5,0x6658,
	/* Array index: 0x1424 */ 0x71DF,0x673D,
	/* Array index: 0x1424 */ 0x71ED,0x7537,
	/* Array index: 0x1424 */ 0x71EE,0x6070,
	/* Array index: 0x1424 */ 0x7206,0x786F,
	/* Array index: 0x1424 */ 0x7210,0x5653,
	/* Array index: 0x1424 */ 0x721B,0x5534,
	/* Array index: 0x1424 */ 0x722A,0x7050,
	/* Array index: 0x1424 */ 0x722C,0x7770,
	/* Array index: 0x1424 */ 0x722D,0x6E33,
	/* Array index: 0x1424 */ 0x7230,0x6A3A,
	/* Array index: 0x1424 */ 0x7232,0x6A53,
	/* Array index: 0x1424 */ 0x7235,0x6D49,
	/* Array index: 0x1424 */ 0x7236,0x5D2B,
	/* Array index: 0x1424 */ 0x724C,0x782B,
	/* Array index: 0x1424 */ 0x7252,0x7460,
	/* Array index: 0x1424 */ 0x7258,0x543A,
	/* Array index: 0x1424 */ 0x7259,0x6433,
	/* Array index: 0x1424 */ 0x725B,0x695A,
	/* Array index: 0x1424 */ 0x725D,0x5E36,
	/* Array index: 0x1424 */ 0x725F,0x593F,
	/* Array index: 0x1424 */ 0x7261,0x5940,
	/* Array index: 0x1424 */ 0x7262,0x566F,
	/* Array index: 0x1424 */ 0x7267,0x594C,
	/* Array index: 0x1424 */ 0x7269,0x5A2A,
	/* Array index: 0x1424 */ 0x7272,0x5F65,
	/* Array index: 0x1424 */ 0x7279,0x7765,
	/* Array index: 0x1424 */ 0x727D,0x4C32,
	/* Array index: 0x1424 */ 0x7280,0x5F79,
	/* Array index: 0x1424 */ 0x7281,0x5760,
	/* Array index: 0x1424 */ 0x72A2,0x543B,
	/* Array index: 0x1424 */ 0x72A7,0x7D7A,
	/* Array index: 0x1424 */ 0x72AC,0x4C33,
	/* Array index: 0x1424 */ 0x72AF,0x5B73,
	/* Array index: 0x1424 */ 0x72C0,0x5F52,
	/* Array index: 0x1424 */ 0x72C2,0x4E4A,
	/* Array index: 0x1424 */ 0x72C4,0x6E5A,
	/* Array index: 0x1424 */ 0x72CE,0x6464,
	/* Array index: 0x1424 */ 0x72D0,0x7B4F,
	/* Array index: 0x1424 */ 0x72D7,0x4F37,
	/* Array index: 0x1424 */ 0x72D9,0x6E43,
	/* Array index: 0x1424 */ 0x72E1,0x4E6A,
	/* Array index: 0x1424 */ 0x72E9,0x622D,
	/* Array index: 0x1424 */ 0x72F8,0x5761,
	/* Array index: 0x1424 */ 0x72F9,0x7A75,
	/* Array index: 0x1424 */ 0x72FC,0x5549,
	/* Array index: 0x1424 */ 0x72FD,0x782C,
	/* Array index: 0x1424 */ 0x730A,0x6759,
	/* Array index: 0x1424 */ 0x7316,0x7369,
	/* Array index: 0x1424 */ 0x7325,0x6865,
	/* Array index: 0x1424 */ 0x7336,0x6B22,
	/* Array index: 0x1424 */ 0x7337,0x6B23,
	/* Array index: 0x1424 */ 0x733E,0x7C42,
	/* Array index: 0x1424 */ 0x733F,0x6A3B,
	/* Array index: 0x1424 */ 0x7344,0x682B,
	/* Array index: 0x1424 */ 0x7345,0x5E62,
	/* Array index: 0x1424 */ 0x7350,0x6D6F,
	/* Array index: 0x1424 */ 0x7352,0x6823,
	/* Array index: 0x1424 */ 0x7357,0x4F71,
	/* Array index: 0x1424 */ 0x7368,0x543C,
	/* Array index: 0x1424 */ 0x736A,0x7C6A,
	/* Array index: 0x1424 */ 0x7370,0x673E,
	/* Array index: 0x1424 */ 0x7372,0x7C72,
	/* Array index: 0x1424 */ 0x7375,0x5634,
	/* Array index: 0x1424 */ 0x7378,0x622E,
	/* Array index: 0x1424 */ 0x737A,0x5337,
	/* Array index: 0x1424 */ 0x737B,0x7A4C,
	/* Array index: 0x1424 */ 0x7384,0x7A5C,
	/* Array index: 0x1424 */ 0x7386,0x6D35,
	/* Array index: 0x1424 */ 0x7387,0x6163,
	/* Array index: 0x1424 */ 0x7389,0x682C,
	/* Array index: 0x1424 */ 0x738B,0x685D,
	/* Array index: 0x1424 */ 0x738E,0x6F69,
	/* Array index: 0x1424 */ 0x739F,0x5A47,
	/* Array index: 0x1424 */ 0x73A7,0x6B49,
	/* Array index: 0x1424 */ 0x73A9,0x684C,
	/* Array index: 0x1424 */ 0x73AD,0x5E37,
	/* Array index: 0x1424 */ 0x73B2,0x563C,
	/* Array index: 0x1424 */ 0x73B3,0x5365,
	/* Array index: 0x1424 */ 0x73B9,0x7A5D,
	/* Array index: 0x1424 */ 0x73C0,0x5A56,
	/* Array index: 0x1424 */ 0x73C2,0x4A31,
	/* Array index: 0x1424 */ 0x73D6,0x4E4B,
	/* Array index: 0x1424 */ 0x73D9,0x4D75,
	/* Array index: 0x1424 */ 0x73DD,0x7D30,
	/* Array index: 0x1424 */ 0x73DE,0x5528,
	/* Array index: 0x1424 */ 0x73E0,0x7141,
	/* Array index: 0x1424 */ 0x73ED,0x5A6C,
	/* Array index: 0x1424 */ 0x73F7,0x596E,
	/* Array index: 0x1424 */ 0x73F9,0x607B,
	/* Array index: 0x1424 */ 0x73FD,0x6F6A,
	/* Array index: 0x1424 */ 0x73FE,0x7A5E,
	/* Array index: 0x1424 */ 0x7401,0x6044,
	/* Array index: 0x1424 */ 0x7413,0x684D,
	/* Array index: 0x1424 */ 0x741B,0x765A,
	/* Array index: 0x1424 */ 0x744B,0x6A54,
	/* Array index: 0x1424 */ 0x7455,0x7942,
	/* Array index: 0x1424 */ 0x746F,0x554B,
	/* Array index: 0x1424 */ 0x747E,0x5048,
	/* Array index: 0x1424 */ 0x7482,0x512F,
	/* Array index: 0x1424 */ 0x7483,0x5763,
	/* Array index: 0x1424 */ 0x7487,0x6046,
	/* Array index: 0x1424 */ 0x7489,0x5622,
	/* Array index: 0x1424 */ 0x748B,0x6D70,
	/* Array index: 0x1424 */ 0x7498,0x5773,
	/* Array index: 0x1424 */ 0x749C,0x7C54,
	/* Array index: 0x1424 */ 0x749E,0x5A57,
	/* Array index: 0x1424 */ 0x749F,0x4C5F,
	/* Array index: 0x1424 */ 0x74A1,0x7254,
	/* Array index: 0x1424 */ 0x74A3,0x5130,
	/* Array index: 0x1424 */ 0x74A5,0x4C60,
	/* Array index: 0x1424 */ 0x74A7,0x5B7D,
	/* Array index: 0x1424 */ 0x74A8,0x733F,
	/* Array index: 0x1424 */ 0x74AA,0x7051,
	/* Array index: 0x1424 */ 0x74B0,0x7C3B,
	/* Array index: 0x1424 */ 0x74B2,0x6230,
	/* Array index: 0x1424 */ 0x74B5,0x6625,
	/* Array index: 0x1424 */ 0x74B9,0x625B,
	/* Array index: 0x1424 */ 0x74BD,0x5F5E,
	/* Array index: 0x1424 */ 0x74BF,0x6047,
	/* Array index: 0x1424 */ 0x74C6,0x726F,
	/* Array index: 0x1424 */ 0x74CA,0x4C61,
	/* Array index: 0x1424 */ 0x74CF,0x566A,
	/* Array index: 0x1424 */ 0x74D4,0x6742,
	/* Array index: 0x1424 */ 0x74D8,0x4E36,
	/* Array index: 0x1424 */ 0x74DA,0x7340,
	/* Array index: 0x1424 */ 0x74DC,0x4D7E,
	/* Array index: 0x1424 */ 0x74E0,0x7B52,
	/* Array index: 0x1424 */ 0x74E2,0x7878,
	/* Array index: 0x1424 */ 0x74E3,0x777B,
	/* Array index: 0x1424 */ 0x74E6,0x683F,
	/* Array index: 0x1424 */ 0x74EE,0x6837,
	/* Array index: 0x1424 */ 0x74F7,0x6D36,
	/* Array index: 0x1424 */ 0x7501,0x5C3A,
	/* Array index: 0x1424 */ 0x7504,0x4C34,
	/* Array index: 0x1424 */ 0x7511,0x7177,
	/* Array index: 0x1424 */ 0x7515,0x6838,
	/* Array index: 0x1424 */ 0x7518,0x4A76,
	/* Array index: 0x1424 */ 0x751A,0x6424,
	/* Array index: 0x1424 */ 0x751B,0x7456,
	/* Array index: 0x1424 */ 0x751F,0x5F66,
	/* Array index: 0x1424 */ 0x7523,0x5F27,
	/* Array index: 0x1424 */ 0x7525,0x5F67,
	/* Array index: 0x1424 */ 0x7526,0x6141,
	/* Array index: 0x1424 */ 0x7528,0x6944,
	/* Array index: 0x1424 */ 0x752B,0x5C4B,
	/* Array index: 0x1424 */ 0x752C,0x6945,
	/* Array index: 0x1424 */ 0x7537,0x517B,
	/* Array index: 0x1424 */ 0x7538,0x6F24,
	/* Array index: 0x1424 */ 0x753A,0x6F6B,
	/* Array index: 0x1424 */ 0x7547,0x5034,
	/* Array index: 0x1424 */ 0x754C,0x4D23,
	/* Array index: 0x1424 */ 0x754F,0x6866,
	/* Array index: 0x1424 */ 0x7551,0x6F25,
	/* Array index: 0x1424 */ 0x7553,0x534C,
	/* Array index: 0x1424 */ 0x7554,0x5A6D,
	/* Array index: 0x1424 */ 0x7562,0x7934,
	/* Array index: 0x1424 */ 0x7565,0x5554,
	/* Array index: 0x1424 */ 0x7566,0x7D4F,
	/* Array index: 0x1424 */ 0x756A,0x5B63,
	/* Array index: 0x1424 */ 0x756F,0x7161,
	/* Array index: 0x1424 */ 0x7570,0x6C36,
	/* Array index: 0x1424 */ 0x7575,0x7B7E,
	/* Array index: 0x1424 */ 0x7576,0x5357,
	/* Array index: 0x1424 */ 0x7578,0x5131,
	/* Array index: 0x1424 */ 0x757A,0x4B31,
	/* Array index: 0x1424 */ 0x757F,0x5132,
	/* Array index: 0x1424 */ 0x7586,0x4B32,
	/* Array index: 0x1424 */ 0x7587,0x7142,
	/* Array index: 0x1424 */ 0x758A,0x7461,
	/* Array index: 0x1424 */ 0x758B,0x7935,
	/* Array index: 0x1424 */ 0x758E,0x6143,
	/* Array index: 0x1424 */ 0x758F,0x6142,
	/* Array index: 0x1424 */ 0x7591,0x6B77,
	/* Array index: 0x1424 */ 0x759D,0x5F28,
	/* Array index: 0x1424 */ 0x75A5,0x4B4A,
	/* Array index: 0x1424 */ 0x75AB,0x6639,
	/* Array index: 0x1424 */ 0x75C2,0x4A32,
	/* Array index: 0x1424 */ 0x75C5,0x5C3B,
	/* Array index: 0x1424 */ 0x75C7,0x7178,
	/* Array index: 0x1424 */ 0x75CD,0x6C37,
	/* Array index: 0x1424 */ 0x75D2,0x654A,
	/* Array index: 0x1424 */ 0x75D4,0x7640,
	/* Array index: 0x1424 */ 0x75D5,0x7D5D,
	/* Array index: 0x1424 */ 0x75D8,0x5463,
	/* Array index: 0x1424 */ 0x75D9,0x4C62,
	/* Array index: 0x1424 */ 0x75DB,0x7754,
	/* Array index: 0x1424 */ 0x75E2,0x5765,
	/* Array index: 0x1424 */ 0x75F0,0x5343,
	/* Array index: 0x1424 */ 0x75F2,0x5826,
	/* Array index: 0x1424 */ 0x75F4,0x7641,
	/* Array index: 0x1424 */ 0x75FA,0x5D76,
	/* Array index: 0x1424 */ 0x75FC,0x4D40,
	/* Array index: 0x1424 */ 0x7600,0x655C,
	/* Array index: 0x1424 */ 0x760D,0x654B,
	/* Array index: 0x1424 */ 0x7619,0x6144,
	/* Array index: 0x1424 */ 0x763B,0x572A,
	/* Array index: 0x1424 */ 0x7642,0x567B,
	/* Array index: 0x1424 */ 0x764C,0x645F,
	/* Array index: 0x1424 */ 0x764E,0x4A56,
	/* Array index: 0x1424 */ 0x7652,0x6B28,
	/* Array index: 0x1424 */ 0x7656,0x5B7E,
	/* Array index: 0x1424 */ 0x7661,0x7642,
	/* Array index: 0x1424 */ 0x7664,0x6F3B,
	/* Array index: 0x1424 */ 0x7669,0x547D,
	/* Array index: 0x1424 */ 0x766C,0x6048,
	/* Array index: 0x1424 */ 0x7670,0x6839,
	/* Array index: 0x1424 */ 0x7672,0x6F26,
	/* Array index: 0x1424 */ 0x7678,0x4D24,
	/* Array index: 0x1424 */ 0x7684,0x6E5C,
	/* Array index: 0x1424 */ 0x7686,0x4B4B,
	/* Array index: 0x1424 */ 0x7687,0x7C55,
	/* Array index: 0x1424 */ 0x768E,0x4E6B,
	/* Array index: 0x1424 */ 0x7690,0x4D41,
	/* Array index: 0x1424 */ 0x7693,0x7B53,
	/* Array index: 0x1424 */ 0x76AE,0x792B,
	/* Array index: 0x1424 */ 0x76BA,0x7554,
	/* Array index: 0x1424 */ 0x76BF,0x5929,
	/* Array index: 0x1424 */ 0x76C2,0x695D,
	/* Array index: 0x1424 */ 0x76C3,0x5B4D,
	/* Array index: 0x1424 */ 0x76C6,0x5D4E,
	/* Array index: 0x1424 */ 0x76C8,0x6743,
	/* Array index: 0x1424 */ 0x76CA,0x6C4C,
	/* Array index: 0x1424 */ 0x76D2,0x796C,
	/* Array index: 0x1424 */ 0x76D6,0x4B4C,
	/* Array index: 0x1424 */ 0x76E7,0x5654,
	/* Array index: 0x1424 */ 0x76EE,0x594D,
	/* Array index: 0x1424 */ 0x76F2,0x586E,
	/* Array index: 0x1424 */ 0x76F4,0x7241,
	/* Array index: 0x1424 */ 0x76F8,0x5F53,
	/* Array index: 0x1424 */ 0x76FC,0x5A70,
	/* Array index: 0x1424 */ 0x76FE,0x626A,
	/* Array index: 0x1424 */ 0x7701,0x607D,
	/* Array index: 0x1424 */ 0x7704,0x5878,
	/* Array index: 0x1424 */ 0x7708,0x772F,
	/* Array index: 0x1424 */ 0x7709,0x5A36,
	/* Array index: 0x1424 */ 0x770B,0x4A57,
	/* Array index: 0x1424 */ 0x771E,0x7258,
	/* Array index: 0x1424 */ 0x7720,0x5879,
	/* Array index: 0x1424 */ 0x7729,0x7A5F,
	/* Array index: 0x1424 */ 0x7737,0x4F6F,
	/* Array index: 0x1424 */ 0x7738,0x5942,
	/* Array index: 0x1424 */ 0x773A,0x7052,
	/* Array index: 0x1424 */ 0x773C,0x6451,
	/* Array index: 0x1424 */ 0x7740,0x7337,
	/* Array index: 0x1424 */ 0x774D,0x7A60,
	/* Array index: 0x1424 */ 0x775B,0x6F6C,
	/* Array index: 0x1424 */ 0x7761,0x6232,
	/* Array index: 0x1424 */ 0x7763,0x543D,
	/* Array index: 0x1424 */ 0x7766,0x594E,
	/* Array index: 0x1424 */ 0x776B,0x7462,
	/* Array index: 0x1424 */ 0x7779,0x5429,
	/* Array index: 0x1424 */ 0x777E,0x4D42,
	/* Array index: 0x1424 */ 0x777F,0x675A,
	/* Array index: 0x1424 */ 0x778B,0x7259,
	/* Array index: 0x1424 */ 0x7791,0x592A,
	/* Array index: 0x1424 */ 0x779E,0x583E,
	/* Array index: 0x1424 */ 0x77A5,0x5C2D,
	/* Array index: 0x1424 */ 0x77AC,0x626B,
	/* Array index: 0x1424 */ 0x77AD,0x567C,
	/* Array index: 0x1424 */ 0x77B0,0x4A79,
	/* Array index: 0x1424 */ 0x77B3,0x545A,
	/* Array index: 0x1424 */ 0x77BB,0x7457,
	/* Array index: 0x1424 */ 0x77BC,0x4C21,
	/* Array index: 0x1424 */ 0x77BF,0x4F3A,
	/* Array index: 0x1424 */ 0x77D7,0x7538,
	/* Array index: 0x1424 */ 0x77DB,0x5943,
	/* Array index: 0x1424 */ 0x77DC,0x5068,
	/* Array index: 0x1424 */ 0x77E2,0x6345,
	/* Array index: 0x1424 */ 0x77E3,0x6B78,
	/* Array index: 0x1424 */ 0x77E5,0x7231,
	/* Array index: 0x1424 */ 0x77E9,0x4F3B,
	/* Array index: 0x1424 */ 0x77F3,0x6034,
	/* Array index: 0x1424 */ 0x7802,0x5E63,
	/* Array index: 0x1424 */ 0x7812,0x5D77,
	/* Array index: 0x1424 */ 0x782C,0x577E,
	/* Array index: 0x1424 */ 0x7832,0x785F,
	/* Array index: 0x1424 */ 0x7834,0x7772,
	/* Array index: 0x1424 */ 0x7845,0x5029,
	/* Array index: 0x1424 */ 0x784F,0x665A,
	/* Array index: 0x1424 */ 0x785D,0x7526,
	/* Array index: 0x1424 */ 0x786B,0x573C,
	/* Array index: 0x1424 */ 0x786C,0x4C63,
	/* Array index: 0x1424 */ 0x786F,0x665B,
	/* Array index: 0x1424 */ 0x787C,0x5D5D,
	/* Array index: 0x1424 */ 0x7881,0x5133,
	/* Array index: 0x1424 */ 0x7887,0x6F6D,
	/* Array index: 0x1424 */ 0x7891,0x5D78,
	/* Array index: 0x1424 */ 0x7897,0x684F,
	/* Array index: 0x1424 */ 0x78A3,0x4A65,
	/* Array index: 0x1424 */ 0x78A7,0x5C21,
	/* Array index: 0x1424 */ 0x78A9,0x6035,
	/* Array index: 0x1424 */ 0x78C1,0x6D38,
	/* Array index: 0x1424 */ 0x78C5,0x5B36,
	/* Array index: 0x1424 */ 0x78CA,0x5670,
	/* Array index: 0x1424 */ 0x78CB,0x732F,
	/* Array index: 0x1424 */ 0x78CE,0x4D25,
	/* Array index: 0x1424 */ 0x78D0,0x5A71,
	/* Array index: 0x1424 */ 0x78E8,0x5828,
	/* Array index: 0x1424 */ 0x78EC,0x4C64,
	/* Array index: 0x1424 */ 0x78EF,0x5134,
	/* Array index: 0x1424 */ 0x78F5,0x4A58,
	/* Array index: 0x1424 */ 0x78FB,0x5A72,
	/* Array index: 0x1424 */ 0x7901,0x7527,
	/* Array index: 0x1424 */ 0x790E,0x7528,
	/* Array index: 0x1424 */ 0x7916,0x6626,
	/* Array index: 0x1424 */ 0x793A,0x6346,
	/* Array index: 0x1424 */ 0x793E,0x5E64,
	/* Array index: 0x1424 */ 0x7940,0x5E65,
	/* Array index: 0x1424 */ 0x7941,0x5135,
	/* Array index: 0x1424 */ 0x7950,0x695E,
	/* Array index: 0x1424 */ 0x7965,0x5F54,
	/* Array index: 0x1424 */ 0x7968,0x7879,
	/* Array index: 0x1424 */ 0x796D,0x702E,
	/* Array index: 0x1424 */ 0x797A,0x5138,
	/* Array index: 0x1424 */ 0x797F,0x565F,
	/* Array index: 0x1424 */ 0x7981,0x5057,
	/* Array index: 0x1424 */ 0x79A6,0x655D,
	/* Array index: 0x1424 */ 0x79A7,0x7D7B,
	/* Array index: 0x1424 */ 0x79AA,0x6049,
	/* Array index: 0x1424 */ 0x79AE,0x5649,
	/* Array index: 0x1424 */ 0x79B1,0x542A,
	/* Array index: 0x1424 */ 0x79B3,0x654C,
	/* Array index: 0x1424 */ 0x79B9,0x6960,
	/* Array index: 0x1424 */ 0x79D1,0x4E21,
	/* Array index: 0x1424 */ 0x79D2,0x7529,
	/* Array index: 0x1424 */ 0x79D5,0x5D79,
	/* Array index: 0x1424 */ 0x79D8,0x5D7A,
	/* Array index: 0x1424 */ 0x79DF,0x7055,
	/* Array index: 0x1424 */ 0x79E4,0x765F,
	/* Array index: 0x1424 */ 0x79E6,0x725A,
	/* Array index: 0x1424 */ 0x79E7,0x646B,
	/* Array index: 0x1424 */ 0x79E9,0x7271,
	/* Array index: 0x1424 */ 0x79FB,0x6C39,
	/* Array index: 0x1424 */ 0x7A00,0x7D7C,
	/* Array index: 0x1424 */ 0x7A05,0x612A,
	/* Array index: 0x1424 */ 0x7A08,0x4A59,
	/* Array index: 0x1424 */ 0x7A0B,0x6F6F,
	/* Array index: 0x1424 */ 0x7A0D,0x752A,
	/* Array index: 0x1424 */ 0x7A14,0x6C79,
	/* Array index: 0x1424 */ 0x7A17,0x782D,
	/* Array index: 0x1424 */ 0x7A19,0x7242,
	/* Array index: 0x1424 */ 0x7A1A,0x7643,
	/* Array index: 0x1424 */ 0x7A1C,0x5752,
	/* Array index: 0x1424 */ 0x7A1F,0x7922,
	/* Array index: 0x1424 */ 0x7A20,0x7056,
	/* Array index: 0x1424 */ 0x7A2E,0x707A,
	/* Array index: 0x1424 */ 0x7A31,0x7660,
	/* Array index: 0x1424 */ 0x7A36,0x6973,
	/* Array index: 0x1424 */ 0x7A37,0x7243,
	/* Array index: 0x1424 */ 0x7A46,0x594F,
	/* Array index: 0x1424 */ 0x7A49,0x7644,
	/* Array index: 0x1424 */ 0x7A4D,0x6E5D,
	/* Array index: 0x1424 */ 0x7A4E,0x6744,
	/* Array index: 0x1424 */ 0x7A57,0x6234,
	/* Array index: 0x1424 */ 0x7A61,0x5F62,
	/* Array index: 0x1424 */ 0x7A62,0x675B,
	/* Array index: 0x1424 */ 0x7A69,0x6831,
	/* Array index: 0x1424 */ 0x7A6B,0x7C2E,
	/* Array index: 0x1424 */ 0x7A70,0x654D,
	/* Array index: 0x1424 */ 0x7A74,0x7A6B,
	/* Array index: 0x1424 */ 0x7A76,0x4F3C,
	/* Array index: 0x1424 */ 0x7A79,0x4F62,
	/* Array index: 0x1424 */ 0x7A7A,0x4D76,
	/* Array index: 0x1424 */ 0x7A7D,0x6F70,
	/* Array index: 0x1424 */ 0x7A7F,0x743E,
	/* Array index: 0x1424 */ 0x7A81,0x544D,
	/* Array index: 0x1424 */ 0x7A84,0x7338,
	/* Array index: 0x1424 */ 0x7A88,0x6921,
	/* Array index: 0x1424 */ 0x7A92,0x7272,
	/* Array index: 0x1424 */ 0x7A93,0x736B,
	/* Array index: 0x1424 */ 0x7A95,0x7057,
	/* Array index: 0x1424 */ 0x7A98,0x4F57,
	/* Array index: 0x1424 */ 0x7A9F,0x4F5F,
	/* Array index: 0x1424 */ 0x7AA9,0x6840,
	/* Array index: 0x1424 */ 0x7AAA,0x6841,
	/* Array index: 0x1424 */ 0x7AAE,0x4F63,
	/* Array index: 0x1424 */ 0x7AAF,0x6922,
	/* Array index: 0x1424 */ 0x7ABA,0x502A,
	/* Array index: 0x1424 */ 0x7AC4,0x7341,
	/* Array index: 0x1424 */ 0x7AC5,0x502B,
	/* Array index: 0x1424 */ 0x7AC7,0x5464,
	/* Array index: 0x1424 */ 0x7ACA,0x6F3C,
	/* Array index: 0x1424 */ 0x7ACB,0x5821,
	/* Array index: 0x1424 */ 0x7AD7,0x595F,
	/* Array index: 0x1424 */ 0x7AD9,0x7357,
	/* Array index: 0x1424 */ 0x7ADD,0x5C3D,
	/* Array index: 0x1424 */ 0x7ADF,0x4C65,
	/* Array index: 0x1424 */ 0x7AE0,0x6D71,
	/* Array index: 0x1424 */ 0x7AE3,0x7162,
	/* Array index: 0x1424 */ 0x7AE5,0x545B,
	/* Array index: 0x1424 */ 0x7AEA,0x6235,
	/* Array index: 0x1424 */ 0x7AED,0x4A66,
	/* Array index: 0x1424 */ 0x7AEF,0x532E,
	/* Array index: 0x1424 */ 0x7AF6,0x4C66,
	/* Array index: 0x1424 */ 0x7AF9,0x7153,
	/* Array index: 0x1424 */ 0x7AFA,0x7567,
	/* Array index: 0x1424 */ 0x7AFF,0x4A5A,
	/* Array index: 0x1424 */ 0x7B0F,0x7B6E,
	/* Array index: 0x1424 */ 0x7B11,0x6145,
	/* Array index: 0x1424 */ 0x7B19,0x5F69,
	/* Array index: 0x1424 */ 0x7B1B,0x6E5E,
	/* Array index: 0x1424 */ 0x7B1E,0x7742,
	/* Array index: 0x1424 */ 0x7B20,0x5822,
	/* Array index: 0x1424 */ 0x7B26,0x5D2C,
	/* Array index: 0x1424 */ 0x7B2C,0x702F,
	/* Array index: 0x1424 */ 0x7B2D,0x563D,
	/* Array index: 0x1424 */ 0x7B39,0x612B,
	/* Array index: 0x1424 */ 0x7B46,0x7936,
	/* Array index: 0x1424 */ 0x7B60,0x5035,
	/* Array index: 0x1424 */ 0x7B6C,0x607E,
	/* Array index: 0x1424 */ 0x7B6E,0x5F7B,
	/* Array index: 0x1424 */ 0x7B75,0x665D,
	/* Array index: 0x1424 */ 0x7B7D,0x6824,
	/* Array index: 0x1424 */ 0x7B87,0x4B4D,
	/* Array index: 0x1424 */ 0x7B8B,0x6F28,
	/* Array index: 0x1424 */ 0x7B8F,0x6E34,
	/* Array index: 0x1424 */ 0x7B94,0x5A58,
	/* Array index: 0x1424 */ 0x7B95,0x5139,
	/* Array index: 0x1424 */ 0x7B97,0x5F29,
	/* Array index: 0x1424 */ 0x7B9A,0x7330,
	/* Array index: 0x1424 */ 0x7B9D,0x4C44,
	/* Array index: 0x1424 */ 0x7BA1,0x4E37,
	/* Array index: 0x1424 */ 0x7BAD,0x6F29,
	/* Array index: 0x1424 */ 0x7BB1,0x5F55,
	/* Array index: 0x1424 */ 0x7BB4,0x6D57,
	/* Array index: 0x1424 */ 0x7BB8,0x6E46,
	/* Array index: 0x1424 */ 0x7BC0,0x6F3D,
	/* Array index: 0x1424 */ 0x7BC1,0x7C56,
	/* Array index: 0x1424 */ 0x7BC4,0x5B74,
	/* Array index: 0x1424 */ 0x7BC6,0x6F2A,
	/* Array index: 0x1424 */ 0x7BC7,0x7839,
	/* Array index: 0x1424 */ 0x7BC9,0x7569,
	/* Array index: 0x1424 */ 0x7BD2,0x6359,
	/* Array index: 0x1424 */ 0x7BE0,0x6146,
	/* Array index: 0x1424 */ 0x7BE4,0x543F,
	/* Array index: 0x1424 */ 0x7BE9,0x5E68,
	/* Array index: 0x1424 */ 0x7C07,0x706A,
	/* Array index: 0x1424 */ 0x7C12,0x7342,
	/* Array index: 0x1424 */ 0x7C1E,0x532F,
	/* Array index: 0x1424 */ 0x7C21,0x4A5B,
	/* Array index: 0x1424 */ 0x7C27,0x7C57,
	/* Array index: 0x1424 */ 0x7C2A,0x6D58,
	/* Array index: 0x1424 */ 0x7C2B,0x6147,
	/* Array index: 0x1424 */ 0x7C43,0x553E,
	/* Array index: 0x1424 */ 0x7C4C,0x7143,
	/* Array index: 0x1424 */ 0x7C4D,0x6E5F,
	/* Array index: 0x1424 */ 0x7C60,0x566B,
	/* Array index: 0x1424 */ 0x7C64,0x7459,
	/* Array index: 0x1424 */ 0x7C6C,0x5766,
	/* Array index: 0x1424 */ 0x7C73,0x5A37,
	/* Array index: 0x1424 */ 0x7C83,0x5D7B,
	/* Array index: 0x1424 */ 0x7C89,0x5D4F,
	/* Array index: 0x1424 */ 0x7C92,0x5823,
	/* Array index: 0x1424 */ 0x7C95,0x5A59,
	/* Array index: 0x1424 */ 0x7C97,0x7058,
	/* Array index: 0x1424 */ 0x7C98,0x6F44,
	/* Array index: 0x1424 */ 0x7C9F,0x6158,
	/* Array index: 0x1424 */ 0x7CA5,0x7154,
	/* Array index: 0x1424 */ 0x7CA7,0x6D72,
	/* Array index: 0x1424 */ 0x7CAE,0x555B,
	/* Array index: 0x1424 */ 0x7CB9,0x6236,
	/* Array index: 0x1424 */ 0x7CBE,0x6F71,
	/* Array index: 0x1424 */ 0x7CCA,0x7B55,
	/* Array index: 0x1424 */ 0x7CD6,0x5358,
	/* Array index: 0x1424 */ 0x7CE7,0x555D,
	/* Array index: 0x1424 */ 0x7CFB,0x4D27,
	/* Array index: 0x1424 */ 0x7CFE,0x502C,
	/* Array index: 0x1424 */ 0x7D00,0x513A,
	/* Array index: 0x1424 */ 0x7D10,0x526F,
	/* Array index: 0x1424 */ 0x7D14,0x626D,
	/* Array index: 0x1424 */ 0x7D33,0x636B,
	/* Array index: 0x1424 */ 0x7D35,0x6E47,
	/* Array index: 0x1424 */ 0x7D39,0x6149,
	/* Array index: 0x1424 */ 0x7D3A,0x4A7A,
	/* Array index: 0x1424 */ 0x7D50,0x4C3F,
	/* Array index: 0x1424 */ 0x7D5E,0x4E6D,
	/* Array index: 0x1424 */ 0x7D61,0x5529,
	/* Array index: 0x1424 */ 0x7D62,0x7A62,
	/* Array index: 0x1424 */ 0x7D66,0x5065,
	/* Array index: 0x1424 */ 0x7D68,0x6B56,
	/* Array index: 0x1424 */ 0x7D6A,0x6C5F,
	/* Array index: 0x1424 */ 0x7D6E,0x5F7C,
	/* Array index: 0x1424 */ 0x7D76,0x6F3E,
	/* Array index: 0x1424 */ 0x7D79,0x4C35,
	/* Array index: 0x1424 */ 0x7D7F,0x4F3D,
	/* Array index: 0x1424 */ 0x7D8E,0x6F72,
	/* Array index: 0x1424 */ 0x7D8F,0x6237,
	/* Array index: 0x1424 */ 0x7D93,0x4C68,
	/* Array index: 0x1424 */ 0x7D9C,0x707C,
	/* Array index: 0x1424 */ 0x7DA0,0x5660,
	/* Array index: 0x1424 */ 0x7DA2,0x7146,
	/* Array index: 0x1424 */ 0x7DAC,0x6238,
	/* Array index: 0x1424 */ 0x7DAD,0x6B2B,
	/* Array index: 0x1424 */ 0x7DB8,0x5746,
	/* Array index: 0x1424 */ 0x7DC7,0x7645,
	/* Array index: 0x1424 */ 0x7DCA,0x514C,
	/* Array index: 0x1424 */ 0x7DCB,0x5D7C,
	/* Array index: 0x1424 */ 0x7DD6,0x5F7D,
	/* Array index: 0x1424 */ 0x7DD8,0x7965,
	/* Array index: 0x1424 */ 0x7DDA,0x604A,
	/* Array index: 0x1424 */ 0x7DE8,0x783A,
	/* Array index: 0x1424 */ 0x7DE9,0x6850,
	/* Array index: 0x1424 */ 0x7DEC,0x587B,
	/* Array index: 0x1424 */ 0x7DEF,0x6A55,
	/* Array index: 0x1424 */ 0x7DF4,0x5623,
	/* Array index: 0x1424 */ 0x7DFB,0x7646,
	/* Array index: 0x1424 */ 0x7E09,0x725B,
	/* Array index: 0x1424 */ 0x7E0A,0x647C,
	/* Array index: 0x1424 */ 0x7E15,0x6832,
	/* Array index: 0x1424 */ 0x7E2B,0x5C6E,
	/* Array index: 0x1424 */ 0x7E2E,0x756A,
	/* Array index: 0x1424 */ 0x7E2F,0x6660,
	/* Array index: 0x1424 */ 0x7E31,0x707D,
	/* Array index: 0x1424 */ 0x7E37,0x572C,
	/* Array index: 0x1424 */ 0x7E3D,0x7545,
	/* Array index: 0x1424 */ 0x7E3E,0x6E60,
	/* Array index: 0x1424 */ 0x7E41,0x5B65,
	/* Array index: 0x1424 */ 0x7E43,0x5D5E,
	/* Array index: 0x1424 */ 0x7E46,0x5970,
	/* Array index: 0x1424 */ 0x7E47,0x6923,
	/* Array index: 0x1424 */ 0x7E52,0x7179,
	/* Array index: 0x1424 */ 0x7E54,0x7244,
	/* Array index: 0x1424 */ 0x7E55,0x604B,
	/* Array index: 0x1424 */ 0x7E5E,0x6924,
	/* Array index: 0x1424 */ 0x7E61,0x6239,
	/* Array index: 0x1424 */ 0x7E70,0x705B,
	/* Array index: 0x1424 */ 0x7E79,0x663A,
	/* Array index: 0x1424 */ 0x7E7C,0x4D29,
	/* Array index: 0x1424 */ 0x7E82,0x7343,
	/* Array index: 0x1424 */ 0x7E8C,0x6159,
	/* Array index: 0x1424 */ 0x7E8F,0x6F2B,
	/* Array index: 0x1424 */ 0x7E93,0x6745,
	/* Array index: 0x1424 */ 0x7E96,0x6069,
	/* Array index: 0x1424 */ 0x7E98,0x7345,
	/* Array index: 0x1424 */ 0x7E9B,0x5440,
	/* Array index: 0x1424 */ 0x7E9C,0x553F,
	/* Array index: 0x1424 */ 0x7F36,0x5D2E,
	/* Array index: 0x1424 */ 0x7F38,0x797C,
	/* Array index: 0x1424 */ 0x7F3A,0x4C40,
	/* Array index: 0x1424 */ 0x7F4C,0x6522,
	/* Array index: 0x1424 */ 0x7F50,0x4E38,
	/* Array index: 0x1424 */ 0x7F54,0x5852,
	/* Array index: 0x1424 */ 0x7F55,0x7956,
	/* Array index: 0x1424 */ 0x7F6A,0x712A,
	/* Array index: 0x1424 */ 0x7F6B,0x4E51,
	/* Array index: 0x1424 */ 0x7F6E,0x7647,
	/* Array index: 0x1424 */ 0x7F70,0x5B6B,
	/* Array index: 0x1424 */ 0x7F72,0x5F7E,
	/* Array index: 0x1424 */ 0x7F75,0x5861,
	/* Array index: 0x1424 */ 0x7F77,0x7773,
	/* Array index: 0x1424 */ 0x7F79,0x5767,
	/* Array index: 0x1424 */ 0x7F85,0x547E,
	/* Array index: 0x1424 */ 0x7F88,0x513C,
	/* Array index: 0x1424 */ 0x7F8A,0x654F,
	/* Array index: 0x1424 */ 0x7F8C,0x4B36,
	/* Array index: 0x1424 */ 0x7F8E,0x5A38,
	/* Array index: 0x1424 */ 0x7F94,0x4D44,
	/* Array index: 0x1424 */ 0x7F9A,0x563E,
	/* Array index: 0x1424 */ 0x7F9E,0x623A,
	/* Array index: 0x1424 */ 0x7FA4,0x4F58,
	/* Array index: 0x1424 */ 0x7FA8,0x604C,
	/* Array index: 0x1424 */ 0x7FA9,0x6B79,
	/* Array index: 0x1424 */ 0x7FB2,0x7D7D,
	/* Array index: 0x1424 */ 0x7FB8,0x5768,
	/* Array index: 0x1424 */ 0x7FB9,0x4B58,
	/* Array index: 0x1424 */ 0x7FBD,0x6962,
	/* Array index: 0x1424 */ 0x7FC1,0x683A,
	/* Array index: 0x1424 */ 0x7FC5,0x6347,
	/* Array index: 0x1424 */ 0x7FCA,0x6C4D,
	/* Array index: 0x1424 */ 0x7FCC,0x6C4E,
	/* Array index: 0x1424 */ 0x7FCE,0x563F,
	/* Array index: 0x1424 */ 0x7FD2,0x6327,
	/* Array index: 0x1424 */ 0x7FD4,0x5F56,
	/* Array index: 0x1424 */ 0x7FD5,0x7D68,
	/* Array index: 0x1424 */ 0x7FE9,0x783B,
	/* Array index: 0x1424 */ 0x7FEB,0x6851,
	/* Array index: 0x1424 */ 0x7FF0,0x7957,
	/* Array index: 0x1424 */ 0x7FF9,0x4E6E,
	/* Array index: 0x1424 */ 0x7FFC,0x6C4F,
	/* Array index: 0x1424 */ 0x8000,0x6925,
	/* Array index: 0x1424 */ 0x8001,0x5655,
	/* Array index: 0x1424 */ 0x8003,0x4D45,
	/* Array index: 0x1424 */ 0x8005,0x6D3A,
	/* Array index: 0x1424 */ 0x8006,0x513D,
	/* Array index: 0x1424 */ 0x8009,0x4F3E,
	/* Array index: 0x1424 */ 0x800C,0x6C3B,
	/* Array index: 0x1424 */ 0x8010,0x5231,
	/* Array index: 0x1424 */ 0x8015,0x4C69,
	/* Array index: 0x1424 */ 0x8017,0x5944,
	/* Array index: 0x1424 */ 0x8018,0x697C,
	/* Array index: 0x1424 */ 0x802D,0x513E,
	/* Array index: 0x1424 */ 0x8033,0x6C3C,
	/* Array index: 0x1424 */ 0x8036,0x652D,
	/* Array index: 0x1424 */ 0x803D,0x7730,
	/* Array index: 0x1424 */ 0x803F,0x4C6A,
	/* Array index: 0x1424 */ 0x8043,0x5344,
	/* Array index: 0x1424 */ 0x8046,0x5640,
	/* Array index: 0x1424 */ 0x804A,0x567D,
	/* Array index: 0x1424 */ 0x8056,0x6121,
	/* Array index: 0x1424 */ 0x8058,0x5E3D,
	/* Array index: 0x1424 */ 0x805A,0x7629,
	/* Array index: 0x1424 */ 0x805E,0x5A24,
	/* Array index: 0x1424 */ 0x8077,0x7245,
	/* Array index: 0x1424 */ 0x8096,0x752B,
	/* Array index: 0x1424 */ 0x809B,0x797D,
	/* Array index: 0x1424 */ 0x809D,0x4A5C,
	/* Array index: 0x1424 */ 0x80A1,0x4D46,
	/* Array index: 0x1424 */ 0x80A2,0x7236,
	/* Array index: 0x1424 */ 0x80A5,0x5D7E,
	/* Array index: 0x1424 */ 0x80A9,0x4C37,
	/* Array index: 0x1424 */ 0x80AA,0x5B38,
	/* Array index: 0x1424 */ 0x80AF,0x5069,
	/* Array index: 0x1424 */ 0x80B1,0x4E5D,
	/* Array index: 0x1424 */ 0x80B2,0x6B40,
	/* Array index: 0x1424 */ 0x80B4,0x7D22,
	/* Array index: 0x1424 */ 0x80BA,0x784B,
	/* Array index: 0x1424 */ 0x80C3,0x6A56,
	/* Array index: 0x1424 */ 0x80C4,0x7130,
	/* Array index: 0x1424 */ 0x80CC,0x5B4E,
	/* Array index: 0x1424 */ 0x80CE,0x7743,
	/* Array index: 0x1424 */ 0x80DA,0x5B4F,
	/* Array index: 0x1424 */ 0x80DB,0x4B24,
	/* Array index: 0x1424 */ 0x80DE,0x7860,
	/* Array index: 0x1424 */ 0x80E1,0x7B57,
	/* Array index: 0x1424 */ 0x80E4,0x6B4A,
	/* Array index: 0x1424 */ 0x80E5,0x6021,
	/* Array index: 0x1424 */ 0x80F1,0x4E4D,
	/* Array index: 0x1424 */ 0x80F4,0x545C,
	/* Array index: 0x1424 */ 0x80F8,0x7D58,
	/* Array index: 0x1424 */ 0x80FD,0x5276,
	/* Array index: 0x1424 */ 0x8102,0x7237,
	/* Array index: 0x1424 */ 0x8118,0x6852,
	/* Array index: 0x1424 */ 0x811A,0x4A45,
	/* Array index: 0x1424 */ 0x811B,0x4C6B,
	/* Array index: 0x1424 */ 0x8123,0x626E,
	/* Array index: 0x1424 */ 0x8129,0x623B,
	/* Array index: 0x1424 */ 0x812B,0x772D,
	/* Array index: 0x1424 */ 0x812F,0x7861,
	/* Array index: 0x1424 */ 0x8139,0x736C,
	/* Array index: 0x1424 */ 0x813E,0x5E21,
	/* Array index: 0x1424 */ 0x814B,0x647D,
	/* Array index: 0x1424 */ 0x814E,0x636C,
	/* Array index: 0x1424 */ 0x8150,0x5D2F,
	/* Array index: 0x1424 */ 0x8151,0x5D30,
	/* Array index: 0x1424 */ 0x8154,0x4B37,
	/* Array index: 0x1424 */ 0x8155,0x6853,
	/* Array index: 0x1424 */ 0x8165,0x6123,
	/* Array index: 0x1424 */ 0x8166,0x5260,
	/* Array index: 0x1424 */ 0x816B,0x707E,
	/* Array index: 0x1424 */ 0x8170,0x6926,
	/* Array index: 0x1424 */ 0x8171,0x4B72,
	/* Array index: 0x1424 */ 0x817F,0x775A,
	/* Array index: 0x1424 */ 0x8180,0x5B39,
	/* Array index: 0x1424 */ 0x8188,0x4C2E,
	/* Array index: 0x1424 */ 0x818A,0x5A5B,
	/* Array index: 0x1424 */ 0x818F,0x4D47,
	/* Array index: 0x1424 */ 0x819A,0x5D31,
	/* Array index: 0x1424 */ 0x819C,0x582F,
	/* Array index: 0x1424 */ 0x819D,0x6323,
	/* Array index: 0x1424 */ 0x81A0,0x4E6F,
	/* Array index: 0x1424 */ 0x81A3,0x7273,
	/* Array index: 0x1424 */ 0x81A8,0x7833,
	/* Array index: 0x1424 */ 0x81B3,0x604E,
	/* Array index: 0x1424 */ 0x81B5,0x757D,
	/* Array index: 0x1424 */ 0x81BA,0x6B6C,
	/* Array index: 0x1424 */ 0x81C6,0x6566,
	/* Array index: 0x1424 */ 0x81CD,0x7030,
	/* Array index: 0x1424 */ 0x81D8,0x5544,
	/* Array index: 0x1424 */ 0x81DF,0x6D74,
	/* Array index: 0x1424 */ 0x81E3,0x636D,
	/* Array index: 0x1424 */ 0x81E5,0x6842,
	/* Array index: 0x1424 */ 0x81E7,0x6D75,
	/* Array index: 0x1424 */ 0x81E8,0x577C,
	/* Array index: 0x1424 */ 0x81EA,0x6D3B,
	/* Array index: 0x1424 */ 0x81ED,0x762B,
	/* Array index: 0x1424 */ 0x81F3,0x7238,
	/* Array index: 0x1424 */ 0x81F4,0x7648,
	/* Array index: 0x1424 */ 0x8205,0x4F40,
	/* Array index: 0x1424 */ 0x8207,0x6628,
	/* Array index: 0x1424 */ 0x8208,0x7D69,
	/* Array index: 0x1424 */ 0x820A,0x4F41,
	/* Array index: 0x1424 */ 0x820C,0x605F,
	/* Array index: 0x1424 */ 0x820D,0x5E6C,
	/* Array index: 0x1424 */ 0x8212,0x6022,
	/* Array index: 0x1424 */ 0x8240,0x5D32,
	/* Array index: 0x1424 */ 0x8245,0x6629,
	/* Array index: 0x1424 */ 0x8247,0x6F73,
	/* Array index: 0x1424 */ 0x8259,0x736D,
	/* Array index: 0x1424 */ 0x8264,0x6B7A,
	/* Array index: 0x1424 */ 0x8266,0x7966,
	/* Array index: 0x1424 */ 0x8276,0x667D,
	/* Array index: 0x1424 */ 0x8278,0x752C,
	/* Array index: 0x1424 */ 0x827E,0x6475,
	/* Array index: 0x1424 */ 0x828B,0x6963,
	/* Array index: 0x1424 */ 0x828D,0x6D4B,
	/* Array index: 0x1424 */ 0x828E,0x4F64,
	/* Array index: 0x1424 */ 0x8292,0x5853,
	/* Array index: 0x1424 */ 0x8299,0x5D33,
	/* Array index: 0x1424 */ 0x829A,0x546C,
	/* Array index: 0x1424 */ 0x829D,0x7239,
	/* Array index: 0x1424 */ 0x829F,0x5F37,
	/* Array index: 0x1424 */ 0x82A5,0x4B4E,
	/* Array index: 0x1424 */ 0x82A6,0x7B58,
	/* Array index: 0x1424 */ 0x82A9,0x5059,
	/* Array index: 0x1424 */ 0x82DB,0x4A34,
	/* Array index: 0x1424 */ 0x82DE,0x7862,
	/* Array index: 0x1424 */ 0x82DF,0x4F42,
	/* Array index: 0x1424 */ 0x82E1,0x6C3E,
	/* Array index: 0x1424 */ 0x82F1,0x6748,
	/* Array index: 0x1424 */ 0x8309,0x5849,
	/* Array index: 0x1424 */ 0x8317,0x592B,
	/* Array index: 0x1424 */ 0x8328,0x6D3C,
	/* Array index: 0x1424 */ 0x832B,0x5854,
	/* Array index: 0x1424 */ 0x832F,0x5C5A,
	/* Array index: 0x1424 */ 0x8331,0x623C,
	/* Array index: 0x1424 */ 0x8340,0x6270,
	/* Array index: 0x1424 */ 0x8347,0x7A3B,
	/* Array index: 0x1424 */ 0x8349,0x752E,
	/* Array index: 0x1424 */ 0x834A,0x7B2A,
	/* Array index: 0x1424 */ 0x834F,0x6C7B,
	/* Array index: 0x1424 */ 0x8351,0x6C3F,
	/* Array index: 0x1424 */ 0x8352,0x7C58,
	/* Array index: 0x1424 */ 0x8373,0x5465,
	/* Array index: 0x1424 */ 0x8377,0x7943,
	/* Array index: 0x1424 */ 0x837B,0x6E62,
	/* Array index: 0x1424 */ 0x8389,0x5769,
	/* Array index: 0x1424 */ 0x838A,0x6D76,
	/* Array index: 0x1424 */ 0x838E,0x5E6D,
	/* Array index: 0x1424 */ 0x8396,0x4C6C,
	/* Array index: 0x1424 */ 0x8398,0x636E,
	/* Array index: 0x1424 */ 0x839E,0x6854,
	/* Array index: 0x1424 */ 0x83A2,0x7A78,
	/* Array index: 0x1424 */ 0x83BD,0x5855,
	/* Array index: 0x1424 */ 0x83C1,0x746A,
	/* Array index: 0x1424 */ 0x83C5,0x4E39,
	/* Array index: 0x1424 */ 0x83C9,0x5661,
	/* Array index: 0x1424 */ 0x83CA,0x4F52,
	/* Array index: 0x1424 */ 0x83CC,0x5036,
	/* Array index: 0x1424 */ 0x83D3,0x4E22,
	/* Array index: 0x1424 */ 0x83D6,0x736E,
	/* Array index: 0x1424 */ 0x83DC,0x7378,
	/* Array index: 0x1424 */ 0x83E9,0x5C4C,
	/* Array index: 0x1424 */ 0x83EB,0x504B,
	/* Array index: 0x1424 */ 0x83F9,0x6E49,
	/* Array index: 0x1424 */ 0x83FD,0x625D,
	/* Array index: 0x1424 */ 0x8403,0x757E,
	/* Array index: 0x1424 */ 0x8404,0x542C,
	/* Array index: 0x1424 */ 0x8429,0x7557,
	/* Array index: 0x1424 */ 0x842C,0x583F,
	/* Array index: 0x1424 */ 0x8431,0x7D40,
	/* Array index: 0x1424 */ 0x8438,0x6B2D,
	/* Array index: 0x1424 */ 0x843D,0x552A,
	/* Array index: 0x1424 */ 0x8449,0x6728,
	/* Array index: 0x1424 */ 0x8457,0x6E4A,
	/* Array index: 0x1424 */ 0x845B,0x4A67,
	/* Array index: 0x1424 */ 0x8461,0x7863,
	/* Array index: 0x1424 */ 0x8463,0x545D,
	/* Array index: 0x1424 */ 0x8466,0x6A58,
	/* Array index: 0x1424 */ 0x846B,0x7B59,
	/* Array index: 0x1424 */ 0x846C,0x6D77,
	/* Array index: 0x1424 */ 0x846F,0x6535,
	/* Array index: 0x1424 */ 0x8475,0x502D,
	/* Array index: 0x1424 */ 0x847A,0x7171,
	/* Array index: 0x1424 */ 0x8490,0x623D,
	/* Array index: 0x1424 */ 0x8494,0x6348,
	/* Array index: 0x1424 */ 0x8499,0x5955,
	/* Array index: 0x1424 */ 0x849C,0x5F2A,
	/* Array index: 0x1424 */ 0x84A1,0x5B3C,
	/* Array index: 0x1424 */ 0x84B2,0x7864,
	/* Array index: 0x1424 */ 0x84B8,0x717A,
	/* Array index: 0x1424 */ 0x84BB,0x6536,
	/* Array index: 0x1424 */ 0x84BC,0x736F,
	/* Array index: 0x1424 */ 0x84BF,0x7B5A,
	/* Array index: 0x1424 */ 0x84C0,0x6160,
	/* Array index: 0x1424 */ 0x84C2,0x592C,
	/* Array index: 0x1424 */ 0x84C4,0x756B,
	/* Array index: 0x1424 */ 0x84C6,0x6036,
	/* Array index: 0x1424 */ 0x84C9,0x6948,
	/* Array index: 0x1424 */ 0x84CB,0x4B4F,
	/* Array index: 0x1424 */ 0x84CD,0x6349,
	/* Array index: 0x1424 */ 0x84D1,0x5E6E,
	/* Array index: 0x1424 */ 0x84DA,0x623E,
	/* Array index: 0x1424 */ 0x84EC,0x5C6F,
	/* Array index: 0x1424 */ 0x84EE,0x5625,
	/* Array index: 0x1424 */ 0x84F4,0x6271,
	/* Array index: 0x1424 */ 0x84FC,0x567E,
	/* Array index: 0x1424 */ 0x8511,0x5921,
	/* Array index: 0x1424 */ 0x8513,0x5840,
	/* Array index: 0x1424 */ 0x8514,0x5C5B,
	/* Array index: 0x1424 */ 0x8517,0x6D3D,
	/* Array index: 0x1424 */ 0x8518,0x5F38,
	/* Array index: 0x1424 */ 0x851A,0x6A25,
	/* Array index: 0x1424 */ 0x851E,0x572D,
	/* Array index: 0x1424 */ 0x8521,0x7379,
	/* Array index: 0x1424 */ 0x8523,0x6D78,
	/* Array index: 0x1424 */ 0x8525,0x7547,
	/* Array index: 0x1424 */ 0x852C,0x614A,
	/* Array index: 0x1424 */ 0x852D,0x6B63,
	/* Array index: 0x1424 */ 0x852F,0x725E,
	/* Array index: 0x1424 */ 0x853D,0x784C,
	/* Array index: 0x1424 */ 0x853F,0x6A59,
	/* Array index: 0x1424 */ 0x8541,0x5346,
	/* Array index: 0x1424 */ 0x8543,0x5B66,
	/* Array index: 0x1424 */ 0x8549,0x752F,
	/* Array index: 0x1424 */ 0x854E,0x4E70,
	/* Array index: 0x1424 */ 0x8553,0x697E,
	/* Array index: 0x1424 */ 0x8559,0x7B36,
	/* Array index: 0x1424 */ 0x8563,0x6272,
	/* Array index: 0x1424 */ 0x856D,0x614B,
	/* Array index: 0x1424 */ 0x8584,0x5A5D,
	/* Array index: 0x1424 */ 0x8587,0x5A39,
	/* Array index: 0x1424 */ 0x858F,0x6B7B,
	/* Array index: 0x1424 */ 0x8591,0x4B39,
	/* Array index: 0x1424 */ 0x8594,0x6D79,
	/* Array index: 0x1424 */ 0x859B,0x6060,
	/* Array index: 0x1424 */ 0x85AF,0x6023,
	/* Array index: 0x1424 */ 0x85B0,0x7D39,
	/* Array index: 0x1424 */ 0x85BA,0x7031,
	/* Array index: 0x1424 */ 0x85C1,0x4D4B,
	/* Array index: 0x1424 */ 0x85C9,0x6D3E,
	/* Array index: 0x1424 */ 0x85D5,0x6964,
	/* Array index: 0x1424 */ 0x85DC,0x556D,
	/* Array index: 0x1424 */ 0x85DD,0x675D,
	/* Array index: 0x1424 */ 0x85E4,0x5476,
	/* Array index: 0x1424 */ 0x85E5,0x6537,
	/* Array index: 0x1424 */ 0x85E9,0x5B67,
	/* Array index: 0x1424 */ 0x85EA,0x623F,
	/* Array index: 0x1424 */ 0x85F7,0x6E4B,
	/* Array index: 0x1424 */ 0x85FA,0x5774,
	/* Array index: 0x1424 */ 0x85FB,0x705D,
	/* Array index: 0x1424 */ 0x85FF,0x4E2B,
	/* Array index: 0x1424 */ 0x8602,0x675E,
	/* Array index: 0x1424 */ 0x8606,0x5656,
	/* Array index: 0x1424 */ 0x8607,0x614C,
	/* Array index: 0x1424 */ 0x860A,0x6833,
	/* Array index: 0x1424 */ 0x8616,0x656E,
	/* Array index: 0x1424 */ 0x8617,0x5C22,
	/* Array index: 0x1424 */ 0x861A,0x6050,
	/* Array index: 0x1424 */ 0x862D,0x5535,
	/* Array index: 0x1424 */ 0x863F,0x5521,
	/* Array index: 0x1424 */ 0x864E,0x7B5B,
	/* Array index: 0x1424 */ 0x8650,0x794B,
	/* Array index: 0x1424 */ 0x8654,0x4B73,
	/* Array index: 0x1424 */ 0x8655,0x7425,
	/* Array index: 0x1424 */ 0x8667,0x7D50,
	/* Array index: 0x1424 */ 0x8679,0x7B76,
	/* Array index: 0x1424 */ 0x868A,0x5A25,
	/* Array index: 0x1424 */ 0x868C,0x5B3D,
	/* Array index: 0x1424 */ 0x8693,0x6C62,
	/* Array index: 0x1424 */ 0x86A3,0x4D77,
	/* Array index: 0x1424 */ 0x86A4,0x705E,
	/* Array index: 0x1424 */ 0x86A9,0x7649,
	/* Array index: 0x1424 */ 0x86C7,0x5E6F,
	/* Array index: 0x1424 */ 0x86CB,0x5331,
	/* Array index: 0x1424 */ 0x86D4,0x7C6E,
	/* Array index: 0x1424 */ 0x86D9,0x6843,
	/* Array index: 0x1424 */ 0x86DB,0x7148,
	/* Array index: 0x1424 */ 0x86DF,0x4E71,
	/* Array index: 0x1424 */ 0x86E4,0x796D,
	/* Array index: 0x1424 */ 0x86ED,0x7274,
	/* Array index: 0x1424 */ 0x86FE,0x6436,
	/* Array index: 0x1424 */ 0x8700,0x7539,
	/* Array index: 0x1424 */ 0x8702,0x5C70,
	/* Array index: 0x1424 */ 0x8703,0x6371,
	/* Array index: 0x1424 */ 0x8708,0x6825,
	/* Array index: 0x1424 */ 0x8718,0x723B,
	/* Array index: 0x1424 */ 0x871A,0x5E24,
	/* Array index: 0x1424 */ 0x871C,0x5A4C,
	/* Array index: 0x1424 */ 0x874E,0x4A69,
	/* Array index: 0x1424 */ 0x8755,0x635A,
	/* Array index: 0x1424 */ 0x8757,0x7C59,
	/* Array index: 0x1424 */ 0x875F,0x6A5A,
	/* Array index: 0x1424 */ 0x8766,0x7944,
	/* Array index: 0x1424 */ 0x8768,0x6324,
	/* Array index: 0x1424 */ 0x8774,0x7B5D,
	/* Array index: 0x1424 */ 0x8776,0x6F4A,
	/* Array index: 0x1424 */ 0x8778,0x6844,
	/* Array index: 0x1424 */ 0x8782,0x554C,
	/* Array index: 0x1424 */ 0x878D,0x6B57,
	/* Array index: 0x1424 */ 0x879F,0x592D,
	/* Array index: 0x1424 */ 0x87A2,0x7B2B,
	/* Array index: 0x1424 */ 0x87B3,0x5359,
	/* Array index: 0x1424 */ 0x87BA,0x5522,
	/* Array index: 0x1424 */ 0x87C4,0x765E,
	/* Array index: 0x1424 */ 0x87E0,0x5A76,
	/* Array index: 0x1424 */ 0x87EC,0x6051,
	/* Array index: 0x1424 */ 0x87EF,0x6928,
	/* Array index: 0x1424 */ 0x87F2,0x7579,
	/* Array index: 0x1424 */ 0x87F9,0x7A2F,
	/* Array index: 0x1424 */ 0x87FB,0x6B7C,
	/* Array index: 0x1424 */ 0x87FE,0x606A,
	/* Array index: 0x1424 */ 0x8805,0x6332,
	/* Array index: 0x1424 */ 0x881F,0x5545,
	/* Array index: 0x1424 */ 0x8822,0x7163,
	/* Array index: 0x1424 */ 0x8823,0x556E,
	/* Array index: 0x1424 */ 0x8831,0x4D4C,
	/* Array index: 0x1424 */ 0x8836,0x6D59,
	/* Array index: 0x1424 */ 0x883B,0x5841,
	/* Array index: 0x1424 */ 0x8840,0x7A6C,
	/* Array index: 0x1424 */ 0x8846,0x716B,
	/* Array index: 0x1424 */ 0x884C,0x7A3C,
	/* Array index: 0x1424 */ 0x884D,0x6662,
	/* Array index: 0x1424 */ 0x8852,0x7A65,
	/* Array index: 0x1424 */ 0x8853,0x627A,
	/* Array index: 0x1424 */ 0x8857,0x4A36,
	/* Array index: 0x1424 */ 0x8859,0x6437,
	/* Array index: 0x1424 */ 0x885B,0x6A5B,
	/* Array index: 0x1424 */ 0x885D,0x757A,
	/* Array index: 0x1424 */ 0x8868,0x787A,
	/* Array index: 0x1424 */ 0x886B,0x5F39,
	/* Array index: 0x1424 */ 0x8870,0x6171,
	/* Array index: 0x1424 */ 0x8872,0x5224,
	/* Array index: 0x1424 */ 0x8877,0x757B,
	/* Array index: 0x1424 */ 0x8888,0x4A37,
	/* Array index: 0x1424 */ 0x888B,0x5367,
	/* Array index: 0x1424 */ 0x888D,0x7865,
	/* Array index: 0x1424 */ 0x8892,0x5332,
	/* Array index: 0x1424 */ 0x8896,0x6240,
	/* Array index: 0x1424 */ 0x8897,0x725F,
	/* Array index: 0x1424 */ 0x889E,0x4D65,
	/* Array index: 0x1424 */ 0x88AB,0x792C,
	/* Array index: 0x1424 */ 0x88B4,0x4D4D,
	/* Array index: 0x1424 */ 0x88C1,0x6E2E,
	/* Array index: 0x1424 */ 0x88C2,0x562E,
	/* Array index: 0x1424 */ 0x88CF,0x576A,
	/* Array index: 0x1424 */ 0x88D4,0x6760,
	/* Array index: 0x1424 */ 0x88D5,0x6B2E,
	/* Array index: 0x1424 */ 0x88D9,0x4F59,
	/* Array index: 0x1424 */ 0x88DC,0x5C4D,
	/* Array index: 0x1424 */ 0x88DD,0x6D7B,
	/* Array index: 0x1424 */ 0x88DF,0x5E70,
	/* Array index: 0x1424 */ 0x88E1,0x576B,
	/* Array index: 0x1424 */ 0x88E8,0x5E25,
	/* Array index: 0x1424 */ 0x88F8,0x5523,
	/* Array index: 0x1424 */ 0x88FD,0x7032,
	/* Array index: 0x1424 */ 0x8907,0x5C5C,
	/* Array index: 0x1424 */ 0x8910,0x4A68,
	/* Array index: 0x1424 */ 0x8912,0x7866,
	/* Array index: 0x1424 */ 0x8913,0x5C4E,
	/* Array index: 0x1424 */ 0x8918,0x6A5C,
	/* Array index: 0x1424 */ 0x8919,0x5B52,
	/* Array index: 0x1424 */ 0x8925,0x6933,
	/* Array index: 0x1424 */ 0x892A,0x775B,
	/* Array index: 0x1424 */ 0x8936,0x6328,
	/* Array index: 0x1424 */ 0x8938,0x572E,
	/* Array index: 0x1424 */ 0x893B,0x6061,
	/* Array index: 0x1424 */ 0x8941,0x4B3A,
	/* Array index: 0x1424 */ 0x8944,0x6551,
	/* Array index: 0x1424 */ 0x895F,0x505C,
	/* Array index: 0x1424 */ 0x8964,0x5541,
	/* Array index: 0x1424 */ 0x896A,0x584A,
	/* Array index: 0x1424 */ 0x8972,0x6329,
	/* Array index: 0x1424 */ 0x897F,0x6024,
	/* Array index: 0x1424 */ 0x8981,0x6929,
	/* Array index: 0x1424 */ 0x8983,0x5347,
	/* Array index: 0x1424 */ 0x8986,0x5C5D,
	/* Array index: 0x1424 */ 0x8987,0x782E,
	/* Array index: 0x1424 */ 0x898B,0x4C38,
	/* Array index: 0x1424 */ 0x898F,0x502E,
	/* Array index: 0x1424 */ 0x8993,0x5872,
	/* Array index: 0x1424 */ 0x8996,0x634A,
	/* Array index: 0x1424 */ 0x89A1,0x4C2F,
	/* Array index: 0x1424 */ 0x89A9,0x542D,
	/* Array index: 0x1424 */ 0x89AA,0x7651,
	/* Array index: 0x1424 */ 0x89B2,0x504C,
	/* Array index: 0x1424 */ 0x89BA,0x4A46,
	/* Array index: 0x1424 */ 0x89BD,0x5542,
	/* Array index: 0x1424 */ 0x89C0,0x4E3A,
	/* Array index: 0x1424 */ 0x89D2,0x4A47,
	/* Array index: 0x1424 */ 0x89E3,0x7A30,
	/* Array index: 0x1424 */ 0x89F4,0x5F58,
	/* Array index: 0x1424 */ 0x89F8,0x753A,
	/* Array index: 0x1424 */ 0x8A00,0x656B,
	/* Array index: 0x1424 */ 0x8A02,0x6F74,
	/* Array index: 0x1424 */ 0x8A03,0x5D35,
	/* Array index: 0x1424 */ 0x8A08,0x4D2A,
	/* Array index: 0x1424 */ 0x8A0A,0x6372,
	/* Array index: 0x1424 */ 0x8A0C,0x7B77,
	/* Array index: 0x1424 */ 0x8A0E,0x7750,
	/* Array index: 0x1424 */ 0x8A13,0x7D3A,
	/* Array index: 0x1424 */ 0x8A1B,0x6845,
	/* Array index: 0x1424 */ 0x8A1D,0x6438,
	/* Array index: 0x1424 */ 0x8A1F,0x6168,
	/* Array index: 0x1424 */ 0x8A23,0x4C41,
	/* Array index: 0x1424 */ 0x8A25,0x526D,
	/* Array index: 0x1424 */ 0x8A2A,0x5B3E,
	/* Array index: 0x1424 */ 0x8A2D,0x6062,
	/* Array index: 0x1424 */ 0x8A31,0x7A49,
	/* Array index: 0x1424 */ 0x8A34,0x614D,
	/* Array index: 0x1424 */ 0x8A36,0x4A38,
	/* Array index: 0x1424 */ 0x8A3A,0x7260,
	/* Array index: 0x1424 */ 0x8A3B,0x7149,
	/* Array index: 0x1424 */ 0x8A50,0x5E71,
	/* Array index: 0x1424 */ 0x8A54,0x705F,
	/* Array index: 0x1424 */ 0x8A55,0x7844,
	/* Array index: 0x1424 */ 0x8A5B,0x6E4C,
	/* Array index: 0x1424 */ 0x8A5E,0x5E72,
	/* Array index: 0x1424 */ 0x8A60,0x6749,
	/* Array index: 0x1424 */ 0x8A62,0x6273,
	/* Array index: 0x1424 */ 0x8A63,0x6761,
	/* Array index: 0x1424 */ 0x8A66,0x634B,
	/* Array index: 0x1424 */ 0x8A69,0x634C,
	/* Array index: 0x1424 */ 0x8A79,0x745A,
	/* Array index: 0x1424 */ 0x8A85,0x714A,
	/* Array index: 0x1424 */ 0x8A87,0x4E23,
	/* Array index: 0x1424 */ 0x8A8C,0x723C,
	/* Array index: 0x1424 */ 0x8A8D,0x6C63,
	/* Array index: 0x1424 */ 0x8A93,0x6025,
	/* Array index: 0x1424 */ 0x8A95,0x772B,
	/* Array index: 0x1424 */ 0x8A98,0x6B2F,
	/* Array index: 0x1424 */ 0x8A9E,0x655E,
	/* Array index: 0x1424 */ 0x8AB0,0x6241,
	/* Array index: 0x1424 */ 0x8AB2,0x4E24,
	/* Array index: 0x1424 */ 0x8AB9,0x5E26,
	/* Array index: 0x1424 */ 0x8ABC,0x6B7E,
	/* Array index: 0x1424 */ 0x8ABE,0x6B5D,
	/* Array index: 0x1424 */ 0x8ABF,0x7060,
	/* Array index: 0x1424 */ 0x8AC2,0x745B,
	/* Array index: 0x1424 */ 0x8AC4,0x6274,
	/* Array index: 0x1424 */ 0x8AC7,0x5348,
	/* Array index: 0x1424 */ 0x8ACB,0x746B,
	/* Array index: 0x1424 */ 0x8ACD,0x6E35,
	/* Array index: 0x1424 */ 0x8ACF,0x7558,
	/* Array index: 0x1424 */ 0x8AD2,0x555F,
	/* Array index: 0x1424 */ 0x8AD6,0x5665,
	/* Array index: 0x1424 */ 0x8ADB,0x6B30,
	/* Array index: 0x1424 */ 0x8ADC,0x7463,
	/* Array index: 0x1424 */ 0x8AE1,0x634D,
	/* Array index: 0x1424 */ 0x8AE6,0x7474,
	/* Array index: 0x1424 */ 0x8AE7,0x7A32,
	/* Array index: 0x1424 */ 0x8AF1,0x7D49,
	/* Array index: 0x1424 */ 0x8B0E,0x5A3A,
	/* Array index: 0x1424 */ 0x8B10,0x5A4D,
	/* Array index: 0x1424 */ 0x8B14,0x794C,
	/* Array index: 0x1424 */ 0x8B20,0x692A,
	/* Array index: 0x1424 */ 0x8B28,0x5948,
	/* Array index: 0x1424 */ 0x8B2B,0x6E63,
	/* Array index: 0x1424 */ 0x8B2C,0x573D,
	/* Array index: 0x1424 */ 0x8B33,0x4F44,
	/* Array index: 0x1424 */ 0x8B39,0x504D,
	/* Array index: 0x1424 */ 0x8B41,0x7C26,
	/* Array index: 0x1424 */ 0x8B49,0x717B,
	/* Array index: 0x1424 */ 0x8B4E,0x7D52,
	/* Array index: 0x1424 */ 0x8B4F,0x5141,
	/* Array index: 0x1424 */ 0x8B58,0x635B,
	/* Array index: 0x1424 */ 0x8B5A,0x5349,
	/* Array index: 0x1424 */ 0x8B5C,0x5C4F,
	/* Array index: 0x1424 */ 0x8B66,0x4C6D,
	/* Array index: 0x1424 */ 0x8B6C,0x5E27,
	/* Array index: 0x1424 */ 0x8B6F,0x663B,
	/* Array index: 0x1424 */ 0x8B70,0x6C21,
	/* Array index: 0x1424 */ 0x8B74,0x4C39,
	/* Array index: 0x1424 */ 0x8B77,0x7B5E,
	/* Array index: 0x1424 */ 0x8B7D,0x6762,
	/* Array index: 0x1424 */ 0x8B80,0x5441,
	/* Array index: 0x1424 */ 0x8B8A,0x5C28,
	/* Array index: 0x1424 */ 0x8B90,0x6242,
	/* Array index: 0x1424 */ 0x8B92,0x7358,
	/* Array index: 0x1424 */ 0x8B93,0x6553,
	/* Array index: 0x1424 */ 0x8B96,0x7359,
	/* Array index: 0x1424 */ 0x8B9A,0x7346,
	/* Array index: 0x1424 */ 0x8C37,0x4D5B,
	/* Array index: 0x1424 */ 0x8C3F,0x4D2C,
	/* Array index: 0x1424 */ 0x8C41,0x7C43,
	/* Array index: 0x1424 */ 0x8C46,0x5467,
	/* Array index: 0x1424 */ 0x8C48,0x5142,
	/* Array index: 0x1424 */ 0x8C4A,0x7925,
	/* Array index: 0x1424 */ 0x8C4C,0x6855,
	/* Array index: 0x1424 */ 0x8C55,0x634E,
	/* Array index: 0x1424 */ 0x8C5A,0x544A,
	/* Array index: 0x1424 */ 0x8C61,0x5F5A,
	/* Array index: 0x1424 */ 0x8C6A,0x7B5F,
	/* Array index: 0x1424 */ 0x8C6B,0x6763,
	/* Array index: 0x1424 */ 0x8C79,0x787B,
	/* Array index: 0x1424 */ 0x8C7A,0x634F,
	/* Array index: 0x1424 */ 0x8C82,0x7530,
	/* Array index: 0x1424 */ 0x8C8A,0x5867,
	/* Array index: 0x1424 */ 0x8C8C,0x5949,
	/* Array index: 0x1424 */ 0x8CD1,0x7261,
	/* Array index: 0x1424 */ 0x8CD3,0x5E39,
	/* Array index: 0x1424 */ 0x8CDA,0x5672,
	/* Array index: 0x1424 */ 0x8CDC,0x5E74,
	/* Array index: 0x1424 */ 0x8CDE,0x5F5B,
	/* Array index: 0x1424 */ 0x8CEA,0x7275,
	/* Array index: 0x1424 */ 0x8CED,0x542E,
	/* Array index: 0x1424 */ 0x8CF4,0x5673,
	/* Array index: 0x1424 */ 0x8D0D,0x606B,
	/* Array index: 0x1424 */ 0x8D13,0x6D7C,
	/* Array index: 0x1424 */ 0x8D16,0x615B,
	/* Array index: 0x1424 */ 0x8D64,0x6E65,
	/* Array index: 0x1424 */ 0x8D66,0x5E75,
	/* Array index: 0x1424 */ 0x8D6B,0x7A53,
	/* Array index: 0x1424 */ 0x8D70,0x714B,
	/* Array index: 0x1424 */ 0x8D73,0x502F,
	/* Array index: 0x1424 */ 0x8D74,0x5D39,
	/* Array index: 0x1424 */ 0x8D77,0x5143,
	/* Array index: 0x1424 */ 0x8D85,0x7531,
	/* Array index: 0x1424 */ 0x8D8A,0x6A46,
	/* Array index: 0x1424 */ 0x8D99,0x7061,
	/* Array index: 0x1424 */ 0x8DA3,0x762C,
	/* Array index: 0x1424 */ 0x8DA8,0x7559,
	/* Array index: 0x1424 */ 0x8DB3,0x706B,
	/* Array index: 0x1424 */ 0x8DBA,0x5D3A,
	/* Array index: 0x1424 */ 0x8DBE,0x723F,
	/* Array index: 0x1424 */ 0x8DC6,0x7745,
	/* Array index: 0x1424 */ 0x8DCB,0x5B22,
	/* Array index: 0x1424 */ 0x8DCC,0x7276,
	/* Array index: 0x1424 */ 0x8DCF,0x4A3A,
	/* Array index: 0x1424 */ 0x8DDB,0x7775,
	/* Array index: 0x1424 */ 0x8DDD,0x4B65,
	/* Array index: 0x1424 */ 0x8DE1,0x6E66,
	/* Array index: 0x1424 */ 0x8DE3,0x6053,
	/* Array index: 0x1424 */ 0x8DE8,0x4E25,
	/* Array index: 0x1424 */ 0x8DEF,0x5658,
	/* Array index: 0x1424 */ 0x8DF3,0x542F,
	/* Array index: 0x1424 */ 0x8E0A,0x6949,
	/* Array index: 0x1424 */ 0x8E0F,0x534E,
	/* Array index: 0x1424 */ 0x8E10,0x7442,
	/* Array index: 0x1424 */ 0x8E1E,0x4B66,
	/* Array index: 0x1424 */ 0x8E2A,0x7121,
	/* Array index: 0x1424 */ 0x8E30,0x6B32,
	/* Array index: 0x1424 */ 0x8E35,0x7122,
	/* Array index: 0x1424 */ 0x8E42,0x6B33,
	/* Array index: 0x1424 */ 0x8E44,0x7034,
	/* Array index: 0x1424 */ 0x8E59,0x756C,
	/* Array index: 0x1424 */ 0x8E5F,0x6E67,
	/* Array index: 0x1424 */ 0x8E60,0x7432,
	/* Array index: 0x1424 */ 0x8E74,0x756D,
	/* Array index: 0x1424 */ 0x8E76,0x4F73,
	/* Array index: 0x1424 */ 0x8E81,0x7062,
	/* Array index: 0x1424 */ 0x8E87,0x6E4E,
	/* Array index: 0x1424 */ 0x8E8A,0x714C,
	/* Array index: 0x1424 */ 0x8E8D,0x6538,
	/* Array index: 0x1424 */ 0x8EC0,0x4F46,
	/* Array index: 0x1424 */ 0x8ED2,0x7A4D,
	/* Array index: 0x1424 */ 0x8EDF,0x6663,
	/* Array index: 0x1424 */ 0x8EEB,0x7262,
	/* Array index: 0x1424 */ 0x8EF8,0x756E,
	/* Array index: 0x1424 */ 0x8EFB,0x4A3B,
	/* Array index: 0x1424 */ 0x8EFE,0x635C,
	/* Array index: 0x1424 */ 0x8F03,0x4E72,
	/* Array index: 0x1424 */ 0x8F05,0x5659,
	/* Array index: 0x1424 */ 0x8F09,0x6E30,
	/* Array index: 0x1424 */ 0x8F2F,0x727E,
	/* Array index: 0x1424 */ 0x8F33,0x714D,
	/* Array index: 0x1424 */ 0x8F38,0x6243,
	/* Array index: 0x1424 */ 0x8F39,0x5C5E,
	/* Array index: 0x1424 */ 0x8F3B,0x5C5F,
	/* Array index: 0x1424 */ 0x8F3E,0x6F2D,
	/* Array index: 0x1424 */ 0x8F3F,0x662B,
	/* Array index: 0x1424 */ 0x8F44,0x795D,
	/* Array index: 0x1424 */ 0x8F45,0x6A3F,
	/* Array index: 0x1424 */ 0x8F49,0x6F2E,
	/* Array index: 0x1424 */ 0x8F4D,0x7450,
	/* Array index: 0x1424 */ 0x8F4E,0x4E73,
	/* Array index: 0x1424 */ 0x8F5D,0x662C,
	/* Array index: 0x1424 */ 0x8F5F,0x4E5E,
	/* Array index: 0x1424 */ 0x8F62,0x5579,
	/* Array index: 0x1424 */ 0x8F9B,0x6374,
	/* Array index: 0x1424 */ 0x8F9C,0x4D50,
	/* Array index: 0x1424 */ 0x8FA3,0x5538,
	/* Array index: 0x1424 */ 0x8FA6,0x777D,
	/* Array index: 0x1424 */ 0x8FA8,0x5C29,
	/* Array index: 0x1424 */ 0x8FC2,0x6966,
	/* Array index: 0x1424 */ 0x8FC5,0x6376,
	/* Array index: 0x1424 */ 0x8FCE,0x674A,
	/* Array index: 0x1424 */ 0x8FD1,0x504E,
	/* Array index: 0x1424 */ 0x8FD4,0x5A77,
	/* Array index: 0x1424 */ 0x8FE6,0x4A3C,
	/* Array index: 0x1424 */ 0x8FEA,0x6E68,
	/* Array index: 0x1424 */ 0x8FEB,0x5A5E,
	/* Array index: 0x1424 */ 0x8FED,0x7277,
	/* Array index: 0x1424 */ 0x8FF0,0x627B,
	/* Array index: 0x1424 */ 0x8FF2,0x4C26,
	/* Array index: 0x1424 */ 0x8FF7,0x5A3B,
	/* Array index: 0x1424 */ 0x8FF9,0x6E69,
	/* Array index: 0x1424 */ 0x8FFD,0x755A,
	/* Array index: 0x1424 */ 0x900B,0x7867,
	/* Array index: 0x1424 */ 0x902E,0x7475,
	/* Array index: 0x1424 */ 0x9031,0x714E,
	/* Array index: 0x1424 */ 0x9032,0x7264,
	/* Array index: 0x1424 */ 0x9035,0x5030,
	/* Array index: 0x1424 */ 0x9038,0x6C6F,
	/* Array index: 0x1424 */ 0x903C,0x793A,
	/* Array index: 0x1424 */ 0x903E,0x6B35,
	/* Array index: 0x1424 */ 0x9041,0x546D,
	/* Array index: 0x1424 */ 0x9042,0x6244,
	/* Array index: 0x1424 */ 0x9047,0x6967,
	/* Array index: 0x1424 */ 0x9059,0x692B,
	/* Array index: 0x1424 */ 0x9069,0x6E6A,
	/* Array index: 0x1424 */ 0x9072,0x7240,
	/* Array index: 0x1424 */ 0x9075,0x7165,
	/* Array index: 0x1424 */ 0x9077,0x7443,
	/* Array index: 0x1424 */ 0x9078,0x6054,
	/* Array index: 0x1424 */ 0x907A,0x6B36,
	/* Array index: 0x1424 */ 0x908F,0x5524,
	/* Array index: 0x1424 */ 0x9091,0x6B69,
	/* Array index: 0x1424 */ 0x9095,0x683B,
	/* Array index: 0x1424 */ 0x9099,0x5857,
	/* Array index: 0x1424 */ 0x90A2,0x7B2E,
	/* Array index: 0x1424 */ 0x90A3,0x5161,
	/* Array index: 0x1424 */ 0x90A6,0x5B40,
	/* Array index: 0x1424 */ 0x90A8,0x753E,
	/* Array index: 0x1424 */ 0x90AA,0x5E77,
	/* Array index: 0x1424 */ 0x90B5,0x6150,
	/* Array index: 0x1424 */ 0x90B8,0x6E50,
	/* Array index: 0x1424 */ 0x90C1,0x6974,
	/* Array index: 0x1424 */ 0x90CA,0x4E74,
	/* Array index: 0x1424 */ 0x90DE,0x554D,
	/* Array index: 0x1424 */ 0x90E1,0x4F5B,
	/* Array index: 0x1424 */ 0x90E8,0x5D3B,
	/* Array index: 0x1424 */ 0x90ED,0x4E2C,
	/* Array index: 0x1424 */ 0x90F5,0x6968,
	/* Array index: 0x1424 */ 0x90FD,0x5434,
	/* Array index: 0x1424 */ 0x9102,0x6447,
	/* Array index: 0x1424 */ 0x9112,0x755B,
	/* Array index: 0x1424 */ 0x9115,0x7A41,
	/* Array index: 0x1424 */ 0x9119,0x5E29,
	/* Array index: 0x1424 */ 0x9127,0x5478,
	/* Array index: 0x1424 */ 0x912D,0x6F77,
	/* Array index: 0x1424 */ 0x9132,0x5333,
	/* Array index: 0x1424 */ 0x9152,0x7150,
	/* Array index: 0x1424 */ 0x9162,0x7532,
	/* Array index: 0x1424 */ 0x9169,0x592E,
	/* Array index: 0x1424 */ 0x916A,0x552C,
	/* Array index: 0x1424 */ 0x916C,0x6246,
	/* Array index: 0x1424 */ 0x9175,0x7D23,
	/* Array index: 0x1424 */ 0x9177,0x7B65,
	/* Array index: 0x1424 */ 0x9178,0x5F2B,
	/* Array index: 0x1424 */ 0x9187,0x6275,
	/* Array index: 0x1424 */ 0x9189,0x762D,
	/* Array index: 0x1424 */ 0x918B,0x7533,
	/* Array index: 0x1424 */ 0x918D,0x7035,
	/* Array index: 0x1424 */ 0x9192,0x6125,
	/* Array index: 0x1424 */ 0x919C,0x755D,
	/* Array index: 0x1424 */ 0x91C0,0x6554,
	/* Array index: 0x1424 */ 0x91C7,0x737A,
	/* Array index: 0x1424 */ 0x91D7,0x6172,
	/* Array index: 0x1424 */ 0x91D8,0x6F79,
	/* Array index: 0x1424 */ 0x91DC,0x5D3C,
	/* Array index: 0x1424 */ 0x91DD,0x765C,
	/* Array index: 0x1424 */ 0x91E3,0x7065,
	/* Array index: 0x1424 */ 0x91E7,0x7444,
	/* Array index: 0x1424 */ 0x91EA,0x6969,
	/* Array index: 0x1424 */ 0x91F5,0x737B,
	/* Array index: 0x1424 */ 0x920D,0x546F,
	/* Array index: 0x1424 */ 0x9217,0x6B4D,
	/* Array index: 0x1424 */ 0x921E,0x5037,
	/* Array index: 0x1424 */ 0x9234,0x5642,
	/* Array index: 0x1424 */ 0x923A,0x682D,
	/* Array index: 0x1424 */ 0x923F,0x6F2F,
	/* Array index: 0x1424 */ 0x9240,0x4B25,
	/* Array index: 0x1424 */ 0x9245,0x4B69,
	/* Array index: 0x1424 */ 0x9249,0x7A68,
	/* Array index: 0x1424 */ 0x9257,0x4C46,
	/* Array index: 0x1424 */ 0x925B,0x6667,
	/* Array index: 0x1424 */ 0x925E,0x6A47,
	/* Array index: 0x1424 */ 0x9280,0x6B5E,
	/* Array index: 0x1424 */ 0x9283,0x7548,
	/* Array index: 0x1424 */ 0x9285,0x545E,
	/* Array index: 0x1424 */ 0x9291,0x6055,
	/* Array index: 0x1424 */ 0x9293,0x6F30,
	/* Array index: 0x1424 */ 0x9296,0x6247,
	/* Array index: 0x1424 */ 0x9298,0x592F,
	/* Array index: 0x1424 */ 0x929C,0x7967,
	/* Array index: 0x1424 */ 0x92B3,0x6765,
	/* Array index: 0x1424 */ 0x92B6,0x4F4A,
	/* Array index: 0x1424 */ 0x92B7,0x6151,
	/* Array index: 0x1424 */ 0x92B9,0x6248,
	/* Array index: 0x1424 */ 0x92CC,0x6F7B,
	/* Array index: 0x1424 */ 0x92CF,0x7A79,
	/* Array index: 0x1424 */ 0x92D2,0x5C72,
	/* Array index: 0x1424 */ 0x92E4,0x6027,
	/* Array index: 0x1424 */ 0x92EA,0x7868,
	/* Array index: 0x1424 */ 0x92F8,0x4B6A,
	/* Array index: 0x1424 */ 0x92FC,0x4B3C,
	/* Array index: 0x1424 */ 0x9304,0x5662,
	/* Array index: 0x1424 */ 0x9310,0x755E,
	/* Array index: 0x1424 */ 0x9318,0x755F,
	/* Array index: 0x1424 */ 0x931A,0x6E36,
	/* Array index: 0x1424 */ 0x932B,0x6038,
	/* Array index: 0x1424 */ 0x932E,0x4D51,
	/* Array index: 0x1424 */ 0x932F,0x7339,
	/* Array index: 0x1424 */ 0x9348,0x674C,
	/* Array index: 0x1424 */ 0x934A,0x5628,
	/* Array index: 0x1424 */ 0x934B,0x4E27,
	/* Array index: 0x1424 */ 0x934D,0x5435,
	/* Array index: 0x1424 */ 0x9354,0x6448,
	/* Array index: 0x1424 */ 0x935B,0x5334,
	/* Array index: 0x1424 */ 0x936E,0x6B39,
	/* Array index: 0x1424 */ 0x9375,0x4B75,
	/* Array index: 0x1424 */ 0x937C,0x765D,
	/* Array index: 0x1424 */ 0x937E,0x7123,
	/* Array index: 0x1424 */ 0x938C,0x4C47,
	/* Array index: 0x1424 */ 0x9394,0x694A,
	/* Array index: 0x1424 */ 0x9396,0x6170,
	/* Array index: 0x1424 */ 0x939A,0x7560,
	/* Array index: 0x1424 */ 0x93A3,0x7B2F,
	/* Array index: 0x1424 */ 0x93A7,0x4B51,
	/* Array index: 0x1424 */ 0x93AC,0x7B60,
	/* Array index: 0x1424 */ 0x93AD,0x7265,
	/* Array index: 0x1424 */ 0x93B0,0x6C70,
	/* Array index: 0x1424 */ 0x93C3,0x706C,
	/* Array index: 0x1424 */ 0x93D1,0x6E6B,
	/* Array index: 0x1424 */ 0x93DE,0x694B,
	/* Array index: 0x1424 */ 0x93E1,0x4C70,
	/* Array index: 0x1424 */ 0x93E4,0x572F,
	/* Array index: 0x1424 */ 0x93F6,0x7321,
	/* Array index: 0x1424 */ 0x9404,0x7C75,
	/* Array index: 0x1424 */ 0x9418,0x7124,
	/* Array index: 0x1424 */ 0x9425,0x6056,
	/* Array index: 0x1424 */ 0x942B,0x6F32,
	/* Array index: 0x1424 */ 0x9435,0x7451,
	/* Array index: 0x1424 */ 0x9438,0x7721,
	/* Array index: 0x1424 */ 0x9444,0x7151,
	/* Array index: 0x1424 */ 0x9451,0x4A7C,
	/* Array index: 0x1424 */ 0x9452,0x4A7D,
	/* Array index: 0x1424 */ 0x945B,0x4E4E,
	/* Array index: 0x1424 */ 0x947D,0x7348,
	/* Array index: 0x1424 */ 0x947F,0x733A,
	/* Array index: 0x1424 */ 0x9577,0x6D7E,
	/* Array index: 0x1424 */ 0x9580,0x5A26,
	/* Array index: 0x1424 */ 0x9583,0x606C,
	/* Array index: 0x1424 */ 0x9589,0x784D,
	/* Array index: 0x1424 */ 0x958B,0x4B52,
	/* Array index: 0x1424 */ 0x9598,0x4B26,
	/* Array index: 0x1424 */ 0x95A8,0x5031,
	/* Array index: 0x1424 */ 0x95AD,0x556F,
	/* Array index: 0x1424 */ 0x95B1,0x6673,
	/* Array index: 0x1424 */ 0x95BB,0x6722,
	/* Array index: 0x1424 */ 0x95BC,0x6459,
	/* Array index: 0x1424 */ 0x95C7,0x6461,
	/* Array index: 0x1424 */ 0x95CA,0x7C44,
	/* Array index: 0x1424 */ 0x95DC,0x4E3C,
	/* Array index: 0x1424 */ 0x95E1,0x7445,
	/* Array index: 0x1424 */ 0x95E2,0x5C23,
	/* Array index: 0x1424 */ 0x961C,0x5D3D,
	/* Array index: 0x1424 */ 0x9621,0x7446,
	/* Array index: 0x1424 */ 0x962A,0x7821,
	/* Array index: 0x1424 */ 0x962E,0x6856,
	/* Array index: 0x1424 */ 0x9632,0x5B41,
	/* Array index: 0x1424 */ 0x963B,0x7066,
	/* Array index: 0x1424 */ 0x963F,0x6439,
	/* Array index: 0x1424 */ 0x9640,0x766D,
	/* Array index: 0x1424 */ 0x9642,0x792E,
	/* Array index: 0x1424 */ 0x9644,0x5D3E,
	/* Array index: 0x1424 */ 0x9650,0x795A,
	/* Array index: 0x1424 */ 0x966A,0x5B56,
	/* Array index: 0x1424 */ 0x9670,0x6B64,
	/* Array index: 0x1424 */ 0x967D,0x6555,
	/* Array index: 0x1424 */ 0x9685,0x696A,
	/* Array index: 0x1424 */ 0x9686,0x574C,
	/* Array index: 0x1424 */ 0x9694,0x4C30,
	/* Array index: 0x1424 */ 0x9695,0x6A22,
	/* Array index: 0x1424 */ 0x96A3,0x5776,
	/* Array index: 0x1424 */ 0x96A7,0x624A,
	/* Array index: 0x1424 */ 0x96A8,0x624B,
	/* Array index: 0x1424 */ 0x96AA,0x7A4F,
	/* Array index: 0x1424 */ 0x96B1,0x6B5F,
	/* Array index: 0x1424 */ 0x96B7,0x564B,
	/* Array index: 0x1424 */ 0x96BB,0x7434,
	/* Array index: 0x1424 */ 0x96D5,0x7067,
	/* Array index: 0x1424 */ 0x96D6,0x624C,
	/* Array index: 0x1424 */ 0x96D9,0x642A,
	/* Array index: 0x1424 */ 0x96DB,0x7561,
	/* Array index: 0x1424 */ 0x96DC,0x6D5A,
	/* Array index: 0x1424 */ 0x96E2,0x576E,
	/* Array index: 0x1424 */ 0x96E3,0x5171,
	/* Array index: 0x1424 */ 0x96EF,0x5A27,
	/* Array index: 0x1424 */ 0x96F0,0x5D54,
	/* Array index: 0x1424 */ 0x96F2,0x6A23,
	/* Array index: 0x1424 */ 0x96F6,0x5643,
	/* Array index: 0x1424 */ 0x96F7,0x5674,
	/* Array index: 0x1424 */ 0x96F9,0x5A5F,
	/* Array index: 0x1424 */ 0x96FB,0x6F33,
	/* Array index: 0x1424 */ 0x9700,0x624D,
	/* Array index: 0x1424 */ 0x9706,0x6F7D,
	/* Array index: 0x1424 */ 0x9707,0x7268,
	/* Array index: 0x1424 */ 0x9711,0x6F45,
	/* Array index: 0x1424 */ 0x9713,0x6767,
	/* Array index: 0x1424 */ 0x9716,0x577D,
	/* Array index: 0x1424 */ 0x9719,0x674E,
	/* Array index: 0x1424 */ 0x971C,0x5F5C,
	/* Array index: 0x1424 */ 0x971E,0x7947,
	/* Array index: 0x1424 */ 0x9727,0x5976,
	/* Array index: 0x1424 */ 0x9730,0x5F2C,
	/* Array index: 0x1424 */ 0x9732,0x565A,
	/* Array index: 0x1424 */ 0x9739,0x5C24,
	/* Array index: 0x1424 */ 0x973D,0x7038,
	/* Array index: 0x1424 */ 0x9742,0x557A,
	/* Array index: 0x1424 */ 0x9744,0x6477,
	/* Array index: 0x1424 */ 0x9748,0x5644,
	/* Array index: 0x1424 */ 0x9751,0x746C,
	/* Array index: 0x1424 */ 0x9756,0x6F7E,
	/* Array index: 0x1424 */ 0x975C,0x7021,
	/* Array index: 0x1424 */ 0x975E,0x5E2A,
	/* Array index: 0x1424 */ 0x9761,0x5A3C,
	/* Array index: 0x1424 */ 0x9762,0x587C,
	/* Array index: 0x1424 */ 0x9769,0x7A54,
	/* Array index: 0x1424 */ 0x976D,0x6C65,
	/* Array index: 0x1424 */ 0x9774,0x7C28,
	/* Array index: 0x1424 */ 0x9777,0x6C66,
	/* Array index: 0x1424 */ 0x977A,0x584B,
	/* Array index: 0x1424 */ 0x978B,0x7B39,
	/* Array index: 0x1424 */ 0x978D,0x6453,
	/* Array index: 0x1424 */ 0x978F,0x4D79,
	/* Array index: 0x1424 */ 0x97A0,0x4F53,
	/* Array index: 0x1424 */ 0x97A8,0x4A6A,
	/* Array index: 0x1424 */ 0x97AB,0x4F54,
	/* Array index: 0x1424 */ 0x97AD,0x783D,
	/* Array index: 0x1424 */ 0x97C6,0x7447,
	/* Array index: 0x1424 */ 0x97CB,0x6A5F,
	/* Array index: 0x1424 */ 0x97D3,0x795B,
	/* Array index: 0x1424 */ 0x97DC,0x5437,
	/* Array index: 0x1424 */ 0x97F3,0x6B65,
	/* Array index: 0x1424 */ 0x97F6,0x6152,
	/* Array index: 0x1424 */ 0x97FB,0x6A24,
	/* Array index: 0x1424 */ 0x9817,0x7776,
	/* Array index: 0x1424 */ 0x9818,0x5645,
	/* Array index: 0x1424 */ 0x982D,0x5469,
	/* Array index: 0x1424 */ 0x9830,0x7A7A,
	/* Array index: 0x1424 */ 0x9838,0x4C72,
	/* Array index: 0x1424 */ 0x9839,0x775D,
	/* Array index: 0x1424 */ 0x983B,0x5E3A,
	/* Array index: 0x1424 */ 0x9846,0x4E28,
	/* Array index: 0x1424 */ 0x9854,0x6454,
	/* Array index: 0x1424 */ 0x9858,0x6A43,
	/* Array index: 0x1424 */ 0x985A,0x6F34,
	/* Array index: 0x1424 */ 0x985E,0x573E,
	/* Array index: 0x1424 */ 0x9865,0x7B62,
	/* Array index: 0x1424 */ 0x9867,0x4D53,
	/* Array index: 0x1424 */ 0x986B,0x6F35,
	/* Array index: 0x1424 */ 0x986F,0x7A69,
	/* Array index: 0x1424 */ 0x98A8,0x7926,
	/* Array index: 0x1424 */ 0x98AF,0x5F3D,
	/* Array index: 0x1424 */ 0x98B1,0x7747,
	/* Array index: 0x1424 */ 0x98C4,0x787D,
	/* Array index: 0x1424 */ 0x98C7,0x787C,
	/* Array index: 0x1424 */ 0x98DB,0x5E2B,
	/* Array index: 0x1424 */ 0x98DC,0x5B68,
	/* Array index: 0x1424 */ 0x98DF,0x635D,
	/* Array index: 0x1424 */ 0x98E1,0x6162,
	/* Array index: 0x1424 */ 0x98E2,0x5146,
	/* Array index: 0x1424 */ 0x98F4,0x6C47,
	/* Array index: 0x1424 */ 0x9903,0x4E75,
	/* Array index: 0x1424 */ 0x9909,0x7A43,
	/* Array index: 0x1424 */ 0x990A,0x6557,
	/* Array index: 0x1424 */ 0x990C,0x6C48,
	/* Array index: 0x1424 */ 0x9910,0x7349,
	/* Array index: 0x1424 */ 0x9913,0x643B,
	/* Array index: 0x1424 */ 0x9918,0x662E,
	/* Array index: 0x1424 */ 0x991E,0x6F36,
	/* Array index: 0x1424 */ 0x9920,0x5C3F,
	/* Array index: 0x1424 */ 0x9928,0x4E3D,
	/* Array index: 0x1424 */ 0x9945,0x5843,
	/* Array index: 0x1424 */ 0x9951,0x5147,
	/* Array index: 0x1424 */ 0x9952,0x692E,
	/* Array index: 0x1424 */ 0x9954,0x683D,
	/* Array index: 0x1424 */ 0x9957,0x7A44,
	/* Array index: 0x1424 */ 0x9996,0x624F,
	/* Array index: 0x1424 */ 0x9999,0x7A45,
	/* Array index: 0x1424 */ 0x999D,0x7938,
	/* Array index: 0x1424 */ 0x99A5,0x5C60,
	/* Array index: 0x1424 */ 0x99A8,0x7B30,
	/* Array index: 0x1424 */ 0x99B1,0x766E,
	/* Array index: 0x1424 */ 0x99B3,0x764C,
	/* Array index: 0x1424 */ 0x99B4,0x6278,
	/* Array index: 0x1424 */ 0x99B9,0x6C71,
	/* Array index: 0x1424 */ 0x99C1,0x5A60,
	/* Array index: 0x1424 */ 0x99D5,0x4A3D,
	/* Array index: 0x1424 */ 0x99D9,0x5D3F,
	/* Array index: 0x1424 */ 0x99DD,0x766F,
	/* Array index: 0x1424 */ 0x99DF,0x5E79,
	/* Array index: 0x1424 */ 0x99ED,0x7A34,
	/* Array index: 0x1424 */ 0x99F1,0x552D,
	/* Array index: 0x1424 */ 0x99FF,0x7167,
	/* Array index: 0x1424 */ 0x9A01,0x5E3E,
	/* Array index: 0x1424 */ 0x9A08,0x5C40,
	/* Array index: 0x1424 */ 0x9A0E,0x5148,
	/* Array index: 0x1424 */ 0x9A0F,0x5149,
	/* Array index: 0x1424 */ 0x9A19,0x783E,
	/* Array index: 0x1424 */ 0x9A2B,0x4B76,
	/* Array index: 0x1424 */ 0x9A30,0x5479,
	/* Array index: 0x1424 */ 0x9A36,0x7562,
	/* Array index: 0x1424 */ 0x9A37,0x6153,
	/* Array index: 0x1424 */ 0x9A40,0x5869,
	/* Array index: 0x1424 */ 0x9A43,0x787E,
	/* Array index: 0x1424 */ 0x9A45,0x4F4C,
	/* Array index: 0x1424 */ 0x9A4D,0x7D24,
	/* Array index: 0x1424 */ 0x9A55,0x4E76,
	/* Array index: 0x1424 */ 0x9A57,0x7A50,
	/* Array index: 0x1424 */ 0x9A5A,0x4C73,
	/* Array index: 0x1424 */ 0x9A5B,0x663E,
	/* Array index: 0x1424 */ 0x9A5F,0x762E,
	/* Array index: 0x1424 */ 0x9A62,0x5570,
	/* Array index: 0x1424 */ 0x9A65,0x514A,
	/* Array index: 0x1424 */ 0x9A69,0x7C3E,
	/* Array index: 0x1424 */ 0x9A6A,0x5571,
	/* Array index: 0x1424 */ 0x9AA8,0x4D69,
	/* Array index: 0x1424 */ 0x9AB8,0x7A35,
	/* Array index: 0x1424 */ 0x9AD3,0x6250,
	/* Array index: 0x1424 */ 0x9AD4,0x7477,
	/* Array index: 0x1424 */ 0x9AD8,0x4D54,
	/* Array index: 0x1424 */ 0x9AE5,0x6723,
	/* Array index: 0x1424 */ 0x9AEE,0x5B25,
	/* Array index: 0x1424 */ 0x9B1A,0x6251,
	/* Array index: 0x1424 */ 0x9B27,0x5722,
	/* Array index: 0x1424 */ 0x9B2A,0x7763,
	/* Array index: 0x1424 */ 0x9B31,0x6A26,
	/* Array index: 0x1424 */ 0x9B3C,0x5021,
	/* Array index: 0x1424 */ 0x9B4F,0x6A60,
	/* Array index: 0x1424 */ 0x9B54,0x582A,
	/* Array index: 0x1424 */ 0x9B5A,0x6560,
	/* Array index: 0x1424 */ 0x9B6F,0x565B,
	/* Array index: 0x1424 */ 0x9B8E,0x6F46,
	/* Array index: 0x1424 */ 0x9B91,0x786A,
	/* Array index: 0x1424 */ 0x9B9F,0x6455,
	/* Array index: 0x1424 */ 0x9BAB,0x4E77,
	/* Array index: 0x1424 */ 0x9BAE,0x6058,
	/* Array index: 0x1424 */ 0x9BC9,0x576F,
	/* Array index: 0x1424 */ 0x9BD6,0x746D,
	/* Array index: 0x1424 */ 0x9BE4,0x4D66,
	/* Array index: 0x1424 */ 0x9BE8,0x4C74,
	/* Array index: 0x1424 */ 0x9C0D,0x7563,
	/* Array index: 0x1424 */ 0x9C10,0x644A,
	/* Array index: 0x1424 */ 0x9C12,0x5C61,
	/* Array index: 0x1424 */ 0x9C15,0x7948,
	/* Array index: 0x1424 */ 0x9C25,0x7C3F,
	/* Array index: 0x1424 */ 0x9C32,0x6827,
	/* Array index: 0x1424 */ 0x9C3B,0x5844,
	/* Array index: 0x1424 */ 0x9C47,0x4B3E,
	/* Array index: 0x1424 */ 0x9C49,0x5C2E,
	/* Array index: 0x1424 */ 0x9C57,0x5777,
	/* Array index: 0x1424 */ 0x9CE5,0x7068,
	/* Array index: 0x1424 */ 0x9CE7,0x5D40,
	/* Array index: 0x1424 */ 0x9CE9,0x4F4D,
	/* Array index: 0x1424 */ 0x9CF3,0x5C73,
	/* Array index: 0x1424 */ 0x9CF4,0x5930,
	/* Array index: 0x1424 */ 0x9CF6,0x6669,
	/* Array index: 0x1424 */ 0x9D09,0x643C,
	/* Array index: 0x1424 */ 0x9D1B,0x6A44,
	/* Array index: 0x1424 */ 0x9D26,0x646C,
	/* Array index: 0x1424 */ 0x9D28,0x6465,
	/* Array index: 0x1424 */ 0x9D3B,0x7B78,
	/* Array index: 0x1424 */ 0x9D51,0x4C3B,
	/* Array index: 0x1424 */ 0x9D5D,0x643D,
	/* Array index: 0x1424 */ 0x9D60,0x4D5C,
	/* Array index: 0x1424 */ 0x9D61,0x5977,
	/* Array index: 0x1424 */ 0x9D6C,0x5D5F,
	/* Array index: 0x1424 */ 0x9D72,0x6D4E,
	/* Array index: 0x1424 */ 0x9DA9,0x5950,
	/* Array index: 0x1424 */ 0x9DAF,0x6523,
	/* Array index: 0x1424 */ 0x9DB4,0x794D,
	/* Array index: 0x1424 */ 0x9DC4,0x4D2E,
	/* Array index: 0x1424 */ 0x9DD7,0x4F4E,
	/* Array index: 0x1424 */ 0x9DF2,0x762F,
	/* Array index: 0x1424 */ 0x9E1A,0x6524,
	/* Array index: 0x1424 */ 0x9E1E,0x5536,
	/* Array index: 0x1424 */ 0x9E75,0x565D,
	/* Array index: 0x1424 */ 0x9E79,0x7969,
	/* Array index: 0x1424 */ 0x9E7D,0x6724,
	/* Array index: 0x1424 */ 0x9E7F,0x5663,
	/* Array index: 0x1424 */ 0x9E92,0x514B,
	/* Array index: 0x1424 */ 0x9E93,0x5664,
	/* Array index: 0x1424 */ 0x9E97,0x5572,
	/* Array index: 0x1424 */ 0x9E9D,0x5E7A,
	/* Array index: 0x1424 */ 0x9E9F,0x5778,
	/* Array index: 0x1424 */ 0x9EA5,0x586A,
	/* Array index: 0x1424 */ 0x9EB4,0x4F55,
	/* Array index: 0x1424 */ 0x9EB5,0x587D,
	/* Array index: 0x1424 */ 0x9EBB,0x582B,
	/* Array index: 0x1424 */ 0x9EBE,0x7D4B,
	/* Array index: 0x1424 */ 0x9EC3,0x7C5C,
	/* Array index: 0x1424 */ 0x9ECD,0x6028,
	/* Array index: 0x1424 */ 0x9ECE,0x5573,
	/* Array index: 0x1424 */ 0x9ED1,0x7D59,
	/* Array index: 0x1424 */ 0x9ED4,0x4C23,
	/* Array index: 0x1424 */ 0x9ED8,0x5979,
	/* Array index: 0x1424 */ 0x9EDB,0x536A,
	/* Array index: 0x1424 */ 0x9EDC,0x7575,
	/* Array index: 0x1424 */ 0x9EDE,0x6F47,
	/* Array index: 0x1424 */ 0x9EE8,0x535A,
	/* Array index: 0x1424 */ 0x9EF4,0x5A3D,
	/* Array index: 0x1424 */ 0x9F07,0x6828,
	/* Array index: 0x1424 */ 0x9F08,0x5C2F,
	/* Array index: 0x1424 */ 0x9F0E,0x7023,
	/* Array index: 0x1424 */ 0x9F13,0x4D55,
	/* Array index: 0x1424 */ 0x9F20,0x6029,
	/* Array index: 0x1424 */ 0x9F3B,0x5E2C,
	/* Array index: 0x1424 */ 0x9F4A,0x703A,
	/* Array index: 0x1424 */ 0x9F4B,0x6E31,
	/* Array index: 0x1424 */ 0x9F4E,0x6E32,
	/* Array index: 0x1424 */ 0x9F52,0x764D,
	/* Array index: 0x1424 */ 0x9F5F,0x6E52,
	/* Array index: 0x1424 */ 0x9F61,0x5646,
	/* Array index: 0x1424 */ 0x9F67,0x6065,
	/* Array index: 0x1424 */ 0x9F6A,0x733B,
	/* Array index: 0x1424 */ 0x9F6C,0x6561,
	/* Array index: 0x1424 */ 0x9F77,0x644B,
	/* Array index: 0x1424 */ 0x9F8D,0x5723,
	/* Array index: 0x1424 */ 0x9F90,0x5B42,
	/* Array index: 0x1424 */ 0x9F95,0x4A7E,
	/* Array index: 0x1424 */ 0x9F9C,0x4F4F,
	/* Array index: 0x1424 */ 0xAC00,0x3021,
	/* Array index: 0x1424 */ 0xAC01,0x3022,
	/* Array index: 0x1424 */ 0xAC04,0x3023,
	/* Array index: 0x1424 */ 0xAC20,0x3035,
	/* Array index: 0x1424 */ 0xAC24,0x3036,
	/* Array index: 0x1424 */ 0xAC38,0x303C,
	/* Array index: 0x1424 */ 0xAC39,0x303D,
	/* Array index: 0x1424 */ 0xAC3C,0x303E,
	/* Array index: 0x1424 */ 0xAC40,0x303F,
	/* Array index: 0x1424 */ 0xAC4B,0x3040,
	/* Array index: 0x1424 */ 0xAC4D,0x3041,
	/* Array index: 0x1424 */ 0xAC54,0x3042,
	/* Array index: 0x1424 */ 0xAC58,0x3043,
	/* Array index: 0x1424 */ 0xAC5C,0x3044,
	/* Array index: 0x1424 */ 0xAC70,0x3045,
	/* Array index: 0x1424 */ 0xAC71,0x3046,
	/* Array index: 0x1424 */ 0xAC74,0x3047,
	/* Array index: 0x1424 */ 0xAC77,0x3048,
	/* Array index: 0x1424 */ 0xAC78,0x3049,
	/* Array index: 0x1424 */ 0xAC7A,0x304A,
	/* Array index: 0x1424 */ 0xAC90,0x3055,
	/* Array index: 0x1424 */ 0xAC94,0x3056,
	/* Array index: 0x1424 */ 0xACC1,0x3067,
	/* Array index: 0x1424 */ 0xACC4,0x3068,
	/* Array index: 0x1424 */ 0xACC8,0x3069,
	/* Array index: 0x1424 */ 0xACCC,0x306A,
	/* Array index: 0x1424 */ 0xACD5,0x306B,
	/* Array index: 0x1424 */ 0xACD7,0x306C,
	/* Array index: 0x1424 */ 0xACE0,0x306D,
	/* Array index: 0x1424 */ 0xACE1,0x306E,
	/* Array index: 0x1424 */ 0xACE4,0x306F,
	/* Array index: 0x1424 */ 0xACE7,0x3070,
	/* Array index: 0x1424 */ 0xACE8,0x3071,
	/* Array index: 0x1424 */ 0xACEA,0x3072,
	/* Array index: 0x1424 */ 0xACEC,0x3073,
	/* Array index: 0x1424 */ 0xACFC,0x307A,
	/* Array index: 0x1424 */ 0xACFD,0x307B,
	/* Array index: 0x1424 */ 0xAD00,0x307C,
	/* Array index: 0x1424 */ 0xAD04,0x307D,
	/* Array index: 0x1424 */ 0xAD06,0x307E,
	/* Array index: 0x1424 */ 0xAD0C,0x3121,
	/* Array index: 0x1424 */ 0xAD0D,0x3122,
	/* Array index: 0x1424 */ 0xAD0F,0x3123,
	/* Array index: 0x1424 */ 0xAD11,0x3124,
	/* Array index: 0x1424 */ 0xAD18,0x3125,
	/* Array index: 0x1424 */ 0xAD1C,0x3126,
	/* Array index: 0x1424 */ 0xAD20,0x3127,
	/* Array index: 0x1424 */ 0xAD29,0x3128,
	/* Array index: 0x1424 */ 0xAD2C,0x3129,
	/* Array index: 0x1424 */ 0xAD2D,0x312A,
	/* Array index: 0x1424 */ 0xAD34,0x312B,
	/* Array index: 0x1424 */ 0xAD35,0x312C,
	/* Array index: 0x1424 */ 0xAD38,0x312D,
	/* Array index: 0x1424 */ 0xAD3C,0x312E,
	/* Array index: 0x1424 */ 0xAD44,0x312F,
	/* Array index: 0x1424 */ 0xAD45,0x3130,
	/* Array index: 0x1424 */ 0xAD47,0x3131,
	/* Array index: 0x1424 */ 0xAD49,0x3132,
	/* Array index: 0x1424 */ 0xAD50,0x3133,
	/* Array index: 0x1424 */ 0xAD54,0x3134,
	/* Array index: 0x1424 */ 0xAD58,0x3135,
	/* Array index: 0x1424 */ 0xAD61,0x3136,
	/* Array index: 0x1424 */ 0xAD63,0x3137,
	/* Array index: 0x1424 */ 0xAD6C,0x3138,
	/* Array index: 0x1424 */ 0xAD6D,0x3139,
	/* Array index: 0x1424 */ 0xAD70,0x313A,
	/* Array index: 0x1424 */ 0xAD88,0x3145,
	/* Array index: 0x1424 */ 0xAD89,0x3146,
	/* Array index: 0x1424 */ 0xAD8C,0x3147,
	/* Array index: 0x1424 */ 0xAD90,0x3148,
	/* Array index: 0x1424 */ 0xAD9C,0x3149,
	/* Array index: 0x1424 */ 0xAD9D,0x314A,
	/* Array index: 0x1424 */ 0xADA4,0x314B,
	/* Array index: 0x1424 */ 0xADB7,0x314C,
	/* Array index: 0x1424 */ 0xADC0,0x314D,
	/* Array index: 0x1424 */ 0xADC1,0x314E,
	/* Array index: 0x1424 */ 0xADC4,0x314F,
	/* Array index: 0x1424 */ 0xADC8,0x3150,
	/* Array index: 0x1424 */ 0xADD0,0x3151,
	/* Array index: 0x1424 */ 0xADD1,0x3152,
	/* Array index: 0x1424 */ 0xADD3,0x3153,
	/* Array index: 0x1424 */ 0xADDC,0x3154,
	/* Array index: 0x1424 */ 0xADE0,0x3155,
	/* Array index: 0x1424 */ 0xADE4,0x3156,
	/* Array index: 0x1424 */ 0xADF8,0x3157,
	/* Array index: 0x1424 */ 0xADF9,0x3158,
	/* Array index: 0x1424 */ 0xADFC,0x3159,
	/* Array index: 0x1424 */ 0xAE08,0x315D,
	/* Array index: 0x1424 */ 0xAE09,0x315E,
	/* Array index: 0x1424 */ 0xAE0B,0x315F,
	/* Array index: 0x1424 */ 0xAE0D,0x3160,
	/* Array index: 0x1424 */ 0xAE14,0x3161,
	/* Array index: 0x1424 */ 0xAE30,0x3162,
	/* Array index: 0x1424 */ 0xAE31,0x3163,
	/* Array index: 0x1424 */ 0xAE34,0x3164,
	/* Array index: 0x1424 */ 0xAE37,0x3165,
	/* Array index: 0x1424 */ 0xAE38,0x3166,
	/* Array index: 0x1424 */ 0xAE3A,0x3167,
	/* Array index: 0x1424 */ 0xAE40,0x3168,
	/* Array index: 0x1424 */ 0xAE41,0x3169,
	/* Array index: 0x1424 */ 0xAE43,0x316A,
	/* Array index: 0x1424 */ 0xAE45,0x316B,
	/* Array index: 0x1424 */ 0xAE46,0x316C,
	/* Array index: 0x1424 */ 0xAE54,0x3172,
	/* Array index: 0x1424 */ 0xAE56,0x3173,
	/* Array index: 0x1424 */ 0xAE65,0x3179,
	/* Array index: 0x1424 */ 0xAE68,0x317A,
	/* Array index: 0x1424 */ 0xAE69,0x317B,
	/* Array index: 0x1424 */ 0xAE6C,0x317C,
	/* Array index: 0x1424 */ 0xAE70,0x317D,
	/* Array index: 0x1424 */ 0xAE84,0x3225,
	/* Array index: 0x1424 */ 0xAE85,0x3226,
	/* Array index: 0x1424 */ 0xAE8C,0x3227,
	/* Array index: 0x1424 */ 0xAEC4,0x322C,
	/* Array index: 0x1424 */ 0xAED8,0x3232,
	/* Array index: 0x1424 */ 0xAED9,0x3233,
	/* Array index: 0x1424 */ 0xAEDC,0x3234,
	/* Array index: 0x1424 */ 0xAEE8,0x3235,
	/* Array index: 0x1424 */ 0xAEEB,0x3236,
	/* Array index: 0x1424 */ 0xAEED,0x3237,
	/* Array index: 0x1424 */ 0xAEF4,0x3238,
	/* Array index: 0x1424 */ 0xAEF8,0x3239,
	/* Array index: 0x1424 */ 0xAEFC,0x323A,
	/* Array index: 0x1424 */ 0xAF07,0x323B,
	/* Array index: 0x1424 */ 0xAF08,0x323C,
	/* Array index: 0x1424 */ 0xAF0D,0x323D,
	/* Array index: 0x1424 */ 0xAF10,0x323E,
	/* Array index: 0x1424 */ 0xAF2C,0x323F,
	/* Array index: 0x1424 */ 0xAF2D,0x3240,
	/* Array index: 0x1424 */ 0xAF30,0x3241,
	/* Array index: 0x1424 */ 0xAF32,0x3242,
	/* Array index: 0x1424 */ 0xAF34,0x3243,
	/* Array index: 0x1424 */ 0xAF3C,0x3244,
	/* Array index: 0x1424 */ 0xAF3D,0x3245,
	/* Array index: 0x1424 */ 0xAF48,0x324A,
	/* Array index: 0x1424 */ 0xAF49,0x324B,
	/* Array index: 0x1424 */ 0xAF50,0x324C,
	/* Array index: 0x1424 */ 0xAF5C,0x324D,
	/* Array index: 0x1424 */ 0xAF5D,0x324E,
	/* Array index: 0x1424 */ 0xAF64,0x324F,
	/* Array index: 0x1424 */ 0xAF65,0x3250,
	/* Array index: 0x1424 */ 0xAF79,0x3251,
	/* Array index: 0x1424 */ 0xAF80,0x3252,
	/* Array index: 0x1424 */ 0xAF84,0x3253,
	/* Array index: 0x1424 */ 0xAF88,0x3254,
	/* Array index: 0x1424 */ 0xAF90,0x3255,
	/* Array index: 0x1424 */ 0xAF91,0x3256,
	/* Array index: 0x1424 */ 0xAF95,0x3257,
	/* Array index: 0x1424 */ 0xAF9C,0x3258,
	/* Array index: 0x1424 */ 0xAFB8,0x3259,
	/* Array index: 0x1424 */ 0xAFB9,0x325A,
	/* Array index: 0x1424 */ 0xAFBC,0x325B,
	/* Array index: 0x1424 */ 0xAFC0,0x325C,
	/* Array index: 0x1424 */ 0xAFD4,0x3263,
	/* Array index: 0x1424 */ 0xAFDC,0x3264,
	/* Array index: 0x1424 */ 0xAFE8,0x3265,
	/* Array index: 0x1424 */ 0xAFE9,0x3266,
	/* Array index: 0x1424 */ 0xAFF0,0x3267,
	/* Array index: 0x1424 */ 0xAFF1,0x3268,
	/* Array index: 0x1424 */ 0xAFF4,0x3269,
	/* Array index: 0x1424 */ 0xAFF8,0x326A,
	/* Array index: 0x1424 */ 0xB000,0x326B,
	/* Array index: 0x1424 */ 0xB001,0x326C,
	/* Array index: 0x1424 */ 0xB004,0x326D,
	/* Array index: 0x1424 */ 0xB00C,0x326E,
	/* Array index: 0x1424 */ 0xB010,0x326F,
	/* Array index: 0x1424 */ 0xB014,0x3270,
	/* Array index: 0x1424 */ 0xB01C,0x3271,
	/* Array index: 0x1424 */ 0xB01D,0x3272,
	/* Array index: 0x1424 */ 0xB028,0x3273,
	/* Array index: 0x1424 */ 0xB044,0x3274,
	/* Array index: 0x1424 */ 0xB045,0x3275,
	/* Array index: 0x1424 */ 0xB048,0x3276,
	/* Array index: 0x1424 */ 0xB04A,0x3277,
	/* Array index: 0x1424 */ 0xB04C,0x3278,
	/* Array index: 0x1424 */ 0xB04E,0x3279,
	/* Array index: 0x1424 */ 0xB05D,0x3321,
	/* Array index: 0x1424 */ 0xB07C,0x3322,
	/* Array index: 0x1424 */ 0xB07D,0x3323,
	/* Array index: 0x1424 */ 0xB080,0x3324,
	/* Array index: 0x1424 */ 0xB084,0x3325,
	/* Array index: 0x1424 */ 0xB08C,0x3326,
	/* Array index: 0x1424 */ 0xB08D,0x3327,
	/* Array index: 0x1424 */ 0xB08F,0x3328,
	/* Array index: 0x1424 */ 0xB091,0x3329,
	/* Array index: 0x1424 */ 0xB0B8,0x333D,
	/* Array index: 0x1424 */ 0xB0BC,0x333E,
	/* Array index: 0x1424 */ 0xB0D0,0x3344,
	/* Array index: 0x1424 */ 0xB0D1,0x3345,
	/* Array index: 0x1424 */ 0xB0D4,0x3346,
	/* Array index: 0x1424 */ 0xB0D8,0x3347,
	/* Array index: 0x1424 */ 0xB0E0,0x3348,
	/* Array index: 0x1424 */ 0xB0E5,0x3349,
	/* Array index: 0x1424 */ 0xB110,0x334E,
	/* Array index: 0x1424 */ 0xB112,0x334F,
	/* Array index: 0x1424 */ 0xB113,0x3350,
	/* Array index: 0x1424 */ 0xB128,0x3359,
	/* Array index: 0x1424 */ 0xB12C,0x335A,
	/* Array index: 0x1424 */ 0xB140,0x3360,
	/* Array index: 0x1424 */ 0xB141,0x3361,
	/* Array index: 0x1424 */ 0xB144,0x3362,
	/* Array index: 0x1424 */ 0xB148,0x3363,
	/* Array index: 0x1424 */ 0xB150,0x3364,
	/* Array index: 0x1424 */ 0xB151,0x3365,
	/* Array index: 0x1424 */ 0xB154,0x3366,
	/* Array index: 0x1424 */ 0xB155,0x3367,
	/* Array index: 0x1424 */ 0xB158,0x3368,
	/* Array index: 0x1424 */ 0xB15C,0x3369,
	/* Array index: 0x1424 */ 0xB160,0x336A,
	/* Array index: 0x1424 */ 0xB178,0x336B,
	/* Array index: 0x1424 */ 0xB179,0x336C,
	/* Array index: 0x1424 */ 0xB17C,0x336D,
	/* Array index: 0x1424 */ 0xB180,0x336E,
	/* Array index: 0x1424 */ 0xB182,0x336F,
	/* Array index: 0x1424 */ 0xB188,0x3370,
	/* Array index: 0x1424 */ 0xB189,0x3371,
	/* Array index: 0x1424 */ 0xB18B,0x3372,
	/* Array index: 0x1424 */ 0xB18D,0x3373,
	/* Array index: 0x1424 */ 0xB198,0x3377,
	/* Array index: 0x1424 */ 0xB19C,0x3378,
	/* Array index: 0x1424 */ 0xB1A8,0x3379,
	/* Array index: 0x1424 */ 0xB1CC,0x337A,
	/* Array index: 0x1424 */ 0xB1D0,0x337B,
	/* Array index: 0x1424 */ 0xB1D4,0x337C,
	/* Array index: 0x1424 */ 0xB1DC,0x337D,
	/* Array index: 0x1424 */ 0xB1DD,0x337E,
	/* Array index: 0x1424 */ 0xB1DF,0x3421,
	/* Array index: 0x1424 */ 0xB1E8,0x3422,
	/* Array index: 0x1424 */ 0xB1E9,0x3423,
	/* Array index: 0x1424 */ 0xB1EC,0x3424,
	/* Array index: 0x1424 */ 0xB1F0,0x3425,
	/* Array index: 0x1424 */ 0xB1F9,0x3426,
	/* Array index: 0x1424 */ 0xB1FB,0x3427,
	/* Array index: 0x1424 */ 0xB1FD,0x3428,
	/* Array index: 0x1424 */ 0xB204,0x3429,
	/* Array index: 0x1424 */ 0xB205,0x342A,
	/* Array index: 0x1424 */ 0xB208,0x342B,
	/* Array index: 0x1424 */ 0xB20B,0x342C,
	/* Array index: 0x1424 */ 0xB20C,0x342D,
	/* Array index: 0x1424 */ 0xB214,0x342E,
	/* Array index: 0x1424 */ 0xB215,0x342F,
	/* Array index: 0x1424 */ 0xB217,0x3430,
	/* Array index: 0x1424 */ 0xB219,0x3431,
	/* Array index: 0x1424 */ 0xB220,0x3432,
	/* Array index: 0x1424 */ 0xB234,0x3433,
	/* Array index: 0x1424 */ 0xB23C,0x3434,
	/* Array index: 0x1424 */ 0xB258,0x3435,
	/* Array index: 0x1424 */ 0xB25C,0x3436,
	/* Array index: 0x1424 */ 0xB260,0x3437,
	/* Array index: 0x1424 */ 0xB268,0x3438,
	/* Array index: 0x1424 */ 0xB269,0x3439,
	/* Array index: 0x1424 */ 0xB274,0x343A,
	/* Array index: 0x1424 */ 0xB275,0x343B,
	/* Array index: 0x1424 */ 0xB27C,0x343C,
	/* Array index: 0x1424 */ 0xB284,0x343D,
	/* Array index: 0x1424 */ 0xB285,0x343E,
	/* Array index: 0x1424 */ 0xB289,0x343F,
	/* Array index: 0x1424 */ 0xB290,0x3440,
	/* Array index: 0x1424 */ 0xB291,0x3441,
	/* Array index: 0x1424 */ 0xB294,0x3442,
	/* Array index: 0x1424 */ 0xB2A0,0x3446,
	/* Array index: 0x1424 */ 0xB2A1,0x3447,
	/* Array index: 0x1424 */ 0xB2A3,0x3448,
	/* Array index: 0x1424 */ 0xB2A5,0x3449,
	/* Array index: 0x1424 */ 0xB2A6,0x344A,
	/* Array index: 0x1424 */ 0xB2AA,0x344B,
	/* Array index: 0x1424 */ 0xB2AC,0x344C,
	/* Array index: 0x1424 */ 0xB2B0,0x344D,
	/* Array index: 0x1424 */ 0xB2B4,0x344E,
	/* Array index: 0x1424 */ 0xB2C8,0x344F,
	/* Array index: 0x1424 */ 0xB2C9,0x3450,
	/* Array index: 0x1424 */ 0xB2CC,0x3451,
	/* Array index: 0x1424 */ 0xB2D0,0x3452,
	/* Array index: 0x1424 */ 0xB2D2,0x3453,
	/* Array index: 0x1424 */ 0xB2D8,0x3454,
	/* Array index: 0x1424 */ 0xB2D9,0x3455,
	/* Array index: 0x1424 */ 0xB2DB,0x3456,
	/* Array index: 0x1424 */ 0xB2DD,0x3457,
	/* Array index: 0x1424 */ 0xB304,0x346D,
	/* Array index: 0x1424 */ 0xB308,0x346E,
	/* Array index: 0x1424 */ 0xB31C,0x3474,
	/* Array index: 0x1424 */ 0xB364,0x347D,
	/* Array index: 0x1424 */ 0xB365,0x347E,
	/* Array index: 0x1424 */ 0xB367,0x3521,
	/* Array index: 0x1424 */ 0xB369,0x3522,
	/* Array index: 0x1424 */ 0xB36B,0x3523,
	/* Array index: 0x1424 */ 0xB36E,0x3524,
	/* Array index: 0x1424 */ 0xB370,0x3525,
	/* Array index: 0x1424 */ 0xB371,0x3526,
	/* Array index: 0x1424 */ 0xB374,0x3527,
	/* Array index: 0x1424 */ 0xB378,0x3528,
	/* Array index: 0x1424 */ 0xB38C,0x352E,
	/* Array index: 0x1424 */ 0xB390,0x352F,
	/* Array index: 0x1424 */ 0xB394,0x3530,
	/* Array index: 0x1424 */ 0xB3A0,0x3531,
	/* Array index: 0x1424 */ 0xB3A1,0x3532,
	/* Array index: 0x1424 */ 0xB3A8,0x3533,
	/* Array index: 0x1424 */ 0xB3AC,0x3534,
	/* Array index: 0x1424 */ 0xB3C4,0x3535,
	/* Array index: 0x1424 */ 0xB3C5,0x3536,
	/* Array index: 0x1424 */ 0xB3C8,0x3537,
	/* Array index: 0x1424 */ 0xB3CB,0x3538,
	/* Array index: 0x1424 */ 0xB3CC,0x3539,
	/* Array index: 0x1424 */ 0xB3CE,0x353A,
	/* Array index: 0x1424 */ 0xB3D0,0x353B,
	/* Array index: 0x1424 */ 0xB3D4,0x353C,
	/* Array index: 0x1424 */ 0xB3D5,0x353D,
	/* Array index: 0x1424 */ 0xB3D7,0x353E,
	/* Array index: 0x1424 */ 0xB3D9,0x353F,
	/* Array index: 0x1424 */ 0xB3DB,0x3540,
	/* Array index: 0x1424 */ 0xB3DD,0x3541,
	/* Array index: 0x1424 */ 0xB3E0,0x3542,
	/* Array index: 0x1424 */ 0xB3E4,0x3543,
	/* Array index: 0x1424 */ 0xB3E8,0x3544,
	/* Array index: 0x1424 */ 0xB3FC,0x3545,
	/* Array index: 0x1424 */ 0xB410,0x3546,
	/* Array index: 0x1424 */ 0xB418,0x3547,
	/* Array index: 0x1424 */ 0xB41C,0x3548,
	/* Array index: 0x1424 */ 0xB420,0x3549,
	/* Array index: 0x1424 */ 0xB428,0x354A,
	/* Array index: 0x1424 */ 0xB429,0x354B,
	/* Array index: 0x1424 */ 0xB42B,0x354C,
	/* Array index: 0x1424 */ 0xB434,0x354D,
	/* Array index: 0x1424 */ 0xB450,0x354E,
	/* Array index: 0x1424 */ 0xB451,0x354F,
	/* Array index: 0x1424 */ 0xB454,0x3550,
	/* Array index: 0x1424 */ 0xB458,0x3551,
	/* Array index: 0x1424 */ 0xB460,0x3552,
	/* Array index: 0x1424 */ 0xB461,0x3553,
	/* Array index: 0x1424 */ 0xB463,0x3554,
	/* Array index: 0x1424 */ 0xB465,0x3555,
	/* Array index: 0x1424 */ 0xB46C,0x3556,
	/* Array index: 0x1424 */ 0xB480,0x3557,
	/* Array index: 0x1424 */ 0xB488,0x3558,
	/* Array index: 0x1424 */ 0xB49D,0x3559,
	/* Array index: 0x1424 */ 0xB4A4,0x355A,
	/* Array index: 0x1424 */ 0xB4A8,0x355B,
	/* Array index: 0x1424 */ 0xB4AC,0x355C,
	/* Array index: 0x1424 */ 0xB4B5,0x355D,
	/* Array index: 0x1424 */ 0xB4B7,0x355E,
	/* Array index: 0x1424 */ 0xB4B9,0x355F,
	/* Array index: 0x1424 */ 0xB4C0,0x3560,
	/* Array index: 0x1424 */ 0xB4C4,0x3561,
	/* Array index: 0x1424 */ 0xB4C8,0x3562,
	/* Array index: 0x1424 */ 0xB4D0,0x3563,
	/* Array index: 0x1424 */ 0xB4D5,0x3564,
	/* Array index: 0x1424 */ 0xB4DC,0x3565,
	/* Array index: 0x1424 */ 0xB4DD,0x3566,
	/* Array index: 0x1424 */ 0xB4E0,0x3567,
	/* Array index: 0x1424 */ 0xB4E3,0x3568,
	/* Array index: 0x1424 */ 0xB4E4,0x3569,
	/* Array index: 0x1424 */ 0xB4E6,0x356A,
	/* Array index: 0x1424 */ 0xB4EC,0x356B,
	/* Array index: 0x1424 */ 0xB4ED,0x356C,
	/* Array index: 0x1424 */ 0xB4EF,0x356D,
	/* Array index: 0x1424 */ 0xB4F1,0x356E,
	/* Array index: 0x1424 */ 0xB4F8,0x356F,
	/* Array index: 0x1424 */ 0xB514,0x3570,
	/* Array index: 0x1424 */ 0xB515,0x3571,
	/* Array index: 0x1424 */ 0xB518,0x3572,
	/* Array index: 0x1424 */ 0xB51B,0x3573,
	/* Array index: 0x1424 */ 0xB51C,0x3574,
	/* Array index: 0x1424 */ 0xB530,0x357B,
	/* Array index: 0x1424 */ 0xB531,0x357C,
	/* Array index: 0x1424 */ 0xB534,0x357D,
	/* Array index: 0x1424 */ 0xB538,0x357E,
	/* Array index: 0x1424 */ 0xB550,0x3629,
	/* Array index: 0x1424 */ 0xB554,0x362A,
	/* Array index: 0x1424 */ 0xB5A0,0x3630,
	/* Array index: 0x1424 */ 0xB5A1,0x3631,
	/* Array index: 0x1424 */ 0xB5A4,0x3632,
	/* Array index: 0x1424 */ 0xB5A8,0x3633,
	/* Array index: 0x1424 */ 0xB5AA,0x3634,
	/* Array index: 0x1424 */ 0xB5AB,0x3635,
	/* Array index: 0x1424 */ 0xB5C0,0x363E,
	/* Array index: 0x1424 */ 0xB5C4,0x363F,
	/* Array index: 0x1424 */ 0xB5D8,0x3645,
	/* Array index: 0x1424 */ 0xB5EC,0x3646,
	/* Array index: 0x1424 */ 0xB610,0x3647,
	/* Array index: 0x1424 */ 0xB611,0x3648,
	/* Array index: 0x1424 */ 0xB614,0x3649,
	/* Array index: 0x1424 */ 0xB618,0x364A,
	/* Array index: 0x1424 */ 0xB625,0x364B,
	/* Array index: 0x1424 */ 0xB62C,0x364C,
	/* Array index: 0x1424 */ 0xB634,0x364D,
	/* Array index: 0x1424 */ 0xB648,0x364E,
	/* Array index: 0x1424 */ 0xB664,0x364F,
	/* Array index: 0x1424 */ 0xB668,0x3650,
	/* Array index: 0x1424 */ 0xB69C,0x3651,
	/* Array index: 0x1424 */ 0xB69D,0x3652,
	/* Array index: 0x1424 */ 0xB6A0,0x3653,
	/* Array index: 0x1424 */ 0xB6A4,0x3654,
	/* Array index: 0x1424 */ 0xB6AB,0x3655,
	/* Array index: 0x1424 */ 0xB6AC,0x3656,
	/* Array index: 0x1424 */ 0xB6B1,0x3657,
	/* Array index: 0x1424 */ 0xB6D4,0x3658,
	/* Array index: 0x1424 */ 0xB6F0,0x3659,
	/* Array index: 0x1424 */ 0xB6F4,0x365A,
	/* Array index: 0x1424 */ 0xB6F8,0x365B,
	/* Array index: 0x1424 */ 0xB700,0x365C,
	/* Array index: 0x1424 */ 0xB701,0x365D,
	/* Array index: 0x1424 */ 0xB705,0x365E,
	/* Array index: 0x1424 */ 0xB728,0x365F,
	/* Array index: 0x1424 */ 0xB729,0x3660,
	/* Array index: 0x1424 */ 0xB72C,0x3661,
	/* Array index: 0x1424 */ 0xB72F,0x3662,
	/* Array index: 0x1424 */ 0xB730,0x3663,
	/* Array index: 0x1424 */ 0xB738,0x3664,
	/* Array index: 0x1424 */ 0xB739,0x3665,
	/* Array index: 0x1424 */ 0xB73B,0x3666,
	/* Array index: 0x1424 */ 0xB744,0x3667,
	/* Array index: 0x1424 */ 0xB748,0x3668,
	/* Array index: 0x1424 */ 0xB74C,0x3669,
	/* Array index: 0x1424 */ 0xB754,0x366A,
	/* Array index: 0x1424 */ 0xB755,0x366B,
	/* Array index: 0x1424 */ 0xB760,0x366C,
	/* Array index: 0x1424 */ 0xB764,0x366D,
	/* Array index: 0x1424 */ 0xB768,0x366E,
	/* Array index: 0x1424 */ 0xB770,0x366F,
	/* Array index: 0x1424 */ 0xB771,0x3670,
	/* Array index: 0x1424 */ 0xB773,0x3671,
	/* Array index: 0x1424 */ 0xB775,0x3672,
	/* Array index: 0x1424 */ 0xB77C,0x3673,
	/* Array index: 0x1424 */ 0xB77D,0x3674,
	/* Array index: 0x1424 */ 0xB780,0x3675,
	/* Array index: 0x1424 */ 0xB784,0x3676,
	/* Array index: 0x1424 */ 0xB79C,0x3723,
	/* Array index: 0x1424 */ 0xB7A0,0x3724,
	/* Array index: 0x1424 */ 0xB7B4,0x372A,
	/* Array index: 0x1424 */ 0xB7B5,0x372B,
	/* Array index: 0x1424 */ 0xB7B8,0x372C,
	/* Array index: 0x1424 */ 0xB7C7,0x372D,
	/* Array index: 0x1424 */ 0xB7C9,0x372E,
	/* Array index: 0x1424 */ 0xB7EC,0x372F,
	/* Array index: 0x1424 */ 0xB7ED,0x3730,
	/* Array index: 0x1424 */ 0xB7F0,0x3731,
	/* Array index: 0x1424 */ 0xB7F4,0x3732,
	/* Array index: 0x1424 */ 0xB80C,0x373B,
	/* Array index: 0x1424 */ 0xB810,0x373C,
	/* Array index: 0x1424 */ 0xB818,0x373D,
	/* Array index: 0x1424 */ 0xB819,0x373E,
	/* Array index: 0x1424 */ 0xB81B,0x373F,
	/* Array index: 0x1424 */ 0xB81D,0x3740,
	/* Array index: 0x1424 */ 0xB824,0x3741,
	/* Array index: 0x1424 */ 0xB825,0x3742,
	/* Array index: 0x1424 */ 0xB828,0x3743,
	/* Array index: 0x1424 */ 0xB82C,0x3744,
	/* Array index: 0x1424 */ 0xB840,0x374A,
	/* Array index: 0x1424 */ 0xB844,0x374B,
	/* Array index: 0x1424 */ 0xB851,0x374C,
	/* Array index: 0x1424 */ 0xB853,0x374D,
	/* Array index: 0x1424 */ 0xB85C,0x374E,
	/* Array index: 0x1424 */ 0xB85D,0x374F,
	/* Array index: 0x1424 */ 0xB860,0x3750,
	/* Array index: 0x1424 */ 0xB864,0x3751,
	/* Array index: 0x1424 */ 0xB86C,0x3752,
	/* Array index: 0x1424 */ 0xB86D,0x3753,
	/* Array index: 0x1424 */ 0xB86F,0x3754,
	/* Array index: 0x1424 */ 0xB871,0x3755,
	/* Array index: 0x1424 */ 0xB878,0x3756,
	/* Array index: 0x1424 */ 0xB87C,0x3757,
	/* Array index: 0x1424 */ 0xB88D,0x3758,
	/* Array index: 0x1424 */ 0xB8A8,0x3759,
	/* Array index: 0x1424 */ 0xB8B0,0x375A,
	/* Array index: 0x1424 */ 0xB8B4,0x375B,
	/* Array index: 0x1424 */ 0xB8B8,0x375C,
	/* Array index: 0x1424 */ 0xB8C0,0x375D,
	/* Array index: 0x1424 */ 0xB8C1,0x375E,
	/* Array index: 0x1424 */ 0xB8C3,0x375F,
	/* Array index: 0x1424 */ 0xB8C5,0x3760,
	/* Array index: 0x1424 */ 0xB8CC,0x3761,
	/* Array index: 0x1424 */ 0xB8D0,0x3762,
	/* Array index: 0x1424 */ 0xB8D4,0x3763,
	/* Array index: 0x1424 */ 0xB8DD,0x3764,
	/* Array index: 0x1424 */ 0xB8DF,0x3765,
	/* Array index: 0x1424 */ 0xB8E1,0x3766,
	/* Array index: 0x1424 */ 0xB8E8,0x3767,
	/* Array index: 0x1424 */ 0xB8E9,0x3768,
	/* Array index: 0x1424 */ 0xB8EC,0x3769,
	/* Array index: 0x1424 */ 0xB8F0,0x376A,
	/* Array index: 0x1424 */ 0xB8F8,0x376B,
	/* Array index: 0x1424 */ 0xB8F9,0x376C,
	/* Array index: 0x1424 */ 0xB8FB,0x376D,
	/* Array index: 0x1424 */ 0xB8FD,0x376E,
	/* Array index: 0x1424 */ 0xB904,0x376F,
	/* Array index: 0x1424 */ 0xB918,0x3770,
	/* Array index: 0x1424 */ 0xB920,0x3771,
	/* Array index: 0x1424 */ 0xB93C,0x3772,
	/* Array index: 0x1424 */ 0xB93D,0x3773,
	/* Array index: 0x1424 */ 0xB940,0x3774,
	/* Array index: 0x1424 */ 0xB944,0x3775,
	/* Array index: 0x1424 */ 0xB94C,0x3776,
	/* Array index: 0x1424 */ 0xB94F,0x3777,
	/* Array index: 0x1424 */ 0xB951,0x3778,
	/* Array index: 0x1424 */ 0xB958,0x3779,
	/* Array index: 0x1424 */ 0xB959,0x377A,
	/* Array index: 0x1424 */ 0xB95C,0x377B,
	/* Array index: 0x1424 */ 0xB960,0x377C,
	/* Array index: 0x1424 */ 0xB968,0x377D,
	/* Array index: 0x1424 */ 0xB969,0x377E,
	/* Array index: 0x1424 */ 0xB96B,0x3821,
	/* Array index: 0x1424 */ 0xB96D,0x3822,
	/* Array index: 0x1424 */ 0xB974,0x3823,
	/* Array index: 0x1424 */ 0xB975,0x3824,
	/* Array index: 0x1424 */ 0xB978,0x3825,
	/* Array index: 0x1424 */ 0xB97C,0x3826,
	/* Array index: 0x1424 */ 0xB984,0x3827,
	/* Array index: 0x1424 */ 0xB985,0x3828,
	/* Array index: 0x1424 */ 0xB987,0x3829,
	/* Array index: 0x1424 */ 0xB989,0x382A,
	/* Array index: 0x1424 */ 0xB98A,0x382B,
	/* Array index: 0x1424 */ 0xB98D,0x382C,
	/* Array index: 0x1424 */ 0xB98E,0x382D,
	/* Array index: 0x1424 */ 0xB9AC,0x382E,
	/* Array index: 0x1424 */ 0xB9AD,0x382F,
	/* Array index: 0x1424 */ 0xB9B0,0x3830,
	/* Array index: 0x1424 */ 0xB9B4,0x3831,
	/* Array index: 0x1424 */ 0xB9BC,0x3832,
	/* Array index: 0x1424 */ 0xB9BD,0x3833,
	/* Array index: 0x1424 */ 0xB9BF,0x3834,
	/* Array index: 0x1424 */ 0xB9C1,0x3835,
	/* Array index: 0x1424 */ 0xB9C8,0x3836,
	/* Array index: 0x1424 */ 0xB9C9,0x3837,
	/* Array index: 0x1424 */ 0xB9D8,0x383E,
	/* Array index: 0x1424 */ 0xB9D9,0x383F,
	/* Array index: 0x1424 */ 0xB9DB,0x3840,
	/* Array index: 0x1424 */ 0xB9DD,0x3841,
	/* Array index: 0x1424 */ 0xB9DE,0x3842,
	/* Array index: 0x1424 */ 0xB9E8,0x3847,
	/* Array index: 0x1424 */ 0xB9EC,0x3848,
	/* Array index: 0x1424 */ 0xBA00,0x384F,
	/* Array index: 0x1424 */ 0xBA01,0x3850,
	/* Array index: 0x1424 */ 0xBA08,0x3851,
	/* Array index: 0x1424 */ 0xBA15,0x3852,
	/* Array index: 0x1424 */ 0xBA38,0x3853,
	/* Array index: 0x1424 */ 0xBA39,0x3854,
	/* Array index: 0x1424 */ 0xBA3C,0x3855,
	/* Array index: 0x1424 */ 0xBA40,0x3856,
	/* Array index: 0x1424 */ 0xBA42,0x3857,
	/* Array index: 0x1424 */ 0xBA48,0x3858,
	/* Array index: 0x1424 */ 0xBA49,0x3859,
	/* Array index: 0x1424 */ 0xBA4B,0x385A,
	/* Array index: 0x1424 */ 0xBA4D,0x385B,
	/* Array index: 0x1424 */ 0xBA4E,0x385C,
	/* Array index: 0x1424 */ 0xBA58,0x3860,
	/* Array index: 0x1424 */ 0xBA5C,0x3861,
	/* Array index: 0x1424 */ 0xBA70,0x3867,
	/* Array index: 0x1424 */ 0xBA71,0x3868,
	/* Array index: 0x1424 */ 0xBA74,0x3869,
	/* Array index: 0x1424 */ 0xBA78,0x386A,
	/* Array index: 0x1424 */ 0xBA8C,0x386F,
	/* Array index: 0x1424 */ 0xBAB0,0x3874,
	/* Array index: 0x1424 */ 0xBAB2,0x3875,
	/* Array index: 0x1424 */ 0xBAB8,0x3876,
	/* Array index: 0x1424 */ 0xBAB9,0x3877,
	/* Array index: 0x1424 */ 0xBABB,0x3878,
	/* Array index: 0x1424 */ 0xBABD,0x3879,
	/* Array index: 0x1424 */ 0xBAC4,0x387A,
	/* Array index: 0x1424 */ 0xBAC8,0x387B,
	/* Array index: 0x1424 */ 0xBAD8,0x387C,
	/* Array index: 0x1424 */ 0xBAD9,0x387D,
	/* Array index: 0x1424 */ 0xBAFC,0x387E,
	/* Array index: 0x1424 */ 0xBB00,0x3921,
	/* Array index: 0x1424 */ 0xBB04,0x3922,
	/* Array index: 0x1424 */ 0xBB0D,0x3923,
	/* Array index: 0x1424 */ 0xBB0F,0x3924,
	/* Array index: 0x1424 */ 0xBB11,0x3925,
	/* Array index: 0x1424 */ 0xBB18,0x3926,
	/* Array index: 0x1424 */ 0xBB1C,0x3927,
	/* Array index: 0x1424 */ 0xBB20,0x3928,
	/* Array index: 0x1424 */ 0xBB29,0x3929,
	/* Array index: 0x1424 */ 0xBB2B,0x392A,
	/* Array index: 0x1424 */ 0xBB44,0x3933,
	/* Array index: 0x1424 */ 0xBB45,0x3934,
	/* Array index: 0x1424 */ 0xBB47,0x3935,
	/* Array index: 0x1424 */ 0xBB49,0x3936,
	/* Array index: 0x1424 */ 0xBB4D,0x3937,
	/* Array index: 0x1424 */ 0xBB4F,0x3938,
	/* Array index: 0x1424 */ 0xBB50,0x3939,
	/* Array index: 0x1424 */ 0xBB54,0x393A,
	/* Array index: 0x1424 */ 0xBB58,0x393B,
	/* Array index: 0x1424 */ 0xBB61,0x393C,
	/* Array index: 0x1424 */ 0xBB63,0x393D,
	/* Array index: 0x1424 */ 0xBB6C,0x393E,
	/* Array index: 0x1424 */ 0xBB88,0x393F,
	/* Array index: 0x1424 */ 0xBB8C,0x3940,
	/* Array index: 0x1424 */ 0xBB90,0x3941,
	/* Array index: 0x1424 */ 0xBBA4,0x3942,
	/* Array index: 0x1424 */ 0xBBA8,0x3943,
	/* Array index: 0x1424 */ 0xBBAC,0x3944,
	/* Array index: 0x1424 */ 0xBBB4,0x3945,
	/* Array index: 0x1424 */ 0xBBB7,0x3946,
	/* Array index: 0x1424 */ 0xBBC0,0x3947,
	/* Array index: 0x1424 */ 0xBBC4,0x3948,
	/* Array index: 0x1424 */ 0xBBC8,0x3949,
	/* Array index: 0x1424 */ 0xBBD0,0x394A,
	/* Array index: 0x1424 */ 0xBBD3,0x394B,
	/* Array index: 0x1424 */ 0xBBF8,0x394C,
	/* Array index: 0x1424 */ 0xBBF9,0x394D,
	/* Array index: 0x1424 */ 0xBBFC,0x394E,
	/* Array index: 0x1424 */ 0xBBFF,0x394F,
	/* Array index: 0x1424 */ 0xBC00,0x3950,
	/* Array index: 0x1424 */ 0xBC02,0x3951,
	/* Array index: 0x1424 */ 0xBC24,0x3963,
	/* Array index: 0x1424 */ 0xBC25,0x3964,
	/* Array index: 0x1424 */ 0xBC27,0x3965,
	/* Array index: 0x1424 */ 0xBC29,0x3966,
	/* Array index: 0x1424 */ 0xBC2D,0x3967,
	/* Array index: 0x1424 */ 0xBC30,0x3968,
	/* Array index: 0x1424 */ 0xBC31,0x3969,
	/* Array index: 0x1424 */ 0xBC34,0x396A,
	/* Array index: 0x1424 */ 0xBC38,0x396B,
	/* Array index: 0x1424 */ 0xBC49,0x3971,
	/* Array index: 0x1424 */ 0xBC4C,0x3972,
	/* Array index: 0x1424 */ 0xBC4D,0x3973,
	/* Array index: 0x1424 */ 0xBC50,0x3974,
	/* Array index: 0x1424 */ 0xBC5D,0x3975,
	/* Array index: 0x1424 */ 0xBC84,0x3976,
	/* Array index: 0x1424 */ 0xBC85,0x3977,
	/* Array index: 0x1424 */ 0xBC88,0x3978,
	/* Array index: 0x1424 */ 0xBC8B,0x3979,
	/* Array index: 0x1424 */ 0xBC8C,0x397A,
	/* Array index: 0x1424 */ 0xBC8E,0x397B,
	/* Array index: 0x1424 */ 0xBC94,0x397C,
	/* Array index: 0x1424 */ 0xBC95,0x397D,
	/* Array index: 0x1424 */ 0xBC97,0x397E,
	/* Array index: 0x1424 */ 0xBC99,0x3A21,
	/* Array index: 0x1424 */ 0xBC9A,0x3A22,
	/* Array index: 0x1424 */ 0xBCA0,0x3A23,
	/* Array index: 0x1424 */ 0xBCA1,0x3A24,
	/* Array index: 0x1424 */ 0xBCA4,0x3A25,
	/* Array index: 0x1424 */ 0xBCA7,0x3A26,
	/* Array index: 0x1424 */ 0xBCA8,0x3A27,
	/* Array index: 0x1424 */ 0xBCBC,0x3A2D,
	/* Array index: 0x1424 */ 0xBCBD,0x3A2E,
	/* Array index: 0x1424 */ 0xBCC0,0x3A2F,
	/* Array index: 0x1424 */ 0xBCC4,0x3A30,
	/* Array index: 0x1424 */ 0xBCD5,0x3A35,
	/* Array index: 0x1424 */ 0xBCD8,0x3A36,
	/* Array index: 0x1424 */ 0xBCDC,0x3A37,
	/* Array index: 0x1424 */ 0xBCFC,0x3A3C,
	/* Array index: 0x1424 */ 0xBD04,0x3A3D,
	/* Array index: 0x1424 */ 0xBD05,0x3A3E,
	/* Array index: 0x1424 */ 0xBD07,0x3A3F,
	/* Array index: 0x1424 */ 0xBD09,0x3A40,
	/* Array index: 0x1424 */ 0xBD10,0x3A41,
	/* Array index: 0x1424 */ 0xBD14,0x3A42,
	/* Array index: 0x1424 */ 0xBD24,0x3A43,
	/* Array index: 0x1424 */ 0xBD2C,0x3A44,
	/* Array index: 0x1424 */ 0xBD40,0x3A45,
	/* Array index: 0x1424 */ 0xBD48,0x3A46,
	/* Array index: 0x1424 */ 0xBD49,0x3A47,
	/* Array index: 0x1424 */ 0xBD4C,0x3A48,
	/* Array index: 0x1424 */ 0xBD50,0x3A49,
	/* Array index: 0x1424 */ 0xBD58,0x3A4A,
	/* Array index: 0x1424 */ 0xBD59,0x3A4B,
	/* Array index: 0x1424 */ 0xBD64,0x3A4C,
	/* Array index: 0x1424 */ 0xBD68,0x3A4D,
	/* Array index: 0x1424 */ 0xBD80,0x3A4E,
	/* Array index: 0x1424 */ 0xBD81,0x3A4F,
	/* Array index: 0x1424 */ 0xBD84,0x3A50,
	/* Array index: 0x1424 */ 0xBD90,0x3A55,
	/* Array index: 0x1424 */ 0xBD91,0x3A56,
	/* Array index: 0x1424 */ 0xBD93,0x3A57,
	/* Array index: 0x1424 */ 0xBD95,0x3A58,
	/* Array index: 0x1424 */ 0xBD99,0x3A59,
	/* Array index: 0x1424 */ 0xBD9A,0x3A5A,
	/* Array index: 0x1424 */ 0xBD9C,0x3A5B,
	/* Array index: 0x1424 */ 0xBDA4,0x3A5C,
	/* Array index: 0x1424 */ 0xBDB0,0x3A5D,
	/* Array index: 0x1424 */ 0xBDB8,0x3A5E,
	/* Array index: 0x1424 */ 0xBDD4,0x3A5F,
	/* Array index: 0x1424 */ 0xBDD5,0x3A60,
	/* Array index: 0x1424 */ 0xBDD8,0x3A61,
	/* Array index: 0x1424 */ 0xBDDC,0x3A62,
	/* Array index: 0x1424 */ 0xBDE9,0x3A63,
	/* Array index: 0x1424 */ 0xBDF0,0x3A64,
	/* Array index: 0x1424 */ 0xBDF4,0x3A65,
	/* Array index: 0x1424 */ 0xBDF8,0x3A66,
	/* Array index: 0x1424 */ 0xBE00,0x3A67,
	/* Array index: 0x1424 */ 0xBE03,0x3A68,
	/* Array index: 0x1424 */ 0xBE05,0x3A69,
	/* Array index: 0x1424 */ 0xBE0C,0x3A6A,
	/* Array index: 0x1424 */ 0xBE0D,0x3A6B,
	/* Array index: 0x1424 */ 0xBE10,0x3A6C,
	/* Array index: 0x1424 */ 0xBE14,0x3A6D,
	/* Array index: 0x1424 */ 0xBE1C,0x3A6E,
	/* Array index: 0x1424 */ 0xBE1D,0x3A6F,
	/* Array index: 0x1424 */ 0xBE1F,0x3A70,
	/* Array index: 0x1424 */ 0xBE44,0x3A71,
	/* Array index: 0x1424 */ 0xBE45,0x3A72,
	/* Array index: 0x1424 */ 0xBE48,0x3A73,
	/* Array index: 0x1424 */ 0xBE4C,0x3A74,
	/* Array index: 0x1424 */ 0xBE4E,0x3A75,
	/* Array index: 0x1424 */ 0xBE54,0x3A76,
	/* Array index: 0x1424 */ 0xBE55,0x3A77,
	/* Array index: 0x1424 */ 0xBE60,0x3A7C,
	/* Array index: 0x1424 */ 0xBE61,0x3A7D,
	/* Array index: 0x1424 */ 0xBE64,0x3A7E,
	/* Array index: 0x1424 */ 0xBE68,0x3B21,
	/* Array index: 0x1424 */ 0xBE6A,0x3B22,
	/* Array index: 0x1424 */ 0xBE80,0x3B2B,
	/* Array index: 0x1424 */ 0xBE84,0x3B2C,
	/* Array index: 0x1424 */ 0xBE98,0x3B32,
	/* Array index: 0x1424 */ 0xBE99,0x3B33,
	/* Array index: 0x1424 */ 0xBEA8,0x3B34,
	/* Array index: 0x1424 */ 0xBED0,0x3B35,
	/* Array index: 0x1424 */ 0xBED1,0x3B36,
	/* Array index: 0x1424 */ 0xBED4,0x3B37,
	/* Array index: 0x1424 */ 0xBED7,0x3B38,
	/* Array index: 0x1424 */ 0xBED8,0x3B39,
	/* Array index: 0x1424 */ 0xBEE0,0x3B3A,
	/* Array index: 0x1424 */ 0xBEEC,0x3B3E,
	/* Array index: 0x1424 */ 0xBF01,0x3B3F,
	/* Array index: 0x1424 */ 0xBF08,0x3B40,
	/* Array index: 0x1424 */ 0xBF09,0x3B41,
	/* Array index: 0x1424 */ 0xBF40,0x3B47,
	/* Array index: 0x1424 */ 0xBF41,0x3B48,
	/* Array index: 0x1424 */ 0xBF44,0x3B49,
	/* Array index: 0x1424 */ 0xBF48,0x3B4A,
	/* Array index: 0x1424 */ 0xBF50,0x3B4B,
	/* Array index: 0x1424 */ 0xBF51,0x3B4C,
	/* Array index: 0x1424 */ 0xBF55,0x3B4D,
	/* Array index: 0x1424 */ 0xBF94,0x3B4E,
	/* Array index: 0x1424 */ 0xBFB0,0x3B4F,
	/* Array index: 0x1424 */ 0xBFC5,0x3B50,
	/* Array index: 0x1424 */ 0xBFCC,0x3B51,
	/* Array index: 0x1424 */ 0xBFCD,0x3B52,
	/* Array index: 0x1424 */ 0xBFD0,0x3B53,
	/* Array index: 0x1424 */ 0xBFD4,0x3B54,
	/* Array index: 0x1424 */ 0xBFDC,0x3B55,
	/* Array index: 0x1424 */ 0xBFDF,0x3B56,
	/* Array index: 0x1424 */ 0xBFE1,0x3B57,
	/* Array index: 0x1424 */ 0xC03C,0x3B58,
	/* Array index: 0x1424 */ 0xC051,0x3B59,
	/* Array index: 0x1424 */ 0xC058,0x3B5A,
	/* Array index: 0x1424 */ 0xC05C,0x3B5B,
	/* Array index: 0x1424 */ 0xC060,0x3B5C,
	/* Array index: 0x1424 */ 0xC068,0x3B5D,
	/* Array index: 0x1424 */ 0xC069,0x3B5E,
	/* Array index: 0x1424 */ 0xC090,0x3B5F,
	/* Array index: 0x1424 */ 0xC091,0x3B60,
	/* Array index: 0x1424 */ 0xC094,0x3B61,
	/* Array index: 0x1424 */ 0xC098,0x3B62,
	/* Array index: 0x1424 */ 0xC0A0,0x3B63,
	/* Array index: 0x1424 */ 0xC0A1,0x3B64,
	/* Array index: 0x1424 */ 0xC0A3,0x3B65,
	/* Array index: 0x1424 */ 0xC0A5,0x3B66,
	/* Array index: 0x1424 */ 0xC0C5,0x3B74,
	/* Array index: 0x1424 */ 0xC0C8,0x3B75,
	/* Array index: 0x1424 */ 0xC0C9,0x3B76,
	/* Array index: 0x1424 */ 0xC0CC,0x3B77,
	/* Array index: 0x1424 */ 0xC0D0,0x3B78,
	/* Array index: 0x1424 */ 0xC0E4,0x3B7E,
	/* Array index: 0x1424 */ 0xC0E5,0x3C21,
	/* Array index: 0x1424 */ 0xC0E8,0x3C22,
	/* Array index: 0x1424 */ 0xC0EC,0x3C23,
	/* Array index: 0x1424 */ 0xC0F4,0x3C24,
	/* Array index: 0x1424 */ 0xC0F5,0x3C25,
	/* Array index: 0x1424 */ 0xC0F7,0x3C26,
	/* Array index: 0x1424 */ 0xC0F9,0x3C27,
	/* Array index: 0x1424 */ 0xC100,0x3C28,
	/* Array index: 0x1424 */ 0xC104,0x3C29,
	/* Array index: 0x1424 */ 0xC108,0x3C2A,
	/* Array index: 0x1424 */ 0xC110,0x3C2B,
	/* Array index: 0x1424 */ 0xC115,0x3C2C,
	/* Array index: 0x1424 */ 0xC136,0x3C3B,
	/* Array index: 0x1424 */ 0xC138,0x3C3C,
	/* Array index: 0x1424 */ 0xC139,0x3C3D,
	/* Array index: 0x1424 */ 0xC13C,0x3C3E,
	/* Array index: 0x1424 */ 0xC140,0x3C3F,
	/* Array index: 0x1424 */ 0xC154,0x3C45,
	/* Array index: 0x1424 */ 0xC155,0x3C46,
	/* Array index: 0x1424 */ 0xC158,0x3C47,
	/* Array index: 0x1424 */ 0xC15C,0x3C48,
	/* Array index: 0x1424 */ 0xC170,0x3C4E,
	/* Array index: 0x1424 */ 0xC174,0x3C4F,
	/* Array index: 0x1424 */ 0xC178,0x3C50,
	/* Array index: 0x1424 */ 0xC185,0x3C51,
	/* Array index: 0x1424 */ 0xC194,0x3C56,
	/* Array index: 0x1424 */ 0xC196,0x3C57,
	/* Array index: 0x1424 */ 0xC19C,0x3C58,
	/* Array index: 0x1424 */ 0xC19D,0x3C59,
	/* Array index: 0x1424 */ 0xC19F,0x3C5A,
	/* Array index: 0x1424 */ 0xC1A1,0x3C5B,
	/* Array index: 0x1424 */ 0xC1A5,0x3C5C,
	/* Array index: 0x1424 */ 0xC1A8,0x3C5D,
	/* Array index: 0x1424 */ 0xC1A9,0x3C5E,
	/* Array index: 0x1424 */ 0xC1AC,0x3C5F,
	/* Array index: 0x1424 */ 0xC1B0,0x3C60,
	/* Array index: 0x1424 */ 0xC1BD,0x3C61,
	/* Array index: 0x1424 */ 0xC1C4,0x3C62,
	/* Array index: 0x1424 */ 0xC1C8,0x3C63,
	/* Array index: 0x1424 */ 0xC1CC,0x3C64,
	/* Array index: 0x1424 */ 0xC1D4,0x3C65,
	/* Array index: 0x1424 */ 0xC1D7,0x3C66,
	/* Array index: 0x1424 */ 0xC1D8,0x3C67,
	/* Array index: 0x1424 */ 0xC1E0,0x3C68,
	/* Array index: 0x1424 */ 0xC1E4,0x3C69,
	/* Array index: 0x1424 */ 0xC1E8,0x3C6A,
	/* Array index: 0x1424 */ 0xC1F0,0x3C6B,
	/* Array index: 0x1424 */ 0xC1F1,0x3C6C,
	/* Array index: 0x1424 */ 0xC1F3,0x3C6D,
	/* Array index: 0x1424 */ 0xC1FC,0x3C6E,
	/* Array index: 0x1424 */ 0xC1FD,0x3C6F,
	/* Array index: 0x1424 */ 0xC200,0x3C70,
	/* Array index: 0x1424 */ 0xC204,0x3C71,
	/* Array index: 0x1424 */ 0xC20C,0x3C72,
	/* Array index: 0x1424 */ 0xC20D,0x3C73,
	/* Array index: 0x1424 */ 0xC20F,0x3C74,
	/* Array index: 0x1424 */ 0xC211,0x3C75,
	/* Array index: 0x1424 */ 0xC218,0x3C76,
	/* Array index: 0x1424 */ 0xC219,0x3C77,
	/* Array index: 0x1424 */ 0xC21C,0x3C78,
	/* Array index: 0x1424 */ 0xC21F,0x3C79,
	/* Array index: 0x1424 */ 0xC220,0x3C7A,
	/* Array index: 0x1424 */ 0xC228,0x3C7B,
	/* Array index: 0x1424 */ 0xC229,0x3C7C,
	/* Array index: 0x1424 */ 0xC22B,0x3C7D,
	/* Array index: 0x1424 */ 0xC22D,0x3C7E,
	/* Array index: 0x1424 */ 0xC22F,0x3D21,
	/* Array index: 0x1424 */ 0xC231,0x3D22,
	/* Array index: 0x1424 */ 0xC232,0x3D23,
	/* Array index: 0x1424 */ 0xC234,0x3D24,
	/* Array index: 0x1424 */ 0xC248,0x3D25,
	/* Array index: 0x1424 */ 0xC250,0x3D26,
	/* Array index: 0x1424 */ 0xC251,0x3D27,
	/* Array index: 0x1424 */ 0xC254,0x3D28,
	/* Array index: 0x1424 */ 0xC258,0x3D29,
	/* Array index: 0x1424 */ 0xC260,0x3D2A,
	/* Array index: 0x1424 */ 0xC265,0x3D2B,
	/* Array index: 0x1424 */ 0xC26C,0x3D2C,
	/* Array index: 0x1424 */ 0xC26D,0x3D2D,
	/* Array index: 0x1424 */ 0xC270,0x3D2E,
	/* Array index: 0x1424 */ 0xC274,0x3D2F,
	/* Array index: 0x1424 */ 0xC27C,0x3D30,
	/* Array index: 0x1424 */ 0xC27D,0x3D31,
	/* Array index: 0x1424 */ 0xC27F,0x3D32,
	/* Array index: 0x1424 */ 0xC281,0x3D33,
	/* Array index: 0x1424 */ 0xC288,0x3D34,
	/* Array index: 0x1424 */ 0xC289,0x3D35,
	/* Array index: 0x1424 */ 0xC290,0x3D36,
	/* Array index: 0x1424 */ 0xC298,0x3D37,
	/* Array index: 0x1424 */ 0xC29B,0x3D38,
	/* Array index: 0x1424 */ 0xC29D,0x3D39,
	/* Array index: 0x1424 */ 0xC2A4,0x3D3A,
	/* Array index: 0x1424 */ 0xC2A5,0x3D3B,
	/* Array index: 0x1424 */ 0xC2A8,0x3D3C,
	/* Array index: 0x1424 */ 0xC2AC,0x3D3D,
	/* Array index: 0x1424 */ 0xC2AD,0x3D3E,
	/* Array index: 0x1424 */ 0xC2B4,0x3D3F,
	/* Array index: 0x1424 */ 0xC2B5,0x3D40,
	/* Array index: 0x1424 */ 0xC2B7,0x3D41,
	/* Array index: 0x1424 */ 0xC2B9,0x3D42,
	/* Array index: 0x1424 */ 0xC2DC,0x3D43,
	/* Array index: 0x1424 */ 0xC2DD,0x3D44,
	/* Array index: 0x1424 */ 0xC2E0,0x3D45,
	/* Array index: 0x1424 */ 0xC2E3,0x3D46,
	/* Array index: 0x1424 */ 0xC2E4,0x3D47,
	/* Array index: 0x1424 */ 0xC2F6,0x3D4D,
	/* Array index: 0x1424 */ 0xC300,0x3D52,
	/* Array index: 0x1424 */ 0xC308,0x3D53,
	/* Array index: 0x1424 */ 0xC309,0x3D54,
	/* Array index: 0x1424 */ 0xC30C,0x3D55,
	/* Array index: 0x1424 */ 0xC30D,0x3D56,
	/* Array index: 0x1424 */ 0xC318,0x3D5A,
	/* Array index: 0x1424 */ 0xC31C,0x3D5B,
	/* Array index: 0x1424 */ 0xC324,0x3D5C,
	/* Array index: 0x1424 */ 0xC325,0x3D5D,
	/* Array index: 0x1424 */ 0xC328,0x3D5E,
	/* Array index: 0x1424 */ 0xC329,0x3D5F,
	/* Array index: 0x1424 */ 0xC345,0x3D60,
	/* Array index: 0x1424 */ 0xC368,0x3D61,
	/* Array index: 0x1424 */ 0xC369,0x3D62,
	/* Array index: 0x1424 */ 0xC36C,0x3D63,
	/* Array index: 0x1424 */ 0xC370,0x3D64,
	/* Array index: 0x1424 */ 0xC372,0x3D65,
	/* Array index: 0x1424 */ 0xC378,0x3D66,
	/* Array index: 0x1424 */ 0xC379,0x3D67,
	/* Array index: 0x1424 */ 0xC37C,0x3D68,
	/* Array index: 0x1424 */ 0xC37D,0x3D69,
	/* Array index: 0x1424 */ 0xC384,0x3D6A,
	/* Array index: 0x1424 */ 0xC388,0x3D6B,
	/* Array index: 0x1424 */ 0xC38C,0x3D6C,
	/* Array index: 0x1424 */ 0xC3C0,0x3D6D,
	/* Array index: 0x1424 */ 0xC3D8,0x3D6E,
	/* Array index: 0x1424 */ 0xC3D9,0x3D6F,
	/* Array index: 0x1424 */ 0xC3DC,0x3D70,
	/* Array index: 0x1424 */ 0xC3DF,0x3D71,
	/* Array index: 0x1424 */ 0xC3E0,0x3D72,
	/* Array index: 0x1424 */ 0xC3E2,0x3D73,
	/* Array index: 0x1424 */ 0xC3E8,0x3D74,
	/* Array index: 0x1424 */ 0xC3E9,0x3D75,
	/* Array index: 0x1424 */ 0xC3ED,0x3D76,
	/* Array index: 0x1424 */ 0xC3F4,0x3D77,
	/* Array index: 0x1424 */ 0xC3F5,0x3D78,
	/* Array index: 0x1424 */ 0xC3F8,0x3D79,
	/* Array index: 0x1424 */ 0xC408,0x3D7A,
	/* Array index: 0x1424 */ 0xC410,0x3D7B,
	/* Array index: 0x1424 */ 0xC424,0x3D7C,
	/* Array index: 0x1424 */ 0xC42C,0x3D7D,
	/* Array index: 0x1424 */ 0xC430,0x3D7E,
	/* Array index: 0x1424 */ 0xC434,0x3E21,
	/* Array index: 0x1424 */ 0xC43C,0x3E22,
	/* Array index: 0x1424 */ 0xC43D,0x3E23,
	/* Array index: 0x1424 */ 0xC448,0x3E24,
	/* Array index: 0x1424 */ 0xC464,0x3E25,
	/* Array index: 0x1424 */ 0xC465,0x3E26,
	/* Array index: 0x1424 */ 0xC468,0x3E27,
	/* Array index: 0x1424 */ 0xC46C,0x3E28,
	/* Array index: 0x1424 */ 0xC474,0x3E29,
	/* Array index: 0x1424 */ 0xC475,0x3E2A,
	/* Array index: 0x1424 */ 0xC479,0x3E2B,
	/* Array index: 0x1424 */ 0xC480,0x3E2C,
	/* Array index: 0x1424 */ 0xC494,0x3E2D,
	/* Array index: 0x1424 */ 0xC49C,0x3E2E,
	/* Array index: 0x1424 */ 0xC4B8,0x3E2F,
	/* Array index: 0x1424 */ 0xC4BC,0x3E30,
	/* Array index: 0x1424 */ 0xC4E9,0x3E31,
	/* Array index: 0x1424 */ 0xC4F0,0x3E32,
	/* Array index: 0x1424 */ 0xC4F1,0x3E33,
	/* Array index: 0x1424 */ 0xC4F4,0x3E34,
	/* Array index: 0x1424 */ 0xC4F8,0x3E35,
	/* Array index: 0x1424 */ 0xC4FA,0x3E36,
	/* Array index: 0x1424 */ 0xC50C,0x3E3A,
	/* Array index: 0x1424 */ 0xC510,0x3E3B,
	/* Array index: 0x1424 */ 0xC514,0x3E3C,
	/* Array index: 0x1424 */ 0xC51C,0x3E3D,
	/* Array index: 0x1424 */ 0xC528,0x3E3E,
	/* Array index: 0x1424 */ 0xC529,0x3E3F,
	/* Array index: 0x1424 */ 0xC52C,0x3E40,
	/* Array index: 0x1424 */ 0xC530,0x3E41,
	/* Array index: 0x1424 */ 0xC538,0x3E42,
	/* Array index: 0x1424 */ 0xC539,0x3E43,
	/* Array index: 0x1424 */ 0xC53B,0x3E44,
	/* Array index: 0x1424 */ 0xC53D,0x3E45,
	/* Array index: 0x1424 */ 0xC564,0x3E58,
	/* Array index: 0x1424 */ 0xC568,0x3E59,
	/* Array index: 0x1424 */ 0xC57C,0x3E5F,
	/* Array index: 0x1424 */ 0xC57D,0x3E60,
	/* Array index: 0x1424 */ 0xC580,0x3E61,
	/* Array index: 0x1424 */ 0xC584,0x3E62,
	/* Array index: 0x1424 */ 0xC587,0x3E63,
	/* Array index: 0x1424 */ 0xC58C,0x3E64,
	/* Array index: 0x1424 */ 0xC58D,0x3E65,
	/* Array index: 0x1424 */ 0xC58F,0x3E66,
	/* Array index: 0x1424 */ 0xC591,0x3E67,
	/* Array index: 0x1424 */ 0xC595,0x3E68,
	/* Array index: 0x1424 */ 0xC597,0x3E69,
	/* Array index: 0x1424 */ 0xC598,0x3E6A,
	/* Array index: 0x1424 */ 0xC59C,0x3E6B,
	/* Array index: 0x1424 */ 0xC5A0,0x3E6C,
	/* Array index: 0x1424 */ 0xC5A9,0x3E6D,
	/* Array index: 0x1424 */ 0xC5B4,0x3E6E,
	/* Array index: 0x1424 */ 0xC5B5,0x3E6F,
	/* Array index: 0x1424 */ 0xC5D4,0x3F23,
	/* Array index: 0x1424 */ 0xC5D8,0x3F24,
	/* Array index: 0x1424 */ 0xC5E0,0x3F25,
	/* Array index: 0x1424 */ 0xC5E1,0x3F26,
	/* Array index: 0x1424 */ 0xC5E3,0x3F27,
	/* Array index: 0x1424 */ 0xC5E5,0x3F28,
	/* Array index: 0x1424 */ 0xC5F4,0x3F2D,
	/* Array index: 0x1424 */ 0xC5F6,0x3F2E,
	/* Array index: 0x1424 */ 0xC5F7,0x3F2F,
	/* Array index: 0x1424 */ 0xC60C,0x3F3A,
	/* Array index: 0x1424 */ 0xC610,0x3F3B,
	/* Array index: 0x1424 */ 0xC624,0x3F40,
	/* Array index: 0x1424 */ 0xC625,0x3F41,
	/* Array index: 0x1424 */ 0xC628,0x3F42,
	/* Array index: 0x1424 */ 0xC640,0x3F4D,
	/* Array index: 0x1424 */ 0xC641,0x3F4E,
	/* Array index: 0x1424 */ 0xC644,0x3F4F,
	/* Array index: 0x1424 */ 0xC648,0x3F50,
	/* Array index: 0x1424 */ 0xC65C,0x3F56,
	/* Array index: 0x1424 */ 0xC65D,0x3F57,
	/* Array index: 0x1424 */ 0xC660,0x3F58,
	/* Array index: 0x1424 */ 0xC66C,0x3F59,
	/* Array index: 0x1424 */ 0xC66F,0x3F5A,
	/* Array index: 0x1424 */ 0xC671,0x3F5B,
	/* Array index: 0x1424 */ 0xC678,0x3F5C,
	/* Array index: 0x1424 */ 0xC679,0x3F5D,
	/* Array index: 0x1424 */ 0xC67C,0x3F5E,
	/* Array index: 0x1424 */ 0xC680,0x3F5F,
	/* Array index: 0x1424 */ 0xC688,0x3F60,
	/* Array index: 0x1424 */ 0xC689,0x3F61,
	/* Array index: 0x1424 */ 0xC68B,0x3F62,
	/* Array index: 0x1424 */ 0xC68D,0x3F63,
	/* Array index: 0x1424 */ 0xC694,0x3F64,
	/* Array index: 0x1424 */ 0xC695,0x3F65,
	/* Array index: 0x1424 */ 0xC698,0x3F66,
	/* Array index: 0x1424 */ 0xC69C,0x3F67,
	/* Array index: 0x1424 */ 0xC6A4,0x3F68,
	/* Array index: 0x1424 */ 0xC6A5,0x3F69,
	/* Array index: 0x1424 */ 0xC6A7,0x3F6A,
	/* Array index: 0x1424 */ 0xC6A9,0x3F6B,
	/* Array index: 0x1424 */ 0xC6B0,0x3F6C,
	/* Array index: 0x1424 */ 0xC6B1,0x3F6D,
	/* Array index: 0x1424 */ 0xC6B4,0x3F6E,
	/* Array index: 0x1424 */ 0xC6C0,0x3F72,
	/* Array index: 0x1424 */ 0xC6C1,0x3F73,
	/* Array index: 0x1424 */ 0xC6C3,0x3F74,
	/* Array index: 0x1424 */ 0xC6C5,0x3F75,
	/* Array index: 0x1424 */ 0xC6CC,0x3F76,
	/* Array index: 0x1424 */ 0xC6CD,0x3F77,
	/* Array index: 0x1424 */ 0xC6D0,0x3F78,
	/* Array index: 0x1424 */ 0xC6D4,0x3F79,
	/* Array index: 0x1424 */ 0xC6DC,0x3F7A,
	/* Array index: 0x1424 */ 0xC6DD,0x3F7B,
	/* Array index: 0x1424 */ 0xC6E0,0x3F7C,
	/* Array index: 0x1424 */ 0xC6E1,0x3F7D,
	/* Array index: 0x1424 */ 0xC6E8,0x3F7E,
	/* Array index: 0x1424 */ 0xC6E9,0x4021,
	/* Array index: 0x1424 */ 0xC6EC,0x4022,
	/* Array index: 0x1424 */ 0xC6F0,0x4023,
	/* Array index: 0x1424 */ 0xC6F8,0x4024,
	/* Array index: 0x1424 */ 0xC6F9,0x4025,
	/* Array index: 0x1424 */ 0xC6FD,0x4026,
	/* Array index: 0x1424 */ 0xC704,0x4027,
	/* Array index: 0x1424 */ 0xC705,0x4028,
	/* Array index: 0x1424 */ 0xC708,0x4029,
	/* Array index: 0x1424 */ 0xC70C,0x402A,
	/* Array index: 0x1424 */ 0xC714,0x402B,
	/* Array index: 0x1424 */ 0xC715,0x402C,
	/* Array index: 0x1424 */ 0xC717,0x402D,
	/* Array index: 0x1424 */ 0xC719,0x402E,
	/* Array index: 0x1424 */ 0xC720,0x402F,
	/* Array index: 0x1424 */ 0xC721,0x4030,
	/* Array index: 0x1424 */ 0xC724,0x4031,
	/* Array index: 0x1424 */ 0xC728,0x4032,
	/* Array index: 0x1424 */ 0xC730,0x4033,
	/* Array index: 0x1424 */ 0xC731,0x4034,
	/* Array index: 0x1424 */ 0xC733,0x4035,
	/* Array index: 0x1424 */ 0xC735,0x4036,
	/* Array index: 0x1424 */ 0xC737,0x4037,
	/* Array index: 0x1424 */ 0xC73C,0x4038,
	/* Array index: 0x1424 */ 0xC73D,0x4039,
	/* Array index: 0x1424 */ 0xC740,0x403A,
	/* Array index: 0x1424 */ 0xC744,0x403B,
	/* Array index: 0x1424 */ 0xC74A,0x403C,
	/* Array index: 0x1424 */ 0xC74C,0x403D,
	/* Array index: 0x1424 */ 0xC74D,0x403E,
	/* Array index: 0x1424 */ 0xC75C,0x4048,
	/* Array index: 0x1424 */ 0xC760,0x4049,
	/* Array index: 0x1424 */ 0xC768,0x404A,
	/* Array index: 0x1424 */ 0xC76B,0x404B,
	/* Array index: 0x1424 */ 0xC774,0x404C,
	/* Array index: 0x1424 */ 0xC775,0x404D,
	/* Array index: 0x1424 */ 0xC778,0x404E,
	/* Array index: 0x1424 */ 0xC78E,0x4059,
	/* Array index: 0x1424 */ 0xC790,0x405A,
	/* Array index: 0x1424 */ 0xC791,0x405B,
	/* Array index: 0x1424 */ 0xC7AC,0x4067,
	/* Array index: 0x1424 */ 0xC7AD,0x4068,
	/* Array index: 0x1424 */ 0xC7B0,0x4069,
	/* Array index: 0x1424 */ 0xC7B4,0x406A,
	/* Array index: 0x1424 */ 0xC7C8,0x4070,
	/* Array index: 0x1424 */ 0xC7C9,0x4071,
	/* Array index: 0x1424 */ 0xC7CC,0x4072,
	/* Array index: 0x1424 */ 0xC7CE,0x4073,
	/* Array index: 0x1424 */ 0xC7D0,0x4074,
	/* Array index: 0x1424 */ 0xC7D8,0x4075,
	/* Array index: 0x1424 */ 0xC7DD,0x4076,
	/* Array index: 0x1424 */ 0xC7E4,0x4077,
	/* Array index: 0x1424 */ 0xC7E8,0x4078,
	/* Array index: 0x1424 */ 0xC7EC,0x4079,
	/* Array index: 0x1424 */ 0xC800,0x407A,
	/* Array index: 0x1424 */ 0xC801,0x407B,
	/* Array index: 0x1424 */ 0xC804,0x407C,
	/* Array index: 0x1424 */ 0xC808,0x407D,
	/* Array index: 0x1424 */ 0xC80A,0x407E,
	/* Array index: 0x1424 */ 0xC810,0x4121,
	/* Array index: 0x1424 */ 0xC811,0x4122,
	/* Array index: 0x1424 */ 0xC813,0x4123,
	/* Array index: 0x1424 */ 0xC815,0x4124,
	/* Array index: 0x1424 */ 0xC816,0x4125,
	/* Array index: 0x1424 */ 0xC81C,0x4126,
	/* Array index: 0x1424 */ 0xC81D,0x4127,
	/* Array index: 0x1424 */ 0xC820,0x4128,
	/* Array index: 0x1424 */ 0xC824,0x4129,
	/* Array index: 0x1424 */ 0xC82C,0x412A,
	/* Array index: 0x1424 */ 0xC82D,0x412B,
	/* Array index: 0x1424 */ 0xC82F,0x412C,
	/* Array index: 0x1424 */ 0xC831,0x412D,
	/* Array index: 0x1424 */ 0xC838,0x412E,
	/* Array index: 0x1424 */ 0xC83C,0x412F,
	/* Array index: 0x1424 */ 0xC840,0x4130,
	/* Array index: 0x1424 */ 0xC848,0x4131,
	/* Array index: 0x1424 */ 0xC849,0x4132,
	/* Array index: 0x1424 */ 0xC84C,0x4133,
	/* Array index: 0x1424 */ 0xC84D,0x4134,
	/* Array index: 0x1424 */ 0xC854,0x4135,
	/* Array index: 0x1424 */ 0xC870,0x4136,
	/* Array index: 0x1424 */ 0xC871,0x4137,
	/* Array index: 0x1424 */ 0xC874,0x4138,
	/* Array index: 0x1424 */ 0xC878,0x4139,
	/* Array index: 0x1424 */ 0xC87A,0x413A,
	/* Array index: 0x1424 */ 0xC880,0x413B,
	/* Array index: 0x1424 */ 0xC881,0x413C,
	/* Array index: 0x1424 */ 0xC894,0x4144,
	/* Array index: 0x1424 */ 0xC89D,0x4145,
	/* Array index: 0x1424 */ 0xC89F,0x4146,
	/* Array index: 0x1424 */ 0xC8A1,0x4147,
	/* Array index: 0x1424 */ 0xC8A8,0x4148,
	/* Array index: 0x1424 */ 0xC8BC,0x4149,
	/* Array index: 0x1424 */ 0xC8BD,0x414A,
	/* Array index: 0x1424 */ 0xC8C4,0x414B,
	/* Array index: 0x1424 */ 0xC8C8,0x414C,
	/* Array index: 0x1424 */ 0xC8CC,0x414D,
	/* Array index: 0x1424 */ 0xC8D4,0x414E,
	/* Array index: 0x1424 */ 0xC8D5,0x414F,
	/* Array index: 0x1424 */ 0xC8D7,0x4150,
	/* Array index: 0x1424 */ 0xC8D9,0x4151,
	/* Array index: 0x1424 */ 0xC8E0,0x4152,
	/* Array index: 0x1424 */ 0xC8E1,0x4153,
	/* Array index: 0x1424 */ 0xC8E4,0x4154,
	/* Array index: 0x1424 */ 0xC8F5,0x4155,
	/* Array index: 0x1424 */ 0xC8FC,0x4156,
	/* Array index: 0x1424 */ 0xC8FD,0x4157,
	/* Array index: 0x1424 */ 0xC900,0x4158,
	/* Array index: 0x1424 */ 0xC90C,0x415C,
	/* Array index: 0x1424 */ 0xC90D,0x415D,
	/* Array index: 0x1424 */ 0xC90F,0x415E,
	/* Array index: 0x1424 */ 0xC911,0x415F,
	/* Array index: 0x1424 */ 0xC918,0x4160,
	/* Array index: 0x1424 */ 0xC92C,0x4161,
	/* Array index: 0x1424 */ 0xC934,0x4162,
	/* Array index: 0x1424 */ 0xC950,0x4163,
	/* Array index: 0x1424 */ 0xC951,0x4164,
	/* Array index: 0x1424 */ 0xC954,0x4165,
	/* Array index: 0x1424 */ 0xC958,0x4166,
	/* Array index: 0x1424 */ 0xC960,0x4167,
	/* Array index: 0x1424 */ 0xC961,0x4168,
	/* Array index: 0x1424 */ 0xC963,0x4169,
	/* Array index: 0x1424 */ 0xC96C,0x416A,
	/* Array index: 0x1424 */ 0xC970,0x416B,
	/* Array index: 0x1424 */ 0xC974,0x416C,
	/* Array index: 0x1424 */ 0xC97C,0x416D,
	/* Array index: 0x1424 */ 0xC988,0x416E,
	/* Array index: 0x1424 */ 0xC989,0x416F,
	/* Array index: 0x1424 */ 0xC98C,0x4170,
	/* Array index: 0x1424 */ 0xC990,0x4171,
	/* Array index: 0x1424 */ 0xC998,0x4172,
	/* Array index: 0x1424 */ 0xC999,0x4173,
	/* Array index: 0x1424 */ 0xC99B,0x4174,
	/* Array index: 0x1424 */ 0xC99D,0x4175,
	/* Array index: 0x1424 */ 0xC9C0,0x4176,
	/* Array index: 0x1424 */ 0xC9C1,0x4177,
	/* Array index: 0x1424 */ 0xC9C4,0x4178,
	/* Array index: 0x1424 */ 0xC9C7,0x4179,
	/* Array index: 0x1424 */ 0xC9C8,0x417A,
	/* Array index: 0x1424 */ 0xC9CA,0x417B,
	/* Array index: 0x1424 */ 0xC9D0,0x417C,
	/* Array index: 0x1424 */ 0xC9D1,0x417D,
	/* Array index: 0x1424 */ 0xC9D3,0x417E,
	/* Array index: 0x1424 */ 0xC9D5,0x4221,
	/* Array index: 0x1424 */ 0xC9D6,0x4222,
	/* Array index: 0x1424 */ 0xC9E0,0x4227,
	/* Array index: 0x1424 */ 0xC9E2,0x4228,
	/* Array index: 0x1424 */ 0xC9E4,0x4229,
	/* Array index: 0x1424 */ 0xC9E7,0x422A,
	/* Array index: 0x1424 */ 0xC9F8,0x4230,
	/* Array index: 0x1424 */ 0xC9F9,0x4231,
	/* Array index: 0x1424 */ 0xC9FC,0x4232,
	/* Array index: 0x1424 */ 0xCA00,0x4233,
	/* Array index: 0x1424 */ 0xCA14,0x4239,
	/* Array index: 0x1424 */ 0xCA18,0x423A,
	/* Array index: 0x1424 */ 0xCA29,0x423B,
	/* Array index: 0x1424 */ 0xCA4C,0x423C,
	/* Array index: 0x1424 */ 0xCA4D,0x423D,
	/* Array index: 0x1424 */ 0xCA50,0x423E,
	/* Array index: 0x1424 */ 0xCA54,0x423F,
	/* Array index: 0x1424 */ 0xCA68,0x4245,
	/* Array index: 0x1424 */ 0xCA7D,0x4246,
	/* Array index: 0x1424 */ 0xCA84,0x4247,
	/* Array index: 0x1424 */ 0xCA98,0x4248,
	/* Array index: 0x1424 */ 0xCABC,0x4249,
	/* Array index: 0x1424 */ 0xCABD,0x424A,
	/* Array index: 0x1424 */ 0xCAC0,0x424B,
	/* Array index: 0x1424 */ 0xCAC4,0x424C,
	/* Array index: 0x1424 */ 0xCACC,0x424D,
	/* Array index: 0x1424 */ 0xCACD,0x424E,
	/* Array index: 0x1424 */ 0xCACF,0x424F,
	/* Array index: 0x1424 */ 0xCAD1,0x4250,
	/* Array index: 0x1424 */ 0xCAD3,0x4251,
	/* Array index: 0x1424 */ 0xCAD8,0x4252,
	/* Array index: 0x1424 */ 0xCAD9,0x4253,
	/* Array index: 0x1424 */ 0xCAE0,0x4254,
	/* Array index: 0x1424 */ 0xCAEC,0x4255,
	/* Array index: 0x1424 */ 0xCAF4,0x4256,
	/* Array index: 0x1424 */ 0xCB08,0x4257,
	/* Array index: 0x1424 */ 0xCB10,0x4258,
	/* Array index: 0x1424 */ 0xCB14,0x4259,
	/* Array index: 0x1424 */ 0xCB18,0x425A,
	/* Array index: 0x1424 */ 0xCB20,0x425B,
	/* Array index: 0x1424 */ 0xCB21,0x425C,
	/* Array index: 0x1424 */ 0xCB41,0x425D,
	/* Array index: 0x1424 */ 0xCB48,0x425E,
	/* Array index: 0x1424 */ 0xCB49,0x425F,
	/* Array index: 0x1424 */ 0xCB4C,0x4260,
	/* Array index: 0x1424 */ 0xCB50,0x4261,
	/* Array index: 0x1424 */ 0xCB58,0x4262,
	/* Array index: 0x1424 */ 0xCB59,0x4263,
	/* Array index: 0x1424 */ 0xCB5D,0x4264,
	/* Array index: 0x1424 */ 0xCB64,0x4265,
	/* Array index: 0x1424 */ 0xCB78,0x4266,
	/* Array index: 0x1424 */ 0xCB79,0x4267,
	/* Array index: 0x1424 */ 0xCB9C,0x4268,
	/* Array index: 0x1424 */ 0xCBB8,0x4269,
	/* Array index: 0x1424 */ 0xCBD4,0x426A,
	/* Array index: 0x1424 */ 0xCBE4,0x426B,
	/* Array index: 0x1424 */ 0xCBE7,0x426C,
	/* Array index: 0x1424 */ 0xCBE9,0x426D,
	/* Array index: 0x1424 */ 0xCC0C,0x426E,
	/* Array index: 0x1424 */ 0xCC0D,0x426F,
	/* Array index: 0x1424 */ 0xCC10,0x4270,
	/* Array index: 0x1424 */ 0xCC14,0x4271,
	/* Array index: 0x1424 */ 0xCC1C,0x4272,
	/* Array index: 0x1424 */ 0xCC1D,0x4273,
	/* Array index: 0x1424 */ 0xCC21,0x4274,
	/* Array index: 0x1424 */ 0xCC22,0x4275,
	/* Array index: 0x1424 */ 0xCC2C,0x4279,
	/* Array index: 0x1424 */ 0xCC2E,0x427A,
	/* Array index: 0x1424 */ 0xCC30,0x427B,
	/* Array index: 0x1424 */ 0xCC44,0x4324,
	/* Array index: 0x1424 */ 0xCC45,0x4325,
	/* Array index: 0x1424 */ 0xCC48,0x4326,
	/* Array index: 0x1424 */ 0xCC4C,0x4327,
	/* Array index: 0x1424 */ 0xCC60,0x432D,
	/* Array index: 0x1424 */ 0xCC64,0x432E,
	/* Array index: 0x1424 */ 0xCC66,0x432F,
	/* Array index: 0x1424 */ 0xCC68,0x4330,
	/* Array index: 0x1424 */ 0xCC70,0x4331,
	/* Array index: 0x1424 */ 0xCC75,0x4332,
	/* Array index: 0x1424 */ 0xCC98,0x4333,
	/* Array index: 0x1424 */ 0xCC99,0x4334,
	/* Array index: 0x1424 */ 0xCC9C,0x4335,
	/* Array index: 0x1424 */ 0xCCA0,0x4336,
	/* Array index: 0x1424 */ 0xCCB4,0x433C,
	/* Array index: 0x1424 */ 0xCCB5,0x433D,
	/* Array index: 0x1424 */ 0xCCB8,0x433E,
	/* Array index: 0x1424 */ 0xCCBC,0x433F,
	/* Array index: 0x1424 */ 0xCCC4,0x4340,
	/* Array index: 0x1424 */ 0xCCC5,0x4341,
	/* Array index: 0x1424 */ 0xCCC7,0x4342,
	/* Array index: 0x1424 */ 0xCCC9,0x4343,
	/* Array index: 0x1424 */ 0xCCD0,0x4344,
	/* Array index: 0x1424 */ 0xCCD4,0x4345,
	/* Array index: 0x1424 */ 0xCCE4,0x4346,
	/* Array index: 0x1424 */ 0xCCEC,0x4347,
	/* Array index: 0x1424 */ 0xCCF0,0x4348,
	/* Array index: 0x1424 */ 0xCD01,0x4349,
	/* Array index: 0x1424 */ 0xCD08,0x434A,
	/* Array index: 0x1424 */ 0xCD09,0x434B,
	/* Array index: 0x1424 */ 0xCD0C,0x434C,
	/* Array index: 0x1424 */ 0xCD10,0x434D,
	/* Array index: 0x1424 */ 0xCD18,0x434E,
	/* Array index: 0x1424 */ 0xCD19,0x434F,
	/* Array index: 0x1424 */ 0xCD1B,0x4350,
	/* Array index: 0x1424 */ 0xCD1D,0x4351,
	/* Array index: 0x1424 */ 0xCD24,0x4352,
	/* Array index: 0x1424 */ 0xCD28,0x4353,
	/* Array index: 0x1424 */ 0xCD2C,0x4354,
	/* Array index: 0x1424 */ 0xCD39,0x4355,
	/* Array index: 0x1424 */ 0xCD5C,0x4356,
	/* Array index: 0x1424 */ 0xCD60,0x4357,
	/* Array index: 0x1424 */ 0xCD64,0x4358,
	/* Array index: 0x1424 */ 0xCD6C,0x4359,
	/* Array index: 0x1424 */ 0xCD6D,0x435A,
	/* Array index: 0x1424 */ 0xCD6F,0x435B,
	/* Array index: 0x1424 */ 0xCD71,0x435C,
	/* Array index: 0x1424 */ 0xCD78,0x435D,
	/* Array index: 0x1424 */ 0xCD88,0x435E,
	/* Array index: 0x1424 */ 0xCD94,0x435F,
	/* Array index: 0x1424 */ 0xCD95,0x4360,
	/* Array index: 0x1424 */ 0xCD98,0x4361,
	/* Array index: 0x1424 */ 0xCD9C,0x4362,
	/* Array index: 0x1424 */ 0xCDA4,0x4363,
	/* Array index: 0x1424 */ 0xCDA5,0x4364,
	/* Array index: 0x1424 */ 0xCDA7,0x4365,
	/* Array index: 0x1424 */ 0xCDA9,0x4366,
	/* Array index: 0x1424 */ 0xCDB0,0x4367,
	/* Array index: 0x1424 */ 0xCDC4,0x4368,
	/* Array index: 0x1424 */ 0xCDCC,0x4369,
	/* Array index: 0x1424 */ 0xCDD0,0x436A,
	/* Array index: 0x1424 */ 0xCDE8,0x436B,
	/* Array index: 0x1424 */ 0xCDEC,0x436C,
	/* Array index: 0x1424 */ 0xCDF0,0x436D,
	/* Array index: 0x1424 */ 0xCDF8,0x436E,
	/* Array index: 0x1424 */ 0xCDF9,0x436F,
	/* Array index: 0x1424 */ 0xCDFB,0x4370,
	/* Array index: 0x1424 */ 0xCDFD,0x4371,
	/* Array index: 0x1424 */ 0xCE04,0x4372,
	/* Array index: 0x1424 */ 0xCE08,0x4373,
	/* Array index: 0x1424 */ 0xCE0C,0x4374,
	/* Array index: 0x1424 */ 0xCE14,0x4375,
	/* Array index: 0x1424 */ 0xCE19,0x4376,
	/* Array index: 0x1424 */ 0xCE20,0x4377,
	/* Array index: 0x1424 */ 0xCE21,0x4378,
	/* Array index: 0x1424 */ 0xCE24,0x4379,
	/* Array index: 0x1424 */ 0xCE28,0x437A,
	/* Array index: 0x1424 */ 0xCE30,0x437B,
	/* Array index: 0x1424 */ 0xCE31,0x437C,
	/* Array index: 0x1424 */ 0xCE33,0x437D,
	/* Array index: 0x1424 */ 0xCE35,0x437E,
	/* Array index: 0x1424 */ 0xCE58,0x4421,
	/* Array index: 0x1424 */ 0xCE59,0x4422,
	/* Array index: 0x1424 */ 0xCE5C,0x4423,
	/* Array index: 0x1424 */ 0xCE68,0x4427,
	/* Array index: 0x1424 */ 0xCE69,0x4428,
	/* Array index: 0x1424 */ 0xCE6B,0x4429,
	/* Array index: 0x1424 */ 0xCE6D,0x442A,
	/* Array index: 0x1424 */ 0xCE74,0x442B,
	/* Array index: 0x1424 */ 0xCE75,0x442C,
	/* Array index: 0x1424 */ 0xCE78,0x442D,
	/* Array index: 0x1424 */ 0xCE7C,0x442E,
	/* Array index: 0x1424 */ 0xCE84,0x442F,
	/* Array index: 0x1424 */ 0xCE85,0x4430,
	/* Array index: 0x1424 */ 0xCE87,0x4431,
	/* Array index: 0x1424 */ 0xCE89,0x4432,
	/* Array index: 0x1424 */ 0xCE90,0x4433,
	/* Array index: 0x1424 */ 0xCE91,0x4434,
	/* Array index: 0x1424 */ 0xCE94,0x4435,
	/* Array index: 0x1424 */ 0xCE98,0x4436,
	/* Array index: 0x1424 */ 0xCEAC,0x443C,
	/* Array index: 0x1424 */ 0xCEAD,0x443D,
	/* Array index: 0x1424 */ 0xCEC1,0x443E,
	/* Array index: 0x1424 */ 0xCEE4,0x443F,
	/* Array index: 0x1424 */ 0xCEE5,0x4440,
	/* Array index: 0x1424 */ 0xCEE8,0x4441,
	/* Array index: 0x1424 */ 0xCEEB,0x4442,
	/* Array index: 0x1424 */ 0xCEEC,0x4443,
	/* Array index: 0x1424 */ 0xCF00,0x4449,
	/* Array index: 0x1424 */ 0xCF01,0x444A,
	/* Array index: 0x1424 */ 0xCF04,0x444B,
	/* Array index: 0x1424 */ 0xCF08,0x444C,
	/* Array index: 0x1424 */ 0xCF10,0x444D,
	/* Array index: 0x1424 */ 0xCF11,0x444E,
	/* Array index: 0x1424 */ 0xCF13,0x444F,
	/* Array index: 0x1424 */ 0xCF15,0x4450,
	/* Array index: 0x1424 */ 0xCF1C,0x4451,
	/* Array index: 0x1424 */ 0xCF20,0x4452,
	/* Array index: 0x1424 */ 0xCF24,0x4453,
	/* Array index: 0x1424 */ 0xCF38,0x4459,
	/* Array index: 0x1424 */ 0xCF54,0x445A,
	/* Array index: 0x1424 */ 0xCF55,0x445B,
	/* Array index: 0x1424 */ 0xCF58,0x445C,
	/* Array index: 0x1424 */ 0xCF5C,0x445D,
	/* Array index: 0x1424 */ 0xCF64,0x445E,
	/* Array index: 0x1424 */ 0xCF65,0x445F,
	/* Array index: 0x1424 */ 0xCF67,0x4460,
	/* Array index: 0x1424 */ 0xCF69,0x4461,
	/* Array index: 0x1424 */ 0xCF70,0x4462,
	/* Array index: 0x1424 */ 0xCF71,0x4463,
	/* Array index: 0x1424 */ 0xCF74,0x4464,
	/* Array index: 0x1424 */ 0xCF78,0x4465,
	/* Array index: 0x1424 */ 0xCF80,0x4466,
	/* Array index: 0x1424 */ 0xCF85,0x4467,
	/* Array index: 0x1424 */ 0xCF8C,0x4468,
	/* Array index: 0x1424 */ 0xCFA1,0x4469,
	/* Array index: 0x1424 */ 0xCFA8,0x446A,
	/* Array index: 0x1424 */ 0xCFB0,0x446B,
	/* Array index: 0x1424 */ 0xCFC4,0x446C,
	/* Array index: 0x1424 */ 0xCFE0,0x446D,
	/* Array index: 0x1424 */ 0xCFE1,0x446E,
	/* Array index: 0x1424 */ 0xCFE4,0x446F,
	/* Array index: 0x1424 */ 0xCFE8,0x4470,
	/* Array index: 0x1424 */ 0xCFF0,0x4471,
	/* Array index: 0x1424 */ 0xCFF1,0x4472,
	/* Array index: 0x1424 */ 0xCFF3,0x4473,
	/* Array index: 0x1424 */ 0xCFF5,0x4474,
	/* Array index: 0x1424 */ 0xCFFC,0x4475,
	/* Array index: 0x1424 */ 0xD000,0x4476,
	/* Array index: 0x1424 */ 0xD004,0x4477,
	/* Array index: 0x1424 */ 0xD011,0x4478,
	/* Array index: 0x1424 */ 0xD018,0x4479,
	/* Array index: 0x1424 */ 0xD02D,0x447A,
	/* Array index: 0x1424 */ 0xD034,0x447B,
	/* Array index: 0x1424 */ 0xD035,0x447C,
	/* Array index: 0x1424 */ 0xD038,0x447D,
	/* Array index: 0x1424 */ 0xD03C,0x447E,
	/* Array index: 0x1424 */ 0xD044,0x4521,
	/* Array index: 0x1424 */ 0xD045,0x4522,
	/* Array index: 0x1424 */ 0xD047,0x4523,
	/* Array index: 0x1424 */ 0xD049,0x4524,
	/* Array index: 0x1424 */ 0xD050,0x4525,
	/* Array index: 0x1424 */ 0xD054,0x4526,
	/* Array index: 0x1424 */ 0xD058,0x4527,
	/* Array index: 0x1424 */ 0xD060,0x4528,
	/* Array index: 0x1424 */ 0xD06C,0x4529,
	/* Array index: 0x1424 */ 0xD06D,0x452A,
	/* Array index: 0x1424 */ 0xD070,0x452B,
	/* Array index: 0x1424 */ 0xD074,0x452C,
	/* Array index: 0x1424 */ 0xD07C,0x452D,
	/* Array index: 0x1424 */ 0xD07D,0x452E,
	/* Array index: 0x1424 */ 0xD081,0x452F,
	/* Array index: 0x1424 */ 0xD0A4,0x4530,
	/* Array index: 0x1424 */ 0xD0A5,0x4531,
	/* Array index: 0x1424 */ 0xD0A8,0x4532,
	/* Array index: 0x1424 */ 0xD0AC,0x4533,
	/* Array index: 0x1424 */ 0xD0B4,0x4534,
	/* Array index: 0x1424 */ 0xD0B5,0x4535,
	/* Array index: 0x1424 */ 0xD0B7,0x4536,
	/* Array index: 0x1424 */ 0xD0B9,0x4537,
	/* Array index: 0x1424 */ 0xD0C0,0x4538,
	/* Array index: 0x1424 */ 0xD0C1,0x4539,
	/* Array index: 0x1424 */ 0xD0C4,0x453A,
	/* Array index: 0x1424 */ 0xD0C8,0x453B,
	/* Array index: 0x1424 */ 0xD0C9,0x453C,
	/* Array index: 0x1424 */ 0xD0DC,0x4542,
	/* Array index: 0x1424 */ 0xD0DD,0x4543,
	/* Array index: 0x1424 */ 0xD0E0,0x4544,
	/* Array index: 0x1424 */ 0xD0E4,0x4545,
	/* Array index: 0x1424 */ 0xD0F8,0x454B,
	/* Array index: 0x1424 */ 0xD10D,0x454C,
	/* Array index: 0x1424 */ 0xD130,0x454D,
	/* Array index: 0x1424 */ 0xD131,0x454E,
	/* Array index: 0x1424 */ 0xD134,0x454F,
	/* Array index: 0x1424 */ 0xD138,0x4550,
	/* Array index: 0x1424 */ 0xD13A,0x4551,
	/* Array index: 0x1424 */ 0xD14C,0x4557,
	/* Array index: 0x1424 */ 0xD14D,0x4558,
	/* Array index: 0x1424 */ 0xD150,0x4559,
	/* Array index: 0x1424 */ 0xD154,0x455A,
	/* Array index: 0x1424 */ 0xD15C,0x455B,
	/* Array index: 0x1424 */ 0xD15D,0x455C,
	/* Array index: 0x1424 */ 0xD15F,0x455D,
	/* Array index: 0x1424 */ 0xD161,0x455E,
	/* Array index: 0x1424 */ 0xD168,0x455F,
	/* Array index: 0x1424 */ 0xD16C,0x4560,
	/* Array index: 0x1424 */ 0xD17C,0x4561,
	/* Array index: 0x1424 */ 0xD184,0x4562,
	/* Array index: 0x1424 */ 0xD188,0x4563,
	/* Array index: 0x1424 */ 0xD1A0,0x4564,
	/* Array index: 0x1424 */ 0xD1A1,0x4565,
	/* Array index: 0x1424 */ 0xD1A4,0x4566,
	/* Array index: 0x1424 */ 0xD1A8,0x4567,
	/* Array index: 0x1424 */ 0xD1B0,0x4568,
	/* Array index: 0x1424 */ 0xD1B1,0x4569,
	/* Array index: 0x1424 */ 0xD1B3,0x456A,
	/* Array index: 0x1424 */ 0xD1B5,0x456B,
	/* Array index: 0x1424 */ 0xD1BA,0x456C,
	/* Array index: 0x1424 */ 0xD1BC,0x456D,
	/* Array index: 0x1424 */ 0xD1C0,0x456E,
	/* Array index: 0x1424 */ 0xD1D8,0x456F,
	/* Array index: 0x1424 */ 0xD1F4,0x4570,
	/* Array index: 0x1424 */ 0xD1F8,0x4571,
	/* Array index: 0x1424 */ 0xD207,0x4572,
	/* Array index: 0x1424 */ 0xD209,0x4573,
	/* Array index: 0x1424 */ 0xD210,0x4574,
	/* Array index: 0x1424 */ 0xD22C,0x4575,
	/* Array index: 0x1424 */ 0xD22D,0x4576,
	/* Array index: 0x1424 */ 0xD230,0x4577,
	/* Array index: 0x1424 */ 0xD234,0x4578,
	/* Array index: 0x1424 */ 0xD23C,0x4579,
	/* Array index: 0x1424 */ 0xD23D,0x457A,
	/* Array index: 0x1424 */ 0xD23F,0x457B,
	/* Array index: 0x1424 */ 0xD241,0x457C,
	/* Array index: 0x1424 */ 0xD248,0x457D,
	/* Array index: 0x1424 */ 0xD25C,0x457E,
	/* Array index: 0x1424 */ 0xD264,0x4621,
	/* Array index: 0x1424 */ 0xD280,0x4622,
	/* Array index: 0x1424 */ 0xD281,0x4623,
	/* Array index: 0x1424 */ 0xD284,0x4624,
	/* Array index: 0x1424 */ 0xD288,0x4625,
	/* Array index: 0x1424 */ 0xD290,0x4626,
	/* Array index: 0x1424 */ 0xD291,0x4627,
	/* Array index: 0x1424 */ 0xD295,0x4628,
	/* Array index: 0x1424 */ 0xD29C,0x4629,
	/* Array index: 0x1424 */ 0xD2A0,0x462A,
	/* Array index: 0x1424 */ 0xD2A4,0x462B,
	/* Array index: 0x1424 */ 0xD2AC,0x462C,
	/* Array index: 0x1424 */ 0xD2B1,0x462D,
	/* Array index: 0x1424 */ 0xD2B8,0x462E,
	/* Array index: 0x1424 */ 0xD2B9,0x462F,
	/* Array index: 0x1424 */ 0xD2BC,0x4630,
	/* Array index: 0x1424 */ 0xD2BF,0x4631,
	/* Array index: 0x1424 */ 0xD2C0,0x4632,
	/* Array index: 0x1424 */ 0xD2C2,0x4633,
	/* Array index: 0x1424 */ 0xD2C8,0x4634,
	/* Array index: 0x1424 */ 0xD2C9,0x4635,
	/* Array index: 0x1424 */ 0xD2CB,0x4636,
	/* Array index: 0x1424 */ 0xD2D4,0x4637,
	/* Array index: 0x1424 */ 0xD2D8,0x4638,
	/* Array index: 0x1424 */ 0xD2DC,0x4639,
	/* Array index: 0x1424 */ 0xD2E4,0x463A,
	/* Array index: 0x1424 */ 0xD2E5,0x463B,
	/* Array index: 0x1424 */ 0xD2F0,0x463C,
	/* Array index: 0x1424 */ 0xD2F1,0x463D,
	/* Array index: 0x1424 */ 0xD2F4,0x463E,
	/* Array index: 0x1424 */ 0xD2F8,0x463F,
	/* Array index: 0x1424 */ 0xD300,0x4640,
	/* Array index: 0x1424 */ 0xD301,0x4641,
	/* Array index: 0x1424 */ 0xD303,0x4642,
	/* Array index: 0x1424 */ 0xD305,0x4643,
	/* Array index: 0x1424 */ 0xD314,0x4648,
	/* Array index: 0x1424 */ 0xD316,0x4649,
	/* Array index: 0x1424 */ 0xD325,0x464F,
	/* Array index: 0x1424 */ 0xD328,0x4650,
	/* Array index: 0x1424 */ 0xD329,0x4651,
	/* Array index: 0x1424 */ 0xD32C,0x4652,
	/* Array index: 0x1424 */ 0xD330,0x4653,
	/* Array index: 0x1424 */ 0xD344,0x4659,
	/* Array index: 0x1424 */ 0xD345,0x465A,
	/* Array index: 0x1424 */ 0xD37C,0x465B,
	/* Array index: 0x1424 */ 0xD37D,0x465C,
	/* Array index: 0x1424 */ 0xD380,0x465D,
	/* Array index: 0x1424 */ 0xD384,0x465E,
	/* Array index: 0x1424 */ 0xD398,0x4664,
	/* Array index: 0x1424 */ 0xD399,0x4665,
	/* Array index: 0x1424 */ 0xD39C,0x4666,
	/* Array index: 0x1424 */ 0xD3A0,0x4667,
	/* Array index: 0x1424 */ 0xD3A8,0x4668,
	/* Array index: 0x1424 */ 0xD3A9,0x4669,
	/* Array index: 0x1424 */ 0xD3AB,0x466A,
	/* Array index: 0x1424 */ 0xD3AD,0x466B,
	/* Array index: 0x1424 */ 0xD3B4,0x466C,
	/* Array index: 0x1424 */ 0xD3B8,0x466D,
	/* Array index: 0x1424 */ 0xD3BC,0x466E,
	/* Array index: 0x1424 */ 0xD3C4,0x466F,
	/* Array index: 0x1424 */ 0xD3C5,0x4670,
	/* Array index: 0x1424 */ 0xD3C8,0x4671,
	/* Array index: 0x1424 */ 0xD3C9,0x4672,
	/* Array index: 0x1424 */ 0xD3D0,0x4673,
	/* Array index: 0x1424 */ 0xD3D8,0x4674,
	/* Array index: 0x1424 */ 0xD3E1,0x4675,
	/* Array index: 0x1424 */ 0xD3E3,0x4676,
	/* Array index: 0x1424 */ 0xD3EC,0x4677,
	/* Array index: 0x1424 */ 0xD3ED,0x4678,
	/* Array index: 0x1424 */ 0xD3F0,0x4679,
	/* Array index: 0x1424 */ 0xD3F4,0x467A,
	/* Array index: 0x1424 */ 0xD3FC,0x467B,
	/* Array index: 0x1424 */ 0xD3FD,0x467C,
	/* Array index: 0x1424 */ 0xD3FF,0x467D,
	/* Array index: 0x1424 */ 0xD401,0x467E,
	/* Array index: 0x1424 */ 0xD408,0x4721,
	/* Array index: 0x1424 */ 0xD41D,0x4722,
	/* Array index: 0x1424 */ 0xD440,0x4723,
	/* Array index: 0x1424 */ 0xD444,0x4724,
	/* Array index: 0x1424 */ 0xD45C,0x4725,
	/* Array index: 0x1424 */ 0xD460,0x4726,
	/* Array index: 0x1424 */ 0xD464,0x4727,
	/* Array index: 0x1424 */ 0xD46D,0x4728,
	/* Array index: 0x1424 */ 0xD46F,0x4729,
	/* Array index: 0x1424 */ 0xD478,0x472A,
	/* Array index: 0x1424 */ 0xD479,0x472B,
	/* Array index: 0x1424 */ 0xD47C,0x472C,
	/* Array index: 0x1424 */ 0xD47F,0x472D,
	/* Array index: 0x1424 */ 0xD480,0x472E,
	/* Array index: 0x1424 */ 0xD482,0x472F,
	/* Array index: 0x1424 */ 0xD488,0x4730,
	/* Array index: 0x1424 */ 0xD489,0x4731,
	/* Array index: 0x1424 */ 0xD48B,0x4732,
	/* Array index: 0x1424 */ 0xD48D,0x4733,
	/* Array index: 0x1424 */ 0xD494,0x4734,
	/* Array index: 0x1424 */ 0xD4A9,0x4735,
	/* Array index: 0x1424 */ 0xD4CC,0x4736,
	/* Array index: 0x1424 */ 0xD4D0,0x4737,
	/* Array index: 0x1424 */ 0xD4D4,0x4738,
	/* Array index: 0x1424 */ 0xD4DC,0x4739,
	/* Array index: 0x1424 */ 0xD4DF,0x473A,
	/* Array index: 0x1424 */ 0xD4E8,0x473B,
	/* Array index: 0x1424 */ 0xD4EC,0x473C,
	/* Array index: 0x1424 */ 0xD4F0,0x473D,
	/* Array index: 0x1424 */ 0xD4F8,0x473E,
	/* Array index: 0x1424 */ 0xD4FB,0x473F,
	/* Array index: 0x1424 */ 0xD4FD,0x4740,
	/* Array index: 0x1424 */ 0xD504,0x4741,
	/* Array index: 0x1424 */ 0xD508,0x4742,
	/* Array index: 0x1424 */ 0xD50C,0x4743,
	/* Array index: 0x1424 */ 0xD514,0x4744,
	/* Array index: 0x1424 */ 0xD515,0x4745,
	/* Array index: 0x1424 */ 0xD517,0x4746,
	/* Array index: 0x1424 */ 0xD53C,0x4747,
	/* Array index: 0x1424 */ 0xD53D,0x4748,
	/* Array index: 0x1424 */ 0xD540,0x4749,
	/* Array index: 0x1424 */ 0xD544,0x474A,
	/* Array index: 0x1424 */ 0xD54C,0x474B,
	/* Array index: 0x1424 */ 0xD54D,0x474C,
	/* Array index: 0x1424 */ 0xD54F,0x474D,
	/* Array index: 0x1424 */ 0xD551,0x474E,
	/* Array index: 0x1424 */ 0xD558,0x474F,
	/* Array index: 0x1424 */ 0xD559,0x4750,
	/* Array index: 0x1424 */ 0xD55C,0x4751,
	/* Array index: 0x1424 */ 0xD560,0x4752,
	/* Array index: 0x1424 */ 0xD565,0x4753,
	/* Array index: 0x1424 */ 0xD568,0x4754,
	/* Array index: 0x1424 */ 0xD569,0x4755,
	/* Array index: 0x1424 */ 0xD56B,0x4756,
	/* Array index: 0x1424 */ 0xD56D,0x4757,
	/* Array index: 0x1424 */ 0xD574,0x4758,
	/* Array index: 0x1424 */ 0xD575,0x4759,
	/* Array index: 0x1424 */ 0xD578,0x475A,
	/* Array index: 0x1424 */ 0xD57C,0x475B,
	/* Array index: 0x1424 */ 0xD590,0x4761,
	/* Array index: 0x1424 */ 0xD5A5,0x4762,
	/* Array index: 0x1424 */ 0xD5C8,0x4763,
	/* Array index: 0x1424 */ 0xD5C9,0x4764,
	/* Array index: 0x1424 */ 0xD5CC,0x4765,
	/* Array index: 0x1424 */ 0xD5D0,0x4766,
	/* Array index: 0x1424 */ 0xD5D2,0x4767,
	/* Array index: 0x1424 */ 0xD5D8,0x4768,
	/* Array index: 0x1424 */ 0xD5D9,0x4769,
	/* Array index: 0x1424 */ 0xD5DB,0x476A,
	/* Array index: 0x1424 */ 0xD5DD,0x476B,
	/* Array index: 0x1424 */ 0xD5E4,0x476C,
	/* Array index: 0x1424 */ 0xD5E5,0x476D,
	/* Array index: 0x1424 */ 0xD5E8,0x476E,
	/* Array index: 0x1424 */ 0xD5EC,0x476F,
	/* Array index: 0x1424 */ 0xD5F4,0x4770,
	/* Array index: 0x1424 */ 0xD5F5,0x4771,
	/* Array index: 0x1424 */ 0xD5F7,0x4772,
	/* Array index: 0x1424 */ 0xD5F9,0x4773,
	/* Array index: 0x1424 */ 0xD600,0x4774,
	/* Array index: 0x1424 */ 0xD601,0x4775,
	/* Array index: 0x1424 */ 0xD604,0x4776,
	/* Array index: 0x1424 */ 0xD608,0x4777,
	/* Array index: 0x1424 */ 0xD61C,0x477D,
	/* Array index: 0x1424 */ 0xD620,0x477E,
	/* Array index: 0x1424 */ 0xD624,0x4821,
	/* Array index: 0x1424 */ 0xD62D,0x4822,
	/* Array index: 0x1424 */ 0xD638,0x4823,
	/* Array index: 0x1424 */ 0xD639,0x4824,
	/* Array index: 0x1424 */ 0xD63C,0x4825,
	/* Array index: 0x1424 */ 0xD640,0x4826,
	/* Array index: 0x1424 */ 0xD645,0x4827,
	/* Array index: 0x1424 */ 0xD648,0x4828,
	/* Array index: 0x1424 */ 0xD649,0x4829,
	/* Array index: 0x1424 */ 0xD64B,0x482A,
	/* Array index: 0x1424 */ 0xD64D,0x482B,
	/* Array index: 0x1424 */ 0xD651,0x482C,
	/* Array index: 0x1424 */ 0xD654,0x482D,
	/* Array index: 0x1424 */ 0xD655,0x482E,
	/* Array index: 0x1424 */ 0xD658,0x482F,
	/* Array index: 0x1424 */ 0xD65C,0x4830,
	/* Array index: 0x1424 */ 0xD667,0x4831,
	/* Array index: 0x1424 */ 0xD669,0x4832,
	/* Array index: 0x1424 */ 0xD670,0x4833,
	/* Array index: 0x1424 */ 0xD671,0x4834,
	/* Array index: 0x1424 */ 0xD674,0x4835,
	/* Array index: 0x1424 */ 0xD683,0x4836,
	/* Array index: 0x1424 */ 0xD685,0x4837,
	/* Array index: 0x1424 */ 0xD68C,0x4838,
	/* Array index: 0x1424 */ 0xD68D,0x4839,
	/* Array index: 0x1424 */ 0xD690,0x483A,
	/* Array index: 0x1424 */ 0xD694,0x483B,
	/* Array index: 0x1424 */ 0xD69D,0x483C,
	/* Array index: 0x1424 */ 0xD69F,0x483D,
	/* Array index: 0x1424 */ 0xD6A1,0x483E,
	/* Array index: 0x1424 */ 0xD6A8,0x483F,
	/* Array index: 0x1424 */ 0xD6AC,0x4840,
	/* Array index: 0x1424 */ 0xD6B0,0x4841,
	/* Array index: 0x1424 */ 0xD6B9,0x4842,
	/* Array index: 0x1424 */ 0xD6BB,0x4843,
	/* Array index: 0x1424 */ 0xD6C4,0x4844,
	/* Array index: 0x1424 */ 0xD6C5,0x4845,
	/* Array index: 0x1424 */ 0xD6C8,0x4846,
	/* Array index: 0x1424 */ 0xD6CC,0x4847,
	/* Array index: 0x1424 */ 0xD6D1,0x4848,
	/* Array index: 0x1424 */ 0xD6D4,0x4849,
	/* Array index: 0x1424 */ 0xD6D7,0x484A,
	/* Array index: 0x1424 */ 0xD6D9,0x484B,
	/* Array index: 0x1424 */ 0xD6E0,0x484C,
	/* Array index: 0x1424 */ 0xD6E4,0x484D,
	/* Array index: 0x1424 */ 0xD6E8,0x484E,
	/* Array index: 0x1424 */ 0xD6F0,0x484F,
	/* Array index: 0x1424 */ 0xD6F5,0x4850,
	/* Array index: 0x1424 */ 0xD6FC,0x4851,
	/* Array index: 0x1424 */ 0xD6FD,0x4852,
	/* Array index: 0x1424 */ 0xD700,0x4853,
	/* Array index: 0x1424 */ 0xD704,0x4854,
	/* Array index: 0x1424 */ 0xD711,0x4855,
	/* Array index: 0x1424 */ 0xD718,0x4856,
	/* Array index: 0x1424 */ 0xD719,0x4857,
	/* Array index: 0x1424 */ 0xD71C,0x4858,
	/* Array index: 0x1424 */ 0xD720,0x4859,
	/* Array index: 0x1424 */ 0xD728,0x485A,
	/* Array index: 0x1424 */ 0xD729,0x485B,
	/* Array index: 0x1424 */ 0xD72B,0x485C,
	/* Array index: 0x1424 */ 0xD72D,0x485D,
	/* Array index: 0x1424 */ 0xD734,0x485E,
	/* Array index: 0x1424 */ 0xD735,0x485F,
	/* Array index: 0x1424 */ 0xD738,0x4860,
	/* Array index: 0x1424 */ 0xD73C,0x4861,
	/* Array index: 0x1424 */ 0xD744,0x4862,
	/* Array index: 0x1424 */ 0xD747,0x4863,
	/* Array index: 0x1424 */ 0xD749,0x4864,
	/* Array index: 0x1424 */ 0xD750,0x4865,
	/* Array index: 0x1424 */ 0xD751,0x4866,
	/* Array index: 0x1424 */ 0xD760,0x486C,
	/* Array index: 0x1424 */ 0xD761,0x486D,
	/* Array index: 0x1424 */ 0xD763,0x486E,
	/* Array index: 0x1424 */ 0xD765,0x486F,
	/* Array index: 0x1424 */ 0xD769,0x4870,
	/* Array index: 0x1424 */ 0xD76C,0x4871,
	/* Array index: 0x1424 */ 0xD770,0x4872,
	/* Array index: 0x1424 */ 0xD774,0x4873,
	/* Array index: 0x1424 */ 0xD77C,0x4874,
	/* Array index: 0x1424 */ 0xD77D,0x4875,
	/* Array index: 0x1424 */ 0xD781,0x4876,
	/* Array index: 0x1424 */ 0xD788,0x4877,
	/* Array index: 0x1424 */ 0xD789,0x4878,
	/* Array index: 0x1424 */ 0xD78C,0x4879,
	/* Array index: 0x1424 */ 0xD790,0x487A,
	/* Array index: 0x1424 */ 0xD798,0x487B,
	/* Array index: 0x1424 */ 0xD799,0x487C,
	/* Array index: 0x1424 */ 0xD79B,0x487D,
	/* Array index: 0x1424 */ 0xD79D,0x487E,
};

#endif /* ICONV_FROM_UCS_CCS_KSX1001 && defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * ksx1001 CCS description table.
 * ======================================================================
 */
const iconv_ccs_t
_iconv_ccs_ksx1001 =
{
	TABLE_VERSION_1, /* Table version */
	ICONV_CCS_KSX1001, /* CCS name */
	TABLE_16BIT, /* Table bits */
#if defined (ICONV_FROM_UCS_CCS_KSX1001) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SIZE_OPTIMIZED,
	(__uint16_t *)&from_ucs_size_ksx1001, /* UCS -> ksx1001 table size-optimized table */
#elif defined (ICONV_FROM_UCS_CCS_KSX1001) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)&from_ucs_speed_ksx1001, /* UCS -> ksx1001 table speed-optimized table */
#else
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)NULL,
#endif
#if defined (ICONV_TO_UCS_CCS_KSX1001) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SIZE_OPTIMIZED,
	(__uint16_t *)&to_ucs_size_ksx1001 /* ksx1001 -> UCS table speed-optimized table */
#elif defined (ICONV_TO_UCS_CCS_KSX1001) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)&to_ucs_speed_ksx1001 /* ksx1001 -> UCS table speed-optimized table */
#else
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)NULL,
#endif
};

#endif /* ICONV_TO_UCS_CCS_KSX1001) || ... */

