/*
 * This file was generated automatically - don't edit it.
 * File contains iconv CCS tables for jis_x0208_1990 encoding.
 */

#include "ccsbi.h"

#if defined (ICONV_TO_UCS_CCS_JIS_X0208_1990) \
 || defined (ICONV_FROM_UCS_CCS_JIS_X0208_1990)

#include <_ansi.h>
#include <sys/types.h>
#include <sys/param.h>
#include "ccs.h"
#include "ccsnames.h"

/*
 * 16-bit jis_x0208_1990 -> UCS speed-optimized table (39936 bytes).
 * ======================================================================
 */
#if defined (ICONV_TO_UCS_CCS_JIS_X0208_1990) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
to_ucs_speed_jis_x0208_1990[] =
{
	/* Heading Block */
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,0x0100,0x0200,0x0300,0x0400,0x0500,0x0600,0x0700,
	0x0800,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	0x0900,0x0A00,0x0B00,0x0C00,0x0D00,0x0E00,0x0F00,0x1000,
	0x1100,0x1200,0x1300,0x1400,0x1500,0x1600,0x1700,0x1800,
	0x1900,0x1A00,0x1B00,0x1C00,0x1D00,0x1E00,0x1F00,0x2000,
	0x2100,0x2200,0x2300,0x2400,0x2500,0x2600,0x2700,0x2800,
	0x2900,0x2A00,0x2B00,0x2C00,0x2D00,0x2E00,0x2F00,0x3000,
	0x3100,0x3200,0x3300,0x3400,0x3500,0x3600,0x3700,0x3800,
	0x3900,0x3A00,0x3B00,0x3C00,0x3D00,0x3E00,0x3F00,0x4000,
	0x4100,0x4200,0x4300,0x4400,0x4500,0x4600,0x4700,0x4800,
	0x4900,0x4A00,0x4B00,0x4C00,0x4D00,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	/* Block 34, Array index 0x0100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3000,0x3001,0x3002,0xFF0C,0xFF0E,0x30FB,0xFF1A,
	0xFF1B,0xFF1F,0xFF01,0x309B,0x309C,0x00B4,0xFF40,0x00A8,
	0xFF3E,0xFFE3,0xFF3F,0x30FD,0x30FE,0x309D,0x309E,0x3003,
	0x4EDD,0x3005,0x3006,0x3007,0x30FC,0x2015,0x2010,0xFF0F,
	0x005C,0x301C,0x2016,0xFF5C,0x2026,0x2025,0x2018,0x2019,
	0x201C,0x201D,0xFF08,0xFF09,0x3014,0x3015,0xFF3B,0xFF3D,
	0xFF5B,0xFF5D,0x3008,0x3009,0x300A,0x300B,0x300C,0x300D,
	0x300E,0x300F,0x3010,0x3011,0xFF0B,0x2212,0x00B1,0x00D7,
	0x00F7,0xFF1D,0x2260,0xFF1C,0xFF1E,0x2266,0x2267,0x221E,
	0x2234,0x2642,0x2640,0x00B0,0x2032,0x2033,0x2103,0xFFE5,
	0xFF04,0x00A2,0x00A3,0xFF05,0xFF03,0xFF06,0xFF0A,0xFF20,
	0x00A7,0x2606,0x2605,0x25CB,0x25CF,0x25CE,0x25C7,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 35, Array index 0x0200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x25C6,0x25A1,0x25A0,0x25B3,0x25B2,0x25BD,0x25BC,
	0x203B,0x3012,0x2192,0x2190,0x2191,0x2193,0x3013,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2208,0x220B,0x2286,0x2287,0x2282,0x2283,
	0x222A,0x2229,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2227,0x2228,0x00AC,0x21D2,0x21D4,0x2200,
	0x2203,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2220,0x22A5,0x2312,0x2202,
	0x2207,0x2261,0x2252,0x226A,0x226B,0x221A,0x223D,0x221D,
	0x2235,0x222B,0x222C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x212B,0x2030,0x266F,0x266D,0x266A,0x2020,
	0x2021,0x00B6,INVALC,INVALC,INVALC,INVALC,0x25EF,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 36, Array index 0x0300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0xFF10,0xFF11,0xFF12,0xFF13,0xFF14,0xFF15,0xFF16,0xFF17,
	0xFF18,0xFF19,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xFF21,0xFF22,0xFF23,0xFF24,0xFF25,0xFF26,0xFF27,
	0xFF28,0xFF29,0xFF2A,0xFF2B,0xFF2C,0xFF2D,0xFF2E,0xFF2F,
	0xFF30,0xFF31,0xFF32,0xFF33,0xFF34,0xFF35,0xFF36,0xFF37,
	0xFF38,0xFF39,0xFF3A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0xFF41,0xFF42,0xFF43,0xFF44,0xFF45,0xFF46,0xFF47,
	0xFF48,0xFF49,0xFF4A,0xFF4B,0xFF4C,0xFF4D,0xFF4E,0xFF4F,
	0xFF50,0xFF51,0xFF52,0xFF53,0xFF54,0xFF55,0xFF56,0xFF57,
	0xFF58,0xFF59,0xFF5A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 37, Array index 0x0400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3041,0x3042,0x3043,0x3044,0x3045,0x3046,0x3047,
	0x3048,0x3049,0x304A,0x304B,0x304C,0x304D,0x304E,0x304F,
	0x3050,0x3051,0x3052,0x3053,0x3054,0x3055,0x3056,0x3057,
	0x3058,0x3059,0x305A,0x305B,0x305C,0x305D,0x305E,0x305F,
	0x3060,0x3061,0x3062,0x3063,0x3064,0x3065,0x3066,0x3067,
	0x3068,0x3069,0x306A,0x306B,0x306C,0x306D,0x306E,0x306F,
	0x3070,0x3071,0x3072,0x3073,0x3074,0x3075,0x3076,0x3077,
	0x3078,0x3079,0x307A,0x307B,0x307C,0x307D,0x307E,0x307F,
	0x3080,0x3081,0x3082,0x3083,0x3084,0x3085,0x3086,0x3087,
	0x3088,0x3089,0x308A,0x308B,0x308C,0x308D,0x308E,0x308F,
	0x3090,0x3091,0x3092,0x3093,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 38, Array index 0x0500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x30A1,0x30A2,0x30A3,0x30A4,0x30A5,0x30A6,0x30A7,
	0x30A8,0x30A9,0x30AA,0x30AB,0x30AC,0x30AD,0x30AE,0x30AF,
	0x30B0,0x30B1,0x30B2,0x30B3,0x30B4,0x30B5,0x30B6,0x30B7,
	0x30B8,0x30B9,0x30BA,0x30BB,0x30BC,0x30BD,0x30BE,0x30BF,
	0x30C0,0x30C1,0x30C2,0x30C3,0x30C4,0x30C5,0x30C6,0x30C7,
	0x30C8,0x30C9,0x30CA,0x30CB,0x30CC,0x30CD,0x30CE,0x30CF,
	0x30D0,0x30D1,0x30D2,0x30D3,0x30D4,0x30D5,0x30D6,0x30D7,
	0x30D8,0x30D9,0x30DA,0x30DB,0x30DC,0x30DD,0x30DE,0x30DF,
	0x30E0,0x30E1,0x30E2,0x30E3,0x30E4,0x30E5,0x30E6,0x30E7,
	0x30E8,0x30E9,0x30EA,0x30EB,0x30EC,0x30ED,0x30EE,0x30EF,
	0x30F0,0x30F1,0x30F2,0x30F3,0x30F4,0x30F5,0x30F6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 39, Array index 0x0600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x0391,0x0392,0x0393,0x0394,0x0395,0x0396,0x0397,
	0x0398,0x0399,0x039A,0x039B,0x039C,0x039D,0x039E,0x039F,
	0x03A0,0x03A1,0x03A3,0x03A4,0x03A5,0x03A6,0x03A7,0x03A8,
	0x03A9,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x03B1,0x03B2,0x03B3,0x03B4,0x03B5,0x03B6,0x03B7,
	0x03B8,0x03B9,0x03BA,0x03BB,0x03BC,0x03BD,0x03BE,0x03BF,
	0x03C0,0x03C1,0x03C3,0x03C4,0x03C5,0x03C6,0x03C7,0x03C8,
	0x03C9,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 40, Array index 0x0700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x0410,0x0411,0x0412,0x0413,0x0414,0x0415,0x0401,
	0x0416,0x0417,0x0418,0x0419,0x041A,0x041B,0x041C,0x041D,
	0x041E,0x041F,0x0420,0x0421,0x0422,0x0423,0x0424,0x0425,
	0x0426,0x0427,0x0428,0x0429,0x042A,0x042B,0x042C,0x042D,
	0x042E,0x042F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x0430,0x0431,0x0432,0x0433,0x0434,0x0435,0x0451,
	0x0436,0x0437,0x0438,0x0439,0x043A,0x043B,0x043C,0x043D,
	0x043E,0x043F,0x0440,0x0441,0x0442,0x0443,0x0444,0x0445,
	0x0446,0x0447,0x0448,0x0449,0x044A,0x044B,0x044C,0x044D,
	0x044E,0x044F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 41, Array index 0x0800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2500,0x2502,0x250C,0x2510,0x2518,0x2514,0x251C,
	0x252C,0x2524,0x2534,0x253C,0x2501,0x2503,0x250F,0x2513,
	0x251B,0x2517,0x2523,0x2533,0x252B,0x253B,0x254B,0x2520,
	0x252F,0x2528,0x2537,0x253F,0x251D,0x2530,0x2525,0x2538,
	0x2542,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 49, Array index 0x0900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E9C,0x5516,0x5A03,0x963F,0x54C0,0x611B,0x6328,
	0x59F6,0x9022,0x8475,0x831C,0x7A50,0x60AA,0x63E1,0x6E25,
	0x65ED,0x8466,0x82A6,0x9BF5,0x6893,0x5727,0x65A1,0x6271,
	0x5B9B,0x59D0,0x867B,0x98F4,0x7D62,0x7DBE,0x9B8E,0x6216,
	0x7C9F,0x88B7,0x5B89,0x5EB5,0x6309,0x6697,0x6848,0x95C7,
	0x978D,0x674F,0x4EE5,0x4F0A,0x4F4D,0x4F9D,0x5049,0x56F2,
	0x5937,0x59D4,0x5A01,0x5C09,0x60DF,0x610F,0x6170,0x6613,
	0x6905,0x70BA,0x754F,0x7570,0x79FB,0x7DAD,0x7DEF,0x80C3,
	0x840E,0x8863,0x8B02,0x9055,0x907A,0x533B,0x4E95,0x4EA5,
	0x57DF,0x80B2,0x90C1,0x78EF,0x4E00,0x58F1,0x6EA2,0x9038,
	0x7A32,0x8328,0x828B,0x9C2F,0x5141,0x5370,0x54BD,0x54E1,
	0x56E0,0x59FB,0x5F15,0x98F2,0x6DEB,0x80E4,0x852D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 50, Array index 0x0A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9662,0x9670,0x96A0,0x97FB,0x540B,0x53F3,0x5B87,
	0x70CF,0x7FBD,0x8FC2,0x96E8,0x536F,0x9D5C,0x7ABA,0x4E11,
	0x7893,0x81FC,0x6E26,0x5618,0x5504,0x6B1D,0x851A,0x9C3B,
	0x59E5,0x53A9,0x6D66,0x74DC,0x958F,0x5642,0x4E91,0x904B,
	0x96F2,0x834F,0x990C,0x53E1,0x55B6,0x5B30,0x5F71,0x6620,
	0x66F3,0x6804,0x6C38,0x6CF3,0x6D29,0x745B,0x76C8,0x7A4E,
	0x9834,0x82F1,0x885B,0x8A60,0x92ED,0x6DB2,0x75AB,0x76CA,
	0x99C5,0x60A6,0x8B01,0x8D8A,0x95B2,0x698E,0x53AD,0x5186,
	0x5712,0x5830,0x5944,0x5BB4,0x5EF6,0x6028,0x63A9,0x63F4,
	0x6CBF,0x6F14,0x708E,0x7114,0x7159,0x71D5,0x733F,0x7E01,
	0x8276,0x82D1,0x8597,0x9060,0x925B,0x9D1B,0x5869,0x65BC,
	0x6C5A,0x7525,0x51F9,0x592E,0x5965,0x5F80,0x5FDC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 51, Array index 0x0B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x62BC,0x65FA,0x6A2A,0x6B27,0x6BB4,0x738B,0x7FC1,
	0x8956,0x9D2C,0x9D0E,0x9EC4,0x5CA1,0x6C96,0x837B,0x5104,
	0x5C4B,0x61B6,0x81C6,0x6876,0x7261,0x4E59,0x4FFA,0x5378,
	0x6069,0x6E29,0x7A4F,0x97F3,0x4E0B,0x5316,0x4EEE,0x4F55,
	0x4F3D,0x4FA1,0x4F73,0x52A0,0x53EF,0x5609,0x590F,0x5AC1,
	0x5BB6,0x5BE1,0x79D1,0x6687,0x679C,0x67B6,0x6B4C,0x6CB3,
	0x706B,0x73C2,0x798D,0x79BE,0x7A3C,0x7B87,0x82B1,0x82DB,
	0x8304,0x8377,0x83EF,0x83D3,0x8766,0x8AB2,0x5629,0x8CA8,
	0x8FE6,0x904E,0x971E,0x868A,0x4FC4,0x5CE8,0x6211,0x7259,
	0x753B,0x81E5,0x82BD,0x86FE,0x8CC0,0x96C5,0x9913,0x99D5,
	0x4ECB,0x4F1A,0x89E3,0x56DE,0x584A,0x58CA,0x5EFB,0x5FEB,
	0x602A,0x6094,0x6062,0x61D0,0x6212,0x62D0,0x6539,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 52, Array index 0x0C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9B41,0x6666,0x68B0,0x6D77,0x7070,0x754C,0x7686,
	0x7D75,0x82A5,0x87F9,0x958B,0x968E,0x8C9D,0x51F1,0x52BE,
	0x5916,0x54B3,0x5BB3,0x5D16,0x6168,0x6982,0x6DAF,0x788D,
	0x84CB,0x8857,0x8A72,0x93A7,0x9AB8,0x6D6C,0x99A8,0x86D9,
	0x57A3,0x67FF,0x86CE,0x920E,0x5283,0x5687,0x5404,0x5ED3,
	0x62E1,0x64B9,0x683C,0x6838,0x6BBB,0x7372,0x78BA,0x7A6B,
	0x899A,0x89D2,0x8D6B,0x8F03,0x90ED,0x95A3,0x9694,0x9769,
	0x5B66,0x5CB3,0x697D,0x984D,0x984E,0x639B,0x7B20,0x6A2B,
	0x6A7F,0x68B6,0x9C0D,0x6F5F,0x5272,0x559D,0x6070,0x62EC,
	0x6D3B,0x6E07,0x6ED1,0x845B,0x8910,0x8F44,0x4E14,0x9C39,
	0x53F6,0x691B,0x6A3A,0x9784,0x682A,0x515C,0x7AC3,0x84B2,
	0x91DC,0x938C,0x565B,0x9D28,0x6822,0x8305,0x8431,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 53, Array index 0x0D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7CA5,0x5208,0x82C5,0x74E6,0x4E7E,0x4F83,0x51A0,
	0x5BD2,0x520A,0x52D8,0x52E7,0x5DFB,0x559A,0x582A,0x59E6,
	0x5B8C,0x5B98,0x5BDB,0x5E72,0x5E79,0x60A3,0x611F,0x6163,
	0x61BE,0x63DB,0x6562,0x67D1,0x6853,0x68FA,0x6B3E,0x6B53,
	0x6C57,0x6F22,0x6F97,0x6F45,0x74B0,0x7518,0x76E3,0x770B,
	0x7AFF,0x7BA1,0x7C21,0x7DE9,0x7F36,0x7FF0,0x809D,0x8266,
	0x839E,0x89B3,0x8ACC,0x8CAB,0x9084,0x9451,0x9593,0x9591,
	0x95A2,0x9665,0x97D3,0x9928,0x8218,0x4E38,0x542B,0x5CB8,
	0x5DCC,0x73A9,0x764C,0x773C,0x5CA9,0x7FEB,0x8D0B,0x96C1,
	0x9811,0x9854,0x9858,0x4F01,0x4F0E,0x5371,0x559C,0x5668,
	0x57FA,0x5947,0x5B09,0x5BC4,0x5C90,0x5E0C,0x5E7E,0x5FCC,
	0x63EE,0x673A,0x65D7,0x65E2,0x671F,0x68CB,0x68C4,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 54, Array index 0x0E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A5F,0x5E30,0x6BC5,0x6C17,0x6C7D,0x757F,0x7948,
	0x5B63,0x7A00,0x7D00,0x5FBD,0x898F,0x8A18,0x8CB4,0x8D77,
	0x8ECC,0x8F1D,0x98E2,0x9A0E,0x9B3C,0x4E80,0x507D,0x5100,
	0x5993,0x5B9C,0x622F,0x6280,0x64EC,0x6B3A,0x72A0,0x7591,
	0x7947,0x7FA9,0x87FB,0x8ABC,0x8B70,0x63AC,0x83CA,0x97A0,
	0x5409,0x5403,0x55AB,0x6854,0x6A58,0x8A70,0x7827,0x6775,
	0x9ECD,0x5374,0x5BA2,0x811A,0x8650,0x9006,0x4E18,0x4E45,
	0x4EC7,0x4F11,0x53CA,0x5438,0x5BAE,0x5F13,0x6025,0x6551,
	0x673D,0x6C42,0x6C72,0x6CE3,0x7078,0x7403,0x7A76,0x7AAE,
	0x7B08,0x7D1A,0x7CFE,0x7D66,0x65E7,0x725B,0x53BB,0x5C45,
	0x5DE8,0x62D2,0x62E0,0x6319,0x6E20,0x865A,0x8A31,0x8DDD,
	0x92F8,0x6F01,0x79A6,0x9B5A,0x4EA8,0x4EAB,0x4EAC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 55, Array index 0x0F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F9B,0x4FA0,0x50D1,0x5147,0x7AF6,0x5171,0x51F6,
	0x5354,0x5321,0x537F,0x53EB,0x55AC,0x5883,0x5CE1,0x5F37,
	0x5F4A,0x602F,0x6050,0x606D,0x631F,0x6559,0x6A4B,0x6CC1,
	0x72C2,0x72ED,0x77EF,0x80F8,0x8105,0x8208,0x854E,0x90F7,
	0x93E1,0x97FF,0x9957,0x9A5A,0x4EF0,0x51DD,0x5C2D,0x6681,
	0x696D,0x5C40,0x66F2,0x6975,0x7389,0x6850,0x7C81,0x50C5,
	0x52E4,0x5747,0x5DFE,0x9326,0x65A4,0x6B23,0x6B3D,0x7434,
	0x7981,0x79BD,0x7B4B,0x7DCA,0x82B9,0x83CC,0x887F,0x895F,
	0x8B39,0x8FD1,0x91D1,0x541F,0x9280,0x4E5D,0x5036,0x53E5,
	0x533A,0x72D7,0x7396,0x77E9,0x82E6,0x8EAF,0x99C6,0x99C8,
	0x99D2,0x5177,0x611A,0x865E,0x55B0,0x7A7A,0x5076,0x5BD3,
	0x9047,0x9685,0x4E32,0x6ADB,0x91E7,0x5C51,0x5C48,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 56, Array index 0x1000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6398,0x7A9F,0x6C93,0x9774,0x8F61,0x7AAA,0x718A,
	0x9688,0x7C82,0x6817,0x7E70,0x6851,0x936C,0x52F2,0x541B,
	0x85AB,0x8A13,0x7FA4,0x8ECD,0x90E1,0x5366,0x8888,0x7941,
	0x4FC2,0x50BE,0x5211,0x5144,0x5553,0x572D,0x73EA,0x578B,
	0x5951,0x5F62,0x5F84,0x6075,0x6176,0x6167,0x61A9,0x63B2,
	0x643A,0x656C,0x666F,0x6842,0x6E13,0x7566,0x7A3D,0x7CFB,
	0x7D4C,0x7D99,0x7E4B,0x7F6B,0x830E,0x834A,0x86CD,0x8A08,
	0x8A63,0x8B66,0x8EFD,0x981A,0x9D8F,0x82B8,0x8FCE,0x9BE8,
	0x5287,0x621F,0x6483,0x6FC0,0x9699,0x6841,0x5091,0x6B20,
	0x6C7A,0x6F54,0x7A74,0x7D50,0x8840,0x8A23,0x6708,0x4EF6,
	0x5039,0x5026,0x5065,0x517C,0x5238,0x5263,0x55A7,0x570F,
	0x5805,0x5ACC,0x5EFA,0x61B2,0x61F8,0x62F3,0x6372,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 57, Array index 0x1100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x691C,0x6A29,0x727D,0x72AC,0x732E,0x7814,0x786F,
	0x7D79,0x770C,0x80A9,0x898B,0x8B19,0x8CE2,0x8ED2,0x9063,
	0x9375,0x967A,0x9855,0x9A13,0x9E78,0x5143,0x539F,0x53B3,
	0x5E7B,0x5F26,0x6E1B,0x6E90,0x7384,0x73FE,0x7D43,0x8237,
	0x8A00,0x8AFA,0x9650,0x4E4E,0x500B,0x53E4,0x547C,0x56FA,
	0x59D1,0x5B64,0x5DF1,0x5EAB,0x5F27,0x6238,0x6545,0x67AF,
	0x6E56,0x72D0,0x7CCA,0x88B4,0x80A1,0x80E1,0x83F0,0x864E,
	0x8A87,0x8DE8,0x9237,0x96C7,0x9867,0x9F13,0x4E94,0x4E92,
	0x4F0D,0x5348,0x5449,0x543E,0x5A2F,0x5F8C,0x5FA1,0x609F,
	0x68A7,0x6A8E,0x745A,0x7881,0x8A9E,0x8AA4,0x8B77,0x9190,
	0x4E5E,0x9BC9,0x4EA4,0x4F7C,0x4FAF,0x5019,0x5016,0x5149,
	0x516C,0x529F,0x52B9,0x52FE,0x539A,0x53E3,0x5411,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 58, Array index 0x1200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x540E,0x5589,0x5751,0x57A2,0x597D,0x5B54,0x5B5D,
	0x5B8F,0x5DE5,0x5DE7,0x5DF7,0x5E78,0x5E83,0x5E9A,0x5EB7,
	0x5F18,0x6052,0x614C,0x6297,0x62D8,0x63A7,0x653B,0x6602,
	0x6643,0x66F4,0x676D,0x6821,0x6897,0x69CB,0x6C5F,0x6D2A,
	0x6D69,0x6E2F,0x6E9D,0x7532,0x7687,0x786C,0x7A3F,0x7CE0,
	0x7D05,0x7D18,0x7D5E,0x7DB1,0x8015,0x8003,0x80AF,0x80B1,
	0x8154,0x818F,0x822A,0x8352,0x884C,0x8861,0x8B1B,0x8CA2,
	0x8CFC,0x90CA,0x9175,0x9271,0x783F,0x92FC,0x95A4,0x964D,
	0x9805,0x9999,0x9AD8,0x9D3B,0x525B,0x52AB,0x53F7,0x5408,
	0x58D5,0x62F7,0x6FE0,0x8C6A,0x8F5F,0x9EB9,0x514B,0x523B,
	0x544A,0x56FD,0x7A40,0x9177,0x9D60,0x9ED2,0x7344,0x6F09,
	0x8170,0x7511,0x5FFD,0x60DA,0x9AA8,0x72DB,0x8FBC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 59, Array index 0x1300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B64,0x9803,0x4ECA,0x56F0,0x5764,0x58BE,0x5A5A,
	0x6068,0x61C7,0x660F,0x6606,0x6839,0x68B1,0x6DF7,0x75D5,
	0x7D3A,0x826E,0x9B42,0x4E9B,0x4F50,0x53C9,0x5506,0x5D6F,
	0x5DE6,0x5DEE,0x67FB,0x6C99,0x7473,0x7802,0x8A50,0x9396,
	0x88DF,0x5750,0x5EA7,0x632B,0x50B5,0x50AC,0x518D,0x6700,
	0x54C9,0x585E,0x59BB,0x5BB0,0x5F69,0x624D,0x63A1,0x683D,
	0x6B73,0x6E08,0x707D,0x91C7,0x7280,0x7815,0x7826,0x796D,
	0x658E,0x7D30,0x83DC,0x88C1,0x8F09,0x969B,0x5264,0x5728,
	0x6750,0x7F6A,0x8CA1,0x51B4,0x5742,0x962A,0x583A,0x698A,
	0x80B4,0x54B2,0x5D0E,0x57FC,0x7895,0x9DFA,0x4F5C,0x524A,
	0x548B,0x643E,0x6628,0x6714,0x67F5,0x7A84,0x7B56,0x7D22,
	0x932F,0x685C,0x9BAD,0x7B39,0x5319,0x518A,0x5237,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 60, Array index 0x1400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5BDF,0x62F6,0x64AE,0x64E6,0x672D,0x6BBA,0x85A9,
	0x96D1,0x7690,0x9BD6,0x634C,0x9306,0x9BAB,0x76BF,0x6652,
	0x4E09,0x5098,0x53C2,0x5C71,0x60E8,0x6492,0x6563,0x685F,
	0x71E6,0x73CA,0x7523,0x7B97,0x7E82,0x8695,0x8B83,0x8CDB,
	0x9178,0x9910,0x65AC,0x66AB,0x6B8B,0x4ED5,0x4ED4,0x4F3A,
	0x4F7F,0x523A,0x53F8,0x53F2,0x55E3,0x56DB,0x58EB,0x59CB,
	0x59C9,0x59FF,0x5B50,0x5C4D,0x5E02,0x5E2B,0x5FD7,0x601D,
	0x6307,0x652F,0x5B5C,0x65AF,0x65BD,0x65E8,0x679D,0x6B62,
	0x6B7B,0x6C0F,0x7345,0x7949,0x79C1,0x7CF8,0x7D19,0x7D2B,
	0x80A2,0x8102,0x81F3,0x8996,0x8A5E,0x8A69,0x8A66,0x8A8C,
	0x8AEE,0x8CC7,0x8CDC,0x96CC,0x98FC,0x6B6F,0x4E8B,0x4F3C,
	0x4F8D,0x5150,0x5B57,0x5BFA,0x6148,0x6301,0x6642,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 61, Array index 0x1500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B21,0x6ECB,0x6CBB,0x723E,0x74BD,0x75D4,0x78C1,
	0x793A,0x800C,0x8033,0x81EA,0x8494,0x8F9E,0x6C50,0x9E7F,
	0x5F0F,0x8B58,0x9D2B,0x7AFA,0x8EF8,0x5B8D,0x96EB,0x4E03,
	0x53F1,0x57F7,0x5931,0x5AC9,0x5BA4,0x6089,0x6E7F,0x6F06,
	0x75BE,0x8CEA,0x5B9F,0x8500,0x7BE0,0x5072,0x67F4,0x829D,
	0x5C61,0x854A,0x7E1E,0x820E,0x5199,0x5C04,0x6368,0x8D66,
	0x659C,0x716E,0x793E,0x7D17,0x8005,0x8B1D,0x8ECA,0x906E,
	0x86C7,0x90AA,0x501F,0x52FA,0x5C3A,0x6753,0x707C,0x7235,
	0x914C,0x91C8,0x932B,0x82E5,0x5BC2,0x5F31,0x60F9,0x4E3B,
	0x53D6,0x5B88,0x624B,0x6731,0x6B8A,0x72E9,0x73E0,0x7A2E,
	0x816B,0x8DA3,0x9152,0x9996,0x5112,0x53D7,0x546A,0x5BFF,
	0x6388,0x6A39,0x7DAC,0x9700,0x56DA,0x53CE,0x5468,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 62, Array index 0x1600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5B97,0x5C31,0x5DDE,0x4FEE,0x6101,0x62FE,0x6D32,
	0x79C0,0x79CB,0x7D42,0x7E4D,0x7FD2,0x81ED,0x821F,0x8490,
	0x8846,0x8972,0x8B90,0x8E74,0x8F2F,0x9031,0x914B,0x916C,
	0x96C6,0x919C,0x4EC0,0x4F4F,0x5145,0x5341,0x5F93,0x620E,
	0x67D4,0x6C41,0x6E0B,0x7363,0x7E26,0x91CD,0x9283,0x53D4,
	0x5919,0x5BBF,0x6DD1,0x795D,0x7E2E,0x7C9B,0x587E,0x719F,
	0x51FA,0x8853,0x8FF0,0x4FCA,0x5CFB,0x6625,0x77AC,0x7AE3,
	0x821C,0x99FF,0x51C6,0x5FAA,0x65EC,0x696F,0x6B89,0x6DF3,
	0x6E96,0x6F64,0x76FE,0x7D14,0x5DE1,0x9075,0x9187,0x9806,
	0x51E6,0x521D,0x6240,0x6691,0x66D9,0x6E1A,0x5EB6,0x7DD2,
	0x7F72,0x66F8,0x85AF,0x85F7,0x8AF8,0x52A9,0x53D9,0x5973,
	0x5E8F,0x5F90,0x6055,0x92E4,0x9664,0x50B7,0x511F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 63, Array index 0x1700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x52DD,0x5320,0x5347,0x53EC,0x54E8,0x5546,0x5531,
	0x5617,0x5968,0x59BE,0x5A3C,0x5BB5,0x5C06,0x5C0F,0x5C11,
	0x5C1A,0x5E84,0x5E8A,0x5EE0,0x5F70,0x627F,0x6284,0x62DB,
	0x638C,0x6377,0x6607,0x660C,0x662D,0x6676,0x677E,0x68A2,
	0x6A1F,0x6A35,0x6CBC,0x6D88,0x6E09,0x6E58,0x713C,0x7126,
	0x7167,0x75C7,0x7701,0x785D,0x7901,0x7965,0x79F0,0x7AE0,
	0x7B11,0x7CA7,0x7D39,0x8096,0x83D6,0x848B,0x8549,0x885D,
	0x88F3,0x8A1F,0x8A3C,0x8A54,0x8A73,0x8C61,0x8CDE,0x91A4,
	0x9266,0x937E,0x9418,0x969C,0x9798,0x4E0A,0x4E08,0x4E1E,
	0x4E57,0x5197,0x5270,0x57CE,0x5834,0x58CC,0x5B22,0x5E38,
	0x60C5,0x64FE,0x6761,0x6756,0x6D44,0x72B6,0x7573,0x7A63,
	0x84B8,0x8B72,0x91B8,0x9320,0x5631,0x57F4,0x98FE,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 64, Array index 0x1800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x62ED,0x690D,0x6B96,0x71ED,0x7E54,0x8077,0x8272,
	0x89E6,0x98DF,0x8755,0x8FB1,0x5C3B,0x4F38,0x4FE1,0x4FB5,
	0x5507,0x5A20,0x5BDD,0x5BE9,0x5FC3,0x614E,0x632F,0x65B0,
	0x664B,0x68EE,0x699B,0x6D78,0x6DF1,0x7533,0x75B9,0x771F,
	0x795E,0x79E6,0x7D33,0x81E3,0x82AF,0x85AA,0x89AA,0x8A3A,
	0x8EAB,0x8F9B,0x9032,0x91DD,0x9707,0x4EBA,0x4EC1,0x5203,
	0x5875,0x58EC,0x5C0B,0x751A,0x5C3D,0x814E,0x8A0A,0x8FC5,
	0x9663,0x976D,0x7B25,0x8ACF,0x9808,0x9162,0x56F3,0x53A8,
	0x9017,0x5439,0x5782,0x5E25,0x63A8,0x6C34,0x708A,0x7761,
	0x7C8B,0x7FE0,0x8870,0x9042,0x9154,0x9310,0x9318,0x968F,
	0x745E,0x9AC4,0x5D07,0x5D69,0x6570,0x67A2,0x8DA8,0x96DB,
	0x636E,0x6749,0x6919,0x83C5,0x9817,0x96C0,0x88FE,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 65, Array index 0x1900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F84,0x647A,0x5BF8,0x4E16,0x702C,0x755D,0x662F,
	0x51C4,0x5236,0x52E2,0x59D3,0x5F81,0x6027,0x6210,0x653F,
	0x6574,0x661F,0x6674,0x68F2,0x6816,0x6B63,0x6E05,0x7272,
	0x751F,0x76DB,0x7CBE,0x8056,0x58F0,0x88FD,0x897F,0x8AA0,
	0x8A93,0x8ACB,0x901D,0x9192,0x9752,0x9759,0x6589,0x7A0E,
	0x8106,0x96BB,0x5E2D,0x60DC,0x621A,0x65A5,0x6614,0x6790,
	0x77F3,0x7A4D,0x7C4D,0x7E3E,0x810A,0x8CAC,0x8D64,0x8DE1,
	0x8E5F,0x78A9,0x5207,0x62D9,0x63A5,0x6442,0x6298,0x8A2D,
	0x7A83,0x7BC0,0x8AAC,0x96EA,0x7D76,0x820C,0x8749,0x4ED9,
	0x5148,0x5343,0x5360,0x5BA3,0x5C02,0x5C16,0x5DDD,0x6226,
	0x6247,0x64B0,0x6813,0x6834,0x6CC9,0x6D45,0x6D17,0x67D3,
	0x6F5C,0x714E,0x717D,0x65CB,0x7A7F,0x7BAD,0x7DDA,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 66, Array index 0x1A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7E4A,0x7FA8,0x817A,0x821B,0x8239,0x85A6,0x8A6E,
	0x8CCE,0x8DF5,0x9078,0x9077,0x92AD,0x9291,0x9583,0x9BAE,
	0x524D,0x5584,0x6F38,0x7136,0x5168,0x7985,0x7E55,0x81B3,
	0x7CCE,0x564C,0x5851,0x5CA8,0x63AA,0x66FE,0x66FD,0x695A,
	0x72D9,0x758F,0x758E,0x790E,0x7956,0x79DF,0x7C97,0x7D20,
	0x7D44,0x8607,0x8A34,0x963B,0x9061,0x9F20,0x50E7,0x5275,
	0x53CC,0x53E2,0x5009,0x55AA,0x58EE,0x594F,0x723D,0x5B8B,
	0x5C64,0x531D,0x60E3,0x60F3,0x635C,0x6383,0x633F,0x63BB,
	0x64CD,0x65E9,0x66F9,0x5DE3,0x69CD,0x69FD,0x6F15,0x71E5,
	0x4E89,0x75E9,0x76F8,0x7A93,0x7CDF,0x7DCF,0x7D9C,0x8061,
	0x8349,0x8358,0x846C,0x84BC,0x85FB,0x88C5,0x8D70,0x9001,
	0x906D,0x9397,0x971C,0x9A12,0x50CF,0x5897,0x618E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 67, Array index 0x1B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x81D3,0x8535,0x8D08,0x9020,0x4FC3,0x5074,0x5247,
	0x5373,0x606F,0x6349,0x675F,0x6E2C,0x8DB3,0x901F,0x4FD7,
	0x5C5E,0x8CCA,0x65CF,0x7D9A,0x5352,0x8896,0x5176,0x63C3,
	0x5B58,0x5B6B,0x5C0A,0x640D,0x6751,0x905C,0x4ED6,0x591A,
	0x592A,0x6C70,0x8A51,0x553E,0x5815,0x59A5,0x60F0,0x6253,
	0x67C1,0x8235,0x6955,0x9640,0x99C4,0x9A28,0x4F53,0x5806,
	0x5BFE,0x8010,0x5CB1,0x5E2F,0x5F85,0x6020,0x614B,0x6234,
	0x66FF,0x6CF0,0x6EDE,0x80CE,0x817F,0x82D4,0x888B,0x8CB8,
	0x9000,0x902E,0x968A,0x9EDB,0x9BDB,0x4EE3,0x53F0,0x5927,
	0x7B2C,0x918D,0x984C,0x9DF9,0x6EDD,0x7027,0x5353,0x5544,
	0x5B85,0x6258,0x629E,0x62D3,0x6CA2,0x6FEF,0x7422,0x8A17,
	0x9438,0x6FC1,0x8AFE,0x8338,0x51E7,0x86F8,0x53EA,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 68, Array index 0x1C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x53E9,0x4F46,0x9054,0x8FB0,0x596A,0x8131,0x5DFD,
	0x7AEA,0x8FBF,0x68DA,0x8C37,0x72F8,0x9C48,0x6A3D,0x8AB0,
	0x4E39,0x5358,0x5606,0x5766,0x62C5,0x63A2,0x65E6,0x6B4E,
	0x6DE1,0x6E5B,0x70AD,0x77ED,0x7AEF,0x7BAA,0x7DBB,0x803D,
	0x80C6,0x86CB,0x8A95,0x935B,0x56E3,0x58C7,0x5F3E,0x65AD,
	0x6696,0x6A80,0x6BB5,0x7537,0x8AC7,0x5024,0x77E5,0x5730,
	0x5F1B,0x6065,0x667A,0x6C60,0x75F4,0x7A1A,0x7F6E,0x81F4,
	0x8718,0x9045,0x99B3,0x7BC9,0x755C,0x7AF9,0x7B51,0x84C4,
	0x9010,0x79E9,0x7A92,0x8336,0x5AE1,0x7740,0x4E2D,0x4EF2,
	0x5B99,0x5FE0,0x62BD,0x663C,0x67F1,0x6CE8,0x866B,0x8877,
	0x8A3B,0x914E,0x92F3,0x99D0,0x6A17,0x7026,0x732A,0x82E7,
	0x8457,0x8CAF,0x4E01,0x5146,0x51CB,0x558B,0x5BF5,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 69, Array index 0x1D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5E16,0x5E33,0x5E81,0x5F14,0x5F35,0x5F6B,0x5FB4,
	0x61F2,0x6311,0x66A2,0x671D,0x6F6E,0x7252,0x753A,0x773A,
	0x8074,0x8139,0x8178,0x8776,0x8ABF,0x8ADC,0x8D85,0x8DF3,
	0x929A,0x9577,0x9802,0x9CE5,0x52C5,0x6357,0x76F4,0x6715,
	0x6C88,0x73CD,0x8CC3,0x93AE,0x9673,0x6D25,0x589C,0x690E,
	0x69CC,0x8FFD,0x939A,0x75DB,0x901A,0x585A,0x6802,0x63B4,
	0x69FB,0x4F43,0x6F2C,0x67D8,0x8FBB,0x8526,0x7DB4,0x9354,
	0x693F,0x6F70,0x576A,0x58F7,0x5B2C,0x7D2C,0x722A,0x540A,
	0x91E3,0x9DB4,0x4EAD,0x4F4E,0x505C,0x5075,0x5243,0x8C9E,
	0x5448,0x5824,0x5B9A,0x5E1D,0x5E95,0x5EAD,0x5EF7,0x5F1F,
	0x608C,0x62B5,0x633A,0x63D0,0x68AF,0x6C40,0x7887,0x798E,
	0x7A0B,0x7DE0,0x8247,0x8A02,0x8AE6,0x8E44,0x9013,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 70, Array index 0x1E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x90B8,0x912D,0x91D8,0x9F0E,0x6CE5,0x6458,0x64E2,
	0x6575,0x6EF4,0x7684,0x7B1B,0x9069,0x93D1,0x6EBA,0x54F2,
	0x5FB9,0x64A4,0x8F4D,0x8FED,0x9244,0x5178,0x586B,0x5929,
	0x5C55,0x5E97,0x6DFB,0x7E8F,0x751C,0x8CBC,0x8EE2,0x985B,
	0x70B9,0x4F1D,0x6BBF,0x6FB1,0x7530,0x96FB,0x514E,0x5410,
	0x5835,0x5857,0x59AC,0x5C60,0x5F92,0x6597,0x675C,0x6E21,
	0x767B,0x83DF,0x8CED,0x9014,0x90FD,0x934D,0x7825,0x783A,
	0x52AA,0x5EA6,0x571F,0x5974,0x6012,0x5012,0x515A,0x51AC,
	0x51CD,0x5200,0x5510,0x5854,0x5858,0x5957,0x5B95,0x5CF6,
	0x5D8B,0x60BC,0x6295,0x642D,0x6771,0x6843,0x68BC,0x68DF,
	0x76D7,0x6DD8,0x6E6F,0x6D9B,0x706F,0x71C8,0x5F53,0x75D8,
	0x7977,0x7B49,0x7B54,0x7B52,0x7CD6,0x7D71,0x5230,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 71, Array index 0x1F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8463,0x8569,0x85E4,0x8A0E,0x8B04,0x8C46,0x8E0F,
	0x9003,0x900F,0x9419,0x9676,0x982D,0x9A30,0x95D8,0x50CD,
	0x52D5,0x540C,0x5802,0x5C0E,0x61A7,0x649E,0x6D1E,0x77B3,
	0x7AE5,0x80F4,0x8404,0x9053,0x9285,0x5CE0,0x9D07,0x533F,
	0x5F97,0x5FB3,0x6D9C,0x7279,0x7763,0x79BF,0x7BE4,0x6BD2,
	0x72EC,0x8AAD,0x6803,0x6A61,0x51F8,0x7A81,0x6934,0x5C4A,
	0x9CF6,0x82EB,0x5BC5,0x9149,0x701E,0x5678,0x5C6F,0x60C7,
	0x6566,0x6C8C,0x8C5A,0x9041,0x9813,0x5451,0x66C7,0x920D,
	0x5948,0x90A3,0x5185,0x4E4D,0x51EA,0x8599,0x8B0E,0x7058,
	0x637A,0x934B,0x6962,0x99B4,0x7E04,0x7577,0x5357,0x6960,
	0x8EDF,0x96E3,0x6C5D,0x4E8C,0x5C3C,0x5F10,0x8FE9,0x5302,
	0x8CD1,0x8089,0x8679,0x5EFF,0x65E5,0x4E73,0x5165,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 72, Array index 0x2000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5982,0x5C3F,0x97EE,0x4EFB,0x598A,0x5FCD,0x8A8D,
	0x6FE1,0x79B0,0x7962,0x5BE7,0x8471,0x732B,0x71B1,0x5E74,
	0x5FF5,0x637B,0x649A,0x71C3,0x7C98,0x4E43,0x5EFC,0x4E4B,
	0x57DC,0x56A2,0x60A9,0x6FC3,0x7D0D,0x80FD,0x8133,0x81BF,
	0x8FB2,0x8997,0x86A4,0x5DF4,0x628A,0x64AD,0x8987,0x6777,
	0x6CE2,0x6D3E,0x7436,0x7834,0x5A46,0x7F75,0x82AD,0x99AC,
	0x4FF3,0x5EC3,0x62DD,0x6392,0x6557,0x676F,0x76C3,0x724C,
	0x80CC,0x80BA,0x8F29,0x914D,0x500D,0x57F9,0x5A92,0x6885,
	0x6973,0x7164,0x72FD,0x8CB7,0x58F2,0x8CE0,0x966A,0x9019,
	0x877F,0x79E4,0x77E7,0x8429,0x4F2F,0x5265,0x535A,0x62CD,
	0x67CF,0x6CCA,0x767D,0x7B94,0x7C95,0x8236,0x8584,0x8FEB,
	0x66DD,0x6F20,0x7206,0x7E1B,0x83AB,0x99C1,0x9EA6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 73, Array index 0x2100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x51FD,0x7BB1,0x7872,0x7BB8,0x8087,0x7B48,0x6AE8,
	0x5E61,0x808C,0x7551,0x7560,0x516B,0x9262,0x6E8C,0x767A,
	0x9197,0x9AEA,0x4F10,0x7F70,0x629C,0x7B4F,0x95A5,0x9CE9,
	0x567A,0x5859,0x86E4,0x96BC,0x4F34,0x5224,0x534A,0x53CD,
	0x53DB,0x5E06,0x642C,0x6591,0x677F,0x6C3E,0x6C4E,0x7248,
	0x72AF,0x73ED,0x7554,0x7E41,0x822C,0x85E9,0x8CA9,0x7BC4,
	0x91C6,0x7169,0x9812,0x98EF,0x633D,0x6669,0x756A,0x76E4,
	0x78D0,0x8543,0x86EE,0x532A,0x5351,0x5426,0x5983,0x5E87,
	0x5F7C,0x60B2,0x6249,0x6279,0x62AB,0x6590,0x6BD4,0x6CCC,
	0x75B2,0x76AE,0x7891,0x79D8,0x7DCB,0x7F77,0x80A5,0x88AB,
	0x8AB9,0x8CBB,0x907F,0x975E,0x98DB,0x6A0B,0x7C38,0x5099,
	0x5C3E,0x5FAE,0x6787,0x6BD8,0x7435,0x7709,0x7F8E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 74, Array index 0x2200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9F3B,0x67CA,0x7A17,0x5339,0x758B,0x9AED,0x5F66,
	0x819D,0x83F1,0x8098,0x5F3C,0x5FC5,0x7562,0x7B46,0x903C,
	0x6867,0x59EB,0x5A9B,0x7D10,0x767E,0x8B2C,0x4FF5,0x5F6A,
	0x6A19,0x6C37,0x6F02,0x74E2,0x7968,0x8868,0x8A55,0x8C79,
	0x5EDF,0x63CF,0x75C5,0x79D2,0x82D7,0x9328,0x92F2,0x849C,
	0x86ED,0x9C2D,0x54C1,0x5F6C,0x658C,0x6D5C,0x7015,0x8CA7,
	0x8CD3,0x983B,0x654F,0x74F6,0x4E0D,0x4ED8,0x57E0,0x592B,
	0x5A66,0x5BCC,0x51A8,0x5E03,0x5E9C,0x6016,0x6276,0x6577,
	0x65A7,0x666E,0x6D6E,0x7236,0x7B26,0x8150,0x819A,0x8299,
	0x8B5C,0x8CA0,0x8CE6,0x8D74,0x961C,0x9644,0x4FAE,0x64AB,
	0x6B66,0x821E,0x8461,0x856A,0x90E8,0x5C01,0x6953,0x98A8,
	0x847A,0x8557,0x4F0F,0x526F,0x5FA9,0x5E45,0x670D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 75, Array index 0x2300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x798F,0x8179,0x8907,0x8986,0x6DF5,0x5F17,0x6255,
	0x6CB8,0x4ECF,0x7269,0x9B92,0x5206,0x543B,0x5674,0x58B3,
	0x61A4,0x626E,0x711A,0x596E,0x7C89,0x7CDE,0x7D1B,0x96F0,
	0x6587,0x805E,0x4E19,0x4F75,0x5175,0x5840,0x5E63,0x5E73,
	0x5F0A,0x67C4,0x4E26,0x853D,0x9589,0x965B,0x7C73,0x9801,
	0x50FB,0x58C1,0x7656,0x78A7,0x5225,0x77A5,0x8511,0x7B86,
	0x504F,0x5909,0x7247,0x7BC7,0x7DE8,0x8FBA,0x8FD4,0x904D,
	0x4FBF,0x52C9,0x5A29,0x5F01,0x97AD,0x4FDD,0x8217,0x92EA,
	0x5703,0x6355,0x6B69,0x752B,0x88DC,0x8F14,0x7A42,0x52DF,
	0x5893,0x6155,0x620A,0x66AE,0x6BCD,0x7C3F,0x83E9,0x5023,
	0x4FF8,0x5305,0x5446,0x5831,0x5949,0x5B9D,0x5CF0,0x5CEF,
	0x5D29,0x5E96,0x62B1,0x6367,0x653E,0x65B9,0x670B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 76, Array index 0x2400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6CD5,0x6CE1,0x70F9,0x7832,0x7E2B,0x80DE,0x82B3,
	0x840C,0x84EC,0x8702,0x8912,0x8A2A,0x8C4A,0x90A6,0x92D2,
	0x98FD,0x9CF3,0x9D6C,0x4E4F,0x4EA1,0x508D,0x5256,0x574A,
	0x59A8,0x5E3D,0x5FD8,0x5FD9,0x623F,0x66B4,0x671B,0x67D0,
	0x68D2,0x5192,0x7D21,0x80AA,0x81A8,0x8B00,0x8C8C,0x8CBF,
	0x927E,0x9632,0x5420,0x982C,0x5317,0x50D5,0x535C,0x58A8,
	0x64B2,0x6734,0x7267,0x7766,0x7A46,0x91E6,0x52C3,0x6CA1,
	0x6B86,0x5800,0x5E4C,0x5954,0x672C,0x7FFB,0x51E1,0x76C6,
	0x6469,0x78E8,0x9B54,0x9EBB,0x57CB,0x59B9,0x6627,0x679A,
	0x6BCE,0x54E9,0x69D9,0x5E55,0x819C,0x6795,0x9BAA,0x67FE,
	0x9C52,0x685D,0x4EA6,0x4FE3,0x53C8,0x62B9,0x672B,0x6CAB,
	0x8FC4,0x4FAD,0x7E6D,0x9EBF,0x4E07,0x6162,0x6E80,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 77, Array index 0x2500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F2B,0x8513,0x5473,0x672A,0x9B45,0x5DF3,0x7B95,
	0x5CAC,0x5BC6,0x871C,0x6E4A,0x84D1,0x7A14,0x8108,0x5999,
	0x7C8D,0x6C11,0x7720,0x52D9,0x5922,0x7121,0x725F,0x77DB,
	0x9727,0x9D61,0x690B,0x5A7F,0x5A18,0x51A5,0x540D,0x547D,
	0x660E,0x76DF,0x8FF7,0x9298,0x9CF4,0x59EA,0x725D,0x6EC5,
	0x514D,0x68C9,0x7DBF,0x7DEC,0x9762,0x9EBA,0x6478,0x6A21,
	0x8302,0x5984,0x5B5F,0x6BDB,0x731B,0x76F2,0x7DB2,0x8017,
	0x8499,0x5132,0x6728,0x9ED9,0x76EE,0x6762,0x52FF,0x9905,
	0x5C24,0x623B,0x7C7E,0x8CB0,0x554F,0x60B6,0x7D0B,0x9580,
	0x5301,0x4E5F,0x51B6,0x591C,0x723A,0x8036,0x91CE,0x5F25,
	0x77E2,0x5384,0x5F79,0x7D04,0x85AC,0x8A33,0x8E8D,0x9756,
	0x67F3,0x85AE,0x9453,0x6109,0x6108,0x6CB9,0x7652,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 78, Array index 0x2600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8AED,0x8F38,0x552F,0x4F51,0x512A,0x52C7,0x53CB,
	0x5BA5,0x5E7D,0x60A0,0x6182,0x63D6,0x6709,0x67DA,0x6E67,
	0x6D8C,0x7336,0x7337,0x7531,0x7950,0x88D5,0x8A98,0x904A,
	0x9091,0x90F5,0x96C4,0x878D,0x5915,0x4E88,0x4F59,0x4E0E,
	0x8A89,0x8F3F,0x9810,0x50AD,0x5E7C,0x5996,0x5BB9,0x5EB8,
	0x63DA,0x63FA,0x64C1,0x66DC,0x694A,0x69D8,0x6D0B,0x6EB6,
	0x7194,0x7528,0x7AAF,0x7F8A,0x8000,0x8449,0x84C9,0x8981,
	0x8B21,0x8E0A,0x9065,0x967D,0x990A,0x617E,0x6291,0x6B32,
	0x6C83,0x6D74,0x7FCC,0x7FFC,0x6DC0,0x7F85,0x87BA,0x88F8,
	0x6765,0x83B1,0x983C,0x96F7,0x6D1B,0x7D61,0x843D,0x916A,
	0x4E71,0x5375,0x5D50,0x6B04,0x6FEB,0x85CD,0x862D,0x89A7,
	0x5229,0x540F,0x5C65,0x674E,0x68A8,0x7406,0x7483,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 79, Array index 0x2700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x75E2,0x88CF,0x88E1,0x91CC,0x96E2,0x9678,0x5F8B,
	0x7387,0x7ACB,0x844E,0x63A0,0x7565,0x5289,0x6D41,0x6E9C,
	0x7409,0x7559,0x786B,0x7C92,0x9686,0x7ADC,0x9F8D,0x4FB6,
	0x616E,0x65C5,0x865C,0x4E86,0x4EAE,0x50DA,0x4E21,0x51CC,
	0x5BEE,0x6599,0x6881,0x6DBC,0x731F,0x7642,0x77AD,0x7A1C,
	0x7CE7,0x826F,0x8AD2,0x907C,0x91CF,0x9675,0x9818,0x529B,
	0x7DD1,0x502B,0x5398,0x6797,0x6DCB,0x71D0,0x7433,0x81E8,
	0x8F2A,0x96A3,0x9C57,0x9E9F,0x7460,0x5841,0x6D99,0x7D2F,
	0x985E,0x4EE4,0x4F36,0x4F8B,0x51B7,0x52B1,0x5DBA,0x601C,
	0x73B2,0x793C,0x82D3,0x9234,0x96B7,0x96F6,0x970A,0x9E97,
	0x9F62,0x66A6,0x6B74,0x5217,0x52A3,0x70C8,0x88C2,0x5EC9,
	0x604B,0x6190,0x6F23,0x7149,0x7C3E,0x7DF4,0x806F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 80, Array index 0x2800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x84EE,0x9023,0x932C,0x5442,0x9B6F,0x6AD3,0x7089,
	0x8CC2,0x8DEF,0x9732,0x52B4,0x5A41,0x5ECA,0x5F04,0x6717,
	0x697C,0x6994,0x6D6A,0x6F0F,0x7262,0x72FC,0x7BED,0x8001,
	0x807E,0x874B,0x90CE,0x516D,0x9E93,0x7984,0x808B,0x9332,
	0x8AD6,0x502D,0x548C,0x8A71,0x6B6A,0x8CC4,0x8107,0x60D1,
	0x67A0,0x9DF2,0x4E99,0x4E98,0x9C10,0x8A6B,0x85C1,0x8568,
	0x6900,0x6E7E,0x7897,0x8155,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 81, Array index 0x2900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F0C,0x4E10,0x4E15,0x4E2A,0x4E31,0x4E36,0x4E3C,
	0x4E3F,0x4E42,0x4E56,0x4E58,0x4E82,0x4E85,0x8C6B,0x4E8A,
	0x8212,0x5F0D,0x4E8E,0x4E9E,0x4E9F,0x4EA0,0x4EA2,0x4EB0,
	0x4EB3,0x4EB6,0x4ECE,0x4ECD,0x4EC4,0x4EC6,0x4EC2,0x4ED7,
	0x4EDE,0x4EED,0x4EDF,0x4EF7,0x4F09,0x4F5A,0x4F30,0x4F5B,
	0x4F5D,0x4F57,0x4F47,0x4F76,0x4F88,0x4F8F,0x4F98,0x4F7B,
	0x4F69,0x4F70,0x4F91,0x4F6F,0x4F86,0x4F96,0x5118,0x4FD4,
	0x4FDF,0x4FCE,0x4FD8,0x4FDB,0x4FD1,0x4FDA,0x4FD0,0x4FE4,
	0x4FE5,0x501A,0x5028,0x5014,0x502A,0x5025,0x5005,0x4F1C,
	0x4FF6,0x5021,0x5029,0x502C,0x4FFE,0x4FEF,0x5011,0x5006,
	0x5043,0x5047,0x6703,0x5055,0x5050,0x5048,0x505A,0x5056,
	0x506C,0x5078,0x5080,0x509A,0x5085,0x50B4,0x50B2,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 82, Array index 0x2A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x50C9,0x50CA,0x50B3,0x50C2,0x50D6,0x50DE,0x50E5,
	0x50ED,0x50E3,0x50EE,0x50F9,0x50F5,0x5109,0x5101,0x5102,
	0x5116,0x5115,0x5114,0x511A,0x5121,0x513A,0x5137,0x513C,
	0x513B,0x513F,0x5140,0x5152,0x514C,0x5154,0x5162,0x7AF8,
	0x5169,0x516A,0x516E,0x5180,0x5182,0x56D8,0x518C,0x5189,
	0x518F,0x5191,0x5193,0x5195,0x5196,0x51A4,0x51A6,0x51A2,
	0x51A9,0x51AA,0x51AB,0x51B3,0x51B1,0x51B2,0x51B0,0x51B5,
	0x51BD,0x51C5,0x51C9,0x51DB,0x51E0,0x8655,0x51E9,0x51ED,
	0x51F0,0x51F5,0x51FE,0x5204,0x520B,0x5214,0x520E,0x5227,
	0x522A,0x522E,0x5233,0x5239,0x524F,0x5244,0x524B,0x524C,
	0x525E,0x5254,0x526A,0x5274,0x5269,0x5273,0x527F,0x527D,
	0x528D,0x5294,0x5292,0x5271,0x5288,0x5291,0x8FA8,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 83, Array index 0x2B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8FA7,0x52AC,0x52AD,0x52BC,0x52B5,0x52C1,0x52CD,
	0x52D7,0x52DE,0x52E3,0x52E6,0x98ED,0x52E0,0x52F3,0x52F5,
	0x52F8,0x52F9,0x5306,0x5308,0x7538,0x530D,0x5310,0x530F,
	0x5315,0x531A,0x5323,0x532F,0x5331,0x5333,0x5338,0x5340,
	0x5346,0x5345,0x4E17,0x5349,0x534D,0x51D6,0x535E,0x5369,
	0x536E,0x5918,0x537B,0x5377,0x5382,0x5396,0x53A0,0x53A6,
	0x53A5,0x53AE,0x53B0,0x53B6,0x53C3,0x7C12,0x96D9,0x53DF,
	0x66FC,0x71EE,0x53EE,0x53E8,0x53ED,0x53FA,0x5401,0x543D,
	0x5440,0x542C,0x542D,0x543C,0x542E,0x5436,0x5429,0x541D,
	0x544E,0x548F,0x5475,0x548E,0x545F,0x5471,0x5477,0x5470,
	0x5492,0x547B,0x5480,0x5476,0x5484,0x5490,0x5486,0x54C7,
	0x54A2,0x54B8,0x54A5,0x54AC,0x54C4,0x54C8,0x54A8,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 84, Array index 0x2C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x54AB,0x54C2,0x54A4,0x54BE,0x54BC,0x54D8,0x54E5,
	0x54E6,0x550F,0x5514,0x54FD,0x54EE,0x54ED,0x54FA,0x54E2,
	0x5539,0x5540,0x5563,0x554C,0x552E,0x555C,0x5545,0x5556,
	0x5557,0x5538,0x5533,0x555D,0x5599,0x5580,0x54AF,0x558A,
	0x559F,0x557B,0x557E,0x5598,0x559E,0x55AE,0x557C,0x5583,
	0x55A9,0x5587,0x55A8,0x55DA,0x55C5,0x55DF,0x55C4,0x55DC,
	0x55E4,0x55D4,0x5614,0x55F7,0x5616,0x55FE,0x55FD,0x561B,
	0x55F9,0x564E,0x5650,0x71DF,0x5634,0x5636,0x5632,0x5638,
	0x566B,0x5664,0x562F,0x566C,0x566A,0x5686,0x5680,0x568A,
	0x56A0,0x5694,0x568F,0x56A5,0x56AE,0x56B6,0x56B4,0x56C2,
	0x56BC,0x56C1,0x56C3,0x56C0,0x56C8,0x56CE,0x56D1,0x56D3,
	0x56D7,0x56EE,0x56F9,0x5700,0x56FF,0x5704,0x5709,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 85, Array index 0x2D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5708,0x570B,0x570D,0x5713,0x5718,0x5716,0x55C7,
	0x571C,0x5726,0x5737,0x5738,0x574E,0x573B,0x5740,0x574F,
	0x5769,0x57C0,0x5788,0x5761,0x577F,0x5789,0x5793,0x57A0,
	0x57B3,0x57A4,0x57AA,0x57B0,0x57C3,0x57C6,0x57D4,0x57D2,
	0x57D3,0x580A,0x57D6,0x57E3,0x580B,0x5819,0x581D,0x5872,
	0x5821,0x5862,0x584B,0x5870,0x6BC0,0x5852,0x583D,0x5879,
	0x5885,0x58B9,0x589F,0x58AB,0x58BA,0x58DE,0x58BB,0x58B8,
	0x58AE,0x58C5,0x58D3,0x58D1,0x58D7,0x58D9,0x58D8,0x58E5,
	0x58DC,0x58E4,0x58DF,0x58EF,0x58FA,0x58F9,0x58FB,0x58FC,
	0x58FD,0x5902,0x590A,0x5910,0x591B,0x68A6,0x5925,0x592C,
	0x592D,0x5932,0x5938,0x593E,0x7AD2,0x5955,0x5950,0x594E,
	0x595A,0x5958,0x5962,0x5960,0x5967,0x596C,0x5969,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 86, Array index 0x2E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5978,0x5981,0x599D,0x4F5E,0x4FAB,0x59A3,0x59B2,
	0x59C6,0x59E8,0x59DC,0x598D,0x59D9,0x59DA,0x5A25,0x5A1F,
	0x5A11,0x5A1C,0x5A09,0x5A1A,0x5A40,0x5A6C,0x5A49,0x5A35,
	0x5A36,0x5A62,0x5A6A,0x5A9A,0x5ABC,0x5ABE,0x5ACB,0x5AC2,
	0x5ABD,0x5AE3,0x5AD7,0x5AE6,0x5AE9,0x5AD6,0x5AFA,0x5AFB,
	0x5B0C,0x5B0B,0x5B16,0x5B32,0x5AD0,0x5B2A,0x5B36,0x5B3E,
	0x5B43,0x5B45,0x5B40,0x5B51,0x5B55,0x5B5A,0x5B5B,0x5B65,
	0x5B69,0x5B70,0x5B73,0x5B75,0x5B78,0x6588,0x5B7A,0x5B80,
	0x5B83,0x5BA6,0x5BB8,0x5BC3,0x5BC7,0x5BC9,0x5BD4,0x5BD0,
	0x5BE4,0x5BE6,0x5BE2,0x5BDE,0x5BE5,0x5BEB,0x5BF0,0x5BF6,
	0x5BF3,0x5C05,0x5C07,0x5C08,0x5C0D,0x5C13,0x5C20,0x5C22,
	0x5C28,0x5C38,0x5C39,0x5C41,0x5C46,0x5C4E,0x5C53,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 87, Array index 0x2F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C50,0x5C4F,0x5B71,0x5C6C,0x5C6E,0x4E62,0x5C76,
	0x5C79,0x5C8C,0x5C91,0x5C94,0x599B,0x5CAB,0x5CBB,0x5CB6,
	0x5CBC,0x5CB7,0x5CC5,0x5CBE,0x5CC7,0x5CD9,0x5CE9,0x5CFD,
	0x5CFA,0x5CED,0x5D8C,0x5CEA,0x5D0B,0x5D15,0x5D17,0x5D5C,
	0x5D1F,0x5D1B,0x5D11,0x5D14,0x5D22,0x5D1A,0x5D19,0x5D18,
	0x5D4C,0x5D52,0x5D4E,0x5D4B,0x5D6C,0x5D73,0x5D76,0x5D87,
	0x5D84,0x5D82,0x5DA2,0x5D9D,0x5DAC,0x5DAE,0x5DBD,0x5D90,
	0x5DB7,0x5DBC,0x5DC9,0x5DCD,0x5DD3,0x5DD2,0x5DD6,0x5DDB,
	0x5DEB,0x5DF2,0x5DF5,0x5E0B,0x5E1A,0x5E19,0x5E11,0x5E1B,
	0x5E36,0x5E37,0x5E44,0x5E43,0x5E40,0x5E4E,0x5E57,0x5E54,
	0x5E5F,0x5E62,0x5E64,0x5E47,0x5E75,0x5E76,0x5E7A,0x9EBC,
	0x5E7F,0x5EA0,0x5EC1,0x5EC2,0x5EC8,0x5ED0,0x5ECF,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 88, Array index 0x3000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5ED6,0x5EE3,0x5EDD,0x5EDA,0x5EDB,0x5EE2,0x5EE1,
	0x5EE8,0x5EE9,0x5EEC,0x5EF1,0x5EF3,0x5EF0,0x5EF4,0x5EF8,
	0x5EFE,0x5F03,0x5F09,0x5F5D,0x5F5C,0x5F0B,0x5F11,0x5F16,
	0x5F29,0x5F2D,0x5F38,0x5F41,0x5F48,0x5F4C,0x5F4E,0x5F2F,
	0x5F51,0x5F56,0x5F57,0x5F59,0x5F61,0x5F6D,0x5F73,0x5F77,
	0x5F83,0x5F82,0x5F7F,0x5F8A,0x5F88,0x5F91,0x5F87,0x5F9E,
	0x5F99,0x5F98,0x5FA0,0x5FA8,0x5FAD,0x5FBC,0x5FD6,0x5FFB,
	0x5FE4,0x5FF8,0x5FF1,0x5FDD,0x60B3,0x5FFF,0x6021,0x6060,
	0x6019,0x6010,0x6029,0x600E,0x6031,0x601B,0x6015,0x602B,
	0x6026,0x600F,0x603A,0x605A,0x6041,0x606A,0x6077,0x605F,
	0x604A,0x6046,0x604D,0x6063,0x6043,0x6064,0x6042,0x606C,
	0x606B,0x6059,0x6081,0x608D,0x60E7,0x6083,0x609A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 89, Array index 0x3100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6084,0x609B,0x6096,0x6097,0x6092,0x60A7,0x608B,
	0x60E1,0x60B8,0x60E0,0x60D3,0x60B4,0x5FF0,0x60BD,0x60C6,
	0x60B5,0x60D8,0x614D,0x6115,0x6106,0x60F6,0x60F7,0x6100,
	0x60F4,0x60FA,0x6103,0x6121,0x60FB,0x60F1,0x610D,0x610E,
	0x6147,0x613E,0x6128,0x6127,0x614A,0x613F,0x613C,0x612C,
	0x6134,0x613D,0x6142,0x6144,0x6173,0x6177,0x6158,0x6159,
	0x615A,0x616B,0x6174,0x616F,0x6165,0x6171,0x615F,0x615D,
	0x6153,0x6175,0x6199,0x6196,0x6187,0x61AC,0x6194,0x619A,
	0x618A,0x6191,0x61AB,0x61AE,0x61CC,0x61CA,0x61C9,0x61F7,
	0x61C8,0x61C3,0x61C6,0x61BA,0x61CB,0x7F79,0x61CD,0x61E6,
	0x61E3,0x61F6,0x61FA,0x61F4,0x61FF,0x61FD,0x61FC,0x61FE,
	0x6200,0x6208,0x6209,0x620D,0x620C,0x6214,0x621B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 90, Array index 0x3200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x621E,0x6221,0x622A,0x622E,0x6230,0x6232,0x6233,
	0x6241,0x624E,0x625E,0x6263,0x625B,0x6260,0x6268,0x627C,
	0x6282,0x6289,0x627E,0x6292,0x6293,0x6296,0x62D4,0x6283,
	0x6294,0x62D7,0x62D1,0x62BB,0x62CF,0x62FF,0x62C6,0x64D4,
	0x62C8,0x62DC,0x62CC,0x62CA,0x62C2,0x62C7,0x629B,0x62C9,
	0x630C,0x62EE,0x62F1,0x6327,0x6302,0x6308,0x62EF,0x62F5,
	0x6350,0x633E,0x634D,0x641C,0x634F,0x6396,0x638E,0x6380,
	0x63AB,0x6376,0x63A3,0x638F,0x6389,0x639F,0x63B5,0x636B,
	0x6369,0x63BE,0x63E9,0x63C0,0x63C6,0x63E3,0x63C9,0x63D2,
	0x63F6,0x63C4,0x6416,0x6434,0x6406,0x6413,0x6426,0x6436,
	0x651D,0x6417,0x6428,0x640F,0x6467,0x646F,0x6476,0x644E,
	0x652A,0x6495,0x6493,0x64A5,0x64A9,0x6488,0x64BC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 91, Array index 0x3300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x64DA,0x64D2,0x64C5,0x64C7,0x64BB,0x64D8,0x64C2,
	0x64F1,0x64E7,0x8209,0x64E0,0x64E1,0x62AC,0x64E3,0x64EF,
	0x652C,0x64F6,0x64F4,0x64F2,0x64FA,0x6500,0x64FD,0x6518,
	0x651C,0x6505,0x6524,0x6523,0x652B,0x6534,0x6535,0x6537,
	0x6536,0x6538,0x754B,0x6548,0x6556,0x6555,0x654D,0x6558,
	0x655E,0x655D,0x6572,0x6578,0x6582,0x6583,0x8B8A,0x659B,
	0x659F,0x65AB,0x65B7,0x65C3,0x65C6,0x65C1,0x65C4,0x65CC,
	0x65D2,0x65DB,0x65D9,0x65E0,0x65E1,0x65F1,0x6772,0x660A,
	0x6603,0x65FB,0x6773,0x6635,0x6636,0x6634,0x661C,0x664F,
	0x6644,0x6649,0x6641,0x665E,0x665D,0x6664,0x6667,0x6668,
	0x665F,0x6662,0x6670,0x6683,0x6688,0x668E,0x6689,0x6684,
	0x6698,0x669D,0x66C1,0x66B9,0x66C9,0x66BE,0x66BC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 92, Array index 0x3400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x66C4,0x66B8,0x66D6,0x66DA,0x66E0,0x663F,0x66E6,
	0x66E9,0x66F0,0x66F5,0x66F7,0x670F,0x6716,0x671E,0x6726,
	0x6727,0x9738,0x672E,0x673F,0x6736,0x6741,0x6738,0x6737,
	0x6746,0x675E,0x6760,0x6759,0x6763,0x6764,0x6789,0x6770,
	0x67A9,0x677C,0x676A,0x678C,0x678B,0x67A6,0x67A1,0x6785,
	0x67B7,0x67EF,0x67B4,0x67EC,0x67B3,0x67E9,0x67B8,0x67E4,
	0x67DE,0x67DD,0x67E2,0x67EE,0x67B9,0x67CE,0x67C6,0x67E7,
	0x6A9C,0x681E,0x6846,0x6829,0x6840,0x684D,0x6832,0x684E,
	0x68B3,0x682B,0x6859,0x6863,0x6877,0x687F,0x689F,0x688F,
	0x68AD,0x6894,0x689D,0x689B,0x6883,0x6AAE,0x68B9,0x6874,
	0x68B5,0x68A0,0x68BA,0x690F,0x688D,0x687E,0x6901,0x68CA,
	0x6908,0x68D8,0x6922,0x6926,0x68E1,0x690C,0x68CD,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 93, Array index 0x3500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x68D4,0x68E7,0x68D5,0x6936,0x6912,0x6904,0x68D7,
	0x68E3,0x6925,0x68F9,0x68E0,0x68EF,0x6928,0x692A,0x691A,
	0x6923,0x6921,0x68C6,0x6979,0x6977,0x695C,0x6978,0x696B,
	0x6954,0x697E,0x696E,0x6939,0x6974,0x693D,0x6959,0x6930,
	0x6961,0x695E,0x695D,0x6981,0x696A,0x69B2,0x69AE,0x69D0,
	0x69BF,0x69C1,0x69D3,0x69BE,0x69CE,0x5BE8,0x69CA,0x69DD,
	0x69BB,0x69C3,0x69A7,0x6A2E,0x6991,0x69A0,0x699C,0x6995,
	0x69B4,0x69DE,0x69E8,0x6A02,0x6A1B,0x69FF,0x6B0A,0x69F9,
	0x69F2,0x69E7,0x6A05,0x69B1,0x6A1E,0x69ED,0x6A14,0x69EB,
	0x6A0A,0x6A12,0x6AC1,0x6A23,0x6A13,0x6A44,0x6A0C,0x6A72,
	0x6A36,0x6A78,0x6A47,0x6A62,0x6A59,0x6A66,0x6A48,0x6A38,
	0x6A22,0x6A90,0x6A8D,0x6AA0,0x6A84,0x6AA2,0x6AA3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 94, Array index 0x3600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A97,0x8617,0x6ABB,0x6AC3,0x6AC2,0x6AB8,0x6AB3,
	0x6AAC,0x6ADE,0x6AD1,0x6ADF,0x6AAA,0x6ADA,0x6AEA,0x6AFB,
	0x6B05,0x8616,0x6AFA,0x6B12,0x6B16,0x9B31,0x6B1F,0x6B38,
	0x6B37,0x76DC,0x6B39,0x98EE,0x6B47,0x6B43,0x6B49,0x6B50,
	0x6B59,0x6B54,0x6B5B,0x6B5F,0x6B61,0x6B78,0x6B79,0x6B7F,
	0x6B80,0x6B84,0x6B83,0x6B8D,0x6B98,0x6B95,0x6B9E,0x6BA4,
	0x6BAA,0x6BAB,0x6BAF,0x6BB2,0x6BB1,0x6BB3,0x6BB7,0x6BBC,
	0x6BC6,0x6BCB,0x6BD3,0x6BDF,0x6BEC,0x6BEB,0x6BF3,0x6BEF,
	0x9EBE,0x6C08,0x6C13,0x6C14,0x6C1B,0x6C24,0x6C23,0x6C5E,
	0x6C55,0x6C62,0x6C6A,0x6C82,0x6C8D,0x6C9A,0x6C81,0x6C9B,
	0x6C7E,0x6C68,0x6C73,0x6C92,0x6C90,0x6CC4,0x6CF1,0x6CD3,
	0x6CBD,0x6CD7,0x6CC5,0x6CDD,0x6CAE,0x6CB1,0x6CBE,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 95, Array index 0x3700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6CBA,0x6CDB,0x6CEF,0x6CD9,0x6CEA,0x6D1F,0x884D,
	0x6D36,0x6D2B,0x6D3D,0x6D38,0x6D19,0x6D35,0x6D33,0x6D12,
	0x6D0C,0x6D63,0x6D93,0x6D64,0x6D5A,0x6D79,0x6D59,0x6D8E,
	0x6D95,0x6FE4,0x6D85,0x6DF9,0x6E15,0x6E0A,0x6DB5,0x6DC7,
	0x6DE6,0x6DB8,0x6DC6,0x6DEC,0x6DDE,0x6DCC,0x6DE8,0x6DD2,
	0x6DC5,0x6DFA,0x6DD9,0x6DE4,0x6DD5,0x6DEA,0x6DEE,0x6E2D,
	0x6E6E,0x6E2E,0x6E19,0x6E72,0x6E5F,0x6E3E,0x6E23,0x6E6B,
	0x6E2B,0x6E76,0x6E4D,0x6E1F,0x6E43,0x6E3A,0x6E4E,0x6E24,
	0x6EFF,0x6E1D,0x6E38,0x6E82,0x6EAA,0x6E98,0x6EC9,0x6EB7,
	0x6ED3,0x6EBD,0x6EAF,0x6EC4,0x6EB2,0x6ED4,0x6ED5,0x6E8F,
	0x6EA5,0x6EC2,0x6E9F,0x6F41,0x6F11,0x704C,0x6EEC,0x6EF8,
	0x6EFE,0x6F3F,0x6EF2,0x6F31,0x6EEF,0x6F32,0x6ECC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 96, Array index 0x3800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F3E,0x6F13,0x6EF7,0x6F86,0x6F7A,0x6F78,0x6F81,
	0x6F80,0x6F6F,0x6F5B,0x6FF3,0x6F6D,0x6F82,0x6F7C,0x6F58,
	0x6F8E,0x6F91,0x6FC2,0x6F66,0x6FB3,0x6FA3,0x6FA1,0x6FA4,
	0x6FB9,0x6FC6,0x6FAA,0x6FDF,0x6FD5,0x6FEC,0x6FD4,0x6FD8,
	0x6FF1,0x6FEE,0x6FDB,0x7009,0x700B,0x6FFA,0x7011,0x7001,
	0x700F,0x6FFE,0x701B,0x701A,0x6F74,0x701D,0x7018,0x701F,
	0x7030,0x703E,0x7032,0x7051,0x7063,0x7099,0x7092,0x70AF,
	0x70F1,0x70AC,0x70B8,0x70B3,0x70AE,0x70DF,0x70CB,0x70DD,
	0x70D9,0x7109,0x70FD,0x711C,0x7119,0x7165,0x7155,0x7188,
	0x7166,0x7162,0x714C,0x7156,0x716C,0x718F,0x71FB,0x7184,
	0x7195,0x71A8,0x71AC,0x71D7,0x71B9,0x71BE,0x71D2,0x71C9,
	0x71D4,0x71CE,0x71E0,0x71EC,0x71E7,0x71F5,0x71FC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 97, Array index 0x3900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x71F9,0x71FF,0x720D,0x7210,0x721B,0x7228,0x722D,
	0x722C,0x7230,0x7232,0x723B,0x723C,0x723F,0x7240,0x7246,
	0x724B,0x7258,0x7274,0x727E,0x7282,0x7281,0x7287,0x7292,
	0x7296,0x72A2,0x72A7,0x72B9,0x72B2,0x72C3,0x72C6,0x72C4,
	0x72CE,0x72D2,0x72E2,0x72E0,0x72E1,0x72F9,0x72F7,0x500F,
	0x7317,0x730A,0x731C,0x7316,0x731D,0x7334,0x732F,0x7329,
	0x7325,0x733E,0x734E,0x734F,0x9ED8,0x7357,0x736A,0x7368,
	0x7370,0x7378,0x7375,0x737B,0x737A,0x73C8,0x73B3,0x73CE,
	0x73BB,0x73C0,0x73E5,0x73EE,0x73DE,0x74A2,0x7405,0x746F,
	0x7425,0x73F8,0x7432,0x743A,0x7455,0x743F,0x745F,0x7459,
	0x7441,0x745C,0x7469,0x7470,0x7463,0x746A,0x7476,0x747E,
	0x748B,0x749E,0x74A7,0x74CA,0x74CF,0x74D4,0x73F1,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 98, Array index 0x3A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x74E0,0x74E3,0x74E7,0x74E9,0x74EE,0x74F2,0x74F0,
	0x74F1,0x74F8,0x74F7,0x7504,0x7503,0x7505,0x750C,0x750E,
	0x750D,0x7515,0x7513,0x751E,0x7526,0x752C,0x753C,0x7544,
	0x754D,0x754A,0x7549,0x755B,0x7546,0x755A,0x7569,0x7564,
	0x7567,0x756B,0x756D,0x7578,0x7576,0x7586,0x7587,0x7574,
	0x758A,0x7589,0x7582,0x7594,0x759A,0x759D,0x75A5,0x75A3,
	0x75C2,0x75B3,0x75C3,0x75B5,0x75BD,0x75B8,0x75BC,0x75B1,
	0x75CD,0x75CA,0x75D2,0x75D9,0x75E3,0x75DE,0x75FE,0x75FF,
	0x75FC,0x7601,0x75F0,0x75FA,0x75F2,0x75F3,0x760B,0x760D,
	0x7609,0x761F,0x7627,0x7620,0x7621,0x7622,0x7624,0x7634,
	0x7630,0x763B,0x7647,0x7648,0x7646,0x765C,0x7658,0x7661,
	0x7662,0x7668,0x7669,0x766A,0x7667,0x766C,0x7670,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 99, Array index 0x3B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7672,0x7676,0x7678,0x767C,0x7680,0x7683,0x7688,
	0x768B,0x768E,0x7696,0x7693,0x7699,0x769A,0x76B0,0x76B4,
	0x76B8,0x76B9,0x76BA,0x76C2,0x76CD,0x76D6,0x76D2,0x76DE,
	0x76E1,0x76E5,0x76E7,0x76EA,0x862F,0x76FB,0x7708,0x7707,
	0x7704,0x7729,0x7724,0x771E,0x7725,0x7726,0x771B,0x7737,
	0x7738,0x7747,0x775A,0x7768,0x776B,0x775B,0x7765,0x777F,
	0x777E,0x7779,0x778E,0x778B,0x7791,0x77A0,0x779E,0x77B0,
	0x77B6,0x77B9,0x77BF,0x77BC,0x77BD,0x77BB,0x77C7,0x77CD,
	0x77D7,0x77DA,0x77DC,0x77E3,0x77EE,0x77FC,0x780C,0x7812,
	0x7926,0x7820,0x792A,0x7845,0x788E,0x7874,0x7886,0x787C,
	0x789A,0x788C,0x78A3,0x78B5,0x78AA,0x78AF,0x78D1,0x78C6,
	0x78CB,0x78D4,0x78BE,0x78BC,0x78C5,0x78CA,0x78EC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 100, Array index 0x3C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x78E7,0x78DA,0x78FD,0x78F4,0x7907,0x7912,0x7911,
	0x7919,0x792C,0x792B,0x7940,0x7960,0x7957,0x795F,0x795A,
	0x7955,0x7953,0x797A,0x797F,0x798A,0x799D,0x79A7,0x9F4B,
	0x79AA,0x79AE,0x79B3,0x79B9,0x79BA,0x79C9,0x79D5,0x79E7,
	0x79EC,0x79E1,0x79E3,0x7A08,0x7A0D,0x7A18,0x7A19,0x7A20,
	0x7A1F,0x7980,0x7A31,0x7A3B,0x7A3E,0x7A37,0x7A43,0x7A57,
	0x7A49,0x7A61,0x7A62,0x7A69,0x9F9D,0x7A70,0x7A79,0x7A7D,
	0x7A88,0x7A97,0x7A95,0x7A98,0x7A96,0x7AA9,0x7AC8,0x7AB0,
	0x7AB6,0x7AC5,0x7AC4,0x7ABF,0x9083,0x7AC7,0x7ACA,0x7ACD,
	0x7ACF,0x7AD5,0x7AD3,0x7AD9,0x7ADA,0x7ADD,0x7AE1,0x7AE2,
	0x7AE6,0x7AED,0x7AF0,0x7B02,0x7B0F,0x7B0A,0x7B06,0x7B33,
	0x7B18,0x7B19,0x7B1E,0x7B35,0x7B28,0x7B36,0x7B50,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 101, Array index 0x3D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7B7A,0x7B04,0x7B4D,0x7B0B,0x7B4C,0x7B45,0x7B75,
	0x7B65,0x7B74,0x7B67,0x7B70,0x7B71,0x7B6C,0x7B6E,0x7B9D,
	0x7B98,0x7B9F,0x7B8D,0x7B9C,0x7B9A,0x7B8B,0x7B92,0x7B8F,
	0x7B5D,0x7B99,0x7BCB,0x7BC1,0x7BCC,0x7BCF,0x7BB4,0x7BC6,
	0x7BDD,0x7BE9,0x7C11,0x7C14,0x7BE6,0x7BE5,0x7C60,0x7C00,
	0x7C07,0x7C13,0x7BF3,0x7BF7,0x7C17,0x7C0D,0x7BF6,0x7C23,
	0x7C27,0x7C2A,0x7C1F,0x7C37,0x7C2B,0x7C3D,0x7C4C,0x7C43,
	0x7C54,0x7C4F,0x7C40,0x7C50,0x7C58,0x7C5F,0x7C64,0x7C56,
	0x7C65,0x7C6C,0x7C75,0x7C83,0x7C90,0x7CA4,0x7CAD,0x7CA2,
	0x7CAB,0x7CA1,0x7CA8,0x7CB3,0x7CB2,0x7CB1,0x7CAE,0x7CB9,
	0x7CBD,0x7CC0,0x7CC5,0x7CC2,0x7CD8,0x7CD2,0x7CDC,0x7CE2,
	0x9B3B,0x7CEF,0x7CF2,0x7CF4,0x7CF6,0x7CFA,0x7D06,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 102, Array index 0x3E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7D02,0x7D1C,0x7D15,0x7D0A,0x7D45,0x7D4B,0x7D2E,
	0x7D32,0x7D3F,0x7D35,0x7D46,0x7D73,0x7D56,0x7D4E,0x7D72,
	0x7D68,0x7D6E,0x7D4F,0x7D63,0x7D93,0x7D89,0x7D5B,0x7D8F,
	0x7D7D,0x7D9B,0x7DBA,0x7DAE,0x7DA3,0x7DB5,0x7DC7,0x7DBD,
	0x7DAB,0x7E3D,0x7DA2,0x7DAF,0x7DDC,0x7DB8,0x7D9F,0x7DB0,
	0x7DD8,0x7DDD,0x7DE4,0x7DDE,0x7DFB,0x7DF2,0x7DE1,0x7E05,
	0x7E0A,0x7E23,0x7E21,0x7E12,0x7E31,0x7E1F,0x7E09,0x7E0B,
	0x7E22,0x7E46,0x7E66,0x7E3B,0x7E35,0x7E39,0x7E43,0x7E37,
	0x7E32,0x7E3A,0x7E67,0x7E5D,0x7E56,0x7E5E,0x7E59,0x7E5A,
	0x7E79,0x7E6A,0x7E69,0x7E7C,0x7E7B,0x7E83,0x7DD5,0x7E7D,
	0x8FAE,0x7E7F,0x7E88,0x7E89,0x7E8C,0x7E92,0x7E90,0x7E93,
	0x7E94,0x7E96,0x7E8E,0x7E9B,0x7E9C,0x7F38,0x7F3A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 103, Array index 0x3F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7F45,0x7F4C,0x7F4D,0x7F4E,0x7F50,0x7F51,0x7F55,
	0x7F54,0x7F58,0x7F5F,0x7F60,0x7F68,0x7F69,0x7F67,0x7F78,
	0x7F82,0x7F86,0x7F83,0x7F88,0x7F87,0x7F8C,0x7F94,0x7F9E,
	0x7F9D,0x7F9A,0x7FA3,0x7FAF,0x7FB2,0x7FB9,0x7FAE,0x7FB6,
	0x7FB8,0x8B71,0x7FC5,0x7FC6,0x7FCA,0x7FD5,0x7FD4,0x7FE1,
	0x7FE6,0x7FE9,0x7FF3,0x7FF9,0x98DC,0x8006,0x8004,0x800B,
	0x8012,0x8018,0x8019,0x801C,0x8021,0x8028,0x803F,0x803B,
	0x804A,0x8046,0x8052,0x8058,0x805A,0x805F,0x8062,0x8068,
	0x8073,0x8072,0x8070,0x8076,0x8079,0x807D,0x807F,0x8084,
	0x8086,0x8085,0x809B,0x8093,0x809A,0x80AD,0x5190,0x80AC,
	0x80DB,0x80E5,0x80D9,0x80DD,0x80C4,0x80DA,0x80D6,0x8109,
	0x80EF,0x80F1,0x811B,0x8129,0x8123,0x812F,0x814B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 104, Array index 0x4000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x968B,0x8146,0x813E,0x8153,0x8151,0x80FC,0x8171,
	0x816E,0x8165,0x8166,0x8174,0x8183,0x8188,0x818A,0x8180,
	0x8182,0x81A0,0x8195,0x81A4,0x81A3,0x815F,0x8193,0x81A9,
	0x81B0,0x81B5,0x81BE,0x81B8,0x81BD,0x81C0,0x81C2,0x81BA,
	0x81C9,0x81CD,0x81D1,0x81D9,0x81D8,0x81C8,0x81DA,0x81DF,
	0x81E0,0x81E7,0x81FA,0x81FB,0x81FE,0x8201,0x8202,0x8205,
	0x8207,0x820A,0x820D,0x8210,0x8216,0x8229,0x822B,0x8238,
	0x8233,0x8240,0x8259,0x8258,0x825D,0x825A,0x825F,0x8264,
	0x8262,0x8268,0x826A,0x826B,0x822E,0x8271,0x8277,0x8278,
	0x827E,0x828D,0x8292,0x82AB,0x829F,0x82BB,0x82AC,0x82E1,
	0x82E3,0x82DF,0x82D2,0x82F4,0x82F3,0x82FA,0x8393,0x8303,
	0x82FB,0x82F9,0x82DE,0x8306,0x82DC,0x8309,0x82D9,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 105, Array index 0x4100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8335,0x8334,0x8316,0x8332,0x8331,0x8340,0x8339,
	0x8350,0x8345,0x832F,0x832B,0x8317,0x8318,0x8385,0x839A,
	0x83AA,0x839F,0x83A2,0x8396,0x8323,0x838E,0x8387,0x838A,
	0x837C,0x83B5,0x8373,0x8375,0x83A0,0x8389,0x83A8,0x83F4,
	0x8413,0x83EB,0x83CE,0x83FD,0x8403,0x83D8,0x840B,0x83C1,
	0x83F7,0x8407,0x83E0,0x83F2,0x840D,0x8422,0x8420,0x83BD,
	0x8438,0x8506,0x83FB,0x846D,0x842A,0x843C,0x855A,0x8484,
	0x8477,0x846B,0x84AD,0x846E,0x8482,0x8469,0x8446,0x842C,
	0x846F,0x8479,0x8435,0x84CA,0x8462,0x84B9,0x84BF,0x849F,
	0x84D9,0x84CD,0x84BB,0x84DA,0x84D0,0x84C1,0x84C6,0x84D6,
	0x84A1,0x8521,0x84FF,0x84F4,0x8517,0x8518,0x852C,0x851F,
	0x8515,0x8514,0x84FC,0x8540,0x8563,0x8558,0x8548,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 106, Array index 0x4200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8541,0x8602,0x854B,0x8555,0x8580,0x85A4,0x8588,
	0x8591,0x858A,0x85A8,0x856D,0x8594,0x859B,0x85EA,0x8587,
	0x859C,0x8577,0x857E,0x8590,0x85C9,0x85BA,0x85CF,0x85B9,
	0x85D0,0x85D5,0x85DD,0x85E5,0x85DC,0x85F9,0x860A,0x8613,
	0x860B,0x85FE,0x85FA,0x8606,0x8622,0x861A,0x8630,0x863F,
	0x864D,0x4E55,0x8654,0x865F,0x8667,0x8671,0x8693,0x86A3,
	0x86A9,0x86AA,0x868B,0x868C,0x86B6,0x86AF,0x86C4,0x86C6,
	0x86B0,0x86C9,0x8823,0x86AB,0x86D4,0x86DE,0x86E9,0x86EC,
	0x86DF,0x86DB,0x86EF,0x8712,0x8706,0x8708,0x8700,0x8703,
	0x86FB,0x8711,0x8709,0x870D,0x86F9,0x870A,0x8734,0x873F,
	0x8737,0x873B,0x8725,0x8729,0x871A,0x8760,0x875F,0x8778,
	0x874C,0x874E,0x8774,0x8757,0x8768,0x876E,0x8759,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 107, Array index 0x4300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8753,0x8763,0x876A,0x8805,0x87A2,0x879F,0x8782,
	0x87AF,0x87CB,0x87BD,0x87C0,0x87D0,0x96D6,0x87AB,0x87C4,
	0x87B3,0x87C7,0x87C6,0x87BB,0x87EF,0x87F2,0x87E0,0x880F,
	0x880D,0x87FE,0x87F6,0x87F7,0x880E,0x87D2,0x8811,0x8816,
	0x8815,0x8822,0x8821,0x8831,0x8836,0x8839,0x8827,0x883B,
	0x8844,0x8842,0x8852,0x8859,0x885E,0x8862,0x886B,0x8881,
	0x887E,0x889E,0x8875,0x887D,0x88B5,0x8872,0x8882,0x8897,
	0x8892,0x88AE,0x8899,0x88A2,0x888D,0x88A4,0x88B0,0x88BF,
	0x88B1,0x88C3,0x88C4,0x88D4,0x88D8,0x88D9,0x88DD,0x88F9,
	0x8902,0x88FC,0x88F4,0x88E8,0x88F2,0x8904,0x890C,0x890A,
	0x8913,0x8943,0x891E,0x8925,0x892A,0x892B,0x8941,0x8944,
	0x893B,0x8936,0x8938,0x894C,0x891D,0x8960,0x895E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 108, Array index 0x4400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8966,0x8964,0x896D,0x896A,0x896F,0x8974,0x8977,
	0x897E,0x8983,0x8988,0x898A,0x8993,0x8998,0x89A1,0x89A9,
	0x89A6,0x89AC,0x89AF,0x89B2,0x89BA,0x89BD,0x89BF,0x89C0,
	0x89DA,0x89DC,0x89DD,0x89E7,0x89F4,0x89F8,0x8A03,0x8A16,
	0x8A10,0x8A0C,0x8A1B,0x8A1D,0x8A25,0x8A36,0x8A41,0x8A5B,
	0x8A52,0x8A46,0x8A48,0x8A7C,0x8A6D,0x8A6C,0x8A62,0x8A85,
	0x8A82,0x8A84,0x8AA8,0x8AA1,0x8A91,0x8AA5,0x8AA6,0x8A9A,
	0x8AA3,0x8AC4,0x8ACD,0x8AC2,0x8ADA,0x8AEB,0x8AF3,0x8AE7,
	0x8AE4,0x8AF1,0x8B14,0x8AE0,0x8AE2,0x8AF7,0x8ADE,0x8ADB,
	0x8B0C,0x8B07,0x8B1A,0x8AE1,0x8B16,0x8B10,0x8B17,0x8B20,
	0x8B33,0x97AB,0x8B26,0x8B2B,0x8B3E,0x8B28,0x8B41,0x8B4C,
	0x8B4F,0x8B4E,0x8B49,0x8B56,0x8B5B,0x8B5A,0x8B6B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 109, Array index 0x4500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8B5F,0x8B6C,0x8B6F,0x8B74,0x8B7D,0x8B80,0x8B8C,
	0x8B8E,0x8B92,0x8B93,0x8B96,0x8B99,0x8B9A,0x8C3A,0x8C41,
	0x8C3F,0x8C48,0x8C4C,0x8C4E,0x8C50,0x8C55,0x8C62,0x8C6C,
	0x8C78,0x8C7A,0x8C82,0x8C89,0x8C85,0x8C8A,0x8C8D,0x8C8E,
	0x8C94,0x8C7C,0x8C98,0x621D,0x8CAD,0x8CAA,0x8CBD,0x8CB2,
	0x8CB3,0x8CAE,0x8CB6,0x8CC8,0x8CC1,0x8CE4,0x8CE3,0x8CDA,
	0x8CFD,0x8CFA,0x8CFB,0x8D04,0x8D05,0x8D0A,0x8D07,0x8D0F,
	0x8D0D,0x8D10,0x9F4E,0x8D13,0x8CCD,0x8D14,0x8D16,0x8D67,
	0x8D6D,0x8D71,0x8D73,0x8D81,0x8D99,0x8DC2,0x8DBE,0x8DBA,
	0x8DCF,0x8DDA,0x8DD6,0x8DCC,0x8DDB,0x8DCB,0x8DEA,0x8DEB,
	0x8DDF,0x8DE3,0x8DFC,0x8E08,0x8E09,0x8DFF,0x8E1D,0x8E1E,
	0x8E10,0x8E1F,0x8E42,0x8E35,0x8E30,0x8E34,0x8E4A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 110, Array index 0x4600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8E47,0x8E49,0x8E4C,0x8E50,0x8E48,0x8E59,0x8E64,
	0x8E60,0x8E2A,0x8E63,0x8E55,0x8E76,0x8E72,0x8E7C,0x8E81,
	0x8E87,0x8E85,0x8E84,0x8E8B,0x8E8A,0x8E93,0x8E91,0x8E94,
	0x8E99,0x8EAA,0x8EA1,0x8EAC,0x8EB0,0x8EC6,0x8EB1,0x8EBE,
	0x8EC5,0x8EC8,0x8ECB,0x8EDB,0x8EE3,0x8EFC,0x8EFB,0x8EEB,
	0x8EFE,0x8F0A,0x8F05,0x8F15,0x8F12,0x8F19,0x8F13,0x8F1C,
	0x8F1F,0x8F1B,0x8F0C,0x8F26,0x8F33,0x8F3B,0x8F39,0x8F45,
	0x8F42,0x8F3E,0x8F4C,0x8F49,0x8F46,0x8F4E,0x8F57,0x8F5C,
	0x8F62,0x8F63,0x8F64,0x8F9C,0x8F9F,0x8FA3,0x8FAD,0x8FAF,
	0x8FB7,0x8FDA,0x8FE5,0x8FE2,0x8FEA,0x8FEF,0x9087,0x8FF4,
	0x9005,0x8FF9,0x8FFA,0x9011,0x9015,0x9021,0x900D,0x901E,
	0x9016,0x900B,0x9027,0x9036,0x9035,0x9039,0x8FF8,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 111, Array index 0x4700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x904F,0x9050,0x9051,0x9052,0x900E,0x9049,0x903E,
	0x9056,0x9058,0x905E,0x9068,0x906F,0x9076,0x96A8,0x9072,
	0x9082,0x907D,0x9081,0x9080,0x908A,0x9089,0x908F,0x90A8,
	0x90AF,0x90B1,0x90B5,0x90E2,0x90E4,0x6248,0x90DB,0x9102,
	0x9112,0x9119,0x9132,0x9130,0x914A,0x9156,0x9158,0x9163,
	0x9165,0x9169,0x9173,0x9172,0x918B,0x9189,0x9182,0x91A2,
	0x91AB,0x91AF,0x91AA,0x91B5,0x91B4,0x91BA,0x91C0,0x91C1,
	0x91C9,0x91CB,0x91D0,0x91D6,0x91DF,0x91E1,0x91DB,0x91FC,
	0x91F5,0x91F6,0x921E,0x91FF,0x9214,0x922C,0x9215,0x9211,
	0x925E,0x9257,0x9245,0x9249,0x9264,0x9248,0x9295,0x923F,
	0x924B,0x9250,0x929C,0x9296,0x9293,0x929B,0x925A,0x92CF,
	0x92B9,0x92B7,0x92E9,0x930F,0x92FA,0x9344,0x932E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 112, Array index 0x4800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9319,0x9322,0x931A,0x9323,0x933A,0x9335,0x933B,
	0x935C,0x9360,0x937C,0x936E,0x9356,0x93B0,0x93AC,0x93AD,
	0x9394,0x93B9,0x93D6,0x93D7,0x93E8,0x93E5,0x93D8,0x93C3,
	0x93DD,0x93D0,0x93C8,0x93E4,0x941A,0x9414,0x9413,0x9403,
	0x9407,0x9410,0x9436,0x942B,0x9435,0x9421,0x943A,0x9441,
	0x9452,0x9444,0x945B,0x9460,0x9462,0x945E,0x946A,0x9229,
	0x9470,0x9475,0x9477,0x947D,0x945A,0x947C,0x947E,0x9481,
	0x947F,0x9582,0x9587,0x958A,0x9594,0x9596,0x9598,0x9599,
	0x95A0,0x95A8,0x95A7,0x95AD,0x95BC,0x95BB,0x95B9,0x95BE,
	0x95CA,0x6FF6,0x95C3,0x95CD,0x95CC,0x95D5,0x95D4,0x95D6,
	0x95DC,0x95E1,0x95E5,0x95E2,0x9621,0x9628,0x962E,0x962F,
	0x9642,0x964C,0x964F,0x964B,0x9677,0x965C,0x965E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 113, Array index 0x4900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x965D,0x965F,0x9666,0x9672,0x966C,0x968D,0x9698,
	0x9695,0x9697,0x96AA,0x96A7,0x96B1,0x96B2,0x96B0,0x96B4,
	0x96B6,0x96B8,0x96B9,0x96CE,0x96CB,0x96C9,0x96CD,0x894D,
	0x96DC,0x970D,0x96D5,0x96F9,0x9704,0x9706,0x9708,0x9713,
	0x970E,0x9711,0x970F,0x9716,0x9719,0x9724,0x972A,0x9730,
	0x9739,0x973D,0x973E,0x9744,0x9746,0x9748,0x9742,0x9749,
	0x975C,0x9760,0x9764,0x9766,0x9768,0x52D2,0x976B,0x9771,
	0x9779,0x9785,0x977C,0x9781,0x977A,0x9786,0x978B,0x978F,
	0x9790,0x979C,0x97A8,0x97A6,0x97A3,0x97B3,0x97B4,0x97C3,
	0x97C6,0x97C8,0x97CB,0x97DC,0x97ED,0x9F4F,0x97F2,0x7ADF,
	0x97F6,0x97F5,0x980F,0x980C,0x9838,0x9824,0x9821,0x9837,
	0x983D,0x9846,0x984F,0x984B,0x986B,0x986F,0x9870,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 114, Array index 0x4A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9871,0x9874,0x9873,0x98AA,0x98AF,0x98B1,0x98B6,
	0x98C4,0x98C3,0x98C6,0x98E9,0x98EB,0x9903,0x9909,0x9912,
	0x9914,0x9918,0x9921,0x991D,0x991E,0x9924,0x9920,0x992C,
	0x992E,0x993D,0x993E,0x9942,0x9949,0x9945,0x9950,0x994B,
	0x9951,0x9952,0x994C,0x9955,0x9997,0x9998,0x99A5,0x99AD,
	0x99AE,0x99BC,0x99DF,0x99DB,0x99DD,0x99D8,0x99D1,0x99ED,
	0x99EE,0x99F1,0x99F2,0x99FB,0x99F8,0x9A01,0x9A0F,0x9A05,
	0x99E2,0x9A19,0x9A2B,0x9A37,0x9A45,0x9A42,0x9A40,0x9A43,
	0x9A3E,0x9A55,0x9A4D,0x9A5B,0x9A57,0x9A5F,0x9A62,0x9A65,
	0x9A64,0x9A69,0x9A6B,0x9A6A,0x9AAD,0x9AB0,0x9ABC,0x9AC0,
	0x9ACF,0x9AD1,0x9AD3,0x9AD4,0x9ADE,0x9ADF,0x9AE2,0x9AE3,
	0x9AE6,0x9AEF,0x9AEB,0x9AEE,0x9AF4,0x9AF1,0x9AF7,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 115, Array index 0x4B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9AFB,0x9B06,0x9B18,0x9B1A,0x9B1F,0x9B22,0x9B23,
	0x9B25,0x9B27,0x9B28,0x9B29,0x9B2A,0x9B2E,0x9B2F,0x9B32,
	0x9B44,0x9B43,0x9B4F,0x9B4D,0x9B4E,0x9B51,0x9B58,0x9B74,
	0x9B93,0x9B83,0x9B91,0x9B96,0x9B97,0x9B9F,0x9BA0,0x9BA8,
	0x9BB4,0x9BC0,0x9BCA,0x9BB9,0x9BC6,0x9BCF,0x9BD1,0x9BD2,
	0x9BE3,0x9BE2,0x9BE4,0x9BD4,0x9BE1,0x9C3A,0x9BF2,0x9BF1,
	0x9BF0,0x9C15,0x9C14,0x9C09,0x9C13,0x9C0C,0x9C06,0x9C08,
	0x9C12,0x9C0A,0x9C04,0x9C2E,0x9C1B,0x9C25,0x9C24,0x9C21,
	0x9C30,0x9C47,0x9C32,0x9C46,0x9C3E,0x9C5A,0x9C60,0x9C67,
	0x9C76,0x9C78,0x9CE7,0x9CEC,0x9CF0,0x9D09,0x9D08,0x9CEB,
	0x9D03,0x9D06,0x9D2A,0x9D26,0x9DAF,0x9D23,0x9D1F,0x9D44,
	0x9D15,0x9D12,0x9D41,0x9D3F,0x9D3E,0x9D46,0x9D48,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 116, Array index 0x4C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9D5D,0x9D5E,0x9D64,0x9D51,0x9D50,0x9D59,0x9D72,
	0x9D89,0x9D87,0x9DAB,0x9D6F,0x9D7A,0x9D9A,0x9DA4,0x9DA9,
	0x9DB2,0x9DC4,0x9DC1,0x9DBB,0x9DB8,0x9DBA,0x9DC6,0x9DCF,
	0x9DC2,0x9DD9,0x9DD3,0x9DF8,0x9DE6,0x9DED,0x9DEF,0x9DFD,
	0x9E1A,0x9E1B,0x9E1E,0x9E75,0x9E79,0x9E7D,0x9E81,0x9E88,
	0x9E8B,0x9E8C,0x9E92,0x9E95,0x9E91,0x9E9D,0x9EA5,0x9EA9,
	0x9EB8,0x9EAA,0x9EAD,0x9761,0x9ECC,0x9ECE,0x9ECF,0x9ED0,
	0x9ED4,0x9EDC,0x9EDE,0x9EDD,0x9EE0,0x9EE5,0x9EE8,0x9EEF,
	0x9EF4,0x9EF6,0x9EF7,0x9EF9,0x9EFB,0x9EFC,0x9EFD,0x9F07,
	0x9F08,0x76B7,0x9F15,0x9F21,0x9F2C,0x9F3E,0x9F4A,0x9F52,
	0x9F54,0x9F63,0x9F5F,0x9F60,0x9F61,0x9F66,0x9F67,0x9F6C,
	0x9F6A,0x9F77,0x9F72,0x9F76,0x9F95,0x9F9C,0x9FA0,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 117, Array index 0x4D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x582F,0x69C7,0x9059,0x7464,0x51DC,0x7199,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
};

#endif /* ICONV_TO_UCS_CCS_JIS_X0208_1990 && !defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit jis_x0208_1990 -> UCS size-optimized table (14276 bytes).
 * ======================================================================
 */
#if defined (ICONV_TO_UCS_CCS_JIS_X0208_1990) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
to_ucs_size_jis_x0208_1990[] =
{
	0x0055, /* Ranges number */
	0x0001, /* Unranged codes number */
	0x1BE0, /* First unranged code index */
	/* Ranges list: first code, last Code, array index. */
	/* Array index: 0x0003 */ 0x2121, 0x217E, 0x0102,
	/* Array index: 0x0006 */ 0x2221, 0x222E, 0x0160,
	/* Array index: 0x0009 */ 0x223A, 0x2241, 0x016E,
	/* Array index: 0x000C */ 0x224A, 0x2250, 0x0176,
	/* Array index: 0x000F */ 0x225C, 0x226A, 0x017D,
	/* Array index: 0x0012 */ 0x2272, 0x2279, 0x018C,
	/* Array index: 0x0015 */ 0x2330, 0x2339, 0x0194,
	/* Array index: 0x0018 */ 0x2341, 0x235A, 0x019E,
	/* Array index: 0x001B */ 0x2361, 0x237A, 0x01B8,
	/* Array index: 0x001E */ 0x2421, 0x2473, 0x01D2,
	/* Array index: 0x0021 */ 0x2521, 0x2576, 0x0225,
	/* Array index: 0x0024 */ 0x2621, 0x2638, 0x027B,
	/* Array index: 0x0027 */ 0x2641, 0x2658, 0x0293,
	/* Array index: 0x002A */ 0x2721, 0x2741, 0x02AB,
	/* Array index: 0x002D */ 0x2751, 0x2771, 0x02CC,
	/* Array index: 0x0030 */ 0x2821, 0x2840, 0x02ED,
	/* Array index: 0x0033 */ 0x3021, 0x307E, 0x030D,
	/* Array index: 0x0036 */ 0x3121, 0x317E, 0x036B,
	/* Array index: 0x0039 */ 0x3221, 0x327E, 0x03C9,
	/* Array index: 0x003C */ 0x3321, 0x337E, 0x0427,
	/* Array index: 0x003F */ 0x3421, 0x347E, 0x0485,
	/* Array index: 0x0042 */ 0x3521, 0x357E, 0x04E3,
	/* Array index: 0x0045 */ 0x3621, 0x367E, 0x0541,
	/* Array index: 0x0048 */ 0x3721, 0x377E, 0x059F,
	/* Array index: 0x004B */ 0x3821, 0x387E, 0x05FD,
	/* Array index: 0x004E */ 0x3921, 0x397E, 0x065B,
	/* Array index: 0x0051 */ 0x3A21, 0x3A7E, 0x06B9,
	/* Array index: 0x0054 */ 0x3B21, 0x3B7E, 0x0717,
	/* Array index: 0x0057 */ 0x3C21, 0x3C7E, 0x0775,
	/* Array index: 0x005A */ 0x3D21, 0x3D7E, 0x07D3,
	/* Array index: 0x005D */ 0x3E21, 0x3E7E, 0x0831,
	/* Array index: 0x0060 */ 0x3F21, 0x3F7E, 0x088F,
	/* Array index: 0x0063 */ 0x4021, 0x407E, 0x08ED,
	/* Array index: 0x0066 */ 0x4121, 0x417E, 0x094B,
	/* Array index: 0x0069 */ 0x4221, 0x427E, 0x09A9,
	/* Array index: 0x006C */ 0x4321, 0x437E, 0x0A07,
	/* Array index: 0x006F */ 0x4421, 0x447E, 0x0A65,
	/* Array index: 0x0072 */ 0x4521, 0x457E, 0x0AC3,
	/* Array index: 0x0075 */ 0x4621, 0x467E, 0x0B21,
	/* Array index: 0x0078 */ 0x4721, 0x477E, 0x0B7F,
	/* Array index: 0x007B */ 0x4821, 0x487E, 0x0BDD,
	/* Array index: 0x007E */ 0x4921, 0x497E, 0x0C3B,
	/* Array index: 0x0081 */ 0x4A21, 0x4A7E, 0x0C99,
	/* Array index: 0x0084 */ 0x4B21, 0x4B7E, 0x0CF7,
	/* Array index: 0x0087 */ 0x4C21, 0x4C7E, 0x0D55,
	/* Array index: 0x008A */ 0x4D21, 0x4D7E, 0x0DB3,
	/* Array index: 0x008D */ 0x4E21, 0x4E7E, 0x0E11,
	/* Array index: 0x0090 */ 0x4F21, 0x4F53, 0x0E6F,
	/* Array index: 0x0093 */ 0x5021, 0x507E, 0x0EA2,
	/* Array index: 0x0096 */ 0x5121, 0x517E, 0x0F00,
	/* Array index: 0x0099 */ 0x5221, 0x527E, 0x0F5E,
	/* Array index: 0x009C */ 0x5321, 0x537E, 0x0FBC,
	/* Array index: 0x009F */ 0x5421, 0x547E, 0x101A,
	/* Array index: 0x00A2 */ 0x5521, 0x557E, 0x1078,
	/* Array index: 0x00A5 */ 0x5621, 0x567E, 0x10D6,
	/* Array index: 0x00A8 */ 0x5721, 0x577E, 0x1134,
	/* Array index: 0x00AB */ 0x5821, 0x587E, 0x1192,
	/* Array index: 0x00AE */ 0x5921, 0x597E, 0x11F0,
	/* Array index: 0x00B1 */ 0x5A21, 0x5A7E, 0x124E,
	/* Array index: 0x00B4 */ 0x5B21, 0x5B7E, 0x12AC,
	/* Array index: 0x00B7 */ 0x5C21, 0x5C7E, 0x130A,
	/* Array index: 0x00BA */ 0x5D21, 0x5D7E, 0x1368,
	/* Array index: 0x00BD */ 0x5E21, 0x5E7E, 0x13C6,
	/* Array index: 0x00C0 */ 0x5F21, 0x5F7E, 0x1424,
	/* Array index: 0x00C3 */ 0x6021, 0x607E, 0x1482,
	/* Array index: 0x00C6 */ 0x6121, 0x617E, 0x14E0,
	/* Array index: 0x00C9 */ 0x6221, 0x627E, 0x153E,
	/* Array index: 0x00CC */ 0x6321, 0x637E, 0x159C,
	/* Array index: 0x00CF */ 0x6421, 0x647E, 0x15FA,
	/* Array index: 0x00D2 */ 0x6521, 0x657E, 0x1658,
	/* Array index: 0x00D5 */ 0x6621, 0x667E, 0x16B6,
	/* Array index: 0x00D8 */ 0x6721, 0x677E, 0x1714,
	/* Array index: 0x00DB */ 0x6821, 0x687E, 0x1772,
	/* Array index: 0x00DE */ 0x6921, 0x697E, 0x17D0,
	/* Array index: 0x00E1 */ 0x6A21, 0x6A7E, 0x182E,
	/* Array index: 0x00E4 */ 0x6B21, 0x6B7E, 0x188C,
	/* Array index: 0x00E7 */ 0x6C21, 0x6C7E, 0x18EA,
	/* Array index: 0x00EA */ 0x6D21, 0x6D7E, 0x1948,
	/* Array index: 0x00ED */ 0x6E21, 0x6E7E, 0x19A6,
	/* Array index: 0x00F0 */ 0x6F21, 0x6F7E, 0x1A04,
	/* Array index: 0x00F3 */ 0x7021, 0x707E, 0x1A62,
	/* Array index: 0x00F6 */ 0x7121, 0x717E, 0x1AC0,
	/* Array index: 0x00F9 */ 0x7221, 0x727E, 0x1B1E,
	/* Array index: 0x00FC */ 0x7321, 0x737E, 0x1B7C,
	/* Array index: 0x00FF */ 0x7421, 0x7426, 0x1BDA,
	/* Ranges content */
	/* Range 0x2121 - 0x217E, array index: 0x0102 */
	0x3000,0x3001,0x3002,0xFF0C,0xFF0E,0x30FB,0xFF1A,0xFF1B,
	0xFF1F,0xFF01,0x309B,0x309C,0x00B4,0xFF40,0x00A8,0xFF3E,
	0xFFE3,0xFF3F,0x30FD,0x30FE,0x309D,0x309E,0x3003,0x4EDD,
	0x3005,0x3006,0x3007,0x30FC,0x2015,0x2010,0xFF0F,0x005C,
	0x301C,0x2016,0xFF5C,0x2026,0x2025,0x2018,0x2019,0x201C,
	0x201D,0xFF08,0xFF09,0x3014,0x3015,0xFF3B,0xFF3D,0xFF5B,
	0xFF5D,0x3008,0x3009,0x300A,0x300B,0x300C,0x300D,0x300E,
	0x300F,0x3010,0x3011,0xFF0B,0x2212,0x00B1,0x00D7,0x00F7,
	0xFF1D,0x2260,0xFF1C,0xFF1E,0x2266,0x2267,0x221E,0x2234,
	0x2642,0x2640,0x00B0,0x2032,0x2033,0x2103,0xFFE5,0xFF04,
	0x00A2,0x00A3,0xFF05,0xFF03,0xFF06,0xFF0A,0xFF20,0x00A7,
	0x2606,0x2605,0x25CB,0x25CF,0x25CE,0x25C7,
	/* Range 0x2221 - 0x222E, array index: 0x0160 */
	0x25C6,0x25A1,0x25A0,0x25B3,0x25B2,0x25BD,0x25BC,0x203B,
	0x3012,0x2192,0x2190,0x2191,0x2193,0x3013,
	/* Range 0x223A - 0x2241, array index: 0x016E */
	0x2208,0x220B,0x2286,0x2287,0x2282,0x2283,0x222A,0x2229,
	/* Range 0x224A - 0x2250, array index: 0x0176 */
	0x2227,0x2228,0x00AC,0x21D2,0x21D4,0x2200,0x2203,
	/* Range 0x225C - 0x226A, array index: 0x017D */
	0x2220,0x22A5,0x2312,0x2202,0x2207,0x2261,0x2252,0x226A,
	0x226B,0x221A,0x223D,0x221D,0x2235,0x222B,0x222C,
	/* Range 0x2272 - 0x2279, array index: 0x018C */
	0x212B,0x2030,0x266F,0x266D,0x266A,0x2020,0x2021,0x00B6,
	/* Range 0x2330 - 0x2339, array index: 0x0194 */
	0xFF10,0xFF11,0xFF12,0xFF13,0xFF14,0xFF15,0xFF16,0xFF17,
	0xFF18,0xFF19,
	/* Range 0x2341 - 0x235A, array index: 0x019E */
	0xFF21,0xFF22,0xFF23,0xFF24,0xFF25,0xFF26,0xFF27,0xFF28,
	0xFF29,0xFF2A,0xFF2B,0xFF2C,0xFF2D,0xFF2E,0xFF2F,0xFF30,
	0xFF31,0xFF32,0xFF33,0xFF34,0xFF35,0xFF36,0xFF37,0xFF38,
	0xFF39,0xFF3A,
	/* Range 0x2361 - 0x237A, array index: 0x01B8 */
	0xFF41,0xFF42,0xFF43,0xFF44,0xFF45,0xFF46,0xFF47,0xFF48,
	0xFF49,0xFF4A,0xFF4B,0xFF4C,0xFF4D,0xFF4E,0xFF4F,0xFF50,
	0xFF51,0xFF52,0xFF53,0xFF54,0xFF55,0xFF56,0xFF57,0xFF58,
	0xFF59,0xFF5A,
	/* Range 0x2421 - 0x2473, array index: 0x01D2 */
	0x3041,0x3042,0x3043,0x3044,0x3045,0x3046,0x3047,0x3048,
	0x3049,0x304A,0x304B,0x304C,0x304D,0x304E,0x304F,0x3050,
	0x3051,0x3052,0x3053,0x3054,0x3055,0x3056,0x3057,0x3058,
	0x3059,0x305A,0x305B,0x305C,0x305D,0x305E,0x305F,0x3060,
	0x3061,0x3062,0x3063,0x3064,0x3065,0x3066,0x3067,0x3068,
	0x3069,0x306A,0x306B,0x306C,0x306D,0x306E,0x306F,0x3070,
	0x3071,0x3072,0x3073,0x3074,0x3075,0x3076,0x3077,0x3078,
	0x3079,0x307A,0x307B,0x307C,0x307D,0x307E,0x307F,0x3080,
	0x3081,0x3082,0x3083,0x3084,0x3085,0x3086,0x3087,0x3088,
	0x3089,0x308A,0x308B,0x308C,0x308D,0x308E,0x308F,0x3090,
	0x3091,0x3092,0x3093,
	/* Range 0x2521 - 0x2576, array index: 0x0225 */
	0x30A1,0x30A2,0x30A3,0x30A4,0x30A5,0x30A6,0x30A7,0x30A8,
	0x30A9,0x30AA,0x30AB,0x30AC,0x30AD,0x30AE,0x30AF,0x30B0,
	0x30B1,0x30B2,0x30B3,0x30B4,0x30B5,0x30B6,0x30B7,0x30B8,
	0x30B9,0x30BA,0x30BB,0x30BC,0x30BD,0x30BE,0x30BF,0x30C0,
	0x30C1,0x30C2,0x30C3,0x30C4,0x30C5,0x30C6,0x30C7,0x30C8,
	0x30C9,0x30CA,0x30CB,0x30CC,0x30CD,0x30CE,0x30CF,0x30D0,
	0x30D1,0x30D2,0x30D3,0x30D4,0x30D5,0x30D6,0x30D7,0x30D8,
	0x30D9,0x30DA,0x30DB,0x30DC,0x30DD,0x30DE,0x30DF,0x30E0,
	0x30E1,0x30E2,0x30E3,0x30E4,0x30E5,0x30E6,0x30E7,0x30E8,
	0x30E9,0x30EA,0x30EB,0x30EC,0x30ED,0x30EE,0x30EF,0x30F0,
	0x30F1,0x30F2,0x30F3,0x30F4,0x30F5,0x30F6,
	/* Range 0x2621 - 0x2638, array index: 0x027B */
	0x0391,0x0392,0x0393,0x0394,0x0395,0x0396,0x0397,0x0398,
	0x0399,0x039A,0x039B,0x039C,0x039D,0x039E,0x039F,0x03A0,
	0x03A1,0x03A3,0x03A4,0x03A5,0x03A6,0x03A7,0x03A8,0x03A9,
	/* Range 0x2641 - 0x2658, array index: 0x0293 */
	0x03B1,0x03B2,0x03B3,0x03B4,0x03B5,0x03B6,0x03B7,0x03B8,
	0x03B9,0x03BA,0x03BB,0x03BC,0x03BD,0x03BE,0x03BF,0x03C0,
	0x03C1,0x03C3,0x03C4,0x03C5,0x03C6,0x03C7,0x03C8,0x03C9,
	/* Range 0x2721 - 0x2741, array index: 0x02AB */
	0x0410,0x0411,0x0412,0x0413,0x0414,0x0415,0x0401,0x0416,
	0x0417,0x0418,0x0419,0x041A,0x041B,0x041C,0x041D,0x041E,
	0x041F,0x0420,0x0421,0x0422,0x0423,0x0424,0x0425,0x0426,
	0x0427,0x0428,0x0429,0x042A,0x042B,0x042C,0x042D,0x042E,
	0x042F,
	/* Range 0x2751 - 0x2771, array index: 0x02CC */
	0x0430,0x0431,0x0432,0x0433,0x0434,0x0435,0x0451,0x0436,
	0x0437,0x0438,0x0439,0x043A,0x043B,0x043C,0x043D,0x043E,
	0x043F,0x0440,0x0441,0x0442,0x0443,0x0444,0x0445,0x0446,
	0x0447,0x0448,0x0449,0x044A,0x044B,0x044C,0x044D,0x044E,
	0x044F,
	/* Range 0x2821 - 0x2840, array index: 0x02ED */
	0x2500,0x2502,0x250C,0x2510,0x2518,0x2514,0x251C,0x252C,
	0x2524,0x2534,0x253C,0x2501,0x2503,0x250F,0x2513,0x251B,
	0x2517,0x2523,0x2533,0x252B,0x253B,0x254B,0x2520,0x252F,
	0x2528,0x2537,0x253F,0x251D,0x2530,0x2525,0x2538,0x2542,
	/* Range 0x3021 - 0x307E, array index: 0x030D */
	0x4E9C,0x5516,0x5A03,0x963F,0x54C0,0x611B,0x6328,0x59F6,
	0x9022,0x8475,0x831C,0x7A50,0x60AA,0x63E1,0x6E25,0x65ED,
	0x8466,0x82A6,0x9BF5,0x6893,0x5727,0x65A1,0x6271,0x5B9B,
	0x59D0,0x867B,0x98F4,0x7D62,0x7DBE,0x9B8E,0x6216,0x7C9F,
	0x88B7,0x5B89,0x5EB5,0x6309,0x6697,0x6848,0x95C7,0x978D,
	0x674F,0x4EE5,0x4F0A,0x4F4D,0x4F9D,0x5049,0x56F2,0x5937,
	0x59D4,0x5A01,0x5C09,0x60DF,0x610F,0x6170,0x6613,0x6905,
	0x70BA,0x754F,0x7570,0x79FB,0x7DAD,0x7DEF,0x80C3,0x840E,
	0x8863,0x8B02,0x9055,0x907A,0x533B,0x4E95,0x4EA5,0x57DF,
	0x80B2,0x90C1,0x78EF,0x4E00,0x58F1,0x6EA2,0x9038,0x7A32,
	0x8328,0x828B,0x9C2F,0x5141,0x5370,0x54BD,0x54E1,0x56E0,
	0x59FB,0x5F15,0x98F2,0x6DEB,0x80E4,0x852D,
	/* Range 0x3121 - 0x317E, array index: 0x036B */
	0x9662,0x9670,0x96A0,0x97FB,0x540B,0x53F3,0x5B87,0x70CF,
	0x7FBD,0x8FC2,0x96E8,0x536F,0x9D5C,0x7ABA,0x4E11,0x7893,
	0x81FC,0x6E26,0x5618,0x5504,0x6B1D,0x851A,0x9C3B,0x59E5,
	0x53A9,0x6D66,0x74DC,0x958F,0x5642,0x4E91,0x904B,0x96F2,
	0x834F,0x990C,0x53E1,0x55B6,0x5B30,0x5F71,0x6620,0x66F3,
	0x6804,0x6C38,0x6CF3,0x6D29,0x745B,0x76C8,0x7A4E,0x9834,
	0x82F1,0x885B,0x8A60,0x92ED,0x6DB2,0x75AB,0x76CA,0x99C5,
	0x60A6,0x8B01,0x8D8A,0x95B2,0x698E,0x53AD,0x5186,0x5712,
	0x5830,0x5944,0x5BB4,0x5EF6,0x6028,0x63A9,0x63F4,0x6CBF,
	0x6F14,0x708E,0x7114,0x7159,0x71D5,0x733F,0x7E01,0x8276,
	0x82D1,0x8597,0x9060,0x925B,0x9D1B,0x5869,0x65BC,0x6C5A,
	0x7525,0x51F9,0x592E,0x5965,0x5F80,0x5FDC,
	/* Range 0x3221 - 0x327E, array index: 0x03C9 */
	0x62BC,0x65FA,0x6A2A,0x6B27,0x6BB4,0x738B,0x7FC1,0x8956,
	0x9D2C,0x9D0E,0x9EC4,0x5CA1,0x6C96,0x837B,0x5104,0x5C4B,
	0x61B6,0x81C6,0x6876,0x7261,0x4E59,0x4FFA,0x5378,0x6069,
	0x6E29,0x7A4F,0x97F3,0x4E0B,0x5316,0x4EEE,0x4F55,0x4F3D,
	0x4FA1,0x4F73,0x52A0,0x53EF,0x5609,0x590F,0x5AC1,0x5BB6,
	0x5BE1,0x79D1,0x6687,0x679C,0x67B6,0x6B4C,0x6CB3,0x706B,
	0x73C2,0x798D,0x79BE,0x7A3C,0x7B87,0x82B1,0x82DB,0x8304,
	0x8377,0x83EF,0x83D3,0x8766,0x8AB2,0x5629,0x8CA8,0x8FE6,
	0x904E,0x971E,0x868A,0x4FC4,0x5CE8,0x6211,0x7259,0x753B,
	0x81E5,0x82BD,0x86FE,0x8CC0,0x96C5,0x9913,0x99D5,0x4ECB,
	0x4F1A,0x89E3,0x56DE,0x584A,0x58CA,0x5EFB,0x5FEB,0x602A,
	0x6094,0x6062,0x61D0,0x6212,0x62D0,0x6539,
	/* Range 0x3321 - 0x337E, array index: 0x0427 */
	0x9B41,0x6666,0x68B0,0x6D77,0x7070,0x754C,0x7686,0x7D75,
	0x82A5,0x87F9,0x958B,0x968E,0x8C9D,0x51F1,0x52BE,0x5916,
	0x54B3,0x5BB3,0x5D16,0x6168,0x6982,0x6DAF,0x788D,0x84CB,
	0x8857,0x8A72,0x93A7,0x9AB8,0x6D6C,0x99A8,0x86D9,0x57A3,
	0x67FF,0x86CE,0x920E,0x5283,0x5687,0x5404,0x5ED3,0x62E1,
	0x64B9,0x683C,0x6838,0x6BBB,0x7372,0x78BA,0x7A6B,0x899A,
	0x89D2,0x8D6B,0x8F03,0x90ED,0x95A3,0x9694,0x9769,0x5B66,
	0x5CB3,0x697D,0x984D,0x984E,0x639B,0x7B20,0x6A2B,0x6A7F,
	0x68B6,0x9C0D,0x6F5F,0x5272,0x559D,0x6070,0x62EC,0x6D3B,
	0x6E07,0x6ED1,0x845B,0x8910,0x8F44,0x4E14,0x9C39,0x53F6,
	0x691B,0x6A3A,0x9784,0x682A,0x515C,0x7AC3,0x84B2,0x91DC,
	0x938C,0x565B,0x9D28,0x6822,0x8305,0x8431,
	/* Range 0x3421 - 0x347E, array index: 0x0485 */
	0x7CA5,0x5208,0x82C5,0x74E6,0x4E7E,0x4F83,0x51A0,0x5BD2,
	0x520A,0x52D8,0x52E7,0x5DFB,0x559A,0x582A,0x59E6,0x5B8C,
	0x5B98,0x5BDB,0x5E72,0x5E79,0x60A3,0x611F,0x6163,0x61BE,
	0x63DB,0x6562,0x67D1,0x6853,0x68FA,0x6B3E,0x6B53,0x6C57,
	0x6F22,0x6F97,0x6F45,0x74B0,0x7518,0x76E3,0x770B,0x7AFF,
	0x7BA1,0x7C21,0x7DE9,0x7F36,0x7FF0,0x809D,0x8266,0x839E,
	0x89B3,0x8ACC,0x8CAB,0x9084,0x9451,0x9593,0x9591,0x95A2,
	0x9665,0x97D3,0x9928,0x8218,0x4E38,0x542B,0x5CB8,0x5DCC,
	0x73A9,0x764C,0x773C,0x5CA9,0x7FEB,0x8D0B,0x96C1,0x9811,
	0x9854,0x9858,0x4F01,0x4F0E,0x5371,0x559C,0x5668,0x57FA,
	0x5947,0x5B09,0x5BC4,0x5C90,0x5E0C,0x5E7E,0x5FCC,0x63EE,
	0x673A,0x65D7,0x65E2,0x671F,0x68CB,0x68C4,
	/* Range 0x3521 - 0x357E, array index: 0x04E3 */
	0x6A5F,0x5E30,0x6BC5,0x6C17,0x6C7D,0x757F,0x7948,0x5B63,
	0x7A00,0x7D00,0x5FBD,0x898F,0x8A18,0x8CB4,0x8D77,0x8ECC,
	0x8F1D,0x98E2,0x9A0E,0x9B3C,0x4E80,0x507D,0x5100,0x5993,
	0x5B9C,0x622F,0x6280,0x64EC,0x6B3A,0x72A0,0x7591,0x7947,
	0x7FA9,0x87FB,0x8ABC,0x8B70,0x63AC,0x83CA,0x97A0,0x5409,
	0x5403,0x55AB,0x6854,0x6A58,0x8A70,0x7827,0x6775,0x9ECD,
	0x5374,0x5BA2,0x811A,0x8650,0x9006,0x4E18,0x4E45,0x4EC7,
	0x4F11,0x53CA,0x5438,0x5BAE,0x5F13,0x6025,0x6551,0x673D,
	0x6C42,0x6C72,0x6CE3,0x7078,0x7403,0x7A76,0x7AAE,0x7B08,
	0x7D1A,0x7CFE,0x7D66,0x65E7,0x725B,0x53BB,0x5C45,0x5DE8,
	0x62D2,0x62E0,0x6319,0x6E20,0x865A,0x8A31,0x8DDD,0x92F8,
	0x6F01,0x79A6,0x9B5A,0x4EA8,0x4EAB,0x4EAC,
	/* Range 0x3621 - 0x367E, array index: 0x0541 */
	0x4F9B,0x4FA0,0x50D1,0x5147,0x7AF6,0x5171,0x51F6,0x5354,
	0x5321,0x537F,0x53EB,0x55AC,0x5883,0x5CE1,0x5F37,0x5F4A,
	0x602F,0x6050,0x606D,0x631F,0x6559,0x6A4B,0x6CC1,0x72C2,
	0x72ED,0x77EF,0x80F8,0x8105,0x8208,0x854E,0x90F7,0x93E1,
	0x97FF,0x9957,0x9A5A,0x4EF0,0x51DD,0x5C2D,0x6681,0x696D,
	0x5C40,0x66F2,0x6975,0x7389,0x6850,0x7C81,0x50C5,0x52E4,
	0x5747,0x5DFE,0x9326,0x65A4,0x6B23,0x6B3D,0x7434,0x7981,
	0x79BD,0x7B4B,0x7DCA,0x82B9,0x83CC,0x887F,0x895F,0x8B39,
	0x8FD1,0x91D1,0x541F,0x9280,0x4E5D,0x5036,0x53E5,0x533A,
	0x72D7,0x7396,0x77E9,0x82E6,0x8EAF,0x99C6,0x99C8,0x99D2,
	0x5177,0x611A,0x865E,0x55B0,0x7A7A,0x5076,0x5BD3,0x9047,
	0x9685,0x4E32,0x6ADB,0x91E7,0x5C51,0x5C48,
	/* Range 0x3721 - 0x377E, array index: 0x059F */
	0x6398,0x7A9F,0x6C93,0x9774,0x8F61,0x7AAA,0x718A,0x9688,
	0x7C82,0x6817,0x7E70,0x6851,0x936C,0x52F2,0x541B,0x85AB,
	0x8A13,0x7FA4,0x8ECD,0x90E1,0x5366,0x8888,0x7941,0x4FC2,
	0x50BE,0x5211,0x5144,0x5553,0x572D,0x73EA,0x578B,0x5951,
	0x5F62,0x5F84,0x6075,0x6176,0x6167,0x61A9,0x63B2,0x643A,
	0x656C,0x666F,0x6842,0x6E13,0x7566,0x7A3D,0x7CFB,0x7D4C,
	0x7D99,0x7E4B,0x7F6B,0x830E,0x834A,0x86CD,0x8A08,0x8A63,
	0x8B66,0x8EFD,0x981A,0x9D8F,0x82B8,0x8FCE,0x9BE8,0x5287,
	0x621F,0x6483,0x6FC0,0x9699,0x6841,0x5091,0x6B20,0x6C7A,
	0x6F54,0x7A74,0x7D50,0x8840,0x8A23,0x6708,0x4EF6,0x5039,
	0x5026,0x5065,0x517C,0x5238,0x5263,0x55A7,0x570F,0x5805,
	0x5ACC,0x5EFA,0x61B2,0x61F8,0x62F3,0x6372,
	/* Range 0x3821 - 0x387E, array index: 0x05FD */
	0x691C,0x6A29,0x727D,0x72AC,0x732E,0x7814,0x786F,0x7D79,
	0x770C,0x80A9,0x898B,0x8B19,0x8CE2,0x8ED2,0x9063,0x9375,
	0x967A,0x9855,0x9A13,0x9E78,0x5143,0x539F,0x53B3,0x5E7B,
	0x5F26,0x6E1B,0x6E90,0x7384,0x73FE,0x7D43,0x8237,0x8A00,
	0x8AFA,0x9650,0x4E4E,0x500B,0x53E4,0x547C,0x56FA,0x59D1,
	0x5B64,0x5DF1,0x5EAB,0x5F27,0x6238,0x6545,0x67AF,0x6E56,
	0x72D0,0x7CCA,0x88B4,0x80A1,0x80E1,0x83F0,0x864E,0x8A87,
	0x8DE8,0x9237,0x96C7,0x9867,0x9F13,0x4E94,0x4E92,0x4F0D,
	0x5348,0x5449,0x543E,0x5A2F,0x5F8C,0x5FA1,0x609F,0x68A7,
	0x6A8E,0x745A,0x7881,0x8A9E,0x8AA4,0x8B77,0x9190,0x4E5E,
	0x9BC9,0x4EA4,0x4F7C,0x4FAF,0x5019,0x5016,0x5149,0x516C,
	0x529F,0x52B9,0x52FE,0x539A,0x53E3,0x5411,
	/* Range 0x3921 - 0x397E, array index: 0x065B */
	0x540E,0x5589,0x5751,0x57A2,0x597D,0x5B54,0x5B5D,0x5B8F,
	0x5DE5,0x5DE7,0x5DF7,0x5E78,0x5E83,0x5E9A,0x5EB7,0x5F18,
	0x6052,0x614C,0x6297,0x62D8,0x63A7,0x653B,0x6602,0x6643,
	0x66F4,0x676D,0x6821,0x6897,0x69CB,0x6C5F,0x6D2A,0x6D69,
	0x6E2F,0x6E9D,0x7532,0x7687,0x786C,0x7A3F,0x7CE0,0x7D05,
	0x7D18,0x7D5E,0x7DB1,0x8015,0x8003,0x80AF,0x80B1,0x8154,
	0x818F,0x822A,0x8352,0x884C,0x8861,0x8B1B,0x8CA2,0x8CFC,
	0x90CA,0x9175,0x9271,0x783F,0x92FC,0x95A4,0x964D,0x9805,
	0x9999,0x9AD8,0x9D3B,0x525B,0x52AB,0x53F7,0x5408,0x58D5,
	0x62F7,0x6FE0,0x8C6A,0x8F5F,0x9EB9,0x514B,0x523B,0x544A,
	0x56FD,0x7A40,0x9177,0x9D60,0x9ED2,0x7344,0x6F09,0x8170,
	0x7511,0x5FFD,0x60DA,0x9AA8,0x72DB,0x8FBC,
	/* Range 0x3A21 - 0x3A7E, array index: 0x06B9 */
	0x6B64,0x9803,0x4ECA,0x56F0,0x5764,0x58BE,0x5A5A,0x6068,
	0x61C7,0x660F,0x6606,0x6839,0x68B1,0x6DF7,0x75D5,0x7D3A,
	0x826E,0x9B42,0x4E9B,0x4F50,0x53C9,0x5506,0x5D6F,0x5DE6,
	0x5DEE,0x67FB,0x6C99,0x7473,0x7802,0x8A50,0x9396,0x88DF,
	0x5750,0x5EA7,0x632B,0x50B5,0x50AC,0x518D,0x6700,0x54C9,
	0x585E,0x59BB,0x5BB0,0x5F69,0x624D,0x63A1,0x683D,0x6B73,
	0x6E08,0x707D,0x91C7,0x7280,0x7815,0x7826,0x796D,0x658E,
	0x7D30,0x83DC,0x88C1,0x8F09,0x969B,0x5264,0x5728,0x6750,
	0x7F6A,0x8CA1,0x51B4,0x5742,0x962A,0x583A,0x698A,0x80B4,
	0x54B2,0x5D0E,0x57FC,0x7895,0x9DFA,0x4F5C,0x524A,0x548B,
	0x643E,0x6628,0x6714,0x67F5,0x7A84,0x7B56,0x7D22,0x932F,
	0x685C,0x9BAD,0x7B39,0x5319,0x518A,0x5237,
	/* Range 0x3B21 - 0x3B7E, array index: 0x0717 */
	0x5BDF,0x62F6,0x64AE,0x64E6,0x672D,0x6BBA,0x85A9,0x96D1,
	0x7690,0x9BD6,0x634C,0x9306,0x9BAB,0x76BF,0x6652,0x4E09,
	0x5098,0x53C2,0x5C71,0x60E8,0x6492,0x6563,0x685F,0x71E6,
	0x73CA,0x7523,0x7B97,0x7E82,0x8695,0x8B83,0x8CDB,0x9178,
	0x9910,0x65AC,0x66AB,0x6B8B,0x4ED5,0x4ED4,0x4F3A,0x4F7F,
	0x523A,0x53F8,0x53F2,0x55E3,0x56DB,0x58EB,0x59CB,0x59C9,
	0x59FF,0x5B50,0x5C4D,0x5E02,0x5E2B,0x5FD7,0x601D,0x6307,
	0x652F,0x5B5C,0x65AF,0x65BD,0x65E8,0x679D,0x6B62,0x6B7B,
	0x6C0F,0x7345,0x7949,0x79C1,0x7CF8,0x7D19,0x7D2B,0x80A2,
	0x8102,0x81F3,0x8996,0x8A5E,0x8A69,0x8A66,0x8A8C,0x8AEE,
	0x8CC7,0x8CDC,0x96CC,0x98FC,0x6B6F,0x4E8B,0x4F3C,0x4F8D,
	0x5150,0x5B57,0x5BFA,0x6148,0x6301,0x6642,
	/* Range 0x3C21 - 0x3C7E, array index: 0x0775 */
	0x6B21,0x6ECB,0x6CBB,0x723E,0x74BD,0x75D4,0x78C1,0x793A,
	0x800C,0x8033,0x81EA,0x8494,0x8F9E,0x6C50,0x9E7F,0x5F0F,
	0x8B58,0x9D2B,0x7AFA,0x8EF8,0x5B8D,0x96EB,0x4E03,0x53F1,
	0x57F7,0x5931,0x5AC9,0x5BA4,0x6089,0x6E7F,0x6F06,0x75BE,
	0x8CEA,0x5B9F,0x8500,0x7BE0,0x5072,0x67F4,0x829D,0x5C61,
	0x854A,0x7E1E,0x820E,0x5199,0x5C04,0x6368,0x8D66,0x659C,
	0x716E,0x793E,0x7D17,0x8005,0x8B1D,0x8ECA,0x906E,0x86C7,
	0x90AA,0x501F,0x52FA,0x5C3A,0x6753,0x707C,0x7235,0x914C,
	0x91C8,0x932B,0x82E5,0x5BC2,0x5F31,0x60F9,0x4E3B,0x53D6,
	0x5B88,0x624B,0x6731,0x6B8A,0x72E9,0x73E0,0x7A2E,0x816B,
	0x8DA3,0x9152,0x9996,0x5112,0x53D7,0x546A,0x5BFF,0x6388,
	0x6A39,0x7DAC,0x9700,0x56DA,0x53CE,0x5468,
	/* Range 0x3D21 - 0x3D7E, array index: 0x07D3 */
	0x5B97,0x5C31,0x5DDE,0x4FEE,0x6101,0x62FE,0x6D32,0x79C0,
	0x79CB,0x7D42,0x7E4D,0x7FD2,0x81ED,0x821F,0x8490,0x8846,
	0x8972,0x8B90,0x8E74,0x8F2F,0x9031,0x914B,0x916C,0x96C6,
	0x919C,0x4EC0,0x4F4F,0x5145,0x5341,0x5F93,0x620E,0x67D4,
	0x6C41,0x6E0B,0x7363,0x7E26,0x91CD,0x9283,0x53D4,0x5919,
	0x5BBF,0x6DD1,0x795D,0x7E2E,0x7C9B,0x587E,0x719F,0x51FA,
	0x8853,0x8FF0,0x4FCA,0x5CFB,0x6625,0x77AC,0x7AE3,0x821C,
	0x99FF,0x51C6,0x5FAA,0x65EC,0x696F,0x6B89,0x6DF3,0x6E96,
	0x6F64,0x76FE,0x7D14,0x5DE1,0x9075,0x9187,0x9806,0x51E6,
	0x521D,0x6240,0x6691,0x66D9,0x6E1A,0x5EB6,0x7DD2,0x7F72,
	0x66F8,0x85AF,0x85F7,0x8AF8,0x52A9,0x53D9,0x5973,0x5E8F,
	0x5F90,0x6055,0x92E4,0x9664,0x50B7,0x511F,
	/* Range 0x3E21 - 0x3E7E, array index: 0x0831 */
	0x52DD,0x5320,0x5347,0x53EC,0x54E8,0x5546,0x5531,0x5617,
	0x5968,0x59BE,0x5A3C,0x5BB5,0x5C06,0x5C0F,0x5C11,0x5C1A,
	0x5E84,0x5E8A,0x5EE0,0x5F70,0x627F,0x6284,0x62DB,0x638C,
	0x6377,0x6607,0x660C,0x662D,0x6676,0x677E,0x68A2,0x6A1F,
	0x6A35,0x6CBC,0x6D88,0x6E09,0x6E58,0x713C,0x7126,0x7167,
	0x75C7,0x7701,0x785D,0x7901,0x7965,0x79F0,0x7AE0,0x7B11,
	0x7CA7,0x7D39,0x8096,0x83D6,0x848B,0x8549,0x885D,0x88F3,
	0x8A1F,0x8A3C,0x8A54,0x8A73,0x8C61,0x8CDE,0x91A4,0x9266,
	0x937E,0x9418,0x969C,0x9798,0x4E0A,0x4E08,0x4E1E,0x4E57,
	0x5197,0x5270,0x57CE,0x5834,0x58CC,0x5B22,0x5E38,0x60C5,
	0x64FE,0x6761,0x6756,0x6D44,0x72B6,0x7573,0x7A63,0x84B8,
	0x8B72,0x91B8,0x9320,0x5631,0x57F4,0x98FE,
	/* Range 0x3F21 - 0x3F7E, array index: 0x088F */
	0x62ED,0x690D,0x6B96,0x71ED,0x7E54,0x8077,0x8272,0x89E6,
	0x98DF,0x8755,0x8FB1,0x5C3B,0x4F38,0x4FE1,0x4FB5,0x5507,
	0x5A20,0x5BDD,0x5BE9,0x5FC3,0x614E,0x632F,0x65B0,0x664B,
	0x68EE,0x699B,0x6D78,0x6DF1,0x7533,0x75B9,0x771F,0x795E,
	0x79E6,0x7D33,0x81E3,0x82AF,0x85AA,0x89AA,0x8A3A,0x8EAB,
	0x8F9B,0x9032,0x91DD,0x9707,0x4EBA,0x4EC1,0x5203,0x5875,
	0x58EC,0x5C0B,0x751A,0x5C3D,0x814E,0x8A0A,0x8FC5,0x9663,
	0x976D,0x7B25,0x8ACF,0x9808,0x9162,0x56F3,0x53A8,0x9017,
	0x5439,0x5782,0x5E25,0x63A8,0x6C34,0x708A,0x7761,0x7C8B,
	0x7FE0,0x8870,0x9042,0x9154,0x9310,0x9318,0x968F,0x745E,
	0x9AC4,0x5D07,0x5D69,0x6570,0x67A2,0x8DA8,0x96DB,0x636E,
	0x6749,0x6919,0x83C5,0x9817,0x96C0,0x88FE,
	/* Range 0x4021 - 0x407E, array index: 0x08ED */
	0x6F84,0x647A,0x5BF8,0x4E16,0x702C,0x755D,0x662F,0x51C4,
	0x5236,0x52E2,0x59D3,0x5F81,0x6027,0x6210,0x653F,0x6574,
	0x661F,0x6674,0x68F2,0x6816,0x6B63,0x6E05,0x7272,0x751F,
	0x76DB,0x7CBE,0x8056,0x58F0,0x88FD,0x897F,0x8AA0,0x8A93,
	0x8ACB,0x901D,0x9192,0x9752,0x9759,0x6589,0x7A0E,0x8106,
	0x96BB,0x5E2D,0x60DC,0x621A,0x65A5,0x6614,0x6790,0x77F3,
	0x7A4D,0x7C4D,0x7E3E,0x810A,0x8CAC,0x8D64,0x8DE1,0x8E5F,
	0x78A9,0x5207,0x62D9,0x63A5,0x6442,0x6298,0x8A2D,0x7A83,
	0x7BC0,0x8AAC,0x96EA,0x7D76,0x820C,0x8749,0x4ED9,0x5148,
	0x5343,0x5360,0x5BA3,0x5C02,0x5C16,0x5DDD,0x6226,0x6247,
	0x64B0,0x6813,0x6834,0x6CC9,0x6D45,0x6D17,0x67D3,0x6F5C,
	0x714E,0x717D,0x65CB,0x7A7F,0x7BAD,0x7DDA,
	/* Range 0x4121 - 0x417E, array index: 0x094B */
	0x7E4A,0x7FA8,0x817A,0x821B,0x8239,0x85A6,0x8A6E,0x8CCE,
	0x8DF5,0x9078,0x9077,0x92AD,0x9291,0x9583,0x9BAE,0x524D,
	0x5584,0x6F38,0x7136,0x5168,0x7985,0x7E55,0x81B3,0x7CCE,
	0x564C,0x5851,0x5CA8,0x63AA,0x66FE,0x66FD,0x695A,0x72D9,
	0x758F,0x758E,0x790E,0x7956,0x79DF,0x7C97,0x7D20,0x7D44,
	0x8607,0x8A34,0x963B,0x9061,0x9F20,0x50E7,0x5275,0x53CC,
	0x53E2,0x5009,0x55AA,0x58EE,0x594F,0x723D,0x5B8B,0x5C64,
	0x531D,0x60E3,0x60F3,0x635C,0x6383,0x633F,0x63BB,0x64CD,
	0x65E9,0x66F9,0x5DE3,0x69CD,0x69FD,0x6F15,0x71E5,0x4E89,
	0x75E9,0x76F8,0x7A93,0x7CDF,0x7DCF,0x7D9C,0x8061,0x8349,
	0x8358,0x846C,0x84BC,0x85FB,0x88C5,0x8D70,0x9001,0x906D,
	0x9397,0x971C,0x9A12,0x50CF,0x5897,0x618E,
	/* Range 0x4221 - 0x427E, array index: 0x09A9 */
	0x81D3,0x8535,0x8D08,0x9020,0x4FC3,0x5074,0x5247,0x5373,
	0x606F,0x6349,0x675F,0x6E2C,0x8DB3,0x901F,0x4FD7,0x5C5E,
	0x8CCA,0x65CF,0x7D9A,0x5352,0x8896,0x5176,0x63C3,0x5B58,
	0x5B6B,0x5C0A,0x640D,0x6751,0x905C,0x4ED6,0x591A,0x592A,
	0x6C70,0x8A51,0x553E,0x5815,0x59A5,0x60F0,0x6253,0x67C1,
	0x8235,0x6955,0x9640,0x99C4,0x9A28,0x4F53,0x5806,0x5BFE,
	0x8010,0x5CB1,0x5E2F,0x5F85,0x6020,0x614B,0x6234,0x66FF,
	0x6CF0,0x6EDE,0x80CE,0x817F,0x82D4,0x888B,0x8CB8,0x9000,
	0x902E,0x968A,0x9EDB,0x9BDB,0x4EE3,0x53F0,0x5927,0x7B2C,
	0x918D,0x984C,0x9DF9,0x6EDD,0x7027,0x5353,0x5544,0x5B85,
	0x6258,0x629E,0x62D3,0x6CA2,0x6FEF,0x7422,0x8A17,0x9438,
	0x6FC1,0x8AFE,0x8338,0x51E7,0x86F8,0x53EA,
	/* Range 0x4321 - 0x437E, array index: 0x0A07 */
	0x53E9,0x4F46,0x9054,0x8FB0,0x596A,0x8131,0x5DFD,0x7AEA,
	0x8FBF,0x68DA,0x8C37,0x72F8,0x9C48,0x6A3D,0x8AB0,0x4E39,
	0x5358,0x5606,0x5766,0x62C5,0x63A2,0x65E6,0x6B4E,0x6DE1,
	0x6E5B,0x70AD,0x77ED,0x7AEF,0x7BAA,0x7DBB,0x803D,0x80C6,
	0x86CB,0x8A95,0x935B,0x56E3,0x58C7,0x5F3E,0x65AD,0x6696,
	0x6A80,0x6BB5,0x7537,0x8AC7,0x5024,0x77E5,0x5730,0x5F1B,
	0x6065,0x667A,0x6C60,0x75F4,0x7A1A,0x7F6E,0x81F4,0x8718,
	0x9045,0x99B3,0x7BC9,0x755C,0x7AF9,0x7B51,0x84C4,0x9010,
	0x79E9,0x7A92,0x8336,0x5AE1,0x7740,0x4E2D,0x4EF2,0x5B99,
	0x5FE0,0x62BD,0x663C,0x67F1,0x6CE8,0x866B,0x8877,0x8A3B,
	0x914E,0x92F3,0x99D0,0x6A17,0x7026,0x732A,0x82E7,0x8457,
	0x8CAF,0x4E01,0x5146,0x51CB,0x558B,0x5BF5,
	/* Range 0x4421 - 0x447E, array index: 0x0A65 */
	0x5E16,0x5E33,0x5E81,0x5F14,0x5F35,0x5F6B,0x5FB4,0x61F2,
	0x6311,0x66A2,0x671D,0x6F6E,0x7252,0x753A,0x773A,0x8074,
	0x8139,0x8178,0x8776,0x8ABF,0x8ADC,0x8D85,0x8DF3,0x929A,
	0x9577,0x9802,0x9CE5,0x52C5,0x6357,0x76F4,0x6715,0x6C88,
	0x73CD,0x8CC3,0x93AE,0x9673,0x6D25,0x589C,0x690E,0x69CC,
	0x8FFD,0x939A,0x75DB,0x901A,0x585A,0x6802,0x63B4,0x69FB,
	0x4F43,0x6F2C,0x67D8,0x8FBB,0x8526,0x7DB4,0x9354,0x693F,
	0x6F70,0x576A,0x58F7,0x5B2C,0x7D2C,0x722A,0x540A,0x91E3,
	0x9DB4,0x4EAD,0x4F4E,0x505C,0x5075,0x5243,0x8C9E,0x5448,
	0x5824,0x5B9A,0x5E1D,0x5E95,0x5EAD,0x5EF7,0x5F1F,0x608C,
	0x62B5,0x633A,0x63D0,0x68AF,0x6C40,0x7887,0x798E,0x7A0B,
	0x7DE0,0x8247,0x8A02,0x8AE6,0x8E44,0x9013,
	/* Range 0x4521 - 0x457E, array index: 0x0AC3 */
	0x90B8,0x912D,0x91D8,0x9F0E,0x6CE5,0x6458,0x64E2,0x6575,
	0x6EF4,0x7684,0x7B1B,0x9069,0x93D1,0x6EBA,0x54F2,0x5FB9,
	0x64A4,0x8F4D,0x8FED,0x9244,0x5178,0x586B,0x5929,0x5C55,
	0x5E97,0x6DFB,0x7E8F,0x751C,0x8CBC,0x8EE2,0x985B,0x70B9,
	0x4F1D,0x6BBF,0x6FB1,0x7530,0x96FB,0x514E,0x5410,0x5835,
	0x5857,0x59AC,0x5C60,0x5F92,0x6597,0x675C,0x6E21,0x767B,
	0x83DF,0x8CED,0x9014,0x90FD,0x934D,0x7825,0x783A,0x52AA,
	0x5EA6,0x571F,0x5974,0x6012,0x5012,0x515A,0x51AC,0x51CD,
	0x5200,0x5510,0x5854,0x5858,0x5957,0x5B95,0x5CF6,0x5D8B,
	0x60BC,0x6295,0x642D,0x6771,0x6843,0x68BC,0x68DF,0x76D7,
	0x6DD8,0x6E6F,0x6D9B,0x706F,0x71C8,0x5F53,0x75D8,0x7977,
	0x7B49,0x7B54,0x7B52,0x7CD6,0x7D71,0x5230,
	/* Range 0x4621 - 0x467E, array index: 0x0B21 */
	0x8463,0x8569,0x85E4,0x8A0E,0x8B04,0x8C46,0x8E0F,0x9003,
	0x900F,0x9419,0x9676,0x982D,0x9A30,0x95D8,0x50CD,0x52D5,
	0x540C,0x5802,0x5C0E,0x61A7,0x649E,0x6D1E,0x77B3,0x7AE5,
	0x80F4,0x8404,0x9053,0x9285,0x5CE0,0x9D07,0x533F,0x5F97,
	0x5FB3,0x6D9C,0x7279,0x7763,0x79BF,0x7BE4,0x6BD2,0x72EC,
	0x8AAD,0x6803,0x6A61,0x51F8,0x7A81,0x6934,0x5C4A,0x9CF6,
	0x82EB,0x5BC5,0x9149,0x701E,0x5678,0x5C6F,0x60C7,0x6566,
	0x6C8C,0x8C5A,0x9041,0x9813,0x5451,0x66C7,0x920D,0x5948,
	0x90A3,0x5185,0x4E4D,0x51EA,0x8599,0x8B0E,0x7058,0x637A,
	0x934B,0x6962,0x99B4,0x7E04,0x7577,0x5357,0x6960,0x8EDF,
	0x96E3,0x6C5D,0x4E8C,0x5C3C,0x5F10,0x8FE9,0x5302,0x8CD1,
	0x8089,0x8679,0x5EFF,0x65E5,0x4E73,0x5165,
	/* Range 0x4721 - 0x477E, array index: 0x0B7F */
	0x5982,0x5C3F,0x97EE,0x4EFB,0x598A,0x5FCD,0x8A8D,0x6FE1,
	0x79B0,0x7962,0x5BE7,0x8471,0x732B,0x71B1,0x5E74,0x5FF5,
	0x637B,0x649A,0x71C3,0x7C98,0x4E43,0x5EFC,0x4E4B,0x57DC,
	0x56A2,0x60A9,0x6FC3,0x7D0D,0x80FD,0x8133,0x81BF,0x8FB2,
	0x8997,0x86A4,0x5DF4,0x628A,0x64AD,0x8987,0x6777,0x6CE2,
	0x6D3E,0x7436,0x7834,0x5A46,0x7F75,0x82AD,0x99AC,0x4FF3,
	0x5EC3,0x62DD,0x6392,0x6557,0x676F,0x76C3,0x724C,0x80CC,
	0x80BA,0x8F29,0x914D,0x500D,0x57F9,0x5A92,0x6885,0x6973,
	0x7164,0x72FD,0x8CB7,0x58F2,0x8CE0,0x966A,0x9019,0x877F,
	0x79E4,0x77E7,0x8429,0x4F2F,0x5265,0x535A,0x62CD,0x67CF,
	0x6CCA,0x767D,0x7B94,0x7C95,0x8236,0x8584,0x8FEB,0x66DD,
	0x6F20,0x7206,0x7E1B,0x83AB,0x99C1,0x9EA6,
	/* Range 0x4821 - 0x487E, array index: 0x0BDD */
	0x51FD,0x7BB1,0x7872,0x7BB8,0x8087,0x7B48,0x6AE8,0x5E61,
	0x808C,0x7551,0x7560,0x516B,0x9262,0x6E8C,0x767A,0x9197,
	0x9AEA,0x4F10,0x7F70,0x629C,0x7B4F,0x95A5,0x9CE9,0x567A,
	0x5859,0x86E4,0x96BC,0x4F34,0x5224,0x534A,0x53CD,0x53DB,
	0x5E06,0x642C,0x6591,0x677F,0x6C3E,0x6C4E,0x7248,0x72AF,
	0x73ED,0x7554,0x7E41,0x822C,0x85E9,0x8CA9,0x7BC4,0x91C6,
	0x7169,0x9812,0x98EF,0x633D,0x6669,0x756A,0x76E4,0x78D0,
	0x8543,0x86EE,0x532A,0x5351,0x5426,0x5983,0x5E87,0x5F7C,
	0x60B2,0x6249,0x6279,0x62AB,0x6590,0x6BD4,0x6CCC,0x75B2,
	0x76AE,0x7891,0x79D8,0x7DCB,0x7F77,0x80A5,0x88AB,0x8AB9,
	0x8CBB,0x907F,0x975E,0x98DB,0x6A0B,0x7C38,0x5099,0x5C3E,
	0x5FAE,0x6787,0x6BD8,0x7435,0x7709,0x7F8E,
	/* Range 0x4921 - 0x497E, array index: 0x0C3B */
	0x9F3B,0x67CA,0x7A17,0x5339,0x758B,0x9AED,0x5F66,0x819D,
	0x83F1,0x8098,0x5F3C,0x5FC5,0x7562,0x7B46,0x903C,0x6867,
	0x59EB,0x5A9B,0x7D10,0x767E,0x8B2C,0x4FF5,0x5F6A,0x6A19,
	0x6C37,0x6F02,0x74E2,0x7968,0x8868,0x8A55,0x8C79,0x5EDF,
	0x63CF,0x75C5,0x79D2,0x82D7,0x9328,0x92F2,0x849C,0x86ED,
	0x9C2D,0x54C1,0x5F6C,0x658C,0x6D5C,0x7015,0x8CA7,0x8CD3,
	0x983B,0x654F,0x74F6,0x4E0D,0x4ED8,0x57E0,0x592B,0x5A66,
	0x5BCC,0x51A8,0x5E03,0x5E9C,0x6016,0x6276,0x6577,0x65A7,
	0x666E,0x6D6E,0x7236,0x7B26,0x8150,0x819A,0x8299,0x8B5C,
	0x8CA0,0x8CE6,0x8D74,0x961C,0x9644,0x4FAE,0x64AB,0x6B66,
	0x821E,0x8461,0x856A,0x90E8,0x5C01,0x6953,0x98A8,0x847A,
	0x8557,0x4F0F,0x526F,0x5FA9,0x5E45,0x670D,
	/* Range 0x4A21 - 0x4A7E, array index: 0x0C99 */
	0x798F,0x8179,0x8907,0x8986,0x6DF5,0x5F17,0x6255,0x6CB8,
	0x4ECF,0x7269,0x9B92,0x5206,0x543B,0x5674,0x58B3,0x61A4,
	0x626E,0x711A,0x596E,0x7C89,0x7CDE,0x7D1B,0x96F0,0x6587,
	0x805E,0x4E19,0x4F75,0x5175,0x5840,0x5E63,0x5E73,0x5F0A,
	0x67C4,0x4E26,0x853D,0x9589,0x965B,0x7C73,0x9801,0x50FB,
	0x58C1,0x7656,0x78A7,0x5225,0x77A5,0x8511,0x7B86,0x504F,
	0x5909,0x7247,0x7BC7,0x7DE8,0x8FBA,0x8FD4,0x904D,0x4FBF,
	0x52C9,0x5A29,0x5F01,0x97AD,0x4FDD,0x8217,0x92EA,0x5703,
	0x6355,0x6B69,0x752B,0x88DC,0x8F14,0x7A42,0x52DF,0x5893,
	0x6155,0x620A,0x66AE,0x6BCD,0x7C3F,0x83E9,0x5023,0x4FF8,
	0x5305,0x5446,0x5831,0x5949,0x5B9D,0x5CF0,0x5CEF,0x5D29,
	0x5E96,0x62B1,0x6367,0x653E,0x65B9,0x670B,
	/* Range 0x4B21 - 0x4B7E, array index: 0x0CF7 */
	0x6CD5,0x6CE1,0x70F9,0x7832,0x7E2B,0x80DE,0x82B3,0x840C,
	0x84EC,0x8702,0x8912,0x8A2A,0x8C4A,0x90A6,0x92D2,0x98FD,
	0x9CF3,0x9D6C,0x4E4F,0x4EA1,0x508D,0x5256,0x574A,0x59A8,
	0x5E3D,0x5FD8,0x5FD9,0x623F,0x66B4,0x671B,0x67D0,0x68D2,
	0x5192,0x7D21,0x80AA,0x81A8,0x8B00,0x8C8C,0x8CBF,0x927E,
	0x9632,0x5420,0x982C,0x5317,0x50D5,0x535C,0x58A8,0x64B2,
	0x6734,0x7267,0x7766,0x7A46,0x91E6,0x52C3,0x6CA1,0x6B86,
	0x5800,0x5E4C,0x5954,0x672C,0x7FFB,0x51E1,0x76C6,0x6469,
	0x78E8,0x9B54,0x9EBB,0x57CB,0x59B9,0x6627,0x679A,0x6BCE,
	0x54E9,0x69D9,0x5E55,0x819C,0x6795,0x9BAA,0x67FE,0x9C52,
	0x685D,0x4EA6,0x4FE3,0x53C8,0x62B9,0x672B,0x6CAB,0x8FC4,
	0x4FAD,0x7E6D,0x9EBF,0x4E07,0x6162,0x6E80,
	/* Range 0x4C21 - 0x4C7E, array index: 0x0D55 */
	0x6F2B,0x8513,0x5473,0x672A,0x9B45,0x5DF3,0x7B95,0x5CAC,
	0x5BC6,0x871C,0x6E4A,0x84D1,0x7A14,0x8108,0x5999,0x7C8D,
	0x6C11,0x7720,0x52D9,0x5922,0x7121,0x725F,0x77DB,0x9727,
	0x9D61,0x690B,0x5A7F,0x5A18,0x51A5,0x540D,0x547D,0x660E,
	0x76DF,0x8FF7,0x9298,0x9CF4,0x59EA,0x725D,0x6EC5,0x514D,
	0x68C9,0x7DBF,0x7DEC,0x9762,0x9EBA,0x6478,0x6A21,0x8302,
	0x5984,0x5B5F,0x6BDB,0x731B,0x76F2,0x7DB2,0x8017,0x8499,
	0x5132,0x6728,0x9ED9,0x76EE,0x6762,0x52FF,0x9905,0x5C24,
	0x623B,0x7C7E,0x8CB0,0x554F,0x60B6,0x7D0B,0x9580,0x5301,
	0x4E5F,0x51B6,0x591C,0x723A,0x8036,0x91CE,0x5F25,0x77E2,
	0x5384,0x5F79,0x7D04,0x85AC,0x8A33,0x8E8D,0x9756,0x67F3,
	0x85AE,0x9453,0x6109,0x6108,0x6CB9,0x7652,
	/* Range 0x4D21 - 0x4D7E, array index: 0x0DB3 */
	0x8AED,0x8F38,0x552F,0x4F51,0x512A,0x52C7,0x53CB,0x5BA5,
	0x5E7D,0x60A0,0x6182,0x63D6,0x6709,0x67DA,0x6E67,0x6D8C,
	0x7336,0x7337,0x7531,0x7950,0x88D5,0x8A98,0x904A,0x9091,
	0x90F5,0x96C4,0x878D,0x5915,0x4E88,0x4F59,0x4E0E,0x8A89,
	0x8F3F,0x9810,0x50AD,0x5E7C,0x5996,0x5BB9,0x5EB8,0x63DA,
	0x63FA,0x64C1,0x66DC,0x694A,0x69D8,0x6D0B,0x6EB6,0x7194,
	0x7528,0x7AAF,0x7F8A,0x8000,0x8449,0x84C9,0x8981,0x8B21,
	0x8E0A,0x9065,0x967D,0x990A,0x617E,0x6291,0x6B32,0x6C83,
	0x6D74,0x7FCC,0x7FFC,0x6DC0,0x7F85,0x87BA,0x88F8,0x6765,
	0x83B1,0x983C,0x96F7,0x6D1B,0x7D61,0x843D,0x916A,0x4E71,
	0x5375,0x5D50,0x6B04,0x6FEB,0x85CD,0x862D,0x89A7,0x5229,
	0x540F,0x5C65,0x674E,0x68A8,0x7406,0x7483,
	/* Range 0x4E21 - 0x4E7E, array index: 0x0E11 */
	0x75E2,0x88CF,0x88E1,0x91CC,0x96E2,0x9678,0x5F8B,0x7387,
	0x7ACB,0x844E,0x63A0,0x7565,0x5289,0x6D41,0x6E9C,0x7409,
	0x7559,0x786B,0x7C92,0x9686,0x7ADC,0x9F8D,0x4FB6,0x616E,
	0x65C5,0x865C,0x4E86,0x4EAE,0x50DA,0x4E21,0x51CC,0x5BEE,
	0x6599,0x6881,0x6DBC,0x731F,0x7642,0x77AD,0x7A1C,0x7CE7,
	0x826F,0x8AD2,0x907C,0x91CF,0x9675,0x9818,0x529B,0x7DD1,
	0x502B,0x5398,0x6797,0x6DCB,0x71D0,0x7433,0x81E8,0x8F2A,
	0x96A3,0x9C57,0x9E9F,0x7460,0x5841,0x6D99,0x7D2F,0x985E,
	0x4EE4,0x4F36,0x4F8B,0x51B7,0x52B1,0x5DBA,0x601C,0x73B2,
	0x793C,0x82D3,0x9234,0x96B7,0x96F6,0x970A,0x9E97,0x9F62,
	0x66A6,0x6B74,0x5217,0x52A3,0x70C8,0x88C2,0x5EC9,0x604B,
	0x6190,0x6F23,0x7149,0x7C3E,0x7DF4,0x806F,
	/* Range 0x4F21 - 0x4F53, array index: 0x0E6F */
	0x84EE,0x9023,0x932C,0x5442,0x9B6F,0x6AD3,0x7089,0x8CC2,
	0x8DEF,0x9732,0x52B4,0x5A41,0x5ECA,0x5F04,0x6717,0x697C,
	0x6994,0x6D6A,0x6F0F,0x7262,0x72FC,0x7BED,0x8001,0x807E,
	0x874B,0x90CE,0x516D,0x9E93,0x7984,0x808B,0x9332,0x8AD6,
	0x502D,0x548C,0x8A71,0x6B6A,0x8CC4,0x8107,0x60D1,0x67A0,
	0x9DF2,0x4E99,0x4E98,0x9C10,0x8A6B,0x85C1,0x8568,0x6900,
	0x6E7E,0x7897,0x8155,
	/* Range 0x5021 - 0x507E, array index: 0x0EA2 */
	0x5F0C,0x4E10,0x4E15,0x4E2A,0x4E31,0x4E36,0x4E3C,0x4E3F,
	0x4E42,0x4E56,0x4E58,0x4E82,0x4E85,0x8C6B,0x4E8A,0x8212,
	0x5F0D,0x4E8E,0x4E9E,0x4E9F,0x4EA0,0x4EA2,0x4EB0,0x4EB3,
	0x4EB6,0x4ECE,0x4ECD,0x4EC4,0x4EC6,0x4EC2,0x4ED7,0x4EDE,
	0x4EED,0x4EDF,0x4EF7,0x4F09,0x4F5A,0x4F30,0x4F5B,0x4F5D,
	0x4F57,0x4F47,0x4F76,0x4F88,0x4F8F,0x4F98,0x4F7B,0x4F69,
	0x4F70,0x4F91,0x4F6F,0x4F86,0x4F96,0x5118,0x4FD4,0x4FDF,
	0x4FCE,0x4FD8,0x4FDB,0x4FD1,0x4FDA,0x4FD0,0x4FE4,0x4FE5,
	0x501A,0x5028,0x5014,0x502A,0x5025,0x5005,0x4F1C,0x4FF6,
	0x5021,0x5029,0x502C,0x4FFE,0x4FEF,0x5011,0x5006,0x5043,
	0x5047,0x6703,0x5055,0x5050,0x5048,0x505A,0x5056,0x506C,
	0x5078,0x5080,0x509A,0x5085,0x50B4,0x50B2,
	/* Range 0x5121 - 0x517E, array index: 0x0F00 */
	0x50C9,0x50CA,0x50B3,0x50C2,0x50D6,0x50DE,0x50E5,0x50ED,
	0x50E3,0x50EE,0x50F9,0x50F5,0x5109,0x5101,0x5102,0x5116,
	0x5115,0x5114,0x511A,0x5121,0x513A,0x5137,0x513C,0x513B,
	0x513F,0x5140,0x5152,0x514C,0x5154,0x5162,0x7AF8,0x5169,
	0x516A,0x516E,0x5180,0x5182,0x56D8,0x518C,0x5189,0x518F,
	0x5191,0x5193,0x5195,0x5196,0x51A4,0x51A6,0x51A2,0x51A9,
	0x51AA,0x51AB,0x51B3,0x51B1,0x51B2,0x51B0,0x51B5,0x51BD,
	0x51C5,0x51C9,0x51DB,0x51E0,0x8655,0x51E9,0x51ED,0x51F0,
	0x51F5,0x51FE,0x5204,0x520B,0x5214,0x520E,0x5227,0x522A,
	0x522E,0x5233,0x5239,0x524F,0x5244,0x524B,0x524C,0x525E,
	0x5254,0x526A,0x5274,0x5269,0x5273,0x527F,0x527D,0x528D,
	0x5294,0x5292,0x5271,0x5288,0x5291,0x8FA8,
	/* Range 0x5221 - 0x527E, array index: 0x0F5E */
	0x8FA7,0x52AC,0x52AD,0x52BC,0x52B5,0x52C1,0x52CD,0x52D7,
	0x52DE,0x52E3,0x52E6,0x98ED,0x52E0,0x52F3,0x52F5,0x52F8,
	0x52F9,0x5306,0x5308,0x7538,0x530D,0x5310,0x530F,0x5315,
	0x531A,0x5323,0x532F,0x5331,0x5333,0x5338,0x5340,0x5346,
	0x5345,0x4E17,0x5349,0x534D,0x51D6,0x535E,0x5369,0x536E,
	0x5918,0x537B,0x5377,0x5382,0x5396,0x53A0,0x53A6,0x53A5,
	0x53AE,0x53B0,0x53B6,0x53C3,0x7C12,0x96D9,0x53DF,0x66FC,
	0x71EE,0x53EE,0x53E8,0x53ED,0x53FA,0x5401,0x543D,0x5440,
	0x542C,0x542D,0x543C,0x542E,0x5436,0x5429,0x541D,0x544E,
	0x548F,0x5475,0x548E,0x545F,0x5471,0x5477,0x5470,0x5492,
	0x547B,0x5480,0x5476,0x5484,0x5490,0x5486,0x54C7,0x54A2,
	0x54B8,0x54A5,0x54AC,0x54C4,0x54C8,0x54A8,
	/* Range 0x5321 - 0x537E, array index: 0x0FBC */
	0x54AB,0x54C2,0x54A4,0x54BE,0x54BC,0x54D8,0x54E5,0x54E6,
	0x550F,0x5514,0x54FD,0x54EE,0x54ED,0x54FA,0x54E2,0x5539,
	0x5540,0x5563,0x554C,0x552E,0x555C,0x5545,0x5556,0x5557,
	0x5538,0x5533,0x555D,0x5599,0x5580,0x54AF,0x558A,0x559F,
	0x557B,0x557E,0x5598,0x559E,0x55AE,0x557C,0x5583,0x55A9,
	0x5587,0x55A8,0x55DA,0x55C5,0x55DF,0x55C4,0x55DC,0x55E4,
	0x55D4,0x5614,0x55F7,0x5616,0x55FE,0x55FD,0x561B,0x55F9,
	0x564E,0x5650,0x71DF,0x5634,0x5636,0x5632,0x5638,0x566B,
	0x5664,0x562F,0x566C,0x566A,0x5686,0x5680,0x568A,0x56A0,
	0x5694,0x568F,0x56A5,0x56AE,0x56B6,0x56B4,0x56C2,0x56BC,
	0x56C1,0x56C3,0x56C0,0x56C8,0x56CE,0x56D1,0x56D3,0x56D7,
	0x56EE,0x56F9,0x5700,0x56FF,0x5704,0x5709,
	/* Range 0x5421 - 0x547E, array index: 0x101A */
	0x5708,0x570B,0x570D,0x5713,0x5718,0x5716,0x55C7,0x571C,
	0x5726,0x5737,0x5738,0x574E,0x573B,0x5740,0x574F,0x5769,
	0x57C0,0x5788,0x5761,0x577F,0x5789,0x5793,0x57A0,0x57B3,
	0x57A4,0x57AA,0x57B0,0x57C3,0x57C6,0x57D4,0x57D2,0x57D3,
	0x580A,0x57D6,0x57E3,0x580B,0x5819,0x581D,0x5872,0x5821,
	0x5862,0x584B,0x5870,0x6BC0,0x5852,0x583D,0x5879,0x5885,
	0x58B9,0x589F,0x58AB,0x58BA,0x58DE,0x58BB,0x58B8,0x58AE,
	0x58C5,0x58D3,0x58D1,0x58D7,0x58D9,0x58D8,0x58E5,0x58DC,
	0x58E4,0x58DF,0x58EF,0x58FA,0x58F9,0x58FB,0x58FC,0x58FD,
	0x5902,0x590A,0x5910,0x591B,0x68A6,0x5925,0x592C,0x592D,
	0x5932,0x5938,0x593E,0x7AD2,0x5955,0x5950,0x594E,0x595A,
	0x5958,0x5962,0x5960,0x5967,0x596C,0x5969,
	/* Range 0x5521 - 0x557E, array index: 0x1078 */
	0x5978,0x5981,0x599D,0x4F5E,0x4FAB,0x59A3,0x59B2,0x59C6,
	0x59E8,0x59DC,0x598D,0x59D9,0x59DA,0x5A25,0x5A1F,0x5A11,
	0x5A1C,0x5A09,0x5A1A,0x5A40,0x5A6C,0x5A49,0x5A35,0x5A36,
	0x5A62,0x5A6A,0x5A9A,0x5ABC,0x5ABE,0x5ACB,0x5AC2,0x5ABD,
	0x5AE3,0x5AD7,0x5AE6,0x5AE9,0x5AD6,0x5AFA,0x5AFB,0x5B0C,
	0x5B0B,0x5B16,0x5B32,0x5AD0,0x5B2A,0x5B36,0x5B3E,0x5B43,
	0x5B45,0x5B40,0x5B51,0x5B55,0x5B5A,0x5B5B,0x5B65,0x5B69,
	0x5B70,0x5B73,0x5B75,0x5B78,0x6588,0x5B7A,0x5B80,0x5B83,
	0x5BA6,0x5BB8,0x5BC3,0x5BC7,0x5BC9,0x5BD4,0x5BD0,0x5BE4,
	0x5BE6,0x5BE2,0x5BDE,0x5BE5,0x5BEB,0x5BF0,0x5BF6,0x5BF3,
	0x5C05,0x5C07,0x5C08,0x5C0D,0x5C13,0x5C20,0x5C22,0x5C28,
	0x5C38,0x5C39,0x5C41,0x5C46,0x5C4E,0x5C53,
	/* Range 0x5621 - 0x567E, array index: 0x10D6 */
	0x5C50,0x5C4F,0x5B71,0x5C6C,0x5C6E,0x4E62,0x5C76,0x5C79,
	0x5C8C,0x5C91,0x5C94,0x599B,0x5CAB,0x5CBB,0x5CB6,0x5CBC,
	0x5CB7,0x5CC5,0x5CBE,0x5CC7,0x5CD9,0x5CE9,0x5CFD,0x5CFA,
	0x5CED,0x5D8C,0x5CEA,0x5D0B,0x5D15,0x5D17,0x5D5C,0x5D1F,
	0x5D1B,0x5D11,0x5D14,0x5D22,0x5D1A,0x5D19,0x5D18,0x5D4C,
	0x5D52,0x5D4E,0x5D4B,0x5D6C,0x5D73,0x5D76,0x5D87,0x5D84,
	0x5D82,0x5DA2,0x5D9D,0x5DAC,0x5DAE,0x5DBD,0x5D90,0x5DB7,
	0x5DBC,0x5DC9,0x5DCD,0x5DD3,0x5DD2,0x5DD6,0x5DDB,0x5DEB,
	0x5DF2,0x5DF5,0x5E0B,0x5E1A,0x5E19,0x5E11,0x5E1B,0x5E36,
	0x5E37,0x5E44,0x5E43,0x5E40,0x5E4E,0x5E57,0x5E54,0x5E5F,
	0x5E62,0x5E64,0x5E47,0x5E75,0x5E76,0x5E7A,0x9EBC,0x5E7F,
	0x5EA0,0x5EC1,0x5EC2,0x5EC8,0x5ED0,0x5ECF,
	/* Range 0x5721 - 0x577E, array index: 0x1134 */
	0x5ED6,0x5EE3,0x5EDD,0x5EDA,0x5EDB,0x5EE2,0x5EE1,0x5EE8,
	0x5EE9,0x5EEC,0x5EF1,0x5EF3,0x5EF0,0x5EF4,0x5EF8,0x5EFE,
	0x5F03,0x5F09,0x5F5D,0x5F5C,0x5F0B,0x5F11,0x5F16,0x5F29,
	0x5F2D,0x5F38,0x5F41,0x5F48,0x5F4C,0x5F4E,0x5F2F,0x5F51,
	0x5F56,0x5F57,0x5F59,0x5F61,0x5F6D,0x5F73,0x5F77,0x5F83,
	0x5F82,0x5F7F,0x5F8A,0x5F88,0x5F91,0x5F87,0x5F9E,0x5F99,
	0x5F98,0x5FA0,0x5FA8,0x5FAD,0x5FBC,0x5FD6,0x5FFB,0x5FE4,
	0x5FF8,0x5FF1,0x5FDD,0x60B3,0x5FFF,0x6021,0x6060,0x6019,
	0x6010,0x6029,0x600E,0x6031,0x601B,0x6015,0x602B,0x6026,
	0x600F,0x603A,0x605A,0x6041,0x606A,0x6077,0x605F,0x604A,
	0x6046,0x604D,0x6063,0x6043,0x6064,0x6042,0x606C,0x606B,
	0x6059,0x6081,0x608D,0x60E7,0x6083,0x609A,
	/* Range 0x5821 - 0x587E, array index: 0x1192 */
	0x6084,0x609B,0x6096,0x6097,0x6092,0x60A7,0x608B,0x60E1,
	0x60B8,0x60E0,0x60D3,0x60B4,0x5FF0,0x60BD,0x60C6,0x60B5,
	0x60D8,0x614D,0x6115,0x6106,0x60F6,0x60F7,0x6100,0x60F4,
	0x60FA,0x6103,0x6121,0x60FB,0x60F1,0x610D,0x610E,0x6147,
	0x613E,0x6128,0x6127,0x614A,0x613F,0x613C,0x612C,0x6134,
	0x613D,0x6142,0x6144,0x6173,0x6177,0x6158,0x6159,0x615A,
	0x616B,0x6174,0x616F,0x6165,0x6171,0x615F,0x615D,0x6153,
	0x6175,0x6199,0x6196,0x6187,0x61AC,0x6194,0x619A,0x618A,
	0x6191,0x61AB,0x61AE,0x61CC,0x61CA,0x61C9,0x61F7,0x61C8,
	0x61C3,0x61C6,0x61BA,0x61CB,0x7F79,0x61CD,0x61E6,0x61E3,
	0x61F6,0x61FA,0x61F4,0x61FF,0x61FD,0x61FC,0x61FE,0x6200,
	0x6208,0x6209,0x620D,0x620C,0x6214,0x621B,
	/* Range 0x5921 - 0x597E, array index: 0x11F0 */
	0x621E,0x6221,0x622A,0x622E,0x6230,0x6232,0x6233,0x6241,
	0x624E,0x625E,0x6263,0x625B,0x6260,0x6268,0x627C,0x6282,
	0x6289,0x627E,0x6292,0x6293,0x6296,0x62D4,0x6283,0x6294,
	0x62D7,0x62D1,0x62BB,0x62CF,0x62FF,0x62C6,0x64D4,0x62C8,
	0x62DC,0x62CC,0x62CA,0x62C2,0x62C7,0x629B,0x62C9,0x630C,
	0x62EE,0x62F1,0x6327,0x6302,0x6308,0x62EF,0x62F5,0x6350,
	0x633E,0x634D,0x641C,0x634F,0x6396,0x638E,0x6380,0x63AB,
	0x6376,0x63A3,0x638F,0x6389,0x639F,0x63B5,0x636B,0x6369,
	0x63BE,0x63E9,0x63C0,0x63C6,0x63E3,0x63C9,0x63D2,0x63F6,
	0x63C4,0x6416,0x6434,0x6406,0x6413,0x6426,0x6436,0x651D,
	0x6417,0x6428,0x640F,0x6467,0x646F,0x6476,0x644E,0x652A,
	0x6495,0x6493,0x64A5,0x64A9,0x6488,0x64BC,
	/* Range 0x5A21 - 0x5A7E, array index: 0x124E */
	0x64DA,0x64D2,0x64C5,0x64C7,0x64BB,0x64D8,0x64C2,0x64F1,
	0x64E7,0x8209,0x64E0,0x64E1,0x62AC,0x64E3,0x64EF,0x652C,
	0x64F6,0x64F4,0x64F2,0x64FA,0x6500,0x64FD,0x6518,0x651C,
	0x6505,0x6524,0x6523,0x652B,0x6534,0x6535,0x6537,0x6536,
	0x6538,0x754B,0x6548,0x6556,0x6555,0x654D,0x6558,0x655E,
	0x655D,0x6572,0x6578,0x6582,0x6583,0x8B8A,0x659B,0x659F,
	0x65AB,0x65B7,0x65C3,0x65C6,0x65C1,0x65C4,0x65CC,0x65D2,
	0x65DB,0x65D9,0x65E0,0x65E1,0x65F1,0x6772,0x660A,0x6603,
	0x65FB,0x6773,0x6635,0x6636,0x6634,0x661C,0x664F,0x6644,
	0x6649,0x6641,0x665E,0x665D,0x6664,0x6667,0x6668,0x665F,
	0x6662,0x6670,0x6683,0x6688,0x668E,0x6689,0x6684,0x6698,
	0x669D,0x66C1,0x66B9,0x66C9,0x66BE,0x66BC,
	/* Range 0x5B21 - 0x5B7E, array index: 0x12AC */
	0x66C4,0x66B8,0x66D6,0x66DA,0x66E0,0x663F,0x66E6,0x66E9,
	0x66F0,0x66F5,0x66F7,0x670F,0x6716,0x671E,0x6726,0x6727,
	0x9738,0x672E,0x673F,0x6736,0x6741,0x6738,0x6737,0x6746,
	0x675E,0x6760,0x6759,0x6763,0x6764,0x6789,0x6770,0x67A9,
	0x677C,0x676A,0x678C,0x678B,0x67A6,0x67A1,0x6785,0x67B7,
	0x67EF,0x67B4,0x67EC,0x67B3,0x67E9,0x67B8,0x67E4,0x67DE,
	0x67DD,0x67E2,0x67EE,0x67B9,0x67CE,0x67C6,0x67E7,0x6A9C,
	0x681E,0x6846,0x6829,0x6840,0x684D,0x6832,0x684E,0x68B3,
	0x682B,0x6859,0x6863,0x6877,0x687F,0x689F,0x688F,0x68AD,
	0x6894,0x689D,0x689B,0x6883,0x6AAE,0x68B9,0x6874,0x68B5,
	0x68A0,0x68BA,0x690F,0x688D,0x687E,0x6901,0x68CA,0x6908,
	0x68D8,0x6922,0x6926,0x68E1,0x690C,0x68CD,
	/* Range 0x5C21 - 0x5C7E, array index: 0x130A */
	0x68D4,0x68E7,0x68D5,0x6936,0x6912,0x6904,0x68D7,0x68E3,
	0x6925,0x68F9,0x68E0,0x68EF,0x6928,0x692A,0x691A,0x6923,
	0x6921,0x68C6,0x6979,0x6977,0x695C,0x6978,0x696B,0x6954,
	0x697E,0x696E,0x6939,0x6974,0x693D,0x6959,0x6930,0x6961,
	0x695E,0x695D,0x6981,0x696A,0x69B2,0x69AE,0x69D0,0x69BF,
	0x69C1,0x69D3,0x69BE,0x69CE,0x5BE8,0x69CA,0x69DD,0x69BB,
	0x69C3,0x69A7,0x6A2E,0x6991,0x69A0,0x699C,0x6995,0x69B4,
	0x69DE,0x69E8,0x6A02,0x6A1B,0x69FF,0x6B0A,0x69F9,0x69F2,
	0x69E7,0x6A05,0x69B1,0x6A1E,0x69ED,0x6A14,0x69EB,0x6A0A,
	0x6A12,0x6AC1,0x6A23,0x6A13,0x6A44,0x6A0C,0x6A72,0x6A36,
	0x6A78,0x6A47,0x6A62,0x6A59,0x6A66,0x6A48,0x6A38,0x6A22,
	0x6A90,0x6A8D,0x6AA0,0x6A84,0x6AA2,0x6AA3,
	/* Range 0x5D21 - 0x5D7E, array index: 0x1368 */
	0x6A97,0x8617,0x6ABB,0x6AC3,0x6AC2,0x6AB8,0x6AB3,0x6AAC,
	0x6ADE,0x6AD1,0x6ADF,0x6AAA,0x6ADA,0x6AEA,0x6AFB,0x6B05,
	0x8616,0x6AFA,0x6B12,0x6B16,0x9B31,0x6B1F,0x6B38,0x6B37,
	0x76DC,0x6B39,0x98EE,0x6B47,0x6B43,0x6B49,0x6B50,0x6B59,
	0x6B54,0x6B5B,0x6B5F,0x6B61,0x6B78,0x6B79,0x6B7F,0x6B80,
	0x6B84,0x6B83,0x6B8D,0x6B98,0x6B95,0x6B9E,0x6BA4,0x6BAA,
	0x6BAB,0x6BAF,0x6BB2,0x6BB1,0x6BB3,0x6BB7,0x6BBC,0x6BC6,
	0x6BCB,0x6BD3,0x6BDF,0x6BEC,0x6BEB,0x6BF3,0x6BEF,0x9EBE,
	0x6C08,0x6C13,0x6C14,0x6C1B,0x6C24,0x6C23,0x6C5E,0x6C55,
	0x6C62,0x6C6A,0x6C82,0x6C8D,0x6C9A,0x6C81,0x6C9B,0x6C7E,
	0x6C68,0x6C73,0x6C92,0x6C90,0x6CC4,0x6CF1,0x6CD3,0x6CBD,
	0x6CD7,0x6CC5,0x6CDD,0x6CAE,0x6CB1,0x6CBE,
	/* Range 0x5E21 - 0x5E7E, array index: 0x13C6 */
	0x6CBA,0x6CDB,0x6CEF,0x6CD9,0x6CEA,0x6D1F,0x884D,0x6D36,
	0x6D2B,0x6D3D,0x6D38,0x6D19,0x6D35,0x6D33,0x6D12,0x6D0C,
	0x6D63,0x6D93,0x6D64,0x6D5A,0x6D79,0x6D59,0x6D8E,0x6D95,
	0x6FE4,0x6D85,0x6DF9,0x6E15,0x6E0A,0x6DB5,0x6DC7,0x6DE6,
	0x6DB8,0x6DC6,0x6DEC,0x6DDE,0x6DCC,0x6DE8,0x6DD2,0x6DC5,
	0x6DFA,0x6DD9,0x6DE4,0x6DD5,0x6DEA,0x6DEE,0x6E2D,0x6E6E,
	0x6E2E,0x6E19,0x6E72,0x6E5F,0x6E3E,0x6E23,0x6E6B,0x6E2B,
	0x6E76,0x6E4D,0x6E1F,0x6E43,0x6E3A,0x6E4E,0x6E24,0x6EFF,
	0x6E1D,0x6E38,0x6E82,0x6EAA,0x6E98,0x6EC9,0x6EB7,0x6ED3,
	0x6EBD,0x6EAF,0x6EC4,0x6EB2,0x6ED4,0x6ED5,0x6E8F,0x6EA5,
	0x6EC2,0x6E9F,0x6F41,0x6F11,0x704C,0x6EEC,0x6EF8,0x6EFE,
	0x6F3F,0x6EF2,0x6F31,0x6EEF,0x6F32,0x6ECC,
	/* Range 0x5F21 - 0x5F7E, array index: 0x1424 */
	0x6F3E,0x6F13,0x6EF7,0x6F86,0x6F7A,0x6F78,0x6F81,0x6F80,
	0x6F6F,0x6F5B,0x6FF3,0x6F6D,0x6F82,0x6F7C,0x6F58,0x6F8E,
	0x6F91,0x6FC2,0x6F66,0x6FB3,0x6FA3,0x6FA1,0x6FA4,0x6FB9,
	0x6FC6,0x6FAA,0x6FDF,0x6FD5,0x6FEC,0x6FD4,0x6FD8,0x6FF1,
	0x6FEE,0x6FDB,0x7009,0x700B,0x6FFA,0x7011,0x7001,0x700F,
	0x6FFE,0x701B,0x701A,0x6F74,0x701D,0x7018,0x701F,0x7030,
	0x703E,0x7032,0x7051,0x7063,0x7099,0x7092,0x70AF,0x70F1,
	0x70AC,0x70B8,0x70B3,0x70AE,0x70DF,0x70CB,0x70DD,0x70D9,
	0x7109,0x70FD,0x711C,0x7119,0x7165,0x7155,0x7188,0x7166,
	0x7162,0x714C,0x7156,0x716C,0x718F,0x71FB,0x7184,0x7195,
	0x71A8,0x71AC,0x71D7,0x71B9,0x71BE,0x71D2,0x71C9,0x71D4,
	0x71CE,0x71E0,0x71EC,0x71E7,0x71F5,0x71FC,
	/* Range 0x6021 - 0x607E, array index: 0x1482 */
	0x71F9,0x71FF,0x720D,0x7210,0x721B,0x7228,0x722D,0x722C,
	0x7230,0x7232,0x723B,0x723C,0x723F,0x7240,0x7246,0x724B,
	0x7258,0x7274,0x727E,0x7282,0x7281,0x7287,0x7292,0x7296,
	0x72A2,0x72A7,0x72B9,0x72B2,0x72C3,0x72C6,0x72C4,0x72CE,
	0x72D2,0x72E2,0x72E0,0x72E1,0x72F9,0x72F7,0x500F,0x7317,
	0x730A,0x731C,0x7316,0x731D,0x7334,0x732F,0x7329,0x7325,
	0x733E,0x734E,0x734F,0x9ED8,0x7357,0x736A,0x7368,0x7370,
	0x7378,0x7375,0x737B,0x737A,0x73C8,0x73B3,0x73CE,0x73BB,
	0x73C0,0x73E5,0x73EE,0x73DE,0x74A2,0x7405,0x746F,0x7425,
	0x73F8,0x7432,0x743A,0x7455,0x743F,0x745F,0x7459,0x7441,
	0x745C,0x7469,0x7470,0x7463,0x746A,0x7476,0x747E,0x748B,
	0x749E,0x74A7,0x74CA,0x74CF,0x74D4,0x73F1,
	/* Range 0x6121 - 0x617E, array index: 0x14E0 */
	0x74E0,0x74E3,0x74E7,0x74E9,0x74EE,0x74F2,0x74F0,0x74F1,
	0x74F8,0x74F7,0x7504,0x7503,0x7505,0x750C,0x750E,0x750D,
	0x7515,0x7513,0x751E,0x7526,0x752C,0x753C,0x7544,0x754D,
	0x754A,0x7549,0x755B,0x7546,0x755A,0x7569,0x7564,0x7567,
	0x756B,0x756D,0x7578,0x7576,0x7586,0x7587,0x7574,0x758A,
	0x7589,0x7582,0x7594,0x759A,0x759D,0x75A5,0x75A3,0x75C2,
	0x75B3,0x75C3,0x75B5,0x75BD,0x75B8,0x75BC,0x75B1,0x75CD,
	0x75CA,0x75D2,0x75D9,0x75E3,0x75DE,0x75FE,0x75FF,0x75FC,
	0x7601,0x75F0,0x75FA,0x75F2,0x75F3,0x760B,0x760D,0x7609,
	0x761F,0x7627,0x7620,0x7621,0x7622,0x7624,0x7634,0x7630,
	0x763B,0x7647,0x7648,0x7646,0x765C,0x7658,0x7661,0x7662,
	0x7668,0x7669,0x766A,0x7667,0x766C,0x7670,
	/* Range 0x6221 - 0x627E, array index: 0x153E */
	0x7672,0x7676,0x7678,0x767C,0x7680,0x7683,0x7688,0x768B,
	0x768E,0x7696,0x7693,0x7699,0x769A,0x76B0,0x76B4,0x76B8,
	0x76B9,0x76BA,0x76C2,0x76CD,0x76D6,0x76D2,0x76DE,0x76E1,
	0x76E5,0x76E7,0x76EA,0x862F,0x76FB,0x7708,0x7707,0x7704,
	0x7729,0x7724,0x771E,0x7725,0x7726,0x771B,0x7737,0x7738,
	0x7747,0x775A,0x7768,0x776B,0x775B,0x7765,0x777F,0x777E,
	0x7779,0x778E,0x778B,0x7791,0x77A0,0x779E,0x77B0,0x77B6,
	0x77B9,0x77BF,0x77BC,0x77BD,0x77BB,0x77C7,0x77CD,0x77D7,
	0x77DA,0x77DC,0x77E3,0x77EE,0x77FC,0x780C,0x7812,0x7926,
	0x7820,0x792A,0x7845,0x788E,0x7874,0x7886,0x787C,0x789A,
	0x788C,0x78A3,0x78B5,0x78AA,0x78AF,0x78D1,0x78C6,0x78CB,
	0x78D4,0x78BE,0x78BC,0x78C5,0x78CA,0x78EC,
	/* Range 0x6321 - 0x637E, array index: 0x159C */
	0x78E7,0x78DA,0x78FD,0x78F4,0x7907,0x7912,0x7911,0x7919,
	0x792C,0x792B,0x7940,0x7960,0x7957,0x795F,0x795A,0x7955,
	0x7953,0x797A,0x797F,0x798A,0x799D,0x79A7,0x9F4B,0x79AA,
	0x79AE,0x79B3,0x79B9,0x79BA,0x79C9,0x79D5,0x79E7,0x79EC,
	0x79E1,0x79E3,0x7A08,0x7A0D,0x7A18,0x7A19,0x7A20,0x7A1F,
	0x7980,0x7A31,0x7A3B,0x7A3E,0x7A37,0x7A43,0x7A57,0x7A49,
	0x7A61,0x7A62,0x7A69,0x9F9D,0x7A70,0x7A79,0x7A7D,0x7A88,
	0x7A97,0x7A95,0x7A98,0x7A96,0x7AA9,0x7AC8,0x7AB0,0x7AB6,
	0x7AC5,0x7AC4,0x7ABF,0x9083,0x7AC7,0x7ACA,0x7ACD,0x7ACF,
	0x7AD5,0x7AD3,0x7AD9,0x7ADA,0x7ADD,0x7AE1,0x7AE2,0x7AE6,
	0x7AED,0x7AF0,0x7B02,0x7B0F,0x7B0A,0x7B06,0x7B33,0x7B18,
	0x7B19,0x7B1E,0x7B35,0x7B28,0x7B36,0x7B50,
	/* Range 0x6421 - 0x647E, array index: 0x15FA */
	0x7B7A,0x7B04,0x7B4D,0x7B0B,0x7B4C,0x7B45,0x7B75,0x7B65,
	0x7B74,0x7B67,0x7B70,0x7B71,0x7B6C,0x7B6E,0x7B9D,0x7B98,
	0x7B9F,0x7B8D,0x7B9C,0x7B9A,0x7B8B,0x7B92,0x7B8F,0x7B5D,
	0x7B99,0x7BCB,0x7BC1,0x7BCC,0x7BCF,0x7BB4,0x7BC6,0x7BDD,
	0x7BE9,0x7C11,0x7C14,0x7BE6,0x7BE5,0x7C60,0x7C00,0x7C07,
	0x7C13,0x7BF3,0x7BF7,0x7C17,0x7C0D,0x7BF6,0x7C23,0x7C27,
	0x7C2A,0x7C1F,0x7C37,0x7C2B,0x7C3D,0x7C4C,0x7C43,0x7C54,
	0x7C4F,0x7C40,0x7C50,0x7C58,0x7C5F,0x7C64,0x7C56,0x7C65,
	0x7C6C,0x7C75,0x7C83,0x7C90,0x7CA4,0x7CAD,0x7CA2,0x7CAB,
	0x7CA1,0x7CA8,0x7CB3,0x7CB2,0x7CB1,0x7CAE,0x7CB9,0x7CBD,
	0x7CC0,0x7CC5,0x7CC2,0x7CD8,0x7CD2,0x7CDC,0x7CE2,0x9B3B,
	0x7CEF,0x7CF2,0x7CF4,0x7CF6,0x7CFA,0x7D06,
	/* Range 0x6521 - 0x657E, array index: 0x1658 */
	0x7D02,0x7D1C,0x7D15,0x7D0A,0x7D45,0x7D4B,0x7D2E,0x7D32,
	0x7D3F,0x7D35,0x7D46,0x7D73,0x7D56,0x7D4E,0x7D72,0x7D68,
	0x7D6E,0x7D4F,0x7D63,0x7D93,0x7D89,0x7D5B,0x7D8F,0x7D7D,
	0x7D9B,0x7DBA,0x7DAE,0x7DA3,0x7DB5,0x7DC7,0x7DBD,0x7DAB,
	0x7E3D,0x7DA2,0x7DAF,0x7DDC,0x7DB8,0x7D9F,0x7DB0,0x7DD8,
	0x7DDD,0x7DE4,0x7DDE,0x7DFB,0x7DF2,0x7DE1,0x7E05,0x7E0A,
	0x7E23,0x7E21,0x7E12,0x7E31,0x7E1F,0x7E09,0x7E0B,0x7E22,
	0x7E46,0x7E66,0x7E3B,0x7E35,0x7E39,0x7E43,0x7E37,0x7E32,
	0x7E3A,0x7E67,0x7E5D,0x7E56,0x7E5E,0x7E59,0x7E5A,0x7E79,
	0x7E6A,0x7E69,0x7E7C,0x7E7B,0x7E83,0x7DD5,0x7E7D,0x8FAE,
	0x7E7F,0x7E88,0x7E89,0x7E8C,0x7E92,0x7E90,0x7E93,0x7E94,
	0x7E96,0x7E8E,0x7E9B,0x7E9C,0x7F38,0x7F3A,
	/* Range 0x6621 - 0x667E, array index: 0x16B6 */
	0x7F45,0x7F4C,0x7F4D,0x7F4E,0x7F50,0x7F51,0x7F55,0x7F54,
	0x7F58,0x7F5F,0x7F60,0x7F68,0x7F69,0x7F67,0x7F78,0x7F82,
	0x7F86,0x7F83,0x7F88,0x7F87,0x7F8C,0x7F94,0x7F9E,0x7F9D,
	0x7F9A,0x7FA3,0x7FAF,0x7FB2,0x7FB9,0x7FAE,0x7FB6,0x7FB8,
	0x8B71,0x7FC5,0x7FC6,0x7FCA,0x7FD5,0x7FD4,0x7FE1,0x7FE6,
	0x7FE9,0x7FF3,0x7FF9,0x98DC,0x8006,0x8004,0x800B,0x8012,
	0x8018,0x8019,0x801C,0x8021,0x8028,0x803F,0x803B,0x804A,
	0x8046,0x8052,0x8058,0x805A,0x805F,0x8062,0x8068,0x8073,
	0x8072,0x8070,0x8076,0x8079,0x807D,0x807F,0x8084,0x8086,
	0x8085,0x809B,0x8093,0x809A,0x80AD,0x5190,0x80AC,0x80DB,
	0x80E5,0x80D9,0x80DD,0x80C4,0x80DA,0x80D6,0x8109,0x80EF,
	0x80F1,0x811B,0x8129,0x8123,0x812F,0x814B,
	/* Range 0x6721 - 0x677E, array index: 0x1714 */
	0x968B,0x8146,0x813E,0x8153,0x8151,0x80FC,0x8171,0x816E,
	0x8165,0x8166,0x8174,0x8183,0x8188,0x818A,0x8180,0x8182,
	0x81A0,0x8195,0x81A4,0x81A3,0x815F,0x8193,0x81A9,0x81B0,
	0x81B5,0x81BE,0x81B8,0x81BD,0x81C0,0x81C2,0x81BA,0x81C9,
	0x81CD,0x81D1,0x81D9,0x81D8,0x81C8,0x81DA,0x81DF,0x81E0,
	0x81E7,0x81FA,0x81FB,0x81FE,0x8201,0x8202,0x8205,0x8207,
	0x820A,0x820D,0x8210,0x8216,0x8229,0x822B,0x8238,0x8233,
	0x8240,0x8259,0x8258,0x825D,0x825A,0x825F,0x8264,0x8262,
	0x8268,0x826A,0x826B,0x822E,0x8271,0x8277,0x8278,0x827E,
	0x828D,0x8292,0x82AB,0x829F,0x82BB,0x82AC,0x82E1,0x82E3,
	0x82DF,0x82D2,0x82F4,0x82F3,0x82FA,0x8393,0x8303,0x82FB,
	0x82F9,0x82DE,0x8306,0x82DC,0x8309,0x82D9,
	/* Range 0x6821 - 0x687E, array index: 0x1772 */
	0x8335,0x8334,0x8316,0x8332,0x8331,0x8340,0x8339,0x8350,
	0x8345,0x832F,0x832B,0x8317,0x8318,0x8385,0x839A,0x83AA,
	0x839F,0x83A2,0x8396,0x8323,0x838E,0x8387,0x838A,0x837C,
	0x83B5,0x8373,0x8375,0x83A0,0x8389,0x83A8,0x83F4,0x8413,
	0x83EB,0x83CE,0x83FD,0x8403,0x83D8,0x840B,0x83C1,0x83F7,
	0x8407,0x83E0,0x83F2,0x840D,0x8422,0x8420,0x83BD,0x8438,
	0x8506,0x83FB,0x846D,0x842A,0x843C,0x855A,0x8484,0x8477,
	0x846B,0x84AD,0x846E,0x8482,0x8469,0x8446,0x842C,0x846F,
	0x8479,0x8435,0x84CA,0x8462,0x84B9,0x84BF,0x849F,0x84D9,
	0x84CD,0x84BB,0x84DA,0x84D0,0x84C1,0x84C6,0x84D6,0x84A1,
	0x8521,0x84FF,0x84F4,0x8517,0x8518,0x852C,0x851F,0x8515,
	0x8514,0x84FC,0x8540,0x8563,0x8558,0x8548,
	/* Range 0x6921 - 0x697E, array index: 0x17D0 */
	0x8541,0x8602,0x854B,0x8555,0x8580,0x85A4,0x8588,0x8591,
	0x858A,0x85A8,0x856D,0x8594,0x859B,0x85EA,0x8587,0x859C,
	0x8577,0x857E,0x8590,0x85C9,0x85BA,0x85CF,0x85B9,0x85D0,
	0x85D5,0x85DD,0x85E5,0x85DC,0x85F9,0x860A,0x8613,0x860B,
	0x85FE,0x85FA,0x8606,0x8622,0x861A,0x8630,0x863F,0x864D,
	0x4E55,0x8654,0x865F,0x8667,0x8671,0x8693,0x86A3,0x86A9,
	0x86AA,0x868B,0x868C,0x86B6,0x86AF,0x86C4,0x86C6,0x86B0,
	0x86C9,0x8823,0x86AB,0x86D4,0x86DE,0x86E9,0x86EC,0x86DF,
	0x86DB,0x86EF,0x8712,0x8706,0x8708,0x8700,0x8703,0x86FB,
	0x8711,0x8709,0x870D,0x86F9,0x870A,0x8734,0x873F,0x8737,
	0x873B,0x8725,0x8729,0x871A,0x8760,0x875F,0x8778,0x874C,
	0x874E,0x8774,0x8757,0x8768,0x876E,0x8759,
	/* Range 0x6A21 - 0x6A7E, array index: 0x182E */
	0x8753,0x8763,0x876A,0x8805,0x87A2,0x879F,0x8782,0x87AF,
	0x87CB,0x87BD,0x87C0,0x87D0,0x96D6,0x87AB,0x87C4,0x87B3,
	0x87C7,0x87C6,0x87BB,0x87EF,0x87F2,0x87E0,0x880F,0x880D,
	0x87FE,0x87F6,0x87F7,0x880E,0x87D2,0x8811,0x8816,0x8815,
	0x8822,0x8821,0x8831,0x8836,0x8839,0x8827,0x883B,0x8844,
	0x8842,0x8852,0x8859,0x885E,0x8862,0x886B,0x8881,0x887E,
	0x889E,0x8875,0x887D,0x88B5,0x8872,0x8882,0x8897,0x8892,
	0x88AE,0x8899,0x88A2,0x888D,0x88A4,0x88B0,0x88BF,0x88B1,
	0x88C3,0x88C4,0x88D4,0x88D8,0x88D9,0x88DD,0x88F9,0x8902,
	0x88FC,0x88F4,0x88E8,0x88F2,0x8904,0x890C,0x890A,0x8913,
	0x8943,0x891E,0x8925,0x892A,0x892B,0x8941,0x8944,0x893B,
	0x8936,0x8938,0x894C,0x891D,0x8960,0x895E,
	/* Range 0x6B21 - 0x6B7E, array index: 0x188C */
	0x8966,0x8964,0x896D,0x896A,0x896F,0x8974,0x8977,0x897E,
	0x8983,0x8988,0x898A,0x8993,0x8998,0x89A1,0x89A9,0x89A6,
	0x89AC,0x89AF,0x89B2,0x89BA,0x89BD,0x89BF,0x89C0,0x89DA,
	0x89DC,0x89DD,0x89E7,0x89F4,0x89F8,0x8A03,0x8A16,0x8A10,
	0x8A0C,0x8A1B,0x8A1D,0x8A25,0x8A36,0x8A41,0x8A5B,0x8A52,
	0x8A46,0x8A48,0x8A7C,0x8A6D,0x8A6C,0x8A62,0x8A85,0x8A82,
	0x8A84,0x8AA8,0x8AA1,0x8A91,0x8AA5,0x8AA6,0x8A9A,0x8AA3,
	0x8AC4,0x8ACD,0x8AC2,0x8ADA,0x8AEB,0x8AF3,0x8AE7,0x8AE4,
	0x8AF1,0x8B14,0x8AE0,0x8AE2,0x8AF7,0x8ADE,0x8ADB,0x8B0C,
	0x8B07,0x8B1A,0x8AE1,0x8B16,0x8B10,0x8B17,0x8B20,0x8B33,
	0x97AB,0x8B26,0x8B2B,0x8B3E,0x8B28,0x8B41,0x8B4C,0x8B4F,
	0x8B4E,0x8B49,0x8B56,0x8B5B,0x8B5A,0x8B6B,
	/* Range 0x6C21 - 0x6C7E, array index: 0x18EA */
	0x8B5F,0x8B6C,0x8B6F,0x8B74,0x8B7D,0x8B80,0x8B8C,0x8B8E,
	0x8B92,0x8B93,0x8B96,0x8B99,0x8B9A,0x8C3A,0x8C41,0x8C3F,
	0x8C48,0x8C4C,0x8C4E,0x8C50,0x8C55,0x8C62,0x8C6C,0x8C78,
	0x8C7A,0x8C82,0x8C89,0x8C85,0x8C8A,0x8C8D,0x8C8E,0x8C94,
	0x8C7C,0x8C98,0x621D,0x8CAD,0x8CAA,0x8CBD,0x8CB2,0x8CB3,
	0x8CAE,0x8CB6,0x8CC8,0x8CC1,0x8CE4,0x8CE3,0x8CDA,0x8CFD,
	0x8CFA,0x8CFB,0x8D04,0x8D05,0x8D0A,0x8D07,0x8D0F,0x8D0D,
	0x8D10,0x9F4E,0x8D13,0x8CCD,0x8D14,0x8D16,0x8D67,0x8D6D,
	0x8D71,0x8D73,0x8D81,0x8D99,0x8DC2,0x8DBE,0x8DBA,0x8DCF,
	0x8DDA,0x8DD6,0x8DCC,0x8DDB,0x8DCB,0x8DEA,0x8DEB,0x8DDF,
	0x8DE3,0x8DFC,0x8E08,0x8E09,0x8DFF,0x8E1D,0x8E1E,0x8E10,
	0x8E1F,0x8E42,0x8E35,0x8E30,0x8E34,0x8E4A,
	/* Range 0x6D21 - 0x6D7E, array index: 0x1948 */
	0x8E47,0x8E49,0x8E4C,0x8E50,0x8E48,0x8E59,0x8E64,0x8E60,
	0x8E2A,0x8E63,0x8E55,0x8E76,0x8E72,0x8E7C,0x8E81,0x8E87,
	0x8E85,0x8E84,0x8E8B,0x8E8A,0x8E93,0x8E91,0x8E94,0x8E99,
	0x8EAA,0x8EA1,0x8EAC,0x8EB0,0x8EC6,0x8EB1,0x8EBE,0x8EC5,
	0x8EC8,0x8ECB,0x8EDB,0x8EE3,0x8EFC,0x8EFB,0x8EEB,0x8EFE,
	0x8F0A,0x8F05,0x8F15,0x8F12,0x8F19,0x8F13,0x8F1C,0x8F1F,
	0x8F1B,0x8F0C,0x8F26,0x8F33,0x8F3B,0x8F39,0x8F45,0x8F42,
	0x8F3E,0x8F4C,0x8F49,0x8F46,0x8F4E,0x8F57,0x8F5C,0x8F62,
	0x8F63,0x8F64,0x8F9C,0x8F9F,0x8FA3,0x8FAD,0x8FAF,0x8FB7,
	0x8FDA,0x8FE5,0x8FE2,0x8FEA,0x8FEF,0x9087,0x8FF4,0x9005,
	0x8FF9,0x8FFA,0x9011,0x9015,0x9021,0x900D,0x901E,0x9016,
	0x900B,0x9027,0x9036,0x9035,0x9039,0x8FF8,
	/* Range 0x6E21 - 0x6E7E, array index: 0x19A6 */
	0x904F,0x9050,0x9051,0x9052,0x900E,0x9049,0x903E,0x9056,
	0x9058,0x905E,0x9068,0x906F,0x9076,0x96A8,0x9072,0x9082,
	0x907D,0x9081,0x9080,0x908A,0x9089,0x908F,0x90A8,0x90AF,
	0x90B1,0x90B5,0x90E2,0x90E4,0x6248,0x90DB,0x9102,0x9112,
	0x9119,0x9132,0x9130,0x914A,0x9156,0x9158,0x9163,0x9165,
	0x9169,0x9173,0x9172,0x918B,0x9189,0x9182,0x91A2,0x91AB,
	0x91AF,0x91AA,0x91B5,0x91B4,0x91BA,0x91C0,0x91C1,0x91C9,
	0x91CB,0x91D0,0x91D6,0x91DF,0x91E1,0x91DB,0x91FC,0x91F5,
	0x91F6,0x921E,0x91FF,0x9214,0x922C,0x9215,0x9211,0x925E,
	0x9257,0x9245,0x9249,0x9264,0x9248,0x9295,0x923F,0x924B,
	0x9250,0x929C,0x9296,0x9293,0x929B,0x925A,0x92CF,0x92B9,
	0x92B7,0x92E9,0x930F,0x92FA,0x9344,0x932E,
	/* Range 0x6F21 - 0x6F7E, array index: 0x1A04 */
	0x9319,0x9322,0x931A,0x9323,0x933A,0x9335,0x933B,0x935C,
	0x9360,0x937C,0x936E,0x9356,0x93B0,0x93AC,0x93AD,0x9394,
	0x93B9,0x93D6,0x93D7,0x93E8,0x93E5,0x93D8,0x93C3,0x93DD,
	0x93D0,0x93C8,0x93E4,0x941A,0x9414,0x9413,0x9403,0x9407,
	0x9410,0x9436,0x942B,0x9435,0x9421,0x943A,0x9441,0x9452,
	0x9444,0x945B,0x9460,0x9462,0x945E,0x946A,0x9229,0x9470,
	0x9475,0x9477,0x947D,0x945A,0x947C,0x947E,0x9481,0x947F,
	0x9582,0x9587,0x958A,0x9594,0x9596,0x9598,0x9599,0x95A0,
	0x95A8,0x95A7,0x95AD,0x95BC,0x95BB,0x95B9,0x95BE,0x95CA,
	0x6FF6,0x95C3,0x95CD,0x95CC,0x95D5,0x95D4,0x95D6,0x95DC,
	0x95E1,0x95E5,0x95E2,0x9621,0x9628,0x962E,0x962F,0x9642,
	0x964C,0x964F,0x964B,0x9677,0x965C,0x965E,
	/* Range 0x7021 - 0x707E, array index: 0x1A62 */
	0x965D,0x965F,0x9666,0x9672,0x966C,0x968D,0x9698,0x9695,
	0x9697,0x96AA,0x96A7,0x96B1,0x96B2,0x96B0,0x96B4,0x96B6,
	0x96B8,0x96B9,0x96CE,0x96CB,0x96C9,0x96CD,0x894D,0x96DC,
	0x970D,0x96D5,0x96F9,0x9704,0x9706,0x9708,0x9713,0x970E,
	0x9711,0x970F,0x9716,0x9719,0x9724,0x972A,0x9730,0x9739,
	0x973D,0x973E,0x9744,0x9746,0x9748,0x9742,0x9749,0x975C,
	0x9760,0x9764,0x9766,0x9768,0x52D2,0x976B,0x9771,0x9779,
	0x9785,0x977C,0x9781,0x977A,0x9786,0x978B,0x978F,0x9790,
	0x979C,0x97A8,0x97A6,0x97A3,0x97B3,0x97B4,0x97C3,0x97C6,
	0x97C8,0x97CB,0x97DC,0x97ED,0x9F4F,0x97F2,0x7ADF,0x97F6,
	0x97F5,0x980F,0x980C,0x9838,0x9824,0x9821,0x9837,0x983D,
	0x9846,0x984F,0x984B,0x986B,0x986F,0x9870,
	/* Range 0x7121 - 0x717E, array index: 0x1AC0 */
	0x9871,0x9874,0x9873,0x98AA,0x98AF,0x98B1,0x98B6,0x98C4,
	0x98C3,0x98C6,0x98E9,0x98EB,0x9903,0x9909,0x9912,0x9914,
	0x9918,0x9921,0x991D,0x991E,0x9924,0x9920,0x992C,0x992E,
	0x993D,0x993E,0x9942,0x9949,0x9945,0x9950,0x994B,0x9951,
	0x9952,0x994C,0x9955,0x9997,0x9998,0x99A5,0x99AD,0x99AE,
	0x99BC,0x99DF,0x99DB,0x99DD,0x99D8,0x99D1,0x99ED,0x99EE,
	0x99F1,0x99F2,0x99FB,0x99F8,0x9A01,0x9A0F,0x9A05,0x99E2,
	0x9A19,0x9A2B,0x9A37,0x9A45,0x9A42,0x9A40,0x9A43,0x9A3E,
	0x9A55,0x9A4D,0x9A5B,0x9A57,0x9A5F,0x9A62,0x9A65,0x9A64,
	0x9A69,0x9A6B,0x9A6A,0x9AAD,0x9AB0,0x9ABC,0x9AC0,0x9ACF,
	0x9AD1,0x9AD3,0x9AD4,0x9ADE,0x9ADF,0x9AE2,0x9AE3,0x9AE6,
	0x9AEF,0x9AEB,0x9AEE,0x9AF4,0x9AF1,0x9AF7,
	/* Range 0x7221 - 0x727E, array index: 0x1B1E */
	0x9AFB,0x9B06,0x9B18,0x9B1A,0x9B1F,0x9B22,0x9B23,0x9B25,
	0x9B27,0x9B28,0x9B29,0x9B2A,0x9B2E,0x9B2F,0x9B32,0x9B44,
	0x9B43,0x9B4F,0x9B4D,0x9B4E,0x9B51,0x9B58,0x9B74,0x9B93,
	0x9B83,0x9B91,0x9B96,0x9B97,0x9B9F,0x9BA0,0x9BA8,0x9BB4,
	0x9BC0,0x9BCA,0x9BB9,0x9BC6,0x9BCF,0x9BD1,0x9BD2,0x9BE3,
	0x9BE2,0x9BE4,0x9BD4,0x9BE1,0x9C3A,0x9BF2,0x9BF1,0x9BF0,
	0x9C15,0x9C14,0x9C09,0x9C13,0x9C0C,0x9C06,0x9C08,0x9C12,
	0x9C0A,0x9C04,0x9C2E,0x9C1B,0x9C25,0x9C24,0x9C21,0x9C30,
	0x9C47,0x9C32,0x9C46,0x9C3E,0x9C5A,0x9C60,0x9C67,0x9C76,
	0x9C78,0x9CE7,0x9CEC,0x9CF0,0x9D09,0x9D08,0x9CEB,0x9D03,
	0x9D06,0x9D2A,0x9D26,0x9DAF,0x9D23,0x9D1F,0x9D44,0x9D15,
	0x9D12,0x9D41,0x9D3F,0x9D3E,0x9D46,0x9D48,
	/* Range 0x7321 - 0x737E, array index: 0x1B7C */
	0x9D5D,0x9D5E,0x9D64,0x9D51,0x9D50,0x9D59,0x9D72,0x9D89,
	0x9D87,0x9DAB,0x9D6F,0x9D7A,0x9D9A,0x9DA4,0x9DA9,0x9DB2,
	0x9DC4,0x9DC1,0x9DBB,0x9DB8,0x9DBA,0x9DC6,0x9DCF,0x9DC2,
	0x9DD9,0x9DD3,0x9DF8,0x9DE6,0x9DED,0x9DEF,0x9DFD,0x9E1A,
	0x9E1B,0x9E1E,0x9E75,0x9E79,0x9E7D,0x9E81,0x9E88,0x9E8B,
	0x9E8C,0x9E92,0x9E95,0x9E91,0x9E9D,0x9EA5,0x9EA9,0x9EB8,
	0x9EAA,0x9EAD,0x9761,0x9ECC,0x9ECE,0x9ECF,0x9ED0,0x9ED4,
	0x9EDC,0x9EDE,0x9EDD,0x9EE0,0x9EE5,0x9EE8,0x9EEF,0x9EF4,
	0x9EF6,0x9EF7,0x9EF9,0x9EFB,0x9EFC,0x9EFD,0x9F07,0x9F08,
	0x76B7,0x9F15,0x9F21,0x9F2C,0x9F3E,0x9F4A,0x9F52,0x9F54,
	0x9F63,0x9F5F,0x9F60,0x9F61,0x9F66,0x9F67,0x9F6C,0x9F6A,
	0x9F77,0x9F72,0x9F76,0x9F95,0x9F9C,0x9FA0,
	/* Range 0x7421 - 0x7426, array index: 0x1BDA */
	0x582F,0x69C7,0x9059,0x7464,0x51DC,0x7199,
	/* Unranged codes (85 codes) */
	/* Array index: 0x1BE0 */ 0x227E,0x25EF,
};

#endif /* ICONV_TO_UCS_CCS_JIS_X0208_1990 && defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit UCS -> jis_x0208_1990 speed-optimized table (48128 bytes).
 * ======================================================================
 */

#if defined (ICONV_FROM_UCS_CCS_JIS_X0208_1990) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
from_ucs_speed_jis_x0208_1990[] =
{
	/* Heading Block */
	0x0100,INVBLK,INVBLK,0x0200,0x0300,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	0x0400,0x0500,0x0600,0x0700,INVBLK,0x0800,0x0900,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	0x0A00,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,0x0B00,0x0C00,
	0x0D00,0x0E00,0x0F00,0x1000,0x1100,0x1200,0x1300,0x1400,
	0x1500,0x1600,0x1700,0x1800,0x1900,0x1A00,0x1B00,0x1C00,
	0x1D00,0x1E00,0x1F00,0x2000,0x2100,0x2200,0x2300,0x2400,
	0x2500,0x2600,0x2700,0x2800,0x2900,0x2A00,0x2B00,0x2C00,
	0x2D00,0x2E00,0x2F00,0x3000,0x3100,0x3200,0x3300,0x3400,
	0x3500,0x3600,0x3700,0x3800,0x3900,0x3A00,0x3B00,0x3C00,
	0x3D00,0x3E00,0x3F00,0x4000,0x4100,0x4200,0x4300,0x4400,
	0x4500,0x4600,0x4700,0x4800,0x4900,0x4A00,0x4B00,0x4C00,
	0x4D00,0x4E00,0x4F00,0x5000,0x5100,0x5200,0x5300,0x5400,
	0x5500,0x5600,0x5700,0x5800,0x5900,0x5A00,0x5B00,0x5C00,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,0x5D00,
	/* Block 1, Array index 0x0100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2140,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2171,0x2172,INVALC,INVALC,INVALC,0x2178,
	0x212F,INVALC,INVALC,INVALC,0x224C,INVALC,INVALC,INVALC,
	0x216B,0x215E,INVALC,INVALC,0x212D,INVALC,0x2279,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x215F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2160,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 4, Array index 0x0200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2621,0x2622,0x2623,0x2624,0x2625,0x2626,0x2627,
	0x2628,0x2629,0x262A,0x262B,0x262C,0x262D,0x262E,0x262F,
	0x2630,0x2631,INVALC,0x2632,0x2633,0x2634,0x2635,0x2636,
	0x2637,0x2638,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2641,0x2642,0x2643,0x2644,0x2645,0x2646,0x2647,
	0x2648,0x2649,0x264A,0x264B,0x264C,0x264D,0x264E,0x264F,
	0x2650,0x2651,INVALC,0x2652,0x2653,0x2654,0x2655,0x2656,
	0x2657,0x2658,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 5, Array index 0x0300 */
	INVALC,0x2727,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2721,0x2722,0x2723,0x2724,0x2725,0x2726,0x2728,0x2729,
	0x272A,0x272B,0x272C,0x272D,0x272E,0x272F,0x2730,0x2731,
	0x2732,0x2733,0x2734,0x2735,0x2736,0x2737,0x2738,0x2739,
	0x273A,0x273B,0x273C,0x273D,0x273E,0x273F,0x2740,0x2741,
	0x2751,0x2752,0x2753,0x2754,0x2755,0x2756,0x2758,0x2759,
	0x275A,0x275B,0x275C,0x275D,0x275E,0x275F,0x2760,0x2761,
	0x2762,0x2763,0x2764,0x2765,0x2766,0x2767,0x2768,0x2769,
	0x276A,0x276B,0x276C,0x276D,0x276E,0x276F,0x2770,0x2771,
	INVALC,0x2757,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 33, Array index 0x0400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x213E,INVALC,INVALC,INVALC,INVALC,0x213D,0x2142,INVALC,
	0x2146,0x2147,INVALC,INVALC,0x2148,0x2149,INVALC,INVALC,
	0x2277,0x2278,INVALC,INVALC,INVALC,0x2145,0x2144,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2273,INVALC,0x216C,0x216D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2228,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 34, Array index 0x0500 */
	INVALC,INVALC,INVALC,0x216E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2272,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x222B,0x222C,0x222A,0x222D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x224D,INVALC,0x224E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 35, Array index 0x0600 */
	0x224F,INVALC,0x225F,0x2250,INVALC,INVALC,INVALC,0x2260,
	0x223A,INVALC,INVALC,0x223B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x215D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2265,INVALC,INVALC,0x2267,0x2167,INVALC,
	0x225C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x224A,
	0x224B,0x2241,0x2240,0x2269,0x226A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2168,0x2268,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2266,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2262,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2162,0x2261,INVALC,INVALC,INVALC,INVALC,0x2165,0x2166,
	INVALC,INVALC,0x2263,0x2264,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x223E,0x223F,INVALC,INVALC,0x223C,0x223D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x225D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 36, Array index 0x0700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x225E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 38, Array index 0x0800 */
	0x2821,0x282C,0x2822,0x282D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2823,INVALC,INVALC,0x282E,
	0x2824,INVALC,INVALC,0x282F,0x2826,INVALC,INVALC,0x2831,
	0x2825,INVALC,INVALC,0x2830,0x2827,0x283C,INVALC,INVALC,
	0x2837,INVALC,INVALC,0x2832,0x2829,0x283E,INVALC,INVALC,
	0x2839,INVALC,INVALC,0x2834,0x2828,INVALC,INVALC,0x2838,
	0x283D,INVALC,INVALC,0x2833,0x282A,INVALC,INVALC,0x283A,
	0x283F,INVALC,INVALC,0x2835,0x282B,INVALC,INVALC,0x283B,
	INVALC,INVALC,0x2840,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2836,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2223,0x2222,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2225,0x2224,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2227,0x2226,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2221,0x217E,
	INVALC,INVALC,INVALC,0x217B,INVALC,INVALC,0x217D,0x217C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x227E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 39, Array index 0x0900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x217A,0x2179,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x216A,INVALC,0x2169,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2276,INVALC,INVALC,0x2275,INVALC,0x2274,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 49, Array index 0x0A00 */
	0x2121,0x2122,0x2123,0x2137,INVALC,0x2139,0x213A,0x213B,
	0x2152,0x2153,0x2154,0x2155,0x2156,0x2157,0x2158,0x2159,
	0x215A,0x215B,0x2229,0x222E,0x214C,0x214D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2141,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2421,0x2422,0x2423,0x2424,0x2425,0x2426,0x2427,
	0x2428,0x2429,0x242A,0x242B,0x242C,0x242D,0x242E,0x242F,
	0x2430,0x2431,0x2432,0x2433,0x2434,0x2435,0x2436,0x2437,
	0x2438,0x2439,0x243A,0x243B,0x243C,0x243D,0x243E,0x243F,
	0x2440,0x2441,0x2442,0x2443,0x2444,0x2445,0x2446,0x2447,
	0x2448,0x2449,0x244A,0x244B,0x244C,0x244D,0x244E,0x244F,
	0x2450,0x2451,0x2452,0x2453,0x2454,0x2455,0x2456,0x2457,
	0x2458,0x2459,0x245A,0x245B,0x245C,0x245D,0x245E,0x245F,
	0x2460,0x2461,0x2462,0x2463,0x2464,0x2465,0x2466,0x2467,
	0x2468,0x2469,0x246A,0x246B,0x246C,0x246D,0x246E,0x246F,
	0x2470,0x2471,0x2472,0x2473,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x212B,0x212C,0x2135,0x2136,INVALC,
	INVALC,0x2521,0x2522,0x2523,0x2524,0x2525,0x2526,0x2527,
	0x2528,0x2529,0x252A,0x252B,0x252C,0x252D,0x252E,0x252F,
	0x2530,0x2531,0x2532,0x2533,0x2534,0x2535,0x2536,0x2537,
	0x2538,0x2539,0x253A,0x253B,0x253C,0x253D,0x253E,0x253F,
	0x2540,0x2541,0x2542,0x2543,0x2544,0x2545,0x2546,0x2547,
	0x2548,0x2549,0x254A,0x254B,0x254C,0x254D,0x254E,0x254F,
	0x2550,0x2551,0x2552,0x2553,0x2554,0x2555,0x2556,0x2557,
	0x2558,0x2559,0x255A,0x255B,0x255C,0x255D,0x255E,0x255F,
	0x2560,0x2561,0x2562,0x2563,0x2564,0x2565,0x2566,0x2567,
	0x2568,0x2569,0x256A,0x256B,0x256C,0x256D,0x256E,0x256F,
	0x2570,0x2571,0x2572,0x2573,0x2574,0x2575,0x2576,INVALC,
	INVALC,INVALC,INVALC,0x2126,0x213C,0x2133,0x2134,INVALC,
	/* Block 79, Array index 0x0B00 */
	0x306C,0x437A,INVALC,0x3C37,INVALC,INVALC,INVALC,0x4B7C,
	0x3E66,0x3B30,0x3E65,0x323C,INVALC,0x4954,0x4D3F,INVALC,
	0x5022,0x312F,INVALC,INVALC,0x336E,0x5023,0x4024,0x5242,
	0x3556,0x4A3A,INVALC,INVALC,INVALC,INVALC,0x3E67,INVALC,
	INVALC,0x4E3E,INVALC,INVALC,INVALC,INVALC,0x4A42,INVALC,
	INVALC,INVALC,0x5024,INVALC,INVALC,0x4366,INVALC,INVALC,
	INVALC,0x5025,0x367A,INVALC,INVALC,INVALC,0x5026,INVALC,
	0x345D,0x4330,INVALC,0x3C67,0x5027,INVALC,INVALC,0x5028,
	INVALC,INVALC,0x5029,0x4735,INVALC,0x3557,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4737,INVALC,0x4663,0x3843,0x4B33,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6949,0x502A,0x3E68,
	0x502B,0x3235,INVALC,INVALC,INVALC,0x3665,0x3870,0x4C69,
	INVALC,INVALC,0x5626,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D70,INVALC,0x467D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3425,INVALC,
	0x3535,INVALC,0x502C,INVALC,INVALC,0x502D,0x4E3B,INVALC,
	0x4D3D,0x4168,0x502F,0x3B76,0x4673,INVALC,0x5032,INVALC,
	INVALC,0x313E,0x385F,INVALC,0x385E,0x3066,INVALC,INVALC,
	0x4F4B,0x4F4A,INVALC,0x3A33,0x3021,INVALC,0x5033,0x5034,
	0x5035,0x4B34,0x5036,INVALC,0x3872,0x3067,0x4B72,INVALC,
	0x357C,INVALC,INVALC,0x357D,0x357E,0x4462,0x4E3C,INVALC,
	0x5037,INVALC,INVALC,0x5038,INVALC,INVALC,0x5039,INVALC,
	INVALC,INVALC,0x3F4D,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D3A,0x3F4E,0x503E,INVALC,0x503C,INVALC,0x503D,0x3558,
	INVALC,INVALC,0x3A23,0x3270,INVALC,0x503B,0x503A,0x4A29,
	INVALC,INVALC,INVALC,INVALC,0x3B46,0x3B45,0x423E,0x503F,
	0x4955,0x4067,INVALC,INVALC,INVALC,0x2138,0x5040,0x5042,
	INVALC,INVALC,INVALC,0x4265,0x4E61,0x304A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5041,0x323E,INVALC,
	0x3644,INVALC,0x4367,INVALC,INVALC,INVALC,0x376F,0x5043,
	INVALC,INVALC,INVALC,0x4724,INVALC,INVALC,INVALC,INVALC,
	/* Block 80, Array index 0x0C00 */
	INVALC,0x346B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5044,0x304B,INVALC,INVALC,0x3860,0x346C,0x497A,
	0x4832,0x3559,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3271,INVALC,0x5067,0x4541,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x476C,
	0x5046,INVALC,INVALC,INVALC,0x483C,INVALC,0x4E62,INVALC,
	0x3F2D,INVALC,0x3B47,INVALC,0x3B77,0x3240,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4451,INVALC,INVALC,0x4322,0x504A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x304C,0x4463,0x3D3B,
	0x3A34,0x4D24,INVALC,0x424E,INVALC,0x323F,INVALC,0x5049,
	INVALC,0x4D3E,0x5045,0x5047,0x3A6E,0x5048,0x5524,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5050,INVALC,INVALC,INVALC,INVALC,INVALC,0x5053,
	0x5051,INVALC,INVALC,0x3242,INVALC,0x4A3B,0x504B,INVALC,
	INVALC,INVALC,INVALC,0x504F,0x3873,INVALC,INVALC,0x3B48,
	INVALC,INVALC,INVALC,0x3426,INVALC,INVALC,0x5054,INVALC,
	0x504C,INVALC,INVALC,0x4E63,INVALC,0x3B78,INVALC,0x504D,
	INVALC,0x5052,INVALC,INVALC,INVALC,INVALC,0x5055,INVALC,
	0x504E,INVALC,INVALC,0x3621,INVALC,0x304D,INVALC,INVALC,
	0x3622,0x3241,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5525,INVALC,0x4B79,0x496E,0x3874,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F2F,0x4E37,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A58,
	INVALC,INVALC,0x3738,0x4225,0x3264,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3D53,INVALC,INVALC,INVALC,0x5059,INVALC,
	0x505E,0x505C,INVALC,INVALC,0x5057,INVALC,INVALC,0x422F,
	0x505A,INVALC,0x505D,0x505B,INVALC,0x4A5D,INVALC,0x5058,
	INVALC,0x3F2E,INVALC,0x4B73,0x505F,0x5060,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D24,0x506D,
	INVALC,INVALC,INVALC,0x4750,INVALC,0x4936,0x5068,INVALC,
	0x4A70,INVALC,0x3236,INVALC,INVALC,INVALC,0x506C,INVALC,
	/* Block 81, Array index 0x0D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5066,0x506F,INVALC,
	INVALC,0x4152,INVALC,0x3844,INVALC,0x475C,INVALC,0x6047,
	INVALC,0x506E,0x455D,INVALC,0x5063,INVALC,0x3876,INVALC,
	INVALC,0x3875,0x5061,INVALC,INVALC,INVALC,INVALC,0x3C5A,
	INVALC,0x5069,INVALC,0x4A6F,0x434D,0x5065,0x3771,INVALC,
	0x5062,0x506A,0x5064,0x4E51,0x506B,0x4F41,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3666,INVALC,
	INVALC,0x3770,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5070,INVALC,INVALC,INVALC,0x5071,
	0x5075,0x304E,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A50,
	0x5074,INVALC,INVALC,INVALC,INVALC,0x5073,0x5077,INVALC,
	INVALC,INVALC,0x5076,INVALC,0x4464,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3772,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5078,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3C45,INVALC,0x4226,0x4465,0x3676,INVALC,
	0x5079,INVALC,INVALC,INVALC,INVALC,0x3536,INVALC,INVALC,
	0x507A,INVALC,INVALC,INVALC,INVALC,0x507C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B35,INVALC,INVALC,
	INVALC,0x3766,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B31,0x4877,0x507B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A45,0x4D43,INVALC,INVALC,
	INVALC,INVALC,0x507E,0x5123,0x507D,0x3A44,INVALC,0x3D7D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3739,INVALC,
	INVALC,INVALC,0x5124,INVALC,INVALC,0x364F,INVALC,INVALC,
	INVALC,0x5121,0x5122,INVALC,INVALC,0x462F,INVALC,0x417C,
	INVALC,0x3623,INVALC,INVALC,INVALC,0x4B4D,0x5125,INVALC,
	INVALC,INVALC,0x4E3D,INVALC,INVALC,INVALC,0x5126,INVALC,
	INVALC,INVALC,INVALC,0x5129,INVALC,0x5127,INVALC,0x414E,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5128,0x512A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x512C,INVALC,INVALC,
	INVALC,0x512B,INVALC,0x4A48,INVALC,INVALC,INVALC,INVALC,
	/* Block 82, Array index 0x0E00 */
	0x3537,0x512E,0x512F,INVALC,0x322F,INVALC,INVALC,INVALC,
	INVALC,0x512D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3C74,INVALC,0x5132,0x5131,0x5130,INVALC,
	0x5056,INVALC,0x5133,INVALC,INVALC,INVALC,INVALC,0x3D7E,
	INVALC,0x5134,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D25,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C59,INVALC,INVALC,INVALC,INVALC,0x5136,
	INVALC,INVALC,0x5135,0x5138,0x5137,INVALC,INVALC,0x5139,
	0x513A,0x3074,INVALC,0x3835,0x373B,0x3D3C,0x437B,0x3624,
	0x4068,0x3877,INVALC,0x396E,0x513C,0x4C48,0x4546,INVALC,
	0x3B79,INVALC,0x513B,INVALC,0x513D,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x455E,INVALC,0x3375,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x513E,INVALC,INVALC,0x467E,INVALC,INVALC,
	0x4134,0x5140,0x5141,0x482C,0x3878,0x4F3B,0x5142,INVALC,
	INVALC,0x3626,INVALC,INVALC,INVALC,0x4A3C,0x4236,0x3671,
	0x4535,INVALC,INVALC,INVALC,0x3773,INVALC,INVALC,INVALC,
	0x5143,INVALC,0x5144,INVALC,INVALC,0x4662,0x315F,INVALC,
	INVALC,0x5147,0x3A7D,INVALC,0x5146,0x3A46,INVALC,0x5148,
	0x666E,0x5149,0x4B41,0x514A,INVALC,0x514B,0x514C,0x3E69,
	INVALC,0x3C4C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3427,INVALC,0x514F,INVALC,0x514D,0x4C3D,0x514E,INVALC,
	0x495A,0x5150,0x5151,0x5152,0x455F,INVALC,INVALC,INVALC,
	0x5156,0x5154,0x5155,0x5153,0x3A63,0x5157,0x4C6A,0x4E64,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5158,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4028,0x5159,0x3D5A,INVALC,
	INVALC,0x515A,INVALC,0x437C,0x4E3F,0x4560,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5245,INVALC,
	INVALC,INVALC,INVALC,0x515B,0x7425,0x3645,INVALC,INVALC,
	0x515C,0x4B5E,INVALC,INVALC,INVALC,INVALC,0x3D68,0x427C,
	INVALC,0x515E,0x4664,INVALC,INVALC,0x515F,INVALC,INVALC,
	0x5160,0x332E,INVALC,INVALC,INVALC,0x5161,0x3627,INVALC,
	0x464C,0x317A,0x3D50,INVALC,INVALC,0x4821,0x5162,INVALC,
	/* Block 83, Array index 0x0F00 */
	0x4561,INVALC,INVALC,0x3F4F,0x5163,INVALC,0x4A2C,0x405A,
	0x3422,INVALC,0x3429,0x5164,INVALC,INVALC,0x5166,INVALC,
	INVALC,0x373A,INVALC,INVALC,0x5165,INVALC,INVALC,0x4E73,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3D69,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x483D,0x4A4C,INVALC,0x5167,
	INVALC,0x4D78,0x5168,INVALC,INVALC,INVALC,0x5169,INVALC,
	0x457E,INVALC,INVALC,0x516A,INVALC,INVALC,0x4029,0x3A7E,
	0x3774,0x516B,0x3B49,0x396F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4466,0x516D,INVALC,INVALC,0x4227,
	INVALC,INVALC,0x3A6F,0x516E,0x516F,0x4130,INVALC,0x516C,
	INVALC,INVALC,INVALC,INVALC,0x5171,INVALC,0x4B36,INVALC,
	INVALC,INVALC,INVALC,0x3964,INVALC,INVALC,0x5170,INVALC,
	INVALC,INVALC,INVALC,0x3775,0x3A5E,0x476D,INVALC,INVALC,
	INVALC,0x5174,0x5172,INVALC,INVALC,INVALC,INVALC,0x497B,
	0x3E6A,0x517B,0x3364,0x5175,0x5173,0x414F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5177,INVALC,0x5176,
	INVALC,INVALC,INVALC,0x3344,INVALC,INVALC,INVALC,0x3760,
	0x517C,0x4E2D,INVALC,INVALC,INVALC,0x5178,INVALC,INVALC,
	INVALC,0x517D,0x517A,INVALC,0x5179,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E4F,INVALC,INVALC,INVALC,0x3879,
	0x3243,INVALC,INVALC,0x4E74,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3D75,0x4558,0x3965,0x5222,0x5223,INVALC,INVALC,
	INVALC,0x4E65,INVALC,INVALC,0x4F2B,0x5225,INVALC,INVALC,
	INVALC,0x387A,INVALC,INVALC,0x5224,INVALC,0x332F,INVALC,
	INVALC,0x5226,INVALC,0x4B56,INVALC,0x443C,INVALC,0x4D26,
	INVALC,0x4A59,INVALC,INVALC,INVALC,0x5227,INVALC,INVALC,
	INVALC,INVALC,0x7055,INVALC,INVALC,0x4630,INVALC,0x5228,
	0x342A,0x4C33,INVALC,INVALC,INVALC,0x3E21,0x5229,0x4A67,
	0x522D,INVALC,0x402A,0x522A,0x3650,INVALC,0x522B,0x342B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x372E,0x522E,INVALC,0x522F,INVALC,INVALC,
	0x5230,0x5231,0x3C5B,INVALC,INVALC,INVALC,0x387B,0x4C5E,
	/* Block 84, Array index 0x1000 */
	INVALC,0x4C68,0x4677,INVALC,INVALC,0x4A71,0x5232,INVALC,
	0x5233,INVALC,INVALC,INVALC,INVALC,0x5235,INVALC,0x5237,
	0x5236,INVALC,INVALC,INVALC,INVALC,0x5238,0x323D,0x4B4C,
	INVALC,0x3A7C,0x5239,INVALC,INVALC,0x4159,INVALC,INVALC,
	0x3E22,0x3629,INVALC,0x523A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x485B,INVALC,INVALC,INVALC,INVALC,0x523B,
	INVALC,0x523C,INVALC,0x523D,INVALC,INVALC,INVALC,INVALC,
	0x523E,0x4924,0x3668,0x3065,INVALC,INVALC,INVALC,0x463F,
	0x523F,0x3D3D,INVALC,0x4069,INVALC,0x5241,0x5240,0x3E23,
	0x3861,0x5243,0x483E,INVALC,INVALC,0x5244,INVALC,INVALC,
	INVALC,0x485C,0x4234,0x426E,0x3628,INVALC,INVALC,0x466E,
	0x4331,INVALC,0x476E,INVALC,0x4B4E,INVALC,0x5246,INVALC,
	0x406A,INVALC,INVALC,INVALC,INVALC,INVALC,0x3735,INVALC,
	INVALC,0x5247,INVALC,INVALC,INVALC,INVALC,0x5248,0x312C,
	0x3075,0x346D,INVALC,0x4228,0x3551,0x4D71,INVALC,0x524B,
	0x3237,INVALC,INVALC,0x524A,INVALC,INVALC,INVALC,0x362A,
	INVALC,INVALC,0x524C,INVALC,0x4C71,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x524D,INVALC,
	0x4E52,INVALC,0x387C,INVALC,INVALC,INVALC,INVALC,0x3836,
	0x524E,INVALC,INVALC,INVALC,INVALC,0x5250,0x524F,INVALC,
	0x3F5F,0x3139,INVALC,INVALC,INVALC,0x315E,0x5251,INVALC,
	0x5252,INVALC,INVALC,0x3837,INVALC,INVALC,0x5253,INVALC,
	INVALC,INVALC,INVALC,0x356E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3B32,0x5254,INVALC,INVALC,INVALC,INVALC,
	0x4B74,0x3A35,0x355A,0x4D27,0x4150,0x483F,0x3C7D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D47,INVALC,0x3C68,0x3C75,
	INVALC,0x3D76,INVALC,0x4840,INVALC,INVALC,INVALC,0x5257,
	INVALC,0x3143,0x4151,0x387D,0x3845,0x3667,INVALC,INVALC,
	0x525B,0x4321,0x427E,0x362B,0x3E24,0x525C,0x525A,0x3244,
	0x4266,0x3C38,0x3B4B,0x3126,INVALC,INVALC,0x3370,0x3966,
	0x3B4A,INVALC,0x525D,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 85, Array index 0x1100 */
	INVALC,0x525E,INVALC,0x3549,0x3346,INVALC,INVALC,INVALC,
	0x3967,0x3548,0x445F,0x3125,0x4631,0x4C3E,0x3921,0x4D79,
	0x4547,0x387E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x372F,INVALC,0x5267,INVALC,0x3663,
	0x4B4A,INVALC,INVALC,INVALC,INVALC,INVALC,0x485D,INVALC,
	INVALC,0x5266,INVALC,0x345E,0x5261,0x5262,0x5264,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5265,INVALC,
	0x355B,0x3F61,INVALC,0x4A2D,0x5263,0x525F,0x3863,INVALC,
	0x5260,INVALC,0x4F24,INVALC,INVALC,INVALC,0x4A72,INVALC,
	0x4468,0x3862,0x3970,INVALC,INVALC,INVALC,0x5268,INVALC,
	INVALC,0x465D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x526C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C7E,INVALC,0x3C76,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x526F,0x526D,INVALC,0x4C23,INVALC,0x526A,0x5273,0x526E,
	INVALC,INVALC,INVALC,0x5271,0x3846,0x4C3F,INVALC,INVALC,
	0x5272,INVALC,INVALC,INVALC,0x5274,INVALC,0x5276,INVALC,
	INVALC,INVALC,INVALC,0x3A70,0x4F42,INVALC,0x526B,0x5269,
	0x5275,INVALC,0x5270,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5278,INVALC,0x5323,0x527A,INVALC,INVALC,
	0x527E,INVALC,INVALC,0x5321,0x527B,INVALC,INVALC,0x533E,
	INVALC,INVALC,0x3A69,0x3331,INVALC,INVALC,INVALC,INVALC,
	0x5279,INVALC,INVALC,INVALC,0x5325,0x3076,0x5324,INVALC,
	0x3025,0x494A,0x5322,INVALC,0x527C,INVALC,INVALC,0x5277,
	0x527D,0x3A48,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5326,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3077,0x532F,INVALC,INVALC,0x5327,0x5328,INVALC,
	0x3E25,0x4B69,INVALC,INVALC,INVALC,0x532D,0x532C,INVALC,
	INVALC,INVALC,0x452F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x532E,INVALC,INVALC,0x532B,INVALC,INVALC,
	/* Block 86, Array index 0x1200 */
	INVALC,INVALC,INVALC,INVALC,0x3134,INVALC,0x3A36,0x3F30,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5329,
	0x4562,INVALC,INVALC,INVALC,0x532A,INVALC,0x3022,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5334,0x4D23,
	INVALC,0x3E27,INVALC,0x533A,INVALC,INVALC,INVALC,INVALC,
	0x5339,0x5330,INVALC,INVALC,INVALC,INVALC,0x4243,INVALC,
	0x5331,INVALC,INVALC,INVALC,0x426F,0x5336,0x3E26,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5333,INVALC,INVALC,0x4C64,
	INVALC,INVALC,INVALC,0x373C,INVALC,INVALC,0x5337,0x5338,
	INVALC,INVALC,INVALC,INVALC,0x5335,0x533B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5332,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5341,0x5346,INVALC,0x5342,INVALC,
	0x533D,INVALC,INVALC,0x5347,0x4131,INVALC,INVALC,0x5349,
	INVALC,0x3922,0x533F,0x437D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5343,0x533C,0x342D,INVALC,0x346E,0x3365,0x5344,0x5340,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3776,
	0x534A,0x5348,0x4153,0x354A,0x362C,INVALC,0x5345,INVALC,
	0x3674,INVALC,INVALC,INVALC,INVALC,INVALC,0x3144,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x534E,0x534C,INVALC,0x5427,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5351,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x534B,INVALC,0x534F,INVALC,INVALC,0x534D,
	INVALC,INVALC,INVALC,0x3B4C,0x5350,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5353,
	INVALC,0x5358,INVALC,INVALC,INVALC,0x5356,0x5355,INVALC,
	/* Block 87, Array index 0x1300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4332,INVALC,
	INVALC,0x3245,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5352,INVALC,0x5354,0x3E28,
	0x3133,INVALC,INVALC,0x5357,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x325E,INVALC,INVALC,INVALC,INVALC,INVALC,0x5362,
	INVALC,0x3E7C,0x535E,INVALC,0x535C,INVALC,0x535D,INVALC,
	0x535F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x313D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4139,INVALC,0x5359,INVALC,
	0x535A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x337A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5361,INVALC,INVALC,INVALC,
	0x346F,INVALC,0x5364,0x5360,0x5363,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4A2E,INVALC,INVALC,INVALC,
	0x4655,INVALC,0x4838,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5366,INVALC,INVALC,INVALC,INVALC,INVALC,0x5365,0x3345,
	INVALC,INVALC,0x5367,INVALC,INVALC,INVALC,INVALC,0x536A,
	INVALC,INVALC,INVALC,INVALC,0x5369,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5368,INVALC,0x4739,INVALC,INVALC,0x536B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x536C,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x536E,INVALC,0x536D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5370,INVALC,INVALC,INVALC,
	0x5373,0x5371,0x536F,0x5372,INVALC,INVALC,INVALC,INVALC,
	0x5374,INVALC,INVALC,INVALC,INVALC,INVALC,0x5375,INVALC,
	INVALC,0x5376,INVALC,0x5377,INVALC,INVALC,INVALC,0x5378,
	0x5145,INVALC,0x3C7C,0x3B4D,INVALC,INVALC,0x3273,INVALC,
	0x3078,INVALC,INVALC,0x4344,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5379,INVALC,
	0x3A24,INVALC,0x304F,0x3F5E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x537A,0x3847,INVALC,INVALC,0x3971,INVALC,0x537C,
	/* Block 88, Array index 0x1400 */
	0x537B,INVALC,INVALC,0x4A60,0x537D,INVALC,INVALC,INVALC,
	0x5421,0x537E,INVALC,0x5422,INVALC,0x5423,INVALC,0x3777,
	INVALC,INVALC,0x3160,0x5424,INVALC,INVALC,0x5426,INVALC,
	0x5425,INVALC,INVALC,INVALC,0x5428,INVALC,INVALC,0x455A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5429,0x3035,
	0x3A5F,INVALC,INVALC,INVALC,INVALC,0x373D,INVALC,INVALC,
	0x434F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x542A,
	0x542B,INVALC,INVALC,0x542D,INVALC,INVALC,INVALC,INVALC,
	0x542E,INVALC,0x3A64,INVALC,INVALC,INVALC,INVALC,0x3651,
	INVALC,INVALC,0x4B37,INVALC,INVALC,INVALC,0x542C,0x542F,
	0x3A41,0x3923,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5433,INVALC,INVALC,0x3A25,INVALC,0x4333,INVALC,
	INVALC,0x5430,0x445A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5434,
	INVALC,INVALC,0x3F62,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5432,0x5435,INVALC,0x373F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5436,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5437,INVALC,0x3924,0x3340,0x5439,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x543A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x543B,INVALC,INVALC,0x5438,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5431,INVALC,INVALC,0x543C,INVALC,INVALC,0x543D,INVALC,
	INVALC,INVALC,INVALC,0x4B64,INVALC,INVALC,0x3E6B,INVALC,
	INVALC,INVALC,0x543F,0x5440,0x543E,INVALC,0x5442,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4738,INVALC,INVALC,0x3068,
	0x4956,INVALC,INVALC,0x5443,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E7D,INVALC,INVALC,0x3C39,
	INVALC,0x475D,0x3470,INVALC,0x3A6B,INVALC,INVALC,INVALC,
	/* Block 89, Array index 0x1500 */
	0x4B59,INVALC,0x4632,INVALC,INVALC,0x3778,0x424F,INVALC,
	INVALC,INVALC,0x5441,0x5444,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4244,INVALC,INVALC,
	INVALC,0x5445,INVALC,INVALC,INVALC,0x5446,INVALC,INVALC,
	INVALC,0x5448,INVALC,INVALC,0x4469,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x342E,INVALC,INVALC,INVALC,INVALC,0x7421,
	0x3161,0x4A73,INVALC,INVALC,0x3E6C,0x4548,INVALC,INVALC,
	INVALC,INVALC,0x3A66,INVALC,INVALC,0x544E,INVALC,INVALC,
	0x4A3D,0x4E5D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3274,0x544A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x413A,0x544D,INVALC,0x4563,INVALC,INVALC,0x4549,
	0x4564,0x4839,0x444D,INVALC,INVALC,INVALC,0x3A49,INVALC,
	INVALC,INVALC,0x5449,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3176,INVALC,0x4536,INVALC,INVALC,INVALC,INVALC,
	0x544B,INVALC,0x5447,INVALC,INVALC,0x3F50,INVALC,INVALC,
	INVALC,0x544F,INVALC,INVALC,INVALC,INVALC,0x3D4E,INVALC,
	INVALC,INVALC,INVALC,0x362D,INVALC,0x5450,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A68,INVALC,INVALC,INVALC,0x417D,
	INVALC,INVALC,INVALC,INVALC,0x4446,INVALC,INVALC,0x5452,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4B4F,INVALC,INVALC,0x5453,INVALC,INVALC,0x5458,INVALC,
	INVALC,INVALC,INVALC,0x4A2F,INVALC,INVALC,INVALC,INVALC,
	0x5457,0x5451,0x5454,0x5456,INVALC,INVALC,0x3A26,INVALC,
	INVALC,0x4A49,INVALC,INVALC,INVALC,0x5459,INVALC,0x4345,
	INVALC,INVALC,0x3275,INVALC,0x3E6D,INVALC,INVALC,INVALC,
	INVALC,0x545B,INVALC,0x545A,INVALC,0x3968,INVALC,0x545C,
	0x545E,0x545D,INVALC,INVALC,0x5460,INVALC,0x5455,0x5462,
	INVALC,INVALC,INVALC,INVALC,0x5461,0x545F,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3B4E,0x3F51,INVALC,0x4154,0x5463,
	0x403C,0x306D,0x4764,INVALC,INVALC,INVALC,INVALC,0x445B,
	INVALC,0x5465,0x5464,0x5466,0x5467,0x5468,INVALC,INVALC,
	/* Block 90, Array index 0x1600 */
	INVALC,INVALC,0x5469,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A51,0x546A,INVALC,INVALC,INVALC,INVALC,0x3246,
	0x546B,INVALC,INVALC,INVALC,INVALC,0x4D3C,0x3330,INVALC,
	0x5249,0x3D48,0x423F,0x546C,0x4C6B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C34,INVALC,INVALC,0x546E,INVALC,0x4267,
	INVALC,0x4537,0x4240,0x4957,0x546F,0x5470,0x317B,INVALC,
	INVALC,0x3C3A,0x5471,INVALC,INVALC,INVALC,INVALC,0x3050,
	0x5472,INVALC,INVALC,INVALC,INVALC,INVALC,0x5473,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3162,INVALC,INVALC,0x3471,
	0x4660,0x4A74,INVALC,INVALC,INVALC,INVALC,0x5477,0x4155,
	0x5476,0x3740,INVALC,INVALC,0x4B5B,0x5475,INVALC,0x4565,
	0x5479,INVALC,0x5478,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x547B,INVALC,0x547A,INVALC,INVALC,0x317C,INVALC,0x547C,
	0x3E29,0x547E,0x4325,INVALC,0x547D,INVALC,0x4A33,INVALC,
	INVALC,INVALC,INVALC,0x3D77,0x455B,INVALC,INVALC,INVALC,
	0x5521,INVALC,INVALC,INVALC,INVALC,0x3925,INVALC,INVALC,
	INVALC,0x5522,0x4721,0x485E,0x4C51,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4725,INVALC,INVALC,0x552B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3538,INVALC,INVALC,0x4D45,INVALC,
	INVALC,0x4C2F,INVALC,0x562C,INVALC,0x5523,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5526,INVALC,0x4245,INVALC,INVALC,
	0x4B38,INVALC,INVALC,INVALC,0x454A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5527,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4B65,INVALC,0x3A4A,INVALC,INVALC,0x3E2A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5528,INVALC,
	INVALC,0x3B50,INVALC,0x3B4F,INVALC,INVALC,INVALC,INVALC,
	0x3039,0x3848,INVALC,0x402B,0x3051,INVALC,INVALC,INVALC,
	INVALC,0x552C,0x552D,INVALC,0x552A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3138,0x342F,INVALC,
	0x5529,INVALC,0x4C45,0x4931,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3028,INVALC,
	INVALC,INVALC,INVALC,0x3079,INVALC,INVALC,INVALC,0x3B51,
	/* Block 91, Array index 0x1700 */
	INVALC,0x3052,INVALC,0x3023,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5532,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5530,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C3C,INVALC,0x5533,INVALC,0x5531,INVALC,INVALC,0x552F,
	0x3F31,INVALC,INVALC,INVALC,INVALC,0x552E,INVALC,INVALC,
	INVALC,0x4A5A,INVALC,INVALC,INVALC,INVALC,INVALC,0x3864,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5537,0x5538,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E2B,INVALC,INVALC,INVALC,
	0x5534,0x4F2C,INVALC,INVALC,INVALC,INVALC,0x474C,INVALC,
	INVALC,0x5536,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3A27,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5539,INVALC,INVALC,INVALC,0x4958,INVALC,
	INVALC,INVALC,0x553A,INVALC,0x5535,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C3B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x475E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x553B,0x4932,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x553C,0x5540,0x553D,INVALC,
	INVALC,0x3247,0x553F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3C3B,INVALC,0x553E,0x3779,INVALC,INVALC,INVALC,
	0x554C,INVALC,INVALC,INVALC,INVALC,INVALC,0x5545,0x5542,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4364,INVALC,0x5541,INVALC,INVALC,0x5543,INVALC,
	INVALC,0x5544,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5546,0x5547,INVALC,INVALC,INVALC,INVALC,
	/* Block 92, Array index 0x1800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3472,INVALC,0x5549,0x5548,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x554A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3E6E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x554D,INVALC,0x445C,INVALC,INVALC,INVALC,
	0x3145,INVALC,0x554B,INVALC,INVALC,INVALC,0x554E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x554F,INVALC,
	0x5552,INVALC,INVALC,0x5550,INVALC,0x5551,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B52,0x5553,INVALC,INVALC,0x3926,0x5554,INVALC,0x3B7A,
	0x4238,INVALC,0x5555,0x5556,0x3B5A,0x3927,INVALC,0x4C52,
	INVALC,INVALC,INVALC,0x3528,0x3849,0x5557,0x3358,INVALC,
	INVALC,0x5558,INVALC,0x4239,INVALC,INVALC,INVALC,INVALC,
	0x5559,0x5623,INVALC,0x555A,INVALC,0x555B,INVALC,INVALC,
	0x555C,INVALC,0x555E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x555F,INVALC,INVALC,0x5560,INVALC,0x4270,INVALC,0x3127,
	0x3C69,0x3042,INVALC,0x4157,0x3430,0x3C35,INVALC,0x3928,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4566,INVALC,0x3D21,
	0x3431,0x4368,0x446A,0x3038,0x3539,0x4A75,INVALC,0x3C42,
	INVALC,INVALC,0x3552,0x406B,0x3C3C,0x4D28,0x5561,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x355C,INVALC,
	0x3A4B,INVALC,INVALC,0x3332,0x3163,0x3E2C,0x3248,INVALC,
	0x5562,0x4D46,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D49,
	INVALC,INVALC,0x3C64,0x5563,0x3473,0x4652,0x4C29,0x5564,
	INVALC,0x5565,INVALC,INVALC,0x4959,INVALC,INVALC,INVALC,
	0x5567,INVALC,0x3428,0x3677,0x5566,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3432,INVALC,0x3F32,0x556B,0x3B21,
	INVALC,0x3249,0x556A,INVALC,0x5568,0x556C,0x5569,0x472B,
	0x5C4D,0x3F33,INVALC,0x556D,INVALC,INVALC,0x4E40,INVALC,
	0x556E,INVALC,INVALC,0x5570,INVALC,0x437E,0x556F,INVALC,
	0x4023,INVALC,0x3B7B,INVALC,INVALC,INVALC,0x4250,0x3C77,
	/* Block 93, Array index 0x1900 */
	INVALC,0x4975,0x406C,INVALC,0x3C4D,0x5571,0x3E2D,0x5572,
	0x5573,0x3053,0x423A,0x3F52,INVALC,0x5574,0x4633,0x3E2E,
	INVALC,0x3E2F,INVALC,0x5575,INVALC,INVALC,0x406D,INVALC,
	INVALC,INVALC,0x3E30,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5576,INVALC,0x5577,INVALC,0x4C60,INVALC,INVALC,INVALC,
	0x5578,INVALC,INVALC,INVALC,INVALC,0x3646,INVALC,INVALC,
	INVALC,0x3D22,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5579,0x557A,0x3C5C,0x3F2C,0x4674,0x3F54,0x4878,0x4722,
	0x3649,0x557B,INVALC,INVALC,INVALC,0x356F,0x557C,INVALC,
	0x367E,INVALC,0x464F,0x3230,INVALC,0x3B53,0x557D,0x5622,
	0x5621,0x367D,INVALC,0x557E,INVALC,0x4538,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4230,INVALC,
	0x454B,0x3C48,INVALC,INVALC,0x4158,0x4D7A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5624,INVALC,0x5625,0x4656,
	INVALC,0x3B33,INVALC,INVALC,INVALC,INVALC,0x5627,INVALC,
	INVALC,0x5628,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5629,INVALC,INVALC,INVALC,
	0x3474,0x562A,INVALC,INVALC,0x562B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x322C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x413B,0x3464,INVALC,0x562D,0x4C28,INVALC,INVALC,INVALC,
	INVALC,0x4252,INVALC,0x3359,INVALC,INVALC,0x562F,0x5631,
	0x345F,INVALC,INVALC,0x562E,0x5630,INVALC,0x5633,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5632,INVALC,0x5634,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5635,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x463D,0x362E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3265,0x5636,0x563B,INVALC,INVALC,0x5639,INVALC,0x4A77,
	0x4A76,INVALC,INVALC,INVALC,INVALC,INVALC,0x4567,INVALC,
	INVALC,INVALC,0x5638,0x3D54,INVALC,0x5637,INVALC,INVALC,
	/* Block 94, Array index 0x1A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3F72,
	INVALC,INVALC,INVALC,0x563C,INVALC,INVALC,0x3A6A,INVALC,
	INVALC,0x5642,INVALC,INVALC,0x5643,0x563D,0x3333,0x563E,
	0x5647,0x5646,0x5645,0x5641,INVALC,INVALC,INVALC,0x5640,
	INVALC,INVALC,0x5644,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A78,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x564B,0x5648,INVALC,0x564A,INVALC,
	0x4D72,INVALC,0x5649,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x563F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3F73,INVALC,INVALC,0x564C,INVALC,INVALC,0x3A37,
	INVALC,INVALC,INVALC,0x564D,INVALC,INVALC,0x564E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5651,INVALC,0x5650,INVALC,INVALC,0x564F,
	INVALC,INVALC,INVALC,0x4568,0x563A,INVALC,INVALC,INVALC,
	0x5657,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5653,INVALC,INVALC,
	INVALC,INVALC,0x5652,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5654,INVALC,0x5655,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5658,
	INVALC,INVALC,0x4E66,INVALC,0x5659,0x5656,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x565A,INVALC,INVALC,0x3460,0x565B,INVALC,INVALC,
	INVALC,INVALC,0x565D,0x565C,INVALC,INVALC,0x565E,INVALC,
	INVALC,INVALC,INVALC,0x565F,INVALC,0x406E,0x3D23,INVALC,
	INVALC,0x3D64,INVALC,0x4163,INVALC,0x3929,0x3A38,0x392A,
	0x3570,INVALC,INVALC,0x5660,INVALC,INVALC,0x3A39,INVALC,
	INVALC,0x384A,0x5661,0x4C26,0x4743,0x5662,INVALC,0x392B,
	INVALC,INVALC,INVALC,0x342C,INVALC,0x4327,0x3652,INVALC,
	/* Block 95, Array index 0x1B00 */
	INVALC,INVALC,0x3B54,0x495B,INVALC,INVALC,0x4841,INVALC,
	INVALC,INVALC,INVALC,0x5663,0x3475,INVALC,INVALC,INVALC,
	INVALC,0x5666,INVALC,INVALC,INVALC,INVALC,0x4421,INVALC,
	INVALC,0x5665,0x5664,0x5667,INVALC,0x446B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F63,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3B55,INVALC,0x404A,INVALC,0x4253,
	0x3522,INVALC,INVALC,0x4422,INVALC,INVALC,0x5668,0x5669,
	0x3E6F,INVALC,INVALC,INVALC,INVALC,0x4B39,INVALC,INVALC,
	0x566C,INVALC,INVALC,0x566B,0x566A,0x497D,INVALC,0x5673,
	INVALC,INVALC,INVALC,INVALC,0x4B5A,INVALC,0x566D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x566F,0x4B6B,INVALC,0x566E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5670,
	INVALC,0x4828,0x5671,0x4A3E,0x5672,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3433,0x4A3F,0x472F,0x5674,0x5675,INVALC,
	0x392C,0x3434,0x5676,0x3838,0x4D44,0x4D29,0x3476,0x5678,
	INVALC,0x4423,INVALC,0x392D,0x3E31,INVALC,INVALC,0x485F,
	INVALC,INVALC,0x3E32,INVALC,INVALC,INVALC,INVALC,0x3D78,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x446C,0x4A79,0x4539,
	INVALC,INVALC,0x392E,INVALC,0x495C,INVALC,INVALC,INVALC,
	0x5679,INVALC,INVALC,INVALC,INVALC,INVALC,0x4559,0x3A42,
	INVALC,INVALC,INVALC,0x384B,INVALC,0x446D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3043,0x3D6E,0x392F,
	0x4D47,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x567A,0x567B,0x4751,INVALC,INVALC,INVALC,INVALC,
	0x567C,0x4E77,0x4F2D,INVALC,INVALC,INVALC,INVALC,0x567E,
	0x567D,INVALC,INVALC,0x3347,INVALC,INVALC,0x5721,INVALC,
	INVALC,INVALC,0x5724,0x5725,INVALC,0x5723,INVALC,0x4940,
	0x3E33,0x5727,0x5726,0x5722,INVALC,INVALC,INVALC,INVALC,
	0x5728,0x5729,INVALC,INVALC,0x572A,INVALC,INVALC,INVALC,
	0x572D,0x572B,INVALC,0x572C,0x572E,INVALC,0x3164,0x446E,
	0x572F,INVALC,0x377A,0x3276,0x4736,INVALC,0x5730,0x467B,
	/* Block 96, Array index 0x1C00 */
	INVALC,0x4A5B,INVALC,0x5731,0x4F2E,INVALC,INVALC,INVALC,
	INVALC,0x5732,0x4A40,0x5735,0x5021,0x5031,INVALC,0x3C30,
	0x4675,0x5736,INVALC,0x355D,0x4424,0x307A,0x5737,0x4A26,
	0x3930,INVALC,INVALC,0x4350,INVALC,INVALC,INVALC,0x446F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4C6F,0x3839,0x384C,
	INVALC,0x5738,INVALC,INVALC,INVALC,0x5739,INVALC,0x573F,
	INVALC,0x3C65,INVALC,INVALC,INVALC,0x4425,INVALC,0x362F,
	0x573A,INVALC,INVALC,INVALC,0x492B,INVALC,0x4346,INVALC,
	INVALC,0x573B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x573C,INVALC,0x3630,INVALC,0x573D,INVALC,0x573E,INVALC,
	INVALC,0x5740,INVALC,0x4576,INVALC,INVALC,0x5741,0x5742,
	INVALC,0x5743,INVALC,INVALC,0x5734,0x5733,INVALC,INVALC,
	INVALC,0x5744,0x3741,INVALC,INVALC,INVALC,0x4927,INVALC,
	INVALC,0x3A4C,0x4937,0x4426,0x494B,0x5745,INVALC,INVALC,
	0x3E34,0x3146,INVALC,0x5746,INVALC,INVALC,INVALC,0x5747,
	INVALC,0x4C72,INVALC,INVALC,0x4860,INVALC,INVALC,0x574A,
	0x317D,0x402C,0x5749,0x5748,0x3742,0x4254,INVALC,0x574E,
	0x574C,INVALC,0x574B,0x4E27,0x3865,INVALC,INVALC,INVALC,
	0x3D79,0x574D,0x454C,0x3D3E,INVALC,INVALC,INVALC,0x4640,
	0x5751,0x5750,INVALC,INVALC,INVALC,INVALC,0x574F,INVALC,
	0x5752,0x3866,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5753,0x497C,0x3D5B,INVALC,INVALC,0x5754,0x4879,INVALC,
	INVALC,INVALC,INVALC,0x4641,0x4427,INVALC,INVALC,INVALC,
	INVALC,0x4530,INVALC,INVALC,0x5755,0x352B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3F34,INVALC,0x492C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3477,0x4726,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5756,0x3B56,
	0x4B3A,0x4B3B,INVALC,INVALC,0x317E,0x575B,INVALC,INVALC,
	0x4369,INVALC,INVALC,INVALC,0x5758,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3277,INVALC,INVALC,INVALC,INVALC,
	0x582D,0x575A,INVALC,INVALC,INVALC,0x4730,INVALC,INVALC,
	0x5759,INVALC,INVALC,0x5757,INVALC,0x397A,INVALC,0x575D,
	/* Block 97, Array index 0x1D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5763,0x5769,
	0x5761,INVALC,0x455C,INVALC,INVALC,0x5766,0x495D,INVALC,
	INVALC,0x5760,INVALC,0x5765,0x4E67,0x3B57,INVALC,INVALC,
	0x4255,0x575E,INVALC,INVALC,INVALC,0x355E,0x5768,0x402D,
	0x3165,0x5762,0x3278,0x5767,INVALC,INVALC,INVALC,0x3631,
	INVALC,0x5764,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x576A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x576C,0x5776,0x5774,INVALC,INVALC,0x5771,INVALC,
	INVALC,INVALC,0x5770,0x4E78,INVALC,0x5772,INVALC,INVALC,
	0x3632,INVALC,0x3931,INVALC,INVALC,0x3D7A,INVALC,INVALC,
	INVALC,0x5779,0x576B,INVALC,INVALC,INVALC,INVALC,0x576F,
	0x575F,INVALC,0x327A,0x5773,0x5775,0x4351,INVALC,INVALC,
	0x3A28,0x3238,0x576D,0x5778,0x5777,0x3633,INVALC,0x4229,
	0x3366,INVALC,INVALC,INVALC,INVALC,0x3743,INVALC,0x576E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x577A,INVALC,0x577D,0x5821,INVALC,INVALC,INVALC,
	INVALC,0x3C3D,INVALC,0x5827,0x4470,0x577B,INVALC,INVALC,
	INVALC,INVALC,0x5825,INVALC,0x3279,INVALC,0x5823,0x5824,
	INVALC,INVALC,0x577E,0x5822,INVALC,INVALC,INVALC,0x3867,
	0x4D2A,INVALC,INVALC,0x3435,INVALC,INVALC,0x3159,0x5826,
	INVALC,0x473A,0x302D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4861,0x575C,0x582C,0x5830,0x4C65,INVALC,
	0x5829,INVALC,INVALC,INVALC,0x4569,0x582E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3E70,0x582F,0x4657,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F47,INVALC,0x582B,INVALC,INVALC,INVALC,INVALC,
	0x5831,INVALC,0x397B,INVALC,0x404B,INVALC,INVALC,0x3054,
	0x582A,0x5828,INVALC,0x415A,INVALC,INVALC,INVALC,0x577C,
	0x3B34,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4246,0x583D,INVALC,0x415B,0x5838,INVALC,0x5835,0x5836,
	INVALC,0x3C66,0x5839,0x583C,INVALC,INVALC,INVALC,INVALC,
	/* Block 98, Array index 0x1E00 */
	0x5837,0x3D25,INVALC,0x583A,INVALC,INVALC,0x5834,INVALC,
	0x4C7C,0x4C7B,INVALC,INVALC,INVALC,0x583E,0x583F,0x3055,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5833,INVALC,INVALC,
	INVALC,INVALC,0x3672,0x3026,INVALC,INVALC,INVALC,0x3436,
	INVALC,0x583B,INVALC,INVALC,INVALC,INVALC,INVALC,0x5843,
	0x5842,INVALC,INVALC,INVALC,0x5847,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5848,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5846,0x5849,0x5841,0x5845,
	INVALC,INVALC,0x584A,INVALC,0x584B,INVALC,INVALC,0x5840,
	0x3B7C,INVALC,0x5844,0x4256,0x3932,0x5832,0x3F35,INVALC,
	INVALC,INVALC,INVALC,0x5858,INVALC,0x4A69,INVALC,INVALC,
	0x584E,0x584F,0x5850,INVALC,INVALC,0x5857,INVALC,0x5856,
	INVALC,INVALC,0x4B7D,0x3437,INVALC,0x5854,INVALC,0x3745,
	0x3334,INVALC,INVALC,0x5851,INVALC,INVALC,0x4E38,0x5853,
	0x3056,0x5855,INVALC,0x584C,0x5852,0x5859,0x3744,0x584D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D5D,INVALC,
	INVALC,INVALC,0x4D2B,INVALC,INVALC,INVALC,INVALC,0x585C,
	INVALC,INVALC,0x5860,INVALC,INVALC,INVALC,0x417E,INVALC,
	0x4E79,0x5861,INVALC,INVALC,0x585E,INVALC,0x585B,INVALC,
	INVALC,0x585A,0x585F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4A30,INVALC,INVALC,0x4634,
	INVALC,0x3746,INVALC,0x5862,0x585D,INVALC,0x5863,INVALC,
	INVALC,INVALC,0x377B,INVALC,INVALC,INVALC,0x3231,INVALC,
	INVALC,INVALC,0x586B,INVALC,INVALC,INVALC,0x3438,INVALC,
	INVALC,INVALC,INVALC,0x5869,INVALC,INVALC,0x586A,0x3A29,
	0x5868,0x5866,0x5865,0x586C,0x5864,0x586E,INVALC,INVALC,
	0x327B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5870,INVALC,INVALC,0x586F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4428,INVALC,0x5873,INVALC,0x5871,0x5867,
	0x377C,INVALC,0x5872,INVALC,0x5876,0x5875,0x5877,0x5874,
	/* Block 99, Array index 0x1F00 */
	0x5878,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5879,0x587A,0x4A6A,INVALC,0x587C,0x587B,0x3D3F,INVALC,
	0x402E,0x3266,0x327C,INVALC,0x587D,INVALC,0x303F,INVALC,
	INVALC,INVALC,0x404C,0x587E,INVALC,0x6C43,0x5921,0x3761,
	INVALC,0x5922,INVALC,INVALC,INVALC,INVALC,0x406F,INVALC,
	INVALC,INVALC,0x5923,INVALC,INVALC,INVALC,0x5924,0x353A,
	0x5925,INVALC,0x5926,0x5927,0x4257,INVALC,INVALC,INVALC,
	0x384D,INVALC,INVALC,0x4C61,INVALC,INVALC,INVALC,0x4B3C,
	0x3D6A,0x5928,INVALC,INVALC,INVALC,INVALC,INVALC,0x4070,
	0x6E3D,0x4862,INVALC,0x3C6A,INVALC,0x3A4D,0x5929,INVALC,
	INVALC,INVALC,INVALC,0x4247,INVALC,0x4A27,INVALC,INVALC,
	0x4271,INVALC,INVALC,0x592C,INVALC,INVALC,0x592A,INVALC,
	0x592D,INVALC,INVALC,0x592B,INVALC,INVALC,INVALC,INVALC,
	0x592E,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A31,INVALC,
	INVALC,0x3037,INVALC,INVALC,INVALC,INVALC,0x495E,INVALC,
	INVALC,0x4863,INVALC,INVALC,0x592F,INVALC,0x5932,0x3E35,
	0x353B,INVALC,0x5930,0x5937,0x3E36,INVALC,INVALC,INVALC,
	INVALC,0x5931,0x4744,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D5E,0x5933,0x5934,0x5938,0x456A,0x5935,0x3933,
	0x405E,INVALC,INVALC,0x5946,0x4834,INVALC,0x4272,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4864,0x5A2D,INVALC,INVALC,INVALC,
	INVALC,0x4A7A,INVALC,INVALC,INVALC,0x4471,INVALC,INVALC,
	INVALC,0x4B75,INVALC,0x593B,0x3221,0x436A,INVALC,INVALC,
	INVALC,INVALC,0x5944,INVALC,INVALC,0x4334,0x593E,0x5945,
	0x5940,0x5947,0x5943,INVALC,0x5942,0x476F,INVALC,0x593C,
	0x327D,0x593A,0x3571,0x4273,0x5936,INVALC,INVALC,0x5939,
	0x3934,0x405B,INVALC,0x3E37,0x5941,0x4752,INVALC,INVALC,
	0x3572,0x3348,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3367,0x3F21,0x5949,0x594E,
	INVALC,0x594A,INVALC,0x377D,INVALC,0x594F,0x3B22,0x3969,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D26,0x593D,
	/* Block 100, Array index 0x2000 */
	INVALC,0x3B7D,0x594C,INVALC,INVALC,INVALC,INVALC,0x3B58,
	0x594D,0x3044,INVALC,INVALC,0x5948,INVALC,INVALC,INVALC,
	INVALC,0x4429,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3573,INVALC,INVALC,INVALC,INVALC,INVALC,0x3634,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x594B,
	0x3027,INVALC,INVALC,0x3A43,INVALC,INVALC,INVALC,0x3F36,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4472,INVALC,INVALC,0x4854,0x5951,0x415E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x422A,INVALC,INVALC,0x3B2B,0x5952,INVALC,0x5954,
	0x5950,INVALC,INVALC,INVALC,INVALC,0x4A61,INVALC,0x443D,
	INVALC,INVALC,INVALC,INVALC,0x415C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A7B,
	0x3C4E,0x5960,INVALC,0x595F,INVALC,INVALC,0x3F78,INVALC,
	INVALC,INVALC,0x377E,INVALC,INVALC,INVALC,0x5959,0x3E39,
	INVALC,INVALC,0x4668,0x4731,INVALC,INVALC,INVALC,INVALC,
	0x5957,INVALC,INVALC,0x415D,INVALC,INVALC,INVALC,INVALC,
	0x3C78,0x595C,INVALC,INVALC,0x3E38,INVALC,0x5956,0x595B,
	INVALC,INVALC,0x4753,INVALC,INVALC,INVALC,0x5955,INVALC,
	0x3721,INVALC,INVALC,0x335D,INVALC,INVALC,INVALC,0x595D,
	0x4E2B,0x3A4E,0x4335,0x595A,INVALC,0x405C,INVALC,0x3935,
	0x3F64,0x3166,0x413C,0x5958,0x3545,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3747,INVALC,0x444F,0x595E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x415F,INVALC,INVALC,0x5961,INVALC,
	0x5963,INVALC,INVALC,0x4237,0x5969,INVALC,0x5964,INVALC,
	INVALC,0x5966,INVALC,INVALC,INVALC,INVALC,INVALC,0x4941,
	0x4473,INVALC,0x5967,INVALC,INVALC,INVALC,0x4D2C,INVALC,
	INVALC,INVALC,0x4D48,0x3439,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x302E,INVALC,0x5965,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5962,INVALC,INVALC,INVALC,INVALC,0x3478,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3167,INVALC,0x5968,INVALC,
	INVALC,INVALC,0x4D49,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 101, Array index 0x2100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x596C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x423B,INVALC,0x5973,
	INVALC,INVALC,INVALC,0x596D,INVALC,INVALC,0x596A,0x5971,
	INVALC,INVALC,INVALC,INVALC,0x5953,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x596E,INVALC,
	0x5972,INVALC,INVALC,INVALC,0x4842,0x456B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x596B,INVALC,0x596F,INVALC,
	INVALC,INVALC,0x3748,INVALC,INVALC,INVALC,0x3A71,INVALC,
	INVALC,INVALC,0x405D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5977,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4526,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5974,
	INVALC,0x4B60,INVALC,INVALC,INVALC,INVALC,INVALC,0x5975,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5976,INVALC,
	0x4C4E,INVALC,0x4022,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3762,INVALC,INVALC,INVALC,INVALC,
	0x597D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3B35,0x597A,INVALC,0x5979,INVALC,INVALC,
	INVALC,INVALC,0x4732,INVALC,INVALC,INVALC,0x4635,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4531,0x597B,INVALC,INVALC,
	INVALC,0x597C,INVALC,0x496F,INVALC,0x4745,0x3B23,INVALC,
	0x4071,INVALC,0x4B50,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3349,INVALC,0x5A25,0x597E,INVALC,INVALC,INVALC,
	INVALC,0x4D4A,0x5A27,INVALC,INVALC,0x5A23,INVALC,0x5A24,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4160,INVALC,INVALC,
	INVALC,INVALC,0x5A22,INVALC,0x593F,INVALC,INVALC,INVALC,
	0x5A26,INVALC,0x5A21,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A2B,0x5A2C,0x4527,0x5A2E,INVALC,INVALC,0x3B24,0x5A29,
	INVALC,INVALC,INVALC,INVALC,0x353C,INVALC,INVALC,0x5A2F,
	INVALC,0x5A28,0x5A33,INVALC,0x5A32,INVALC,0x5A31,INVALC,
	INVALC,INVALC,0x5A34,INVALC,INVALC,0x5A36,0x3E71,INVALC,
	/* Block 102, Array index 0x2200 */
	0x5A35,INVALC,INVALC,INVALC,INVALC,0x5A39,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A37,INVALC,INVALC,INVALC,0x5A38,0x5970,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5A3B,0x5A3A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5978,0x5A3C,0x5A30,INVALC,INVALC,0x3B59,
	INVALC,INVALC,INVALC,INVALC,0x5A3D,0x5A3E,0x5A40,0x5A3F,
	0x5A41,0x327E,INVALC,0x3936,INVALC,INVALC,0x4A7C,0x402F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x384E,INVALC,INVALC,
	0x5A43,INVALC,INVALC,INVALC,INVALC,0x5A46,INVALC,0x4952,
	INVALC,0x355F,INVALC,INVALC,INVALC,0x5A45,0x5A44,0x4754,
	0x5A47,0x3635,INVALC,INVALC,INVALC,0x5A49,0x5A48,INVALC,
	INVALC,INVALC,0x343A,0x3B36,INVALC,INVALC,0x4658,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3749,INVALC,INVALC,INVALC,
	0x3F74,INVALC,0x5A4A,INVALC,0x4030,0x4528,INVALC,0x495F,
	0x5A4B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A4C,0x5A4D,INVALC,INVALC,INVALC,0x4A38,
	0x555D,0x4046,INVALC,INVALC,0x494C,INVALC,0x3A58,INVALC,
	0x4865,0x4843,INVALC,INVALC,INVALC,INVALC,INVALC,0x454D,
	INVALC,0x4E41,INVALC,0x5A4F,0x3C50,INVALC,INVALC,0x5A50,
	INVALC,0x3036,INVALC,INVALC,0x3654,0x404D,INVALC,0x4960,
	INVALC,INVALC,INVALC,0x5A51,0x3B42,0x4347,INVALC,0x3B5B,
	0x3F37,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A52,
	INVALC,0x4A7D,INVALC,INVALC,0x3177,0x3B5C,INVALC,INVALC,
	INVALC,0x5A55,INVALC,0x5A53,0x5A56,0x4E39,0x5A54,INVALC,
	INVALC,INVALC,INVALC,0x407B,0x5A57,INVALC,INVALC,0x4232,
	INVALC,INVALC,0x5A58,INVALC,INVALC,INVALC,INVALC,0x347A,
	INVALC,0x5A5A,INVALC,0x5A59,INVALC,INVALC,INVALC,INVALC,
	0x5A5B,0x5A5C,0x347B,INVALC,INVALC,0x467C,0x4336,0x356C,
	0x3B5D,0x4161,INVALC,INVALC,0x3D5C,0x3030,INVALC,INVALC,
	INVALC,0x5A5D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3222,0x5A61,INVALC,INVALC,INVALC,INVALC,
	/* Block 103, Array index 0x2300 */
	INVALC,INVALC,0x3937,0x5A60,INVALC,INVALC,0x3A2B,0x3E3A,
	INVALC,INVALC,0x5A5F,INVALC,0x3E3B,INVALC,0x4C40,0x3A2A,
	INVALC,INVALC,INVALC,0x3057,0x404E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A66,INVALC,INVALC,0x4031,
	0x3147,INVALC,INVALC,INVALC,INVALC,0x3D55,INVALC,0x4B66,
	0x3A72,INVALC,INVALC,INVALC,INVALC,0x3E3C,INVALC,0x4027,
	INVALC,INVALC,INVALC,INVALC,0x5A65,0x5A63,0x5A64,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x436B,INVALC,INVALC,0x5B26,
	INVALC,0x5A6A,0x3B7E,0x3938,0x5A68,INVALC,INVALC,INVALC,
	INVALC,0x5A69,INVALC,0x3F38,INVALC,INVALC,INVALC,0x5A67,
	INVALC,INVALC,0x3B2F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5A6C,0x5A6B,0x5A70,
	INVALC,INVALC,0x5A71,INVALC,0x5A6D,INVALC,0x3322,0x5A6E,
	0x5A6F,0x4855,INVALC,INVALC,INVALC,INVALC,0x4961,0x374A,
	0x5A72,INVALC,INVALC,INVALC,0x4032,INVALC,0x3E3D,INVALC,
	INVALC,INVALC,0x4352,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3647,INVALC,0x5A73,0x5A77,INVALC,INVALC,0x324B,
	0x5A74,0x5A76,INVALC,INVALC,INVALC,INVALC,0x5A75,INVALC,
	INVALC,0x3D6B,INVALC,INVALC,INVALC,INVALC,0x4348,0x3045,
	0x5A78,INVALC,INVALC,INVALC,INVALC,0x5A79,INVALC,INVALC,
	INVALC,INVALC,0x442A,INVALC,INVALC,INVALC,0x4E71,INVALC,
	INVALC,INVALC,INVALC,0x3B43,INVALC,INVALC,0x4A6B,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4B3D,INVALC,INVALC,INVALC,
	0x5B22,0x5A7B,INVALC,INVALC,0x5A7E,INVALC,0x5A7D,INVALC,
	INVALC,0x5A7A,INVALC,INVALC,0x5B21,INVALC,INVALC,0x465E,
	INVALC,0x5A7C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B23,INVALC,
	INVALC,0x3D6C,0x5B24,INVALC,0x4D4B,0x4778,INVALC,INVALC,
	0x5B25,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B27,INVALC,
	INVALC,0x5B28,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5B29,INVALC,0x364A,0x3148,0x3939,0x5B2A,INVALC,0x5B2B,
	0x3D71,0x4162,INVALC,INVALC,0x5258,0x413E,0x413D,0x4258,
	/* Block 104, Array index 0x2400 */
	0x3A47,INVALC,INVALC,0x5072,INVALC,INVALC,INVALC,INVALC,
	0x376E,0x4D2D,INVALC,0x4A7E,INVALC,0x497E,INVALC,0x5B2C,
	INVALC,INVALC,INVALC,INVALC,0x3A73,0x443F,0x5B2D,0x4F2F,
	INVALC,INVALC,INVALC,0x4B3E,INVALC,0x442B,0x5B2E,0x347C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B2F,0x5B30,
	0x4C5A,INVALC,0x4C24,0x4B76,0x4B5C,0x3B25,0x5B32,INVALC,
	INVALC,0x3C6B,INVALC,INVALC,0x4B51,INVALC,0x5B34,0x5B37,
	0x5B36,INVALC,0x3479,INVALC,INVALC,0x3560,INVALC,0x5B33,
	INVALC,0x5B35,INVALC,INVALC,INVALC,INVALC,0x5B38,INVALC,
	INVALC,0x3F79,INVALC,INVALC,INVALC,INVALC,0x4D7B,0x3049,
	0x3A60,0x423C,INVALC,0x3C5D,INVALC,INVALC,0x3E73,INVALC,
	INVALC,0x5B3B,INVALC,INVALC,0x454E,INVALC,0x5B39,0x422B,
	0x5B3A,0x3E72,0x4C5D,0x5B3C,0x5B3D,0x4D68,INVALC,INVALC,
	INVALC,INVALC,0x5B42,INVALC,INVALC,0x393A,INVALC,0x4755,
	0x5B3F,0x456C,0x5A5E,0x5A62,INVALC,0x354F,INVALC,0x4747,
	INVALC,INVALC,INVALC,INVALC,0x5B41,INVALC,0x3E3E,0x4844,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5B47,INVALC,0x487A,
	INVALC,0x5B3E,INVALC,0x5B44,0x5B43,INVALC,INVALC,INVALC,
	0x404F,INVALC,INVALC,INVALC,INVALC,0x4B6D,INVALC,0x4E53,
	INVALC,INVALC,0x4B67,INVALC,0x324C,0x3B5E,INVALC,INVALC,
	0x4F48,0x5B46,0x3F75,INVALC,INVALC,INVALC,0x5B45,INVALC,
	INVALC,0x5B40,INVALC,INVALC,INVALC,INVALC,INVALC,0x384F,
	INVALC,INVALC,INVALC,0x5B4C,0x5B4A,INVALC,0x324D,0x5B48,
	0x5B4E,0x5B54,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4248,INVALC,INVALC,0x4A41,INVALC,0x5B56,INVALC,
	INVALC,INVALC,0x4922,INVALC,INVALC,INVALC,0x5B55,0x4770,
	0x4B3F,0x343B,INVALC,0x4077,0x3D40,INVALC,INVALC,INVALC,
	0x4453,INVALC,0x4D2E,INVALC,INVALC,0x5B51,0x5B50,INVALC,
	INVALC,INVALC,0x5B52,INVALC,0x5B4F,INVALC,INVALC,0x5B57,
	INVALC,0x5B4D,INVALC,INVALC,0x5B4B,INVALC,0x5B53,0x5B49,
	INVALC,0x436C,INVALC,0x4C78,0x3C46,0x3A74,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3A3A,INVALC,INVALC,0x4B6F,0x3341,
	/* Block 105, Array index 0x2500 */
	INVALC,INVALC,0x444E,0x464A,0x3149,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4072,INVALC,INVALC,0x4034,0x372A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B59,INVALC,
	INVALC,0x393B,0x337C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5B5B,0x3374,0x5B61,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5B5E,INVALC,0x4073,INVALC,INVALC,INVALC,
	0x334B,0x3A2C,INVALC,INVALC,0x334A,0x3A4F,INVALC,INVALC,
	0x5B5C,0x3765,0x374B,0x456D,INVALC,INVALC,0x5B5A,INVALC,
	0x3046,INVALC,INVALC,INVALC,INVALC,0x5B5D,0x5B5F,INVALC,
	0x364D,0x372C,INVALC,0x343C,0x354B,INVALC,INVALC,INVALC,
	INVALC,0x5B62,INVALC,INVALC,0x3A79,0x4B71,INVALC,0x3B37,
	INVALC,INVALC,INVALC,0x5B63,INVALC,INVALC,INVALC,0x4930,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5B6F,INVALC,0x3233,0x5B64,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B75,0x5B65,
	INVALC,0x4E42,INVALC,0x5B6C,INVALC,0x475F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5B74,INVALC,0x5B67,
	INVALC,INVALC,INVALC,0x3034,0x5B69,INVALC,INVALC,0x393C,
	INVALC,INVALC,INVALC,0x5B6B,INVALC,0x5B6A,INVALC,0x5B66,
	0x5B71,INVALC,0x3E3F,INVALC,INVALC,INVALC,0x546D,0x3868,
	0x4D7C,INVALC,INVALC,INVALC,INVALC,0x5B68,INVALC,0x4474,
	0x3323,0x3A2D,INVALC,0x5B60,INVALC,0x5B70,0x3361,INVALC,
	INVALC,0x5B6E,0x5B72,INVALC,0x456E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x347E,INVALC,0x5C32,INVALC,
	INVALC,0x4C49,0x5B77,0x347D,INVALC,0x5B7E,INVALC,INVALC,
	INVALC,INVALC,0x4B40,INVALC,0x5C21,0x5C23,INVALC,0x5C27,
	0x5B79,INVALC,0x432A,INVALC,INVALC,INVALC,INVALC,0x456F,
	0x5C2B,0x5B7C,INVALC,0x5C28,INVALC,INVALC,INVALC,0x5C22,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3F39,0x5C2C,
	INVALC,INVALC,0x4033,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C2A,0x343D,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 106, Array index 0x2600 */
	0x4F50,0x5B76,INVALC,INVALC,0x5C26,0x3058,INVALC,INVALC,
	0x5B78,INVALC,INVALC,0x4C3A,0x5B7D,0x3F22,0x4447,0x5B73,
	INVALC,INVALC,0x5C25,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3F7A,0x5C2F,0x3371,0x3821,INVALC,INVALC,INVALC,
	INVALC,0x5C31,0x5B7A,0x5C30,INVALC,0x5C29,0x5B7B,INVALC,
	0x5C2D,INVALC,0x5C2E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5C3F,INVALC,INVALC,INVALC,0x464E,INVALC,0x5C24,INVALC,
	INVALC,0x5C3B,INVALC,INVALC,INVALC,0x5C3D,INVALC,0x4458,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D4C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4976,0x5C38,0x424A,INVALC,INVALC,
	INVALC,0x5C3E,0x413F,INVALC,0x5C35,0x5C42,0x5C41,INVALC,
	0x466F,0x5C40,0x466A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C44,0x5C37,INVALC,0x3648,0x5C3A,0x3D5D,
	INVALC,INVALC,INVALC,0x4760,0x5C3C,0x364B,INVALC,0x5C34,
	0x5C36,0x5C33,INVALC,INVALC,0x4F30,0x335A,0x5C39,INVALC,
	INVALC,0x5C43,0x3335,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3A67,INVALC,INVALC,INVALC,0x315D,INVALC,
	INVALC,0x5C54,INVALC,INVALC,0x4F31,0x5C57,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3F3A,0x5C56,INVALC,INVALC,INVALC,
	0x5C55,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C52,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C46,INVALC,
	INVALC,0x5C63,0x5C45,INVALC,0x5C58,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5C50,INVALC,INVALC,0x5C4B,0x5C48,
	INVALC,0x5C49,INVALC,0x5C51,INVALC,INVALC,INVALC,0x7422,
	INVALC,INVALC,0x5C4E,0x393D,0x4448,0x4164,0x5C4C,INVALC,
	0x5C47,INVALC,INVALC,0x5C4A,INVALC,INVALC,INVALC,INVALC,
	0x4D4D,0x4B6A,INVALC,INVALC,INVALC,0x5C4F,0x5C59,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C61,
	0x5C5A,INVALC,INVALC,0x5C67,INVALC,0x5C65,INVALC,INVALC,
	INVALC,INVALC,0x5C60,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C5F,INVALC,0x4450,INVALC,0x4165,INVALC,0x5C5D,
	/* Block 107, Array index 0x2700 */
	INVALC,INVALC,0x5C5B,INVALC,INVALC,0x5C62,INVALC,INVALC,
	INVALC,INVALC,0x5C68,0x4875,0x5C6E,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C69,0x5C6C,0x5C66,INVALC,INVALC,0x4374,
	INVALC,0x4938,INVALC,0x5C5C,INVALC,INVALC,0x5C64,0x3E40,
	INVALC,0x4C4F,0x5C78,0x5C6B,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3822,0x3223,0x335F,INVALC,INVALC,0x5C53,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3E41,0x5C70,INVALC,
	0x5C77,0x3C79,0x3372,INVALC,INVALC,0x432E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5C6D,INVALC,INVALC,0x5C72,
	0x5C76,INVALC,INVALC,0x3636,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x354C,0x5C74,INVALC,INVALC,INVALC,INVALC,INVALC,0x3521,
	INVALC,0x464B,0x5C73,INVALC,INVALC,INVALC,0x5C75,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C6F,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5C71,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3360,
	0x4349,INVALC,INVALC,INVALC,0x5C7C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5C7A,0x3869,INVALC,
	0x5C79,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D21,
	INVALC,INVALC,INVALC,INVALC,0x5B58,INVALC,INVALC,INVALC,
	0x5C7B,INVALC,0x5C7D,0x5C7E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D2C,INVALC,0x5D28,INVALC,0x5B6D,INVALC,
	INVALC,INVALC,INVALC,0x5D27,INVALC,INVALC,INVALC,INVALC,
	0x5D26,INVALC,INVALC,0x5D23,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C6A,0x5D25,0x5D24,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D2A,INVALC,0x4F26,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D2D,0x367B,INVALC,INVALC,0x5D29,0x5D2B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4827,INVALC,0x5D2E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D32,0x5D2F,INVALC,INVALC,INVALC,INVALC,
	/* Block 108, Array index 0x2800 */
	INVALC,INVALC,INVALC,INVALC,0x4D73,0x5D30,INVALC,INVALC,
	INVALC,INVALC,0x5C5E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D33,INVALC,INVALC,INVALC,0x5D34,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3135,INVALC,0x5D36,
	0x3767,0x3C21,INVALC,0x3655,INVALC,INVALC,INVALC,0x3224,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D5F,INVALC,INVALC,INVALC,INVALC,0x5D38,
	0x5D37,0x5D3A,0x353D,INVALC,INVALC,0x3656,0x343E,INVALC,
	INVALC,INVALC,INVALC,0x5D3D,INVALC,INVALC,INVALC,0x5D3C,
	INVALC,0x5D3E,INVALC,INVALC,0x324E,INVALC,0x4337,INVALC,
	0x5D3F,INVALC,INVALC,0x343F,0x5D41,INVALC,INVALC,INVALC,
	INVALC,0x5D40,INVALC,0x5D42,INVALC,INVALC,INVALC,0x5D43,
	INVALC,0x5D44,0x3B5F,0x4035,0x3A21,INVALC,0x4970,INVALC,
	INVALC,0x4A62,0x4F44,INVALC,INVALC,INVALC,INVALC,0x3B75,
	INVALC,INVALC,INVALC,0x3A50,0x4E72,INVALC,INVALC,INVALC,
	0x5D45,0x5D46,INVALC,0x3B60,INVALC,INVALC,INVALC,0x5D47,
	0x5D48,INVALC,INVALC,0x5D4A,0x5D49,INVALC,0x4B58,INVALC,
	INVALC,0x3D5E,0x3C6C,0x3B44,INVALC,0x5D4B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5D4D,0x3F23,INVALC,
	0x5D4C,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D4E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D4F,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D50,0x5D51,INVALC,INVALC,INVALC,0x5D52,
	INVALC,0x5D54,0x5D53,0x5D55,0x3225,0x434A,INVALC,0x5D56,
	INVALC,INVALC,0x3B26,0x334C,0x5D57,INVALC,INVALC,0x4542,
	0x544C,INVALC,INVALC,INVALC,INVALC,0x3523,0x5D58,INVALC,
	INVALC,INVALC,INVALC,0x5D59,INVALC,0x4A6C,0x4B68,INVALC,
	INVALC,INVALC,0x4647,0x5D5A,0x4866,INVALC,INVALC,INVALC,
	0x487B,INVALC,INVALC,0x4C53,INVALC,INVALC,INVALC,0x5D5B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D5D,0x5D5C,INVALC,INVALC,0x5D5F,
	INVALC,INVALC,INVALC,0x5D5E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 109, Array index 0x2900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5D61,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B61,
	INVALC,0x4C31,INVALC,0x5D62,0x5D63,INVALC,INVALC,0x3524,
	INVALC,INVALC,INVALC,0x5D64,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D66,0x5D65,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F65,INVALC,INVALC,0x4939,
	0x314A,INVALC,INVALC,INVALC,INVALC,INVALC,0x4845,INVALC,
	0x4475,0x3D41,0x3561,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4846,INVALC,
	0x3C2E,INVALC,INVALC,INVALC,INVALC,0x5D68,INVALC,0x3440,
	INVALC,INVALC,0x3178,INVALC,INVALC,0x4672,0x5D67,0x393E,
	0x4353,INVALC,0x5D69,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5D71,INVALC,0x5D6A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4241,INVALC,0x3562,0x5D72,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3768,INVALC,INVALC,0x3525,0x5D70,INVALC,
	INVALC,0x5D6E,0x5D6B,0x4D60,INVALC,INVALC,INVALC,INVALC,
	0x4440,INVALC,INVALC,INVALC,0x4659,0x5D6C,INVALC,INVALC,
	0x5D74,INVALC,0x5D73,0x3723,INVALC,INVALC,0x322D,INVALC,
	INVALC,0x3A3B,0x5D6D,0x5D6F,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4B57,0x4274,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4B77,INVALC,INVALC,0x5D7C,INVALC,
	INVALC,0x5D7D,INVALC,0x324F,INVALC,INVALC,INVALC,INVALC,
	0x4A28,0x4C7D,0x5E21,0x3C23,0x3E42,0x5D78,0x5D7E,0x3168,
	INVALC,0x3637,INVALC,INVALC,0x5D75,0x5D7A,INVALC,INVALC,
	INVALC,0x4074,0x4771,INVALC,0x4867,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D77,INVALC,0x4B21,INVALC,0x5D79,
	INVALC,0x5E24,INVALC,0x5E22,INVALC,0x5D7B,INVALC,INVALC,
	INVALC,0x4B22,0x4748,0x3563,INVALC,0x4525,INVALC,INVALC,
	0x436D,INVALC,0x5E25,INVALC,INVALC,INVALC,INVALC,0x5E23,
	0x4259,0x5D76,INVALC,0x314B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 110, Array index 0x2A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4D4E,0x5E30,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5E2F,INVALC,INVALC,INVALC,INVALC,0x4076,
	INVALC,0x5E2C,INVALC,0x4D6C,INVALC,INVALC,0x4636,0x5E26,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4445,INVALC,INVALC,
	INVALC,0x314C,0x393F,0x5E29,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3D27,0x5E2E,INVALC,0x5E2D,0x5E28,INVALC,
	0x5E2B,INVALC,INVALC,0x3368,INVALC,0x5E2A,0x4749,INVALC,
	INVALC,0x4E2E,INVALC,INVALC,0x3E74,0x4075,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5E36,0x5E34,INVALC,0x494D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E31,0x5E33,INVALC,0x313A,INVALC,
	INVALC,0x3940,0x4F32,INVALC,0x333D,INVALC,0x4962,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D61,INVALC,INVALC,0x3324,
	0x3F3B,0x5E35,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E3A,INVALC,INVALC,
	0x3E43,INVALC,INVALC,INVALC,0x4D30,INVALC,0x5E37,INVALC,
	INVALC,INVALC,INVALC,0x5E32,INVALC,0x5E38,INVALC,INVALC,
	INVALC,0x4E5E,INVALC,0x4573,0x4642,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3336,
	INVALC,INVALC,0x3155,INVALC,INVALC,0x5E3E,INVALC,INVALC,
	0x5E41,INVALC,INVALC,INVALC,0x4E43,INVALC,INVALC,INVALC,
	0x4D64,INVALC,INVALC,INVALC,INVALC,0x5E48,0x5E42,0x5E3F,
	INVALC,INVALC,INVALC,0x4E54,0x5E45,INVALC,INVALC,INVALC,
	INVALC,0x3D4A,0x5E47,INVALC,INVALC,0x5E4C,INVALC,INVALC,
	0x4571,0x5E4A,INVALC,INVALC,INVALC,INVALC,0x5E44,INVALC,
	INVALC,0x4338,INVALC,INVALC,0x5E4B,INVALC,0x5E40,INVALC,
	0x5E46,INVALC,0x5E4D,0x307C,0x5E43,INVALC,0x5E4E,INVALC,
	INVALC,0x3F3C,INVALC,0x3D5F,INVALC,0x4A25,INVALC,0x3A2E,
	INVALC,0x5E3B,0x5E49,0x453A,INVALC,INVALC,INVALC,INVALC,
	/* Block 111, Array index 0x2B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4036,INVALC,0x3369,
	0x3A51,0x3E44,0x5E3D,0x3D42,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x374C,INVALC,0x5E3C,INVALC,INVALC,
	INVALC,0x5E52,0x3D6D,0x383A,INVALC,0x5E61,INVALC,0x5E5B,
	0x3574,0x454F,INVALC,0x5E56,0x5E5F,0x302F,0x3132,INVALC,
	INVALC,0x3239,INVALC,0x5E58,0x422C,0x5E4F,0x5E51,0x3941,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5E62,INVALC,0x5E5D,INVALC,INVALC,INVALC,0x5E55,INVALC,
	INVALC,INVALC,INVALC,0x5E5C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C2B,INVALC,INVALC,0x5E5A,0x5E5E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3850,INVALC,
	0x3E45,INVALC,INVALC,0x4339,INVALC,INVALC,INVALC,0x5E54,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D2F,
	INVALC,INVALC,INVALC,0x5E57,INVALC,INVALC,0x5E50,0x4572,
	INVALC,INVALC,0x5E53,INVALC,INVALC,INVALC,0x5E59,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F51,0x3C3E,
	0x4B7E,INVALC,0x5E63,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x482E,INVALC,INVALC,0x5E6F,
	0x383B,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D60,INVALC,
	0x5E65,INVALC,INVALC,INVALC,0x4E2F,0x3942,INVALC,0x5E72,
	INVALC,INVALC,0x306E,INVALC,INVALC,0x5E70,INVALC,INVALC,
	INVALC,INVALC,0x5E64,INVALC,INVALC,INVALC,INVALC,0x5E6A,
	INVALC,INVALC,0x5E6C,INVALC,INVALC,INVALC,0x4D4F,0x5E67,
	INVALC,INVALC,0x452E,INVALC,INVALC,0x5E69,INVALC,INVALC,
	INVALC,INVALC,0x5E71,INVALC,0x5E6B,0x4C47,INVALC,INVALC,
	INVALC,0x5E66,INVALC,0x3C22,0x5E7E,INVALC,INVALC,INVALC,
	INVALC,0x336A,INVALC,0x5E68,0x5E6D,0x5E6E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x426C,0x425A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E76,INVALC,INVALC,0x5E7C,
	INVALC,INVALC,0x5E7A,INVALC,0x4529,INVALC,INVALC,0x5F23,
	0x5E77,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E78,0x5E60,
	/* Block 112, Array index 0x2C00 */
	INVALC,0x3579,0x493A,INVALC,INVALC,INVALC,0x3C3F,INVALC,
	INVALC,0x3977,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F33,
	INVALC,0x5E74,INVALC,0x5F22,0x3169,0x4166,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4779,INVALC,0x3441,0x4E7A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4C21,0x4452,INVALC,INVALC,INVALC,
	INVALC,0x5E7B,0x5E7D,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4132,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F21,0x5E79,
	INVALC,0x5E73,INVALC,INVALC,INVALC,0x3443,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3769,INVALC,INVALC,INVALC,
	0x5F2F,INVALC,INVALC,0x5F2A,0x4078,INVALC,INVALC,0x3363,
	INVALC,INVALC,INVALC,INVALC,0x3D61,INVALC,0x5F33,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F2C,0x442C,0x5F29,
	0x4459,INVALC,INVALC,INVALC,0x5F4C,INVALC,INVALC,INVALC,
	0x5F26,INVALC,0x5F25,INVALC,0x5F2E,INVALC,INVALC,INVALC,
	0x5F28,0x5F27,0x5F2D,INVALC,0x4021,INVALC,0x5F24,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F30,INVALC,
	INVALC,0x5F31,INVALC,INVALC,INVALC,INVALC,INVALC,0x3442,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F36,INVALC,0x5F35,0x5F37,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5F3A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4543,INVALC,0x5F34,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F38,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3763,0x4279,0x5F32,0x473B,INVALC,INVALC,0x5F39,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F3E,0x5F3C,INVALC,INVALC,
	0x5F3F,INVALC,INVALC,0x5F42,INVALC,INVALC,INVALC,0x5F3B,
	0x396A,0x4728,INVALC,INVALC,0x5E39,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4D74,0x5F3D,INVALC,0x5F41,0x4275,
	INVALC,0x5F40,INVALC,0x5F2B,INVALC,INVALC,0x6F69,INVALC,
	INVALC,INVALC,0x5F45,INVALC,INVALC,INVALC,0x5F49,INVALC,
	/* Block 113, Array index 0x2D00 */
	INVALC,0x5F47,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F43,INVALC,0x5F44,INVALC,INVALC,INVALC,0x5F48,
	INVALC,0x5F46,INVALC,INVALC,INVALC,0x494E,INVALC,INVALC,
	0x5F4E,INVALC,0x5F4B,0x5F4A,INVALC,0x5F4D,0x4654,0x5F4F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4375,0x426D,
	INVALC,INVALC,INVALC,INVALC,0x4025,INVALC,INVALC,INVALC,
	0x5F50,INVALC,0x5F52,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F51,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E75,INVALC,INVALC,INVALC,
	INVALC,0x5F53,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4667,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5F54,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3250,INVALC,INVALC,INVALC,0x4574,
	0x3325,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3564,INVALC,INVALC,INVALC,0x3C5E,0x3A52,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F27,0x3F66,INVALC,INVALC,INVALC,0x316A,INVALC,
	INVALC,INVALC,0x5F56,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F55,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F59,0x433A,0x5F5C,0x5F57,
	INVALC,INVALC,INVALC,0x5F5B,INVALC,INVALC,INVALC,INVALC,
	0x5F5A,0x4540,0x3059,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4E75,INVALC,INVALC,0x5F5E,INVALC,INVALC,INVALC,0x3128,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F60,INVALC,INVALC,INVALC,0x5F5F,INVALC,0x5F5D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F58,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4B23,INVALC,INVALC,INVALC,0x5F62,INVALC,INVALC,
	/* Block 114, Array index 0x2E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F61,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x316B,INVALC,INVALC,INVALC,
	INVALC,0x5F64,0x4A32,INVALC,0x5F63,INVALC,INVALC,INVALC,
	INVALC,0x4C35,INVALC,INVALC,INVALC,INVALC,0x3E47,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4133,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E46,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E7B,INVALC,INVALC,0x5F6A,INVALC,0x4079,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F66,0x5F6B,INVALC,
	INVALC,0x316C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5F69,INVALC,0x4761,0x5F65,0x5F68,0x3E48,
	INVALC,0x4851,INVALC,INVALC,0x5F6C,INVALC,0x3C51,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x407A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F6F,INVALC,INVALC,INVALC,
	0x5F67,INVALC,0x3727,INVALC,INVALC,INVALC,INVALC,0x5F6D,
	INVALC,INVALC,INVALC,INVALC,0x4D50,0x5F70,INVALC,INVALC,
	INVALC,0x7426,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D4F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5F71,INVALC,INVALC,INVALC,0x5F72,INVALC,INVALC,INVALC,
	INVALC,0x472E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F74,INVALC,INVALC,INVALC,INVALC,0x5F75,INVALC,
	INVALC,INVALC,INVALC,0x4733,INVALC,INVALC,INVALC,INVALC,
	0x4575,0x5F77,INVALC,INVALC,INVALC,INVALC,0x5F79,INVALC,
	0x4E55,INVALC,0x5F76,INVALC,0x5F78,0x316D,INVALC,0x5F73,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x535B,
	0x5F7A,INVALC,INVALC,INVALC,INVALC,0x4167,0x3B38,0x5F7C,
	INVALC,INVALC,INVALC,INVALC,0x5F7B,0x3F24,0x5259,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F7D,INVALC,INVALC,
	INVALC,0x6021,INVALC,0x5F6E,0x5F7E,INVALC,INVALC,0x6022,
	/* Block 115, Array index 0x2F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x477A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6023,INVALC,INVALC,
	0x6024,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6025,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6026,INVALC,0x445E,INVALC,0x6028,0x6027,INVALC,INVALC,
	0x6029,INVALC,0x602A,INVALC,INVALC,0x3C5F,0x4963,INVALC,
	INVALC,INVALC,0x4C6C,0x602B,0x602C,0x4156,0x3C24,0x602D,
	0x602E,INVALC,INVALC,INVALC,INVALC,INVALC,0x602F,0x4A52,
	0x4847,INVALC,INVALC,0x6030,0x4757,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x442D,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6031,0x3267,INVALC,0x356D,INVALC,0x4C46,INVALC,0x4C36,
	INVALC,0x3234,0x4F34,INVALC,INVALC,INVALC,INVALC,0x4B52,
	INVALC,0x4A2A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4037,INVALC,0x6032,INVALC,INVALC,INVALC,
	INVALC,0x4643,INVALC,INVALC,INVALC,0x3823,0x6033,INVALC,
	0x3A54,0x6035,0x6034,INVALC,INVALC,INVALC,INVALC,0x6036,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6037,INVALC,INVALC,INVALC,0x6038,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x353E,INVALC,0x6039,INVALC,INVALC,INVALC,INVALC,0x603A,
	INVALC,INVALC,INVALC,INVALC,0x3824,INVALC,INVALC,0x4848,
	INVALC,INVALC,0x603C,INVALC,INVALC,INVALC,0x3E75,INVALC,
	INVALC,0x603B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3638,0x603D,0x603F,INVALC,0x603E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6040,INVALC,
	0x3851,INVALC,0x6041,INVALC,INVALC,INVALC,INVALC,0x3669,
	INVALC,0x4140,INVALC,0x397D,INVALC,INVALC,INVALC,INVALC,
	0x6043,0x6044,0x6042,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3C6D,INVALC,INVALC,0x4648,0x3639,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6046,
	0x432C,0x6045,INVALC,INVALC,0x4F35,0x4762,INVALC,INVALC,
	/* Block 116, Array index 0x3000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6049,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x604B,0x6048,
	INVALC,INVALC,INVALC,0x4C54,0x604A,0x604C,INVALC,0x4E44,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6050,INVALC,INVALC,
	INVALC,0x604F,0x4376,0x472D,INVALC,INVALC,0x3825,0x604E,
	INVALC,INVALC,INVALC,INVALC,0x604D,INVALC,0x4D31,0x4D32,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6051,0x316E,
	INVALC,INVALC,INVALC,INVALC,0x3976,0x3B62,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6052,0x6053,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6055,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3D43,INVALC,INVALC,INVALC,INVALC,
	0x6057,INVALC,0x6056,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6058,INVALC,0x334D,INVALC,INVALC,0x605A,INVALC,INVALC,
	0x6059,INVALC,0x605C,0x605B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x383C,INVALC,INVALC,0x4E28,
	INVALC,0x364C,INVALC,0x3226,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x366A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3461,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E68,0x605E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6060,INVALC,INVALC,INVALC,INVALC,
	0x6061,INVALC,0x3251,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x605D,INVALC,0x3B39,INVALC,INVALC,0x4441,0x605F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6064,INVALC,
	0x3C6E,INVALC,INVALC,INVALC,INVALC,0x6062,INVALC,INVALC,
	INVALC,INVALC,0x373E,INVALC,INVALC,0x4849,0x6063,INVALC,
	INVALC,0x607E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6069,INVALC,INVALC,INVALC,INVALC,INVALC,0x383D,INVALC,
	/* Block 117, Array index 0x3100 */
	INVALC,INVALC,INVALC,0x3565,INVALC,0x6066,0x4D7D,INVALC,
	INVALC,0x4E30,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4276,INVALC,INVALC,0x6068,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x606A,0x4E56,0x3657,0x487C,0x474A,INVALC,
	INVALC,INVALC,0x606B,INVALC,INVALC,INVALC,INVALC,0x606D,
	INVALC,0x6070,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x606C,INVALC,INVALC,
	INVALC,0x606F,0x386A,0x314D,0x6071,INVALC,0x3F70,0x606E,
	0x4E5C,INVALC,INVALC,0x6074,0x7424,INVALC,INVALC,INVALC,
	INVALC,0x6072,0x6075,INVALC,INVALC,INVALC,INVALC,0x6067,
	0x6073,INVALC,INVALC,0x3A3C,INVALC,INVALC,0x6076,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6077,INVALC,
	INVALC,INVALC,INVALC,0x4D7E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6078,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6079,INVALC,
	INVALC,INVALC,0x6065,INVALC,INVALC,INVALC,INVALC,0x607A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3444,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C25,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x607B,INVALC,INVALC,INVALC,INVALC,0x607C,
	INVALC,INVALC,INVALC,INVALC,0x607D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x313B,INVALC,INVALC,INVALC,
	0x6121,INVALC,0x493B,0x6122,INVALC,INVALC,0x3424,0x6123,
	INVALC,0x6124,INVALC,INVALC,INVALC,INVALC,0x6125,INVALC,
	0x6127,0x6128,0x6126,INVALC,INVALC,INVALC,0x4953,0x612A,
	0x6129,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 118, Array index 0x3200 */
	INVALC,INVALC,INVALC,0x612C,0x612B,0x612D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x612E,0x6130,0x612F,INVALC,
	INVALC,0x3979,INVALC,0x6132,INVALC,0x6131,INVALC,INVALC,
	0x3445,INVALC,0x3F53,INVALC,0x453C,INVALC,0x6133,0x4038,
	INVALC,INVALC,INVALC,0x3B3A,INVALC,0x3179,0x6134,INVALC,
	0x4D51,INVALC,INVALC,0x4A63,0x6135,INVALC,INVALC,INVALC,
	0x4544,0x4D33,0x3943,0x3F3D,INVALC,INVALC,INVALC,0x434B,
	0x5234,INVALC,0x442E,0x3268,0x6136,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6137,INVALC,0x613C,INVALC,
	INVALC,0x613A,0x6139,0x5A42,0x3326,0x6138,INVALC,0x305A,
	INVALC,0x482A,INVALC,INVALC,0x484A,INVALC,INVALC,INVALC,
	INVALC,0x4E31,0x613D,0x613B,0x435C,0x4026,INVALC,INVALC,
	0x482B,INVALC,0x492D,INVALC,0x613F,0x4E2C,0x374D,0x6140,
	INVALC,0x613E,0x4856,0x6141,INVALC,0x6142,INVALC,INVALC,
	0x305B,INVALC,INVALC,0x3E76,0x6147,INVALC,0x6144,0x466D,
	0x6143,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3526,
	INVALC,INVALC,0x614A,INVALC,INVALC,INVALC,0x6145,0x6146,
	INVALC,0x6149,0x6148,0x4925,INVALC,INVALC,0x4142,0x4141,
	INVALC,0x353F,INVALC,INVALC,0x614B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x614C,INVALC,INVALC,0x614D,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x614F,INVALC,0x614E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3156,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6157,0x4868,0x6151,INVALC,0x6153,INVALC,INVALC,
	0x6155,0x3F3E,INVALC,INVALC,0x6156,0x6154,0x3C40,INVALC,
	INVALC,INVALC,0x6150,0x6152,INVALC,0x4942,INVALC,0x3E49,
	INVALC,INVALC,0x6159,INVALC,INVALC,0x6158,INVALC,INVALC,
	INVALC,INVALC,0x615A,INVALC,0x3C26,0x3A2F,INVALC,INVALC,
	0x4577,0x615B,INVALC,0x444B,INVALC,INVALC,0x615D,INVALC,
	INVALC,INVALC,0x4E21,0x615C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4169,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6162,INVALC,0x6164,0x6165,0x4354,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6163,INVALC,0x6160,INVALC,0x615E,0x615F,
	/* Block 119, Array index 0x3300 */
	INVALC,0x6161,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6168,INVALC,0x6166,INVALC,0x6167,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6169,
	0x616B,0x616C,0x616D,INVALC,0x616E,INVALC,INVALC,0x616A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6170,INVALC,INVALC,INVALC,0x616F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6171,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E45,INVALC,INVALC,INVALC,0x6174,0x6172,
	0x6173,INVALC,INVALC,INVALC,0x3462,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C7E,INVALC,INVALC,INVALC,0x4A4A,INVALC,
	0x6176,INVALC,INVALC,INVALC,0x6175,INVALC,INVALC,INVALC,
	INVALC,0x6177,0x6178,INVALC,INVALC,INVALC,INVALC,0x617C,
	0x6179,0x617A,0x617B,INVALC,0x617D,INVALC,INVALC,INVALC,
	0x617E,INVALC,0x6221,INVALC,INVALC,INVALC,0x6222,INVALC,
	0x6223,INVALC,0x482F,0x4550,0x6224,0x4772,0x4934,INVALC,
	0x6225,INVALC,INVALC,0x6226,0x452A,INVALC,0x3327,0x3944,
	0x6227,INVALC,INVALC,0x6228,INVALC,INVALC,0x6229,INVALC,
	0x3B29,INVALC,INVALC,0x622B,INVALC,INVALC,0x622A,INVALC,
	INVALC,0x622C,0x622D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4869,INVALC,
	0x622E,INVALC,INVALC,INVALC,0x622F,INVALC,INVALC,0x7369,
	0x6230,0x6231,0x6232,INVALC,INVALC,INVALC,INVALC,0x3B2E,
	INVALC,INVALC,0x6233,0x4756,INVALC,INVALC,0x4B5F,INVALC,
	0x314E,INVALC,0x3157,INVALC,INVALC,0x6234,INVALC,INVALC,
	INVALC,INVALC,0x6236,INVALC,INVALC,INVALC,0x6235,0x4570,
	INVALC,INVALC,INVALC,0x4039,0x5D39,INVALC,0x6237,0x4C41,
	INVALC,0x6238,INVALC,0x3446,0x4857,0x6239,INVALC,0x623A,
	INVALC,INVALC,0x623B,INVALC,INVALC,INVALC,0x4C5C,INVALC,
	INVALC,INVALC,0x4C55,INVALC,0x443E,INVALC,INVALC,INVALC,
	0x416A,INVALC,INVALC,0x623D,INVALC,INVALC,0x3D62,INVALC,
	/* Block 120, Array index 0x3400 */
	INVALC,0x3E4A,INVALC,INVALC,0x6240,INVALC,INVALC,0x623F,
	0x623E,0x487D,INVALC,0x3447,0x3829,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6246,INVALC,INVALC,0x6243,0x3F3F,
	0x4C32,INVALC,INVALC,INVALC,0x6242,0x6244,0x6245,INVALC,
	INVALC,0x6241,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6247,
	0x6248,INVALC,0x442F,INVALC,0x3463,INVALC,INVALC,INVALC,
	0x4365,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6249,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x624A,0x624D,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3F67,INVALC,0x4644,INVALC,0x624E,0x4B53,INVALC,
	0x624B,INVALC,INVALC,0x624C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6251,INVALC,INVALC,INVALC,INVALC,0x6250,0x624F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6253,INVALC,INVALC,0x6252,INVALC,
	INVALC,0x6254,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6256,INVALC,
	0x6255,INVALC,INVALC,INVALC,INVALC,0x4A4D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D56,0x4E46,INVALC,INVALC,
	0x6257,INVALC,INVALC,0x4637,INVALC,INVALC,0x6258,INVALC,
	INVALC,0x6259,INVALC,0x625D,0x625B,0x625C,INVALC,0x625A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x625E,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x625F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6260,
	INVALC,INVALC,0x6261,0x4C37,0x6262,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C70,0x6263,INVALC,0x434E,INVALC,0x476A,
	INVALC,0x366B,INVALC,INVALC,INVALC,0x433B,0x6264,0x363A,
	INVALC,INVALC,INVALC,0x4050,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6265,INVALC,INVALC,INVALC,
	/* Block 121, Array index 0x3500 */
	INVALC,INVALC,0x3A3D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6266,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6267,INVALC,0x3826,0x3A55,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6269,INVALC,INVALC,INVALC,INVALC,0x4556,0x3A56,0x354E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4B24,INVALC,0x474B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4557,INVALC,INVALC,INVALC,INVALC,0x395C,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x626B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3E4B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E32,0x3945,INVALC,INVALC,0x3827,
	INVALC,INVALC,0x4823,INVALC,0x626D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x626F,INVALC,INVALC,INVALC,
	INVALC,0x386B,INVALC,INVALC,INVALC,INVALC,0x626E,0x4476,
	INVALC,INVALC,INVALC,INVALC,0x6271,0x3337,0x626C,INVALC,
	INVALC,0x486A,INVALC,0x3130,INVALC,0x3A6C,INVALC,0x4F52,
	INVALC,INVALC,0x6270,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6272,INVALC,INVALC,INVALC,0x4A4B,
	INVALC,0x4059,0x6274,INVALC,INVALC,INVALC,INVALC,0x6275,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6273,INVALC,INVALC,
	INVALC,INVALC,0x334E,INVALC,0x627B,INVALC,0x627A,INVALC,
	INVALC,0x3C27,INVALC,INVALC,INVALC,0x627C,0x6277,INVALC,
	INVALC,INVALC,0x627D,0x6278,INVALC,INVALC,INVALC,INVALC,
	0x4858,0x6276,INVALC,INVALC,0x6279,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6322,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6321,
	0x4B61,INVALC,INVALC,INVALC,0x627E,INVALC,INVALC,0x306B,
	INVALC,INVALC,INVALC,INVALC,0x6324,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6323,INVALC,INVALC,
	/* Block 122, Array index 0x3600 */
	INVALC,0x3E4C,INVALC,INVALC,INVALC,INVALC,INVALC,0x6325,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4143,INVALC,
	INVALC,0x6327,0x6326,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6328,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6268,INVALC,
	INVALC,INVALC,0x626A,0x632A,0x6329,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3C28,INVALC,0x4E69,INVALC,0x3C52,INVALC,
	0x632B,0x3737,INVALC,INVALC,INVALC,INVALC,INVALC,0x3540,
	0x3527,0x3B63,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D34,INVALC,INVALC,0x6331,INVALC,0x6330,0x4144,0x632D,
	INVALC,INVALC,0x632F,INVALC,INVALC,0x3D4B,0x3F40,0x632E,
	0x632C,INVALC,0x472A,INVALC,INVALC,0x3E4D,INVALC,INVALC,
	0x493C,INVALC,INVALC,INVALC,INVALC,0x3A57,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4578,
	INVALC,INVALC,0x6332,INVALC,INVALC,INVALC,INVALC,0x6333,
	0x6349,0x3658,INVALC,INVALC,0x4F3D,0x4135,INVALC,INVALC,
	INVALC,INVALC,0x6334,INVALC,INVALC,0x3252,0x4477,0x4A21,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6335,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x357A,0x6336,
	INVALC,INVALC,0x6338,INVALC,INVALC,INVALC,0x6339,INVALC,
	0x4729,INVALC,INVALC,0x633A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x633B,0x633C,INVALC,INVALC,0x3659,0x3253,0x4645,
	0x3D28,0x3B64,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x633D,INVALC,0x3D29,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x324A,0x4943,INVALC,INVALC,0x633E,INVALC,INVALC,
	0x486B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4145,
	INVALC,0x6341,INVALC,0x6342,0x4769,INVALC,0x3F41,0x633F,
	INVALC,0x4361,INVALC,INVALC,0x6340,INVALC,INVALC,INVALC,
	0x3E4E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x305C,INVALC,INVALC,INVALC,INVALC,
	/* Block 123, Array index 0x3700 */
	0x3529,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6343,INVALC,INVALC,0x4478,INVALC,0x6344,0x4047,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4C2D,INVALC,INVALC,0x4923,
	0x6345,0x6346,0x4355,INVALC,0x4E47,INVALC,INVALC,0x6348,
	0x6347,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C6F,INVALC,
	INVALC,0x634A,0x3070,INVALC,INVALC,INVALC,INVALC,0x634D,
	INVALC,INVALC,INVALC,0x634B,0x3254,0x374E,0x634C,0x3946,
	0x3972,INVALC,0x4A66,0x634E,INVALC,INVALC,0x4B54,INVALC,
	INVALC,0x6350,INVALC,INVALC,INVALC,0x4051,0x314F,0x323A,
	0x302C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x634F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6351,0x6352,0x3E77,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6353,INVALC,0x334F,INVALC,INVALC,INVALC,INVALC,
	0x6355,INVALC,INVALC,INVALC,0x376A,INVALC,0x3566,INVALC,
	INVALC,0x6356,0x3675,INVALC,INVALC,0x6357,INVALC,0x407C,
	INVALC,0x464D,INVALC,0x4060,0x3A75,INVALC,INVALC,INVALC,
	0x6358,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4362,0x416B,INVALC,0x635A,0x635C,0x6359,
	0x635B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3722,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x635D,0x3726,INVALC,INVALC,INVALC,0x3567,0x4D52,
	0x635F,INVALC,INVALC,INVALC,INVALC,INVALC,0x6360,INVALC,
	INVALC,INVALC,0x312E,INVALC,INVALC,INVALC,INVALC,0x6363,
	INVALC,INVALC,INVALC,0x3376,0x6362,0x6361,INVALC,0x6365,
	0x635E,INVALC,0x6366,0x4E29,INVALC,0x6367,INVALC,0x6368,
	INVALC,INVALC,0x5474,0x636A,INVALC,0x6369,INVALC,INVALC,
	INVALC,0x636B,0x636C,INVALC,0x4E35,0x636D,INVALC,0x706F,
	0x3E4F,0x636E,0x636F,0x3D57,INVALC,0x4638,0x6370,INVALC,
	INVALC,INVALC,0x4328,INVALC,INVALC,0x6371,INVALC,0x433C,
	0x6372,INVALC,INVALC,INVALC,INVALC,INVALC,0x3625,INVALC,
	0x513F,0x435D,0x3C33,INVALC,INVALC,INVALC,INVALC,0x3448,
	/* Block 124, Array index 0x3800 */
	INVALC,INVALC,0x6373,INVALC,0x6422,INVALC,0x6376,INVALC,
	0x3568,INVALC,0x6375,0x6424,INVALC,INVALC,INVALC,0x6374,
	INVALC,0x3E50,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6378,0x6379,INVALC,0x452B,INVALC,INVALC,0x637A,INVALC,
	0x335E,INVALC,INVALC,INVALC,INVALC,0x3F5A,0x4964,INVALC,
	0x637C,INVALC,INVALC,INVALC,0x4268,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6377,INVALC,0x637B,0x637D,INVALC,
	INVALC,0x3A7B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6426,0x492E,INVALC,
	0x4826,0x4579,INVALC,0x365A,0x6425,0x6423,INVALC,0x4835,
	0x637E,0x435E,0x457B,INVALC,0x457A,INVALC,0x3A76,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6438,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6428,INVALC,0x642A,
	INVALC,INVALC,INVALC,INVALC,0x642D,INVALC,0x642E,INVALC,
	0x642B,0x642C,INVALC,INVALC,0x6429,0x6427,INVALC,INVALC,
	INVALC,INVALC,0x6421,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A4F,0x3255,
	INVALC,INVALC,INVALC,0x6435,INVALC,0x6432,INVALC,0x6437,
	INVALC,INVALC,0x6436,INVALC,0x4773,0x4C27,INVALC,0x3B3B,
	0x6430,0x6439,0x6434,INVALC,0x6433,0x642F,INVALC,0x6431,
	INVALC,0x3449,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x433D,INVALC,INVALC,0x407D,INVALC,INVALC,
	INVALC,0x4822,INVALC,INVALC,0x643E,INVALC,INVALC,INVALC,
	0x4824,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4061,0x643B,INVALC,INVALC,0x484F,INVALC,0x643F,0x4A53,
	INVALC,0x435B,INVALC,0x643A,0x643C,INVALC,INVALC,0x643D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6440,INVALC,INVALC,
	0x3C44,INVALC,INVALC,INVALC,0x4646,0x6445,0x6444,INVALC,
	INVALC,0x6441,INVALC,INVALC,INVALC,0x4F36,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x644A,INVALC,INVALC,0x644E,0x644B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 125, Array index 0x3900 */
	0x6447,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6448,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x644D,INVALC,INVALC,
	INVALC,0x6442,0x5255,0x6449,0x6443,INVALC,INVALC,0x644C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6452,
	INVALC,0x344A,INVALC,0x644F,INVALC,INVALC,INVALC,0x6450,
	INVALC,INVALC,0x6451,0x6454,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6453,
	0x4876,INVALC,INVALC,INVALC,INVALC,0x6455,0x4E7C,0x4A6D,
	0x645A,INVALC,INVALC,0x6457,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6456,0x4052,INVALC,0x6459,
	0x645B,INVALC,INVALC,INVALC,0x6458,INVALC,0x645F,INVALC,
	0x645C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x645D,
	0x6446,INVALC,INVALC,INVALC,0x645E,0x6460,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6461,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A46,INVALC,0x6462,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C62,INVALC,
	INVALC,0x364E,0x3729,0x6463,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A34,INVALC,0x3F68,INVALC,0x4C30,INVALC,INVALC,
	0x6464,INVALC,0x4E33,INVALC,INVALC,0x4774,INVALC,0x4146,
	0x4734,INVALC,INVALC,0x3D4D,INVALC,INVALC,INVALC,0x3040,
	INVALC,0x6469,0x6467,INVALC,0x6465,0x3421,INVALC,0x3E51,
	0x646A,INVALC,INVALC,0x6468,INVALC,0x6466,0x646E,INVALC,
	INVALC,0x646D,0x646C,0x646B,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x646F,INVALC,INVALC,INVALC,0x6470,0x403A,INVALC,
	0x6471,INVALC,0x6473,INVALC,INVALC,0x6472,INVALC,INVALC,
	INVALC,INVALC,0x3852,INVALC,INVALC,INVALC,0x4138,INVALC,
	INVALC,INVALC,0x6475,INVALC,INVALC,INVALC,0x457C,INVALC,
	0x6474,INVALC,INVALC,INVALC,0x6476,INVALC,0x4A35,0x416C,
	0x3947,INVALC,0x6477,INVALC,INVALC,INVALC,INVALC,0x4E48,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6479,
	INVALC,INVALC,0x647A,INVALC,0x647B,INVALC,0x647C,INVALC,
	0x3B65,INVALC,0x647D,0x374F,INVALC,INVALC,0x356A,INVALC,
	/* Block 126, Array index 0x3A00 */
	0x352A,INVALC,0x6521,INVALC,0x4C73,0x3948,0x647E,INVALC,
	INVALC,INVALC,0x6524,0x4C66,INVALC,0x473C,INVALC,INVALC,
	0x4933,INVALC,INVALC,INVALC,0x3D63,0x6523,INVALC,0x3C53,
	0x3949,0x3B66,0x3569,0x4A36,0x6522,INVALC,INVALC,INVALC,
	0x4147,0x4B42,0x3A77,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3B67,0x445D,INVALC,0x6527,0x4E5F,
	0x3A59,INVALC,0x6528,0x3F42,INVALC,0x652A,INVALC,INVALC,
	INVALC,0x3E52,0x3A30,INVALC,INVALC,INVALC,INVALC,0x6529,
	INVALC,INVALC,0x3D2A,0x383E,0x4148,0x6525,0x652B,INVALC,
	INVALC,INVALC,INVALC,0x6526,0x3750,INVALC,0x652E,0x6532,
	0x376B,INVALC,INVALC,INVALC,INVALC,INVALC,0x652D,INVALC,
	INVALC,INVALC,INVALC,0x6536,INVALC,INVALC,0x394A,INVALC,
	INVALC,0x4D6D,0x303C,0x6533,INVALC,INVALC,0x356B,INVALC,
	0x6530,INVALC,INVALC,INVALC,INVALC,INVALC,0x6531,INVALC,
	INVALC,0x457D,0x652F,0x652C,INVALC,0x3328,0x4064,INVALC,
	INVALC,0x3828,INVALC,INVALC,INVALC,0x6538,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6535,INVALC,INVALC,INVALC,INVALC,INVALC,0x6537,
	INVALC,INVALC,INVALC,0x6534,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3751,0x4233,0x6539,0x416E,INVALC,INVALC,0x6546,
	INVALC,INVALC,0x6542,0x653C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6540,0x3C7A,0x305D,0x653B,0x6543,
	0x6547,0x394B,0x4C56,INVALC,0x4456,0x653D,INVALC,INVALC,
	0x6545,INVALC,0x653A,0x433E,INVALC,0x653F,0x303D,0x4C4A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x653E,
	INVALC,INVALC,0x365B,0x486C,INVALC,INVALC,INVALC,0x416D,
	INVALC,0x4E50,0x3D6F,INVALC,INVALC,0x656E,INVALC,INVALC,
	0x6548,INVALC,0x407E,INVALC,0x6544,0x6549,0x654B,INVALC,
	0x4479,0x654E,INVALC,INVALC,0x654A,INVALC,INVALC,INVALC,
	0x4A54,0x344B,INVALC,INVALC,0x4C4B,INVALC,INVALC,0x305E,
	INVALC,INVALC,0x654D,INVALC,0x4E7D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x654C,INVALC,INVALC,INVALC,INVALC,
	/* Block 127, Array index 0x3B00 */
	INVALC,0x316F,INVALC,INVALC,0x466C,0x654F,INVALC,INVALC,
	INVALC,0x6556,0x6550,0x6557,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6553,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x477B,INVALC,INVALC,0x3C4A,0x6555,
	INVALC,0x6552,0x6558,0x6551,INVALC,INVALC,0x3D44,INVALC,
	INVALC,INVALC,INVALC,0x4B25,INVALC,INVALC,0x3D4C,INVALC,
	INVALC,0x6554,0x6560,INVALC,INVALC,0x655C,INVALC,0x655F,
	INVALC,0x655D,0x6561,0x655B,INVALC,0x6541,0x4053,INVALC,
	INVALC,0x484B,INVALC,0x655E,INVALC,INVALC,0x6559,INVALC,
	INVALC,INVALC,0x4121,0x3752,INVALC,0x3D2B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F25,0x4136,0x6564,INVALC,
	INVALC,0x6566,0x6567,INVALC,INVALC,0x6563,0x6565,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x655A,0x6562,
	INVALC,0x656A,0x6569,INVALC,INVALC,0x4B7A,INVALC,INVALC,
	0x372B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6568,INVALC,0x656C,0x656B,0x656F,INVALC,0x6571,
	INVALC,INVALC,0x3B3C,0x656D,INVALC,INVALC,INVALC,INVALC,
	0x6572,0x6573,INVALC,INVALC,0x6574,INVALC,0x657A,0x453B,
	0x6576,INVALC,0x6575,0x6577,0x6578,INVALC,0x6579,INVALC,
	INVALC,INVALC,INVALC,0x657B,0x657C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 128, Array index 0x3C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x344C,INVALC,
	0x657D,INVALC,0x657E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6621,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6622,0x6623,0x6624,INVALC,
	0x6625,0x6626,INVALC,INVALC,0x6628,0x6627,INVALC,INVALC,
	0x6629,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x662A,
	0x662B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x662E,
	0x662C,0x662D,0x3A61,0x3753,INVALC,INVALC,0x4356,INVALC,
	0x4833,INVALC,0x3D70,INVALC,INVALC,0x474D,INVALC,0x486D,
	0x662F,0x586D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6630,0x6632,INVALC,0x4D65,0x6631,0x6634,
	0x6633,INVALC,0x4D53,INVALC,0x6635,INVALC,0x487E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6636,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6639,INVALC,INVALC,0x6638,0x6637,INVALC,
	INVALC,INVALC,INVALC,0x663A,0x3732,INVALC,INVALC,INVALC,
	0x4122,0x3541,INVALC,INVALC,INVALC,INVALC,0x663E,0x663B,
	INVALC,INVALC,0x663C,INVALC,INVALC,INVALC,0x663F,INVALC,
	0x6640,0x663D,INVALC,INVALC,INVALC,0x3129,INVALC,INVALC,
	INVALC,0x3227,INVALC,INVALC,INVALC,0x6642,0x6643,INVALC,
	INVALC,INVALC,0x6644,INVALC,0x4D62,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3D2C,INVALC,0x6646,0x6645,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F69,0x6647,INVALC,INVALC,INVALC,INVALC,0x6648,INVALC,
	INVALC,0x6649,INVALC,0x3465,INVALC,INVALC,INVALC,INVALC,
	0x344D,INVALC,INVALC,0x664A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x664B,INVALC,0x4B5D,0x4D63,INVALC,INVALC,INVALC,
	/* Block 129, Array index 0x3D00 */
	0x4D54,0x4F37,INVALC,0x394D,0x664E,0x3C54,0x664D,INVALC,
	INVALC,INVALC,INVALC,0x664F,0x3C29,INVALC,INVALC,INVALC,
	0x4251,INVALC,0x6650,INVALC,INVALC,0x394C,INVALC,0x4C57,
	0x6651,0x6652,INVALC,INVALC,0x6653,INVALC,INVALC,INVALC,
	INVALC,0x6654,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6655,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3C2A,INVALC,INVALC,0x4C6D,INVALC,
	INVALC,INVALC,INVALC,0x6657,INVALC,0x433F,INVALC,0x6656,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6659,INVALC,
	INVALC,INVALC,0x6658,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x665A,INVALC,INVALC,INVALC,0x403B,INVALC,
	0x665B,INVALC,0x665C,INVALC,INVALC,INVALC,0x4A39,0x665D,
	INVALC,0x416F,0x665E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x665F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E7E,
	0x6662,INVALC,0x6661,0x6660,0x4430,INVALC,0x6663,0x3F26,
	INVALC,0x6664,INVALC,INVALC,INVALC,0x6665,0x4F38,0x6666,
	INVALC,INVALC,INVALC,INVALC,0x6667,0x6669,0x6668,0x4825,
	INVALC,0x4679,INVALC,0x4F3E,0x4829,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x666B,INVALC,INVALC,0x3E53,INVALC,
	0x492A,INVALC,0x666C,0x666A,INVALC,0x344E,INVALC,INVALC,
	INVALC,0x3854,0x3B68,INVALC,INVALC,0x486E,INVALC,INVALC,
	INVALC,0x382A,0x4B43,INVALC,0x666F,0x666D,INVALC,0x394E,
	INVALC,0x394F,0x3069,INVALC,0x3A68,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4759,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x305F,0x6674,INVALC,0x4340,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4758,INVALC,0x425B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6676,INVALC,
	INVALC,0x6672,0x6675,0x6670,INVALC,0x6673,0x4B26,INVALC,
	INVALC,0x3855,INVALC,INVALC,0x307D,0x6671,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6678,
	INVALC,0x6679,INVALC,INVALC,0x4639,INVALC,INVALC,INVALC,
	0x363B,INVALC,INVALC,INVALC,0x6726,0x473D,INVALC,INVALC,
	/* Block 130, Array index 0x3E00 */
	INVALC,INVALC,0x3B69,INVALC,INVALC,0x363C,0x4048,0x4F46,
	0x4C2E,0x6677,0x4054,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3553,0x667A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x667C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x667B,INVALC,INVALC,INVALC,INVALC,INVALC,0x667D,
	INVALC,0x4326,INVALC,0x473E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4431,INVALC,INVALC,INVALC,INVALC,0x6723,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6722,INVALC,
	INVALC,INVALC,INVALC,0x667E,INVALC,INVALC,0x3F55,INVALC,
	0x4965,0x6725,INVALC,0x6724,0x3950,0x4F53,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6735,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6729,0x672A,INVALC,
	INVALC,INVALC,INVALC,0x3C70,INVALC,INVALC,0x6728,INVALC,
	0x3978,0x6727,INVALC,INVALC,0x672B,INVALC,INVALC,INVALC,
	0x4432,0x4A22,0x4123,INVALC,INVALC,INVALC,INVALC,0x425C,
	0x672F,INVALC,0x6730,0x672C,INVALC,INVALC,INVALC,INVALC,
	0x672D,INVALC,0x672E,INVALC,INVALC,INVALC,INVALC,0x3951,
	INVALC,INVALC,INVALC,0x6736,INVALC,0x6732,INVALC,INVALC,
	INVALC,INVALC,0x4966,INVALC,0x4B6C,0x4928,INVALC,INVALC,
	0x6731,INVALC,INVALC,0x6734,0x6733,INVALC,INVALC,INVALC,
	0x4B44,0x6737,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6738,INVALC,INVALC,0x4137,INVALC,0x6739,INVALC,INVALC,
	0x673B,INVALC,0x673F,INVALC,INVALC,0x673C,0x673A,0x473F,
	0x673D,INVALC,0x673E,INVALC,INVALC,INVALC,0x3232,INVALC,
	0x6745,0x6740,INVALC,INVALC,INVALC,0x6741,INVALC,INVALC,
	INVALC,0x6742,INVALC,0x4221,INVALC,INVALC,INVALC,INVALC,
	0x6744,0x6743,0x6746,INVALC,INVALC,INVALC,INVALC,0x6747,
	0x6748,INVALC,INVALC,0x3F43,INVALC,0x3269,INVALC,0x6749,
	0x4E57,INVALC,0x3C2B,INVALC,INVALC,0x3D2D,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3B6A,0x4357,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x674A,0x674B,0x3131,INVALC,0x674C,INVALC,
	/* Block 131, Array index 0x3F00 */
	INVALC,0x674D,0x674E,INVALC,INVALC,0x674F,INVALC,0x6750,
	0x363D,0x5A2A,0x6751,INVALC,0x4065,0x6752,0x3C4B,INVALC,
	0x6753,INVALC,0x5030,INVALC,INVALC,INVALC,0x6754,0x4A5E,
	0x345C,INVALC,INVALC,0x4124,0x3D58,INVALC,0x4971,0x3D2E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6755,0x3952,0x6756,0x484C,INVALC,0x6764,INVALC,
	INVALC,INVALC,INVALC,0x6758,INVALC,0x4249,0x4775,0x383F,
	0x6757,0x4125,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6759,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x447A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x675B,0x675A,0x675D,INVALC,INVALC,0x675C,INVALC,0x675E,
	INVALC,INVALC,0x6760,INVALC,0x675F,INVALC,0x344F,INVALC,
	0x6761,INVALC,0x6762,0x6763,INVALC,INVALC,0x3A31,0x4E49,
	INVALC,0x6765,0x3F27,INVALC,INVALC,INVALC,0x3170,0x6766,
	0x6767,INVALC,INVALC,INVALC,INVALC,INVALC,0x6768,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3072,INVALC,0x6769,INVALC,INVALC,
	INVALC,INVALC,0x676A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4967,INVALC,INVALC,INVALC,0x3C47,INVALC,0x676C,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3329,0x3032,INVALC,
	INVALC,INVALC,INVALC,0x676B,0x676E,0x474E,INVALC,0x3F44,
	INVALC,0x3256,INVALC,0x4B27,INVALC,INVALC,INVALC,INVALC,
	0x375D,0x365C,INVALC,0x676D,INVALC,0x326A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3423,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3171,0x6772,0x4E6A,0x425D,INVALC,INVALC,0x4944,
	INVALC,0x677E,INVALC,0x3257,0x677C,INVALC,0x677A,0x6771,
	INVALC,0x676F,INVALC,0x6770,INVALC,0x3C63,0x366C,0x4377,
	INVALC,INVALC,INVALC,0x4651,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3151,INVALC,0x6774,0x6773,INVALC,INVALC,INVALC,
	INVALC,0x6779,0x6775,0x6778,INVALC,INVALC,INVALC,INVALC,
	/* Block 132, Array index 0x4000 */
	INVALC,INVALC,0x4C50,0x6777,0x3258,0x337D,0x677B,INVALC,
	INVALC,0x677D,INVALC,INVALC,INVALC,INVALC,0x3754,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6823,0x682C,
	0x682D,INVALC,INVALC,INVALC,0x302B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6834,INVALC,INVALC,INVALC,INVALC,
	0x3071,INVALC,INVALC,0x682B,INVALC,INVALC,INVALC,0x682A,
	INVALC,0x6825,0x6824,INVALC,0x6822,0x6821,0x4363,INVALC,
	0x427B,0x6827,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6826,INVALC,INVALC,INVALC,INVALC,0x6829,INVALC,INVALC,
	INVALC,0x4170,0x3755,INVALC,INVALC,INVALC,INVALC,0x3141,
	0x6828,INVALC,0x3953,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4171,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x683A,INVALC,0x683B,INVALC,0x3259,
	INVALC,INVALC,INVALC,0x322E,0x6838,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x682E,INVALC,0x6836,
	INVALC,0x683D,0x6837,INVALC,INVALC,INVALC,0x6835,INVALC,
	INVALC,INVALC,INVALC,0x6776,INVALC,INVALC,0x6833,INVALC,
	INVALC,INVALC,0x682F,INVALC,INVALC,INVALC,0x3450,0x6831,
	0x683C,INVALC,0x6832,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x683E,INVALC,0x6830,0x477C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D69,INVALC,INVALC,INVALC,0x6839,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x684F,INVALC,INVALC,
	INVALC,0x6847,INVALC,INVALC,INVALC,0x3F7B,INVALC,INVALC,
	INVALC,INVALC,0x3546,INVALC,0x365D,INVALC,0x6842,INVALC,
	INVALC,INVALC,INVALC,0x325B,INVALC,INVALC,0x3E54,INVALC,
	0x6845,INVALC,INVALC,INVALC,0x3A5A,INVALC,INVALC,0x4551,
	0x684A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A6E,INVALC,0x6841,INVALC,INVALC,INVALC,0x325A,
	0x3856,0x4929,0x684B,INVALC,0x683F,INVALC,INVALC,0x6848,
	INVALC,INVALC,INVALC,0x6852,INVALC,0x6843,INVALC,INVALC,
	/* Block 133, Array index 0x4100 */
	INVALC,INVALC,INVALC,0x6844,0x463A,INVALC,INVALC,0x6849,
	INVALC,INVALC,INVALC,0x6846,0x4B28,0x684C,0x3060,INVALC,
	INVALC,INVALC,INVALC,0x6840,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x684E,INVALC,0x684D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x476B,0x6854,INVALC,0x685F,INVALC,INVALC,INVALC,
	INVALC,0x337E,INVALC,INVALC,INVALC,0x6862,INVALC,INVALC,
	0x6850,INVALC,INVALC,INVALC,0x6855,0x4D6E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x685E,INVALC,
	INVALC,0x4D55,INVALC,INVALC,INVALC,INVALC,0x4E2A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4378,
	INVALC,INVALC,INVALC,0x336B,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4972,0x6864,0x4621,INVALC,INVALC,0x3031,INVALC,
	INVALC,0x685D,INVALC,0x6859,0x4172,0x6853,0x685B,0x6860,
	INVALC,0x472C,INVALC,INVALC,INVALC,0x302A,INVALC,0x6858,
	INVALC,0x6861,0x4978,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x685C,INVALC,0x6857,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3E55,INVALC,INVALC,INVALC,INVALC,
	0x3D2F,INVALC,INVALC,INVALC,0x3C2C,INVALC,INVALC,INVALC,
	INVALC,0x4C58,INVALC,INVALC,0x4947,INVALC,INVALC,0x6867,
	INVALC,0x6870,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x685A,INVALC,INVALC,
	INVALC,INVALC,0x3377,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E78,0x6865,INVALC,0x686A,0x4173,INVALC,INVALC,0x6866,
	INVALC,0x686D,INVALC,INVALC,0x435F,INVALC,0x686E,INVALC,
	INVALC,0x4D56,0x6863,0x3338,INVALC,0x6869,INVALC,INVALC,
	0x686C,0x4C2C,INVALC,INVALC,INVALC,INVALC,0x686F,INVALC,
	INVALC,0x6868,0x686B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4B29,INVALC,0x4F21,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6873,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x687A,INVALC,INVALC,0x6872,
	/* Block 134, Array index 0x4200 */
	0x3C43,INVALC,INVALC,INVALC,INVALC,INVALC,0x6851,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A4E,INVALC,0x4C22,0x6879,0x6878,INVALC,0x6874,
	0x6875,INVALC,0x3136,INVALC,INVALC,INVALC,INVALC,0x6877,
	INVALC,0x6871,INVALC,INVALC,INVALC,INVALC,0x4455,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6876,0x307E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4222,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4A43,INVALC,INVALC,
	0x687B,0x6921,INVALC,0x4859,INVALC,INVALC,INVALC,INVALC,
	0x687E,0x3E56,0x3C49,0x6923,INVALC,INVALC,0x363E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6924,INVALC,0x4979,
	0x687D,INVALC,0x6856,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x687C,INVALC,INVALC,INVALC,INVALC,
	0x4F4F,0x4622,0x4973,INVALC,INVALC,0x692B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6931,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6932,INVALC,
	0x6925,INVALC,INVALC,INVALC,0x4776,INVALC,INVALC,0x692F,
	0x6927,INVALC,0x6929,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6933,0x6928,INVALC,INVALC,0x692C,INVALC,INVALC,0x3172,
	INVALC,0x4665,INVALC,0x692D,0x6930,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6926,INVALC,0x4126,INVALC,
	0x692A,0x3B27,0x3F45,0x3730,0x4C74,INVALC,0x4C79,0x3D72,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6937,0x6935,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F4E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6934,INVALC,INVALC,INVALC,0x4D75,INVALC,0x6936,
	0x6938,INVALC,INVALC,INVALC,INVALC,0x6939,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x693C,0x693A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4623,0x693B,INVALC,INVALC,
	INVALC,0x484D,0x692E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D73,
	INVALC,0x693D,0x6942,0x4174,INVALC,INVALC,0x6941,INVALC,
	/* Block 135, Array index 0x4300 */
	INVALC,INVALC,0x6922,INVALC,INVALC,INVALC,0x6943,0x4149,
	INVALC,INVALC,0x693E,0x6940,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x693F,INVALC,INVALC,0x5D31,0x5D22,
	INVALC,INVALC,0x6945,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6944,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4D76,INVALC,0x623C,
	0x6946,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6947,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6948,0x3857,INVALC,
	0x3554,INVALC,INVALC,INVALC,0x694A,0x515D,INVALC,INVALC,
	INVALC,INVALC,0x3575,INVALC,0x4E3A,INVALC,0x3673,0x694B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x694C,
	INVALC,INVALC,INVALC,0x436E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x694D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x467A,INVALC,0x303A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3263,0x6952,0x6953,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x694E,INVALC,0x3B3D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x694F,0x4742,INVALC,INVALC,INVALC,
	INVALC,0x6950,0x6951,0x695B,INVALC,INVALC,INVALC,0x6955,
	0x6958,INVALC,INVALC,INVALC,INVALC,INVALC,0x6954,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6956,INVALC,0x6957,0x3C58,
	INVALC,0x6959,INVALC,0x4341,INVALC,0x3756,0x3342,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x695C,INVALC,INVALC,INVALC,
	INVALC,0x333F,INVALC,0x6961,INVALC,INVALC,0x695D,0x6960,
	INVALC,INVALC,INVALC,INVALC,0x483A,INVALC,INVALC,INVALC,
	INVALC,0x695E,INVALC,INVALC,0x695F,0x4948,0x485A,0x6962,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x427D,0x696C,INVALC,0x6968,INVALC,INVALC,0x326B,INVALC,
	/* Block 136, Array index 0x4400 */
	0x6966,INVALC,0x4B2A,0x6967,INVALC,INVALC,0x6964,INVALC,
	0x6965,0x696A,0x696D,INVALC,INVALC,0x696B,INVALC,INVALC,
	INVALC,0x6969,0x6963,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4358,INVALC,0x6974,INVALC,0x4C2A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6972,INVALC,INVALC,
	INVALC,0x6973,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x696E,INVALC,INVALC,0x6970,
	INVALC,INVALC,INVALC,0x6971,INVALC,INVALC,INVALC,0x696F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4066,INVALC,0x4F39,0x6978,INVALC,0x6979,INVALC,
	INVALC,INVALC,INVALC,0x6A21,INVALC,0x3F2A,INVALC,0x697B,
	INVALC,0x697E,INVALC,INVALC,INVALC,INVALC,INVALC,0x6976,
	0x6975,INVALC,INVALC,0x6A22,INVALC,INVALC,0x325C,INVALC,
	0x697C,INVALC,0x6A23,INVALC,INVALC,INVALC,0x697D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x697A,INVALC,0x4433,INVALC,
	0x6977,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4768,
	INVALC,INVALC,0x6A27,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4D3B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A26,
	INVALC,INVALC,0x6A25,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6A2E,INVALC,INVALC,INVALC,0x6A28,
	INVALC,INVALC,INVALC,0x6A30,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D66,0x6A33,INVALC,0x6A2A,INVALC,INVALC,
	0x6A2B,INVALC,INVALC,INVALC,0x6A2F,INVALC,0x6A32,0x6A31,
	INVALC,INVALC,INVALC,0x6A29,INVALC,INVALC,INVALC,INVALC,
	0x6A2C,INVALC,0x6A3D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6A36,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A34,
	INVALC,INVALC,0x6A35,INVALC,INVALC,INVALC,0x6A3A,0x6A3B,
	INVALC,0x332A,INVALC,0x3542,INVALC,INVALC,0x6A39,INVALC,
	/* Block 137, Array index 0x4500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6A24,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6A38,0x6A3C,0x6A37,
	INVALC,0x6A3E,INVALC,INVALC,INVALC,0x6A40,0x6A3F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A42,0x6A41,0x695A,INVALC,INVALC,INVALC,0x6A46,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A43,INVALC,INVALC,INVALC,INVALC,0x6A44,INVALC,
	INVALC,0x6A45,INVALC,0x6A47,INVALC,INVALC,INVALC,INVALC,
	0x376C,INVALC,0x6A49,INVALC,0x6A48,INVALC,0x3D30,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3954,0x5E27,INVALC,INVALC,
	INVALC,INVALC,0x6A4A,0x3D51,INVALC,INVALC,INVALC,0x3339,
	INVALC,0x6A4B,INVALC,0x3152,INVALC,0x3E57,0x6A4C,INVALC,
	INVALC,0x3955,0x6A4D,0x3061,INVALC,INVALC,INVALC,INVALC,
	0x493D,INVALC,INVALC,0x6A4E,INVALC,INVALC,INVALC,INVALC,
	0x3F6A,INVALC,0x6A55,INVALC,INVALC,0x6A52,INVALC,0x436F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6A53,0x6A50,0x365E,
	INVALC,0x6A4F,0x6A56,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3736,INVALC,INVALC,0x425E,INVALC,0x6A5C,INVALC,INVALC,
	INVALC,INVALC,0x6A58,INVALC,INVALC,INVALC,0x4235,0x6A57,
	INVALC,0x6A5A,INVALC,INVALC,INVALC,INVALC,0x6A51,INVALC,
	INVALC,INVALC,0x6A5B,INVALC,0x6A5D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x486F,INVALC,INVALC,0x6A59,INVALC,
	0x6A5E,0x6A60,INVALC,INVALC,0x3853,0x6A54,INVALC,0x3041,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A5F,
	INVALC,0x3A5B,0x4E76,0x6A61,0x6A62,0x4175,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E22,
	INVALC,INVALC,INVALC,INVALC,0x6A63,0x4D35,INVALC,INVALC,
	0x6A64,0x6A65,INVALC,INVALC,0x4A64,0x6A66,INVALC,0x3A40,
	INVALC,0x4E23,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6A6B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6A6C,0x3E58,0x6A6A,INVALC,INVALC,INVALC,
	0x4D67,0x6A67,INVALC,INVALC,0x6A69,0x403D,0x3F7E,INVALC,
	/* Block 138, Array index 0x4600 */
	INVALC,INVALC,0x6A68,INVALC,0x6A6D,INVALC,INVALC,0x4A23,
	INVALC,INVALC,0x6A6F,INVALC,0x6A6E,INVALC,INVALC,INVALC,
	0x336C,INVALC,0x4B2B,0x6A70,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6A7C,0x6A72,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6A73,INVALC,INVALC,
	INVALC,INVALC,0x6A74,0x6A75,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A79,INVALC,
	0x6A7A,INVALC,INVALC,0x6A78,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A76,INVALC,0x6A71,0x6A77,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6A7B,0x7037,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3228,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A7E,0x365F,
	0x6A7D,INVALC,INVALC,INVALC,0x6B22,INVALC,0x6B21,INVALC,
	INVALC,INVALC,0x6B24,INVALC,INVALC,0x6B23,INVALC,0x6B25,
	INVALC,INVALC,0x3D31,INVALC,0x6B26,INVALC,INVALC,0x6B27,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B28,0x403E,
	INVALC,0x4D57,INVALC,0x6B29,INVALC,INVALC,0x4A24,0x4746,
	0x6B2A,INVALC,0x6B2B,0x382B,INVALC,INVALC,INVALC,0x352C,
	INVALC,INVALC,INVALC,0x6B2C,INVALC,INVALC,0x3B6B,0x4741,
	0x6B2D,INVALC,0x3350,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B2E,INVALC,INVALC,INVALC,INVALC,0x6B30,0x4D77,
	INVALC,0x6B2F,0x3F46,INVALC,0x6B31,INVALC,INVALC,0x6B32,
	INVALC,INVALC,0x6B33,0x3451,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6B34,INVALC,INVALC,0x6B35,INVALC,0x6B36,
	0x6B37,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3351,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6B38,INVALC,0x6B39,0x6B3A,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3272,INVALC,INVALC,0x3F28,0x6B3B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6B3C,INVALC,INVALC,INVALC,
	0x6B3D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 139, Array index 0x4700 */
	0x3840,INVALC,0x447B,0x6B3E,INVALC,INVALC,INVALC,INVALC,
	0x3757,INVALC,0x3F56,INVALC,0x6B41,INVALC,0x4624,INVALC,
	0x6B40,INVALC,INVALC,0x3731,INVALC,INVALC,0x6B3F,0x4277,
	0x352D,INVALC,INVALC,0x6B42,INVALC,0x6B43,INVALC,0x3E59,
	INVALC,INVALC,INVALC,0x376D,INVALC,0x6B44,INVALC,INVALC,
	INVALC,INVALC,0x4B2C,INVALC,INVALC,0x405F,INVALC,INVALC,
	INVALC,0x3576,INVALC,0x4C75,0x414A,INVALC,0x6B45,INVALC,
	INVALC,INVALC,0x3F47,0x4370,0x3E5A,INVALC,INVALC,INVALC,
	INVALC,0x6B46,INVALC,INVALC,INVALC,INVALC,0x6B49,INVALC,
	0x6B4A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A3E,0x4242,0x6B48,INVALC,0x3E5B,0x493E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6B47,INVALC,INVALC,0x3B6C,INVALC,
	0x3153,INVALC,0x6B4E,0x3758,INVALC,INVALC,0x3B6E,INVALC,
	INVALC,0x3B6D,INVALC,0x4F4D,0x6B4D,0x6B4C,0x4127,INVALC,
	0x354D,0x4F43,0x333A,0x3E5C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6B4B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6B50,INVALC,0x6B51,0x6B4F,INVALC,0x3858,
	INVALC,0x4D40,INVALC,INVALC,0x3B6F,0x4727,INVALC,INVALC,
	INVALC,0x6B54,INVALC,0x4040,INVALC,0x4342,INVALC,INVALC,
	0x4D36,INVALC,0x6B57,INVALC,INVALC,INVALC,0x386C,INVALC,
	0x403F,0x6B53,INVALC,0x6B58,0x386D,0x6B55,0x6B56,INVALC,
	0x6B52,INVALC,INVALC,INVALC,0x4062,0x4649,INVALC,INVALC,
	0x432F,INVALC,0x325D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4870,INVALC,INVALC,0x3543,INVALC,INVALC,0x4434,
	INVALC,INVALC,0x6B5B,INVALC,0x6B59,INVALC,INVALC,0x434C,
	INVALC,INVALC,INVALC,0x4041,0x3452,0x6B5A,INVALC,0x3F5B,
	INVALC,INVALC,0x4E4A,INVALC,INVALC,INVALC,0x4F40,INVALC,
	INVALC,INVALC,0x6B5C,0x6B67,0x4435,INVALC,0x6B66,INVALC,
	0x6B63,0x6B6B,0x6B64,INVALC,0x6B60,INVALC,0x447C,0x6B5F,
	INVALC,INVALC,INVALC,0x6B5D,INVALC,0x4D21,0x3B70,INVALC,
	INVALC,0x6B61,INVALC,0x6B5E,INVALC,INVALC,INVALC,0x6B65,
	0x3D74,INVALC,0x3841,INVALC,INVALC,INVALC,0x427A,INVALC,
	/* Block 140, Array index 0x4800 */
	0x4B45,0x315A,0x3062,INVALC,0x4625,INVALC,INVALC,0x6B69,
	INVALC,INVALC,INVALC,INVALC,0x6B68,INVALC,0x4666,INVALC,
	0x6B6D,INVALC,INVALC,INVALC,0x6B62,INVALC,0x6B6C,0x6B6E,
	INVALC,0x382C,0x6B6A,0x3956,INVALC,0x3C55,INVALC,INVALC,
	0x6B6F,0x4D58,INVALC,INVALC,INVALC,INVALC,0x6B72,INVALC,
	0x6B75,INVALC,INVALC,0x6B73,0x4935,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6B70,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3660,INVALC,INVALC,INVALC,INVALC,0x6B74,INVALC,
	INVALC,0x6B76,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B7A,INVALC,INVALC,0x6B77,INVALC,0x6B79,0x6B78,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B7B,INVALC,
	0x3C31,INVALC,0x6B7D,0x6B7C,0x4968,INVALC,INVALC,0x6C21,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3759,INVALC,
	INVALC,INVALC,INVALC,0x6B7E,0x6C22,INVALC,INVALC,0x6C23,
	0x3544,0x6641,0x3E79,INVALC,0x6C24,INVALC,INVALC,0x386E,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6C25,INVALC,INVALC,
	0x6C26,INVALC,INVALC,0x3B3E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A4E,INVALC,0x6C27,INVALC,0x6C28,INVALC,
	0x3D32,INVALC,0x6C29,0x6C2A,INVALC,INVALC,0x6C2B,INVALC,
	INVALC,0x6C2C,0x6C2D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 141, Array index 0x4900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x432B,
	INVALC,INVALC,0x6C2E,INVALC,INVALC,INVALC,INVALC,0x6C30,
	INVALC,0x6C2F,INVALC,INVALC,INVALC,INVALC,0x4626,INVALC,
	0x6C31,INVALC,0x4B2D,INVALC,0x6C32,INVALC,0x6C33,INVALC,
	0x6C34,INVALC,INVALC,INVALC,INVALC,0x6C35,INVALC,INVALC,
	INVALC,INVALC,0x465A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3E5D,0x6C36,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x396B,0x502E,0x6C37,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6C38,0x493F,0x6C39,INVALC,0x6C41,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6C3A,INVALC,INVALC,0x6C3C,INVALC,INVALC,
	INVALC,0x6C3B,0x6C3D,INVALC,0x4B46,0x6C3E,0x6C3F,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6C40,INVALC,INVALC,INVALC,
	0x6C42,INVALC,INVALC,INVALC,INVALC,0x332D,0x4467,INVALC,
	0x4969,0x3A62,0x3957,INVALC,INVALC,INVALC,INVALC,0x494F,
	0x325F,0x484E,0x6C45,0x3453,0x4055,0x6C44,0x6C49,0x4379,
	0x4C63,INVALC,0x6C47,0x6C48,0x352E,INVALC,0x6C4A,0x4763,
	0x425F,INVALC,INVALC,0x4871,0x453D,0x6C46,INVALC,0x4B47,
	0x326C,0x6C4C,0x4F28,0x4442,0x4F45,INVALC,INVALC,0x3B71,
	0x6C4B,INVALC,0x4231,INVALC,INVALC,0x6C5C,0x4128,INVALC,
	INVALC,0x4678,INVALC,0x4950,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6C4F,0x3B3F,0x3B72,INVALC,0x3E5E,INVALC,
	0x4765,INVALC,0x382D,0x6C4E,0x6C4D,INVALC,0x496A,INVALC,
	INVALC,INVALC,0x3C41,INVALC,INVALC,0x4552,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6C51,0x6C52,0x3958,0x6C50,INVALC,INVALC,
	/* Block 142, Array index 0x4A00 */
	INVALC,INVALC,INVALC,INVALC,0x6C53,0x6C54,INVALC,0x6C56,
	0x4223,INVALC,0x6C55,0x3466,INVALC,0x6C58,INVALC,0x6C57,
	0x6C59,INVALC,INVALC,0x6C5B,0x6C5D,INVALC,0x6C5E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4056,INVALC,0x3C4F,0x6C5F,
	INVALC,INVALC,INVALC,0x3352,INVALC,0x6C60,INVALC,INVALC,
	0x4176,0x6C61,INVALC,0x6C62,0x496B,INVALC,INVALC,0x352F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6C63,INVALC,INVALC,INVALC,0x4436,INVALC,INVALC,
	INVALC,INVALC,0x315B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6C64,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3C71,INVALC,INVALC,INVALC,INVALC,
	0x3F76,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x422D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6C67,INVALC,INVALC,INVALC,0x6C66,INVALC,
	INVALC,INVALC,0x6C65,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6C6D,0x6C6B,INVALC,INVALC,0x6C68,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6C6A,INVALC,
	INVALC,INVALC,0x6C69,0x6C6C,INVALC,0x3577,INVALC,0x6C70,
	INVALC,0x4057,INVALC,0x6C71,INVALC,INVALC,INVALC,INVALC,
	0x3859,INVALC,0x6C6E,0x6C6F,INVALC,INVALC,INVALC,0x4F29,
	INVALC,INVALC,INVALC,0x4437,INVALC,0x4129,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6C72,INVALC,INVALC,0x6C75,
	/* Block 143, Array index 0x4B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6C73,0x6C74,0x4D59,INVALC,INVALC,INVALC,INVALC,0x4627,
	0x6C78,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6C76,0x6C77,0x6C79,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6D29,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6C7C,INVALC,INVALC,INVALC,0x6C7D,0x6C7B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6C7A,INVALC,0x447D,INVALC,INVALC,0x6D21,
	0x6D25,0x6D22,0x6C7E,INVALC,0x6D23,INVALC,INVALC,INVALC,
	0x6D24,INVALC,INVALC,INVALC,INVALC,0x6D2B,INVALC,INVALC,
	INVALC,0x6D26,INVALC,INVALC,INVALC,INVALC,INVALC,0x4058,
	0x6D28,INVALC,INVALC,0x6D2A,0x6D27,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6D2D,INVALC,0x3D33,INVALC,0x6D2C,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6D2E,INVALC,INVALC,INVALC,
	INVALC,0x6D2F,INVALC,INVALC,0x6D32,0x6D31,INVALC,0x6D30,
	INVALC,INVALC,0x6D34,0x6D33,INVALC,0x4C76,INVALC,INVALC,
	INVALC,0x6D36,INVALC,0x6D35,0x6D37,INVALC,INVALC,INVALC,
	INVALC,0x6D38,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6D3A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6D39,0x3F48,0x6D3B,INVALC,INVALC,0x366D,
	0x6D3C,0x6D3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D3F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6D40,0x6D3D,INVALC,
	0x6D41,INVALC,0x3C56,0x6D42,0x3530,0x3733,INVALC,INVALC,
	INVALC,INVALC,0x382E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6D43,INVALC,INVALC,INVALC,0x4670,
	INVALC,INVALC,0x453E,0x6D44,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6D47,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C34,INVALC,INVALC,0x6D46,0x6D45,0x375A,0x6D48,INVALC,
	/* Block 144, Array index 0x4C00 */
	INVALC,INVALC,INVALC,0x3353,INVALC,0x6D4A,INVALC,INVALC,
	INVALC,0x3A5C,0x6D49,INVALC,0x6D52,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6D4C,0x6D4E,0x4A65,0x6D4B,INVALC,INVALC,
	INVALC,0x6D4D,INVALC,0x6D51,0x6D4F,0x3531,INVALC,0x6D50,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D53,INVALC,
	INVALC,0x475A,0x4E58,INVALC,INVALC,INVALC,INVALC,0x3D34,
	INVALC,INVALC,INVALC,0x6D54,INVALC,INVALC,INVALC,INVALC,
	0x4D22,0x6D56,INVALC,0x6D55,INVALC,INVALC,0x6D59,0x4D41,
	INVALC,INVALC,0x6D58,INVALC,0x336D,0x6D57,0x6D5C,INVALC,
	INVALC,0x6D5B,INVALC,INVALC,0x6D5A,0x4532,0x6D5D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D5E,
	INVALC,INVALC,INVALC,INVALC,0x6D5F,INVALC,INVALC,0x396C,
	INVALC,0x3725,0x6D60,0x6D61,0x6D62,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3F49,0x6D63,INVALC,0x3C2D,0x6D64,
	INVALC,INVALC,INVALC,0x6D65,INVALC,INVALC,INVALC,0x5221,
	0x517E,INVALC,INVALC,INVALC,INVALC,0x6D66,0x6570,0x6D67,
	0x4324,0x3F2B,0x4740,INVALC,INVALC,INVALC,INVALC,0x6D68,
	INVALC,INVALC,0x4A55,0x4454,0x397E,INVALC,INVALC,0x4329,
	INVALC,INVALC,0x312A,INVALC,0x4B78,0x3F57,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x375E,INVALC,
	INVALC,0x3661,INVALC,INVALC,0x4A56,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6D69,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6D6B,INVALC,INVALC,0x6D6A,0x3260,INVALC,
	INVALC,0x4676,0x6D6C,0x4777,INVALC,0x4533,INVALC,0x6D6D,
	0x3D52,INVALC,INVALC,INVALC,0x6D6F,INVALC,INVALC,0x4C42,
	0x6D7E,0x6D71,0x6D72,INVALC,INVALC,0x4449,INVALC,INVALC,
	/* Block 145, Array index 0x4D00 */
	0x4260,0x4177,INVALC,0x4628,INVALC,0x6D70,0x3555,INVALC,
	INVALC,INVALC,INVALC,0x6D79,INVALC,0x6D76,0x6E25,0x4629,
	0x4360,0x6D73,INVALC,0x447E,0x4553,0x6D74,0x6D78,0x3F60,
	INVALC,0x4767,0x444C,INVALC,INVALC,0x4042,0x6D77,0x422E,
	0x4224,0x6D75,0x3029,0x4F22,INVALC,INVALC,INVALC,0x6D7A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4261,INVALC,
	INVALC,0x3D35,0x3F4A,INVALC,INVALC,0x6D7C,0x6D7B,INVALC,
	0x306F,0x6D7D,INVALC,INVALC,0x492F,INVALC,0x6E27,INVALC,
	INVALC,0x465B,0x3F6B,INVALC,INVALC,0x4359,INVALC,0x3678,
	INVALC,0x6E26,0x4D37,0x313F,INVALC,0x4A57,0x3261,0x6E21,
	0x6E22,0x6E23,0x6E24,0x463B,0x4323,0x3063,0x6E28,INVALC,
	0x6E29,0x7423,INVALC,INVALC,0x423D,INVALC,0x6E2A,INVALC,
	0x3173,0x414C,INVALC,0x382F,INVALC,0x4D5A,INVALC,INVALC,
	0x6E2B,0x452C,INVALC,INVALC,INVALC,0x4178,0x3C57,0x6E2C,
	INVALC,INVALC,0x6E2F,INVALC,INVALC,0x3D65,0x6E2D,0x412B,
	0x412A,INVALC,0x3064,INVALC,0x4E4B,0x6E31,INVALC,0x4872,
	0x6E33,0x6E32,0x6E30,0x6364,0x3454,INVALC,INVALC,0x6D6E,
	INVALC,0x6E35,0x6E34,INVALC,INVALC,INVALC,INVALC,0x6E36,
	INVALC,0x4D38,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4661,INVALC,INVALC,0x4B2E,INVALC,
	0x6E37,INVALC,0x3C59,INVALC,INVALC,INVALC,INVALC,0x6E38,
	INVALC,0x6E39,INVALC,INVALC,INVALC,0x6E3A,INVALC,INVALC,
	0x4521,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x306A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3959,INVALC,INVALC,INVALC,0x4F3A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6E3E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3734,0x6E3B,INVALC,0x6E3C,INVALC,INVALC,INVALC,
	0x4974,INVALC,INVALC,INVALC,INVALC,0x3354,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4D39,INVALC,0x363F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4554,INVALC,INVALC,
	/* Block 146, Array index 0x4E00 */
	INVALC,INVALC,0x6E3F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6E40,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6E41,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4522,INVALC,INVALC,
	0x6E43,INVALC,0x6E42,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4653,0x6E44,0x3D36,0x3C60,0x475B,0x4371,INVALC,
	INVALC,INVALC,0x3C72,INVALC,0x3F6C,INVALC,0x6E45,INVALC,
	0x6E46,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F5D,0x6E47,INVALC,0x6E48,INVALC,INVALC,
	INVALC,0x6E49,0x4D6F,INVALC,0x3D37,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6E4B,0x6E4A,INVALC,0x395A,INVALC,0x3973,
	0x3B40,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6E4E,INVALC,INVALC,INVALC,INVALC,0x3D66,
	INVALC,0x6E4D,INVALC,0x6E4C,INVALC,0x4269,INVALC,INVALC,
	0x386F,INVALC,0x4043,INVALC,INVALC,INVALC,INVALC,0x4830,
	INVALC,INVALC,INVALC,INVALC,0x3D39,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6E4F,INVALC,0x3E5F,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6E52,0x6E50,INVALC,INVALC,INVALC,0x6E51,
	INVALC,INVALC,INVALC,INVALC,0x6E54,0x6E53,INVALC,INVALC,
	0x3E7A,INVALC,0x6E55,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6E56,0x6E57,INVALC,INVALC,INVALC,INVALC,0x4850,0x3A53,
	0x3C61,0x6E58,INVALC,0x6E59,0x4E24,0x3D45,0x4C6E,0x4E4C,
	0x6E5A,0x3662,INVALC,INVALC,INVALC,INVALC,0x6E5B,INVALC,
	0x4523,INVALC,INVALC,0x6E5E,0x3378,0x3F4B,INVALC,0x6E5C,
	INVALC,0x6E5D,INVALC,0x4460,INVALC,INVALC,0x4B55,0x367C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6E60,0x6E61,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6E5F,INVALC,INVALC,0x6E63,
	/* Block 147, Array index 0x4F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x465F,0x3343,INVALC,
	INVALC,0x6E67,INVALC,INVALC,0x6E64,0x6E66,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E62,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F4F,INVALC,INVALC,0x6E65,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4E6B,INVALC,INVALC,0x385A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E6F,
	INVALC,INVALC,INVALC,INVALC,0x4534,0x6E6A,INVALC,INVALC,
	0x6E6D,0x6E6B,INVALC,0x6E70,INVALC,INVALC,INVALC,INVALC,
	0x6E71,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E69,
	INVALC,INVALC,0x6E76,0x3174,INVALC,INVALC,0x6E68,INVALC,
	INVALC,INVALC,0x482D,INVALC,0x6E6C,INVALC,0x3E60,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x395B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B48,INVALC,
	0x3664,INVALC,INVALC,0x3D46,INVALC,0x463C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x412D,INVALC,0x6E74,INVALC,0x6E6E,0x6E73,INVALC,
	0x4C43,INVALC,0x4438,0x6E75,0x6E72,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x412C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E79,
	INVALC,0x6E78,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E77,
	INVALC,INVALC,0x4B2F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D7B,INVALC,INVALC,INVALC,
	INVALC,0x6E7A,0x4A5F,INVALC,INVALC,0x3154,INVALC,INVALC,
	INVALC,INVALC,0x4946,0x4372,INVALC,INVALC,INVALC,INVALC,
	0x3578,INVALC,0x6E7C,INVALC,0x395D,INVALC,INVALC,INVALC,
	/* Block 148, Array index 0x5000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B2C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6E7B,
	0x3F6D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F6E,0x6F21,0x6F23,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E7B,INVALC,0x6F22,0x6F24,INVALC,INVALC,0x3653,INVALC,
	0x4945,INVALC,INVALC,0x3C62,0x4F23,INVALC,0x6E7E,0x3A78,
	INVALC,INVALC,0x4F3F,INVALC,INVALC,0x6F26,INVALC,INVALC,
	INVALC,INVALC,0x6F25,0x6F27,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6E7D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4669,INVALC,0x4555,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4457,INVALC,0x6F2C,INVALC,
	INVALC,INVALC,INVALC,0x4343,0x6F28,INVALC,INVALC,INVALC,
	0x6F29,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x372D,INVALC,0x6F2B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3830,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6F2A,INVALC,0x3E61,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3379,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6F30,INVALC,0x3A3F,0x4179,
	INVALC,INVALC,0x444A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x333B,
	INVALC,INVALC,INVALC,INVALC,0x6F2E,0x6F2F,0x4443,INVALC,
	0x6F2D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F31,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6F37,INVALC,INVALC,INVALC,INVALC,
	0x6F3A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F39,0x452D,INVALC,INVALC,INVALC,INVALC,0x6F32,0x6F33,
	0x6F36,INVALC,INVALC,INVALC,INVALC,0x6F38,INVALC,INVALC,
	INVALC,0x3640,INVALC,INVALC,0x6F3B,0x6F35,INVALC,INVALC,
	0x6F34,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 149, Array index 0x5100 */
	INVALC,INVALC,INVALC,0x6F3F,INVALC,INVALC,INVALC,0x6F40,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F41,INVALC,INVALC,0x6F3E,0x6F3D,INVALC,INVALC,INVALC,
	0x3E62,0x462A,0x6F3C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F45,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6F43,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6F44,0x6F42,INVALC,
	0x4278,INVALC,0x6F46,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F47,INVALC,INVALC,0x6F49,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3455,0x6F48,0x4C7A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6F54,0x6F4A,INVALC,INVALC,0x6F4D,INVALC,
	0x6F4B,INVALC,0x6F4C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6F4E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F50,INVALC,INVALC,INVALC,INVALC,0x6F51,INVALC,0x6F52,
	INVALC,INVALC,INVALC,INVALC,0x6F55,0x6F53,0x6F56,0x6F58,
	INVALC,0x6F57,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 150, Array index 0x5200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4439,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C67,INVALC,0x6F59,0x412E,INVALC,INVALC,INVALC,0x6F5A,
	INVALC,0x4A44,0x6F5B,0x332B,INVALC,INVALC,INVALC,0x313C,
	INVALC,0x3457,INVALC,0x3456,0x6F5C,INVALC,0x6F5D,INVALC,
	0x6F5E,0x6F5F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F60,INVALC,0x3458,0x3355,0x395E,0x4836,INVALC,0x6F62,
	0x6F61,INVALC,INVALC,INVALC,INVALC,0x6F63,INVALC,INVALC,
	INVALC,INVALC,0x315C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6F66,INVALC,0x6F65,0x6F64,INVALC,0x6F67,INVALC,
	INVALC,INVALC,INVALC,0x6F6A,INVALC,INVALC,INVALC,0x3047,
	INVALC,INVALC,0x6F68,INVALC,0x6F6C,0x6F6B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6F6E,0x6F6D,0x6F6F,INVALC,
	0x462E,INVALC,INVALC,INVALC,0x6F70,INVALC,INVALC,INVALC,
	INVALC,0x6F71,0x6F73,INVALC,INVALC,0x6F72,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 151, Array index 0x5300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x496C,INVALC,INVALC,INVALC,
	INVALC,0x6F74,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6F75,INVALC,0x3A65,INVALC,INVALC,INVALC,0x6F76,0x6F77,
	INVALC,INVALC,0x4B49,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x414B,INVALC,INVALC,INVALC,0x3024,
	0x424B,INVALC,0x6F78,INVALC,0x496D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6F7B,0x6F79,0x395F,INVALC,0x6F7A,
	0x3842,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A45,0x6F7D,0x7021,0x6F7E,0x7022,
	INVALC,INVALC,0x3121,0x3F58,0x3D7C,0x3459,0x7023,INVALC,
	INVALC,INVALC,0x4766,INVALC,0x7025,INVALC,INVALC,INVALC,
	0x3122,INVALC,0x7024,0x4444,INVALC,0x4E4D,0x462B,0x6F7C,
	0x4E26,INVALC,0x3831,INVALC,INVALC,0x4D5B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3679,0x4E34,INVALC,
	0x3728,INVALC,0x4262,0x6721,INVALC,0x7026,0x332C,0x3F6F,
	INVALC,INVALC,INVALC,INVALC,0x3356,0x7028,INVALC,0x7029,
	0x7027,0x3764,INVALC,0x3A5D,0x3E63,INVALC,INVALC,INVALC,
	0x3123,INVALC,INVALC,0x4E59,INVALC,INVALC,INVALC,0x702B,
	0x6E2E,INVALC,0x702A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x702E,0x702C,0x702D,INVALC,0x702F,INVALC,0x7030,0x4E6C,
	0x7031,0x7032,INVALC,0x4049,0x483B,INVALC,INVALC,INVALC,
	0x3F7D,0x3467,INVALC,INVALC,0x4D3A,0x326D,0x3D38,0x385B,
	INVALC,0x7035,INVALC,0x7034,0x3B73,0x7036,0x7033,INVALC,
	INVALC,0x3B28,INVALC,INVALC,INVALC,0x703A,0x6A2D,INVALC,
	INVALC,0x5256,INVALC,0x3F77,0x7038,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E25,0x4671,INVALC,INVALC,INVALC,INVALC,
	0x312B,INVALC,0x4063,0x3C36,INVALC,INVALC,INVALC,INVALC,
	0x4A37,INVALC,0x3140,INVALC,INVALC,INVALC,0x4E6D,0x4D6B,
	INVALC,0x703B,INVALC,0x4545,INVALC,INVALC,INVALC,INVALC,
	/* Block 152, Array index 0x5400 */
	0x3C7B,INVALC,INVALC,INVALC,0x703C,INVALC,0x703D,0x3F4C,
	0x703E,INVALC,0x4E6E,INVALC,INVALC,0x7039,0x7040,0x7042,
	INVALC,0x7041,INVALC,0x703F,INVALC,INVALC,0x7043,INVALC,
	INVALC,0x7044,INVALC,INVALC,0x417A,INVALC,0x3262,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7045,INVALC,INVALC,0x4C38,
	INVALC,INVALC,0x7046,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7047,INVALC,0x4F2A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5B31,0x7048,INVALC,INVALC,INVALC,0x7049,0x704A,INVALC,
	INVALC,INVALC,0x704E,INVALC,0x704B,INVALC,0x704C,INVALC,
	0x704D,0x704F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4044,INVALC,INVALC,INVALC,0x4C77,INVALC,
	INVALC,0x4045,INVALC,INVALC,0x7050,INVALC,0x4873,INVALC,
	0x7051,0x7353,0x4C4C,INVALC,0x7052,INVALC,0x7053,INVALC,
	0x7054,0x3357,INVALC,0x7056,INVALC,0x3F59,INVALC,INVALC,
	INVALC,0x7057,INVALC,INVALC,0x3724,INVALC,INVALC,INVALC,
	INVALC,0x7058,0x705C,INVALC,0x705A,INVALC,INVALC,INVALC,
	INVALC,0x705B,INVALC,INVALC,0x3373,0x7059,0x705D,INVALC,
	INVALC,INVALC,INVALC,0x705E,INVALC,0x3048,INVALC,0x705F,
	0x7060,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E64,INVALC,INVALC,INVALC,0x7061,INVALC,INVALC,INVALC,
	0x3547,INVALC,INVALC,0x7064,INVALC,INVALC,0x7063,INVALC,
	0x7062,INVALC,INVALC,0x6B71,INVALC,0x4A5C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7065,0x7066,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7067,INVALC,INVALC,0x7068,INVALC,
	0x7069,INVALC,INVALC,0x706A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x345A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x706B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x706C,0x4723,INVALC,
	INVALC,INVALC,0x706E,0x323B,INVALC,0x7071,0x7070,INVALC,
	INVALC,INVALC,INVALC,0x3124,INVALC,INVALC,INVALC,0x3641,
	/* Block 153, Array index 0x5500 */
	INVALC,0x4A47,0x443A,0x3A22,INVALC,0x3960,0x3D67,INVALC,
	0x3F5C,INVALC,INVALC,INVALC,0x7073,INVALC,INVALC,0x7072,
	0x4D42,0x3468,0x4852,0x465C,INVALC,INVALC,INVALC,0x3F7C,
	0x4E4E,INVALC,0x375B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7076,INVALC,INVALC,0x7075,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4B4B,0x462C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3150,INVALC,INVALC,0x7077,
	0x7074,INVALC,INVALC,0x4951,0x4D6A,0x7078,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7079,INVALC,
	INVALC,INVALC,INVALC,0x707B,0x426A,0x335B,0x335C,0x707A,
	INVALC,INVALC,INVALC,INVALC,0x3469,0x3832,INVALC,INVALC,
	0x346A,INVALC,INVALC,0x453F,INVALC,INVALC,0x4E60,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x385C,
	INVALC,INVALC,INVALC,0x707C,INVALC,INVALC,INVALC,0x707D,
	0x707E,0x7121,INVALC,0x7123,0x7122,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4977,INVALC,0x7124,INVALC,INVALC,INVALC,INVALC,0x7125,
	INVALC,0x7126,INVALC,INVALC,INVALC,INVALC,0x7127,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7129,0x7128,INVALC,0x712A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4874,0x664C,INVALC,INVALC,0x3F29,
	INVALC,INVALC,0x3532,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x712B,INVALC,0x712C,INVALC,0x522C,0x5D3B,0x4853,
	INVALC,INVALC,0x307B,INVALC,0x303B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B74,0x4B30,0x3E7E,INVALC,
	/* Block 154, Array index 0x5600 */
	INVALC,INVALC,INVALC,0x712D,INVALC,0x4C5F,INVALC,INVALC,
	INVALC,0x712E,0x4D5C,INVALC,0x3142,INVALC,INVALC,INVALC,
	0x3B41,INVALC,0x712F,0x326E,0x7130,INVALC,INVALC,INVALC,
	0x7131,INVALC,INVALC,INVALC,INVALC,0x7133,0x7134,INVALC,
	0x7136,0x7132,INVALC,INVALC,0x7135,INVALC,INVALC,INVALC,
	0x345B,INVALC,INVALC,INVALC,0x7137,INVALC,0x7138,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7139,0x713A,INVALC,
	INVALC,INVALC,0x713B,INVALC,INVALC,0x713D,INVALC,INVALC,
	INVALC,0x713C,INVALC,0x713F,0x7142,INVALC,INVALC,INVALC,
	0x713E,0x7140,0x7141,INVALC,INVALC,0x7143,INVALC,0x3642,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C73,0x7144,
	0x7145,0x3961,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7146,INVALC,INVALC,
	0x333E,INVALC,INVALC,INVALC,0x474F,0x7147,0x7148,INVALC,
	INVALC,INVALC,INVALC,0x435A,0x466B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7149,INVALC,INVALC,INVALC,
	INVALC,0x477D,INVALC,INVALC,0x424C,0x3158,0x366E,INVALC,
	0x366F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4373,0x714E,0x3670,INVALC,INVALC,0x326F,INVALC,INVALC,
	0x714D,INVALC,INVALC,0x714B,INVALC,0x714C,INVALC,0x714A,
	INVALC,INVALC,0x7158,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x714F,0x7150,INVALC,
	INVALC,0x7151,0x7152,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7154,INVALC,INVALC,0x7153,INVALC,INVALC,INVALC,0x3D59,
	/* Block 155, Array index 0x5700 */
	INVALC,0x7155,INVALC,INVALC,INVALC,0x7157,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3533,0x7156,
	INVALC,INVALC,0x417B,0x3833,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7159,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x424D,INVALC,INVALC,0x715A,INVALC,INVALC,INVALC,INVALC,
	0x462D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x715B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7160,INVALC,
	0x715E,INVALC,0x715D,0x715F,INVALC,0x715C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7162,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7161,INVALC,0x7164,
	INVALC,INVALC,0x3643,0x7163,INVALC,INVALC,INVALC,0x7165,
	INVALC,INVALC,0x7166,INVALC,0x7168,0x7167,INVALC,INVALC,
	INVALC,0x7169,0x716B,0x716A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x397C,INVALC,INVALC,INVALC,INVALC,0x716C,INVALC,INVALC,
	0x716D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x333C,INVALC,INVALC,INVALC,0x716E,INVALC,INVALC,INVALC,
	0x716F,INVALC,INVALC,INVALC,0x3F71,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7170,
	INVALC,0x7171,INVALC,0x7172,0x7173,INVALC,INVALC,INVALC,
	0x3962,INVALC,INVALC,INVALC,INVALC,INVALC,0x7174,0x7175,
	INVALC,INVALC,0x7176,0x7177,INVALC,INVALC,0x7178,INVALC,
	INVALC,INVALC,0x4831,0x717A,INVALC,0x4926,0x717B,0x7179,
	INVALC,0x717D,INVALC,INVALC,0x717C,INVALC,INVALC,0x717E,
	INVALC,INVALC,INVALC,0x7221,INVALC,INVALC,INVALC,INVALC,
	/* Block 156, Array index 0x5800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7222,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7223,INVALC,0x7224,INVALC,INVALC,INVALC,INVALC,0x7225,
	INVALC,INVALC,0x7226,0x7227,INVALC,0x7228,INVALC,0x7229,
	0x722A,0x722B,0x722C,INVALC,INVALC,INVALC,0x722D,0x722E,
	INVALC,0x5D35,0x722F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6478,0x3534,INVALC,INVALC,INVALC,
	INVALC,0x3321,0x3A32,0x7231,0x7230,0x4C25,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7233,0x7234,0x7232,
	INVALC,0x7235,INVALC,INVALC,0x4B62,INVALC,INVALC,INVALC,
	0x7236,INVALC,0x357B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F25,
	INVALC,INVALC,INVALC,INVALC,0x7237,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7239,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x303E,INVALC,
	INVALC,0x723A,0x4A2B,0x7238,INVALC,INVALC,0x723B,0x723C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x723D,
	0x723E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x723F,INVALC,0x4B6E,0x3B2D,INVALC,0x3A7A,0x412F,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7240,INVALC,INVALC,INVALC,
	INVALC,0x7243,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7241,INVALC,INVALC,INVALC,INVALC,INVALC,0x7244,INVALC,
	INVALC,0x3871,0x7242,INVALC,INVALC,INVALC,INVALC,0x7245,
	INVALC,0x7246,0x7247,INVALC,0x724B,INVALC,0x3B2A,INVALC,
	INVALC,INVALC,INVALC,0x4264,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x724C,0x7249,0x7248,0x724A,INVALC,INVALC,INVALC,
	0x375F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7250,0x724F,0x724E,INVALC,INVALC,0x3033,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 157, Array index 0x5900 */
	INVALC,INVALC,INVALC,INVALC,0x725A,INVALC,0x7256,INVALC,
	0x7257,0x7253,0x7259,INVALC,0x7255,0x3362,INVALC,INVALC,
	0x4F4C,INVALC,0x7258,0x7254,0x7252,0x7251,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x725C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x725F,INVALC,INVALC,0x725E,0x725D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4949,0x725B,0x3073,
	0x7260,INVALC,0x7262,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x336F,0x724D,0x3137,INVALC,INVALC,0x7264,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7263,0x7261,
	0x432D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4B70,INVALC,INVALC,INVALC,INVALC,0x4E5A,
	INVALC,INVALC,0x7265,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7266,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7267,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7268,INVALC,
	0x7269,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x443B,INVALC,0x726A,
	INVALC,0x4837,INVALC,0x726F,0x726B,INVALC,INVALC,INVALC,
	0x726C,INVALC,INVALC,0x4B31,0x4C44,INVALC,0x4650,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 158, Array index 0x5A00 */
	INVALC,INVALC,INVALC,0x7270,INVALC,INVALC,0x7271,0x463E,
	0x726E,0x726D,INVALC,INVALC,INVALC,INVALC,0x322A,INVALC,
	INVALC,INVALC,0x7279,INVALC,INVALC,0x7278,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3175,INVALC,INVALC,INVALC,0x7276,
	INVALC,INVALC,INVALC,0x7275,INVALC,INVALC,0x7273,INVALC,
	0x337B,INVALC,0x7272,0x3C32,0x3229,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3963,INVALC,INVALC,0x727C,0x727B,
	INVALC,0x727A,INVALC,INVALC,0x7277,INVALC,0x727D,INVALC,
	0x727E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7325,0x7324,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7326,INVALC,INVALC,0x312D,0x7321,0x7322,INVALC,
	0x3974,0x4C39,INVALC,INVALC,0x7323,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4B32,INVALC,INVALC,0x732B,
	INVALC,INVALC,0x7327,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x732C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7329,
	INVALC,0x7328,INVALC,INVALC,INVALC,INVALC,INVALC,0x375C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x732D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x732E,INVALC,INVALC,INVALC,
	INVALC,0x732F,INVALC,0x732A,INVALC,INVALC,INVALC,0x7274,
	INVALC,INVALC,0x7330,INVALC,0x4461,INVALC,INVALC,INVALC,
	0x7334,INVALC,0x7335,0x7333,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7332,0x7338,INVALC,0x7331,INVALC,0x7336,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7337,
	INVALC,INVALC,INVALC,0x733A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7339,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x733C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x733D,INVALC,0x733E,
	INVALC,INVALC,0x4F49,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x733B,0x426B,0x3A6D,INVALC,INVALC,0x733F,INVALC,INVALC,
	/* Block 159, Array index 0x5B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7340,0x7341,INVALC,INVALC,0x7342,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7343,INVALC,INVALC,
	0x3834,0x7344,INVALC,INVALC,INVALC,0x7345,INVALC,0x3C2F,
	INVALC,0x7346,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7347,INVALC,INVALC,0x7348,0x7349,INVALC,INVALC,INVALC,
	INVALC,0x734C,0x734A,0x4F3C,INVALC,0x734B,INVALC,0x4E6F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x734D,INVALC,0x4E5B,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x734E,0x477E,INVALC,
	INVALC,0x734F,0x7351,INVALC,INVALC,0x7352,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7350,0x396D,0x4C4D,0x4B63,0x5677,INVALC,0x5D60,0x4B7B,
	INVALC,INVALC,INVALC,INVALC,0x322B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7354,0x3550,0x7355,0x7356,
	0x7357,INVALC,0x3975,INVALC,0x7358,INVALC,INVALC,INVALC,
	0x6054,0x4C5B,INVALC,0x4263,0x7359,0x735B,0x735A,INVALC,
	0x735C,INVALC,INVALC,INVALC,INVALC,0x735D,INVALC,INVALC,
	0x735E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x735F,
	INVALC,INVALC,INVALC,INVALC,0x7360,INVALC,0x7361,0x7362,
	INVALC,0x7363,INVALC,0x7364,0x7365,0x7366,INVALC,INVALC,
	/* Block 160, Array index 0x5C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7367,
	0x7368,INVALC,INVALC,INVALC,INVALC,INVALC,0x4524,INVALC,
	INVALC,INVALC,INVALC,0x385D,INVALC,0x736A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x414D,0x736B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x736C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4921,INVALC,INVALC,0x736D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x736E,0x6337,INVALC,INVALC,0x6C5A,0x706D,
	INVALC,INVALC,0x736F,INVALC,0x7370,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7372,
	0x7373,0x7374,0x4E70,0x7371,INVALC,INVALC,0x7375,0x7376,
	INVALC,INVALC,0x7378,INVALC,0x7377,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x737A,INVALC,INVALC,INVALC,0x737B,0x7379,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E36,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x737C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x737D,0x6354,INVALC,INVALC,
	0x737E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 256, Array index 0x5D00 */
	INVALC,0x212A,INVALC,0x2174,0x2170,0x2173,0x2175,INVALC,
	0x214A,0x214B,0x2176,0x215C,0x2124,INVALC,0x2125,0x213F,
	0x2330,0x2331,0x2332,0x2333,0x2334,0x2335,0x2336,0x2337,
	0x2338,0x2339,0x2127,0x2128,0x2163,0x2161,0x2164,0x2129,
	0x2177,0x2341,0x2342,0x2343,0x2344,0x2345,0x2346,0x2347,
	0x2348,0x2349,0x234A,0x234B,0x234C,0x234D,0x234E,0x234F,
	0x2350,0x2351,0x2352,0x2353,0x2354,0x2355,0x2356,0x2357,
	0x2358,0x2359,0x235A,0x214E,INVALC,0x214F,0x2130,0x2132,
	0x212E,0x2361,0x2362,0x2363,0x2364,0x2365,0x2366,0x2367,
	0x2368,0x2369,0x236A,0x236B,0x236C,0x236D,0x236E,0x236F,
	0x2370,0x2371,0x2372,0x2373,0x2374,0x2375,0x2376,0x2377,
	0x2378,0x2379,0x237A,0x2150,0x2143,0x2151,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2131,INVALC,0x216F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
};

#endif /* ICONV_FROM_UCS_CCS_JIS_X0208_1990 && !defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit UCS -> jis_x0208_1990 size-optimized table (25440 bytes).
 * ======================================================================
 */

#if defined (ICONV_FROM_UCS_CCS_JIS_X0208_1990) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
from_ucs_size_jis_x0208_1990[] =
{
	0x01E9, /* Ranges number */
	0x0E1F, /* Unranged codes number */
	0x1572, /* First unranged code index */
	/* Ranges list: first code, last Code, array index. */
	/* Array index: 0x0003 */ 0x0391, 0x03A9, 0x05BE,
	/* Array index: 0x0006 */ 0x03B1, 0x03C9, 0x05D7,
	/* Array index: 0x0009 */ 0x0410, 0x0451, 0x05F0,
	/* Array index: 0x000C */ 0x2015, 0x2021, 0x0632,
	/* Array index: 0x000F */ 0x2190, 0x2193, 0x063F,
	/* Array index: 0x0012 */ 0x2227, 0x222C, 0x0643,
	/* Array index: 0x0015 */ 0x2500, 0x2503, 0x0649,
	/* Array index: 0x0018 */ 0x2517, 0x251D, 0x064D,
	/* Array index: 0x001B */ 0x2523, 0x2525, 0x0654,
	/* Array index: 0x001E */ 0x3000, 0x3015, 0x0657,
	/* Array index: 0x0021 */ 0x3041, 0x3093, 0x066D,
	/* Array index: 0x0024 */ 0x309B, 0x30F6, 0x06C0,
	/* Array index: 0x0027 */ 0x30FB, 0x30FE, 0x071C,
	/* Array index: 0x002A */ 0x4E07, 0x4E19, 0x0720,
	/* Array index: 0x002D */ 0x4E38, 0x4E3C, 0x0733,
	/* Array index: 0x0030 */ 0x4E4B, 0x4E4F, 0x0738,
	/* Array index: 0x0033 */ 0x4E55, 0x4E5F, 0x073D,
	/* Array index: 0x0036 */ 0x4E85, 0x4EB0, 0x0748,
	/* Array index: 0x0039 */ 0x4EC0, 0x4ECF, 0x0774,
	/* Array index: 0x003C */ 0x4ED4, 0x4EE5, 0x0784,
	/* Array index: 0x003F */ 0x4F09, 0x4F11, 0x0796,
	/* Array index: 0x0042 */ 0x4F4D, 0x4F5E, 0x079F,
	/* Array index: 0x0045 */ 0x4FAB, 0x4FAF, 0x07B1,
	/* Array index: 0x0048 */ 0x4FC2, 0x4FC4, 0x07B6,
	/* Array index: 0x004B */ 0x4FD7, 0x4FE5, 0x07B9,
	/* Array index: 0x004E */ 0x5021, 0x502D, 0x07C8,
	/* Array index: 0x0051 */ 0x5047, 0x5049, 0x07D5,
	/* Array index: 0x0054 */ 0x5072, 0x5078, 0x07D8,
	/* Array index: 0x0057 */ 0x5098, 0x509A, 0x07DF,
	/* Array index: 0x005A */ 0x50B2, 0x50B7, 0x07E2,
	/* Array index: 0x005D */ 0x5100, 0x5104, 0x07E8,
	/* Array index: 0x0060 */ 0x5112, 0x511A, 0x07ED,
	/* Array index: 0x0063 */ 0x513A, 0x5154, 0x07F6,
	/* Array index: 0x0066 */ 0x5168, 0x516E, 0x0811,
	/* Array index: 0x0069 */ 0x5175, 0x5178, 0x0818,
	/* Array index: 0x006C */ 0x5189, 0x5199, 0x081C,
	/* Array index: 0x006F */ 0x51A2, 0x51B7, 0x082D,
	/* Array index: 0x0072 */ 0x51C4, 0x51C6, 0x0843,
	/* Array index: 0x0075 */ 0x51C9, 0x51CD, 0x0846,
	/* Array index: 0x0078 */ 0x51DB, 0x51E1, 0x084B,
	/* Array index: 0x007B */ 0x51E6, 0x51EA, 0x0852,
	/* Array index: 0x007E */ 0x51F5, 0x520B, 0x0857,
	/* Array index: 0x0081 */ 0x5236, 0x523B, 0x086E,
	/* Array index: 0x0084 */ 0x524A, 0x524F, 0x0874,
	/* Array index: 0x0087 */ 0x5263, 0x5265, 0x087A,
	/* Array index: 0x008A */ 0x526F, 0x5275, 0x087D,
	/* Array index: 0x008D */ 0x5287, 0x5289, 0x0884,
	/* Array index: 0x0090 */ 0x52A9, 0x52AD, 0x0887,
	/* Array index: 0x0093 */ 0x52D5, 0x52E7, 0x088C,
	/* Array index: 0x0096 */ 0x52F8, 0x52FA, 0x089F,
	/* Array index: 0x0099 */ 0x52FE, 0x5308, 0x08A2,
	/* Array index: 0x009C */ 0x5315, 0x531A, 0x08AD,
	/* Array index: 0x009F */ 0x5338, 0x534A, 0x08B3,
	/* Array index: 0x00A2 */ 0x5351, 0x5360, 0x08C6,
	/* Array index: 0x00A5 */ 0x536E, 0x5378, 0x08D6,
	/* Array index: 0x00A8 */ 0x53A5, 0x53A9, 0x08E1,
	/* Array index: 0x00AB */ 0x53C8, 0x53CE, 0x08E6,
	/* Array index: 0x00AE */ 0x53DF, 0x53FA, 0x08ED,
	/* Array index: 0x00B1 */ 0x5408, 0x5411, 0x0909,
	/* Array index: 0x00B4 */ 0x5429, 0x542E, 0x0913,
	/* Array index: 0x00B7 */ 0x5438, 0x5442, 0x0919,
	/* Array index: 0x00BA */ 0x5446, 0x544A, 0x0924,
	/* Array index: 0x00BD */ 0x5473, 0x547D, 0x0929,
	/* Array index: 0x00C0 */ 0x548B, 0x5492, 0x0934,
	/* Array index: 0x00C3 */ 0x54BC, 0x54C9, 0x093C,
	/* Array index: 0x00C6 */ 0x54E5, 0x54E9, 0x094A,
	/* Array index: 0x00C9 */ 0x5544, 0x5546, 0x094F,
	/* Array index: 0x00CC */ 0x5587, 0x558B, 0x0952,
	/* Array index: 0x00CF */ 0x5598, 0x559F, 0x0957,
	/* Array index: 0x00D2 */ 0x55A7, 0x55B0, 0x095F,
	/* Array index: 0x00D5 */ 0x5614, 0x5618, 0x0969,
	/* Array index: 0x00D8 */ 0x5668, 0x566C, 0x096E,
	/* Array index: 0x00DB */ 0x56C0, 0x56C3, 0x0973,
	/* Array index: 0x00DE */ 0x56D7, 0x56DB, 0x0977,
	/* Array index: 0x00E1 */ 0x5726, 0x5728, 0x097C,
	/* Array index: 0x00E4 */ 0x574E, 0x5751, 0x097F,
	/* Array index: 0x00E7 */ 0x57A0, 0x57A4, 0x0983,
	/* Array index: 0x00EA */ 0x57D2, 0x57D6, 0x0988,
	/* Array index: 0x00ED */ 0x582F, 0x5835, 0x098D,
	/* Array index: 0x00F0 */ 0x5857, 0x585A, 0x0994,
	/* Array index: 0x00F3 */ 0x58B8, 0x58BB, 0x0998,
	/* Array index: 0x00F6 */ 0x58D5, 0x58D9, 0x099C,
	/* Array index: 0x00F9 */ 0x58EB, 0x58F2, 0x09A1,
	/* Array index: 0x00FC */ 0x58F7, 0x58FD, 0x09A9,
	/* Array index: 0x00FF */ 0x5915, 0x591C, 0x09B0,
	/* Array index: 0x0102 */ 0x5927, 0x5932, 0x09B8,
	/* Array index: 0x0105 */ 0x5947, 0x5949, 0x09C4,
	/* Array index: 0x0108 */ 0x594E, 0x595A, 0x09C7,
	/* Array index: 0x010B */ 0x5965, 0x596E, 0x09D4,
	/* Array index: 0x010E */ 0x5981, 0x5984, 0x09DE,
	/* Array index: 0x0111 */ 0x59D0, 0x59D4, 0x09E2,
	/* Array index: 0x0114 */ 0x5ABC, 0x5AC2, 0x09E7,
	/* Array index: 0x0117 */ 0x5B54, 0x5B66, 0x09EE,
	/* Array index: 0x011A */ 0x5B85, 0x5B8F, 0x0A01,
	/* Array index: 0x011D */ 0x5B95, 0x5BA6, 0x0A0C,
	/* Array index: 0x0120 */ 0x5BB3, 0x5BB9, 0x0A1E,
	/* Array index: 0x0123 */ 0x5BC2, 0x5BC9, 0x0A25,
	/* Array index: 0x0126 */ 0x5BD0, 0x5BD4, 0x0A2D,
	/* Array index: 0x0129 */ 0x5BDB, 0x5BEB, 0x0A32,
	/* Array index: 0x012C */ 0x5BFE, 0x5C13, 0x0A43,
	/* Array index: 0x012F */ 0x5C38, 0x5C41, 0x0A59,
	/* Array index: 0x0132 */ 0x5C4A, 0x5C55, 0x0A63,
	/* Array index: 0x0135 */ 0x5CA8, 0x5CAC, 0x0A6F,
	/* Array index: 0x0138 */ 0x5CB6, 0x5CBE, 0x0A74,
	/* Array index: 0x013B */ 0x5CE8, 0x5CEA, 0x0A7D,
	/* Array index: 0x013E */ 0x5D14, 0x5D1B, 0x0A80,
	/* Array index: 0x0141 */ 0x5DE3, 0x5DE8, 0x0A88,
	/* Array index: 0x0144 */ 0x5DF1, 0x5DF7, 0x0A8E,
	/* Array index: 0x0147 */ 0x5E19, 0x5E1D, 0x0A95,
	/* Array index: 0x014A */ 0x5E36, 0x5E38, 0x0A9A,
	/* Array index: 0x014D */ 0x5E43, 0x5E47, 0x0A9D,
	/* Array index: 0x0150 */ 0x5E5F, 0x5E64, 0x0AA2,
	/* Array index: 0x0153 */ 0x5E72, 0x5E84, 0x0AA8,
	/* Array index: 0x0156 */ 0x5E95, 0x5E97, 0x0ABB,
	/* Array index: 0x0159 */ 0x5EB5, 0x5EB8, 0x0ABE,
	/* Array index: 0x015C */ 0x5EC1, 0x5EC3, 0x0AC2,
	/* Array index: 0x015F */ 0x5EC8, 0x5ECA, 0x0AC5,
	/* Array index: 0x0162 */ 0x5EDD, 0x5EE3, 0x0AC8,
	/* Array index: 0x0165 */ 0x5EF0, 0x5F04, 0x0ACF,
	/* Array index: 0x0168 */ 0x5F09, 0x5F18, 0x0AE4,
	/* Array index: 0x016B */ 0x5F25, 0x5F29, 0x0AF4,
	/* Array index: 0x016E */ 0x5F69, 0x5F73, 0x0AF9,
	/* Array index: 0x0171 */ 0x5F7F, 0x5F99, 0x0B04,
	/* Array index: 0x0174 */ 0x5FA8, 0x5FAE, 0x0B1F,
	/* Array index: 0x0177 */ 0x5FD6, 0x5FDD, 0x0B26,
	/* Array index: 0x017A */ 0x600E, 0x6016, 0x0B2E,
	/* Array index: 0x017D */ 0x6019, 0x602B, 0x0B37,
	/* Array index: 0x0180 */ 0x6041, 0x6043, 0x0B4A,
	/* Array index: 0x0183 */ 0x605F, 0x6070, 0x0B4D,
	/* Array index: 0x0186 */ 0x6089, 0x608D, 0x0B5F,
	/* Array index: 0x0189 */ 0x60A6, 0x60AA, 0x0B64,
	/* Array index: 0x018C */ 0x60B2, 0x60B8, 0x0B69,
	/* Array index: 0x018F */ 0x60C5, 0x60C7, 0x0B70,
	/* Array index: 0x0192 */ 0x60DF, 0x60E3, 0x0B73,
	/* Array index: 0x0195 */ 0x60F0, 0x60FB, 0x0B78,
	/* Array index: 0x0198 */ 0x610D, 0x610F, 0x0B84,
	/* Array index: 0x019B */ 0x613C, 0x613F, 0x0B87,
	/* Array index: 0x019E */ 0x6147, 0x614E, 0x0B8B,
	/* Array index: 0x01A1 */ 0x6158, 0x615A, 0x0B93,
	/* Array index: 0x01A4 */ 0x616E, 0x6177, 0x0B96,
	/* Array index: 0x01A7 */ 0x61C6, 0x61CD, 0x0BA0,
	/* Array index: 0x01AA */ 0x61F4, 0x6200, 0x0BA8,
	/* Array index: 0x01AD */ 0x6208, 0x6216, 0x0BB5,
	/* Array index: 0x01B0 */ 0x621A, 0x6221, 0x0BC4,
	/* Array index: 0x01B3 */ 0x622E, 0x6234, 0x0BCC,
	/* Array index: 0x01B6 */ 0x623F, 0x6241, 0x0BD3,
	/* Array index: 0x01B9 */ 0x6247, 0x624E, 0x0BD6,
	/* Array index: 0x01BC */ 0x627C, 0x6284, 0x0BDE,
	/* Array index: 0x01BF */ 0x6291, 0x629E, 0x0BE7,
	/* Array index: 0x01C2 */ 0x62B9, 0x62BD, 0x0BF5,
	/* Array index: 0x01C5 */ 0x62C5, 0x62E1, 0x0BFA,
	/* Array index: 0x01C8 */ 0x62EC, 0x62F7, 0x0C17,
	/* Array index: 0x01CB */ 0x62FE, 0x6302, 0x0C23,
	/* Array index: 0x01CE */ 0x6307, 0x6309, 0x0C28,
	/* Array index: 0x01D1 */ 0x633D, 0x633F, 0x0C2B,
	/* Array index: 0x01D4 */ 0x634C, 0x6350, 0x0C2E,
	/* Array index: 0x01D7 */ 0x6367, 0x636B, 0x0C33,
	/* Array index: 0x01DA */ 0x639F, 0x63AC, 0x0C38,
	/* Array index: 0x01DD */ 0x64E0, 0x64E7, 0x0C46,
	/* Array index: 0x01E0 */ 0x652A, 0x652C, 0x0C4E,
	/* Array index: 0x01E3 */ 0x6534, 0x653F, 0x0C51,
	/* Array index: 0x01E6 */ 0x6555, 0x6559, 0x0C5D,
	/* Array index: 0x01E9 */ 0x6574, 0x6578, 0x0C62,
	/* Array index: 0x01EC */ 0x6587, 0x6589, 0x0C67,
	/* Array index: 0x01EF */ 0x65AB, 0x65B0, 0x0C6A,
	/* Array index: 0x01F2 */ 0x65C1, 0x65C6, 0x0C70,
	/* Array index: 0x01F5 */ 0x65E0, 0x65ED, 0x0C76,
	/* Array index: 0x01F8 */ 0x6634, 0x6636, 0x0C84,
	/* Array index: 0x01FB */ 0x663F, 0x6644, 0x0C87,
	/* Array index: 0x01FE */ 0x665D, 0x665F, 0x0C8D,
	/* Array index: 0x0201 */ 0x6664, 0x6669, 0x0C90,
	/* Array index: 0x0204 */ 0x666E, 0x6670, 0x0C96,
	/* Array index: 0x0207 */ 0x6683, 0x6689, 0x0C99,
	/* Array index: 0x020A */ 0x6696, 0x6698, 0x0CA0,
	/* Array index: 0x020D */ 0x66D9, 0x66DD, 0x0CA3,
	/* Array index: 0x0210 */ 0x66F0, 0x6700, 0x0CA8,
	/* Array index: 0x0213 */ 0x6714, 0x6717, 0x0CB9,
	/* Array index: 0x0216 */ 0x671B, 0x671F, 0x0CBD,
	/* Array index: 0x0219 */ 0x6726, 0x672E, 0x0CC2,
	/* Array index: 0x021C */ 0x6734, 0x673A, 0x0CCB,
	/* Array index: 0x021F */ 0x674E, 0x6753, 0x0CD2,
	/* Array index: 0x0222 */ 0x675C, 0x6765, 0x0CD8,
	/* Array index: 0x0225 */ 0x676D, 0x6777, 0x0CE2,
	/* Array index: 0x0228 */ 0x679C, 0x67A2, 0x0CED,
	/* Array index: 0x022B */ 0x67B3, 0x67B9, 0x0CF4,
	/* Array index: 0x022E */ 0x67CE, 0x67D4, 0x0CFB,
	/* Array index: 0x0231 */ 0x67F1, 0x67F5, 0x0D02,
	/* Array index: 0x0234 */ 0x67FE, 0x6804, 0x0D07,
	/* Array index: 0x0237 */ 0x6829, 0x682B, 0x0D0E,
	/* Array index: 0x023A */ 0x683C, 0x6843, 0x0D11,
	/* Array index: 0x023D */ 0x684D, 0x6854, 0x0D19,
	/* Array index: 0x0240 */ 0x68A6, 0x68A8, 0x0D21,
	/* Array index: 0x0243 */ 0x68AD, 0x68BC, 0x0D24,
	/* Array index: 0x0246 */ 0x68C9, 0x68CD, 0x0D34,
	/* Array index: 0x0249 */ 0x68D4, 0x68DA, 0x0D39,
	/* Array index: 0x024C */ 0x68DF, 0x68E3, 0x0D40,
	/* Array index: 0x024F */ 0x690B, 0x690F, 0x0D45,
	/* Array index: 0x0252 */ 0x6919, 0x691C, 0x0D4A,
	/* Array index: 0x0255 */ 0x6921, 0x692A, 0x0D4E,
	/* Array index: 0x0258 */ 0x6953, 0x6955, 0x0D58,
	/* Array index: 0x025B */ 0x6959, 0x6962, 0x0D5B,
	/* Array index: 0x025E */ 0x696A, 0x6982, 0x0D65,
	/* Array index: 0x0261 */ 0x69CA, 0x69D0, 0x0D7E,
	/* Array index: 0x0264 */ 0x6A0A, 0x6A0C, 0x0D85,
	/* Array index: 0x0267 */ 0x6A12, 0x6A14, 0x0D88,
	/* Array index: 0x026A */ 0x6A1E, 0x6A23, 0x0D8B,
	/* Array index: 0x026D */ 0x6A29, 0x6A2B, 0x0D91,
	/* Array index: 0x0270 */ 0x6A35, 0x6A3A, 0x0D94,
	/* Array index: 0x0273 */ 0x6AC1, 0x6AC3, 0x0D9A,
	/* Array index: 0x0276 */ 0x6B1D, 0x6B23, 0x0D9D,
	/* Array index: 0x0279 */ 0x6B37, 0x6B3E, 0x0DA4,
	/* Array index: 0x027C */ 0x6B5F, 0x6B6A, 0x0DAC,
	/* Array index: 0x027F */ 0x6B89, 0x6B8D, 0x0DB8,
	/* Array index: 0x0282 */ 0x6BAF, 0x6BC0, 0x0DBD,
	/* Array index: 0x0285 */ 0x6BD2, 0x6BD4, 0x0DCF,
	/* Array index: 0x0288 */ 0x6C3E, 0x6C42, 0x0DD2,
	/* Array index: 0x028B */ 0x6C5D, 0x6C62, 0x0DD7,
	/* Array index: 0x028E */ 0x6C7D, 0x6C83, 0x0DDD,
	/* Array index: 0x0291 */ 0x6C99, 0x6C9B, 0x0DE4,
	/* Array index: 0x0294 */ 0x6CB8, 0x6CC5, 0x0DE7,
	/* Array index: 0x0297 */ 0x6CE1, 0x6CE5, 0x0DF5,
	/* Array index: 0x029A */ 0x6CEF, 0x6CF3, 0x0DFA,
	/* Array index: 0x029D */ 0x6D29, 0x6D2B, 0x0DFF,
	/* Array index: 0x02A0 */ 0x6D32, 0x6D38, 0x0E02,
	/* Array index: 0x02A3 */ 0x6D77, 0x6D79, 0x0E09,
	/* Array index: 0x02A6 */ 0x6DC5, 0x6DC7, 0x0E0C,
	/* Array index: 0x02A9 */ 0x6DE8, 0x6DEE, 0x0E0F,
	/* Array index: 0x02AC */ 0x6DF7, 0x6DFB, 0x0E16,
	/* Array index: 0x02AF */ 0x6E05, 0x6E0B, 0x0E1B,
	/* Array index: 0x02B2 */ 0x6E19, 0x6E26, 0x0E22,
	/* Array index: 0x02B5 */ 0x6E29, 0x6E2F, 0x0E30,
	/* Array index: 0x02B8 */ 0x6E7E, 0x6E82, 0x0E37,
	/* Array index: 0x02BB */ 0x6ED1, 0x6ED5, 0x0E3C,
	/* Array index: 0x02BE */ 0x6EFE, 0x6F02, 0x0E41,
	/* Array index: 0x02C1 */ 0x6F11, 0x6F15, 0x0E46,
	/* Array index: 0x02C4 */ 0x6F6D, 0x6F70, 0x0E4B,
	/* Array index: 0x02C7 */ 0x6F80, 0x6F86, 0x0E4F,
	/* Array index: 0x02CA */ 0x6FC0, 0x6FC3, 0x0E56,
	/* Array index: 0x02CD */ 0x6FDF, 0x6FE1, 0x0E5A,
	/* Array index: 0x02D0 */ 0x6FEB, 0x6FF3, 0x0E5D,
	/* Array index: 0x02D3 */ 0x701A, 0x701F, 0x0E66,
	/* Array index: 0x02D6 */ 0x70AC, 0x70AF, 0x0E6C,
	/* Array index: 0x02D9 */ 0x70B8, 0x70BA, 0x0E70,
	/* Array index: 0x02DC */ 0x7162, 0x7169, 0x0E73,
	/* Array index: 0x02DF */ 0x71E5, 0x71E7, 0x0E7B,
	/* Array index: 0x02E2 */ 0x71EC, 0x71EE, 0x0E7E,
	/* Array index: 0x02E5 */ 0x723A, 0x7240, 0x0E81,
	/* Array index: 0x02E8 */ 0x7246, 0x724C, 0x0E88,
	/* Array index: 0x02EB */ 0x727D, 0x7282, 0x0E8F,
	/* Array index: 0x02EE */ 0x72C2, 0x72C6, 0x0E95,
	/* Array index: 0x02F1 */ 0x72E0, 0x72E2, 0x0E9A,
	/* Array index: 0x02F4 */ 0x72F7, 0x72FD, 0x0E9D,
	/* Array index: 0x02F7 */ 0x731B, 0x731F, 0x0EA4,
	/* Array index: 0x02FA */ 0x7329, 0x732F, 0x0EA9,
	/* Array index: 0x02FD */ 0x7432, 0x7436, 0x0EB0,
	/* Array index: 0x0300 */ 0x7459, 0x7464, 0x0EB5,
	/* Array index: 0x0303 */ 0x74EE, 0x74F8, 0x0EC1,
	/* Array index: 0x0306 */ 0x7503, 0x7505, 0x0ECC,
	/* Array index: 0x0309 */ 0x750C, 0x750E, 0x0ECF,
	/* Array index: 0x030C */ 0x7530, 0x7533, 0x0ED2,
	/* Array index: 0x030F */ 0x7537, 0x753C, 0x0ED6,
	/* Array index: 0x0312 */ 0x7549, 0x7551, 0x0EDC,
	/* Array index: 0x0315 */ 0x7559, 0x755D, 0x0EE5,
	/* Array index: 0x0318 */ 0x7562, 0x756D, 0x0EEA,
	/* Array index: 0x031B */ 0x7573, 0x7578, 0x0EF6,
	/* Array index: 0x031E */ 0x7586, 0x7591, 0x0EFC,
	/* Array index: 0x0321 */ 0x75B1, 0x75BE, 0x0F08,
	/* Array index: 0x0324 */ 0x75F0, 0x75F4, 0x0F16,
	/* Array index: 0x0327 */ 0x761F, 0x7624, 0x0F1B,
	/* Array index: 0x032A */ 0x7646, 0x7648, 0x0F21,
	/* Array index: 0x032D */ 0x7667, 0x766C, 0x0F24,
	/* Array index: 0x0330 */ 0x7678, 0x7688, 0x0F2A,
	/* Array index: 0x0333 */ 0x76B7, 0x76BA, 0x0F3B,
	/* Array index: 0x0336 */ 0x76DB, 0x76E7, 0x0F3F,
	/* Array index: 0x0339 */ 0x7707, 0x770C, 0x0F4C,
	/* Array index: 0x033C */ 0x771E, 0x7726, 0x0F52,
	/* Array index: 0x033F */ 0x77B9, 0x77BF, 0x0F5B,
	/* Array index: 0x0342 */ 0x77DA, 0x77DC, 0x0F62,
	/* Array index: 0x0345 */ 0x77ED, 0x77EF, 0x0F65,
	/* Array index: 0x0348 */ 0x7825, 0x7827, 0x0F68,
	/* Array index: 0x034B */ 0x788C, 0x788E, 0x0F6B,
	/* Array index: 0x034E */ 0x792A, 0x792C, 0x0F6E,
	/* Array index: 0x0351 */ 0x7947, 0x7949, 0x0F71,
	/* Array index: 0x0354 */ 0x7953, 0x7957, 0x0F74,
	/* Array index: 0x0357 */ 0x795D, 0x7962, 0x0F79,
	/* Array index: 0x035A */ 0x797F, 0x7985, 0x0F7F,
	/* Array index: 0x035D */ 0x798D, 0x798F, 0x0F86,
	/* Array index: 0x0360 */ 0x79B9, 0x79C1, 0x0F89,
	/* Array index: 0x0363 */ 0x79E3, 0x79E9, 0x0F92,
	/* Array index: 0x0366 */ 0x7A17, 0x7A20, 0x0F99,
	/* Array index: 0x0369 */ 0x7A3B, 0x7A43, 0x0FA3,
	/* Array index: 0x036C */ 0x7A4D, 0x7A50, 0x0FAC,
	/* Array index: 0x036F */ 0x7A61, 0x7A63, 0x0FB0,
	/* Array index: 0x0372 */ 0x7A92, 0x7A98, 0x0FB3,
	/* Array index: 0x0375 */ 0x7AAE, 0x7AB0, 0x0FBA,
	/* Array index: 0x0378 */ 0x7AC3, 0x7AD5, 0x0FBD,
	/* Array index: 0x037B */ 0x7AD9, 0x7AE6, 0x0FD0,
	/* Array index: 0x037E */ 0x7AF6, 0x7AFA, 0x0FDE,
	/* Array index: 0x0381 */ 0x7B45, 0x7B56, 0x0FE3,
	/* Array index: 0x0384 */ 0x7B94, 0x7BA1, 0x0FF5,
	/* Array index: 0x0387 */ 0x7BE4, 0x7BE6, 0x1003,
	/* Array index: 0x038A */ 0x7C11, 0x7C14, 0x1006,
	/* Array index: 0x038D */ 0x7C3D, 0x7C40, 0x100A,
	/* Array index: 0x0390 */ 0x7C4C, 0x7C50, 0x100E,
	/* Array index: 0x0393 */ 0x7C81, 0x7C83, 0x1013,
	/* Array index: 0x0396 */ 0x7CA1, 0x7CA8, 0x1016,
	/* Array index: 0x0399 */ 0x7CAD, 0x7CB3, 0x101E,
	/* Array index: 0x039C */ 0x7CDC, 0x7CE2, 0x1025,
	/* Array index: 0x039F */ 0x7D02, 0x7D06, 0x102C,
	/* Array index: 0x03A2 */ 0x7D14, 0x7D22, 0x1031,
	/* Array index: 0x03A5 */ 0x7D2B, 0x7D35, 0x1040,
	/* Array index: 0x03A8 */ 0x7D42, 0x7D46, 0x104B,
	/* Array index: 0x03AB */ 0x7D4B, 0x7D50, 0x1050,
	/* Array index: 0x03AE */ 0x7D61, 0x7D63, 0x1056,
	/* Array index: 0x03B1 */ 0x7D71, 0x7D76, 0x1059,
	/* Array index: 0x03B4 */ 0x7D99, 0x7D9C, 0x105F,
	/* Array index: 0x03B7 */ 0x7DAB, 0x7DB5, 0x1063,
	/* Array index: 0x03BA */ 0x7DBA, 0x7DBF, 0x106E,
	/* Array index: 0x03BD */ 0x7DDA, 0x7DE1, 0x1074,
	/* Array index: 0x03C0 */ 0x7E09, 0x7E0B, 0x107C,
	/* Array index: 0x03C3 */ 0x7E1E, 0x7E23, 0x107F,
	/* Array index: 0x03C6 */ 0x7E37, 0x7E3E, 0x1085,
	/* Array index: 0x03C9 */ 0x7E54, 0x7E5E, 0x108D,
	/* Array index: 0x03CC */ 0x7E66, 0x7E6A, 0x1098,
	/* Array index: 0x03CF */ 0x7E79, 0x7E83, 0x109D,
	/* Array index: 0x03D2 */ 0x7E8C, 0x7E96, 0x10A8,
	/* Array index: 0x03D5 */ 0x7F4C, 0x7F55, 0x10B3,
	/* Array index: 0x03D8 */ 0x7F67, 0x7F6B, 0x10BD,
	/* Array index: 0x03DB */ 0x7F75, 0x7F79, 0x10C2,
	/* Array index: 0x03DE */ 0x7F82, 0x7F8E, 0x10C7,
	/* Array index: 0x03E1 */ 0x8000, 0x8006, 0x10D4,
	/* Array index: 0x03E4 */ 0x8015, 0x8019, 0x10DB,
	/* Array index: 0x03E7 */ 0x805E, 0x8062, 0x10E0,
	/* Array index: 0x03EA */ 0x806F, 0x807F, 0x10E5,
	/* Array index: 0x03ED */ 0x8084, 0x808C, 0x10F6,
	/* Array index: 0x03F0 */ 0x80A9, 0x80B4, 0x10FF,
	/* Array index: 0x03F3 */ 0x80D9, 0x80DE, 0x110B,
	/* Array index: 0x03F6 */ 0x8105, 0x810A, 0x1111,
	/* Array index: 0x03F9 */ 0x8150, 0x8155, 0x1117,
	/* Array index: 0x03FC */ 0x8178, 0x817A, 0x111D,
	/* Array index: 0x03FF */ 0x817F, 0x8183, 0x1120,
	/* Array index: 0x0402 */ 0x81BD, 0x81C2, 0x1125,
	/* Array index: 0x0405 */ 0x81D8, 0x81DA, 0x112B,
	/* Array index: 0x0408 */ 0x81FA, 0x8202, 0x112E,
	/* Array index: 0x040B */ 0x8205, 0x821F, 0x1137,
	/* Array index: 0x040E */ 0x8229, 0x822E, 0x1152,
	/* Array index: 0x0411 */ 0x8233, 0x8239, 0x1158,
	/* Array index: 0x0414 */ 0x8258, 0x825A, 0x115F,
	/* Array index: 0x0417 */ 0x826E, 0x8278, 0x1162,
	/* Array index: 0x041A */ 0x82AB, 0x82B3, 0x116D,
	/* Array index: 0x041D */ 0x82D1, 0x82D4, 0x1176,
	/* Array index: 0x0420 */ 0x82DB, 0x82E7, 0x117A,
	/* Array index: 0x0423 */ 0x82F9, 0x82FB, 0x1187,
	/* Array index: 0x0426 */ 0x8302, 0x8306, 0x118A,
	/* Array index: 0x0429 */ 0x8316, 0x8318, 0x118F,
	/* Array index: 0x042C */ 0x8331, 0x8339, 0x1192,
	/* Array index: 0x042F */ 0x839E, 0x83A2, 0x119B,
	/* Array index: 0x0432 */ 0x83EF, 0x83F4, 0x11A0,
	/* Array index: 0x0435 */ 0x840B, 0x840E, 0x11A6,
	/* Array index: 0x0438 */ 0x8461, 0x8463, 0x11AA,
	/* Array index: 0x043B */ 0x8469, 0x8471, 0x11AD,
	/* Array index: 0x043E */ 0x84B8, 0x84BC, 0x11B6,
	/* Array index: 0x0441 */ 0x84C9, 0x84D1, 0x11BB,
	/* Array index: 0x0444 */ 0x8511, 0x851A, 0x11C4,
	/* Array index: 0x0447 */ 0x8548, 0x854B, 0x11CE,
	/* Array index: 0x044A */ 0x8568, 0x856A, 0x11D2,
	/* Array index: 0x044D */ 0x85A6, 0x85AF, 0x11D5,
	/* Array index: 0x0450 */ 0x85F7, 0x85FB, 0x11DF,
	/* Array index: 0x0453 */ 0x868A, 0x868C, 0x11E4,
	/* Array index: 0x0456 */ 0x86A9, 0x86AB, 0x11E7,
	/* Array index: 0x0459 */ 0x86EC, 0x86EF, 0x11EA,
	/* Array index: 0x045C */ 0x8706, 0x870A, 0x11EE,
	/* Array index: 0x045F */ 0x880D, 0x8811, 0x11F3,
	/* Array index: 0x0462 */ 0x8821, 0x8823, 0x11F8,
	/* Array index: 0x0465 */ 0x885D, 0x8863, 0x11FB,
	/* Array index: 0x0468 */ 0x887D, 0x8882, 0x1202,
	/* Array index: 0x046B */ 0x88BF, 0x88C5, 0x1208,
	/* Array index: 0x046E */ 0x88F2, 0x88F4, 0x120F,
	/* Array index: 0x0471 */ 0x88F8, 0x88FE, 0x1212,
	/* Array index: 0x0474 */ 0x895E, 0x8960, 0x1219,
	/* Array index: 0x0477 */ 0x8986, 0x898B, 0x121C,
	/* Array index: 0x047A */ 0x8996, 0x899A, 0x1222,
	/* Array index: 0x047D */ 0x89A6, 0x89AC, 0x1227,
	/* Array index: 0x0480 */ 0x8A16, 0x8A18, 0x122E,
	/* Array index: 0x0483 */ 0x8A3A, 0x8A3C, 0x1231,
	/* Array index: 0x0486 */ 0x8A50, 0x8A55, 0x1234,
	/* Array index: 0x0489 */ 0x8A69, 0x8A73, 0x123A,
	/* Array index: 0x048C */ 0x8AA0, 0x8AA8, 0x1245,
	/* Array index: 0x048F */ 0x8ACB, 0x8ACF, 0x124E,
	/* Array index: 0x0492 */ 0x8ADA, 0x8AE7, 0x1253,
	/* Array index: 0x0495 */ 0x8AFE, 0x8B04, 0x1261,
	/* Array index: 0x0498 */ 0x8B16, 0x8B21, 0x1268,
	/* Array index: 0x049B */ 0x8B58, 0x8B5C, 0x1274,
	/* Array index: 0x049E */ 0x8B6B, 0x8B74, 0x1279,
	/* Array index: 0x04A1 */ 0x8C6A, 0x8C6C, 0x1283,
	/* Array index: 0x04A4 */ 0x8C78, 0x8C7C, 0x1286,
	/* Array index: 0x04A7 */ 0x8C89, 0x8C8E, 0x128B,
	/* Array index: 0x04AA */ 0x8C9D, 0x8CA2, 0x1291,
	/* Array index: 0x04AD */ 0x8CA7, 0x8CCE, 0x1297,
	/* Array index: 0x04B0 */ 0x8CDA, 0x8CE6, 0x12BF,
	/* Array index: 0x04B3 */ 0x8CFA, 0x8CFD, 0x12CC,
	/* Array index: 0x04B6 */ 0x8D04, 0x8D16, 0x12D0,
	/* Array index: 0x04B9 */ 0x8D70, 0x8D74, 0x12E3,
	/* Array index: 0x04BC */ 0x8E08, 0x8E0A, 0x12E8,
	/* Array index: 0x04BF */ 0x8E1D, 0x8E1F, 0x12EB,
	/* Array index: 0x04C2 */ 0x8E47, 0x8E4C, 0x12EE,
	/* Array index: 0x04C5 */ 0x8EAA, 0x8EB1, 0x12F4,
	/* Array index: 0x04C8 */ 0x8EC8, 0x8ECD, 0x12FC,
	/* Array index: 0x04CB */ 0x8EFB, 0x8EFE, 0x1302,
	/* Array index: 0x04CE */ 0x8F12, 0x8F15, 0x1306,
	/* Array index: 0x04D1 */ 0x8F19, 0x8F1F, 0x130A,
	/* Array index: 0x04D4 */ 0x8F42, 0x8F46, 0x1311,
	/* Array index: 0x04D7 */ 0x8F4C, 0x8F4E, 0x1316,
	/* Array index: 0x04DA */ 0x8F5F, 0x8F64, 0x1319,
	/* Array index: 0x04DD */ 0x8F9B, 0x8F9F, 0x131F,
	/* Array index: 0x04E0 */ 0x8FAD, 0x8FB2, 0x1324,
	/* Array index: 0x04E3 */ 0x8FBA, 0x8FBC, 0x132A,
	/* Array index: 0x04E6 */ 0x8FE5, 0x8FF0, 0x132D,
	/* Array index: 0x04E9 */ 0x8FF7, 0x8FFA, 0x1339,
	/* Array index: 0x04EC */ 0x900B, 0x9023, 0x133D,
	/* Array index: 0x04EF */ 0x9035, 0x9039, 0x1356,
	/* Array index: 0x04F2 */ 0x9047, 0x9059, 0x135B,
	/* Array index: 0x04F5 */ 0x906D, 0x906F, 0x136E,
	/* Array index: 0x04F8 */ 0x9075, 0x9084, 0x1371,
	/* Array index: 0x04FB */ 0x9149, 0x914E, 0x1381,
	/* Array index: 0x04FE */ 0x91C6, 0x91D1, 0x1387,
	/* Array index: 0x0501 */ 0x91DB, 0x91E7, 0x1393,
	/* Array index: 0x0504 */ 0x9298, 0x929C, 0x13A0,
	/* Array index: 0x0507 */ 0x9318, 0x931A, 0x13A5,
	/* Array index: 0x050A */ 0x932B, 0x932F, 0x13A8,
	/* Array index: 0x050D */ 0x93AC, 0x93B0, 0x13AD,
	/* Array index: 0x0510 */ 0x93D6, 0x93D8, 0x13B2,
	/* Array index: 0x0513 */ 0x9418, 0x941A, 0x13B5,
	/* Array index: 0x0516 */ 0x9451, 0x9453, 0x13B8,
	/* Array index: 0x0519 */ 0x947C, 0x9481, 0x13BB,
	/* Array index: 0x051C */ 0x9587, 0x958B, 0x13C1,
	/* Array index: 0x051F */ 0x95A0, 0x95A8, 0x13C6,
	/* Array index: 0x0522 */ 0x95D4, 0x95D8, 0x13CF,
	/* Array index: 0x0525 */ 0x964B, 0x9650, 0x13D4,
	/* Array index: 0x0528 */ 0x965B, 0x9666, 0x13DA,
	/* Array index: 0x052B */ 0x9672, 0x967A, 0x13E6,
	/* Array index: 0x052E */ 0x968A, 0x968F, 0x13EF,
	/* Array index: 0x0531 */ 0x9694, 0x969C, 0x13F5,
	/* Array index: 0x0534 */ 0x96B0, 0x96BC, 0x13FE,
	/* Array index: 0x0537 */ 0x96C0, 0x96CE, 0x140B,
	/* Array index: 0x053A */ 0x9704, 0x9713, 0x141A,
	/* Array index: 0x053D */ 0x975E, 0x976D, 0x142A,
	/* Array index: 0x0540 */ 0x9784, 0x9786, 0x143A,
	/* Array index: 0x0543 */ 0x97F2, 0x97F6, 0x143D,
	/* Array index: 0x0546 */ 0x97FF, 0x9808, 0x1442,
	/* Array index: 0x0549 */ 0x980F, 0x9813, 0x144C,
	/* Array index: 0x054C */ 0x9837, 0x983D, 0x1451,
	/* Array index: 0x054F */ 0x984B, 0x984F, 0x1458,
	/* Array index: 0x0552 */ 0x986F, 0x9874, 0x145D,
	/* Array index: 0x0555 */ 0x98EB, 0x98EF, 0x1463,
	/* Array index: 0x0558 */ 0x98FC, 0x98FE, 0x1468,
	/* Array index: 0x055B */ 0x9910, 0x9914, 0x146B,
	/* Array index: 0x055E */ 0x991D, 0x9921, 0x1470,
	/* Array index: 0x0561 */ 0x9950, 0x9952, 0x1475,
	/* Array index: 0x0564 */ 0x9996, 0x9999, 0x1478,
	/* Array index: 0x0567 */ 0x99AC, 0x99AE, 0x147C,
	/* Array index: 0x056A */ 0x99C4, 0x99C8, 0x147F,
	/* Array index: 0x056D */ 0x99D0, 0x99D2, 0x1484,
	/* Array index: 0x0570 */ 0x9A69, 0x9A6B, 0x1487,
	/* Array index: 0x0573 */ 0x9AEA, 0x9AF1, 0x148A,
	/* Array index: 0x0576 */ 0x9B25, 0x9B2A, 0x1492,
	/* Array index: 0x0579 */ 0x9B2E, 0x9B32, 0x1498,
	/* Array index: 0x057C */ 0x9B41, 0x9B45, 0x149D,
	/* Array index: 0x057F */ 0x9B4D, 0x9B51, 0x14A2,
	/* Array index: 0x0582 */ 0x9B91, 0x9B97, 0x14A7,
	/* Array index: 0x0585 */ 0x9BAA, 0x9BAE, 0x14AE,
	/* Array index: 0x0588 */ 0x9BE1, 0x9BE4, 0x14B3,
	/* Array index: 0x058B */ 0x9BF0, 0x9BF2, 0x14B7,
	/* Array index: 0x058E */ 0x9C06, 0x9C0D, 0x14BA,
	/* Array index: 0x0591 */ 0x9C10, 0x9C15, 0x14C2,
	/* Array index: 0x0594 */ 0x9C2D, 0x9C32, 0x14C8,
	/* Array index: 0x0597 */ 0x9C39, 0x9C3B, 0x14CE,
	/* Array index: 0x059A */ 0x9C46, 0x9C48, 0x14D1,
	/* Array index: 0x059D */ 0x9D06, 0x9D09, 0x14D4,
	/* Array index: 0x05A0 */ 0x9D28, 0x9D2C, 0x14D8,
	/* Array index: 0x05A3 */ 0x9D5C, 0x9D61, 0x14DD,
	/* Array index: 0x05A6 */ 0x9DF8, 0x9DFA, 0x14E3,
	/* Array index: 0x05A9 */ 0x9E91, 0x9E97, 0x14E6,
	/* Array index: 0x05AC */ 0x9EB8, 0x9EBF, 0x14ED,
	/* Array index: 0x05AF */ 0x9ECC, 0x9ED4, 0x14F5,
	/* Array index: 0x05B2 */ 0x9ED8, 0x9EE0, 0x14FE,
	/* Array index: 0x05B5 */ 0x9EF9, 0x9EFD, 0x1507,
	/* Array index: 0x05B8 */ 0x9F5F, 0x9F67, 0x150C,
	/* Array index: 0x05BB */ 0xFF01, 0xFF5D, 0x1515,
	/* Ranges content */
	/* Range 0x0391 - 0x03A9, array index: 0x05BE */
	0x2621,0x2622,0x2623,0x2624,0x2625,0x2626,0x2627,0x2628,
	0x2629,0x262A,0x262B,0x262C,0x262D,0x262E,0x262F,0x2630,
	0x2631,INVALC,0x2632,0x2633,0x2634,0x2635,0x2636,0x2637,
	0x2638,
	/* Range 0x03B1 - 0x03C9, array index: 0x05D7 */
	0x2641,0x2642,0x2643,0x2644,0x2645,0x2646,0x2647,0x2648,
	0x2649,0x264A,0x264B,0x264C,0x264D,0x264E,0x264F,0x2650,
	0x2651,INVALC,0x2652,0x2653,0x2654,0x2655,0x2656,0x2657,
	0x2658,
	/* Range 0x0410 - 0x0451, array index: 0x05F0 */
	0x2721,0x2722,0x2723,0x2724,0x2725,0x2726,0x2728,0x2729,
	0x272A,0x272B,0x272C,0x272D,0x272E,0x272F,0x2730,0x2731,
	0x2732,0x2733,0x2734,0x2735,0x2736,0x2737,0x2738,0x2739,
	0x273A,0x273B,0x273C,0x273D,0x273E,0x273F,0x2740,0x2741,
	0x2751,0x2752,0x2753,0x2754,0x2755,0x2756,0x2758,0x2759,
	0x275A,0x275B,0x275C,0x275D,0x275E,0x275F,0x2760,0x2761,
	0x2762,0x2763,0x2764,0x2765,0x2766,0x2767,0x2768,0x2769,
	0x276A,0x276B,0x276C,0x276D,0x276E,0x276F,0x2770,0x2771,
	INVALC,0x2757,
	/* Range 0x2015 - 0x2021, array index: 0x0632 */
	0x213D,0x2142,INVALC,0x2146,0x2147,INVALC,INVALC,0x2148,
	0x2149,INVALC,INVALC,0x2277,0x2278,
	/* Range 0x2190 - 0x2193, array index: 0x063F */
	0x222B,0x222C,0x222A,0x222D,
	/* Range 0x2227 - 0x222C, array index: 0x0643 */
	0x224A,0x224B,0x2241,0x2240,0x2269,0x226A,
	/* Range 0x2500 - 0x2503, array index: 0x0649 */
	0x2821,0x282C,0x2822,0x282D,
	/* Range 0x2517 - 0x251D, array index: 0x064D */
	0x2831,0x2825,INVALC,INVALC,0x2830,0x2827,0x283C,
	/* Range 0x2523 - 0x2525, array index: 0x0654 */
	0x2832,0x2829,0x283E,
	/* Range 0x3000 - 0x3015, array index: 0x0657 */
	0x2121,0x2122,0x2123,0x2137,INVALC,0x2139,0x213A,0x213B,
	0x2152,0x2153,0x2154,0x2155,0x2156,0x2157,0x2158,0x2159,
	0x215A,0x215B,0x2229,0x222E,0x214C,0x214D,
	/* Range 0x3041 - 0x3093, array index: 0x066D */
	0x2421,0x2422,0x2423,0x2424,0x2425,0x2426,0x2427,0x2428,
	0x2429,0x242A,0x242B,0x242C,0x242D,0x242E,0x242F,0x2430,
	0x2431,0x2432,0x2433,0x2434,0x2435,0x2436,0x2437,0x2438,
	0x2439,0x243A,0x243B,0x243C,0x243D,0x243E,0x243F,0x2440,
	0x2441,0x2442,0x2443,0x2444,0x2445,0x2446,0x2447,0x2448,
	0x2449,0x244A,0x244B,0x244C,0x244D,0x244E,0x244F,0x2450,
	0x2451,0x2452,0x2453,0x2454,0x2455,0x2456,0x2457,0x2458,
	0x2459,0x245A,0x245B,0x245C,0x245D,0x245E,0x245F,0x2460,
	0x2461,0x2462,0x2463,0x2464,0x2465,0x2466,0x2467,0x2468,
	0x2469,0x246A,0x246B,0x246C,0x246D,0x246E,0x246F,0x2470,
	0x2471,0x2472,0x2473,
	/* Range 0x309B - 0x30F6, array index: 0x06C0 */
	0x212B,0x212C,0x2135,0x2136,INVALC,INVALC,0x2521,0x2522,
	0x2523,0x2524,0x2525,0x2526,0x2527,0x2528,0x2529,0x252A,
	0x252B,0x252C,0x252D,0x252E,0x252F,0x2530,0x2531,0x2532,
	0x2533,0x2534,0x2535,0x2536,0x2537,0x2538,0x2539,0x253A,
	0x253B,0x253C,0x253D,0x253E,0x253F,0x2540,0x2541,0x2542,
	0x2543,0x2544,0x2545,0x2546,0x2547,0x2548,0x2549,0x254A,
	0x254B,0x254C,0x254D,0x254E,0x254F,0x2550,0x2551,0x2552,
	0x2553,0x2554,0x2555,0x2556,0x2557,0x2558,0x2559,0x255A,
	0x255B,0x255C,0x255D,0x255E,0x255F,0x2560,0x2561,0x2562,
	0x2563,0x2564,0x2565,0x2566,0x2567,0x2568,0x2569,0x256A,
	0x256B,0x256C,0x256D,0x256E,0x256F,0x2570,0x2571,0x2572,
	0x2573,0x2574,0x2575,0x2576,
	/* Range 0x30FB - 0x30FE, array index: 0x071C */
	0x2126,0x213C,0x2133,0x2134,
	/* Range 0x4E07 - 0x4E19, array index: 0x0720 */
	0x4B7C,0x3E66,0x3B30,0x3E65,0x323C,INVALC,0x4954,0x4D3F,
	INVALC,0x5022,0x312F,INVALC,INVALC,0x336E,0x5023,0x4024,
	0x5242,0x3556,0x4A3A,
	/* Range 0x4E38 - 0x4E3C, array index: 0x0733 */
	0x345D,0x4330,INVALC,0x3C67,0x5027,
	/* Range 0x4E4B - 0x4E4F, array index: 0x0738 */
	0x4737,INVALC,0x4663,0x3843,0x4B33,
	/* Range 0x4E55 - 0x4E5F, array index: 0x073D */
	0x6949,0x502A,0x3E68,0x502B,0x3235,INVALC,INVALC,INVALC,
	0x3665,0x3870,0x4C69,
	/* Range 0x4E85 - 0x4EB0, array index: 0x0748 */
	0x502D,0x4E3B,INVALC,0x4D3D,0x4168,0x502F,0x3B76,0x4673,
	INVALC,0x5032,INVALC,INVALC,0x313E,0x385F,INVALC,0x385E,
	0x3066,INVALC,INVALC,0x4F4B,0x4F4A,INVALC,0x3A33,0x3021,
	INVALC,0x5033,0x5034,0x5035,0x4B34,0x5036,INVALC,0x3872,
	0x3067,0x4B72,INVALC,0x357C,INVALC,INVALC,0x357D,0x357E,
	0x4462,0x4E3C,INVALC,0x5037,
	/* Range 0x4EC0 - 0x4ECF, array index: 0x0774 */
	0x3D3A,0x3F4E,0x503E,INVALC,0x503C,INVALC,0x503D,0x3558,
	INVALC,INVALC,0x3A23,0x3270,INVALC,0x503B,0x503A,0x4A29,
	/* Range 0x4ED4 - 0x4EE5, array index: 0x0784 */
	0x3B46,0x3B45,0x423E,0x503F,0x4955,0x4067,INVALC,INVALC,
	INVALC,0x2138,0x5040,0x5042,INVALC,INVALC,INVALC,0x4265,
	0x4E61,0x304A,
	/* Range 0x4F09 - 0x4F11, array index: 0x0796 */
	0x5044,0x304B,INVALC,INVALC,0x3860,0x346C,0x497A,0x4832,
	0x3559,
	/* Range 0x4F4D - 0x4F5E, array index: 0x079F */
	0x304C,0x4463,0x3D3B,0x3A34,0x4D24,INVALC,0x424E,INVALC,
	0x323F,INVALC,0x5049,INVALC,0x4D3E,0x5045,0x5047,0x3A6E,
	0x5048,0x5524,
	/* Range 0x4FAB - 0x4FAF, array index: 0x07B1 */
	0x5525,INVALC,0x4B79,0x496E,0x3874,
	/* Range 0x4FC2 - 0x4FC4, array index: 0x07B6 */
	0x3738,0x4225,0x3264,
	/* Range 0x4FD7 - 0x4FE5, array index: 0x07B9 */
	0x422F,0x505A,INVALC,0x505D,0x505B,INVALC,0x4A5D,INVALC,
	0x5058,INVALC,0x3F2E,INVALC,0x4B73,0x505F,0x5060,
	/* Range 0x5021 - 0x502D, array index: 0x07C8 */
	0x5069,INVALC,0x4A6F,0x434D,0x5065,0x3771,INVALC,0x5062,
	0x506A,0x5064,0x4E51,0x506B,0x4F41,
	/* Range 0x5047 - 0x5049, array index: 0x07D5 */
	0x5071,0x5075,0x304E,
	/* Range 0x5072 - 0x5078, array index: 0x07D8 */
	0x3C45,INVALC,0x4226,0x4465,0x3676,INVALC,0x5079,
	/* Range 0x5098 - 0x509A, array index: 0x07DF */
	0x3B31,0x4877,0x507B,
	/* Range 0x50B2 - 0x50B7, array index: 0x07E2 */
	0x507E,0x5123,0x507D,0x3A44,INVALC,0x3D7D,
	/* Range 0x5100 - 0x5104, array index: 0x07E8 */
	0x3537,0x512E,0x512F,INVALC,0x322F,
	/* Range 0x5112 - 0x511A, array index: 0x07ED */
	0x3C74,INVALC,0x5132,0x5131,0x5130,INVALC,0x5056,INVALC,
	0x5133,
	/* Range 0x513A - 0x5154, array index: 0x07F6 */
	0x5135,0x5138,0x5137,INVALC,INVALC,0x5139,0x513A,0x3074,
	INVALC,0x3835,0x373B,0x3D3C,0x437B,0x3624,0x4068,0x3877,
	INVALC,0x396E,0x513C,0x4C48,0x4546,INVALC,0x3B79,INVALC,
	0x513B,INVALC,0x513D,
	/* Range 0x5168 - 0x516E, array index: 0x0811 */
	0x4134,0x5140,0x5141,0x482C,0x3878,0x4F3B,0x5142,
	/* Range 0x5175 - 0x5178, array index: 0x0818 */
	0x4A3C,0x4236,0x3671,0x4535,
	/* Range 0x5189 - 0x5199, array index: 0x081C */
	0x5147,0x3A7D,INVALC,0x5146,0x3A46,INVALC,0x5148,0x666E,
	0x5149,0x4B41,0x514A,INVALC,0x514B,0x514C,0x3E69,INVALC,
	0x3C4C,
	/* Range 0x51A2 - 0x51B7, array index: 0x082D */
	0x514F,INVALC,0x514D,0x4C3D,0x514E,INVALC,0x495A,0x5150,
	0x5151,0x5152,0x455F,INVALC,INVALC,INVALC,0x5156,0x5154,
	0x5155,0x5153,0x3A63,0x5157,0x4C6A,0x4E64,
	/* Range 0x51C4 - 0x51C6, array index: 0x0843 */
	0x4028,0x5159,0x3D5A,
	/* Range 0x51C9 - 0x51CD, array index: 0x0846 */
	0x515A,INVALC,0x437C,0x4E3F,0x4560,
	/* Range 0x51DB - 0x51E1, array index: 0x084B */
	0x515B,0x7425,0x3645,INVALC,INVALC,0x515C,0x4B5E,
	/* Range 0x51E6 - 0x51EA, array index: 0x0852 */
	0x3D68,0x427C,INVALC,0x515E,0x4664,
	/* Range 0x51F5 - 0x520B, array index: 0x0857 */
	0x5161,0x3627,INVALC,0x464C,0x317A,0x3D50,INVALC,INVALC,
	0x4821,0x5162,INVALC,0x4561,INVALC,INVALC,0x3F4F,0x5163,
	INVALC,0x4A2C,0x405A,0x3422,INVALC,0x3429,0x5164,
	/* Range 0x5236 - 0x523B, array index: 0x086E */
	0x4029,0x3A7E,0x3774,0x516B,0x3B49,0x396F,
	/* Range 0x524A - 0x524F, array index: 0x0874 */
	0x3A6F,0x516E,0x516F,0x4130,INVALC,0x516C,
	/* Range 0x5263 - 0x5265, array index: 0x087A */
	0x3775,0x3A5E,0x476D,
	/* Range 0x526F - 0x5275, array index: 0x087D */
	0x497B,0x3E6A,0x517B,0x3364,0x5175,0x5173,0x414F,
	/* Range 0x5287 - 0x5289, array index: 0x0884 */
	0x3760,0x517C,0x4E2D,
	/* Range 0x52A9 - 0x52AD, array index: 0x0887 */
	0x3D75,0x4558,0x3965,0x5222,0x5223,
	/* Range 0x52D5 - 0x52E7, array index: 0x088C */
	0x4630,INVALC,0x5228,0x342A,0x4C33,INVALC,INVALC,INVALC,
	0x3E21,0x5229,0x4A67,0x522D,INVALC,0x402A,0x522A,0x3650,
	INVALC,0x522B,0x342B,
	/* Range 0x52F8 - 0x52FA, array index: 0x089F */
	0x5230,0x5231,0x3C5B,
	/* Range 0x52FE - 0x5308, array index: 0x08A2 */
	0x387B,0x4C5E,INVALC,0x4C68,0x4677,INVALC,INVALC,0x4A71,
	0x5232,INVALC,0x5233,
	/* Range 0x5315 - 0x531A, array index: 0x08AD */
	0x5238,0x323D,0x4B4C,INVALC,0x3A7C,0x5239,
	/* Range 0x5338 - 0x534A, array index: 0x08B3 */
	0x523E,0x4924,0x3668,0x3065,INVALC,INVALC,INVALC,0x463F,
	0x523F,0x3D3D,INVALC,0x4069,INVALC,0x5241,0x5240,0x3E23,
	0x3861,0x5243,0x483E,
	/* Range 0x5351 - 0x5360, array index: 0x08C6 */
	0x485C,0x4234,0x426E,0x3628,INVALC,INVALC,0x466E,0x4331,
	INVALC,0x476E,INVALC,0x4B4E,INVALC,0x5246,INVALC,0x406A,
	/* Range 0x536E - 0x5378, array index: 0x08D6 */
	0x5248,0x312C,0x3075,0x346D,INVALC,0x4228,0x3551,0x4D71,
	INVALC,0x524B,0x3237,
	/* Range 0x53A5 - 0x53A9, array index: 0x08E1 */
	0x5250,0x524F,INVALC,0x3F5F,0x3139,
	/* Range 0x53C8 - 0x53CE, array index: 0x08E6 */
	0x4B74,0x3A35,0x355A,0x4D27,0x4150,0x483F,0x3C7D,
	/* Range 0x53DF - 0x53FA, array index: 0x08ED */
	0x5257,INVALC,0x3143,0x4151,0x387D,0x3845,0x3667,INVALC,
	INVALC,0x525B,0x4321,0x427E,0x362B,0x3E24,0x525C,0x525A,
	0x3244,0x4266,0x3C38,0x3B4B,0x3126,INVALC,INVALC,0x3370,
	0x3966,0x3B4A,INVALC,0x525D,
	/* Range 0x5408 - 0x5411, array index: 0x0909 */
	0x3967,0x3548,0x445F,0x3125,0x4631,0x4C3E,0x3921,0x4D79,
	0x4547,0x387E,
	/* Range 0x5429 - 0x542E, array index: 0x0913 */
	0x5266,INVALC,0x345E,0x5261,0x5262,0x5264,
	/* Range 0x5438 - 0x5442, array index: 0x0919 */
	0x355B,0x3F61,INVALC,0x4A2D,0x5263,0x525F,0x3863,INVALC,
	0x5260,INVALC,0x4F24,
	/* Range 0x5446 - 0x544A, array index: 0x0924 */
	0x4A72,INVALC,0x4468,0x3862,0x3970,
	/* Range 0x5473 - 0x547D, array index: 0x0929 */
	0x4C23,INVALC,0x526A,0x5273,0x526E,INVALC,INVALC,INVALC,
	0x5271,0x3846,0x4C3F,
	/* Range 0x548B - 0x5492, array index: 0x0934 */
	0x3A70,0x4F42,INVALC,0x526B,0x5269,0x5275,INVALC,0x5270,
	/* Range 0x54BC - 0x54C9, array index: 0x093C */
	0x5325,0x3076,0x5324,INVALC,0x3025,0x494A,0x5322,INVALC,
	0x527C,INVALC,INVALC,0x5277,0x527D,0x3A48,
	/* Range 0x54E5 - 0x54E9, array index: 0x094A */
	0x5327,0x5328,INVALC,0x3E25,0x4B69,
	/* Range 0x5544 - 0x5546, array index: 0x094F */
	0x426F,0x5336,0x3E26,
	/* Range 0x5587 - 0x558B, array index: 0x0952 */
	0x5349,INVALC,0x3922,0x533F,0x437D,
	/* Range 0x5598 - 0x559F, array index: 0x0957 */
	0x5343,0x533C,0x342D,INVALC,0x346E,0x3365,0x5344,0x5340,
	/* Range 0x55A7 - 0x55B0, array index: 0x095F */
	0x3776,0x534A,0x5348,0x4153,0x354A,0x362C,INVALC,0x5345,
	INVALC,0x3674,
	/* Range 0x5614 - 0x5618, array index: 0x0969 */
	0x5352,INVALC,0x5354,0x3E28,0x3133,
	/* Range 0x5668 - 0x566C, array index: 0x096E */
	0x346F,INVALC,0x5364,0x5360,0x5363,
	/* Range 0x56C0 - 0x56C3, array index: 0x0973 */
	0x5373,0x5371,0x536F,0x5372,
	/* Range 0x56D7 - 0x56DB, array index: 0x0977 */
	0x5378,0x5145,INVALC,0x3C7C,0x3B4D,
	/* Range 0x5726 - 0x5728, array index: 0x097C */
	0x5429,0x3035,0x3A5F,
	/* Range 0x574E - 0x5751, array index: 0x097F */
	0x542C,0x542F,0x3A41,0x3923,
	/* Range 0x57A0 - 0x57A4, array index: 0x0983 */
	0x5437,INVALC,0x3924,0x3340,0x5439,
	/* Range 0x57D2 - 0x57D6, array index: 0x0988 */
	0x543F,0x5440,0x543E,INVALC,0x5442,
	/* Range 0x582F - 0x5835, array index: 0x098D */
	0x7421,0x3161,0x4A73,INVALC,INVALC,0x3E6C,0x4548,
	/* Range 0x5857 - 0x585A, array index: 0x0994 */
	0x4549,0x4564,0x4839,0x444D,
	/* Range 0x58B8 - 0x58BB, array index: 0x0998 */
	0x5457,0x5451,0x5454,0x5456,
	/* Range 0x58D5 - 0x58D9, array index: 0x099C */
	0x3968,INVALC,0x545C,0x545E,0x545D,
	/* Range 0x58EB - 0x58F2, array index: 0x09A1 */
	0x3B4E,0x3F51,INVALC,0x4154,0x5463,0x403C,0x306D,0x4764,
	/* Range 0x58F7 - 0x58FD, array index: 0x09A9 */
	0x445B,INVALC,0x5465,0x5464,0x5466,0x5467,0x5468,
	/* Range 0x5915 - 0x591C, array index: 0x09B0 */
	0x4D3C,0x3330,INVALC,0x5249,0x3D48,0x423F,0x546C,0x4C6B,
	/* Range 0x5927 - 0x5932, array index: 0x09B8 */
	0x4267,INVALC,0x4537,0x4240,0x4957,0x546F,0x5470,0x317B,
	INVALC,INVALC,0x3C3A,0x5471,
	/* Range 0x5947 - 0x5949, array index: 0x09C4 */
	0x3471,0x4660,0x4A74,
	/* Range 0x594E - 0x595A, array index: 0x09C7 */
	0x5477,0x4155,0x5476,0x3740,INVALC,INVALC,0x4B5B,0x5475,
	INVALC,0x4565,0x5479,INVALC,0x5478,
	/* Range 0x5965 - 0x596E, array index: 0x09D4 */
	0x317C,INVALC,0x547C,0x3E29,0x547E,0x4325,INVALC,0x547D,
	INVALC,0x4A33,
	/* Range 0x5981 - 0x5984, array index: 0x09DE */
	0x5522,0x4721,0x485E,0x4C51,
	/* Range 0x59D0 - 0x59D4, array index: 0x09E2 */
	0x3039,0x3848,INVALC,0x402B,0x3051,
	/* Range 0x5ABC - 0x5AC2, array index: 0x09E7 */
	0x553C,0x5540,0x553D,INVALC,INVALC,0x3247,0x553F,
	/* Range 0x5B54 - 0x5B66, array index: 0x09EE */
	0x3926,0x5554,INVALC,0x3B7A,0x4238,INVALC,0x5555,0x5556,
	0x3B5A,0x3927,INVALC,0x4C52,INVALC,INVALC,INVALC,0x3528,
	0x3849,0x5557,0x3358,
	/* Range 0x5B85 - 0x5B8F, array index: 0x0A01 */
	0x4270,INVALC,0x3127,0x3C69,0x3042,INVALC,0x4157,0x3430,
	0x3C35,INVALC,0x3928,
	/* Range 0x5B95 - 0x5BA6, array index: 0x0A0C */
	0x4566,INVALC,0x3D21,0x3431,0x4368,0x446A,0x3038,0x3539,
	0x4A75,INVALC,0x3C42,INVALC,INVALC,0x3552,0x406B,0x3C3C,
	0x4D28,0x5561,
	/* Range 0x5BB3 - 0x5BB9, array index: 0x0A1E */
	0x3332,0x3163,0x3E2C,0x3248,INVALC,0x5562,0x4D46,
	/* Range 0x5BC2 - 0x5BC9, array index: 0x0A25 */
	0x3C64,0x5563,0x3473,0x4652,0x4C29,0x5564,INVALC,0x5565,
	/* Range 0x5BD0 - 0x5BD4, array index: 0x0A2D */
	0x5567,INVALC,0x3428,0x3677,0x5566,
	/* Range 0x5BDB - 0x5BEB, array index: 0x0A32 */
	0x3432,INVALC,0x3F32,0x556B,0x3B21,INVALC,0x3249,0x556A,
	INVALC,0x5568,0x556C,0x5569,0x472B,0x5C4D,0x3F33,INVALC,
	0x556D,
	/* Range 0x5BFE - 0x5C13, array index: 0x0A43 */
	0x4250,0x3C77,INVALC,0x4975,0x406C,INVALC,0x3C4D,0x5571,
	0x3E2D,0x5572,0x5573,0x3053,0x423A,0x3F52,INVALC,0x5574,
	0x4633,0x3E2E,INVALC,0x3E2F,INVALC,0x5575,
	/* Range 0x5C38 - 0x5C41, array index: 0x0A59 */
	0x5579,0x557A,0x3C5C,0x3F2C,0x4674,0x3F54,0x4878,0x4722,
	0x3649,0x557B,
	/* Range 0x5C4A - 0x5C55, array index: 0x0A63 */
	0x464F,0x3230,INVALC,0x3B53,0x557D,0x5622,0x5621,0x367D,
	INVALC,0x557E,INVALC,0x4538,
	/* Range 0x5CA8 - 0x5CAC, array index: 0x0A6F */
	0x413B,0x3464,INVALC,0x562D,0x4C28,
	/* Range 0x5CB6 - 0x5CBE, array index: 0x0A74 */
	0x562F,0x5631,0x345F,INVALC,INVALC,0x562E,0x5630,INVALC,
	0x5633,
	/* Range 0x5CE8 - 0x5CEA, array index: 0x0A7D */
	0x3265,0x5636,0x563B,
	/* Range 0x5D14 - 0x5D1B, array index: 0x0A80 */
	0x5643,0x563D,0x3333,0x563E,0x5647,0x5646,0x5645,0x5641,
	/* Range 0x5DE3 - 0x5DE8, array index: 0x0A88 */
	0x4163,INVALC,0x3929,0x3A38,0x392A,0x3570,
	/* Range 0x5DF1 - 0x5DF7, array index: 0x0A8E */
	0x384A,0x5661,0x4C26,0x4743,0x5662,INVALC,0x392B,
	/* Range 0x5E19 - 0x5E1D, array index: 0x0A95 */
	0x5665,0x5664,0x5667,INVALC,0x446B,
	/* Range 0x5E36 - 0x5E38, array index: 0x0A9A */
	0x5668,0x5669,0x3E6F,
	/* Range 0x5E43 - 0x5E47, array index: 0x0A9D */
	0x566B,0x566A,0x497D,INVALC,0x5673,
	/* Range 0x5E5F - 0x5E64, array index: 0x0AA2 */
	0x5670,INVALC,0x4828,0x5671,0x4A3E,0x5672,
	/* Range 0x5E72 - 0x5E84, array index: 0x0AA8 */
	0x3433,0x4A3F,0x472F,0x5674,0x5675,INVALC,0x392C,0x3434,
	0x5676,0x3838,0x4D44,0x4D29,0x3476,0x5678,INVALC,0x4423,
	INVALC,0x392D,0x3E31,
	/* Range 0x5E95 - 0x5E97, array index: 0x0ABB */
	0x446C,0x4A79,0x4539,
	/* Range 0x5EB5 - 0x5EB8, array index: 0x0ABE */
	0x3043,0x3D6E,0x392F,0x4D47,
	/* Range 0x5EC1 - 0x5EC3, array index: 0x0AC2 */
	0x567A,0x567B,0x4751,
	/* Range 0x5EC8 - 0x5ECA, array index: 0x0AC5 */
	0x567C,0x4E77,0x4F2D,
	/* Range 0x5EDD - 0x5EE3, array index: 0x0AC8 */
	0x5723,INVALC,0x4940,0x3E33,0x5727,0x5726,0x5722,
	/* Range 0x5EF0 - 0x5F04, array index: 0x0ACF */
	0x572D,0x572B,INVALC,0x572C,0x572E,INVALC,0x3164,0x446E,
	0x572F,INVALC,0x377A,0x3276,0x4736,INVALC,0x5730,0x467B,
	INVALC,0x4A5B,INVALC,0x5731,0x4F2E,
	/* Range 0x5F09 - 0x5F18, array index: 0x0AE4 */
	0x5732,0x4A40,0x5735,0x5021,0x5031,INVALC,0x3C30,0x4675,
	0x5736,INVALC,0x355D,0x4424,0x307A,0x5737,0x4A26,0x3930,
	/* Range 0x5F25 - 0x5F29, array index: 0x0AF4 */
	0x4C6F,0x3839,0x384C,INVALC,0x5738,
	/* Range 0x5F69 - 0x5F73, array index: 0x0AF9 */
	0x3A4C,0x4937,0x4426,0x494B,0x5745,INVALC,INVALC,0x3E34,
	0x3146,INVALC,0x5746,
	/* Range 0x5F7F - 0x5F99, array index: 0x0B04 */
	0x574A,0x317D,0x402C,0x5749,0x5748,0x3742,0x4254,INVALC,
	0x574E,0x574C,INVALC,0x574B,0x4E27,0x3865,INVALC,INVALC,
	INVALC,0x3D79,0x574D,0x454C,0x3D3E,INVALC,INVALC,INVALC,
	0x4640,0x5751,0x5750,
	/* Range 0x5FA8 - 0x5FAE, array index: 0x0B1F */
	0x5753,0x497C,0x3D5B,INVALC,INVALC,0x5754,0x4879,
	/* Range 0x5FD6 - 0x5FDD, array index: 0x0B26 */
	0x5756,0x3B56,0x4B3A,0x4B3B,INVALC,INVALC,0x317E,0x575B,
	/* Range 0x600E - 0x6016, array index: 0x0B2E */
	0x5763,0x5769,0x5761,INVALC,0x455C,INVALC,INVALC,0x5766,
	0x495D,
	/* Range 0x6019 - 0x602B, array index: 0x0B37 */
	0x5760,INVALC,0x5765,0x4E67,0x3B57,INVALC,INVALC,0x4255,
	0x575E,INVALC,INVALC,INVALC,0x355E,0x5768,0x402D,0x3165,
	0x5762,0x3278,0x5767,
	/* Range 0x6041 - 0x6043, array index: 0x0B4A */
	0x576C,0x5776,0x5774,
	/* Range 0x605F - 0x6070, array index: 0x0B4D */
	0x576F,0x575F,INVALC,0x327A,0x5773,0x5775,0x4351,INVALC,
	INVALC,0x3A28,0x3238,0x576D,0x5778,0x5777,0x3633,INVALC,
	0x4229,0x3366,
	/* Range 0x6089 - 0x608D, array index: 0x0B5F */
	0x3C3D,INVALC,0x5827,0x4470,0x577B,
	/* Range 0x60A6 - 0x60AA, array index: 0x0B64 */
	0x3159,0x5826,INVALC,0x473A,0x302D,
	/* Range 0x60B2 - 0x60B8, array index: 0x0B69 */
	0x4861,0x575C,0x582C,0x5830,0x4C65,INVALC,0x5829,
	/* Range 0x60C5 - 0x60C7, array index: 0x0B70 */
	0x3E70,0x582F,0x4657,
	/* Range 0x60DF - 0x60E3, array index: 0x0B73 */
	0x3054,0x582A,0x5828,INVALC,0x415A,
	/* Range 0x60F0 - 0x60FB, array index: 0x0B78 */
	0x4246,0x583D,INVALC,0x415B,0x5838,INVALC,0x5835,0x5836,
	INVALC,0x3C66,0x5839,0x583C,
	/* Range 0x610D - 0x610F, array index: 0x0B84 */
	0x583E,0x583F,0x3055,
	/* Range 0x613C - 0x613F, array index: 0x0B87 */
	0x5846,0x5849,0x5841,0x5845,
	/* Range 0x6147 - 0x614E, array index: 0x0B8B */
	0x5840,0x3B7C,INVALC,0x5844,0x4256,0x3932,0x5832,0x3F35,
	/* Range 0x6158 - 0x615A, array index: 0x0B93 */
	0x584E,0x584F,0x5850,
	/* Range 0x616E - 0x6177, array index: 0x0B96 */
	0x4E38,0x5853,0x3056,0x5855,INVALC,0x584C,0x5852,0x5859,
	0x3744,0x584D,
	/* Range 0x61C6 - 0x61CD, array index: 0x0BA0 */
	0x586A,0x3A29,0x5868,0x5866,0x5865,0x586C,0x5864,0x586E,
	/* Range 0x61F4 - 0x6200, array index: 0x0BA8 */
	0x5873,INVALC,0x5871,0x5867,0x377C,INVALC,0x5872,INVALC,
	0x5876,0x5875,0x5877,0x5874,0x5878,
	/* Range 0x6208 - 0x6216, array index: 0x0BB5 */
	0x5879,0x587A,0x4A6A,INVALC,0x587C,0x587B,0x3D3F,INVALC,
	0x402E,0x3266,0x327C,INVALC,0x587D,INVALC,0x303F,
	/* Range 0x621A - 0x6221, array index: 0x0BC4 */
	0x404C,0x587E,INVALC,0x6C43,0x5921,0x3761,INVALC,0x5922,
	/* Range 0x622E - 0x6234, array index: 0x0BCC */
	0x5924,0x353A,0x5925,INVALC,0x5926,0x5927,0x4257,
	/* Range 0x623F - 0x6241, array index: 0x0BD3 */
	0x4B3C,0x3D6A,0x5928,
	/* Range 0x6247 - 0x624E, array index: 0x0BD6 */
	0x4070,0x6E3D,0x4862,INVALC,0x3C6A,INVALC,0x3A4D,0x5929,
	/* Range 0x627C - 0x6284, array index: 0x0BDE */
	0x592F,INVALC,0x5932,0x3E35,0x353B,INVALC,0x5930,0x5937,
	0x3E36,
	/* Range 0x6291 - 0x629E, array index: 0x0BE7 */
	0x4D5E,0x5933,0x5934,0x5938,0x456A,0x5935,0x3933,0x405E,
	INVALC,INVALC,0x5946,0x4834,INVALC,0x4272,
	/* Range 0x62B9 - 0x62BD, array index: 0x0BF5 */
	0x4B75,INVALC,0x593B,0x3221,0x436A,
	/* Range 0x62C5 - 0x62E1, array index: 0x0BFA */
	0x4334,0x593E,0x5945,0x5940,0x5947,0x5943,INVALC,0x5942,
	0x476F,INVALC,0x593C,0x327D,0x593A,0x3571,0x4273,0x5936,
	INVALC,INVALC,0x5939,0x3934,0x405B,INVALC,0x3E37,0x5941,
	0x4752,INVALC,INVALC,0x3572,0x3348,
	/* Range 0x62EC - 0x62F7, array index: 0x0C17 */
	0x3367,0x3F21,0x5949,0x594E,INVALC,0x594A,INVALC,0x377D,
	INVALC,0x594F,0x3B22,0x3969,
	/* Range 0x62FE - 0x6302, array index: 0x0C23 */
	0x3D26,0x593D,INVALC,0x3B7D,0x594C,
	/* Range 0x6307 - 0x6309, array index: 0x0C28 */
	0x3B58,0x594D,0x3044,
	/* Range 0x633D - 0x633F, array index: 0x0C2B */
	0x4854,0x5951,0x415E,
	/* Range 0x634C - 0x6350, array index: 0x0C2E */
	0x3B2B,0x5952,INVALC,0x5954,0x5950,
	/* Range 0x6367 - 0x636B, array index: 0x0C33 */
	0x4A7B,0x3C4E,0x5960,INVALC,0x595F,
	/* Range 0x639F - 0x63AC, array index: 0x0C38 */
	0x595D,0x4E2B,0x3A4E,0x4335,0x595A,INVALC,0x405C,INVALC,
	0x3935,0x3F64,0x3166,0x413C,0x5958,0x3545,
	/* Range 0x64E0 - 0x64E7, array index: 0x0C46 */
	0x5A2B,0x5A2C,0x4527,0x5A2E,INVALC,INVALC,0x3B24,0x5A29,
	/* Range 0x652A - 0x652C, array index: 0x0C4E */
	0x5978,0x5A3C,0x5A30,
	/* Range 0x6534 - 0x653F, array index: 0x0C51 */
	0x5A3D,0x5A3E,0x5A40,0x5A3F,0x5A41,0x327E,INVALC,0x3936,
	INVALC,INVALC,0x4A7C,0x402F,
	/* Range 0x6555 - 0x6559, array index: 0x0C5D */
	0x5A45,0x5A44,0x4754,0x5A47,0x3635,
	/* Range 0x6574 - 0x6578, array index: 0x0C62 */
	0x4030,0x4528,INVALC,0x495F,0x5A4B,
	/* Range 0x6587 - 0x6589, array index: 0x0C67 */
	0x4A38,0x555D,0x4046,
	/* Range 0x65AB - 0x65B0, array index: 0x0C6A */
	0x5A51,0x3B42,0x4347,INVALC,0x3B5B,0x3F37,
	/* Range 0x65C1 - 0x65C6, array index: 0x0C70 */
	0x5A55,INVALC,0x5A53,0x5A56,0x4E39,0x5A54,
	/* Range 0x65E0 - 0x65ED, array index: 0x0C76 */
	0x5A5B,0x5A5C,0x347B,INVALC,INVALC,0x467C,0x4336,0x356C,
	0x3B5D,0x4161,INVALC,INVALC,0x3D5C,0x3030,
	/* Range 0x6634 - 0x6636, array index: 0x0C84 */
	0x5A65,0x5A63,0x5A64,
	/* Range 0x663F - 0x6644, array index: 0x0C87 */
	0x5B26,INVALC,0x5A6A,0x3B7E,0x3938,0x5A68,
	/* Range 0x665D - 0x665F, array index: 0x0C8D */
	0x5A6C,0x5A6B,0x5A70,
	/* Range 0x6664 - 0x6669, array index: 0x0C90 */
	0x5A6D,INVALC,0x3322,0x5A6E,0x5A6F,0x4855,
	/* Range 0x666E - 0x6670, array index: 0x0C96 */
	0x4961,0x374A,0x5A72,
	/* Range 0x6683 - 0x6689, array index: 0x0C99 */
	0x5A73,0x5A77,INVALC,INVALC,0x324B,0x5A74,0x5A76,
	/* Range 0x6696 - 0x6698, array index: 0x0CA0 */
	0x4348,0x3045,0x5A78,
	/* Range 0x66D9 - 0x66DD, array index: 0x0CA3 */
	0x3D6C,0x5B24,INVALC,0x4D4B,0x4778,
	/* Range 0x66F0 - 0x6700, array index: 0x0CA8 */
	0x5B29,INVALC,0x364A,0x3148,0x3939,0x5B2A,INVALC,0x5B2B,
	0x3D71,0x4162,INVALC,INVALC,0x5258,0x413E,0x413D,0x4258,
	0x3A47,
	/* Range 0x6714 - 0x6717, array index: 0x0CB9 */
	0x3A73,0x443F,0x5B2D,0x4F2F,
	/* Range 0x671B - 0x671F, array index: 0x0CBD */
	0x4B3E,INVALC,0x442B,0x5B2E,0x347C,
	/* Range 0x6726 - 0x672E, array index: 0x0CC2 */
	0x5B2F,0x5B30,0x4C5A,INVALC,0x4C24,0x4B76,0x4B5C,0x3B25,
	0x5B32,
	/* Range 0x6734 - 0x673A, array index: 0x0CCB */
	0x4B51,INVALC,0x5B34,0x5B37,0x5B36,INVALC,0x3479,
	/* Range 0x674E - 0x6753, array index: 0x0CD2 */
	0x4D7B,0x3049,0x3A60,0x423C,INVALC,0x3C5D,
	/* Range 0x675C - 0x6765, array index: 0x0CD8 */
	0x454E,INVALC,0x5B39,0x422B,0x5B3A,0x3E72,0x4C5D,0x5B3C,
	0x5B3D,0x4D68,
	/* Range 0x676D - 0x6777, array index: 0x0CE2 */
	0x393A,INVALC,0x4755,0x5B3F,0x456C,0x5A5E,0x5A62,INVALC,
	0x354F,INVALC,0x4747,
	/* Range 0x679C - 0x67A2, array index: 0x0CED */
	0x324C,0x3B5E,INVALC,INVALC,0x4F48,0x5B46,0x3F75,
	/* Range 0x67B3 - 0x67B9, array index: 0x0CF4 */
	0x5B4C,0x5B4A,INVALC,0x324D,0x5B48,0x5B4E,0x5B54,
	/* Range 0x67CE - 0x67D4, array index: 0x0CFB */
	0x5B55,0x4770,0x4B3F,0x343B,INVALC,0x4077,0x3D40,
	/* Range 0x67F1 - 0x67F5, array index: 0x0D02 */
	0x436C,INVALC,0x4C78,0x3C46,0x3A74,
	/* Range 0x67FE - 0x6804, array index: 0x0D07 */
	0x4B6F,0x3341,INVALC,INVALC,0x444E,0x464A,0x3149,
	/* Range 0x6829 - 0x682B, array index: 0x0D0E */
	0x5B5B,0x3374,0x5B61,
	/* Range 0x683C - 0x6843, array index: 0x0D11 */
	0x334A,0x3A4F,INVALC,INVALC,0x5B5C,0x3765,0x374B,0x456D,
	/* Range 0x684D - 0x6854, array index: 0x0D19 */
	0x5B5D,0x5B5F,INVALC,0x364D,0x372C,INVALC,0x343C,0x354B,
	/* Range 0x68A6 - 0x68A8, array index: 0x0D21 */
	0x546D,0x3868,0x4D7C,
	/* Range 0x68AD - 0x68BC, array index: 0x0D24 */
	0x5B68,INVALC,0x4474,0x3323,0x3A2D,INVALC,0x5B60,INVALC,
	0x5B70,0x3361,INVALC,INVALC,0x5B6E,0x5B72,INVALC,0x456E,
	/* Range 0x68C9 - 0x68CD, array index: 0x0D34 */
	0x4C49,0x5B77,0x347D,INVALC,0x5B7E,
	/* Range 0x68D4 - 0x68DA, array index: 0x0D39 */
	0x5C21,0x5C23,INVALC,0x5C27,0x5B79,INVALC,0x432A,
	/* Range 0x68DF - 0x68E3, array index: 0x0D40 */
	0x456F,0x5C2B,0x5B7C,INVALC,0x5C28,
	/* Range 0x690B - 0x690F, array index: 0x0D45 */
	0x4C3A,0x5B7D,0x3F22,0x4447,0x5B73,
	/* Range 0x6919 - 0x691C, array index: 0x0D4A */
	0x3F7A,0x5C2F,0x3371,0x3821,
	/* Range 0x6921 - 0x692A, array index: 0x0D4E */
	0x5C31,0x5B7A,0x5C30,INVALC,0x5C29,0x5B7B,INVALC,0x5C2D,
	INVALC,0x5C2E,
	/* Range 0x6953 - 0x6955, array index: 0x0D58 */
	0x4976,0x5C38,0x424A,
	/* Range 0x6959 - 0x6962, array index: 0x0D5B */
	0x5C3E,0x413F,INVALC,0x5C35,0x5C42,0x5C41,INVALC,0x466F,
	0x5C40,0x466A,
	/* Range 0x696A - 0x6982, array index: 0x0D65 */
	0x5C44,0x5C37,INVALC,0x3648,0x5C3A,0x3D5D,INVALC,INVALC,
	INVALC,0x4760,0x5C3C,0x364B,INVALC,0x5C34,0x5C36,0x5C33,
	INVALC,INVALC,0x4F30,0x335A,0x5C39,INVALC,INVALC,0x5C43,
	0x3335,
	/* Range 0x69CA - 0x69D0, array index: 0x0D7E */
	0x5C4E,0x393D,0x4448,0x4164,0x5C4C,INVALC,0x5C47,
	/* Range 0x6A0A - 0x6A0C, array index: 0x0D85 */
	0x5C68,0x4875,0x5C6E,
	/* Range 0x6A12 - 0x6A14, array index: 0x0D88 */
	0x5C69,0x5C6C,0x5C66,
	/* Range 0x6A1E - 0x6A23, array index: 0x0D8B */
	0x5C64,0x3E40,INVALC,0x4C4F,0x5C78,0x5C6B,
	/* Range 0x6A29 - 0x6A2B, array index: 0x0D91 */
	0x3822,0x3223,0x335F,
	/* Range 0x6A35 - 0x6A3A, array index: 0x0D94 */
	0x3E41,0x5C70,INVALC,0x5C77,0x3C79,0x3372,
	/* Range 0x6AC1 - 0x6AC3, array index: 0x0D9A */
	0x5C6A,0x5D25,0x5D24,
	/* Range 0x6B1D - 0x6B23, array index: 0x0D9D */
	0x3135,INVALC,0x5D36,0x3767,0x3C21,INVALC,0x3655,
	/* Range 0x6B37 - 0x6B3E, array index: 0x0DA4 */
	0x5D38,0x5D37,0x5D3A,0x353D,INVALC,INVALC,0x3656,0x343E,
	/* Range 0x6B5F - 0x6B6A, array index: 0x0DAC */
	0x5D43,INVALC,0x5D44,0x3B5F,0x4035,0x3A21,INVALC,0x4970,
	INVALC,INVALC,0x4A62,0x4F44,
	/* Range 0x6B89 - 0x6B8D, array index: 0x0DB8 */
	0x3D5E,0x3C6C,0x3B44,INVALC,0x5D4B,
	/* Range 0x6BAF - 0x6BC0, array index: 0x0DBD */
	0x5D52,INVALC,0x5D54,0x5D53,0x5D55,0x3225,0x434A,INVALC,
	0x5D56,INVALC,INVALC,0x3B26,0x334C,0x5D57,INVALC,INVALC,
	0x4542,0x544C,
	/* Range 0x6BD2 - 0x6BD4, array index: 0x0DCF */
	0x4647,0x5D5A,0x4866,
	/* Range 0x6C3E - 0x6C42, array index: 0x0DD2 */
	0x4845,INVALC,0x4475,0x3D41,0x3561,
	/* Range 0x6C5D - 0x6C62, array index: 0x0DD7 */
	0x4672,0x5D67,0x393E,0x4353,INVALC,0x5D69,
	/* Range 0x6C7D - 0x6C83, array index: 0x0DDD */
	0x3525,0x5D70,INVALC,INVALC,0x5D6E,0x5D6B,0x4D60,
	/* Range 0x6C99 - 0x6C9B, array index: 0x0DE4 */
	0x3A3B,0x5D6D,0x5D6F,
	/* Range 0x6CB8 - 0x6CC5, array index: 0x0DE7 */
	0x4A28,0x4C7D,0x5E21,0x3C23,0x3E42,0x5D78,0x5D7E,0x3168,
	INVALC,0x3637,INVALC,INVALC,0x5D75,0x5D7A,
	/* Range 0x6CE1 - 0x6CE5, array index: 0x0DF5 */
	0x4B22,0x4748,0x3563,INVALC,0x4525,
	/* Range 0x6CEF - 0x6CF3, array index: 0x0DFA */
	0x5E23,0x4259,0x5D76,INVALC,0x314B,
	/* Range 0x6D29 - 0x6D2B, array index: 0x0DFF */
	0x314C,0x393F,0x5E29,
	/* Range 0x6D32 - 0x6D38, array index: 0x0E02 */
	0x3D27,0x5E2E,INVALC,0x5E2D,0x5E28,INVALC,0x5E2B,
	/* Range 0x6D77 - 0x6D79, array index: 0x0E09 */
	0x3324,0x3F3B,0x5E35,
	/* Range 0x6DC5 - 0x6DC7, array index: 0x0E0C */
	0x5E48,0x5E42,0x5E3F,
	/* Range 0x6DE8 - 0x6DEE, array index: 0x0E0F */
	0x5E46,INVALC,0x5E4D,0x307C,0x5E43,INVALC,0x5E4E,
	/* Range 0x6DF7 - 0x6DFB, array index: 0x0E16 */
	0x3A2E,INVALC,0x5E3B,0x5E49,0x453A,
	/* Range 0x6E05 - 0x6E0B, array index: 0x0E1B */
	0x4036,INVALC,0x3369,0x3A51,0x3E44,0x5E3D,0x3D42,
	/* Range 0x6E19 - 0x6E26, array index: 0x0E22 */
	0x5E52,0x3D6D,0x383A,INVALC,0x5E61,INVALC,0x5E5B,0x3574,
	0x454F,INVALC,0x5E56,0x5E5F,0x302F,0x3132,
	/* Range 0x6E29 - 0x6E2F, array index: 0x0E30 */
	0x3239,INVALC,0x5E58,0x422C,0x5E4F,0x5E51,0x3941,
	/* Range 0x6E7E - 0x6E82, array index: 0x0E37 */
	0x4F51,0x3C3E,0x4B7E,INVALC,0x5E63,
	/* Range 0x6ED1 - 0x6ED5, array index: 0x0E3C */
	0x336A,INVALC,0x5E68,0x5E6D,0x5E6E,
	/* Range 0x6EFE - 0x6F02, array index: 0x0E41 */
	0x5E78,0x5E60,INVALC,0x3579,0x493A,
	/* Range 0x6F11 - 0x6F15, array index: 0x0E46 */
	0x5E74,INVALC,0x5F22,0x3169,0x4166,
	/* Range 0x6F6D - 0x6F70, array index: 0x0E4B */
	0x5F2C,0x442C,0x5F29,0x4459,
	/* Range 0x6F80 - 0x6F86, array index: 0x0E4F */
	0x5F28,0x5F27,0x5F2D,INVALC,0x4021,INVALC,0x5F24,
	/* Range 0x6FC0 - 0x6FC3, array index: 0x0E56 */
	0x3763,0x4279,0x5F32,0x473B,
	/* Range 0x6FDF - 0x6FE1, array index: 0x0E5A */
	0x5F3B,0x396A,0x4728,
	/* Range 0x6FEB - 0x6FF3, array index: 0x0E5D */
	0x4D74,0x5F3D,INVALC,0x5F41,0x4275,INVALC,0x5F40,INVALC,
	0x5F2B,
	/* Range 0x701A - 0x701F, array index: 0x0E66 */
	0x5F4B,0x5F4A,INVALC,0x5F4D,0x4654,0x5F4F,
	/* Range 0x70AC - 0x70AF, array index: 0x0E6C */
	0x5F59,0x433A,0x5F5C,0x5F57,
	/* Range 0x70B8 - 0x70BA, array index: 0x0E70 */
	0x5F5A,0x4540,0x3059,
	/* Range 0x7162 - 0x7169, array index: 0x0E73 */
	0x5F69,INVALC,0x4761,0x5F65,0x5F68,0x3E48,INVALC,0x4851,
	/* Range 0x71E5 - 0x71E7, array index: 0x0E7B */
	0x4167,0x3B38,0x5F7C,
	/* Range 0x71EC - 0x71EE, array index: 0x0E7E */
	0x5F7B,0x3F24,0x5259,
	/* Range 0x723A - 0x7240, array index: 0x0E81 */
	0x4C6C,0x602B,0x602C,0x4156,0x3C24,0x602D,0x602E,
	/* Range 0x7246 - 0x724C, array index: 0x0E88 */
	0x602F,0x4A52,0x4847,INVALC,INVALC,0x6030,0x4757,
	/* Range 0x727D - 0x7282, array index: 0x0E8F */
	0x3823,0x6033,INVALC,0x3A54,0x6035,0x6034,
	/* Range 0x72C2 - 0x72C6, array index: 0x0E95 */
	0x3638,0x603D,0x603F,INVALC,0x603E,
	/* Range 0x72E0 - 0x72E2, array index: 0x0E9A */
	0x6043,0x6044,0x6042,
	/* Range 0x72F7 - 0x72FD, array index: 0x0E9D */
	0x6046,0x432C,0x6045,INVALC,INVALC,0x4F35,0x4762,
	/* Range 0x731B - 0x731F, array index: 0x0EA4 */
	0x4C54,0x604A,0x604C,INVALC,0x4E44,
	/* Range 0x7329 - 0x732F, array index: 0x0EA9 */
	0x604F,0x4376,0x472D,INVALC,INVALC,0x3825,0x604E,
	/* Range 0x7432 - 0x7436, array index: 0x0EB0 */
	0x606A,0x4E56,0x3657,0x487C,0x474A,
	/* Range 0x7459 - 0x7464, array index: 0x0EB5 */
	0x606F,0x386A,0x314D,0x6071,INVALC,0x3F70,0x606E,0x4E5C,
	INVALC,INVALC,0x6074,0x7424,
	/* Range 0x74EE - 0x74F8, array index: 0x0EC1 */
	0x6125,INVALC,0x6127,0x6128,0x6126,INVALC,INVALC,INVALC,
	0x4953,0x612A,0x6129,
	/* Range 0x7503 - 0x7505, array index: 0x0ECC */
	0x612C,0x612B,0x612D,
	/* Range 0x750C - 0x750E, array index: 0x0ECF */
	0x612E,0x6130,0x612F,
	/* Range 0x7530 - 0x7533, array index: 0x0ED2 */
	0x4544,0x4D33,0x3943,0x3F3D,
	/* Range 0x7537 - 0x753C, array index: 0x0ED6 */
	0x434B,0x5234,INVALC,0x442E,0x3268,0x6136,
	/* Range 0x7549 - 0x7551, array index: 0x0EDC */
	0x613A,0x6139,0x5A42,0x3326,0x6138,INVALC,0x305A,INVALC,
	0x482A,
	/* Range 0x7559 - 0x755D, array index: 0x0EE5 */
	0x4E31,0x613D,0x613B,0x435C,0x4026,
	/* Range 0x7562 - 0x756D, array index: 0x0EEA */
	0x492D,INVALC,0x613F,0x4E2C,0x374D,0x6140,INVALC,0x613E,
	0x4856,0x6141,INVALC,0x6142,
	/* Range 0x7573 - 0x7578, array index: 0x0EF6 */
	0x3E76,0x6147,INVALC,0x6144,0x466D,0x6143,
	/* Range 0x7586 - 0x7591, array index: 0x0EFC */
	0x6145,0x6146,INVALC,0x6149,0x6148,0x4925,INVALC,INVALC,
	0x4142,0x4141,INVALC,0x353F,
	/* Range 0x75B1 - 0x75BE, array index: 0x0F08 */
	0x6157,0x4868,0x6151,INVALC,0x6153,INVALC,INVALC,0x6155,
	0x3F3E,INVALC,INVALC,0x6156,0x6154,0x3C40,
	/* Range 0x75F0 - 0x75F4, array index: 0x0F16 */
	0x6162,INVALC,0x6164,0x6165,0x4354,
	/* Range 0x761F - 0x7624, array index: 0x0F1B */
	0x6169,0x616B,0x616C,0x616D,INVALC,0x616E,
	/* Range 0x7646 - 0x7648, array index: 0x0F21 */
	0x6174,0x6172,0x6173,
	/* Range 0x7667 - 0x766C, array index: 0x0F24 */
	0x617C,0x6179,0x617A,0x617B,INVALC,0x617D,
	/* Range 0x7678 - 0x7688, array index: 0x0F2A */
	0x6223,INVALC,0x482F,0x4550,0x6224,0x4772,0x4934,INVALC,
	0x6225,INVALC,INVALC,0x6226,0x452A,INVALC,0x3327,0x3944,
	0x6227,
	/* Range 0x76B7 - 0x76BA, array index: 0x0F3B */
	0x7369,0x6230,0x6231,0x6232,
	/* Range 0x76DB - 0x76E7, array index: 0x0F3F */
	0x4039,0x5D39,INVALC,0x6237,0x4C41,INVALC,0x6238,INVALC,
	0x3446,0x4857,0x6239,INVALC,0x623A,
	/* Range 0x7707 - 0x770C, array index: 0x0F4C */
	0x623F,0x623E,0x487D,INVALC,0x3447,0x3829,
	/* Range 0x771E - 0x7726, array index: 0x0F52 */
	0x6243,0x3F3F,0x4C32,INVALC,INVALC,INVALC,0x6242,0x6244,
	0x6245,
	/* Range 0x77B9 - 0x77BF, array index: 0x0F5B */
	0x6259,INVALC,0x625D,0x625B,0x625C,INVALC,0x625A,
	/* Range 0x77DA - 0x77DC, array index: 0x0F62 */
	0x6261,0x4C37,0x6262,
	/* Range 0x77ED - 0x77EF, array index: 0x0F65 */
	0x433B,0x6264,0x363A,
	/* Range 0x7825 - 0x7827, array index: 0x0F68 */
	0x4556,0x3A56,0x354E,
	/* Range 0x788C - 0x788E, array index: 0x0F6B */
	0x6271,0x3337,0x626C,
	/* Range 0x792A - 0x792C, array index: 0x0F6E */
	0x626A,0x632A,0x6329,
	/* Range 0x7947 - 0x7949, array index: 0x0F71 */
	0x3540,0x3527,0x3B63,
	/* Range 0x7953 - 0x7957, array index: 0x0F74 */
	0x6331,INVALC,0x6330,0x4144,0x632D,
	/* Range 0x795D - 0x7962, array index: 0x0F79 */
	0x3D4B,0x3F40,0x632E,0x632C,INVALC,0x472A,
	/* Range 0x797F - 0x7985, array index: 0x0F7F */
	0x6333,0x6349,0x3658,INVALC,INVALC,0x4F3D,0x4135,
	/* Range 0x798D - 0x798F, array index: 0x0F86 */
	0x3252,0x4477,0x4A21,
	/* Range 0x79B9 - 0x79C1, array index: 0x0F89 */
	0x633B,0x633C,INVALC,INVALC,0x3659,0x3253,0x4645,0x3D28,
	0x3B64,
	/* Range 0x79E3 - 0x79E9, array index: 0x0F92 */
	0x6342,0x4769,INVALC,0x3F41,0x633F,INVALC,0x4361,
	/* Range 0x7A17 - 0x7A20, array index: 0x0F99 */
	0x4923,0x6345,0x6346,0x4355,INVALC,0x4E47,INVALC,INVALC,
	0x6348,0x6347,
	/* Range 0x7A3B - 0x7A43, array index: 0x0FA3 */
	0x634B,0x3254,0x374E,0x634C,0x3946,0x3972,INVALC,0x4A66,
	0x634E,
	/* Range 0x7A4D - 0x7A50, array index: 0x0FAC */
	0x4051,0x314F,0x323A,0x302C,
	/* Range 0x7A61 - 0x7A63, array index: 0x0FB0 */
	0x6351,0x6352,0x3E77,
	/* Range 0x7A92 - 0x7A98, array index: 0x0FB3 */
	0x4362,0x416B,INVALC,0x635A,0x635C,0x6359,0x635B,
	/* Range 0x7AAE - 0x7AB0, array index: 0x0FBA */
	0x3567,0x4D52,0x635F,
	/* Range 0x7AC3 - 0x7AD5, array index: 0x0FBD */
	0x3376,0x6362,0x6361,INVALC,0x6365,0x635E,INVALC,0x6366,
	0x4E29,INVALC,0x6367,INVALC,0x6368,INVALC,INVALC,0x5474,
	0x636A,INVALC,0x6369,
	/* Range 0x7AD9 - 0x7AE6, array index: 0x0FD0 */
	0x636B,0x636C,INVALC,0x4E35,0x636D,INVALC,0x706F,0x3E4F,
	0x636E,0x636F,0x3D57,INVALC,0x4638,0x6370,
	/* Range 0x7AF6 - 0x7AFA, array index: 0x0FDE */
	0x3625,INVALC,0x513F,0x435D,0x3C33,
	/* Range 0x7B45 - 0x7B56, array index: 0x0FE3 */
	0x6426,0x492E,INVALC,0x4826,0x4579,INVALC,0x365A,0x6425,
	0x6423,INVALC,0x4835,0x637E,0x435E,0x457B,INVALC,0x457A,
	INVALC,0x3A76,
	/* Range 0x7B94 - 0x7BA1, array index: 0x0FF5 */
	0x4773,0x4C27,INVALC,0x3B3B,0x6430,0x6439,0x6434,INVALC,
	0x6433,0x642F,INVALC,0x6431,INVALC,0x3449,
	/* Range 0x7BE4 - 0x7BE6, array index: 0x1003 */
	0x4646,0x6445,0x6444,
	/* Range 0x7C11 - 0x7C14, array index: 0x1006 */
	0x6442,0x5255,0x6449,0x6443,
	/* Range 0x7C3D - 0x7C40, array index: 0x100A */
	0x6455,0x4E7C,0x4A6D,0x645A,
	/* Range 0x7C4C - 0x7C50, array index: 0x100E */
	0x6456,0x4052,INVALC,0x6459,0x645B,
	/* Range 0x7C81 - 0x7C83, array index: 0x1013 */
	0x364E,0x3729,0x6463,
	/* Range 0x7CA1 - 0x7CA8, array index: 0x1016 */
	0x6469,0x6467,INVALC,0x6465,0x3421,INVALC,0x3E51,0x646A,
	/* Range 0x7CAD - 0x7CB3, array index: 0x101E */
	0x6466,0x646E,INVALC,INVALC,0x646D,0x646C,0x646B,
	/* Range 0x7CDC - 0x7CE2, array index: 0x1025 */
	0x6476,INVALC,0x4A35,0x416C,0x3947,INVALC,0x6477,
	/* Range 0x7D02 - 0x7D06, array index: 0x102C */
	0x6521,INVALC,0x4C73,0x3948,0x647E,
	/* Range 0x7D14 - 0x7D22, array index: 0x1031 */
	0x3D63,0x6523,INVALC,0x3C53,0x3949,0x3B66,0x3569,0x4A36,
	0x6522,INVALC,INVALC,INVALC,0x4147,0x4B42,0x3A77,
	/* Range 0x7D2B - 0x7D35, array index: 0x1040 */
	0x3B67,0x445D,INVALC,0x6527,0x4E5F,0x3A59,INVALC,0x6528,
	0x3F42,INVALC,0x652A,
	/* Range 0x7D42 - 0x7D46, array index: 0x104B */
	0x3D2A,0x383E,0x4148,0x6525,0x652B,
	/* Range 0x7D4B - 0x7D50, array index: 0x1050 */
	0x6526,0x3750,INVALC,0x652E,0x6532,0x376B,
	/* Range 0x7D61 - 0x7D63, array index: 0x1056 */
	0x4D6D,0x303C,0x6533,
	/* Range 0x7D71 - 0x7D76, array index: 0x1059 */
	0x457D,0x652F,0x652C,INVALC,0x3328,0x4064,
	/* Range 0x7D99 - 0x7D9C, array index: 0x105F */
	0x3751,0x4233,0x6539,0x416E,
	/* Range 0x7DAB - 0x7DB5, array index: 0x1063 */
	0x6540,0x3C7A,0x305D,0x653B,0x6543,0x6547,0x394B,0x4C56,
	INVALC,0x4456,0x653D,
	/* Range 0x7DBA - 0x7DBF, array index: 0x106E */
	0x653A,0x433E,INVALC,0x653F,0x303D,0x4C4A,
	/* Range 0x7DDA - 0x7DE1, array index: 0x1074 */
	0x407E,INVALC,0x6544,0x6549,0x654B,INVALC,0x4479,0x654E,
	/* Range 0x7E09 - 0x7E0B, array index: 0x107C */
	0x6556,0x6550,0x6557,
	/* Range 0x7E1E - 0x7E23, array index: 0x107F */
	0x3C4A,0x6555,INVALC,0x6552,0x6558,0x6551,
	/* Range 0x7E37 - 0x7E3E, array index: 0x1085 */
	0x655F,INVALC,0x655D,0x6561,0x655B,INVALC,0x6541,0x4053,
	/* Range 0x7E54 - 0x7E5E, array index: 0x108D */
	0x3F25,0x4136,0x6564,INVALC,INVALC,0x6566,0x6567,INVALC,
	INVALC,0x6563,0x6565,
	/* Range 0x7E66 - 0x7E6A, array index: 0x1098 */
	0x655A,0x6562,INVALC,0x656A,0x6569,
	/* Range 0x7E79 - 0x7E83, array index: 0x109D */
	0x6568,INVALC,0x656C,0x656B,0x656F,INVALC,0x6571,INVALC,
	INVALC,0x3B3C,0x656D,
	/* Range 0x7E8C - 0x7E96, array index: 0x10A8 */
	0x6574,INVALC,0x657A,0x453B,0x6576,INVALC,0x6575,0x6577,
	0x6578,INVALC,0x6579,
	/* Range 0x7F4C - 0x7F55, array index: 0x10B3 */
	0x6622,0x6623,0x6624,INVALC,0x6625,0x6626,INVALC,INVALC,
	0x6628,0x6627,
	/* Range 0x7F67 - 0x7F6B, array index: 0x10BD */
	0x662E,0x662C,0x662D,0x3A61,0x3753,
	/* Range 0x7F75 - 0x7F79, array index: 0x10C2 */
	0x474D,INVALC,0x486D,0x662F,0x586D,
	/* Range 0x7F82 - 0x7F8E, array index: 0x10C7 */
	0x6630,0x6632,INVALC,0x4D65,0x6631,0x6634,0x6633,INVALC,
	0x4D53,INVALC,0x6635,INVALC,0x487E,
	/* Range 0x8000 - 0x8006, array index: 0x10D4 */
	0x4D54,0x4F37,INVALC,0x394D,0x664E,0x3C54,0x664D,
	/* Range 0x8015 - 0x8019, array index: 0x10DB */
	0x394C,INVALC,0x4C57,0x6651,0x6652,
	/* Range 0x805E - 0x8062, array index: 0x10E0 */
	0x4A39,0x665D,INVALC,0x416F,0x665E,
	/* Range 0x806F - 0x807F, array index: 0x10E5 */
	0x4E7E,0x6662,INVALC,0x6661,0x6660,0x4430,INVALC,0x6663,
	0x3F26,INVALC,0x6664,INVALC,INVALC,INVALC,0x6665,0x4F38,
	0x6666,
	/* Range 0x8084 - 0x808C, array index: 0x10F6 */
	0x6667,0x6669,0x6668,0x4825,INVALC,0x4679,INVALC,0x4F3E,
	0x4829,
	/* Range 0x80A9 - 0x80B4, array index: 0x10FF */
	0x382A,0x4B43,INVALC,0x666F,0x666D,INVALC,0x394E,INVALC,
	0x394F,0x3069,INVALC,0x3A68,
	/* Range 0x80D9 - 0x80DE, array index: 0x110B */
	0x6672,0x6675,0x6670,INVALC,0x6673,0x4B26,
	/* Range 0x8105 - 0x810A, array index: 0x1111 */
	0x363C,0x4048,0x4F46,0x4C2E,0x6677,0x4054,
	/* Range 0x8150 - 0x8155, array index: 0x1117 */
	0x4965,0x6725,INVALC,0x6724,0x3950,0x4F53,
	/* Range 0x8178 - 0x817A, array index: 0x111D */
	0x4432,0x4A22,0x4123,
	/* Range 0x817F - 0x8183, array index: 0x1120 */
	0x425C,0x672F,INVALC,0x6730,0x672C,
	/* Range 0x81BD - 0x81C2, array index: 0x1125 */
	0x673C,0x673A,0x473F,0x673D,INVALC,0x673E,
	/* Range 0x81D8 - 0x81DA, array index: 0x112B */
	0x6744,0x6743,0x6746,
	/* Range 0x81FA - 0x8202, array index: 0x112E */
	0x674A,0x674B,0x3131,INVALC,0x674C,INVALC,INVALC,0x674D,
	0x674E,
	/* Range 0x8205 - 0x821F, array index: 0x1137 */
	0x674F,INVALC,0x6750,0x363D,0x5A2A,0x6751,INVALC,0x4065,
	0x6752,0x3C4B,INVALC,0x6753,INVALC,0x5030,INVALC,INVALC,
	INVALC,0x6754,0x4A5E,0x345C,INVALC,INVALC,0x4124,0x3D58,
	INVALC,0x4971,0x3D2E,
	/* Range 0x8229 - 0x822E, array index: 0x1152 */
	0x6755,0x3952,0x6756,0x484C,INVALC,0x6764,
	/* Range 0x8233 - 0x8239, array index: 0x1158 */
	0x6758,INVALC,0x4249,0x4775,0x383F,0x6757,0x4125,
	/* Range 0x8258 - 0x825A, array index: 0x115F */
	0x675B,0x675A,0x675D,
	/* Range 0x826E - 0x8278, array index: 0x1162 */
	0x3A31,0x4E49,INVALC,0x6765,0x3F27,INVALC,INVALC,INVALC,
	0x3170,0x6766,0x6767,
	/* Range 0x82AB - 0x82B3, array index: 0x116D */
	0x676B,0x676E,0x474E,INVALC,0x3F44,INVALC,0x3256,INVALC,
	0x4B27,
	/* Range 0x82D1 - 0x82D4, array index: 0x1176 */
	0x3171,0x6772,0x4E6A,0x425D,
	/* Range 0x82DB - 0x82E7, array index: 0x117A */
	0x3257,0x677C,INVALC,0x677A,0x6771,INVALC,0x676F,INVALC,
	0x6770,INVALC,0x3C63,0x366C,0x4377,
	/* Range 0x82F9 - 0x82FB, array index: 0x1187 */
	0x6779,0x6775,0x6778,
	/* Range 0x8302 - 0x8306, array index: 0x118A */
	0x4C50,0x6777,0x3258,0x337D,0x677B,
	/* Range 0x8316 - 0x8318, array index: 0x118F */
	0x6823,0x682C,0x682D,
	/* Range 0x8331 - 0x8339, array index: 0x1192 */
	0x6825,0x6824,INVALC,0x6822,0x6821,0x4363,INVALC,0x427B,
	0x6827,
	/* Range 0x839E - 0x83A2, array index: 0x119B */
	0x3450,0x6831,0x683C,INVALC,0x6832,
	/* Range 0x83EF - 0x83F4, array index: 0x11A0 */
	0x325A,0x3856,0x4929,0x684B,INVALC,0x683F,
	/* Range 0x840B - 0x840E, array index: 0x11A6 */
	0x6846,0x4B28,0x684C,0x3060,
	/* Range 0x8461 - 0x8463, array index: 0x11AA */
	0x4972,0x6864,0x4621,
	/* Range 0x8469 - 0x8471, array index: 0x11AD */
	0x685D,INVALC,0x6859,0x4172,0x6853,0x685B,0x6860,INVALC,
	0x472C,
	/* Range 0x84B8 - 0x84BC, array index: 0x11B6 */
	0x3E78,0x6865,INVALC,0x686A,0x4173,
	/* Range 0x84C9 - 0x84D1, array index: 0x11BB */
	0x4D56,0x6863,0x3338,INVALC,0x6869,INVALC,INVALC,0x686C,
	0x4C2C,
	/* Range 0x8511 - 0x851A, array index: 0x11C4 */
	0x4A4E,INVALC,0x4C22,0x6879,0x6878,INVALC,0x6874,0x6875,
	INVALC,0x3136,
	/* Range 0x8548 - 0x854B, array index: 0x11CE */
	0x687E,0x3E56,0x3C49,0x6923,
	/* Range 0x8568 - 0x856A, array index: 0x11D2 */
	0x4F4F,0x4622,0x4973,
	/* Range 0x85A6 - 0x85AF, array index: 0x11D5 */
	0x4126,INVALC,0x692A,0x3B27,0x3F45,0x3730,0x4C74,INVALC,
	0x4C79,0x3D72,
	/* Range 0x85F7 - 0x85FB, array index: 0x11DF */
	0x3D73,INVALC,0x693D,0x6942,0x4174,
	/* Range 0x868A - 0x868C, array index: 0x11E4 */
	0x3263,0x6952,0x6953,
	/* Range 0x86A9 - 0x86AB, array index: 0x11E7 */
	0x6950,0x6951,0x695B,
	/* Range 0x86EC - 0x86EF, array index: 0x11EA */
	0x695F,0x4948,0x485A,0x6962,
	/* Range 0x8706 - 0x870A, array index: 0x11EE */
	0x6964,INVALC,0x6965,0x696A,0x696D,
	/* Range 0x880D - 0x8811, array index: 0x11F3 */
	0x6A38,0x6A3C,0x6A37,INVALC,0x6A3E,
	/* Range 0x8821 - 0x8823, array index: 0x11F8 */
	0x6A42,0x6A41,0x695A,
	/* Range 0x885D - 0x8863, array index: 0x11FB */
	0x3E57,0x6A4C,INVALC,INVALC,0x3955,0x6A4D,0x3061,
	/* Range 0x887D - 0x8882, array index: 0x1202 */
	0x6A53,0x6A50,0x365E,INVALC,0x6A4F,0x6A56,
	/* Range 0x88BF - 0x88C5, array index: 0x1208 */
	0x6A5F,INVALC,0x3A5B,0x4E76,0x6A61,0x6A62,0x4175,
	/* Range 0x88F2 - 0x88F4, array index: 0x120F */
	0x6A6C,0x3E58,0x6A6A,
	/* Range 0x88F8 - 0x88FE, array index: 0x1212 */
	0x4D67,0x6A67,INVALC,INVALC,0x6A69,0x403D,0x3F7E,
	/* Range 0x895E - 0x8960, array index: 0x1219 */
	0x6A7E,0x365F,0x6A7D,
	/* Range 0x8986 - 0x898B, array index: 0x121C */
	0x4A24,0x4746,0x6B2A,INVALC,0x6B2B,0x382B,
	/* Range 0x8996 - 0x899A, array index: 0x1222 */
	0x3B6B,0x4741,0x6B2D,INVALC,0x3350,
	/* Range 0x89A6 - 0x89AC, array index: 0x1227 */
	0x6B30,0x4D77,INVALC,0x6B2F,0x3F46,INVALC,0x6B31,
	/* Range 0x8A16 - 0x8A18, array index: 0x122E */
	0x6B3F,0x4277,0x352D,
	/* Range 0x8A3A - 0x8A3C, array index: 0x1231 */
	0x3F47,0x4370,0x3E5A,
	/* Range 0x8A50 - 0x8A55, array index: 0x1234 */
	0x3A3E,0x4242,0x6B48,INVALC,0x3E5B,0x493E,
	/* Range 0x8A69 - 0x8A73, array index: 0x123A */
	0x3B6D,INVALC,0x4F4D,0x6B4D,0x6B4C,0x4127,INVALC,0x354D,
	0x4F43,0x333A,0x3E5C,
	/* Range 0x8AA0 - 0x8AA8, array index: 0x1245 */
	0x403F,0x6B53,INVALC,0x6B58,0x386D,0x6B55,0x6B56,INVALC,
	0x6B52,
	/* Range 0x8ACB - 0x8ACF, array index: 0x124E */
	0x4041,0x3452,0x6B5A,INVALC,0x3F5B,
	/* Range 0x8ADA - 0x8AE7, array index: 0x1253 */
	0x6B5C,0x6B67,0x4435,INVALC,0x6B66,INVALC,0x6B63,0x6B6B,
	0x6B64,INVALC,0x6B60,INVALC,0x447C,0x6B5F,
	/* Range 0x8AFE - 0x8B04, array index: 0x1261 */
	0x427A,INVALC,0x4B45,0x315A,0x3062,INVALC,0x4625,
	/* Range 0x8B16 - 0x8B21, array index: 0x1268 */
	0x6B6C,0x6B6E,INVALC,0x382C,0x6B6A,0x3956,INVALC,0x3C55,
	INVALC,INVALC,0x6B6F,0x4D58,
	/* Range 0x8B58 - 0x8B5C, array index: 0x1274 */
	0x3C31,INVALC,0x6B7D,0x6B7C,0x4968,
	/* Range 0x8B6B - 0x8B74, array index: 0x1279 */
	0x6B7E,0x6C22,INVALC,INVALC,0x6C23,0x3544,0x6641,0x3E79,
	INVALC,0x6C24,
	/* Range 0x8C6A - 0x8C6C, array index: 0x1283 */
	0x396B,0x502E,0x6C37,
	/* Range 0x8C78 - 0x8C7C, array index: 0x1286 */
	0x6C38,0x493F,0x6C39,INVALC,0x6C41,
	/* Range 0x8C89 - 0x8C8E, array index: 0x128B */
	0x6C3B,0x6C3D,INVALC,0x4B46,0x6C3E,0x6C3F,
	/* Range 0x8C9D - 0x8CA2, array index: 0x1291 */
	0x332D,0x4467,INVALC,0x4969,0x3A62,0x3957,
	/* Range 0x8CA7 - 0x8CCE, array index: 0x1297 */
	0x494F,0x325F,0x484E,0x6C45,0x3453,0x4055,0x6C44,0x6C49,
	0x4379,0x4C63,INVALC,0x6C47,0x6C48,0x352E,INVALC,0x6C4A,
	0x4763,0x425F,INVALC,INVALC,0x4871,0x453D,0x6C46,INVALC,
	0x4B47,0x326C,0x6C4C,0x4F28,0x4442,0x4F45,INVALC,INVALC,
	0x3B71,0x6C4B,INVALC,0x4231,INVALC,INVALC,0x6C5C,0x4128,
	/* Range 0x8CDA - 0x8CE6, array index: 0x12BF */
	0x6C4F,0x3B3F,0x3B72,INVALC,0x3E5E,INVALC,0x4765,INVALC,
	0x382D,0x6C4E,0x6C4D,INVALC,0x496A,
	/* Range 0x8CFA - 0x8CFD, array index: 0x12CC */
	0x6C51,0x6C52,0x3958,0x6C50,
	/* Range 0x8D04 - 0x8D16, array index: 0x12D0 */
	0x6C53,0x6C54,INVALC,0x6C56,0x4223,INVALC,0x6C55,0x3466,
	INVALC,0x6C58,INVALC,0x6C57,0x6C59,INVALC,INVALC,0x6C5B,
	0x6C5D,INVALC,0x6C5E,
	/* Range 0x8D70 - 0x8D74, array index: 0x12E3 */
	0x4176,0x6C61,INVALC,0x6C62,0x496B,
	/* Range 0x8E08 - 0x8E0A, array index: 0x12E8 */
	0x6C73,0x6C74,0x4D59,
	/* Range 0x8E1D - 0x8E1F, array index: 0x12EB */
	0x6C76,0x6C77,0x6C79,
	/* Range 0x8E47 - 0x8E4C, array index: 0x12EE */
	0x6D21,0x6D25,0x6D22,0x6C7E,INVALC,0x6D23,
	/* Range 0x8EAA - 0x8EB1, array index: 0x12F4 */
	0x6D39,0x3F48,0x6D3B,INVALC,INVALC,0x366D,0x6D3C,0x6D3E,
	/* Range 0x8EC8 - 0x8ECD, array index: 0x12FC */
	0x6D41,INVALC,0x3C56,0x6D42,0x3530,0x3733,
	/* Range 0x8EFB - 0x8EFE, array index: 0x1302 */
	0x6D46,0x6D45,0x375A,0x6D48,
	/* Range 0x8F12 - 0x8F15, array index: 0x1306 */
	0x6D4C,0x6D4E,0x4A65,0x6D4B,
	/* Range 0x8F19 - 0x8F1F, array index: 0x130A */
	0x6D4D,INVALC,0x6D51,0x6D4F,0x3531,INVALC,0x6D50,
	/* Range 0x8F42 - 0x8F46, array index: 0x1311 */
	0x6D58,INVALC,0x336D,0x6D57,0x6D5C,
	/* Range 0x8F4C - 0x8F4E, array index: 0x1316 */
	0x6D5A,0x4532,0x6D5D,
	/* Range 0x8F5F - 0x8F64, array index: 0x1319 */
	0x396C,INVALC,0x3725,0x6D60,0x6D61,0x6D62,
	/* Range 0x8F9B - 0x8F9F, array index: 0x131F */
	0x3F49,0x6D63,INVALC,0x3C2D,0x6D64,
	/* Range 0x8FAD - 0x8FB2, array index: 0x1324 */
	0x6D66,0x6570,0x6D67,0x4324,0x3F2B,0x4740,
	/* Range 0x8FBA - 0x8FBC, array index: 0x132A */
	0x4A55,0x4454,0x397E,
	/* Range 0x8FE5 - 0x8FF0, array index: 0x132D */
	0x6D6A,0x3260,INVALC,INVALC,0x4676,0x6D6C,0x4777,INVALC,
	0x4533,INVALC,0x6D6D,0x3D52,
	/* Range 0x8FF7 - 0x8FFA, array index: 0x1339 */
	0x4C42,0x6D7E,0x6D71,0x6D72,
	/* Range 0x900B - 0x9023, array index: 0x133D */
	0x6D79,INVALC,0x6D76,0x6E25,0x4629,0x4360,0x6D73,INVALC,
	0x447E,0x4553,0x6D74,0x6D78,0x3F60,INVALC,0x4767,0x444C,
	INVALC,INVALC,0x4042,0x6D77,0x422E,0x4224,0x6D75,0x3029,
	0x4F22,
	/* Range 0x9035 - 0x9039, array index: 0x1356 */
	0x6D7C,0x6D7B,INVALC,0x306F,0x6D7D,
	/* Range 0x9047 - 0x9059, array index: 0x135B */
	0x3678,INVALC,0x6E26,0x4D37,0x313F,INVALC,0x4A57,0x3261,
	0x6E21,0x6E22,0x6E23,0x6E24,0x463B,0x4323,0x3063,0x6E28,
	INVALC,0x6E29,0x7423,
	/* Range 0x906D - 0x906F, array index: 0x136E */
	0x4178,0x3C57,0x6E2C,
	/* Range 0x9075 - 0x9084, array index: 0x1371 */
	0x3D65,0x6E2D,0x412B,0x412A,INVALC,0x3064,INVALC,0x4E4B,
	0x6E31,INVALC,0x4872,0x6E33,0x6E32,0x6E30,0x6364,0x3454,
	/* Range 0x9149 - 0x914E, array index: 0x1381 */
	0x4653,0x6E44,0x3D36,0x3C60,0x475B,0x4371,
	/* Range 0x91C6 - 0x91D1, array index: 0x1387 */
	0x4850,0x3A53,0x3C61,0x6E58,INVALC,0x6E59,0x4E24,0x3D45,
	0x4C6E,0x4E4C,0x6E5A,0x3662,
	/* Range 0x91DB - 0x91E7, array index: 0x1393 */
	0x6E5E,0x3378,0x3F4B,INVALC,0x6E5C,INVALC,0x6E5D,INVALC,
	0x4460,INVALC,INVALC,0x4B55,0x367C,
	/* Range 0x9298 - 0x929C, array index: 0x13A0 */
	0x4C43,INVALC,0x4438,0x6E75,0x6E72,
	/* Range 0x9318 - 0x931A, array index: 0x13A5 */
	0x3F6E,0x6F21,0x6F23,
	/* Range 0x932B - 0x932F, array index: 0x13A8 */
	0x3C62,0x4F23,INVALC,0x6E7E,0x3A78,
	/* Range 0x93AC - 0x93B0, array index: 0x13AD */
	0x6F2E,0x6F2F,0x4443,INVALC,0x6F2D,
	/* Range 0x93D6 - 0x93D8, array index: 0x13B2 */
	0x6F32,0x6F33,0x6F36,
	/* Range 0x9418 - 0x941A, array index: 0x13B5 */
	0x3E62,0x462A,0x6F3C,
	/* Range 0x9451 - 0x9453, array index: 0x13B8 */
	0x3455,0x6F48,0x4C7A,
	/* Range 0x947C - 0x9481, array index: 0x13BB */
	0x6F55,0x6F53,0x6F56,0x6F58,INVALC,0x6F57,
	/* Range 0x9587 - 0x958B, array index: 0x13C1 */
	0x6F5A,INVALC,0x4A44,0x6F5B,0x332B,
	/* Range 0x95A0 - 0x95A8, array index: 0x13C6 */
	0x6F60,INVALC,0x3458,0x3355,0x395E,0x4836,INVALC,0x6F62,
	0x6F61,
	/* Range 0x95D4 - 0x95D8, array index: 0x13CF */
	0x6F6E,0x6F6D,0x6F6F,INVALC,0x462E,
	/* Range 0x964B - 0x9650, array index: 0x13D4 */
	0x6F7B,0x6F79,0x395F,INVALC,0x6F7A,0x3842,
	/* Range 0x965B - 0x9666, array index: 0x13DA */
	0x4A45,0x6F7D,0x7021,0x6F7E,0x7022,INVALC,INVALC,0x3121,
	0x3F58,0x3D7C,0x3459,0x7023,
	/* Range 0x9672 - 0x967A, array index: 0x13E6 */
	0x7024,0x4444,INVALC,0x4E4D,0x462B,0x6F7C,0x4E26,INVALC,
	0x3831,
	/* Range 0x968A - 0x968F, array index: 0x13EF */
	0x4262,0x6721,INVALC,0x7026,0x332C,0x3F6F,
	/* Range 0x9694 - 0x969C, array index: 0x13F5 */
	0x3356,0x7028,INVALC,0x7029,0x7027,0x3764,INVALC,0x3A5D,
	0x3E63,
	/* Range 0x96B0 - 0x96BC, array index: 0x13FE */
	0x702E,0x702C,0x702D,INVALC,0x702F,INVALC,0x7030,0x4E6C,
	0x7031,0x7032,INVALC,0x4049,0x483B,
	/* Range 0x96C0 - 0x96CE, array index: 0x140B */
	0x3F7D,0x3467,INVALC,INVALC,0x4D3A,0x326D,0x3D38,0x385B,
	INVALC,0x7035,INVALC,0x7034,0x3B73,0x7036,0x7033,
	/* Range 0x9704 - 0x9713, array index: 0x141A */
	0x703C,INVALC,0x703D,0x3F4C,0x703E,INVALC,0x4E6E,INVALC,
	INVALC,0x7039,0x7040,0x7042,INVALC,0x7041,INVALC,0x703F,
	/* Range 0x975E - 0x976D, array index: 0x142A */
	0x4873,INVALC,0x7051,0x7353,0x4C4C,INVALC,0x7052,INVALC,
	0x7053,INVALC,0x7054,0x3357,INVALC,0x7056,INVALC,0x3F59,
	/* Range 0x9784 - 0x9786, array index: 0x143A */
	0x3373,0x7059,0x705D,
	/* Range 0x97F2 - 0x97F6, array index: 0x143D */
	0x706E,0x323B,INVALC,0x7071,0x7070,
	/* Range 0x97FF - 0x9808, array index: 0x1442 */
	0x3641,INVALC,0x4A47,0x443A,0x3A22,INVALC,0x3960,0x3D67,
	INVALC,0x3F5C,
	/* Range 0x980F - 0x9813, array index: 0x144C */
	0x7072,0x4D42,0x3468,0x4852,0x465C,
	/* Range 0x9837 - 0x983D, array index: 0x1451 */
	0x7077,0x7074,INVALC,INVALC,0x4951,0x4D6A,0x7078,
	/* Range 0x984B - 0x984F, array index: 0x1458 */
	0x707B,0x426A,0x335B,0x335C,0x707A,
	/* Range 0x986F - 0x9874, array index: 0x145D */
	0x707D,0x707E,0x7121,INVALC,0x7123,0x7122,
	/* Range 0x98EB - 0x98EF, array index: 0x1463 */
	0x712C,INVALC,0x522C,0x5D3B,0x4853,
	/* Range 0x98FC - 0x98FE, array index: 0x1468 */
	0x3B74,0x4B30,0x3E7E,
	/* Range 0x9910 - 0x9914, array index: 0x146B */
	0x3B41,INVALC,0x712F,0x326E,0x7130,
	/* Range 0x991D - 0x9921, array index: 0x1470 */
	0x7133,0x7134,INVALC,0x7136,0x7132,
	/* Range 0x9950 - 0x9952, array index: 0x1475 */
	0x713E,0x7140,0x7141,
	/* Range 0x9996 - 0x9999, array index: 0x1478 */
	0x3C73,0x7144,0x7145,0x3961,
	/* Range 0x99AC - 0x99AE, array index: 0x147C */
	0x474F,0x7147,0x7148,
	/* Range 0x99C4 - 0x99C8, array index: 0x147F */
	0x424C,0x3158,0x366E,INVALC,0x366F,
	/* Range 0x99D0 - 0x99D2, array index: 0x1484 */
	0x4373,0x714E,0x3670,
	/* Range 0x9A69 - 0x9A6B, array index: 0x1487 */
	0x7169,0x716B,0x716A,
	/* Range 0x9AEA - 0x9AF1, array index: 0x148A */
	0x4831,0x717A,INVALC,0x4926,0x717B,0x7179,INVALC,0x717D,
	/* Range 0x9B25 - 0x9B2A, array index: 0x1492 */
	0x7228,INVALC,0x7229,0x722A,0x722B,0x722C,
	/* Range 0x9B2E - 0x9B32, array index: 0x1498 */
	0x722D,0x722E,INVALC,0x5D35,0x722F,
	/* Range 0x9B41 - 0x9B45, array index: 0x149D */
	0x3321,0x3A32,0x7231,0x7230,0x4C25,
	/* Range 0x9B4D - 0x9B51, array index: 0x14A2 */
	0x7233,0x7234,0x7232,INVALC,0x7235,
	/* Range 0x9B91 - 0x9B97, array index: 0x14A7 */
	0x723A,0x4A2B,0x7238,INVALC,INVALC,0x723B,0x723C,
	/* Range 0x9BAA - 0x9BAE, array index: 0x14AE */
	0x4B6E,0x3B2D,INVALC,0x3A7A,0x412F,
	/* Range 0x9BE1 - 0x9BE4, array index: 0x14B3 */
	0x724C,0x7249,0x7248,0x724A,
	/* Range 0x9BF0 - 0x9BF2, array index: 0x14B7 */
	0x7250,0x724F,0x724E,
	/* Range 0x9C06 - 0x9C0D, array index: 0x14BA */
	0x7256,INVALC,0x7257,0x7253,0x7259,INVALC,0x7255,0x3362,
	/* Range 0x9C10 - 0x9C15, array index: 0x14C2 */
	0x4F4C,INVALC,0x7258,0x7254,0x7252,0x7251,
	/* Range 0x9C2D - 0x9C32, array index: 0x14C8 */
	0x4949,0x725B,0x3073,0x7260,INVALC,0x7262,
	/* Range 0x9C39 - 0x9C3B, array index: 0x14CE */
	0x336F,0x724D,0x3137,
	/* Range 0x9C46 - 0x9C48, array index: 0x14D1 */
	0x7263,0x7261,0x432D,
	/* Range 0x9D06 - 0x9D09, array index: 0x14D4 */
	0x7271,0x463E,0x726E,0x726D,
	/* Range 0x9D28 - 0x9D2C, array index: 0x14D8 */
	0x337B,INVALC,0x7272,0x3C32,0x3229,
	/* Range 0x9D5C - 0x9D61, array index: 0x14DD */
	0x312D,0x7321,0x7322,INVALC,0x3974,0x4C39,
	/* Range 0x9DF8 - 0x9DFA, array index: 0x14E3 */
	0x733B,0x426B,0x3A6D,
	/* Range 0x9E91 - 0x9E97, array index: 0x14E6 */
	0x734C,0x734A,0x4F3C,INVALC,0x734B,INVALC,0x4E6F,
	/* Range 0x9EB8 - 0x9EBF, array index: 0x14ED */
	0x7350,0x396D,0x4C4D,0x4B63,0x5677,INVALC,0x5D60,0x4B7B,
	/* Range 0x9ECC - 0x9ED4, array index: 0x14F5 */
	0x7354,0x3550,0x7355,0x7356,0x7357,INVALC,0x3975,INVALC,
	0x7358,
	/* Range 0x9ED8 - 0x9EE0, array index: 0x14FE */
	0x6054,0x4C5B,INVALC,0x4263,0x7359,0x735B,0x735A,INVALC,
	0x735C,
	/* Range 0x9EF9 - 0x9EFD, array index: 0x1507 */
	0x7363,INVALC,0x7364,0x7365,0x7366,
	/* Range 0x9F5F - 0x9F67, array index: 0x150C */
	0x7372,0x7373,0x7374,0x4E70,0x7371,INVALC,INVALC,0x7375,
	0x7376,
	/* Range 0xFF01 - 0xFF5D, array index: 0x1515 */
	0x212A,INVALC,0x2174,0x2170,0x2173,0x2175,INVALC,0x214A,
	0x214B,0x2176,0x215C,0x2124,INVALC,0x2125,0x213F,0x2330,
	0x2331,0x2332,0x2333,0x2334,0x2335,0x2336,0x2337,0x2338,
	0x2339,0x2127,0x2128,0x2163,0x2161,0x2164,0x2129,0x2177,
	0x2341,0x2342,0x2343,0x2344,0x2345,0x2346,0x2347,0x2348,
	0x2349,0x234A,0x234B,0x234C,0x234D,0x234E,0x234F,0x2350,
	0x2351,0x2352,0x2353,0x2354,0x2355,0x2356,0x2357,0x2358,
	0x2359,0x235A,0x214E,INVALC,0x214F,0x2130,0x2132,0x212E,
	0x2361,0x2362,0x2363,0x2364,0x2365,0x2366,0x2367,0x2368,
	0x2369,0x236A,0x236B,0x236C,0x236D,0x236E,0x236F,0x2370,
	0x2371,0x2372,0x2373,0x2374,0x2375,0x2376,0x2377,0x2378,
	0x2379,0x237A,0x2150,0x2143,0x2151,
	/* Unranged codes (489 codes) */
	/* Array index: 0x1572 */ 0x005C,0x2140,
	/* Array index: 0x1572 */ 0x00A2,0x2171,
	/* Array index: 0x1572 */ 0x00A3,0x2172,
	/* Array index: 0x1572 */ 0x00A7,0x2178,
	/* Array index: 0x1572 */ 0x00A8,0x212F,
	/* Array index: 0x1572 */ 0x00AC,0x224C,
	/* Array index: 0x1572 */ 0x00B0,0x216B,
	/* Array index: 0x1572 */ 0x00B1,0x215E,
	/* Array index: 0x1572 */ 0x00B4,0x212D,
	/* Array index: 0x1572 */ 0x00B6,0x2279,
	/* Array index: 0x1572 */ 0x00D7,0x215F,
	/* Array index: 0x1572 */ 0x00F7,0x2160,
	/* Array index: 0x1572 */ 0x0401,0x2727,
	/* Array index: 0x1572 */ 0x2010,0x213E,
	/* Array index: 0x1572 */ 0x2025,0x2145,
	/* Array index: 0x1572 */ 0x2026,0x2144,
	/* Array index: 0x1572 */ 0x2030,0x2273,
	/* Array index: 0x1572 */ 0x2032,0x216C,
	/* Array index: 0x1572 */ 0x2033,0x216D,
	/* Array index: 0x1572 */ 0x203B,0x2228,
	/* Array index: 0x1572 */ 0x2103,0x216E,
	/* Array index: 0x1572 */ 0x212B,0x2272,
	/* Array index: 0x1572 */ 0x21D2,0x224D,
	/* Array index: 0x1572 */ 0x21D4,0x224E,
	/* Array index: 0x1572 */ 0x2200,0x224F,
	/* Array index: 0x1572 */ 0x2202,0x225F,
	/* Array index: 0x1572 */ 0x2203,0x2250,
	/* Array index: 0x1572 */ 0x2207,0x2260,
	/* Array index: 0x1572 */ 0x2208,0x223A,
	/* Array index: 0x1572 */ 0x220B,0x223B,
	/* Array index: 0x1572 */ 0x2212,0x215D,
	/* Array index: 0x1572 */ 0x221A,0x2265,
	/* Array index: 0x1572 */ 0x221D,0x2267,
	/* Array index: 0x1572 */ 0x221E,0x2167,
	/* Array index: 0x1572 */ 0x2220,0x225C,
	/* Array index: 0x1572 */ 0x2234,0x2168,
	/* Array index: 0x1572 */ 0x2235,0x2268,
	/* Array index: 0x1572 */ 0x223D,0x2266,
	/* Array index: 0x1572 */ 0x2252,0x2262,
	/* Array index: 0x1572 */ 0x2260,0x2162,
	/* Array index: 0x1572 */ 0x2261,0x2261,
	/* Array index: 0x1572 */ 0x2266,0x2165,
	/* Array index: 0x1572 */ 0x2267,0x2166,
	/* Array index: 0x1572 */ 0x226A,0x2263,
	/* Array index: 0x1572 */ 0x226B,0x2264,
	/* Array index: 0x1572 */ 0x2282,0x223E,
	/* Array index: 0x1572 */ 0x2283,0x223F,
	/* Array index: 0x1572 */ 0x2286,0x223C,
	/* Array index: 0x1572 */ 0x2287,0x223D,
	/* Array index: 0x1572 */ 0x22A5,0x225D,
	/* Array index: 0x1572 */ 0x2312,0x225E,
	/* Array index: 0x1572 */ 0x250C,0x2823,
	/* Array index: 0x1572 */ 0x250F,0x282E,
	/* Array index: 0x1572 */ 0x2510,0x2824,
	/* Array index: 0x1572 */ 0x2513,0x282F,
	/* Array index: 0x1572 */ 0x2514,0x2826,
	/* Array index: 0x1572 */ 0x2520,0x2837,
	/* Array index: 0x1572 */ 0x2528,0x2839,
	/* Array index: 0x1572 */ 0x252B,0x2834,
	/* Array index: 0x1572 */ 0x252C,0x2828,
	/* Array index: 0x1572 */ 0x252F,0x2838,
	/* Array index: 0x1572 */ 0x2530,0x283D,
	/* Array index: 0x1572 */ 0x2533,0x2833,
	/* Array index: 0x1572 */ 0x2534,0x282A,
	/* Array index: 0x1572 */ 0x2537,0x283A,
	/* Array index: 0x1572 */ 0x2538,0x283F,
	/* Array index: 0x1572 */ 0x253B,0x2835,
	/* Array index: 0x1572 */ 0x253C,0x282B,
	/* Array index: 0x1572 */ 0x253F,0x283B,
	/* Array index: 0x1572 */ 0x2542,0x2840,
	/* Array index: 0x1572 */ 0x254B,0x2836,
	/* Array index: 0x1572 */ 0x25A0,0x2223,
	/* Array index: 0x1572 */ 0x25A1,0x2222,
	/* Array index: 0x1572 */ 0x25B2,0x2225,
	/* Array index: 0x1572 */ 0x25B3,0x2224,
	/* Array index: 0x1572 */ 0x25BC,0x2227,
	/* Array index: 0x1572 */ 0x25BD,0x2226,
	/* Array index: 0x1572 */ 0x25C6,0x2221,
	/* Array index: 0x1572 */ 0x25C7,0x217E,
	/* Array index: 0x1572 */ 0x25CB,0x217B,
	/* Array index: 0x1572 */ 0x25CE,0x217D,
	/* Array index: 0x1572 */ 0x25CF,0x217C,
	/* Array index: 0x1572 */ 0x25EF,0x227E,
	/* Array index: 0x1572 */ 0x2605,0x217A,
	/* Array index: 0x1572 */ 0x2606,0x2179,
	/* Array index: 0x1572 */ 0x2640,0x216A,
	/* Array index: 0x1572 */ 0x2642,0x2169,
	/* Array index: 0x1572 */ 0x266A,0x2276,
	/* Array index: 0x1572 */ 0x266D,0x2275,
	/* Array index: 0x1572 */ 0x266F,0x2274,
	/* Array index: 0x1572 */ 0x301C,0x2141,
	/* Array index: 0x1572 */ 0x4E00,0x306C,
	/* Array index: 0x1572 */ 0x4E01,0x437A,
	/* Array index: 0x1572 */ 0x4E03,0x3C37,
	/* Array index: 0x1572 */ 0x4E1E,0x3E67,
	/* Array index: 0x1572 */ 0x4E21,0x4E3E,
	/* Array index: 0x1572 */ 0x4E26,0x4A42,
	/* Array index: 0x1572 */ 0x4E2A,0x5024,
	/* Array index: 0x1572 */ 0x4E2D,0x4366,
	/* Array index: 0x1572 */ 0x4E31,0x5025,
	/* Array index: 0x1572 */ 0x4E32,0x367A,
	/* Array index: 0x1572 */ 0x4E36,0x5026,
	/* Array index: 0x1572 */ 0x4E3F,0x5028,
	/* Array index: 0x1572 */ 0x4E42,0x5029,
	/* Array index: 0x1572 */ 0x4E43,0x4735,
	/* Array index: 0x1572 */ 0x4E45,0x3557,
	/* Array index: 0x1572 */ 0x4E62,0x5626,
	/* Array index: 0x1572 */ 0x4E71,0x4D70,
	/* Array index: 0x1572 */ 0x4E73,0x467D,
	/* Array index: 0x1572 */ 0x4E7E,0x3425,
	/* Array index: 0x1572 */ 0x4E80,0x3535,
	/* Array index: 0x1572 */ 0x4E82,0x502C,
	/* Array index: 0x1572 */ 0x4EB3,0x5038,
	/* Array index: 0x1572 */ 0x4EB6,0x5039,
	/* Array index: 0x1572 */ 0x4EBA,0x3F4D,
	/* Array index: 0x1572 */ 0x4EED,0x5041,
	/* Array index: 0x1572 */ 0x4EEE,0x323E,
	/* Array index: 0x1572 */ 0x4EF0,0x3644,
	/* Array index: 0x1572 */ 0x4EF2,0x4367,
	/* Array index: 0x1572 */ 0x4EF6,0x376F,
	/* Array index: 0x1572 */ 0x4EF7,0x5043,
	/* Array index: 0x1572 */ 0x4EFB,0x4724,
	/* Array index: 0x1572 */ 0x4F01,0x346B,
	/* Array index: 0x1572 */ 0x4F1A,0x3271,
	/* Array index: 0x1572 */ 0x4F1C,0x5067,
	/* Array index: 0x1572 */ 0x4F1D,0x4541,
	/* Array index: 0x1572 */ 0x4F2F,0x476C,
	/* Array index: 0x1572 */ 0x4F30,0x5046,
	/* Array index: 0x1572 */ 0x4F34,0x483C,
	/* Array index: 0x1572 */ 0x4F36,0x4E62,
	/* Array index: 0x1572 */ 0x4F38,0x3F2D,
	/* Array index: 0x1572 */ 0x4F3A,0x3B47,
	/* Array index: 0x1572 */ 0x4F3C,0x3B77,
	/* Array index: 0x1572 */ 0x4F3D,0x3240,
	/* Array index: 0x1572 */ 0x4F43,0x4451,
	/* Array index: 0x1572 */ 0x4F46,0x4322,
	/* Array index: 0x1572 */ 0x4F47,0x504A,
	/* Array index: 0x1572 */ 0x4F69,0x5050,
	/* Array index: 0x1572 */ 0x4F6F,0x5053,
	/* Array index: 0x1572 */ 0x4F70,0x5051,
	/* Array index: 0x1572 */ 0x4F73,0x3242,
	/* Array index: 0x1572 */ 0x4F75,0x4A3B,
	/* Array index: 0x1572 */ 0x4F76,0x504B,
	/* Array index: 0x1572 */ 0x4F7B,0x504F,
	/* Array index: 0x1572 */ 0x4F7C,0x3873,
	/* Array index: 0x1572 */ 0x4F7F,0x3B48,
	/* Array index: 0x1572 */ 0x4F83,0x3426,
	/* Array index: 0x1572 */ 0x4F86,0x5054,
	/* Array index: 0x1572 */ 0x4F88,0x504C,
	/* Array index: 0x1572 */ 0x4F8B,0x4E63,
	/* Array index: 0x1572 */ 0x4F8D,0x3B78,
	/* Array index: 0x1572 */ 0x4F8F,0x504D,
	/* Array index: 0x1572 */ 0x4F91,0x5052,
	/* Array index: 0x1572 */ 0x4F96,0x5055,
	/* Array index: 0x1572 */ 0x4F98,0x504E,
	/* Array index: 0x1572 */ 0x4F9B,0x3621,
	/* Array index: 0x1572 */ 0x4F9D,0x304D,
	/* Array index: 0x1572 */ 0x4FA0,0x3622,
	/* Array index: 0x1572 */ 0x4FA1,0x3241,
	/* Array index: 0x1572 */ 0x4FB5,0x3F2F,
	/* Array index: 0x1572 */ 0x4FB6,0x4E37,
	/* Array index: 0x1572 */ 0x4FBF,0x4A58,
	/* Array index: 0x1572 */ 0x4FCA,0x3D53,
	/* Array index: 0x1572 */ 0x4FCE,0x5059,
	/* Array index: 0x1572 */ 0x4FD0,0x505E,
	/* Array index: 0x1572 */ 0x4FD1,0x505C,
	/* Array index: 0x1572 */ 0x4FD4,0x5057,
	/* Array index: 0x1572 */ 0x4FEE,0x3D24,
	/* Array index: 0x1572 */ 0x4FEF,0x506D,
	/* Array index: 0x1572 */ 0x4FF3,0x4750,
	/* Array index: 0x1572 */ 0x4FF5,0x4936,
	/* Array index: 0x1572 */ 0x4FF6,0x5068,
	/* Array index: 0x1572 */ 0x4FF8,0x4A70,
	/* Array index: 0x1572 */ 0x4FFA,0x3236,
	/* Array index: 0x1572 */ 0x4FFE,0x506C,
	/* Array index: 0x1572 */ 0x5005,0x5066,
	/* Array index: 0x1572 */ 0x5006,0x506F,
	/* Array index: 0x1572 */ 0x5009,0x4152,
	/* Array index: 0x1572 */ 0x500B,0x3844,
	/* Array index: 0x1572 */ 0x500D,0x475C,
	/* Array index: 0x1572 */ 0x500F,0x6047,
	/* Array index: 0x1572 */ 0x5011,0x506E,
	/* Array index: 0x1572 */ 0x5012,0x455D,
	/* Array index: 0x1572 */ 0x5014,0x5063,
	/* Array index: 0x1572 */ 0x5016,0x3876,
	/* Array index: 0x1572 */ 0x5019,0x3875,
	/* Array index: 0x1572 */ 0x501A,0x5061,
	/* Array index: 0x1572 */ 0x501F,0x3C5A,
	/* Array index: 0x1572 */ 0x5036,0x3666,
	/* Array index: 0x1572 */ 0x5039,0x3770,
	/* Array index: 0x1572 */ 0x5043,0x5070,
	/* Array index: 0x1572 */ 0x504F,0x4A50,
	/* Array index: 0x1572 */ 0x5050,0x5074,
	/* Array index: 0x1572 */ 0x5055,0x5073,
	/* Array index: 0x1572 */ 0x5056,0x5077,
	/* Array index: 0x1572 */ 0x505A,0x5076,
	/* Array index: 0x1572 */ 0x505C,0x4464,
	/* Array index: 0x1572 */ 0x5065,0x3772,
	/* Array index: 0x1572 */ 0x506C,0x5078,
	/* Array index: 0x1572 */ 0x507D,0x3536,
	/* Array index: 0x1572 */ 0x5080,0x507A,
	/* Array index: 0x1572 */ 0x5085,0x507C,
	/* Array index: 0x1572 */ 0x508D,0x4B35,
	/* Array index: 0x1572 */ 0x5091,0x3766,
	/* Array index: 0x1572 */ 0x50AC,0x3A45,
	/* Array index: 0x1572 */ 0x50AD,0x4D43,
	/* Array index: 0x1572 */ 0x50BE,0x3739,
	/* Array index: 0x1572 */ 0x50C2,0x5124,
	/* Array index: 0x1572 */ 0x50C5,0x364F,
	/* Array index: 0x1572 */ 0x50C9,0x5121,
	/* Array index: 0x1572 */ 0x50CA,0x5122,
	/* Array index: 0x1572 */ 0x50CD,0x462F,
	/* Array index: 0x1572 */ 0x50CF,0x417C,
	/* Array index: 0x1572 */ 0x50D1,0x3623,
	/* Array index: 0x1572 */ 0x50D5,0x4B4D,
	/* Array index: 0x1572 */ 0x50D6,0x5125,
	/* Array index: 0x1572 */ 0x50DA,0x4E3D,
	/* Array index: 0x1572 */ 0x50DE,0x5126,
	/* Array index: 0x1572 */ 0x50E3,0x5129,
	/* Array index: 0x1572 */ 0x50E5,0x5127,
	/* Array index: 0x1572 */ 0x50E7,0x414E,
	/* Array index: 0x1572 */ 0x50ED,0x5128,
	/* Array index: 0x1572 */ 0x50EE,0x512A,
	/* Array index: 0x1572 */ 0x50F5,0x512C,
	/* Array index: 0x1572 */ 0x50F9,0x512B,
	/* Array index: 0x1572 */ 0x50FB,0x4A48,
	/* Array index: 0x1572 */ 0x5109,0x512D,
	/* Array index: 0x1572 */ 0x511F,0x3D7E,
	/* Array index: 0x1572 */ 0x5121,0x5134,
	/* Array index: 0x1572 */ 0x512A,0x4D25,
	/* Array index: 0x1572 */ 0x5132,0x4C59,
	/* Array index: 0x1572 */ 0x5137,0x5136,
	/* Array index: 0x1572 */ 0x515A,0x455E,
	/* Array index: 0x1572 */ 0x515C,0x3375,
	/* Array index: 0x1572 */ 0x5162,0x513E,
	/* Array index: 0x1572 */ 0x5165,0x467E,
	/* Array index: 0x1572 */ 0x5171,0x3626,
	/* Array index: 0x1572 */ 0x517C,0x3773,
	/* Array index: 0x1572 */ 0x5180,0x5143,
	/* Array index: 0x1572 */ 0x5182,0x5144,
	/* Array index: 0x1572 */ 0x5185,0x4662,
	/* Array index: 0x1572 */ 0x5186,0x315F,
	/* Array index: 0x1572 */ 0x51A0,0x3427,
	/* Array index: 0x1572 */ 0x51BD,0x5158,
	/* Array index: 0x1572 */ 0x51D6,0x5245,
	/* Array index: 0x1572 */ 0x51ED,0x515F,
	/* Array index: 0x1572 */ 0x51F0,0x5160,
	/* Array index: 0x1572 */ 0x51F1,0x332E,
	/* Array index: 0x1572 */ 0x520E,0x5166,
	/* Array index: 0x1572 */ 0x5211,0x373A,
	/* Array index: 0x1572 */ 0x5214,0x5165,
	/* Array index: 0x1572 */ 0x5217,0x4E73,
	/* Array index: 0x1572 */ 0x521D,0x3D69,
	/* Array index: 0x1572 */ 0x5224,0x483D,
	/* Array index: 0x1572 */ 0x5225,0x4A4C,
	/* Array index: 0x1572 */ 0x5227,0x5167,
	/* Array index: 0x1572 */ 0x5229,0x4D78,
	/* Array index: 0x1572 */ 0x522A,0x5168,
	/* Array index: 0x1572 */ 0x522E,0x5169,
	/* Array index: 0x1572 */ 0x5230,0x457E,
	/* Array index: 0x1572 */ 0x5233,0x516A,
	/* Array index: 0x1572 */ 0x5243,0x4466,
	/* Array index: 0x1572 */ 0x5244,0x516D,
	/* Array index: 0x1572 */ 0x5247,0x4227,
	/* Array index: 0x1572 */ 0x5254,0x5171,
	/* Array index: 0x1572 */ 0x5256,0x4B36,
	/* Array index: 0x1572 */ 0x525B,0x3964,
	/* Array index: 0x1572 */ 0x525E,0x5170,
	/* Array index: 0x1572 */ 0x5269,0x5174,
	/* Array index: 0x1572 */ 0x526A,0x5172,
	/* Array index: 0x1572 */ 0x527D,0x5177,
	/* Array index: 0x1572 */ 0x527F,0x5176,
	/* Array index: 0x1572 */ 0x5283,0x3344,
	/* Array index: 0x1572 */ 0x528D,0x5178,
	/* Array index: 0x1572 */ 0x5291,0x517D,
	/* Array index: 0x1572 */ 0x5292,0x517A,
	/* Array index: 0x1572 */ 0x5294,0x5179,
	/* Array index: 0x1572 */ 0x529B,0x4E4F,
	/* Array index: 0x1572 */ 0x529F,0x3879,
	/* Array index: 0x1572 */ 0x52A0,0x3243,
	/* Array index: 0x1572 */ 0x52A3,0x4E74,
	/* Array index: 0x1572 */ 0x52B1,0x4E65,
	/* Array index: 0x1572 */ 0x52B4,0x4F2B,
	/* Array index: 0x1572 */ 0x52B5,0x5225,
	/* Array index: 0x1572 */ 0x52B9,0x387A,
	/* Array index: 0x1572 */ 0x52BC,0x5224,
	/* Array index: 0x1572 */ 0x52BE,0x332F,
	/* Array index: 0x1572 */ 0x52C1,0x5226,
	/* Array index: 0x1572 */ 0x52C3,0x4B56,
	/* Array index: 0x1572 */ 0x52C5,0x443C,
	/* Array index: 0x1572 */ 0x52C7,0x4D26,
	/* Array index: 0x1572 */ 0x52C9,0x4A59,
	/* Array index: 0x1572 */ 0x52CD,0x5227,
	/* Array index: 0x1572 */ 0x52D2,0x7055,
	/* Array index: 0x1572 */ 0x52F2,0x372E,
	/* Array index: 0x1572 */ 0x52F3,0x522E,
	/* Array index: 0x1572 */ 0x52F5,0x522F,
	/* Array index: 0x1572 */ 0x530D,0x5235,
	/* Array index: 0x1572 */ 0x530F,0x5237,
	/* Array index: 0x1572 */ 0x5310,0x5236,
	/* Array index: 0x1572 */ 0x531D,0x4159,
	/* Array index: 0x1572 */ 0x5320,0x3E22,
	/* Array index: 0x1572 */ 0x5321,0x3629,
	/* Array index: 0x1572 */ 0x5323,0x523A,
	/* Array index: 0x1572 */ 0x532A,0x485B,
	/* Array index: 0x1572 */ 0x532F,0x523B,
	/* Array index: 0x1572 */ 0x5331,0x523C,
	/* Array index: 0x1572 */ 0x5333,0x523D,
	/* Array index: 0x1572 */ 0x534D,0x5244,
	/* Array index: 0x1572 */ 0x5366,0x3735,
	/* Array index: 0x1572 */ 0x5369,0x5247,
	/* Array index: 0x1572 */ 0x537B,0x524A,
	/* Array index: 0x1572 */ 0x537F,0x362A,
	/* Array index: 0x1572 */ 0x5382,0x524C,
	/* Array index: 0x1572 */ 0x5384,0x4C71,
	/* Array index: 0x1572 */ 0x5396,0x524D,
	/* Array index: 0x1572 */ 0x5398,0x4E52,
	/* Array index: 0x1572 */ 0x539A,0x387C,
	/* Array index: 0x1572 */ 0x539F,0x3836,
	/* Array index: 0x1572 */ 0x53A0,0x524E,
	/* Array index: 0x1572 */ 0x53AD,0x315E,
	/* Array index: 0x1572 */ 0x53AE,0x5251,
	/* Array index: 0x1572 */ 0x53B0,0x5252,
	/* Array index: 0x1572 */ 0x53B3,0x3837,
	/* Array index: 0x1572 */ 0x53B6,0x5253,
	/* Array index: 0x1572 */ 0x53BB,0x356E,
	/* Array index: 0x1572 */ 0x53C2,0x3B32,
	/* Array index: 0x1572 */ 0x53C3,0x5254,
	/* Array index: 0x1572 */ 0x53D4,0x3D47,
	/* Array index: 0x1572 */ 0x53D6,0x3C68,
	/* Array index: 0x1572 */ 0x53D7,0x3C75,
	/* Array index: 0x1572 */ 0x53D9,0x3D76,
	/* Array index: 0x1572 */ 0x53DB,0x4840,
	/* Array index: 0x1572 */ 0x5401,0x525E,
	/* Array index: 0x1572 */ 0x5403,0x3549,
	/* Array index: 0x1572 */ 0x5404,0x3346,
	/* Array index: 0x1572 */ 0x541B,0x372F,
	/* Array index: 0x1572 */ 0x541D,0x5267,
	/* Array index: 0x1572 */ 0x541F,0x3663,
	/* Array index: 0x1572 */ 0x5420,0x4B4A,
	/* Array index: 0x1572 */ 0x5426,0x485D,
	/* Array index: 0x1572 */ 0x5436,0x5265,
	/* Array index: 0x1572 */ 0x544E,0x5268,
	/* Array index: 0x1572 */ 0x5451,0x465D,
	/* Array index: 0x1572 */ 0x545F,0x526C,
	/* Array index: 0x1572 */ 0x5468,0x3C7E,
	/* Array index: 0x1572 */ 0x546A,0x3C76,
	/* Array index: 0x1572 */ 0x5470,0x526F,
	/* Array index: 0x1572 */ 0x5471,0x526D,
	/* Array index: 0x1572 */ 0x5480,0x5272,
	/* Array index: 0x1572 */ 0x5484,0x5274,
	/* Array index: 0x1572 */ 0x5486,0x5276,
	/* Array index: 0x1572 */ 0x54A2,0x5278,
	/* Array index: 0x1572 */ 0x54A4,0x5323,
	/* Array index: 0x1572 */ 0x54A5,0x527A,
	/* Array index: 0x1572 */ 0x54A8,0x527E,
	/* Array index: 0x1572 */ 0x54AB,0x5321,
	/* Array index: 0x1572 */ 0x54AC,0x527B,
	/* Array index: 0x1572 */ 0x54AF,0x533E,
	/* Array index: 0x1572 */ 0x54B2,0x3A69,
	/* Array index: 0x1572 */ 0x54B3,0x3331,
	/* Array index: 0x1572 */ 0x54B8,0x5279,
	/* Array index: 0x1572 */ 0x54D8,0x5326,
	/* Array index: 0x1572 */ 0x54E1,0x3077,
	/* Array index: 0x1572 */ 0x54E2,0x532F,
	/* Array index: 0x1572 */ 0x54ED,0x532D,
	/* Array index: 0x1572 */ 0x54EE,0x532C,
	/* Array index: 0x1572 */ 0x54F2,0x452F,
	/* Array index: 0x1572 */ 0x54FA,0x532E,
	/* Array index: 0x1572 */ 0x54FD,0x532B,
	/* Array index: 0x1572 */ 0x5504,0x3134,
	/* Array index: 0x1572 */ 0x5506,0x3A36,
	/* Array index: 0x1572 */ 0x5507,0x3F30,
	/* Array index: 0x1572 */ 0x550F,0x5329,
	/* Array index: 0x1572 */ 0x5510,0x4562,
	/* Array index: 0x1572 */ 0x5514,0x532A,
	/* Array index: 0x1572 */ 0x5516,0x3022,
	/* Array index: 0x1572 */ 0x552E,0x5334,
	/* Array index: 0x1572 */ 0x552F,0x4D23,
	/* Array index: 0x1572 */ 0x5531,0x3E27,
	/* Array index: 0x1572 */ 0x5533,0x533A,
	/* Array index: 0x1572 */ 0x5538,0x5339,
	/* Array index: 0x1572 */ 0x5539,0x5330,
	/* Array index: 0x1572 */ 0x553E,0x4243,
	/* Array index: 0x1572 */ 0x5540,0x5331,
	/* Array index: 0x1572 */ 0x554C,0x5333,
	/* Array index: 0x1572 */ 0x554F,0x4C64,
	/* Array index: 0x1572 */ 0x5553,0x373C,
	/* Array index: 0x1572 */ 0x5556,0x5337,
	/* Array index: 0x1572 */ 0x5557,0x5338,
	/* Array index: 0x1572 */ 0x555C,0x5335,
	/* Array index: 0x1572 */ 0x555D,0x533B,
	/* Array index: 0x1572 */ 0x5563,0x5332,
	/* Array index: 0x1572 */ 0x557B,0x5341,
	/* Array index: 0x1572 */ 0x557C,0x5346,
	/* Array index: 0x1572 */ 0x557E,0x5342,
	/* Array index: 0x1572 */ 0x5580,0x533D,
	/* Array index: 0x1572 */ 0x5583,0x5347,
	/* Array index: 0x1572 */ 0x5584,0x4131,
	/* Array index: 0x1572 */ 0x55B6,0x3144,
	/* Array index: 0x1572 */ 0x55C4,0x534E,
	/* Array index: 0x1572 */ 0x55C5,0x534C,
	/* Array index: 0x1572 */ 0x55C7,0x5427,
	/* Array index: 0x1572 */ 0x55D4,0x5351,
	/* Array index: 0x1572 */ 0x55DA,0x534B,
	/* Array index: 0x1572 */ 0x55DC,0x534F,
	/* Array index: 0x1572 */ 0x55DF,0x534D,
	/* Array index: 0x1572 */ 0x55E3,0x3B4C,
	/* Array index: 0x1572 */ 0x55E4,0x5350,
	/* Array index: 0x1572 */ 0x55F7,0x5353,
	/* Array index: 0x1572 */ 0x55F9,0x5358,
	/* Array index: 0x1572 */ 0x55FD,0x5356,
	/* Array index: 0x1572 */ 0x55FE,0x5355,
	/* Array index: 0x1572 */ 0x5606,0x4332,
	/* Array index: 0x1572 */ 0x5609,0x3245,
	/* Array index: 0x1572 */ 0x561B,0x5357,
	/* Array index: 0x1572 */ 0x5629,0x325E,
	/* Array index: 0x1572 */ 0x562F,0x5362,
	/* Array index: 0x1572 */ 0x5631,0x3E7C,
	/* Array index: 0x1572 */ 0x5632,0x535E,
	/* Array index: 0x1572 */ 0x5634,0x535C,
	/* Array index: 0x1572 */ 0x5636,0x535D,
	/* Array index: 0x1572 */ 0x5638,0x535F,
	/* Array index: 0x1572 */ 0x5642,0x313D,
	/* Array index: 0x1572 */ 0x564C,0x4139,
	/* Array index: 0x1572 */ 0x564E,0x5359,
	/* Array index: 0x1572 */ 0x5650,0x535A,
	/* Array index: 0x1572 */ 0x565B,0x337A,
	/* Array index: 0x1572 */ 0x5664,0x5361,
	/* Array index: 0x1572 */ 0x5674,0x4A2E,
	/* Array index: 0x1572 */ 0x5678,0x4655,
	/* Array index: 0x1572 */ 0x567A,0x4838,
	/* Array index: 0x1572 */ 0x5680,0x5366,
	/* Array index: 0x1572 */ 0x5686,0x5365,
	/* Array index: 0x1572 */ 0x5687,0x3345,
	/* Array index: 0x1572 */ 0x568A,0x5367,
	/* Array index: 0x1572 */ 0x568F,0x536A,
	/* Array index: 0x1572 */ 0x5694,0x5369,
	/* Array index: 0x1572 */ 0x56A0,0x5368,
	/* Array index: 0x1572 */ 0x56A2,0x4739,
	/* Array index: 0x1572 */ 0x56A5,0x536B,
	/* Array index: 0x1572 */ 0x56AE,0x536C,
	/* Array index: 0x1572 */ 0x56B4,0x536E,
	/* Array index: 0x1572 */ 0x56B6,0x536D,
	/* Array index: 0x1572 */ 0x56BC,0x5370,
	/* Array index: 0x1572 */ 0x56C8,0x5374,
	/* Array index: 0x1572 */ 0x56CE,0x5375,
	/* Array index: 0x1572 */ 0x56D1,0x5376,
	/* Array index: 0x1572 */ 0x56D3,0x5377,
	/* Array index: 0x1572 */ 0x56DE,0x3273,
	/* Array index: 0x1572 */ 0x56E0,0x3078,
	/* Array index: 0x1572 */ 0x56E3,0x4344,
	/* Array index: 0x1572 */ 0x56EE,0x5379,
	/* Array index: 0x1572 */ 0x56F0,0x3A24,
	/* Array index: 0x1572 */ 0x56F2,0x304F,
	/* Array index: 0x1572 */ 0x56F3,0x3F5E,
	/* Array index: 0x1572 */ 0x56F9,0x537A,
	/* Array index: 0x1572 */ 0x56FA,0x3847,
	/* Array index: 0x1572 */ 0x56FD,0x3971,
	/* Array index: 0x1572 */ 0x56FF,0x537C,
	/* Array index: 0x1572 */ 0x5700,0x537B,
	/* Array index: 0x1572 */ 0x5703,0x4A60,
	/* Array index: 0x1572 */ 0x5704,0x537D,
	/* Array index: 0x1572 */ 0x5708,0x5421,
	/* Array index: 0x1572 */ 0x5709,0x537E,
	/* Array index: 0x1572 */ 0x570B,0x5422,
	/* Array index: 0x1572 */ 0x570D,0x5423,
	/* Array index: 0x1572 */ 0x570F,0x3777,
	/* Array index: 0x1572 */ 0x5712,0x3160,
	/* Array index: 0x1572 */ 0x5713,0x5424,
	/* Array index: 0x1572 */ 0x5716,0x5426,
	/* Array index: 0x1572 */ 0x5718,0x5425,
	/* Array index: 0x1572 */ 0x571C,0x5428,
	/* Array index: 0x1572 */ 0x571F,0x455A,
	/* Array index: 0x1572 */ 0x572D,0x373D,
	/* Array index: 0x1572 */ 0x5730,0x434F,
	/* Array index: 0x1572 */ 0x5737,0x542A,
	/* Array index: 0x1572 */ 0x5738,0x542B,
	/* Array index: 0x1572 */ 0x573B,0x542D,
	/* Array index: 0x1572 */ 0x5740,0x542E,
	/* Array index: 0x1572 */ 0x5742,0x3A64,
	/* Array index: 0x1572 */ 0x5747,0x3651,
	/* Array index: 0x1572 */ 0x574A,0x4B37,
	/* Array index: 0x1572 */ 0x5761,0x5433,
	/* Array index: 0x1572 */ 0x5764,0x3A25,
	/* Array index: 0x1572 */ 0x5766,0x4333,
	/* Array index: 0x1572 */ 0x5769,0x5430,
	/* Array index: 0x1572 */ 0x576A,0x445A,
	/* Array index: 0x1572 */ 0x577F,0x5434,
	/* Array index: 0x1572 */ 0x5782,0x3F62,
	/* Array index: 0x1572 */ 0x5788,0x5432,
	/* Array index: 0x1572 */ 0x5789,0x5435,
	/* Array index: 0x1572 */ 0x578B,0x373F,
	/* Array index: 0x1572 */ 0x5793,0x5436,
	/* Array index: 0x1572 */ 0x57AA,0x543A,
	/* Array index: 0x1572 */ 0x57B0,0x543B,
	/* Array index: 0x1572 */ 0x57B3,0x5438,
	/* Array index: 0x1572 */ 0x57C0,0x5431,
	/* Array index: 0x1572 */ 0x57C3,0x543C,
	/* Array index: 0x1572 */ 0x57C6,0x543D,
	/* Array index: 0x1572 */ 0x57CB,0x4B64,
	/* Array index: 0x1572 */ 0x57CE,0x3E6B,
	/* Array index: 0x1572 */ 0x57DC,0x4738,
	/* Array index: 0x1572 */ 0x57DF,0x3068,
	/* Array index: 0x1572 */ 0x57E0,0x4956,
	/* Array index: 0x1572 */ 0x57E3,0x5443,
	/* Array index: 0x1572 */ 0x57F4,0x3E7D,
	/* Array index: 0x1572 */ 0x57F7,0x3C39,
	/* Array index: 0x1572 */ 0x57F9,0x475D,
	/* Array index: 0x1572 */ 0x57FA,0x3470,
	/* Array index: 0x1572 */ 0x57FC,0x3A6B,
	/* Array index: 0x1572 */ 0x5800,0x4B59,
	/* Array index: 0x1572 */ 0x5802,0x4632,
	/* Array index: 0x1572 */ 0x5805,0x3778,
	/* Array index: 0x1572 */ 0x5806,0x424F,
	/* Array index: 0x1572 */ 0x580A,0x5441,
	/* Array index: 0x1572 */ 0x580B,0x5444,
	/* Array index: 0x1572 */ 0x5815,0x4244,
	/* Array index: 0x1572 */ 0x5819,0x5445,
	/* Array index: 0x1572 */ 0x581D,0x5446,
	/* Array index: 0x1572 */ 0x5821,0x5448,
	/* Array index: 0x1572 */ 0x5824,0x4469,
	/* Array index: 0x1572 */ 0x582A,0x342E,
	/* Array index: 0x1572 */ 0x583A,0x3A66,
	/* Array index: 0x1572 */ 0x583D,0x544E,
	/* Array index: 0x1572 */ 0x5840,0x4A3D,
	/* Array index: 0x1572 */ 0x5841,0x4E5D,
	/* Array index: 0x1572 */ 0x584A,0x3274,
	/* Array index: 0x1572 */ 0x584B,0x544A,
	/* Array index: 0x1572 */ 0x5851,0x413A,
	/* Array index: 0x1572 */ 0x5852,0x544D,
	/* Array index: 0x1572 */ 0x5854,0x4563,
	/* Array index: 0x1572 */ 0x585E,0x3A49,
	/* Array index: 0x1572 */ 0x5862,0x5449,
	/* Array index: 0x1572 */ 0x5869,0x3176,
	/* Array index: 0x1572 */ 0x586B,0x4536,
	/* Array index: 0x1572 */ 0x5870,0x544B,
	/* Array index: 0x1572 */ 0x5872,0x5447,
	/* Array index: 0x1572 */ 0x5875,0x3F50,
	/* Array index: 0x1572 */ 0x5879,0x544F,
	/* Array index: 0x1572 */ 0x587E,0x3D4E,
	/* Array index: 0x1572 */ 0x5883,0x362D,
	/* Array index: 0x1572 */ 0x5885,0x5450,
	/* Array index: 0x1572 */ 0x5893,0x4A68,
	/* Array index: 0x1572 */ 0x5897,0x417D,
	/* Array index: 0x1572 */ 0x589C,0x4446,
	/* Array index: 0x1572 */ 0x589F,0x5452,
	/* Array index: 0x1572 */ 0x58A8,0x4B4F,
	/* Array index: 0x1572 */ 0x58AB,0x5453,
	/* Array index: 0x1572 */ 0x58AE,0x5458,
	/* Array index: 0x1572 */ 0x58B3,0x4A2F,
	/* Array index: 0x1572 */ 0x58BE,0x3A26,
	/* Array index: 0x1572 */ 0x58C1,0x4A49,
	/* Array index: 0x1572 */ 0x58C5,0x5459,
	/* Array index: 0x1572 */ 0x58C7,0x4345,
	/* Array index: 0x1572 */ 0x58CA,0x3275,
	/* Array index: 0x1572 */ 0x58CC,0x3E6D,
	/* Array index: 0x1572 */ 0x58D1,0x545B,
	/* Array index: 0x1572 */ 0x58D3,0x545A,
	/* Array index: 0x1572 */ 0x58DC,0x5460,
	/* Array index: 0x1572 */ 0x58DE,0x5455,
	/* Array index: 0x1572 */ 0x58DF,0x5462,
	/* Array index: 0x1572 */ 0x58E4,0x5461,
	/* Array index: 0x1572 */ 0x58E5,0x545F,
	/* Array index: 0x1572 */ 0x5902,0x5469,
	/* Array index: 0x1572 */ 0x5909,0x4A51,
	/* Array index: 0x1572 */ 0x590A,0x546A,
	/* Array index: 0x1572 */ 0x590F,0x3246,
	/* Array index: 0x1572 */ 0x5910,0x546B,
	/* Array index: 0x1572 */ 0x5922,0x4C34,
	/* Array index: 0x1572 */ 0x5925,0x546E,
	/* Array index: 0x1572 */ 0x5937,0x3050,
	/* Array index: 0x1572 */ 0x5938,0x5472,
	/* Array index: 0x1572 */ 0x593E,0x5473,
	/* Array index: 0x1572 */ 0x5944,0x3162,
	/* Array index: 0x1572 */ 0x5960,0x547B,
	/* Array index: 0x1572 */ 0x5962,0x547A,
	/* Array index: 0x1572 */ 0x5973,0x3D77,
	/* Array index: 0x1572 */ 0x5974,0x455B,
	/* Array index: 0x1572 */ 0x5978,0x5521,
	/* Array index: 0x1572 */ 0x597D,0x3925,
	/* Array index: 0x1572 */ 0x598A,0x4725,
	/* Array index: 0x1572 */ 0x598D,0x552B,
	/* Array index: 0x1572 */ 0x5993,0x3538,
	/* Array index: 0x1572 */ 0x5996,0x4D45,
	/* Array index: 0x1572 */ 0x5999,0x4C2F,
	/* Array index: 0x1572 */ 0x599B,0x562C,
	/* Array index: 0x1572 */ 0x599D,0x5523,
	/* Array index: 0x1572 */ 0x59A3,0x5526,
	/* Array index: 0x1572 */ 0x59A5,0x4245,
	/* Array index: 0x1572 */ 0x59A8,0x4B38,
	/* Array index: 0x1572 */ 0x59AC,0x454A,
	/* Array index: 0x1572 */ 0x59B2,0x5527,
	/* Array index: 0x1572 */ 0x59B9,0x4B65,
	/* Array index: 0x1572 */ 0x59BB,0x3A4A,
	/* Array index: 0x1572 */ 0x59BE,0x3E2A,
	/* Array index: 0x1572 */ 0x59C6,0x5528,
	/* Array index: 0x1572 */ 0x59C9,0x3B50,
	/* Array index: 0x1572 */ 0x59CB,0x3B4F,
	/* Array index: 0x1572 */ 0x59D9,0x552C,
	/* Array index: 0x1572 */ 0x59DA,0x552D,
	/* Array index: 0x1572 */ 0x59DC,0x552A,
	/* Array index: 0x1572 */ 0x59E5,0x3138,
	/* Array index: 0x1572 */ 0x59E6,0x342F,
	/* Array index: 0x1572 */ 0x59E8,0x5529,
	/* Array index: 0x1572 */ 0x59EA,0x4C45,
	/* Array index: 0x1572 */ 0x59EB,0x4931,
	/* Array index: 0x1572 */ 0x59F6,0x3028,
	/* Array index: 0x1572 */ 0x59FB,0x3079,
	/* Array index: 0x1572 */ 0x59FF,0x3B51,
	/* Array index: 0x1572 */ 0x5A01,0x3052,
	/* Array index: 0x1572 */ 0x5A03,0x3023,
	/* Array index: 0x1572 */ 0x5A09,0x5532,
	/* Array index: 0x1572 */ 0x5A11,0x5530,
	/* Array index: 0x1572 */ 0x5A18,0x4C3C,
	/* Array index: 0x1572 */ 0x5A1A,0x5533,
	/* Array index: 0x1572 */ 0x5A1C,0x5531,
	/* Array index: 0x1572 */ 0x5A1F,0x552F,
	/* Array index: 0x1572 */ 0x5A20,0x3F31,
	/* Array index: 0x1572 */ 0x5A25,0x552E,
	/* Array index: 0x1572 */ 0x5A29,0x4A5A,
	/* Array index: 0x1572 */ 0x5A2F,0x3864,
	/* Array index: 0x1572 */ 0x5A35,0x5537,
	/* Array index: 0x1572 */ 0x5A36,0x5538,
	/* Array index: 0x1572 */ 0x5A3C,0x3E2B,
	/* Array index: 0x1572 */ 0x5A40,0x5534,
	/* Array index: 0x1572 */ 0x5A41,0x4F2C,
	/* Array index: 0x1572 */ 0x5A46,0x474C,
	/* Array index: 0x1572 */ 0x5A49,0x5536,
	/* Array index: 0x1572 */ 0x5A5A,0x3A27,
	/* Array index: 0x1572 */ 0x5A62,0x5539,
	/* Array index: 0x1572 */ 0x5A66,0x4958,
	/* Array index: 0x1572 */ 0x5A6A,0x553A,
	/* Array index: 0x1572 */ 0x5A6C,0x5535,
	/* Array index: 0x1572 */ 0x5A7F,0x4C3B,
	/* Array index: 0x1572 */ 0x5A92,0x475E,
	/* Array index: 0x1572 */ 0x5A9A,0x553B,
	/* Array index: 0x1572 */ 0x5A9B,0x4932,
	/* Array index: 0x1572 */ 0x5AC9,0x3C3B,
	/* Array index: 0x1572 */ 0x5ACB,0x553E,
	/* Array index: 0x1572 */ 0x5ACC,0x3779,
	/* Array index: 0x1572 */ 0x5AD0,0x554C,
	/* Array index: 0x1572 */ 0x5AD6,0x5545,
	/* Array index: 0x1572 */ 0x5AD7,0x5542,
	/* Array index: 0x1572 */ 0x5AE1,0x4364,
	/* Array index: 0x1572 */ 0x5AE3,0x5541,
	/* Array index: 0x1572 */ 0x5AE6,0x5543,
	/* Array index: 0x1572 */ 0x5AE9,0x5544,
	/* Array index: 0x1572 */ 0x5AFA,0x5546,
	/* Array index: 0x1572 */ 0x5AFB,0x5547,
	/* Array index: 0x1572 */ 0x5B09,0x3472,
	/* Array index: 0x1572 */ 0x5B0B,0x5549,
	/* Array index: 0x1572 */ 0x5B0C,0x5548,
	/* Array index: 0x1572 */ 0x5B16,0x554A,
	/* Array index: 0x1572 */ 0x5B22,0x3E6E,
	/* Array index: 0x1572 */ 0x5B2A,0x554D,
	/* Array index: 0x1572 */ 0x5B2C,0x445C,
	/* Array index: 0x1572 */ 0x5B30,0x3145,
	/* Array index: 0x1572 */ 0x5B32,0x554B,
	/* Array index: 0x1572 */ 0x5B36,0x554E,
	/* Array index: 0x1572 */ 0x5B3E,0x554F,
	/* Array index: 0x1572 */ 0x5B40,0x5552,
	/* Array index: 0x1572 */ 0x5B43,0x5550,
	/* Array index: 0x1572 */ 0x5B45,0x5551,
	/* Array index: 0x1572 */ 0x5B50,0x3B52,
	/* Array index: 0x1572 */ 0x5B51,0x5553,
	/* Array index: 0x1572 */ 0x5B69,0x5558,
	/* Array index: 0x1572 */ 0x5B6B,0x4239,
	/* Array index: 0x1572 */ 0x5B70,0x5559,
	/* Array index: 0x1572 */ 0x5B71,0x5623,
	/* Array index: 0x1572 */ 0x5B73,0x555A,
	/* Array index: 0x1572 */ 0x5B75,0x555B,
	/* Array index: 0x1572 */ 0x5B78,0x555C,
	/* Array index: 0x1572 */ 0x5B7A,0x555E,
	/* Array index: 0x1572 */ 0x5B80,0x555F,
	/* Array index: 0x1572 */ 0x5B83,0x5560,
	/* Array index: 0x1572 */ 0x5BAE,0x355C,
	/* Array index: 0x1572 */ 0x5BB0,0x3A4B,
	/* Array index: 0x1572 */ 0x5BBF,0x3D49,
	/* Array index: 0x1572 */ 0x5BCC,0x4959,
	/* Array index: 0x1572 */ 0x5BEE,0x4E40,
	/* Array index: 0x1572 */ 0x5BF0,0x556E,
	/* Array index: 0x1572 */ 0x5BF3,0x5570,
	/* Array index: 0x1572 */ 0x5BF5,0x437E,
	/* Array index: 0x1572 */ 0x5BF6,0x556F,
	/* Array index: 0x1572 */ 0x5BF8,0x4023,
	/* Array index: 0x1572 */ 0x5BFA,0x3B7B,
	/* Array index: 0x1572 */ 0x5C16,0x406D,
	/* Array index: 0x1572 */ 0x5C1A,0x3E30,
	/* Array index: 0x1572 */ 0x5C20,0x5576,
	/* Array index: 0x1572 */ 0x5C22,0x5577,
	/* Array index: 0x1572 */ 0x5C24,0x4C60,
	/* Array index: 0x1572 */ 0x5C28,0x5578,
	/* Array index: 0x1572 */ 0x5C2D,0x3646,
	/* Array index: 0x1572 */ 0x5C31,0x3D22,
	/* Array index: 0x1572 */ 0x5C45,0x356F,
	/* Array index: 0x1572 */ 0x5C46,0x557C,
	/* Array index: 0x1572 */ 0x5C48,0x367E,
	/* Array index: 0x1572 */ 0x5C5E,0x4230,
	/* Array index: 0x1572 */ 0x5C60,0x454B,
	/* Array index: 0x1572 */ 0x5C61,0x3C48,
	/* Array index: 0x1572 */ 0x5C64,0x4158,
	/* Array index: 0x1572 */ 0x5C65,0x4D7A,
	/* Array index: 0x1572 */ 0x5C6C,0x5624,
	/* Array index: 0x1572 */ 0x5C6E,0x5625,
	/* Array index: 0x1572 */ 0x5C6F,0x4656,
	/* Array index: 0x1572 */ 0x5C71,0x3B33,
	/* Array index: 0x1572 */ 0x5C76,0x5627,
	/* Array index: 0x1572 */ 0x5C79,0x5628,
	/* Array index: 0x1572 */ 0x5C8C,0x5629,
	/* Array index: 0x1572 */ 0x5C90,0x3474,
	/* Array index: 0x1572 */ 0x5C91,0x562A,
	/* Array index: 0x1572 */ 0x5C94,0x562B,
	/* Array index: 0x1572 */ 0x5CA1,0x322C,
	/* Array index: 0x1572 */ 0x5CB1,0x4252,
	/* Array index: 0x1572 */ 0x5CB3,0x3359,
	/* Array index: 0x1572 */ 0x5CC5,0x5632,
	/* Array index: 0x1572 */ 0x5CC7,0x5634,
	/* Array index: 0x1572 */ 0x5CD9,0x5635,
	/* Array index: 0x1572 */ 0x5CE0,0x463D,
	/* Array index: 0x1572 */ 0x5CE1,0x362E,
	/* Array index: 0x1572 */ 0x5CED,0x5639,
	/* Array index: 0x1572 */ 0x5CEF,0x4A77,
	/* Array index: 0x1572 */ 0x5CF0,0x4A76,
	/* Array index: 0x1572 */ 0x5CF6,0x4567,
	/* Array index: 0x1572 */ 0x5CFA,0x5638,
	/* Array index: 0x1572 */ 0x5CFB,0x3D54,
	/* Array index: 0x1572 */ 0x5CFD,0x5637,
	/* Array index: 0x1572 */ 0x5D07,0x3F72,
	/* Array index: 0x1572 */ 0x5D0B,0x563C,
	/* Array index: 0x1572 */ 0x5D0E,0x3A6A,
	/* Array index: 0x1572 */ 0x5D11,0x5642,
	/* Array index: 0x1572 */ 0x5D1F,0x5640,
	/* Array index: 0x1572 */ 0x5D22,0x5644,
	/* Array index: 0x1572 */ 0x5D29,0x4A78,
	/* Array index: 0x1572 */ 0x5D4B,0x564B,
	/* Array index: 0x1572 */ 0x5D4C,0x5648,
	/* Array index: 0x1572 */ 0x5D4E,0x564A,
	/* Array index: 0x1572 */ 0x5D50,0x4D72,
	/* Array index: 0x1572 */ 0x5D52,0x5649,
	/* Array index: 0x1572 */ 0x5D5C,0x563F,
	/* Array index: 0x1572 */ 0x5D69,0x3F73,
	/* Array index: 0x1572 */ 0x5D6C,0x564C,
	/* Array index: 0x1572 */ 0x5D6F,0x3A37,
	/* Array index: 0x1572 */ 0x5D73,0x564D,
	/* Array index: 0x1572 */ 0x5D76,0x564E,
	/* Array index: 0x1572 */ 0x5D82,0x5651,
	/* Array index: 0x1572 */ 0x5D84,0x5650,
	/* Array index: 0x1572 */ 0x5D87,0x564F,
	/* Array index: 0x1572 */ 0x5D8B,0x4568,
	/* Array index: 0x1572 */ 0x5D8C,0x563A,
	/* Array index: 0x1572 */ 0x5D90,0x5657,
	/* Array index: 0x1572 */ 0x5D9D,0x5653,
	/* Array index: 0x1572 */ 0x5DA2,0x5652,
	/* Array index: 0x1572 */ 0x5DAC,0x5654,
	/* Array index: 0x1572 */ 0x5DAE,0x5655,
	/* Array index: 0x1572 */ 0x5DB7,0x5658,
	/* Array index: 0x1572 */ 0x5DBA,0x4E66,
	/* Array index: 0x1572 */ 0x5DBC,0x5659,
	/* Array index: 0x1572 */ 0x5DBD,0x5656,
	/* Array index: 0x1572 */ 0x5DC9,0x565A,
	/* Array index: 0x1572 */ 0x5DCC,0x3460,
	/* Array index: 0x1572 */ 0x5DCD,0x565B,
	/* Array index: 0x1572 */ 0x5DD2,0x565D,
	/* Array index: 0x1572 */ 0x5DD3,0x565C,
	/* Array index: 0x1572 */ 0x5DD6,0x565E,
	/* Array index: 0x1572 */ 0x5DDB,0x565F,
	/* Array index: 0x1572 */ 0x5DDD,0x406E,
	/* Array index: 0x1572 */ 0x5DDE,0x3D23,
	/* Array index: 0x1572 */ 0x5DE1,0x3D64,
	/* Array index: 0x1572 */ 0x5DEB,0x5660,
	/* Array index: 0x1572 */ 0x5DEE,0x3A39,
	/* Array index: 0x1572 */ 0x5DFB,0x342C,
	/* Array index: 0x1572 */ 0x5DFD,0x4327,
	/* Array index: 0x1572 */ 0x5DFE,0x3652,
	/* Array index: 0x1572 */ 0x5E02,0x3B54,
	/* Array index: 0x1572 */ 0x5E03,0x495B,
	/* Array index: 0x1572 */ 0x5E06,0x4841,
	/* Array index: 0x1572 */ 0x5E0B,0x5663,
	/* Array index: 0x1572 */ 0x5E0C,0x3475,
	/* Array index: 0x1572 */ 0x5E11,0x5666,
	/* Array index: 0x1572 */ 0x5E16,0x4421,
	/* Array index: 0x1572 */ 0x5E25,0x3F63,
	/* Array index: 0x1572 */ 0x5E2B,0x3B55,
	/* Array index: 0x1572 */ 0x5E2D,0x404A,
	/* Array index: 0x1572 */ 0x5E2F,0x4253,
	/* Array index: 0x1572 */ 0x5E30,0x3522,
	/* Array index: 0x1572 */ 0x5E33,0x4422,
	/* Array index: 0x1572 */ 0x5E3D,0x4B39,
	/* Array index: 0x1572 */ 0x5E40,0x566C,
	/* Array index: 0x1572 */ 0x5E4C,0x4B5A,
	/* Array index: 0x1572 */ 0x5E4E,0x566D,
	/* Array index: 0x1572 */ 0x5E54,0x566F,
	/* Array index: 0x1572 */ 0x5E55,0x4B6B,
	/* Array index: 0x1572 */ 0x5E57,0x566E,
	/* Array index: 0x1572 */ 0x5E87,0x485F,
	/* Array index: 0x1572 */ 0x5E8A,0x3E32,
	/* Array index: 0x1572 */ 0x5E8F,0x3D78,
	/* Array index: 0x1572 */ 0x5E9A,0x392E,
	/* Array index: 0x1572 */ 0x5E9C,0x495C,
	/* Array index: 0x1572 */ 0x5EA0,0x5679,
	/* Array index: 0x1572 */ 0x5EA6,0x4559,
	/* Array index: 0x1572 */ 0x5EA7,0x3A42,
	/* Array index: 0x1572 */ 0x5EAB,0x384B,
	/* Array index: 0x1572 */ 0x5EAD,0x446D,
	/* Array index: 0x1572 */ 0x5ECF,0x567E,
	/* Array index: 0x1572 */ 0x5ED0,0x567D,
	/* Array index: 0x1572 */ 0x5ED3,0x3347,
	/* Array index: 0x1572 */ 0x5ED6,0x5721,
	/* Array index: 0x1572 */ 0x5EDA,0x5724,
	/* Array index: 0x1572 */ 0x5EDB,0x5725,
	/* Array index: 0x1572 */ 0x5EE8,0x5728,
	/* Array index: 0x1572 */ 0x5EE9,0x5729,
	/* Array index: 0x1572 */ 0x5EEC,0x572A,
	/* Array index: 0x1572 */ 0x5F1B,0x4350,
	/* Array index: 0x1572 */ 0x5F1F,0x446F,
	/* Array index: 0x1572 */ 0x5F2D,0x5739,
	/* Array index: 0x1572 */ 0x5F2F,0x573F,
	/* Array index: 0x1572 */ 0x5F31,0x3C65,
	/* Array index: 0x1572 */ 0x5F35,0x4425,
	/* Array index: 0x1572 */ 0x5F37,0x362F,
	/* Array index: 0x1572 */ 0x5F38,0x573A,
	/* Array index: 0x1572 */ 0x5F3C,0x492B,
	/* Array index: 0x1572 */ 0x5F3E,0x4346,
	/* Array index: 0x1572 */ 0x5F41,0x573B,
	/* Array index: 0x1572 */ 0x5F48,0x573C,
	/* Array index: 0x1572 */ 0x5F4A,0x3630,
	/* Array index: 0x1572 */ 0x5F4C,0x573D,
	/* Array index: 0x1572 */ 0x5F4E,0x573E,
	/* Array index: 0x1572 */ 0x5F51,0x5740,
	/* Array index: 0x1572 */ 0x5F53,0x4576,
	/* Array index: 0x1572 */ 0x5F56,0x5741,
	/* Array index: 0x1572 */ 0x5F57,0x5742,
	/* Array index: 0x1572 */ 0x5F59,0x5743,
	/* Array index: 0x1572 */ 0x5F5C,0x5734,
	/* Array index: 0x1572 */ 0x5F5D,0x5733,
	/* Array index: 0x1572 */ 0x5F61,0x5744,
	/* Array index: 0x1572 */ 0x5F62,0x3741,
	/* Array index: 0x1572 */ 0x5F66,0x4927,
	/* Array index: 0x1572 */ 0x5F77,0x5747,
	/* Array index: 0x1572 */ 0x5F79,0x4C72,
	/* Array index: 0x1572 */ 0x5F7C,0x4860,
	/* Array index: 0x1572 */ 0x5F9E,0x574F,
	/* Array index: 0x1572 */ 0x5FA0,0x5752,
	/* Array index: 0x1572 */ 0x5FA1,0x3866,
	/* Array index: 0x1572 */ 0x5FB3,0x4641,
	/* Array index: 0x1572 */ 0x5FB4,0x4427,
	/* Array index: 0x1572 */ 0x5FB9,0x4530,
	/* Array index: 0x1572 */ 0x5FBC,0x5755,
	/* Array index: 0x1572 */ 0x5FBD,0x352B,
	/* Array index: 0x1572 */ 0x5FC3,0x3F34,
	/* Array index: 0x1572 */ 0x5FC5,0x492C,
	/* Array index: 0x1572 */ 0x5FCC,0x3477,
	/* Array index: 0x1572 */ 0x5FCD,0x4726,
	/* Array index: 0x1572 */ 0x5FE0,0x4369,
	/* Array index: 0x1572 */ 0x5FE4,0x5758,
	/* Array index: 0x1572 */ 0x5FEB,0x3277,
	/* Array index: 0x1572 */ 0x5FF0,0x582D,
	/* Array index: 0x1572 */ 0x5FF1,0x575A,
	/* Array index: 0x1572 */ 0x5FF5,0x4730,
	/* Array index: 0x1572 */ 0x5FF8,0x5759,
	/* Array index: 0x1572 */ 0x5FFB,0x5757,
	/* Array index: 0x1572 */ 0x5FFD,0x397A,
	/* Array index: 0x1572 */ 0x5FFF,0x575D,
	/* Array index: 0x1572 */ 0x602F,0x3631,
	/* Array index: 0x1572 */ 0x6031,0x5764,
	/* Array index: 0x1572 */ 0x603A,0x576A,
	/* Array index: 0x1572 */ 0x6046,0x5771,
	/* Array index: 0x1572 */ 0x604A,0x5770,
	/* Array index: 0x1572 */ 0x604B,0x4E78,
	/* Array index: 0x1572 */ 0x604D,0x5772,
	/* Array index: 0x1572 */ 0x6050,0x3632,
	/* Array index: 0x1572 */ 0x6052,0x3931,
	/* Array index: 0x1572 */ 0x6055,0x3D7A,
	/* Array index: 0x1572 */ 0x6059,0x5779,
	/* Array index: 0x1572 */ 0x605A,0x576B,
	/* Array index: 0x1572 */ 0x6075,0x3743,
	/* Array index: 0x1572 */ 0x6077,0x576E,
	/* Array index: 0x1572 */ 0x6081,0x577A,
	/* Array index: 0x1572 */ 0x6083,0x577D,
	/* Array index: 0x1572 */ 0x6084,0x5821,
	/* Array index: 0x1572 */ 0x6092,0x5825,
	/* Array index: 0x1572 */ 0x6094,0x3279,
	/* Array index: 0x1572 */ 0x6096,0x5823,
	/* Array index: 0x1572 */ 0x6097,0x5824,
	/* Array index: 0x1572 */ 0x609A,0x577E,
	/* Array index: 0x1572 */ 0x609B,0x5822,
	/* Array index: 0x1572 */ 0x609F,0x3867,
	/* Array index: 0x1572 */ 0x60A0,0x4D2A,
	/* Array index: 0x1572 */ 0x60A3,0x3435,
	/* Array index: 0x1572 */ 0x60BC,0x4569,
	/* Array index: 0x1572 */ 0x60BD,0x582E,
	/* Array index: 0x1572 */ 0x60D1,0x4F47,
	/* Array index: 0x1572 */ 0x60D3,0x582B,
	/* Array index: 0x1572 */ 0x60D8,0x5831,
	/* Array index: 0x1572 */ 0x60DA,0x397B,
	/* Array index: 0x1572 */ 0x60DC,0x404B,
	/* Array index: 0x1572 */ 0x60E7,0x577C,
	/* Array index: 0x1572 */ 0x60E8,0x3B34,
	/* Array index: 0x1572 */ 0x6100,0x5837,
	/* Array index: 0x1572 */ 0x6101,0x3D25,
	/* Array index: 0x1572 */ 0x6103,0x583A,
	/* Array index: 0x1572 */ 0x6106,0x5834,
	/* Array index: 0x1572 */ 0x6108,0x4C7C,
	/* Array index: 0x1572 */ 0x6109,0x4C7B,
	/* Array index: 0x1572 */ 0x6115,0x5833,
	/* Array index: 0x1572 */ 0x611A,0x3672,
	/* Array index: 0x1572 */ 0x611B,0x3026,
	/* Array index: 0x1572 */ 0x611F,0x3436,
	/* Array index: 0x1572 */ 0x6121,0x583B,
	/* Array index: 0x1572 */ 0x6127,0x5843,
	/* Array index: 0x1572 */ 0x6128,0x5842,
	/* Array index: 0x1572 */ 0x612C,0x5847,
	/* Array index: 0x1572 */ 0x6134,0x5848,
	/* Array index: 0x1572 */ 0x6142,0x584A,
	/* Array index: 0x1572 */ 0x6144,0x584B,
	/* Array index: 0x1572 */ 0x6153,0x5858,
	/* Array index: 0x1572 */ 0x6155,0x4A69,
	/* Array index: 0x1572 */ 0x615D,0x5857,
	/* Array index: 0x1572 */ 0x615F,0x5856,
	/* Array index: 0x1572 */ 0x6162,0x4B7D,
	/* Array index: 0x1572 */ 0x6163,0x3437,
	/* Array index: 0x1572 */ 0x6165,0x5854,
	/* Array index: 0x1572 */ 0x6167,0x3745,
	/* Array index: 0x1572 */ 0x6168,0x3334,
	/* Array index: 0x1572 */ 0x616B,0x5851,
	/* Array index: 0x1572 */ 0x617E,0x4D5D,
	/* Array index: 0x1572 */ 0x6182,0x4D2B,
	/* Array index: 0x1572 */ 0x6187,0x585C,
	/* Array index: 0x1572 */ 0x618A,0x5860,
	/* Array index: 0x1572 */ 0x618E,0x417E,
	/* Array index: 0x1572 */ 0x6190,0x4E79,
	/* Array index: 0x1572 */ 0x6191,0x5861,
	/* Array index: 0x1572 */ 0x6194,0x585E,
	/* Array index: 0x1572 */ 0x6196,0x585B,
	/* Array index: 0x1572 */ 0x6199,0x585A,
	/* Array index: 0x1572 */ 0x619A,0x585F,
	/* Array index: 0x1572 */ 0x61A4,0x4A30,
	/* Array index: 0x1572 */ 0x61A7,0x4634,
	/* Array index: 0x1572 */ 0x61A9,0x3746,
	/* Array index: 0x1572 */ 0x61AB,0x5862,
	/* Array index: 0x1572 */ 0x61AC,0x585D,
	/* Array index: 0x1572 */ 0x61AE,0x5863,
	/* Array index: 0x1572 */ 0x61B2,0x377B,
	/* Array index: 0x1572 */ 0x61B6,0x3231,
	/* Array index: 0x1572 */ 0x61BA,0x586B,
	/* Array index: 0x1572 */ 0x61BE,0x3438,
	/* Array index: 0x1572 */ 0x61C3,0x5869,
	/* Array index: 0x1572 */ 0x61D0,0x327B,
	/* Array index: 0x1572 */ 0x61E3,0x5870,
	/* Array index: 0x1572 */ 0x61E6,0x586F,
	/* Array index: 0x1572 */ 0x61F2,0x4428,
	/* Array index: 0x1572 */ 0x6226,0x406F,
	/* Array index: 0x1572 */ 0x622A,0x5923,
	/* Array index: 0x1572 */ 0x6238,0x384D,
	/* Array index: 0x1572 */ 0x623B,0x4C61,
	/* Array index: 0x1572 */ 0x6253,0x4247,
	/* Array index: 0x1572 */ 0x6255,0x4A27,
	/* Array index: 0x1572 */ 0x6258,0x4271,
	/* Array index: 0x1572 */ 0x625B,0x592C,
	/* Array index: 0x1572 */ 0x625E,0x592A,
	/* Array index: 0x1572 */ 0x6260,0x592D,
	/* Array index: 0x1572 */ 0x6263,0x592B,
	/* Array index: 0x1572 */ 0x6268,0x592E,
	/* Array index: 0x1572 */ 0x626E,0x4A31,
	/* Array index: 0x1572 */ 0x6271,0x3037,
	/* Array index: 0x1572 */ 0x6276,0x495E,
	/* Array index: 0x1572 */ 0x6279,0x4863,
	/* Array index: 0x1572 */ 0x6289,0x5931,
	/* Array index: 0x1572 */ 0x628A,0x4744,
	/* Array index: 0x1572 */ 0x62AB,0x4864,
	/* Array index: 0x1572 */ 0x62AC,0x5A2D,
	/* Array index: 0x1572 */ 0x62B1,0x4A7A,
	/* Array index: 0x1572 */ 0x62B5,0x4471,
	/* Array index: 0x1572 */ 0x62C2,0x5944,
	/* Array index: 0x1572 */ 0x630C,0x5948,
	/* Array index: 0x1572 */ 0x6311,0x4429,
	/* Array index: 0x1572 */ 0x6319,0x3573,
	/* Array index: 0x1572 */ 0x631F,0x3634,
	/* Array index: 0x1572 */ 0x6327,0x594B,
	/* Array index: 0x1572 */ 0x6328,0x3027,
	/* Array index: 0x1572 */ 0x632B,0x3A43,
	/* Array index: 0x1572 */ 0x632F,0x3F36,
	/* Array index: 0x1572 */ 0x633A,0x4472,
	/* Array index: 0x1572 */ 0x6349,0x422A,
	/* Array index: 0x1572 */ 0x6355,0x4A61,
	/* Array index: 0x1572 */ 0x6357,0x443D,
	/* Array index: 0x1572 */ 0x635C,0x415C,
	/* Array index: 0x1572 */ 0x636E,0x3F78,
	/* Array index: 0x1572 */ 0x6372,0x377E,
	/* Array index: 0x1572 */ 0x6376,0x5959,
	/* Array index: 0x1572 */ 0x6377,0x3E39,
	/* Array index: 0x1572 */ 0x637A,0x4668,
	/* Array index: 0x1572 */ 0x637B,0x4731,
	/* Array index: 0x1572 */ 0x6380,0x5957,
	/* Array index: 0x1572 */ 0x6383,0x415D,
	/* Array index: 0x1572 */ 0x6388,0x3C78,
	/* Array index: 0x1572 */ 0x6389,0x595C,
	/* Array index: 0x1572 */ 0x638C,0x3E38,
	/* Array index: 0x1572 */ 0x638E,0x5956,
	/* Array index: 0x1572 */ 0x638F,0x595B,
	/* Array index: 0x1572 */ 0x6392,0x4753,
	/* Array index: 0x1572 */ 0x6396,0x5955,
	/* Array index: 0x1572 */ 0x6398,0x3721,
	/* Array index: 0x1572 */ 0x639B,0x335D,
	/* Array index: 0x1572 */ 0x63B2,0x3747,
	/* Array index: 0x1572 */ 0x63B4,0x444F,
	/* Array index: 0x1572 */ 0x63B5,0x595E,
	/* Array index: 0x1572 */ 0x63BB,0x415F,
	/* Array index: 0x1572 */ 0x63BE,0x5961,
	/* Array index: 0x1572 */ 0x63C0,0x5963,
	/* Array index: 0x1572 */ 0x63C3,0x4237,
	/* Array index: 0x1572 */ 0x63C4,0x5969,
	/* Array index: 0x1572 */ 0x63C6,0x5964,
	/* Array index: 0x1572 */ 0x63C9,0x5966,
	/* Array index: 0x1572 */ 0x63CF,0x4941,
	/* Array index: 0x1572 */ 0x63D0,0x4473,
	/* Array index: 0x1572 */ 0x63D2,0x5967,
	/* Array index: 0x1572 */ 0x63D6,0x4D2C,
	/* Array index: 0x1572 */ 0x63DA,0x4D48,
	/* Array index: 0x1572 */ 0x63DB,0x3439,
	/* Array index: 0x1572 */ 0x63E1,0x302E,
	/* Array index: 0x1572 */ 0x63E3,0x5965,
	/* Array index: 0x1572 */ 0x63E9,0x5962,
	/* Array index: 0x1572 */ 0x63EE,0x3478,
	/* Array index: 0x1572 */ 0x63F4,0x3167,
	/* Array index: 0x1572 */ 0x63F6,0x5968,
	/* Array index: 0x1572 */ 0x63FA,0x4D49,
	/* Array index: 0x1572 */ 0x6406,0x596C,
	/* Array index: 0x1572 */ 0x640D,0x423B,
	/* Array index: 0x1572 */ 0x640F,0x5973,
	/* Array index: 0x1572 */ 0x6413,0x596D,
	/* Array index: 0x1572 */ 0x6416,0x596A,
	/* Array index: 0x1572 */ 0x6417,0x5971,
	/* Array index: 0x1572 */ 0x641C,0x5953,
	/* Array index: 0x1572 */ 0x6426,0x596E,
	/* Array index: 0x1572 */ 0x6428,0x5972,
	/* Array index: 0x1572 */ 0x642C,0x4842,
	/* Array index: 0x1572 */ 0x642D,0x456B,
	/* Array index: 0x1572 */ 0x6434,0x596B,
	/* Array index: 0x1572 */ 0x6436,0x596F,
	/* Array index: 0x1572 */ 0x643A,0x3748,
	/* Array index: 0x1572 */ 0x643E,0x3A71,
	/* Array index: 0x1572 */ 0x6442,0x405D,
	/* Array index: 0x1572 */ 0x644E,0x5977,
	/* Array index: 0x1572 */ 0x6458,0x4526,
	/* Array index: 0x1572 */ 0x6467,0x5974,
	/* Array index: 0x1572 */ 0x6469,0x4B60,
	/* Array index: 0x1572 */ 0x646F,0x5975,
	/* Array index: 0x1572 */ 0x6476,0x5976,
	/* Array index: 0x1572 */ 0x6478,0x4C4E,
	/* Array index: 0x1572 */ 0x647A,0x4022,
	/* Array index: 0x1572 */ 0x6483,0x3762,
	/* Array index: 0x1572 */ 0x6488,0x597D,
	/* Array index: 0x1572 */ 0x6492,0x3B35,
	/* Array index: 0x1572 */ 0x6493,0x597A,
	/* Array index: 0x1572 */ 0x6495,0x5979,
	/* Array index: 0x1572 */ 0x649A,0x4732,
	/* Array index: 0x1572 */ 0x649E,0x4635,
	/* Array index: 0x1572 */ 0x64A4,0x4531,
	/* Array index: 0x1572 */ 0x64A5,0x597B,
	/* Array index: 0x1572 */ 0x64A9,0x597C,
	/* Array index: 0x1572 */ 0x64AB,0x496F,
	/* Array index: 0x1572 */ 0x64AD,0x4745,
	/* Array index: 0x1572 */ 0x64AE,0x3B23,
	/* Array index: 0x1572 */ 0x64B0,0x4071,
	/* Array index: 0x1572 */ 0x64B2,0x4B50,
	/* Array index: 0x1572 */ 0x64B9,0x3349,
	/* Array index: 0x1572 */ 0x64BB,0x5A25,
	/* Array index: 0x1572 */ 0x64BC,0x597E,
	/* Array index: 0x1572 */ 0x64C1,0x4D4A,
	/* Array index: 0x1572 */ 0x64C2,0x5A27,
	/* Array index: 0x1572 */ 0x64C5,0x5A23,
	/* Array index: 0x1572 */ 0x64C7,0x5A24,
	/* Array index: 0x1572 */ 0x64CD,0x4160,
	/* Array index: 0x1572 */ 0x64D2,0x5A22,
	/* Array index: 0x1572 */ 0x64D4,0x593F,
	/* Array index: 0x1572 */ 0x64D8,0x5A26,
	/* Array index: 0x1572 */ 0x64DA,0x5A21,
	/* Array index: 0x1572 */ 0x64EC,0x353C,
	/* Array index: 0x1572 */ 0x64EF,0x5A2F,
	/* Array index: 0x1572 */ 0x64F1,0x5A28,
	/* Array index: 0x1572 */ 0x64F2,0x5A33,
	/* Array index: 0x1572 */ 0x64F4,0x5A32,
	/* Array index: 0x1572 */ 0x64F6,0x5A31,
	/* Array index: 0x1572 */ 0x64FA,0x5A34,
	/* Array index: 0x1572 */ 0x64FD,0x5A36,
	/* Array index: 0x1572 */ 0x64FE,0x3E71,
	/* Array index: 0x1572 */ 0x6500,0x5A35,
	/* Array index: 0x1572 */ 0x6505,0x5A39,
	/* Array index: 0x1572 */ 0x6518,0x5A37,
	/* Array index: 0x1572 */ 0x651C,0x5A38,
	/* Array index: 0x1572 */ 0x651D,0x5970,
	/* Array index: 0x1572 */ 0x6523,0x5A3B,
	/* Array index: 0x1572 */ 0x6524,0x5A3A,
	/* Array index: 0x1572 */ 0x652F,0x3B59,
	/* Array index: 0x1572 */ 0x6545,0x384E,
	/* Array index: 0x1572 */ 0x6548,0x5A43,
	/* Array index: 0x1572 */ 0x654D,0x5A46,
	/* Array index: 0x1572 */ 0x654F,0x4952,
	/* Array index: 0x1572 */ 0x6551,0x355F,
	/* Array index: 0x1572 */ 0x655D,0x5A49,
	/* Array index: 0x1572 */ 0x655E,0x5A48,
	/* Array index: 0x1572 */ 0x6562,0x343A,
	/* Array index: 0x1572 */ 0x6563,0x3B36,
	/* Array index: 0x1572 */ 0x6566,0x4658,
	/* Array index: 0x1572 */ 0x656C,0x3749,
	/* Array index: 0x1572 */ 0x6570,0x3F74,
	/* Array index: 0x1572 */ 0x6572,0x5A4A,
	/* Array index: 0x1572 */ 0x6582,0x5A4C,
	/* Array index: 0x1572 */ 0x6583,0x5A4D,
	/* Array index: 0x1572 */ 0x658C,0x494C,
	/* Array index: 0x1572 */ 0x658E,0x3A58,
	/* Array index: 0x1572 */ 0x6590,0x4865,
	/* Array index: 0x1572 */ 0x6591,0x4843,
	/* Array index: 0x1572 */ 0x6597,0x454D,
	/* Array index: 0x1572 */ 0x6599,0x4E41,
	/* Array index: 0x1572 */ 0x659B,0x5A4F,
	/* Array index: 0x1572 */ 0x659C,0x3C50,
	/* Array index: 0x1572 */ 0x659F,0x5A50,
	/* Array index: 0x1572 */ 0x65A1,0x3036,
	/* Array index: 0x1572 */ 0x65A4,0x3654,
	/* Array index: 0x1572 */ 0x65A5,0x404D,
	/* Array index: 0x1572 */ 0x65A7,0x4960,
	/* Array index: 0x1572 */ 0x65B7,0x5A52,
	/* Array index: 0x1572 */ 0x65B9,0x4A7D,
	/* Array index: 0x1572 */ 0x65BC,0x3177,
	/* Array index: 0x1572 */ 0x65BD,0x3B5C,
	/* Array index: 0x1572 */ 0x65CB,0x407B,
	/* Array index: 0x1572 */ 0x65CC,0x5A57,
	/* Array index: 0x1572 */ 0x65CF,0x4232,
	/* Array index: 0x1572 */ 0x65D2,0x5A58,
	/* Array index: 0x1572 */ 0x65D7,0x347A,
	/* Array index: 0x1572 */ 0x65D9,0x5A5A,
	/* Array index: 0x1572 */ 0x65DB,0x5A59,
	/* Array index: 0x1572 */ 0x65F1,0x5A5D,
	/* Array index: 0x1572 */ 0x65FA,0x3222,
	/* Array index: 0x1572 */ 0x65FB,0x5A61,
	/* Array index: 0x1572 */ 0x6602,0x3937,
	/* Array index: 0x1572 */ 0x6603,0x5A60,
	/* Array index: 0x1572 */ 0x6606,0x3A2B,
	/* Array index: 0x1572 */ 0x6607,0x3E3A,
	/* Array index: 0x1572 */ 0x660A,0x5A5F,
	/* Array index: 0x1572 */ 0x660C,0x3E3B,
	/* Array index: 0x1572 */ 0x660E,0x4C40,
	/* Array index: 0x1572 */ 0x660F,0x3A2A,
	/* Array index: 0x1572 */ 0x6613,0x3057,
	/* Array index: 0x1572 */ 0x6614,0x404E,
	/* Array index: 0x1572 */ 0x661C,0x5A66,
	/* Array index: 0x1572 */ 0x661F,0x4031,
	/* Array index: 0x1572 */ 0x6620,0x3147,
	/* Array index: 0x1572 */ 0x6625,0x3D55,
	/* Array index: 0x1572 */ 0x6627,0x4B66,
	/* Array index: 0x1572 */ 0x6628,0x3A72,
	/* Array index: 0x1572 */ 0x662D,0x3E3C,
	/* Array index: 0x1572 */ 0x662F,0x4027,
	/* Array index: 0x1572 */ 0x663C,0x436B,
	/* Array index: 0x1572 */ 0x6649,0x5A69,
	/* Array index: 0x1572 */ 0x664B,0x3F38,
	/* Array index: 0x1572 */ 0x664F,0x5A67,
	/* Array index: 0x1572 */ 0x6652,0x3B2F,
	/* Array index: 0x1572 */ 0x6662,0x5A71,
	/* Array index: 0x1572 */ 0x6674,0x4032,
	/* Array index: 0x1572 */ 0x6676,0x3E3D,
	/* Array index: 0x1572 */ 0x667A,0x4352,
	/* Array index: 0x1572 */ 0x6681,0x3647,
	/* Array index: 0x1572 */ 0x668E,0x5A75,
	/* Array index: 0x1572 */ 0x6691,0x3D6B,
	/* Array index: 0x1572 */ 0x669D,0x5A79,
	/* Array index: 0x1572 */ 0x66A2,0x442A,
	/* Array index: 0x1572 */ 0x66A6,0x4E71,
	/* Array index: 0x1572 */ 0x66AB,0x3B43,
	/* Array index: 0x1572 */ 0x66AE,0x4A6B,
	/* Array index: 0x1572 */ 0x66B4,0x4B3D,
	/* Array index: 0x1572 */ 0x66B8,0x5B22,
	/* Array index: 0x1572 */ 0x66B9,0x5A7B,
	/* Array index: 0x1572 */ 0x66BC,0x5A7E,
	/* Array index: 0x1572 */ 0x66BE,0x5A7D,
	/* Array index: 0x1572 */ 0x66C1,0x5A7A,
	/* Array index: 0x1572 */ 0x66C4,0x5B21,
	/* Array index: 0x1572 */ 0x66C7,0x465E,
	/* Array index: 0x1572 */ 0x66C9,0x5A7C,
	/* Array index: 0x1572 */ 0x66D6,0x5B23,
	/* Array index: 0x1572 */ 0x66E0,0x5B25,
	/* Array index: 0x1572 */ 0x66E6,0x5B27,
	/* Array index: 0x1572 */ 0x66E9,0x5B28,
	/* Array index: 0x1572 */ 0x6703,0x5072,
	/* Array index: 0x1572 */ 0x6708,0x376E,
	/* Array index: 0x1572 */ 0x6709,0x4D2D,
	/* Array index: 0x1572 */ 0x670B,0x4A7E,
	/* Array index: 0x1572 */ 0x670D,0x497E,
	/* Array index: 0x1572 */ 0x670F,0x5B2C,
	/* Array index: 0x1572 */ 0x6731,0x3C6B,
	/* Array index: 0x1572 */ 0x673D,0x3560,
	/* Array index: 0x1572 */ 0x673F,0x5B33,
	/* Array index: 0x1572 */ 0x6741,0x5B35,
	/* Array index: 0x1572 */ 0x6746,0x5B38,
	/* Array index: 0x1572 */ 0x6749,0x3F79,
	/* Array index: 0x1572 */ 0x6756,0x3E73,
	/* Array index: 0x1572 */ 0x6759,0x5B3B,
	/* Array index: 0x1572 */ 0x676A,0x5B42,
	/* Array index: 0x1572 */ 0x677C,0x5B41,
	/* Array index: 0x1572 */ 0x677E,0x3E3E,
	/* Array index: 0x1572 */ 0x677F,0x4844,
	/* Array index: 0x1572 */ 0x6785,0x5B47,
	/* Array index: 0x1572 */ 0x6787,0x487A,
	/* Array index: 0x1572 */ 0x6789,0x5B3E,
	/* Array index: 0x1572 */ 0x678B,0x5B44,
	/* Array index: 0x1572 */ 0x678C,0x5B43,
	/* Array index: 0x1572 */ 0x6790,0x404F,
	/* Array index: 0x1572 */ 0x6795,0x4B6D,
	/* Array index: 0x1572 */ 0x6797,0x4E53,
	/* Array index: 0x1572 */ 0x679A,0x4B67,
	/* Array index: 0x1572 */ 0x67A6,0x5B45,
	/* Array index: 0x1572 */ 0x67A9,0x5B40,
	/* Array index: 0x1572 */ 0x67AF,0x384F,
	/* Array index: 0x1572 */ 0x67C1,0x4248,
	/* Array index: 0x1572 */ 0x67C4,0x4A41,
	/* Array index: 0x1572 */ 0x67C6,0x5B56,
	/* Array index: 0x1572 */ 0x67CA,0x4922,
	/* Array index: 0x1572 */ 0x67D8,0x4453,
	/* Array index: 0x1572 */ 0x67DA,0x4D2E,
	/* Array index: 0x1572 */ 0x67DD,0x5B51,
	/* Array index: 0x1572 */ 0x67DE,0x5B50,
	/* Array index: 0x1572 */ 0x67E2,0x5B52,
	/* Array index: 0x1572 */ 0x67E4,0x5B4F,
	/* Array index: 0x1572 */ 0x67E7,0x5B57,
	/* Array index: 0x1572 */ 0x67E9,0x5B4D,
	/* Array index: 0x1572 */ 0x67EC,0x5B4B,
	/* Array index: 0x1572 */ 0x67EE,0x5B53,
	/* Array index: 0x1572 */ 0x67EF,0x5B49,
	/* Array index: 0x1572 */ 0x67FB,0x3A3A,
	/* Array index: 0x1572 */ 0x6813,0x4072,
	/* Array index: 0x1572 */ 0x6816,0x4034,
	/* Array index: 0x1572 */ 0x6817,0x372A,
	/* Array index: 0x1572 */ 0x681E,0x5B59,
	/* Array index: 0x1572 */ 0x6821,0x393B,
	/* Array index: 0x1572 */ 0x6822,0x337C,
	/* Array index: 0x1572 */ 0x6832,0x5B5E,
	/* Array index: 0x1572 */ 0x6834,0x4073,
	/* Array index: 0x1572 */ 0x6838,0x334B,
	/* Array index: 0x1572 */ 0x6839,0x3A2C,
	/* Array index: 0x1572 */ 0x6846,0x5B5A,
	/* Array index: 0x1572 */ 0x6848,0x3046,
	/* Array index: 0x1572 */ 0x6859,0x5B62,
	/* Array index: 0x1572 */ 0x685C,0x3A79,
	/* Array index: 0x1572 */ 0x685D,0x4B71,
	/* Array index: 0x1572 */ 0x685F,0x3B37,
	/* Array index: 0x1572 */ 0x6863,0x5B63,
	/* Array index: 0x1572 */ 0x6867,0x4930,
	/* Array index: 0x1572 */ 0x6874,0x5B6F,
	/* Array index: 0x1572 */ 0x6876,0x3233,
	/* Array index: 0x1572 */ 0x6877,0x5B64,
	/* Array index: 0x1572 */ 0x687E,0x5B75,
	/* Array index: 0x1572 */ 0x687F,0x5B65,
	/* Array index: 0x1572 */ 0x6881,0x4E42,
	/* Array index: 0x1572 */ 0x6883,0x5B6C,
	/* Array index: 0x1572 */ 0x6885,0x475F,
	/* Array index: 0x1572 */ 0x688D,0x5B74,
	/* Array index: 0x1572 */ 0x688F,0x5B67,
	/* Array index: 0x1572 */ 0x6893,0x3034,
	/* Array index: 0x1572 */ 0x6894,0x5B69,
	/* Array index: 0x1572 */ 0x6897,0x393C,
	/* Array index: 0x1572 */ 0x689B,0x5B6B,
	/* Array index: 0x1572 */ 0x689D,0x5B6A,
	/* Array index: 0x1572 */ 0x689F,0x5B66,
	/* Array index: 0x1572 */ 0x68A0,0x5B71,
	/* Array index: 0x1572 */ 0x68A2,0x3E3F,
	/* Array index: 0x1572 */ 0x68C4,0x347E,
	/* Array index: 0x1572 */ 0x68C6,0x5C32,
	/* Array index: 0x1572 */ 0x68D2,0x4B40,
	/* Array index: 0x1572 */ 0x68E7,0x5C22,
	/* Array index: 0x1572 */ 0x68EE,0x3F39,
	/* Array index: 0x1572 */ 0x68EF,0x5C2C,
	/* Array index: 0x1572 */ 0x68F2,0x4033,
	/* Array index: 0x1572 */ 0x68F9,0x5C2A,
	/* Array index: 0x1572 */ 0x68FA,0x343D,
	/* Array index: 0x1572 */ 0x6900,0x4F50,
	/* Array index: 0x1572 */ 0x6901,0x5B76,
	/* Array index: 0x1572 */ 0x6904,0x5C26,
	/* Array index: 0x1572 */ 0x6905,0x3058,
	/* Array index: 0x1572 */ 0x6908,0x5B78,
	/* Array index: 0x1572 */ 0x6912,0x5C25,
	/* Array index: 0x1572 */ 0x6930,0x5C3F,
	/* Array index: 0x1572 */ 0x6934,0x464E,
	/* Array index: 0x1572 */ 0x6936,0x5C24,
	/* Array index: 0x1572 */ 0x6939,0x5C3B,
	/* Array index: 0x1572 */ 0x693D,0x5C3D,
	/* Array index: 0x1572 */ 0x693F,0x4458,
	/* Array index: 0x1572 */ 0x694A,0x4D4C,
	/* Array index: 0x1572 */ 0x698A,0x3A67,
	/* Array index: 0x1572 */ 0x698E,0x315D,
	/* Array index: 0x1572 */ 0x6991,0x5C54,
	/* Array index: 0x1572 */ 0x6994,0x4F31,
	/* Array index: 0x1572 */ 0x6995,0x5C57,
	/* Array index: 0x1572 */ 0x699B,0x3F3A,
	/* Array index: 0x1572 */ 0x699C,0x5C56,
	/* Array index: 0x1572 */ 0x69A0,0x5C55,
	/* Array index: 0x1572 */ 0x69A7,0x5C52,
	/* Array index: 0x1572 */ 0x69AE,0x5C46,
	/* Array index: 0x1572 */ 0x69B1,0x5C63,
	/* Array index: 0x1572 */ 0x69B2,0x5C45,
	/* Array index: 0x1572 */ 0x69B4,0x5C58,
	/* Array index: 0x1572 */ 0x69BB,0x5C50,
	/* Array index: 0x1572 */ 0x69BE,0x5C4B,
	/* Array index: 0x1572 */ 0x69BF,0x5C48,
	/* Array index: 0x1572 */ 0x69C1,0x5C49,
	/* Array index: 0x1572 */ 0x69C3,0x5C51,
	/* Array index: 0x1572 */ 0x69C7,0x7422,
	/* Array index: 0x1572 */ 0x69D3,0x5C4A,
	/* Array index: 0x1572 */ 0x69D8,0x4D4D,
	/* Array index: 0x1572 */ 0x69D9,0x4B6A,
	/* Array index: 0x1572 */ 0x69DD,0x5C4F,
	/* Array index: 0x1572 */ 0x69DE,0x5C59,
	/* Array index: 0x1572 */ 0x69E7,0x5C61,
	/* Array index: 0x1572 */ 0x69E8,0x5C5A,
	/* Array index: 0x1572 */ 0x69EB,0x5C67,
	/* Array index: 0x1572 */ 0x69ED,0x5C65,
	/* Array index: 0x1572 */ 0x69F2,0x5C60,
	/* Array index: 0x1572 */ 0x69F9,0x5C5F,
	/* Array index: 0x1572 */ 0x69FB,0x4450,
	/* Array index: 0x1572 */ 0x69FD,0x4165,
	/* Array index: 0x1572 */ 0x69FF,0x5C5D,
	/* Array index: 0x1572 */ 0x6A02,0x5C5B,
	/* Array index: 0x1572 */ 0x6A05,0x5C62,
	/* Array index: 0x1572 */ 0x6A17,0x4374,
	/* Array index: 0x1572 */ 0x6A19,0x4938,
	/* Array index: 0x1572 */ 0x6A1B,0x5C5C,
	/* Array index: 0x1572 */ 0x6A2E,0x5C53,
	/* Array index: 0x1572 */ 0x6A3D,0x432E,
	/* Array index: 0x1572 */ 0x6A44,0x5C6D,
	/* Array index: 0x1572 */ 0x6A47,0x5C72,
	/* Array index: 0x1572 */ 0x6A48,0x5C76,
	/* Array index: 0x1572 */ 0x6A4B,0x3636,
	/* Array index: 0x1572 */ 0x6A58,0x354C,
	/* Array index: 0x1572 */ 0x6A59,0x5C74,
	/* Array index: 0x1572 */ 0x6A5F,0x3521,
	/* Array index: 0x1572 */ 0x6A61,0x464B,
	/* Array index: 0x1572 */ 0x6A62,0x5C73,
	/* Array index: 0x1572 */ 0x6A66,0x5C75,
	/* Array index: 0x1572 */ 0x6A72,0x5C6F,
	/* Array index: 0x1572 */ 0x6A78,0x5C71,
	/* Array index: 0x1572 */ 0x6A7F,0x3360,
	/* Array index: 0x1572 */ 0x6A80,0x4349,
	/* Array index: 0x1572 */ 0x6A84,0x5C7C,
	/* Array index: 0x1572 */ 0x6A8D,0x5C7A,
	/* Array index: 0x1572 */ 0x6A8E,0x3869,
	/* Array index: 0x1572 */ 0x6A90,0x5C79,
	/* Array index: 0x1572 */ 0x6A97,0x5D21,
	/* Array index: 0x1572 */ 0x6A9C,0x5B58,
	/* Array index: 0x1572 */ 0x6AA0,0x5C7B,
	/* Array index: 0x1572 */ 0x6AA2,0x5C7D,
	/* Array index: 0x1572 */ 0x6AA3,0x5C7E,
	/* Array index: 0x1572 */ 0x6AAA,0x5D2C,
	/* Array index: 0x1572 */ 0x6AAC,0x5D28,
	/* Array index: 0x1572 */ 0x6AAE,0x5B6D,
	/* Array index: 0x1572 */ 0x6AB3,0x5D27,
	/* Array index: 0x1572 */ 0x6AB8,0x5D26,
	/* Array index: 0x1572 */ 0x6ABB,0x5D23,
	/* Array index: 0x1572 */ 0x6AD1,0x5D2A,
	/* Array index: 0x1572 */ 0x6AD3,0x4F26,
	/* Array index: 0x1572 */ 0x6ADA,0x5D2D,
	/* Array index: 0x1572 */ 0x6ADB,0x367B,
	/* Array index: 0x1572 */ 0x6ADE,0x5D29,
	/* Array index: 0x1572 */ 0x6ADF,0x5D2B,
	/* Array index: 0x1572 */ 0x6AE8,0x4827,
	/* Array index: 0x1572 */ 0x6AEA,0x5D2E,
	/* Array index: 0x1572 */ 0x6AFA,0x5D32,
	/* Array index: 0x1572 */ 0x6AFB,0x5D2F,
	/* Array index: 0x1572 */ 0x6B04,0x4D73,
	/* Array index: 0x1572 */ 0x6B05,0x5D30,
	/* Array index: 0x1572 */ 0x6B0A,0x5C5E,
	/* Array index: 0x1572 */ 0x6B12,0x5D33,
	/* Array index: 0x1572 */ 0x6B16,0x5D34,
	/* Array index: 0x1572 */ 0x6B27,0x3224,
	/* Array index: 0x1572 */ 0x6B32,0x4D5F,
	/* Array index: 0x1572 */ 0x6B43,0x5D3D,
	/* Array index: 0x1572 */ 0x6B47,0x5D3C,
	/* Array index: 0x1572 */ 0x6B49,0x5D3E,
	/* Array index: 0x1572 */ 0x6B4C,0x324E,
	/* Array index: 0x1572 */ 0x6B4E,0x4337,
	/* Array index: 0x1572 */ 0x6B50,0x5D3F,
	/* Array index: 0x1572 */ 0x6B53,0x343F,
	/* Array index: 0x1572 */ 0x6B54,0x5D41,
	/* Array index: 0x1572 */ 0x6B59,0x5D40,
	/* Array index: 0x1572 */ 0x6B5B,0x5D42,
	/* Array index: 0x1572 */ 0x6B6F,0x3B75,
	/* Array index: 0x1572 */ 0x6B73,0x3A50,
	/* Array index: 0x1572 */ 0x6B74,0x4E72,
	/* Array index: 0x1572 */ 0x6B78,0x5D45,
	/* Array index: 0x1572 */ 0x6B79,0x5D46,
	/* Array index: 0x1572 */ 0x6B7B,0x3B60,
	/* Array index: 0x1572 */ 0x6B7F,0x5D47,
	/* Array index: 0x1572 */ 0x6B80,0x5D48,
	/* Array index: 0x1572 */ 0x6B83,0x5D4A,
	/* Array index: 0x1572 */ 0x6B84,0x5D49,
	/* Array index: 0x1572 */ 0x6B86,0x4B58,
	/* Array index: 0x1572 */ 0x6B95,0x5D4D,
	/* Array index: 0x1572 */ 0x6B96,0x3F23,
	/* Array index: 0x1572 */ 0x6B98,0x5D4C,
	/* Array index: 0x1572 */ 0x6B9E,0x5D4E,
	/* Array index: 0x1572 */ 0x6BA4,0x5D4F,
	/* Array index: 0x1572 */ 0x6BAA,0x5D50,
	/* Array index: 0x1572 */ 0x6BAB,0x5D51,
	/* Array index: 0x1572 */ 0x6BC5,0x3523,
	/* Array index: 0x1572 */ 0x6BC6,0x5D58,
	/* Array index: 0x1572 */ 0x6BCB,0x5D59,
	/* Array index: 0x1572 */ 0x6BCD,0x4A6C,
	/* Array index: 0x1572 */ 0x6BCE,0x4B68,
	/* Array index: 0x1572 */ 0x6BD8,0x487B,
	/* Array index: 0x1572 */ 0x6BDB,0x4C53,
	/* Array index: 0x1572 */ 0x6BDF,0x5D5B,
	/* Array index: 0x1572 */ 0x6BEB,0x5D5D,
	/* Array index: 0x1572 */ 0x6BEC,0x5D5C,
	/* Array index: 0x1572 */ 0x6BEF,0x5D5F,
	/* Array index: 0x1572 */ 0x6BF3,0x5D5E,
	/* Array index: 0x1572 */ 0x6C08,0x5D61,
	/* Array index: 0x1572 */ 0x6C0F,0x3B61,
	/* Array index: 0x1572 */ 0x6C11,0x4C31,
	/* Array index: 0x1572 */ 0x6C13,0x5D62,
	/* Array index: 0x1572 */ 0x6C14,0x5D63,
	/* Array index: 0x1572 */ 0x6C17,0x3524,
	/* Array index: 0x1572 */ 0x6C1B,0x5D64,
	/* Array index: 0x1572 */ 0x6C23,0x5D66,
	/* Array index: 0x1572 */ 0x6C24,0x5D65,
	/* Array index: 0x1572 */ 0x6C34,0x3F65,
	/* Array index: 0x1572 */ 0x6C37,0x4939,
	/* Array index: 0x1572 */ 0x6C38,0x314A,
	/* Array index: 0x1572 */ 0x6C4E,0x4846,
	/* Array index: 0x1572 */ 0x6C50,0x3C2E,
	/* Array index: 0x1572 */ 0x6C55,0x5D68,
	/* Array index: 0x1572 */ 0x6C57,0x3440,
	/* Array index: 0x1572 */ 0x6C5A,0x3178,
	/* Array index: 0x1572 */ 0x6C68,0x5D71,
	/* Array index: 0x1572 */ 0x6C6A,0x5D6A,
	/* Array index: 0x1572 */ 0x6C70,0x4241,
	/* Array index: 0x1572 */ 0x6C72,0x3562,
	/* Array index: 0x1572 */ 0x6C73,0x5D72,
	/* Array index: 0x1572 */ 0x6C7A,0x3768,
	/* Array index: 0x1572 */ 0x6C88,0x4440,
	/* Array index: 0x1572 */ 0x6C8C,0x4659,
	/* Array index: 0x1572 */ 0x6C8D,0x5D6C,
	/* Array index: 0x1572 */ 0x6C90,0x5D74,
	/* Array index: 0x1572 */ 0x6C92,0x5D73,
	/* Array index: 0x1572 */ 0x6C93,0x3723,
	/* Array index: 0x1572 */ 0x6C96,0x322D,
	/* Array index: 0x1572 */ 0x6CA1,0x4B57,
	/* Array index: 0x1572 */ 0x6CA2,0x4274,
	/* Array index: 0x1572 */ 0x6CAB,0x4B77,
	/* Array index: 0x1572 */ 0x6CAE,0x5D7C,
	/* Array index: 0x1572 */ 0x6CB1,0x5D7D,
	/* Array index: 0x1572 */ 0x6CB3,0x324F,
	/* Array index: 0x1572 */ 0x6CC9,0x4074,
	/* Array index: 0x1572 */ 0x6CCA,0x4771,
	/* Array index: 0x1572 */ 0x6CCC,0x4867,
	/* Array index: 0x1572 */ 0x6CD3,0x5D77,
	/* Array index: 0x1572 */ 0x6CD5,0x4B21,
	/* Array index: 0x1572 */ 0x6CD7,0x5D79,
	/* Array index: 0x1572 */ 0x6CD9,0x5E24,
	/* Array index: 0x1572 */ 0x6CDB,0x5E22,
	/* Array index: 0x1572 */ 0x6CDD,0x5D7B,
	/* Array index: 0x1572 */ 0x6CE8,0x436D,
	/* Array index: 0x1572 */ 0x6CEA,0x5E25,
	/* Array index: 0x1572 */ 0x6D0B,0x4D4E,
	/* Array index: 0x1572 */ 0x6D0C,0x5E30,
	/* Array index: 0x1572 */ 0x6D12,0x5E2F,
	/* Array index: 0x1572 */ 0x6D17,0x4076,
	/* Array index: 0x1572 */ 0x6D19,0x5E2C,
	/* Array index: 0x1572 */ 0x6D1B,0x4D6C,
	/* Array index: 0x1572 */ 0x6D1E,0x4636,
	/* Array index: 0x1572 */ 0x6D1F,0x5E26,
	/* Array index: 0x1572 */ 0x6D25,0x4445,
	/* Array index: 0x1572 */ 0x6D3B,0x3368,
	/* Array index: 0x1572 */ 0x6D3D,0x5E2A,
	/* Array index: 0x1572 */ 0x6D3E,0x4749,
	/* Array index: 0x1572 */ 0x6D41,0x4E2E,
	/* Array index: 0x1572 */ 0x6D44,0x3E74,
	/* Array index: 0x1572 */ 0x6D45,0x4075,
	/* Array index: 0x1572 */ 0x6D59,0x5E36,
	/* Array index: 0x1572 */ 0x6D5A,0x5E34,
	/* Array index: 0x1572 */ 0x6D5C,0x494D,
	/* Array index: 0x1572 */ 0x6D63,0x5E31,
	/* Array index: 0x1572 */ 0x6D64,0x5E33,
	/* Array index: 0x1572 */ 0x6D66,0x313A,
	/* Array index: 0x1572 */ 0x6D69,0x3940,
	/* Array index: 0x1572 */ 0x6D6A,0x4F32,
	/* Array index: 0x1572 */ 0x6D6C,0x333D,
	/* Array index: 0x1572 */ 0x6D6E,0x4962,
	/* Array index: 0x1572 */ 0x6D74,0x4D61,
	/* Array index: 0x1572 */ 0x6D85,0x5E3A,
	/* Array index: 0x1572 */ 0x6D88,0x3E43,
	/* Array index: 0x1572 */ 0x6D8C,0x4D30,
	/* Array index: 0x1572 */ 0x6D8E,0x5E37,
	/* Array index: 0x1572 */ 0x6D93,0x5E32,
	/* Array index: 0x1572 */ 0x6D95,0x5E38,
	/* Array index: 0x1572 */ 0x6D99,0x4E5E,
	/* Array index: 0x1572 */ 0x6D9B,0x4573,
	/* Array index: 0x1572 */ 0x6D9C,0x4642,
	/* Array index: 0x1572 */ 0x6DAF,0x3336,
	/* Array index: 0x1572 */ 0x6DB2,0x3155,
	/* Array index: 0x1572 */ 0x6DB5,0x5E3E,
	/* Array index: 0x1572 */ 0x6DB8,0x5E41,
	/* Array index: 0x1572 */ 0x6DBC,0x4E43,
	/* Array index: 0x1572 */ 0x6DC0,0x4D64,
	/* Array index: 0x1572 */ 0x6DCB,0x4E54,
	/* Array index: 0x1572 */ 0x6DCC,0x5E45,
	/* Array index: 0x1572 */ 0x6DD1,0x3D4A,
	/* Array index: 0x1572 */ 0x6DD2,0x5E47,
	/* Array index: 0x1572 */ 0x6DD5,0x5E4C,
	/* Array index: 0x1572 */ 0x6DD8,0x4571,
	/* Array index: 0x1572 */ 0x6DD9,0x5E4A,
	/* Array index: 0x1572 */ 0x6DDE,0x5E44,
	/* Array index: 0x1572 */ 0x6DE1,0x4338,
	/* Array index: 0x1572 */ 0x6DE4,0x5E4B,
	/* Array index: 0x1572 */ 0x6DE6,0x5E40,
	/* Array index: 0x1572 */ 0x6DF1,0x3F3C,
	/* Array index: 0x1572 */ 0x6DF3,0x3D5F,
	/* Array index: 0x1572 */ 0x6DF5,0x4A25,
	/* Array index: 0x1572 */ 0x6E13,0x374C,
	/* Array index: 0x1572 */ 0x6E15,0x5E3C,
	/* Array index: 0x1572 */ 0x6E38,0x5E62,
	/* Array index: 0x1572 */ 0x6E3A,0x5E5D,
	/* Array index: 0x1572 */ 0x6E3E,0x5E55,
	/* Array index: 0x1572 */ 0x6E43,0x5E5C,
	/* Array index: 0x1572 */ 0x6E4A,0x4C2B,
	/* Array index: 0x1572 */ 0x6E4D,0x5E5A,
	/* Array index: 0x1572 */ 0x6E4E,0x5E5E,
	/* Array index: 0x1572 */ 0x6E56,0x3850,
	/* Array index: 0x1572 */ 0x6E58,0x3E45,
	/* Array index: 0x1572 */ 0x6E5B,0x4339,
	/* Array index: 0x1572 */ 0x6E5F,0x5E54,
	/* Array index: 0x1572 */ 0x6E67,0x4D2F,
	/* Array index: 0x1572 */ 0x6E6B,0x5E57,
	/* Array index: 0x1572 */ 0x6E6E,0x5E50,
	/* Array index: 0x1572 */ 0x6E6F,0x4572,
	/* Array index: 0x1572 */ 0x6E72,0x5E53,
	/* Array index: 0x1572 */ 0x6E76,0x5E59,
	/* Array index: 0x1572 */ 0x6E8C,0x482E,
	/* Array index: 0x1572 */ 0x6E8F,0x5E6F,
	/* Array index: 0x1572 */ 0x6E90,0x383B,
	/* Array index: 0x1572 */ 0x6E96,0x3D60,
	/* Array index: 0x1572 */ 0x6E98,0x5E65,
	/* Array index: 0x1572 */ 0x6E9C,0x4E2F,
	/* Array index: 0x1572 */ 0x6E9D,0x3942,
	/* Array index: 0x1572 */ 0x6E9F,0x5E72,
	/* Array index: 0x1572 */ 0x6EA2,0x306E,
	/* Array index: 0x1572 */ 0x6EA5,0x5E70,
	/* Array index: 0x1572 */ 0x6EAA,0x5E64,
	/* Array index: 0x1572 */ 0x6EAF,0x5E6A,
	/* Array index: 0x1572 */ 0x6EB2,0x5E6C,
	/* Array index: 0x1572 */ 0x6EB6,0x4D4F,
	/* Array index: 0x1572 */ 0x6EB7,0x5E67,
	/* Array index: 0x1572 */ 0x6EBA,0x452E,
	/* Array index: 0x1572 */ 0x6EBD,0x5E69,
	/* Array index: 0x1572 */ 0x6EC2,0x5E71,
	/* Array index: 0x1572 */ 0x6EC4,0x5E6B,
	/* Array index: 0x1572 */ 0x6EC5,0x4C47,
	/* Array index: 0x1572 */ 0x6EC9,0x5E66,
	/* Array index: 0x1572 */ 0x6ECB,0x3C22,
	/* Array index: 0x1572 */ 0x6ECC,0x5E7E,
	/* Array index: 0x1572 */ 0x6EDD,0x426C,
	/* Array index: 0x1572 */ 0x6EDE,0x425A,
	/* Array index: 0x1572 */ 0x6EEC,0x5E76,
	/* Array index: 0x1572 */ 0x6EEF,0x5E7C,
	/* Array index: 0x1572 */ 0x6EF2,0x5E7A,
	/* Array index: 0x1572 */ 0x6EF4,0x4529,
	/* Array index: 0x1572 */ 0x6EF7,0x5F23,
	/* Array index: 0x1572 */ 0x6EF8,0x5E77,
	/* Array index: 0x1572 */ 0x6F06,0x3C3F,
	/* Array index: 0x1572 */ 0x6F09,0x3977,
	/* Array index: 0x1572 */ 0x6F0F,0x4F33,
	/* Array index: 0x1572 */ 0x6F20,0x4779,
	/* Array index: 0x1572 */ 0x6F22,0x3441,
	/* Array index: 0x1572 */ 0x6F23,0x4E7A,
	/* Array index: 0x1572 */ 0x6F2B,0x4C21,
	/* Array index: 0x1572 */ 0x6F2C,0x4452,
	/* Array index: 0x1572 */ 0x6F31,0x5E7B,
	/* Array index: 0x1572 */ 0x6F32,0x5E7D,
	/* Array index: 0x1572 */ 0x6F38,0x4132,
	/* Array index: 0x1572 */ 0x6F3E,0x5F21,
	/* Array index: 0x1572 */ 0x6F3F,0x5E79,
	/* Array index: 0x1572 */ 0x6F41,0x5E73,
	/* Array index: 0x1572 */ 0x6F45,0x3443,
	/* Array index: 0x1572 */ 0x6F54,0x3769,
	/* Array index: 0x1572 */ 0x6F58,0x5F2F,
	/* Array index: 0x1572 */ 0x6F5B,0x5F2A,
	/* Array index: 0x1572 */ 0x6F5C,0x4078,
	/* Array index: 0x1572 */ 0x6F5F,0x3363,
	/* Array index: 0x1572 */ 0x6F64,0x3D61,
	/* Array index: 0x1572 */ 0x6F66,0x5F33,
	/* Array index: 0x1572 */ 0x6F74,0x5F4C,
	/* Array index: 0x1572 */ 0x6F78,0x5F26,
	/* Array index: 0x1572 */ 0x6F7A,0x5F25,
	/* Array index: 0x1572 */ 0x6F7C,0x5F2E,
	/* Array index: 0x1572 */ 0x6F8E,0x5F30,
	/* Array index: 0x1572 */ 0x6F91,0x5F31,
	/* Array index: 0x1572 */ 0x6F97,0x3442,
	/* Array index: 0x1572 */ 0x6FA1,0x5F36,
	/* Array index: 0x1572 */ 0x6FA3,0x5F35,
	/* Array index: 0x1572 */ 0x6FA4,0x5F37,
	/* Array index: 0x1572 */ 0x6FAA,0x5F3A,
	/* Array index: 0x1572 */ 0x6FB1,0x4543,
	/* Array index: 0x1572 */ 0x6FB3,0x5F34,
	/* Array index: 0x1572 */ 0x6FB9,0x5F38,
	/* Array index: 0x1572 */ 0x6FC6,0x5F39,
	/* Array index: 0x1572 */ 0x6FD4,0x5F3E,
	/* Array index: 0x1572 */ 0x6FD5,0x5F3C,
	/* Array index: 0x1572 */ 0x6FD8,0x5F3F,
	/* Array index: 0x1572 */ 0x6FDB,0x5F42,
	/* Array index: 0x1572 */ 0x6FE4,0x5E39,
	/* Array index: 0x1572 */ 0x6FF6,0x6F69,
	/* Array index: 0x1572 */ 0x6FFA,0x5F45,
	/* Array index: 0x1572 */ 0x6FFE,0x5F49,
	/* Array index: 0x1572 */ 0x7001,0x5F47,
	/* Array index: 0x1572 */ 0x7009,0x5F43,
	/* Array index: 0x1572 */ 0x700B,0x5F44,
	/* Array index: 0x1572 */ 0x700F,0x5F48,
	/* Array index: 0x1572 */ 0x7011,0x5F46,
	/* Array index: 0x1572 */ 0x7015,0x494E,
	/* Array index: 0x1572 */ 0x7018,0x5F4E,
	/* Array index: 0x1572 */ 0x7026,0x4375,
	/* Array index: 0x1572 */ 0x7027,0x426D,
	/* Array index: 0x1572 */ 0x702C,0x4025,
	/* Array index: 0x1572 */ 0x7030,0x5F50,
	/* Array index: 0x1572 */ 0x7032,0x5F52,
	/* Array index: 0x1572 */ 0x703E,0x5F51,
	/* Array index: 0x1572 */ 0x704C,0x5E75,
	/* Array index: 0x1572 */ 0x7051,0x5F53,
	/* Array index: 0x1572 */ 0x7058,0x4667,
	/* Array index: 0x1572 */ 0x7063,0x5F54,
	/* Array index: 0x1572 */ 0x706B,0x3250,
	/* Array index: 0x1572 */ 0x706F,0x4574,
	/* Array index: 0x1572 */ 0x7070,0x3325,
	/* Array index: 0x1572 */ 0x7078,0x3564,
	/* Array index: 0x1572 */ 0x707C,0x3C5E,
	/* Array index: 0x1572 */ 0x707D,0x3A52,
	/* Array index: 0x1572 */ 0x7089,0x4F27,
	/* Array index: 0x1572 */ 0x708A,0x3F66,
	/* Array index: 0x1572 */ 0x708E,0x316A,
	/* Array index: 0x1572 */ 0x7092,0x5F56,
	/* Array index: 0x1572 */ 0x7099,0x5F55,
	/* Array index: 0x1572 */ 0x70B3,0x5F5B,
	/* Array index: 0x1572 */ 0x70C8,0x4E75,
	/* Array index: 0x1572 */ 0x70CB,0x5F5E,
	/* Array index: 0x1572 */ 0x70CF,0x3128,
	/* Array index: 0x1572 */ 0x70D9,0x5F60,
	/* Array index: 0x1572 */ 0x70DD,0x5F5F,
	/* Array index: 0x1572 */ 0x70DF,0x5F5D,
	/* Array index: 0x1572 */ 0x70F1,0x5F58,
	/* Array index: 0x1572 */ 0x70F9,0x4B23,
	/* Array index: 0x1572 */ 0x70FD,0x5F62,
	/* Array index: 0x1572 */ 0x7109,0x5F61,
	/* Array index: 0x1572 */ 0x7114,0x316B,
	/* Array index: 0x1572 */ 0x7119,0x5F64,
	/* Array index: 0x1572 */ 0x711A,0x4A32,
	/* Array index: 0x1572 */ 0x711C,0x5F63,
	/* Array index: 0x1572 */ 0x7121,0x4C35,
	/* Array index: 0x1572 */ 0x7126,0x3E47,
	/* Array index: 0x1572 */ 0x7136,0x4133,
	/* Array index: 0x1572 */ 0x713C,0x3E46,
	/* Array index: 0x1572 */ 0x7149,0x4E7B,
	/* Array index: 0x1572 */ 0x714C,0x5F6A,
	/* Array index: 0x1572 */ 0x714E,0x4079,
	/* Array index: 0x1572 */ 0x7155,0x5F66,
	/* Array index: 0x1572 */ 0x7156,0x5F6B,
	/* Array index: 0x1572 */ 0x7159,0x316C,
	/* Array index: 0x1572 */ 0x716C,0x5F6C,
	/* Array index: 0x1572 */ 0x716E,0x3C51,
	/* Array index: 0x1572 */ 0x717D,0x407A,
	/* Array index: 0x1572 */ 0x7184,0x5F6F,
	/* Array index: 0x1572 */ 0x7188,0x5F67,
	/* Array index: 0x1572 */ 0x718A,0x3727,
	/* Array index: 0x1572 */ 0x718F,0x5F6D,
	/* Array index: 0x1572 */ 0x7194,0x4D50,
	/* Array index: 0x1572 */ 0x7195,0x5F70,
	/* Array index: 0x1572 */ 0x7199,0x7426,
	/* Array index: 0x1572 */ 0x719F,0x3D4F,
	/* Array index: 0x1572 */ 0x71A8,0x5F71,
	/* Array index: 0x1572 */ 0x71AC,0x5F72,
	/* Array index: 0x1572 */ 0x71B1,0x472E,
	/* Array index: 0x1572 */ 0x71B9,0x5F74,
	/* Array index: 0x1572 */ 0x71BE,0x5F75,
	/* Array index: 0x1572 */ 0x71C3,0x4733,
	/* Array index: 0x1572 */ 0x71C8,0x4575,
	/* Array index: 0x1572 */ 0x71C9,0x5F77,
	/* Array index: 0x1572 */ 0x71CE,0x5F79,
	/* Array index: 0x1572 */ 0x71D0,0x4E55,
	/* Array index: 0x1572 */ 0x71D2,0x5F76,
	/* Array index: 0x1572 */ 0x71D4,0x5F78,
	/* Array index: 0x1572 */ 0x71D5,0x316D,
	/* Array index: 0x1572 */ 0x71D7,0x5F73,
	/* Array index: 0x1572 */ 0x71DF,0x535B,
	/* Array index: 0x1572 */ 0x71E0,0x5F7A,
	/* Array index: 0x1572 */ 0x71F5,0x5F7D,
	/* Array index: 0x1572 */ 0x71F9,0x6021,
	/* Array index: 0x1572 */ 0x71FB,0x5F6E,
	/* Array index: 0x1572 */ 0x71FC,0x5F7E,
	/* Array index: 0x1572 */ 0x71FF,0x6022,
	/* Array index: 0x1572 */ 0x7206,0x477A,
	/* Array index: 0x1572 */ 0x720D,0x6023,
	/* Array index: 0x1572 */ 0x7210,0x6024,
	/* Array index: 0x1572 */ 0x721B,0x6025,
	/* Array index: 0x1572 */ 0x7228,0x6026,
	/* Array index: 0x1572 */ 0x722A,0x445E,
	/* Array index: 0x1572 */ 0x722C,0x6028,
	/* Array index: 0x1572 */ 0x722D,0x6027,
	/* Array index: 0x1572 */ 0x7230,0x6029,
	/* Array index: 0x1572 */ 0x7232,0x602A,
	/* Array index: 0x1572 */ 0x7235,0x3C5F,
	/* Array index: 0x1572 */ 0x7236,0x4963,
	/* Array index: 0x1572 */ 0x7252,0x442D,
	/* Array index: 0x1572 */ 0x7258,0x6031,
	/* Array index: 0x1572 */ 0x7259,0x3267,
	/* Array index: 0x1572 */ 0x725B,0x356D,
	/* Array index: 0x1572 */ 0x725D,0x4C46,
	/* Array index: 0x1572 */ 0x725F,0x4C36,
	/* Array index: 0x1572 */ 0x7261,0x3234,
	/* Array index: 0x1572 */ 0x7262,0x4F34,
	/* Array index: 0x1572 */ 0x7267,0x4B52,
	/* Array index: 0x1572 */ 0x7269,0x4A2A,
	/* Array index: 0x1572 */ 0x7272,0x4037,
	/* Array index: 0x1572 */ 0x7274,0x6032,
	/* Array index: 0x1572 */ 0x7279,0x4643,
	/* Array index: 0x1572 */ 0x7287,0x6036,
	/* Array index: 0x1572 */ 0x7292,0x6037,
	/* Array index: 0x1572 */ 0x7296,0x6038,
	/* Array index: 0x1572 */ 0x72A0,0x353E,
	/* Array index: 0x1572 */ 0x72A2,0x6039,
	/* Array index: 0x1572 */ 0x72A7,0x603A,
	/* Array index: 0x1572 */ 0x72AC,0x3824,
	/* Array index: 0x1572 */ 0x72AF,0x4848,
	/* Array index: 0x1572 */ 0x72B2,0x603C,
	/* Array index: 0x1572 */ 0x72B6,0x3E75,
	/* Array index: 0x1572 */ 0x72B9,0x603B,
	/* Array index: 0x1572 */ 0x72CE,0x6040,
	/* Array index: 0x1572 */ 0x72D0,0x3851,
	/* Array index: 0x1572 */ 0x72D2,0x6041,
	/* Array index: 0x1572 */ 0x72D7,0x3669,
	/* Array index: 0x1572 */ 0x72D9,0x4140,
	/* Array index: 0x1572 */ 0x72DB,0x397D,
	/* Array index: 0x1572 */ 0x72E9,0x3C6D,
	/* Array index: 0x1572 */ 0x72EC,0x4648,
	/* Array index: 0x1572 */ 0x72ED,0x3639,
	/* Array index: 0x1572 */ 0x730A,0x6049,
	/* Array index: 0x1572 */ 0x7316,0x604B,
	/* Array index: 0x1572 */ 0x7317,0x6048,
	/* Array index: 0x1572 */ 0x7325,0x6050,
	/* Array index: 0x1572 */ 0x7334,0x604D,
	/* Array index: 0x1572 */ 0x7336,0x4D31,
	/* Array index: 0x1572 */ 0x7337,0x4D32,
	/* Array index: 0x1572 */ 0x733E,0x6051,
	/* Array index: 0x1572 */ 0x733F,0x316E,
	/* Array index: 0x1572 */ 0x7344,0x3976,
	/* Array index: 0x1572 */ 0x7345,0x3B62,
	/* Array index: 0x1572 */ 0x734E,0x6052,
	/* Array index: 0x1572 */ 0x734F,0x6053,
	/* Array index: 0x1572 */ 0x7357,0x6055,
	/* Array index: 0x1572 */ 0x7363,0x3D43,
	/* Array index: 0x1572 */ 0x7368,0x6057,
	/* Array index: 0x1572 */ 0x736A,0x6056,
	/* Array index: 0x1572 */ 0x7370,0x6058,
	/* Array index: 0x1572 */ 0x7372,0x334D,
	/* Array index: 0x1572 */ 0x7375,0x605A,
	/* Array index: 0x1572 */ 0x7378,0x6059,
	/* Array index: 0x1572 */ 0x737A,0x605C,
	/* Array index: 0x1572 */ 0x737B,0x605B,
	/* Array index: 0x1572 */ 0x7384,0x383C,
	/* Array index: 0x1572 */ 0x7387,0x4E28,
	/* Array index: 0x1572 */ 0x7389,0x364C,
	/* Array index: 0x1572 */ 0x738B,0x3226,
	/* Array index: 0x1572 */ 0x7396,0x366A,
	/* Array index: 0x1572 */ 0x73A9,0x3461,
	/* Array index: 0x1572 */ 0x73B2,0x4E68,
	/* Array index: 0x1572 */ 0x73B3,0x605E,
	/* Array index: 0x1572 */ 0x73BB,0x6060,
	/* Array index: 0x1572 */ 0x73C0,0x6061,
	/* Array index: 0x1572 */ 0x73C2,0x3251,
	/* Array index: 0x1572 */ 0x73C8,0x605D,
	/* Array index: 0x1572 */ 0x73CA,0x3B39,
	/* Array index: 0x1572 */ 0x73CD,0x4441,
	/* Array index: 0x1572 */ 0x73CE,0x605F,
	/* Array index: 0x1572 */ 0x73DE,0x6064,
	/* Array index: 0x1572 */ 0x73E0,0x3C6E,
	/* Array index: 0x1572 */ 0x73E5,0x6062,
	/* Array index: 0x1572 */ 0x73EA,0x373E,
	/* Array index: 0x1572 */ 0x73ED,0x4849,
	/* Array index: 0x1572 */ 0x73EE,0x6063,
	/* Array index: 0x1572 */ 0x73F1,0x607E,
	/* Array index: 0x1572 */ 0x73F8,0x6069,
	/* Array index: 0x1572 */ 0x73FE,0x383D,
	/* Array index: 0x1572 */ 0x7403,0x3565,
	/* Array index: 0x1572 */ 0x7405,0x6066,
	/* Array index: 0x1572 */ 0x7406,0x4D7D,
	/* Array index: 0x1572 */ 0x7409,0x4E30,
	/* Array index: 0x1572 */ 0x7422,0x4276,
	/* Array index: 0x1572 */ 0x7425,0x6068,
	/* Array index: 0x1572 */ 0x743A,0x606B,
	/* Array index: 0x1572 */ 0x743F,0x606D,
	/* Array index: 0x1572 */ 0x7441,0x6070,
	/* Array index: 0x1572 */ 0x7455,0x606C,
	/* Array index: 0x1572 */ 0x7469,0x6072,
	/* Array index: 0x1572 */ 0x746A,0x6075,
	/* Array index: 0x1572 */ 0x746F,0x6067,
	/* Array index: 0x1572 */ 0x7470,0x6073,
	/* Array index: 0x1572 */ 0x7473,0x3A3C,
	/* Array index: 0x1572 */ 0x7476,0x6076,
	/* Array index: 0x1572 */ 0x747E,0x6077,
	/* Array index: 0x1572 */ 0x7483,0x4D7E,
	/* Array index: 0x1572 */ 0x748B,0x6078,
	/* Array index: 0x1572 */ 0x749E,0x6079,
	/* Array index: 0x1572 */ 0x74A2,0x6065,
	/* Array index: 0x1572 */ 0x74A7,0x607A,
	/* Array index: 0x1572 */ 0x74B0,0x3444,
	/* Array index: 0x1572 */ 0x74BD,0x3C25,
	/* Array index: 0x1572 */ 0x74CA,0x607B,
	/* Array index: 0x1572 */ 0x74CF,0x607C,
	/* Array index: 0x1572 */ 0x74D4,0x607D,
	/* Array index: 0x1572 */ 0x74DC,0x313B,
	/* Array index: 0x1572 */ 0x74E0,0x6121,
	/* Array index: 0x1572 */ 0x74E2,0x493B,
	/* Array index: 0x1572 */ 0x74E3,0x6122,
	/* Array index: 0x1572 */ 0x74E6,0x3424,
	/* Array index: 0x1572 */ 0x74E7,0x6123,
	/* Array index: 0x1572 */ 0x74E9,0x6124,
	/* Array index: 0x1572 */ 0x7511,0x3979,
	/* Array index: 0x1572 */ 0x7513,0x6132,
	/* Array index: 0x1572 */ 0x7515,0x6131,
	/* Array index: 0x1572 */ 0x7518,0x3445,
	/* Array index: 0x1572 */ 0x751A,0x3F53,
	/* Array index: 0x1572 */ 0x751C,0x453C,
	/* Array index: 0x1572 */ 0x751E,0x6133,
	/* Array index: 0x1572 */ 0x751F,0x4038,
	/* Array index: 0x1572 */ 0x7523,0x3B3A,
	/* Array index: 0x1572 */ 0x7525,0x3179,
	/* Array index: 0x1572 */ 0x7526,0x6134,
	/* Array index: 0x1572 */ 0x7528,0x4D51,
	/* Array index: 0x1572 */ 0x752B,0x4A63,
	/* Array index: 0x1572 */ 0x752C,0x6135,
	/* Array index: 0x1572 */ 0x7544,0x6137,
	/* Array index: 0x1572 */ 0x7546,0x613C,
	/* Array index: 0x1572 */ 0x7554,0x484A,
	/* Array index: 0x1572 */ 0x7560,0x482B,
	/* Array index: 0x1572 */ 0x7570,0x305B,
	/* Array index: 0x1572 */ 0x757F,0x3526,
	/* Array index: 0x1572 */ 0x7582,0x614A,
	/* Array index: 0x1572 */ 0x7594,0x614B,
	/* Array index: 0x1572 */ 0x759A,0x614C,
	/* Array index: 0x1572 */ 0x759D,0x614D,
	/* Array index: 0x1572 */ 0x75A3,0x614F,
	/* Array index: 0x1572 */ 0x75A5,0x614E,
	/* Array index: 0x1572 */ 0x75AB,0x3156,
	/* Array index: 0x1572 */ 0x75C2,0x6150,
	/* Array index: 0x1572 */ 0x75C3,0x6152,
	/* Array index: 0x1572 */ 0x75C5,0x4942,
	/* Array index: 0x1572 */ 0x75C7,0x3E49,
	/* Array index: 0x1572 */ 0x75CA,0x6159,
	/* Array index: 0x1572 */ 0x75CD,0x6158,
	/* Array index: 0x1572 */ 0x75D2,0x615A,
	/* Array index: 0x1572 */ 0x75D4,0x3C26,
	/* Array index: 0x1572 */ 0x75D5,0x3A2F,
	/* Array index: 0x1572 */ 0x75D8,0x4577,
	/* Array index: 0x1572 */ 0x75D9,0x615B,
	/* Array index: 0x1572 */ 0x75DB,0x444B,
	/* Array index: 0x1572 */ 0x75DE,0x615D,
	/* Array index: 0x1572 */ 0x75E2,0x4E21,
	/* Array index: 0x1572 */ 0x75E3,0x615C,
	/* Array index: 0x1572 */ 0x75E9,0x4169,
	/* Array index: 0x1572 */ 0x75FA,0x6163,
	/* Array index: 0x1572 */ 0x75FC,0x6160,
	/* Array index: 0x1572 */ 0x75FE,0x615E,
	/* Array index: 0x1572 */ 0x75FF,0x615F,
	/* Array index: 0x1572 */ 0x7601,0x6161,
	/* Array index: 0x1572 */ 0x7609,0x6168,
	/* Array index: 0x1572 */ 0x760B,0x6166,
	/* Array index: 0x1572 */ 0x760D,0x6167,
	/* Array index: 0x1572 */ 0x7627,0x616A,
	/* Array index: 0x1572 */ 0x7630,0x6170,
	/* Array index: 0x1572 */ 0x7634,0x616F,
	/* Array index: 0x1572 */ 0x763B,0x6171,
	/* Array index: 0x1572 */ 0x7642,0x4E45,
	/* Array index: 0x1572 */ 0x764C,0x3462,
	/* Array index: 0x1572 */ 0x7652,0x4C7E,
	/* Array index: 0x1572 */ 0x7656,0x4A4A,
	/* Array index: 0x1572 */ 0x7658,0x6176,
	/* Array index: 0x1572 */ 0x765C,0x6175,
	/* Array index: 0x1572 */ 0x7661,0x6177,
	/* Array index: 0x1572 */ 0x7662,0x6178,
	/* Array index: 0x1572 */ 0x7670,0x617E,
	/* Array index: 0x1572 */ 0x7672,0x6221,
	/* Array index: 0x1572 */ 0x7676,0x6222,
	/* Array index: 0x1572 */ 0x768B,0x6228,
	/* Array index: 0x1572 */ 0x768E,0x6229,
	/* Array index: 0x1572 */ 0x7690,0x3B29,
	/* Array index: 0x1572 */ 0x7693,0x622B,
	/* Array index: 0x1572 */ 0x7696,0x622A,
	/* Array index: 0x1572 */ 0x7699,0x622C,
	/* Array index: 0x1572 */ 0x769A,0x622D,
	/* Array index: 0x1572 */ 0x76AE,0x4869,
	/* Array index: 0x1572 */ 0x76B0,0x622E,
	/* Array index: 0x1572 */ 0x76B4,0x622F,
	/* Array index: 0x1572 */ 0x76BF,0x3B2E,
	/* Array index: 0x1572 */ 0x76C2,0x6233,
	/* Array index: 0x1572 */ 0x76C3,0x4756,
	/* Array index: 0x1572 */ 0x76C6,0x4B5F,
	/* Array index: 0x1572 */ 0x76C8,0x314E,
	/* Array index: 0x1572 */ 0x76CA,0x3157,
	/* Array index: 0x1572 */ 0x76CD,0x6234,
	/* Array index: 0x1572 */ 0x76D2,0x6236,
	/* Array index: 0x1572 */ 0x76D6,0x6235,
	/* Array index: 0x1572 */ 0x76D7,0x4570,
	/* Array index: 0x1572 */ 0x76EA,0x623B,
	/* Array index: 0x1572 */ 0x76EE,0x4C5C,
	/* Array index: 0x1572 */ 0x76F2,0x4C55,
	/* Array index: 0x1572 */ 0x76F4,0x443E,
	/* Array index: 0x1572 */ 0x76F8,0x416A,
	/* Array index: 0x1572 */ 0x76FB,0x623D,
	/* Array index: 0x1572 */ 0x76FE,0x3D62,
	/* Array index: 0x1572 */ 0x7701,0x3E4A,
	/* Array index: 0x1572 */ 0x7704,0x6240,
	/* Array index: 0x1572 */ 0x771B,0x6246,
	/* Array index: 0x1572 */ 0x7729,0x6241,
	/* Array index: 0x1572 */ 0x7737,0x6247,
	/* Array index: 0x1572 */ 0x7738,0x6248,
	/* Array index: 0x1572 */ 0x773A,0x442F,
	/* Array index: 0x1572 */ 0x773C,0x3463,
	/* Array index: 0x1572 */ 0x7740,0x4365,
	/* Array index: 0x1572 */ 0x7747,0x6249,
	/* Array index: 0x1572 */ 0x775A,0x624A,
	/* Array index: 0x1572 */ 0x775B,0x624D,
	/* Array index: 0x1572 */ 0x7761,0x3F67,
	/* Array index: 0x1572 */ 0x7763,0x4644,
	/* Array index: 0x1572 */ 0x7765,0x624E,
	/* Array index: 0x1572 */ 0x7766,0x4B53,
	/* Array index: 0x1572 */ 0x7768,0x624B,
	/* Array index: 0x1572 */ 0x776B,0x624C,
	/* Array index: 0x1572 */ 0x7779,0x6251,
	/* Array index: 0x1572 */ 0x777E,0x6250,
	/* Array index: 0x1572 */ 0x777F,0x624F,
	/* Array index: 0x1572 */ 0x778B,0x6253,
	/* Array index: 0x1572 */ 0x778E,0x6252,
	/* Array index: 0x1572 */ 0x7791,0x6254,
	/* Array index: 0x1572 */ 0x779E,0x6256,
	/* Array index: 0x1572 */ 0x77A0,0x6255,
	/* Array index: 0x1572 */ 0x77A5,0x4A4D,
	/* Array index: 0x1572 */ 0x77AC,0x3D56,
	/* Array index: 0x1572 */ 0x77AD,0x4E46,
	/* Array index: 0x1572 */ 0x77B0,0x6257,
	/* Array index: 0x1572 */ 0x77B3,0x4637,
	/* Array index: 0x1572 */ 0x77B6,0x6258,
	/* Array index: 0x1572 */ 0x77C7,0x625E,
	/* Array index: 0x1572 */ 0x77CD,0x625F,
	/* Array index: 0x1572 */ 0x77D7,0x6260,
	/* Array index: 0x1572 */ 0x77E2,0x4C70,
	/* Array index: 0x1572 */ 0x77E3,0x6263,
	/* Array index: 0x1572 */ 0x77E5,0x434E,
	/* Array index: 0x1572 */ 0x77E7,0x476A,
	/* Array index: 0x1572 */ 0x77E9,0x366B,
	/* Array index: 0x1572 */ 0x77F3,0x4050,
	/* Array index: 0x1572 */ 0x77FC,0x6265,
	/* Array index: 0x1572 */ 0x7802,0x3A3D,
	/* Array index: 0x1572 */ 0x780C,0x6266,
	/* Array index: 0x1572 */ 0x7812,0x6267,
	/* Array index: 0x1572 */ 0x7814,0x3826,
	/* Array index: 0x1572 */ 0x7815,0x3A55,
	/* Array index: 0x1572 */ 0x7820,0x6269,
	/* Array index: 0x1572 */ 0x7832,0x4B24,
	/* Array index: 0x1572 */ 0x7834,0x474B,
	/* Array index: 0x1572 */ 0x783A,0x4557,
	/* Array index: 0x1572 */ 0x783F,0x395C,
	/* Array index: 0x1572 */ 0x7845,0x626B,
	/* Array index: 0x1572 */ 0x785D,0x3E4B,
	/* Array index: 0x1572 */ 0x786B,0x4E32,
	/* Array index: 0x1572 */ 0x786C,0x3945,
	/* Array index: 0x1572 */ 0x786F,0x3827,
	/* Array index: 0x1572 */ 0x7872,0x4823,
	/* Array index: 0x1572 */ 0x7874,0x626D,
	/* Array index: 0x1572 */ 0x787C,0x626F,
	/* Array index: 0x1572 */ 0x7881,0x386B,
	/* Array index: 0x1572 */ 0x7886,0x626E,
	/* Array index: 0x1572 */ 0x7887,0x4476,
	/* Array index: 0x1572 */ 0x7891,0x486A,
	/* Array index: 0x1572 */ 0x7893,0x3130,
	/* Array index: 0x1572 */ 0x7895,0x3A6C,
	/* Array index: 0x1572 */ 0x7897,0x4F52,
	/* Array index: 0x1572 */ 0x789A,0x6270,
	/* Array index: 0x1572 */ 0x78A3,0x6272,
	/* Array index: 0x1572 */ 0x78A7,0x4A4B,
	/* Array index: 0x1572 */ 0x78A9,0x4059,
	/* Array index: 0x1572 */ 0x78AA,0x6274,
	/* Array index: 0x1572 */ 0x78AF,0x6275,
	/* Array index: 0x1572 */ 0x78B5,0x6273,
	/* Array index: 0x1572 */ 0x78BA,0x334E,
	/* Array index: 0x1572 */ 0x78BC,0x627B,
	/* Array index: 0x1572 */ 0x78BE,0x627A,
	/* Array index: 0x1572 */ 0x78C1,0x3C27,
	/* Array index: 0x1572 */ 0x78C5,0x627C,
	/* Array index: 0x1572 */ 0x78C6,0x6277,
	/* Array index: 0x1572 */ 0x78CA,0x627D,
	/* Array index: 0x1572 */ 0x78CB,0x6278,
	/* Array index: 0x1572 */ 0x78D0,0x4858,
	/* Array index: 0x1572 */ 0x78D1,0x6276,
	/* Array index: 0x1572 */ 0x78D4,0x6279,
	/* Array index: 0x1572 */ 0x78DA,0x6322,
	/* Array index: 0x1572 */ 0x78E7,0x6321,
	/* Array index: 0x1572 */ 0x78E8,0x4B61,
	/* Array index: 0x1572 */ 0x78EC,0x627E,
	/* Array index: 0x1572 */ 0x78EF,0x306B,
	/* Array index: 0x1572 */ 0x78F4,0x6324,
	/* Array index: 0x1572 */ 0x78FD,0x6323,
	/* Array index: 0x1572 */ 0x7901,0x3E4C,
	/* Array index: 0x1572 */ 0x7907,0x6325,
	/* Array index: 0x1572 */ 0x790E,0x4143,
	/* Array index: 0x1572 */ 0x7911,0x6327,
	/* Array index: 0x1572 */ 0x7912,0x6326,
	/* Array index: 0x1572 */ 0x7919,0x6328,
	/* Array index: 0x1572 */ 0x7926,0x6268,
	/* Array index: 0x1572 */ 0x793A,0x3C28,
	/* Array index: 0x1572 */ 0x793C,0x4E69,
	/* Array index: 0x1572 */ 0x793E,0x3C52,
	/* Array index: 0x1572 */ 0x7940,0x632B,
	/* Array index: 0x1572 */ 0x7941,0x3737,
	/* Array index: 0x1572 */ 0x7950,0x4D34,
	/* Array index: 0x1572 */ 0x795A,0x632F,
	/* Array index: 0x1572 */ 0x7965,0x3E4D,
	/* Array index: 0x1572 */ 0x7968,0x493C,
	/* Array index: 0x1572 */ 0x796D,0x3A57,
	/* Array index: 0x1572 */ 0x7977,0x4578,
	/* Array index: 0x1572 */ 0x797A,0x6332,
	/* Array index: 0x1572 */ 0x798A,0x6334,
	/* Array index: 0x1572 */ 0x799D,0x6335,
	/* Array index: 0x1572 */ 0x79A6,0x357A,
	/* Array index: 0x1572 */ 0x79A7,0x6336,
	/* Array index: 0x1572 */ 0x79AA,0x6338,
	/* Array index: 0x1572 */ 0x79AE,0x6339,
	/* Array index: 0x1572 */ 0x79B0,0x4729,
	/* Array index: 0x1572 */ 0x79B3,0x633A,
	/* Array index: 0x1572 */ 0x79C9,0x633D,
	/* Array index: 0x1572 */ 0x79CB,0x3D29,
	/* Array index: 0x1572 */ 0x79D1,0x324A,
	/* Array index: 0x1572 */ 0x79D2,0x4943,
	/* Array index: 0x1572 */ 0x79D5,0x633E,
	/* Array index: 0x1572 */ 0x79D8,0x486B,
	/* Array index: 0x1572 */ 0x79DF,0x4145,
	/* Array index: 0x1572 */ 0x79E1,0x6341,
	/* Array index: 0x1572 */ 0x79EC,0x6340,
	/* Array index: 0x1572 */ 0x79F0,0x3E4E,
	/* Array index: 0x1572 */ 0x79FB,0x305C,
	/* Array index: 0x1572 */ 0x7A00,0x3529,
	/* Array index: 0x1572 */ 0x7A08,0x6343,
	/* Array index: 0x1572 */ 0x7A0B,0x4478,
	/* Array index: 0x1572 */ 0x7A0D,0x6344,
	/* Array index: 0x1572 */ 0x7A0E,0x4047,
	/* Array index: 0x1572 */ 0x7A14,0x4C2D,
	/* Array index: 0x1572 */ 0x7A2E,0x3C6F,
	/* Array index: 0x1572 */ 0x7A31,0x634A,
	/* Array index: 0x1572 */ 0x7A32,0x3070,
	/* Array index: 0x1572 */ 0x7A37,0x634D,
	/* Array index: 0x1572 */ 0x7A46,0x4B54,
	/* Array index: 0x1572 */ 0x7A49,0x6350,
	/* Array index: 0x1572 */ 0x7A57,0x634F,
	/* Array index: 0x1572 */ 0x7A69,0x6353,
	/* Array index: 0x1572 */ 0x7A6B,0x334F,
	/* Array index: 0x1572 */ 0x7A70,0x6355,
	/* Array index: 0x1572 */ 0x7A74,0x376A,
	/* Array index: 0x1572 */ 0x7A76,0x3566,
	/* Array index: 0x1572 */ 0x7A79,0x6356,
	/* Array index: 0x1572 */ 0x7A7A,0x3675,
	/* Array index: 0x1572 */ 0x7A7D,0x6357,
	/* Array index: 0x1572 */ 0x7A7F,0x407C,
	/* Array index: 0x1572 */ 0x7A81,0x464D,
	/* Array index: 0x1572 */ 0x7A83,0x4060,
	/* Array index: 0x1572 */ 0x7A84,0x3A75,
	/* Array index: 0x1572 */ 0x7A88,0x6358,
	/* Array index: 0x1572 */ 0x7A9F,0x3722,
	/* Array index: 0x1572 */ 0x7AA9,0x635D,
	/* Array index: 0x1572 */ 0x7AAA,0x3726,
	/* Array index: 0x1572 */ 0x7AB6,0x6360,
	/* Array index: 0x1572 */ 0x7ABA,0x312E,
	/* Array index: 0x1572 */ 0x7ABF,0x6363,
	/* Array index: 0x1572 */ 0x7AEA,0x4328,
	/* Array index: 0x1572 */ 0x7AED,0x6371,
	/* Array index: 0x1572 */ 0x7AEF,0x433C,
	/* Array index: 0x1572 */ 0x7AF0,0x6372,
	/* Array index: 0x1572 */ 0x7AFF,0x3448,
	/* Array index: 0x1572 */ 0x7B02,0x6373,
	/* Array index: 0x1572 */ 0x7B04,0x6422,
	/* Array index: 0x1572 */ 0x7B06,0x6376,
	/* Array index: 0x1572 */ 0x7B08,0x3568,
	/* Array index: 0x1572 */ 0x7B0A,0x6375,
	/* Array index: 0x1572 */ 0x7B0B,0x6424,
	/* Array index: 0x1572 */ 0x7B0F,0x6374,
	/* Array index: 0x1572 */ 0x7B11,0x3E50,
	/* Array index: 0x1572 */ 0x7B18,0x6378,
	/* Array index: 0x1572 */ 0x7B19,0x6379,
	/* Array index: 0x1572 */ 0x7B1B,0x452B,
	/* Array index: 0x1572 */ 0x7B1E,0x637A,
	/* Array index: 0x1572 */ 0x7B20,0x335E,
	/* Array index: 0x1572 */ 0x7B25,0x3F5A,
	/* Array index: 0x1572 */ 0x7B26,0x4964,
	/* Array index: 0x1572 */ 0x7B28,0x637C,
	/* Array index: 0x1572 */ 0x7B2C,0x4268,
	/* Array index: 0x1572 */ 0x7B33,0x6377,
	/* Array index: 0x1572 */ 0x7B35,0x637B,
	/* Array index: 0x1572 */ 0x7B36,0x637D,
	/* Array index: 0x1572 */ 0x7B39,0x3A7B,
	/* Array index: 0x1572 */ 0x7B5D,0x6438,
	/* Array index: 0x1572 */ 0x7B65,0x6428,
	/* Array index: 0x1572 */ 0x7B67,0x642A,
	/* Array index: 0x1572 */ 0x7B6C,0x642D,
	/* Array index: 0x1572 */ 0x7B6E,0x642E,
	/* Array index: 0x1572 */ 0x7B70,0x642B,
	/* Array index: 0x1572 */ 0x7B71,0x642C,
	/* Array index: 0x1572 */ 0x7B74,0x6429,
	/* Array index: 0x1572 */ 0x7B75,0x6427,
	/* Array index: 0x1572 */ 0x7B7A,0x6421,
	/* Array index: 0x1572 */ 0x7B86,0x4A4F,
	/* Array index: 0x1572 */ 0x7B87,0x3255,
	/* Array index: 0x1572 */ 0x7B8B,0x6435,
	/* Array index: 0x1572 */ 0x7B8D,0x6432,
	/* Array index: 0x1572 */ 0x7B8F,0x6437,
	/* Array index: 0x1572 */ 0x7B92,0x6436,
	/* Array index: 0x1572 */ 0x7BAA,0x433D,
	/* Array index: 0x1572 */ 0x7BAD,0x407D,
	/* Array index: 0x1572 */ 0x7BB1,0x4822,
	/* Array index: 0x1572 */ 0x7BB4,0x643E,
	/* Array index: 0x1572 */ 0x7BB8,0x4824,
	/* Array index: 0x1572 */ 0x7BC0,0x4061,
	/* Array index: 0x1572 */ 0x7BC1,0x643B,
	/* Array index: 0x1572 */ 0x7BC4,0x484F,
	/* Array index: 0x1572 */ 0x7BC6,0x643F,
	/* Array index: 0x1572 */ 0x7BC7,0x4A53,
	/* Array index: 0x1572 */ 0x7BC9,0x435B,
	/* Array index: 0x1572 */ 0x7BCB,0x643A,
	/* Array index: 0x1572 */ 0x7BCC,0x643C,
	/* Array index: 0x1572 */ 0x7BCF,0x643D,
	/* Array index: 0x1572 */ 0x7BDD,0x6440,
	/* Array index: 0x1572 */ 0x7BE0,0x3C44,
	/* Array index: 0x1572 */ 0x7BE9,0x6441,
	/* Array index: 0x1572 */ 0x7BED,0x4F36,
	/* Array index: 0x1572 */ 0x7BF3,0x644A,
	/* Array index: 0x1572 */ 0x7BF6,0x644E,
	/* Array index: 0x1572 */ 0x7BF7,0x644B,
	/* Array index: 0x1572 */ 0x7C00,0x6447,
	/* Array index: 0x1572 */ 0x7C07,0x6448,
	/* Array index: 0x1572 */ 0x7C0D,0x644D,
	/* Array index: 0x1572 */ 0x7C17,0x644C,
	/* Array index: 0x1572 */ 0x7C1F,0x6452,
	/* Array index: 0x1572 */ 0x7C21,0x344A,
	/* Array index: 0x1572 */ 0x7C23,0x644F,
	/* Array index: 0x1572 */ 0x7C27,0x6450,
	/* Array index: 0x1572 */ 0x7C2A,0x6451,
	/* Array index: 0x1572 */ 0x7C2B,0x6454,
	/* Array index: 0x1572 */ 0x7C37,0x6453,
	/* Array index: 0x1572 */ 0x7C38,0x4876,
	/* Array index: 0x1572 */ 0x7C43,0x6457,
	/* Array index: 0x1572 */ 0x7C54,0x6458,
	/* Array index: 0x1572 */ 0x7C56,0x645F,
	/* Array index: 0x1572 */ 0x7C58,0x645C,
	/* Array index: 0x1572 */ 0x7C5F,0x645D,
	/* Array index: 0x1572 */ 0x7C60,0x6446,
	/* Array index: 0x1572 */ 0x7C64,0x645E,
	/* Array index: 0x1572 */ 0x7C65,0x6460,
	/* Array index: 0x1572 */ 0x7C6C,0x6461,
	/* Array index: 0x1572 */ 0x7C73,0x4A46,
	/* Array index: 0x1572 */ 0x7C75,0x6462,
	/* Array index: 0x1572 */ 0x7C7E,0x4C62,
	/* Array index: 0x1572 */ 0x7C89,0x4A34,
	/* Array index: 0x1572 */ 0x7C8B,0x3F68,
	/* Array index: 0x1572 */ 0x7C8D,0x4C30,
	/* Array index: 0x1572 */ 0x7C90,0x6464,
	/* Array index: 0x1572 */ 0x7C92,0x4E33,
	/* Array index: 0x1572 */ 0x7C95,0x4774,
	/* Array index: 0x1572 */ 0x7C97,0x4146,
	/* Array index: 0x1572 */ 0x7C98,0x4734,
	/* Array index: 0x1572 */ 0x7C9B,0x3D4D,
	/* Array index: 0x1572 */ 0x7C9F,0x3040,
	/* Array index: 0x1572 */ 0x7CAB,0x6468,
	/* Array index: 0x1572 */ 0x7CB9,0x646F,
	/* Array index: 0x1572 */ 0x7CBD,0x6470,
	/* Array index: 0x1572 */ 0x7CBE,0x403A,
	/* Array index: 0x1572 */ 0x7CC0,0x6471,
	/* Array index: 0x1572 */ 0x7CC2,0x6473,
	/* Array index: 0x1572 */ 0x7CC5,0x6472,
	/* Array index: 0x1572 */ 0x7CCA,0x3852,
	/* Array index: 0x1572 */ 0x7CCE,0x4138,
	/* Array index: 0x1572 */ 0x7CD2,0x6475,
	/* Array index: 0x1572 */ 0x7CD6,0x457C,
	/* Array index: 0x1572 */ 0x7CD8,0x6474,
	/* Array index: 0x1572 */ 0x7CE7,0x4E48,
	/* Array index: 0x1572 */ 0x7CEF,0x6479,
	/* Array index: 0x1572 */ 0x7CF2,0x647A,
	/* Array index: 0x1572 */ 0x7CF4,0x647B,
	/* Array index: 0x1572 */ 0x7CF6,0x647C,
	/* Array index: 0x1572 */ 0x7CF8,0x3B65,
	/* Array index: 0x1572 */ 0x7CFA,0x647D,
	/* Array index: 0x1572 */ 0x7CFB,0x374F,
	/* Array index: 0x1572 */ 0x7CFE,0x356A,
	/* Array index: 0x1572 */ 0x7D00,0x352A,
	/* Array index: 0x1572 */ 0x7D0A,0x6524,
	/* Array index: 0x1572 */ 0x7D0B,0x4C66,
	/* Array index: 0x1572 */ 0x7D0D,0x473C,
	/* Array index: 0x1572 */ 0x7D10,0x4933,
	/* Array index: 0x1572 */ 0x7D39,0x3E52,
	/* Array index: 0x1572 */ 0x7D3A,0x3A30,
	/* Array index: 0x1572 */ 0x7D3F,0x6529,
	/* Array index: 0x1572 */ 0x7D56,0x652D,
	/* Array index: 0x1572 */ 0x7D5B,0x6536,
	/* Array index: 0x1572 */ 0x7D5E,0x394A,
	/* Array index: 0x1572 */ 0x7D66,0x356B,
	/* Array index: 0x1572 */ 0x7D68,0x6530,
	/* Array index: 0x1572 */ 0x7D6E,0x6531,
	/* Array index: 0x1572 */ 0x7D79,0x3828,
	/* Array index: 0x1572 */ 0x7D7D,0x6538,
	/* Array index: 0x1572 */ 0x7D89,0x6535,
	/* Array index: 0x1572 */ 0x7D8F,0x6537,
	/* Array index: 0x1572 */ 0x7D93,0x6534,
	/* Array index: 0x1572 */ 0x7D9F,0x6546,
	/* Array index: 0x1572 */ 0x7DA2,0x6542,
	/* Array index: 0x1572 */ 0x7DA3,0x653C,
	/* Array index: 0x1572 */ 0x7DB8,0x6545,
	/* Array index: 0x1572 */ 0x7DC7,0x653E,
	/* Array index: 0x1572 */ 0x7DCA,0x365B,
	/* Array index: 0x1572 */ 0x7DCB,0x486C,
	/* Array index: 0x1572 */ 0x7DCF,0x416D,
	/* Array index: 0x1572 */ 0x7DD1,0x4E50,
	/* Array index: 0x1572 */ 0x7DD2,0x3D6F,
	/* Array index: 0x1572 */ 0x7DD5,0x656E,
	/* Array index: 0x1572 */ 0x7DD8,0x6548,
	/* Array index: 0x1572 */ 0x7DE4,0x654A,
	/* Array index: 0x1572 */ 0x7DE8,0x4A54,
	/* Array index: 0x1572 */ 0x7DE9,0x344B,
	/* Array index: 0x1572 */ 0x7DEC,0x4C4B,
	/* Array index: 0x1572 */ 0x7DEF,0x305E,
	/* Array index: 0x1572 */ 0x7DF2,0x654D,
	/* Array index: 0x1572 */ 0x7DF4,0x4E7D,
	/* Array index: 0x1572 */ 0x7DFB,0x654C,
	/* Array index: 0x1572 */ 0x7E01,0x316F,
	/* Array index: 0x1572 */ 0x7E04,0x466C,
	/* Array index: 0x1572 */ 0x7E05,0x654F,
	/* Array index: 0x1572 */ 0x7E12,0x6553,
	/* Array index: 0x1572 */ 0x7E1B,0x477B,
	/* Array index: 0x1572 */ 0x7E26,0x3D44,
	/* Array index: 0x1572 */ 0x7E2B,0x4B25,
	/* Array index: 0x1572 */ 0x7E2E,0x3D4C,
	/* Array index: 0x1572 */ 0x7E31,0x6554,
	/* Array index: 0x1572 */ 0x7E32,0x6560,
	/* Array index: 0x1572 */ 0x7E35,0x655C,
	/* Array index: 0x1572 */ 0x7E41,0x484B,
	/* Array index: 0x1572 */ 0x7E43,0x655E,
	/* Array index: 0x1572 */ 0x7E46,0x6559,
	/* Array index: 0x1572 */ 0x7E4A,0x4121,
	/* Array index: 0x1572 */ 0x7E4B,0x3752,
	/* Array index: 0x1572 */ 0x7E4D,0x3D2B,
	/* Array index: 0x1572 */ 0x7E6D,0x4B7A,
	/* Array index: 0x1572 */ 0x7E70,0x372B,
	/* Array index: 0x1572 */ 0x7E88,0x6572,
	/* Array index: 0x1572 */ 0x7E89,0x6573,
	/* Array index: 0x1572 */ 0x7E9B,0x657B,
	/* Array index: 0x1572 */ 0x7E9C,0x657C,
	/* Array index: 0x1572 */ 0x7F36,0x344C,
	/* Array index: 0x1572 */ 0x7F38,0x657D,
	/* Array index: 0x1572 */ 0x7F3A,0x657E,
	/* Array index: 0x1572 */ 0x7F45,0x6621,
	/* Array index: 0x1572 */ 0x7F58,0x6629,
	/* Array index: 0x1572 */ 0x7F5F,0x662A,
	/* Array index: 0x1572 */ 0x7F60,0x662B,
	/* Array index: 0x1572 */ 0x7F6E,0x4356,
	/* Array index: 0x1572 */ 0x7F70,0x4833,
	/* Array index: 0x1572 */ 0x7F72,0x3D70,
	/* Array index: 0x1572 */ 0x7F94,0x6636,
	/* Array index: 0x1572 */ 0x7F9A,0x6639,
	/* Array index: 0x1572 */ 0x7F9D,0x6638,
	/* Array index: 0x1572 */ 0x7F9E,0x6637,
	/* Array index: 0x1572 */ 0x7FA3,0x663A,
	/* Array index: 0x1572 */ 0x7FA4,0x3732,
	/* Array index: 0x1572 */ 0x7FA8,0x4122,
	/* Array index: 0x1572 */ 0x7FA9,0x3541,
	/* Array index: 0x1572 */ 0x7FAE,0x663E,
	/* Array index: 0x1572 */ 0x7FAF,0x663B,
	/* Array index: 0x1572 */ 0x7FB2,0x663C,
	/* Array index: 0x1572 */ 0x7FB6,0x663F,
	/* Array index: 0x1572 */ 0x7FB8,0x6640,
	/* Array index: 0x1572 */ 0x7FB9,0x663D,
	/* Array index: 0x1572 */ 0x7FBD,0x3129,
	/* Array index: 0x1572 */ 0x7FC1,0x3227,
	/* Array index: 0x1572 */ 0x7FC5,0x6642,
	/* Array index: 0x1572 */ 0x7FC6,0x6643,
	/* Array index: 0x1572 */ 0x7FCA,0x6644,
	/* Array index: 0x1572 */ 0x7FCC,0x4D62,
	/* Array index: 0x1572 */ 0x7FD2,0x3D2C,
	/* Array index: 0x1572 */ 0x7FD4,0x6646,
	/* Array index: 0x1572 */ 0x7FD5,0x6645,
	/* Array index: 0x1572 */ 0x7FE0,0x3F69,
	/* Array index: 0x1572 */ 0x7FE1,0x6647,
	/* Array index: 0x1572 */ 0x7FE6,0x6648,
	/* Array index: 0x1572 */ 0x7FE9,0x6649,
	/* Array index: 0x1572 */ 0x7FEB,0x3465,
	/* Array index: 0x1572 */ 0x7FF0,0x344D,
	/* Array index: 0x1572 */ 0x7FF3,0x664A,
	/* Array index: 0x1572 */ 0x7FF9,0x664B,
	/* Array index: 0x1572 */ 0x7FFB,0x4B5D,
	/* Array index: 0x1572 */ 0x7FFC,0x4D63,
	/* Array index: 0x1572 */ 0x800B,0x664F,
	/* Array index: 0x1572 */ 0x800C,0x3C29,
	/* Array index: 0x1572 */ 0x8010,0x4251,
	/* Array index: 0x1572 */ 0x8012,0x6650,
	/* Array index: 0x1572 */ 0x801C,0x6653,
	/* Array index: 0x1572 */ 0x8021,0x6654,
	/* Array index: 0x1572 */ 0x8028,0x6655,
	/* Array index: 0x1572 */ 0x8033,0x3C2A,
	/* Array index: 0x1572 */ 0x8036,0x4C6D,
	/* Array index: 0x1572 */ 0x803B,0x6657,
	/* Array index: 0x1572 */ 0x803D,0x433F,
	/* Array index: 0x1572 */ 0x803F,0x6656,
	/* Array index: 0x1572 */ 0x8046,0x6659,
	/* Array index: 0x1572 */ 0x804A,0x6658,
	/* Array index: 0x1572 */ 0x8052,0x665A,
	/* Array index: 0x1572 */ 0x8056,0x403B,
	/* Array index: 0x1572 */ 0x8058,0x665B,
	/* Array index: 0x1572 */ 0x805A,0x665C,
	/* Array index: 0x1572 */ 0x8068,0x665F,
	/* Array index: 0x1572 */ 0x8093,0x666B,
	/* Array index: 0x1572 */ 0x8096,0x3E53,
	/* Array index: 0x1572 */ 0x8098,0x492A,
	/* Array index: 0x1572 */ 0x809A,0x666C,
	/* Array index: 0x1572 */ 0x809B,0x666A,
	/* Array index: 0x1572 */ 0x809D,0x344E,
	/* Array index: 0x1572 */ 0x80A1,0x3854,
	/* Array index: 0x1572 */ 0x80A2,0x3B68,
	/* Array index: 0x1572 */ 0x80A5,0x486E,
	/* Array index: 0x1572 */ 0x80BA,0x4759,
	/* Array index: 0x1572 */ 0x80C3,0x305F,
	/* Array index: 0x1572 */ 0x80C4,0x6674,
	/* Array index: 0x1572 */ 0x80C6,0x4340,
	/* Array index: 0x1572 */ 0x80CC,0x4758,
	/* Array index: 0x1572 */ 0x80CE,0x425B,
	/* Array index: 0x1572 */ 0x80D6,0x6676,
	/* Array index: 0x1572 */ 0x80E1,0x3855,
	/* Array index: 0x1572 */ 0x80E4,0x307D,
	/* Array index: 0x1572 */ 0x80E5,0x6671,
	/* Array index: 0x1572 */ 0x80EF,0x6678,
	/* Array index: 0x1572 */ 0x80F1,0x6679,
	/* Array index: 0x1572 */ 0x80F4,0x4639,
	/* Array index: 0x1572 */ 0x80F8,0x363B,
	/* Array index: 0x1572 */ 0x80FC,0x6726,
	/* Array index: 0x1572 */ 0x80FD,0x473D,
	/* Array index: 0x1572 */ 0x8102,0x3B69,
	/* Array index: 0x1572 */ 0x811A,0x3553,
	/* Array index: 0x1572 */ 0x811B,0x667A,
	/* Array index: 0x1572 */ 0x8123,0x667C,
	/* Array index: 0x1572 */ 0x8129,0x667B,
	/* Array index: 0x1572 */ 0x812F,0x667D,
	/* Array index: 0x1572 */ 0x8131,0x4326,
	/* Array index: 0x1572 */ 0x8133,0x473E,
	/* Array index: 0x1572 */ 0x8139,0x4431,
	/* Array index: 0x1572 */ 0x813E,0x6723,
	/* Array index: 0x1572 */ 0x8146,0x6722,
	/* Array index: 0x1572 */ 0x814B,0x667E,
	/* Array index: 0x1572 */ 0x814E,0x3F55,
	/* Array index: 0x1572 */ 0x815F,0x6735,
	/* Array index: 0x1572 */ 0x8165,0x6729,
	/* Array index: 0x1572 */ 0x8166,0x672A,
	/* Array index: 0x1572 */ 0x816B,0x3C70,
	/* Array index: 0x1572 */ 0x816E,0x6728,
	/* Array index: 0x1572 */ 0x8170,0x3978,
	/* Array index: 0x1572 */ 0x8171,0x6727,
	/* Array index: 0x1572 */ 0x8174,0x672B,
	/* Array index: 0x1572 */ 0x8188,0x672D,
	/* Array index: 0x1572 */ 0x818A,0x672E,
	/* Array index: 0x1572 */ 0x818F,0x3951,
	/* Array index: 0x1572 */ 0x8193,0x6736,
	/* Array index: 0x1572 */ 0x8195,0x6732,
	/* Array index: 0x1572 */ 0x819A,0x4966,
	/* Array index: 0x1572 */ 0x819C,0x4B6C,
	/* Array index: 0x1572 */ 0x819D,0x4928,
	/* Array index: 0x1572 */ 0x81A0,0x6731,
	/* Array index: 0x1572 */ 0x81A3,0x6734,
	/* Array index: 0x1572 */ 0x81A4,0x6733,
	/* Array index: 0x1572 */ 0x81A8,0x4B44,
	/* Array index: 0x1572 */ 0x81A9,0x6737,
	/* Array index: 0x1572 */ 0x81B0,0x6738,
	/* Array index: 0x1572 */ 0x81B3,0x4137,
	/* Array index: 0x1572 */ 0x81B5,0x6739,
	/* Array index: 0x1572 */ 0x81B8,0x673B,
	/* Array index: 0x1572 */ 0x81BA,0x673F,
	/* Array index: 0x1572 */ 0x81C6,0x3232,
	/* Array index: 0x1572 */ 0x81C8,0x6745,
	/* Array index: 0x1572 */ 0x81C9,0x6740,
	/* Array index: 0x1572 */ 0x81CD,0x6741,
	/* Array index: 0x1572 */ 0x81D1,0x6742,
	/* Array index: 0x1572 */ 0x81D3,0x4221,
	/* Array index: 0x1572 */ 0x81DF,0x6747,
	/* Array index: 0x1572 */ 0x81E0,0x6748,
	/* Array index: 0x1572 */ 0x81E3,0x3F43,
	/* Array index: 0x1572 */ 0x81E5,0x3269,
	/* Array index: 0x1572 */ 0x81E7,0x6749,
	/* Array index: 0x1572 */ 0x81E8,0x4E57,
	/* Array index: 0x1572 */ 0x81EA,0x3C2B,
	/* Array index: 0x1572 */ 0x81ED,0x3D2D,
	/* Array index: 0x1572 */ 0x81F3,0x3B6A,
	/* Array index: 0x1572 */ 0x81F4,0x4357,
	/* Array index: 0x1572 */ 0x8240,0x6759,
	/* Array index: 0x1572 */ 0x8247,0x447A,
	/* Array index: 0x1572 */ 0x825D,0x675C,
	/* Array index: 0x1572 */ 0x825F,0x675E,
	/* Array index: 0x1572 */ 0x8262,0x6760,
	/* Array index: 0x1572 */ 0x8264,0x675F,
	/* Array index: 0x1572 */ 0x8266,0x344F,
	/* Array index: 0x1572 */ 0x8268,0x6761,
	/* Array index: 0x1572 */ 0x826A,0x6762,
	/* Array index: 0x1572 */ 0x826B,0x6763,
	/* Array index: 0x1572 */ 0x827E,0x6768,
	/* Array index: 0x1572 */ 0x828B,0x3072,
	/* Array index: 0x1572 */ 0x828D,0x6769,
	/* Array index: 0x1572 */ 0x8292,0x676A,
	/* Array index: 0x1572 */ 0x8299,0x4967,
	/* Array index: 0x1572 */ 0x829D,0x3C47,
	/* Array index: 0x1572 */ 0x829F,0x676C,
	/* Array index: 0x1572 */ 0x82A5,0x3329,
	/* Array index: 0x1572 */ 0x82A6,0x3032,
	/* Array index: 0x1572 */ 0x82B8,0x375D,
	/* Array index: 0x1572 */ 0x82B9,0x365C,
	/* Array index: 0x1572 */ 0x82BB,0x676D,
	/* Array index: 0x1572 */ 0x82BD,0x326A,
	/* Array index: 0x1572 */ 0x82C5,0x3423,
	/* Array index: 0x1572 */ 0x82D7,0x4944,
	/* Array index: 0x1572 */ 0x82D9,0x677E,
	/* Array index: 0x1572 */ 0x82EB,0x4651,
	/* Array index: 0x1572 */ 0x82F1,0x3151,
	/* Array index: 0x1572 */ 0x82F3,0x6774,
	/* Array index: 0x1572 */ 0x82F4,0x6773,
	/* Array index: 0x1572 */ 0x8309,0x677D,
	/* Array index: 0x1572 */ 0x830E,0x3754,
	/* Array index: 0x1572 */ 0x831C,0x302B,
	/* Array index: 0x1572 */ 0x8323,0x6834,
	/* Array index: 0x1572 */ 0x8328,0x3071,
	/* Array index: 0x1572 */ 0x832B,0x682B,
	/* Array index: 0x1572 */ 0x832F,0x682A,
	/* Array index: 0x1572 */ 0x8340,0x6826,
	/* Array index: 0x1572 */ 0x8345,0x6829,
	/* Array index: 0x1572 */ 0x8349,0x4170,
	/* Array index: 0x1572 */ 0x834A,0x3755,
	/* Array index: 0x1572 */ 0x834F,0x3141,
	/* Array index: 0x1572 */ 0x8350,0x6828,
	/* Array index: 0x1572 */ 0x8352,0x3953,
	/* Array index: 0x1572 */ 0x8358,0x4171,
	/* Array index: 0x1572 */ 0x8373,0x683A,
	/* Array index: 0x1572 */ 0x8375,0x683B,
	/* Array index: 0x1572 */ 0x8377,0x3259,
	/* Array index: 0x1572 */ 0x837B,0x322E,
	/* Array index: 0x1572 */ 0x837C,0x6838,
	/* Array index: 0x1572 */ 0x8385,0x682E,
	/* Array index: 0x1572 */ 0x8387,0x6836,
	/* Array index: 0x1572 */ 0x8389,0x683D,
	/* Array index: 0x1572 */ 0x838A,0x6837,
	/* Array index: 0x1572 */ 0x838E,0x6835,
	/* Array index: 0x1572 */ 0x8393,0x6776,
	/* Array index: 0x1572 */ 0x8396,0x6833,
	/* Array index: 0x1572 */ 0x839A,0x682F,
	/* Array index: 0x1572 */ 0x83A8,0x683E,
	/* Array index: 0x1572 */ 0x83AA,0x6830,
	/* Array index: 0x1572 */ 0x83AB,0x477C,
	/* Array index: 0x1572 */ 0x83B1,0x4D69,
	/* Array index: 0x1572 */ 0x83B5,0x6839,
	/* Array index: 0x1572 */ 0x83BD,0x684F,
	/* Array index: 0x1572 */ 0x83C1,0x6847,
	/* Array index: 0x1572 */ 0x83C5,0x3F7B,
	/* Array index: 0x1572 */ 0x83CA,0x3546,
	/* Array index: 0x1572 */ 0x83CC,0x365D,
	/* Array index: 0x1572 */ 0x83CE,0x6842,
	/* Array index: 0x1572 */ 0x83D3,0x325B,
	/* Array index: 0x1572 */ 0x83D6,0x3E54,
	/* Array index: 0x1572 */ 0x83D8,0x6845,
	/* Array index: 0x1572 */ 0x83DC,0x3A5A,
	/* Array index: 0x1572 */ 0x83DF,0x4551,
	/* Array index: 0x1572 */ 0x83E0,0x684A,
	/* Array index: 0x1572 */ 0x83E9,0x4A6E,
	/* Array index: 0x1572 */ 0x83EB,0x6841,
	/* Array index: 0x1572 */ 0x83F7,0x6848,
	/* Array index: 0x1572 */ 0x83FB,0x6852,
	/* Array index: 0x1572 */ 0x83FD,0x6843,
	/* Array index: 0x1572 */ 0x8403,0x6844,
	/* Array index: 0x1572 */ 0x8404,0x463A,
	/* Array index: 0x1572 */ 0x8407,0x6849,
	/* Array index: 0x1572 */ 0x8413,0x6840,
	/* Array index: 0x1572 */ 0x8420,0x684E,
	/* Array index: 0x1572 */ 0x8422,0x684D,
	/* Array index: 0x1572 */ 0x8429,0x476B,
	/* Array index: 0x1572 */ 0x842A,0x6854,
	/* Array index: 0x1572 */ 0x842C,0x685F,
	/* Array index: 0x1572 */ 0x8431,0x337E,
	/* Array index: 0x1572 */ 0x8435,0x6862,
	/* Array index: 0x1572 */ 0x8438,0x6850,
	/* Array index: 0x1572 */ 0x843C,0x6855,
	/* Array index: 0x1572 */ 0x843D,0x4D6E,
	/* Array index: 0x1572 */ 0x8446,0x685E,
	/* Array index: 0x1572 */ 0x8449,0x4D55,
	/* Array index: 0x1572 */ 0x844E,0x4E2A,
	/* Array index: 0x1572 */ 0x8457,0x4378,
	/* Array index: 0x1572 */ 0x845B,0x336B,
	/* Array index: 0x1572 */ 0x8466,0x3031,
	/* Array index: 0x1572 */ 0x8475,0x302A,
	/* Array index: 0x1572 */ 0x8477,0x6858,
	/* Array index: 0x1572 */ 0x8479,0x6861,
	/* Array index: 0x1572 */ 0x847A,0x4978,
	/* Array index: 0x1572 */ 0x8482,0x685C,
	/* Array index: 0x1572 */ 0x8484,0x6857,
	/* Array index: 0x1572 */ 0x848B,0x3E55,
	/* Array index: 0x1572 */ 0x8490,0x3D2F,
	/* Array index: 0x1572 */ 0x8494,0x3C2C,
	/* Array index: 0x1572 */ 0x8499,0x4C58,
	/* Array index: 0x1572 */ 0x849C,0x4947,
	/* Array index: 0x1572 */ 0x849F,0x6867,
	/* Array index: 0x1572 */ 0x84A1,0x6870,
	/* Array index: 0x1572 */ 0x84AD,0x685A,
	/* Array index: 0x1572 */ 0x84B2,0x3377,
	/* Array index: 0x1572 */ 0x84BF,0x6866,
	/* Array index: 0x1572 */ 0x84C1,0x686D,
	/* Array index: 0x1572 */ 0x84C4,0x435F,
	/* Array index: 0x1572 */ 0x84C6,0x686E,
	/* Array index: 0x1572 */ 0x84D6,0x686F,
	/* Array index: 0x1572 */ 0x84D9,0x6868,
	/* Array index: 0x1572 */ 0x84DA,0x686B,
	/* Array index: 0x1572 */ 0x84EC,0x4B29,
	/* Array index: 0x1572 */ 0x84EE,0x4F21,
	/* Array index: 0x1572 */ 0x84F4,0x6873,
	/* Array index: 0x1572 */ 0x84FC,0x687A,
	/* Array index: 0x1572 */ 0x84FF,0x6872,
	/* Array index: 0x1572 */ 0x8500,0x3C43,
	/* Array index: 0x1572 */ 0x8506,0x6851,
	/* Array index: 0x1572 */ 0x851F,0x6877,
	/* Array index: 0x1572 */ 0x8521,0x6871,
	/* Array index: 0x1572 */ 0x8526,0x4455,
	/* Array index: 0x1572 */ 0x852C,0x6876,
	/* Array index: 0x1572 */ 0x852D,0x307E,
	/* Array index: 0x1572 */ 0x8535,0x4222,
	/* Array index: 0x1572 */ 0x853D,0x4A43,
	/* Array index: 0x1572 */ 0x8540,0x687B,
	/* Array index: 0x1572 */ 0x8541,0x6921,
	/* Array index: 0x1572 */ 0x8543,0x4859,
	/* Array index: 0x1572 */ 0x854E,0x363E,
	/* Array index: 0x1572 */ 0x8555,0x6924,
	/* Array index: 0x1572 */ 0x8557,0x4979,
	/* Array index: 0x1572 */ 0x8558,0x687D,
	/* Array index: 0x1572 */ 0x855A,0x6856,
	/* Array index: 0x1572 */ 0x8563,0x687C,
	/* Array index: 0x1572 */ 0x856D,0x692B,
	/* Array index: 0x1572 */ 0x8577,0x6931,
	/* Array index: 0x1572 */ 0x857E,0x6932,
	/* Array index: 0x1572 */ 0x8580,0x6925,
	/* Array index: 0x1572 */ 0x8584,0x4776,
	/* Array index: 0x1572 */ 0x8587,0x692F,
	/* Array index: 0x1572 */ 0x8588,0x6927,
	/* Array index: 0x1572 */ 0x858A,0x6929,
	/* Array index: 0x1572 */ 0x8590,0x6933,
	/* Array index: 0x1572 */ 0x8591,0x6928,
	/* Array index: 0x1572 */ 0x8594,0x692C,
	/* Array index: 0x1572 */ 0x8597,0x3172,
	/* Array index: 0x1572 */ 0x8599,0x4665,
	/* Array index: 0x1572 */ 0x859B,0x692D,
	/* Array index: 0x1572 */ 0x859C,0x6930,
	/* Array index: 0x1572 */ 0x85A4,0x6926,
	/* Array index: 0x1572 */ 0x85B9,0x6937,
	/* Array index: 0x1572 */ 0x85BA,0x6935,
	/* Array index: 0x1572 */ 0x85C1,0x4F4E,
	/* Array index: 0x1572 */ 0x85C9,0x6934,
	/* Array index: 0x1572 */ 0x85CD,0x4D75,
	/* Array index: 0x1572 */ 0x85CF,0x6936,
	/* Array index: 0x1572 */ 0x85D0,0x6938,
	/* Array index: 0x1572 */ 0x85D5,0x6939,
	/* Array index: 0x1572 */ 0x85DC,0x693C,
	/* Array index: 0x1572 */ 0x85DD,0x693A,
	/* Array index: 0x1572 */ 0x85E4,0x4623,
	/* Array index: 0x1572 */ 0x85E5,0x693B,
	/* Array index: 0x1572 */ 0x85E9,0x484D,
	/* Array index: 0x1572 */ 0x85EA,0x692E,
	/* Array index: 0x1572 */ 0x85FE,0x6941,
	/* Array index: 0x1572 */ 0x8602,0x6922,
	/* Array index: 0x1572 */ 0x8606,0x6943,
	/* Array index: 0x1572 */ 0x8607,0x4149,
	/* Array index: 0x1572 */ 0x860A,0x693E,
	/* Array index: 0x1572 */ 0x860B,0x6940,
	/* Array index: 0x1572 */ 0x8613,0x693F,
	/* Array index: 0x1572 */ 0x8616,0x5D31,
	/* Array index: 0x1572 */ 0x8617,0x5D22,
	/* Array index: 0x1572 */ 0x861A,0x6945,
	/* Array index: 0x1572 */ 0x8622,0x6944,
	/* Array index: 0x1572 */ 0x862D,0x4D76,
	/* Array index: 0x1572 */ 0x862F,0x623C,
	/* Array index: 0x1572 */ 0x8630,0x6946,
	/* Array index: 0x1572 */ 0x863F,0x6947,
	/* Array index: 0x1572 */ 0x864D,0x6948,
	/* Array index: 0x1572 */ 0x864E,0x3857,
	/* Array index: 0x1572 */ 0x8650,0x3554,
	/* Array index: 0x1572 */ 0x8654,0x694A,
	/* Array index: 0x1572 */ 0x8655,0x515D,
	/* Array index: 0x1572 */ 0x865A,0x3575,
	/* Array index: 0x1572 */ 0x865C,0x4E3A,
	/* Array index: 0x1572 */ 0x865E,0x3673,
	/* Array index: 0x1572 */ 0x865F,0x694B,
	/* Array index: 0x1572 */ 0x8667,0x694C,
	/* Array index: 0x1572 */ 0x866B,0x436E,
	/* Array index: 0x1572 */ 0x8671,0x694D,
	/* Array index: 0x1572 */ 0x8679,0x467A,
	/* Array index: 0x1572 */ 0x867B,0x303A,
	/* Array index: 0x1572 */ 0x8693,0x694E,
	/* Array index: 0x1572 */ 0x8695,0x3B3D,
	/* Array index: 0x1572 */ 0x86A3,0x694F,
	/* Array index: 0x1572 */ 0x86A4,0x4742,
	/* Array index: 0x1572 */ 0x86AF,0x6955,
	/* Array index: 0x1572 */ 0x86B0,0x6958,
	/* Array index: 0x1572 */ 0x86B6,0x6954,
	/* Array index: 0x1572 */ 0x86C4,0x6956,
	/* Array index: 0x1572 */ 0x86C6,0x6957,
	/* Array index: 0x1572 */ 0x86C7,0x3C58,
	/* Array index: 0x1572 */ 0x86C9,0x6959,
	/* Array index: 0x1572 */ 0x86CB,0x4341,
	/* Array index: 0x1572 */ 0x86CD,0x3756,
	/* Array index: 0x1572 */ 0x86CE,0x3342,
	/* Array index: 0x1572 */ 0x86D4,0x695C,
	/* Array index: 0x1572 */ 0x86D9,0x333F,
	/* Array index: 0x1572 */ 0x86DB,0x6961,
	/* Array index: 0x1572 */ 0x86DE,0x695D,
	/* Array index: 0x1572 */ 0x86DF,0x6960,
	/* Array index: 0x1572 */ 0x86E4,0x483A,
	/* Array index: 0x1572 */ 0x86E9,0x695E,
	/* Array index: 0x1572 */ 0x86F8,0x427D,
	/* Array index: 0x1572 */ 0x86F9,0x696C,
	/* Array index: 0x1572 */ 0x86FB,0x6968,
	/* Array index: 0x1572 */ 0x86FE,0x326B,
	/* Array index: 0x1572 */ 0x8700,0x6966,
	/* Array index: 0x1572 */ 0x8702,0x4B2A,
	/* Array index: 0x1572 */ 0x8703,0x6967,
	/* Array index: 0x1572 */ 0x870D,0x696B,
	/* Array index: 0x1572 */ 0x8711,0x6969,
	/* Array index: 0x1572 */ 0x8712,0x6963,
	/* Array index: 0x1572 */ 0x8718,0x4358,
	/* Array index: 0x1572 */ 0x871A,0x6974,
	/* Array index: 0x1572 */ 0x871C,0x4C2A,
	/* Array index: 0x1572 */ 0x8725,0x6972,
	/* Array index: 0x1572 */ 0x8729,0x6973,
	/* Array index: 0x1572 */ 0x8734,0x696E,
	/* Array index: 0x1572 */ 0x8737,0x6970,
	/* Array index: 0x1572 */ 0x873B,0x6971,
	/* Array index: 0x1572 */ 0x873F,0x696F,
	/* Array index: 0x1572 */ 0x8749,0x4066,
	/* Array index: 0x1572 */ 0x874B,0x4F39,
	/* Array index: 0x1572 */ 0x874C,0x6978,
	/* Array index: 0x1572 */ 0x874E,0x6979,
	/* Array index: 0x1572 */ 0x8753,0x6A21,
	/* Array index: 0x1572 */ 0x8755,0x3F2A,
	/* Array index: 0x1572 */ 0x8757,0x697B,
	/* Array index: 0x1572 */ 0x8759,0x697E,
	/* Array index: 0x1572 */ 0x875F,0x6976,
	/* Array index: 0x1572 */ 0x8760,0x6975,
	/* Array index: 0x1572 */ 0x8763,0x6A22,
	/* Array index: 0x1572 */ 0x8766,0x325C,
	/* Array index: 0x1572 */ 0x8768,0x697C,
	/* Array index: 0x1572 */ 0x876A,0x6A23,
	/* Array index: 0x1572 */ 0x876E,0x697D,
	/* Array index: 0x1572 */ 0x8774,0x697A,
	/* Array index: 0x1572 */ 0x8776,0x4433,
	/* Array index: 0x1572 */ 0x8778,0x6977,
	/* Array index: 0x1572 */ 0x877F,0x4768,
	/* Array index: 0x1572 */ 0x8782,0x6A27,
	/* Array index: 0x1572 */ 0x878D,0x4D3B,
	/* Array index: 0x1572 */ 0x879F,0x6A26,
	/* Array index: 0x1572 */ 0x87A2,0x6A25,
	/* Array index: 0x1572 */ 0x87AB,0x6A2E,
	/* Array index: 0x1572 */ 0x87AF,0x6A28,
	/* Array index: 0x1572 */ 0x87B3,0x6A30,
	/* Array index: 0x1572 */ 0x87BA,0x4D66,
	/* Array index: 0x1572 */ 0x87BB,0x6A33,
	/* Array index: 0x1572 */ 0x87BD,0x6A2A,
	/* Array index: 0x1572 */ 0x87C0,0x6A2B,
	/* Array index: 0x1572 */ 0x87C4,0x6A2F,
	/* Array index: 0x1572 */ 0x87C6,0x6A32,
	/* Array index: 0x1572 */ 0x87C7,0x6A31,
	/* Array index: 0x1572 */ 0x87CB,0x6A29,
	/* Array index: 0x1572 */ 0x87D0,0x6A2C,
	/* Array index: 0x1572 */ 0x87D2,0x6A3D,
	/* Array index: 0x1572 */ 0x87E0,0x6A36,
	/* Array index: 0x1572 */ 0x87EF,0x6A34,
	/* Array index: 0x1572 */ 0x87F2,0x6A35,
	/* Array index: 0x1572 */ 0x87F6,0x6A3A,
	/* Array index: 0x1572 */ 0x87F7,0x6A3B,
	/* Array index: 0x1572 */ 0x87F9,0x332A,
	/* Array index: 0x1572 */ 0x87FB,0x3542,
	/* Array index: 0x1572 */ 0x87FE,0x6A39,
	/* Array index: 0x1572 */ 0x8805,0x6A24,
	/* Array index: 0x1572 */ 0x8815,0x6A40,
	/* Array index: 0x1572 */ 0x8816,0x6A3F,
	/* Array index: 0x1572 */ 0x8827,0x6A46,
	/* Array index: 0x1572 */ 0x8831,0x6A43,
	/* Array index: 0x1572 */ 0x8836,0x6A44,
	/* Array index: 0x1572 */ 0x8839,0x6A45,
	/* Array index: 0x1572 */ 0x883B,0x6A47,
	/* Array index: 0x1572 */ 0x8840,0x376C,
	/* Array index: 0x1572 */ 0x8842,0x6A49,
	/* Array index: 0x1572 */ 0x8844,0x6A48,
	/* Array index: 0x1572 */ 0x8846,0x3D30,
	/* Array index: 0x1572 */ 0x884C,0x3954,
	/* Array index: 0x1572 */ 0x884D,0x5E27,
	/* Array index: 0x1572 */ 0x8852,0x6A4A,
	/* Array index: 0x1572 */ 0x8853,0x3D51,
	/* Array index: 0x1572 */ 0x8857,0x3339,
	/* Array index: 0x1572 */ 0x8859,0x6A4B,
	/* Array index: 0x1572 */ 0x885B,0x3152,
	/* Array index: 0x1572 */ 0x8868,0x493D,
	/* Array index: 0x1572 */ 0x886B,0x6A4E,
	/* Array index: 0x1572 */ 0x8870,0x3F6A,
	/* Array index: 0x1572 */ 0x8872,0x6A55,
	/* Array index: 0x1572 */ 0x8875,0x6A52,
	/* Array index: 0x1572 */ 0x8877,0x436F,
	/* Array index: 0x1572 */ 0x8888,0x3736,
	/* Array index: 0x1572 */ 0x888B,0x425E,
	/* Array index: 0x1572 */ 0x888D,0x6A5C,
	/* Array index: 0x1572 */ 0x8892,0x6A58,
	/* Array index: 0x1572 */ 0x8896,0x4235,
	/* Array index: 0x1572 */ 0x8897,0x6A57,
	/* Array index: 0x1572 */ 0x8899,0x6A5A,
	/* Array index: 0x1572 */ 0x889E,0x6A51,
	/* Array index: 0x1572 */ 0x88A2,0x6A5B,
	/* Array index: 0x1572 */ 0x88A4,0x6A5D,
	/* Array index: 0x1572 */ 0x88AB,0x486F,
	/* Array index: 0x1572 */ 0x88AE,0x6A59,
	/* Array index: 0x1572 */ 0x88B0,0x6A5E,
	/* Array index: 0x1572 */ 0x88B1,0x6A60,
	/* Array index: 0x1572 */ 0x88B4,0x3853,
	/* Array index: 0x1572 */ 0x88B5,0x6A54,
	/* Array index: 0x1572 */ 0x88B7,0x3041,
	/* Array index: 0x1572 */ 0x88CF,0x4E22,
	/* Array index: 0x1572 */ 0x88D4,0x6A63,
	/* Array index: 0x1572 */ 0x88D5,0x4D35,
	/* Array index: 0x1572 */ 0x88D8,0x6A64,
	/* Array index: 0x1572 */ 0x88D9,0x6A65,
	/* Array index: 0x1572 */ 0x88DC,0x4A64,
	/* Array index: 0x1572 */ 0x88DD,0x6A66,
	/* Array index: 0x1572 */ 0x88DF,0x3A40,
	/* Array index: 0x1572 */ 0x88E1,0x4E23,
	/* Array index: 0x1572 */ 0x88E8,0x6A6B,
	/* Array index: 0x1572 */ 0x8902,0x6A68,
	/* Array index: 0x1572 */ 0x8904,0x6A6D,
	/* Array index: 0x1572 */ 0x8907,0x4A23,
	/* Array index: 0x1572 */ 0x890A,0x6A6F,
	/* Array index: 0x1572 */ 0x890C,0x6A6E,
	/* Array index: 0x1572 */ 0x8910,0x336C,
	/* Array index: 0x1572 */ 0x8912,0x4B2B,
	/* Array index: 0x1572 */ 0x8913,0x6A70,
	/* Array index: 0x1572 */ 0x891D,0x6A7C,
	/* Array index: 0x1572 */ 0x891E,0x6A72,
	/* Array index: 0x1572 */ 0x8925,0x6A73,
	/* Array index: 0x1572 */ 0x892A,0x6A74,
	/* Array index: 0x1572 */ 0x892B,0x6A75,
	/* Array index: 0x1572 */ 0x8936,0x6A79,
	/* Array index: 0x1572 */ 0x8938,0x6A7A,
	/* Array index: 0x1572 */ 0x893B,0x6A78,
	/* Array index: 0x1572 */ 0x8941,0x6A76,
	/* Array index: 0x1572 */ 0x8943,0x6A71,
	/* Array index: 0x1572 */ 0x8944,0x6A77,
	/* Array index: 0x1572 */ 0x894C,0x6A7B,
	/* Array index: 0x1572 */ 0x894D,0x7037,
	/* Array index: 0x1572 */ 0x8956,0x3228,
	/* Array index: 0x1572 */ 0x8964,0x6B22,
	/* Array index: 0x1572 */ 0x8966,0x6B21,
	/* Array index: 0x1572 */ 0x896A,0x6B24,
	/* Array index: 0x1572 */ 0x896D,0x6B23,
	/* Array index: 0x1572 */ 0x896F,0x6B25,
	/* Array index: 0x1572 */ 0x8972,0x3D31,
	/* Array index: 0x1572 */ 0x8974,0x6B26,
	/* Array index: 0x1572 */ 0x8977,0x6B27,
	/* Array index: 0x1572 */ 0x897E,0x6B28,
	/* Array index: 0x1572 */ 0x897F,0x403E,
	/* Array index: 0x1572 */ 0x8981,0x4D57,
	/* Array index: 0x1572 */ 0x8983,0x6B29,
	/* Array index: 0x1572 */ 0x898F,0x352C,
	/* Array index: 0x1572 */ 0x8993,0x6B2C,
	/* Array index: 0x1572 */ 0x89A1,0x6B2E,
	/* Array index: 0x1572 */ 0x89AF,0x6B32,
	/* Array index: 0x1572 */ 0x89B2,0x6B33,
	/* Array index: 0x1572 */ 0x89B3,0x3451,
	/* Array index: 0x1572 */ 0x89BA,0x6B34,
	/* Array index: 0x1572 */ 0x89BD,0x6B35,
	/* Array index: 0x1572 */ 0x89BF,0x6B36,
	/* Array index: 0x1572 */ 0x89C0,0x6B37,
	/* Array index: 0x1572 */ 0x89D2,0x3351,
	/* Array index: 0x1572 */ 0x89DA,0x6B38,
	/* Array index: 0x1572 */ 0x89DC,0x6B39,
	/* Array index: 0x1572 */ 0x89DD,0x6B3A,
	/* Array index: 0x1572 */ 0x89E3,0x3272,
	/* Array index: 0x1572 */ 0x89E6,0x3F28,
	/* Array index: 0x1572 */ 0x89E7,0x6B3B,
	/* Array index: 0x1572 */ 0x89F4,0x6B3C,
	/* Array index: 0x1572 */ 0x89F8,0x6B3D,
	/* Array index: 0x1572 */ 0x8A00,0x3840,
	/* Array index: 0x1572 */ 0x8A02,0x447B,
	/* Array index: 0x1572 */ 0x8A03,0x6B3E,
	/* Array index: 0x1572 */ 0x8A08,0x3757,
	/* Array index: 0x1572 */ 0x8A0A,0x3F56,
	/* Array index: 0x1572 */ 0x8A0C,0x6B41,
	/* Array index: 0x1572 */ 0x8A0E,0x4624,
	/* Array index: 0x1572 */ 0x8A10,0x6B40,
	/* Array index: 0x1572 */ 0x8A13,0x3731,
	/* Array index: 0x1572 */ 0x8A1B,0x6B42,
	/* Array index: 0x1572 */ 0x8A1D,0x6B43,
	/* Array index: 0x1572 */ 0x8A1F,0x3E59,
	/* Array index: 0x1572 */ 0x8A23,0x376D,
	/* Array index: 0x1572 */ 0x8A25,0x6B44,
	/* Array index: 0x1572 */ 0x8A2A,0x4B2C,
	/* Array index: 0x1572 */ 0x8A2D,0x405F,
	/* Array index: 0x1572 */ 0x8A31,0x3576,
	/* Array index: 0x1572 */ 0x8A33,0x4C75,
	/* Array index: 0x1572 */ 0x8A34,0x414A,
	/* Array index: 0x1572 */ 0x8A36,0x6B45,
	/* Array index: 0x1572 */ 0x8A41,0x6B46,
	/* Array index: 0x1572 */ 0x8A46,0x6B49,
	/* Array index: 0x1572 */ 0x8A48,0x6B4A,
	/* Array index: 0x1572 */ 0x8A5B,0x6B47,
	/* Array index: 0x1572 */ 0x8A5E,0x3B6C,
	/* Array index: 0x1572 */ 0x8A60,0x3153,
	/* Array index: 0x1572 */ 0x8A62,0x6B4E,
	/* Array index: 0x1572 */ 0x8A63,0x3758,
	/* Array index: 0x1572 */ 0x8A66,0x3B6E,
	/* Array index: 0x1572 */ 0x8A7C,0x6B4B,
	/* Array index: 0x1572 */ 0x8A82,0x6B50,
	/* Array index: 0x1572 */ 0x8A84,0x6B51,
	/* Array index: 0x1572 */ 0x8A85,0x6B4F,
	/* Array index: 0x1572 */ 0x8A87,0x3858,
	/* Array index: 0x1572 */ 0x8A89,0x4D40,
	/* Array index: 0x1572 */ 0x8A8C,0x3B6F,
	/* Array index: 0x1572 */ 0x8A8D,0x4727,
	/* Array index: 0x1572 */ 0x8A91,0x6B54,
	/* Array index: 0x1572 */ 0x8A93,0x4040,
	/* Array index: 0x1572 */ 0x8A95,0x4342,
	/* Array index: 0x1572 */ 0x8A98,0x4D36,
	/* Array index: 0x1572 */ 0x8A9A,0x6B57,
	/* Array index: 0x1572 */ 0x8A9E,0x386C,
	/* Array index: 0x1572 */ 0x8AAC,0x4062,
	/* Array index: 0x1572 */ 0x8AAD,0x4649,
	/* Array index: 0x1572 */ 0x8AB0,0x432F,
	/* Array index: 0x1572 */ 0x8AB2,0x325D,
	/* Array index: 0x1572 */ 0x8AB9,0x4870,
	/* Array index: 0x1572 */ 0x8ABC,0x3543,
	/* Array index: 0x1572 */ 0x8ABF,0x4434,
	/* Array index: 0x1572 */ 0x8AC2,0x6B5B,
	/* Array index: 0x1572 */ 0x8AC4,0x6B59,
	/* Array index: 0x1572 */ 0x8AC7,0x434C,
	/* Array index: 0x1572 */ 0x8AD2,0x4E4A,
	/* Array index: 0x1572 */ 0x8AD6,0x4F40,
	/* Array index: 0x1572 */ 0x8AEB,0x6B5D,
	/* Array index: 0x1572 */ 0x8AED,0x4D21,
	/* Array index: 0x1572 */ 0x8AEE,0x3B70,
	/* Array index: 0x1572 */ 0x8AF1,0x6B61,
	/* Array index: 0x1572 */ 0x8AF3,0x6B5E,
	/* Array index: 0x1572 */ 0x8AF7,0x6B65,
	/* Array index: 0x1572 */ 0x8AF8,0x3D74,
	/* Array index: 0x1572 */ 0x8AFA,0x3841,
	/* Array index: 0x1572 */ 0x8B07,0x6B69,
	/* Array index: 0x1572 */ 0x8B0C,0x6B68,
	/* Array index: 0x1572 */ 0x8B0E,0x4666,
	/* Array index: 0x1572 */ 0x8B10,0x6B6D,
	/* Array index: 0x1572 */ 0x8B14,0x6B62,
	/* Array index: 0x1572 */ 0x8B26,0x6B72,
	/* Array index: 0x1572 */ 0x8B28,0x6B75,
	/* Array index: 0x1572 */ 0x8B2B,0x6B73,
	/* Array index: 0x1572 */ 0x8B2C,0x4935,
	/* Array index: 0x1572 */ 0x8B33,0x6B70,
	/* Array index: 0x1572 */ 0x8B39,0x3660,
	/* Array index: 0x1572 */ 0x8B3E,0x6B74,
	/* Array index: 0x1572 */ 0x8B41,0x6B76,
	/* Array index: 0x1572 */ 0x8B49,0x6B7A,
	/* Array index: 0x1572 */ 0x8B4C,0x6B77,
	/* Array index: 0x1572 */ 0x8B4E,0x6B79,
	/* Array index: 0x1572 */ 0x8B4F,0x6B78,
	/* Array index: 0x1572 */ 0x8B56,0x6B7B,
	/* Array index: 0x1572 */ 0x8B5F,0x6C21,
	/* Array index: 0x1572 */ 0x8B66,0x3759,
	/* Array index: 0x1572 */ 0x8B77,0x386E,
	/* Array index: 0x1572 */ 0x8B7D,0x6C25,
	/* Array index: 0x1572 */ 0x8B80,0x6C26,
	/* Array index: 0x1572 */ 0x8B83,0x3B3E,
	/* Array index: 0x1572 */ 0x8B8A,0x5A4E,
	/* Array index: 0x1572 */ 0x8B8C,0x6C27,
	/* Array index: 0x1572 */ 0x8B8E,0x6C28,
	/* Array index: 0x1572 */ 0x8B90,0x3D32,
	/* Array index: 0x1572 */ 0x8B92,0x6C29,
	/* Array index: 0x1572 */ 0x8B93,0x6C2A,
	/* Array index: 0x1572 */ 0x8B96,0x6C2B,
	/* Array index: 0x1572 */ 0x8B99,0x6C2C,
	/* Array index: 0x1572 */ 0x8B9A,0x6C2D,
	/* Array index: 0x1572 */ 0x8C37,0x432B,
	/* Array index: 0x1572 */ 0x8C3A,0x6C2E,
	/* Array index: 0x1572 */ 0x8C3F,0x6C30,
	/* Array index: 0x1572 */ 0x8C41,0x6C2F,
	/* Array index: 0x1572 */ 0x8C46,0x4626,
	/* Array index: 0x1572 */ 0x8C48,0x6C31,
	/* Array index: 0x1572 */ 0x8C4A,0x4B2D,
	/* Array index: 0x1572 */ 0x8C4C,0x6C32,
	/* Array index: 0x1572 */ 0x8C4E,0x6C33,
	/* Array index: 0x1572 */ 0x8C50,0x6C34,
	/* Array index: 0x1572 */ 0x8C55,0x6C35,
	/* Array index: 0x1572 */ 0x8C5A,0x465A,
	/* Array index: 0x1572 */ 0x8C61,0x3E5D,
	/* Array index: 0x1572 */ 0x8C62,0x6C36,
	/* Array index: 0x1572 */ 0x8C82,0x6C3A,
	/* Array index: 0x1572 */ 0x8C85,0x6C3C,
	/* Array index: 0x1572 */ 0x8C94,0x6C40,
	/* Array index: 0x1572 */ 0x8C98,0x6C42,
	/* Array index: 0x1572 */ 0x8CD1,0x4678,
	/* Array index: 0x1572 */ 0x8CD3,0x4950,
	/* Array index: 0x1572 */ 0x8CEA,0x3C41,
	/* Array index: 0x1572 */ 0x8CED,0x4552,
	/* Array index: 0x1572 */ 0x8D64,0x4056,
	/* Array index: 0x1572 */ 0x8D66,0x3C4F,
	/* Array index: 0x1572 */ 0x8D67,0x6C5F,
	/* Array index: 0x1572 */ 0x8D6B,0x3352,
	/* Array index: 0x1572 */ 0x8D6D,0x6C60,
	/* Array index: 0x1572 */ 0x8D77,0x352F,
	/* Array index: 0x1572 */ 0x8D81,0x6C63,
	/* Array index: 0x1572 */ 0x8D85,0x4436,
	/* Array index: 0x1572 */ 0x8D8A,0x315B,
	/* Array index: 0x1572 */ 0x8D99,0x6C64,
	/* Array index: 0x1572 */ 0x8DA3,0x3C71,
	/* Array index: 0x1572 */ 0x8DA8,0x3F76,
	/* Array index: 0x1572 */ 0x8DB3,0x422D,
	/* Array index: 0x1572 */ 0x8DBA,0x6C67,
	/* Array index: 0x1572 */ 0x8DBE,0x6C66,
	/* Array index: 0x1572 */ 0x8DC2,0x6C65,
	/* Array index: 0x1572 */ 0x8DCB,0x6C6D,
	/* Array index: 0x1572 */ 0x8DCC,0x6C6B,
	/* Array index: 0x1572 */ 0x8DCF,0x6C68,
	/* Array index: 0x1572 */ 0x8DD6,0x6C6A,
	/* Array index: 0x1572 */ 0x8DDA,0x6C69,
	/* Array index: 0x1572 */ 0x8DDB,0x6C6C,
	/* Array index: 0x1572 */ 0x8DDD,0x3577,
	/* Array index: 0x1572 */ 0x8DDF,0x6C70,
	/* Array index: 0x1572 */ 0x8DE1,0x4057,
	/* Array index: 0x1572 */ 0x8DE3,0x6C71,
	/* Array index: 0x1572 */ 0x8DE8,0x3859,
	/* Array index: 0x1572 */ 0x8DEA,0x6C6E,
	/* Array index: 0x1572 */ 0x8DEB,0x6C6F,
	/* Array index: 0x1572 */ 0x8DEF,0x4F29,
	/* Array index: 0x1572 */ 0x8DF3,0x4437,
	/* Array index: 0x1572 */ 0x8DF5,0x4129,
	/* Array index: 0x1572 */ 0x8DFC,0x6C72,
	/* Array index: 0x1572 */ 0x8DFF,0x6C75,
	/* Array index: 0x1572 */ 0x8E0F,0x4627,
	/* Array index: 0x1572 */ 0x8E10,0x6C78,
	/* Array index: 0x1572 */ 0x8E2A,0x6D29,
	/* Array index: 0x1572 */ 0x8E30,0x6C7C,
	/* Array index: 0x1572 */ 0x8E34,0x6C7D,
	/* Array index: 0x1572 */ 0x8E35,0x6C7B,
	/* Array index: 0x1572 */ 0x8E42,0x6C7A,
	/* Array index: 0x1572 */ 0x8E44,0x447D,
	/* Array index: 0x1572 */ 0x8E50,0x6D24,
	/* Array index: 0x1572 */ 0x8E55,0x6D2B,
	/* Array index: 0x1572 */ 0x8E59,0x6D26,
	/* Array index: 0x1572 */ 0x8E5F,0x4058,
	/* Array index: 0x1572 */ 0x8E60,0x6D28,
	/* Array index: 0x1572 */ 0x8E63,0x6D2A,
	/* Array index: 0x1572 */ 0x8E64,0x6D27,
	/* Array index: 0x1572 */ 0x8E72,0x6D2D,
	/* Array index: 0x1572 */ 0x8E74,0x3D33,
	/* Array index: 0x1572 */ 0x8E76,0x6D2C,
	/* Array index: 0x1572 */ 0x8E7C,0x6D2E,
	/* Array index: 0x1572 */ 0x8E81,0x6D2F,
	/* Array index: 0x1572 */ 0x8E84,0x6D32,
	/* Array index: 0x1572 */ 0x8E85,0x6D31,
	/* Array index: 0x1572 */ 0x8E87,0x6D30,
	/* Array index: 0x1572 */ 0x8E8A,0x6D34,
	/* Array index: 0x1572 */ 0x8E8B,0x6D33,
	/* Array index: 0x1572 */ 0x8E8D,0x4C76,
	/* Array index: 0x1572 */ 0x8E91,0x6D36,
	/* Array index: 0x1572 */ 0x8E93,0x6D35,
	/* Array index: 0x1572 */ 0x8E94,0x6D37,
	/* Array index: 0x1572 */ 0x8E99,0x6D38,
	/* Array index: 0x1572 */ 0x8EA1,0x6D3A,
	/* Array index: 0x1572 */ 0x8EBE,0x6D3F,
	/* Array index: 0x1572 */ 0x8EC5,0x6D40,
	/* Array index: 0x1572 */ 0x8EC6,0x6D3D,
	/* Array index: 0x1572 */ 0x8ED2,0x382E,
	/* Array index: 0x1572 */ 0x8EDB,0x6D43,
	/* Array index: 0x1572 */ 0x8EDF,0x4670,
	/* Array index: 0x1572 */ 0x8EE2,0x453E,
	/* Array index: 0x1572 */ 0x8EE3,0x6D44,
	/* Array index: 0x1572 */ 0x8EEB,0x6D47,
	/* Array index: 0x1572 */ 0x8EF8,0x3C34,
	/* Array index: 0x1572 */ 0x8F03,0x3353,
	/* Array index: 0x1572 */ 0x8F05,0x6D4A,
	/* Array index: 0x1572 */ 0x8F09,0x3A5C,
	/* Array index: 0x1572 */ 0x8F0A,0x6D49,
	/* Array index: 0x1572 */ 0x8F0C,0x6D52,
	/* Array index: 0x1572 */ 0x8F26,0x6D53,
	/* Array index: 0x1572 */ 0x8F29,0x475A,
	/* Array index: 0x1572 */ 0x8F2A,0x4E58,
	/* Array index: 0x1572 */ 0x8F2F,0x3D34,
	/* Array index: 0x1572 */ 0x8F33,0x6D54,
	/* Array index: 0x1572 */ 0x8F38,0x4D22,
	/* Array index: 0x1572 */ 0x8F39,0x6D56,
	/* Array index: 0x1572 */ 0x8F3B,0x6D55,
	/* Array index: 0x1572 */ 0x8F3E,0x6D59,
	/* Array index: 0x1572 */ 0x8F3F,0x4D41,
	/* Array index: 0x1572 */ 0x8F49,0x6D5B,
	/* Array index: 0x1572 */ 0x8F57,0x6D5E,
	/* Array index: 0x1572 */ 0x8F5C,0x6D5F,
	/* Array index: 0x1572 */ 0x8FA3,0x6D65,
	/* Array index: 0x1572 */ 0x8FA7,0x5221,
	/* Array index: 0x1572 */ 0x8FA8,0x517E,
	/* Array index: 0x1572 */ 0x8FB7,0x6D68,
	/* Array index: 0x1572 */ 0x8FBF,0x4329,
	/* Array index: 0x1572 */ 0x8FC2,0x312A,
	/* Array index: 0x1572 */ 0x8FC4,0x4B78,
	/* Array index: 0x1572 */ 0x8FC5,0x3F57,
	/* Array index: 0x1572 */ 0x8FCE,0x375E,
	/* Array index: 0x1572 */ 0x8FD1,0x3661,
	/* Array index: 0x1572 */ 0x8FD4,0x4A56,
	/* Array index: 0x1572 */ 0x8FDA,0x6D69,
	/* Array index: 0x1572 */ 0x8FE2,0x6D6B,
	/* Array index: 0x1572 */ 0x8FF4,0x6D6F,
	/* Array index: 0x1572 */ 0x8FFD,0x4449,
	/* Array index: 0x1572 */ 0x9000,0x4260,
	/* Array index: 0x1572 */ 0x9001,0x4177,
	/* Array index: 0x1572 */ 0x9003,0x4628,
	/* Array index: 0x1572 */ 0x9005,0x6D70,
	/* Array index: 0x1572 */ 0x9006,0x3555,
	/* Array index: 0x1572 */ 0x9027,0x6D7A,
	/* Array index: 0x1572 */ 0x902E,0x4261,
	/* Array index: 0x1572 */ 0x9031,0x3D35,
	/* Array index: 0x1572 */ 0x9032,0x3F4A,
	/* Array index: 0x1572 */ 0x903C,0x492F,
	/* Array index: 0x1572 */ 0x903E,0x6E27,
	/* Array index: 0x1572 */ 0x9041,0x465B,
	/* Array index: 0x1572 */ 0x9042,0x3F6B,
	/* Array index: 0x1572 */ 0x9045,0x4359,
	/* Array index: 0x1572 */ 0x905C,0x423D,
	/* Array index: 0x1572 */ 0x905E,0x6E2A,
	/* Array index: 0x1572 */ 0x9060,0x3173,
	/* Array index: 0x1572 */ 0x9061,0x414C,
	/* Array index: 0x1572 */ 0x9063,0x382F,
	/* Array index: 0x1572 */ 0x9065,0x4D5A,
	/* Array index: 0x1572 */ 0x9068,0x6E2B,
	/* Array index: 0x1572 */ 0x9069,0x452C,
	/* Array index: 0x1572 */ 0x9072,0x6E2F,
	/* Array index: 0x1572 */ 0x9087,0x6D6E,
	/* Array index: 0x1572 */ 0x9089,0x6E35,
	/* Array index: 0x1572 */ 0x908A,0x6E34,
	/* Array index: 0x1572 */ 0x908F,0x6E36,
	/* Array index: 0x1572 */ 0x9091,0x4D38,
	/* Array index: 0x1572 */ 0x90A3,0x4661,
	/* Array index: 0x1572 */ 0x90A6,0x4B2E,
	/* Array index: 0x1572 */ 0x90A8,0x6E37,
	/* Array index: 0x1572 */ 0x90AA,0x3C59,
	/* Array index: 0x1572 */ 0x90AF,0x6E38,
	/* Array index: 0x1572 */ 0x90B1,0x6E39,
	/* Array index: 0x1572 */ 0x90B5,0x6E3A,
	/* Array index: 0x1572 */ 0x90B8,0x4521,
	/* Array index: 0x1572 */ 0x90C1,0x306A,
	/* Array index: 0x1572 */ 0x90CA,0x3959,
	/* Array index: 0x1572 */ 0x90CE,0x4F3A,
	/* Array index: 0x1572 */ 0x90DB,0x6E3E,
	/* Array index: 0x1572 */ 0x90E1,0x3734,
	/* Array index: 0x1572 */ 0x90E2,0x6E3B,
	/* Array index: 0x1572 */ 0x90E4,0x6E3C,
	/* Array index: 0x1572 */ 0x90E8,0x4974,
	/* Array index: 0x1572 */ 0x90ED,0x3354,
	/* Array index: 0x1572 */ 0x90F5,0x4D39,
	/* Array index: 0x1572 */ 0x90F7,0x363F,
	/* Array index: 0x1572 */ 0x90FD,0x4554,
	/* Array index: 0x1572 */ 0x9102,0x6E3F,
	/* Array index: 0x1572 */ 0x9112,0x6E40,
	/* Array index: 0x1572 */ 0x9119,0x6E41,
	/* Array index: 0x1572 */ 0x912D,0x4522,
	/* Array index: 0x1572 */ 0x9130,0x6E43,
	/* Array index: 0x1572 */ 0x9132,0x6E42,
	/* Array index: 0x1572 */ 0x9152,0x3C72,
	/* Array index: 0x1572 */ 0x9154,0x3F6C,
	/* Array index: 0x1572 */ 0x9156,0x6E45,
	/* Array index: 0x1572 */ 0x9158,0x6E46,
	/* Array index: 0x1572 */ 0x9162,0x3F5D,
	/* Array index: 0x1572 */ 0x9163,0x6E47,
	/* Array index: 0x1572 */ 0x9165,0x6E48,
	/* Array index: 0x1572 */ 0x9169,0x6E49,
	/* Array index: 0x1572 */ 0x916A,0x4D6F,
	/* Array index: 0x1572 */ 0x916C,0x3D37,
	/* Array index: 0x1572 */ 0x9172,0x6E4B,
	/* Array index: 0x1572 */ 0x9173,0x6E4A,
	/* Array index: 0x1572 */ 0x9175,0x395A,
	/* Array index: 0x1572 */ 0x9177,0x3973,
	/* Array index: 0x1572 */ 0x9178,0x3B40,
	/* Array index: 0x1572 */ 0x9182,0x6E4E,
	/* Array index: 0x1572 */ 0x9187,0x3D66,
	/* Array index: 0x1572 */ 0x9189,0x6E4D,
	/* Array index: 0x1572 */ 0x918B,0x6E4C,
	/* Array index: 0x1572 */ 0x918D,0x4269,
	/* Array index: 0x1572 */ 0x9190,0x386F,
	/* Array index: 0x1572 */ 0x9192,0x4043,
	/* Array index: 0x1572 */ 0x9197,0x4830,
	/* Array index: 0x1572 */ 0x919C,0x3D39,
	/* Array index: 0x1572 */ 0x91A2,0x6E4F,
	/* Array index: 0x1572 */ 0x91A4,0x3E5F,
	/* Array index: 0x1572 */ 0x91AA,0x6E52,
	/* Array index: 0x1572 */ 0x91AB,0x6E50,
	/* Array index: 0x1572 */ 0x91AF,0x6E51,
	/* Array index: 0x1572 */ 0x91B4,0x6E54,
	/* Array index: 0x1572 */ 0x91B5,0x6E53,
	/* Array index: 0x1572 */ 0x91B8,0x3E7A,
	/* Array index: 0x1572 */ 0x91BA,0x6E55,
	/* Array index: 0x1572 */ 0x91C0,0x6E56,
	/* Array index: 0x1572 */ 0x91C1,0x6E57,
	/* Array index: 0x1572 */ 0x91D6,0x6E5B,
	/* Array index: 0x1572 */ 0x91D8,0x4523,
	/* Array index: 0x1572 */ 0x91F5,0x6E60,
	/* Array index: 0x1572 */ 0x91F6,0x6E61,
	/* Array index: 0x1572 */ 0x91FC,0x6E5F,
	/* Array index: 0x1572 */ 0x91FF,0x6E63,
	/* Array index: 0x1572 */ 0x920D,0x465F,
	/* Array index: 0x1572 */ 0x920E,0x3343,
	/* Array index: 0x1572 */ 0x9211,0x6E67,
	/* Array index: 0x1572 */ 0x9214,0x6E64,
	/* Array index: 0x1572 */ 0x9215,0x6E66,
	/* Array index: 0x1572 */ 0x921E,0x6E62,
	/* Array index: 0x1572 */ 0x9229,0x6F4F,
	/* Array index: 0x1572 */ 0x922C,0x6E65,
	/* Array index: 0x1572 */ 0x9234,0x4E6B,
	/* Array index: 0x1572 */ 0x9237,0x385A,
	/* Array index: 0x1572 */ 0x923F,0x6E6F,
	/* Array index: 0x1572 */ 0x9244,0x4534,
	/* Array index: 0x1572 */ 0x9245,0x6E6A,
	/* Array index: 0x1572 */ 0x9248,0x6E6D,
	/* Array index: 0x1572 */ 0x9249,0x6E6B,
	/* Array index: 0x1572 */ 0x924B,0x6E70,
	/* Array index: 0x1572 */ 0x9250,0x6E71,
	/* Array index: 0x1572 */ 0x9257,0x6E69,
	/* Array index: 0x1572 */ 0x925A,0x6E76,
	/* Array index: 0x1572 */ 0x925B,0x3174,
	/* Array index: 0x1572 */ 0x925E,0x6E68,
	/* Array index: 0x1572 */ 0x9262,0x482D,
	/* Array index: 0x1572 */ 0x9264,0x6E6C,
	/* Array index: 0x1572 */ 0x9266,0x3E60,
	/* Array index: 0x1572 */ 0x9271,0x395B,
	/* Array index: 0x1572 */ 0x927E,0x4B48,
	/* Array index: 0x1572 */ 0x9280,0x3664,
	/* Array index: 0x1572 */ 0x9283,0x3D46,
	/* Array index: 0x1572 */ 0x9285,0x463C,
	/* Array index: 0x1572 */ 0x9291,0x412D,
	/* Array index: 0x1572 */ 0x9293,0x6E74,
	/* Array index: 0x1572 */ 0x9295,0x6E6E,
	/* Array index: 0x1572 */ 0x9296,0x6E73,
	/* Array index: 0x1572 */ 0x92AD,0x412C,
	/* Array index: 0x1572 */ 0x92B7,0x6E79,
	/* Array index: 0x1572 */ 0x92B9,0x6E78,
	/* Array index: 0x1572 */ 0x92CF,0x6E77,
	/* Array index: 0x1572 */ 0x92D2,0x4B2F,
	/* Array index: 0x1572 */ 0x92E4,0x3D7B,
	/* Array index: 0x1572 */ 0x92E9,0x6E7A,
	/* Array index: 0x1572 */ 0x92EA,0x4A5F,
	/* Array index: 0x1572 */ 0x92ED,0x3154,
	/* Array index: 0x1572 */ 0x92F2,0x4946,
	/* Array index: 0x1572 */ 0x92F3,0x4372,
	/* Array index: 0x1572 */ 0x92F8,0x3578,
	/* Array index: 0x1572 */ 0x92FA,0x6E7C,
	/* Array index: 0x1572 */ 0x92FC,0x395D,
	/* Array index: 0x1572 */ 0x9306,0x3B2C,
	/* Array index: 0x1572 */ 0x930F,0x6E7B,
	/* Array index: 0x1572 */ 0x9310,0x3F6D,
	/* Array index: 0x1572 */ 0x9320,0x3E7B,
	/* Array index: 0x1572 */ 0x9322,0x6F22,
	/* Array index: 0x1572 */ 0x9323,0x6F24,
	/* Array index: 0x1572 */ 0x9326,0x3653,
	/* Array index: 0x1572 */ 0x9328,0x4945,
	/* Array index: 0x1572 */ 0x9332,0x4F3F,
	/* Array index: 0x1572 */ 0x9335,0x6F26,
	/* Array index: 0x1572 */ 0x933A,0x6F25,
	/* Array index: 0x1572 */ 0x933B,0x6F27,
	/* Array index: 0x1572 */ 0x9344,0x6E7D,
	/* Array index: 0x1572 */ 0x934B,0x4669,
	/* Array index: 0x1572 */ 0x934D,0x4555,
	/* Array index: 0x1572 */ 0x9354,0x4457,
	/* Array index: 0x1572 */ 0x9356,0x6F2C,
	/* Array index: 0x1572 */ 0x935B,0x4343,
	/* Array index: 0x1572 */ 0x935C,0x6F28,
	/* Array index: 0x1572 */ 0x9360,0x6F29,
	/* Array index: 0x1572 */ 0x936C,0x372D,
	/* Array index: 0x1572 */ 0x936E,0x6F2B,
	/* Array index: 0x1572 */ 0x9375,0x3830,
	/* Array index: 0x1572 */ 0x937C,0x6F2A,
	/* Array index: 0x1572 */ 0x937E,0x3E61,
	/* Array index: 0x1572 */ 0x938C,0x3379,
	/* Array index: 0x1572 */ 0x9394,0x6F30,
	/* Array index: 0x1572 */ 0x9396,0x3A3F,
	/* Array index: 0x1572 */ 0x9397,0x4179,
	/* Array index: 0x1572 */ 0x939A,0x444A,
	/* Array index: 0x1572 */ 0x93A7,0x333B,
	/* Array index: 0x1572 */ 0x93B9,0x6F31,
	/* Array index: 0x1572 */ 0x93C3,0x6F37,
	/* Array index: 0x1572 */ 0x93C8,0x6F3A,
	/* Array index: 0x1572 */ 0x93D0,0x6F39,
	/* Array index: 0x1572 */ 0x93D1,0x452D,
	/* Array index: 0x1572 */ 0x93DD,0x6F38,
	/* Array index: 0x1572 */ 0x93E1,0x3640,
	/* Array index: 0x1572 */ 0x93E4,0x6F3B,
	/* Array index: 0x1572 */ 0x93E5,0x6F35,
	/* Array index: 0x1572 */ 0x93E8,0x6F34,
	/* Array index: 0x1572 */ 0x9403,0x6F3F,
	/* Array index: 0x1572 */ 0x9407,0x6F40,
	/* Array index: 0x1572 */ 0x9410,0x6F41,
	/* Array index: 0x1572 */ 0x9413,0x6F3E,
	/* Array index: 0x1572 */ 0x9414,0x6F3D,
	/* Array index: 0x1572 */ 0x9421,0x6F45,
	/* Array index: 0x1572 */ 0x942B,0x6F43,
	/* Array index: 0x1572 */ 0x9435,0x6F44,
	/* Array index: 0x1572 */ 0x9436,0x6F42,
	/* Array index: 0x1572 */ 0x9438,0x4278,
	/* Array index: 0x1572 */ 0x943A,0x6F46,
	/* Array index: 0x1572 */ 0x9441,0x6F47,
	/* Array index: 0x1572 */ 0x9444,0x6F49,
	/* Array index: 0x1572 */ 0x945A,0x6F54,
	/* Array index: 0x1572 */ 0x945B,0x6F4A,
	/* Array index: 0x1572 */ 0x945E,0x6F4D,
	/* Array index: 0x1572 */ 0x9460,0x6F4B,
	/* Array index: 0x1572 */ 0x9462,0x6F4C,
	/* Array index: 0x1572 */ 0x946A,0x6F4E,
	/* Array index: 0x1572 */ 0x9470,0x6F50,
	/* Array index: 0x1572 */ 0x9475,0x6F51,
	/* Array index: 0x1572 */ 0x9477,0x6F52,
	/* Array index: 0x1572 */ 0x9577,0x4439,
	/* Array index: 0x1572 */ 0x9580,0x4C67,
	/* Array index: 0x1572 */ 0x9582,0x6F59,
	/* Array index: 0x1572 */ 0x9583,0x412E,
	/* Array index: 0x1572 */ 0x958F,0x313C,
	/* Array index: 0x1572 */ 0x9591,0x3457,
	/* Array index: 0x1572 */ 0x9593,0x3456,
	/* Array index: 0x1572 */ 0x9594,0x6F5C,
	/* Array index: 0x1572 */ 0x9596,0x6F5D,
	/* Array index: 0x1572 */ 0x9598,0x6F5E,
	/* Array index: 0x1572 */ 0x9599,0x6F5F,
	/* Array index: 0x1572 */ 0x95AD,0x6F63,
	/* Array index: 0x1572 */ 0x95B2,0x315C,
	/* Array index: 0x1572 */ 0x95B9,0x6F66,
	/* Array index: 0x1572 */ 0x95BB,0x6F65,
	/* Array index: 0x1572 */ 0x95BC,0x6F64,
	/* Array index: 0x1572 */ 0x95BE,0x6F67,
	/* Array index: 0x1572 */ 0x95C3,0x6F6A,
	/* Array index: 0x1572 */ 0x95C7,0x3047,
	/* Array index: 0x1572 */ 0x95CA,0x6F68,
	/* Array index: 0x1572 */ 0x95CC,0x6F6C,
	/* Array index: 0x1572 */ 0x95CD,0x6F6B,
	/* Array index: 0x1572 */ 0x95DC,0x6F70,
	/* Array index: 0x1572 */ 0x95E1,0x6F71,
	/* Array index: 0x1572 */ 0x95E2,0x6F73,
	/* Array index: 0x1572 */ 0x95E5,0x6F72,
	/* Array index: 0x1572 */ 0x961C,0x496C,
	/* Array index: 0x1572 */ 0x9621,0x6F74,
	/* Array index: 0x1572 */ 0x9628,0x6F75,
	/* Array index: 0x1572 */ 0x962A,0x3A65,
	/* Array index: 0x1572 */ 0x962E,0x6F76,
	/* Array index: 0x1572 */ 0x962F,0x6F77,
	/* Array index: 0x1572 */ 0x9632,0x4B49,
	/* Array index: 0x1572 */ 0x963B,0x414B,
	/* Array index: 0x1572 */ 0x963F,0x3024,
	/* Array index: 0x1572 */ 0x9640,0x424B,
	/* Array index: 0x1572 */ 0x9642,0x6F78,
	/* Array index: 0x1572 */ 0x9644,0x496D,
	/* Array index: 0x1572 */ 0x966A,0x4766,
	/* Array index: 0x1572 */ 0x966C,0x7025,
	/* Array index: 0x1572 */ 0x9670,0x3122,
	/* Array index: 0x1572 */ 0x967D,0x4D5B,
	/* Array index: 0x1572 */ 0x9685,0x3679,
	/* Array index: 0x1572 */ 0x9686,0x4E34,
	/* Array index: 0x1572 */ 0x9688,0x3728,
	/* Array index: 0x1572 */ 0x96A0,0x3123,
	/* Array index: 0x1572 */ 0x96A3,0x4E59,
	/* Array index: 0x1572 */ 0x96A7,0x702B,
	/* Array index: 0x1572 */ 0x96A8,0x6E2E,
	/* Array index: 0x1572 */ 0x96AA,0x702A,
	/* Array index: 0x1572 */ 0x96D1,0x3B28,
	/* Array index: 0x1572 */ 0x96D5,0x703A,
	/* Array index: 0x1572 */ 0x96D6,0x6A2D,
	/* Array index: 0x1572 */ 0x96D9,0x5256,
	/* Array index: 0x1572 */ 0x96DB,0x3F77,
	/* Array index: 0x1572 */ 0x96DC,0x7038,
	/* Array index: 0x1572 */ 0x96E2,0x4E25,
	/* Array index: 0x1572 */ 0x96E3,0x4671,
	/* Array index: 0x1572 */ 0x96E8,0x312B,
	/* Array index: 0x1572 */ 0x96EA,0x4063,
	/* Array index: 0x1572 */ 0x96EB,0x3C36,
	/* Array index: 0x1572 */ 0x96F0,0x4A37,
	/* Array index: 0x1572 */ 0x96F2,0x3140,
	/* Array index: 0x1572 */ 0x96F6,0x4E6D,
	/* Array index: 0x1572 */ 0x96F7,0x4D6B,
	/* Array index: 0x1572 */ 0x96F9,0x703B,
	/* Array index: 0x1572 */ 0x96FB,0x4545,
	/* Array index: 0x1572 */ 0x9700,0x3C7B,
	/* Array index: 0x1572 */ 0x9716,0x7043,
	/* Array index: 0x1572 */ 0x9719,0x7044,
	/* Array index: 0x1572 */ 0x971C,0x417A,
	/* Array index: 0x1572 */ 0x971E,0x3262,
	/* Array index: 0x1572 */ 0x9724,0x7045,
	/* Array index: 0x1572 */ 0x9727,0x4C38,
	/* Array index: 0x1572 */ 0x972A,0x7046,
	/* Array index: 0x1572 */ 0x9730,0x7047,
	/* Array index: 0x1572 */ 0x9732,0x4F2A,
	/* Array index: 0x1572 */ 0x9738,0x5B31,
	/* Array index: 0x1572 */ 0x9739,0x7048,
	/* Array index: 0x1572 */ 0x973D,0x7049,
	/* Array index: 0x1572 */ 0x973E,0x704A,
	/* Array index: 0x1572 */ 0x9742,0x704E,
	/* Array index: 0x1572 */ 0x9744,0x704B,
	/* Array index: 0x1572 */ 0x9746,0x704C,
	/* Array index: 0x1572 */ 0x9748,0x704D,
	/* Array index: 0x1572 */ 0x9749,0x704F,
	/* Array index: 0x1572 */ 0x9752,0x4044,
	/* Array index: 0x1572 */ 0x9756,0x4C77,
	/* Array index: 0x1572 */ 0x9759,0x4045,
	/* Array index: 0x1572 */ 0x975C,0x7050,
	/* Array index: 0x1572 */ 0x9771,0x7057,
	/* Array index: 0x1572 */ 0x9774,0x3724,
	/* Array index: 0x1572 */ 0x9779,0x7058,
	/* Array index: 0x1572 */ 0x977A,0x705C,
	/* Array index: 0x1572 */ 0x977C,0x705A,
	/* Array index: 0x1572 */ 0x9781,0x705B,
	/* Array index: 0x1572 */ 0x978B,0x705E,
	/* Array index: 0x1572 */ 0x978D,0x3048,
	/* Array index: 0x1572 */ 0x978F,0x705F,
	/* Array index: 0x1572 */ 0x9790,0x7060,
	/* Array index: 0x1572 */ 0x9798,0x3E64,
	/* Array index: 0x1572 */ 0x979C,0x7061,
	/* Array index: 0x1572 */ 0x97A0,0x3547,
	/* Array index: 0x1572 */ 0x97A3,0x7064,
	/* Array index: 0x1572 */ 0x97A6,0x7063,
	/* Array index: 0x1572 */ 0x97A8,0x7062,
	/* Array index: 0x1572 */ 0x97AB,0x6B71,
	/* Array index: 0x1572 */ 0x97AD,0x4A5C,
	/* Array index: 0x1572 */ 0x97B3,0x7065,
	/* Array index: 0x1572 */ 0x97B4,0x7066,
	/* Array index: 0x1572 */ 0x97C3,0x7067,
	/* Array index: 0x1572 */ 0x97C6,0x7068,
	/* Array index: 0x1572 */ 0x97C8,0x7069,
	/* Array index: 0x1572 */ 0x97CB,0x706A,
	/* Array index: 0x1572 */ 0x97D3,0x345A,
	/* Array index: 0x1572 */ 0x97DC,0x706B,
	/* Array index: 0x1572 */ 0x97ED,0x706C,
	/* Array index: 0x1572 */ 0x97EE,0x4723,
	/* Array index: 0x1572 */ 0x97FB,0x3124,
	/* Array index: 0x1572 */ 0x980C,0x7073,
	/* Array index: 0x1572 */ 0x9817,0x3F7C,
	/* Array index: 0x1572 */ 0x9818,0x4E4E,
	/* Array index: 0x1572 */ 0x981A,0x375B,
	/* Array index: 0x1572 */ 0x9821,0x7076,
	/* Array index: 0x1572 */ 0x9824,0x7075,
	/* Array index: 0x1572 */ 0x982C,0x4B4B,
	/* Array index: 0x1572 */ 0x982D,0x462C,
	/* Array index: 0x1572 */ 0x9834,0x3150,
	/* Array index: 0x1572 */ 0x9846,0x7079,
	/* Array index: 0x1572 */ 0x9854,0x3469,
	/* Array index: 0x1572 */ 0x9855,0x3832,
	/* Array index: 0x1572 */ 0x9858,0x346A,
	/* Array index: 0x1572 */ 0x985B,0x453F,
	/* Array index: 0x1572 */ 0x985E,0x4E60,
	/* Array index: 0x1572 */ 0x9867,0x385C,
	/* Array index: 0x1572 */ 0x986B,0x707C,
	/* Array index: 0x1572 */ 0x98A8,0x4977,
	/* Array index: 0x1572 */ 0x98AA,0x7124,
	/* Array index: 0x1572 */ 0x98AF,0x7125,
	/* Array index: 0x1572 */ 0x98B1,0x7126,
	/* Array index: 0x1572 */ 0x98B6,0x7127,
	/* Array index: 0x1572 */ 0x98C3,0x7129,
	/* Array index: 0x1572 */ 0x98C4,0x7128,
	/* Array index: 0x1572 */ 0x98C6,0x712A,
	/* Array index: 0x1572 */ 0x98DB,0x4874,
	/* Array index: 0x1572 */ 0x98DC,0x664C,
	/* Array index: 0x1572 */ 0x98DF,0x3F29,
	/* Array index: 0x1572 */ 0x98E2,0x3532,
	/* Array index: 0x1572 */ 0x98E9,0x712B,
	/* Array index: 0x1572 */ 0x98F2,0x307B,
	/* Array index: 0x1572 */ 0x98F4,0x303B,
	/* Array index: 0x1572 */ 0x9903,0x712D,
	/* Array index: 0x1572 */ 0x9905,0x4C5F,
	/* Array index: 0x1572 */ 0x9909,0x712E,
	/* Array index: 0x1572 */ 0x990A,0x4D5C,
	/* Array index: 0x1572 */ 0x990C,0x3142,
	/* Array index: 0x1572 */ 0x9918,0x7131,
	/* Array index: 0x1572 */ 0x9924,0x7135,
	/* Array index: 0x1572 */ 0x9928,0x345B,
	/* Array index: 0x1572 */ 0x992C,0x7137,
	/* Array index: 0x1572 */ 0x992E,0x7138,
	/* Array index: 0x1572 */ 0x993D,0x7139,
	/* Array index: 0x1572 */ 0x993E,0x713A,
	/* Array index: 0x1572 */ 0x9942,0x713B,
	/* Array index: 0x1572 */ 0x9945,0x713D,
	/* Array index: 0x1572 */ 0x9949,0x713C,
	/* Array index: 0x1572 */ 0x994B,0x713F,
	/* Array index: 0x1572 */ 0x994C,0x7142,
	/* Array index: 0x1572 */ 0x9955,0x7143,
	/* Array index: 0x1572 */ 0x9957,0x3642,
	/* Array index: 0x1572 */ 0x99A5,0x7146,
	/* Array index: 0x1572 */ 0x99A8,0x333E,
	/* Array index: 0x1572 */ 0x99B3,0x435A,
	/* Array index: 0x1572 */ 0x99B4,0x466B,
	/* Array index: 0x1572 */ 0x99BC,0x7149,
	/* Array index: 0x1572 */ 0x99C1,0x477D,
	/* Array index: 0x1572 */ 0x99D5,0x326F,
	/* Array index: 0x1572 */ 0x99D8,0x714D,
	/* Array index: 0x1572 */ 0x99DB,0x714B,
	/* Array index: 0x1572 */ 0x99DD,0x714C,
	/* Array index: 0x1572 */ 0x99DF,0x714A,
	/* Array index: 0x1572 */ 0x99E2,0x7158,
	/* Array index: 0x1572 */ 0x99ED,0x714F,
	/* Array index: 0x1572 */ 0x99EE,0x7150,
	/* Array index: 0x1572 */ 0x99F1,0x7151,
	/* Array index: 0x1572 */ 0x99F2,0x7152,
	/* Array index: 0x1572 */ 0x99F8,0x7154,
	/* Array index: 0x1572 */ 0x99FB,0x7153,
	/* Array index: 0x1572 */ 0x99FF,0x3D59,
	/* Array index: 0x1572 */ 0x9A01,0x7155,
	/* Array index: 0x1572 */ 0x9A05,0x7157,
	/* Array index: 0x1572 */ 0x9A0E,0x3533,
	/* Array index: 0x1572 */ 0x9A0F,0x7156,
	/* Array index: 0x1572 */ 0x9A12,0x417B,
	/* Array index: 0x1572 */ 0x9A13,0x3833,
	/* Array index: 0x1572 */ 0x9A19,0x7159,
	/* Array index: 0x1572 */ 0x9A28,0x424D,
	/* Array index: 0x1572 */ 0x9A2B,0x715A,
	/* Array index: 0x1572 */ 0x9A30,0x462D,
	/* Array index: 0x1572 */ 0x9A37,0x715B,
	/* Array index: 0x1572 */ 0x9A3E,0x7160,
	/* Array index: 0x1572 */ 0x9A40,0x715E,
	/* Array index: 0x1572 */ 0x9A42,0x715D,
	/* Array index: 0x1572 */ 0x9A43,0x715F,
	/* Array index: 0x1572 */ 0x9A45,0x715C,
	/* Array index: 0x1572 */ 0x9A4D,0x7162,
	/* Array index: 0x1572 */ 0x9A55,0x7161,
	/* Array index: 0x1572 */ 0x9A57,0x7164,
	/* Array index: 0x1572 */ 0x9A5A,0x3643,
	/* Array index: 0x1572 */ 0x9A5B,0x7163,
	/* Array index: 0x1572 */ 0x9A5F,0x7165,
	/* Array index: 0x1572 */ 0x9A62,0x7166,
	/* Array index: 0x1572 */ 0x9A64,0x7168,
	/* Array index: 0x1572 */ 0x9A65,0x7167,
	/* Array index: 0x1572 */ 0x9AA8,0x397C,
	/* Array index: 0x1572 */ 0x9AAD,0x716C,
	/* Array index: 0x1572 */ 0x9AB0,0x716D,
	/* Array index: 0x1572 */ 0x9AB8,0x333C,
	/* Array index: 0x1572 */ 0x9ABC,0x716E,
	/* Array index: 0x1572 */ 0x9AC0,0x716F,
	/* Array index: 0x1572 */ 0x9AC4,0x3F71,
	/* Array index: 0x1572 */ 0x9ACF,0x7170,
	/* Array index: 0x1572 */ 0x9AD1,0x7171,
	/* Array index: 0x1572 */ 0x9AD3,0x7172,
	/* Array index: 0x1572 */ 0x9AD4,0x7173,
	/* Array index: 0x1572 */ 0x9AD8,0x3962,
	/* Array index: 0x1572 */ 0x9ADE,0x7174,
	/* Array index: 0x1572 */ 0x9ADF,0x7175,
	/* Array index: 0x1572 */ 0x9AE2,0x7176,
	/* Array index: 0x1572 */ 0x9AE3,0x7177,
	/* Array index: 0x1572 */ 0x9AE6,0x7178,
	/* Array index: 0x1572 */ 0x9AF4,0x717C,
	/* Array index: 0x1572 */ 0x9AF7,0x717E,
	/* Array index: 0x1572 */ 0x9AFB,0x7221,
	/* Array index: 0x1572 */ 0x9B06,0x7222,
	/* Array index: 0x1572 */ 0x9B18,0x7223,
	/* Array index: 0x1572 */ 0x9B1A,0x7224,
	/* Array index: 0x1572 */ 0x9B1F,0x7225,
	/* Array index: 0x1572 */ 0x9B22,0x7226,
	/* Array index: 0x1572 */ 0x9B23,0x7227,
	/* Array index: 0x1572 */ 0x9B3B,0x6478,
	/* Array index: 0x1572 */ 0x9B3C,0x3534,
	/* Array index: 0x1572 */ 0x9B54,0x4B62,
	/* Array index: 0x1572 */ 0x9B58,0x7236,
	/* Array index: 0x1572 */ 0x9B5A,0x357B,
	/* Array index: 0x1572 */ 0x9B6F,0x4F25,
	/* Array index: 0x1572 */ 0x9B74,0x7237,
	/* Array index: 0x1572 */ 0x9B83,0x7239,
	/* Array index: 0x1572 */ 0x9B8E,0x303E,
	/* Array index: 0x1572 */ 0x9B9F,0x723D,
	/* Array index: 0x1572 */ 0x9BA0,0x723E,
	/* Array index: 0x1572 */ 0x9BA8,0x723F,
	/* Array index: 0x1572 */ 0x9BB4,0x7240,
	/* Array index: 0x1572 */ 0x9BB9,0x7243,
	/* Array index: 0x1572 */ 0x9BC0,0x7241,
	/* Array index: 0x1572 */ 0x9BC6,0x7244,
	/* Array index: 0x1572 */ 0x9BC9,0x3871,
	/* Array index: 0x1572 */ 0x9BCA,0x7242,
	/* Array index: 0x1572 */ 0x9BCF,0x7245,
	/* Array index: 0x1572 */ 0x9BD1,0x7246,
	/* Array index: 0x1572 */ 0x9BD2,0x7247,
	/* Array index: 0x1572 */ 0x9BD4,0x724B,
	/* Array index: 0x1572 */ 0x9BD6,0x3B2A,
	/* Array index: 0x1572 */ 0x9BDB,0x4264,
	/* Array index: 0x1572 */ 0x9BE8,0x375F,
	/* Array index: 0x1572 */ 0x9BF5,0x3033,
	/* Array index: 0x1572 */ 0x9C04,0x725A,
	/* Array index: 0x1572 */ 0x9C1B,0x725C,
	/* Array index: 0x1572 */ 0x9C21,0x725F,
	/* Array index: 0x1572 */ 0x9C24,0x725E,
	/* Array index: 0x1572 */ 0x9C25,0x725D,
	/* Array index: 0x1572 */ 0x9C3E,0x7264,
	/* Array index: 0x1572 */ 0x9C52,0x4B70,
	/* Array index: 0x1572 */ 0x9C57,0x4E5A,
	/* Array index: 0x1572 */ 0x9C5A,0x7265,
	/* Array index: 0x1572 */ 0x9C60,0x7266,
	/* Array index: 0x1572 */ 0x9C67,0x7267,
	/* Array index: 0x1572 */ 0x9C76,0x7268,
	/* Array index: 0x1572 */ 0x9C78,0x7269,
	/* Array index: 0x1572 */ 0x9CE5,0x443B,
	/* Array index: 0x1572 */ 0x9CE7,0x726A,
	/* Array index: 0x1572 */ 0x9CE9,0x4837,
	/* Array index: 0x1572 */ 0x9CEB,0x726F,
	/* Array index: 0x1572 */ 0x9CEC,0x726B,
	/* Array index: 0x1572 */ 0x9CF0,0x726C,
	/* Array index: 0x1572 */ 0x9CF3,0x4B31,
	/* Array index: 0x1572 */ 0x9CF4,0x4C44,
	/* Array index: 0x1572 */ 0x9CF6,0x4650,
	/* Array index: 0x1572 */ 0x9D03,0x7270,
	/* Array index: 0x1572 */ 0x9D0E,0x322A,
	/* Array index: 0x1572 */ 0x9D12,0x7279,
	/* Array index: 0x1572 */ 0x9D15,0x7278,
	/* Array index: 0x1572 */ 0x9D1B,0x3175,
	/* Array index: 0x1572 */ 0x9D1F,0x7276,
	/* Array index: 0x1572 */ 0x9D23,0x7275,
	/* Array index: 0x1572 */ 0x9D26,0x7273,
	/* Array index: 0x1572 */ 0x9D3B,0x3963,
	/* Array index: 0x1572 */ 0x9D3E,0x727C,
	/* Array index: 0x1572 */ 0x9D3F,0x727B,
	/* Array index: 0x1572 */ 0x9D41,0x727A,
	/* Array index: 0x1572 */ 0x9D44,0x7277,
	/* Array index: 0x1572 */ 0x9D46,0x727D,
	/* Array index: 0x1572 */ 0x9D48,0x727E,
	/* Array index: 0x1572 */ 0x9D50,0x7325,
	/* Array index: 0x1572 */ 0x9D51,0x7324,
	/* Array index: 0x1572 */ 0x9D59,0x7326,
	/* Array index: 0x1572 */ 0x9D64,0x7323,
	/* Array index: 0x1572 */ 0x9D6C,0x4B32,
	/* Array index: 0x1572 */ 0x9D6F,0x732B,
	/* Array index: 0x1572 */ 0x9D72,0x7327,
	/* Array index: 0x1572 */ 0x9D7A,0x732C,
	/* Array index: 0x1572 */ 0x9D87,0x7329,
	/* Array index: 0x1572 */ 0x9D89,0x7328,
	/* Array index: 0x1572 */ 0x9D8F,0x375C,
	/* Array index: 0x1572 */ 0x9D9A,0x732D,
	/* Array index: 0x1572 */ 0x9DA4,0x732E,
	/* Array index: 0x1572 */ 0x9DA9,0x732F,
	/* Array index: 0x1572 */ 0x9DAB,0x732A,
	/* Array index: 0x1572 */ 0x9DAF,0x7274,
	/* Array index: 0x1572 */ 0x9DB2,0x7330,
	/* Array index: 0x1572 */ 0x9DB4,0x4461,
	/* Array index: 0x1572 */ 0x9DB8,0x7334,
	/* Array index: 0x1572 */ 0x9DBA,0x7335,
	/* Array index: 0x1572 */ 0x9DBB,0x7333,
	/* Array index: 0x1572 */ 0x9DC1,0x7332,
	/* Array index: 0x1572 */ 0x9DC2,0x7338,
	/* Array index: 0x1572 */ 0x9DC4,0x7331,
	/* Array index: 0x1572 */ 0x9DC6,0x7336,
	/* Array index: 0x1572 */ 0x9DCF,0x7337,
	/* Array index: 0x1572 */ 0x9DD3,0x733A,
	/* Array index: 0x1572 */ 0x9DD9,0x7339,
	/* Array index: 0x1572 */ 0x9DE6,0x733C,
	/* Array index: 0x1572 */ 0x9DED,0x733D,
	/* Array index: 0x1572 */ 0x9DEF,0x733E,
	/* Array index: 0x1572 */ 0x9DF2,0x4F49,
	/* Array index: 0x1572 */ 0x9DFD,0x733F,
	/* Array index: 0x1572 */ 0x9E1A,0x7340,
	/* Array index: 0x1572 */ 0x9E1B,0x7341,
	/* Array index: 0x1572 */ 0x9E1E,0x7342,
	/* Array index: 0x1572 */ 0x9E75,0x7343,
	/* Array index: 0x1572 */ 0x9E78,0x3834,
	/* Array index: 0x1572 */ 0x9E79,0x7344,
	/* Array index: 0x1572 */ 0x9E7D,0x7345,
	/* Array index: 0x1572 */ 0x9E7F,0x3C2F,
	/* Array index: 0x1572 */ 0x9E81,0x7346,
	/* Array index: 0x1572 */ 0x9E88,0x7347,
	/* Array index: 0x1572 */ 0x9E8B,0x7348,
	/* Array index: 0x1572 */ 0x9E8C,0x7349,
	/* Array index: 0x1572 */ 0x9E9D,0x734D,
	/* Array index: 0x1572 */ 0x9E9F,0x4E5B,
	/* Array index: 0x1572 */ 0x9EA5,0x734E,
	/* Array index: 0x1572 */ 0x9EA6,0x477E,
	/* Array index: 0x1572 */ 0x9EA9,0x734F,
	/* Array index: 0x1572 */ 0x9EAA,0x7351,
	/* Array index: 0x1572 */ 0x9EAD,0x7352,
	/* Array index: 0x1572 */ 0x9EC4,0x322B,
	/* Array index: 0x1572 */ 0x9EE5,0x735D,
	/* Array index: 0x1572 */ 0x9EE8,0x735E,
	/* Array index: 0x1572 */ 0x9EEF,0x735F,
	/* Array index: 0x1572 */ 0x9EF4,0x7360,
	/* Array index: 0x1572 */ 0x9EF6,0x7361,
	/* Array index: 0x1572 */ 0x9EF7,0x7362,
	/* Array index: 0x1572 */ 0x9F07,0x7367,
	/* Array index: 0x1572 */ 0x9F08,0x7368,
	/* Array index: 0x1572 */ 0x9F0E,0x4524,
	/* Array index: 0x1572 */ 0x9F13,0x385D,
	/* Array index: 0x1572 */ 0x9F15,0x736A,
	/* Array index: 0x1572 */ 0x9F20,0x414D,
	/* Array index: 0x1572 */ 0x9F21,0x736B,
	/* Array index: 0x1572 */ 0x9F2C,0x736C,
	/* Array index: 0x1572 */ 0x9F3B,0x4921,
	/* Array index: 0x1572 */ 0x9F3E,0x736D,
	/* Array index: 0x1572 */ 0x9F4A,0x736E,
	/* Array index: 0x1572 */ 0x9F4B,0x6337,
	/* Array index: 0x1572 */ 0x9F4E,0x6C5A,
	/* Array index: 0x1572 */ 0x9F4F,0x706D,
	/* Array index: 0x1572 */ 0x9F52,0x736F,
	/* Array index: 0x1572 */ 0x9F54,0x7370,
	/* Array index: 0x1572 */ 0x9F6A,0x7378,
	/* Array index: 0x1572 */ 0x9F6C,0x7377,
	/* Array index: 0x1572 */ 0x9F72,0x737A,
	/* Array index: 0x1572 */ 0x9F76,0x737B,
	/* Array index: 0x1572 */ 0x9F77,0x7379,
	/* Array index: 0x1572 */ 0x9F8D,0x4E36,
	/* Array index: 0x1572 */ 0x9F95,0x737C,
	/* Array index: 0x1572 */ 0x9F9C,0x737D,
	/* Array index: 0x1572 */ 0x9F9D,0x6354,
	/* Array index: 0x1572 */ 0x9FA0,0x737E,
	/* Array index: 0x1572 */ 0xFFE3,0x2131,
	/* Array index: 0x1572 */ 0xFFE5,0x216F,
};

#endif /* ICONV_FROM_UCS_CCS_JIS_X0208_1990 && defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * jis_x0208_1990 CCS description table.
 * ======================================================================
 */
const iconv_ccs_t
_iconv_ccs_jis_x0208_1990 =
{
	TABLE_VERSION_1, /* Table version */
	ICONV_CCS_JIS_X0208_1990, /* CCS name */
	TABLE_16BIT, /* Table bits */
#if defined (ICONV_FROM_UCS_CCS_JIS_X0208_1990) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SIZE_OPTIMIZED,
	(__uint16_t *)&from_ucs_size_jis_x0208_1990, /* UCS -> jis_x0208_1990 table size-optimized table */
#elif defined (ICONV_FROM_UCS_CCS_JIS_X0208_1990) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)&from_ucs_speed_jis_x0208_1990, /* UCS -> jis_x0208_1990 table speed-optimized table */
#else
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)NULL,
#endif
#if defined (ICONV_TO_UCS_CCS_JIS_X0208_1990) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SIZE_OPTIMIZED,
	(__uint16_t *)&to_ucs_size_jis_x0208_1990 /* jis_x0208_1990 -> UCS table speed-optimized table */
#elif defined (ICONV_TO_UCS_CCS_JIS_X0208_1990) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)&to_ucs_speed_jis_x0208_1990 /* jis_x0208_1990 -> UCS table speed-optimized table */
#else
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)NULL,
#endif
};

#endif /* ICONV_TO_UCS_CCS_JIS_X0208_1990) || ... */

