/*
 * This file was generated automatically - don't edit it.
 * File contains iconv CCS tables for cns11643_plane14 encoding.
 */

#include "ccsbi.h"

#if defined (ICONV_TO_UCS_CCS_CNS11643_PLANE14) \
 || defined (ICONV_FROM_UCS_CCS_CNS11643_PLANE14)

#include <_ansi.h>
#include <sys/types.h>
#include <sys/param.h>
#include "ccs.h"
#include "ccsnames.h"

/*
 * 16-bit cns11643_plane14 -> UCS speed-optimized table (36864 bytes).
 * ======================================================================
 */
#if defined (ICONV_TO_UCS_CCS_CNS11643_PLANE14) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
to_ucs_speed_cns11643_plane14[] =
{
	/* Heading Block */
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,0x0100,0x0200,0x0300,0x0400,0x0500,0x0600,0x0700,
	0x0800,0x0900,0x0A00,0x0B00,0x0C00,0x0D00,0x0E00,0x0F00,
	0x1000,0x1100,0x1200,0x1300,0x1400,0x1500,0x1600,0x1700,
	0x1800,0x1900,0x1A00,0x1B00,0x1C00,0x1D00,0x1E00,0x1F00,
	0x2000,0x2100,0x2200,0x2300,0x2400,0x2500,0x2600,0x2700,
	0x2800,0x2900,0x2A00,0x2B00,0x2C00,0x2D00,0x2E00,0x2F00,
	0x3000,0x3100,0x3200,0x3300,0x3400,0x3500,0x3600,0x3700,
	0x3800,0x3900,0x3A00,0x3B00,0x3C00,0x3D00,0x3E00,0x3F00,
	0x4000,0x4100,0x4200,0x4300,0x4400,0x4500,0x4600,0x4700,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	/* Block 34, Array index 0x0100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E28,0x4E36,0x4E3F,0x4E85,0x4E05,0x4E04,0x5182,
	0x5196,0x5338,0x5369,0x53B6,0x4E2A,0x4E87,0x4E49,0x51E2,
	0x4E46,0x4E8F,0x4EBC,0x4EBE,0x5166,0x51E3,0x5204,0x529C,
	INVALC,0x5902,0x590A,0x5B80,0x5DDB,0x5E7A,0x5E7F,0x5EF4,
	0x5F50,0x5F51,0x5F61,0x961D,INVALC,0x4E63,0x4E62,0x4EA3,
	0x5185,0x4EC5,0x4ECF,0x4ECE,0x4ECC,0x5184,0x5186,INVALC,
	INVALC,0x51E4,0x5205,0x529E,0x529D,0x52FD,0x5300,0x533A,
	INVALC,0x5346,0x535D,0x5386,0x53B7,INVALC,0x53CC,INVALC,
	0x53CE,0x5721,INVALC,0x5E00,0x5F0C,0x6237,0x6238,0x6534,
	0x6535,0x65E0,INVALC,0x738D,0x4E97,0x4EE0,INVALC,INVALC,
	0x4EE7,INVALC,0x4EE6,INVALC,INVALC,INVALC,INVALC,0x56D8,
	0x518B,0x518C,0x5199,0x51E5,INVALC,0x520B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 35, Array index 0x0200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5304,0x5303,0x5307,INVALC,0x531E,0x535F,
	0x536D,0x5389,0x53BA,0x53D0,INVALC,0x53F6,0x53F7,0x53F9,
	INVALC,0x53F4,INVALC,INVALC,0x5724,0x5904,0x5918,0x5932,
	0x5930,0x5934,INVALC,0x5975,INVALC,0x5B82,0x5BF9,0x5C14,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E81,
	0x5E83,0x5F0D,0x5F52,INVALC,0x5FCA,0x5FC7,0x6239,INVALC,
	0x624F,0x65E7,0x672F,0x6B7A,0x6C39,INVALC,INVALC,0x6C37,
	0x6C44,0x6C45,0x738C,0x7592,0x7676,0x9093,0x9092,INVALC,
	INVALC,0x4E21,0x4E20,0x4E22,0x4E68,0x4E89,0x4E98,0x4EF9,
	0x4EEF,INVALC,INVALC,0x4EF8,0x4F06,0x4F03,0x4EFC,0x4EEE,
	0x4F16,INVALC,0x4F28,0x4F1C,0x4F07,0x4F1A,0x4EFA,0x4F17,
	0x514A,INVALC,0x5172,INVALC,0x51B4,0x51B3,0x51B2,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 36, Array index 0x0300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x51E8,INVALC,0x5214,0x520F,0x5215,0x5218,
	0x52A8,INVALC,0x534B,0x534F,INVALC,0x5350,INVALC,0x538B,
	INVALC,0x53BE,INVALC,0x53D2,0x5416,0x53FF,INVALC,0x5400,
	INVALC,0x5405,0x5413,0x5415,INVALC,INVALC,0x56E3,0x5735,
	0x5736,0x5731,0x5732,0x58EE,0x5905,0x4E54,INVALC,0x5936,
	INVALC,INVALC,INVALC,0x597A,INVALC,0x5986,INVALC,INVALC,
	0x5B86,0x5F53,0x5C18,INVALC,0x5C3D,0x5C78,INVALC,INVALC,
	INVALC,INVALC,0x5C80,INVALC,0x5E08,INVALC,INVALC,INVALC,
	INVALC,0x5EF5,0x5F0E,INVALC,INVALC,INVALC,0x5FD3,0x5FDA,
	INVALC,0x5FDB,INVALC,0x620F,0x625D,0x625F,0x6267,0x6257,
	0x9F50,INVALC,0x65EB,0x65EA,INVALC,0x6737,INVALC,0x6732,
	0x6736,0x6B22,0x6BCE,INVALC,0x6C58,0x6C51,0x6C77,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 37, Array index 0x0400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6C3C,INVALC,0x6C5A,INVALC,0x6C53,0x706F,0x7072,
	0x706E,INVALC,INVALC,0x7073,0x72B1,0x72B2,INVALC,0x738F,
	INVALC,INVALC,INVALC,0x793C,INVALC,0x808D,0x808E,INVALC,
	0x827B,INVALC,0x8D71,0x8FB9,0x9096,0x909A,INVALC,0x4E24,
	0x4E71,INVALC,0x4E9C,0x4F45,0x4F4A,0x4F39,0x4F37,INVALC,
	0x4F32,0x4F42,INVALC,0x4F44,0x4F4B,INVALC,0x4F40,0x4F35,
	0x4F31,0x5151,INVALC,0x5150,0x514E,INVALC,INVALC,0x519D,
	INVALC,0x51B5,0x51B8,0x51EC,0x5223,0x5227,0x5226,0x521F,
	0x522B,0x5220,0x52B4,0x52B3,INVALC,0x5325,0x533B,0x5374,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x544D,INVALC,INVALC,
	0x543A,INVALC,INVALC,0x5444,0x544C,0x5423,0x541A,0x5432,
	0x544B,0x5421,INVALC,0x5434,0x5449,0x5450,0x5422,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 38, Array index 0x0500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x543F,0x5451,0x545A,0x542F,INVALC,0x56E9,0x56F2,
	0x56F3,0x56EF,0x56ED,0x56EC,0x56E6,0x5748,INVALC,0x5744,
	0x573F,0x573C,0x5753,0x5756,INVALC,0x575F,0x5743,0x5758,
	0x5757,INVALC,INVALC,INVALC,0x5746,INVALC,0x573D,INVALC,
	0x5742,0x5754,0x5755,0x58F1,0x58F2,0x58F0,0x590B,0x9EA6,
	0x56F1,0x593D,INVALC,0x5994,0x598C,INVALC,0x599C,INVALC,
	INVALC,0x599F,INVALC,0x599B,INVALC,0x5989,0x599A,INVALC,
	0x6588,INVALC,0x5B8D,INVALC,0x5BFE,0x5BFF,0x5BFD,0x5C2B,
	INVALC,0x5C84,0x5C8E,0x5C9C,INVALC,INVALC,0x5C85,0x5DF5,
	0x5E09,INVALC,INVALC,0x5E0B,INVALC,0x5E92,0x5E90,0x5F03,
	INVALC,0x5F1E,0x5F63,INVALC,0x5FE7,0x5FFE,0x5FE6,0x5FDC,
	0x5FCE,INVALC,0x5FFC,0x5FDF,0x5FEC,0x5FF6,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 39, Array index 0x0600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5FF2,0x5FF0,0x5FF9,INVALC,0x6213,INVALC,INVALC,
	0x623B,0x623C,0x6282,INVALC,INVALC,INVALC,0x6278,0x628B,
	INVALC,0x629E,0x62A5,0x629B,0x629C,0x6299,0x628D,0x6285,
	0x629D,0x6275,INVALC,INVALC,INVALC,0x65F6,INVALC,INVALC,
	INVALC,0x66F5,0x675B,INVALC,0x6754,0x6752,INVALC,0x6758,
	0x6744,0x674A,0x6761,INVALC,0x6C7F,0x6C91,0x6C9E,INVALC,
	0x6C6E,0x6C7C,0x6C9F,0x6C75,INVALC,0x6C56,0x6CA2,0x6C79,
	INVALC,0x6CA1,INVALC,0x6CAA,0x6CA0,INVALC,0x7079,0x7077,
	0x707E,INVALC,0x7075,0x707B,0x7264,INVALC,0x72BB,0x72BC,
	0x72C7,0x72B9,0x72BE,0x72B6,INVALC,INVALC,0x7398,INVALC,
	INVALC,INVALC,INVALC,0x7593,0x7680,INVALC,0x7683,0x76C0,
	0x76C1,INVALC,INVALC,0x77F4,0x77F5,INVALC,0x7ACC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 40, Array index 0x0700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7ACD,0x7CFA,0x809F,0x8091,0x8097,0x8094,INVALC,
	0x8286,0x828C,INVALC,0x8295,INVALC,0x866C,INVALC,0x8FB5,
	0x8FBE,0x8FC7,INVALC,0x8FC1,0x90A9,0x90A4,INVALC,INVALC,
	INVALC,0x90A8,0x9627,0x9626,0x962B,0x9633,0x9634,0x9629,
	0x4E3D,INVALC,0x4E9D,0x4F93,0x4F8A,INVALC,INVALC,0x4F6D,
	0x4F8E,0x4FA0,0x4FA2,0x4FA1,0x4F9F,0x4FA3,INVALC,0x4F72,
	INVALC,0x4F8C,0x5156,INVALC,INVALC,0x5190,INVALC,INVALC,
	INVALC,0x51ED,0x51FE,0x522F,INVALC,0x523C,0x5234,0x5239,
	0x52B9,0x52B5,0x52BF,0x5355,INVALC,0x5376,0x537A,0x5393,
	INVALC,0x53C1,0x53C2,0x53D5,0x5485,INVALC,0x545F,0x5493,
	0x5489,0x5479,0x9EFE,0x548F,0x5469,0x546D,INVALC,0x5494,
	0x546A,0x548A,INVALC,0x56FD,0x56FB,0x56F8,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 41, Array index 0x0800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x56FC,0x56F6,0x5765,0x5781,0x5763,0x5767,INVALC,
	0x576E,0x5778,0x577F,INVALC,INVALC,0x58F3,0x594B,0x594C,
	INVALC,INVALC,INVALC,0x59AD,INVALC,0x59C4,INVALC,0x59C2,
	0x59B0,INVALC,INVALC,INVALC,INVALC,0x59BF,INVALC,0x59C9,
	0x59B8,0x59AC,INVALC,INVALC,INVALC,0x59B7,0x59D7,INVALC,
	0x5B60,INVALC,0x5B96,0x5B9E,0x5B94,0x5B9F,0x5B9D,INVALC,
	0x5C00,0x5C19,INVALC,INVALC,0x5C49,0x5C4A,INVALC,0x5CBB,
	0x5CC1,INVALC,INVALC,INVALC,0x5CB9,0x5C9E,0x5CB4,0x5CBA,
	0x5DF6,0x5E13,0x5E12,0x5E77,INVALC,0x5E98,INVALC,0x5E99,
	0x5E9D,0x5EF8,INVALC,0x5EF9,INVALC,0x5F06,0x5F21,INVALC,
	0x5F25,0x5F55,INVALC,INVALC,INVALC,0x5F84,0x5F83,0x6030,
	0x6007,INVALC,0x6036,INVALC,INVALC,INVALC,0x5FE9,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 42, Array index 0x0900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x603D,0x6008,INVALC,INVALC,0x62BA,0x62B2,INVALC,
	0x62B7,0x62E4,0x62A7,INVALC,INVALC,INVALC,0x62D5,0x62E1,
	0x62DD,0x62A6,0x62C1,0x62C5,0x62C0,0x62DF,0x62E0,0x62DE,
	INVALC,0x6589,INVALC,0x65A6,0x65BA,INVALC,0x65FF,INVALC,
	0x6617,0x6618,0x6601,0x65FE,INVALC,0x670C,INVALC,0x676B,
	0x6796,0x6782,0x678A,INVALC,0x67A3,INVALC,0x67A2,0x678F,
	INVALC,0x67F9,0x6780,0x6B26,0x6B27,0x6B68,0x6B69,INVALC,
	0x6B81,0x6BB4,0x6BD1,INVALC,INVALC,0x6C1C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6C97,0x6C6C,0x6CDF,INVALC,0x6CEA,
	INVALC,0x6CE4,0x6CD8,0x6CB2,0x6CCE,0x6CC8,INVALC,0x708B,
	0x7088,0x7090,0x708F,INVALC,0x7087,0x7089,0x708D,0x7081,
	INVALC,0x708C,INVALC,INVALC,0x7240,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 43, Array index 0x0A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7265,0x7266,0x7268,INVALC,INVALC,0x72CD,0x72D3,
	0x72DB,INVALC,0x72CF,0x73A7,0x73A3,0x739E,INVALC,0x73AF,
	INVALC,INVALC,0x73AA,0x739C,INVALC,0x7542,0x7544,0x753B,
	0x7541,INVALC,0x759B,0x759E,INVALC,0x79C4,0x79C3,0x79C6,
	INVALC,INVALC,0x79C7,INVALC,0x79CA,INVALC,INVALC,0x7ACF,
	0x7C76,0x7C74,0x7CFF,0x7CFC,INVALC,INVALC,0x7F59,0x80A8,
	INVALC,INVALC,0x80B0,INVALC,0x80B3,INVALC,0x80A4,0x80B6,
	0x80A7,0x80AC,INVALC,0x80A6,0x5367,0x820E,0x82C4,0x833E,
	0x829C,INVALC,INVALC,INVALC,INVALC,INVALC,0x82AA,INVALC,
	0x82C9,INVALC,INVALC,0x82A6,0x82B2,INVALC,INVALC,INVALC,
	0x8FCC,0x8FD9,0x8FCA,0x8FD8,0x8FCF,0x90B7,INVALC,0x90AD,
	0x90B9,0x9637,INVALC,0x9641,0x963E,0x96B6,0x9751,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 44, Array index 0x0B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9763,0x4E57,0x4E79,0x4EB2,0x4EB0,0x4EAF,0x4EB1,
	0x4FD2,0x4FD5,INVALC,0x4FBE,0x4FB8,0x4FB0,0x4FB1,0x4FC8,
	INVALC,INVALC,0x4FC6,0x4FCC,0x4FE5,0x4FE3,0x4FB4,0x516A,
	INVALC,0x519F,INVALC,0x51C1,INVALC,0x51C2,0x51C3,0x5245,
	0x5248,INVALC,INVALC,0x524F,INVALC,INVALC,0x52C5,0x52CA,
	0x52C4,0x5327,0x5358,0x537D,INVALC,0x53DD,0x53DC,0x53DA,
	0x53D9,0x54B9,INVALC,0x54D0,0x54B4,0x54CA,INVALC,0x54A3,
	0x54DA,0x54A4,INVALC,0x54B2,0x549E,0x549F,0x54B5,INVALC,
	INVALC,0x54CD,INVALC,0x54CC,INVALC,0x5700,0x57AC,0x5791,
	0x578E,0x578D,0x5792,0x57A1,0x5790,0x57A6,0x57A8,INVALC,
	0x579C,0x5796,0x57A7,INVALC,INVALC,INVALC,INVALC,0x58F5,
	INVALC,0x5909,0x5908,INVALC,0x5952,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 45, Array index 0x0C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x59DF,INVALC,0x59EB,0x59EF,0x59F0,0x59D5,0x5A0D,
	0x5A04,0x59F9,0x5A02,0x59F8,0x59E2,0x59D9,0x59E7,0x5B6A,
	INVALC,INVALC,0x5BAB,INVALC,0x5C1B,0x5C2F,INVALC,0x663C,
	INVALC,INVALC,INVALC,0x5CD1,0x5CDC,0x5CE6,0x5CE1,0x5CCD,
	INVALC,0x5CE2,0x5CDD,0x5CE5,0x5DFB,0x5DFA,0x5E1E,INVALC,
	0x5EA1,INVALC,INVALC,0x5EFC,0x5EFB,0x5F2F,INVALC,INVALC,
	0x5F66,INVALC,INVALC,INVALC,0x605C,INVALC,0x604E,0x6051,
	INVALC,INVALC,0x6023,0x6031,0x607C,0x6052,INVALC,0x6060,
	0x604A,0x6061,INVALC,0x6218,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x631F,0x6317,0x62EA,0x6321,0x6304,
	0x6305,INVALC,0x6531,0x6544,0x6540,INVALC,0x6542,0x65BE,
	INVALC,0x6629,0x661B,INVALC,0x6623,0x662C,0x661A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 46, Array index 0x0D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6630,0x663B,0x661E,0x6637,0x6638,INVALC,0x670E,
	INVALC,INVALC,0x67E8,0x67D6,INVALC,0x67C7,0x67BC,0x6852,
	0x67BF,0x67D5,0x67FE,0x8363,0x67FB,INVALC,0x67B1,0x6801,
	0x6805,0x6800,0x67D7,INVALC,0x6B2A,0x6B6B,INVALC,INVALC,
	INVALC,INVALC,0x6BE1,INVALC,INVALC,0x6D23,0x6CFF,0x6D14,
	0x6D05,0x6D13,0x6D06,0x6D21,INVALC,0x6D15,0x6CAF,0x6CF4,
	0x6D02,0x6D45,INVALC,0x6D26,INVALC,0x6D44,INVALC,0x6D24,
	0x70A5,INVALC,0x70A3,INVALC,0x70A2,0x70BB,0x70A0,0x70AA,
	INVALC,INVALC,0x70A8,0x70B6,0x70B2,0x70A7,INVALC,INVALC,
	0x70B9,0x722E,INVALC,0x723C,INVALC,0x726D,INVALC,INVALC,
	0x72E7,0x72ED,INVALC,0x72EC,0x72E5,0x72E2,INVALC,0x73C4,
	0x73BD,0x73CF,0x73C9,0x73C1,0x73D0,INVALC,0x73CE,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 47, Array index 0x0E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x74ED,0x74EB,INVALC,0x74EF,0x7549,0x7550,0x7546,
	0x754A,INVALC,0x754D,0x75A6,INVALC,INVALC,INVALC,0x75A8,
	INVALC,INVALC,0x76C7,0x76FF,INVALC,0x76FD,0x77E6,0x780A,
	INVALC,0x7804,0x780B,0x7807,INVALC,0x7815,0x7808,INVALC,
	0x79D3,0x79D4,0x79D0,0x79D7,0x7A7C,INVALC,INVALC,0x7A7D,
	0x7A83,0x7A82,INVALC,0x7AD4,0x7AD5,0x7AD3,0x7AD0,0x7AD2,
	0x7AFE,0x7AFC,0x7C77,0x7C7C,0x7C7B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7F8F,0x80D3,INVALC,
	0x80CB,0x80D2,INVALC,0x8109,0x80E2,0x80DF,0x80C6,INVALC,
	0x8224,0x82F7,0x82D8,0x82DD,INVALC,INVALC,0x82F8,0x82FC,
	INVALC,INVALC,0x82E9,INVALC,0x82EE,INVALC,0x82D0,0x830E,
	0x82E2,0x830B,0x82FD,0x5179,0x8676,INVALC,0x8678,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 48, Array index 0x0F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x8675,0x867D,INVALC,0x8842,0x8866,
	INVALC,0x898C,0x8A05,INVALC,0x8A06,INVALC,0x8C9F,INVALC,
	0x8FF1,0x8FE7,0x8FE9,0x8FEF,0x90C2,0x90BC,INVALC,0x90C6,
	0x90C0,INVALC,INVALC,0x90CD,0x90C9,INVALC,0x90C4,INVALC,
	0x9581,INVALC,0x9CEC,0x5032,0x4FF9,0x501D,0x4FFF,0x5004,
	0x4FF0,0x5003,INVALC,0x5002,0x4FFC,0x4FF2,0x5024,0x5008,
	0x5036,0x502E,INVALC,0x5010,0x5038,0x5039,0x4FFD,0x5056,
	0x4FFB,0x51A3,0x51A6,0x51A1,INVALC,INVALC,0x51C7,0x51C9,
	0x5260,0x5264,0x5259,0x5265,0x5267,0x5257,0x5263,INVALC,
	0x5253,INVALC,0x52CF,INVALC,0x52CE,0x52D0,0x52D1,0x52CC,
	INVALC,INVALC,INVALC,0x550D,0x54F4,INVALC,0x5513,0x54EF,
	0x54F5,0x54F9,0x5502,0x5500,INVALC,INVALC,0x5518,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 49, Array index 0x1000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x54F0,0x54F6,INVALC,INVALC,0x5519,INVALC,0x5705,
	0x57C9,INVALC,0x57B7,0x57CD,INVALC,INVALC,INVALC,0x57BE,
	0x57BB,INVALC,0x57DB,0x57C8,0x57C4,0x57C5,0x57D1,0x57CA,
	0x57C0,INVALC,INVALC,0x5A21,0x5A2A,INVALC,0x5A1D,INVALC,
	0x5A0B,INVALC,INVALC,INVALC,INVALC,0x5A22,INVALC,INVALC,
	0x5A24,INVALC,0x5A14,0x5A31,INVALC,0x5A2F,0x5A1A,0x5A12,
	INVALC,INVALC,0x5A26,INVALC,INVALC,0x5BBC,0x5BBB,0x5BB7,
	0x5C05,0x5C06,0x5C52,0x5C53,INVALC,INVALC,0x5CFA,0x5CEB,
	INVALC,0x5CF3,0x5CF5,0x5CE9,0x5CEF,INVALC,0x5E2A,0x5E30,
	0x5E2E,0x5E2C,0x5E2F,0x5EAF,0x5EA9,INVALC,0x5EFD,0x5F32,
	0x5F8E,0x5F93,0x5F8F,0x604F,0x6099,INVALC,0x607E,INVALC,
	0x6074,0x604B,0x6073,0x6075,INVALC,INVALC,0x6056,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 50, Array index 0x1100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x60A9,0x608B,0x60A6,INVALC,0x6093,0x60AE,0x609E,
	0x60A7,0x6245,INVALC,INVALC,0x632E,INVALC,0x6352,0x6330,
	0x635B,INVALC,0x6319,0x631B,INVALC,0x6331,0x635D,0x6337,
	0x6335,0x6353,INVALC,0x635C,0x633F,0x654B,INVALC,INVALC,
	0x658B,INVALC,0x659A,0x6650,0x6646,0x664E,0x6640,INVALC,
	0x664B,0x6648,INVALC,0x6660,0x6644,0x664D,INVALC,0x6837,
	0x6824,INVALC,INVALC,0x681B,0x6836,INVALC,0x682C,0x6819,
	0x6856,0x6847,0x683E,0x681E,INVALC,0x6815,0x6822,0x6827,
	0x6859,0x6858,0x6855,0x6830,0x6823,0x6B2E,0x6B2B,0x6B30,
	0x6B6C,INVALC,0x6B8B,INVALC,0x6BE9,0x6BEA,0x6BE5,0x6D6B,
	INVALC,INVALC,0x6D73,0x6D57,INVALC,INVALC,0x6D5D,0x6D56,
	0x6D8F,0x6D5B,0x6D1C,0x6D9A,0x6D9B,0x6D99,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 51, Array index 0x1200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6D81,0x6D71,INVALC,INVALC,0x6D72,0x6D5C,0x6D96,
	0x70C4,0x70DB,0x70CC,0x70D0,0x70E3,0x70DF,INVALC,0x70D6,
	0x70EE,0x70D5,INVALC,INVALC,INVALC,INVALC,0x727A,INVALC,
	0x72F5,0x7302,INVALC,INVALC,0x73E2,0x73EC,0x73D5,0x73F9,
	0x73DF,0x73E6,INVALC,INVALC,INVALC,INVALC,0x73E4,0x73E1,
	0x74F3,INVALC,INVALC,INVALC,INVALC,0x7556,0x7555,0x7558,
	0x7557,0x755E,0x75C3,INVALC,INVALC,0x75B4,INVALC,0x75B1,
	INVALC,INVALC,0x76CB,0x76CC,0x772A,INVALC,0x7716,0x770F,
	INVALC,INVALC,0x773F,0x772B,0x770E,0x7724,INVALC,0x7721,
	0x7718,0x77DD,INVALC,INVALC,0x7824,0x7836,INVALC,0x7958,
	0x7959,INVALC,0x7962,0x79DA,0x79D9,INVALC,0x79E1,0x79E5,
	0x79E8,0x79DB,INVALC,0x79E2,0x79F0,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 52, Array index 0x1300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x7ADA,0x7ADD,INVALC,0x7ADB,0x7ADC,
	INVALC,INVALC,0x7B0D,0x7B0B,0x7B14,0x7C8E,0x7C86,INVALC,
	0x7C87,0x7C83,0x7C8B,INVALC,INVALC,INVALC,INVALC,0x7D24,
	INVALC,INVALC,INVALC,0x7D25,0x7F62,0x7F93,0x7F99,0x7F97,
	INVALC,INVALC,0x7FC4,0x7FC6,0x800A,INVALC,INVALC,0x8040,
	0x803C,0x803B,0x80F6,0x80FF,0x80EE,0x8104,0x8103,0x8107,
	INVALC,INVALC,0x80F7,INVALC,INVALC,0x822D,INVALC,0x8227,
	0x8229,0x831F,0x8357,INVALC,INVALC,INVALC,INVALC,0x8321,
	INVALC,INVALC,0x8318,0x8358,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8684,0x869F,0x869B,0x8689,0x86A6,0x8692,0x868F,
	0x86A0,0x884F,0x8878,0x887A,0x886E,0x887B,0x8884,0x8873,
	INVALC,INVALC,0x8A0D,0x8A0B,0x8A19,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 53, Array index 0x1400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x8FF9,0x9009,0x9008,
	INVALC,0x90DE,0x9151,INVALC,INVALC,0x91DB,0x91DF,0x91DE,
	0x91D6,0x91E0,0x9585,0x9660,0x9659,INVALC,0x9656,INVALC,
	INVALC,0x96BD,INVALC,INVALC,0x5042,0x5059,INVALC,0x5044,
	0x5066,0x5052,0x5054,0x5071,0x5050,0x507B,0x507C,0x5058,
	INVALC,INVALC,0x5079,0x506C,0x5078,0x51A8,0x51D1,0x51CF,
	0x5268,0x5276,0x52D4,INVALC,0x53A0,0x53C4,INVALC,0x5558,
	0x554C,0x5568,INVALC,0x5549,INVALC,INVALC,0x555D,0x5529,
	INVALC,0x5554,0x5553,INVALC,0x555A,INVALC,0x553A,0x553F,
	0x552B,0x57EA,INVALC,0x57EF,INVALC,INVALC,0x57DD,0x57FE,
	INVALC,0x57DE,0x57E6,INVALC,0x57E8,0x57FF,0x5803,0x58F7,
	0x68A6,0x591F,INVALC,0x595B,0x595D,0x595E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 54, Array index 0x1500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A2B,INVALC,0x5A3B,INVALC,INVALC,0x5A61,
	0x5A3A,0x5A6E,0x5A4B,0x5A6B,INVALC,INVALC,0x5A45,0x5A4E,
	0x5A68,0x5A3D,0x5A71,0x5A3F,0x5A6F,0x5A75,INVALC,0x5A73,
	0x5A2C,0x5A59,0x5A54,0x5A4F,0x5A63,INVALC,INVALC,0x5BC8,
	INVALC,0x5BC3,INVALC,0x5C5B,0x5C61,INVALC,0x5D21,0x5D0A,
	0x5D09,INVALC,0x5D2C,0x5D08,INVALC,INVALC,0x5D2A,0x5D15,
	INVALC,0x5D10,0x5D13,INVALC,0x5D2F,0x5D18,INVALC,0x5DE3,
	0x5E39,0x5E35,0x5E3A,0x5E32,INVALC,INVALC,INVALC,INVALC,
	0x5EBB,0x5EBA,0x5F34,0x5F39,INVALC,INVALC,INVALC,INVALC,
	0x6098,INVALC,0x60D0,INVALC,INVALC,INVALC,0x60D7,0x60AA,
	INVALC,0x60A1,0x60A4,INVALC,0x60EE,INVALC,0x60E7,INVALC,
	INVALC,0x60DE,INVALC,INVALC,0x637E,0x638B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 55, Array index 0x1600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6379,0x6386,0x6393,INVALC,0x6373,0x636A,
	INVALC,0x636C,INVALC,0x637F,INVALC,0x63B2,0x63BA,INVALC,
	INVALC,0x6366,0x6374,INVALC,0x655A,INVALC,0x654E,0x654D,
	0x658D,0x658E,0x65AD,INVALC,0x65C7,0x65CA,INVALC,0x65C9,
	INVALC,0x65E3,0x6657,INVALC,0x6663,0x6667,0x671A,0x6719,
	0x6716,INVALC,INVALC,0x689E,0x68B6,0x6898,0x6873,INVALC,
	0x689A,0x688E,0x68B7,0x68DB,0x68A5,0x686C,0x68C1,0x6884,
	INVALC,INVALC,0x6895,0x687A,0x6899,INVALC,0x68B8,0x68B9,
	0x6870,INVALC,0x6B35,INVALC,0x6B90,0x6BBB,0x6BED,INVALC,
	INVALC,INVALC,0x6DC1,0x6DC3,0x6DCE,INVALC,INVALC,0x6DAD,
	0x6E04,INVALC,0x6DB9,INVALC,0x6DE7,INVALC,0x6E08,0x6E06,
	INVALC,0x6E0A,0x6DB0,INVALC,0x6DF8,0x6E0C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 56, Array index 0x1700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6DB1,INVALC,0x6E02,0x6E07,0x6E09,0x6E01,0x6E17,
	0x6DFF,0x6E12,INVALC,INVALC,0x7103,0x7107,0x7101,0x70F5,
	0x70F1,0x7108,0x70F2,0x710F,INVALC,0x70FE,INVALC,INVALC,
	INVALC,0x731A,0x7310,0x730E,0x7402,0x73F3,INVALC,INVALC,
	0x73FB,INVALC,INVALC,INVALC,0x751B,0x7523,0x7561,0x7568,
	INVALC,0x7567,0x75D3,INVALC,INVALC,0x7690,INVALC,INVALC,
	0x76D5,0x76D7,0x76D6,0x7730,INVALC,0x7726,INVALC,0x7740,
	INVALC,0x771E,INVALC,INVALC,INVALC,0x7847,INVALC,0x784B,
	0x7851,0x784F,0x7842,0x7846,INVALC,0x796E,0x796C,0x79F2,
	INVALC,0x79F1,0x79F5,0x79F3,0x79F9,INVALC,INVALC,INVALC,
	0x7A9A,0x7A93,0x7A91,0x7AE1,INVALC,INVALC,0x7B21,0x7B1C,
	0x7B16,0x7B17,0x7B36,0x7B1F,INVALC,0x7C93,0x7C99,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 57, Array index 0x1800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7C9A,0x7C9C,INVALC,0x7D49,INVALC,0x7D34,0x7D37,
	INVALC,0x7D2D,INVALC,0x7D4C,INVALC,INVALC,0x7D48,INVALC,
	INVALC,0x7F3B,INVALC,INVALC,INVALC,INVALC,0x8008,0x801A,
	INVALC,0x801D,INVALC,0x8049,0x8045,0x8044,0x7C9B,INVALC,
	INVALC,0x812A,0x812E,INVALC,INVALC,0x8131,INVALC,0x811A,
	0x8134,0x8117,INVALC,INVALC,INVALC,0x831D,0x8371,0x8384,
	0x8380,0x8372,0x83A1,INVALC,0x8379,0x8391,INVALC,0x839F,
	0x83AD,INVALC,INVALC,0x8323,INVALC,0x8385,0x839C,0x83B7,
	0x8658,0x865A,INVALC,0x8657,0x86B2,INVALC,0x86AE,INVALC,
	INVALC,INVALC,0x8845,0x889C,0x8894,0x88A3,0x888F,0x88A5,
	0x88A9,0x88A6,0x888A,0x88A0,0x8890,0x8992,0x8991,0x8994,
	INVALC,0x8A26,0x8A32,0x8A28,INVALC,INVALC,0x8A1C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 58, Array index 0x1900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x8A2B,0x8A20,INVALC,0x8A29,INVALC,INVALC,
	INVALC,0x8A21,0x8C3A,INVALC,0x8C5B,0x8C58,0x8C7C,INVALC,
	0x8CA6,0x8CAE,0x8CAD,0x8D65,INVALC,0x8D7E,INVALC,0x8D7C,
	0x8D7F,0x8D7A,0x8DBD,INVALC,INVALC,0x8DC0,0x8DBB,0x8EAD,
	0x8EAF,0x8ED6,INVALC,INVALC,INVALC,INVALC,INVALC,0x8ED9,
	INVALC,INVALC,0x9012,0x900E,0x9025,INVALC,0x9013,0x90EE,
	INVALC,0x90AB,0x90F7,INVALC,0x9159,0x9154,0x91F2,0x91F0,
	0x91E5,0x91F6,INVALC,INVALC,0x9587,INVALC,0x965A,INVALC,
	INVALC,0x966E,INVALC,INVALC,INVALC,0x9679,INVALC,0x98E1,
	0x98E6,INVALC,0x9EC4,0x9ED2,0x4E80,INVALC,0x4E81,0x508F,
	0x5097,0x5088,0x5089,INVALC,INVALC,0x5081,0x5160,INVALC,
	INVALC,0x5E42,0x51D3,INVALC,INVALC,0x51D2,0x51D6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 59, Array index 0x1A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5273,INVALC,0x5270,INVALC,INVALC,INVALC,0x53A8,
	0x53A6,0x53C5,0x5597,0x55DE,INVALC,INVALC,0x5596,0x55B4,
	INVALC,0x5585,INVALC,0x559B,0x55A0,INVALC,0x5559,INVALC,
	0x5586,INVALC,INVALC,0x55AF,0x557A,INVALC,INVALC,INVALC,
	0x559E,INVALC,0x55A9,0x570F,0x570E,0x581A,INVALC,0x581F,
	INVALC,0x583C,0x5818,0x583E,0x5826,INVALC,0x583A,INVALC,
	0x5822,INVALC,0x58FB,0x5963,0x5964,INVALC,0x5AA8,0x5AA3,
	0x5A82,0x5A88,0x5AA1,0x5A85,0x5A98,INVALC,0x5A99,INVALC,
	0x5A89,0x5A81,0x5A96,0x5A80,INVALC,INVALC,0x5A91,INVALC,
	INVALC,INVALC,INVALC,0x5ACF,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A87,0x5AA0,INVALC,0x5A79,INVALC,0x5A86,
	0x5AAB,0x5AAA,0x5AA4,0x5A8D,0x5A7E,INVALC,0x5BD5,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 60, Array index 0x1B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5C1E,0x5C5F,0x5C5E,0x5D44,
	0x5D3E,INVALC,0x5D48,0x5D1C,INVALC,0x5D5B,0x5D4D,INVALC,
	INVALC,0x5D57,INVALC,0x5D53,0x5D4F,INVALC,0x5D3B,0x5D46,
	INVALC,INVALC,0x5E46,0x5E47,INVALC,0x5E48,0x5EC0,0x5EBD,
	0x5EBF,INVALC,0x5F11,INVALC,0x5F3E,0x5F3B,INVALC,0x5F3A,
	INVALC,INVALC,INVALC,0x5FA7,INVALC,0x60EA,INVALC,0x6107,
	0x6122,0x610C,INVALC,INVALC,0x60B3,0x60D6,0x60D2,INVALC,
	0x60E3,0x60E5,0x60E9,INVALC,INVALC,0x6111,0x60FD,INVALC,
	INVALC,0x611E,0x6120,0x6121,0x621E,INVALC,0x63E2,0x63DE,
	0x63E6,INVALC,INVALC,INVALC,INVALC,0x63F8,INVALC,0x63FE,
	0x63C1,0x63BF,0x63F7,0x63D1,0x655F,0x6560,0x6561,INVALC,
	INVALC,0x65D1,INVALC,INVALC,0x667D,0x666B,0x667F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 61, Array index 0x1C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6673,0x6681,0x666D,0x6669,INVALC,
	INVALC,0x671E,0x68ED,INVALC,INVALC,INVALC,INVALC,0x6903,
	INVALC,0x68FE,0x68E5,0x691E,0x6902,INVALC,INVALC,0x6909,
	0x68CA,0x6900,INVALC,0x6901,0x6918,0x68E2,0x68CF,INVALC,
	0x692E,0x68C5,0x68FF,INVALC,0x691C,0x68C3,INVALC,0x6B6F,
	INVALC,0x6B6E,INVALC,0x6BBE,INVALC,0x6BF4,0x6C2D,INVALC,
	0x6DB6,0x6E75,0x6E1E,INVALC,0x6E18,INVALC,0x6E48,INVALC,
	0x6E4F,INVALC,0x6E42,0x6E6A,0x6E70,0x6DFE,INVALC,INVALC,
	0x6E6D,INVALC,0x6E7B,0x6E7E,0x6E59,INVALC,0x6E57,INVALC,
	0x6E80,0x6E50,INVALC,0x6E29,0x6E76,0x6E2A,0x6E4C,0x712A,
	INVALC,0x7135,0x712C,0x7137,0x711D,INVALC,INVALC,0x7138,
	INVALC,0x7134,0x712B,0x7133,0x7127,0x7124,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 62, Array index 0x1D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x712D,0x7232,0x7283,0x7282,0x7287,0x7306,0x7324,
	0x7338,0x732A,0x732C,0x732B,INVALC,0x732F,0x7328,0x7417,
	INVALC,INVALC,0x7419,0x7438,INVALC,0x741F,0x7414,0x743C,
	0x73F7,0x741C,0x7415,0x7418,0x7439,0x74F9,0x7524,INVALC,
	INVALC,INVALC,0x756E,0x756D,0x7571,0x758E,INVALC,0x75E5,
	INVALC,INVALC,INVALC,INVALC,0x7694,0x76B3,INVALC,0x76D9,
	INVALC,0x7748,0x7749,0x7743,INVALC,INVALC,0x7742,0x77DF,
	INVALC,0x7863,0x7876,INVALC,0x785F,0x7866,0x7966,0x7971,
	INVALC,INVALC,0x7976,0x7984,0x7975,0x79FF,0x7A07,INVALC,
	0x7A0E,0x7A09,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7AE7,0x7AE2,0x7B55,INVALC,INVALC,0x7B43,0x7B57,0x7B6C,
	0x7B42,0x7B53,INVALC,0x7B41,INVALC,INVALC,0x7CA7,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 63, Array index 0x1E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7CA0,0x7CA6,0x7CA4,0x7D74,INVALC,0x7D59,INVALC,
	0x7D60,0x7D57,0x7D6C,0x7D7E,0x7D64,INVALC,0x7D5A,0x7D5D,
	INVALC,INVALC,INVALC,0x7D76,0x7D4D,0x7D75,INVALC,0x7FD3,
	0x7FD6,INVALC,INVALC,0x8060,0x804E,0x8145,0x813B,INVALC,
	0x8148,0x8142,0x8149,0x8140,0x8114,0x8141,INVALC,0x81EF,
	0x81F6,0x8203,INVALC,0x83ED,INVALC,0x83DA,0x8418,0x83D2,
	0x8408,INVALC,0x8400,INVALC,INVALC,INVALC,0x8417,0x8346,
	0x8414,0x83D3,0x8405,0x841F,0x8402,0x8416,0x83CD,0x83E6,
	INVALC,0x865D,0x86D5,0x86E1,INVALC,INVALC,INVALC,INVALC,
	0x86EE,0x8847,0x8846,INVALC,INVALC,0x88BB,INVALC,0x88BF,
	0x88B4,INVALC,0x88B5,INVALC,0x899A,0x8A43,INVALC,INVALC,
	0x8A5A,INVALC,INVALC,INVALC,0x8A35,0x8A38,0x8A42,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 64, Array index 0x1F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8A49,0x8A5D,0x8A4B,0x8A3D,INVALC,INVALC,INVALC,
	INVALC,0x8C60,0x8C5E,0x8C7F,0x8C7E,0x8C83,INVALC,0x8CB1,
	0x8D87,INVALC,INVALC,0x8D88,0x8D83,INVALC,INVALC,0x8D86,
	0x8D8B,0x8D82,0x8DCA,0x8DD2,INVALC,INVALC,0x8DD4,0x8DC9,
	0x8EB0,INVALC,INVALC,INVALC,0x8EF2,0x8EE4,0x8EF3,0x8EEA,
	INVALC,0x8EFD,INVALC,0x8F9D,0x902B,0x902A,INVALC,0x9028,
	0x9029,0x902C,INVALC,INVALC,0x903A,0x9030,0x9037,0x903B,
	INVALC,0x910A,INVALC,INVALC,INVALC,0x91FE,0x9220,INVALC,
	0x920B,INVALC,0x9218,0x9222,INVALC,0x921B,0x9208,INVALC,
	0x920E,0x9213,INVALC,INVALC,0x9595,INVALC,INVALC,INVALC,
	0x968C,0x967B,0x967F,0x9681,INVALC,0x9682,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x96EE,0x96ED,INVALC,0x96EC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 65, Array index 0x2000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x975F,0x976F,INVALC,0x976D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x98F0,INVALC,INVALC,INVALC,0x9AA9,
	INVALC,INVALC,0x9AE0,0x4EB7,INVALC,INVALC,0x50CC,0x50BC,
	INVALC,0x50AA,0x50B9,INVALC,0x50AB,0x50C3,0x50CD,0x517E,
	0x527E,0x5279,INVALC,INVALC,0x52E1,0x52E0,0x52E7,0x5380,
	0x53AB,0x53AA,0x53A9,0x53E0,0x55EA,INVALC,0x55D7,INVALC,
	INVALC,0x55C1,0x5715,INVALC,0x586C,INVALC,0x585C,0x5850,
	0x5861,0x586A,0x5869,0x5856,0x5860,0x5866,0x585F,0x5923,
	0x5966,0x5968,INVALC,INVALC,0x5ACE,INVALC,0x5AC5,0x5AC3,
	INVALC,INVALC,0x5AD0,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5B74,0x5B76,0x5BDC,0x5BD7,0x5BDA,0x5BDB,INVALC,
	0x5C20,0x5D6D,0x5D66,INVALC,0x5D64,0x5D6E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 66, Array index 0x2100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D60,0x5F42,0x5F5A,0x5F6E,INVALC,INVALC,0x6130,
	0x613A,0x612A,0x6143,0x6119,0x6131,INVALC,0x613D,INVALC,
	INVALC,INVALC,0x6408,0x6432,0x6438,INVALC,0x6431,INVALC,
	0x6419,INVALC,0x6411,INVALC,INVALC,0x6429,0x641D,INVALC,
	INVALC,INVALC,0x643C,INVALC,0x6446,0x6447,INVALC,INVALC,
	0x643A,0x6407,INVALC,0x656B,INVALC,0x6570,0x656D,INVALC,
	0x65E4,0x6693,INVALC,INVALC,INVALC,INVALC,0x668F,INVALC,
	INVALC,0x6692,INVALC,0x668E,INVALC,0x6946,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6931,INVALC,INVALC,
	0x693E,INVALC,0x697C,0x6943,INVALC,0x6973,INVALC,0x6955,
	INVALC,INVALC,0x6985,0x694D,0x6950,0x6947,0x6967,0x6936,
	0x6964,0x6961,INVALC,0x697D,0x6B44,0x6B40,0x6B71,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 67, Array index 0x2200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B73,0x6B9C,INVALC,INVALC,INVALC,0x6BC1,INVALC,
	0x6BFA,0x6C31,0x6C32,INVALC,INVALC,0x6EB8,0x6EA8,INVALC,
	0x6E91,0x6EBB,INVALC,0x6E9A,INVALC,INVALC,0x6EA9,INVALC,
	INVALC,0x6EB5,0x6E6C,0x6EE8,INVALC,0x6EDD,0x6EDA,0x6EE6,
	0x6EAC,INVALC,INVALC,INVALC,0x6ED9,0x6EE3,0x6EE9,0x6EDB,
	INVALC,0x716F,INVALC,INVALC,0x7148,INVALC,0x714A,0x716B,
	INVALC,0x714F,0x7157,0x7174,INVALC,INVALC,INVALC,0x7145,
	0x7151,0x716D,INVALC,0x7251,0x7250,0x724E,INVALC,0x7341,
	INVALC,0x732E,0x7346,INVALC,0x7427,INVALC,0x7448,0x7453,
	0x743D,INVALC,0x745D,0x7456,INVALC,0x741E,0x7447,0x7443,
	0x7458,0x7449,INVALC,0x744C,0x7445,0x743E,INVALC,0x7501,
	0x751E,INVALC,INVALC,0x757A,0x75EE,0x7602,0x7697,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 68, Array index 0x2300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7698,INVALC,INVALC,INVALC,0x775D,0x7764,0x7753,
	0x7758,0x7882,0x7890,0x788A,INVALC,0x787A,0x787D,INVALC,
	0x788B,0x7878,INVALC,INVALC,0x788D,0x7888,0x7892,0x7881,
	0x797E,0x7983,INVALC,INVALC,INVALC,0x7980,INVALC,INVALC,
	INVALC,0x7A0F,INVALC,INVALC,0x7A1D,INVALC,0x7AA1,0x7AA4,
	INVALC,0x7AE9,0x7AEA,INVALC,0x7B62,0x7B6B,INVALC,0x7B5E,
	INVALC,0x7B79,INVALC,INVALC,0x7B6F,0x7B68,INVALC,INVALC,
	0x7CAE,INVALC,INVALC,INVALC,0x7CB0,INVALC,0x7D90,INVALC,
	0x7D8A,INVALC,0x7D8B,0x7D99,0x7D95,INVALC,0x7D87,0x7D78,
	0x7D97,0x7D89,0x7D98,INVALC,INVALC,INVALC,0x7FA3,INVALC,
	INVALC,INVALC,0x7FDD,0x8057,INVALC,0x8163,0x816A,0x816C,
	INVALC,INVALC,INVALC,0x815D,0x8175,INVALC,0x815F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 69, Array index 0x2400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x817D,0x816D,INVALC,INVALC,0x8241,0x844F,
	0x8484,INVALC,0x847F,INVALC,0x8448,0x842A,0x847B,0x8472,
	0x8464,0x842E,0x845C,0x8453,INVALC,0x8441,0x84C8,INVALC,
	0x8462,0x8480,0x843E,0x8483,0x8471,INVALC,0x844A,0x8455,
	0x8458,INVALC,INVALC,INVALC,0x86FC,0x86FD,0x8715,INVALC,
	0x8716,0x86FF,INVALC,INVALC,INVALC,0x8858,0x88CF,0x88E0,
	INVALC,INVALC,INVALC,INVALC,0x89E7,0x8A6A,0x8A80,INVALC,
	0x8A6F,0x8A65,INVALC,0x8A78,0x8A7D,0x8A88,INVALC,INVALC,
	0x8A64,0x8A7E,INVALC,0x8A67,0x8C63,0x8C88,INVALC,0x8CCD,
	INVALC,0x8CC9,INVALC,0x8DED,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x8EB1,INVALC,INVALC,0x8F04,0x8F9E,
	0x8FA0,0x9043,0x9046,0x9048,0x9045,0x9040,0x904C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 70, Array index 0x2500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x910C,0x9113,0x9115,INVALC,0x916B,
	0x9167,0x925D,0x9255,0x9235,INVALC,0x9259,0x922F,0x923C,
	0x928F,0x925C,0x926A,0x9262,0x925F,0x926B,0x926E,0x923B,
	0x9244,0x9241,0x959A,INVALC,0x9599,INVALC,INVALC,INVALC,
	0x968F,INVALC,0x9696,INVALC,INVALC,INVALC,0x96F4,0x96FC,
	INVALC,0x9755,INVALC,0x9779,INVALC,INVALC,INVALC,0x97EE,
	0x97F5,INVALC,0x980B,INVALC,0x98F3,INVALC,INVALC,0x98F7,
	0x98FF,0x98F5,INVALC,0x98EC,0x98F1,INVALC,INVALC,0x999A,
	INVALC,0x9AE2,0x9B3D,0x9B5D,0x9CE8,INVALC,0x9CEB,0x9CEF,
	0x9CEE,0x9E81,0x9F14,0x50D0,0x50D9,0x50DC,0x50D8,INVALC,
	0x50E1,0x50EB,INVALC,INVALC,0x50F4,0x50E2,0x50DE,INVALC,
	INVALC,INVALC,0x51F4,INVALC,INVALC,INVALC,0x52ED,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 71, Array index 0x2600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x52EA,INVALC,0x5332,INVALC,0x53AE,0x53B0,INVALC,
	0x55FB,0x5603,0x560B,INVALC,0x5607,INVALC,0x55F8,INVALC,
	0x5628,0x561E,INVALC,0x5618,0x5611,0x5651,0x5605,0x5717,
	0x5892,INVALC,0x588C,INVALC,0x5878,0x5884,0x5873,0x58AD,
	0x5897,0x5895,0x5877,0x5872,0x5896,0x588D,0x5910,INVALC,
	0x596C,INVALC,0x5AE7,INVALC,0x5AE4,INVALC,INVALC,0x5AEF,
	0x5626,INVALC,INVALC,0x5AF0,0x5D7B,INVALC,0x5D83,INVALC,
	INVALC,0x5D8B,0x5D8C,INVALC,0x5D78,0x5E52,INVALC,INVALC,
	0x5ED0,0x5ECF,INVALC,0x5FB3,0x5FB4,INVALC,INVALC,INVALC,
	0x617B,INVALC,0x616F,0x6181,0x613C,0x6142,0x6138,0x6133,
	INVALC,0x6160,0x6169,0x617D,0x6186,0x622C,0x6228,INVALC,
	0x644C,INVALC,0x6457,0x647C,INVALC,INVALC,0x6455,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 72, Array index 0x2700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6462,0x6471,0x646A,0x6456,0x643B,0x6481,INVALC,
	0x644F,0x647E,0x6464,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6571,INVALC,INVALC,0x66A5,0x669A,0x669C,INVALC,0x66A6,
	INVALC,0x66A4,0x698F,0x69C5,0x69C8,0x6992,0x69B2,INVALC,
	INVALC,INVALC,0x69E3,0x69C0,0x69D6,0x69D1,0x699F,0x69A2,
	0x69D2,INVALC,INVALC,INVALC,0x69E1,0x69D5,0x699D,INVALC,
	INVALC,0x6998,INVALC,0x6B74,0x6BA1,INVALC,0x6EF0,0x6EF3,
	INVALC,INVALC,0x6F1B,0x6F0C,0x6F1D,0x6F34,0x6F28,0x6F17,
	INVALC,0x6F44,0x6F42,0x6F04,0x6F11,0x6EFA,0x6F4A,0x7191,
	0x718E,INVALC,0x718B,0x718D,0x717F,0x718C,0x717E,0x717C,
	0x7183,INVALC,0x7188,INVALC,INVALC,0x7294,INVALC,0x7355,
	0x7353,0x734F,0x7354,0x746C,0x7465,0x7466,0x7461,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 73, Array index 0x2800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x746B,0x7468,0x7476,INVALC,0x7460,INVALC,0x7474,
	0x7506,0x760E,INVALC,0x7607,INVALC,INVALC,0x76B9,INVALC,
	0x76B7,0x76E2,INVALC,0x7774,0x7777,0x7776,0x7775,INVALC,
	0x7778,0x7771,INVALC,0x777A,0x715B,0x777B,0x78A6,0x78AE,
	0x78B8,INVALC,INVALC,INVALC,0x78B1,0x78AF,INVALC,0x7989,
	0x7987,INVALC,INVALC,0x7A29,INVALC,0x7A2A,INVALC,0x7A2D,
	0x7A2C,INVALC,0x7A32,INVALC,0x7AEC,0x7AF0,0x7B81,0x7B9E,
	0x7B83,INVALC,0x7B92,INVALC,0x7BA3,0x7B9F,0x7B93,INVALC,
	0x7B86,0x7CB8,0x7CB7,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x7DC8,0x7DB6,INVALC,0x7DD1,INVALC,0x7DA8,0x7DAB,INVALC,
	0x7DB3,0x7DCD,INVALC,0x7DCF,0x7DA4,INVALC,INVALC,0x7F41,
	0x7F6F,0x7F71,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 74, Array index 0x2900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x8023,0x805B,INVALC,0x8061,0x805F,0x8181,
	INVALC,INVALC,0x8184,0x8213,INVALC,0x824A,0x824C,INVALC,
	INVALC,INVALC,0x84BD,0x8495,INVALC,0x8492,0x84C3,INVALC,
	0x8496,0x84A5,0x84B5,0x84B3,0x84A3,0x84E4,0x84D8,0x84D5,
	INVALC,0x84B7,0x84AD,0x84DA,0x8493,0x8736,INVALC,INVALC,
	INVALC,0x873D,0x872B,0x8747,0x8739,INVALC,0x8745,0x871D,
	INVALC,0x88FF,0x88EA,INVALC,0x88F5,INVALC,0x8900,0x88ED,
	0x8903,0x88E9,INVALC,INVALC,0x89EA,INVALC,0x8A9B,0x8A8E,
	0x8AA2,INVALC,0x8A9C,0x8A94,0x8A90,0x8AA9,0x8AAC,INVALC,
	0x8A9F,INVALC,INVALC,0x8A9D,INVALC,0x8C67,INVALC,INVALC,
	0x8CD0,0x8CD6,0x8CD4,0x8D98,0x8D9A,0x8D97,INVALC,INVALC,
	INVALC,0x8E0B,0x8E08,0x8E01,0x8EB4,0x8EB3,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 75, Array index 0x2A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8FA1,0x8FA2,INVALC,0x905A,INVALC,0x9061,0x905F,
	INVALC,INVALC,0x9125,0x917B,0x9176,0x917C,INVALC,0x9289,
	0x92F6,0x92B1,0x92AD,0x9292,0x9281,0x9284,INVALC,0x92AE,
	0x9290,0x929E,INVALC,INVALC,INVALC,0x95A2,0x95A7,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x96A0,0x969D,0x969F,0x96D0,
	INVALC,0x96D1,INVALC,INVALC,0x9759,INVALC,0x9764,INVALC,
	INVALC,INVALC,0x9819,INVALC,0x9814,0x9815,0x981A,INVALC,
	INVALC,INVALC,INVALC,0x9906,INVALC,0x98F8,0x9901,INVALC,
	0x99BE,0x99BC,0x99B7,0x99B6,0x99C0,INVALC,0x99B8,INVALC,
	INVALC,INVALC,0x99C4,INVALC,0x99BF,INVALC,0x9ADA,0x9AE4,
	0x9AE9,0x9AE8,0x9AEA,0x9AE5,INVALC,0x9B26,INVALC,INVALC,
	0x9B40,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 76, Array index 0x2B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x9EBD,INVALC,INVALC,INVALC,INVALC,0x510E,
	INVALC,0x50F7,INVALC,0x50FC,0x510D,0x5101,0x51DA,0x51D9,
	0x51DB,0x5286,0x528E,0x52EE,0x5333,0x53B1,INVALC,0x5647,
	0x562D,0x5654,INVALC,0x564B,0x5652,0x5631,0x5644,0x5656,
	0x5650,0x562B,INVALC,0x564D,0x5637,0x564F,0x58A2,0x58B7,
	INVALC,0x58B2,INVALC,0x58AA,0x58B5,0x58B0,INVALC,0x58B4,
	0x58A4,0x58A7,INVALC,0x5926,0x5AFE,INVALC,0x5B04,INVALC,
	0x5AFC,INVALC,0x5B06,0x5B0A,0x5AFA,0x5B0D,0x5B00,0x5B0E,
	INVALC,INVALC,INVALC,0x5D91,INVALC,0x5D8F,0x5D90,0x5D98,
	0x5DA4,0x5D9B,0x5DA3,0x5D96,0x5DE4,0x5E5A,INVALC,INVALC,
	0x5E5E,INVALC,0x5FB8,0x6157,0x615C,0x61A6,0x6195,0x6188,
	INVALC,0x61A3,0x618F,INVALC,0x6164,INVALC,0x6159,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 77, Array index 0x2C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6178,INVALC,0x6185,0x6187,0x619E,INVALC,INVALC,
	0x6198,0x619C,INVALC,INVALC,0x622F,0x6480,0x649B,0x648E,
	0x648D,0x6494,0x64C6,INVALC,0x64A8,0x6483,INVALC,0x64B9,
	0x6486,0x64B4,0x64AF,0x6491,INVALC,0x64AA,0x64A1,0x64A7,
	0x66B6,0x66B3,INVALC,0x66BC,0x66AC,INVALC,0x66AD,0x6A0E,
	INVALC,0x6A1C,0x6A1A,INVALC,INVALC,0x6A0B,INVALC,0x69EF,
	0x6A0C,0x69F0,0x6A22,INVALC,0x69D8,INVALC,0x6A12,0x69FA,
	INVALC,0x6A2A,INVALC,0x6A10,INVALC,INVALC,0x6A29,0x69F9,
	0x69EA,0x6A2C,0x6A24,INVALC,0x69E9,0x6B52,0x6B4F,0x6B53,
	INVALC,INVALC,0x6F10,0x6F65,0x6F75,INVALC,INVALC,INVALC,
	INVALC,0x6FD0,INVALC,0x6F5C,0x6F3D,0x6F71,INVALC,0x6F91,
	0x6F0B,0x6F79,0x6F81,0x6F8F,INVALC,0x6F59,0x6F74,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 78, Array index 0x2D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x71AE,INVALC,0x71A3,0x71AD,INVALC,INVALC,
	0x71AB,0x71A6,0x71A2,INVALC,0x52F2,0x7257,0x7255,0x7299,
	0x734B,0x747A,INVALC,INVALC,INVALC,0x748C,0x7484,INVALC,
	INVALC,0x7482,0x7493,0x747B,INVALC,0x7509,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x778A,INVALC,0x7790,INVALC,
	0x78C6,0x78D3,0x78C0,0x78D2,0x78C7,0x78C2,INVALC,0x799F,
	0x799D,0x799E,INVALC,0x7A41,INVALC,0x7A38,0x7A3A,0x7A42,
	INVALC,INVALC,0x7A3E,0x7AB0,0x7BAE,0x7BB3,INVALC,INVALC,
	0x7BBF,INVALC,INVALC,0x7BCD,INVALC,0x7BB2,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7CC4,0x7CCD,0x7CC2,
	0x7CC6,0x7CC3,0x7CC9,0x7CC7,INVALC,0x7DF8,INVALC,0x7DED,
	0x7DE2,INVALC,INVALC,INVALC,0x7DDC,0x7E02,0x7E01,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 79, Array index 0x2E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7DD6,INVALC,0x7DE4,0x7DFE,INVALC,0x7E00,
	0x7DFC,0x7DFD,INVALC,0x7DF5,0x7DFF,INVALC,0x7DEB,0x7DE5,
	0x7F78,0x7FAE,0x7FE7,INVALC,0x8065,0x806A,0x8066,0x8068,
	0x806B,0x8194,0x81A1,0x8192,0x8196,0x8193,INVALC,INVALC,
	0x8501,INVALC,0x84F8,INVALC,0x84F5,INVALC,0x8504,INVALC,
	INVALC,INVALC,INVALC,0x851B,0x8503,0x8533,0x8534,0x84ED,
	INVALC,INVALC,0x8535,INVALC,0x8505,INVALC,INVALC,INVALC,
	INVALC,0x877D,INVALC,INVALC,INVALC,0x8771,INVALC,0x885C,
	0x88E6,0x890F,0x891B,INVALC,0x89A9,0x89A5,0x89EE,0x8AB1,
	INVALC,0x8ACC,0x8ACE,INVALC,0x8AB7,INVALC,0x8AB5,0x8AE9,
	0x8AB4,INVALC,0x8AB3,0x8AC1,0x8AAF,0x8ACA,0x8AD0,INVALC,
	INVALC,INVALC,0x8C8E,INVALC,INVALC,0x8CE9,0x8CDB,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 80, Array index 0x2F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x8CEB,0x8DA4,INVALC,0x8DA2,0x8D9D,INVALC,
	INVALC,INVALC,INVALC,0x8E2A,0x8E28,INVALC,INVALC,0x8EB8,
	0x8EB6,0x8EB9,0x8EB7,0x8F22,0x8F2B,0x8F27,0x8F19,0x8FA4,
	INVALC,0x8FB3,INVALC,0x9071,0x906A,INVALC,INVALC,0x9188,
	0x918C,0x92BF,0x92B8,0x92BE,0x92DC,0x92E5,INVALC,INVALC,
	0x92D4,0x92D6,INVALC,0x92DA,0x92ED,0x92F3,0x92DB,INVALC,
	0x92B9,0x92E2,0x92EB,0x95AF,INVALC,0x95B2,0x95B3,INVALC,
	INVALC,INVALC,0x96A3,0x96A5,INVALC,INVALC,INVALC,INVALC,
	0x970A,INVALC,0x9787,0x9789,0x978C,0x97EF,0x982A,0x9822,
	INVALC,0x981F,INVALC,0x9919,INVALC,0x99CA,0x99DA,INVALC,
	INVALC,INVALC,0x99DE,0x99C8,0x99E0,INVALC,0x9AB6,0x9AB5,
	INVALC,0x9AF4,INVALC,0x9B6B,0x9B69,0x9B72,0x9B63,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 81, Array index 0x3000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x9D0D,INVALC,0x9D01,0x9D0C,INVALC,0x9CF8,
	INVALC,INVALC,0x9CFE,0x9D02,0x9E84,INVALC,0x9EAB,0x9EAA,
	0x511D,0x5116,INVALC,0x512B,0x511E,0x511B,0x5290,0x5294,
	0x5314,INVALC,INVALC,0x5667,INVALC,0x567B,INVALC,0x565F,
	0x5661,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x58C3,0x58CA,0x58BB,0x58C0,0x58C4,0x5901,0x5B1F,0x5B18,
	0x5B11,0x5B15,INVALC,0x5B12,0x5B1C,INVALC,0x5B22,0x5B79,
	0x5DA6,INVALC,0x5DB3,0x5DAB,0x5EEA,INVALC,0x5F5B,INVALC,
	INVALC,0x61B7,0x61CE,0x61B9,0x61BD,0x61CF,0x61C0,0x6199,
	0x6197,INVALC,0x61BB,0x61D0,0x61C4,0x6231,INVALC,0x64D3,
	0x64C0,INVALC,INVALC,INVALC,INVALC,0x64DC,0x64D1,0x64C8,
	INVALC,0x64D5,0x66C3,INVALC,INVALC,0x66BF,0x66C5,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 82, Array index 0x3100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x66CD,0x66C1,0x6706,INVALC,0x6724,0x6A63,
	0x6A42,0x6A52,INVALC,0x6A43,0x6A33,INVALC,0x6A6C,0x6A57,
	INVALC,0x6A4C,0x6A6E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6A37,INVALC,0x6A71,0x6A4A,0x6A36,INVALC,0x6A53,INVALC,
	0x6A45,0x6A70,INVALC,INVALC,0x6A5C,0x6B58,0x6B57,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6FBB,INVALC,INVALC,
	0x6FBE,INVALC,INVALC,INVALC,0x6FB5,0x6FD3,0x6F9F,INVALC,
	0x6FB7,0x6FF5,0x71B7,INVALC,0x71BB,INVALC,0x71D1,INVALC,
	0x71BA,INVALC,0x71B6,0x71CC,INVALC,INVALC,0x71D3,0x749B,
	INVALC,INVALC,0x7496,0x74A2,0x749D,0x750A,0x750E,INVALC,
	0x7581,0x762C,0x7637,0x7636,0x763B,INVALC,0x76A1,INVALC,
	INVALC,0x7798,INVALC,0x7796,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 83, Array index 0x3200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x78D6,0x78EB,INVALC,0x78DC,INVALC,0x79A5,0x79A9,
	0x9834,0x7A53,0x7A45,INVALC,0x7A4F,INVALC,0x7ABD,0x7ABB,
	0x7AF1,INVALC,INVALC,0x7BEC,0x7BED,INVALC,INVALC,0x7CD3,
	INVALC,0x7CE1,INVALC,0x7E19,INVALC,INVALC,INVALC,0x7E27,
	0x7E26,INVALC,INVALC,0x806E,0x81AF,INVALC,INVALC,0x81AD,
	INVALC,0x81AA,0x8218,INVALC,INVALC,INVALC,INVALC,0x856F,
	0x854C,INVALC,0x8542,INVALC,0x855C,0x8570,0x855F,INVALC,
	0x855A,0x854B,0x853F,0x878A,INVALC,0x878B,0x87A1,0x878E,
	INVALC,INVALC,0x8799,0x885E,0x885F,0x8924,0x89A7,0x8AEA,
	0x8AFD,0x8AF9,0x8AE3,0x8AE5,INVALC,INVALC,0x8AEC,INVALC,
	INVALC,INVALC,INVALC,0x8CF2,INVALC,0x8CEF,INVALC,0x8DA6,
	INVALC,INVALC,INVALC,0x8E3B,0x8E43,INVALC,0x8E32,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 84, Array index 0x3300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8F31,0x8F30,INVALC,0x8F2D,0x8F3C,0x8FA7,0x8FA5,
	INVALC,INVALC,INVALC,0x9137,0x9195,0x918E,INVALC,0x9196,
	INVALC,0x9345,0x930A,INVALC,INVALC,0x92FD,0x9317,0x931C,
	0x9307,0x9331,0x9332,0x932C,0x9330,0x9303,0x9305,INVALC,
	0x95C2,INVALC,0x95B8,INVALC,0x95C1,INVALC,INVALC,INVALC,
	0x96AB,0x96B7,INVALC,INVALC,0x9715,0x9714,INVALC,INVALC,
	0x970C,0x9717,INVALC,0x9793,INVALC,0x97D2,INVALC,INVALC,
	0x9836,0x9831,0x9833,0x983C,0x982E,0x983A,INVALC,0x983D,
	INVALC,0x98B5,0x9922,0x9923,0x9920,0x991C,0x991D,INVALC,
	0x99A0,INVALC,0x99EF,0x99E8,0x99EB,INVALC,INVALC,INVALC,
	0x99E1,0x99E6,INVALC,INVALC,0x9AF8,0x9AF5,INVALC,INVALC,
	0x9B83,0x9B94,0x9B84,INVALC,0x9B8B,0x9B8F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 85, Array index 0x3400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9B8C,INVALC,0x9B89,INVALC,0x9B8E,INVALC,INVALC,
	INVALC,0x9D24,0x9D0F,INVALC,0x9D13,0x9D0A,INVALC,INVALC,
	INVALC,INVALC,0x9D2A,0x9D1A,INVALC,0x9D27,0x9D16,0x9D21,
	INVALC,0x9E85,0x9EAC,0x9EC6,0x9EC5,0x9ED7,0x9F53,INVALC,
	0x5128,0x5127,0x51DF,INVALC,0x5335,0x53B3,INVALC,0x568A,
	0x567D,0x5689,INVALC,0x58CD,0x58D0,INVALC,0x5B2B,0x5B33,
	0x5B29,0x5B35,0x5B31,0x5B37,0x5C36,0x5DBE,INVALC,0x5DB9,
	INVALC,0x5DBB,INVALC,0x61E2,0x61DB,0x61DD,0x61DC,0x61DA,
	INVALC,0x61D9,INVALC,INVALC,0x64DF,INVALC,INVALC,0x64E1,
	INVALC,0x64EE,INVALC,0x65B5,0x66D4,0x66D5,INVALC,0x66D0,
	0x66D1,0x66CE,0x66D7,INVALC,INVALC,0x6A7D,0x6A8A,INVALC,
	0x6AA7,INVALC,0x6A99,0x6A82,0x6A88,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 86, Array index 0x3500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A86,INVALC,0x6A98,0x6A9D,INVALC,INVALC,0x6A8F,
	INVALC,0x6AAA,INVALC,0x6B5D,INVALC,0x6C0A,INVALC,0x6FD7,
	0x6FD6,0x6FE5,INVALC,INVALC,INVALC,0x6FD9,0x6FDA,0x6FEA,
	INVALC,0x6FF6,INVALC,INVALC,0x71E3,INVALC,0x71E9,INVALC,
	0x71EB,0x71EF,0x71F3,0x71EA,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7371,INVALC,0x74AE,INVALC,0x74B3,INVALC,0x74AC,
	INVALC,INVALC,0x7583,0x7645,0x764E,0x7644,0x76A3,0x76A5,
	0x77A6,0x77A4,INVALC,0x77A9,0x77AF,INVALC,INVALC,INVALC,
	0x78F0,0x78F8,0x78F1,INVALC,0x7A49,INVALC,INVALC,INVALC,
	0x7AC2,0x7AF2,0x7AF3,0x7BFA,INVALC,0x7BF6,0x7BFC,0x7C18,
	0x7C08,0x7C12,INVALC,INVALC,0x7CDB,0x7CDA,INVALC,INVALC,
	INVALC,0x7E2C,0x7E4D,INVALC,INVALC,0x7F46,0x7FF6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 87, Array index 0x3600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x802B,0x8074,0x81B8,0x81C8,INVALC,INVALC,INVALC,
	0x8592,0x8593,INVALC,0x857F,0x85AB,0x8597,INVALC,INVALC,
	0x85AC,INVALC,INVALC,INVALC,0x87CE,INVALC,0x87CD,INVALC,
	INVALC,0x87C1,0x87B1,0x87C7,INVALC,0x8940,INVALC,0x893F,
	0x8939,INVALC,0x8943,INVALC,INVALC,INVALC,0x89AB,INVALC,
	0x8B1F,0x8B09,0x8B0C,INVALC,INVALC,0x8C40,INVALC,0x8C96,
	INVALC,0x8CF6,0x8CF7,INVALC,0x8E46,0x8E4F,INVALC,INVALC,
	INVALC,0x8F3D,0x8F41,0x9366,0x9378,0x935D,0x9369,0x9374,
	0x937D,0x936E,0x9372,0x9373,0x9362,0x9348,0x9353,0x935F,
	0x9368,INVALC,0x937F,0x936B,INVALC,0x95C4,INVALC,0x96AF,
	0x96AD,0x96B2,INVALC,INVALC,0x971A,0x971B,INVALC,INVALC,
	INVALC,INVALC,0x979B,0x979F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 88, Array index 0x3700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x9840,INVALC,
	0x9847,INVALC,0x98B7,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x99A2,INVALC,INVALC,0x9A00,0x99F3,INVALC,INVALC,0x99F5,
	INVALC,INVALC,0x9ABD,0x9B00,0x9B02,INVALC,0x9B34,0x9B49,
	0x9B9F,INVALC,0x9BA3,0x9BCD,0x9B99,0x9B9D,INVALC,INVALC,
	0x9D39,INVALC,0x9D44,INVALC,INVALC,0x9D35,INVALC,INVALC,
	0x9EAF,INVALC,0x512F,INVALC,INVALC,0x9F8E,INVALC,0x569F,
	0x569B,0x569E,0x5696,0x5694,0x56A0,INVALC,0x5B3B,INVALC,
	INVALC,0x5B3A,0x5DC1,0x5F4D,0x5F5D,0x61F3,INVALC,INVALC,
	INVALC,INVALC,0x64F6,0x64E5,0x64EA,0x64E7,0x6505,INVALC,
	0x64F9,INVALC,INVALC,INVALC,0x6AAB,0x6AED,0x6AB2,0x6AB0,
	0x6AB5,0x6ABE,0x6AC1,0x6AC8,INVALC,0x6AC0,0x6ABC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 89, Array index 0x3800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6AB1,0x6AC4,0x6ABF,INVALC,INVALC,0x7008,0x7003,
	0x6FFD,0x7010,0x7002,0x7013,INVALC,0x71FA,0x7200,0x74B9,
	0x74BC,INVALC,0x765B,0x7651,0x764F,0x76EB,0x77B8,INVALC,
	0x77B9,0x77C1,0x77C0,0x77BE,0x790B,INVALC,0x7907,0x790A,
	0x7908,INVALC,0x790D,0x7906,0x7915,0x79AF,INVALC,INVALC,
	INVALC,0x7AF5,INVALC,INVALC,0x7C2E,INVALC,0x7C1B,INVALC,
	0x7C1A,0x7C24,INVALC,INVALC,0x7CE6,0x7CE3,INVALC,INVALC,
	0x7E5D,0x7E4F,0x7E66,0x7E5B,0x7F47,0x7FB4,INVALC,INVALC,
	INVALC,0x7FFA,0x802E,INVALC,INVALC,0x81CE,INVALC,INVALC,
	0x8219,INVALC,INVALC,0x85CC,0x85B2,INVALC,0x85BB,0x85C1,
	INVALC,INVALC,INVALC,0x87E9,0x87EE,0x87F0,0x87D6,0x880E,
	0x87DA,0x8948,0x894A,0x894E,0x894D,0x89B1,0x89B0,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 90, Array index 0x3900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x89B3,INVALC,0x8B38,0x8B32,INVALC,0x8B2D,INVALC,
	0x8B34,INVALC,0x8B29,0x8C74,INVALC,INVALC,0x8D03,INVALC,
	INVALC,0x8DA9,0x8E58,INVALC,INVALC,0x8EBF,0x8EC1,0x8F4A,
	0x8FAC,INVALC,0x9089,0x913D,0x913C,0x91A9,0x93A0,INVALC,
	0x9390,INVALC,0x9393,0x938B,0x93AD,0x93BB,0x93B8,INVALC,
	INVALC,0x939C,0x95D8,0x95D7,INVALC,INVALC,INVALC,0x975D,
	0x97A9,0x97DA,INVALC,INVALC,INVALC,INVALC,0x9854,INVALC,
	0x9855,0x984B,INVALC,0x983F,0x98B9,INVALC,INVALC,INVALC,
	INVALC,0x9938,0x9936,0x9940,INVALC,0x993B,0x9939,0x99A4,
	INVALC,INVALC,0x9A08,0x9A0C,INVALC,0x9A10,INVALC,0x9B07,
	INVALC,0x9BD2,INVALC,0x9BC2,0x9BBB,0x9BCC,0x9BCB,INVALC,
	INVALC,0x9D4D,0x9D63,0x9D4E,INVALC,0x9D50,0x9D55,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 91, Array index 0x3A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x9D5E,INVALC,0x9E90,0x9EB2,0x9EB1,INVALC,
	0x9ECA,0x9F02,0x9F27,0x9F26,INVALC,0x56AF,0x58E0,0x58DC,
	INVALC,0x5B39,INVALC,INVALC,0x5B7C,0x5BF3,INVALC,INVALC,
	0x5C6B,0x5DC4,0x650B,0x6508,0x650A,INVALC,INVALC,0x65DC,
	INVALC,INVALC,0x66E1,0x66DF,0x6ACE,0x6AD4,0x6AE3,0x6AD7,
	0x6AE2,INVALC,INVALC,INVALC,INVALC,0x6AD8,0x6AD5,0x6AD2,
	INVALC,INVALC,0x701E,0x702C,0x7025,0x6FF3,0x7204,0x7208,
	0x7215,INVALC,0x74C4,0x74C9,0x74C7,0x74C8,0x76A9,0x77C6,
	0x77C5,0x7918,0x791A,0x7920,INVALC,0x7A66,0x7A64,0x7A6A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x7C35,0x7C34,
	INVALC,INVALC,0x7E6C,INVALC,0x7E6E,0x7E71,INVALC,0x81D4,
	0x81D6,0x821A,0x8262,0x8265,0x8276,0x85DB,0x85D6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 92, Array index 0x3B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x85E7,INVALC,INVALC,0x85F4,INVALC,0x87FD,
	0x87D5,0x8807,INVALC,0x880F,0x87F8,INVALC,INVALC,0x8987,
	INVALC,0x89B5,0x89F5,INVALC,0x8B3F,0x8B43,0x8B4C,INVALC,
	0x8D0B,0x8E6B,0x8E68,0x8E70,0x8E75,0x8E77,INVALC,0x8EC3,
	INVALC,0x93E9,0x93EA,0x93CB,0x93C5,0x93C6,INVALC,0x93ED,
	0x93D3,INVALC,0x93E5,INVALC,INVALC,0x93DB,0x93EB,0x93E0,
	0x93C1,INVALC,INVALC,0x95DD,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x97B2,0x97B4,0x97B1,
	0x97B5,0x97F2,INVALC,INVALC,INVALC,0x9856,INVALC,INVALC,
	INVALC,0x9944,INVALC,0x9A26,0x9A1F,0x9A18,0x9A21,0x9A17,
	INVALC,0x9B09,INVALC,INVALC,0x9BC5,0x9BDF,INVALC,0x9BE3,
	INVALC,0x9BE9,0x9BEE,INVALC,INVALC,0x9D66,0x9D7A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 93, Array index 0x3C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x9D6E,0x9D91,0x9D83,0x9D76,0x9D7E,0x9D6D,
	INVALC,0x9E95,0x9EE3,INVALC,INVALC,0x9F03,0x9F04,INVALC,
	0x9F17,INVALC,0x5136,INVALC,0x5336,INVALC,0x5B42,INVALC,
	INVALC,0x5B44,0x5B46,0x5B7E,0x5DCA,0x5DC8,0x5DCC,0x5EF0,
	INVALC,0x6585,0x66E5,0x66E7,INVALC,INVALC,INVALC,0x6AF4,
	INVALC,0x6AE9,INVALC,INVALC,INVALC,INVALC,INVALC,0x703D,
	INVALC,0x7036,INVALC,0x7216,INVALC,0x7212,0x720F,0x7217,
	0x7211,0x720B,INVALC,INVALC,0x74CD,0x74D0,0x74CC,0x74CE,
	0x74D1,INVALC,0x7589,INVALC,0x7A6F,0x7C4B,0x7C44,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x7E7F,0x8B71,INVALC,0x802F,
	0x807A,0x807B,0x807C,INVALC,INVALC,INVALC,0x85FC,0x8610,
	0x8602,INVALC,INVALC,0x85EE,0x8603,INVALC,0x860D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 94, Array index 0x3D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8613,0x8608,0x860F,0x8818,0x8812,INVALC,INVALC,
	0x8967,0x8965,0x89BB,0x8B69,0x8B62,INVALC,0x8B6E,INVALC,
	0x8B61,INVALC,0x8B64,0x8B4D,0x8C51,INVALC,INVALC,0x8E83,
	0x8EC6,INVALC,0x941F,INVALC,0x9404,0x9417,0x9408,0x9405,
	INVALC,0x93F3,0x941E,0x9402,0x941A,0x941B,0x9427,0x941C,
	INVALC,0x96B5,INVALC,INVALC,0x9733,INVALC,0x9734,0x9731,
	0x97B8,0x97BA,INVALC,0x97FC,INVALC,INVALC,0x98C3,INVALC,
	0x994D,INVALC,0x9A2F,INVALC,INVALC,INVALC,0x9AC9,INVALC,
	0x9AC8,0x9AC4,0x9B2A,0x9B38,0x9B50,INVALC,0x9C0A,0x9BFB,
	0x9C04,0x9BFC,0x9BFE,INVALC,INVALC,INVALC,0x9C02,0x9BF6,
	0x9C1B,0x9BF9,0x9C15,0x9C10,0x9BFF,0x9C00,0x9C0C,INVALC,
	INVALC,0x9D95,0x9DA5,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 95, Array index 0x3E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9E98,0x9EC1,INVALC,0x9F5A,0x5164,0x56BB,INVALC,
	0x58E6,0x5B49,0x5BF7,INVALC,INVALC,0x5DD0,INVALC,0x5FC2,
	INVALC,0x6511,INVALC,0x6AFF,0x6AFE,0x6AFD,INVALC,0x6B01,
	INVALC,INVALC,0x704B,0x704D,0x7047,0x74D3,0x7668,0x7667,
	INVALC,INVALC,0x77D1,0x7930,0x7932,0x792E,INVALC,0x9F9D,
	0x7AC9,0x7AC8,INVALC,0x7C56,0x7C51,INVALC,INVALC,INVALC,
	0x7E85,0x7E89,0x7E8E,0x7E84,INVALC,0x826A,0x862B,0x862F,
	0x8628,INVALC,0x8616,0x8615,0x861D,0x881A,INVALC,INVALC,
	INVALC,0x89BC,0x8B75,0x8B7C,INVALC,0x8D11,0x8D12,0x8F5C,
	0x91BB,INVALC,0x93F4,INVALC,INVALC,0x942D,INVALC,INVALC,
	0x96E4,0x9737,0x9736,0x9767,0x97BE,0x97BD,0x97E2,0x9868,
	0x9866,0x98C8,0x98CA,0x98C7,0x98DC,INVALC,0x994F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 96, Array index 0x3F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x99A9,0x9A3C,INVALC,0x9A3B,0x9ACE,INVALC,0x9B14,
	0x9B53,INVALC,0x9C2E,INVALC,0x9C1F,INVALC,INVALC,INVALC,
	INVALC,0x9DB0,0x9DBD,INVALC,INVALC,0x9DAE,0x9DC4,0x9E7B,
	INVALC,INVALC,0x9E9E,INVALC,0x9F05,INVALC,0x9F69,0x9FA1,
	0x56C7,0x571D,0x5B4A,0x5DD3,INVALC,0x5F72,0x6202,INVALC,
	0x6235,0x6527,0x651E,0x651F,INVALC,INVALC,0x6B07,0x6B06,
	INVALC,INVALC,0x7054,0x721C,0x7220,0x7AF8,INVALC,0x7C5D,
	0x7C58,INVALC,0x7E92,0x7F4E,INVALC,INVALC,INVALC,0x8827,
	INVALC,0x8B81,0x8B83,INVALC,0x8C44,INVALC,INVALC,INVALC,
	INVALC,0x9442,0x944D,0x9454,0x944E,INVALC,0x9443,INVALC,
	INVALC,0x973C,0x9740,0x97C0,INVALC,INVALC,INVALC,INVALC,
	0x995A,0x9A51,INVALC,0x9ADD,INVALC,INVALC,0x9C38,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 97, Array index 0x4000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x9C45,0x9C3A,INVALC,0x9C35,INVALC,INVALC,
	INVALC,0x9EF1,INVALC,0x9F93,0x529A,INVALC,INVALC,0x8641,
	0x5DD7,INVALC,0x6528,INVALC,INVALC,INVALC,0x7053,0x7059,
	INVALC,0x7221,INVALC,0x766F,0x7937,0x79B5,0x7C62,0x7C5E,
	0x7CF5,INVALC,INVALC,0x863D,INVALC,0x882D,0x8989,0x8B8D,
	0x8B87,0x8B90,0x8D1A,0x8E99,INVALC,INVALC,INVALC,0x945F,
	INVALC,INVALC,0x9456,0x9461,0x945B,0x945A,0x945C,0x9465,
	INVALC,0x9741,INVALC,INVALC,0x986E,0x986C,0x986D,INVALC,
	0x99AA,0x9A5C,0x9A58,0x9ADE,INVALC,0x9C4F,0x9C51,INVALC,
	0x9C53,INVALC,INVALC,INVALC,0x9DFC,0x9F39,INVALC,0x513E,
	INVALC,0x56D2,INVALC,0x5B4F,0x6B14,INVALC,0x7A72,0x7A73,
	INVALC,INVALC,INVALC,0x8B91,INVALC,INVALC,0x91BF,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 98, Array index 0x4100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x946C,INVALC,INVALC,0x96E6,0x9745,INVALC,
	0x97C8,0x97E4,0x995D,INVALC,0x9B21,INVALC,0x9B2C,0x9B57,
	INVALC,INVALC,0x9C5D,0x9C61,0x9C65,0x9E08,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x9F45,INVALC,INVALC,0x6205,0x66EF,
	0x6B1B,0x6B1D,0x7225,0x7224,0x7C6D,INVALC,0x8642,0x8649,
	INVALC,0x8978,0x898A,0x8B97,INVALC,0x8C9B,0x8D1C,INVALC,
	0x8EA2,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9C6C,INVALC,0x9C6F,INVALC,0x9E0E,INVALC,0x9F08,
	0x9F1D,0x9FA3,INVALC,INVALC,0x5F60,0x6B1C,INVALC,INVALC,
	INVALC,0x7CF3,INVALC,0x8B9B,0x8EA7,0x91C4,INVALC,0x947A,
	INVALC,INVALC,0x9A61,0x9A63,0x9AD7,0x9C76,INVALC,0x9FA5,
	INVALC,0x7067,INVALC,0x72AB,0x864A,0x897D,0x8B9D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 99, Array index 0x4200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8C53,0x8F65,0x947B,INVALC,0x98CD,0x98DD,INVALC,
	0x9B30,0x9E16,INVALC,INVALC,INVALC,INVALC,INVALC,0x96E7,
	0x9E18,0x9EA2,INVALC,0x9F7C,INVALC,0x7E9E,0x9484,INVALC,
	0x9E1C,INVALC,0x7C71,0x97CA,INVALC,INVALC,INVALC,0x9EA3,
	INVALC,0x9C7B,0x9F97,INVALC,INVALC,0x9750,INVALC,INVALC,
	INVALC,0x5727,0x5C13,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5FC8,INVALC,INVALC,INVALC,INVALC,INVALC,0x6765,
	INVALC,INVALC,0x52BD,INVALC,0x5B66,INVALC,0x65F9,0x6788,
	0x6CE6,0x6CCB,INVALC,0x4FBD,0x5F8D,INVALC,0x6018,0x6048,
	INVALC,0x6B29,0x70A6,INVALC,0x7706,INVALC,INVALC,INVALC,
	0x5A10,0x5CFC,0x5CFE,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x70C9,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 100, Array index 0x4300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x9579,INVALC,0x96BA,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x7B29,0x8128,INVALC,
	0x8A2E,INVALC,INVALC,INVALC,0x9AD9,INVALC,0x582B,0x5845,
	INVALC,0x63FA,INVALC,INVALC,INVALC,0x6E86,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5867,INVALC,0x5BDD,0x656E,INVALC,
	INVALC,INVALC,0x8C87,INVALC,0x50D2,0x50DF,INVALC,INVALC,
	INVALC,INVALC,0x69BA,INVALC,0x6B9D,INVALC,0x8059,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6F8A,INVALC,INVALC,0x7BC3,0x7BC2,
	INVALC,INVALC,INVALC,INVALC,0x90F6,INVALC,0x9823,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x71CD,0x7499,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x9842,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 101, Array index 0x4400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x7F84,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x8D0E,INVALC,0x9861,INVALC,INVALC,0x8B73,INVALC,0x9C27,
	INVALC,0x9458,0x77D6,0x9B2D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4F66,0x4F68,0x4FE7,0x503F,INVALC,0x50A6,0x510F,0x523E,
	0x5324,0x5365,0x539B,0x517F,0x54CB,0x5573,0x5571,0x556B,
	0x55F4,0x5622,0x5620,0x5692,0x56BA,0x5691,0x56B0,0x5759,
	0x578A,0x580F,0x5812,0x5813,0x5847,0x589B,0x5900,0x594D,
	0x5AD1,0x5AD3,0x5B67,0x5C57,0x5C77,0x5CD5,0x5D75,0x5D8E,
	0x5DA5,0x5DB6,0x5DBF,0x5E65,0x5ECD,0x5EED,0x5F94,0x5F9A,
	0x5FBA,0x6125,0x6150,0x62A3,0x6360,0x6364,0x63B6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 102, Array index 0x4500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6403,0x64B6,0x651A,0x7A25,0x5C21,0x66E2,0x6702,
	0x67A4,0x67AC,0x6810,0x6806,0x685E,0x685A,0x692C,0x6929,
	0x6A2D,0x6A77,0x6A7A,0x6ACA,0x6AE6,0x6AF5,0x6B0D,0x6B0E,
	0x6BDC,0x6BDD,0x6BF6,0x6C1E,0x6C63,0x6DA5,0x6E0F,0x6E8A,
	0x6E84,0x6E8B,0x6E7C,0x6F4C,0x6F48,0x6F49,0x6F9D,0x6F99,
	0x6FF8,0x702E,0x702D,0x705C,0x79CC,0x70BF,0x70EA,0x70E5,
	0x7111,0x7112,0x713F,0x7139,0x713B,0x713D,0x7177,0x7175,
	0x7176,0x7171,0x7196,0x7193,0x71B4,0x71DD,0x71DE,0x720E,
	0x5911,0x7218,0x7347,0x7348,0x73EF,0x7412,0x743B,0x74A4,
	0x748D,0x74B4,0x7673,0x7677,0x76BC,0x7819,0x781B,0x783D,
	0x7853,0x7854,0x7858,0x78B7,0x78D8,0x78EE,0x7922,0x794D,
	0x7986,0x7999,0x79A3,0x79BC,0x7AA7,0x7B37,0x7B59,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 103, Array index 0x4600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7BD0,0x7C2F,0x7C32,0x7C42,0x7C4E,0x7C68,0x7CA9,
	0x7CED,0x7DD0,0x7E07,0x7DD3,0x7E64,0x7F40,INVALC,0x8041,
	0x8063,0x80BB,0x6711,0x6725,0x8248,0x8310,0x8362,0x8312,
	0x8421,0x841E,0x84E2,0x84DE,0x84E1,0x8573,0x85D4,0x85F5,
	0x8637,0x8645,0x8672,0x874A,0x87A9,0x87A5,0x87F5,0x8834,
	0x8850,0x8887,0x8954,0x8984,0x8B03,0x8C52,0x8CD8,0x8D0C,
	0x8D18,0x8DB0,0x8EBC,0x8ED5,0x8FAA,0x909C,INVALC,0x915C,
	0x922B,0x9221,0x9273,0x92F4,0x92F5,0x933F,0x9342,0x9386,
	0x93BE,0x93BC,0x93BD,0x93F1,0x93F2,0x93EF,0x9422,0x9423,
	0x9424,0x9467,0x9466,0x9597,0x95CE,0x95E7,0x973B,0x974D,
	0x98E4,0x9942,0x9B1D,0x9B98,INVALC,0x9D49,0x6449,0x5E71,
	0x5E85,0x61D3,0x990E,0x8002,0x781E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 104, Array index 0x4700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5528,0x5572,0x55BA,0x55F0,0x55EE,0x56B8,0x56B9,
	0x56C4,0x8053,0x92B0,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
};

#endif /* ICONV_TO_UCS_CCS_CNS11643_PLANE14 && !defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit cns11643_plane14 -> UCS size-optimized table (12822 bytes).
 * ======================================================================
 */
#if defined (ICONV_TO_UCS_CCS_CNS11643_PLANE14) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
to_ucs_size_cns11643_plane14[] =
{
	0x00DE, /* Ranges number */
	0x01A6, /* Unranged codes number */
	0x15BF, /* First unranged code index */
	/* Ranges list: first code, last Code, array index. */
	/* Array index: 0x0003 */ 0x2121, 0x216D, 0x029D,
	/* Array index: 0x0006 */ 0x2177, 0x217D, 0x02EA,
	/* Array index: 0x0009 */ 0x2222, 0x223F, 0x02F1,
	/* Array index: 0x000C */ 0x2247, 0x227E, 0x030F,
	/* Array index: 0x000F */ 0x2322, 0x2347, 0x0347,
	/* Array index: 0x0012 */ 0x2350, 0x2355, 0x036D,
	/* Array index: 0x0015 */ 0x2369, 0x237E, 0x0373,
	/* Array index: 0x0018 */ 0x2423, 0x242F, 0x0389,
	/* Array index: 0x001B */ 0x2438, 0x2454, 0x0396,
	/* Array index: 0x001E */ 0x2457, 0x2467, 0x03B3,
	/* Array index: 0x0021 */ 0x2473, 0x247E, 0x03C4,
	/* Array index: 0x0024 */ 0x2521, 0x2538, 0x03D0,
	/* Array index: 0x0027 */ 0x253E, 0x254E, 0x03E8,
	/* Array index: 0x002A */ 0x255A, 0x2568, 0x03F9,
	/* Array index: 0x002D */ 0x256B, 0x257D, 0x0408,
	/* Array index: 0x0030 */ 0x2621, 0x262A, 0x041B,
	/* Array index: 0x0033 */ 0x262E, 0x2639, 0x0425,
	/* Array index: 0x0036 */ 0x2641, 0x266B, 0x0431,
	/* Array index: 0x0039 */ 0x2673, 0x267E, 0x045C,
	/* Array index: 0x003C */ 0x2721, 0x2752, 0x0468,
	/* Array index: 0x003F */ 0x2759, 0x277D, 0x049A,
	/* Array index: 0x0042 */ 0x2821, 0x282F, 0x04BF,
	/* Array index: 0x0045 */ 0x283D, 0x2841, 0x04CE,
	/* Array index: 0x0048 */ 0x2848, 0x287A, 0x04D3,
	/* Array index: 0x004B */ 0x2925, 0x295A, 0x0506,
	/* Array index: 0x004E */ 0x2963, 0x2979, 0x053C,
	/* Array index: 0x0051 */ 0x2A21, 0x2A3F, 0x0553,
	/* Array index: 0x0054 */ 0x2A47, 0x2A4F, 0x0572,
	/* Array index: 0x0057 */ 0x2A54, 0x2A60, 0x057B,
	/* Array index: 0x005A */ 0x2A70, 0x2A7E, 0x0588,
	/* Array index: 0x005D */ 0x2B21, 0x2B40, 0x0597,
	/* Array index: 0x0060 */ 0x2B46, 0x2B5E, 0x05B7,
	/* Array index: 0x0063 */ 0x2B63, 0x2B72, 0x05D0,
	/* Array index: 0x0066 */ 0x2C21, 0x2C2F, 0x05E0,
	/* Array index: 0x0069 */ 0x2C3B, 0x2C4D, 0x05EF,
	/* Array index: 0x006C */ 0x2C56, 0x2C63, 0x0602,
	/* Array index: 0x006F */ 0x2C6B, 0x2C7E, 0x0610,
	/* Array index: 0x0072 */ 0x2D21, 0x2D3D, 0x0624,
	/* Array index: 0x0075 */ 0x2D45, 0x2D7E, 0x0641,
	/* Array index: 0x0078 */ 0x2E21, 0x2E2B, 0x067B,
	/* Array index: 0x007B */ 0x2E32, 0x2E54, 0x0686,
	/* Array index: 0x007E */ 0x2E5D, 0x2E6F, 0x06A9,
	/* Array index: 0x0081 */ 0x2E74, 0x2E7E, 0x06BC,
	/* Array index: 0x0084 */ 0x2F23, 0x2F6F, 0x06C7,
	/* Array index: 0x0087 */ 0x2F73, 0x2F7B, 0x0714,
	/* Array index: 0x008A */ 0x3027, 0x302B, 0x071D,
	/* Array index: 0x008D */ 0x302F, 0x3040, 0x0722,
	/* Array index: 0x0090 */ 0x304A, 0x304F, 0x0734,
	/* Array index: 0x0093 */ 0x3055, 0x307B, 0x073A,
	/* Array index: 0x0096 */ 0x3121, 0x3129, 0x0761,
	/* Array index: 0x0099 */ 0x312C, 0x313D, 0x076A,
	/* Array index: 0x009C */ 0x3140, 0x317D, 0x077C,
	/* Array index: 0x009F */ 0x3221, 0x3231, 0x07BA,
	/* Array index: 0x00A2 */ 0x3238, 0x3241, 0x07CB,
	/* Array index: 0x00A5 */ 0x3246, 0x3248, 0x07D5,
	/* Array index: 0x00A8 */ 0x324D, 0x3252, 0x07D8,
	/* Array index: 0x00AB */ 0x325A, 0x327C, 0x07DE,
	/* Array index: 0x00AE */ 0x3323, 0x3332, 0x0801,
	/* Array index: 0x00B1 */ 0x333B, 0x334F, 0x0811,
	/* Array index: 0x00B4 */ 0x3355, 0x335A, 0x0826,
	/* Array index: 0x00B7 */ 0x3369, 0x337C, 0x082C,
	/* Array index: 0x00BA */ 0x3425, 0x3436, 0x0840,
	/* Array index: 0x00BD */ 0x343C, 0x347D, 0x0852,
	/* Array index: 0x00C0 */ 0x3527, 0x353C, 0x0894,
	/* Array index: 0x00C3 */ 0x3543, 0x355B, 0x08AA,
	/* Array index: 0x00C6 */ 0x3560, 0x3563, 0x08C3,
	/* Array index: 0x00C9 */ 0x356E, 0x3576, 0x08C7,
	/* Array index: 0x00CC */ 0x3622, 0x367D, 0x08D0,
	/* Array index: 0x00CF */ 0x3721, 0x373D, 0x092C,
	/* Array index: 0x00D2 */ 0x3744, 0x374A, 0x0949,
	/* Array index: 0x00D5 */ 0x3750, 0x3759, 0x0950,
	/* Array index: 0x00D8 */ 0x375D, 0x377E, 0x095A,
	/* Array index: 0x00DB */ 0x3839, 0x3842, 0x097C,
	/* Array index: 0x00DE */ 0x3845, 0x3858, 0x0986,
	/* Array index: 0x00E1 */ 0x385B, 0x387B, 0x099A,
	/* Array index: 0x00E4 */ 0x3929, 0x3941, 0x09BB,
	/* Array index: 0x00E7 */ 0x394A, 0x3959, 0x09D4,
	/* Array index: 0x00EA */ 0x3967, 0x397E, 0x09E4,
	/* Array index: 0x00ED */ 0x3A27, 0x3A3C, 0x09FC,
	/* Array index: 0x00F0 */ 0x3A40, 0x3A63, 0x0A12,
	/* Array index: 0x00F3 */ 0x3A75, 0x3A7E, 0x0A36,
	/* Array index: 0x00F6 */ 0x3B24, 0x3B2E, 0x0A40,
	/* Array index: 0x00F9 */ 0x3B33, 0x3B47, 0x0A4B,
	/* Array index: 0x00FC */ 0x3B4D, 0x3B68, 0x0A60,
	/* Array index: 0x00FF */ 0x3B6D, 0x3B76, 0x0A7C,
	/* Array index: 0x0102 */ 0x3B7C, 0x3B7E, 0x0A86,
	/* Array index: 0x0105 */ 0x3C23, 0x3C2A, 0x0A89,
	/* Array index: 0x0108 */ 0x3C2F, 0x3C5D, 0x0A91,
	/* Array index: 0x010B */ 0x3C60, 0x3C74, 0x0AC0,
	/* Array index: 0x010E */ 0x3C77, 0x3C7D, 0x0AD5,
	/* Array index: 0x0111 */ 0x3D21, 0x3D47, 0x0ADC,
	/* Array index: 0x0114 */ 0x3D4F, 0x3D69, 0x0B03,
	/* Array index: 0x0117 */ 0x3D70, 0x3D7B, 0x0B1E,
	/* Array index: 0x011A */ 0x3E21, 0x3E63, 0x0B2A,
	/* Array index: 0x011D */ 0x3E68, 0x3E6A, 0x0B6D,
	/* Array index: 0x0120 */ 0x3E7C, 0x3E7E, 0x0B70,
	/* Array index: 0x0123 */ 0x3F21, 0x3F24, 0x0B73,
	/* Array index: 0x0126 */ 0x3F29, 0x3F59, 0x0B77,
	/* Array index: 0x0129 */ 0x3F62, 0x3F69, 0x0BA8,
	/* Array index: 0x012C */ 0x3F70, 0x3F75, 0x0BB0,
	/* Array index: 0x012F */ 0x4036, 0x4061, 0x0BB6,
	/* Array index: 0x0132 */ 0x4071, 0x407D, 0x0BE2,
	/* Array index: 0x0135 */ 0x4121, 0x413E, 0x0BEF,
	/* Array index: 0x0138 */ 0x414D, 0x4151, 0x0C0D,
	/* Array index: 0x013B */ 0x4172, 0x417E, 0x0C12,
	/* Array index: 0x013E */ 0x4226, 0x4233, 0x0C1F,
	/* Array index: 0x0141 */ 0x4239, 0x4249, 0x0C2D,
	/* Array index: 0x0144 */ 0x424E, 0x427E, 0x0C3E,
	/* Array index: 0x0147 */ 0x4325, 0x4339, 0x0C6F,
	/* Array index: 0x014A */ 0x4346, 0x4355, 0x0C84,
	/* Array index: 0x014D */ 0x4360, 0x436A, 0x0C94,
	/* Array index: 0x0150 */ 0x4372, 0x4377, 0x0C9F,
	/* Array index: 0x0153 */ 0x4422, 0x444F, 0x0CA5,
	/* Array index: 0x0156 */ 0x4454, 0x446B, 0x0CD3,
	/* Array index: 0x0159 */ 0x4476, 0x447E, 0x0CEB,
	/* Array index: 0x015C */ 0x4523, 0x453C, 0x0CF4,
	/* Array index: 0x015F */ 0x4557, 0x455C, 0x0D0E,
	/* Array index: 0x0162 */ 0x455F, 0x4576, 0x0D14,
	/* Array index: 0x0165 */ 0x4625, 0x4664, 0x0D2C,
	/* Array index: 0x0168 */ 0x4668, 0x467B, 0x0D6C,
	/* Array index: 0x016B */ 0x4721, 0x472A, 0x0D80,
	/* Array index: 0x016E */ 0x4733, 0x474E, 0x0D8A,
	/* Array index: 0x0171 */ 0x4753, 0x4772, 0x0DA6,
	/* Array index: 0x0174 */ 0x4775, 0x477E, 0x0DC6,
	/* Array index: 0x0177 */ 0x4821, 0x482B, 0x0DD0,
	/* Array index: 0x017A */ 0x4830, 0x4840, 0x0DDB,
	/* Array index: 0x017D */ 0x4844, 0x4848, 0x0DEC,
	/* Array index: 0x0180 */ 0x4852, 0x4862, 0x0DF1,
	/* Array index: 0x0183 */ 0x486D, 0x4879, 0x0E02,
	/* Array index: 0x0186 */ 0x4922, 0x492E, 0x0E0F,
	/* Array index: 0x0189 */ 0x4932, 0x4959, 0x0E1C,
	/* Array index: 0x018C */ 0x495C, 0x4968, 0x0E44,
	/* Array index: 0x018F */ 0x4970, 0x497D, 0x0E51,
	/* Array index: 0x0192 */ 0x4A26, 0x4A39, 0x0E5F,
	/* Array index: 0x0195 */ 0x4A44, 0x4A49, 0x0E73,
	/* Array index: 0x0198 */ 0x4A52, 0x4A56, 0x0E79,
	/* Array index: 0x019B */ 0x4A5D, 0x4A66, 0x0E7E,
	/* Array index: 0x019E */ 0x4A6C, 0x4A75, 0x0E88,
	/* Array index: 0x01A1 */ 0x4B29, 0x4B5F, 0x0E92,
	/* Array index: 0x01A4 */ 0x4B63, 0x4B6D, 0x0EC9,
	/* Array index: 0x01A7 */ 0x4B70, 0x4B7E, 0x0ED4,
	/* Array index: 0x01AA */ 0x4C21, 0x4C4A, 0x0EE3,
	/* Array index: 0x01AD */ 0x4C4D, 0x4C6C, 0x0F0D,
	/* Array index: 0x01B0 */ 0x4C71, 0x4C7E, 0x0F2D,
	/* Array index: 0x01B3 */ 0x4D24, 0x4D31, 0x0F3B,
	/* Array index: 0x01B6 */ 0x4D35, 0x4D3D, 0x0F49,
	/* Array index: 0x01B9 */ 0x4D46, 0x4D5D, 0x0F52,
	/* Array index: 0x01BC */ 0x4D6D, 0x4D7E, 0x0F6A,
	/* Array index: 0x01BF */ 0x4E24, 0x4E3D, 0x0F7C,
	/* Array index: 0x01C2 */ 0x4E4B, 0x4E4F, 0x0F96,
	/* Array index: 0x01C5 */ 0x4E5D, 0x4E76, 0x0F9B,
	/* Array index: 0x01C8 */ 0x4F22, 0x4F26, 0x0FB5,
	/* Array index: 0x01CB */ 0x4F2B, 0x4F56, 0x0FBA,
	/* Array index: 0x01CE */ 0x4F60, 0x4F7E, 0x0FE6,
	/* Array index: 0x01D1 */ 0x502A, 0x5038, 0x1005,
	/* Array index: 0x01D4 */ 0x5048, 0x5070, 0x1014,
	/* Array index: 0x01D7 */ 0x5075, 0x507E, 0x103D,
	/* Array index: 0x01DA */ 0x5122, 0x5132, 0x1047,
	/* Array index: 0x01DD */ 0x5138, 0x5146, 0x1058,
	/* Array index: 0x01E0 */ 0x5154, 0x5176, 0x1067,
	/* Array index: 0x01E3 */ 0x5224, 0x5234, 0x108A,
	/* Array index: 0x01E6 */ 0x5252, 0x526B, 0x109B,
	/* Array index: 0x01E9 */ 0x5321, 0x5344, 0x10B5,
	/* Array index: 0x01EC */ 0x5358, 0x536C, 0x10D9,
	/* Array index: 0x01EF */ 0x5374, 0x537D, 0x10EE,
	/* Array index: 0x01F2 */ 0x5429, 0x542D, 0x10F8,
	/* Array index: 0x01F5 */ 0x5432, 0x5461, 0x10FD,
	/* Array index: 0x01F8 */ 0x5469, 0x547C, 0x112D,
	/* Array index: 0x01FB */ 0x552D, 0x5539, 0x1141,
	/* Array index: 0x01FE */ 0x553E, 0x5543, 0x114E,
	/* Array index: 0x0201 */ 0x5552, 0x5575, 0x1154,
	/* Array index: 0x0204 */ 0x5621, 0x5624, 0x1178,
	/* Array index: 0x0207 */ 0x5628, 0x562D, 0x117C,
	/* Array index: 0x020A */ 0x5639, 0x5642, 0x1182,
	/* Array index: 0x020D */ 0x5646, 0x564A, 0x118C,
	/* Array index: 0x0210 */ 0x5651, 0x5675, 0x1191,
	/* Array index: 0x0213 */ 0x573A, 0x5745, 0x11B6,
	/* Array index: 0x0216 */ 0x5755, 0x5765, 0x11C2,
	/* Array index: 0x0219 */ 0x576A, 0x577E, 0x11D3,
	/* Array index: 0x021C */ 0x5821, 0x5845, 0x11E8,
	/* Array index: 0x021F */ 0x5854, 0x585D, 0x120D,
	/* Array index: 0x0222 */ 0x586B, 0x587E, 0x1217,
	/* Array index: 0x0225 */ 0x5931, 0x5951, 0x122B,
	/* Array index: 0x0228 */ 0x5958, 0x595C, 0x124C,
	/* Array index: 0x022B */ 0x5961, 0x597E, 0x1251,
	/* Array index: 0x022E */ 0x5A22, 0x5A3C, 0x126F,
	/* Array index: 0x0231 */ 0x5A42, 0x5A48, 0x128A,
	/* Array index: 0x0234 */ 0x5A4D, 0x5A67, 0x1291,
	/* Array index: 0x0237 */ 0x5A74, 0x5A7E, 0x12AC,
	/* Array index: 0x023A */ 0x5B25, 0x5B2C, 0x12B7,
	/* Array index: 0x023D */ 0x5B31, 0x5B50, 0x12BF,
	/* Array index: 0x0240 */ 0x5B5D, 0x5B61, 0x12DF,
	/* Array index: 0x0243 */ 0x5B69, 0x5B7E, 0x12E4,
	/* Array index: 0x0246 */ 0x5C22, 0x5C43, 0x12FA,
	/* Array index: 0x0249 */ 0x5C53, 0x5C66, 0x131C,
	/* Array index: 0x024C */ 0x5C6C, 0x5C7E, 0x1330,
	/* Array index: 0x024F */ 0x5D21, 0x5D49, 0x1343,
	/* Array index: 0x0252 */ 0x5D4C, 0x5D53, 0x136C,
	/* Array index: 0x0255 */ 0x5D5E, 0x5D7A, 0x1374,
	/* Array index: 0x0258 */ 0x5E21, 0x5E2A, 0x1391,
	/* Array index: 0x025B */ 0x5E31, 0x5E6A, 0x139B,
	/* Array index: 0x025E */ 0x5E70, 0x5E7E, 0x13D5,
	/* Array index: 0x0261 */ 0x5F21, 0x5F2C, 0x13E4,
	/* Array index: 0x0264 */ 0x5F31, 0x5F37, 0x13F0,
	/* Array index: 0x0267 */ 0x5F3C, 0x5F5B, 0x13F7,
	/* Array index: 0x026A */ 0x5F69, 0x5F73, 0x1417,
	/* Array index: 0x026D */ 0x6039, 0x6040, 0x1422,
	/* Array index: 0x0270 */ 0x6043, 0x604B, 0x142A,
	/* Array index: 0x0273 */ 0x6052, 0x6068, 0x1433,
	/* Array index: 0x0276 */ 0x6073, 0x6077, 0x144A,
	/* Array index: 0x0279 */ 0x6125, 0x6135, 0x144F,
	/* Array index: 0x027C */ 0x613E, 0x6150, 0x1460,
	/* Array index: 0x027F */ 0x615D, 0x6165, 0x1473,
	/* Array index: 0x0282 */ 0x6169, 0x617E, 0x147C,
	/* Array index: 0x0285 */ 0x6221, 0x6229, 0x1492,
	/* Array index: 0x0288 */ 0x622F, 0x623B, 0x149B,
	/* Array index: 0x028B */ 0x625C, 0x6272, 0x14A8,
	/* Array index: 0x028E */ 0x642F, 0x6433, 0x14BF,
	/* Array index: 0x0291 */ 0x6448, 0x647E, 0x14C4,
	/* Array index: 0x0294 */ 0x6521, 0x657E, 0x14FB,
	/* Array index: 0x0297 */ 0x6621, 0x667C, 0x1559,
	/* Array index: 0x029A */ 0x6721, 0x672A, 0x15B5,
	/* Ranges content */
	/* Range 0x2121 - 0x216D, array index: 0x029D */
	0x4E28,0x4E36,0x4E3F,0x4E85,0x4E05,0x4E04,0x5182,0x5196,
	0x5338,0x5369,0x53B6,0x4E2A,0x4E87,0x4E49,0x51E2,0x4E46,
	0x4E8F,0x4EBC,0x4EBE,0x5166,0x51E3,0x5204,0x529C,INVALC,
	0x5902,0x590A,0x5B80,0x5DDB,0x5E7A,0x5E7F,0x5EF4,0x5F50,
	0x5F51,0x5F61,0x961D,INVALC,0x4E63,0x4E62,0x4EA3,0x5185,
	0x4EC5,0x4ECF,0x4ECE,0x4ECC,0x5184,0x5186,INVALC,INVALC,
	0x51E4,0x5205,0x529E,0x529D,0x52FD,0x5300,0x533A,INVALC,
	0x5346,0x535D,0x5386,0x53B7,INVALC,0x53CC,INVALC,0x53CE,
	0x5721,INVALC,0x5E00,0x5F0C,0x6237,0x6238,0x6534,0x6535,
	0x65E0,INVALC,0x738D,0x4E97,0x4EE0,
	/* Range 0x2177 - 0x217D, array index: 0x02EA */
	0x56D8,0x518B,0x518C,0x5199,0x51E5,INVALC,0x520B,
	/* Range 0x2222 - 0x223F, array index: 0x02F1 */
	0x5304,0x5303,0x5307,INVALC,0x531E,0x535F,0x536D,0x5389,
	0x53BA,0x53D0,INVALC,0x53F6,0x53F7,0x53F9,INVALC,0x53F4,
	INVALC,INVALC,0x5724,0x5904,0x5918,0x5932,0x5930,0x5934,
	INVALC,0x5975,INVALC,0x5B82,0x5BF9,0x5C14,
	/* Range 0x2247 - 0x227E, array index: 0x030F */
	0x5E81,0x5E83,0x5F0D,0x5F52,INVALC,0x5FCA,0x5FC7,0x6239,
	INVALC,0x624F,0x65E7,0x672F,0x6B7A,0x6C39,INVALC,INVALC,
	0x6C37,0x6C44,0x6C45,0x738C,0x7592,0x7676,0x9093,0x9092,
	INVALC,INVALC,0x4E21,0x4E20,0x4E22,0x4E68,0x4E89,0x4E98,
	0x4EF9,0x4EEF,INVALC,INVALC,0x4EF8,0x4F06,0x4F03,0x4EFC,
	0x4EEE,0x4F16,INVALC,0x4F28,0x4F1C,0x4F07,0x4F1A,0x4EFA,
	0x4F17,0x514A,INVALC,0x5172,INVALC,0x51B4,0x51B3,0x51B2,
	/* Range 0x2322 - 0x2347, array index: 0x0347 */
	0x51E8,INVALC,0x5214,0x520F,0x5215,0x5218,0x52A8,INVALC,
	0x534B,0x534F,INVALC,0x5350,INVALC,0x538B,INVALC,0x53BE,
	INVALC,0x53D2,0x5416,0x53FF,INVALC,0x5400,INVALC,0x5405,
	0x5413,0x5415,INVALC,INVALC,0x56E3,0x5735,0x5736,0x5731,
	0x5732,0x58EE,0x5905,0x4E54,INVALC,0x5936,
	/* Range 0x2350 - 0x2355, array index: 0x036D */
	0x5B86,0x5F53,0x5C18,INVALC,0x5C3D,0x5C78,
	/* Range 0x2369 - 0x237E, array index: 0x0373 */
	0x5FDB,INVALC,0x620F,0x625D,0x625F,0x6267,0x6257,0x9F50,
	INVALC,0x65EB,0x65EA,INVALC,0x6737,INVALC,0x6732,0x6736,
	0x6B22,0x6BCE,INVALC,0x6C58,0x6C51,0x6C77,
	/* Range 0x2423 - 0x242F, array index: 0x0389 */
	0x6C5A,INVALC,0x6C53,0x706F,0x7072,0x706E,INVALC,INVALC,
	0x7073,0x72B1,0x72B2,INVALC,0x738F,
	/* Range 0x2438 - 0x2454, array index: 0x0396 */
	0x827B,INVALC,0x8D71,0x8FB9,0x9096,0x909A,INVALC,0x4E24,
	0x4E71,INVALC,0x4E9C,0x4F45,0x4F4A,0x4F39,0x4F37,INVALC,
	0x4F32,0x4F42,INVALC,0x4F44,0x4F4B,INVALC,0x4F40,0x4F35,
	0x4F31,0x5151,INVALC,0x5150,0x514E,
	/* Range 0x2457 - 0x2467, array index: 0x03B3 */
	0x519D,INVALC,0x51B5,0x51B8,0x51EC,0x5223,0x5227,0x5226,
	0x521F,0x522B,0x5220,0x52B4,0x52B3,INVALC,0x5325,0x533B,
	0x5374,
	/* Range 0x2473 - 0x247E, array index: 0x03C4 */
	0x5444,0x544C,0x5423,0x541A,0x5432,0x544B,0x5421,INVALC,
	0x5434,0x5449,0x5450,0x5422,
	/* Range 0x2521 - 0x2538, array index: 0x03D0 */
	0x543F,0x5451,0x545A,0x542F,INVALC,0x56E9,0x56F2,0x56F3,
	0x56EF,0x56ED,0x56EC,0x56E6,0x5748,INVALC,0x5744,0x573F,
	0x573C,0x5753,0x5756,INVALC,0x575F,0x5743,0x5758,0x5757,
	/* Range 0x253E - 0x254E, array index: 0x03E8 */
	0x573D,INVALC,0x5742,0x5754,0x5755,0x58F1,0x58F2,0x58F0,
	0x590B,0x9EA6,0x56F1,0x593D,INVALC,0x5994,0x598C,INVALC,
	0x599C,
	/* Range 0x255A - 0x2568, array index: 0x03F9 */
	0x5B8D,INVALC,0x5BFE,0x5BFF,0x5BFD,0x5C2B,INVALC,0x5C84,
	0x5C8E,0x5C9C,INVALC,INVALC,0x5C85,0x5DF5,0x5E09,
	/* Range 0x256B - 0x257D, array index: 0x0408 */
	0x5E0B,INVALC,0x5E92,0x5E90,0x5F03,INVALC,0x5F1E,0x5F63,
	INVALC,0x5FE7,0x5FFE,0x5FE6,0x5FDC,0x5FCE,INVALC,0x5FFC,
	0x5FDF,0x5FEC,0x5FF6,
	/* Range 0x2621 - 0x262A, array index: 0x041B */
	0x5FF2,0x5FF0,0x5FF9,INVALC,0x6213,INVALC,INVALC,0x623B,
	0x623C,0x6282,
	/* Range 0x262E - 0x2639, array index: 0x0425 */
	0x6278,0x628B,INVALC,0x629E,0x62A5,0x629B,0x629C,0x6299,
	0x628D,0x6285,0x629D,0x6275,
	/* Range 0x2641 - 0x266B, array index: 0x0431 */
	0x66F5,0x675B,INVALC,0x6754,0x6752,INVALC,0x6758,0x6744,
	0x674A,0x6761,INVALC,0x6C7F,0x6C91,0x6C9E,INVALC,0x6C6E,
	0x6C7C,0x6C9F,0x6C75,INVALC,0x6C56,0x6CA2,0x6C79,INVALC,
	0x6CA1,INVALC,0x6CAA,0x6CA0,INVALC,0x7079,0x7077,0x707E,
	INVALC,0x7075,0x707B,0x7264,INVALC,0x72BB,0x72BC,0x72C7,
	0x72B9,0x72BE,0x72B6,
	/* Range 0x2673 - 0x267E, array index: 0x045C */
	0x7593,0x7680,INVALC,0x7683,0x76C0,0x76C1,INVALC,INVALC,
	0x77F4,0x77F5,INVALC,0x7ACC,
	/* Range 0x2721 - 0x2752, array index: 0x0468 */
	0x7ACD,0x7CFA,0x809F,0x8091,0x8097,0x8094,INVALC,0x8286,
	0x828C,INVALC,0x8295,INVALC,0x866C,INVALC,0x8FB5,0x8FBE,
	0x8FC7,INVALC,0x8FC1,0x90A9,0x90A4,INVALC,INVALC,INVALC,
	0x90A8,0x9627,0x9626,0x962B,0x9633,0x9634,0x9629,0x4E3D,
	INVALC,0x4E9D,0x4F93,0x4F8A,INVALC,INVALC,0x4F6D,0x4F8E,
	0x4FA0,0x4FA2,0x4FA1,0x4F9F,0x4FA3,INVALC,0x4F72,INVALC,
	0x4F8C,0x5156,
	/* Range 0x2759 - 0x277D, array index: 0x049A */
	0x51ED,0x51FE,0x522F,INVALC,0x523C,0x5234,0x5239,0x52B9,
	0x52B5,0x52BF,0x5355,INVALC,0x5376,0x537A,0x5393,INVALC,
	0x53C1,0x53C2,0x53D5,0x5485,INVALC,0x545F,0x5493,0x5489,
	0x5479,0x9EFE,0x548F,0x5469,0x546D,INVALC,0x5494,0x546A,
	0x548A,INVALC,0x56FD,0x56FB,0x56F8,
	/* Range 0x2821 - 0x282F, array index: 0x04BF */
	0x56FC,0x56F6,0x5765,0x5781,0x5763,0x5767,INVALC,0x576E,
	0x5778,0x577F,INVALC,INVALC,0x58F3,0x594B,0x594C,
	/* Range 0x283D - 0x2841, array index: 0x04CE */
	0x59BF,INVALC,0x59C9,0x59B8,0x59AC,
	/* Range 0x2848 - 0x287A, array index: 0x04D3 */
	0x5B60,INVALC,0x5B96,0x5B9E,0x5B94,0x5B9F,0x5B9D,INVALC,
	0x5C00,0x5C19,INVALC,INVALC,0x5C49,0x5C4A,INVALC,0x5CBB,
	0x5CC1,INVALC,INVALC,INVALC,0x5CB9,0x5C9E,0x5CB4,0x5CBA,
	0x5DF6,0x5E13,0x5E12,0x5E77,INVALC,0x5E98,INVALC,0x5E99,
	0x5E9D,0x5EF8,INVALC,0x5EF9,INVALC,0x5F06,0x5F21,INVALC,
	0x5F25,0x5F55,INVALC,INVALC,INVALC,0x5F84,0x5F83,0x6030,
	0x6007,INVALC,0x6036,
	/* Range 0x2925 - 0x295A, array index: 0x0506 */
	0x62BA,0x62B2,INVALC,0x62B7,0x62E4,0x62A7,INVALC,INVALC,
	INVALC,0x62D5,0x62E1,0x62DD,0x62A6,0x62C1,0x62C5,0x62C0,
	0x62DF,0x62E0,0x62DE,INVALC,0x6589,INVALC,0x65A6,0x65BA,
	INVALC,0x65FF,INVALC,0x6617,0x6618,0x6601,0x65FE,INVALC,
	0x670C,INVALC,0x676B,0x6796,0x6782,0x678A,INVALC,0x67A3,
	INVALC,0x67A2,0x678F,INVALC,0x67F9,0x6780,0x6B26,0x6B27,
	0x6B68,0x6B69,INVALC,0x6B81,0x6BB4,0x6BD1,
	/* Range 0x2963 - 0x2979, array index: 0x053C */
	0x6C97,0x6C6C,0x6CDF,INVALC,0x6CEA,INVALC,0x6CE4,0x6CD8,
	0x6CB2,0x6CCE,0x6CC8,INVALC,0x708B,0x7088,0x7090,0x708F,
	INVALC,0x7087,0x7089,0x708D,0x7081,INVALC,0x708C,
	/* Range 0x2A21 - 0x2A3F, array index: 0x0553 */
	0x7265,0x7266,0x7268,INVALC,INVALC,0x72CD,0x72D3,0x72DB,
	INVALC,0x72CF,0x73A7,0x73A3,0x739E,INVALC,0x73AF,INVALC,
	INVALC,0x73AA,0x739C,INVALC,0x7542,0x7544,0x753B,0x7541,
	INVALC,0x759B,0x759E,INVALC,0x79C4,0x79C3,0x79C6,
	/* Range 0x2A47 - 0x2A4F, array index: 0x0572 */
	0x7ACF,0x7C76,0x7C74,0x7CFF,0x7CFC,INVALC,INVALC,0x7F59,
	0x80A8,
	/* Range 0x2A54 - 0x2A60, array index: 0x057B */
	0x80B3,INVALC,0x80A4,0x80B6,0x80A7,0x80AC,INVALC,0x80A6,
	0x5367,0x820E,0x82C4,0x833E,0x829C,
	/* Range 0x2A70 - 0x2A7E, array index: 0x0588 */
	0x8FCC,0x8FD9,0x8FCA,0x8FD8,0x8FCF,0x90B7,INVALC,0x90AD,
	0x90B9,0x9637,INVALC,0x9641,0x963E,0x96B6,0x9751,
	/* Range 0x2B21 - 0x2B40, array index: 0x0597 */
	0x9763,0x4E57,0x4E79,0x4EB2,0x4EB0,0x4EAF,0x4EB1,0x4FD2,
	0x4FD5,INVALC,0x4FBE,0x4FB8,0x4FB0,0x4FB1,0x4FC8,INVALC,
	INVALC,0x4FC6,0x4FCC,0x4FE5,0x4FE3,0x4FB4,0x516A,INVALC,
	0x519F,INVALC,0x51C1,INVALC,0x51C2,0x51C3,0x5245,0x5248,
	/* Range 0x2B46 - 0x2B5E, array index: 0x05B7 */
	0x52C5,0x52CA,0x52C4,0x5327,0x5358,0x537D,INVALC,0x53DD,
	0x53DC,0x53DA,0x53D9,0x54B9,INVALC,0x54D0,0x54B4,0x54CA,
	INVALC,0x54A3,0x54DA,0x54A4,INVALC,0x54B2,0x549E,0x549F,
	0x54B5,
	/* Range 0x2B63 - 0x2B72, array index: 0x05D0 */
	0x54CC,INVALC,0x5700,0x57AC,0x5791,0x578E,0x578D,0x5792,
	0x57A1,0x5790,0x57A6,0x57A8,INVALC,0x579C,0x5796,0x57A7,
	/* Range 0x2C21 - 0x2C2F, array index: 0x05E0 */
	0x59DF,INVALC,0x59EB,0x59EF,0x59F0,0x59D5,0x5A0D,0x5A04,
	0x59F9,0x5A02,0x59F8,0x59E2,0x59D9,0x59E7,0x5B6A,
	/* Range 0x2C3B - 0x2C4D, array index: 0x05EF */
	0x5CD1,0x5CDC,0x5CE6,0x5CE1,0x5CCD,INVALC,0x5CE2,0x5CDD,
	0x5CE5,0x5DFB,0x5DFA,0x5E1E,INVALC,0x5EA1,INVALC,INVALC,
	0x5EFC,0x5EFB,0x5F2F,
	/* Range 0x2C56 - 0x2C63, array index: 0x0602 */
	0x604E,0x6051,INVALC,INVALC,0x6023,0x6031,0x607C,0x6052,
	INVALC,0x6060,0x604A,0x6061,INVALC,0x6218,
	/* Range 0x2C6B - 0x2C7E, array index: 0x0610 */
	0x631F,0x6317,0x62EA,0x6321,0x6304,0x6305,INVALC,0x6531,
	0x6544,0x6540,INVALC,0x6542,0x65BE,INVALC,0x6629,0x661B,
	INVALC,0x6623,0x662C,0x661A,
	/* Range 0x2D21 - 0x2D3D, array index: 0x0624 */
	0x6630,0x663B,0x661E,0x6637,0x6638,INVALC,0x670E,INVALC,
	INVALC,0x67E8,0x67D6,INVALC,0x67C7,0x67BC,0x6852,0x67BF,
	0x67D5,0x67FE,0x8363,0x67FB,INVALC,0x67B1,0x6801,0x6805,
	0x6800,0x67D7,INVALC,0x6B2A,0x6B6B,
	/* Range 0x2D45 - 0x2D7E, array index: 0x0641 */
	0x6D23,0x6CFF,0x6D14,0x6D05,0x6D13,0x6D06,0x6D21,INVALC,
	0x6D15,0x6CAF,0x6CF4,0x6D02,0x6D45,INVALC,0x6D26,INVALC,
	0x6D44,INVALC,0x6D24,0x70A5,INVALC,0x70A3,INVALC,0x70A2,
	0x70BB,0x70A0,0x70AA,INVALC,INVALC,0x70A8,0x70B6,0x70B2,
	0x70A7,INVALC,INVALC,0x70B9,0x722E,INVALC,0x723C,INVALC,
	0x726D,INVALC,INVALC,0x72E7,0x72ED,INVALC,0x72EC,0x72E5,
	0x72E2,INVALC,0x73C4,0x73BD,0x73CF,0x73C9,0x73C1,0x73D0,
	INVALC,0x73CE,
	/* Range 0x2E21 - 0x2E2B, array index: 0x067B */
	0x74ED,0x74EB,INVALC,0x74EF,0x7549,0x7550,0x7546,0x754A,
	INVALC,0x754D,0x75A6,
	/* Range 0x2E32 - 0x2E54, array index: 0x0686 */
	0x76C7,0x76FF,INVALC,0x76FD,0x77E6,0x780A,INVALC,0x7804,
	0x780B,0x7807,INVALC,0x7815,0x7808,INVALC,0x79D3,0x79D4,
	0x79D0,0x79D7,0x7A7C,INVALC,INVALC,0x7A7D,0x7A83,0x7A82,
	INVALC,0x7AD4,0x7AD5,0x7AD3,0x7AD0,0x7AD2,0x7AFE,0x7AFC,
	0x7C77,0x7C7C,0x7C7B,
	/* Range 0x2E5D - 0x2E6F, array index: 0x06A9 */
	0x7F8F,0x80D3,INVALC,0x80CB,0x80D2,INVALC,0x8109,0x80E2,
	0x80DF,0x80C6,INVALC,0x8224,0x82F7,0x82D8,0x82DD,INVALC,
	INVALC,0x82F8,0x82FC,
	/* Range 0x2E74 - 0x2E7E, array index: 0x06BC */
	0x82EE,INVALC,0x82D0,0x830E,0x82E2,0x830B,0x82FD,0x5179,
	0x8676,INVALC,0x8678,
	/* Range 0x2F23 - 0x2F6F, array index: 0x06C7 */
	0x8675,0x867D,INVALC,0x8842,0x8866,INVALC,0x898C,0x8A05,
	INVALC,0x8A06,INVALC,0x8C9F,INVALC,0x8FF1,0x8FE7,0x8FE9,
	0x8FEF,0x90C2,0x90BC,INVALC,0x90C6,0x90C0,INVALC,INVALC,
	0x90CD,0x90C9,INVALC,0x90C4,INVALC,0x9581,INVALC,0x9CEC,
	0x5032,0x4FF9,0x501D,0x4FFF,0x5004,0x4FF0,0x5003,INVALC,
	0x5002,0x4FFC,0x4FF2,0x5024,0x5008,0x5036,0x502E,INVALC,
	0x5010,0x5038,0x5039,0x4FFD,0x5056,0x4FFB,0x51A3,0x51A6,
	0x51A1,INVALC,INVALC,0x51C7,0x51C9,0x5260,0x5264,0x5259,
	0x5265,0x5267,0x5257,0x5263,INVALC,0x5253,INVALC,0x52CF,
	INVALC,0x52CE,0x52D0,0x52D1,0x52CC,
	/* Range 0x2F73 - 0x2F7B, array index: 0x0714 */
	0x550D,0x54F4,INVALC,0x5513,0x54EF,0x54F5,0x54F9,0x5502,
	0x5500,
	/* Range 0x3027 - 0x302B, array index: 0x071D */
	0x5705,0x57C9,INVALC,0x57B7,0x57CD,
	/* Range 0x302F - 0x3040, array index: 0x0722 */
	0x57BE,0x57BB,INVALC,0x57DB,0x57C8,0x57C4,0x57C5,0x57D1,
	0x57CA,0x57C0,INVALC,INVALC,0x5A21,0x5A2A,INVALC,0x5A1D,
	INVALC,0x5A0B,
	/* Range 0x304A - 0x304F, array index: 0x0734 */
	0x5A14,0x5A31,INVALC,0x5A2F,0x5A1A,0x5A12,
	/* Range 0x3055 - 0x307B, array index: 0x073A */
	0x5BBC,0x5BBB,0x5BB7,0x5C05,0x5C06,0x5C52,0x5C53,INVALC,
	INVALC,0x5CFA,0x5CEB,INVALC,0x5CF3,0x5CF5,0x5CE9,0x5CEF,
	INVALC,0x5E2A,0x5E30,0x5E2E,0x5E2C,0x5E2F,0x5EAF,0x5EA9,
	INVALC,0x5EFD,0x5F32,0x5F8E,0x5F93,0x5F8F,0x604F,0x6099,
	INVALC,0x607E,INVALC,0x6074,0x604B,0x6073,0x6075,
	/* Range 0x3121 - 0x3129, array index: 0x0761 */
	0x60A9,0x608B,0x60A6,INVALC,0x6093,0x60AE,0x609E,0x60A7,
	0x6245,
	/* Range 0x312C - 0x313D, array index: 0x076A */
	0x632E,INVALC,0x6352,0x6330,0x635B,INVALC,0x6319,0x631B,
	INVALC,0x6331,0x635D,0x6337,0x6335,0x6353,INVALC,0x635C,
	0x633F,0x654B,
	/* Range 0x3140 - 0x317D, array index: 0x077C */
	0x658B,INVALC,0x659A,0x6650,0x6646,0x664E,0x6640,INVALC,
	0x664B,0x6648,INVALC,0x6660,0x6644,0x664D,INVALC,0x6837,
	0x6824,INVALC,INVALC,0x681B,0x6836,INVALC,0x682C,0x6819,
	0x6856,0x6847,0x683E,0x681E,INVALC,0x6815,0x6822,0x6827,
	0x6859,0x6858,0x6855,0x6830,0x6823,0x6B2E,0x6B2B,0x6B30,
	0x6B6C,INVALC,0x6B8B,INVALC,0x6BE9,0x6BEA,0x6BE5,0x6D6B,
	INVALC,INVALC,0x6D73,0x6D57,INVALC,INVALC,0x6D5D,0x6D56,
	0x6D8F,0x6D5B,0x6D1C,0x6D9A,0x6D9B,0x6D99,
	/* Range 0x3221 - 0x3231, array index: 0x07BA */
	0x6D81,0x6D71,INVALC,INVALC,0x6D72,0x6D5C,0x6D96,0x70C4,
	0x70DB,0x70CC,0x70D0,0x70E3,0x70DF,INVALC,0x70D6,0x70EE,
	0x70D5,
	/* Range 0x3238 - 0x3241, array index: 0x07CB */
	0x72F5,0x7302,INVALC,INVALC,0x73E2,0x73EC,0x73D5,0x73F9,
	0x73DF,0x73E6,
	/* Range 0x3246 - 0x3248, array index: 0x07D5 */
	0x73E4,0x73E1,0x74F3,
	/* Range 0x324D - 0x3252, array index: 0x07D8 */
	0x7556,0x7555,0x7558,0x7557,0x755E,0x75C3,
	/* Range 0x325A - 0x327C, array index: 0x07DE */
	0x76CB,0x76CC,0x772A,INVALC,0x7716,0x770F,INVALC,INVALC,
	0x773F,0x772B,0x770E,0x7724,INVALC,0x7721,0x7718,0x77DD,
	INVALC,INVALC,0x7824,0x7836,INVALC,0x7958,0x7959,INVALC,
	0x7962,0x79DA,0x79D9,INVALC,0x79E1,0x79E5,0x79E8,0x79DB,
	INVALC,0x79E2,0x79F0,
	/* Range 0x3323 - 0x3332, array index: 0x0801 */
	0x7ADA,0x7ADD,INVALC,0x7ADB,0x7ADC,INVALC,INVALC,0x7B0D,
	0x7B0B,0x7B14,0x7C8E,0x7C86,INVALC,0x7C87,0x7C83,0x7C8B,
	/* Range 0x333B - 0x334F, array index: 0x0811 */
	0x7D25,0x7F62,0x7F93,0x7F99,0x7F97,INVALC,INVALC,0x7FC4,
	0x7FC6,0x800A,INVALC,INVALC,0x8040,0x803C,0x803B,0x80F6,
	0x80FF,0x80EE,0x8104,0x8103,0x8107,
	/* Range 0x3355 - 0x335A, array index: 0x0826 */
	0x822D,INVALC,0x8227,0x8229,0x831F,0x8357,
	/* Range 0x3369 - 0x337C, array index: 0x082C */
	0x8684,0x869F,0x869B,0x8689,0x86A6,0x8692,0x868F,0x86A0,
	0x884F,0x8878,0x887A,0x886E,0x887B,0x8884,0x8873,INVALC,
	INVALC,0x8A0D,0x8A0B,0x8A19,
	/* Range 0x3425 - 0x3436, array index: 0x0840 */
	0x8FF9,0x9009,0x9008,INVALC,0x90DE,0x9151,INVALC,INVALC,
	0x91DB,0x91DF,0x91DE,0x91D6,0x91E0,0x9585,0x9660,0x9659,
	INVALC,0x9656,
	/* Range 0x343C - 0x347D, array index: 0x0852 */
	0x5042,0x5059,INVALC,0x5044,0x5066,0x5052,0x5054,0x5071,
	0x5050,0x507B,0x507C,0x5058,INVALC,INVALC,0x5079,0x506C,
	0x5078,0x51A8,0x51D1,0x51CF,0x5268,0x5276,0x52D4,INVALC,
	0x53A0,0x53C4,INVALC,0x5558,0x554C,0x5568,INVALC,0x5549,
	INVALC,INVALC,0x555D,0x5529,INVALC,0x5554,0x5553,INVALC,
	0x555A,INVALC,0x553A,0x553F,0x552B,0x57EA,INVALC,0x57EF,
	INVALC,INVALC,0x57DD,0x57FE,INVALC,0x57DE,0x57E6,INVALC,
	0x57E8,0x57FF,0x5803,0x58F7,0x68A6,0x591F,INVALC,0x595B,
	0x595D,0x595E,
	/* Range 0x3527 - 0x353C, array index: 0x0894 */
	0x5A61,0x5A3A,0x5A6E,0x5A4B,0x5A6B,INVALC,INVALC,0x5A45,
	0x5A4E,0x5A68,0x5A3D,0x5A71,0x5A3F,0x5A6F,0x5A75,INVALC,
	0x5A73,0x5A2C,0x5A59,0x5A54,0x5A4F,0x5A63,
	/* Range 0x3543 - 0x355B, array index: 0x08AA */
	0x5C5B,0x5C61,INVALC,0x5D21,0x5D0A,0x5D09,INVALC,0x5D2C,
	0x5D08,INVALC,INVALC,0x5D2A,0x5D15,INVALC,0x5D10,0x5D13,
	INVALC,0x5D2F,0x5D18,INVALC,0x5DE3,0x5E39,0x5E35,0x5E3A,
	0x5E32,
	/* Range 0x3560 - 0x3563, array index: 0x08C3 */
	0x5EBB,0x5EBA,0x5F34,0x5F39,
	/* Range 0x356E - 0x3576, array index: 0x08C7 */
	0x60D7,0x60AA,INVALC,0x60A1,0x60A4,INVALC,0x60EE,INVALC,
	0x60E7,
	/* Range 0x3622 - 0x367D, array index: 0x08D0 */
	0x6379,0x6386,0x6393,INVALC,0x6373,0x636A,INVALC,0x636C,
	INVALC,0x637F,INVALC,0x63B2,0x63BA,INVALC,INVALC,0x6366,
	0x6374,INVALC,0x655A,INVALC,0x654E,0x654D,0x658D,0x658E,
	0x65AD,INVALC,0x65C7,0x65CA,INVALC,0x65C9,INVALC,0x65E3,
	0x6657,INVALC,0x6663,0x6667,0x671A,0x6719,0x6716,INVALC,
	INVALC,0x689E,0x68B6,0x6898,0x6873,INVALC,0x689A,0x688E,
	0x68B7,0x68DB,0x68A5,0x686C,0x68C1,0x6884,INVALC,INVALC,
	0x6895,0x687A,0x6899,INVALC,0x68B8,0x68B9,0x6870,INVALC,
	0x6B35,INVALC,0x6B90,0x6BBB,0x6BED,INVALC,INVALC,INVALC,
	0x6DC1,0x6DC3,0x6DCE,INVALC,INVALC,0x6DAD,0x6E04,INVALC,
	0x6DB9,INVALC,0x6DE7,INVALC,0x6E08,0x6E06,INVALC,0x6E0A,
	0x6DB0,INVALC,0x6DF8,0x6E0C,
	/* Range 0x3721 - 0x373D, array index: 0x092C */
	0x6DB1,INVALC,0x6E02,0x6E07,0x6E09,0x6E01,0x6E17,0x6DFF,
	0x6E12,INVALC,INVALC,0x7103,0x7107,0x7101,0x70F5,0x70F1,
	0x7108,0x70F2,0x710F,INVALC,0x70FE,INVALC,INVALC,INVALC,
	0x731A,0x7310,0x730E,0x7402,0x73F3,
	/* Range 0x3744 - 0x374A, array index: 0x0949 */
	0x751B,0x7523,0x7561,0x7568,INVALC,0x7567,0x75D3,
	/* Range 0x3750 - 0x3759, array index: 0x0950 */
	0x76D5,0x76D7,0x76D6,0x7730,INVALC,0x7726,INVALC,0x7740,
	INVALC,0x771E,
	/* Range 0x375D - 0x377E, array index: 0x095A */
	0x7847,INVALC,0x784B,0x7851,0x784F,0x7842,0x7846,INVALC,
	0x796E,0x796C,0x79F2,INVALC,0x79F1,0x79F5,0x79F3,0x79F9,
	INVALC,INVALC,INVALC,0x7A9A,0x7A93,0x7A91,0x7AE1,INVALC,
	INVALC,0x7B21,0x7B1C,0x7B16,0x7B17,0x7B36,0x7B1F,INVALC,
	0x7C93,0x7C99,
	/* Range 0x3839 - 0x3842, array index: 0x097C */
	0x801D,INVALC,0x8049,0x8045,0x8044,0x7C9B,INVALC,INVALC,
	0x812A,0x812E,
	/* Range 0x3845 - 0x3858, array index: 0x0986 */
	0x8131,INVALC,0x811A,0x8134,0x8117,INVALC,INVALC,INVALC,
	0x831D,0x8371,0x8384,0x8380,0x8372,0x83A1,INVALC,0x8379,
	0x8391,INVALC,0x839F,0x83AD,
	/* Range 0x385B - 0x387B, array index: 0x099A */
	0x8323,INVALC,0x8385,0x839C,0x83B7,0x8658,0x865A,INVALC,
	0x8657,0x86B2,INVALC,0x86AE,INVALC,INVALC,INVALC,0x8845,
	0x889C,0x8894,0x88A3,0x888F,0x88A5,0x88A9,0x88A6,0x888A,
	0x88A0,0x8890,0x8992,0x8991,0x8994,INVALC,0x8A26,0x8A32,
	0x8A28,
	/* Range 0x3929 - 0x3941, array index: 0x09BB */
	0x8A21,0x8C3A,INVALC,0x8C5B,0x8C58,0x8C7C,INVALC,0x8CA6,
	0x8CAE,0x8CAD,0x8D65,INVALC,0x8D7E,INVALC,0x8D7C,0x8D7F,
	0x8D7A,0x8DBD,INVALC,INVALC,0x8DC0,0x8DBB,0x8EAD,0x8EAF,
	0x8ED6,
	/* Range 0x394A - 0x3959, array index: 0x09D4 */
	0x9012,0x900E,0x9025,INVALC,0x9013,0x90EE,INVALC,0x90AB,
	0x90F7,INVALC,0x9159,0x9154,0x91F2,0x91F0,0x91E5,0x91F6,
	/* Range 0x3967 - 0x397E, array index: 0x09E4 */
	0x98E1,0x98E6,INVALC,0x9EC4,0x9ED2,0x4E80,INVALC,0x4E81,
	0x508F,0x5097,0x5088,0x5089,INVALC,INVALC,0x5081,0x5160,
	INVALC,INVALC,0x5E42,0x51D3,INVALC,INVALC,0x51D2,0x51D6,
	/* Range 0x3A27 - 0x3A3C, array index: 0x09FC */
	0x53A8,0x53A6,0x53C5,0x5597,0x55DE,INVALC,INVALC,0x5596,
	0x55B4,INVALC,0x5585,INVALC,0x559B,0x55A0,INVALC,0x5559,
	INVALC,0x5586,INVALC,INVALC,0x55AF,0x557A,
	/* Range 0x3A40 - 0x3A63, array index: 0x0A12 */
	0x559E,INVALC,0x55A9,0x570F,0x570E,0x581A,INVALC,0x581F,
	INVALC,0x583C,0x5818,0x583E,0x5826,INVALC,0x583A,INVALC,
	0x5822,INVALC,0x58FB,0x5963,0x5964,INVALC,0x5AA8,0x5AA3,
	0x5A82,0x5A88,0x5AA1,0x5A85,0x5A98,INVALC,0x5A99,INVALC,
	0x5A89,0x5A81,0x5A96,0x5A80,
	/* Range 0x3A75 - 0x3A7E, array index: 0x0A36 */
	0x5A79,INVALC,0x5A86,0x5AAB,0x5AAA,0x5AA4,0x5A8D,0x5A7E,
	INVALC,0x5BD5,
	/* Range 0x3B24 - 0x3B2E, array index: 0x0A40 */
	0x5C1E,0x5C5F,0x5C5E,0x5D44,0x5D3E,INVALC,0x5D48,0x5D1C,
	INVALC,0x5D5B,0x5D4D,
	/* Range 0x3B33 - 0x3B47, array index: 0x0A4B */
	0x5D53,0x5D4F,INVALC,0x5D3B,0x5D46,INVALC,INVALC,0x5E46,
	0x5E47,INVALC,0x5E48,0x5EC0,0x5EBD,0x5EBF,INVALC,0x5F11,
	INVALC,0x5F3E,0x5F3B,INVALC,0x5F3A,
	/* Range 0x3B4D - 0x3B68, array index: 0x0A60 */
	0x60EA,INVALC,0x6107,0x6122,0x610C,INVALC,INVALC,0x60B3,
	0x60D6,0x60D2,INVALC,0x60E3,0x60E5,0x60E9,INVALC,INVALC,
	0x6111,0x60FD,INVALC,INVALC,0x611E,0x6120,0x6121,0x621E,
	INVALC,0x63E2,0x63DE,0x63E6,
	/* Range 0x3B6D - 0x3B76, array index: 0x0A7C */
	0x63F8,INVALC,0x63FE,0x63C1,0x63BF,0x63F7,0x63D1,0x655F,
	0x6560,0x6561,
	/* Range 0x3B7C - 0x3B7E, array index: 0x0A86 */
	0x667D,0x666B,0x667F,
	/* Range 0x3C23 - 0x3C2A, array index: 0x0A89 */
	0x6673,0x6681,0x666D,0x6669,INVALC,INVALC,0x671E,0x68ED,
	/* Range 0x3C2F - 0x3C5D, array index: 0x0A91 */
	0x6903,INVALC,0x68FE,0x68E5,0x691E,0x6902,INVALC,INVALC,
	0x6909,0x68CA,0x6900,INVALC,0x6901,0x6918,0x68E2,0x68CF,
	INVALC,0x692E,0x68C5,0x68FF,INVALC,0x691C,0x68C3,INVALC,
	0x6B6F,INVALC,0x6B6E,INVALC,0x6BBE,INVALC,0x6BF4,0x6C2D,
	INVALC,0x6DB6,0x6E75,0x6E1E,INVALC,0x6E18,INVALC,0x6E48,
	INVALC,0x6E4F,INVALC,0x6E42,0x6E6A,0x6E70,0x6DFE,
	/* Range 0x3C60 - 0x3C74, array index: 0x0AC0 */
	0x6E6D,INVALC,0x6E7B,0x6E7E,0x6E59,INVALC,0x6E57,INVALC,
	0x6E80,0x6E50,INVALC,0x6E29,0x6E76,0x6E2A,0x6E4C,0x712A,
	INVALC,0x7135,0x712C,0x7137,0x711D,
	/* Range 0x3C77 - 0x3C7D, array index: 0x0AD5 */
	0x7138,INVALC,0x7134,0x712B,0x7133,0x7127,0x7124,
	/* Range 0x3D21 - 0x3D47, array index: 0x0ADC */
	0x712D,0x7232,0x7283,0x7282,0x7287,0x7306,0x7324,0x7338,
	0x732A,0x732C,0x732B,INVALC,0x732F,0x7328,0x7417,INVALC,
	INVALC,0x7419,0x7438,INVALC,0x741F,0x7414,0x743C,0x73F7,
	0x741C,0x7415,0x7418,0x7439,0x74F9,0x7524,INVALC,INVALC,
	INVALC,0x756E,0x756D,0x7571,0x758E,INVALC,0x75E5,
	/* Range 0x3D4F - 0x3D69, array index: 0x0B03 */
	0x76D9,INVALC,0x7748,0x7749,0x7743,INVALC,INVALC,0x7742,
	0x77DF,INVALC,0x7863,0x7876,INVALC,0x785F,0x7866,0x7966,
	0x7971,INVALC,INVALC,0x7976,0x7984,0x7975,0x79FF,0x7A07,
	INVALC,0x7A0E,0x7A09,
	/* Range 0x3D70 - 0x3D7B, array index: 0x0B1E */
	0x7AE7,0x7AE2,0x7B55,INVALC,INVALC,0x7B43,0x7B57,0x7B6C,
	0x7B42,0x7B53,INVALC,0x7B41,
	/* Range 0x3E21 - 0x3E63, array index: 0x0B2A */
	0x7CA0,0x7CA6,0x7CA4,0x7D74,INVALC,0x7D59,INVALC,0x7D60,
	0x7D57,0x7D6C,0x7D7E,0x7D64,INVALC,0x7D5A,0x7D5D,INVALC,
	INVALC,INVALC,0x7D76,0x7D4D,0x7D75,INVALC,0x7FD3,0x7FD6,
	INVALC,INVALC,0x8060,0x804E,0x8145,0x813B,INVALC,0x8148,
	0x8142,0x8149,0x8140,0x8114,0x8141,INVALC,0x81EF,0x81F6,
	0x8203,INVALC,0x83ED,INVALC,0x83DA,0x8418,0x83D2,0x8408,
	INVALC,0x8400,INVALC,INVALC,INVALC,0x8417,0x8346,0x8414,
	0x83D3,0x8405,0x841F,0x8402,0x8416,0x83CD,0x83E6,INVALC,
	0x865D,0x86D5,0x86E1,
	/* Range 0x3E68 - 0x3E6A, array index: 0x0B6D */
	0x86EE,0x8847,0x8846,
	/* Range 0x3E7C - 0x3E7E, array index: 0x0B70 */
	0x8A35,0x8A38,0x8A42,
	/* Range 0x3F21 - 0x3F24, array index: 0x0B73 */
	0x8A49,0x8A5D,0x8A4B,0x8A3D,
	/* Range 0x3F29 - 0x3F59, array index: 0x0B77 */
	0x8C60,0x8C5E,0x8C7F,0x8C7E,0x8C83,INVALC,0x8CB1,0x8D87,
	INVALC,INVALC,0x8D88,0x8D83,INVALC,INVALC,0x8D86,0x8D8B,
	0x8D82,0x8DCA,0x8DD2,INVALC,INVALC,0x8DD4,0x8DC9,0x8EB0,
	INVALC,INVALC,INVALC,0x8EF2,0x8EE4,0x8EF3,0x8EEA,INVALC,
	0x8EFD,INVALC,0x8F9D,0x902B,0x902A,INVALC,0x9028,0x9029,
	0x902C,INVALC,INVALC,0x903A,0x9030,0x9037,0x903B,INVALC,
	0x910A,
	/* Range 0x3F62 - 0x3F69, array index: 0x0BA8 */
	0x9218,0x9222,INVALC,0x921B,0x9208,INVALC,0x920E,0x9213,
	/* Range 0x3F70 - 0x3F75, array index: 0x0BB0 */
	0x968C,0x967B,0x967F,0x9681,INVALC,0x9682,
	/* Range 0x4036 - 0x4061, array index: 0x0BB6 */
	0x50CC,0x50BC,INVALC,0x50AA,0x50B9,INVALC,0x50AB,0x50C3,
	0x50CD,0x517E,0x527E,0x5279,INVALC,INVALC,0x52E1,0x52E0,
	0x52E7,0x5380,0x53AB,0x53AA,0x53A9,0x53E0,0x55EA,INVALC,
	0x55D7,INVALC,INVALC,0x55C1,0x5715,INVALC,0x586C,INVALC,
	0x585C,0x5850,0x5861,0x586A,0x5869,0x5856,0x5860,0x5866,
	0x585F,0x5923,0x5966,0x5968,
	/* Range 0x4071 - 0x407D, array index: 0x0BE2 */
	0x5B74,0x5B76,0x5BDC,0x5BD7,0x5BDA,0x5BDB,INVALC,0x5C20,
	0x5D6D,0x5D66,INVALC,0x5D64,0x5D6E,
	/* Range 0x4121 - 0x413E, array index: 0x0BEF */
	0x5D60,0x5F42,0x5F5A,0x5F6E,INVALC,INVALC,0x6130,0x613A,
	0x612A,0x6143,0x6119,0x6131,INVALC,0x613D,INVALC,INVALC,
	INVALC,0x6408,0x6432,0x6438,INVALC,0x6431,INVALC,0x6419,
	INVALC,0x6411,INVALC,INVALC,0x6429,0x641D,
	/* Range 0x414D - 0x4151, array index: 0x0C0D */
	0x6570,0x656D,INVALC,0x65E4,0x6693,
	/* Range 0x4172 - 0x417E, array index: 0x0C12 */
	0x6985,0x694D,0x6950,0x6947,0x6967,0x6936,0x6964,0x6961,
	INVALC,0x697D,0x6B44,0x6B40,0x6B71,
	/* Range 0x4226 - 0x4233, array index: 0x0C1F */
	0x6BC1,INVALC,0x6BFA,0x6C31,0x6C32,INVALC,INVALC,0x6EB8,
	0x6EA8,INVALC,0x6E91,0x6EBB,INVALC,0x6E9A,
	/* Range 0x4239 - 0x4249, array index: 0x0C2D */
	0x6EB5,0x6E6C,0x6EE8,INVALC,0x6EDD,0x6EDA,0x6EE6,0x6EAC,
	INVALC,INVALC,INVALC,0x6ED9,0x6EE3,0x6EE9,0x6EDB,INVALC,
	0x716F,
	/* Range 0x424E - 0x427E, array index: 0x0C3E */
	0x714A,0x716B,INVALC,0x714F,0x7157,0x7174,INVALC,INVALC,
	INVALC,0x7145,0x7151,0x716D,INVALC,0x7251,0x7250,0x724E,
	INVALC,0x7341,INVALC,0x732E,0x7346,INVALC,0x7427,INVALC,
	0x7448,0x7453,0x743D,INVALC,0x745D,0x7456,INVALC,0x741E,
	0x7447,0x7443,0x7458,0x7449,INVALC,0x744C,0x7445,0x743E,
	INVALC,0x7501,0x751E,INVALC,INVALC,0x757A,0x75EE,0x7602,
	0x7697,
	/* Range 0x4325 - 0x4339, array index: 0x0C6F */
	0x775D,0x7764,0x7753,0x7758,0x7882,0x7890,0x788A,INVALC,
	0x787A,0x787D,INVALC,0x788B,0x7878,INVALC,INVALC,0x788D,
	0x7888,0x7892,0x7881,0x797E,0x7983,
	/* Range 0x4346 - 0x4355, array index: 0x0C84 */
	0x7AA1,0x7AA4,INVALC,0x7AE9,0x7AEA,INVALC,0x7B62,0x7B6B,
	INVALC,0x7B5E,INVALC,0x7B79,INVALC,INVALC,0x7B6F,0x7B68,
	/* Range 0x4360 - 0x436A, array index: 0x0C94 */
	0x7D8A,INVALC,0x7D8B,0x7D99,0x7D95,INVALC,0x7D87,0x7D78,
	0x7D97,0x7D89,0x7D98,
	/* Range 0x4372 - 0x4377, array index: 0x0C9F */
	0x7FDD,0x8057,INVALC,0x8163,0x816A,0x816C,
	/* Range 0x4422 - 0x444F, array index: 0x0CA5 */
	0x817D,0x816D,INVALC,INVALC,0x8241,0x844F,0x8484,INVALC,
	0x847F,INVALC,0x8448,0x842A,0x847B,0x8472,0x8464,0x842E,
	0x845C,0x8453,INVALC,0x8441,0x84C8,INVALC,0x8462,0x8480,
	0x843E,0x8483,0x8471,INVALC,0x844A,0x8455,0x8458,INVALC,
	INVALC,INVALC,0x86FC,0x86FD,0x8715,INVALC,0x8716,0x86FF,
	INVALC,INVALC,INVALC,0x8858,0x88CF,0x88E0,
	/* Range 0x4454 - 0x446B, array index: 0x0CD3 */
	0x89E7,0x8A6A,0x8A80,INVALC,0x8A6F,0x8A65,INVALC,0x8A78,
	0x8A7D,0x8A88,INVALC,INVALC,0x8A64,0x8A7E,INVALC,0x8A67,
	0x8C63,0x8C88,INVALC,0x8CCD,INVALC,0x8CC9,INVALC,0x8DED,
	/* Range 0x4476 - 0x447E, array index: 0x0CEB */
	0x8F04,0x8F9E,0x8FA0,0x9043,0x9046,0x9048,0x9045,0x9040,
	0x904C,
	/* Range 0x4523 - 0x453C, array index: 0x0CF4 */
	0x910C,0x9113,0x9115,INVALC,0x916B,0x9167,0x925D,0x9255,
	0x9235,INVALC,0x9259,0x922F,0x923C,0x928F,0x925C,0x926A,
	0x9262,0x925F,0x926B,0x926E,0x923B,0x9244,0x9241,0x959A,
	INVALC,0x9599,
	/* Range 0x4557 - 0x455C, array index: 0x0D0E */
	0x98F7,0x98FF,0x98F5,INVALC,0x98EC,0x98F1,
	/* Range 0x455F - 0x4576, array index: 0x0D14 */
	0x999A,INVALC,0x9AE2,0x9B3D,0x9B5D,0x9CE8,INVALC,0x9CEB,
	0x9CEF,0x9CEE,0x9E81,0x9F14,0x50D0,0x50D9,0x50DC,0x50D8,
	INVALC,0x50E1,0x50EB,INVALC,INVALC,0x50F4,0x50E2,0x50DE,
	/* Range 0x4625 - 0x4664, array index: 0x0D2C */
	0x53AE,0x53B0,INVALC,0x55FB,0x5603,0x560B,INVALC,0x5607,
	INVALC,0x55F8,INVALC,0x5628,0x561E,INVALC,0x5618,0x5611,
	0x5651,0x5605,0x5717,0x5892,INVALC,0x588C,INVALC,0x5878,
	0x5884,0x5873,0x58AD,0x5897,0x5895,0x5877,0x5872,0x5896,
	0x588D,0x5910,INVALC,0x596C,INVALC,0x5AE7,INVALC,0x5AE4,
	INVALC,INVALC,0x5AEF,0x5626,INVALC,INVALC,0x5AF0,0x5D7B,
	INVALC,0x5D83,INVALC,INVALC,0x5D8B,0x5D8C,INVALC,0x5D78,
	0x5E52,INVALC,INVALC,0x5ED0,0x5ECF,INVALC,0x5FB3,0x5FB4,
	/* Range 0x4668 - 0x467B, array index: 0x0D6C */
	0x617B,INVALC,0x616F,0x6181,0x613C,0x6142,0x6138,0x6133,
	INVALC,0x6160,0x6169,0x617D,0x6186,0x622C,0x6228,INVALC,
	0x644C,INVALC,0x6457,0x647C,
	/* Range 0x4721 - 0x472A, array index: 0x0D80 */
	0x6462,0x6471,0x646A,0x6456,0x643B,0x6481,INVALC,0x644F,
	0x647E,0x6464,
	/* Range 0x4733 - 0x474E, array index: 0x0D8A */
	0x66A5,0x669A,0x669C,INVALC,0x66A6,INVALC,0x66A4,0x698F,
	0x69C5,0x69C8,0x6992,0x69B2,INVALC,INVALC,INVALC,0x69E3,
	0x69C0,0x69D6,0x69D1,0x699F,0x69A2,0x69D2,INVALC,INVALC,
	INVALC,0x69E1,0x69D5,0x699D,
	/* Range 0x4753 - 0x4772, array index: 0x0DA6 */
	0x6B74,0x6BA1,INVALC,0x6EF0,0x6EF3,INVALC,INVALC,0x6F1B,
	0x6F0C,0x6F1D,0x6F34,0x6F28,0x6F17,INVALC,0x6F44,0x6F42,
	0x6F04,0x6F11,0x6EFA,0x6F4A,0x7191,0x718E,INVALC,0x718B,
	0x718D,0x717F,0x718C,0x717E,0x717C,0x7183,INVALC,0x7188,
	/* Range 0x4775 - 0x477E, array index: 0x0DC6 */
	0x7294,INVALC,0x7355,0x7353,0x734F,0x7354,0x746C,0x7465,
	0x7466,0x7461,
	/* Range 0x4821 - 0x482B, array index: 0x0DD0 */
	0x746B,0x7468,0x7476,INVALC,0x7460,INVALC,0x7474,0x7506,
	0x760E,INVALC,0x7607,
	/* Range 0x4830 - 0x4840, array index: 0x0DDB */
	0x76B7,0x76E2,INVALC,0x7774,0x7777,0x7776,0x7775,INVALC,
	0x7778,0x7771,INVALC,0x777A,0x715B,0x777B,0x78A6,0x78AE,
	0x78B8,
	/* Range 0x4844 - 0x4848, array index: 0x0DEC */
	0x78B1,0x78AF,INVALC,0x7989,0x7987,
	/* Range 0x4852 - 0x4862, array index: 0x0DF1 */
	0x7A32,INVALC,0x7AEC,0x7AF0,0x7B81,0x7B9E,0x7B83,INVALC,
	0x7B92,INVALC,0x7BA3,0x7B9F,0x7B93,INVALC,0x7B86,0x7CB8,
	0x7CB7,
	/* Range 0x486D - 0x4879, array index: 0x0E02 */
	0x7DA8,0x7DAB,INVALC,0x7DB3,0x7DCD,INVALC,0x7DCF,0x7DA4,
	INVALC,INVALC,0x7F41,0x7F6F,0x7F71,
	/* Range 0x4922 - 0x492E, array index: 0x0E0F */
	0x8023,0x805B,INVALC,0x8061,0x805F,0x8181,INVALC,INVALC,
	0x8184,0x8213,INVALC,0x824A,0x824C,
	/* Range 0x4932 - 0x4959, array index: 0x0E1C */
	0x84BD,0x8495,INVALC,0x8492,0x84C3,INVALC,0x8496,0x84A5,
	0x84B5,0x84B3,0x84A3,0x84E4,0x84D8,0x84D5,INVALC,0x84B7,
	0x84AD,0x84DA,0x8493,0x8736,INVALC,INVALC,INVALC,0x873D,
	0x872B,0x8747,0x8739,INVALC,0x8745,0x871D,INVALC,0x88FF,
	0x88EA,INVALC,0x88F5,INVALC,0x8900,0x88ED,0x8903,0x88E9,
	/* Range 0x495C - 0x4968, array index: 0x0E44 */
	0x89EA,INVALC,0x8A9B,0x8A8E,0x8AA2,INVALC,0x8A9C,0x8A94,
	0x8A90,0x8AA9,0x8AAC,INVALC,0x8A9F,
	/* Range 0x4970 - 0x497D, array index: 0x0E51 */
	0x8CD0,0x8CD6,0x8CD4,0x8D98,0x8D9A,0x8D97,INVALC,INVALC,
	INVALC,0x8E0B,0x8E08,0x8E01,0x8EB4,0x8EB3,
	/* Range 0x4A26 - 0x4A39, array index: 0x0E5F */
	0x9061,0x905F,INVALC,INVALC,0x9125,0x917B,0x9176,0x917C,
	INVALC,0x9289,0x92F6,0x92B1,0x92AD,0x9292,0x9281,0x9284,
	INVALC,0x92AE,0x9290,0x929E,
	/* Range 0x4A44 - 0x4A49, array index: 0x0E73 */
	0x96A0,0x969D,0x969F,0x96D0,INVALC,0x96D1,
	/* Range 0x4A52 - 0x4A56, array index: 0x0E79 */
	0x9819,INVALC,0x9814,0x9815,0x981A,
	/* Range 0x4A5D - 0x4A66, array index: 0x0E7E */
	0x98F8,0x9901,INVALC,0x99BE,0x99BC,0x99B7,0x99B6,0x99C0,
	INVALC,0x99B8,
	/* Range 0x4A6C - 0x4A75, array index: 0x0E88 */
	0x99BF,INVALC,0x9ADA,0x9AE4,0x9AE9,0x9AE8,0x9AEA,0x9AE5,
	INVALC,0x9B26,
	/* Range 0x4B29 - 0x4B5F, array index: 0x0E92 */
	0x50F7,INVALC,0x50FC,0x510D,0x5101,0x51DA,0x51D9,0x51DB,
	0x5286,0x528E,0x52EE,0x5333,0x53B1,INVALC,0x5647,0x562D,
	0x5654,INVALC,0x564B,0x5652,0x5631,0x5644,0x5656,0x5650,
	0x562B,INVALC,0x564D,0x5637,0x564F,0x58A2,0x58B7,INVALC,
	0x58B2,INVALC,0x58AA,0x58B5,0x58B0,INVALC,0x58B4,0x58A4,
	0x58A7,INVALC,0x5926,0x5AFE,INVALC,0x5B04,INVALC,0x5AFC,
	INVALC,0x5B06,0x5B0A,0x5AFA,0x5B0D,0x5B00,0x5B0E,
	/* Range 0x4B63 - 0x4B6D, array index: 0x0EC9 */
	0x5D91,INVALC,0x5D8F,0x5D90,0x5D98,0x5DA4,0x5D9B,0x5DA3,
	0x5D96,0x5DE4,0x5E5A,
	/* Range 0x4B70 - 0x4B7E, array index: 0x0ED4 */
	0x5E5E,INVALC,0x5FB8,0x6157,0x615C,0x61A6,0x6195,0x6188,
	INVALC,0x61A3,0x618F,INVALC,0x6164,INVALC,0x6159,
	/* Range 0x4C21 - 0x4C4A, array index: 0x0EE3 */
	0x6178,INVALC,0x6185,0x6187,0x619E,INVALC,INVALC,0x6198,
	0x619C,INVALC,INVALC,0x622F,0x6480,0x649B,0x648E,0x648D,
	0x6494,0x64C6,INVALC,0x64A8,0x6483,INVALC,0x64B9,0x6486,
	0x64B4,0x64AF,0x6491,INVALC,0x64AA,0x64A1,0x64A7,0x66B6,
	0x66B3,INVALC,0x66BC,0x66AC,INVALC,0x66AD,0x6A0E,INVALC,
	0x6A1C,0x6A1A,
	/* Range 0x4C4D - 0x4C6C, array index: 0x0F0D */
	0x6A0B,INVALC,0x69EF,0x6A0C,0x69F0,0x6A22,INVALC,0x69D8,
	INVALC,0x6A12,0x69FA,INVALC,0x6A2A,INVALC,0x6A10,INVALC,
	INVALC,0x6A29,0x69F9,0x69EA,0x6A2C,0x6A24,INVALC,0x69E9,
	0x6B52,0x6B4F,0x6B53,INVALC,INVALC,0x6F10,0x6F65,0x6F75,
	/* Range 0x4C71 - 0x4C7E, array index: 0x0F2D */
	0x6FD0,INVALC,0x6F5C,0x6F3D,0x6F71,INVALC,0x6F91,0x6F0B,
	0x6F79,0x6F81,0x6F8F,INVALC,0x6F59,0x6F74,
	/* Range 0x4D24 - 0x4D31, array index: 0x0F3B */
	0x71A3,0x71AD,INVALC,INVALC,0x71AB,0x71A6,0x71A2,INVALC,
	0x52F2,0x7257,0x7255,0x7299,0x734B,0x747A,
	/* Range 0x4D35 - 0x4D3D, array index: 0x0F49 */
	0x748C,0x7484,INVALC,INVALC,0x7482,0x7493,0x747B,INVALC,
	0x7509,
	/* Range 0x4D46 - 0x4D5D, array index: 0x0F52 */
	0x7790,INVALC,0x78C6,0x78D3,0x78C0,0x78D2,0x78C7,0x78C2,
	INVALC,0x799F,0x799D,0x799E,INVALC,0x7A41,INVALC,0x7A38,
	0x7A3A,0x7A42,INVALC,INVALC,0x7A3E,0x7AB0,0x7BAE,0x7BB3,
	/* Range 0x4D6D - 0x4D7E, array index: 0x0F6A */
	0x7CC4,0x7CCD,0x7CC2,0x7CC6,0x7CC3,0x7CC9,0x7CC7,INVALC,
	0x7DF8,INVALC,0x7DED,0x7DE2,INVALC,INVALC,INVALC,0x7DDC,
	0x7E02,0x7E01,
	/* Range 0x4E24 - 0x4E3D, array index: 0x0F7C */
	0x7DE4,0x7DFE,INVALC,0x7E00,0x7DFC,0x7DFD,INVALC,0x7DF5,
	0x7DFF,INVALC,0x7DEB,0x7DE5,0x7F78,0x7FAE,0x7FE7,INVALC,
	0x8065,0x806A,0x8066,0x8068,0x806B,0x8194,0x81A1,0x8192,
	0x8196,0x8193,
	/* Range 0x4E4B - 0x4E4F, array index: 0x0F96 */
	0x851B,0x8503,0x8533,0x8534,0x84ED,
	/* Range 0x4E5D - 0x4E76, array index: 0x0F9B */
	0x8771,INVALC,0x885C,0x88E6,0x890F,0x891B,INVALC,0x89A9,
	0x89A5,0x89EE,0x8AB1,INVALC,0x8ACC,0x8ACE,INVALC,0x8AB7,
	INVALC,0x8AB5,0x8AE9,0x8AB4,INVALC,0x8AB3,0x8AC1,0x8AAF,
	0x8ACA,0x8AD0,
	/* Range 0x4F22 - 0x4F26, array index: 0x0FB5 */
	0x8CEB,0x8DA4,INVALC,0x8DA2,0x8D9D,
	/* Range 0x4F2B - 0x4F56, array index: 0x0FBA */
	0x8E2A,0x8E28,INVALC,INVALC,0x8EB8,0x8EB6,0x8EB9,0x8EB7,
	0x8F22,0x8F2B,0x8F27,0x8F19,0x8FA4,INVALC,0x8FB3,INVALC,
	0x9071,0x906A,INVALC,INVALC,0x9188,0x918C,0x92BF,0x92B8,
	0x92BE,0x92DC,0x92E5,INVALC,INVALC,0x92D4,0x92D6,INVALC,
	0x92DA,0x92ED,0x92F3,0x92DB,INVALC,0x92B9,0x92E2,0x92EB,
	0x95AF,INVALC,0x95B2,0x95B3,
	/* Range 0x4F60 - 0x4F7E, array index: 0x0FE6 */
	0x970A,INVALC,0x9787,0x9789,0x978C,0x97EF,0x982A,0x9822,
	INVALC,0x981F,INVALC,0x9919,INVALC,0x99CA,0x99DA,INVALC,
	INVALC,INVALC,0x99DE,0x99C8,0x99E0,INVALC,0x9AB6,0x9AB5,
	INVALC,0x9AF4,INVALC,0x9B6B,0x9B69,0x9B72,0x9B63,
	/* Range 0x502A - 0x5038, array index: 0x1005 */
	0x9CFE,0x9D02,0x9E84,INVALC,0x9EAB,0x9EAA,0x511D,0x5116,
	INVALC,0x512B,0x511E,0x511B,0x5290,0x5294,0x5314,
	/* Range 0x5048 - 0x5070, array index: 0x1014 */
	0x58C3,0x58CA,0x58BB,0x58C0,0x58C4,0x5901,0x5B1F,0x5B18,
	0x5B11,0x5B15,INVALC,0x5B12,0x5B1C,INVALC,0x5B22,0x5B79,
	0x5DA6,INVALC,0x5DB3,0x5DAB,0x5EEA,INVALC,0x5F5B,INVALC,
	INVALC,0x61B7,0x61CE,0x61B9,0x61BD,0x61CF,0x61C0,0x6199,
	0x6197,INVALC,0x61BB,0x61D0,0x61C4,0x6231,INVALC,0x64D3,
	0x64C0,
	/* Range 0x5075 - 0x507E, array index: 0x103D */
	0x64DC,0x64D1,0x64C8,INVALC,0x64D5,0x66C3,INVALC,INVALC,
	0x66BF,0x66C5,
	/* Range 0x5122 - 0x5132, array index: 0x1047 */
	0x66CD,0x66C1,0x6706,INVALC,0x6724,0x6A63,0x6A42,0x6A52,
	INVALC,0x6A43,0x6A33,INVALC,0x6A6C,0x6A57,INVALC,0x6A4C,
	0x6A6E,
	/* Range 0x5138 - 0x5146, array index: 0x1058 */
	0x6A37,INVALC,0x6A71,0x6A4A,0x6A36,INVALC,0x6A53,INVALC,
	0x6A45,0x6A70,INVALC,INVALC,0x6A5C,0x6B58,0x6B57,
	/* Range 0x5154 - 0x5176, array index: 0x1067 */
	0x6FB5,0x6FD3,0x6F9F,INVALC,0x6FB7,0x6FF5,0x71B7,INVALC,
	0x71BB,INVALC,0x71D1,INVALC,0x71BA,INVALC,0x71B6,0x71CC,
	INVALC,INVALC,0x71D3,0x749B,INVALC,INVALC,0x7496,0x74A2,
	0x749D,0x750A,0x750E,INVALC,0x7581,0x762C,0x7637,0x7636,
	0x763B,INVALC,0x76A1,
	/* Range 0x5224 - 0x5234, array index: 0x108A */
	0x78DC,INVALC,0x79A5,0x79A9,0x9834,0x7A53,0x7A45,INVALC,
	0x7A4F,INVALC,0x7ABD,0x7ABB,0x7AF1,INVALC,INVALC,0x7BEC,
	0x7BED,
	/* Range 0x5252 - 0x526B, array index: 0x109B */
	0x8542,INVALC,0x855C,0x8570,0x855F,INVALC,0x855A,0x854B,
	0x853F,0x878A,INVALC,0x878B,0x87A1,0x878E,INVALC,INVALC,
	0x8799,0x885E,0x885F,0x8924,0x89A7,0x8AEA,0x8AFD,0x8AF9,
	0x8AE3,0x8AE5,
	/* Range 0x5321 - 0x5344, array index: 0x10B5 */
	0x8F31,0x8F30,INVALC,0x8F2D,0x8F3C,0x8FA7,0x8FA5,INVALC,
	INVALC,INVALC,0x9137,0x9195,0x918E,INVALC,0x9196,INVALC,
	0x9345,0x930A,INVALC,INVALC,0x92FD,0x9317,0x931C,0x9307,
	0x9331,0x9332,0x932C,0x9330,0x9303,0x9305,INVALC,0x95C2,
	INVALC,0x95B8,INVALC,0x95C1,
	/* Range 0x5358 - 0x536C, array index: 0x10D9 */
	0x9836,0x9831,0x9833,0x983C,0x982E,0x983A,INVALC,0x983D,
	INVALC,0x98B5,0x9922,0x9923,0x9920,0x991C,0x991D,INVALC,
	0x99A0,INVALC,0x99EF,0x99E8,0x99EB,
	/* Range 0x5374 - 0x537D, array index: 0x10EE */
	0x9AF8,0x9AF5,INVALC,INVALC,0x9B83,0x9B94,0x9B84,INVALC,
	0x9B8B,0x9B8F,
	/* Range 0x5429 - 0x542D, array index: 0x10F8 */
	0x9D24,0x9D0F,INVALC,0x9D13,0x9D0A,
	/* Range 0x5432 - 0x5461, array index: 0x10FD */
	0x9D2A,0x9D1A,INVALC,0x9D27,0x9D16,0x9D21,INVALC,0x9E85,
	0x9EAC,0x9EC6,0x9EC5,0x9ED7,0x9F53,INVALC,0x5128,0x5127,
	0x51DF,INVALC,0x5335,0x53B3,INVALC,0x568A,0x567D,0x5689,
	INVALC,0x58CD,0x58D0,INVALC,0x5B2B,0x5B33,0x5B29,0x5B35,
	0x5B31,0x5B37,0x5C36,0x5DBE,INVALC,0x5DB9,INVALC,0x5DBB,
	INVALC,0x61E2,0x61DB,0x61DD,0x61DC,0x61DA,INVALC,0x61D9,
	/* Range 0x5469 - 0x547C, array index: 0x112D */
	0x64EE,INVALC,0x65B5,0x66D4,0x66D5,INVALC,0x66D0,0x66D1,
	0x66CE,0x66D7,INVALC,INVALC,0x6A7D,0x6A8A,INVALC,0x6AA7,
	INVALC,0x6A99,0x6A82,0x6A88,
	/* Range 0x552D - 0x5539, array index: 0x1141 */
	0x6C0A,INVALC,0x6FD7,0x6FD6,0x6FE5,INVALC,INVALC,INVALC,
	0x6FD9,0x6FDA,0x6FEA,INVALC,0x6FF6,
	/* Range 0x553E - 0x5543, array index: 0x114E */
	0x71E9,INVALC,0x71EB,0x71EF,0x71F3,0x71EA,
	/* Range 0x5552 - 0x5575, array index: 0x1154 */
	0x7583,0x7645,0x764E,0x7644,0x76A3,0x76A5,0x77A6,0x77A4,
	INVALC,0x77A9,0x77AF,INVALC,INVALC,INVALC,0x78F0,0x78F8,
	0x78F1,INVALC,0x7A49,INVALC,INVALC,INVALC,0x7AC2,0x7AF2,
	0x7AF3,0x7BFA,INVALC,0x7BF6,0x7BFC,0x7C18,0x7C08,0x7C12,
	INVALC,INVALC,0x7CDB,0x7CDA,
	/* Range 0x5621 - 0x5624, array index: 0x1178 */
	0x802B,0x8074,0x81B8,0x81C8,
	/* Range 0x5628 - 0x562D, array index: 0x117C */
	0x8592,0x8593,INVALC,0x857F,0x85AB,0x8597,
	/* Range 0x5639 - 0x5642, array index: 0x1182 */
	0x87C1,0x87B1,0x87C7,INVALC,0x8940,INVALC,0x893F,0x8939,
	INVALC,0x8943,
	/* Range 0x5646 - 0x564A, array index: 0x118C */
	0x89AB,INVALC,0x8B1F,0x8B09,0x8B0C,
	/* Range 0x5651 - 0x5675, array index: 0x1191 */
	0x8CF6,0x8CF7,INVALC,0x8E46,0x8E4F,INVALC,INVALC,INVALC,
	0x8F3D,0x8F41,0x9366,0x9378,0x935D,0x9369,0x9374,0x937D,
	0x936E,0x9372,0x9373,0x9362,0x9348,0x9353,0x935F,0x9368,
	INVALC,0x937F,0x936B,INVALC,0x95C4,INVALC,0x96AF,0x96AD,
	0x96B2,INVALC,INVALC,0x971A,0x971B,
	/* Range 0x573A - 0x5745, array index: 0x11B6 */
	0x9ABD,0x9B00,0x9B02,INVALC,0x9B34,0x9B49,0x9B9F,INVALC,
	0x9BA3,0x9BCD,0x9B99,0x9B9D,
	/* Range 0x5755 - 0x5765, array index: 0x11C2 */
	0x9F8E,INVALC,0x569F,0x569B,0x569E,0x5696,0x5694,0x56A0,
	INVALC,0x5B3B,INVALC,INVALC,0x5B3A,0x5DC1,0x5F4D,0x5F5D,
	0x61F3,
	/* Range 0x576A - 0x577E, array index: 0x11D3 */
	0x64F6,0x64E5,0x64EA,0x64E7,0x6505,INVALC,0x64F9,INVALC,
	INVALC,INVALC,0x6AAB,0x6AED,0x6AB2,0x6AB0,0x6AB5,0x6ABE,
	0x6AC1,0x6AC8,INVALC,0x6AC0,0x6ABC,
	/* Range 0x5821 - 0x5845, array index: 0x11E8 */
	0x6AB1,0x6AC4,0x6ABF,INVALC,INVALC,0x7008,0x7003,0x6FFD,
	0x7010,0x7002,0x7013,INVALC,0x71FA,0x7200,0x74B9,0x74BC,
	INVALC,0x765B,0x7651,0x764F,0x76EB,0x77B8,INVALC,0x77B9,
	0x77C1,0x77C0,0x77BE,0x790B,INVALC,0x7907,0x790A,0x7908,
	INVALC,0x790D,0x7906,0x7915,0x79AF,
	/* Range 0x5854 - 0x585D, array index: 0x120D */
	0x7CE6,0x7CE3,INVALC,INVALC,0x7E5D,0x7E4F,0x7E66,0x7E5B,
	0x7F47,0x7FB4,
	/* Range 0x586B - 0x587E, array index: 0x1217 */
	0x85CC,0x85B2,INVALC,0x85BB,0x85C1,INVALC,INVALC,INVALC,
	0x87E9,0x87EE,0x87F0,0x87D6,0x880E,0x87DA,0x8948,0x894A,
	0x894E,0x894D,0x89B1,0x89B0,
	/* Range 0x5931 - 0x5951, array index: 0x122B */
	0x8DA9,0x8E58,INVALC,INVALC,0x8EBF,0x8EC1,0x8F4A,0x8FAC,
	INVALC,0x9089,0x913D,0x913C,0x91A9,0x93A0,INVALC,0x9390,
	INVALC,0x9393,0x938B,0x93AD,0x93BB,0x93B8,INVALC,INVALC,
	0x939C,0x95D8,0x95D7,INVALC,INVALC,INVALC,0x975D,0x97A9,
	0x97DA,
	/* Range 0x5958 - 0x595C, array index: 0x124C */
	0x9855,0x984B,INVALC,0x983F,0x98B9,
	/* Range 0x5961 - 0x597E, array index: 0x1251 */
	0x9938,0x9936,0x9940,INVALC,0x993B,0x9939,0x99A4,INVALC,
	INVALC,0x9A08,0x9A0C,INVALC,0x9A10,INVALC,0x9B07,INVALC,
	0x9BD2,INVALC,0x9BC2,0x9BBB,0x9BCC,0x9BCB,INVALC,INVALC,
	0x9D4D,0x9D63,0x9D4E,INVALC,0x9D50,0x9D55,
	/* Range 0x5A22 - 0x5A3C, array index: 0x126F */
	0x9D5E,INVALC,0x9E90,0x9EB2,0x9EB1,INVALC,0x9ECA,0x9F02,
	0x9F27,0x9F26,INVALC,0x56AF,0x58E0,0x58DC,INVALC,0x5B39,
	INVALC,INVALC,0x5B7C,0x5BF3,INVALC,INVALC,0x5C6B,0x5DC4,
	0x650B,0x6508,0x650A,
	/* Range 0x5A42 - 0x5A48, array index: 0x128A */
	0x66E1,0x66DF,0x6ACE,0x6AD4,0x6AE3,0x6AD7,0x6AE2,
	/* Range 0x5A4D - 0x5A67, array index: 0x1291 */
	0x6AD8,0x6AD5,0x6AD2,INVALC,INVALC,0x701E,0x702C,0x7025,
	0x6FF3,0x7204,0x7208,0x7215,INVALC,0x74C4,0x74C9,0x74C7,
	0x74C8,0x76A9,0x77C6,0x77C5,0x7918,0x791A,0x7920,INVALC,
	0x7A66,0x7A64,0x7A6A,
	/* Range 0x5A74 - 0x5A7E, array index: 0x12AC */
	0x7E6E,0x7E71,INVALC,0x81D4,0x81D6,0x821A,0x8262,0x8265,
	0x8276,0x85DB,0x85D6,
	/* Range 0x5B25 - 0x5B2C, array index: 0x12B7 */
	0x85F4,INVALC,0x87FD,0x87D5,0x8807,INVALC,0x880F,0x87F8,
	/* Range 0x5B31 - 0x5B50, array index: 0x12BF */
	0x89B5,0x89F5,INVALC,0x8B3F,0x8B43,0x8B4C,INVALC,0x8D0B,
	0x8E6B,0x8E68,0x8E70,0x8E75,0x8E77,INVALC,0x8EC3,INVALC,
	0x93E9,0x93EA,0x93CB,0x93C5,0x93C6,INVALC,0x93ED,0x93D3,
	INVALC,0x93E5,INVALC,INVALC,0x93DB,0x93EB,0x93E0,0x93C1,
	/* Range 0x5B5D - 0x5B61, array index: 0x12DF */
	0x97B2,0x97B4,0x97B1,0x97B5,0x97F2,
	/* Range 0x5B69 - 0x5B7E, array index: 0x12E4 */
	0x9944,INVALC,0x9A26,0x9A1F,0x9A18,0x9A21,0x9A17,INVALC,
	0x9B09,INVALC,INVALC,0x9BC5,0x9BDF,INVALC,0x9BE3,INVALC,
	0x9BE9,0x9BEE,INVALC,INVALC,0x9D66,0x9D7A,
	/* Range 0x5C22 - 0x5C43, array index: 0x12FA */
	0x9D6E,0x9D91,0x9D83,0x9D76,0x9D7E,0x9D6D,INVALC,0x9E95,
	0x9EE3,INVALC,INVALC,0x9F03,0x9F04,INVALC,0x9F17,INVALC,
	0x5136,INVALC,0x5336,INVALC,0x5B42,INVALC,INVALC,0x5B44,
	0x5B46,0x5B7E,0x5DCA,0x5DC8,0x5DCC,0x5EF0,INVALC,0x6585,
	0x66E5,0x66E7,
	/* Range 0x5C53 - 0x5C66, array index: 0x131C */
	0x7216,INVALC,0x7212,0x720F,0x7217,0x7211,0x720B,INVALC,
	INVALC,0x74CD,0x74D0,0x74CC,0x74CE,0x74D1,INVALC,0x7589,
	INVALC,0x7A6F,0x7C4B,0x7C44,
	/* Range 0x5C6C - 0x5C7E, array index: 0x1330 */
	0x7E7F,0x8B71,INVALC,0x802F,0x807A,0x807B,0x807C,INVALC,
	INVALC,INVALC,0x85FC,0x8610,0x8602,INVALC,INVALC,0x85EE,
	0x8603,INVALC,0x860D,
	/* Range 0x5D21 - 0x5D49, array index: 0x1343 */
	0x8613,0x8608,0x860F,0x8818,0x8812,INVALC,INVALC,0x8967,
	0x8965,0x89BB,0x8B69,0x8B62,INVALC,0x8B6E,INVALC,0x8B61,
	INVALC,0x8B64,0x8B4D,0x8C51,INVALC,INVALC,0x8E83,0x8EC6,
	INVALC,0x941F,INVALC,0x9404,0x9417,0x9408,0x9405,INVALC,
	0x93F3,0x941E,0x9402,0x941A,0x941B,0x9427,0x941C,INVALC,
	0x96B5,
	/* Range 0x5D4C - 0x5D53, array index: 0x136C */
	0x9733,INVALC,0x9734,0x9731,0x97B8,0x97BA,INVALC,0x97FC,
	/* Range 0x5D5E - 0x5D7A, array index: 0x1374 */
	0x9AC9,INVALC,0x9AC8,0x9AC4,0x9B2A,0x9B38,0x9B50,INVALC,
	0x9C0A,0x9BFB,0x9C04,0x9BFC,0x9BFE,INVALC,INVALC,INVALC,
	0x9C02,0x9BF6,0x9C1B,0x9BF9,0x9C15,0x9C10,0x9BFF,0x9C00,
	0x9C0C,INVALC,INVALC,0x9D95,0x9DA5,
	/* Range 0x5E21 - 0x5E2A, array index: 0x1391 */
	0x9E98,0x9EC1,INVALC,0x9F5A,0x5164,0x56BB,INVALC,0x58E6,
	0x5B49,0x5BF7,
	/* Range 0x5E31 - 0x5E6A, array index: 0x139B */
	0x6511,INVALC,0x6AFF,0x6AFE,0x6AFD,INVALC,0x6B01,INVALC,
	INVALC,0x704B,0x704D,0x7047,0x74D3,0x7668,0x7667,INVALC,
	INVALC,0x77D1,0x7930,0x7932,0x792E,INVALC,0x9F9D,0x7AC9,
	0x7AC8,INVALC,0x7C56,0x7C51,INVALC,INVALC,INVALC,0x7E85,
	0x7E89,0x7E8E,0x7E84,INVALC,0x826A,0x862B,0x862F,0x8628,
	INVALC,0x8616,0x8615,0x861D,0x881A,INVALC,INVALC,INVALC,
	0x89BC,0x8B75,0x8B7C,INVALC,0x8D11,0x8D12,0x8F5C,0x91BB,
	INVALC,0x93F4,
	/* Range 0x5E70 - 0x5E7E, array index: 0x13D5 */
	0x96E4,0x9737,0x9736,0x9767,0x97BE,0x97BD,0x97E2,0x9868,
	0x9866,0x98C8,0x98CA,0x98C7,0x98DC,INVALC,0x994F,
	/* Range 0x5F21 - 0x5F2C, array index: 0x13E4 */
	0x99A9,0x9A3C,INVALC,0x9A3B,0x9ACE,INVALC,0x9B14,0x9B53,
	INVALC,0x9C2E,INVALC,0x9C1F,
	/* Range 0x5F31 - 0x5F37, array index: 0x13F0 */
	0x9DB0,0x9DBD,INVALC,INVALC,0x9DAE,0x9DC4,0x9E7B,
	/* Range 0x5F3C - 0x5F5B, array index: 0x13F7 */
	0x9F05,INVALC,0x9F69,0x9FA1,0x56C7,0x571D,0x5B4A,0x5DD3,
	INVALC,0x5F72,0x6202,INVALC,0x6235,0x6527,0x651E,0x651F,
	INVALC,INVALC,0x6B07,0x6B06,INVALC,INVALC,0x7054,0x721C,
	0x7220,0x7AF8,INVALC,0x7C5D,0x7C58,INVALC,0x7E92,0x7F4E,
	/* Range 0x5F69 - 0x5F73, array index: 0x1417 */
	0x9442,0x944D,0x9454,0x944E,INVALC,0x9443,INVALC,INVALC,
	0x973C,0x9740,0x97C0,
	/* Range 0x6039 - 0x6040, array index: 0x1422 */
	0x7221,INVALC,0x766F,0x7937,0x79B5,0x7C62,0x7C5E,0x7CF5,
	/* Range 0x6043 - 0x604B, array index: 0x142A */
	0x863D,INVALC,0x882D,0x8989,0x8B8D,0x8B87,0x8B90,0x8D1A,
	0x8E99,
	/* Range 0x6052 - 0x6068, array index: 0x1433 */
	0x9456,0x9461,0x945B,0x945A,0x945C,0x9465,INVALC,0x9741,
	INVALC,INVALC,0x986E,0x986C,0x986D,INVALC,0x99AA,0x9A5C,
	0x9A58,0x9ADE,INVALC,0x9C4F,0x9C51,INVALC,0x9C53,
	/* Range 0x6073 - 0x6077, array index: 0x144A */
	0x5B4F,0x6B14,INVALC,0x7A72,0x7A73,
	/* Range 0x6125 - 0x6135, array index: 0x144F */
	0x96E6,0x9745,INVALC,0x97C8,0x97E4,0x995D,INVALC,0x9B21,
	INVALC,0x9B2C,0x9B57,INVALC,INVALC,0x9C5D,0x9C61,0x9C65,
	0x9E08,
	/* Range 0x613E - 0x6150, array index: 0x1460 */
	0x6205,0x66EF,0x6B1B,0x6B1D,0x7225,0x7224,0x7C6D,INVALC,
	0x8642,0x8649,INVALC,0x8978,0x898A,0x8B97,INVALC,0x8C9B,
	0x8D1C,INVALC,0x8EA2,
	/* Range 0x615D - 0x6165, array index: 0x1473 */
	0x9E0E,INVALC,0x9F08,0x9F1D,0x9FA3,INVALC,INVALC,0x5F60,
	0x6B1C,
	/* Range 0x6169 - 0x617E, array index: 0x147C */
	0x7CF3,INVALC,0x8B9B,0x8EA7,0x91C4,INVALC,0x947A,INVALC,
	INVALC,0x9A61,0x9A63,0x9AD7,0x9C76,INVALC,0x9FA5,INVALC,
	0x7067,INVALC,0x72AB,0x864A,0x897D,0x8B9D,
	/* Range 0x6221 - 0x6229, array index: 0x1492 */
	0x8C53,0x8F65,0x947B,INVALC,0x98CD,0x98DD,INVALC,0x9B30,
	0x9E16,
	/* Range 0x622F - 0x623B, array index: 0x149B */
	0x96E7,0x9E18,0x9EA2,INVALC,0x9F7C,INVALC,0x7E9E,0x9484,
	INVALC,0x9E1C,INVALC,0x7C71,0x97CA,
	/* Range 0x625C - 0x6272, array index: 0x14A8 */
	0x5B66,INVALC,0x65F9,0x6788,0x6CE6,0x6CCB,INVALC,0x4FBD,
	0x5F8D,INVALC,0x6018,0x6048,INVALC,0x6B29,0x70A6,INVALC,
	0x7706,INVALC,INVALC,INVALC,0x5A10,0x5CFC,0x5CFE,
	/* Range 0x642F - 0x6433, array index: 0x14BF */
	0x9C27,INVALC,0x9458,0x77D6,0x9B2D,
	/* Range 0x6448 - 0x647E, array index: 0x14C4 */
	0x4F66,0x4F68,0x4FE7,0x503F,INVALC,0x50A6,0x510F,0x523E,
	0x5324,0x5365,0x539B,0x517F,0x54CB,0x5573,0x5571,0x556B,
	0x55F4,0x5622,0x5620,0x5692,0x56BA,0x5691,0x56B0,0x5759,
	0x578A,0x580F,0x5812,0x5813,0x5847,0x589B,0x5900,0x594D,
	0x5AD1,0x5AD3,0x5B67,0x5C57,0x5C77,0x5CD5,0x5D75,0x5D8E,
	0x5DA5,0x5DB6,0x5DBF,0x5E65,0x5ECD,0x5EED,0x5F94,0x5F9A,
	0x5FBA,0x6125,0x6150,0x62A3,0x6360,0x6364,0x63B6,
	/* Range 0x6521 - 0x657E, array index: 0x14FB */
	0x6403,0x64B6,0x651A,0x7A25,0x5C21,0x66E2,0x6702,0x67A4,
	0x67AC,0x6810,0x6806,0x685E,0x685A,0x692C,0x6929,0x6A2D,
	0x6A77,0x6A7A,0x6ACA,0x6AE6,0x6AF5,0x6B0D,0x6B0E,0x6BDC,
	0x6BDD,0x6BF6,0x6C1E,0x6C63,0x6DA5,0x6E0F,0x6E8A,0x6E84,
	0x6E8B,0x6E7C,0x6F4C,0x6F48,0x6F49,0x6F9D,0x6F99,0x6FF8,
	0x702E,0x702D,0x705C,0x79CC,0x70BF,0x70EA,0x70E5,0x7111,
	0x7112,0x713F,0x7139,0x713B,0x713D,0x7177,0x7175,0x7176,
	0x7171,0x7196,0x7193,0x71B4,0x71DD,0x71DE,0x720E,0x5911,
	0x7218,0x7347,0x7348,0x73EF,0x7412,0x743B,0x74A4,0x748D,
	0x74B4,0x7673,0x7677,0x76BC,0x7819,0x781B,0x783D,0x7853,
	0x7854,0x7858,0x78B7,0x78D8,0x78EE,0x7922,0x794D,0x7986,
	0x7999,0x79A3,0x79BC,0x7AA7,0x7B37,0x7B59,
	/* Range 0x6621 - 0x667C, array index: 0x1559 */
	0x7BD0,0x7C2F,0x7C32,0x7C42,0x7C4E,0x7C68,0x7CA9,0x7CED,
	0x7DD0,0x7E07,0x7DD3,0x7E64,0x7F40,INVALC,0x8041,0x8063,
	0x80BB,0x6711,0x6725,0x8248,0x8310,0x8362,0x8312,0x8421,
	0x841E,0x84E2,0x84DE,0x84E1,0x8573,0x85D4,0x85F5,0x8637,
	0x8645,0x8672,0x874A,0x87A9,0x87A5,0x87F5,0x8834,0x8850,
	0x8887,0x8954,0x8984,0x8B03,0x8C52,0x8CD8,0x8D0C,0x8D18,
	0x8DB0,0x8EBC,0x8ED5,0x8FAA,0x909C,INVALC,0x915C,0x922B,
	0x9221,0x9273,0x92F4,0x92F5,0x933F,0x9342,0x9386,0x93BE,
	0x93BC,0x93BD,0x93F1,0x93F2,0x93EF,0x9422,0x9423,0x9424,
	0x9467,0x9466,0x9597,0x95CE,0x95E7,0x973B,0x974D,0x98E4,
	0x9942,0x9B1D,0x9B98,INVALC,0x9D49,0x6449,0x5E71,0x5E85,
	0x61D3,0x990E,0x8002,0x781E,
	/* Range 0x6721 - 0x672A, array index: 0x15B5 */
	0x5528,0x5572,0x55BA,0x55F0,0x55EE,0x56B8,0x56B9,0x56C4,
	0x8053,0x92B0,
	/* Unranged codes (222 codes) */
	/* Array index: 0x15BF */ 0x2170,0x4EE7,
	/* Array index: 0x15BF */ 0x2172,0x4EE6,
	/* Array index: 0x15BF */ 0x234B,0x597A,
	/* Array index: 0x15BF */ 0x234D,0x5986,
	/* Array index: 0x15BF */ 0x235A,0x5C80,
	/* Array index: 0x15BF */ 0x235C,0x5E08,
	/* Array index: 0x15BF */ 0x2361,0x5EF5,
	/* Array index: 0x15BF */ 0x2362,0x5F0E,
	/* Array index: 0x15BF */ 0x2366,0x5FD3,
	/* Array index: 0x15BF */ 0x2367,0x5FDA,
	/* Array index: 0x15BF */ 0x2421,0x6C3C,
	/* Array index: 0x15BF */ 0x2433,0x793C,
	/* Array index: 0x15BF */ 0x2435,0x808D,
	/* Array index: 0x15BF */ 0x2436,0x808E,
	/* Array index: 0x15BF */ 0x246D,0x544D,
	/* Array index: 0x15BF */ 0x2470,0x543A,
	/* Array index: 0x15BF */ 0x253C,0x5746,
	/* Array index: 0x15BF */ 0x2551,0x599F,
	/* Array index: 0x15BF */ 0x2553,0x599B,
	/* Array index: 0x15BF */ 0x2555,0x5989,
	/* Array index: 0x15BF */ 0x2556,0x599A,
	/* Array index: 0x15BF */ 0x2558,0x6588,
	/* Array index: 0x15BF */ 0x263D,0x65F6,
	/* Array index: 0x15BF */ 0x266E,0x7398,
	/* Array index: 0x15BF */ 0x2755,0x5190,
	/* Array index: 0x15BF */ 0x2833,0x59AD,
	/* Array index: 0x15BF */ 0x2835,0x59C4,
	/* Array index: 0x15BF */ 0x2837,0x59C2,
	/* Array index: 0x15BF */ 0x2838,0x59B0,
	/* Array index: 0x15BF */ 0x2845,0x59B7,
	/* Array index: 0x15BF */ 0x2846,0x59D7,
	/* Array index: 0x15BF */ 0x287E,0x5FE9,
	/* Array index: 0x15BF */ 0x2921,0x603D,
	/* Array index: 0x15BF */ 0x2922,0x6008,
	/* Array index: 0x15BF */ 0x295D,0x6C1C,
	/* Array index: 0x15BF */ 0x297C,0x7240,
	/* Array index: 0x15BF */ 0x2A42,0x79C7,
	/* Array index: 0x15BF */ 0x2A44,0x79CA,
	/* Array index: 0x15BF */ 0x2A52,0x80B0,
	/* Array index: 0x15BF */ 0x2A66,0x82AA,
	/* Array index: 0x15BF */ 0x2A68,0x82C9,
	/* Array index: 0x15BF */ 0x2A6B,0x82A6,
	/* Array index: 0x15BF */ 0x2A6C,0x82B2,
	/* Array index: 0x15BF */ 0x2B43,0x524F,
	/* Array index: 0x15BF */ 0x2B61,0x54CD,
	/* Array index: 0x15BF */ 0x2B77,0x58F5,
	/* Array index: 0x15BF */ 0x2B79,0x5909,
	/* Array index: 0x15BF */ 0x2B7A,0x5908,
	/* Array index: 0x15BF */ 0x2B7C,0x5952,
	/* Array index: 0x15BF */ 0x2C32,0x5BAB,
	/* Array index: 0x15BF */ 0x2C34,0x5C1B,
	/* Array index: 0x15BF */ 0x2C35,0x5C2F,
	/* Array index: 0x15BF */ 0x2C37,0x663C,
	/* Array index: 0x15BF */ 0x2C50,0x5F66,
	/* Array index: 0x15BF */ 0x2C54,0x605C,
	/* Array index: 0x15BF */ 0x2D42,0x6BE1,
	/* Array index: 0x15BF */ 0x2E2F,0x75A8,
	/* Array index: 0x15BF */ 0x2E72,0x82E9,
	/* Array index: 0x15BF */ 0x2F7E,0x5518,
	/* Array index: 0x15BF */ 0x3021,0x54F0,
	/* Array index: 0x15BF */ 0x3022,0x54F6,
	/* Array index: 0x15BF */ 0x3025,0x5519,
	/* Array index: 0x15BF */ 0x3045,0x5A22,
	/* Array index: 0x15BF */ 0x3048,0x5A24,
	/* Array index: 0x15BF */ 0x3052,0x5A26,
	/* Array index: 0x15BF */ 0x307E,0x6056,
	/* Array index: 0x15BF */ 0x3236,0x727A,
	/* Array index: 0x15BF */ 0x3255,0x75B4,
	/* Array index: 0x15BF */ 0x3257,0x75B1,
	/* Array index: 0x15BF */ 0x3337,0x7D24,
	/* Array index: 0x15BF */ 0x3352,0x80F7,
	/* Array index: 0x15BF */ 0x335F,0x8321,
	/* Array index: 0x15BF */ 0x3362,0x8318,
	/* Array index: 0x15BF */ 0x3363,0x8358,
	/* Array index: 0x15BF */ 0x3439,0x96BD,
	/* Array index: 0x15BF */ 0x3522,0x5A2B,
	/* Array index: 0x15BF */ 0x3524,0x5A3B,
	/* Array index: 0x15BF */ 0x353F,0x5BC8,
	/* Array index: 0x15BF */ 0x3541,0x5BC3,
	/* Array index: 0x15BF */ 0x3568,0x6098,
	/* Array index: 0x15BF */ 0x356A,0x60D0,
	/* Array index: 0x15BF */ 0x3579,0x60DE,
	/* Array index: 0x15BF */ 0x357C,0x637E,
	/* Array index: 0x15BF */ 0x357D,0x638B,
	/* Array index: 0x15BF */ 0x3740,0x73FB,
	/* Array index: 0x15BF */ 0x374D,0x7690,
	/* Array index: 0x15BF */ 0x3821,0x7C9A,
	/* Array index: 0x15BF */ 0x3822,0x7C9C,
	/* Array index: 0x15BF */ 0x3824,0x7D49,
	/* Array index: 0x15BF */ 0x3826,0x7D34,
	/* Array index: 0x15BF */ 0x3827,0x7D37,
	/* Array index: 0x15BF */ 0x3829,0x7D2D,
	/* Array index: 0x15BF */ 0x382B,0x7D4C,
	/* Array index: 0x15BF */ 0x382E,0x7D48,
	/* Array index: 0x15BF */ 0x3831,0x7F3B,
	/* Array index: 0x15BF */ 0x3836,0x8008,
	/* Array index: 0x15BF */ 0x3837,0x801A,
	/* Array index: 0x15BF */ 0x387E,0x8A1C,
	/* Array index: 0x15BF */ 0x3922,0x8A2B,
	/* Array index: 0x15BF */ 0x3923,0x8A20,
	/* Array index: 0x15BF */ 0x3925,0x8A29,
	/* Array index: 0x15BF */ 0x3947,0x8ED9,
	/* Array index: 0x15BF */ 0x395C,0x9587,
	/* Array index: 0x15BF */ 0x395E,0x965A,
	/* Array index: 0x15BF */ 0x3961,0x966E,
	/* Array index: 0x15BF */ 0x3965,0x9679,
	/* Array index: 0x15BF */ 0x3A21,0x5273,
	/* Array index: 0x15BF */ 0x3A23,0x5270,
	/* Array index: 0x15BF */ 0x3A66,0x5A91,
	/* Array index: 0x15BF */ 0x3A6B,0x5ACF,
	/* Array index: 0x15BF */ 0x3A72,0x5A87,
	/* Array index: 0x15BF */ 0x3A73,0x5AA0,
	/* Array index: 0x15BF */ 0x3B31,0x5D57,
	/* Array index: 0x15BF */ 0x3B4B,0x5FA7,
	/* Array index: 0x15BF */ 0x3B79,0x65D1,
	/* Array index: 0x15BF */ 0x3D4C,0x7694,
	/* Array index: 0x15BF */ 0x3D4D,0x76B3,
	/* Array index: 0x15BF */ 0x3D7E,0x7CA7,
	/* Array index: 0x15BF */ 0x3E6D,0x88BB,
	/* Array index: 0x15BF */ 0x3E6F,0x88BF,
	/* Array index: 0x15BF */ 0x3E70,0x88B4,
	/* Array index: 0x15BF */ 0x3E72,0x88B5,
	/* Array index: 0x15BF */ 0x3E74,0x899A,
	/* Array index: 0x15BF */ 0x3E75,0x8A43,
	/* Array index: 0x15BF */ 0x3E78,0x8A5A,
	/* Array index: 0x15BF */ 0x3F5D,0x91FE,
	/* Array index: 0x15BF */ 0x3F5E,0x9220,
	/* Array index: 0x15BF */ 0x3F60,0x920B,
	/* Array index: 0x15BF */ 0x3F6C,0x9595,
	/* Array index: 0x15BF */ 0x3F7B,0x96EE,
	/* Array index: 0x15BF */ 0x3F7C,0x96ED,
	/* Array index: 0x15BF */ 0x3F7E,0x96EC,
	/* Array index: 0x15BF */ 0x4021,0x975F,
	/* Array index: 0x15BF */ 0x4022,0x976F,
	/* Array index: 0x15BF */ 0x4024,0x976D,
	/* Array index: 0x15BF */ 0x402B,0x98F0,
	/* Array index: 0x15BF */ 0x402F,0x9AA9,
	/* Array index: 0x15BF */ 0x4032,0x9AE0,
	/* Array index: 0x15BF */ 0x4033,0x4EB7,
	/* Array index: 0x15BF */ 0x4064,0x5ACE,
	/* Array index: 0x15BF */ 0x4066,0x5AC5,
	/* Array index: 0x15BF */ 0x4067,0x5AC3,
	/* Array index: 0x15BF */ 0x406A,0x5AD0,
	/* Array index: 0x15BF */ 0x4142,0x643C,
	/* Array index: 0x15BF */ 0x4144,0x6446,
	/* Array index: 0x15BF */ 0x4145,0x6447,
	/* Array index: 0x15BF */ 0x4148,0x643A,
	/* Array index: 0x15BF */ 0x4149,0x6407,
	/* Array index: 0x15BF */ 0x414B,0x656B,
	/* Array index: 0x15BF */ 0x4156,0x668F,
	/* Array index: 0x15BF */ 0x4159,0x6692,
	/* Array index: 0x15BF */ 0x415B,0x668E,
	/* Array index: 0x15BF */ 0x415D,0x6946,
	/* Array index: 0x15BF */ 0x4165,0x6931,
	/* Array index: 0x15BF */ 0x4168,0x693E,
	/* Array index: 0x15BF */ 0x416A,0x697C,
	/* Array index: 0x15BF */ 0x416B,0x6943,
	/* Array index: 0x15BF */ 0x416D,0x6973,
	/* Array index: 0x15BF */ 0x416F,0x6955,
	/* Array index: 0x15BF */ 0x4221,0x6B73,
	/* Array index: 0x15BF */ 0x4222,0x6B9C,
	/* Array index: 0x15BF */ 0x4236,0x6EA9,
	/* Array index: 0x15BF */ 0x424C,0x7148,
	/* Array index: 0x15BF */ 0x4321,0x7698,
	/* Array index: 0x15BF */ 0x433D,0x7980,
	/* Array index: 0x15BF */ 0x4341,0x7A0F,
	/* Array index: 0x15BF */ 0x4344,0x7A1D,
	/* Array index: 0x15BF */ 0x4358,0x7CAE,
	/* Array index: 0x15BF */ 0x435C,0x7CB0,
	/* Array index: 0x15BF */ 0x435E,0x7D90,
	/* Array index: 0x15BF */ 0x436E,0x7FA3,
	/* Array index: 0x15BF */ 0x437B,0x815D,
	/* Array index: 0x15BF */ 0x437C,0x8175,
	/* Array index: 0x15BF */ 0x437E,0x815F,
	/* Array index: 0x15BF */ 0x4473,0x8EB1,
	/* Array index: 0x15BF */ 0x4540,0x968F,
	/* Array index: 0x15BF */ 0x4542,0x9696,
	/* Array index: 0x15BF */ 0x4546,0x96F4,
	/* Array index: 0x15BF */ 0x4547,0x96FC,
	/* Array index: 0x15BF */ 0x4549,0x9755,
	/* Array index: 0x15BF */ 0x454B,0x9779,
	/* Array index: 0x15BF */ 0x454F,0x97EE,
	/* Array index: 0x15BF */ 0x4550,0x97F5,
	/* Array index: 0x15BF */ 0x4552,0x980B,
	/* Array index: 0x15BF */ 0x4554,0x98F3,
	/* Array index: 0x15BF */ 0x457A,0x51F4,
	/* Array index: 0x15BF */ 0x457E,0x52ED,
	/* Array index: 0x15BF */ 0x4621,0x52EA,
	/* Array index: 0x15BF */ 0x4623,0x5332,
	/* Array index: 0x15BF */ 0x467E,0x6455,
	/* Array index: 0x15BF */ 0x4730,0x6571,
	/* Array index: 0x15BF */ 0x4751,0x6998,
	/* Array index: 0x15BF */ 0x482E,0x76B9,
	/* Array index: 0x15BF */ 0x484B,0x7A29,
	/* Array index: 0x15BF */ 0x484D,0x7A2A,
	/* Array index: 0x15BF */ 0x484F,0x7A2D,
	/* Array index: 0x15BF */ 0x4850,0x7A2C,
	/* Array index: 0x15BF */ 0x4868,0x7DC8,
	/* Array index: 0x15BF */ 0x4869,0x7DB6,
	/* Array index: 0x15BF */ 0x486B,0x7DD1,
	/* Array index: 0x15BF */ 0x496B,0x8A9D,
	/* Array index: 0x15BF */ 0x496D,0x8C67,
	/* Array index: 0x15BF */ 0x4A21,0x8FA1,
	/* Array index: 0x15BF */ 0x4A22,0x8FA2,
	/* Array index: 0x15BF */ 0x4A24,0x905A,
	/* Array index: 0x15BF */ 0x4A3D,0x95A2,
	/* Array index: 0x15BF */ 0x4A3E,0x95A7,
	/* Array index: 0x15BF */ 0x4A4C,0x9759,
	/* Array index: 0x15BF */ 0x4A4E,0x9764,
	/* Array index: 0x15BF */ 0x4A5B,0x9906,
	/* Array index: 0x15BF */ 0x4A6A,0x99C4,
	/* Array index: 0x15BF */ 0x4A78,0x9B40,
	/* Array index: 0x15BF */ 0x4B22,0x9EBD,
	/* Array index: 0x15BF */ 0x4B27,0x510E,
	/* Array index: 0x15BF */ 0x4D22,0x71AE,
	/* Array index: 0x15BF */ 0x4D44,0x778A,
	/* Array index: 0x15BF */ 0x4D60,0x7BBF,
	/* Array index: 0x15BF */ 0x4D63,0x7BCD,
	/* Array index: 0x15BF */ 0x4D65,0x7BB2,
	/* Array index: 0x15BF */ 0x4E22,0x7DD6,
	/* Array index: 0x15BF */ 0x4E40,0x8501,
	/* Array index: 0x15BF */ 0x4E42,0x84F8,
	/* Array index: 0x15BF */ 0x4E44,0x84F5,
	/* Array index: 0x15BF */ 0x4E46,0x8504,
	/* Array index: 0x15BF */ 0x4E52,0x8535,
	/* Array index: 0x15BF */ 0x4E54,0x8505,
	/* Array index: 0x15BF */ 0x4E59,0x877D,
	/* Array index: 0x15BF */ 0x4E7A,0x8C8E,
	/* Array index: 0x15BF */ 0x4E7D,0x8CE9,
	/* Array index: 0x15BF */ 0x4E7E,0x8CDB,
	/* Array index: 0x15BF */ 0x4F5A,0x96A3,
	/* Array index: 0x15BF */ 0x4F5B,0x96A5,
	/* Array index: 0x15BF */ 0x5022,0x9D0D,
	/* Array index: 0x15BF */ 0x5024,0x9D01,
	/* Array index: 0x15BF */ 0x5025,0x9D0C,
	/* Array index: 0x15BF */ 0x5027,0x9CF8,
	/* Array index: 0x15BF */ 0x503B,0x5667,
	/* Array index: 0x15BF */ 0x503D,0x567B,
	/* Array index: 0x15BF */ 0x503F,0x565F,
	/* Array index: 0x15BF */ 0x5040,0x5661,
	/* Array index: 0x15BF */ 0x514D,0x6FBB,
	/* Array index: 0x15BF */ 0x5150,0x6FBE,
	/* Array index: 0x15BF */ 0x5179,0x7798,
	/* Array index: 0x15BF */ 0x517B,0x7796,
	/* Array index: 0x15BF */ 0x5221,0x78D6,
	/* Array index: 0x15BF */ 0x5222,0x78EB,
	/* Array index: 0x15BF */ 0x5237,0x7CD3,
	/* Array index: 0x15BF */ 0x5239,0x7CE1,
	/* Array index: 0x15BF */ 0x523B,0x7E19,
	/* Array index: 0x15BF */ 0x523F,0x7E27,
	/* Array index: 0x15BF */ 0x5240,0x7E26,
	/* Array index: 0x15BF */ 0x5243,0x806E,
	/* Array index: 0x15BF */ 0x5244,0x81AF,
	/* Array index: 0x15BF */ 0x5247,0x81AD,
	/* Array index: 0x15BF */ 0x5249,0x81AA,
	/* Array index: 0x15BF */ 0x524A,0x8218,
	/* Array index: 0x15BF */ 0x524F,0x856F,
	/* Array index: 0x15BF */ 0x5250,0x854C,
	/* Array index: 0x15BF */ 0x526E,0x8AEC,
	/* Array index: 0x15BF */ 0x5273,0x8CF2,
	/* Array index: 0x15BF */ 0x5275,0x8CEF,
	/* Array index: 0x15BF */ 0x5277,0x8DA6,
	/* Array index: 0x15BF */ 0x527B,0x8E3B,
	/* Array index: 0x15BF */ 0x527C,0x8E43,
	/* Array index: 0x15BF */ 0x527E,0x8E32,
	/* Array index: 0x15BF */ 0x5348,0x96AB,
	/* Array index: 0x15BF */ 0x5349,0x96B7,
	/* Array index: 0x15BF */ 0x534C,0x9715,
	/* Array index: 0x15BF */ 0x534D,0x9714,
	/* Array index: 0x15BF */ 0x5350,0x970C,
	/* Array index: 0x15BF */ 0x5351,0x9717,
	/* Array index: 0x15BF */ 0x5353,0x9793,
	/* Array index: 0x15BF */ 0x5355,0x97D2,
	/* Array index: 0x15BF */ 0x5370,0x99E1,
	/* Array index: 0x15BF */ 0x5371,0x99E6,
	/* Array index: 0x15BF */ 0x5421,0x9B8C,
	/* Array index: 0x15BF */ 0x5423,0x9B89,
	/* Array index: 0x15BF */ 0x5425,0x9B8E,
	/* Array index: 0x15BF */ 0x5464,0x64DF,
	/* Array index: 0x15BF */ 0x5467,0x64E1,
	/* Array index: 0x15BF */ 0x5521,0x6A86,
	/* Array index: 0x15BF */ 0x5523,0x6A98,
	/* Array index: 0x15BF */ 0x5524,0x6A9D,
	/* Array index: 0x15BF */ 0x5527,0x6A8F,
	/* Array index: 0x15BF */ 0x5529,0x6AAA,
	/* Array index: 0x15BF */ 0x552B,0x6B5D,
	/* Array index: 0x15BF */ 0x553C,0x71E3,
	/* Array index: 0x15BF */ 0x5549,0x7371,
	/* Array index: 0x15BF */ 0x554B,0x74AE,
	/* Array index: 0x15BF */ 0x554D,0x74B3,
	/* Array index: 0x15BF */ 0x554F,0x74AC,
	/* Array index: 0x15BF */ 0x5579,0x7E2C,
	/* Array index: 0x15BF */ 0x557A,0x7E4D,
	/* Array index: 0x15BF */ 0x557D,0x7F46,
	/* Array index: 0x15BF */ 0x557E,0x7FF6,
	/* Array index: 0x15BF */ 0x5630,0x85AC,
	/* Array index: 0x15BF */ 0x5634,0x87CE,
	/* Array index: 0x15BF */ 0x5636,0x87CD,
	/* Array index: 0x15BF */ 0x564D,0x8C40,
	/* Array index: 0x15BF */ 0x564F,0x8C96,
	/* Array index: 0x15BF */ 0x567A,0x979B,
	/* Array index: 0x15BF */ 0x567B,0x979F,
	/* Array index: 0x15BF */ 0x5726,0x9840,
	/* Array index: 0x15BF */ 0x5728,0x9847,
	/* Array index: 0x15BF */ 0x572A,0x98B7,
	/* Array index: 0x15BF */ 0x5730,0x99A2,
	/* Array index: 0x15BF */ 0x5733,0x9A00,
	/* Array index: 0x15BF */ 0x5734,0x99F3,
	/* Array index: 0x15BF */ 0x5737,0x99F5,
	/* Array index: 0x15BF */ 0x5748,0x9D39,
	/* Array index: 0x15BF */ 0x574A,0x9D44,
	/* Array index: 0x15BF */ 0x574D,0x9D35,
	/* Array index: 0x15BF */ 0x5750,0x9EAF,
	/* Array index: 0x15BF */ 0x5752,0x512F,
	/* Array index: 0x15BF */ 0x5849,0x7AF5,
	/* Array index: 0x15BF */ 0x584C,0x7C2E,
	/* Array index: 0x15BF */ 0x584E,0x7C1B,
	/* Array index: 0x15BF */ 0x5850,0x7C1A,
	/* Array index: 0x15BF */ 0x5851,0x7C24,
	/* Array index: 0x15BF */ 0x5861,0x7FFA,
	/* Array index: 0x15BF */ 0x5862,0x802E,
	/* Array index: 0x15BF */ 0x5865,0x81CE,
	/* Array index: 0x15BF */ 0x5868,0x8219,
	/* Array index: 0x15BF */ 0x5921,0x89B3,
	/* Array index: 0x15BF */ 0x5923,0x8B38,
	/* Array index: 0x15BF */ 0x5924,0x8B32,
	/* Array index: 0x15BF */ 0x5926,0x8B2D,
	/* Array index: 0x15BF */ 0x5928,0x8B34,
	/* Array index: 0x15BF */ 0x592A,0x8B29,
	/* Array index: 0x15BF */ 0x592B,0x8C74,
	/* Array index: 0x15BF */ 0x592E,0x8D03,
	/* Array index: 0x15BF */ 0x5956,0x9854,
	/* Array index: 0x15BF */ 0x5A3F,0x65DC,
	/* Array index: 0x15BF */ 0x5A6E,0x7C35,
	/* Array index: 0x15BF */ 0x5A6F,0x7C34,
	/* Array index: 0x15BF */ 0x5A72,0x7E6C,
	/* Array index: 0x15BF */ 0x5B22,0x85E7,
	/* Array index: 0x15BF */ 0x5B2F,0x8987,
	/* Array index: 0x15BF */ 0x5B53,0x95DD,
	/* Array index: 0x15BF */ 0x5B65,0x9856,
	/* Array index: 0x15BF */ 0x5C47,0x6AF4,
	/* Array index: 0x15BF */ 0x5C49,0x6AE9,
	/* Array index: 0x15BF */ 0x5C4F,0x703D,
	/* Array index: 0x15BF */ 0x5C51,0x7036,
	/* Array index: 0x15BF */ 0x5D56,0x98C3,
	/* Array index: 0x15BF */ 0x5D58,0x994D,
	/* Array index: 0x15BF */ 0x5D5A,0x9A2F,
	/* Array index: 0x15BF */ 0x5E2D,0x5DD0,
	/* Array index: 0x15BF */ 0x5E2F,0x5FC2,
	/* Array index: 0x15BF */ 0x5E6D,0x942D,
	/* Array index: 0x15BF */ 0x5F3A,0x9E9E,
	/* Array index: 0x15BF */ 0x5F5F,0x8827,
	/* Array index: 0x15BF */ 0x5F61,0x8B81,
	/* Array index: 0x15BF */ 0x5F62,0x8B83,
	/* Array index: 0x15BF */ 0x5F64,0x8C44,
	/* Array index: 0x15BF */ 0x5F78,0x995A,
	/* Array index: 0x15BF */ 0x5F79,0x9A51,
	/* Array index: 0x15BF */ 0x5F7B,0x9ADD,
	/* Array index: 0x15BF */ 0x5F7E,0x9C38,
	/* Array index: 0x15BF */ 0x6022,0x9C45,
	/* Array index: 0x15BF */ 0x6023,0x9C3A,
	/* Array index: 0x15BF */ 0x6025,0x9C35,
	/* Array index: 0x15BF */ 0x6029,0x9EF1,
	/* Array index: 0x15BF */ 0x602B,0x9F93,
	/* Array index: 0x15BF */ 0x602C,0x529A,
	/* Array index: 0x15BF */ 0x602F,0x8641,
	/* Array index: 0x15BF */ 0x6030,0x5DD7,
	/* Array index: 0x15BF */ 0x6032,0x6528,
	/* Array index: 0x15BF */ 0x6036,0x7053,
	/* Array index: 0x15BF */ 0x6037,0x7059,
	/* Array index: 0x15BF */ 0x604F,0x945F,
	/* Array index: 0x15BF */ 0x606C,0x9DFC,
	/* Array index: 0x15BF */ 0x606D,0x9F39,
	/* Array index: 0x15BF */ 0x606F,0x513E,
	/* Array index: 0x15BF */ 0x6071,0x56D2,
	/* Array index: 0x15BF */ 0x607B,0x8B91,
	/* Array index: 0x15BF */ 0x607E,0x91BF,
	/* Array index: 0x15BF */ 0x6122,0x946C,
	/* Array index: 0x15BF */ 0x613B,0x9F45,
	/* Array index: 0x15BF */ 0x6159,0x9C6C,
	/* Array index: 0x15BF */ 0x615B,0x9C6F,
	/* Array index: 0x15BF */ 0x623F,0x9EA3,
	/* Array index: 0x15BF */ 0x6241,0x9C7B,
	/* Array index: 0x15BF */ 0x6242,0x9F97,
	/* Array index: 0x15BF */ 0x6245,0x9750,
	/* Array index: 0x15BF */ 0x6249,0x5727,
	/* Array index: 0x15BF */ 0x624A,0x5C13,
	/* Array index: 0x15BF */ 0x6251,0x5FC8,
	/* Array index: 0x15BF */ 0x6257,0x6765,
	/* Array index: 0x15BF */ 0x625A,0x52BD,
	/* Array index: 0x15BF */ 0x6279,0x70C9,
	/* Array index: 0x15BF */ 0x6323,0x9579,
	/* Array index: 0x15BF */ 0x6325,0x96BA,
	/* Array index: 0x15BF */ 0x632D,0x7B29,
	/* Array index: 0x15BF */ 0x632E,0x8128,
	/* Array index: 0x15BF */ 0x6330,0x8A2E,
	/* Array index: 0x15BF */ 0x6334,0x9AD9,
	/* Array index: 0x15BF */ 0x6336,0x582B,
	/* Array index: 0x15BF */ 0x6337,0x5845,
	/* Array index: 0x15BF */ 0x6339,0x63FA,
	/* Array index: 0x15BF */ 0x633D,0x6E86,
	/* Array index: 0x15BF */ 0x6343,0x5867,
	/* Array index: 0x15BF */ 0x6345,0x5BDD,
	/* Array index: 0x15BF */ 0x6346,0x656E,
	/* Array index: 0x15BF */ 0x634A,0x8C87,
	/* Array index: 0x15BF */ 0x634C,0x50D2,
	/* Array index: 0x15BF */ 0x634D,0x50DF,
	/* Array index: 0x15BF */ 0x6352,0x69BA,
	/* Array index: 0x15BF */ 0x6354,0x6B9D,
	/* Array index: 0x15BF */ 0x6356,0x8059,
	/* Array index: 0x15BF */ 0x6363,0x6F8A,
	/* Array index: 0x15BF */ 0x6366,0x7BC3,
	/* Array index: 0x15BF */ 0x6367,0x7BC2,
	/* Array index: 0x15BF */ 0x636C,0x90F6,
	/* Array index: 0x15BF */ 0x636E,0x9823,
	/* Array index: 0x15BF */ 0x6374,0x71CD,
	/* Array index: 0x15BF */ 0x6375,0x7499,
	/* Array index: 0x15BF */ 0x637B,0x9842,
	/* Array index: 0x15BF */ 0x6422,0x7F84,
	/* Array index: 0x15BF */ 0x6428,0x8D0E,
	/* Array index: 0x15BF */ 0x642A,0x9861,
	/* Array index: 0x15BF */ 0x642D,0x8B73,
};

#endif /* ICONV_TO_UCS_CCS_CNS11643_PLANE14 && defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit UCS -> cns11643_plane14 speed-optimized table (42496 bytes).
 * ======================================================================
 */

#if defined (ICONV_FROM_UCS_CCS_CNS11643_PLANE14) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
from_ucs_speed_cns11643_plane14[] =
{
	/* Heading Block */
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,0x0100,0x0200,
	0x0300,0x0400,0x0500,0x0600,0x0700,0x0800,0x0900,0x0A00,
	0x0B00,0x0C00,0x0D00,0x0E00,0x0F00,0x1000,0x1100,0x1200,
	0x1300,0x1400,0x1500,0x1600,0x1700,0x1800,0x1900,0x1A00,
	0x1B00,0x1C00,0x1D00,0x1E00,0x1F00,0x2000,0x2100,0x2200,
	0x2300,0x2400,0x2500,0x2600,0x2700,0x2800,0x2900,0x2A00,
	0x2B00,0x2C00,0x2D00,0x2E00,0x2F00,0x3000,0x3100,0x3200,
	0x3300,0x3400,0x3500,0x3600,0x3700,0x3800,0x3900,0x3A00,
	0x3B00,0x3C00,0x3D00,0x3E00,0x3F00,0x4000,0x4100,0x4200,
	0x4300,0x4400,0x4500,0x4600,0x4700,0x4800,0x4900,0x4A00,
	0x4B00,0x4C00,0x4D00,0x4E00,0x4F00,0x5000,0x5100,0x5200,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	/* Block 79, Array index 0x0100 */
	INVALC,INVALC,INVALC,INVALC,0x2126,0x2125,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2262,0x2261,0x2263,INVALC,0x243F,INVALC,INVALC,INVALC,
	0x2121,INVALC,0x212C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2122,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2740,INVALC,0x2123,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2130,INVALC,
	INVALC,0x212E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2345,INVALC,INVALC,0x2B22,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2146,0x2145,INVALC,INVALC,INVALC,INVALC,
	0x2264,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2440,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2B23,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x396C,0x396E,INVALC,INVALC,INVALC,0x2124,INVALC,0x212D,
	INVALC,0x2265,INVALC,INVALC,INVALC,INVALC,INVALC,0x2131,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x216C,
	0x2266,INVALC,INVALC,INVALC,0x2442,0x2742,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2147,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2B26,
	0x2B25,0x2B27,0x2B24,INVALC,INVALC,INVALC,INVALC,0x4033,
	INVALC,INVALC,INVALC,INVALC,0x2132,INVALC,0x2133,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2149,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x214C,INVALC,0x214B,0x214A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x216D,INVALC,INVALC,INVALC,INVALC,INVALC,0x2172,0x2170,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x226F,0x2268,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x226B,0x2267,0x2276,INVALC,0x226E,INVALC,INVALC,INVALC,
	/* Block 80, Array index 0x0200 */
	INVALC,INVALC,INVALC,0x226D,INVALC,INVALC,0x226C,0x2274,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2270,0x2277,
	INVALC,INVALC,0x2275,INVALC,0x2273,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2272,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2450,0x2448,INVALC,INVALC,0x244F,INVALC,0x2446,
	INVALC,0x2445,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x244E,INVALC,0x2449,INVALC,0x244B,0x2443,INVALC,INVALC,
	INVALC,INVALC,0x2444,0x244C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6448,INVALC,
	0x6449,INVALC,INVALC,INVALC,INVALC,0x2747,INVALC,INVALC,
	INVALC,INVALC,0x274F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2744,INVALC,0x2751,INVALC,0x2748,INVALC,
	INVALC,INVALC,INVALC,0x2743,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x274C,
	0x2749,0x274B,0x274A,0x274D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2B2D,0x2B2E,INVALC,INVALC,0x2B36,INVALC,INVALC,INVALC,
	0x2B2C,INVALC,INVALC,INVALC,INVALC,0x6263,0x2B2B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2B32,INVALC,
	0x2B2F,INVALC,INVALC,INVALC,0x2B33,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2B28,INVALC,INVALC,0x2B29,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2B35,INVALC,0x2B34,INVALC,0x644A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2F48,INVALC,0x2F4D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2F44,INVALC,0x2F58,0x2F4C,0x2F56,INVALC,0x2F46,
	/* Block 81, Array index 0x0300 */
	INVALC,INVALC,0x2F4B,0x2F49,0x2F47,INVALC,INVALC,INVALC,
	0x2F4F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2F53,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2F45,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2F4E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2F51,INVALC,
	INVALC,INVALC,0x2F43,INVALC,INVALC,INVALC,0x2F50,INVALC,
	0x2F54,0x2F55,INVALC,INVALC,INVALC,INVALC,INVALC,0x644B,
	INVALC,INVALC,0x343C,INVALC,0x343F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3444,INVALC,0x3441,INVALC,0x3442,INVALC,0x2F57,INVALC,
	0x3447,0x343D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3440,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x344B,INVALC,INVALC,INVALC,
	INVALC,0x3443,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x344C,0x344A,INVALC,0x3445,0x3446,INVALC,INVALC,INVALC,
	INVALC,0x3975,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3971,0x3972,INVALC,INVALC,INVALC,INVALC,INVALC,0x396F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3970,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x644D,INVALC,
	INVALC,INVALC,0x4039,0x403C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x403A,INVALC,INVALC,0x4037,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x403D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4036,0x403E,INVALC,INVALC,
	0x456B,INVALC,0x634C,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x456E,0x456C,INVALC,INVALC,0x456D,INVALC,0x4576,0x634D,
	INVALC,0x4570,0x4575,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4571,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4574,INVALC,INVALC,0x4B29,
	INVALC,INVALC,INVALC,INVALC,0x4B2B,INVALC,INVALC,INVALC,
	/* Block 82, Array index 0x0400 */
	INVALC,0x4B2D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B2C,0x4B27,0x644E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5031,INVALC,
	INVALC,INVALC,INVALC,0x5035,INVALC,0x5030,0x5034,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5441,
	0x5440,INVALC,INVALC,0x5033,INVALC,INVALC,INVALC,0x5752,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C32,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x606F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2278,INVALC,INVALC,INVALC,0x2454,INVALC,
	0x2453,0x2451,INVALC,INVALC,INVALC,INVALC,0x2752,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3976,INVALC,INVALC,INVALC,0x5E25,INVALC,0x2134,INVALC,
	INVALC,INVALC,0x2B37,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x227A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2E7B,INVALC,INVALC,INVALC,INVALC,0x403F,0x6453,
	INVALC,INVALC,0x2127,INVALC,0x214D,0x2148,0x214E,INVALC,
	INVALC,INVALC,INVALC,0x2178,0x2179,INVALC,INVALC,INVALC,
	0x2755,INVALC,INVALC,INVALC,INVALC,INVALC,0x2128,INVALC,
	INVALC,0x217A,INVALC,INVALC,INVALC,0x2457,INVALC,0x2B39,
	INVALC,0x2F5B,INVALC,0x2F59,INVALC,INVALC,0x2F5A,INVALC,
	0x344D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x227E,0x227D,0x227C,0x2459,INVALC,INVALC,
	0x245A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2B3B,0x2B3D,0x2B3E,INVALC,INVALC,INVALC,0x2F5E,
	INVALC,0x2F5F,INVALC,INVALC,INVALC,INVALC,INVALC,0x344F,
	INVALC,0x344E,0x397D,0x397A,INVALC,INVALC,0x397E,INVALC,
	INVALC,0x4B2F,0x4B2E,0x4B30,INVALC,INVALC,INVALC,0x5442,
	INVALC,INVALC,0x212F,0x2135,0x2151,0x217B,INVALC,INVALC,
	0x2322,INVALC,INVALC,INVALC,0x245B,0x2759,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x457A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x275A,INVALC,
	/* Block 83, Array index 0x0500 */
	INVALC,INVALC,INVALC,INVALC,0x2136,0x2152,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x217D,INVALC,INVALC,INVALC,0x2325,
	INVALC,INVALC,INVALC,INVALC,0x2324,0x2326,INVALC,INVALC,
	0x2327,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x245F,
	0x2461,INVALC,INVALC,0x245C,INVALC,INVALC,0x245E,0x245D,
	INVALC,INVALC,INVALC,0x2460,INVALC,INVALC,INVALC,0x275B,
	INVALC,INVALC,INVALC,INVALC,0x275E,INVALC,INVALC,INVALC,
	INVALC,0x275F,INVALC,INVALC,0x275D,INVALC,0x644F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2B3F,INVALC,INVALC,
	0x2B40,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2B43,
	INVALC,INVALC,INVALC,0x2F68,INVALC,INVALC,INVALC,0x2F65,
	INVALC,0x2F62,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2F60,INVALC,INVALC,0x2F66,0x2F61,0x2F63,INVALC,0x2F64,
	0x3450,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A23,INVALC,INVALC,0x3A21,INVALC,INVALC,0x3451,INVALC,
	INVALC,0x4041,INVALC,INVALC,INVALC,INVALC,0x4040,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B31,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B32,INVALC,
	0x5036,INVALC,INVALC,INVALC,0x5037,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x602C,INVALC,0x2137,0x2154,0x2153,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2328,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2463,0x2462,0x2761,INVALC,INVALC,
	INVALC,0x2760,INVALC,INVALC,INVALC,0x625A,INVALC,0x2762,
	INVALC,INVALC,INVALC,INVALC,0x2B48,0x2B46,INVALC,INVALC,
	INVALC,INVALC,0x2B47,INVALC,0x2F6F,INVALC,0x2F6C,0x2F6A,
	0x2F6D,0x2F6E,INVALC,INVALC,0x3452,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4045,0x4044,INVALC,INVALC,INVALC,INVALC,INVALC,0x4046,
	INVALC,INVALC,0x4621,INVALC,INVALC,0x457E,0x4B33,INVALC,
	INVALC,INVALC,0x4D2C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2155,INVALC,INVALC,
	/* Block 84, Array index 0x0600 */
	0x2156,INVALC,INVALC,0x2223,0x2222,INVALC,INVALC,0x2224,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5038,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2226,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6450,0x2465,INVALC,0x2B49,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4623,0x4B34,INVALC,0x5444,0x5C34,INVALC,
	0x2129,INVALC,0x2157,0x2466,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2159,INVALC,
	INVALC,INVALC,INVALC,0x232A,INVALC,INVALC,INVALC,0x232B,
	0x232D,INVALC,INVALC,INVALC,INVALC,0x2763,INVALC,INVALC,
	0x2B4A,INVALC,INVALC,INVALC,INVALC,0x215A,INVALC,0x2227,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6451,INVALC,0x2A5C,
	INVALC,0x212A,INVALC,INVALC,INVALC,0x2228,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2467,INVALC,0x2765,INVALC,
	INVALC,INVALC,0x2766,INVALC,INVALC,0x2B4B,INVALC,INVALC,
	0x4047,INVALC,INVALC,INVALC,INVALC,INVALC,0x215B,INVALC,
	INVALC,0x2229,INVALC,0x232F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2767,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6452,INVALC,INVALC,INVALC,INVALC,
	0x3454,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A28,INVALC,
	0x3A27,0x404A,0x4049,0x4048,INVALC,INVALC,0x4625,INVALC,
	0x4626,0x4B35,INVALC,0x5445,INVALC,INVALC,0x212B,0x215C,
	INVALC,INVALC,0x222A,INVALC,INVALC,INVALC,0x2331,INVALC,
	INVALC,0x2769,0x276A,INVALC,0x3455,0x3A29,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x215E,INVALC,0x2160,INVALC,
	0x222B,INVALC,0x2333,INVALC,INVALC,0x276B,INVALC,INVALC,
	INVALC,0x2B50,0x2B4F,INVALC,0x2B4E,0x2B4D,INVALC,INVALC,
	0x404B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2231,INVALC,0x222D,0x222E,
	INVALC,0x222F,INVALC,INVALC,INVALC,INVALC,INVALC,0x2335,
	/* Block 85, Array index 0x0700 */
	0x2337,INVALC,INVALC,INVALC,INVALC,0x2339,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x233A,INVALC,0x233B,0x2334,INVALC,
	INVALC,INVALC,0x2476,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2479,0x247E,0x2475,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2524,
	INVALC,INVALC,0x2477,INVALC,0x247B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2470,INVALC,INVALC,INVALC,INVALC,0x2521,
	INVALC,INVALC,INVALC,INVALC,0x2473,INVALC,INVALC,INVALC,
	INVALC,0x247C,INVALC,0x2478,0x2474,0x246D,INVALC,INVALC,
	0x247D,0x2522,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2523,INVALC,INVALC,INVALC,INVALC,0x276E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2774,0x2778,INVALC,INVALC,0x2775,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2771,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x276C,INVALC,INVALC,
	INVALC,0x2770,0x2779,INVALC,INVALC,INVALC,INVALC,0x2773,
	INVALC,INVALC,INVALC,0x276F,0x2777,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2B5C,0x2B5D,
	INVALC,INVALC,INVALC,0x2B57,0x2B59,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2B5B,INVALC,0x2B54,0x2B5E,INVALC,INVALC,
	INVALC,0x2B51,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2B55,0x6454,0x2B63,0x2B61,INVALC,INVALC,
	0x2B53,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2B58,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2F77,
	0x3021,INVALC,INVALC,INVALC,0x2F74,0x2F78,0x3022,INVALC,
	INVALC,0x2F79,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 86, Array index 0x0800 */
	0x2F7B,INVALC,0x2F7A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2F73,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2F76,INVALC,INVALC,INVALC,INVALC,
	0x2F7E,0x3025,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6721,0x345F,INVALC,0x3468,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3466,INVALC,INVALC,INVALC,INVALC,0x3467,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x345B,INVALC,INVALC,0x3458,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3462,0x3461,INVALC,INVALC,INVALC,
	0x3457,0x3A36,0x3464,INVALC,INVALC,0x345E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3459,INVALC,INVALC,0x6457,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6456,0x6722,0x6455,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3A3C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3A31,0x3A38,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A2E,0x3A2A,
	INVALC,INVALC,INVALC,0x3A33,INVALC,INVALC,0x3A40,INVALC,
	0x3A34,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3A42,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A3B,
	INVALC,INVALC,INVALC,INVALC,0x3A2F,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6723,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4051,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x404E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A2B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x404C,INVALC,INVALC,INVALC,0x6725,INVALC,
	0x6724,INVALC,INVALC,INVALC,0x6458,INVALC,INVALC,INVALC,
	0x462E,INVALC,INVALC,0x4628,INVALC,INVALC,INVALC,INVALC,
	/* Block 87, Array index 0x0900 */
	INVALC,INVALC,INVALC,0x4629,INVALC,0x4636,INVALC,0x462C,
	INVALC,INVALC,INVALC,0x462A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4634,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4633,INVALC,INVALC,INVALC,INVALC,INVALC,0x4631,INVALC,
	0x645A,INVALC,0x6459,INVALC,INVALC,INVALC,0x4650,INVALC,
	0x4630,INVALC,INVALC,0x4B41,INVALC,0x4B38,INVALC,INVALC,
	INVALC,0x4B3D,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B44,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4B3E,INVALC,INVALC,0x4B37,
	INVALC,INVALC,INVALC,0x4B3B,INVALC,0x4B43,INVALC,0x4B45,
	0x4B40,0x4635,0x4B3C,INVALC,0x4B39,INVALC,0x4B3F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x503F,
	INVALC,0x5040,INVALC,INVALC,INVALC,INVALC,INVALC,0x503B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x503D,INVALC,0x5448,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5449,0x5447,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x645D,0x645B,INVALC,0x575B,INVALC,0x575A,INVALC,
	INVALC,INVALC,INVALC,0x5758,INVALC,INVALC,0x5759,0x5757,
	0x575C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A2D,
	0x645E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6726,0x6727,0x645C,0x5E26,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6728,INVALC,INVALC,0x5F40,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6071,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2177,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x233E,INVALC,INVALC,0x252C,INVALC,
	INVALC,0x2526,INVALC,INVALC,0x252B,0x252A,INVALC,0x2529,
	INVALC,0x2548,0x2527,0x2528,INVALC,INVALC,0x2822,INVALC,
	0x277D,INVALC,INVALC,0x277C,0x2821,0x277B,INVALC,INVALC,
	/* Block 88, Array index 0x0A00 */
	0x2B65,INVALC,INVALC,INVALC,INVALC,0x3027,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A44,0x3A43,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4052,INVALC,0x4637,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F41,INVALC,INVALC,
	INVALC,0x2161,INVALC,INVALC,0x2234,INVALC,INVALC,0x6249,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2341,0x2342,INVALC,INVALC,0x233F,0x2340,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2531,0x253E,INVALC,0x2530,
	INVALC,INVALC,0x2540,0x2536,0x252F,INVALC,0x253C,INVALC,
	0x252D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2532,0x2541,0x2542,0x2533,0x2538,
	0x2537,0x645F,INVALC,INVALC,INVALC,INVALC,INVALC,0x2535,
	INVALC,INVALC,INVALC,0x2825,INVALC,0x2823,INVALC,0x2826,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2828,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2829,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x282A,
	INVALC,0x2824,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6460,INVALC,INVALC,0x2B69,0x2B68,INVALC,
	0x2B6C,0x2B67,0x2B6A,INVALC,INVALC,INVALC,0x2B71,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2B70,INVALC,INVALC,INVALC,
	INVALC,0x2B6B,INVALC,INVALC,INVALC,INVALC,0x2B6D,0x2B72,
	0x2B6E,INVALC,INVALC,INVALC,0x2B66,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x302A,
	INVALC,INVALC,INVALC,0x3030,INVALC,INVALC,0x302F,INVALC,
	0x3038,INVALC,INVALC,INVALC,0x3034,0x3035,INVALC,INVALC,
	0x3033,0x3028,0x3037,INVALC,INVALC,0x302B,INVALC,INVALC,
	INVALC,0x3036,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3032,INVALC,0x346E,0x3471,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3472,INVALC,
	0x3474,INVALC,0x3469,INVALC,INVALC,INVALC,INVALC,0x346B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x346F,0x3475,
	/* Block 89, Array index 0x0B00 */
	INVALC,INVALC,INVALC,0x3476,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6461,
	INVALC,INVALC,0x6462,0x6463,INVALC,INVALC,INVALC,INVALC,
	0x3A4A,INVALC,0x3A45,INVALC,INVALC,INVALC,INVALC,0x3A47,
	INVALC,INVALC,0x3A50,INVALC,INVALC,INVALC,0x3A4C,INVALC,
	INVALC,INVALC,INVALC,0x6336,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3A4E,INVALC,0x3A49,INVALC,0x3A4B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6337,INVALC,0x6464,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4057,INVALC,INVALC,INVALC,INVALC,INVALC,0x405B,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4056,INVALC,INVALC,0x405E,
	0x405C,0x4058,INVALC,INVALC,INVALC,INVALC,0x405D,0x6343,
	INVALC,0x405A,0x4059,INVALC,0x4054,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4643,0x463E,INVALC,INVALC,INVALC,0x4642,
	0x463C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x463D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x463A,0x4645,INVALC,INVALC,
	INVALC,INVALC,0x4638,INVALC,INVALC,0x4641,0x4644,0x4640,
	INVALC,INVALC,INVALC,0x6465,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4B46,INVALC,0x4B50,INVALC,INVALC,0x4B51,
	INVALC,INVALC,0x4B4B,INVALC,INVALC,0x463F,INVALC,INVALC,
	0x4B4D,INVALC,0x4B49,INVALC,0x4B4F,0x4B4C,INVALC,0x4B47,
	INVALC,INVALC,INVALC,0x504A,INVALC,INVALC,INVALC,INVALC,
	0x504B,INVALC,INVALC,0x5048,0x504C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5049,INVALC,INVALC,0x544B,INVALC,INVALC,
	0x544C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A2F,INVALC,INVALC,INVALC,
	0x5A2E,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E28,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2343,INVALC,
	0x2545,0x2543,0x2544,0x282D,INVALC,0x2B77,INVALC,0x3477,
	INVALC,INVALC,INVALC,0x3A52,INVALC,INVALC,INVALC,INVALC,
	/* Block 90, Array index 0x0C00 */
	0x6466,0x504D,0x2139,INVALC,0x2235,0x2344,INVALC,INVALC,
	0x2B7A,0x2B79,0x213A,0x2546,INVALC,INVALC,INVALC,INVALC,
	0x4646,0x6560,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2236,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3479,
	INVALC,INVALC,INVALC,0x405F,INVALC,INVALC,0x4B53,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2238,INVALC,0x2237,INVALC,0x2239,INVALC,0x2347,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2549,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x282E,0x282F,0x6467,INVALC,INVALC,
	INVALC,INVALC,0x2B7C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x347B,INVALC,0x347C,0x347D,INVALC,
	INVALC,INVALC,INVALC,0x3A53,0x3A54,INVALC,0x4060,INVALC,
	0x4061,INVALC,INVALC,INVALC,0x4648,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x223B,INVALC,INVALC,
	INVALC,INVALC,0x234B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x234D,INVALC,
	INVALC,0x2555,INVALC,INVALC,0x254C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x254B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2556,0x2553,0x254E,INVALC,INVALC,0x2551,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2841,0x2833,INVALC,INVALC,
	0x2838,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2845,
	0x2840,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x283D,
	INVALC,INVALC,0x2837,INVALC,0x2835,INVALC,INVALC,INVALC,
	INVALC,0x283F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2C26,INVALC,0x2846,
	INVALC,0x2C2D,INVALC,INVALC,INVALC,INVALC,INVALC,0x2C21,
	INVALC,INVALC,0x2C2C,INVALC,INVALC,INVALC,INVALC,0x2C2E,
	INVALC,INVALC,INVALC,0x2C23,INVALC,INVALC,INVALC,0x2C24,
	0x2C25,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2C2B,0x2C29,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 91, Array index 0x0D00 */
	INVALC,INVALC,0x2C2A,INVALC,0x2C28,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3040,INVALC,0x2C27,INVALC,INVALC,
	0x6270,INVALC,0x304F,INVALC,0x304A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x304E,INVALC,INVALC,0x303E,INVALC,INVALC,
	INVALC,0x303B,0x3045,INVALC,0x3048,INVALC,0x3052,INVALC,
	INVALC,INVALC,0x303C,0x3522,0x3538,INVALC,INVALC,0x304D,
	INVALC,0x304B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3528,0x3524,INVALC,0x3531,INVALC,0x3533,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x352E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x352A,INVALC,INVALC,0x352F,0x353B,
	INVALC,INVALC,INVALC,INVALC,0x353A,INVALC,INVALC,INVALC,
	INVALC,0x3539,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3527,INVALC,0x353C,INVALC,INVALC,INVALC,INVALC,
	0x3530,INVALC,INVALC,0x352B,INVALC,INVALC,0x3529,0x3534,
	INVALC,0x3532,INVALC,0x3537,INVALC,0x3535,INVALC,INVALC,
	INVALC,0x3A75,INVALC,INVALC,INVALC,INVALC,0x3A7C,INVALC,
	0x3A63,0x3A61,0x3A58,INVALC,INVALC,0x3A5B,0x3A77,0x3A72,
	0x3A59,0x3A60,INVALC,INVALC,INVALC,0x3A7B,INVALC,INVALC,
	INVALC,0x3A66,INVALC,INVALC,INVALC,INVALC,0x3A62,INVALC,
	0x3A5C,0x3A5E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A73,0x3A5A,INVALC,0x3A57,0x3A7A,INVALC,INVALC,INVALC,
	0x3A56,INVALC,0x3A79,0x3A78,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4067,INVALC,0x4066,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4064,0x3A6B,
	0x406A,0x6468,INVALC,0x6469,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x464C,INVALC,INVALC,0x464A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x464F,
	0x4653,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4B5C,INVALC,0x4B58,INVALC,0x4B54,INVALC,
	/* Block 92, Array index 0x0E00 */
	0x4B5E,INVALC,INVALC,INVALC,0x4B56,INVALC,0x4B5A,INVALC,
	INVALC,INVALC,0x4B5B,INVALC,INVALC,0x4B5D,0x4B5F,INVALC,
	INVALC,0x5050,0x5053,INVALC,INVALC,0x5051,INVALC,INVALC,
	0x504F,INVALC,INVALC,INVALC,0x5054,INVALC,INVALC,0x504E,
	INVALC,INVALC,0x5056,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5450,INVALC,0x544E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5452,INVALC,0x544F,INVALC,0x5451,INVALC,0x5453,
	INVALC,0x5A31,0x5761,0x575E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C36,INVALC,0x5C39,INVALC,0x5C3A,INVALC,
	INVALC,0x5E29,0x5F42,INVALC,INVALC,INVALC,INVALC,0x6073,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2848,INVALC,INVALC,INVALC,INVALC,INVALC,0x625C,0x646A,
	INVALC,INVALC,0x2C2F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4071,INVALC,0x4072,INVALC,
	INVALC,0x5057,INVALC,INVALC,0x5A34,INVALC,0x5C3B,INVALC,
	0x213B,INVALC,0x223D,INVALC,INVALC,INVALC,0x2350,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x255A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x284C,INVALC,0x284A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x284E,0x284B,0x284D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2C32,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3057,
	INVALC,INVALC,INVALC,0x3056,0x3055,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3541,INVALC,INVALC,INVALC,INVALC,
	0x353F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3A7E,INVALC,0x4074,
	INVALC,INVALC,0x4075,0x4076,0x4073,0x6345,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5A35,INVALC,INVALC,INVALC,0x5E2A,
	INVALC,0x223E,INVALC,INVALC,INVALC,0x255E,0x255C,0x255D,
	/* Block 93, Array index 0x0F00 */
	0x2850,INVALC,INVALC,INVALC,INVALC,0x3058,0x3059,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x624A,0x223F,INVALC,INVALC,INVALC,
	0x2352,0x2851,INVALC,0x2C34,INVALC,INVALC,0x3B24,INVALC,
	0x4078,0x6525,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x255F,INVALC,INVALC,INVALC,0x2C35,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5454,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2354,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2854,0x2855,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x305A,0x305B,INVALC,INVALC,INVALC,0x646B,
	INVALC,INVALC,INVALC,0x3543,INVALC,INVALC,0x3B26,0x3B25,
	INVALC,0x3544,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5A38,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x646C,
	0x2355,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x235A,INVALC,INVALC,INVALC,0x2561,0x2566,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2562,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2563,INVALC,0x285D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x285E,INVALC,INVALC,INVALC,
	INVALC,0x285C,0x285F,0x2857,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2858,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2C3F,INVALC,INVALC,
	INVALC,0x2C3B,INVALC,INVALC,INVALC,0x646D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2C3C,0x2C42,INVALC,INVALC,
	INVALC,0x2C3E,0x2C41,INVALC,INVALC,0x2C43,0x2C3D,INVALC,
	INVALC,0x3063,INVALC,0x305F,INVALC,INVALC,INVALC,0x3064,
	INVALC,INVALC,INVALC,0x3061,INVALC,0x3062,INVALC,INVALC,
	INVALC,INVALC,0x305E,INVALC,0x6271,INVALC,0x6272,INVALC,
	/* Block 94, Array index 0x1000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x354B,0x3548,0x3547,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3551,INVALC,INVALC,0x3552,INVALC,0x354F,INVALC,INVALC,
	0x3555,INVALC,INVALC,INVALC,0x3B2B,INVALC,INVALC,INVALC,
	INVALC,0x3546,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x354E,INVALC,0x354A,INVALC,INVALC,0x3554,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3B36,INVALC,INVALC,0x3B28,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3B27,INVALC,0x3B37,INVALC,
	0x3B2A,INVALC,INVALC,INVALC,INVALC,0x3B2E,INVALC,0x3B34,
	INVALC,INVALC,INVALC,0x3B33,INVALC,INVALC,INVALC,0x3B31,
	INVALC,INVALC,INVALC,0x3B2D,INVALC,INVALC,INVALC,INVALC,
	0x4121,INVALC,INVALC,INVALC,0x407C,INVALC,0x407A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4079,0x407D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x646E,INVALC,INVALC,
	0x465C,INVALC,INVALC,0x4654,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4656,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4659,0x465A,INVALC,0x646F,0x4B65,
	0x4B66,0x4B63,INVALC,INVALC,INVALC,INVALC,0x4B6B,INVALC,
	0x4B67,INVALC,INVALC,0x4B69,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4B6A,0x4B68,0x6470,0x5058,INVALC,
	INVALC,INVALC,INVALC,0x505B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x505A,INVALC,INVALC,0x6471,INVALC,
	INVALC,0x5457,INVALC,0x5459,INVALC,INVALC,0x5455,0x6472,
	INVALC,0x5762,INVALC,INVALC,0x5A39,INVALC,INVALC,INVALC,
	0x5C3D,INVALC,0x5C3C,INVALC,0x5C3E,INVALC,INVALC,INVALC,
	0x5E2D,INVALC,INVALC,0x5F43,INVALC,INVALC,INVALC,0x6030,
	INVALC,INVALC,INVALC,0x213C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3557,0x4B6C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2567,0x2860,INVALC,
	INVALC,INVALC,0x2C45,0x2C44,INVALC,INVALC,INVALC,INVALC,
	/* Block 95, Array index 0x1100 */
	0x2163,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x235C,0x2568,INVALC,0x256B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2862,0x2861,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2C46,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3066,INVALC,0x3069,INVALC,0x3068,0x306A,
	0x3067,INVALC,0x355B,INVALC,INVALC,0x3559,INVALC,INVALC,
	INVALC,0x3558,0x355A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3979,INVALC,INVALC,INVALC,0x3B3A,0x3B3B,
	0x3B3D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x465D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4B6D,INVALC,INVALC,INVALC,0x4B70,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6473,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6677,INVALC,INVALC,INVALC,INVALC,INVALC,0x2863,
	INVALC,INVALC,0x213D,INVALC,INVALC,INVALC,INVALC,0x213E,
	INVALC,0x2247,INVALC,0x2248,INVALC,0x6678,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x256E,INVALC,0x256D,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2865,0x2867,INVALC,INVALC,INVALC,0x2868,INVALC,INVALC,
	INVALC,0x2C48,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x306C,INVALC,INVALC,INVALC,INVALC,INVALC,0x306B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3561,0x3560,INVALC,0x3B3F,INVALC,0x3B40,
	0x3B3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6474,INVALC,0x4661,
	0x4660,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x505C,INVALC,INVALC,0x6475,INVALC,INVALC,
	0x5C3F,INVALC,INVALC,INVALC,0x213F,0x2361,INVALC,INVALC,
	0x2869,0x286B,INVALC,0x2C4C,0x2C4B,0x306E,INVALC,INVALC,
	/* Block 96, Array index 0x1200 */
	INVALC,INVALC,INVALC,0x256F,INVALC,INVALC,0x286D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2164,0x2249,0x2362,INVALC,
	INVALC,0x3B42,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2571,INVALC,
	INVALC,0x286E,INVALC,INVALC,INVALC,0x2870,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2C4D,
	INVALC,INVALC,0x306F,INVALC,0x3562,INVALC,INVALC,INVALC,
	INVALC,0x3563,0x3B47,0x3B45,INVALC,INVALC,0x3B44,INVALC,
	INVALC,INVALC,0x4122,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5763,INVALC,INVALC,
	0x2140,0x2141,0x224A,0x2351,INVALC,0x2871,INVALC,INVALC,
	INVALC,INVALC,0x4123,0x505E,INVALC,0x5764,INVALC,INVALC,
	0x6164,0x2142,INVALC,0x2572,INVALC,INVALC,0x2C50,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4124,INVALC,
	INVALC,INVALC,0x5F45,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2876,0x2875,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6264,0x3070,0x3072,
	INVALC,INVALC,INVALC,0x3071,0x6476,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6477,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B4B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4663,0x4664,INVALC,INVALC,INVALC,
	0x4B72,INVALC,0x6478,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5E2F,INVALC,INVALC,INVALC,INVALC,0x224D,
	0x6251,INVALC,0x224C,INVALC,INVALC,INVALC,0x2578,INVALC,
	INVALC,INVALC,INVALC,0x2366,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2367,0x2369,0x2577,INVALC,INVALC,0x257B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2576,0x2574,
	INVALC,0x287E,INVALC,INVALC,0x257C,INVALC,INVALC,INVALC,
	0x2622,INVALC,0x2621,INVALC,INVALC,INVALC,0x257D,INVALC,
	INVALC,0x2623,INVALC,INVALC,0x257A,INVALC,0x2575,INVALC,
	/* Block 97, Array index 0x1300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2878,
	0x2922,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6266,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2C5A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2877,0x2C5B,INVALC,INVALC,INVALC,INVALC,0x287A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2921,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6267,INVALC,0x2C60,0x3079,INVALC,INVALC,0x2C56,0x3073,
	INVALC,0x2C57,0x2C5D,INVALC,INVALC,INVALC,0x307E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2C54,INVALC,INVALC,INVALC,
	0x2C5F,0x2C61,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x307A,0x3078,0x307B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2C5C,INVALC,0x3076,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3122,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3125,INVALC,INVALC,INVALC,INVALC,
	0x3568,0x3074,INVALC,INVALC,INVALC,INVALC,0x3127,INVALC,
	INVALC,0x3571,INVALC,INVALC,0x3572,INVALC,0x3123,0x3128,
	INVALC,0x3121,0x356F,INVALC,INVALC,INVALC,0x3126,INVALC,
	INVALC,INVALC,INVALC,0x3B54,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x356A,INVALC,0x3B56,INVALC,INVALC,INVALC,0x3B55,0x356E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3579,INVALC,
	INVALC,INVALC,INVALC,0x3B58,INVALC,0x3B59,INVALC,0x3576,
	INVALC,0x3B5A,0x3B4D,INVALC,INVALC,INVALC,0x3574,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3B5E,INVALC,INVALC,
	/* Block 98, Array index 0x1400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B4F,
	INVALC,INVALC,INVALC,INVALC,0x3B51,INVALC,INVALC,INVALC,
	INVALC,0x3B5D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x412B,INVALC,INVALC,INVALC,INVALC,0x3B61,INVALC,
	0x3B62,0x3B63,0x3B50,INVALC,INVALC,0x6479,INVALC,INVALC,
	INVALC,INVALC,0x4129,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4127,0x412C,INVALC,0x466F,INVALC,INVALC,INVALC,INVALC,
	0x466E,INVALC,0x4128,INVALC,0x466C,0x412E,INVALC,INVALC,
	INVALC,INVALC,0x466D,0x412A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x647A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B73,
	INVALC,0x4B7E,INVALC,INVALC,0x4B74,INVALC,INVALC,INVALC,
	0x4671,INVALC,INVALC,INVALC,0x4B7C,INVALC,INVALC,INVALC,
	INVALC,0x4672,INVALC,INVALC,INVALC,INVALC,INVALC,0x466A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C21,INVALC,INVALC,0x4668,INVALC,0x4673,INVALC,INVALC,
	INVALC,0x466B,INVALC,INVALC,INVALC,0x4C23,0x4674,0x4C24,
	0x4B77,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B7A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B76,INVALC,0x5068,
	0x4C28,0x5067,INVALC,INVALC,0x4C29,INVALC,0x4C25,INVALC,
	INVALC,INVALC,INVALC,0x4B79,INVALC,INVALC,0x4B75,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5061,
	INVALC,0x5063,INVALC,0x506A,INVALC,0x5064,INVALC,INVALC,
	0x5066,INVALC,INVALC,INVALC,0x506C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5062,0x5065,
	0x506B,INVALC,INVALC,0x6679,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5461,0x545F,0x545C,0x545E,0x545D,INVALC,INVALC,
	INVALC,INVALC,0x545B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5765,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 99, Array index 0x1500 */
	INVALC,INVALC,0x5F46,INVALC,INVALC,0x613E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x236B,
	INVALC,INVALC,INVALC,0x2625,INVALC,INVALC,INVALC,INVALC,
	0x2C63,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B64,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4676,INVALC,INVALC,INVALC,0x4675,INVALC,INVALC,0x4C2C,
	INVALC,0x506D,INVALC,INVALC,INVALC,0x5F48,INVALC,0x2165,
	0x2166,0x224E,INVALC,0x2628,0x2629,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3129,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2250,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x236F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x236C,INVALC,0x236D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x236E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2639,INVALC,INVALC,
	0x262E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x262A,INVALC,INVALC,0x2637,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x262F,INVALC,0x2636,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2635,INVALC,0x2633,0x2634,0x2638,0x2631,INVALC,
	INVALC,INVALC,INVALC,0x647B,INVALC,0x2632,0x2931,0x292A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2926,INVALC,INVALC,INVALC,INVALC,0x2928,
	INVALC,INVALC,0x2925,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2934,0x2932,INVALC,INVALC,INVALC,0x2933,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x292E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2930,0x2937,0x2935,
	0x2936,0x292F,INVALC,INVALC,0x2929,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2C6D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 100, Array index 0x1600 */
	INVALC,INVALC,INVALC,INVALC,0x2C6F,0x2C70,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2C6C,
	INVALC,0x3132,INVALC,0x3133,INVALC,INVALC,INVALC,0x2C6B,
	INVALC,0x2C6E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x312C,INVALC,
	0x312F,0x3135,INVALC,INVALC,INVALC,0x3138,INVALC,0x3137,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x313C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x312E,0x3139,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3130,0x313B,0x3136,INVALC,INVALC,
	0x647C,INVALC,INVALC,INVALC,0x647D,INVALC,0x3631,INVALC,
	INVALC,INVALC,0x3627,INVALC,0x3629,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3626,0x3632,INVALC,INVALC,INVALC,
	INVALC,0x3622,INVALC,INVALC,INVALC,INVALC,0x357C,0x362B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3623,INVALC,
	INVALC,INVALC,INVALC,0x357D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3624,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x362D,INVALC,INVALC,INVALC,0x647E,INVALC,
	INVALC,INVALC,0x362E,INVALC,INVALC,INVALC,INVALC,0x3B71,
	INVALC,0x3B70,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3B73,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B67,INVALC,
	INVALC,INVALC,0x3B66,INVALC,INVALC,INVALC,0x3B68,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B72,
	0x3B6D,INVALC,0x6339,INVALC,INVALC,INVALC,0x3B6F,INVALC,
	/* Block 101, Array index 0x1700 */
	INVALC,INVALC,INVALC,0x6521,INVALC,INVALC,INVALC,0x4149,
	0x4132,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x413A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4138,INVALC,INVALC,INVALC,0x413E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x413D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4136,0x4133,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4134,INVALC,0x4148,0x4725,0x4142,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4144,0x4145,
	INVALC,0x6676,INVALC,INVALC,0x4678,INVALC,INVALC,0x4728,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x467E,0x4724,0x467A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4721,INVALC,0x472A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4723,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4722,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x467B,INVALC,0x4729,INVALC,
	0x4C2D,0x4726,INVALC,0x4C35,INVALC,INVALC,0x4C38,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4C30,0x4C2F,INVALC,
	INVALC,0x4C3B,INVALC,INVALC,0x4C31,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4C2E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C3E,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C3F,
	0x4C34,INVALC,0x4C3D,INVALC,INVALC,INVALC,INVALC,0x4C3A,
	INVALC,INVALC,INVALC,INVALC,0x4C39,INVALC,0x6522,INVALC,
	INVALC,0x4C37,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5070,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C32,INVALC,
	0x5077,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5076,INVALC,0x506F,INVALC,0x5079,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5075,INVALC,INVALC,0x5464,
	INVALC,0x5467,INVALC,INVALC,INVALC,0x576B,INVALC,0x576D,
	INVALC,INVALC,0x576C,INVALC,INVALC,INVALC,0x5469,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x576A,INVALC,
	INVALC,0x5770,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 102, Array index 0x1800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x576E,INVALC,INVALC,
	0x5A3B,INVALC,0x5A3C,0x5A3A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5E31,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6523,INVALC,INVALC,INVALC,0x5F4A,0x5F4B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F49,
	0x6032,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2C72,INVALC,INVALC,0x2167,0x2168,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2C74,INVALC,0x2C76,INVALC,0x2C73,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x313D,INVALC,0x3637,0x3636,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3634,INVALC,INVALC,INVALC,INVALC,0x3B74,
	0x3B75,0x3B76,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x414B,INVALC,0x414E,0x6346,INVALC,
	0x414D,0x4730,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5C41,INVALC,INVALC,
	0x2558,0x2939,INVALC,0x3140,INVALC,0x3638,0x3639,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3142,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x293B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x363A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x546B,INVALC,INVALC,
	INVALC,INVALC,0x293C,INVALC,INVALC,INVALC,0x2C77,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x363C,
	INVALC,0x363F,0x363D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3B79,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A3F,INVALC,INVALC,INVALC,
	0x2169,INVALC,INVALC,0x3641,0x4150,INVALC,INVALC,0x2251,
	INVALC,INVALC,0x2373,0x2372,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x263D,INVALC,
	INVALC,0x625E,INVALC,INVALC,INVALC,INVALC,0x2943,0x293E,
	/* Block 103, Array index 0x1900 */
	INVALC,0x2942,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2940,
	0x2941,INVALC,0x2C7E,0x2C7A,INVALC,INVALC,0x2D23,INVALC,
	INVALC,INVALC,INVALC,0x2C7C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2C79,INVALC,INVALC,0x2C7D,INVALC,INVALC,INVALC,
	0x2D21,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2D24,
	0x2D25,INVALC,INVALC,0x2D22,0x2C37,INVALC,INVALC,INVALC,
	0x3146,INVALC,INVALC,INVALC,0x314C,INVALC,0x3144,INVALC,
	0x3149,INVALC,INVALC,0x3148,INVALC,0x314D,0x3145,INVALC,
	0x3143,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3642,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x314B,INVALC,INVALC,0x3644,INVALC,INVALC,INVALC,0x3645,
	INVALC,0x3C26,INVALC,0x3B7D,INVALC,0x3C25,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3C23,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3B7C,INVALC,0x3B7E,
	INVALC,0x3C24,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x415B,0x4156,
	INVALC,INVALC,0x4159,0x4151,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4734,INVALC,0x4735,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4739,0x4733,0x4737,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4C44,0x4C46,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4C41,INVALC,INVALC,0x4C40,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4C43,INVALC,INVALC,0x507D,
	INVALC,0x5123,INVALC,0x507A,INVALC,0x507E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5122,0x5471,INVALC,
	0x546F,0x5470,INVALC,INVALC,0x546C,0x546D,INVALC,0x5472,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A43,
	INVALC,0x5A42,0x6526,INVALC,INVALC,0x5C42,INVALC,0x5C43,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x613F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2641,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 104, Array index 0x1A00 */
	INVALC,INVALC,0x6527,INVALC,INVALC,INVALC,0x5124,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2945,INVALC,0x2D27,INVALC,
	INVALC,0x6632,INVALC,INVALC,INVALC,INVALC,0x3648,INVALC,
	INVALC,0x3647,0x3646,INVALC,INVALC,INVALC,0x3C29,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5126,0x6633,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2252,
	INVALC,INVALC,0x2377,INVALC,INVALC,INVALC,0x2378,0x2375,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2648,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2649,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2645,INVALC,0x2644,INVALC,INVALC,INVALC,
	0x2647,INVALC,INVALC,0x2642,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x264A,INVALC,INVALC,INVALC,0x6257,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2947,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2952,INVALC,0x2949,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x625F,INVALC,0x294A,INVALC,INVALC,INVALC,INVALC,0x294F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2948,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x294E,0x294C,0x6528,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6529,INVALC,INVALC,INVALC,
	INVALC,0x2D36,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2D2E,INVALC,INVALC,0x2D30,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2D2D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2D31,0x2D2B,0x2D3A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2D2A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2951,INVALC,0x2D34,INVALC,INVALC,0x2D32,INVALC,
	/* Block 105, Array index 0x1B00 */
	0x2D39,0x2D37,INVALC,INVALC,INVALC,0x2D38,0x652B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x652A,INVALC,INVALC,INVALC,INVALC,0x315D,INVALC,INVALC,
	INVALC,0x3157,INVALC,0x3153,INVALC,INVALC,0x315B,INVALC,
	INVALC,INVALC,0x315E,0x3164,0x3150,INVALC,INVALC,0x315F,
	INVALC,INVALC,INVALC,INVALC,0x3156,INVALC,INVALC,INVALC,
	0x3163,INVALC,INVALC,INVALC,INVALC,INVALC,0x3154,0x314F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x315A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3159,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2D2F,INVALC,INVALC,0x3162,0x3158,INVALC,
	0x3161,0x3160,0x652D,INVALC,INVALC,INVALC,0x652C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3655,INVALC,INVALC,INVALC,
	0x3660,INVALC,INVALC,0x364E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x365B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3657,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3651,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x365A,INVALC,INVALC,
	0x364D,0x365C,0x3650,INVALC,INVALC,INVALC,0x364B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3654,0x3478,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x364C,0x3652,
	0x365E,0x365F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3656,INVALC,0x3C45,INVALC,0x3C41,INVALC,INVALC,
	INVALC,INVALC,0x3C38,INVALC,INVALC,INVALC,INVALC,0x3C3E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3653,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3C3D,INVALC,INVALC,0x3C32,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C2A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C31,0x3C42,
	/* Block 106, Array index 0x1C00 */
	0x3C39,0x3C3B,0x3C34,0x3C2F,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3C37,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C3C,INVALC,INVALC,INVALC,0x3C44,INVALC,0x3C33,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x652F,INVALC,INVALC,0x652E,INVALC,0x3C40,INVALC,
	INVALC,0x4165,INVALC,INVALC,INVALC,INVALC,0x4177,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4168,INVALC,
	INVALC,INVALC,INVALC,0x416B,INVALC,INVALC,0x415D,0x4175,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4173,INVALC,INVALC,
	0x4174,INVALC,INVALC,INVALC,INVALC,0x416F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4179,INVALC,INVALC,0x4178,INVALC,INVALC,0x4176,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x416D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x416A,0x417B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4172,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x473A,
	INVALC,INVALC,0x473D,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4751,INVALC,INVALC,INVALC,INVALC,0x474E,INVALC,0x4746,
	INVALC,INVALC,0x4747,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x473E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6352,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4743,INVALC,INVALC,INVALC,INVALC,0x473B,INVALC,INVALC,
	0x473C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4745,0x4748,INVALC,INVALC,0x474D,0x4744,INVALC,
	0x4C54,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x474C,INVALC,0x4742,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C64,0x4C60,INVALC,INVALC,INVALC,INVALC,0x4C4F,
	0x4C51,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C5F,0x4C57,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 107, Array index 0x1D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4C4D,0x4C50,INVALC,0x4C47,INVALC,
	0x4C5B,INVALC,0x4C56,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C4A,INVALC,0x4C49,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C52,INVALC,0x4C62,INVALC,INVALC,INVALC,
	INVALC,0x4C5E,0x4C59,INVALC,0x4C61,0x6530,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x512C,INVALC,INVALC,0x513C,0x5138,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5128,0x512B,INVALC,0x5140,INVALC,INVALC,
	INVALC,INVALC,0x513B,INVALC,0x5131,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5129,0x513E,INVALC,INVALC,INVALC,0x512F,
	INVALC,INVALC,INVALC,INVALC,0x5144,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5127,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x512E,INVALC,0x5132,INVALC,
	0x5141,0x513A,INVALC,INVALC,INVALC,INVALC,INVALC,0x6531,
	INVALC,INVALC,0x6532,INVALC,INVALC,0x5475,INVALC,INVALC,
	INVALC,INVALC,0x547B,INVALC,INVALC,INVALC,0x5521,INVALC,
	0x547C,INVALC,0x5476,INVALC,INVALC,INVALC,INVALC,0x5527,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5523,0x547A,INVALC,INVALC,INVALC,0x5524,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5478,
	INVALC,INVALC,0x5529,0x5774,INVALC,INVALC,INVALC,INVALC,
	0x5777,0x5821,0x5776,INVALC,INVALC,0x5778,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x577E,INVALC,0x5779,0x5823,
	0x577D,0x577A,INVALC,INVALC,0x5822,INVALC,INVALC,INVALC,
	0x577B,INVALC,0x6533,INVALC,INVALC,INVALC,0x5A44,INVALC,
	INVALC,INVALC,0x5A4F,INVALC,0x5A45,0x5A4E,INVALC,0x5A47,
	0x5A4D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A48,0x5A46,INVALC,INVALC,0x6534,INVALC,
	INVALC,0x5C49,INVALC,INVALC,INVALC,0x5775,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5C47,0x6535,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E35,0x5E34,0x5E33,
	/* Block 108, Array index 0x1E00 */
	INVALC,0x5E37,INVALC,INVALC,INVALC,INVALC,0x5F4F,0x5F4E,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6536,0x6537,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6074,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6140,0x6165,0x6141,INVALC,INVALC,
	INVALC,INVALC,0x2379,INVALC,INVALC,INVALC,0x2953,0x2954,
	INVALC,0x6269,0x2D3C,0x3166,INVALC,INVALC,0x3165,INVALC,
	0x3167,INVALC,INVALC,INVALC,INVALC,0x3662,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x417D,INVALC,INVALC,INVALC,0x417C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C66,
	INVALC,INVALC,0x4C65,0x4C67,INVALC,INVALC,INVALC,0x5146,
	0x5145,INVALC,INVALC,INVALC,INVALC,0x552B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2955,0x2956,INVALC,0x2D3D,0x3168,INVALC,0x3C49,0x3C47,
	INVALC,0x417E,INVALC,0x4221,0x4753,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2253,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2958,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x316A,INVALC,INVALC,INVALC,INVALC,
	0x3664,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4222,0x6354,INVALC,INVALC,
	INVALC,0x4754,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2959,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3665,INVALC,INVALC,0x3C4B,INVALC,
	INVALC,0x4226,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x237A,INVALC,
	INVALC,0x295A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6538,0x6539,INVALC,INVALC,
	INVALC,0x2D42,INVALC,INVALC,INVALC,0x316E,INVALC,INVALC,
	INVALC,0x316C,0x316D,INVALC,INVALC,0x3666,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C4D,INVALC,0x653A,INVALC,
	INVALC,INVALC,0x4228,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 109, Array index 0x1F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x552D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x295D,INVALC,0x653B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C4E,INVALC,INVALC,
	INVALC,0x4229,0x422A,INVALC,INVALC,INVALC,INVALC,0x2257,
	INVALC,0x2254,INVALC,INVALC,0x2421,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2258,0x2259,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x237D,INVALC,0x2425,INVALC,INVALC,0x2655,INVALC,
	0x237C,INVALC,0x2423,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x653C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2964,INVALC,0x2650,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2653,INVALC,0x237E,
	INVALC,0x2657,INVALC,INVALC,0x2651,INVALC,INVALC,0x264C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x264D,INVALC,INVALC,INVALC,INVALC,INVALC,0x2963,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x264E,0x2652,
	0x265C,0x2659,0x2656,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x265B,INVALC,INVALC,INVALC,INVALC,0x2D4E,
	INVALC,INVALC,0x296B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x296D,INVALC,INVALC,0x6261,INVALC,INVALC,0x296C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x296A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2965,
	INVALC,INVALC,INVALC,INVALC,0x2969,INVALC,0x6260,INVALC,
	INVALC,INVALC,0x2967,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2D4F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2D46,
	/* Block 110, Array index 0x2000 */
	INVALC,INVALC,0x2D50,INVALC,INVALC,0x2D48,0x2D4A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2D49,0x2D47,0x2D4D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x317A,INVALC,INVALC,INVALC,
	INVALC,0x2D4B,INVALC,0x2D45,0x2D57,INVALC,0x2D53,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2D55,0x2D51,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3177,0x3173,
	INVALC,INVALC,INVALC,0x3179,0x3226,0x3176,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x316F,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3222,0x3225,0x3172,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3221,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3178,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3227,INVALC,
	INVALC,0x317D,0x317B,0x317C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x653D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x366F,INVALC,INVALC,
	0x367A,0x3721,INVALC,INVALC,INVALC,INVALC,0x3C50,INVALC,
	INVALC,0x3672,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x366A,INVALC,0x366B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x366C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3674,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x367C,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C5D,0x3728,
	/* Block 111, Array index 0x2100 */
	INVALC,0x3726,0x3723,INVALC,0x3670,INVALC,0x3677,0x3724,
	0x3676,0x3725,0x3679,INVALC,0x367D,INVALC,INVALC,0x653E,
	INVALC,INVALC,0x3729,INVALC,INVALC,INVALC,INVALC,0x3727,
	0x3C54,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C52,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3C6B,0x3C6D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3C5A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C56,INVALC,INVALC,INVALC,0x3C6E,INVALC,INVALC,0x3C58,
	0x3C69,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C66,
	INVALC,0x3C64,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3C5B,INVALC,0x423A,0x3C60,INVALC,INVALC,
	0x3C5C,INVALC,INVALC,INVALC,INVALC,0x3C51,0x3C6C,INVALC,
	INVALC,INVALC,INVALC,0x3C62,0x6542,INVALC,0x3C63,INVALC,
	0x3C68,INVALC,INVALC,INVALC,0x6540,INVALC,0x633D,INVALC,
	INVALC,INVALC,0x653F,0x6541,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4230,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4233,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x422E,0x4236,INVALC,INVALC,0x4240,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4239,INVALC,INVALC,
	0x422D,INVALC,INVALC,0x4231,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4244,0x423E,0x4247,INVALC,0x423D,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4245,INVALC,INVALC,0x423F,INVALC,
	0x423B,0x4246,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4756,INVALC,INVALC,0x4757,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4765,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 112, Array index 0x2200 */
	INVALC,INVALC,INVALC,INVALC,0x4763,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4C78,0x475B,INVALC,INVALC,INVALC,
	0x4C6A,0x4764,INVALC,INVALC,INVALC,INVALC,INVALC,0x475F,
	INVALC,INVALC,INVALC,0x475A,INVALC,0x475C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x475E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x475D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4C74,INVALC,INVALC,
	INVALC,INVALC,0x4762,INVALC,0x4761,INVALC,INVALC,INVALC,
	0x6544,0x6545,0x4766,INVALC,0x6543,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C7D,INVALC,INVALC,0x4C73,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4C6B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C75,INVALC,INVALC,0x4C7E,0x4C6C,INVALC,INVALC,
	INVALC,0x4C79,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C7A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6363,INVALC,INVALC,INVALC,INVALC,0x4C7B,
	INVALC,0x4C77,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6547,INVALC,INVALC,INVALC,0x6546,INVALC,0x5156,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5154,INVALC,0x5158,
	INVALC,INVALC,INVALC,0x514D,INVALC,INVALC,0x5150,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C71,INVALC,INVALC,0x5155,INVALC,INVALC,0x5530,0x552F,
	INVALC,0x5535,0x5536,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5531,INVALC,INVALC,
	INVALC,INVALC,0x5537,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5A55,INVALC,0x5159,0x5539,INVALC,
	0x6548,INVALC,INVALC,INVALC,INVALC,0x5828,INVALC,INVALC,
	/* Block 113, Array index 0x2300 */
	INVALC,INVALC,0x582A,0x5827,INVALC,INVALC,INVALC,INVALC,
	0x5826,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5829,INVALC,INVALC,0x582B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A52,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5A54,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A53,0x654A,0x6549,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C51,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5C4F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E3C,
	INVALC,INVALC,INVALC,0x5E3A,INVALC,0x5E3B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6036,0x5F52,INVALC,INVALC,INVALC,
	INVALC,0x6037,INVALC,INVALC,0x654B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6179,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2428,0x2426,
	INVALC,INVALC,0x2427,0x242B,INVALC,0x2662,INVALC,0x265F,
	INVALC,0x265E,INVALC,0x2663,INVALC,INVALC,0x2660,INVALC,
	INVALC,0x2977,INVALC,INVALC,INVALC,INVALC,INVALC,0x2974,
	0x2970,0x2975,INVALC,0x296F,0x2979,0x2976,INVALC,0x2972,
	0x2971,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2D5E,INVALC,0x2D5C,0x2D5A,INVALC,0x2D58,0x626A,0x2D65,
	0x2D62,INVALC,0x2D5F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2D64,INVALC,INVALC,INVALC,0x2D63,INVALC,
	INVALC,0x2D68,INVALC,0x2D5D,INVALC,INVALC,INVALC,0x654D,
	INVALC,INVALC,INVALC,INVALC,0x3228,INVALC,INVALC,INVALC,
	INVALC,0x6279,INVALC,INVALC,0x322A,INVALC,INVALC,INVALC,
	0x322B,INVALC,INVALC,INVALC,INVALC,0x3231,0x322F,INVALC,
	INVALC,INVALC,INVALC,0x3229,INVALC,INVALC,INVALC,0x322D,
	INVALC,INVALC,INVALC,0x322C,INVALC,0x654F,INVALC,INVALC,
	INVALC,INVALC,0x654E,INVALC,INVALC,INVALC,0x3230,INVALC,
	INVALC,0x3730,0x3732,INVALC,INVALC,0x372F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3735,INVALC,
	/* Block 114, Array index 0x2400 */
	INVALC,0x372E,INVALC,0x372C,INVALC,INVALC,INVALC,0x372D,
	0x3731,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3733,
	INVALC,0x6550,0x6551,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C74,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C7D,INVALC,INVALC,0x3C7C,
	INVALC,INVALC,0x3C6F,0x3C7A,0x3C72,0x3D21,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3C7B,0x3C79,0x3C71,INVALC,0x3C73,
	0x3C77,0x6553,INVALC,0x6554,INVALC,0x6555,INVALC,0x6552,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4257,INVALC,INVALC,
	0x424C,INVALC,0x424E,INVALC,INVALC,INVALC,INVALC,0x4251,
	INVALC,0x4258,INVALC,INVALC,INVALC,INVALC,INVALC,0x4252,
	INVALC,INVALC,INVALC,0x483C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x424F,INVALC,0x4259,INVALC,0x4249,
	INVALC,0x6559,INVALC,INVALC,0x4253,0x6557,0x6558,0x6556,
	INVALC,INVALC,INVALC,INVALC,0x476F,INVALC,0x476E,0x476C,
	INVALC,INVALC,INVALC,0x4770,INVALC,INVALC,INVALC,INVALC,
	0x4772,INVALC,INVALC,0x476A,0x476D,0x476B,0x4768,INVALC,
	INVALC,0x4767,INVALC,0x655B,INVALC,INVALC,0x655A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D2A,0x4D24,INVALC,INVALC,0x4D29,INVALC,
	INVALC,INVALC,INVALC,0x4D28,INVALC,0x4D25,0x4D22,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x655C,INVALC,0x5162,0x515A,
	INVALC,INVALC,0x5160,0x515C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5163,0x6374,INVALC,INVALC,
	INVALC,0x515E,INVALC,0x5166,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x655D,0x655E,INVALC,
	INVALC,INVALC,INVALC,0x553C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x553E,0x5543,0x5540,INVALC,INVALC,INVALC,0x5541,
	INVALC,INVALC,INVALC,0x5542,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x582D,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 115, Array index 0x2500 */
	0x582E,INVALC,INVALC,INVALC,0x5A56,INVALC,INVALC,INVALC,
	0x5A57,INVALC,INVALC,0x5C59,INVALC,INVALC,0x655F,0x5C56,
	INVALC,0x5C58,0x5C55,INVALC,INVALC,0x5A58,0x5C53,0x5C57,
	0x6561,INVALC,INVALC,INVALC,0x5F53,INVALC,INVALC,INVALC,
	0x5F54,0x6039,INVALC,INVALC,0x6143,0x6142,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2D69,INVALC,
	INVALC,INVALC,0x3D22,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2D6B,INVALC,INVALC,INVALC,
	0x297C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x425D,INVALC,
	0x425C,0x425B,INVALC,INVALC,INVALC,0x4D2E,INVALC,0x4D2D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2664,0x2A21,0x2A22,INVALC,
	0x2A23,INVALC,INVALC,INVALC,INVALC,0x2D6D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3236,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3D24,0x3D23,INVALC,INVALC,INVALC,0x3D25,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4775,INVALC,INVALC,INVALC,
	INVALC,0x4D2F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x617B,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x242C,0x242D,INVALC,INVALC,INVALC,0x266B,INVALC,
	INVALC,0x2669,INVALC,0x2666,0x2667,INVALC,0x266A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2668,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2A26,INVALC,0x2A2A,
	INVALC,INVALC,INVALC,0x2A27,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2A28,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2D75,INVALC,INVALC,0x2D74,INVALC,0x2D70,
	INVALC,INVALC,INVALC,INVALC,0x2D73,0x2D71,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3238,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 116, Array index 0x2600 */
	INVALC,INVALC,0x3239,INVALC,INVALC,INVALC,0x3D26,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x373B,INVALC,
	0x373A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3739,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D27,INVALC,INVALC,INVALC,
	0x3D2E,INVALC,0x3D29,0x3D2B,0x3D2A,INVALC,0x4261,0x3D2D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D28,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x425F,INVALC,INVALC,INVALC,INVALC,0x4262,0x6562,
	0x6563,INVALC,INVALC,0x4D30,INVALC,INVALC,INVALC,0x4779,
	INVALC,INVALC,INVALC,0x4778,0x477A,0x4777,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5549,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x225A,0x216B,INVALC,0x242F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x266E,INVALC,INVALC,INVALC,0x2A33,INVALC,0x2A2D,INVALC,
	INVALC,INVALC,INVALC,0x2A2C,INVALC,INVALC,INVALC,0x2A2B,
	INVALC,INVALC,0x2A32,INVALC,INVALC,INVALC,INVALC,0x2A2F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2D78,INVALC,INVALC,
	INVALC,0x2D7B,INVALC,INVALC,0x2D77,INVALC,INVALC,INVALC,
	INVALC,0x2D7A,INVALC,INVALC,INVALC,INVALC,0x2D7E,0x2D79,
	0x2D7C,INVALC,INVALC,INVALC,INVALC,0x323E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3240,
	INVALC,0x3247,0x323C,INVALC,0x3246,INVALC,0x3241,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x323D,INVALC,INVALC,0x6564,
	INVALC,INVALC,INVALC,0x373D,INVALC,INVALC,INVALC,0x3D38,
	INVALC,0x323F,INVALC,0x3740,INVALC,INVALC,INVALC,INVALC,
	/* Block 117, Array index 0x2700 */
	INVALC,INVALC,0x373C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6565,INVALC,0x3D36,0x3D3A,INVALC,0x3D2F,
	0x3D3B,0x3D32,INVALC,INVALC,0x3D39,INVALC,0x426D,0x3D35,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4264,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D33,0x3D3C,INVALC,0x6566,0x3D37,0x4268,0x4275,INVALC,
	INVALC,INVALC,INVALC,0x426F,INVALC,0x4274,INVALC,0x426E,
	0x4266,0x4271,INVALC,INVALC,0x4273,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4267,INVALC,INVALC,0x426B,INVALC,
	0x4270,INVALC,INVALC,INVALC,INVALC,0x426A,INVALC,INVALC,
	0x4825,0x477E,INVALC,INVALC,INVALC,0x477C,0x477D,INVALC,
	0x4822,INVALC,INVALC,0x4821,0x477B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4827,INVALC,0x4823,INVALC,
	INVALC,INVALC,0x4D31,0x4D3B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D39,INVALC,0x4D36,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D35,0x6568,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4D3A,INVALC,INVALC,0x516A,INVALC,
	INVALC,0x6375,INVALC,0x5167,INVALC,0x516C,INVALC,INVALC,
	INVALC,INVALC,0x516B,INVALC,0x6567,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x554F,INVALC,0x554B,INVALC,
	INVALC,INVALC,INVALC,0x554D,0x6569,INVALC,INVALC,INVALC,
	INVALC,0x582F,INVALC,INVALC,0x5830,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A5A,INVALC,INVALC,0x5A5C,
	0x5A5D,0x5A5B,INVALC,INVALC,0x5C5E,0x5C5C,0x5C5F,INVALC,
	0x5C5D,0x5C60,INVALC,0x5E3D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2E22,INVALC,0x2E21,INVALC,0x2E24,
	INVALC,INVALC,INVALC,0x3248,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3D3D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 118, Array index 0x2800 */
	INVALC,0x4277,INVALC,INVALC,INVALC,INVALC,0x4828,INVALC,
	INVALC,0x4D3D,0x516D,INVALC,INVALC,INVALC,0x516E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3744,INVALC,INVALC,0x4278,INVALC,
	INVALC,INVALC,INVALC,0x3745,0x3D3E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2A37,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2A38,0x2A35,INVALC,0x2A36,INVALC,0x2E27,INVALC,
	INVALC,0x2E25,0x2E28,INVALC,INVALC,0x2E2A,INVALC,INVALC,
	0x2E26,INVALC,INVALC,INVALC,INVALC,0x324E,0x324D,0x3250,
	0x324F,INVALC,INVALC,INVALC,INVALC,INVALC,0x3251,INVALC,
	INVALC,0x3746,INVALC,INVALC,INVALC,INVALC,INVALC,0x3749,
	0x3747,INVALC,INVALC,INVALC,INVALC,0x3D43,0x3D42,INVALC,
	INVALC,0x3D44,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x427B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5170,INVALC,0x5552,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C62,INVALC,INVALC,INVALC,INVALC,0x3D45,INVALC,
	INVALC,INVALC,0x225B,0x2673,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2A3A,INVALC,INVALC,0x2A3B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2E2B,INVALC,
	0x2E2F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3257,INVALC,INVALC,0x3255,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3252,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x374A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3D47,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x427C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 119, Array index 0x2900 */
	INVALC,INVALC,0x427D,INVALC,INVALC,INVALC,INVALC,0x482B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4829,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5171,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5173,0x5172,
	INVALC,INVALC,INVALC,0x5174,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5555,0x5553,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5554,0x5834,
	INVALC,0x5833,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5832,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E3F,
	0x5E3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x603B,
	INVALC,INVALC,INVALC,0x656A,INVALC,INVALC,0x225C,0x656B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2674,INVALC,INVALC,0x2676,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x374D,INVALC,INVALC,INVALC,0x3D4C,INVALC,INVALC,0x427E,
	0x4321,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5176,INVALC,0x5556,INVALC,0x5557,INVALC,INVALC,
	INVALC,0x5A5E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3D4D,INVALC,INVALC,INVALC,0x4830,
	INVALC,0x482E,INVALC,INVALC,0x656C,INVALC,INVALC,INVALC,
	0x2677,0x2678,INVALC,INVALC,INVALC,INVALC,INVALC,0x2E32,
	INVALC,INVALC,INVALC,0x325A,0x325B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3750,0x3752,0x3751,
	INVALC,0x3D4F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4831,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5835,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2E35,INVALC,0x2E33,
	/* Block 120, Array index 0x2A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x626C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3264,0x325F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x325E,INVALC,
	0x3268,INVALC,INVALC,INVALC,INVALC,INVALC,0x3759,INVALC,
	INVALC,0x3267,INVALC,INVALC,0x3265,INVALC,0x3755,INVALC,
	INVALC,INVALC,0x325C,0x3263,INVALC,INVALC,INVALC,INVALC,
	0x3753,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3262,
	0x3757,INVALC,0x3D56,0x3D53,INVALC,INVALC,INVALC,INVALC,
	0x3D51,0x3D52,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4327,INVALC,INVALC,INVALC,INVALC,
	0x4328,INVALC,INVALC,INVALC,INVALC,0x4325,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4326,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4839,INVALC,INVALC,0x4833,0x4836,0x4835,0x4834,
	0x4838,INVALC,0x483B,0x483D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D44,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D46,INVALC,INVALC,INVALC,INVALC,INVALC,0x517B,INVALC,
	0x5179,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5559,INVALC,0x5558,INVALC,
	INVALC,0x555B,INVALC,INVALC,INVALC,INVALC,INVALC,0x555C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5836,0x5838,INVALC,INVALC,INVALC,INVALC,0x583B,INVALC,
	0x583A,0x5839,INVALC,INVALC,INVALC,0x5A60,0x5A5F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5E42,INVALC,INVALC,INVALC,INVALC,0x6432,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3269,INVALC,0x3D57,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2E36,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x267B,0x267C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 121, Array index 0x2B00 */
	INVALC,INVALC,INVALC,INVALC,0x2E39,INVALC,INVALC,0x2E3B,
	0x2E3E,INVALC,0x2E37,0x2E3A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2E3D,INVALC,INVALC,
	INVALC,0x656D,INVALC,0x656E,INVALC,INVALC,0x667C,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x326C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x326D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x656F,INVALC,INVALC,
	INVALC,INVALC,0x3762,INVALC,INVALC,INVALC,0x3763,0x375D,
	INVALC,INVALC,INVALC,0x375F,INVALC,INVALC,INVALC,0x3761,
	INVALC,0x3760,INVALC,0x6570,0x6571,INVALC,INVALC,INVALC,
	0x6572,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D5C,
	INVALC,INVALC,INVALC,0x3D59,INVALC,INVALC,0x3D5D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D5A,INVALC,
	0x4331,INVALC,0x432D,INVALC,INVALC,0x432E,INVALC,INVALC,
	INVALC,0x4337,0x4329,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4335,INVALC,0x432B,0x4330,INVALC,0x4334,INVALC,INVALC,
	0x432A,INVALC,0x4336,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x483E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x483F,0x4845,
	INVALC,0x4844,INVALC,INVALC,INVALC,INVALC,INVALC,0x6573,
	0x4840,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D4A,INVALC,0x4D4D,INVALC,INVALC,INVALC,0x4D48,0x4D4C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D4B,0x4D49,INVALC,INVALC,0x5221,INVALC,
	0x6574,INVALC,INVALC,INVALC,0x5224,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5222,INVALC,INVALC,0x6575,INVALC,
	0x5560,0x5562,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5561,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 122, Array index 0x2C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5843,0x583E,
	0x5840,INVALC,0x583F,0x583C,INVALC,0x5842,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5844,INVALC,INVALC,
	0x5A61,INVALC,0x5A62,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A63,INVALC,0x6576,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E45,INVALC,
	0x5E43,INVALC,0x5E44,INVALC,INVALC,INVALC,INVALC,0x603C,
	INVALC,INVALC,INVALC,INVALC,0x2433,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6577,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x326F,0x3270,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3272,INVALC,INVALC,INVALC,0x3D5E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3766,INVALC,0x3765,INVALC,
	INVALC,0x3D5F,INVALC,INVALC,INVALC,0x3D64,0x3D62,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4338,INVALC,
	0x433D,INVALC,INVALC,0x4339,0x3D63,INVALC,0x6578,0x4848,
	INVALC,0x4847,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6579,INVALC,INVALC,INVALC,0x4D50,0x4D51,0x4D4F,
	INVALC,INVALC,INVALC,0x657A,INVALC,0x5226,INVALC,INVALC,
	INVALC,0x5227,INVALC,INVALC,INVALC,INVALC,INVALC,0x5845,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x603D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x657B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2A3E,0x2A3D,INVALC,0x2A3F,0x2A42,
	INVALC,INVALC,0x2A44,INVALC,0x654C,INVALC,INVALC,INVALC,
	0x2E42,INVALC,INVALC,0x2E40,0x2E41,INVALC,INVALC,0x2E43,
	INVALC,0x3274,0x3273,0x3279,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3276,0x327B,INVALC,INVALC,0x3277,INVALC,INVALC,
	0x3278,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x327C,0x3769,0x3767,0x376B,INVALC,0x376A,INVALC,INVALC,
	INVALC,0x376C,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D65,
	/* Block 123, Array index 0x2D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D66,
	INVALC,0x3D69,INVALC,INVALC,INVALC,INVALC,0x3D68,0x4341,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4344,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6524,INVALC,INVALC,
	INVALC,0x484B,0x484D,INVALC,0x4850,0x484F,INVALC,INVALC,
	INVALC,INVALC,0x4852,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D55,INVALC,0x4D56,INVALC,INVALC,INVALC,0x4D5A,INVALC,
	INVALC,0x4D53,0x4D57,INVALC,INVALC,0x522A,INVALC,INVALC,
	INVALC,0x5564,INVALC,INVALC,INVALC,INVALC,INVALC,0x522C,
	INVALC,INVALC,INVALC,0x5229,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A66,INVALC,0x5A65,INVALC,
	INVALC,INVALC,0x5A67,INVALC,INVALC,INVALC,INVALC,0x5C64,
	INVALC,INVALC,0x6076,0x6077,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2E44,0x2E47,INVALC,INVALC,
	INVALC,INVALC,0x2E49,0x2E48,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3772,INVALC,0x3771,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3770,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4346,INVALC,INVALC,0x4347,INVALC,INVALC,0x657C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D5B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x522F,INVALC,0x522E,INVALC,INVALC,
	INVALC,INVALC,0x5568,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5E49,0x5E48,INVALC,INVALC,0x267E,0x2721,INVALC,0x2A47,
	0x2E4E,INVALC,0x2E4F,0x2E4D,0x2E4B,0x2E4C,INVALC,INVALC,
	INVALC,INVALC,0x3323,0x3326,0x3327,0x3324,INVALC,INVALC,
	INVALC,0x3773,0x3D71,INVALC,INVALC,INVALC,INVALC,0x3D70,
	INVALC,0x4349,0x434A,INVALC,0x4854,INVALC,INVALC,INVALC,
	0x4855,0x5230,0x5569,0x556A,INVALC,0x5849,INVALC,INVALC,
	0x5F55,INVALC,INVALC,INVALC,0x2E51,INVALC,0x2E50,INVALC,
	/* Block 124, Array index 0x2E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x332B,INVALC,0x332A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x332C,INVALC,0x3778,0x3779,
	INVALC,INVALC,INVALC,INVALC,0x3777,INVALC,INVALC,0x377B,
	INVALC,0x3776,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x632D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x377A,0x657D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3D7B,0x3D78,0x3D75,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3D79,INVALC,0x3D72,INVALC,0x3D76,
	INVALC,0x657E,INVALC,INVALC,INVALC,INVALC,0x434F,INVALC,
	INVALC,INVALC,0x434C,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4355,INVALC,INVALC,0x434D,0x3D77,INVALC,INVALC,0x4354,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4351,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4856,INVALC,0x4858,INVALC,INVALC,0x4860,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x485A,0x485E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4857,0x485D,
	INVALC,INVALC,INVALC,0x485C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D5C,INVALC,
	INVALC,INVALC,0x4D65,0x4D5D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D60,
	INVALC,INVALC,0x6367,0x6366,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4D63,INVALC,INVALC,
	0x6621,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5233,0x5234,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x556D,INVALC,
	INVALC,INVALC,0x556B,INVALC,0x556E,INVALC,INVALC,INVALC,
	/* Block 125, Array index 0x2F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5570,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5571,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x556F,INVALC,0x5850,0x584E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5851,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x584C,0x6622,
	INVALC,INVALC,0x6623,INVALC,0x5A6F,0x5A6E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6624,INVALC,0x5C66,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5C65,INVALC,INVALC,0x6625,INVALC,
	INVALC,0x5E4C,INVALC,INVALC,INVALC,INVALC,0x5E4B,INVALC,
	0x5F58,INVALC,INVALC,INVALC,INVALC,0x5F57,0x603F,INVALC,
	INVALC,INVALC,0x603E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6626,INVALC,INVALC,INVALC,INVALC,0x6144,INVALC,INVALC,
	INVALC,0x623A,INVALC,INVALC,0x2A49,INVALC,0x2A48,0x2E52,
	INVALC,INVALC,INVALC,0x2E54,0x2E53,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3331,INVALC,INVALC,0x332E,0x3330,
	INVALC,INVALC,INVALC,0x3332,INVALC,INVALC,0x332D,INVALC,
	INVALC,INVALC,INVALC,0x377D,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x377E,0x3821,0x383E,0x3822,INVALC,INVALC,INVALC,
	0x3E21,INVALC,INVALC,INVALC,0x3E23,INVALC,0x3E22,0x3D7E,
	INVALC,0x6627,INVALC,INVALC,INVALC,INVALC,0x4358,INVALC,
	0x435C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4862,
	0x4861,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D6F,0x4D71,0x4D6D,INVALC,0x4D70,0x4D73,
	INVALC,0x4D72,INVALC,INVALC,INVALC,0x4D6E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5237,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5575,0x5574,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5239,INVALC,0x5855,INVALC,INVALC,0x5854,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6628,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6169,INVALC,0x6040,INVALC,INVALC,
	INVALC,INVALC,0x2722,INVALC,0x2A4B,INVALC,INVALC,0x2A4A,
	/* Block 126, Array index 0x3000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3337,0x333B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3829,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3826,INVALC,INVALC,0x3827,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x382E,0x3824,INVALC,INVALC,0x382B,0x3E34,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E29,
	INVALC,0x3E26,0x3E2E,INVALC,INVALC,0x3E2F,INVALC,INVALC,
	0x3E28,INVALC,INVALC,INVALC,0x3E2C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E2A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E24,0x3E35,0x3E33,INVALC,
	0x4367,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E2B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4366,
	INVALC,0x4369,0x4360,0x4362,INVALC,INVALC,INVALC,INVALC,
	0x435E,INVALC,INVALC,INVALC,INVALC,0x4364,INVALC,0x4368,
	0x436A,0x4363,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4874,INVALC,INVALC,INVALC,
	0x486D,INVALC,INVALC,0x486E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4870,INVALC,INVALC,0x4869,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4868,INVALC,INVALC,INVALC,INVALC,0x4871,INVALC,0x4873,
	0x6629,0x486B,INVALC,0x662B,INVALC,INVALC,0x4E22,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D7C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D78,INVALC,0x4E24,0x4E2F,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E2E,INVALC,0x4D77,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E2B,INVALC,INVALC,
	0x4D75,INVALC,INVALC,INVALC,0x4E28,0x4E29,0x4E25,0x4E2C,
	/* Block 127, Array index 0x3100 */
	0x4E27,0x4D7E,0x4D7D,INVALC,INVALC,INVALC,INVALC,0x662A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x523B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5240,0x523F,
	INVALC,INVALC,INVALC,INVALC,0x5579,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x557A,INVALC,0x5859,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x585B,INVALC,0x5858,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x662C,INVALC,0x585A,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A72,INVALC,0x5A74,INVALC,
	INVALC,0x5A75,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C6C,
	INVALC,INVALC,INVALC,INVALC,0x5E53,0x5E50,INVALC,INVALC,
	INVALC,0x5E51,INVALC,INVALC,INVALC,INVALC,0x5E52,INVALC,
	INVALC,INVALC,0x5F5A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6235,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 128, Array index 0x3200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3831,INVALC,INVALC,INVALC,INVALC,
	0x662D,0x4877,INVALC,INVALC,INVALC,INVALC,0x557D,0x585C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F5B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2A4E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x333C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4878,
	INVALC,0x4879,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4E30,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6422,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2E5D,
	INVALC,INVALC,INVALC,0x333D,INVALC,INVALC,INVALC,0x333F,
	INVALC,0x333E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x436E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E31,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x585D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3342,INVALC,0x3343,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3E37,INVALC,INVALC,0x3E38,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4372,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E32,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x557E,INVALC,
	INVALC,INVALC,0x5861,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 129, Array index 0x3300 */
	INVALC,INVALC,0x667B,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3836,INVALC,0x3344,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3837,INVALC,INVALC,0x3839,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4922,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5621,INVALC,INVALC,0x5862,0x5C6F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3349,0x3348,INVALC,INVALC,INVALC,
	0x3347,0x662F,INVALC,INVALC,0x383D,0x383C,INVALC,INVALC,
	INVALC,0x383B,INVALC,INVALC,INVALC,INVALC,0x3E3C,INVALC,
	INVALC,INVALC,INVALC,0x6729,INVALC,INVALC,INVALC,0x4373,
	INVALC,0x6356,INVALC,0x4923,INVALC,INVALC,INVALC,0x4926,
	0x3E3B,0x4925,INVALC,0x6630,INVALC,0x4E34,0x4E36,INVALC,
	0x4E37,INVALC,0x4E35,0x4E38,INVALC,INVALC,0x5243,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5622,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C70,0x5C71,0x5C72,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2435,0x2436,INVALC,
	INVALC,0x2724,INVALC,INVALC,0x2726,INVALC,INVALC,0x2725,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2723,
	INVALC,INVALC,INVALC,INVALC,0x2A56,INVALC,0x2A5B,0x2A58,
	0x2A4F,INVALC,INVALC,INVALC,0x2A59,INVALC,INVALC,INVALC,
	0x2A52,INVALC,INVALC,0x2A54,INVALC,INVALC,0x2A57,INVALC,
	INVALC,INVALC,INVALC,0x6631,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2E66,INVALC,
	INVALC,INVALC,INVALC,0x2E60,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2E61,0x2E5E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2E65,
	INVALC,INVALC,0x2E64,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x334C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x334A,0x3352,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x334B,
	/* Block 130, Array index 0x3400 */
	INVALC,INVALC,INVALC,0x334E,0x334D,INVALC,INVALC,0x334F,
	INVALC,0x2E63,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E44,INVALC,INVALC,0x3849,
	INVALC,INVALC,0x3847,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x632E,INVALC,0x3841,INVALC,INVALC,INVALC,0x3842,INVALC,
	INVALC,0x3845,INVALC,INVALC,0x3848,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3E3E,INVALC,INVALC,INVALC,INVALC,
	0x3E43,0x3E45,0x3E41,INVALC,INVALC,0x3E3D,INVALC,INVALC,
	0x3E40,0x3E42,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x437B,INVALC,0x437E,
	INVALC,INVALC,INVALC,0x4375,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4376,INVALC,0x4377,0x4423,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x437C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4422,INVALC,INVALC,
	INVALC,0x4927,INVALC,INVALC,0x492A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E3B,0x4E3D,0x4E39,INVALC,0x4E3C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E3A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5249,INVALC,INVALC,0x5247,INVALC,0x5244,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5623,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5624,INVALC,INVALC,INVALC,INVALC,INVALC,0x5865,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5A77,INVALC,0x5A78,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E47,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E48,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 131, Array index 0x3500 */
	INVALC,INVALC,INVALC,0x3E49,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2A5D,INVALC,
	INVALC,INVALC,INVALC,0x492B,INVALC,INVALC,INVALC,INVALC,
	0x524A,0x5868,0x5A79,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2E68,INVALC,INVALC,0x3357,
	INVALC,0x3358,INVALC,INVALC,INVALC,0x3355,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4426,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6634,INVALC,0x492D,INVALC,0x492E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A7A,INVALC,INVALC,0x5A7B,INVALC,INVALC,
	INVALC,INVALC,0x5E55,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A7C,INVALC,
	INVALC,INVALC,INVALC,0x2438,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2728,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2729,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x272B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2A60,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2A6B,INVALC,
	INVALC,INVALC,0x2A66,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2A6C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2A5E,INVALC,INVALC,INVALC,
	INVALC,0x2A68,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2E76,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2E6A,INVALC,INVALC,INVALC,INVALC,0x2E6B,INVALC,INVALC,
	INVALC,INVALC,0x2E78,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2E72,INVALC,INVALC,INVALC,INVALC,0x2E74,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2E69,
	0x2E6E,INVALC,INVALC,INVALC,0x2E6F,0x2E7A,INVALC,INVALC,
	/* Block 132, Array index 0x3600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x2E79,INVALC,INVALC,0x2E77,INVALC,
	0x6635,INVALC,0x6637,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3362,INVALC,INVALC,INVALC,INVALC,0x384D,INVALC,0x3359,
	INVALC,0x335F,INVALC,0x385B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2A5F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E57,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x335A,
	0x3363,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6636,0x2D33,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x384E,0x3851,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3854,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3850,INVALC,INVALC,INVALC,0x384F,0x385D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3855,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x385E,INVALC,INVALC,0x3857,
	INVALC,0x3852,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3858,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x385F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3E5E,INVALC,INVALC,
	INVALC,INVALC,0x3E4F,0x3E59,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3E4D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E5F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3E4B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 133, Array index 0x3700 */
	0x3E52,INVALC,0x3E5C,INVALC,INVALC,0x3E5A,INVALC,INVALC,
	0x3E50,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E58,INVALC,0x3E5D,0x3E56,
	0x3E4E,INVALC,INVALC,INVALC,INVALC,INVALC,0x6639,0x3E5B,
	INVALC,0x6638,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x442D,INVALC,INVALC,INVALC,0x4431,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x443A,INVALC,
	INVALC,0x4435,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x442C,INVALC,0x443E,INVALC,INVALC,INVALC,INVALC,0x4427,
	INVALC,INVALC,INVALC,0x4433,INVALC,0x443F,INVALC,INVALC,
	0x4440,INVALC,INVALC,INVALC,0x4432,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4438,INVALC,0x4430,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x443C,0x442F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x442E,INVALC,INVALC,INVALC,0x442A,
	0x4439,INVALC,INVALC,0x443B,0x4428,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4935,0x4944,INVALC,0x4933,0x4938,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x493C,INVALC,0x4939,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4942,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x493B,INVALC,0x493A,INVALC,0x4941,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4932,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4936,INVALC,INVALC,INVALC,INVALC,
	0x4436,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x493F,INVALC,INVALC,
	0x493E,INVALC,0x4943,INVALC,INVALC,INVALC,0x663B,INVALC,
	INVALC,0x663C,0x663A,INVALC,0x493D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E4F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E44,INVALC,INVALC,
	0x4E42,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 134, Array index 0x3800 */
	INVALC,0x4E40,INVALC,0x4E4C,0x4E46,0x4E54,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E4B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E4D,0x4E4E,0x4E52,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x525A,
	INVALC,INVALC,0x5252,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5259,0x5250,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5258,INVALC,0x5254,INVALC,INVALC,0x5256,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x524F,
	0x5255,INVALC,INVALC,0x663D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x562B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5628,0x5629,INVALC,INVALC,INVALC,0x562D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x562C,0x5630,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x586C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x586E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x586F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x586B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x663E,INVALC,0x5A7E,INVALC,
	INVALC,INVALC,INVALC,0x5A7D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B22,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C7B,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5B25,0x663F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5C76,INVALC,INVALC,INVALC,
	/* Block 135, Array index 0x3900 */
	INVALC,INVALC,0x5C78,0x5C7C,INVALC,INVALC,INVALC,INVALC,
	0x5D22,INVALC,INVALC,INVALC,INVALC,0x5C7E,INVALC,0x5D23,
	0x5C77,INVALC,INVALC,0x5D21,INVALC,0x5E5B,0x5E5A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E5C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5E58,INVALC,INVALC,0x5E56,INVALC,INVALC,INVALC,0x5E57,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6640,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6043,INVALC,INVALC,
	INVALC,0x602F,0x6146,INVALC,INVALC,0x6641,INVALC,INVALC,
	INVALC,0x6147,0x617C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3863,
	0x3860,INVALC,0x3861,INVALC,INVALC,0x3E61,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x272D,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6642,INVALC,INVALC,0x2F23,0x2E7C,INVALC,
	0x2E7E,INVALC,INVALC,INVALC,INVALC,0x2F24,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3369,INVALC,INVALC,INVALC,
	INVALC,0x336C,INVALC,INVALC,INVALC,INVALC,INVALC,0x336F,
	INVALC,INVALC,0x336E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x336B,INVALC,INVALC,INVALC,0x336A,
	0x3370,INVALC,INVALC,INVALC,INVALC,INVALC,0x336D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3866,INVALC,
	INVALC,INVALC,0x3864,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3E62,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3E63,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E68,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4444,0x4445,INVALC,0x4449,
	/* Block 136, Array index 0x3A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4446,0x4448,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x494F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x494A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4945,INVALC,
	INVALC,0x494C,INVALC,INVALC,INVALC,0x4949,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x494E,INVALC,0x494B,
	INVALC,INVALC,0x6643,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E5D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E59,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x525B,0x525D,INVALC,INVALC,0x525F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5262,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x525E,INVALC,INVALC,INVALC,0x6645,INVALC,INVALC,
	INVALC,0x6644,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x563A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5639,INVALC,INVALC,INVALC,INVALC,INVALC,0x563B,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5636,0x5634,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5B28,0x5876,INVALC,
	INVALC,INVALC,0x5878,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5873,INVALC,INVALC,INVALC,INVALC,0x5874,INVALC,
	0x5875,INVALC,INVALC,INVALC,INVALC,0x6646,INVALC,INVALC,
	0x5B2C,INVALC,INVALC,INVALC,INVALC,0x5B27,INVALC,INVALC,
	/* Block 137, Array index 0x3B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B29,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5877,0x5B2B,
	INVALC,INVALC,0x5D25,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5D24,INVALC,0x5E5D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F5F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6045,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6647,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2F26,INVALC,INVALC,0x386A,0x3E6A,0x3E69,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3371,
	0x6648,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x444D,INVALC,INVALC,INVALC,0x4E5F,INVALC,0x5263,0x5264,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2F27,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3374,INVALC,
	INVALC,INVALC,INVALC,0x3377,INVALC,INVALC,INVALC,INVALC,
	0x3372,INVALC,0x3373,0x3375,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3376,INVALC,INVALC,0x6649,
	INVALC,INVALC,0x3872,INVALC,INVALC,INVALC,INVALC,0x386E,
	0x3874,INVALC,INVALC,INVALC,0x386C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x386B,INVALC,INVALC,INVALC,
	0x3873,INVALC,INVALC,0x386D,INVALC,0x386F,0x3871,INVALC,
	INVALC,0x3870,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E70,0x3E72,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3E6D,INVALC,INVALC,INVALC,0x3E6F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x444E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x444F,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E60,INVALC,
	INVALC,0x4959,0x4952,INVALC,INVALC,0x4957,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4954,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4951,
	/* Block 138, Array index 0x3C00 */
	0x4956,INVALC,INVALC,0x4958,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E61,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E62,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5265,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5640,INVALC,INVALC,INVALC,INVALC,INVALC,0x563F,
	0x563D,INVALC,INVALC,0x5642,INVALC,INVALC,INVALC,INVALC,
	0x5879,INVALC,0x587A,INVALC,INVALC,0x587C,0x587B,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x664A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5D29,INVALC,0x5D28,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6149,INVALC,INVALC,INVALC,INVALC,0x617D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x664B,INVALC,INVALC,0x5B2F,
	INVALC,0x6046,0x614A,INVALC,0x2F29,INVALC,INVALC,INVALC,
	INVALC,0x3876,0x3875,INVALC,0x3877,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3E74,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E65,INVALC,0x5266,
	INVALC,0x4E64,INVALC,0x5646,INVALC,INVALC,INVALC,INVALC,
	0x587E,0x587D,INVALC,0x5921,INVALC,0x5B31,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D2A,0x5E61,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4454,
	INVALC,INVALC,0x495C,INVALC,INVALC,INVALC,0x4E66,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5B32,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 139, Array index 0x3D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2F2A,0x2F2C,INVALC,
	INVALC,INVALC,INVALC,0x337B,INVALC,0x337A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x337C,INVALC,INVALC,0x387E,INVALC,INVALC,INVALC,
	0x3923,0x3929,INVALC,INVALC,INVALC,INVALC,0x3879,INVALC,
	0x387B,0x3925,INVALC,0x3922,INVALC,INVALC,0x6330,INVALC,
	INVALC,INVALC,0x387A,INVALC,INVALC,0x3E7C,INVALC,INVALC,
	0x3E7D,INVALC,INVALC,INVALC,INVALC,0x3F24,INVALC,INVALC,
	INVALC,INVALC,0x3E7E,0x3E75,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3F21,INVALC,0x3F23,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3E78,INVALC,INVALC,0x3F22,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4460,0x4459,INVALC,0x4463,
	INVALC,INVALC,0x4455,INVALC,INVALC,INVALC,INVALC,0x4458,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x445B,INVALC,INVALC,INVALC,INVALC,0x445C,0x4461,INVALC,
	0x4456,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x445D,INVALC,INVALC,INVALC,INVALC,INVALC,0x495F,INVALC,
	0x4964,INVALC,INVALC,INVALC,0x4963,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x495E,0x4962,0x496B,INVALC,0x4968,
	INVALC,INVALC,0x4960,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4965,INVALC,INVALC,0x4966,INVALC,INVALC,0x4E74,
	INVALC,0x4E67,INVALC,0x4E72,0x4E70,0x4E6E,INVALC,0x4E6C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E73,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E75,INVALC,0x4E69,INVALC,0x4E6A,INVALC,
	0x4E76,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x526A,INVALC,0x526B,INVALC,INVALC,
	INVALC,0x4E6F,0x5267,INVALC,0x526E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5269,INVALC,INVALC,INVALC,0x5268,INVALC,INVALC,
	/* Block 140, Array index 0x3E00 */
	INVALC,INVALC,INVALC,0x664C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5649,INVALC,INVALC,0x564A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5648,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x592A,INVALC,INVALC,INVALC,0x5926,INVALC,INVALC,
	INVALC,INVALC,0x5924,INVALC,0x5928,INVALC,INVALC,INVALC,
	0x5923,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B34,
	INVALC,INVALC,INVALC,0x5B35,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5B36,0x5D33,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D30,0x5D2C,INVALC,0x5D32,INVALC,INVALC,INVALC,
	INVALC,0x5D2B,INVALC,INVALC,INVALC,INVALC,0x5D2E,INVALC,
	INVALC,0x5C6D,INVALC,0x642D,INVALC,0x5E62,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E63,INVALC,INVALC,INVALC,
	INVALC,0x5F61,INVALC,0x5F62,INVALC,INVALC,INVALC,0x6048,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6047,INVALC,INVALC,
	0x6049,0x607B,INVALC,INVALC,INVALC,INVALC,INVALC,0x614B,
	INVALC,INVALC,INVALC,0x616B,INVALC,0x617E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 141, Array index 0x3F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x392A,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x564D,INVALC,INVALC,INVALC,0x5F64,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D34,0x664D,0x6221,INVALC,INVALC,INVALC,INVALC,
	0x392D,INVALC,INVALC,0x392C,INVALC,INVALC,0x3F2A,INVALC,
	0x3F29,INVALC,INVALC,0x4464,INVALC,INVALC,INVALC,0x496D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x592B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x392E,INVALC,0x3F2C,0x3F2B,
	INVALC,INVALC,INVALC,0x3F2D,INVALC,INVALC,INVALC,0x634A,
	0x4465,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E7A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x564F,INVALC,
	INVALC,INVALC,INVALC,0x614D,INVALC,INVALC,INVALC,0x2F2E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3930,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3932,0x3931,INVALC,
	INVALC,0x3F2F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4469,INVALC,INVALC,INVALC,0x4467,INVALC,INVALC,
	0x4970,INVALC,INVALC,INVALC,0x4972,INVALC,0x4971,INVALC,
	0x664E,INVALC,INVALC,0x4E7E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E7D,INVALC,0x4F22,INVALC,INVALC,INVALC,0x5275,
	INVALC,INVALC,0x5273,INVALC,INVALC,INVALC,0x5651,0x5652,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 142, Array index 0x4000 */
	INVALC,INVALC,INVALC,0x592E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5B38,0x664F,INVALC,0x6428,INVALC,
	INVALC,0x5E65,0x5E66,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6650,INVALC,0x604A,INVALC,0x614E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3933,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x243A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3939,INVALC,0x3937,INVALC,0x3935,0x3938,
	INVALC,INVALC,0x3F39,0x3F34,INVALC,INVALC,0x3F37,0x3F30,
	0x3F33,INVALC,INVALC,0x3F38,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4975,
	0x4973,INVALC,0x4974,INVALC,INVALC,0x4F26,INVALC,INVALC,
	INVALC,INVALC,0x4F25,INVALC,0x4F23,INVALC,0x5277,INVALC,
	INVALC,0x5931,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6651,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x393E,INVALC,0x393A,INVALC,INVALC,
	0x393D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3F3F,0x3F3A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F3B,INVALC,0x3F3E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x446B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 143, Array index 0x4100 */
	INVALC,0x497B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x497A,INVALC,INVALC,0x4979,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4F2C,INVALC,0x4F2B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x527E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x527B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x527C,INVALC,INVALC,0x5654,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5655,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5932,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5B3A,INVALC,INVALC,0x5B39,INVALC,INVALC,INVALC,INVALC,
	0x5B3B,INVALC,INVALC,INVALC,INVALC,0x5B3C,INVALC,0x5B3D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D37,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x604B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6150,INVALC,INVALC,INVALC,INVALC,0x616C,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x393F,INVALC,0x3940,
	0x3F40,0x4473,INVALC,0x497D,0x497C,INVALC,0x4F30,0x4F32,
	0x4F2F,0x4F31,INVALC,INVALC,0x6652,INVALC,INVALC,0x5935,
	INVALC,0x5936,INVALC,0x5B3F,INVALC,INVALC,0x5D38,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6653,0x3941,INVALC,
	INVALC,0x3947,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F45,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F47,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F44,0x3F46,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F49,INVALC,INVALC,
	/* Block 144, Array index 0x4200 */
	INVALC,INVALC,INVALC,INVALC,0x4476,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F36,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F33,INVALC,INVALC,INVALC,INVALC,0x4F35,
	INVALC,INVALC,INVALC,0x4F34,INVALC,0x5324,INVALC,INVALC,
	0x5322,0x5321,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5325,0x5659,INVALC,INVALC,
	INVALC,0x565A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5937,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E67,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6222,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F4B,0x4477,INVALC,
	0x4478,0x4A21,0x4A22,INVALC,0x4F37,0x5327,INVALC,0x5326,
	INVALC,INVALC,0x6654,INVALC,0x5938,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4F39,INVALC,0x272F,INVALC,INVALC,
	INVALC,0x243B,INVALC,INVALC,INVALC,INVALC,0x2730,INVALC,
	INVALC,0x2733,INVALC,INVALC,INVALC,INVALC,INVALC,0x2731,
	INVALC,INVALC,0x2A72,INVALC,0x2A70,INVALC,INVALC,0x2A74,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2A73,0x2A71,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2F31,
	INVALC,0x2F32,INVALC,INVALC,INVALC,INVALC,INVALC,0x2F33,
	INVALC,0x2F30,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3425,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 145, Array index 0x4300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3427,0x3426,INVALC,INVALC,INVALC,INVALC,0x394B,INVALC,
	INVALC,INVALC,0x394A,0x394E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x394C,INVALC,INVALC,
	0x3F4F,0x3F50,0x3F4D,0x3F4C,0x3F51,INVALC,INVALC,INVALC,
	0x3F55,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3F56,
	INVALC,INVALC,0x3F54,0x3F57,INVALC,INVALC,INVALC,INVALC,
	0x447D,INVALC,INVALC,0x4479,INVALC,0x447C,0x447A,INVALC,
	0x447B,INVALC,INVALC,INVALC,0x447E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A24,INVALC,INVALC,INVALC,INVALC,0x4A27,
	INVALC,0x4A26,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F3C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F3B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x593A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x225E,0x225D,INVALC,INVALC,0x243C,INVALC,
	INVALC,INVALC,0x243D,INVALC,0x6655,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2735,INVALC,INVALC,INVALC,
	0x2739,0x2734,INVALC,0x3951,INVALC,0x2A77,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2A75,
	INVALC,0x2A78,INVALC,INVALC,0x2F35,INVALC,INVALC,INVALC,
	0x2F38,INVALC,0x2F34,INVALC,0x2F3E,INVALC,0x2F37,INVALC,
	INVALC,0x2F3C,INVALC,INVALC,INVALC,0x2F3B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3429,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x394F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x636C,0x3952,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 146, Array index 0x4400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F59,INVALC,0x4523,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4524,INVALC,0x4525,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4A2A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x532B,
	INVALC,INVALC,INVALC,INVALC,0x593C,0x593B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x342A,INVALC,INVALC,0x3955,INVALC,INVALC,INVALC,
	INVALC,0x3954,INVALC,INVALC,0x6657,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4528,
	INVALC,INVALC,INVALC,0x4527,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A2C,INVALC,
	INVALC,INVALC,INVALC,0x4A2B,0x4A2D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4F3F,INVALC,INVALC,INVALC,0x4F40,INVALC,0x532D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x532C,0x532F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x593D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E68,INVALC,INVALC,INVALC,0x607E,
	INVALC,INVALC,INVALC,INVALC,0x616D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3430,INVALC,
	INVALC,INVALC,INVALC,0x342D,INVALC,INVALC,0x342F,0x342E,
	0x3431,INVALC,INVALC,INVALC,INVALC,0x3958,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3957,INVALC,0x3956,INVALC,INVALC,INVALC,0x3959,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3F5D,INVALC,
	/* Block 147, Array index 0x4500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3F66,INVALC,INVALC,0x3F60,INVALC,INVALC,0x3F68,INVALC,
	INVALC,INVALC,INVALC,0x3F69,INVALC,INVALC,INVALC,INVALC,
	0x3F62,INVALC,INVALC,0x3F65,INVALC,INVALC,INVALC,INVALC,
	0x3F5E,0x6659,0x3F63,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6658,INVALC,INVALC,INVALC,0x452E,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x452B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4537,0x452F,INVALC,INVALC,INVALC,
	INVALC,0x4539,INVALC,INVALC,0x4538,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x452A,INVALC,INVALC,
	INVALC,0x452D,INVALC,INVALC,0x4531,0x4529,INVALC,0x4534,
	INVALC,INVALC,0x4533,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4532,0x4535,INVALC,INVALC,0x4536,INVALC,
	INVALC,INVALC,INVALC,0x665A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A34,INVALC,INVALC,0x4A35,INVALC,INVALC,INVALC,
	INVALC,0x4A2F,INVALC,INVALC,INVALC,INVALC,INVALC,0x4530,
	0x4A38,INVALC,0x4A33,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A39,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4A32,0x4A37,INVALC,
	0x672A,0x4A31,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4F42,0x4F50,INVALC,INVALC,INVALC,INVALC,0x4F43,0x4F41,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F48,INVALC,0x4F49,INVALC,
	INVALC,INVALC,0x4F4B,0x4F4E,0x4F44,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F51,INVALC,INVALC,0x4F45,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4F52,INVALC,0x4F4C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4F4D,0x665B,0x665C,0x4A30,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5335,INVALC,INVALC,
	/* Block 148, Array index 0x4600 */
	INVALC,INVALC,INVALC,0x533D,INVALC,0x533E,INVALC,0x5338,
	INVALC,INVALC,0x5332,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5336,
	INVALC,INVALC,INVALC,INVALC,0x5337,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x533B,INVALC,INVALC,INVALC,
	0x533C,0x5339,0x533A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x665D,
	INVALC,INVALC,0x665E,INVALC,INVALC,0x5331,INVALC,INVALC,
	0x5665,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5666,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x565D,INVALC,0x5667,
	INVALC,INVALC,0x5664,INVALC,INVALC,INVALC,0x565B,INVALC,
	0x5668,0x565E,INVALC,0x566B,INVALC,INVALC,0x5661,INVALC,
	INVALC,INVALC,0x5662,0x5663,0x565F,INVALC,INVALC,INVALC,
	0x565C,INVALC,INVALC,INVALC,INVALC,0x5660,INVALC,0x566A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x665F,INVALC,
	INVALC,INVALC,INVALC,0x5943,INVALC,INVALC,INVALC,INVALC,
	0x5940,INVALC,INVALC,0x5942,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5949,INVALC,INVALC,INVALC,
	0x593E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5944,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5946,INVALC,INVALC,0x5945,0x6661,0x6662,0x6660,INVALC,
	INVALC,0x5B50,INVALC,INVALC,INVALC,0x5B44,0x5B45,INVALC,
	INVALC,INVALC,INVALC,0x5B43,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5B48,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5B4D,INVALC,INVALC,INVALC,INVALC,
	0x5B4F,INVALC,INVALC,INVALC,INVALC,0x5B4A,INVALC,INVALC,
	INVALC,0x5B41,0x5B42,0x5B4E,INVALC,0x5B47,INVALC,0x6665,
	INVALC,0x6663,0x6664,0x5D41,0x5E6A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 149, Array index 0x4700 */
	INVALC,INVALC,0x5D43,INVALC,0x5D3C,0x5D3F,INVALC,INVALC,
	0x5D3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D3D,
	INVALC,INVALC,0x5D44,0x5D45,0x5D47,INVALC,0x5D42,0x5D3A,
	INVALC,INVALC,0x6666,0x6667,0x6668,INVALC,INVALC,0x5D46,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E6D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5F69,0x5F6E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F6A,0x5F6C,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F6B,INVALC,0x6052,INVALC,
	0x6431,INVALC,0x6055,0x6054,0x6056,INVALC,INVALC,0x604F,
	INVALC,0x6053,INVALC,INVALC,INVALC,0x6057,0x666A,0x6669,
	INVALC,INVALC,INVALC,INVALC,0x6122,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x616F,0x6223,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6236,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 150, Array index 0x4800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6323,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2F40,INVALC,INVALC,INVALC,0x3432,INVALC,0x395C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F6C,INVALC,0x666B,
	INVALC,0x453C,0x453A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A3D,INVALC,INVALC,INVALC,INVALC,0x4A3E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F53,
	INVALC,INVALC,0x4F55,0x4F56,INVALC,INVALC,INVALC,INVALC,
	0x5342,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5344,0x5340,INVALC,0x566D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x666C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x594B,
	0x594A,INVALC,INVALC,INVALC,INVALC,0x5B53,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x666D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 151, Array index 0x4900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2143,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x273B,0x273A,
	INVALC,0x273F,INVALC,0x273C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x273D,0x273E,INVALC,INVALC,0x2A79,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2A7C,INVALC,
	INVALC,0x2A7B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3436,INVALC,
	INVALC,0x3434,0x395E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3433,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3961,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3965,INVALC,0x3F71,INVALC,INVALC,INVALC,0x3F72,
	INVALC,0x3F73,0x3F75,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F70,INVALC,INVALC,0x4540,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4542,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4A45,INVALC,0x4A46,
	0x4A44,INVALC,INVALC,0x4F5A,INVALC,0x4F5B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5348,INVALC,0x5670,INVALC,0x566F,
	INVALC,INVALC,0x5671,INVALC,INVALC,0x5D49,0x2A7D,0x5349,
	INVALC,INVALC,0x6325,INVALC,INVALC,0x3439,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A47,0x4A49,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E70,INVALC,0x6125,0x622F,
	INVALC,INVALC,INVALC,INVALC,0x3F7E,0x3F7C,0x3F7B,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4546,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4547,INVALC,INVALC,INVALC,
	/* Block 152, Array index 0x4A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F60,INVALC,0x5350,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x534D,0x534C,INVALC,0x5351,
	INVALC,INVALC,0x5674,0x5675,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D4F,INVALC,0x5D4C,0x5D4E,INVALC,0x5E72,0x5E71,
	INVALC,INVALC,INVALC,0x666E,0x5F71,INVALC,INVALC,INVALC,
	0x5F72,0x6059,INVALC,INVALC,INVALC,0x6126,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x666F,INVALC,INVALC,
	0x6245,0x2A7E,INVALC,INVALC,INVALC,0x4549,INVALC,INVALC,
	INVALC,0x4A4C,INVALC,INVALC,INVALC,0x594F,INVALC,0x4021,
	INVALC,INVALC,INVALC,0x2B21,0x4A4E,INVALC,INVALC,0x5E73,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4024,INVALC,0x4022,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x454B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F62,
	INVALC,0x4F63,INVALC,INVALC,0x4F64,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5353,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x567A,INVALC,INVALC,INVALC,0x567B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5950,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5B5F,0x5B5D,INVALC,0x5B5E,0x5B60,INVALC,INVALC,
	0x5D50,INVALC,0x5D51,INVALC,INVALC,0x5E75,0x5E74,INVALC,
	0x5F73,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6128,INVALC,0x623B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5355,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5951,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5E76,INVALC,0x6129,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x454F,0x4F65,
	INVALC,INVALC,0x5B61,INVALC,INVALC,0x4550,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D53,INVALC,INVALC,INVALC,
	/* Block 153, Array index 0x4B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4552,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4A54,0x4A55,INVALC,INVALC,
	INVALC,0x4A52,0x4A56,INVALC,INVALC,INVALC,INVALC,0x4F69,
	INVALC,INVALC,0x4F67,0x636E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F66,INVALC,INVALC,INVALC,0x535C,INVALC,
	INVALC,0x5359,INVALC,0x535A,0x5228,INVALC,0x5358,INVALC,
	INVALC,INVALC,0x535D,INVALC,0x535B,0x535F,INVALC,0x595B,
	0x5726,INVALC,0x637B,INVALC,INVALC,INVALC,INVALC,0x5728,
	INVALC,INVALC,INVALC,0x5959,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5956,0x5958,0x5B65,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x642A,INVALC,INVALC,INVALC,INVALC,0x5E78,INVALC,
	0x5E77,INVALC,INVALC,INVALC,0x605D,0x605E,0x605C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5361,INVALC,0x572A,
	INVALC,0x595C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D56,INVALC,INVALC,INVALC,0x5E7B,
	0x5E79,INVALC,0x5E7A,INVALC,INVALC,0x6225,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E7C,0x6226,INVALC,INVALC,
	INVALC,0x3967,INVALC,INVALC,0x6670,INVALC,0x3968,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x455B,INVALC,INVALC,INVALC,
	0x402B,0x455C,INVALC,0x4554,INVALC,0x4559,INVALC,0x4557,
	0x4A5D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4558,
	/* Block 154, Array index 0x4C00 */
	INVALC,0x4A5E,INVALC,INVALC,INVALC,INVALC,0x4A5B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x667A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F6B,INVALC,INVALC,0x5365,0x5366,INVALC,INVALC,
	0x5364,INVALC,0x5362,0x5363,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5962,INVALC,
	0x5961,0x5966,INVALC,0x5965,INVALC,INVALC,INVALC,INVALC,
	0x5963,INVALC,0x6671,INVALC,0x5B69,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5D58,INVALC,0x5E7E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5F78,INVALC,INVALC,0x612A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x455F,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5368,INVALC,0x5730,INVALC,0x5967,INVALC,INVALC,INVALC,
	INVALC,0x5F21,0x6060,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A63,0x4A62,
	0x4A66,INVALC,INVALC,INVALC,0x4A61,INVALC,0x4A60,0x4A6C,
	0x4A64,INVALC,INVALC,INVALC,0x4A6A,INVALC,INVALC,INVALC,
	0x4F73,INVALC,0x4F6D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F6E,INVALC,INVALC,INVALC,0x4F72,INVALC,
	0x4F74,0x5370,INVALC,INVALC,INVALC,INVALC,0x5371,INVALC,
	0x536B,INVALC,INVALC,0x536C,INVALC,INVALC,INVALC,0x536A,
	INVALC,INVALC,INVALC,0x5734,INVALC,0x5737,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 155, Array index 0x4D00 */
	0x5733,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x596A,INVALC,INVALC,INVALC,0x596B,INVALC,INVALC,INVALC,
	0x596D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B6F,
	0x5B6D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B6C,
	INVALC,0x5B6E,INVALC,INVALC,INVALC,INVALC,0x5B6B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D5A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5F24,0x5F22,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F79,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6062,INVALC,INVALC,INVALC,0x6061,INVALC,INVALC,INVALC,
	INVALC,0x6172,INVALC,0x6173,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x402F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4F77,0x4F76,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x573A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D61,INVALC,INVALC,INVALC,
	0x5D60,0x5D5E,INVALC,INVALC,INVALC,INVALC,0x5F25,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6174,
	INVALC,0x6334,0x4A6E,INVALC,INVALC,0x5F7B,0x6063,INVALC,
	0x4032,INVALC,0x4561,INVALC,0x4A6F,0x4A73,INVALC,INVALC,
	0x4A71,0x4A70,0x4A72,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F79,0x5375,INVALC,INVALC,
	0x5374,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 156, Array index 0x4E00 */
	0x573B,INVALC,0x573C,INVALC,INVALC,INVALC,INVALC,0x596F,
	INVALC,0x5B71,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F27,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6672,INVALC,INVALC,
	INVALC,0x612C,INVALC,INVALC,INVALC,INVALC,0x4A75,INVALC,
	INVALC,INVALC,0x5D62,INVALC,0x612E,0x6433,INVALC,INVALC,
	0x6228,INVALC,INVALC,INVALC,0x573E,INVALC,INVALC,INVALC,
	0x5D63,INVALC,INVALC,INVALC,INVALC,0x4562,INVALC,INVALC,
	0x4A78,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x573F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5D64,INVALC,INVALC,0x5F28,INVALC,INVALC,INVALC,0x612F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4563,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4F7E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F7C,INVALC,0x4F7B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F7D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5378,0x537A,INVALC,INVALC,INVALC,
	INVALC,0x5423,INVALC,0x537C,0x5421,INVALC,0x5425,0x537D,
	INVALC,INVALC,INVALC,INVALC,0x5379,INVALC,INVALC,INVALC,
	0x6673,0x5744,INVALC,INVALC,INVALC,0x5745,INVALC,0x5740,
	INVALC,INVALC,INVALC,0x5742,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5974,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5973,INVALC,INVALC,0x5B74,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5976,0x5975,0x5743,INVALC,INVALC,
	INVALC,INVALC,0x5971,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B75,
	INVALC,INVALC,INVALC,0x5B77,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5B79,INVALC,INVALC,INVALC,INVALC,0x5B7A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D6F,INVALC,
	INVALC,0x5D71,INVALC,0x5D67,0x5D69,INVALC,0x5D6A,0x5D74,
	/* Block 157, Array index 0x4F00 */
	0x5D75,INVALC,0x5D6E,INVALC,0x5D68,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5D66,INVALC,0x5D76,INVALC,INVALC,INVALC,
	0x5D73,INVALC,INVALC,INVALC,INVALC,0x5D72,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D70,INVALC,INVALC,INVALC,0x5F2C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x642F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F2A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6025,INVALC,INVALC,
	0x5F7E,INVALC,0x6023,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6022,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6065,
	INVALC,0x6066,INVALC,0x6068,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6132,INVALC,INVALC,
	INVALC,0x6133,INVALC,INVALC,INVALC,0x6134,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6159,INVALC,INVALC,0x615B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6175,INVALC,
	INVALC,INVALC,INVALC,0x6241,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4564,INVALC,INVALC,0x4566,0x2F42,INVALC,0x4568,0x4567,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5027,INVALC,INVALC,INVALC,INVALC,INVALC,0x502A,INVALC,
	/* Block 158, Array index 0x5000 */
	INVALC,0x5024,0x502B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x542D,INVALC,0x5025,0x5022,INVALC,0x542A,
	INVALC,INVALC,INVALC,0x542C,INVALC,INVALC,0x5436,INVALC,
	INVALC,INVALC,0x5433,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5437,INVALC,INVALC,0x5429,INVALC,INVALC,0x5435,
	INVALC,INVALC,0x5432,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x574D,INVALC,INVALC,
	INVALC,0x5748,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x574A,INVALC,INVALC,INVALC,
	INVALC,0x6675,INVALC,INVALC,INVALC,0x5979,0x597B,INVALC,
	0x597D,INVALC,INVALC,INVALC,INVALC,0x597E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A22,INVALC,
	INVALC,INVALC,INVALC,0x597A,INVALC,INVALC,0x5B7D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5C27,0x5C22,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C25,INVALC,
	INVALC,INVALC,0x5B7E,INVALC,INVALC,INVALC,0x5C26,INVALC,
	INVALC,INVALC,INVALC,0x5C24,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C23,INVALC,INVALC,INVALC,0x5D79,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5D7A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F35,INVALC,
	0x5F31,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F32,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F36,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x606C,INVALC,INVALC,INVALC,
	/* Block 159, Array index 0x5100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6135,INVALC,INVALC,INVALC,INVALC,INVALC,0x615D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6229,INVALC,
	0x6230,INVALC,INVALC,INVALC,0x6238,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5F37,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4569,INVALC,INVALC,0x502C,0x5439,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A24,INVALC,INVALC,INVALC,INVALC,0x5C29,INVALC,INVALC,
	0x5E21,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F3A,INVALC,
	INVALC,INVALC,0x6231,0x623F,INVALC,INVALC,0x2547,INVALC,
	INVALC,INVALC,0x502F,0x502E,0x543A,INVALC,INVALC,0x5750,
	INVALC,0x5A26,0x5A25,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B22,INVALC,INVALC,
	INVALC,0x5E22,INVALC,INVALC,0x396A,0x543C,0x543B,INVALC,
	INVALC,INVALC,0x5A28,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x396B,INVALC,INVALC,INVALC,INVALC,0x543D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5C2A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6029,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2772,INVALC,
	/* Block 160, Array index 0x5200 */
	INVALC,INVALC,0x5A29,0x5C2D,0x5C2E,0x5F3C,INVALC,INVALC,
	0x615F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x456A,INVALC,INVALC,0x5C30,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6160,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A2B,0x5A2A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x606D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x613B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2370,INVALC,INVALC,0x543E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5E24,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6233,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5755,INVALC,
	INVALC,INVALC,INVALC,0x602B,INVALC,INVALC,INVALC,0x6242,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E47,INVALC,INVALC,
	INVALC,0x5F3F,INVALC,0x6161,INVALC,0x6177,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
};

#endif /* ICONV_FROM_UCS_CCS_CNS11643_PLANE14 && !defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit UCS -> cns11643_plane14 size-optimized table (16576 bytes).
 * ======================================================================
 */

#if defined (ICONV_FROM_UCS_CCS_CNS11643_PLANE14) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
from_ucs_size_cns11643_plane14[] =
{
	0x00C5, /* Ranges number */
	0x0D04, /* Unranged codes number */
	0x0658, /* First unranged code index */
	/* Ranges list: first code, last Code, array index. */
	/* Array index: 0x0003 */ 0x4E20, 0x4E24, 0x0252,
	/* Array index: 0x0006 */ 0x4EAF, 0x4EB2, 0x0257,
	/* Array index: 0x0009 */ 0x4EF8, 0x4EFC, 0x025B,
	/* Array index: 0x000C */ 0x4F9F, 0x4FA3, 0x0260,
	/* Array index: 0x000F */ 0x4FF9, 0x5004, 0x0265,
	/* Array index: 0x0012 */ 0x5078, 0x507C, 0x0271,
	/* Array index: 0x0015 */ 0x50DE, 0x50E2, 0x0276,
	/* Array index: 0x0018 */ 0x510D, 0x510F, 0x027B,
	/* Array index: 0x001B */ 0x5182, 0x5186, 0x027E,
	/* Array index: 0x001E */ 0x51B2, 0x51B5, 0x0283,
	/* Array index: 0x0021 */ 0x51C1, 0x51C3, 0x0287,
	/* Array index: 0x0024 */ 0x51CF, 0x51D3, 0x028A,
	/* Array index: 0x0027 */ 0x51D9, 0x51DB, 0x028F,
	/* Array index: 0x002A */ 0x51E2, 0x51E5, 0x0292,
	/* Array index: 0x002D */ 0x5263, 0x5268, 0x0296,
	/* Array index: 0x0030 */ 0x529A, 0x529E, 0x029C,
	/* Array index: 0x0033 */ 0x52B3, 0x52B5, 0x02A1,
	/* Array index: 0x0036 */ 0x52CC, 0x52D1, 0x02A4,
	/* Array index: 0x0039 */ 0x5332, 0x533B, 0x02AA,
	/* Array index: 0x003C */ 0x53A6, 0x53AB, 0x02B4,
	/* Array index: 0x003F */ 0x53C1, 0x53C5, 0x02BA,
	/* Array index: 0x0042 */ 0x53D9, 0x53DD, 0x02BF,
	/* Array index: 0x0045 */ 0x5421, 0x5423, 0x02C4,
	/* Array index: 0x0048 */ 0x5449, 0x5451, 0x02C7,
	/* Array index: 0x004B */ 0x54CA, 0x54CD, 0x02D0,
	/* Array index: 0x004E */ 0x54F4, 0x54F6, 0x02D4,
	/* Array index: 0x0051 */ 0x5558, 0x555A, 0x02D7,
	/* Array index: 0x0054 */ 0x5571, 0x5573, 0x02DA,
	/* Array index: 0x0057 */ 0x564D, 0x5656, 0x02DD,
	/* Array index: 0x005A */ 0x569E, 0x56A0, 0x02E7,
	/* Array index: 0x005D */ 0x56B8, 0x56BB, 0x02EA,
	/* Array index: 0x0060 */ 0x56EF, 0x56F3, 0x02EE,
	/* Array index: 0x0063 */ 0x56FB, 0x56FD, 0x02F3,
	/* Array index: 0x0066 */ 0x5742, 0x5748, 0x02F6,
	/* Array index: 0x0069 */ 0x5753, 0x5759, 0x02FD,
	/* Array index: 0x006C */ 0x578D, 0x5792, 0x0304,
	/* Array index: 0x006F */ 0x57A6, 0x57A8, 0x030A,
	/* Array index: 0x0072 */ 0x57C4, 0x57CA, 0x030D,
	/* Array index: 0x0075 */ 0x585F, 0x5861, 0x0314,
	/* Array index: 0x0078 */ 0x5866, 0x586C, 0x0317,
	/* Array index: 0x007B */ 0x5895, 0x5897, 0x031E,
	/* Array index: 0x007E */ 0x58EE, 0x58F7, 0x0321,
	/* Array index: 0x0081 */ 0x5900, 0x590B, 0x032B,
	/* Array index: 0x0084 */ 0x594B, 0x594D, 0x0337,
	/* Array index: 0x0087 */ 0x599A, 0x599C, 0x033A,
	/* Array index: 0x008A */ 0x5A2A, 0x5A2C, 0x033D,
	/* Array index: 0x008D */ 0x5A7E, 0x5A89, 0x0340,
	/* Array index: 0x0090 */ 0x5AA0, 0x5AA4, 0x034C,
	/* Array index: 0x0093 */ 0x5ACE, 0x5AD3, 0x0351,
	/* Array index: 0x0096 */ 0x5B37, 0x5B3B, 0x0357,
	/* Array index: 0x0099 */ 0x5B9D, 0x5B9F, 0x035C,
	/* Array index: 0x009C */ 0x5BDA, 0x5BDD, 0x035F,
	/* Array index: 0x009F */ 0x5BFD, 0x5C00, 0x0363,
	/* Array index: 0x00A2 */ 0x5CB9, 0x5CBB, 0x0367,
	/* Array index: 0x00A5 */ 0x5D08, 0x5D0A, 0x036A,
	/* Array index: 0x00A8 */ 0x5D8B, 0x5D91, 0x036D,
	/* Array index: 0x00AB */ 0x5DA3, 0x5DA6, 0x0374,
	/* Array index: 0x00AE */ 0x5E2C, 0x5E32, 0x0378,
	/* Array index: 0x00B1 */ 0x5E46, 0x5E48, 0x037F,
	/* Array index: 0x00B4 */ 0x5EF8, 0x5EFD, 0x0382,
	/* Array index: 0x00B7 */ 0x5F0C, 0x5F0E, 0x0388,
	/* Array index: 0x00BA */ 0x5F39, 0x5F3B, 0x038B,
	/* Array index: 0x00BD */ 0x5F50, 0x5F55, 0x038E,
	/* Array index: 0x00C0 */ 0x5F8D, 0x5F8F, 0x0394,
	/* Array index: 0x00C3 */ 0x5FDA, 0x5FDC, 0x0397,
	/* Array index: 0x00C6 */ 0x604E, 0x6052, 0x039A,
	/* Array index: 0x00C9 */ 0x6073, 0x6075, 0x039F,
	/* Array index: 0x00CC */ 0x60A6, 0x60AA, 0x03A2,
	/* Array index: 0x00CF */ 0x611E, 0x6122, 0x03A7,
	/* Array index: 0x00D2 */ 0x6185, 0x6188, 0x03AC,
	/* Array index: 0x00D5 */ 0x6195, 0x6199, 0x03B0,
	/* Array index: 0x00D8 */ 0x61CE, 0x61D0, 0x03B5,
	/* Array index: 0x00DB */ 0x61D9, 0x61DD, 0x03B8,
	/* Array index: 0x00DE */ 0x6235, 0x623C, 0x03BD,
	/* Array index: 0x00E1 */ 0x6299, 0x629E, 0x03C5,
	/* Array index: 0x00E4 */ 0x62A3, 0x62A7, 0x03CB,
	/* Array index: 0x00E7 */ 0x62DD, 0x62E1, 0x03D0,
	/* Array index: 0x00EA */ 0x635B, 0x635D, 0x03D5,
	/* Array index: 0x00ED */ 0x6438, 0x643C, 0x03D8,
	/* Array index: 0x00F0 */ 0x6455, 0x6457, 0x03DD,
	/* Array index: 0x00F3 */ 0x655F, 0x6561, 0x03E0,
	/* Array index: 0x00F6 */ 0x656D, 0x6571, 0x03E3,
	/* Array index: 0x00F9 */ 0x6617, 0x661B, 0x03E8,
	/* Array index: 0x00FC */ 0x66A4, 0x66A6, 0x03ED,
	/* Array index: 0x00FF */ 0x66CD, 0x66D7, 0x03F0,
	/* Array index: 0x0102 */ 0x67A2, 0x67A4, 0x03FB,
	/* Array index: 0x0105 */ 0x67D5, 0x67D7, 0x03FE,
	/* Array index: 0x0108 */ 0x6822, 0x6824, 0x0401,
	/* Array index: 0x010B */ 0x6855, 0x685A, 0x0404,
	/* Array index: 0x010E */ 0x6898, 0x689A, 0x040A,
	/* Array index: 0x0111 */ 0x68B6, 0x68B9, 0x040D,
	/* Array index: 0x0114 */ 0x68FE, 0x6903, 0x0411,
	/* Array index: 0x0117 */ 0x6A29, 0x6A2D, 0x0417,
	/* Array index: 0x011A */ 0x6AB0, 0x6AB2, 0x041C,
	/* Array index: 0x011D */ 0x6ABC, 0x6AC1, 0x041F,
	/* Array index: 0x0120 */ 0x6AD4, 0x6AD8, 0x0425,
	/* Array index: 0x0123 */ 0x6AFD, 0x6B01, 0x042A,
	/* Array index: 0x0126 */ 0x6B1B, 0x6B1D, 0x042F,
	/* Array index: 0x0129 */ 0x6B26, 0x6B2B, 0x0432,
	/* Array index: 0x012C */ 0x6B68, 0x6B74, 0x0438,
	/* Array index: 0x012F */ 0x6C9E, 0x6CA2, 0x0445,
	/* Array index: 0x0132 */ 0x6D13, 0x6D15, 0x044A,
	/* Array index: 0x0135 */ 0x6D5B, 0x6D5D, 0x044D,
	/* Array index: 0x0138 */ 0x6D71, 0x6D73, 0x0450,
	/* Array index: 0x013B */ 0x6D99, 0x6D9B, 0x0453,
	/* Array index: 0x013E */ 0x6DFE, 0x6E0C, 0x0456,
	/* Array index: 0x0141 */ 0x6ED9, 0x6EDD, 0x0465,
	/* Array index: 0x0144 */ 0x6F48, 0x6F4C, 0x046A,
	/* Array index: 0x0147 */ 0x6FD6, 0x6FDA, 0x046F,
	/* Array index: 0x014A */ 0x702C, 0x702E, 0x0474,
	/* Array index: 0x014D */ 0x7087, 0x7090, 0x0477,
	/* Array index: 0x0150 */ 0x70A2, 0x70AA, 0x0481,
	/* Array index: 0x0153 */ 0x712A, 0x712D, 0x048A,
	/* Array index: 0x0156 */ 0x7133, 0x713F, 0x048E,
	/* Array index: 0x0159 */ 0x7174, 0x7177, 0x049B,
	/* Array index: 0x015C */ 0x718B, 0x718E, 0x049F,
	/* Array index: 0x015F */ 0x71E9, 0x71EB, 0x04A3,
	/* Array index: 0x0162 */ 0x720E, 0x7218, 0x04A6,
	/* Array index: 0x0165 */ 0x7264, 0x7268, 0x04B1,
	/* Array index: 0x0168 */ 0x7328, 0x732F, 0x04B6,
	/* Array index: 0x016B */ 0x7346, 0x7348, 0x04BE,
	/* Array index: 0x016E */ 0x7353, 0x7355, 0x04C1,
	/* Array index: 0x0171 */ 0x73CE, 0x73D0, 0x04C4,
	/* Array index: 0x0174 */ 0x7414, 0x7419, 0x04C7,
	/* Array index: 0x0177 */ 0x7438, 0x743E, 0x04CD,
	/* Array index: 0x017A */ 0x7445, 0x7449, 0x04D4,
	/* Array index: 0x017D */ 0x74C7, 0x74D3, 0x04D9,
	/* Array index: 0x0180 */ 0x7555, 0x7558, 0x04E6,
	/* Array index: 0x0183 */ 0x76D5, 0x76D9, 0x04EA,
	/* Array index: 0x0186 */ 0x773F, 0x7743, 0x04EF,
	/* Array index: 0x0189 */ 0x7774, 0x777B, 0x04F4,
	/* Array index: 0x018C */ 0x7807, 0x780B, 0x04FC,
	/* Array index: 0x018F */ 0x7906, 0x790D, 0x0501,
	/* Array index: 0x0192 */ 0x7983, 0x7989, 0x0509,
	/* Array index: 0x0195 */ 0x799D, 0x799F, 0x0510,
	/* Array index: 0x0198 */ 0x79C3, 0x79C7, 0x0513,
	/* Array index: 0x019B */ 0x79D7, 0x79DB, 0x0518,
	/* Array index: 0x019E */ 0x79F0, 0x79F5, 0x051D,
	/* Array index: 0x01A1 */ 0x7A29, 0x7A2D, 0x0523,
	/* Array index: 0x01A4 */ 0x7ACC, 0x7AD5, 0x0528,
	/* Array index: 0x01A7 */ 0x7ADA, 0x7ADD, 0x0532,
	/* Array index: 0x01AA */ 0x7AF0, 0x7AF5, 0x0536,
	/* Array index: 0x01AD */ 0x7B41, 0x7B43, 0x053C,
	/* Array index: 0x01B0 */ 0x7C99, 0x7C9C, 0x053F,
	/* Array index: 0x01B3 */ 0x7CC2, 0x7CC9, 0x0543,
	/* Array index: 0x01B6 */ 0x7D74, 0x7D78, 0x054B,
	/* Array index: 0x01B9 */ 0x7D87, 0x7D8B, 0x0550,
	/* Array index: 0x01BC */ 0x7D95, 0x7D99, 0x0555,
	/* Array index: 0x01BF */ 0x7DCD, 0x7DD3, 0x055A,
	/* Array index: 0x01C2 */ 0x7DFC, 0x7E02, 0x0561,
	/* Array index: 0x01C5 */ 0x805F, 0x806B, 0x0568,
	/* Array index: 0x01C8 */ 0x807A, 0x807C, 0x0575,
	/* Array index: 0x01CB */ 0x80A4, 0x80A8, 0x0578,
	/* Array index: 0x01CE */ 0x8140, 0x8142, 0x057D,
	/* Array index: 0x01D1 */ 0x8192, 0x8196, 0x0580,
	/* Array index: 0x01D4 */ 0x8218, 0x821A, 0x0585,
	/* Array index: 0x01D7 */ 0x8414, 0x8418, 0x0588,
	/* Array index: 0x01DA */ 0x8492, 0x8496, 0x058D,
	/* Array index: 0x01DD */ 0x8501, 0x8505, 0x0592,
	/* Array index: 0x01E0 */ 0x8533, 0x8535, 0x0597,
	/* Array index: 0x01E3 */ 0x8845, 0x8847, 0x059A,
	/* Array index: 0x01E6 */ 0x8A9B, 0x8A9F, 0x059D,
	/* Array index: 0x01E9 */ 0x8AB1, 0x8AB7, 0x05A2,
	/* Array index: 0x01EC */ 0x8C51, 0x8C53, 0x05A9,
	/* Array index: 0x01EF */ 0x8D82, 0x8D88, 0x05AC,
	/* Array index: 0x01F2 */ 0x8EAD, 0x8EB9, 0x05B3,
	/* Array index: 0x01F5 */ 0x8F9D, 0x8FA7, 0x05C0,
	/* Array index: 0x01F8 */ 0x9028, 0x902C, 0x05CB,
	/* Array index: 0x01FB */ 0x91DE, 0x91E0, 0x05D0,
	/* Array index: 0x01FE */ 0x9220, 0x9222, 0x05D3,
	/* Array index: 0x0201 */ 0x92AD, 0x92B1, 0x05D6,
	/* Array index: 0x0204 */ 0x92DA, 0x92DC, 0x05DB,
	/* Array index: 0x0207 */ 0x92F3, 0x92F6, 0x05DE,
	/* Array index: 0x020A */ 0x9330, 0x9332, 0x05E2,
	/* Array index: 0x020D */ 0x9372, 0x9374, 0x05E5,
	/* Array index: 0x0210 */ 0x93BB, 0x93BE, 0x05E8,
	/* Array index: 0x0213 */ 0x93E9, 0x93F4, 0x05EC,
	/* Array index: 0x0216 */ 0x941A, 0x9424, 0x05F8,
	/* Array index: 0x0219 */ 0x9458, 0x945C, 0x0603,
	/* Array index: 0x021C */ 0x9465, 0x9467, 0x0608,
	/* Array index: 0x021F */ 0x96B5, 0x96B7, 0x060B,
	/* Array index: 0x0222 */ 0x96EC, 0x96EE, 0x060E,
	/* Array index: 0x0225 */ 0x9733, 0x9737, 0x0611,
	/* Array index: 0x0228 */ 0x97B1, 0x97B5, 0x0616,
	/* Array index: 0x022B */ 0x983C, 0x9842, 0x061B,
	/* Array index: 0x022E */ 0x9854, 0x9856, 0x0622,
	/* Array index: 0x0231 */ 0x986C, 0x986E, 0x0625,
	/* Array index: 0x0234 */ 0x99B6, 0x99B8, 0x0628,
	/* Array index: 0x0237 */ 0x99BC, 0x99C0, 0x062B,
	/* Array index: 0x023A */ 0x9AE4, 0x9AEA, 0x0630,
	/* Array index: 0x023D */ 0x9B8B, 0x9B8F, 0x0637,
	/* Array index: 0x0240 */ 0x9BCB, 0x9BCD, 0x063C,
	/* Array index: 0x0243 */ 0x9BFB, 0x9C04, 0x063F,
	/* Array index: 0x0246 */ 0x9CEB, 0x9CEF, 0x0649,
	/* Array index: 0x0249 */ 0x9EAA, 0x9EAC, 0x064E,
	/* Array index: 0x024C */ 0x9EC4, 0x9EC6, 0x0651,
	/* Array index: 0x024F */ 0x9F02, 0x9F05, 0x0654,
	/* Ranges content */
	/* Range 0x4E20 - 0x4E24, array index: 0x0252 */
	0x2262,0x2261,0x2263,INVALC,0x243F,
	/* Range 0x4EAF - 0x4EB2, array index: 0x0257 */
	0x2B26,0x2B25,0x2B27,0x2B24,
	/* Range 0x4EF8 - 0x4EFC, array index: 0x025B */
	0x226B,0x2267,0x2276,INVALC,0x226E,
	/* Range 0x4F9F - 0x4FA3, array index: 0x0260 */
	0x274C,0x2749,0x274B,0x274A,0x274D,
	/* Range 0x4FF9 - 0x5004, array index: 0x0265 */
	0x2F44,INVALC,0x2F58,0x2F4C,0x2F56,INVALC,0x2F46,INVALC,
	INVALC,0x2F4B,0x2F49,0x2F47,
	/* Range 0x5078 - 0x507C, array index: 0x0271 */
	0x344C,0x344A,INVALC,0x3445,0x3446,
	/* Range 0x50DE - 0x50E2, array index: 0x0276 */
	0x4576,0x634D,INVALC,0x4570,0x4575,
	/* Range 0x510D - 0x510F, array index: 0x027B */
	0x4B2C,0x4B27,0x644E,
	/* Range 0x5182 - 0x5186, array index: 0x027E */
	0x2127,INVALC,0x214D,0x2148,0x214E,
	/* Range 0x51B2 - 0x51B5, array index: 0x0283 */
	0x227E,0x227D,0x227C,0x2459,
	/* Range 0x51C1 - 0x51C3, array index: 0x0287 */
	0x2B3B,0x2B3D,0x2B3E,
	/* Range 0x51CF - 0x51D3, array index: 0x028A */
	0x344F,INVALC,0x344E,0x397D,0x397A,
	/* Range 0x51D9 - 0x51DB, array index: 0x028F */
	0x4B2F,0x4B2E,0x4B30,
	/* Range 0x51E2 - 0x51E5, array index: 0x0292 */
	0x212F,0x2135,0x2151,0x217B,
	/* Range 0x5263 - 0x5268, array index: 0x0296 */
	0x2F66,0x2F61,0x2F63,INVALC,0x2F64,0x3450,
	/* Range 0x529A - 0x529E, array index: 0x029C */
	0x602C,INVALC,0x2137,0x2154,0x2153,
	/* Range 0x52B3 - 0x52B5, array index: 0x02A1 */
	0x2463,0x2462,0x2761,
	/* Range 0x52CC - 0x52D1, array index: 0x02A4 */
	0x2F6F,INVALC,0x2F6C,0x2F6A,0x2F6D,0x2F6E,
	/* Range 0x5332 - 0x533B, array index: 0x02AA */
	0x4623,0x4B34,INVALC,0x5444,0x5C34,INVALC,0x2129,INVALC,
	0x2157,0x2466,
	/* Range 0x53A6 - 0x53AB, array index: 0x02B4 */
	0x3A28,INVALC,0x3A27,0x404A,0x4049,0x4048,
	/* Range 0x53C1 - 0x53C5, array index: 0x02BA */
	0x2769,0x276A,INVALC,0x3455,0x3A29,
	/* Range 0x53D9 - 0x53DD, array index: 0x02BF */
	0x2B50,0x2B4F,INVALC,0x2B4E,0x2B4D,
	/* Range 0x5421 - 0x5423, array index: 0x02C4 */
	0x2479,0x247E,0x2475,
	/* Range 0x5449 - 0x5451, array index: 0x02C7 */
	0x247C,INVALC,0x2478,0x2474,0x246D,INVALC,INVALC,0x247D,
	0x2522,
	/* Range 0x54CA - 0x54CD, array index: 0x02D0 */
	0x2B55,0x6454,0x2B63,0x2B61,
	/* Range 0x54F4 - 0x54F6, array index: 0x02D4 */
	0x2F74,0x2F78,0x3022,
	/* Range 0x5558 - 0x555A, array index: 0x02D7 */
	0x3457,0x3A36,0x3464,
	/* Range 0x5571 - 0x5573, array index: 0x02DA */
	0x6456,0x6722,0x6455,
	/* Range 0x564D - 0x5656, array index: 0x02DD */
	0x4B43,INVALC,0x4B45,0x4B40,0x4635,0x4B3C,INVALC,0x4B39,
	INVALC,0x4B3F,
	/* Range 0x569E - 0x56A0, array index: 0x02E7 */
	0x5759,0x5757,0x575C,
	/* Range 0x56B8 - 0x56BB, array index: 0x02EA */
	0x6726,0x6727,0x645C,0x5E26,
	/* Range 0x56EF - 0x56F3, array index: 0x02EE */
	0x2529,INVALC,0x2548,0x2527,0x2528,
	/* Range 0x56FB - 0x56FD, array index: 0x02F3 */
	0x277C,0x2821,0x277B,
	/* Range 0x5742 - 0x5748, array index: 0x02F6 */
	0x2540,0x2536,0x252F,INVALC,0x253C,INVALC,0x252D,
	/* Range 0x5753 - 0x5759, array index: 0x02FD */
	0x2532,0x2541,0x2542,0x2533,0x2538,0x2537,0x645F,
	/* Range 0x578D - 0x5792, array index: 0x0304 */
	0x2B69,0x2B68,INVALC,0x2B6C,0x2B67,0x2B6A,
	/* Range 0x57A6 - 0x57A8, array index: 0x030A */
	0x2B6D,0x2B72,0x2B6E,
	/* Range 0x57C4 - 0x57CA, array index: 0x030D */
	0x3034,0x3035,INVALC,INVALC,0x3033,0x3028,0x3037,
	/* Range 0x585F - 0x5861, array index: 0x0314 */
	0x405E,0x405C,0x4058,
	/* Range 0x5866 - 0x586C, array index: 0x0317 */
	0x405D,0x6343,INVALC,0x405A,0x4059,INVALC,0x4054,
	/* Range 0x5895 - 0x5897, array index: 0x031E */
	0x4641,0x4644,0x4640,
	/* Range 0x58EE - 0x58F7, array index: 0x0321 */
	0x2343,INVALC,0x2545,0x2543,0x2544,0x282D,INVALC,0x2B77,
	INVALC,0x3477,
	/* Range 0x5900 - 0x590B, array index: 0x032B */
	0x6466,0x504D,0x2139,INVALC,0x2235,0x2344,INVALC,INVALC,
	0x2B7A,0x2B79,0x213A,0x2546,
	/* Range 0x594B - 0x594D, array index: 0x0337 */
	0x282E,0x282F,0x6467,
	/* Range 0x599A - 0x599C, array index: 0x033A */
	0x2556,0x2553,0x254E,
	/* Range 0x5A2A - 0x5A2C, array index: 0x033D */
	0x303C,0x3522,0x3538,
	/* Range 0x5A7E - 0x5A89, array index: 0x0340 */
	0x3A7C,INVALC,0x3A63,0x3A61,0x3A58,INVALC,INVALC,0x3A5B,
	0x3A77,0x3A72,0x3A59,0x3A60,
	/* Range 0x5AA0 - 0x5AA4, array index: 0x034C */
	0x3A73,0x3A5A,INVALC,0x3A57,0x3A7A,
	/* Range 0x5ACE - 0x5AD3, array index: 0x0351 */
	0x4064,0x3A6B,0x406A,0x6468,INVALC,0x6469,
	/* Range 0x5B37 - 0x5B3B, array index: 0x0357 */
	0x5453,INVALC,0x5A31,0x5761,0x575E,
	/* Range 0x5B9D - 0x5B9F, array index: 0x035C */
	0x284E,0x284B,0x284D,
	/* Range 0x5BDA - 0x5BDD, array index: 0x035F */
	0x4075,0x4076,0x4073,0x6345,
	/* Range 0x5BFD - 0x5C00, array index: 0x0363 */
	0x255E,0x255C,0x255D,0x2850,
	/* Range 0x5CB9 - 0x5CBB, array index: 0x0367 */
	0x285C,0x285F,0x2857,
	/* Range 0x5D08 - 0x5D0A, array index: 0x036A */
	0x354B,0x3548,0x3547,
	/* Range 0x5D8B - 0x5D91, array index: 0x036D */
	0x4659,0x465A,INVALC,0x646F,0x4B65,0x4B66,0x4B63,
	/* Range 0x5DA3 - 0x5DA6, array index: 0x0374 */
	0x4B6A,0x4B68,0x6470,0x5058,
	/* Range 0x5E2C - 0x5E32, array index: 0x0378 */
	0x3069,INVALC,0x3068,0x306A,0x3067,INVALC,0x355B,
	/* Range 0x5E46 - 0x5E48, array index: 0x037F */
	0x3B3A,0x3B3B,0x3B3D,
	/* Range 0x5EF8 - 0x5EFD, array index: 0x0382 */
	0x2869,0x286B,INVALC,0x2C4C,0x2C4B,0x306E,
	/* Range 0x5F0C - 0x5F0E, array index: 0x0388 */
	0x2164,0x2249,0x2362,
	/* Range 0x5F39 - 0x5F3B, array index: 0x038B */
	0x3563,0x3B47,0x3B45,
	/* Range 0x5F50 - 0x5F55, array index: 0x038E */
	0x2140,0x2141,0x224A,0x2351,INVALC,0x2871,
	/* Range 0x5F8D - 0x5F8F, array index: 0x0394 */
	0x6264,0x3070,0x3072,
	/* Range 0x5FDA - 0x5FDC, array index: 0x0397 */
	0x2367,0x2369,0x2577,
	/* Range 0x604E - 0x6052, array index: 0x039A */
	0x2C56,0x3073,INVALC,0x2C57,0x2C5D,
	/* Range 0x6073 - 0x6075, array index: 0x039F */
	0x307A,0x3078,0x307B,
	/* Range 0x60A6 - 0x60AA, array index: 0x03A2 */
	0x3123,0x3128,INVALC,0x3121,0x356F,
	/* Range 0x611E - 0x6122, array index: 0x03A7 */
	0x3B61,INVALC,0x3B62,0x3B63,0x3B50,
	/* Range 0x6185 - 0x6188, array index: 0x03AC */
	0x4C23,0x4674,0x4C24,0x4B77,
	/* Range 0x6195 - 0x6199, array index: 0x03B0 */
	0x4B76,INVALC,0x5068,0x4C28,0x5067,
	/* Range 0x61CE - 0x61D0, array index: 0x03B5 */
	0x5062,0x5065,0x506B,
	/* Range 0x61D9 - 0x61DD, array index: 0x03B8 */
	0x5461,0x545F,0x545C,0x545E,0x545D,
	/* Range 0x6235 - 0x623C, array index: 0x03BD */
	0x5F48,INVALC,0x2165,0x2166,0x224E,INVALC,0x2628,0x2629,
	/* Range 0x6299 - 0x629E, array index: 0x03C5 */
	0x2635,INVALC,0x2633,0x2634,0x2638,0x2631,
	/* Range 0x62A3 - 0x62A7, array index: 0x03CB */
	0x647B,INVALC,0x2632,0x2931,0x292A,
	/* Range 0x62DD - 0x62E1, array index: 0x03D0 */
	0x2930,0x2937,0x2935,0x2936,0x292F,
	/* Range 0x635B - 0x635D, array index: 0x03D5 */
	0x3130,0x313B,0x3136,
	/* Range 0x6438 - 0x643C, array index: 0x03D8 */
	0x4134,INVALC,0x4148,0x4725,0x4142,
	/* Range 0x6455 - 0x6457, array index: 0x03DD */
	0x467E,0x4724,0x467A,
	/* Range 0x655F - 0x6561, array index: 0x03E0 */
	0x3B74,0x3B75,0x3B76,
	/* Range 0x656D - 0x6571, array index: 0x03E3 */
	0x414E,0x6346,INVALC,0x414D,0x4730,
	/* Range 0x6617 - 0x661B, array index: 0x03E8 */
	0x2940,0x2941,INVALC,0x2C7E,0x2C7A,
	/* Range 0x66A4 - 0x66A6, array index: 0x03ED */
	0x4739,0x4733,0x4737,
	/* Range 0x66CD - 0x66D7, array index: 0x03F0 */
	0x5122,0x5471,INVALC,0x546F,0x5470,INVALC,INVALC,0x546C,
	0x546D,INVALC,0x5472,
	/* Range 0x67A2 - 0x67A4, array index: 0x03FB */
	0x294E,0x294C,0x6528,
	/* Range 0x67D5 - 0x67D7, array index: 0x03FE */
	0x2D31,0x2D2B,0x2D3A,
	/* Range 0x6822 - 0x6824, array index: 0x0401 */
	0x315E,0x3164,0x3150,
	/* Range 0x6855 - 0x685A, array index: 0x0404 */
	0x3162,0x3158,INVALC,0x3161,0x3160,0x652D,
	/* Range 0x6898 - 0x689A, array index: 0x040A */
	0x364D,0x365C,0x3650,
	/* Range 0x68B6 - 0x68B9, array index: 0x040D */
	0x364C,0x3652,0x365E,0x365F,
	/* Range 0x68FE - 0x6903, array index: 0x0411 */
	0x3C31,0x3C42,0x3C39,0x3C3B,0x3C34,0x3C2F,
	/* Range 0x6A29 - 0x6A2D, array index: 0x0417 */
	0x4C5E,0x4C59,INVALC,0x4C61,0x6530,
	/* Range 0x6AB0 - 0x6AB2, array index: 0x041C */
	0x5777,0x5821,0x5776,
	/* Range 0x6ABC - 0x6AC1, array index: 0x041F */
	0x577E,INVALC,0x5779,0x5823,0x577D,0x577A,
	/* Range 0x6AD4 - 0x6AD8, array index: 0x0425 */
	0x5A45,0x5A4E,INVALC,0x5A47,0x5A4D,
	/* Range 0x6AFD - 0x6B01, array index: 0x042A */
	0x5E35,0x5E34,0x5E33,INVALC,0x5E37,
	/* Range 0x6B1B - 0x6B1D, array index: 0x042F */
	0x6140,0x6165,0x6141,
	/* Range 0x6B26 - 0x6B2B, array index: 0x0432 */
	0x2953,0x2954,INVALC,0x6269,0x2D3C,0x3166,
	/* Range 0x6B68 - 0x6B74, array index: 0x0438 */
	0x2955,0x2956,INVALC,0x2D3D,0x3168,INVALC,0x3C49,0x3C47,
	INVALC,0x417E,INVALC,0x4221,0x4753,
	/* Range 0x6C9E - 0x6CA2, array index: 0x0445 */
	0x264E,0x2652,0x265C,0x2659,0x2656,
	/* Range 0x6D13 - 0x6D15, array index: 0x044A */
	0x2D49,0x2D47,0x2D4D,
	/* Range 0x6D5B - 0x6D5D, array index: 0x044D */
	0x3179,0x3226,0x3176,
	/* Range 0x6D71 - 0x6D73, array index: 0x0450 */
	0x3222,0x3225,0x3172,
	/* Range 0x6D99 - 0x6D9B, array index: 0x0453 */
	0x317D,0x317B,0x317C,
	/* Range 0x6DFE - 0x6E0C, array index: 0x0456 */
	0x3C5D,0x3728,INVALC,0x3726,0x3723,INVALC,0x3670,INVALC,
	0x3677,0x3724,0x3676,0x3725,0x3679,INVALC,0x367D,
	/* Range 0x6ED9 - 0x6EDD, array index: 0x0465 */
	0x4244,0x423E,0x4247,INVALC,0x423D,
	/* Range 0x6F48 - 0x6F4C, array index: 0x046A */
	0x6544,0x6545,0x4766,INVALC,0x6543,
	/* Range 0x6FD6 - 0x6FDA, array index: 0x046F */
	0x5530,0x552F,INVALC,0x5535,0x5536,
	/* Range 0x702C - 0x702E, array index: 0x0474 */
	0x5A53,0x654A,0x6549,
	/* Range 0x7087 - 0x7090, array index: 0x0477 */
	0x2974,0x2970,0x2975,INVALC,0x296F,0x2979,0x2976,INVALC,
	0x2972,0x2971,
	/* Range 0x70A2 - 0x70AA, array index: 0x0481 */
	0x2D5C,0x2D5A,INVALC,0x2D58,0x626A,0x2D65,0x2D62,INVALC,
	0x2D5F,
	/* Range 0x712A - 0x712D, array index: 0x048A */
	0x3C6F,0x3C7A,0x3C72,0x3D21,
	/* Range 0x7133 - 0x713F, array index: 0x048E */
	0x3C7B,0x3C79,0x3C71,INVALC,0x3C73,0x3C77,0x6553,INVALC,
	0x6554,INVALC,0x6555,INVALC,0x6552,
	/* Range 0x7174 - 0x7177, array index: 0x049B */
	0x4253,0x6557,0x6558,0x6556,
	/* Range 0x718B - 0x718E, array index: 0x049F */
	0x476A,0x476D,0x476B,0x4768,
	/* Range 0x71E9 - 0x71EB, array index: 0x04A3 */
	0x553E,0x5543,0x5540,
	/* Range 0x720E - 0x7218, array index: 0x04A6 */
	0x655F,0x5C56,INVALC,0x5C58,0x5C55,INVALC,INVALC,0x5A58,
	0x5C53,0x5C57,0x6561,
	/* Range 0x7264 - 0x7268, array index: 0x04B1 */
	0x2664,0x2A21,0x2A22,INVALC,0x2A23,
	/* Range 0x7328 - 0x732F, array index: 0x04B6 */
	0x3D2E,INVALC,0x3D29,0x3D2B,0x3D2A,INVALC,0x4261,0x3D2D,
	/* Range 0x7346 - 0x7348, array index: 0x04BE */
	0x4262,0x6562,0x6563,
	/* Range 0x7353 - 0x7355, array index: 0x04C1 */
	0x4778,0x477A,0x4777,
	/* Range 0x73CE - 0x73D0, array index: 0x04C4 */
	0x2D7E,0x2D79,0x2D7C,
	/* Range 0x7414 - 0x7419, array index: 0x04C7 */
	0x3D36,0x3D3A,INVALC,0x3D2F,0x3D3B,0x3D32,
	/* Range 0x7438 - 0x743E, array index: 0x04CD */
	0x3D33,0x3D3C,INVALC,0x6566,0x3D37,0x4268,0x4275,
	/* Range 0x7445 - 0x7449, array index: 0x04D4 */
	0x4274,INVALC,0x426E,0x4266,0x4271,
	/* Range 0x74C7 - 0x74D3, array index: 0x04D9 */
	0x5A5C,0x5A5D,0x5A5B,INVALC,INVALC,0x5C5E,0x5C5C,0x5C5F,
	INVALC,0x5C5D,0x5C60,INVALC,0x5E3D,
	/* Range 0x7555 - 0x7558, array index: 0x04E6 */
	0x324E,0x324D,0x3250,0x324F,
	/* Range 0x76D5 - 0x76D9, array index: 0x04EA */
	0x3750,0x3752,0x3751,INVALC,0x3D4F,
	/* Range 0x773F - 0x7743, array index: 0x04EF */
	0x3262,0x3757,INVALC,0x3D56,0x3D53,
	/* Range 0x7774 - 0x777B, array index: 0x04F4 */
	0x4833,0x4836,0x4835,0x4834,0x4838,INVALC,0x483B,0x483D,
	/* Range 0x7807 - 0x780B, array index: 0x04FC */
	0x2E3B,0x2E3E,INVALC,0x2E37,0x2E3A,
	/* Range 0x7906 - 0x790D, array index: 0x0501 */
	0x5843,0x583E,0x5840,INVALC,0x583F,0x583C,INVALC,0x5842,
	/* Range 0x7983 - 0x7989, array index: 0x0509 */
	0x4339,0x3D63,INVALC,0x6578,0x4848,INVALC,0x4847,
	/* Range 0x799D - 0x799F, array index: 0x0510 */
	0x4D50,0x4D51,0x4D4F,
	/* Range 0x79C3 - 0x79C7, array index: 0x0513 */
	0x2A3E,0x2A3D,INVALC,0x2A3F,0x2A42,
	/* Range 0x79D7 - 0x79DB, array index: 0x0518 */
	0x2E43,INVALC,0x3274,0x3273,0x3279,
	/* Range 0x79F0 - 0x79F5, array index: 0x051D */
	0x327C,0x3769,0x3767,0x376B,INVALC,0x376A,
	/* Range 0x7A29 - 0x7A2D, array index: 0x0523 */
	0x484B,0x484D,INVALC,0x4850,0x484F,
	/* Range 0x7ACC - 0x7AD5, array index: 0x0528 */
	0x267E,0x2721,INVALC,0x2A47,0x2E4E,INVALC,0x2E4F,0x2E4D,
	0x2E4B,0x2E4C,
	/* Range 0x7ADA - 0x7ADD, array index: 0x0532 */
	0x3323,0x3326,0x3327,0x3324,
	/* Range 0x7AF0 - 0x7AF5, array index: 0x0536 */
	0x4855,0x5230,0x5569,0x556A,INVALC,0x5849,
	/* Range 0x7B41 - 0x7B43, array index: 0x053C */
	0x3D7B,0x3D78,0x3D75,
	/* Range 0x7C99 - 0x7C9C, array index: 0x053F */
	0x377E,0x3821,0x383E,0x3822,
	/* Range 0x7CC2 - 0x7CC9, array index: 0x0543 */
	0x4D6F,0x4D71,0x4D6D,INVALC,0x4D70,0x4D73,INVALC,0x4D72,
	/* Range 0x7D74 - 0x7D78, array index: 0x054B */
	0x3E24,0x3E35,0x3E33,INVALC,0x4367,
	/* Range 0x7D87 - 0x7D8B, array index: 0x0550 */
	0x4366,INVALC,0x4369,0x4360,0x4362,
	/* Range 0x7D95 - 0x7D99, array index: 0x0555 */
	0x4364,INVALC,0x4368,0x436A,0x4363,
	/* Range 0x7DCD - 0x7DD3, array index: 0x055A */
	0x4871,INVALC,0x4873,0x6629,0x486B,INVALC,0x662B,
	/* Range 0x7DFC - 0x7E02, array index: 0x0561 */
	0x4E28,0x4E29,0x4E25,0x4E2C,0x4E27,0x4D7E,0x4D7D,
	/* Range 0x805F - 0x806B, array index: 0x0568 */
	0x4926,0x3E3B,0x4925,INVALC,0x6630,INVALC,0x4E34,0x4E36,
	INVALC,0x4E37,INVALC,0x4E35,0x4E38,
	/* Range 0x807A - 0x807C, array index: 0x0575 */
	0x5C70,0x5C71,0x5C72,
	/* Range 0x80A4 - 0x80A8, array index: 0x0578 */
	0x2A56,INVALC,0x2A5B,0x2A58,0x2A4F,
	/* Range 0x8140 - 0x8142, array index: 0x057D */
	0x3E43,0x3E45,0x3E41,
	/* Range 0x8192 - 0x8196, array index: 0x0580 */
	0x4E3B,0x4E3D,0x4E39,INVALC,0x4E3C,
	/* Range 0x8218 - 0x821A, array index: 0x0585 */
	0x524A,0x5868,0x5A79,
	/* Range 0x8414 - 0x8418, array index: 0x0588 */
	0x3E58,INVALC,0x3E5D,0x3E56,0x3E4E,
	/* Range 0x8492 - 0x8496, array index: 0x058D */
	0x4935,0x4944,INVALC,0x4933,0x4938,
	/* Range 0x8501 - 0x8505, array index: 0x0592 */
	0x4E40,INVALC,0x4E4C,0x4E46,0x4E54,
	/* Range 0x8533 - 0x8535, array index: 0x0597 */
	0x4E4D,0x4E4E,0x4E52,
	/* Range 0x8845 - 0x8847, array index: 0x059A */
	0x386A,0x3E6A,0x3E69,
	/* Range 0x8A9B - 0x8A9F, array index: 0x059D */
	0x495E,0x4962,0x496B,INVALC,0x4968,
	/* Range 0x8AB1 - 0x8AB7, array index: 0x05A2 */
	0x4E67,INVALC,0x4E72,0x4E70,0x4E6E,INVALC,0x4E6C,
	/* Range 0x8C51 - 0x8C53, array index: 0x05A9 */
	0x5D34,0x664D,0x6221,
	/* Range 0x8D82 - 0x8D88, array index: 0x05AC */
	0x3F39,0x3F34,INVALC,INVALC,0x3F37,0x3F30,0x3F33,
	/* Range 0x8EAD - 0x8EB9, array index: 0x05B3 */
	0x393F,INVALC,0x3940,0x3F40,0x4473,INVALC,0x497D,0x497C,
	INVALC,0x4F30,0x4F32,0x4F2F,0x4F31,
	/* Range 0x8F9D - 0x8FA7, array index: 0x05C0 */
	0x3F4B,0x4477,INVALC,0x4478,0x4A21,0x4A22,INVALC,0x4F37,
	0x5327,INVALC,0x5326,
	/* Range 0x9028 - 0x902C, array index: 0x05CB */
	0x3F4F,0x3F50,0x3F4D,0x3F4C,0x3F51,
	/* Range 0x91DE - 0x91E0, array index: 0x05D0 */
	0x342F,0x342E,0x3431,
	/* Range 0x9220 - 0x9222, array index: 0x05D3 */
	0x3F5E,0x6659,0x3F63,
	/* Range 0x92AD - 0x92B1, array index: 0x05D6 */
	0x4A32,0x4A37,INVALC,0x672A,0x4A31,
	/* Range 0x92DA - 0x92DC, array index: 0x05DB */
	0x4F4B,0x4F4E,0x4F44,
	/* Range 0x92F3 - 0x92F6, array index: 0x05DE */
	0x4F4D,0x665B,0x665C,0x4A30,
	/* Range 0x9330 - 0x9332, array index: 0x05E2 */
	0x533C,0x5339,0x533A,
	/* Range 0x9372 - 0x9374, array index: 0x05E5 */
	0x5662,0x5663,0x565F,
	/* Range 0x93BB - 0x93BE, array index: 0x05E8 */
	0x5945,0x6661,0x6662,0x6660,
	/* Range 0x93E9 - 0x93F4, array index: 0x05EC */
	0x5B41,0x5B42,0x5B4E,INVALC,0x5B47,INVALC,0x6665,INVALC,
	0x6663,0x6664,0x5D41,0x5E6A,
	/* Range 0x941A - 0x9424, array index: 0x05F8 */
	0x5D44,0x5D45,0x5D47,INVALC,0x5D42,0x5D3A,INVALC,INVALC,
	0x6666,0x6667,0x6668,
	/* Range 0x9458 - 0x945C, array index: 0x0603 */
	0x6431,INVALC,0x6055,0x6054,0x6056,
	/* Range 0x9465 - 0x9467, array index: 0x0608 */
	0x6057,0x666A,0x6669,
	/* Range 0x96B5 - 0x96B7, array index: 0x060B */
	0x5D49,0x2A7D,0x5349,
	/* Range 0x96EC - 0x96EE, array index: 0x060E */
	0x3F7E,0x3F7C,0x3F7B,
	/* Range 0x9733 - 0x9737, array index: 0x0611 */
	0x5D4C,0x5D4E,INVALC,0x5E72,0x5E71,
	/* Range 0x97B1 - 0x97B5, array index: 0x0616 */
	0x5B5F,0x5B5D,INVALC,0x5B5E,0x5B60,
	/* Range 0x983C - 0x9842, array index: 0x061B */
	0x535B,0x535F,INVALC,0x595B,0x5726,INVALC,0x637B,
	/* Range 0x9854 - 0x9856, array index: 0x0622 */
	0x5956,0x5958,0x5B65,
	/* Range 0x986C - 0x986E, array index: 0x0625 */
	0x605D,0x605E,0x605C,
	/* Range 0x99B6 - 0x99B8, array index: 0x0628 */
	0x4A63,0x4A62,0x4A66,
	/* Range 0x99BC - 0x99C0, array index: 0x062B */
	0x4A61,INVALC,0x4A60,0x4A6C,0x4A64,
	/* Range 0x9AE4 - 0x9AEA, array index: 0x0630 */
	0x4A6F,0x4A73,INVALC,INVALC,0x4A71,0x4A70,0x4A72,
	/* Range 0x9B8B - 0x9B8F, array index: 0x0637 */
	0x537C,0x5421,INVALC,0x5425,0x537D,
	/* Range 0x9BCB - 0x9BCD, array index: 0x063C */
	0x5976,0x5975,0x5743,
	/* Range 0x9BFB - 0x9C04, array index: 0x063F */
	0x5D67,0x5D69,INVALC,0x5D6A,0x5D74,0x5D75,INVALC,0x5D6E,
	INVALC,0x5D68,
	/* Range 0x9CEB - 0x9CEF, array index: 0x0649 */
	0x4566,0x2F42,INVALC,0x4568,0x4567,
	/* Range 0x9EAA - 0x9EAC, array index: 0x064E */
	0x502F,0x502E,0x543A,
	/* Range 0x9EC4 - 0x9EC6, array index: 0x0651 */
	0x396A,0x543C,0x543B,
	/* Range 0x9F02 - 0x9F05, array index: 0x0654 */
	0x5A29,0x5C2D,0x5C2E,0x5F3C,
	/* Unranged codes (197 codes) */
	/* Array index: 0x0658 */ 0x4E04,0x2126,
	/* Array index: 0x0658 */ 0x4E05,0x2125,
	/* Array index: 0x0658 */ 0x4E28,0x2121,
	/* Array index: 0x0658 */ 0x4E2A,0x212C,
	/* Array index: 0x0658 */ 0x4E36,0x2122,
	/* Array index: 0x0658 */ 0x4E3D,0x2740,
	/* Array index: 0x0658 */ 0x4E3F,0x2123,
	/* Array index: 0x0658 */ 0x4E46,0x2130,
	/* Array index: 0x0658 */ 0x4E49,0x212E,
	/* Array index: 0x0658 */ 0x4E54,0x2345,
	/* Array index: 0x0658 */ 0x4E57,0x2B22,
	/* Array index: 0x0658 */ 0x4E62,0x2146,
	/* Array index: 0x0658 */ 0x4E63,0x2145,
	/* Array index: 0x0658 */ 0x4E68,0x2264,
	/* Array index: 0x0658 */ 0x4E71,0x2440,
	/* Array index: 0x0658 */ 0x4E79,0x2B23,
	/* Array index: 0x0658 */ 0x4E80,0x396C,
	/* Array index: 0x0658 */ 0x4E81,0x396E,
	/* Array index: 0x0658 */ 0x4E85,0x2124,
	/* Array index: 0x0658 */ 0x4E87,0x212D,
	/* Array index: 0x0658 */ 0x4E89,0x2265,
	/* Array index: 0x0658 */ 0x4E8F,0x2131,
	/* Array index: 0x0658 */ 0x4E97,0x216C,
	/* Array index: 0x0658 */ 0x4E98,0x2266,
	/* Array index: 0x0658 */ 0x4E9C,0x2442,
	/* Array index: 0x0658 */ 0x4E9D,0x2742,
	/* Array index: 0x0658 */ 0x4EA3,0x2147,
	/* Array index: 0x0658 */ 0x4EB7,0x4033,
	/* Array index: 0x0658 */ 0x4EBC,0x2132,
	/* Array index: 0x0658 */ 0x4EBE,0x2133,
	/* Array index: 0x0658 */ 0x4EC5,0x2149,
	/* Array index: 0x0658 */ 0x4ECC,0x214C,
	/* Array index: 0x0658 */ 0x4ECE,0x214B,
	/* Array index: 0x0658 */ 0x4ECF,0x214A,
	/* Array index: 0x0658 */ 0x4EE0,0x216D,
	/* Array index: 0x0658 */ 0x4EE6,0x2172,
	/* Array index: 0x0658 */ 0x4EE7,0x2170,
	/* Array index: 0x0658 */ 0x4EEE,0x226F,
	/* Array index: 0x0658 */ 0x4EEF,0x2268,
	/* Array index: 0x0658 */ 0x4F03,0x226D,
	/* Array index: 0x0658 */ 0x4F06,0x226C,
	/* Array index: 0x0658 */ 0x4F07,0x2274,
	/* Array index: 0x0658 */ 0x4F16,0x2270,
	/* Array index: 0x0658 */ 0x4F17,0x2277,
	/* Array index: 0x0658 */ 0x4F1A,0x2275,
	/* Array index: 0x0658 */ 0x4F1C,0x2273,
	/* Array index: 0x0658 */ 0x4F28,0x2272,
	/* Array index: 0x0658 */ 0x4F31,0x2450,
	/* Array index: 0x0658 */ 0x4F32,0x2448,
	/* Array index: 0x0658 */ 0x4F35,0x244F,
	/* Array index: 0x0658 */ 0x4F37,0x2446,
	/* Array index: 0x0658 */ 0x4F39,0x2445,
	/* Array index: 0x0658 */ 0x4F40,0x244E,
	/* Array index: 0x0658 */ 0x4F42,0x2449,
	/* Array index: 0x0658 */ 0x4F44,0x244B,
	/* Array index: 0x0658 */ 0x4F45,0x2443,
	/* Array index: 0x0658 */ 0x4F4A,0x2444,
	/* Array index: 0x0658 */ 0x4F4B,0x244C,
	/* Array index: 0x0658 */ 0x4F66,0x6448,
	/* Array index: 0x0658 */ 0x4F68,0x6449,
	/* Array index: 0x0658 */ 0x4F6D,0x2747,
	/* Array index: 0x0658 */ 0x4F72,0x274F,
	/* Array index: 0x0658 */ 0x4F8A,0x2744,
	/* Array index: 0x0658 */ 0x4F8C,0x2751,
	/* Array index: 0x0658 */ 0x4F8E,0x2748,
	/* Array index: 0x0658 */ 0x4F93,0x2743,
	/* Array index: 0x0658 */ 0x4FB0,0x2B2D,
	/* Array index: 0x0658 */ 0x4FB1,0x2B2E,
	/* Array index: 0x0658 */ 0x4FB4,0x2B36,
	/* Array index: 0x0658 */ 0x4FB8,0x2B2C,
	/* Array index: 0x0658 */ 0x4FBD,0x6263,
	/* Array index: 0x0658 */ 0x4FBE,0x2B2B,
	/* Array index: 0x0658 */ 0x4FC6,0x2B32,
	/* Array index: 0x0658 */ 0x4FC8,0x2B2F,
	/* Array index: 0x0658 */ 0x4FCC,0x2B33,
	/* Array index: 0x0658 */ 0x4FD2,0x2B28,
	/* Array index: 0x0658 */ 0x4FD5,0x2B29,
	/* Array index: 0x0658 */ 0x4FE3,0x2B35,
	/* Array index: 0x0658 */ 0x4FE5,0x2B34,
	/* Array index: 0x0658 */ 0x4FE7,0x644A,
	/* Array index: 0x0658 */ 0x4FF0,0x2F48,
	/* Array index: 0x0658 */ 0x4FF2,0x2F4D,
	/* Array index: 0x0658 */ 0x5008,0x2F4F,
	/* Array index: 0x0658 */ 0x5010,0x2F53,
	/* Array index: 0x0658 */ 0x501D,0x2F45,
	/* Array index: 0x0658 */ 0x5024,0x2F4E,
	/* Array index: 0x0658 */ 0x502E,0x2F51,
	/* Array index: 0x0658 */ 0x5032,0x2F43,
	/* Array index: 0x0658 */ 0x5036,0x2F50,
	/* Array index: 0x0658 */ 0x5038,0x2F54,
	/* Array index: 0x0658 */ 0x5039,0x2F55,
	/* Array index: 0x0658 */ 0x503F,0x644B,
	/* Array index: 0x0658 */ 0x5042,0x343C,
	/* Array index: 0x0658 */ 0x5044,0x343F,
	/* Array index: 0x0658 */ 0x5050,0x3444,
	/* Array index: 0x0658 */ 0x5052,0x3441,
	/* Array index: 0x0658 */ 0x5054,0x3442,
	/* Array index: 0x0658 */ 0x5056,0x2F57,
	/* Array index: 0x0658 */ 0x5058,0x3447,
	/* Array index: 0x0658 */ 0x5059,0x343D,
	/* Array index: 0x0658 */ 0x5066,0x3440,
	/* Array index: 0x0658 */ 0x506C,0x344B,
	/* Array index: 0x0658 */ 0x5071,0x3443,
	/* Array index: 0x0658 */ 0x5081,0x3975,
	/* Array index: 0x0658 */ 0x5088,0x3971,
	/* Array index: 0x0658 */ 0x5089,0x3972,
	/* Array index: 0x0658 */ 0x508F,0x396F,
	/* Array index: 0x0658 */ 0x5097,0x3970,
	/* Array index: 0x0658 */ 0x50A6,0x644D,
	/* Array index: 0x0658 */ 0x50AA,0x4039,
	/* Array index: 0x0658 */ 0x50AB,0x403C,
	/* Array index: 0x0658 */ 0x50B9,0x403A,
	/* Array index: 0x0658 */ 0x50BC,0x4037,
	/* Array index: 0x0658 */ 0x50C3,0x403D,
	/* Array index: 0x0658 */ 0x50CC,0x4036,
	/* Array index: 0x0658 */ 0x50CD,0x403E,
	/* Array index: 0x0658 */ 0x50D0,0x456B,
	/* Array index: 0x0658 */ 0x50D2,0x634C,
	/* Array index: 0x0658 */ 0x50D8,0x456E,
	/* Array index: 0x0658 */ 0x50D9,0x456C,
	/* Array index: 0x0658 */ 0x50DC,0x456D,
	/* Array index: 0x0658 */ 0x50EB,0x4571,
	/* Array index: 0x0658 */ 0x50F4,0x4574,
	/* Array index: 0x0658 */ 0x50F7,0x4B29,
	/* Array index: 0x0658 */ 0x50FC,0x4B2B,
	/* Array index: 0x0658 */ 0x5101,0x4B2D,
	/* Array index: 0x0658 */ 0x5116,0x5031,
	/* Array index: 0x0658 */ 0x511B,0x5035,
	/* Array index: 0x0658 */ 0x511D,0x5030,
	/* Array index: 0x0658 */ 0x511E,0x5034,
	/* Array index: 0x0658 */ 0x5127,0x5441,
	/* Array index: 0x0658 */ 0x5128,0x5440,
	/* Array index: 0x0658 */ 0x512B,0x5033,
	/* Array index: 0x0658 */ 0x512F,0x5752,
	/* Array index: 0x0658 */ 0x5136,0x5C32,
	/* Array index: 0x0658 */ 0x513E,0x606F,
	/* Array index: 0x0658 */ 0x514A,0x2278,
	/* Array index: 0x0658 */ 0x514E,0x2454,
	/* Array index: 0x0658 */ 0x5150,0x2453,
	/* Array index: 0x0658 */ 0x5151,0x2451,
	/* Array index: 0x0658 */ 0x5156,0x2752,
	/* Array index: 0x0658 */ 0x5160,0x3976,
	/* Array index: 0x0658 */ 0x5164,0x5E25,
	/* Array index: 0x0658 */ 0x5166,0x2134,
	/* Array index: 0x0658 */ 0x516A,0x2B37,
	/* Array index: 0x0658 */ 0x5172,0x227A,
	/* Array index: 0x0658 */ 0x5179,0x2E7B,
	/* Array index: 0x0658 */ 0x517E,0x403F,
	/* Array index: 0x0658 */ 0x517F,0x6453,
	/* Array index: 0x0658 */ 0x518B,0x2178,
	/* Array index: 0x0658 */ 0x518C,0x2179,
	/* Array index: 0x0658 */ 0x5190,0x2755,
	/* Array index: 0x0658 */ 0x5196,0x2128,
	/* Array index: 0x0658 */ 0x5199,0x217A,
	/* Array index: 0x0658 */ 0x519D,0x2457,
	/* Array index: 0x0658 */ 0x519F,0x2B39,
	/* Array index: 0x0658 */ 0x51A1,0x2F5B,
	/* Array index: 0x0658 */ 0x51A3,0x2F59,
	/* Array index: 0x0658 */ 0x51A6,0x2F5A,
	/* Array index: 0x0658 */ 0x51A8,0x344D,
	/* Array index: 0x0658 */ 0x51B8,0x245A,
	/* Array index: 0x0658 */ 0x51C7,0x2F5E,
	/* Array index: 0x0658 */ 0x51C9,0x2F5F,
	/* Array index: 0x0658 */ 0x51D6,0x397E,
	/* Array index: 0x0658 */ 0x51DF,0x5442,
	/* Array index: 0x0658 */ 0x51E8,0x2322,
	/* Array index: 0x0658 */ 0x51EC,0x245B,
	/* Array index: 0x0658 */ 0x51ED,0x2759,
	/* Array index: 0x0658 */ 0x51F4,0x457A,
	/* Array index: 0x0658 */ 0x51FE,0x275A,
	/* Array index: 0x0658 */ 0x5204,0x2136,
	/* Array index: 0x0658 */ 0x5205,0x2152,
	/* Array index: 0x0658 */ 0x520B,0x217D,
	/* Array index: 0x0658 */ 0x520F,0x2325,
	/* Array index: 0x0658 */ 0x5214,0x2324,
	/* Array index: 0x0658 */ 0x5215,0x2326,
	/* Array index: 0x0658 */ 0x5218,0x2327,
	/* Array index: 0x0658 */ 0x521F,0x245F,
	/* Array index: 0x0658 */ 0x5220,0x2461,
	/* Array index: 0x0658 */ 0x5223,0x245C,
	/* Array index: 0x0658 */ 0x5226,0x245E,
	/* Array index: 0x0658 */ 0x5227,0x245D,
	/* Array index: 0x0658 */ 0x522B,0x2460,
	/* Array index: 0x0658 */ 0x522F,0x275B,
	/* Array index: 0x0658 */ 0x5234,0x275E,
	/* Array index: 0x0658 */ 0x5239,0x275F,
	/* Array index: 0x0658 */ 0x523C,0x275D,
	/* Array index: 0x0658 */ 0x523E,0x644F,
	/* Array index: 0x0658 */ 0x5245,0x2B3F,
	/* Array index: 0x0658 */ 0x5248,0x2B40,
	/* Array index: 0x0658 */ 0x524F,0x2B43,
	/* Array index: 0x0658 */ 0x5253,0x2F68,
	/* Array index: 0x0658 */ 0x5257,0x2F65,
	/* Array index: 0x0658 */ 0x5259,0x2F62,
	/* Array index: 0x0658 */ 0x5260,0x2F60,
	/* Array index: 0x0658 */ 0x5270,0x3A23,
	/* Array index: 0x0658 */ 0x5273,0x3A21,
	/* Array index: 0x0658 */ 0x5276,0x3451,
	/* Array index: 0x0658 */ 0x5279,0x4041,
	/* Array index: 0x0658 */ 0x527E,0x4040,
	/* Array index: 0x0658 */ 0x5286,0x4B31,
	/* Array index: 0x0658 */ 0x528E,0x4B32,
	/* Array index: 0x0658 */ 0x5290,0x5036,
	/* Array index: 0x0658 */ 0x5294,0x5037,
	/* Array index: 0x0658 */ 0x52A8,0x2328,
	/* Array index: 0x0658 */ 0x52B9,0x2760,
	/* Array index: 0x0658 */ 0x52BD,0x625A,
	/* Array index: 0x0658 */ 0x52BF,0x2762,
	/* Array index: 0x0658 */ 0x52C4,0x2B48,
	/* Array index: 0x0658 */ 0x52C5,0x2B46,
	/* Array index: 0x0658 */ 0x52CA,0x2B47,
	/* Array index: 0x0658 */ 0x52D4,0x3452,
	/* Array index: 0x0658 */ 0x52E0,0x4045,
	/* Array index: 0x0658 */ 0x52E1,0x4044,
	/* Array index: 0x0658 */ 0x52E7,0x4046,
	/* Array index: 0x0658 */ 0x52EA,0x4621,
	/* Array index: 0x0658 */ 0x52ED,0x457E,
	/* Array index: 0x0658 */ 0x52EE,0x4B33,
	/* Array index: 0x0658 */ 0x52F2,0x4D2C,
	/* Array index: 0x0658 */ 0x52FD,0x2155,
	/* Array index: 0x0658 */ 0x5300,0x2156,
	/* Array index: 0x0658 */ 0x5303,0x2223,
	/* Array index: 0x0658 */ 0x5304,0x2222,
	/* Array index: 0x0658 */ 0x5307,0x2224,
	/* Array index: 0x0658 */ 0x5314,0x5038,
	/* Array index: 0x0658 */ 0x531E,0x2226,
	/* Array index: 0x0658 */ 0x5324,0x6450,
	/* Array index: 0x0658 */ 0x5325,0x2465,
	/* Array index: 0x0658 */ 0x5327,0x2B49,
	/* Array index: 0x0658 */ 0x5346,0x2159,
	/* Array index: 0x0658 */ 0x534B,0x232A,
	/* Array index: 0x0658 */ 0x534F,0x232B,
	/* Array index: 0x0658 */ 0x5350,0x232D,
	/* Array index: 0x0658 */ 0x5355,0x2763,
	/* Array index: 0x0658 */ 0x5358,0x2B4A,
	/* Array index: 0x0658 */ 0x535D,0x215A,
	/* Array index: 0x0658 */ 0x535F,0x2227,
	/* Array index: 0x0658 */ 0x5365,0x6451,
	/* Array index: 0x0658 */ 0x5367,0x2A5C,
	/* Array index: 0x0658 */ 0x5369,0x212A,
	/* Array index: 0x0658 */ 0x536D,0x2228,
	/* Array index: 0x0658 */ 0x5374,0x2467,
	/* Array index: 0x0658 */ 0x5376,0x2765,
	/* Array index: 0x0658 */ 0x537A,0x2766,
	/* Array index: 0x0658 */ 0x537D,0x2B4B,
	/* Array index: 0x0658 */ 0x5380,0x4047,
	/* Array index: 0x0658 */ 0x5386,0x215B,
	/* Array index: 0x0658 */ 0x5389,0x2229,
	/* Array index: 0x0658 */ 0x538B,0x232F,
	/* Array index: 0x0658 */ 0x5393,0x2767,
	/* Array index: 0x0658 */ 0x539B,0x6452,
	/* Array index: 0x0658 */ 0x53A0,0x3454,
	/* Array index: 0x0658 */ 0x53AE,0x4625,
	/* Array index: 0x0658 */ 0x53B0,0x4626,
	/* Array index: 0x0658 */ 0x53B1,0x4B35,
	/* Array index: 0x0658 */ 0x53B3,0x5445,
	/* Array index: 0x0658 */ 0x53B6,0x212B,
	/* Array index: 0x0658 */ 0x53B7,0x215C,
	/* Array index: 0x0658 */ 0x53BA,0x222A,
	/* Array index: 0x0658 */ 0x53BE,0x2331,
	/* Array index: 0x0658 */ 0x53CC,0x215E,
	/* Array index: 0x0658 */ 0x53CE,0x2160,
	/* Array index: 0x0658 */ 0x53D0,0x222B,
	/* Array index: 0x0658 */ 0x53D2,0x2333,
	/* Array index: 0x0658 */ 0x53D5,0x276B,
	/* Array index: 0x0658 */ 0x53E0,0x404B,
	/* Array index: 0x0658 */ 0x53F4,0x2231,
	/* Array index: 0x0658 */ 0x53F6,0x222D,
	/* Array index: 0x0658 */ 0x53F7,0x222E,
	/* Array index: 0x0658 */ 0x53F9,0x222F,
	/* Array index: 0x0658 */ 0x53FF,0x2335,
	/* Array index: 0x0658 */ 0x5400,0x2337,
	/* Array index: 0x0658 */ 0x5405,0x2339,
	/* Array index: 0x0658 */ 0x5413,0x233A,
	/* Array index: 0x0658 */ 0x5415,0x233B,
	/* Array index: 0x0658 */ 0x5416,0x2334,
	/* Array index: 0x0658 */ 0x541A,0x2476,
	/* Array index: 0x0658 */ 0x542F,0x2524,
	/* Array index: 0x0658 */ 0x5432,0x2477,
	/* Array index: 0x0658 */ 0x5434,0x247B,
	/* Array index: 0x0658 */ 0x543A,0x2470,
	/* Array index: 0x0658 */ 0x543F,0x2521,
	/* Array index: 0x0658 */ 0x5444,0x2473,
	/* Array index: 0x0658 */ 0x545A,0x2523,
	/* Array index: 0x0658 */ 0x545F,0x276E,
	/* Array index: 0x0658 */ 0x5469,0x2774,
	/* Array index: 0x0658 */ 0x546A,0x2778,
	/* Array index: 0x0658 */ 0x546D,0x2775,
	/* Array index: 0x0658 */ 0x5479,0x2771,
	/* Array index: 0x0658 */ 0x5485,0x276C,
	/* Array index: 0x0658 */ 0x5489,0x2770,
	/* Array index: 0x0658 */ 0x548A,0x2779,
	/* Array index: 0x0658 */ 0x548F,0x2773,
	/* Array index: 0x0658 */ 0x5493,0x276F,
	/* Array index: 0x0658 */ 0x5494,0x2777,
	/* Array index: 0x0658 */ 0x549E,0x2B5C,
	/* Array index: 0x0658 */ 0x549F,0x2B5D,
	/* Array index: 0x0658 */ 0x54A3,0x2B57,
	/* Array index: 0x0658 */ 0x54A4,0x2B59,
	/* Array index: 0x0658 */ 0x54B2,0x2B5B,
	/* Array index: 0x0658 */ 0x54B4,0x2B54,
	/* Array index: 0x0658 */ 0x54B5,0x2B5E,
	/* Array index: 0x0658 */ 0x54B9,0x2B51,
	/* Array index: 0x0658 */ 0x54D0,0x2B53,
	/* Array index: 0x0658 */ 0x54DA,0x2B58,
	/* Array index: 0x0658 */ 0x54EF,0x2F77,
	/* Array index: 0x0658 */ 0x54F0,0x3021,
	/* Array index: 0x0658 */ 0x54F9,0x2F79,
	/* Array index: 0x0658 */ 0x5500,0x2F7B,
	/* Array index: 0x0658 */ 0x5502,0x2F7A,
	/* Array index: 0x0658 */ 0x550D,0x2F73,
	/* Array index: 0x0658 */ 0x5513,0x2F76,
	/* Array index: 0x0658 */ 0x5518,0x2F7E,
	/* Array index: 0x0658 */ 0x5519,0x3025,
	/* Array index: 0x0658 */ 0x5528,0x6721,
	/* Array index: 0x0658 */ 0x5529,0x345F,
	/* Array index: 0x0658 */ 0x552B,0x3468,
	/* Array index: 0x0658 */ 0x553A,0x3466,
	/* Array index: 0x0658 */ 0x553F,0x3467,
	/* Array index: 0x0658 */ 0x5549,0x345B,
	/* Array index: 0x0658 */ 0x554C,0x3458,
	/* Array index: 0x0658 */ 0x5553,0x3462,
	/* Array index: 0x0658 */ 0x5554,0x3461,
	/* Array index: 0x0658 */ 0x555D,0x345E,
	/* Array index: 0x0658 */ 0x5568,0x3459,
	/* Array index: 0x0658 */ 0x556B,0x6457,
	/* Array index: 0x0658 */ 0x557A,0x3A3C,
	/* Array index: 0x0658 */ 0x5585,0x3A31,
	/* Array index: 0x0658 */ 0x5586,0x3A38,
	/* Array index: 0x0658 */ 0x5596,0x3A2E,
	/* Array index: 0x0658 */ 0x5597,0x3A2A,
	/* Array index: 0x0658 */ 0x559B,0x3A33,
	/* Array index: 0x0658 */ 0x559E,0x3A40,
	/* Array index: 0x0658 */ 0x55A0,0x3A34,
	/* Array index: 0x0658 */ 0x55A9,0x3A42,
	/* Array index: 0x0658 */ 0x55AF,0x3A3B,
	/* Array index: 0x0658 */ 0x55B4,0x3A2F,
	/* Array index: 0x0658 */ 0x55BA,0x6723,
	/* Array index: 0x0658 */ 0x55C1,0x4051,
	/* Array index: 0x0658 */ 0x55D7,0x404E,
	/* Array index: 0x0658 */ 0x55DE,0x3A2B,
	/* Array index: 0x0658 */ 0x55EA,0x404C,
	/* Array index: 0x0658 */ 0x55EE,0x6725,
	/* Array index: 0x0658 */ 0x55F0,0x6724,
	/* Array index: 0x0658 */ 0x55F4,0x6458,
	/* Array index: 0x0658 */ 0x55F8,0x462E,
	/* Array index: 0x0658 */ 0x55FB,0x4628,
	/* Array index: 0x0658 */ 0x5603,0x4629,
	/* Array index: 0x0658 */ 0x5605,0x4636,
	/* Array index: 0x0658 */ 0x5607,0x462C,
	/* Array index: 0x0658 */ 0x560B,0x462A,
	/* Array index: 0x0658 */ 0x5611,0x4634,
	/* Array index: 0x0658 */ 0x5618,0x4633,
	/* Array index: 0x0658 */ 0x561E,0x4631,
	/* Array index: 0x0658 */ 0x5620,0x645A,
	/* Array index: 0x0658 */ 0x5622,0x6459,
	/* Array index: 0x0658 */ 0x5626,0x4650,
	/* Array index: 0x0658 */ 0x5628,0x4630,
	/* Array index: 0x0658 */ 0x562B,0x4B41,
	/* Array index: 0x0658 */ 0x562D,0x4B38,
	/* Array index: 0x0658 */ 0x5631,0x4B3D,
	/* Array index: 0x0658 */ 0x5637,0x4B44,
	/* Array index: 0x0658 */ 0x5644,0x4B3E,
	/* Array index: 0x0658 */ 0x5647,0x4B37,
	/* Array index: 0x0658 */ 0x564B,0x4B3B,
	/* Array index: 0x0658 */ 0x565F,0x503F,
	/* Array index: 0x0658 */ 0x5661,0x5040,
	/* Array index: 0x0658 */ 0x5667,0x503B,
	/* Array index: 0x0658 */ 0x567B,0x503D,
	/* Array index: 0x0658 */ 0x567D,0x5448,
	/* Array index: 0x0658 */ 0x5689,0x5449,
	/* Array index: 0x0658 */ 0x568A,0x5447,
	/* Array index: 0x0658 */ 0x5691,0x645D,
	/* Array index: 0x0658 */ 0x5692,0x645B,
	/* Array index: 0x0658 */ 0x5694,0x575B,
	/* Array index: 0x0658 */ 0x5696,0x575A,
	/* Array index: 0x0658 */ 0x569B,0x5758,
	/* Array index: 0x0658 */ 0x56AF,0x5A2D,
	/* Array index: 0x0658 */ 0x56B0,0x645E,
	/* Array index: 0x0658 */ 0x56C4,0x6728,
	/* Array index: 0x0658 */ 0x56C7,0x5F40,
	/* Array index: 0x0658 */ 0x56D2,0x6071,
	/* Array index: 0x0658 */ 0x56D8,0x2177,
	/* Array index: 0x0658 */ 0x56E3,0x233E,
	/* Array index: 0x0658 */ 0x56E6,0x252C,
	/* Array index: 0x0658 */ 0x56E9,0x2526,
	/* Array index: 0x0658 */ 0x56EC,0x252B,
	/* Array index: 0x0658 */ 0x56ED,0x252A,
	/* Array index: 0x0658 */ 0x56F6,0x2822,
	/* Array index: 0x0658 */ 0x56F8,0x277D,
	/* Array index: 0x0658 */ 0x5700,0x2B65,
	/* Array index: 0x0658 */ 0x5705,0x3027,
	/* Array index: 0x0658 */ 0x570E,0x3A44,
	/* Array index: 0x0658 */ 0x570F,0x3A43,
	/* Array index: 0x0658 */ 0x5715,0x4052,
	/* Array index: 0x0658 */ 0x5717,0x4637,
	/* Array index: 0x0658 */ 0x571D,0x5F41,
	/* Array index: 0x0658 */ 0x5721,0x2161,
	/* Array index: 0x0658 */ 0x5724,0x2234,
	/* Array index: 0x0658 */ 0x5727,0x6249,
	/* Array index: 0x0658 */ 0x5731,0x2341,
	/* Array index: 0x0658 */ 0x5732,0x2342,
	/* Array index: 0x0658 */ 0x5735,0x233F,
	/* Array index: 0x0658 */ 0x5736,0x2340,
	/* Array index: 0x0658 */ 0x573C,0x2531,
	/* Array index: 0x0658 */ 0x573D,0x253E,
	/* Array index: 0x0658 */ 0x573F,0x2530,
	/* Array index: 0x0658 */ 0x575F,0x2535,
	/* Array index: 0x0658 */ 0x5763,0x2825,
	/* Array index: 0x0658 */ 0x5765,0x2823,
	/* Array index: 0x0658 */ 0x5767,0x2826,
	/* Array index: 0x0658 */ 0x576E,0x2828,
	/* Array index: 0x0658 */ 0x5778,0x2829,
	/* Array index: 0x0658 */ 0x577F,0x282A,
	/* Array index: 0x0658 */ 0x5781,0x2824,
	/* Array index: 0x0658 */ 0x578A,0x6460,
	/* Array index: 0x0658 */ 0x5796,0x2B71,
	/* Array index: 0x0658 */ 0x579C,0x2B70,
	/* Array index: 0x0658 */ 0x57A1,0x2B6B,
	/* Array index: 0x0658 */ 0x57AC,0x2B66,
	/* Array index: 0x0658 */ 0x57B7,0x302A,
	/* Array index: 0x0658 */ 0x57BB,0x3030,
	/* Array index: 0x0658 */ 0x57BE,0x302F,
	/* Array index: 0x0658 */ 0x57C0,0x3038,
	/* Array index: 0x0658 */ 0x57CD,0x302B,
	/* Array index: 0x0658 */ 0x57D1,0x3036,
	/* Array index: 0x0658 */ 0x57DB,0x3032,
	/* Array index: 0x0658 */ 0x57DD,0x346E,
	/* Array index: 0x0658 */ 0x57DE,0x3471,
	/* Array index: 0x0658 */ 0x57E6,0x3472,
	/* Array index: 0x0658 */ 0x57E8,0x3474,
	/* Array index: 0x0658 */ 0x57EA,0x3469,
	/* Array index: 0x0658 */ 0x57EF,0x346B,
	/* Array index: 0x0658 */ 0x57FE,0x346F,
	/* Array index: 0x0658 */ 0x57FF,0x3475,
	/* Array index: 0x0658 */ 0x5803,0x3476,
	/* Array index: 0x0658 */ 0x580F,0x6461,
	/* Array index: 0x0658 */ 0x5812,0x6462,
	/* Array index: 0x0658 */ 0x5813,0x6463,
	/* Array index: 0x0658 */ 0x5818,0x3A4A,
	/* Array index: 0x0658 */ 0x581A,0x3A45,
	/* Array index: 0x0658 */ 0x581F,0x3A47,
	/* Array index: 0x0658 */ 0x5822,0x3A50,
	/* Array index: 0x0658 */ 0x5826,0x3A4C,
	/* Array index: 0x0658 */ 0x582B,0x6336,
	/* Array index: 0x0658 */ 0x583A,0x3A4E,
	/* Array index: 0x0658 */ 0x583C,0x3A49,
	/* Array index: 0x0658 */ 0x583E,0x3A4B,
	/* Array index: 0x0658 */ 0x5845,0x6337,
	/* Array index: 0x0658 */ 0x5847,0x6464,
	/* Array index: 0x0658 */ 0x5850,0x4057,
	/* Array index: 0x0658 */ 0x5856,0x405B,
	/* Array index: 0x0658 */ 0x585C,0x4056,
	/* Array index: 0x0658 */ 0x5872,0x4643,
	/* Array index: 0x0658 */ 0x5873,0x463E,
	/* Array index: 0x0658 */ 0x5877,0x4642,
	/* Array index: 0x0658 */ 0x5878,0x463C,
	/* Array index: 0x0658 */ 0x5884,0x463D,
	/* Array index: 0x0658 */ 0x588C,0x463A,
	/* Array index: 0x0658 */ 0x588D,0x4645,
	/* Array index: 0x0658 */ 0x5892,0x4638,
	/* Array index: 0x0658 */ 0x589B,0x6465,
	/* Array index: 0x0658 */ 0x58A2,0x4B46,
	/* Array index: 0x0658 */ 0x58A4,0x4B50,
	/* Array index: 0x0658 */ 0x58A7,0x4B51,
	/* Array index: 0x0658 */ 0x58AA,0x4B4B,
	/* Array index: 0x0658 */ 0x58AD,0x463F,
	/* Array index: 0x0658 */ 0x58B0,0x4B4D,
	/* Array index: 0x0658 */ 0x58B2,0x4B49,
	/* Array index: 0x0658 */ 0x58B4,0x4B4F,
	/* Array index: 0x0658 */ 0x58B5,0x4B4C,
	/* Array index: 0x0658 */ 0x58B7,0x4B47,
	/* Array index: 0x0658 */ 0x58BB,0x504A,
	/* Array index: 0x0658 */ 0x58C0,0x504B,
	/* Array index: 0x0658 */ 0x58C3,0x5048,
	/* Array index: 0x0658 */ 0x58C4,0x504C,
	/* Array index: 0x0658 */ 0x58CA,0x5049,
	/* Array index: 0x0658 */ 0x58CD,0x544B,
	/* Array index: 0x0658 */ 0x58D0,0x544C,
	/* Array index: 0x0658 */ 0x58DC,0x5A2F,
	/* Array index: 0x0658 */ 0x58E0,0x5A2E,
	/* Array index: 0x0658 */ 0x58E6,0x5E28,
	/* Array index: 0x0658 */ 0x58FB,0x3A52,
	/* Array index: 0x0658 */ 0x5910,0x4646,
	/* Array index: 0x0658 */ 0x5911,0x6560,
	/* Array index: 0x0658 */ 0x5918,0x2236,
	/* Array index: 0x0658 */ 0x591F,0x3479,
	/* Array index: 0x0658 */ 0x5923,0x405F,
	/* Array index: 0x0658 */ 0x5926,0x4B53,
	/* Array index: 0x0658 */ 0x5930,0x2238,
	/* Array index: 0x0658 */ 0x5932,0x2237,
	/* Array index: 0x0658 */ 0x5934,0x2239,
	/* Array index: 0x0658 */ 0x5936,0x2347,
	/* Array index: 0x0658 */ 0x593D,0x2549,
	/* Array index: 0x0658 */ 0x5952,0x2B7C,
	/* Array index: 0x0658 */ 0x595B,0x347B,
	/* Array index: 0x0658 */ 0x595D,0x347C,
	/* Array index: 0x0658 */ 0x595E,0x347D,
	/* Array index: 0x0658 */ 0x5963,0x3A53,
	/* Array index: 0x0658 */ 0x5964,0x3A54,
	/* Array index: 0x0658 */ 0x5966,0x4060,
	/* Array index: 0x0658 */ 0x5968,0x4061,
	/* Array index: 0x0658 */ 0x596C,0x4648,
	/* Array index: 0x0658 */ 0x5975,0x223B,
	/* Array index: 0x0658 */ 0x597A,0x234B,
	/* Array index: 0x0658 */ 0x5986,0x234D,
	/* Array index: 0x0658 */ 0x5989,0x2555,
	/* Array index: 0x0658 */ 0x598C,0x254C,
	/* Array index: 0x0658 */ 0x5994,0x254B,
	/* Array index: 0x0658 */ 0x599F,0x2551,
	/* Array index: 0x0658 */ 0x59AC,0x2841,
	/* Array index: 0x0658 */ 0x59AD,0x2833,
	/* Array index: 0x0658 */ 0x59B0,0x2838,
	/* Array index: 0x0658 */ 0x59B7,0x2845,
	/* Array index: 0x0658 */ 0x59B8,0x2840,
	/* Array index: 0x0658 */ 0x59BF,0x283D,
	/* Array index: 0x0658 */ 0x59C2,0x2837,
	/* Array index: 0x0658 */ 0x59C4,0x2835,
	/* Array index: 0x0658 */ 0x59C9,0x283F,
	/* Array index: 0x0658 */ 0x59D5,0x2C26,
	/* Array index: 0x0658 */ 0x59D7,0x2846,
	/* Array index: 0x0658 */ 0x59D9,0x2C2D,
	/* Array index: 0x0658 */ 0x59DF,0x2C21,
	/* Array index: 0x0658 */ 0x59E2,0x2C2C,
	/* Array index: 0x0658 */ 0x59E7,0x2C2E,
	/* Array index: 0x0658 */ 0x59EB,0x2C23,
	/* Array index: 0x0658 */ 0x59EF,0x2C24,
	/* Array index: 0x0658 */ 0x59F0,0x2C25,
	/* Array index: 0x0658 */ 0x59F8,0x2C2B,
	/* Array index: 0x0658 */ 0x59F9,0x2C29,
	/* Array index: 0x0658 */ 0x5A02,0x2C2A,
	/* Array index: 0x0658 */ 0x5A04,0x2C28,
	/* Array index: 0x0658 */ 0x5A0B,0x3040,
	/* Array index: 0x0658 */ 0x5A0D,0x2C27,
	/* Array index: 0x0658 */ 0x5A10,0x6270,
	/* Array index: 0x0658 */ 0x5A12,0x304F,
	/* Array index: 0x0658 */ 0x5A14,0x304A,
	/* Array index: 0x0658 */ 0x5A1A,0x304E,
	/* Array index: 0x0658 */ 0x5A1D,0x303E,
	/* Array index: 0x0658 */ 0x5A21,0x303B,
	/* Array index: 0x0658 */ 0x5A22,0x3045,
	/* Array index: 0x0658 */ 0x5A24,0x3048,
	/* Array index: 0x0658 */ 0x5A26,0x3052,
	/* Array index: 0x0658 */ 0x5A2F,0x304D,
	/* Array index: 0x0658 */ 0x5A31,0x304B,
	/* Array index: 0x0658 */ 0x5A3A,0x3528,
	/* Array index: 0x0658 */ 0x5A3B,0x3524,
	/* Array index: 0x0658 */ 0x5A3D,0x3531,
	/* Array index: 0x0658 */ 0x5A3F,0x3533,
	/* Array index: 0x0658 */ 0x5A45,0x352E,
	/* Array index: 0x0658 */ 0x5A4B,0x352A,
	/* Array index: 0x0658 */ 0x5A4E,0x352F,
	/* Array index: 0x0658 */ 0x5A4F,0x353B,
	/* Array index: 0x0658 */ 0x5A54,0x353A,
	/* Array index: 0x0658 */ 0x5A59,0x3539,
	/* Array index: 0x0658 */ 0x5A61,0x3527,
	/* Array index: 0x0658 */ 0x5A63,0x353C,
	/* Array index: 0x0658 */ 0x5A68,0x3530,
	/* Array index: 0x0658 */ 0x5A6B,0x352B,
	/* Array index: 0x0658 */ 0x5A6E,0x3529,
	/* Array index: 0x0658 */ 0x5A6F,0x3534,
	/* Array index: 0x0658 */ 0x5A71,0x3532,
	/* Array index: 0x0658 */ 0x5A73,0x3537,
	/* Array index: 0x0658 */ 0x5A75,0x3535,
	/* Array index: 0x0658 */ 0x5A79,0x3A75,
	/* Array index: 0x0658 */ 0x5A8D,0x3A7B,
	/* Array index: 0x0658 */ 0x5A91,0x3A66,
	/* Array index: 0x0658 */ 0x5A96,0x3A62,
	/* Array index: 0x0658 */ 0x5A98,0x3A5C,
	/* Array index: 0x0658 */ 0x5A99,0x3A5E,
	/* Array index: 0x0658 */ 0x5AA8,0x3A56,
	/* Array index: 0x0658 */ 0x5AAA,0x3A79,
	/* Array index: 0x0658 */ 0x5AAB,0x3A78,
	/* Array index: 0x0658 */ 0x5AC3,0x4067,
	/* Array index: 0x0658 */ 0x5AC5,0x4066,
	/* Array index: 0x0658 */ 0x5AE4,0x464C,
	/* Array index: 0x0658 */ 0x5AE7,0x464A,
	/* Array index: 0x0658 */ 0x5AEF,0x464F,
	/* Array index: 0x0658 */ 0x5AF0,0x4653,
	/* Array index: 0x0658 */ 0x5AFA,0x4B5C,
	/* Array index: 0x0658 */ 0x5AFC,0x4B58,
	/* Array index: 0x0658 */ 0x5AFE,0x4B54,
	/* Array index: 0x0658 */ 0x5B00,0x4B5E,
	/* Array index: 0x0658 */ 0x5B04,0x4B56,
	/* Array index: 0x0658 */ 0x5B06,0x4B5A,
	/* Array index: 0x0658 */ 0x5B0A,0x4B5B,
	/* Array index: 0x0658 */ 0x5B0D,0x4B5D,
	/* Array index: 0x0658 */ 0x5B0E,0x4B5F,
	/* Array index: 0x0658 */ 0x5B11,0x5050,
	/* Array index: 0x0658 */ 0x5B12,0x5053,
	/* Array index: 0x0658 */ 0x5B15,0x5051,
	/* Array index: 0x0658 */ 0x5B18,0x504F,
	/* Array index: 0x0658 */ 0x5B1C,0x5054,
	/* Array index: 0x0658 */ 0x5B1F,0x504E,
	/* Array index: 0x0658 */ 0x5B22,0x5056,
	/* Array index: 0x0658 */ 0x5B29,0x5450,
	/* Array index: 0x0658 */ 0x5B2B,0x544E,
	/* Array index: 0x0658 */ 0x5B31,0x5452,
	/* Array index: 0x0658 */ 0x5B33,0x544F,
	/* Array index: 0x0658 */ 0x5B35,0x5451,
	/* Array index: 0x0658 */ 0x5B42,0x5C36,
	/* Array index: 0x0658 */ 0x5B44,0x5C39,
	/* Array index: 0x0658 */ 0x5B46,0x5C3A,
	/* Array index: 0x0658 */ 0x5B49,0x5E29,
	/* Array index: 0x0658 */ 0x5B4A,0x5F42,
	/* Array index: 0x0658 */ 0x5B4F,0x6073,
	/* Array index: 0x0658 */ 0x5B60,0x2848,
	/* Array index: 0x0658 */ 0x5B66,0x625C,
	/* Array index: 0x0658 */ 0x5B67,0x646A,
	/* Array index: 0x0658 */ 0x5B6A,0x2C2F,
	/* Array index: 0x0658 */ 0x5B74,0x4071,
	/* Array index: 0x0658 */ 0x5B76,0x4072,
	/* Array index: 0x0658 */ 0x5B79,0x5057,
	/* Array index: 0x0658 */ 0x5B7C,0x5A34,
	/* Array index: 0x0658 */ 0x5B7E,0x5C3B,
	/* Array index: 0x0658 */ 0x5B80,0x213B,
	/* Array index: 0x0658 */ 0x5B82,0x223D,
	/* Array index: 0x0658 */ 0x5B86,0x2350,
	/* Array index: 0x0658 */ 0x5B8D,0x255A,
	/* Array index: 0x0658 */ 0x5B94,0x284C,
	/* Array index: 0x0658 */ 0x5B96,0x284A,
	/* Array index: 0x0658 */ 0x5BAB,0x2C32,
	/* Array index: 0x0658 */ 0x5BB7,0x3057,
	/* Array index: 0x0658 */ 0x5BBB,0x3056,
	/* Array index: 0x0658 */ 0x5BBC,0x3055,
	/* Array index: 0x0658 */ 0x5BC3,0x3541,
	/* Array index: 0x0658 */ 0x5BC8,0x353F,
	/* Array index: 0x0658 */ 0x5BD5,0x3A7E,
	/* Array index: 0x0658 */ 0x5BD7,0x4074,
	/* Array index: 0x0658 */ 0x5BF3,0x5A35,
	/* Array index: 0x0658 */ 0x5BF7,0x5E2A,
	/* Array index: 0x0658 */ 0x5BF9,0x223E,
	/* Array index: 0x0658 */ 0x5C05,0x3058,
	/* Array index: 0x0658 */ 0x5C06,0x3059,
	/* Array index: 0x0658 */ 0x5C13,0x624A,
	/* Array index: 0x0658 */ 0x5C14,0x223F,
	/* Array index: 0x0658 */ 0x5C18,0x2352,
	/* Array index: 0x0658 */ 0x5C19,0x2851,
	/* Array index: 0x0658 */ 0x5C1B,0x2C34,
	/* Array index: 0x0658 */ 0x5C1E,0x3B24,
	/* Array index: 0x0658 */ 0x5C20,0x4078,
	/* Array index: 0x0658 */ 0x5C21,0x6525,
	/* Array index: 0x0658 */ 0x5C2B,0x255F,
	/* Array index: 0x0658 */ 0x5C2F,0x2C35,
	/* Array index: 0x0658 */ 0x5C36,0x5454,
	/* Array index: 0x0658 */ 0x5C3D,0x2354,
	/* Array index: 0x0658 */ 0x5C49,0x2854,
	/* Array index: 0x0658 */ 0x5C4A,0x2855,
	/* Array index: 0x0658 */ 0x5C52,0x305A,
	/* Array index: 0x0658 */ 0x5C53,0x305B,
	/* Array index: 0x0658 */ 0x5C57,0x646B,
	/* Array index: 0x0658 */ 0x5C5B,0x3543,
	/* Array index: 0x0658 */ 0x5C5E,0x3B26,
	/* Array index: 0x0658 */ 0x5C5F,0x3B25,
	/* Array index: 0x0658 */ 0x5C61,0x3544,
	/* Array index: 0x0658 */ 0x5C6B,0x5A38,
	/* Array index: 0x0658 */ 0x5C77,0x646C,
	/* Array index: 0x0658 */ 0x5C78,0x2355,
	/* Array index: 0x0658 */ 0x5C80,0x235A,
	/* Array index: 0x0658 */ 0x5C84,0x2561,
	/* Array index: 0x0658 */ 0x5C85,0x2566,
	/* Array index: 0x0658 */ 0x5C8E,0x2562,
	/* Array index: 0x0658 */ 0x5C9C,0x2563,
	/* Array index: 0x0658 */ 0x5C9E,0x285D,
	/* Array index: 0x0658 */ 0x5CB4,0x285E,
	/* Array index: 0x0658 */ 0x5CC1,0x2858,
	/* Array index: 0x0658 */ 0x5CCD,0x2C3F,
	/* Array index: 0x0658 */ 0x5CD1,0x2C3B,
	/* Array index: 0x0658 */ 0x5CD5,0x646D,
	/* Array index: 0x0658 */ 0x5CDC,0x2C3C,
	/* Array index: 0x0658 */ 0x5CDD,0x2C42,
	/* Array index: 0x0658 */ 0x5CE1,0x2C3E,
	/* Array index: 0x0658 */ 0x5CE2,0x2C41,
	/* Array index: 0x0658 */ 0x5CE5,0x2C43,
	/* Array index: 0x0658 */ 0x5CE6,0x2C3D,
	/* Array index: 0x0658 */ 0x5CE9,0x3063,
	/* Array index: 0x0658 */ 0x5CEB,0x305F,
	/* Array index: 0x0658 */ 0x5CEF,0x3064,
	/* Array index: 0x0658 */ 0x5CF3,0x3061,
	/* Array index: 0x0658 */ 0x5CF5,0x3062,
	/* Array index: 0x0658 */ 0x5CFA,0x305E,
	/* Array index: 0x0658 */ 0x5CFC,0x6271,
	/* Array index: 0x0658 */ 0x5CFE,0x6272,
	/* Array index: 0x0658 */ 0x5D10,0x3551,
	/* Array index: 0x0658 */ 0x5D13,0x3552,
	/* Array index: 0x0658 */ 0x5D15,0x354F,
	/* Array index: 0x0658 */ 0x5D18,0x3555,
	/* Array index: 0x0658 */ 0x5D1C,0x3B2B,
	/* Array index: 0x0658 */ 0x5D21,0x3546,
	/* Array index: 0x0658 */ 0x5D2A,0x354E,
	/* Array index: 0x0658 */ 0x5D2C,0x354A,
	/* Array index: 0x0658 */ 0x5D2F,0x3554,
	/* Array index: 0x0658 */ 0x5D3B,0x3B36,
	/* Array index: 0x0658 */ 0x5D3E,0x3B28,
	/* Array index: 0x0658 */ 0x5D44,0x3B27,
	/* Array index: 0x0658 */ 0x5D46,0x3B37,
	/* Array index: 0x0658 */ 0x5D48,0x3B2A,
	/* Array index: 0x0658 */ 0x5D4D,0x3B2E,
	/* Array index: 0x0658 */ 0x5D4F,0x3B34,
	/* Array index: 0x0658 */ 0x5D53,0x3B33,
	/* Array index: 0x0658 */ 0x5D57,0x3B31,
	/* Array index: 0x0658 */ 0x5D5B,0x3B2D,
	/* Array index: 0x0658 */ 0x5D60,0x4121,
	/* Array index: 0x0658 */ 0x5D64,0x407C,
	/* Array index: 0x0658 */ 0x5D66,0x407A,
	/* Array index: 0x0658 */ 0x5D6D,0x4079,
	/* Array index: 0x0658 */ 0x5D6E,0x407D,
	/* Array index: 0x0658 */ 0x5D75,0x646E,
	/* Array index: 0x0658 */ 0x5D78,0x465C,
	/* Array index: 0x0658 */ 0x5D7B,0x4654,
	/* Array index: 0x0658 */ 0x5D83,0x4656,
	/* Array index: 0x0658 */ 0x5D96,0x4B6B,
	/* Array index: 0x0658 */ 0x5D98,0x4B67,
	/* Array index: 0x0658 */ 0x5D9B,0x4B69,
	/* Array index: 0x0658 */ 0x5DAB,0x505B,
	/* Array index: 0x0658 */ 0x5DB3,0x505A,
	/* Array index: 0x0658 */ 0x5DB6,0x6471,
	/* Array index: 0x0658 */ 0x5DB9,0x5457,
	/* Array index: 0x0658 */ 0x5DBB,0x5459,
	/* Array index: 0x0658 */ 0x5DBE,0x5455,
	/* Array index: 0x0658 */ 0x5DBF,0x6472,
	/* Array index: 0x0658 */ 0x5DC1,0x5762,
	/* Array index: 0x0658 */ 0x5DC4,0x5A39,
	/* Array index: 0x0658 */ 0x5DC8,0x5C3D,
	/* Array index: 0x0658 */ 0x5DCA,0x5C3C,
	/* Array index: 0x0658 */ 0x5DCC,0x5C3E,
	/* Array index: 0x0658 */ 0x5DD0,0x5E2D,
	/* Array index: 0x0658 */ 0x5DD3,0x5F43,
	/* Array index: 0x0658 */ 0x5DD7,0x6030,
	/* Array index: 0x0658 */ 0x5DDB,0x213C,
	/* Array index: 0x0658 */ 0x5DE3,0x3557,
	/* Array index: 0x0658 */ 0x5DE4,0x4B6C,
	/* Array index: 0x0658 */ 0x5DF5,0x2567,
	/* Array index: 0x0658 */ 0x5DF6,0x2860,
	/* Array index: 0x0658 */ 0x5DFA,0x2C45,
	/* Array index: 0x0658 */ 0x5DFB,0x2C44,
	/* Array index: 0x0658 */ 0x5E00,0x2163,
	/* Array index: 0x0658 */ 0x5E08,0x235C,
	/* Array index: 0x0658 */ 0x5E09,0x2568,
	/* Array index: 0x0658 */ 0x5E0B,0x256B,
	/* Array index: 0x0658 */ 0x5E12,0x2862,
	/* Array index: 0x0658 */ 0x5E13,0x2861,
	/* Array index: 0x0658 */ 0x5E1E,0x2C46,
	/* Array index: 0x0658 */ 0x5E2A,0x3066,
	/* Array index: 0x0658 */ 0x5E35,0x3559,
	/* Array index: 0x0658 */ 0x5E39,0x3558,
	/* Array index: 0x0658 */ 0x5E3A,0x355A,
	/* Array index: 0x0658 */ 0x5E42,0x3979,
	/* Array index: 0x0658 */ 0x5E52,0x465D,
	/* Array index: 0x0658 */ 0x5E5A,0x4B6D,
	/* Array index: 0x0658 */ 0x5E5E,0x4B70,
	/* Array index: 0x0658 */ 0x5E65,0x6473,
	/* Array index: 0x0658 */ 0x5E71,0x6677,
	/* Array index: 0x0658 */ 0x5E77,0x2863,
	/* Array index: 0x0658 */ 0x5E7A,0x213D,
	/* Array index: 0x0658 */ 0x5E7F,0x213E,
	/* Array index: 0x0658 */ 0x5E81,0x2247,
	/* Array index: 0x0658 */ 0x5E83,0x2248,
	/* Array index: 0x0658 */ 0x5E85,0x6678,
	/* Array index: 0x0658 */ 0x5E90,0x256E,
	/* Array index: 0x0658 */ 0x5E92,0x256D,
	/* Array index: 0x0658 */ 0x5E98,0x2865,
	/* Array index: 0x0658 */ 0x5E99,0x2867,
	/* Array index: 0x0658 */ 0x5E9D,0x2868,
	/* Array index: 0x0658 */ 0x5EA1,0x2C48,
	/* Array index: 0x0658 */ 0x5EA9,0x306C,
	/* Array index: 0x0658 */ 0x5EAF,0x306B,
	/* Array index: 0x0658 */ 0x5EBA,0x3561,
	/* Array index: 0x0658 */ 0x5EBB,0x3560,
	/* Array index: 0x0658 */ 0x5EBD,0x3B3F,
	/* Array index: 0x0658 */ 0x5EBF,0x3B40,
	/* Array index: 0x0658 */ 0x5EC0,0x3B3E,
	/* Array index: 0x0658 */ 0x5ECD,0x6474,
	/* Array index: 0x0658 */ 0x5ECF,0x4661,
	/* Array index: 0x0658 */ 0x5ED0,0x4660,
	/* Array index: 0x0658 */ 0x5EEA,0x505C,
	/* Array index: 0x0658 */ 0x5EED,0x6475,
	/* Array index: 0x0658 */ 0x5EF0,0x5C3F,
	/* Array index: 0x0658 */ 0x5EF4,0x213F,
	/* Array index: 0x0658 */ 0x5EF5,0x2361,
	/* Array index: 0x0658 */ 0x5F03,0x256F,
	/* Array index: 0x0658 */ 0x5F06,0x286D,
	/* Array index: 0x0658 */ 0x5F11,0x3B42,
	/* Array index: 0x0658 */ 0x5F1E,0x2571,
	/* Array index: 0x0658 */ 0x5F21,0x286E,
	/* Array index: 0x0658 */ 0x5F25,0x2870,
	/* Array index: 0x0658 */ 0x5F2F,0x2C4D,
	/* Array index: 0x0658 */ 0x5F32,0x306F,
	/* Array index: 0x0658 */ 0x5F34,0x3562,
	/* Array index: 0x0658 */ 0x5F3E,0x3B44,
	/* Array index: 0x0658 */ 0x5F42,0x4122,
	/* Array index: 0x0658 */ 0x5F4D,0x5763,
	/* Array index: 0x0658 */ 0x5F5A,0x4123,
	/* Array index: 0x0658 */ 0x5F5B,0x505E,
	/* Array index: 0x0658 */ 0x5F5D,0x5764,
	/* Array index: 0x0658 */ 0x5F60,0x6164,
	/* Array index: 0x0658 */ 0x5F61,0x2142,
	/* Array index: 0x0658 */ 0x5F63,0x2572,
	/* Array index: 0x0658 */ 0x5F66,0x2C50,
	/* Array index: 0x0658 */ 0x5F6E,0x4124,
	/* Array index: 0x0658 */ 0x5F72,0x5F45,
	/* Array index: 0x0658 */ 0x5F83,0x2876,
	/* Array index: 0x0658 */ 0x5F84,0x2875,
	/* Array index: 0x0658 */ 0x5F93,0x3071,
	/* Array index: 0x0658 */ 0x5F94,0x6476,
	/* Array index: 0x0658 */ 0x5F9A,0x6477,
	/* Array index: 0x0658 */ 0x5FA7,0x3B4B,
	/* Array index: 0x0658 */ 0x5FB3,0x4663,
	/* Array index: 0x0658 */ 0x5FB4,0x4664,
	/* Array index: 0x0658 */ 0x5FB8,0x4B72,
	/* Array index: 0x0658 */ 0x5FBA,0x6478,
	/* Array index: 0x0658 */ 0x5FC2,0x5E2F,
	/* Array index: 0x0658 */ 0x5FC7,0x224D,
	/* Array index: 0x0658 */ 0x5FC8,0x6251,
	/* Array index: 0x0658 */ 0x5FCA,0x224C,
	/* Array index: 0x0658 */ 0x5FCE,0x2578,
	/* Array index: 0x0658 */ 0x5FD3,0x2366,
	/* Array index: 0x0658 */ 0x5FDF,0x257B,
	/* Array index: 0x0658 */ 0x5FE6,0x2576,
	/* Array index: 0x0658 */ 0x5FE7,0x2574,
	/* Array index: 0x0658 */ 0x5FE9,0x287E,
	/* Array index: 0x0658 */ 0x5FEC,0x257C,
	/* Array index: 0x0658 */ 0x5FF0,0x2622,
	/* Array index: 0x0658 */ 0x5FF2,0x2621,
	/* Array index: 0x0658 */ 0x5FF6,0x257D,
	/* Array index: 0x0658 */ 0x5FF9,0x2623,
	/* Array index: 0x0658 */ 0x5FFC,0x257A,
	/* Array index: 0x0658 */ 0x5FFE,0x2575,
	/* Array index: 0x0658 */ 0x6007,0x2878,
	/* Array index: 0x0658 */ 0x6008,0x2922,
	/* Array index: 0x0658 */ 0x6018,0x6266,
	/* Array index: 0x0658 */ 0x6023,0x2C5A,
	/* Array index: 0x0658 */ 0x6030,0x2877,
	/* Array index: 0x0658 */ 0x6031,0x2C5B,
	/* Array index: 0x0658 */ 0x6036,0x287A,
	/* Array index: 0x0658 */ 0x603D,0x2921,
	/* Array index: 0x0658 */ 0x6048,0x6267,
	/* Array index: 0x0658 */ 0x604A,0x2C60,
	/* Array index: 0x0658 */ 0x604B,0x3079,
	/* Array index: 0x0658 */ 0x6056,0x307E,
	/* Array index: 0x0658 */ 0x605C,0x2C54,
	/* Array index: 0x0658 */ 0x6060,0x2C5F,
	/* Array index: 0x0658 */ 0x6061,0x2C61,
	/* Array index: 0x0658 */ 0x607C,0x2C5C,
	/* Array index: 0x0658 */ 0x607E,0x3076,
	/* Array index: 0x0658 */ 0x608B,0x3122,
	/* Array index: 0x0658 */ 0x6093,0x3125,
	/* Array index: 0x0658 */ 0x6098,0x3568,
	/* Array index: 0x0658 */ 0x6099,0x3074,
	/* Array index: 0x0658 */ 0x609E,0x3127,
	/* Array index: 0x0658 */ 0x60A1,0x3571,
	/* Array index: 0x0658 */ 0x60A4,0x3572,
	/* Array index: 0x0658 */ 0x60AE,0x3126,
	/* Array index: 0x0658 */ 0x60B3,0x3B54,
	/* Array index: 0x0658 */ 0x60D0,0x356A,
	/* Array index: 0x0658 */ 0x60D2,0x3B56,
	/* Array index: 0x0658 */ 0x60D6,0x3B55,
	/* Array index: 0x0658 */ 0x60D7,0x356E,
	/* Array index: 0x0658 */ 0x60DE,0x3579,
	/* Array index: 0x0658 */ 0x60E3,0x3B58,
	/* Array index: 0x0658 */ 0x60E5,0x3B59,
	/* Array index: 0x0658 */ 0x60E7,0x3576,
	/* Array index: 0x0658 */ 0x60E9,0x3B5A,
	/* Array index: 0x0658 */ 0x60EA,0x3B4D,
	/* Array index: 0x0658 */ 0x60EE,0x3574,
	/* Array index: 0x0658 */ 0x60FD,0x3B5E,
	/* Array index: 0x0658 */ 0x6107,0x3B4F,
	/* Array index: 0x0658 */ 0x610C,0x3B51,
	/* Array index: 0x0658 */ 0x6111,0x3B5D,
	/* Array index: 0x0658 */ 0x6119,0x412B,
	/* Array index: 0x0658 */ 0x6125,0x6479,
	/* Array index: 0x0658 */ 0x612A,0x4129,
	/* Array index: 0x0658 */ 0x6130,0x4127,
	/* Array index: 0x0658 */ 0x6131,0x412C,
	/* Array index: 0x0658 */ 0x6133,0x466F,
	/* Array index: 0x0658 */ 0x6138,0x466E,
	/* Array index: 0x0658 */ 0x613A,0x4128,
	/* Array index: 0x0658 */ 0x613C,0x466C,
	/* Array index: 0x0658 */ 0x613D,0x412E,
	/* Array index: 0x0658 */ 0x6142,0x466D,
	/* Array index: 0x0658 */ 0x6143,0x412A,
	/* Array index: 0x0658 */ 0x6150,0x647A,
	/* Array index: 0x0658 */ 0x6157,0x4B73,
	/* Array index: 0x0658 */ 0x6159,0x4B7E,
	/* Array index: 0x0658 */ 0x615C,0x4B74,
	/* Array index: 0x0658 */ 0x6160,0x4671,
	/* Array index: 0x0658 */ 0x6164,0x4B7C,
	/* Array index: 0x0658 */ 0x6169,0x4672,
	/* Array index: 0x0658 */ 0x616F,0x466A,
	/* Array index: 0x0658 */ 0x6178,0x4C21,
	/* Array index: 0x0658 */ 0x617B,0x4668,
	/* Array index: 0x0658 */ 0x617D,0x4673,
	/* Array index: 0x0658 */ 0x6181,0x466B,
	/* Array index: 0x0658 */ 0x618F,0x4B7A,
	/* Array index: 0x0658 */ 0x619C,0x4C29,
	/* Array index: 0x0658 */ 0x619E,0x4C25,
	/* Array index: 0x0658 */ 0x61A3,0x4B79,
	/* Array index: 0x0658 */ 0x61A6,0x4B75,
	/* Array index: 0x0658 */ 0x61B7,0x5061,
	/* Array index: 0x0658 */ 0x61B9,0x5063,
	/* Array index: 0x0658 */ 0x61BB,0x506A,
	/* Array index: 0x0658 */ 0x61BD,0x5064,
	/* Array index: 0x0658 */ 0x61C0,0x5066,
	/* Array index: 0x0658 */ 0x61C4,0x506C,
	/* Array index: 0x0658 */ 0x61D3,0x6679,
	/* Array index: 0x0658 */ 0x61E2,0x545B,
	/* Array index: 0x0658 */ 0x61F3,0x5765,
	/* Array index: 0x0658 */ 0x6202,0x5F46,
	/* Array index: 0x0658 */ 0x6205,0x613E,
	/* Array index: 0x0658 */ 0x620F,0x236B,
	/* Array index: 0x0658 */ 0x6213,0x2625,
	/* Array index: 0x0658 */ 0x6218,0x2C63,
	/* Array index: 0x0658 */ 0x621E,0x3B64,
	/* Array index: 0x0658 */ 0x6228,0x4676,
	/* Array index: 0x0658 */ 0x622C,0x4675,
	/* Array index: 0x0658 */ 0x622F,0x4C2C,
	/* Array index: 0x0658 */ 0x6231,0x506D,
	/* Array index: 0x0658 */ 0x6245,0x3129,
	/* Array index: 0x0658 */ 0x624F,0x2250,
	/* Array index: 0x0658 */ 0x6257,0x236F,
	/* Array index: 0x0658 */ 0x625D,0x236C,
	/* Array index: 0x0658 */ 0x625F,0x236D,
	/* Array index: 0x0658 */ 0x6267,0x236E,
	/* Array index: 0x0658 */ 0x6275,0x2639,
	/* Array index: 0x0658 */ 0x6278,0x262E,
	/* Array index: 0x0658 */ 0x6282,0x262A,
	/* Array index: 0x0658 */ 0x6285,0x2637,
	/* Array index: 0x0658 */ 0x628B,0x262F,
	/* Array index: 0x0658 */ 0x628D,0x2636,
	/* Array index: 0x0658 */ 0x62B2,0x2926,
	/* Array index: 0x0658 */ 0x62B7,0x2928,
	/* Array index: 0x0658 */ 0x62BA,0x2925,
	/* Array index: 0x0658 */ 0x62C0,0x2934,
	/* Array index: 0x0658 */ 0x62C1,0x2932,
	/* Array index: 0x0658 */ 0x62C5,0x2933,
	/* Array index: 0x0658 */ 0x62D5,0x292E,
	/* Array index: 0x0658 */ 0x62E4,0x2929,
	/* Array index: 0x0658 */ 0x62EA,0x2C6D,
	/* Array index: 0x0658 */ 0x6304,0x2C6F,
	/* Array index: 0x0658 */ 0x6305,0x2C70,
	/* Array index: 0x0658 */ 0x6317,0x2C6C,
	/* Array index: 0x0658 */ 0x6319,0x3132,
	/* Array index: 0x0658 */ 0x631B,0x3133,
	/* Array index: 0x0658 */ 0x631F,0x2C6B,
	/* Array index: 0x0658 */ 0x6321,0x2C6E,
	/* Array index: 0x0658 */ 0x632E,0x312C,
	/* Array index: 0x0658 */ 0x6330,0x312F,
	/* Array index: 0x0658 */ 0x6331,0x3135,
	/* Array index: 0x0658 */ 0x6335,0x3138,
	/* Array index: 0x0658 */ 0x6337,0x3137,
	/* Array index: 0x0658 */ 0x633F,0x313C,
	/* Array index: 0x0658 */ 0x6352,0x312E,
	/* Array index: 0x0658 */ 0x6353,0x3139,
	/* Array index: 0x0658 */ 0x6360,0x647C,
	/* Array index: 0x0658 */ 0x6364,0x647D,
	/* Array index: 0x0658 */ 0x6366,0x3631,
	/* Array index: 0x0658 */ 0x636A,0x3627,
	/* Array index: 0x0658 */ 0x636C,0x3629,
	/* Array index: 0x0658 */ 0x6373,0x3626,
	/* Array index: 0x0658 */ 0x6374,0x3632,
	/* Array index: 0x0658 */ 0x6379,0x3622,
	/* Array index: 0x0658 */ 0x637E,0x357C,
	/* Array index: 0x0658 */ 0x637F,0x362B,
	/* Array index: 0x0658 */ 0x6386,0x3623,
	/* Array index: 0x0658 */ 0x638B,0x357D,
	/* Array index: 0x0658 */ 0x6393,0x3624,
	/* Array index: 0x0658 */ 0x63B2,0x362D,
	/* Array index: 0x0658 */ 0x63B6,0x647E,
	/* Array index: 0x0658 */ 0x63BA,0x362E,
	/* Array index: 0x0658 */ 0x63BF,0x3B71,
	/* Array index: 0x0658 */ 0x63C1,0x3B70,
	/* Array index: 0x0658 */ 0x63D1,0x3B73,
	/* Array index: 0x0658 */ 0x63DE,0x3B67,
	/* Array index: 0x0658 */ 0x63E2,0x3B66,
	/* Array index: 0x0658 */ 0x63E6,0x3B68,
	/* Array index: 0x0658 */ 0x63F7,0x3B72,
	/* Array index: 0x0658 */ 0x63F8,0x3B6D,
	/* Array index: 0x0658 */ 0x63FA,0x6339,
	/* Array index: 0x0658 */ 0x63FE,0x3B6F,
	/* Array index: 0x0658 */ 0x6403,0x6521,
	/* Array index: 0x0658 */ 0x6407,0x4149,
	/* Array index: 0x0658 */ 0x6408,0x4132,
	/* Array index: 0x0658 */ 0x6411,0x413A,
	/* Array index: 0x0658 */ 0x6419,0x4138,
	/* Array index: 0x0658 */ 0x641D,0x413E,
	/* Array index: 0x0658 */ 0x6429,0x413D,
	/* Array index: 0x0658 */ 0x6431,0x4136,
	/* Array index: 0x0658 */ 0x6432,0x4133,
	/* Array index: 0x0658 */ 0x6446,0x4144,
	/* Array index: 0x0658 */ 0x6447,0x4145,
	/* Array index: 0x0658 */ 0x6449,0x6676,
	/* Array index: 0x0658 */ 0x644C,0x4678,
	/* Array index: 0x0658 */ 0x644F,0x4728,
	/* Array index: 0x0658 */ 0x6462,0x4721,
	/* Array index: 0x0658 */ 0x6464,0x472A,
	/* Array index: 0x0658 */ 0x646A,0x4723,
	/* Array index: 0x0658 */ 0x6471,0x4722,
	/* Array index: 0x0658 */ 0x647C,0x467B,
	/* Array index: 0x0658 */ 0x647E,0x4729,
	/* Array index: 0x0658 */ 0x6480,0x4C2D,
	/* Array index: 0x0658 */ 0x6481,0x4726,
	/* Array index: 0x0658 */ 0x6483,0x4C35,
	/* Array index: 0x0658 */ 0x6486,0x4C38,
	/* Array index: 0x0658 */ 0x648D,0x4C30,
	/* Array index: 0x0658 */ 0x648E,0x4C2F,
	/* Array index: 0x0658 */ 0x6491,0x4C3B,
	/* Array index: 0x0658 */ 0x6494,0x4C31,
	/* Array index: 0x0658 */ 0x649B,0x4C2E,
	/* Array index: 0x0658 */ 0x64A1,0x4C3E,
	/* Array index: 0x0658 */ 0x64A7,0x4C3F,
	/* Array index: 0x0658 */ 0x64A8,0x4C34,
	/* Array index: 0x0658 */ 0x64AA,0x4C3D,
	/* Array index: 0x0658 */ 0x64AF,0x4C3A,
	/* Array index: 0x0658 */ 0x64B4,0x4C39,
	/* Array index: 0x0658 */ 0x64B6,0x6522,
	/* Array index: 0x0658 */ 0x64B9,0x4C37,
	/* Array index: 0x0658 */ 0x64C0,0x5070,
	/* Array index: 0x0658 */ 0x64C6,0x4C32,
	/* Array index: 0x0658 */ 0x64C8,0x5077,
	/* Array index: 0x0658 */ 0x64D1,0x5076,
	/* Array index: 0x0658 */ 0x64D3,0x506F,
	/* Array index: 0x0658 */ 0x64D5,0x5079,
	/* Array index: 0x0658 */ 0x64DC,0x5075,
	/* Array index: 0x0658 */ 0x64DF,0x5464,
	/* Array index: 0x0658 */ 0x64E1,0x5467,
	/* Array index: 0x0658 */ 0x64E5,0x576B,
	/* Array index: 0x0658 */ 0x64E7,0x576D,
	/* Array index: 0x0658 */ 0x64EA,0x576C,
	/* Array index: 0x0658 */ 0x64EE,0x5469,
	/* Array index: 0x0658 */ 0x64F6,0x576A,
	/* Array index: 0x0658 */ 0x64F9,0x5770,
	/* Array index: 0x0658 */ 0x6505,0x576E,
	/* Array index: 0x0658 */ 0x6508,0x5A3B,
	/* Array index: 0x0658 */ 0x650A,0x5A3C,
	/* Array index: 0x0658 */ 0x650B,0x5A3A,
	/* Array index: 0x0658 */ 0x6511,0x5E31,
	/* Array index: 0x0658 */ 0x651A,0x6523,
	/* Array index: 0x0658 */ 0x651E,0x5F4A,
	/* Array index: 0x0658 */ 0x651F,0x5F4B,
	/* Array index: 0x0658 */ 0x6527,0x5F49,
	/* Array index: 0x0658 */ 0x6528,0x6032,
	/* Array index: 0x0658 */ 0x6531,0x2C72,
	/* Array index: 0x0658 */ 0x6534,0x2167,
	/* Array index: 0x0658 */ 0x6535,0x2168,
	/* Array index: 0x0658 */ 0x6540,0x2C74,
	/* Array index: 0x0658 */ 0x6542,0x2C76,
	/* Array index: 0x0658 */ 0x6544,0x2C73,
	/* Array index: 0x0658 */ 0x654B,0x313D,
	/* Array index: 0x0658 */ 0x654D,0x3637,
	/* Array index: 0x0658 */ 0x654E,0x3636,
	/* Array index: 0x0658 */ 0x655A,0x3634,
	/* Array index: 0x0658 */ 0x656B,0x414B,
	/* Array index: 0x0658 */ 0x6585,0x5C41,
	/* Array index: 0x0658 */ 0x6588,0x2558,
	/* Array index: 0x0658 */ 0x6589,0x2939,
	/* Array index: 0x0658 */ 0x658B,0x3140,
	/* Array index: 0x0658 */ 0x658D,0x3638,
	/* Array index: 0x0658 */ 0x658E,0x3639,
	/* Array index: 0x0658 */ 0x659A,0x3142,
	/* Array index: 0x0658 */ 0x65A6,0x293B,
	/* Array index: 0x0658 */ 0x65AD,0x363A,
	/* Array index: 0x0658 */ 0x65B5,0x546B,
	/* Array index: 0x0658 */ 0x65BA,0x293C,
	/* Array index: 0x0658 */ 0x65BE,0x2C77,
	/* Array index: 0x0658 */ 0x65C7,0x363C,
	/* Array index: 0x0658 */ 0x65C9,0x363F,
	/* Array index: 0x0658 */ 0x65CA,0x363D,
	/* Array index: 0x0658 */ 0x65D1,0x3B79,
	/* Array index: 0x0658 */ 0x65DC,0x5A3F,
	/* Array index: 0x0658 */ 0x65E0,0x2169,
	/* Array index: 0x0658 */ 0x65E3,0x3641,
	/* Array index: 0x0658 */ 0x65E4,0x4150,
	/* Array index: 0x0658 */ 0x65E7,0x2251,
	/* Array index: 0x0658 */ 0x65EA,0x2373,
	/* Array index: 0x0658 */ 0x65EB,0x2372,
	/* Array index: 0x0658 */ 0x65F6,0x263D,
	/* Array index: 0x0658 */ 0x65F9,0x625E,
	/* Array index: 0x0658 */ 0x65FE,0x2943,
	/* Array index: 0x0658 */ 0x65FF,0x293E,
	/* Array index: 0x0658 */ 0x6601,0x2942,
	/* Array index: 0x0658 */ 0x661E,0x2D23,
	/* Array index: 0x0658 */ 0x6623,0x2C7C,
	/* Array index: 0x0658 */ 0x6629,0x2C79,
	/* Array index: 0x0658 */ 0x662C,0x2C7D,
	/* Array index: 0x0658 */ 0x6630,0x2D21,
	/* Array index: 0x0658 */ 0x6637,0x2D24,
	/* Array index: 0x0658 */ 0x6638,0x2D25,
	/* Array index: 0x0658 */ 0x663B,0x2D22,
	/* Array index: 0x0658 */ 0x663C,0x2C37,
	/* Array index: 0x0658 */ 0x6640,0x3146,
	/* Array index: 0x0658 */ 0x6644,0x314C,
	/* Array index: 0x0658 */ 0x6646,0x3144,
	/* Array index: 0x0658 */ 0x6648,0x3149,
	/* Array index: 0x0658 */ 0x664B,0x3148,
	/* Array index: 0x0658 */ 0x664D,0x314D,
	/* Array index: 0x0658 */ 0x664E,0x3145,
	/* Array index: 0x0658 */ 0x6650,0x3143,
	/* Array index: 0x0658 */ 0x6657,0x3642,
	/* Array index: 0x0658 */ 0x6660,0x314B,
	/* Array index: 0x0658 */ 0x6663,0x3644,
	/* Array index: 0x0658 */ 0x6667,0x3645,
	/* Array index: 0x0658 */ 0x6669,0x3C26,
	/* Array index: 0x0658 */ 0x666B,0x3B7D,
	/* Array index: 0x0658 */ 0x666D,0x3C25,
	/* Array index: 0x0658 */ 0x6673,0x3C23,
	/* Array index: 0x0658 */ 0x667D,0x3B7C,
	/* Array index: 0x0658 */ 0x667F,0x3B7E,
	/* Array index: 0x0658 */ 0x6681,0x3C24,
	/* Array index: 0x0658 */ 0x668E,0x415B,
	/* Array index: 0x0658 */ 0x668F,0x4156,
	/* Array index: 0x0658 */ 0x6692,0x4159,
	/* Array index: 0x0658 */ 0x6693,0x4151,
	/* Array index: 0x0658 */ 0x669A,0x4734,
	/* Array index: 0x0658 */ 0x669C,0x4735,
	/* Array index: 0x0658 */ 0x66AC,0x4C44,
	/* Array index: 0x0658 */ 0x66AD,0x4C46,
	/* Array index: 0x0658 */ 0x66B3,0x4C41,
	/* Array index: 0x0658 */ 0x66B6,0x4C40,
	/* Array index: 0x0658 */ 0x66BC,0x4C43,
	/* Array index: 0x0658 */ 0x66BF,0x507D,
	/* Array index: 0x0658 */ 0x66C1,0x5123,
	/* Array index: 0x0658 */ 0x66C3,0x507A,
	/* Array index: 0x0658 */ 0x66C5,0x507E,
	/* Array index: 0x0658 */ 0x66DF,0x5A43,
	/* Array index: 0x0658 */ 0x66E1,0x5A42,
	/* Array index: 0x0658 */ 0x66E2,0x6526,
	/* Array index: 0x0658 */ 0x66E5,0x5C42,
	/* Array index: 0x0658 */ 0x66E7,0x5C43,
	/* Array index: 0x0658 */ 0x66EF,0x613F,
	/* Array index: 0x0658 */ 0x66F5,0x2641,
	/* Array index: 0x0658 */ 0x6702,0x6527,
	/* Array index: 0x0658 */ 0x6706,0x5124,
	/* Array index: 0x0658 */ 0x670C,0x2945,
	/* Array index: 0x0658 */ 0x670E,0x2D27,
	/* Array index: 0x0658 */ 0x6711,0x6632,
	/* Array index: 0x0658 */ 0x6716,0x3648,
	/* Array index: 0x0658 */ 0x6719,0x3647,
	/* Array index: 0x0658 */ 0x671A,0x3646,
	/* Array index: 0x0658 */ 0x671E,0x3C29,
	/* Array index: 0x0658 */ 0x6724,0x5126,
	/* Array index: 0x0658 */ 0x6725,0x6633,
	/* Array index: 0x0658 */ 0x672F,0x2252,
	/* Array index: 0x0658 */ 0x6732,0x2377,
	/* Array index: 0x0658 */ 0x6736,0x2378,
	/* Array index: 0x0658 */ 0x6737,0x2375,
	/* Array index: 0x0658 */ 0x6744,0x2648,
	/* Array index: 0x0658 */ 0x674A,0x2649,
	/* Array index: 0x0658 */ 0x6752,0x2645,
	/* Array index: 0x0658 */ 0x6754,0x2644,
	/* Array index: 0x0658 */ 0x6758,0x2647,
	/* Array index: 0x0658 */ 0x675B,0x2642,
	/* Array index: 0x0658 */ 0x6761,0x264A,
	/* Array index: 0x0658 */ 0x6765,0x6257,
	/* Array index: 0x0658 */ 0x676B,0x2947,
	/* Array index: 0x0658 */ 0x6780,0x2952,
	/* Array index: 0x0658 */ 0x6782,0x2949,
	/* Array index: 0x0658 */ 0x6788,0x625F,
	/* Array index: 0x0658 */ 0x678A,0x294A,
	/* Array index: 0x0658 */ 0x678F,0x294F,
	/* Array index: 0x0658 */ 0x6796,0x2948,
	/* Array index: 0x0658 */ 0x67AC,0x6529,
	/* Array index: 0x0658 */ 0x67B1,0x2D36,
	/* Array index: 0x0658 */ 0x67BC,0x2D2E,
	/* Array index: 0x0658 */ 0x67BF,0x2D30,
	/* Array index: 0x0658 */ 0x67C7,0x2D2D,
	/* Array index: 0x0658 */ 0x67E8,0x2D2A,
	/* Array index: 0x0658 */ 0x67F9,0x2951,
	/* Array index: 0x0658 */ 0x67FB,0x2D34,
	/* Array index: 0x0658 */ 0x67FE,0x2D32,
	/* Array index: 0x0658 */ 0x6800,0x2D39,
	/* Array index: 0x0658 */ 0x6801,0x2D37,
	/* Array index: 0x0658 */ 0x6805,0x2D38,
	/* Array index: 0x0658 */ 0x6806,0x652B,
	/* Array index: 0x0658 */ 0x6810,0x652A,
	/* Array index: 0x0658 */ 0x6815,0x315D,
	/* Array index: 0x0658 */ 0x6819,0x3157,
	/* Array index: 0x0658 */ 0x681B,0x3153,
	/* Array index: 0x0658 */ 0x681E,0x315B,
	/* Array index: 0x0658 */ 0x6827,0x315F,
	/* Array index: 0x0658 */ 0x682C,0x3156,
	/* Array index: 0x0658 */ 0x6830,0x3163,
	/* Array index: 0x0658 */ 0x6836,0x3154,
	/* Array index: 0x0658 */ 0x6837,0x314F,
	/* Array index: 0x0658 */ 0x683E,0x315A,
	/* Array index: 0x0658 */ 0x6847,0x3159,
	/* Array index: 0x0658 */ 0x6852,0x2D2F,
	/* Array index: 0x0658 */ 0x685E,0x652C,
	/* Array index: 0x0658 */ 0x686C,0x3655,
	/* Array index: 0x0658 */ 0x6870,0x3660,
	/* Array index: 0x0658 */ 0x6873,0x364E,
	/* Array index: 0x0658 */ 0x687A,0x365B,
	/* Array index: 0x0658 */ 0x6884,0x3657,
	/* Array index: 0x0658 */ 0x688E,0x3651,
	/* Array index: 0x0658 */ 0x6895,0x365A,
	/* Array index: 0x0658 */ 0x689E,0x364B,
	/* Array index: 0x0658 */ 0x68A5,0x3654,
	/* Array index: 0x0658 */ 0x68A6,0x3478,
	/* Array index: 0x0658 */ 0x68C1,0x3656,
	/* Array index: 0x0658 */ 0x68C3,0x3C45,
	/* Array index: 0x0658 */ 0x68C5,0x3C41,
	/* Array index: 0x0658 */ 0x68CA,0x3C38,
	/* Array index: 0x0658 */ 0x68CF,0x3C3E,
	/* Array index: 0x0658 */ 0x68DB,0x3653,
	/* Array index: 0x0658 */ 0x68E2,0x3C3D,
	/* Array index: 0x0658 */ 0x68E5,0x3C32,
	/* Array index: 0x0658 */ 0x68ED,0x3C2A,
	/* Array index: 0x0658 */ 0x6909,0x3C37,
	/* Array index: 0x0658 */ 0x6918,0x3C3C,
	/* Array index: 0x0658 */ 0x691C,0x3C44,
	/* Array index: 0x0658 */ 0x691E,0x3C33,
	/* Array index: 0x0658 */ 0x6929,0x652F,
	/* Array index: 0x0658 */ 0x692C,0x652E,
	/* Array index: 0x0658 */ 0x692E,0x3C40,
	/* Array index: 0x0658 */ 0x6931,0x4165,
	/* Array index: 0x0658 */ 0x6936,0x4177,
	/* Array index: 0x0658 */ 0x693E,0x4168,
	/* Array index: 0x0658 */ 0x6943,0x416B,
	/* Array index: 0x0658 */ 0x6946,0x415D,
	/* Array index: 0x0658 */ 0x6947,0x4175,
	/* Array index: 0x0658 */ 0x694D,0x4173,
	/* Array index: 0x0658 */ 0x6950,0x4174,
	/* Array index: 0x0658 */ 0x6955,0x416F,
	/* Array index: 0x0658 */ 0x6961,0x4179,
	/* Array index: 0x0658 */ 0x6964,0x4178,
	/* Array index: 0x0658 */ 0x6967,0x4176,
	/* Array index: 0x0658 */ 0x6973,0x416D,
	/* Array index: 0x0658 */ 0x697C,0x416A,
	/* Array index: 0x0658 */ 0x697D,0x417B,
	/* Array index: 0x0658 */ 0x6985,0x4172,
	/* Array index: 0x0658 */ 0x698F,0x473A,
	/* Array index: 0x0658 */ 0x6992,0x473D,
	/* Array index: 0x0658 */ 0x6998,0x4751,
	/* Array index: 0x0658 */ 0x699D,0x474E,
	/* Array index: 0x0658 */ 0x699F,0x4746,
	/* Array index: 0x0658 */ 0x69A2,0x4747,
	/* Array index: 0x0658 */ 0x69B2,0x473E,
	/* Array index: 0x0658 */ 0x69BA,0x6352,
	/* Array index: 0x0658 */ 0x69C0,0x4743,
	/* Array index: 0x0658 */ 0x69C5,0x473B,
	/* Array index: 0x0658 */ 0x69C8,0x473C,
	/* Array index: 0x0658 */ 0x69D1,0x4745,
	/* Array index: 0x0658 */ 0x69D2,0x4748,
	/* Array index: 0x0658 */ 0x69D5,0x474D,
	/* Array index: 0x0658 */ 0x69D6,0x4744,
	/* Array index: 0x0658 */ 0x69D8,0x4C54,
	/* Array index: 0x0658 */ 0x69E1,0x474C,
	/* Array index: 0x0658 */ 0x69E3,0x4742,
	/* Array index: 0x0658 */ 0x69E9,0x4C64,
	/* Array index: 0x0658 */ 0x69EA,0x4C60,
	/* Array index: 0x0658 */ 0x69EF,0x4C4F,
	/* Array index: 0x0658 */ 0x69F0,0x4C51,
	/* Array index: 0x0658 */ 0x69F9,0x4C5F,
	/* Array index: 0x0658 */ 0x69FA,0x4C57,
	/* Array index: 0x0658 */ 0x6A0B,0x4C4D,
	/* Array index: 0x0658 */ 0x6A0C,0x4C50,
	/* Array index: 0x0658 */ 0x6A0E,0x4C47,
	/* Array index: 0x0658 */ 0x6A10,0x4C5B,
	/* Array index: 0x0658 */ 0x6A12,0x4C56,
	/* Array index: 0x0658 */ 0x6A1A,0x4C4A,
	/* Array index: 0x0658 */ 0x6A1C,0x4C49,
	/* Array index: 0x0658 */ 0x6A22,0x4C52,
	/* Array index: 0x0658 */ 0x6A24,0x4C62,
	/* Array index: 0x0658 */ 0x6A33,0x512C,
	/* Array index: 0x0658 */ 0x6A36,0x513C,
	/* Array index: 0x0658 */ 0x6A37,0x5138,
	/* Array index: 0x0658 */ 0x6A42,0x5128,
	/* Array index: 0x0658 */ 0x6A43,0x512B,
	/* Array index: 0x0658 */ 0x6A45,0x5140,
	/* Array index: 0x0658 */ 0x6A4A,0x513B,
	/* Array index: 0x0658 */ 0x6A4C,0x5131,
	/* Array index: 0x0658 */ 0x6A52,0x5129,
	/* Array index: 0x0658 */ 0x6A53,0x513E,
	/* Array index: 0x0658 */ 0x6A57,0x512F,
	/* Array index: 0x0658 */ 0x6A5C,0x5144,
	/* Array index: 0x0658 */ 0x6A63,0x5127,
	/* Array index: 0x0658 */ 0x6A6C,0x512E,
	/* Array index: 0x0658 */ 0x6A6E,0x5132,
	/* Array index: 0x0658 */ 0x6A70,0x5141,
	/* Array index: 0x0658 */ 0x6A71,0x513A,
	/* Array index: 0x0658 */ 0x6A77,0x6531,
	/* Array index: 0x0658 */ 0x6A7A,0x6532,
	/* Array index: 0x0658 */ 0x6A7D,0x5475,
	/* Array index: 0x0658 */ 0x6A82,0x547B,
	/* Array index: 0x0658 */ 0x6A86,0x5521,
	/* Array index: 0x0658 */ 0x6A88,0x547C,
	/* Array index: 0x0658 */ 0x6A8A,0x5476,
	/* Array index: 0x0658 */ 0x6A8F,0x5527,
	/* Array index: 0x0658 */ 0x6A98,0x5523,
	/* Array index: 0x0658 */ 0x6A99,0x547A,
	/* Array index: 0x0658 */ 0x6A9D,0x5524,
	/* Array index: 0x0658 */ 0x6AA7,0x5478,
	/* Array index: 0x0658 */ 0x6AAA,0x5529,
	/* Array index: 0x0658 */ 0x6AAB,0x5774,
	/* Array index: 0x0658 */ 0x6AB5,0x5778,
	/* Array index: 0x0658 */ 0x6AC4,0x5822,
	/* Array index: 0x0658 */ 0x6AC8,0x577B,
	/* Array index: 0x0658 */ 0x6ACA,0x6533,
	/* Array index: 0x0658 */ 0x6ACE,0x5A44,
	/* Array index: 0x0658 */ 0x6AD2,0x5A4F,
	/* Array index: 0x0658 */ 0x6AE2,0x5A48,
	/* Array index: 0x0658 */ 0x6AE3,0x5A46,
	/* Array index: 0x0658 */ 0x6AE6,0x6534,
	/* Array index: 0x0658 */ 0x6AE9,0x5C49,
	/* Array index: 0x0658 */ 0x6AED,0x5775,
	/* Array index: 0x0658 */ 0x6AF4,0x5C47,
	/* Array index: 0x0658 */ 0x6AF5,0x6535,
	/* Array index: 0x0658 */ 0x6B06,0x5F4F,
	/* Array index: 0x0658 */ 0x6B07,0x5F4E,
	/* Array index: 0x0658 */ 0x6B0D,0x6536,
	/* Array index: 0x0658 */ 0x6B0E,0x6537,
	/* Array index: 0x0658 */ 0x6B14,0x6074,
	/* Array index: 0x0658 */ 0x6B22,0x2379,
	/* Array index: 0x0658 */ 0x6B2E,0x3165,
	/* Array index: 0x0658 */ 0x6B30,0x3167,
	/* Array index: 0x0658 */ 0x6B35,0x3662,
	/* Array index: 0x0658 */ 0x6B40,0x417D,
	/* Array index: 0x0658 */ 0x6B44,0x417C,
	/* Array index: 0x0658 */ 0x6B4F,0x4C66,
	/* Array index: 0x0658 */ 0x6B52,0x4C65,
	/* Array index: 0x0658 */ 0x6B53,0x4C67,
	/* Array index: 0x0658 */ 0x6B57,0x5146,
	/* Array index: 0x0658 */ 0x6B58,0x5145,
	/* Array index: 0x0658 */ 0x6B5D,0x552B,
	/* Array index: 0x0658 */ 0x6B7A,0x2253,
	/* Array index: 0x0658 */ 0x6B81,0x2958,
	/* Array index: 0x0658 */ 0x6B8B,0x316A,
	/* Array index: 0x0658 */ 0x6B90,0x3664,
	/* Array index: 0x0658 */ 0x6B9C,0x4222,
	/* Array index: 0x0658 */ 0x6B9D,0x6354,
	/* Array index: 0x0658 */ 0x6BA1,0x4754,
	/* Array index: 0x0658 */ 0x6BB4,0x2959,
	/* Array index: 0x0658 */ 0x6BBB,0x3665,
	/* Array index: 0x0658 */ 0x6BBE,0x3C4B,
	/* Array index: 0x0658 */ 0x6BC1,0x4226,
	/* Array index: 0x0658 */ 0x6BCE,0x237A,
	/* Array index: 0x0658 */ 0x6BD1,0x295A,
	/* Array index: 0x0658 */ 0x6BDC,0x6538,
	/* Array index: 0x0658 */ 0x6BDD,0x6539,
	/* Array index: 0x0658 */ 0x6BE1,0x2D42,
	/* Array index: 0x0658 */ 0x6BE5,0x316E,
	/* Array index: 0x0658 */ 0x6BE9,0x316C,
	/* Array index: 0x0658 */ 0x6BEA,0x316D,
	/* Array index: 0x0658 */ 0x6BED,0x3666,
	/* Array index: 0x0658 */ 0x6BF4,0x3C4D,
	/* Array index: 0x0658 */ 0x6BF6,0x653A,
	/* Array index: 0x0658 */ 0x6BFA,0x4228,
	/* Array index: 0x0658 */ 0x6C0A,0x552D,
	/* Array index: 0x0658 */ 0x6C1C,0x295D,
	/* Array index: 0x0658 */ 0x6C1E,0x653B,
	/* Array index: 0x0658 */ 0x6C2D,0x3C4E,
	/* Array index: 0x0658 */ 0x6C31,0x4229,
	/* Array index: 0x0658 */ 0x6C32,0x422A,
	/* Array index: 0x0658 */ 0x6C37,0x2257,
	/* Array index: 0x0658 */ 0x6C39,0x2254,
	/* Array index: 0x0658 */ 0x6C3C,0x2421,
	/* Array index: 0x0658 */ 0x6C44,0x2258,
	/* Array index: 0x0658 */ 0x6C45,0x2259,
	/* Array index: 0x0658 */ 0x6C51,0x237D,
	/* Array index: 0x0658 */ 0x6C53,0x2425,
	/* Array index: 0x0658 */ 0x6C56,0x2655,
	/* Array index: 0x0658 */ 0x6C58,0x237C,
	/* Array index: 0x0658 */ 0x6C5A,0x2423,
	/* Array index: 0x0658 */ 0x6C63,0x653C,
	/* Array index: 0x0658 */ 0x6C6C,0x2964,
	/* Array index: 0x0658 */ 0x6C6E,0x2650,
	/* Array index: 0x0658 */ 0x6C75,0x2653,
	/* Array index: 0x0658 */ 0x6C77,0x237E,
	/* Array index: 0x0658 */ 0x6C79,0x2657,
	/* Array index: 0x0658 */ 0x6C7C,0x2651,
	/* Array index: 0x0658 */ 0x6C7F,0x264C,
	/* Array index: 0x0658 */ 0x6C91,0x264D,
	/* Array index: 0x0658 */ 0x6C97,0x2963,
	/* Array index: 0x0658 */ 0x6CAA,0x265B,
	/* Array index: 0x0658 */ 0x6CAF,0x2D4E,
	/* Array index: 0x0658 */ 0x6CB2,0x296B,
	/* Array index: 0x0658 */ 0x6CC8,0x296D,
	/* Array index: 0x0658 */ 0x6CCB,0x6261,
	/* Array index: 0x0658 */ 0x6CCE,0x296C,
	/* Array index: 0x0658 */ 0x6CD8,0x296A,
	/* Array index: 0x0658 */ 0x6CDF,0x2965,
	/* Array index: 0x0658 */ 0x6CE4,0x2969,
	/* Array index: 0x0658 */ 0x6CE6,0x6260,
	/* Array index: 0x0658 */ 0x6CEA,0x2967,
	/* Array index: 0x0658 */ 0x6CF4,0x2D4F,
	/* Array index: 0x0658 */ 0x6CFF,0x2D46,
	/* Array index: 0x0658 */ 0x6D02,0x2D50,
	/* Array index: 0x0658 */ 0x6D05,0x2D48,
	/* Array index: 0x0658 */ 0x6D06,0x2D4A,
	/* Array index: 0x0658 */ 0x6D1C,0x317A,
	/* Array index: 0x0658 */ 0x6D21,0x2D4B,
	/* Array index: 0x0658 */ 0x6D23,0x2D45,
	/* Array index: 0x0658 */ 0x6D24,0x2D57,
	/* Array index: 0x0658 */ 0x6D26,0x2D53,
	/* Array index: 0x0658 */ 0x6D44,0x2D55,
	/* Array index: 0x0658 */ 0x6D45,0x2D51,
	/* Array index: 0x0658 */ 0x6D56,0x3177,
	/* Array index: 0x0658 */ 0x6D57,0x3173,
	/* Array index: 0x0658 */ 0x6D6B,0x316F,
	/* Array index: 0x0658 */ 0x6D81,0x3221,
	/* Array index: 0x0658 */ 0x6D8F,0x3178,
	/* Array index: 0x0658 */ 0x6D96,0x3227,
	/* Array index: 0x0658 */ 0x6DA5,0x653D,
	/* Array index: 0x0658 */ 0x6DAD,0x366F,
	/* Array index: 0x0658 */ 0x6DB0,0x367A,
	/* Array index: 0x0658 */ 0x6DB1,0x3721,
	/* Array index: 0x0658 */ 0x6DB6,0x3C50,
	/* Array index: 0x0658 */ 0x6DB9,0x3672,
	/* Array index: 0x0658 */ 0x6DC1,0x366A,
	/* Array index: 0x0658 */ 0x6DC3,0x366B,
	/* Array index: 0x0658 */ 0x6DCE,0x366C,
	/* Array index: 0x0658 */ 0x6DE7,0x3674,
	/* Array index: 0x0658 */ 0x6DF8,0x367C,
	/* Array index: 0x0658 */ 0x6E0F,0x653E,
	/* Array index: 0x0658 */ 0x6E12,0x3729,
	/* Array index: 0x0658 */ 0x6E17,0x3727,
	/* Array index: 0x0658 */ 0x6E18,0x3C54,
	/* Array index: 0x0658 */ 0x6E1E,0x3C52,
	/* Array index: 0x0658 */ 0x6E29,0x3C6B,
	/* Array index: 0x0658 */ 0x6E2A,0x3C6D,
	/* Array index: 0x0658 */ 0x6E42,0x3C5A,
	/* Array index: 0x0658 */ 0x6E48,0x3C56,
	/* Array index: 0x0658 */ 0x6E4C,0x3C6E,
	/* Array index: 0x0658 */ 0x6E4F,0x3C58,
	/* Array index: 0x0658 */ 0x6E50,0x3C69,
	/* Array index: 0x0658 */ 0x6E57,0x3C66,
	/* Array index: 0x0658 */ 0x6E59,0x3C64,
	/* Array index: 0x0658 */ 0x6E6A,0x3C5B,
	/* Array index: 0x0658 */ 0x6E6C,0x423A,
	/* Array index: 0x0658 */ 0x6E6D,0x3C60,
	/* Array index: 0x0658 */ 0x6E70,0x3C5C,
	/* Array index: 0x0658 */ 0x6E75,0x3C51,
	/* Array index: 0x0658 */ 0x6E76,0x3C6C,
	/* Array index: 0x0658 */ 0x6E7B,0x3C62,
	/* Array index: 0x0658 */ 0x6E7C,0x6542,
	/* Array index: 0x0658 */ 0x6E7E,0x3C63,
	/* Array index: 0x0658 */ 0x6E80,0x3C68,
	/* Array index: 0x0658 */ 0x6E84,0x6540,
	/* Array index: 0x0658 */ 0x6E86,0x633D,
	/* Array index: 0x0658 */ 0x6E8A,0x653F,
	/* Array index: 0x0658 */ 0x6E8B,0x6541,
	/* Array index: 0x0658 */ 0x6E91,0x4230,
	/* Array index: 0x0658 */ 0x6E9A,0x4233,
	/* Array index: 0x0658 */ 0x6EA8,0x422E,
	/* Array index: 0x0658 */ 0x6EA9,0x4236,
	/* Array index: 0x0658 */ 0x6EAC,0x4240,
	/* Array index: 0x0658 */ 0x6EB5,0x4239,
	/* Array index: 0x0658 */ 0x6EB8,0x422D,
	/* Array index: 0x0658 */ 0x6EBB,0x4231,
	/* Array index: 0x0658 */ 0x6EE3,0x4245,
	/* Array index: 0x0658 */ 0x6EE6,0x423F,
	/* Array index: 0x0658 */ 0x6EE8,0x423B,
	/* Array index: 0x0658 */ 0x6EE9,0x4246,
	/* Array index: 0x0658 */ 0x6EF0,0x4756,
	/* Array index: 0x0658 */ 0x6EF3,0x4757,
	/* Array index: 0x0658 */ 0x6EFA,0x4765,
	/* Array index: 0x0658 */ 0x6F04,0x4763,
	/* Array index: 0x0658 */ 0x6F0B,0x4C78,
	/* Array index: 0x0658 */ 0x6F0C,0x475B,
	/* Array index: 0x0658 */ 0x6F10,0x4C6A,
	/* Array index: 0x0658 */ 0x6F11,0x4764,
	/* Array index: 0x0658 */ 0x6F17,0x475F,
	/* Array index: 0x0658 */ 0x6F1B,0x475A,
	/* Array index: 0x0658 */ 0x6F1D,0x475C,
	/* Array index: 0x0658 */ 0x6F28,0x475E,
	/* Array index: 0x0658 */ 0x6F34,0x475D,
	/* Array index: 0x0658 */ 0x6F3D,0x4C74,
	/* Array index: 0x0658 */ 0x6F42,0x4762,
	/* Array index: 0x0658 */ 0x6F44,0x4761,
	/* Array index: 0x0658 */ 0x6F59,0x4C7D,
	/* Array index: 0x0658 */ 0x6F5C,0x4C73,
	/* Array index: 0x0658 */ 0x6F65,0x4C6B,
	/* Array index: 0x0658 */ 0x6F71,0x4C75,
	/* Array index: 0x0658 */ 0x6F74,0x4C7E,
	/* Array index: 0x0658 */ 0x6F75,0x4C6C,
	/* Array index: 0x0658 */ 0x6F79,0x4C79,
	/* Array index: 0x0658 */ 0x6F81,0x4C7A,
	/* Array index: 0x0658 */ 0x6F8A,0x6363,
	/* Array index: 0x0658 */ 0x6F8F,0x4C7B,
	/* Array index: 0x0658 */ 0x6F91,0x4C77,
	/* Array index: 0x0658 */ 0x6F99,0x6547,
	/* Array index: 0x0658 */ 0x6F9D,0x6546,
	/* Array index: 0x0658 */ 0x6F9F,0x5156,
	/* Array index: 0x0658 */ 0x6FB5,0x5154,
	/* Array index: 0x0658 */ 0x6FB7,0x5158,
	/* Array index: 0x0658 */ 0x6FBB,0x514D,
	/* Array index: 0x0658 */ 0x6FBE,0x5150,
	/* Array index: 0x0658 */ 0x6FD0,0x4C71,
	/* Array index: 0x0658 */ 0x6FD3,0x5155,
	/* Array index: 0x0658 */ 0x6FE5,0x5531,
	/* Array index: 0x0658 */ 0x6FEA,0x5537,
	/* Array index: 0x0658 */ 0x6FF3,0x5A55,
	/* Array index: 0x0658 */ 0x6FF5,0x5159,
	/* Array index: 0x0658 */ 0x6FF6,0x5539,
	/* Array index: 0x0658 */ 0x6FF8,0x6548,
	/* Array index: 0x0658 */ 0x6FFD,0x5828,
	/* Array index: 0x0658 */ 0x7002,0x582A,
	/* Array index: 0x0658 */ 0x7003,0x5827,
	/* Array index: 0x0658 */ 0x7008,0x5826,
	/* Array index: 0x0658 */ 0x7010,0x5829,
	/* Array index: 0x0658 */ 0x7013,0x582B,
	/* Array index: 0x0658 */ 0x701E,0x5A52,
	/* Array index: 0x0658 */ 0x7025,0x5A54,
	/* Array index: 0x0658 */ 0x7036,0x5C51,
	/* Array index: 0x0658 */ 0x703D,0x5C4F,
	/* Array index: 0x0658 */ 0x7047,0x5E3C,
	/* Array index: 0x0658 */ 0x704B,0x5E3A,
	/* Array index: 0x0658 */ 0x704D,0x5E3B,
	/* Array index: 0x0658 */ 0x7053,0x6036,
	/* Array index: 0x0658 */ 0x7054,0x5F52,
	/* Array index: 0x0658 */ 0x7059,0x6037,
	/* Array index: 0x0658 */ 0x705C,0x654B,
	/* Array index: 0x0658 */ 0x7067,0x6179,
	/* Array index: 0x0658 */ 0x706E,0x2428,
	/* Array index: 0x0658 */ 0x706F,0x2426,
	/* Array index: 0x0658 */ 0x7072,0x2427,
	/* Array index: 0x0658 */ 0x7073,0x242B,
	/* Array index: 0x0658 */ 0x7075,0x2662,
	/* Array index: 0x0658 */ 0x7077,0x265F,
	/* Array index: 0x0658 */ 0x7079,0x265E,
	/* Array index: 0x0658 */ 0x707B,0x2663,
	/* Array index: 0x0658 */ 0x707E,0x2660,
	/* Array index: 0x0658 */ 0x7081,0x2977,
	/* Array index: 0x0658 */ 0x70A0,0x2D5E,
	/* Array index: 0x0658 */ 0x70B2,0x2D64,
	/* Array index: 0x0658 */ 0x70B6,0x2D63,
	/* Array index: 0x0658 */ 0x70B9,0x2D68,
	/* Array index: 0x0658 */ 0x70BB,0x2D5D,
	/* Array index: 0x0658 */ 0x70BF,0x654D,
	/* Array index: 0x0658 */ 0x70C4,0x3228,
	/* Array index: 0x0658 */ 0x70C9,0x6279,
	/* Array index: 0x0658 */ 0x70CC,0x322A,
	/* Array index: 0x0658 */ 0x70D0,0x322B,
	/* Array index: 0x0658 */ 0x70D5,0x3231,
	/* Array index: 0x0658 */ 0x70D6,0x322F,
	/* Array index: 0x0658 */ 0x70DB,0x3229,
	/* Array index: 0x0658 */ 0x70DF,0x322D,
	/* Array index: 0x0658 */ 0x70E3,0x322C,
	/* Array index: 0x0658 */ 0x70E5,0x654F,
	/* Array index: 0x0658 */ 0x70EA,0x654E,
	/* Array index: 0x0658 */ 0x70EE,0x3230,
	/* Array index: 0x0658 */ 0x70F1,0x3730,
	/* Array index: 0x0658 */ 0x70F2,0x3732,
	/* Array index: 0x0658 */ 0x70F5,0x372F,
	/* Array index: 0x0658 */ 0x70FE,0x3735,
	/* Array index: 0x0658 */ 0x7101,0x372E,
	/* Array index: 0x0658 */ 0x7103,0x372C,
	/* Array index: 0x0658 */ 0x7107,0x372D,
	/* Array index: 0x0658 */ 0x7108,0x3731,
	/* Array index: 0x0658 */ 0x710F,0x3733,
	/* Array index: 0x0658 */ 0x7111,0x6550,
	/* Array index: 0x0658 */ 0x7112,0x6551,
	/* Array index: 0x0658 */ 0x711D,0x3C74,
	/* Array index: 0x0658 */ 0x7124,0x3C7D,
	/* Array index: 0x0658 */ 0x7127,0x3C7C,
	/* Array index: 0x0658 */ 0x7145,0x4257,
	/* Array index: 0x0658 */ 0x7148,0x424C,
	/* Array index: 0x0658 */ 0x714A,0x424E,
	/* Array index: 0x0658 */ 0x714F,0x4251,
	/* Array index: 0x0658 */ 0x7151,0x4258,
	/* Array index: 0x0658 */ 0x7157,0x4252,
	/* Array index: 0x0658 */ 0x715B,0x483C,
	/* Array index: 0x0658 */ 0x716B,0x424F,
	/* Array index: 0x0658 */ 0x716D,0x4259,
	/* Array index: 0x0658 */ 0x716F,0x4249,
	/* Array index: 0x0658 */ 0x7171,0x6559,
	/* Array index: 0x0658 */ 0x717C,0x476F,
	/* Array index: 0x0658 */ 0x717E,0x476E,
	/* Array index: 0x0658 */ 0x717F,0x476C,
	/* Array index: 0x0658 */ 0x7183,0x4770,
	/* Array index: 0x0658 */ 0x7188,0x4772,
	/* Array index: 0x0658 */ 0x7191,0x4767,
	/* Array index: 0x0658 */ 0x7193,0x655B,
	/* Array index: 0x0658 */ 0x7196,0x655A,
	/* Array index: 0x0658 */ 0x71A2,0x4D2A,
	/* Array index: 0x0658 */ 0x71A3,0x4D24,
	/* Array index: 0x0658 */ 0x71A6,0x4D29,
	/* Array index: 0x0658 */ 0x71AB,0x4D28,
	/* Array index: 0x0658 */ 0x71AD,0x4D25,
	/* Array index: 0x0658 */ 0x71AE,0x4D22,
	/* Array index: 0x0658 */ 0x71B4,0x655C,
	/* Array index: 0x0658 */ 0x71B6,0x5162,
	/* Array index: 0x0658 */ 0x71B7,0x515A,
	/* Array index: 0x0658 */ 0x71BA,0x5160,
	/* Array index: 0x0658 */ 0x71BB,0x515C,
	/* Array index: 0x0658 */ 0x71CC,0x5163,
	/* Array index: 0x0658 */ 0x71CD,0x6374,
	/* Array index: 0x0658 */ 0x71D1,0x515E,
	/* Array index: 0x0658 */ 0x71D3,0x5166,
	/* Array index: 0x0658 */ 0x71DD,0x655D,
	/* Array index: 0x0658 */ 0x71DE,0x655E,
	/* Array index: 0x0658 */ 0x71E3,0x553C,
	/* Array index: 0x0658 */ 0x71EF,0x5541,
	/* Array index: 0x0658 */ 0x71F3,0x5542,
	/* Array index: 0x0658 */ 0x71FA,0x582D,
	/* Array index: 0x0658 */ 0x7200,0x582E,
	/* Array index: 0x0658 */ 0x7204,0x5A56,
	/* Array index: 0x0658 */ 0x7208,0x5A57,
	/* Array index: 0x0658 */ 0x720B,0x5C59,
	/* Array index: 0x0658 */ 0x721C,0x5F53,
	/* Array index: 0x0658 */ 0x7220,0x5F54,
	/* Array index: 0x0658 */ 0x7221,0x6039,
	/* Array index: 0x0658 */ 0x7224,0x6143,
	/* Array index: 0x0658 */ 0x7225,0x6142,
	/* Array index: 0x0658 */ 0x722E,0x2D69,
	/* Array index: 0x0658 */ 0x7232,0x3D22,
	/* Array index: 0x0658 */ 0x723C,0x2D6B,
	/* Array index: 0x0658 */ 0x7240,0x297C,
	/* Array index: 0x0658 */ 0x724E,0x425D,
	/* Array index: 0x0658 */ 0x7250,0x425C,
	/* Array index: 0x0658 */ 0x7251,0x425B,
	/* Array index: 0x0658 */ 0x7255,0x4D2E,
	/* Array index: 0x0658 */ 0x7257,0x4D2D,
	/* Array index: 0x0658 */ 0x726D,0x2D6D,
	/* Array index: 0x0658 */ 0x727A,0x3236,
	/* Array index: 0x0658 */ 0x7282,0x3D24,
	/* Array index: 0x0658 */ 0x7283,0x3D23,
	/* Array index: 0x0658 */ 0x7287,0x3D25,
	/* Array index: 0x0658 */ 0x7294,0x4775,
	/* Array index: 0x0658 */ 0x7299,0x4D2F,
	/* Array index: 0x0658 */ 0x72AB,0x617B,
	/* Array index: 0x0658 */ 0x72B1,0x242C,
	/* Array index: 0x0658 */ 0x72B2,0x242D,
	/* Array index: 0x0658 */ 0x72B6,0x266B,
	/* Array index: 0x0658 */ 0x72B9,0x2669,
	/* Array index: 0x0658 */ 0x72BB,0x2666,
	/* Array index: 0x0658 */ 0x72BC,0x2667,
	/* Array index: 0x0658 */ 0x72BE,0x266A,
	/* Array index: 0x0658 */ 0x72C7,0x2668,
	/* Array index: 0x0658 */ 0x72CD,0x2A26,
	/* Array index: 0x0658 */ 0x72CF,0x2A2A,
	/* Array index: 0x0658 */ 0x72D3,0x2A27,
	/* Array index: 0x0658 */ 0x72DB,0x2A28,
	/* Array index: 0x0658 */ 0x72E2,0x2D75,
	/* Array index: 0x0658 */ 0x72E5,0x2D74,
	/* Array index: 0x0658 */ 0x72E7,0x2D70,
	/* Array index: 0x0658 */ 0x72EC,0x2D73,
	/* Array index: 0x0658 */ 0x72ED,0x2D71,
	/* Array index: 0x0658 */ 0x72F5,0x3238,
	/* Array index: 0x0658 */ 0x7302,0x3239,
	/* Array index: 0x0658 */ 0x7306,0x3D26,
	/* Array index: 0x0658 */ 0x730E,0x373B,
	/* Array index: 0x0658 */ 0x7310,0x373A,
	/* Array index: 0x0658 */ 0x731A,0x3739,
	/* Array index: 0x0658 */ 0x7324,0x3D27,
	/* Array index: 0x0658 */ 0x7338,0x3D28,
	/* Array index: 0x0658 */ 0x7341,0x425F,
	/* Array index: 0x0658 */ 0x734B,0x4D30,
	/* Array index: 0x0658 */ 0x734F,0x4779,
	/* Array index: 0x0658 */ 0x7371,0x5549,
	/* Array index: 0x0658 */ 0x738C,0x225A,
	/* Array index: 0x0658 */ 0x738D,0x216B,
	/* Array index: 0x0658 */ 0x738F,0x242F,
	/* Array index: 0x0658 */ 0x7398,0x266E,
	/* Array index: 0x0658 */ 0x739C,0x2A33,
	/* Array index: 0x0658 */ 0x739E,0x2A2D,
	/* Array index: 0x0658 */ 0x73A3,0x2A2C,
	/* Array index: 0x0658 */ 0x73A7,0x2A2B,
	/* Array index: 0x0658 */ 0x73AA,0x2A32,
	/* Array index: 0x0658 */ 0x73AF,0x2A2F,
	/* Array index: 0x0658 */ 0x73BD,0x2D78,
	/* Array index: 0x0658 */ 0x73C1,0x2D7B,
	/* Array index: 0x0658 */ 0x73C4,0x2D77,
	/* Array index: 0x0658 */ 0x73C9,0x2D7A,
	/* Array index: 0x0658 */ 0x73D5,0x323E,
	/* Array index: 0x0658 */ 0x73DF,0x3240,
	/* Array index: 0x0658 */ 0x73E1,0x3247,
	/* Array index: 0x0658 */ 0x73E2,0x323C,
	/* Array index: 0x0658 */ 0x73E4,0x3246,
	/* Array index: 0x0658 */ 0x73E6,0x3241,
	/* Array index: 0x0658 */ 0x73EC,0x323D,
	/* Array index: 0x0658 */ 0x73EF,0x6564,
	/* Array index: 0x0658 */ 0x73F3,0x373D,
	/* Array index: 0x0658 */ 0x73F7,0x3D38,
	/* Array index: 0x0658 */ 0x73F9,0x323F,
	/* Array index: 0x0658 */ 0x73FB,0x3740,
	/* Array index: 0x0658 */ 0x7402,0x373C,
	/* Array index: 0x0658 */ 0x7412,0x6565,
	/* Array index: 0x0658 */ 0x741C,0x3D39,
	/* Array index: 0x0658 */ 0x741E,0x426D,
	/* Array index: 0x0658 */ 0x741F,0x3D35,
	/* Array index: 0x0658 */ 0x7427,0x4264,
	/* Array index: 0x0658 */ 0x7443,0x426F,
	/* Array index: 0x0658 */ 0x744C,0x4273,
	/* Array index: 0x0658 */ 0x7453,0x4267,
	/* Array index: 0x0658 */ 0x7456,0x426B,
	/* Array index: 0x0658 */ 0x7458,0x4270,
	/* Array index: 0x0658 */ 0x745D,0x426A,
	/* Array index: 0x0658 */ 0x7460,0x4825,
	/* Array index: 0x0658 */ 0x7461,0x477E,
	/* Array index: 0x0658 */ 0x7465,0x477C,
	/* Array index: 0x0658 */ 0x7466,0x477D,
	/* Array index: 0x0658 */ 0x7468,0x4822,
	/* Array index: 0x0658 */ 0x746B,0x4821,
	/* Array index: 0x0658 */ 0x746C,0x477B,
	/* Array index: 0x0658 */ 0x7474,0x4827,
	/* Array index: 0x0658 */ 0x7476,0x4823,
	/* Array index: 0x0658 */ 0x747A,0x4D31,
	/* Array index: 0x0658 */ 0x747B,0x4D3B,
	/* Array index: 0x0658 */ 0x7482,0x4D39,
	/* Array index: 0x0658 */ 0x7484,0x4D36,
	/* Array index: 0x0658 */ 0x748C,0x4D35,
	/* Array index: 0x0658 */ 0x748D,0x6568,
	/* Array index: 0x0658 */ 0x7493,0x4D3A,
	/* Array index: 0x0658 */ 0x7496,0x516A,
	/* Array index: 0x0658 */ 0x7499,0x6375,
	/* Array index: 0x0658 */ 0x749B,0x5167,
	/* Array index: 0x0658 */ 0x749D,0x516C,
	/* Array index: 0x0658 */ 0x74A2,0x516B,
	/* Array index: 0x0658 */ 0x74A4,0x6567,
	/* Array index: 0x0658 */ 0x74AC,0x554F,
	/* Array index: 0x0658 */ 0x74AE,0x554B,
	/* Array index: 0x0658 */ 0x74B3,0x554D,
	/* Array index: 0x0658 */ 0x74B4,0x6569,
	/* Array index: 0x0658 */ 0x74B9,0x582F,
	/* Array index: 0x0658 */ 0x74BC,0x5830,
	/* Array index: 0x0658 */ 0x74C4,0x5A5A,
	/* Array index: 0x0658 */ 0x74EB,0x2E22,
	/* Array index: 0x0658 */ 0x74ED,0x2E21,
	/* Array index: 0x0658 */ 0x74EF,0x2E24,
	/* Array index: 0x0658 */ 0x74F3,0x3248,
	/* Array index: 0x0658 */ 0x74F9,0x3D3D,
	/* Array index: 0x0658 */ 0x7501,0x4277,
	/* Array index: 0x0658 */ 0x7506,0x4828,
	/* Array index: 0x0658 */ 0x7509,0x4D3D,
	/* Array index: 0x0658 */ 0x750A,0x516D,
	/* Array index: 0x0658 */ 0x750E,0x516E,
	/* Array index: 0x0658 */ 0x751B,0x3744,
	/* Array index: 0x0658 */ 0x751E,0x4278,
	/* Array index: 0x0658 */ 0x7523,0x3745,
	/* Array index: 0x0658 */ 0x7524,0x3D3E,
	/* Array index: 0x0658 */ 0x753B,0x2A37,
	/* Array index: 0x0658 */ 0x7541,0x2A38,
	/* Array index: 0x0658 */ 0x7542,0x2A35,
	/* Array index: 0x0658 */ 0x7544,0x2A36,
	/* Array index: 0x0658 */ 0x7546,0x2E27,
	/* Array index: 0x0658 */ 0x7549,0x2E25,
	/* Array index: 0x0658 */ 0x754A,0x2E28,
	/* Array index: 0x0658 */ 0x754D,0x2E2A,
	/* Array index: 0x0658 */ 0x7550,0x2E26,
	/* Array index: 0x0658 */ 0x755E,0x3251,
	/* Array index: 0x0658 */ 0x7561,0x3746,
	/* Array index: 0x0658 */ 0x7567,0x3749,
	/* Array index: 0x0658 */ 0x7568,0x3747,
	/* Array index: 0x0658 */ 0x756D,0x3D43,
	/* Array index: 0x0658 */ 0x756E,0x3D42,
	/* Array index: 0x0658 */ 0x7571,0x3D44,
	/* Array index: 0x0658 */ 0x757A,0x427B,
	/* Array index: 0x0658 */ 0x7581,0x5170,
	/* Array index: 0x0658 */ 0x7583,0x5552,
	/* Array index: 0x0658 */ 0x7589,0x5C62,
	/* Array index: 0x0658 */ 0x758E,0x3D45,
	/* Array index: 0x0658 */ 0x7592,0x225B,
	/* Array index: 0x0658 */ 0x7593,0x2673,
	/* Array index: 0x0658 */ 0x759B,0x2A3A,
	/* Array index: 0x0658 */ 0x759E,0x2A3B,
	/* Array index: 0x0658 */ 0x75A6,0x2E2B,
	/* Array index: 0x0658 */ 0x75A8,0x2E2F,
	/* Array index: 0x0658 */ 0x75B1,0x3257,
	/* Array index: 0x0658 */ 0x75B4,0x3255,
	/* Array index: 0x0658 */ 0x75C3,0x3252,
	/* Array index: 0x0658 */ 0x75D3,0x374A,
	/* Array index: 0x0658 */ 0x75E5,0x3D47,
	/* Array index: 0x0658 */ 0x75EE,0x427C,
	/* Array index: 0x0658 */ 0x7602,0x427D,
	/* Array index: 0x0658 */ 0x7607,0x482B,
	/* Array index: 0x0658 */ 0x760E,0x4829,
	/* Array index: 0x0658 */ 0x762C,0x5171,
	/* Array index: 0x0658 */ 0x7636,0x5173,
	/* Array index: 0x0658 */ 0x7637,0x5172,
	/* Array index: 0x0658 */ 0x763B,0x5174,
	/* Array index: 0x0658 */ 0x7644,0x5555,
	/* Array index: 0x0658 */ 0x7645,0x5553,
	/* Array index: 0x0658 */ 0x764E,0x5554,
	/* Array index: 0x0658 */ 0x764F,0x5834,
	/* Array index: 0x0658 */ 0x7651,0x5833,
	/* Array index: 0x0658 */ 0x765B,0x5832,
	/* Array index: 0x0658 */ 0x7667,0x5E3F,
	/* Array index: 0x0658 */ 0x7668,0x5E3E,
	/* Array index: 0x0658 */ 0x766F,0x603B,
	/* Array index: 0x0658 */ 0x7673,0x656A,
	/* Array index: 0x0658 */ 0x7676,0x225C,
	/* Array index: 0x0658 */ 0x7677,0x656B,
	/* Array index: 0x0658 */ 0x7680,0x2674,
	/* Array index: 0x0658 */ 0x7683,0x2676,
	/* Array index: 0x0658 */ 0x7690,0x374D,
	/* Array index: 0x0658 */ 0x7694,0x3D4C,
	/* Array index: 0x0658 */ 0x7697,0x427E,
	/* Array index: 0x0658 */ 0x7698,0x4321,
	/* Array index: 0x0658 */ 0x76A1,0x5176,
	/* Array index: 0x0658 */ 0x76A3,0x5556,
	/* Array index: 0x0658 */ 0x76A5,0x5557,
	/* Array index: 0x0658 */ 0x76A9,0x5A5E,
	/* Array index: 0x0658 */ 0x76B3,0x3D4D,
	/* Array index: 0x0658 */ 0x76B7,0x4830,
	/* Array index: 0x0658 */ 0x76B9,0x482E,
	/* Array index: 0x0658 */ 0x76BC,0x656C,
	/* Array index: 0x0658 */ 0x76C0,0x2677,
	/* Array index: 0x0658 */ 0x76C1,0x2678,
	/* Array index: 0x0658 */ 0x76C7,0x2E32,
	/* Array index: 0x0658 */ 0x76CB,0x325A,
	/* Array index: 0x0658 */ 0x76CC,0x325B,
	/* Array index: 0x0658 */ 0x76E2,0x4831,
	/* Array index: 0x0658 */ 0x76EB,0x5835,
	/* Array index: 0x0658 */ 0x76FD,0x2E35,
	/* Array index: 0x0658 */ 0x76FF,0x2E33,
	/* Array index: 0x0658 */ 0x7706,0x626C,
	/* Array index: 0x0658 */ 0x770E,0x3264,
	/* Array index: 0x0658 */ 0x770F,0x325F,
	/* Array index: 0x0658 */ 0x7716,0x325E,
	/* Array index: 0x0658 */ 0x7718,0x3268,
	/* Array index: 0x0658 */ 0x771E,0x3759,
	/* Array index: 0x0658 */ 0x7721,0x3267,
	/* Array index: 0x0658 */ 0x7724,0x3265,
	/* Array index: 0x0658 */ 0x7726,0x3755,
	/* Array index: 0x0658 */ 0x772A,0x325C,
	/* Array index: 0x0658 */ 0x772B,0x3263,
	/* Array index: 0x0658 */ 0x7730,0x3753,
	/* Array index: 0x0658 */ 0x7748,0x3D51,
	/* Array index: 0x0658 */ 0x7749,0x3D52,
	/* Array index: 0x0658 */ 0x7753,0x4327,
	/* Array index: 0x0658 */ 0x7758,0x4328,
	/* Array index: 0x0658 */ 0x775D,0x4325,
	/* Array index: 0x0658 */ 0x7764,0x4326,
	/* Array index: 0x0658 */ 0x7771,0x4839,
	/* Array index: 0x0658 */ 0x778A,0x4D44,
	/* Array index: 0x0658 */ 0x7790,0x4D46,
	/* Array index: 0x0658 */ 0x7796,0x517B,
	/* Array index: 0x0658 */ 0x7798,0x5179,
	/* Array index: 0x0658 */ 0x77A4,0x5559,
	/* Array index: 0x0658 */ 0x77A6,0x5558,
	/* Array index: 0x0658 */ 0x77A9,0x555B,
	/* Array index: 0x0658 */ 0x77AF,0x555C,
	/* Array index: 0x0658 */ 0x77B8,0x5836,
	/* Array index: 0x0658 */ 0x77B9,0x5838,
	/* Array index: 0x0658 */ 0x77BE,0x583B,
	/* Array index: 0x0658 */ 0x77C0,0x583A,
	/* Array index: 0x0658 */ 0x77C1,0x5839,
	/* Array index: 0x0658 */ 0x77C5,0x5A60,
	/* Array index: 0x0658 */ 0x77C6,0x5A5F,
	/* Array index: 0x0658 */ 0x77D1,0x5E42,
	/* Array index: 0x0658 */ 0x77D6,0x6432,
	/* Array index: 0x0658 */ 0x77DD,0x3269,
	/* Array index: 0x0658 */ 0x77DF,0x3D57,
	/* Array index: 0x0658 */ 0x77E6,0x2E36,
	/* Array index: 0x0658 */ 0x77F4,0x267B,
	/* Array index: 0x0658 */ 0x77F5,0x267C,
	/* Array index: 0x0658 */ 0x7804,0x2E39,
	/* Array index: 0x0658 */ 0x7815,0x2E3D,
	/* Array index: 0x0658 */ 0x7819,0x656D,
	/* Array index: 0x0658 */ 0x781B,0x656E,
	/* Array index: 0x0658 */ 0x781E,0x667C,
	/* Array index: 0x0658 */ 0x7824,0x326C,
	/* Array index: 0x0658 */ 0x7836,0x326D,
	/* Array index: 0x0658 */ 0x783D,0x656F,
	/* Array index: 0x0658 */ 0x7842,0x3762,
	/* Array index: 0x0658 */ 0x7846,0x3763,
	/* Array index: 0x0658 */ 0x7847,0x375D,
	/* Array index: 0x0658 */ 0x784B,0x375F,
	/* Array index: 0x0658 */ 0x784F,0x3761,
	/* Array index: 0x0658 */ 0x7851,0x3760,
	/* Array index: 0x0658 */ 0x7853,0x6570,
	/* Array index: 0x0658 */ 0x7854,0x6571,
	/* Array index: 0x0658 */ 0x7858,0x6572,
	/* Array index: 0x0658 */ 0x785F,0x3D5C,
	/* Array index: 0x0658 */ 0x7863,0x3D59,
	/* Array index: 0x0658 */ 0x7866,0x3D5D,
	/* Array index: 0x0658 */ 0x7876,0x3D5A,
	/* Array index: 0x0658 */ 0x7878,0x4331,
	/* Array index: 0x0658 */ 0x787A,0x432D,
	/* Array index: 0x0658 */ 0x787D,0x432E,
	/* Array index: 0x0658 */ 0x7881,0x4337,
	/* Array index: 0x0658 */ 0x7882,0x4329,
	/* Array index: 0x0658 */ 0x7888,0x4335,
	/* Array index: 0x0658 */ 0x788A,0x432B,
	/* Array index: 0x0658 */ 0x788B,0x4330,
	/* Array index: 0x0658 */ 0x788D,0x4334,
	/* Array index: 0x0658 */ 0x7890,0x432A,
	/* Array index: 0x0658 */ 0x7892,0x4336,
	/* Array index: 0x0658 */ 0x78A6,0x483E,
	/* Array index: 0x0658 */ 0x78AE,0x483F,
	/* Array index: 0x0658 */ 0x78AF,0x4845,
	/* Array index: 0x0658 */ 0x78B1,0x4844,
	/* Array index: 0x0658 */ 0x78B7,0x6573,
	/* Array index: 0x0658 */ 0x78B8,0x4840,
	/* Array index: 0x0658 */ 0x78C0,0x4D4A,
	/* Array index: 0x0658 */ 0x78C2,0x4D4D,
	/* Array index: 0x0658 */ 0x78C6,0x4D48,
	/* Array index: 0x0658 */ 0x78C7,0x4D4C,
	/* Array index: 0x0658 */ 0x78D2,0x4D4B,
	/* Array index: 0x0658 */ 0x78D3,0x4D49,
	/* Array index: 0x0658 */ 0x78D6,0x5221,
	/* Array index: 0x0658 */ 0x78D8,0x6574,
	/* Array index: 0x0658 */ 0x78DC,0x5224,
	/* Array index: 0x0658 */ 0x78EB,0x5222,
	/* Array index: 0x0658 */ 0x78EE,0x6575,
	/* Array index: 0x0658 */ 0x78F0,0x5560,
	/* Array index: 0x0658 */ 0x78F1,0x5562,
	/* Array index: 0x0658 */ 0x78F8,0x5561,
	/* Array index: 0x0658 */ 0x7915,0x5844,
	/* Array index: 0x0658 */ 0x7918,0x5A61,
	/* Array index: 0x0658 */ 0x791A,0x5A62,
	/* Array index: 0x0658 */ 0x7920,0x5A63,
	/* Array index: 0x0658 */ 0x7922,0x6576,
	/* Array index: 0x0658 */ 0x792E,0x5E45,
	/* Array index: 0x0658 */ 0x7930,0x5E43,
	/* Array index: 0x0658 */ 0x7932,0x5E44,
	/* Array index: 0x0658 */ 0x7937,0x603C,
	/* Array index: 0x0658 */ 0x793C,0x2433,
	/* Array index: 0x0658 */ 0x794D,0x6577,
	/* Array index: 0x0658 */ 0x7958,0x326F,
	/* Array index: 0x0658 */ 0x7959,0x3270,
	/* Array index: 0x0658 */ 0x7962,0x3272,
	/* Array index: 0x0658 */ 0x7966,0x3D5E,
	/* Array index: 0x0658 */ 0x796C,0x3766,
	/* Array index: 0x0658 */ 0x796E,0x3765,
	/* Array index: 0x0658 */ 0x7971,0x3D5F,
	/* Array index: 0x0658 */ 0x7975,0x3D64,
	/* Array index: 0x0658 */ 0x7976,0x3D62,
	/* Array index: 0x0658 */ 0x797E,0x4338,
	/* Array index: 0x0658 */ 0x7980,0x433D,
	/* Array index: 0x0658 */ 0x7999,0x6579,
	/* Array index: 0x0658 */ 0x79A3,0x657A,
	/* Array index: 0x0658 */ 0x79A5,0x5226,
	/* Array index: 0x0658 */ 0x79A9,0x5227,
	/* Array index: 0x0658 */ 0x79AF,0x5845,
	/* Array index: 0x0658 */ 0x79B5,0x603D,
	/* Array index: 0x0658 */ 0x79BC,0x657B,
	/* Array index: 0x0658 */ 0x79CA,0x2A44,
	/* Array index: 0x0658 */ 0x79CC,0x654C,
	/* Array index: 0x0658 */ 0x79D0,0x2E42,
	/* Array index: 0x0658 */ 0x79D3,0x2E40,
	/* Array index: 0x0658 */ 0x79D4,0x2E41,
	/* Array index: 0x0658 */ 0x79E1,0x3276,
	/* Array index: 0x0658 */ 0x79E2,0x327B,
	/* Array index: 0x0658 */ 0x79E5,0x3277,
	/* Array index: 0x0658 */ 0x79E8,0x3278,
	/* Array index: 0x0658 */ 0x79F9,0x376C,
	/* Array index: 0x0658 */ 0x79FF,0x3D65,
	/* Array index: 0x0658 */ 0x7A07,0x3D66,
	/* Array index: 0x0658 */ 0x7A09,0x3D69,
	/* Array index: 0x0658 */ 0x7A0E,0x3D68,
	/* Array index: 0x0658 */ 0x7A0F,0x4341,
	/* Array index: 0x0658 */ 0x7A1D,0x4344,
	/* Array index: 0x0658 */ 0x7A25,0x6524,
	/* Array index: 0x0658 */ 0x7A32,0x4852,
	/* Array index: 0x0658 */ 0x7A38,0x4D55,
	/* Array index: 0x0658 */ 0x7A3A,0x4D56,
	/* Array index: 0x0658 */ 0x7A3E,0x4D5A,
	/* Array index: 0x0658 */ 0x7A41,0x4D53,
	/* Array index: 0x0658 */ 0x7A42,0x4D57,
	/* Array index: 0x0658 */ 0x7A45,0x522A,
	/* Array index: 0x0658 */ 0x7A49,0x5564,
	/* Array index: 0x0658 */ 0x7A4F,0x522C,
	/* Array index: 0x0658 */ 0x7A53,0x5229,
	/* Array index: 0x0658 */ 0x7A64,0x5A66,
	/* Array index: 0x0658 */ 0x7A66,0x5A65,
	/* Array index: 0x0658 */ 0x7A6A,0x5A67,
	/* Array index: 0x0658 */ 0x7A6F,0x5C64,
	/* Array index: 0x0658 */ 0x7A72,0x6076,
	/* Array index: 0x0658 */ 0x7A73,0x6077,
	/* Array index: 0x0658 */ 0x7A7C,0x2E44,
	/* Array index: 0x0658 */ 0x7A7D,0x2E47,
	/* Array index: 0x0658 */ 0x7A82,0x2E49,
	/* Array index: 0x0658 */ 0x7A83,0x2E48,
	/* Array index: 0x0658 */ 0x7A91,0x3772,
	/* Array index: 0x0658 */ 0x7A93,0x3771,
	/* Array index: 0x0658 */ 0x7A9A,0x3770,
	/* Array index: 0x0658 */ 0x7AA1,0x4346,
	/* Array index: 0x0658 */ 0x7AA4,0x4347,
	/* Array index: 0x0658 */ 0x7AA7,0x657C,
	/* Array index: 0x0658 */ 0x7AB0,0x4D5B,
	/* Array index: 0x0658 */ 0x7ABB,0x522F,
	/* Array index: 0x0658 */ 0x7ABD,0x522E,
	/* Array index: 0x0658 */ 0x7AC2,0x5568,
	/* Array index: 0x0658 */ 0x7AC8,0x5E49,
	/* Array index: 0x0658 */ 0x7AC9,0x5E48,
	/* Array index: 0x0658 */ 0x7AE1,0x3773,
	/* Array index: 0x0658 */ 0x7AE2,0x3D71,
	/* Array index: 0x0658 */ 0x7AE7,0x3D70,
	/* Array index: 0x0658 */ 0x7AE9,0x4349,
	/* Array index: 0x0658 */ 0x7AEA,0x434A,
	/* Array index: 0x0658 */ 0x7AEC,0x4854,
	/* Array index: 0x0658 */ 0x7AF8,0x5F55,
	/* Array index: 0x0658 */ 0x7AFC,0x2E51,
	/* Array index: 0x0658 */ 0x7AFE,0x2E50,
	/* Array index: 0x0658 */ 0x7B0B,0x332B,
	/* Array index: 0x0658 */ 0x7B0D,0x332A,
	/* Array index: 0x0658 */ 0x7B14,0x332C,
	/* Array index: 0x0658 */ 0x7B16,0x3778,
	/* Array index: 0x0658 */ 0x7B17,0x3779,
	/* Array index: 0x0658 */ 0x7B1C,0x3777,
	/* Array index: 0x0658 */ 0x7B1F,0x377B,
	/* Array index: 0x0658 */ 0x7B21,0x3776,
	/* Array index: 0x0658 */ 0x7B29,0x632D,
	/* Array index: 0x0658 */ 0x7B36,0x377A,
	/* Array index: 0x0658 */ 0x7B37,0x657D,
	/* Array index: 0x0658 */ 0x7B53,0x3D79,
	/* Array index: 0x0658 */ 0x7B55,0x3D72,
	/* Array index: 0x0658 */ 0x7B57,0x3D76,
	/* Array index: 0x0658 */ 0x7B59,0x657E,
	/* Array index: 0x0658 */ 0x7B5E,0x434F,
	/* Array index: 0x0658 */ 0x7B62,0x434C,
	/* Array index: 0x0658 */ 0x7B68,0x4355,
	/* Array index: 0x0658 */ 0x7B6B,0x434D,
	/* Array index: 0x0658 */ 0x7B6C,0x3D77,
	/* Array index: 0x0658 */ 0x7B6F,0x4354,
	/* Array index: 0x0658 */ 0x7B79,0x4351,
	/* Array index: 0x0658 */ 0x7B81,0x4856,
	/* Array index: 0x0658 */ 0x7B83,0x4858,
	/* Array index: 0x0658 */ 0x7B86,0x4860,
	/* Array index: 0x0658 */ 0x7B92,0x485A,
	/* Array index: 0x0658 */ 0x7B93,0x485E,
	/* Array index: 0x0658 */ 0x7B9E,0x4857,
	/* Array index: 0x0658 */ 0x7B9F,0x485D,
	/* Array index: 0x0658 */ 0x7BA3,0x485C,
	/* Array index: 0x0658 */ 0x7BAE,0x4D5C,
	/* Array index: 0x0658 */ 0x7BB2,0x4D65,
	/* Array index: 0x0658 */ 0x7BB3,0x4D5D,
	/* Array index: 0x0658 */ 0x7BBF,0x4D60,
	/* Array index: 0x0658 */ 0x7BC2,0x6367,
	/* Array index: 0x0658 */ 0x7BC3,0x6366,
	/* Array index: 0x0658 */ 0x7BCD,0x4D63,
	/* Array index: 0x0658 */ 0x7BD0,0x6621,
	/* Array index: 0x0658 */ 0x7BEC,0x5233,
	/* Array index: 0x0658 */ 0x7BED,0x5234,
	/* Array index: 0x0658 */ 0x7BF6,0x556D,
	/* Array index: 0x0658 */ 0x7BFA,0x556B,
	/* Array index: 0x0658 */ 0x7BFC,0x556E,
	/* Array index: 0x0658 */ 0x7C08,0x5570,
	/* Array index: 0x0658 */ 0x7C12,0x5571,
	/* Array index: 0x0658 */ 0x7C18,0x556F,
	/* Array index: 0x0658 */ 0x7C1A,0x5850,
	/* Array index: 0x0658 */ 0x7C1B,0x584E,
	/* Array index: 0x0658 */ 0x7C24,0x5851,
	/* Array index: 0x0658 */ 0x7C2E,0x584C,
	/* Array index: 0x0658 */ 0x7C2F,0x6622,
	/* Array index: 0x0658 */ 0x7C32,0x6623,
	/* Array index: 0x0658 */ 0x7C34,0x5A6F,
	/* Array index: 0x0658 */ 0x7C35,0x5A6E,
	/* Array index: 0x0658 */ 0x7C42,0x6624,
	/* Array index: 0x0658 */ 0x7C44,0x5C66,
	/* Array index: 0x0658 */ 0x7C4B,0x5C65,
	/* Array index: 0x0658 */ 0x7C4E,0x6625,
	/* Array index: 0x0658 */ 0x7C51,0x5E4C,
	/* Array index: 0x0658 */ 0x7C56,0x5E4B,
	/* Array index: 0x0658 */ 0x7C58,0x5F58,
	/* Array index: 0x0658 */ 0x7C5D,0x5F57,
	/* Array index: 0x0658 */ 0x7C5E,0x603F,
	/* Array index: 0x0658 */ 0x7C62,0x603E,
	/* Array index: 0x0658 */ 0x7C68,0x6626,
	/* Array index: 0x0658 */ 0x7C6D,0x6144,
	/* Array index: 0x0658 */ 0x7C71,0x623A,
	/* Array index: 0x0658 */ 0x7C74,0x2A49,
	/* Array index: 0x0658 */ 0x7C76,0x2A48,
	/* Array index: 0x0658 */ 0x7C77,0x2E52,
	/* Array index: 0x0658 */ 0x7C7B,0x2E54,
	/* Array index: 0x0658 */ 0x7C7C,0x2E53,
	/* Array index: 0x0658 */ 0x7C83,0x3331,
	/* Array index: 0x0658 */ 0x7C86,0x332E,
	/* Array index: 0x0658 */ 0x7C87,0x3330,
	/* Array index: 0x0658 */ 0x7C8B,0x3332,
	/* Array index: 0x0658 */ 0x7C8E,0x332D,
	/* Array index: 0x0658 */ 0x7C93,0x377D,
	/* Array index: 0x0658 */ 0x7CA0,0x3E21,
	/* Array index: 0x0658 */ 0x7CA4,0x3E23,
	/* Array index: 0x0658 */ 0x7CA6,0x3E22,
	/* Array index: 0x0658 */ 0x7CA7,0x3D7E,
	/* Array index: 0x0658 */ 0x7CA9,0x6627,
	/* Array index: 0x0658 */ 0x7CAE,0x4358,
	/* Array index: 0x0658 */ 0x7CB0,0x435C,
	/* Array index: 0x0658 */ 0x7CB7,0x4862,
	/* Array index: 0x0658 */ 0x7CB8,0x4861,
	/* Array index: 0x0658 */ 0x7CCD,0x4D6E,
	/* Array index: 0x0658 */ 0x7CD3,0x5237,
	/* Array index: 0x0658 */ 0x7CDA,0x5575,
	/* Array index: 0x0658 */ 0x7CDB,0x5574,
	/* Array index: 0x0658 */ 0x7CE1,0x5239,
	/* Array index: 0x0658 */ 0x7CE3,0x5855,
	/* Array index: 0x0658 */ 0x7CE6,0x5854,
	/* Array index: 0x0658 */ 0x7CED,0x6628,
	/* Array index: 0x0658 */ 0x7CF3,0x6169,
	/* Array index: 0x0658 */ 0x7CF5,0x6040,
	/* Array index: 0x0658 */ 0x7CFA,0x2722,
	/* Array index: 0x0658 */ 0x7CFC,0x2A4B,
	/* Array index: 0x0658 */ 0x7CFF,0x2A4A,
	/* Array index: 0x0658 */ 0x7D24,0x3337,
	/* Array index: 0x0658 */ 0x7D25,0x333B,
	/* Array index: 0x0658 */ 0x7D2D,0x3829,
	/* Array index: 0x0658 */ 0x7D34,0x3826,
	/* Array index: 0x0658 */ 0x7D37,0x3827,
	/* Array index: 0x0658 */ 0x7D48,0x382E,
	/* Array index: 0x0658 */ 0x7D49,0x3824,
	/* Array index: 0x0658 */ 0x7D4C,0x382B,
	/* Array index: 0x0658 */ 0x7D4D,0x3E34,
	/* Array index: 0x0658 */ 0x7D57,0x3E29,
	/* Array index: 0x0658 */ 0x7D59,0x3E26,
	/* Array index: 0x0658 */ 0x7D5A,0x3E2E,
	/* Array index: 0x0658 */ 0x7D5D,0x3E2F,
	/* Array index: 0x0658 */ 0x7D60,0x3E28,
	/* Array index: 0x0658 */ 0x7D64,0x3E2C,
	/* Array index: 0x0658 */ 0x7D6C,0x3E2A,
	/* Array index: 0x0658 */ 0x7D7E,0x3E2B,
	/* Array index: 0x0658 */ 0x7D90,0x435E,
	/* Array index: 0x0658 */ 0x7DA4,0x4874,
	/* Array index: 0x0658 */ 0x7DA8,0x486D,
	/* Array index: 0x0658 */ 0x7DAB,0x486E,
	/* Array index: 0x0658 */ 0x7DB3,0x4870,
	/* Array index: 0x0658 */ 0x7DB6,0x4869,
	/* Array index: 0x0658 */ 0x7DC8,0x4868,
	/* Array index: 0x0658 */ 0x7DD6,0x4E22,
	/* Array index: 0x0658 */ 0x7DDC,0x4D7C,
	/* Array index: 0x0658 */ 0x7DE2,0x4D78,
	/* Array index: 0x0658 */ 0x7DE4,0x4E24,
	/* Array index: 0x0658 */ 0x7DE5,0x4E2F,
	/* Array index: 0x0658 */ 0x7DEB,0x4E2E,
	/* Array index: 0x0658 */ 0x7DED,0x4D77,
	/* Array index: 0x0658 */ 0x7DF5,0x4E2B,
	/* Array index: 0x0658 */ 0x7DF8,0x4D75,
	/* Array index: 0x0658 */ 0x7E07,0x662A,
	/* Array index: 0x0658 */ 0x7E19,0x523B,
	/* Array index: 0x0658 */ 0x7E26,0x5240,
	/* Array index: 0x0658 */ 0x7E27,0x523F,
	/* Array index: 0x0658 */ 0x7E2C,0x5579,
	/* Array index: 0x0658 */ 0x7E4D,0x557A,
	/* Array index: 0x0658 */ 0x7E4F,0x5859,
	/* Array index: 0x0658 */ 0x7E5B,0x585B,
	/* Array index: 0x0658 */ 0x7E5D,0x5858,
	/* Array index: 0x0658 */ 0x7E64,0x662C,
	/* Array index: 0x0658 */ 0x7E66,0x585A,
	/* Array index: 0x0658 */ 0x7E6C,0x5A72,
	/* Array index: 0x0658 */ 0x7E6E,0x5A74,
	/* Array index: 0x0658 */ 0x7E71,0x5A75,
	/* Array index: 0x0658 */ 0x7E7F,0x5C6C,
	/* Array index: 0x0658 */ 0x7E84,0x5E53,
	/* Array index: 0x0658 */ 0x7E85,0x5E50,
	/* Array index: 0x0658 */ 0x7E89,0x5E51,
	/* Array index: 0x0658 */ 0x7E8E,0x5E52,
	/* Array index: 0x0658 */ 0x7E92,0x5F5A,
	/* Array index: 0x0658 */ 0x7E9E,0x6235,
	/* Array index: 0x0658 */ 0x7F3B,0x3831,
	/* Array index: 0x0658 */ 0x7F40,0x662D,
	/* Array index: 0x0658 */ 0x7F41,0x4877,
	/* Array index: 0x0658 */ 0x7F46,0x557D,
	/* Array index: 0x0658 */ 0x7F47,0x585C,
	/* Array index: 0x0658 */ 0x7F4E,0x5F5B,
	/* Array index: 0x0658 */ 0x7F59,0x2A4E,
	/* Array index: 0x0658 */ 0x7F62,0x333C,
	/* Array index: 0x0658 */ 0x7F6F,0x4878,
	/* Array index: 0x0658 */ 0x7F71,0x4879,
	/* Array index: 0x0658 */ 0x7F78,0x4E30,
	/* Array index: 0x0658 */ 0x7F84,0x6422,
	/* Array index: 0x0658 */ 0x7F8F,0x2E5D,
	/* Array index: 0x0658 */ 0x7F93,0x333D,
	/* Array index: 0x0658 */ 0x7F97,0x333F,
	/* Array index: 0x0658 */ 0x7F99,0x333E,
	/* Array index: 0x0658 */ 0x7FA3,0x436E,
	/* Array index: 0x0658 */ 0x7FAE,0x4E31,
	/* Array index: 0x0658 */ 0x7FB4,0x585D,
	/* Array index: 0x0658 */ 0x7FC4,0x3342,
	/* Array index: 0x0658 */ 0x7FC6,0x3343,
	/* Array index: 0x0658 */ 0x7FD3,0x3E37,
	/* Array index: 0x0658 */ 0x7FD6,0x3E38,
	/* Array index: 0x0658 */ 0x7FDD,0x4372,
	/* Array index: 0x0658 */ 0x7FE7,0x4E32,
	/* Array index: 0x0658 */ 0x7FF6,0x557E,
	/* Array index: 0x0658 */ 0x7FFA,0x5861,
	/* Array index: 0x0658 */ 0x8002,0x667B,
	/* Array index: 0x0658 */ 0x8008,0x3836,
	/* Array index: 0x0658 */ 0x800A,0x3344,
	/* Array index: 0x0658 */ 0x801A,0x3837,
	/* Array index: 0x0658 */ 0x801D,0x3839,
	/* Array index: 0x0658 */ 0x8023,0x4922,
	/* Array index: 0x0658 */ 0x802B,0x5621,
	/* Array index: 0x0658 */ 0x802E,0x5862,
	/* Array index: 0x0658 */ 0x802F,0x5C6F,
	/* Array index: 0x0658 */ 0x803B,0x3349,
	/* Array index: 0x0658 */ 0x803C,0x3348,
	/* Array index: 0x0658 */ 0x8040,0x3347,
	/* Array index: 0x0658 */ 0x8041,0x662F,
	/* Array index: 0x0658 */ 0x8044,0x383D,
	/* Array index: 0x0658 */ 0x8045,0x383C,
	/* Array index: 0x0658 */ 0x8049,0x383B,
	/* Array index: 0x0658 */ 0x804E,0x3E3C,
	/* Array index: 0x0658 */ 0x8053,0x6729,
	/* Array index: 0x0658 */ 0x8057,0x4373,
	/* Array index: 0x0658 */ 0x8059,0x6356,
	/* Array index: 0x0658 */ 0x805B,0x4923,
	/* Array index: 0x0658 */ 0x806E,0x5243,
	/* Array index: 0x0658 */ 0x8074,0x5622,
	/* Array index: 0x0658 */ 0x808D,0x2435,
	/* Array index: 0x0658 */ 0x808E,0x2436,
	/* Array index: 0x0658 */ 0x8091,0x2724,
	/* Array index: 0x0658 */ 0x8094,0x2726,
	/* Array index: 0x0658 */ 0x8097,0x2725,
	/* Array index: 0x0658 */ 0x809F,0x2723,
	/* Array index: 0x0658 */ 0x80AC,0x2A59,
	/* Array index: 0x0658 */ 0x80B0,0x2A52,
	/* Array index: 0x0658 */ 0x80B3,0x2A54,
	/* Array index: 0x0658 */ 0x80B6,0x2A57,
	/* Array index: 0x0658 */ 0x80BB,0x6631,
	/* Array index: 0x0658 */ 0x80C6,0x2E66,
	/* Array index: 0x0658 */ 0x80CB,0x2E60,
	/* Array index: 0x0658 */ 0x80D2,0x2E61,
	/* Array index: 0x0658 */ 0x80D3,0x2E5E,
	/* Array index: 0x0658 */ 0x80DF,0x2E65,
	/* Array index: 0x0658 */ 0x80E2,0x2E64,
	/* Array index: 0x0658 */ 0x80EE,0x334C,
	/* Array index: 0x0658 */ 0x80F6,0x334A,
	/* Array index: 0x0658 */ 0x80F7,0x3352,
	/* Array index: 0x0658 */ 0x80FF,0x334B,
	/* Array index: 0x0658 */ 0x8103,0x334E,
	/* Array index: 0x0658 */ 0x8104,0x334D,
	/* Array index: 0x0658 */ 0x8107,0x334F,
	/* Array index: 0x0658 */ 0x8109,0x2E63,
	/* Array index: 0x0658 */ 0x8114,0x3E44,
	/* Array index: 0x0658 */ 0x8117,0x3849,
	/* Array index: 0x0658 */ 0x811A,0x3847,
	/* Array index: 0x0658 */ 0x8128,0x632E,
	/* Array index: 0x0658 */ 0x812A,0x3841,
	/* Array index: 0x0658 */ 0x812E,0x3842,
	/* Array index: 0x0658 */ 0x8131,0x3845,
	/* Array index: 0x0658 */ 0x8134,0x3848,
	/* Array index: 0x0658 */ 0x813B,0x3E3E,
	/* Array index: 0x0658 */ 0x8145,0x3E3D,
	/* Array index: 0x0658 */ 0x8148,0x3E40,
	/* Array index: 0x0658 */ 0x8149,0x3E42,
	/* Array index: 0x0658 */ 0x815D,0x437B,
	/* Array index: 0x0658 */ 0x815F,0x437E,
	/* Array index: 0x0658 */ 0x8163,0x4375,
	/* Array index: 0x0658 */ 0x816A,0x4376,
	/* Array index: 0x0658 */ 0x816C,0x4377,
	/* Array index: 0x0658 */ 0x816D,0x4423,
	/* Array index: 0x0658 */ 0x8175,0x437C,
	/* Array index: 0x0658 */ 0x817D,0x4422,
	/* Array index: 0x0658 */ 0x8181,0x4927,
	/* Array index: 0x0658 */ 0x8184,0x492A,
	/* Array index: 0x0658 */ 0x81A1,0x4E3A,
	/* Array index: 0x0658 */ 0x81AA,0x5249,
	/* Array index: 0x0658 */ 0x81AD,0x5247,
	/* Array index: 0x0658 */ 0x81AF,0x5244,
	/* Array index: 0x0658 */ 0x81B8,0x5623,
	/* Array index: 0x0658 */ 0x81C8,0x5624,
	/* Array index: 0x0658 */ 0x81CE,0x5865,
	/* Array index: 0x0658 */ 0x81D4,0x5A77,
	/* Array index: 0x0658 */ 0x81D6,0x5A78,
	/* Array index: 0x0658 */ 0x81EF,0x3E47,
	/* Array index: 0x0658 */ 0x81F6,0x3E48,
	/* Array index: 0x0658 */ 0x8203,0x3E49,
	/* Array index: 0x0658 */ 0x820E,0x2A5D,
	/* Array index: 0x0658 */ 0x8213,0x492B,
	/* Array index: 0x0658 */ 0x8224,0x2E68,
	/* Array index: 0x0658 */ 0x8227,0x3357,
	/* Array index: 0x0658 */ 0x8229,0x3358,
	/* Array index: 0x0658 */ 0x822D,0x3355,
	/* Array index: 0x0658 */ 0x8241,0x4426,
	/* Array index: 0x0658 */ 0x8248,0x6634,
	/* Array index: 0x0658 */ 0x824A,0x492D,
	/* Array index: 0x0658 */ 0x824C,0x492E,
	/* Array index: 0x0658 */ 0x8262,0x5A7A,
	/* Array index: 0x0658 */ 0x8265,0x5A7B,
	/* Array index: 0x0658 */ 0x826A,0x5E55,
	/* Array index: 0x0658 */ 0x8276,0x5A7C,
	/* Array index: 0x0658 */ 0x827B,0x2438,
	/* Array index: 0x0658 */ 0x8286,0x2728,
	/* Array index: 0x0658 */ 0x828C,0x2729,
	/* Array index: 0x0658 */ 0x8295,0x272B,
	/* Array index: 0x0658 */ 0x829C,0x2A60,
	/* Array index: 0x0658 */ 0x82A6,0x2A6B,
	/* Array index: 0x0658 */ 0x82AA,0x2A66,
	/* Array index: 0x0658 */ 0x82B2,0x2A6C,
	/* Array index: 0x0658 */ 0x82C4,0x2A5E,
	/* Array index: 0x0658 */ 0x82C9,0x2A68,
	/* Array index: 0x0658 */ 0x82D0,0x2E76,
	/* Array index: 0x0658 */ 0x82D8,0x2E6A,
	/* Array index: 0x0658 */ 0x82DD,0x2E6B,
	/* Array index: 0x0658 */ 0x82E2,0x2E78,
	/* Array index: 0x0658 */ 0x82E9,0x2E72,
	/* Array index: 0x0658 */ 0x82EE,0x2E74,
	/* Array index: 0x0658 */ 0x82F7,0x2E69,
	/* Array index: 0x0658 */ 0x82F8,0x2E6E,
	/* Array index: 0x0658 */ 0x82FC,0x2E6F,
	/* Array index: 0x0658 */ 0x82FD,0x2E7A,
	/* Array index: 0x0658 */ 0x830B,0x2E79,
	/* Array index: 0x0658 */ 0x830E,0x2E77,
	/* Array index: 0x0658 */ 0x8310,0x6635,
	/* Array index: 0x0658 */ 0x8312,0x6637,
	/* Array index: 0x0658 */ 0x8318,0x3362,
	/* Array index: 0x0658 */ 0x831D,0x384D,
	/* Array index: 0x0658 */ 0x831F,0x3359,
	/* Array index: 0x0658 */ 0x8321,0x335F,
	/* Array index: 0x0658 */ 0x8323,0x385B,
	/* Array index: 0x0658 */ 0x833E,0x2A5F,
	/* Array index: 0x0658 */ 0x8346,0x3E57,
	/* Array index: 0x0658 */ 0x8357,0x335A,
	/* Array index: 0x0658 */ 0x8358,0x3363,
	/* Array index: 0x0658 */ 0x8362,0x6636,
	/* Array index: 0x0658 */ 0x8363,0x2D33,
	/* Array index: 0x0658 */ 0x8371,0x384E,
	/* Array index: 0x0658 */ 0x8372,0x3851,
	/* Array index: 0x0658 */ 0x8379,0x3854,
	/* Array index: 0x0658 */ 0x8380,0x3850,
	/* Array index: 0x0658 */ 0x8384,0x384F,
	/* Array index: 0x0658 */ 0x8385,0x385D,
	/* Array index: 0x0658 */ 0x8391,0x3855,
	/* Array index: 0x0658 */ 0x839C,0x385E,
	/* Array index: 0x0658 */ 0x839F,0x3857,
	/* Array index: 0x0658 */ 0x83A1,0x3852,
	/* Array index: 0x0658 */ 0x83AD,0x3858,
	/* Array index: 0x0658 */ 0x83B7,0x385F,
	/* Array index: 0x0658 */ 0x83CD,0x3E5E,
	/* Array index: 0x0658 */ 0x83D2,0x3E4F,
	/* Array index: 0x0658 */ 0x83D3,0x3E59,
	/* Array index: 0x0658 */ 0x83DA,0x3E4D,
	/* Array index: 0x0658 */ 0x83E6,0x3E5F,
	/* Array index: 0x0658 */ 0x83ED,0x3E4B,
	/* Array index: 0x0658 */ 0x8400,0x3E52,
	/* Array index: 0x0658 */ 0x8402,0x3E5C,
	/* Array index: 0x0658 */ 0x8405,0x3E5A,
	/* Array index: 0x0658 */ 0x8408,0x3E50,
	/* Array index: 0x0658 */ 0x841E,0x6639,
	/* Array index: 0x0658 */ 0x841F,0x3E5B,
	/* Array index: 0x0658 */ 0x8421,0x6638,
	/* Array index: 0x0658 */ 0x842A,0x442D,
	/* Array index: 0x0658 */ 0x842E,0x4431,
	/* Array index: 0x0658 */ 0x843E,0x443A,
	/* Array index: 0x0658 */ 0x8441,0x4435,
	/* Array index: 0x0658 */ 0x8448,0x442C,
	/* Array index: 0x0658 */ 0x844A,0x443E,
	/* Array index: 0x0658 */ 0x844F,0x4427,
	/* Array index: 0x0658 */ 0x8453,0x4433,
	/* Array index: 0x0658 */ 0x8455,0x443F,
	/* Array index: 0x0658 */ 0x8458,0x4440,
	/* Array index: 0x0658 */ 0x845C,0x4432,
	/* Array index: 0x0658 */ 0x8462,0x4438,
	/* Array index: 0x0658 */ 0x8464,0x4430,
	/* Array index: 0x0658 */ 0x8471,0x443C,
	/* Array index: 0x0658 */ 0x8472,0x442F,
	/* Array index: 0x0658 */ 0x847B,0x442E,
	/* Array index: 0x0658 */ 0x847F,0x442A,
	/* Array index: 0x0658 */ 0x8480,0x4439,
	/* Array index: 0x0658 */ 0x8483,0x443B,
	/* Array index: 0x0658 */ 0x8484,0x4428,
	/* Array index: 0x0658 */ 0x84A3,0x493C,
	/* Array index: 0x0658 */ 0x84A5,0x4939,
	/* Array index: 0x0658 */ 0x84AD,0x4942,
	/* Array index: 0x0658 */ 0x84B3,0x493B,
	/* Array index: 0x0658 */ 0x84B5,0x493A,
	/* Array index: 0x0658 */ 0x84B7,0x4941,
	/* Array index: 0x0658 */ 0x84BD,0x4932,
	/* Array index: 0x0658 */ 0x84C3,0x4936,
	/* Array index: 0x0658 */ 0x84C8,0x4436,
	/* Array index: 0x0658 */ 0x84D5,0x493F,
	/* Array index: 0x0658 */ 0x84D8,0x493E,
	/* Array index: 0x0658 */ 0x84DA,0x4943,
	/* Array index: 0x0658 */ 0x84DE,0x663B,
	/* Array index: 0x0658 */ 0x84E1,0x663C,
	/* Array index: 0x0658 */ 0x84E2,0x663A,
	/* Array index: 0x0658 */ 0x84E4,0x493D,
	/* Array index: 0x0658 */ 0x84ED,0x4E4F,
	/* Array index: 0x0658 */ 0x84F5,0x4E44,
	/* Array index: 0x0658 */ 0x84F8,0x4E42,
	/* Array index: 0x0658 */ 0x851B,0x4E4B,
	/* Array index: 0x0658 */ 0x853F,0x525A,
	/* Array index: 0x0658 */ 0x8542,0x5252,
	/* Array index: 0x0658 */ 0x854B,0x5259,
	/* Array index: 0x0658 */ 0x854C,0x5250,
	/* Array index: 0x0658 */ 0x855A,0x5258,
	/* Array index: 0x0658 */ 0x855C,0x5254,
	/* Array index: 0x0658 */ 0x855F,0x5256,
	/* Array index: 0x0658 */ 0x856F,0x524F,
	/* Array index: 0x0658 */ 0x8570,0x5255,
	/* Array index: 0x0658 */ 0x8573,0x663D,
	/* Array index: 0x0658 */ 0x857F,0x562B,
	/* Array index: 0x0658 */ 0x8592,0x5628,
	/* Array index: 0x0658 */ 0x8593,0x5629,
	/* Array index: 0x0658 */ 0x8597,0x562D,
	/* Array index: 0x0658 */ 0x85AB,0x562C,
	/* Array index: 0x0658 */ 0x85AC,0x5630,
	/* Array index: 0x0658 */ 0x85B2,0x586C,
	/* Array index: 0x0658 */ 0x85BB,0x586E,
	/* Array index: 0x0658 */ 0x85C1,0x586F,
	/* Array index: 0x0658 */ 0x85CC,0x586B,
	/* Array index: 0x0658 */ 0x85D4,0x663E,
	/* Array index: 0x0658 */ 0x85D6,0x5A7E,
	/* Array index: 0x0658 */ 0x85DB,0x5A7D,
	/* Array index: 0x0658 */ 0x85E7,0x5B22,
	/* Array index: 0x0658 */ 0x85EE,0x5C7B,
	/* Array index: 0x0658 */ 0x85F4,0x5B25,
	/* Array index: 0x0658 */ 0x85F5,0x663F,
	/* Array index: 0x0658 */ 0x85FC,0x5C76,
	/* Array index: 0x0658 */ 0x8602,0x5C78,
	/* Array index: 0x0658 */ 0x8603,0x5C7C,
	/* Array index: 0x0658 */ 0x8608,0x5D22,
	/* Array index: 0x0658 */ 0x860D,0x5C7E,
	/* Array index: 0x0658 */ 0x860F,0x5D23,
	/* Array index: 0x0658 */ 0x8610,0x5C77,
	/* Array index: 0x0658 */ 0x8613,0x5D21,
	/* Array index: 0x0658 */ 0x8615,0x5E5B,
	/* Array index: 0x0658 */ 0x8616,0x5E5A,
	/* Array index: 0x0658 */ 0x861D,0x5E5C,
	/* Array index: 0x0658 */ 0x8628,0x5E58,
	/* Array index: 0x0658 */ 0x862B,0x5E56,
	/* Array index: 0x0658 */ 0x862F,0x5E57,
	/* Array index: 0x0658 */ 0x8637,0x6640,
	/* Array index: 0x0658 */ 0x863D,0x6043,
	/* Array index: 0x0658 */ 0x8641,0x602F,
	/* Array index: 0x0658 */ 0x8642,0x6146,
	/* Array index: 0x0658 */ 0x8645,0x6641,
	/* Array index: 0x0658 */ 0x8649,0x6147,
	/* Array index: 0x0658 */ 0x864A,0x617C,
	/* Array index: 0x0658 */ 0x8657,0x3863,
	/* Array index: 0x0658 */ 0x8658,0x3860,
	/* Array index: 0x0658 */ 0x865A,0x3861,
	/* Array index: 0x0658 */ 0x865D,0x3E61,
	/* Array index: 0x0658 */ 0x866C,0x272D,
	/* Array index: 0x0658 */ 0x8672,0x6642,
	/* Array index: 0x0658 */ 0x8675,0x2F23,
	/* Array index: 0x0658 */ 0x8676,0x2E7C,
	/* Array index: 0x0658 */ 0x8678,0x2E7E,
	/* Array index: 0x0658 */ 0x867D,0x2F24,
	/* Array index: 0x0658 */ 0x8684,0x3369,
	/* Array index: 0x0658 */ 0x8689,0x336C,
	/* Array index: 0x0658 */ 0x868F,0x336F,
	/* Array index: 0x0658 */ 0x8692,0x336E,
	/* Array index: 0x0658 */ 0x869B,0x336B,
	/* Array index: 0x0658 */ 0x869F,0x336A,
	/* Array index: 0x0658 */ 0x86A0,0x3370,
	/* Array index: 0x0658 */ 0x86A6,0x336D,
	/* Array index: 0x0658 */ 0x86AE,0x3866,
	/* Array index: 0x0658 */ 0x86B2,0x3864,
	/* Array index: 0x0658 */ 0x86D5,0x3E62,
	/* Array index: 0x0658 */ 0x86E1,0x3E63,
	/* Array index: 0x0658 */ 0x86EE,0x3E68,
	/* Array index: 0x0658 */ 0x86FC,0x4444,
	/* Array index: 0x0658 */ 0x86FD,0x4445,
	/* Array index: 0x0658 */ 0x86FF,0x4449,
	/* Array index: 0x0658 */ 0x8715,0x4446,
	/* Array index: 0x0658 */ 0x8716,0x4448,
	/* Array index: 0x0658 */ 0x871D,0x494F,
	/* Array index: 0x0658 */ 0x872B,0x494A,
	/* Array index: 0x0658 */ 0x8736,0x4945,
	/* Array index: 0x0658 */ 0x8739,0x494C,
	/* Array index: 0x0658 */ 0x873D,0x4949,
	/* Array index: 0x0658 */ 0x8745,0x494E,
	/* Array index: 0x0658 */ 0x8747,0x494B,
	/* Array index: 0x0658 */ 0x874A,0x6643,
	/* Array index: 0x0658 */ 0x8771,0x4E5D,
	/* Array index: 0x0658 */ 0x877D,0x4E59,
	/* Array index: 0x0658 */ 0x878A,0x525B,
	/* Array index: 0x0658 */ 0x878B,0x525D,
	/* Array index: 0x0658 */ 0x878E,0x525F,
	/* Array index: 0x0658 */ 0x8799,0x5262,
	/* Array index: 0x0658 */ 0x87A1,0x525E,
	/* Array index: 0x0658 */ 0x87A5,0x6645,
	/* Array index: 0x0658 */ 0x87A9,0x6644,
	/* Array index: 0x0658 */ 0x87B1,0x563A,
	/* Array index: 0x0658 */ 0x87C1,0x5639,
	/* Array index: 0x0658 */ 0x87C7,0x563B,
	/* Array index: 0x0658 */ 0x87CD,0x5636,
	/* Array index: 0x0658 */ 0x87CE,0x5634,
	/* Array index: 0x0658 */ 0x87D5,0x5B28,
	/* Array index: 0x0658 */ 0x87D6,0x5876,
	/* Array index: 0x0658 */ 0x87DA,0x5878,
	/* Array index: 0x0658 */ 0x87E9,0x5873,
	/* Array index: 0x0658 */ 0x87EE,0x5874,
	/* Array index: 0x0658 */ 0x87F0,0x5875,
	/* Array index: 0x0658 */ 0x87F5,0x6646,
	/* Array index: 0x0658 */ 0x87F8,0x5B2C,
	/* Array index: 0x0658 */ 0x87FD,0x5B27,
	/* Array index: 0x0658 */ 0x8807,0x5B29,
	/* Array index: 0x0658 */ 0x880E,0x5877,
	/* Array index: 0x0658 */ 0x880F,0x5B2B,
	/* Array index: 0x0658 */ 0x8812,0x5D25,
	/* Array index: 0x0658 */ 0x8818,0x5D24,
	/* Array index: 0x0658 */ 0x881A,0x5E5D,
	/* Array index: 0x0658 */ 0x8827,0x5F5F,
	/* Array index: 0x0658 */ 0x882D,0x6045,
	/* Array index: 0x0658 */ 0x8834,0x6647,
	/* Array index: 0x0658 */ 0x8842,0x2F26,
	/* Array index: 0x0658 */ 0x884F,0x3371,
	/* Array index: 0x0658 */ 0x8850,0x6648,
	/* Array index: 0x0658 */ 0x8858,0x444D,
	/* Array index: 0x0658 */ 0x885C,0x4E5F,
	/* Array index: 0x0658 */ 0x885E,0x5263,
	/* Array index: 0x0658 */ 0x885F,0x5264,
	/* Array index: 0x0658 */ 0x8866,0x2F27,
	/* Array index: 0x0658 */ 0x886E,0x3374,
	/* Array index: 0x0658 */ 0x8873,0x3377,
	/* Array index: 0x0658 */ 0x8878,0x3372,
	/* Array index: 0x0658 */ 0x887A,0x3373,
	/* Array index: 0x0658 */ 0x887B,0x3375,
	/* Array index: 0x0658 */ 0x8884,0x3376,
	/* Array index: 0x0658 */ 0x8887,0x6649,
	/* Array index: 0x0658 */ 0x888A,0x3872,
	/* Array index: 0x0658 */ 0x888F,0x386E,
	/* Array index: 0x0658 */ 0x8890,0x3874,
	/* Array index: 0x0658 */ 0x8894,0x386C,
	/* Array index: 0x0658 */ 0x889C,0x386B,
	/* Array index: 0x0658 */ 0x88A0,0x3873,
	/* Array index: 0x0658 */ 0x88A3,0x386D,
	/* Array index: 0x0658 */ 0x88A5,0x386F,
	/* Array index: 0x0658 */ 0x88A6,0x3871,
	/* Array index: 0x0658 */ 0x88A9,0x3870,
	/* Array index: 0x0658 */ 0x88B4,0x3E70,
	/* Array index: 0x0658 */ 0x88B5,0x3E72,
	/* Array index: 0x0658 */ 0x88BB,0x3E6D,
	/* Array index: 0x0658 */ 0x88BF,0x3E6F,
	/* Array index: 0x0658 */ 0x88CF,0x444E,
	/* Array index: 0x0658 */ 0x88E0,0x444F,
	/* Array index: 0x0658 */ 0x88E6,0x4E60,
	/* Array index: 0x0658 */ 0x88E9,0x4959,
	/* Array index: 0x0658 */ 0x88EA,0x4952,
	/* Array index: 0x0658 */ 0x88ED,0x4957,
	/* Array index: 0x0658 */ 0x88F5,0x4954,
	/* Array index: 0x0658 */ 0x88FF,0x4951,
	/* Array index: 0x0658 */ 0x8900,0x4956,
	/* Array index: 0x0658 */ 0x8903,0x4958,
	/* Array index: 0x0658 */ 0x890F,0x4E61,
	/* Array index: 0x0658 */ 0x891B,0x4E62,
	/* Array index: 0x0658 */ 0x8924,0x5265,
	/* Array index: 0x0658 */ 0x8939,0x5640,
	/* Array index: 0x0658 */ 0x893F,0x563F,
	/* Array index: 0x0658 */ 0x8940,0x563D,
	/* Array index: 0x0658 */ 0x8943,0x5642,
	/* Array index: 0x0658 */ 0x8948,0x5879,
	/* Array index: 0x0658 */ 0x894A,0x587A,
	/* Array index: 0x0658 */ 0x894D,0x587C,
	/* Array index: 0x0658 */ 0x894E,0x587B,
	/* Array index: 0x0658 */ 0x8954,0x664A,
	/* Array index: 0x0658 */ 0x8965,0x5D29,
	/* Array index: 0x0658 */ 0x8967,0x5D28,
	/* Array index: 0x0658 */ 0x8978,0x6149,
	/* Array index: 0x0658 */ 0x897D,0x617D,
	/* Array index: 0x0658 */ 0x8984,0x664B,
	/* Array index: 0x0658 */ 0x8987,0x5B2F,
	/* Array index: 0x0658 */ 0x8989,0x6046,
	/* Array index: 0x0658 */ 0x898A,0x614A,
	/* Array index: 0x0658 */ 0x898C,0x2F29,
	/* Array index: 0x0658 */ 0x8991,0x3876,
	/* Array index: 0x0658 */ 0x8992,0x3875,
	/* Array index: 0x0658 */ 0x8994,0x3877,
	/* Array index: 0x0658 */ 0x899A,0x3E74,
	/* Array index: 0x0658 */ 0x89A5,0x4E65,
	/* Array index: 0x0658 */ 0x89A7,0x5266,
	/* Array index: 0x0658 */ 0x89A9,0x4E64,
	/* Array index: 0x0658 */ 0x89AB,0x5646,
	/* Array index: 0x0658 */ 0x89B0,0x587E,
	/* Array index: 0x0658 */ 0x89B1,0x587D,
	/* Array index: 0x0658 */ 0x89B3,0x5921,
	/* Array index: 0x0658 */ 0x89B5,0x5B31,
	/* Array index: 0x0658 */ 0x89BB,0x5D2A,
	/* Array index: 0x0658 */ 0x89BC,0x5E61,
	/* Array index: 0x0658 */ 0x89E7,0x4454,
	/* Array index: 0x0658 */ 0x89EA,0x495C,
	/* Array index: 0x0658 */ 0x89EE,0x4E66,
	/* Array index: 0x0658 */ 0x89F5,0x5B32,
	/* Array index: 0x0658 */ 0x8A05,0x2F2A,
	/* Array index: 0x0658 */ 0x8A06,0x2F2C,
	/* Array index: 0x0658 */ 0x8A0B,0x337B,
	/* Array index: 0x0658 */ 0x8A0D,0x337A,
	/* Array index: 0x0658 */ 0x8A19,0x337C,
	/* Array index: 0x0658 */ 0x8A1C,0x387E,
	/* Array index: 0x0658 */ 0x8A20,0x3923,
	/* Array index: 0x0658 */ 0x8A21,0x3929,
	/* Array index: 0x0658 */ 0x8A26,0x3879,
	/* Array index: 0x0658 */ 0x8A28,0x387B,
	/* Array index: 0x0658 */ 0x8A29,0x3925,
	/* Array index: 0x0658 */ 0x8A2B,0x3922,
	/* Array index: 0x0658 */ 0x8A2E,0x6330,
	/* Array index: 0x0658 */ 0x8A32,0x387A,
	/* Array index: 0x0658 */ 0x8A35,0x3E7C,
	/* Array index: 0x0658 */ 0x8A38,0x3E7D,
	/* Array index: 0x0658 */ 0x8A3D,0x3F24,
	/* Array index: 0x0658 */ 0x8A42,0x3E7E,
	/* Array index: 0x0658 */ 0x8A43,0x3E75,
	/* Array index: 0x0658 */ 0x8A49,0x3F21,
	/* Array index: 0x0658 */ 0x8A4B,0x3F23,
	/* Array index: 0x0658 */ 0x8A5A,0x3E78,
	/* Array index: 0x0658 */ 0x8A5D,0x3F22,
	/* Array index: 0x0658 */ 0x8A64,0x4460,
	/* Array index: 0x0658 */ 0x8A65,0x4459,
	/* Array index: 0x0658 */ 0x8A67,0x4463,
	/* Array index: 0x0658 */ 0x8A6A,0x4455,
	/* Array index: 0x0658 */ 0x8A6F,0x4458,
	/* Array index: 0x0658 */ 0x8A78,0x445B,
	/* Array index: 0x0658 */ 0x8A7D,0x445C,
	/* Array index: 0x0658 */ 0x8A7E,0x4461,
	/* Array index: 0x0658 */ 0x8A80,0x4456,
	/* Array index: 0x0658 */ 0x8A88,0x445D,
	/* Array index: 0x0658 */ 0x8A8E,0x495F,
	/* Array index: 0x0658 */ 0x8A90,0x4964,
	/* Array index: 0x0658 */ 0x8A94,0x4963,
	/* Array index: 0x0658 */ 0x8AA2,0x4960,
	/* Array index: 0x0658 */ 0x8AA9,0x4965,
	/* Array index: 0x0658 */ 0x8AAC,0x4966,
	/* Array index: 0x0658 */ 0x8AAF,0x4E74,
	/* Array index: 0x0658 */ 0x8AC1,0x4E73,
	/* Array index: 0x0658 */ 0x8ACA,0x4E75,
	/* Array index: 0x0658 */ 0x8ACC,0x4E69,
	/* Array index: 0x0658 */ 0x8ACE,0x4E6A,
	/* Array index: 0x0658 */ 0x8AD0,0x4E76,
	/* Array index: 0x0658 */ 0x8AE3,0x526A,
	/* Array index: 0x0658 */ 0x8AE5,0x526B,
	/* Array index: 0x0658 */ 0x8AE9,0x4E6F,
	/* Array index: 0x0658 */ 0x8AEA,0x5267,
	/* Array index: 0x0658 */ 0x8AEC,0x526E,
	/* Array index: 0x0658 */ 0x8AF9,0x5269,
	/* Array index: 0x0658 */ 0x8AFD,0x5268,
	/* Array index: 0x0658 */ 0x8B03,0x664C,
	/* Array index: 0x0658 */ 0x8B09,0x5649,
	/* Array index: 0x0658 */ 0x8B0C,0x564A,
	/* Array index: 0x0658 */ 0x8B1F,0x5648,
	/* Array index: 0x0658 */ 0x8B29,0x592A,
	/* Array index: 0x0658 */ 0x8B2D,0x5926,
	/* Array index: 0x0658 */ 0x8B32,0x5924,
	/* Array index: 0x0658 */ 0x8B34,0x5928,
	/* Array index: 0x0658 */ 0x8B38,0x5923,
	/* Array index: 0x0658 */ 0x8B3F,0x5B34,
	/* Array index: 0x0658 */ 0x8B43,0x5B35,
	/* Array index: 0x0658 */ 0x8B4C,0x5B36,
	/* Array index: 0x0658 */ 0x8B4D,0x5D33,
	/* Array index: 0x0658 */ 0x8B61,0x5D30,
	/* Array index: 0x0658 */ 0x8B62,0x5D2C,
	/* Array index: 0x0658 */ 0x8B64,0x5D32,
	/* Array index: 0x0658 */ 0x8B69,0x5D2B,
	/* Array index: 0x0658 */ 0x8B6E,0x5D2E,
	/* Array index: 0x0658 */ 0x8B71,0x5C6D,
	/* Array index: 0x0658 */ 0x8B73,0x642D,
	/* Array index: 0x0658 */ 0x8B75,0x5E62,
	/* Array index: 0x0658 */ 0x8B7C,0x5E63,
	/* Array index: 0x0658 */ 0x8B81,0x5F61,
	/* Array index: 0x0658 */ 0x8B83,0x5F62,
	/* Array index: 0x0658 */ 0x8B87,0x6048,
	/* Array index: 0x0658 */ 0x8B8D,0x6047,
	/* Array index: 0x0658 */ 0x8B90,0x6049,
	/* Array index: 0x0658 */ 0x8B91,0x607B,
	/* Array index: 0x0658 */ 0x8B97,0x614B,
	/* Array index: 0x0658 */ 0x8B9B,0x616B,
	/* Array index: 0x0658 */ 0x8B9D,0x617E,
	/* Array index: 0x0658 */ 0x8C3A,0x392A,
	/* Array index: 0x0658 */ 0x8C40,0x564D,
	/* Array index: 0x0658 */ 0x8C44,0x5F64,
	/* Array index: 0x0658 */ 0x8C58,0x392D,
	/* Array index: 0x0658 */ 0x8C5B,0x392C,
	/* Array index: 0x0658 */ 0x8C5E,0x3F2A,
	/* Array index: 0x0658 */ 0x8C60,0x3F29,
	/* Array index: 0x0658 */ 0x8C63,0x4464,
	/* Array index: 0x0658 */ 0x8C67,0x496D,
	/* Array index: 0x0658 */ 0x8C74,0x592B,
	/* Array index: 0x0658 */ 0x8C7C,0x392E,
	/* Array index: 0x0658 */ 0x8C7E,0x3F2C,
	/* Array index: 0x0658 */ 0x8C7F,0x3F2B,
	/* Array index: 0x0658 */ 0x8C83,0x3F2D,
	/* Array index: 0x0658 */ 0x8C87,0x634A,
	/* Array index: 0x0658 */ 0x8C88,0x4465,
	/* Array index: 0x0658 */ 0x8C8E,0x4E7A,
	/* Array index: 0x0658 */ 0x8C96,0x564F,
	/* Array index: 0x0658 */ 0x8C9B,0x614D,
	/* Array index: 0x0658 */ 0x8C9F,0x2F2E,
	/* Array index: 0x0658 */ 0x8CA6,0x3930,
	/* Array index: 0x0658 */ 0x8CAD,0x3932,
	/* Array index: 0x0658 */ 0x8CAE,0x3931,
	/* Array index: 0x0658 */ 0x8CB1,0x3F2F,
	/* Array index: 0x0658 */ 0x8CC9,0x4469,
	/* Array index: 0x0658 */ 0x8CCD,0x4467,
	/* Array index: 0x0658 */ 0x8CD0,0x4970,
	/* Array index: 0x0658 */ 0x8CD4,0x4972,
	/* Array index: 0x0658 */ 0x8CD6,0x4971,
	/* Array index: 0x0658 */ 0x8CD8,0x664E,
	/* Array index: 0x0658 */ 0x8CDB,0x4E7E,
	/* Array index: 0x0658 */ 0x8CE9,0x4E7D,
	/* Array index: 0x0658 */ 0x8CEB,0x4F22,
	/* Array index: 0x0658 */ 0x8CEF,0x5275,
	/* Array index: 0x0658 */ 0x8CF2,0x5273,
	/* Array index: 0x0658 */ 0x8CF6,0x5651,
	/* Array index: 0x0658 */ 0x8CF7,0x5652,
	/* Array index: 0x0658 */ 0x8D03,0x592E,
	/* Array index: 0x0658 */ 0x8D0B,0x5B38,
	/* Array index: 0x0658 */ 0x8D0C,0x664F,
	/* Array index: 0x0658 */ 0x8D0E,0x6428,
	/* Array index: 0x0658 */ 0x8D11,0x5E65,
	/* Array index: 0x0658 */ 0x8D12,0x5E66,
	/* Array index: 0x0658 */ 0x8D18,0x6650,
	/* Array index: 0x0658 */ 0x8D1A,0x604A,
	/* Array index: 0x0658 */ 0x8D1C,0x614E,
	/* Array index: 0x0658 */ 0x8D65,0x3933,
	/* Array index: 0x0658 */ 0x8D71,0x243A,
	/* Array index: 0x0658 */ 0x8D7A,0x3939,
	/* Array index: 0x0658 */ 0x8D7C,0x3937,
	/* Array index: 0x0658 */ 0x8D7E,0x3935,
	/* Array index: 0x0658 */ 0x8D7F,0x3938,
	/* Array index: 0x0658 */ 0x8D8B,0x3F38,
	/* Array index: 0x0658 */ 0x8D97,0x4975,
	/* Array index: 0x0658 */ 0x8D98,0x4973,
	/* Array index: 0x0658 */ 0x8D9A,0x4974,
	/* Array index: 0x0658 */ 0x8D9D,0x4F26,
	/* Array index: 0x0658 */ 0x8DA2,0x4F25,
	/* Array index: 0x0658 */ 0x8DA4,0x4F23,
	/* Array index: 0x0658 */ 0x8DA6,0x5277,
	/* Array index: 0x0658 */ 0x8DA9,0x5931,
	/* Array index: 0x0658 */ 0x8DB0,0x6651,
	/* Array index: 0x0658 */ 0x8DBB,0x393E,
	/* Array index: 0x0658 */ 0x8DBD,0x393A,
	/* Array index: 0x0658 */ 0x8DC0,0x393D,
	/* Array index: 0x0658 */ 0x8DC9,0x3F3F,
	/* Array index: 0x0658 */ 0x8DCA,0x3F3A,
	/* Array index: 0x0658 */ 0x8DD2,0x3F3B,
	/* Array index: 0x0658 */ 0x8DD4,0x3F3E,
	/* Array index: 0x0658 */ 0x8DED,0x446B,
	/* Array index: 0x0658 */ 0x8E01,0x497B,
	/* Array index: 0x0658 */ 0x8E08,0x497A,
	/* Array index: 0x0658 */ 0x8E0B,0x4979,
	/* Array index: 0x0658 */ 0x8E28,0x4F2C,
	/* Array index: 0x0658 */ 0x8E2A,0x4F2B,
	/* Array index: 0x0658 */ 0x8E32,0x527E,
	/* Array index: 0x0658 */ 0x8E3B,0x527B,
	/* Array index: 0x0658 */ 0x8E43,0x527C,
	/* Array index: 0x0658 */ 0x8E46,0x5654,
	/* Array index: 0x0658 */ 0x8E4F,0x5655,
	/* Array index: 0x0658 */ 0x8E58,0x5932,
	/* Array index: 0x0658 */ 0x8E68,0x5B3A,
	/* Array index: 0x0658 */ 0x8E6B,0x5B39,
	/* Array index: 0x0658 */ 0x8E70,0x5B3B,
	/* Array index: 0x0658 */ 0x8E75,0x5B3C,
	/* Array index: 0x0658 */ 0x8E77,0x5B3D,
	/* Array index: 0x0658 */ 0x8E83,0x5D37,
	/* Array index: 0x0658 */ 0x8E99,0x604B,
	/* Array index: 0x0658 */ 0x8EA2,0x6150,
	/* Array index: 0x0658 */ 0x8EA7,0x616C,
	/* Array index: 0x0658 */ 0x8EBC,0x6652,
	/* Array index: 0x0658 */ 0x8EBF,0x5935,
	/* Array index: 0x0658 */ 0x8EC1,0x5936,
	/* Array index: 0x0658 */ 0x8EC3,0x5B3F,
	/* Array index: 0x0658 */ 0x8EC6,0x5D38,
	/* Array index: 0x0658 */ 0x8ED5,0x6653,
	/* Array index: 0x0658 */ 0x8ED6,0x3941,
	/* Array index: 0x0658 */ 0x8ED9,0x3947,
	/* Array index: 0x0658 */ 0x8EE4,0x3F45,
	/* Array index: 0x0658 */ 0x8EEA,0x3F47,
	/* Array index: 0x0658 */ 0x8EF2,0x3F44,
	/* Array index: 0x0658 */ 0x8EF3,0x3F46,
	/* Array index: 0x0658 */ 0x8EFD,0x3F49,
	/* Array index: 0x0658 */ 0x8F04,0x4476,
	/* Array index: 0x0658 */ 0x8F19,0x4F36,
	/* Array index: 0x0658 */ 0x8F22,0x4F33,
	/* Array index: 0x0658 */ 0x8F27,0x4F35,
	/* Array index: 0x0658 */ 0x8F2B,0x4F34,
	/* Array index: 0x0658 */ 0x8F2D,0x5324,
	/* Array index: 0x0658 */ 0x8F30,0x5322,
	/* Array index: 0x0658 */ 0x8F31,0x5321,
	/* Array index: 0x0658 */ 0x8F3C,0x5325,
	/* Array index: 0x0658 */ 0x8F3D,0x5659,
	/* Array index: 0x0658 */ 0x8F41,0x565A,
	/* Array index: 0x0658 */ 0x8F4A,0x5937,
	/* Array index: 0x0658 */ 0x8F5C,0x5E67,
	/* Array index: 0x0658 */ 0x8F65,0x6222,
	/* Array index: 0x0658 */ 0x8FAA,0x6654,
	/* Array index: 0x0658 */ 0x8FAC,0x5938,
	/* Array index: 0x0658 */ 0x8FB3,0x4F39,
	/* Array index: 0x0658 */ 0x8FB5,0x272F,
	/* Array index: 0x0658 */ 0x8FB9,0x243B,
	/* Array index: 0x0658 */ 0x8FBE,0x2730,
	/* Array index: 0x0658 */ 0x8FC1,0x2733,
	/* Array index: 0x0658 */ 0x8FC7,0x2731,
	/* Array index: 0x0658 */ 0x8FCA,0x2A72,
	/* Array index: 0x0658 */ 0x8FCC,0x2A70,
	/* Array index: 0x0658 */ 0x8FCF,0x2A74,
	/* Array index: 0x0658 */ 0x8FD8,0x2A73,
	/* Array index: 0x0658 */ 0x8FD9,0x2A71,
	/* Array index: 0x0658 */ 0x8FE7,0x2F31,
	/* Array index: 0x0658 */ 0x8FE9,0x2F32,
	/* Array index: 0x0658 */ 0x8FEF,0x2F33,
	/* Array index: 0x0658 */ 0x8FF1,0x2F30,
	/* Array index: 0x0658 */ 0x8FF9,0x3425,
	/* Array index: 0x0658 */ 0x9008,0x3427,
	/* Array index: 0x0658 */ 0x9009,0x3426,
	/* Array index: 0x0658 */ 0x900E,0x394B,
	/* Array index: 0x0658 */ 0x9012,0x394A,
	/* Array index: 0x0658 */ 0x9013,0x394E,
	/* Array index: 0x0658 */ 0x9025,0x394C,
	/* Array index: 0x0658 */ 0x9030,0x3F55,
	/* Array index: 0x0658 */ 0x9037,0x3F56,
	/* Array index: 0x0658 */ 0x903A,0x3F54,
	/* Array index: 0x0658 */ 0x903B,0x3F57,
	/* Array index: 0x0658 */ 0x9040,0x447D,
	/* Array index: 0x0658 */ 0x9043,0x4479,
	/* Array index: 0x0658 */ 0x9045,0x447C,
	/* Array index: 0x0658 */ 0x9046,0x447A,
	/* Array index: 0x0658 */ 0x9048,0x447B,
	/* Array index: 0x0658 */ 0x904C,0x447E,
	/* Array index: 0x0658 */ 0x905A,0x4A24,
	/* Array index: 0x0658 */ 0x905F,0x4A27,
	/* Array index: 0x0658 */ 0x9061,0x4A26,
	/* Array index: 0x0658 */ 0x906A,0x4F3C,
	/* Array index: 0x0658 */ 0x9071,0x4F3B,
	/* Array index: 0x0658 */ 0x9089,0x593A,
	/* Array index: 0x0658 */ 0x9092,0x225E,
	/* Array index: 0x0658 */ 0x9093,0x225D,
	/* Array index: 0x0658 */ 0x9096,0x243C,
	/* Array index: 0x0658 */ 0x909A,0x243D,
	/* Array index: 0x0658 */ 0x909C,0x6655,
	/* Array index: 0x0658 */ 0x90A4,0x2735,
	/* Array index: 0x0658 */ 0x90A8,0x2739,
	/* Array index: 0x0658 */ 0x90A9,0x2734,
	/* Array index: 0x0658 */ 0x90AB,0x3951,
	/* Array index: 0x0658 */ 0x90AD,0x2A77,
	/* Array index: 0x0658 */ 0x90B7,0x2A75,
	/* Array index: 0x0658 */ 0x90B9,0x2A78,
	/* Array index: 0x0658 */ 0x90BC,0x2F35,
	/* Array index: 0x0658 */ 0x90C0,0x2F38,
	/* Array index: 0x0658 */ 0x90C2,0x2F34,
	/* Array index: 0x0658 */ 0x90C4,0x2F3E,
	/* Array index: 0x0658 */ 0x90C6,0x2F37,
	/* Array index: 0x0658 */ 0x90C9,0x2F3C,
	/* Array index: 0x0658 */ 0x90CD,0x2F3B,
	/* Array index: 0x0658 */ 0x90DE,0x3429,
	/* Array index: 0x0658 */ 0x90EE,0x394F,
	/* Array index: 0x0658 */ 0x90F6,0x636C,
	/* Array index: 0x0658 */ 0x90F7,0x3952,
	/* Array index: 0x0658 */ 0x910A,0x3F59,
	/* Array index: 0x0658 */ 0x910C,0x4523,
	/* Array index: 0x0658 */ 0x9113,0x4524,
	/* Array index: 0x0658 */ 0x9115,0x4525,
	/* Array index: 0x0658 */ 0x9125,0x4A2A,
	/* Array index: 0x0658 */ 0x9137,0x532B,
	/* Array index: 0x0658 */ 0x913C,0x593C,
	/* Array index: 0x0658 */ 0x913D,0x593B,
	/* Array index: 0x0658 */ 0x9151,0x342A,
	/* Array index: 0x0658 */ 0x9154,0x3955,
	/* Array index: 0x0658 */ 0x9159,0x3954,
	/* Array index: 0x0658 */ 0x915C,0x6657,
	/* Array index: 0x0658 */ 0x9167,0x4528,
	/* Array index: 0x0658 */ 0x916B,0x4527,
	/* Array index: 0x0658 */ 0x9176,0x4A2C,
	/* Array index: 0x0658 */ 0x917B,0x4A2B,
	/* Array index: 0x0658 */ 0x917C,0x4A2D,
	/* Array index: 0x0658 */ 0x9188,0x4F3F,
	/* Array index: 0x0658 */ 0x918C,0x4F40,
	/* Array index: 0x0658 */ 0x918E,0x532D,
	/* Array index: 0x0658 */ 0x9195,0x532C,
	/* Array index: 0x0658 */ 0x9196,0x532F,
	/* Array index: 0x0658 */ 0x91A9,0x593D,
	/* Array index: 0x0658 */ 0x91BB,0x5E68,
	/* Array index: 0x0658 */ 0x91BF,0x607E,
	/* Array index: 0x0658 */ 0x91C4,0x616D,
	/* Array index: 0x0658 */ 0x91D6,0x3430,
	/* Array index: 0x0658 */ 0x91DB,0x342D,
	/* Array index: 0x0658 */ 0x91E5,0x3958,
	/* Array index: 0x0658 */ 0x91F0,0x3957,
	/* Array index: 0x0658 */ 0x91F2,0x3956,
	/* Array index: 0x0658 */ 0x91F6,0x3959,
	/* Array index: 0x0658 */ 0x91FE,0x3F5D,
	/* Array index: 0x0658 */ 0x9208,0x3F66,
	/* Array index: 0x0658 */ 0x920B,0x3F60,
	/* Array index: 0x0658 */ 0x920E,0x3F68,
	/* Array index: 0x0658 */ 0x9213,0x3F69,
	/* Array index: 0x0658 */ 0x9218,0x3F62,
	/* Array index: 0x0658 */ 0x921B,0x3F65,
	/* Array index: 0x0658 */ 0x922B,0x6658,
	/* Array index: 0x0658 */ 0x922F,0x452E,
	/* Array index: 0x0658 */ 0x9235,0x452B,
	/* Array index: 0x0658 */ 0x923B,0x4537,
	/* Array index: 0x0658 */ 0x923C,0x452F,
	/* Array index: 0x0658 */ 0x9241,0x4539,
	/* Array index: 0x0658 */ 0x9244,0x4538,
	/* Array index: 0x0658 */ 0x9255,0x452A,
	/* Array index: 0x0658 */ 0x9259,0x452D,
	/* Array index: 0x0658 */ 0x925C,0x4531,
	/* Array index: 0x0658 */ 0x925D,0x4529,
	/* Array index: 0x0658 */ 0x925F,0x4534,
	/* Array index: 0x0658 */ 0x9262,0x4533,
	/* Array index: 0x0658 */ 0x926A,0x4532,
	/* Array index: 0x0658 */ 0x926B,0x4535,
	/* Array index: 0x0658 */ 0x926E,0x4536,
	/* Array index: 0x0658 */ 0x9273,0x665A,
	/* Array index: 0x0658 */ 0x9281,0x4A34,
	/* Array index: 0x0658 */ 0x9284,0x4A35,
	/* Array index: 0x0658 */ 0x9289,0x4A2F,
	/* Array index: 0x0658 */ 0x928F,0x4530,
	/* Array index: 0x0658 */ 0x9290,0x4A38,
	/* Array index: 0x0658 */ 0x9292,0x4A33,
	/* Array index: 0x0658 */ 0x929E,0x4A39,
	/* Array index: 0x0658 */ 0x92B8,0x4F42,
	/* Array index: 0x0658 */ 0x92B9,0x4F50,
	/* Array index: 0x0658 */ 0x92BE,0x4F43,
	/* Array index: 0x0658 */ 0x92BF,0x4F41,
	/* Array index: 0x0658 */ 0x92D4,0x4F48,
	/* Array index: 0x0658 */ 0x92D6,0x4F49,
	/* Array index: 0x0658 */ 0x92E2,0x4F51,
	/* Array index: 0x0658 */ 0x92E5,0x4F45,
	/* Array index: 0x0658 */ 0x92EB,0x4F52,
	/* Array index: 0x0658 */ 0x92ED,0x4F4C,
	/* Array index: 0x0658 */ 0x92FD,0x5335,
	/* Array index: 0x0658 */ 0x9303,0x533D,
	/* Array index: 0x0658 */ 0x9305,0x533E,
	/* Array index: 0x0658 */ 0x9307,0x5338,
	/* Array index: 0x0658 */ 0x930A,0x5332,
	/* Array index: 0x0658 */ 0x9317,0x5336,
	/* Array index: 0x0658 */ 0x931C,0x5337,
	/* Array index: 0x0658 */ 0x932C,0x533B,
	/* Array index: 0x0658 */ 0x933F,0x665D,
	/* Array index: 0x0658 */ 0x9342,0x665E,
	/* Array index: 0x0658 */ 0x9345,0x5331,
	/* Array index: 0x0658 */ 0x9348,0x5665,
	/* Array index: 0x0658 */ 0x9353,0x5666,
	/* Array index: 0x0658 */ 0x935D,0x565D,
	/* Array index: 0x0658 */ 0x935F,0x5667,
	/* Array index: 0x0658 */ 0x9362,0x5664,
	/* Array index: 0x0658 */ 0x9366,0x565B,
	/* Array index: 0x0658 */ 0x9368,0x5668,
	/* Array index: 0x0658 */ 0x9369,0x565E,
	/* Array index: 0x0658 */ 0x936B,0x566B,
	/* Array index: 0x0658 */ 0x936E,0x5661,
	/* Array index: 0x0658 */ 0x9378,0x565C,
	/* Array index: 0x0658 */ 0x937D,0x5660,
	/* Array index: 0x0658 */ 0x937F,0x566A,
	/* Array index: 0x0658 */ 0x9386,0x665F,
	/* Array index: 0x0658 */ 0x938B,0x5943,
	/* Array index: 0x0658 */ 0x9390,0x5940,
	/* Array index: 0x0658 */ 0x9393,0x5942,
	/* Array index: 0x0658 */ 0x939C,0x5949,
	/* Array index: 0x0658 */ 0x93A0,0x593E,
	/* Array index: 0x0658 */ 0x93AD,0x5944,
	/* Array index: 0x0658 */ 0x93B8,0x5946,
	/* Array index: 0x0658 */ 0x93C1,0x5B50,
	/* Array index: 0x0658 */ 0x93C5,0x5B44,
	/* Array index: 0x0658 */ 0x93C6,0x5B45,
	/* Array index: 0x0658 */ 0x93CB,0x5B43,
	/* Array index: 0x0658 */ 0x93D3,0x5B48,
	/* Array index: 0x0658 */ 0x93DB,0x5B4D,
	/* Array index: 0x0658 */ 0x93E0,0x5B4F,
	/* Array index: 0x0658 */ 0x93E5,0x5B4A,
	/* Array index: 0x0658 */ 0x9402,0x5D43,
	/* Array index: 0x0658 */ 0x9404,0x5D3C,
	/* Array index: 0x0658 */ 0x9405,0x5D3F,
	/* Array index: 0x0658 */ 0x9408,0x5D3E,
	/* Array index: 0x0658 */ 0x9417,0x5D3D,
	/* Array index: 0x0658 */ 0x9427,0x5D46,
	/* Array index: 0x0658 */ 0x942D,0x5E6D,
	/* Array index: 0x0658 */ 0x9442,0x5F69,
	/* Array index: 0x0658 */ 0x9443,0x5F6E,
	/* Array index: 0x0658 */ 0x944D,0x5F6A,
	/* Array index: 0x0658 */ 0x944E,0x5F6C,
	/* Array index: 0x0658 */ 0x9454,0x5F6B,
	/* Array index: 0x0658 */ 0x9456,0x6052,
	/* Array index: 0x0658 */ 0x945F,0x604F,
	/* Array index: 0x0658 */ 0x9461,0x6053,
	/* Array index: 0x0658 */ 0x946C,0x6122,
	/* Array index: 0x0658 */ 0x947A,0x616F,
	/* Array index: 0x0658 */ 0x947B,0x6223,
	/* Array index: 0x0658 */ 0x9484,0x6236,
	/* Array index: 0x0658 */ 0x9579,0x6323,
	/* Array index: 0x0658 */ 0x9581,0x2F40,
	/* Array index: 0x0658 */ 0x9585,0x3432,
	/* Array index: 0x0658 */ 0x9587,0x395C,
	/* Array index: 0x0658 */ 0x9595,0x3F6C,
	/* Array index: 0x0658 */ 0x9597,0x666B,
	/* Array index: 0x0658 */ 0x9599,0x453C,
	/* Array index: 0x0658 */ 0x959A,0x453A,
	/* Array index: 0x0658 */ 0x95A2,0x4A3D,
	/* Array index: 0x0658 */ 0x95A7,0x4A3E,
	/* Array index: 0x0658 */ 0x95AF,0x4F53,
	/* Array index: 0x0658 */ 0x95B2,0x4F55,
	/* Array index: 0x0658 */ 0x95B3,0x4F56,
	/* Array index: 0x0658 */ 0x95B8,0x5342,
	/* Array index: 0x0658 */ 0x95C1,0x5344,
	/* Array index: 0x0658 */ 0x95C2,0x5340,
	/* Array index: 0x0658 */ 0x95C4,0x566D,
	/* Array index: 0x0658 */ 0x95CE,0x666C,
	/* Array index: 0x0658 */ 0x95D7,0x594B,
	/* Array index: 0x0658 */ 0x95D8,0x594A,
	/* Array index: 0x0658 */ 0x95DD,0x5B53,
	/* Array index: 0x0658 */ 0x95E7,0x666D,
	/* Array index: 0x0658 */ 0x961D,0x2143,
	/* Array index: 0x0658 */ 0x9626,0x273B,
	/* Array index: 0x0658 */ 0x9627,0x273A,
	/* Array index: 0x0658 */ 0x9629,0x273F,
	/* Array index: 0x0658 */ 0x962B,0x273C,
	/* Array index: 0x0658 */ 0x9633,0x273D,
	/* Array index: 0x0658 */ 0x9634,0x273E,
	/* Array index: 0x0658 */ 0x9637,0x2A79,
	/* Array index: 0x0658 */ 0x963E,0x2A7C,
	/* Array index: 0x0658 */ 0x9641,0x2A7B,
	/* Array index: 0x0658 */ 0x9656,0x3436,
	/* Array index: 0x0658 */ 0x9659,0x3434,
	/* Array index: 0x0658 */ 0x965A,0x395E,
	/* Array index: 0x0658 */ 0x9660,0x3433,
	/* Array index: 0x0658 */ 0x966E,0x3961,
	/* Array index: 0x0658 */ 0x9679,0x3965,
	/* Array index: 0x0658 */ 0x967B,0x3F71,
	/* Array index: 0x0658 */ 0x967F,0x3F72,
	/* Array index: 0x0658 */ 0x9681,0x3F73,
	/* Array index: 0x0658 */ 0x9682,0x3F75,
	/* Array index: 0x0658 */ 0x968C,0x3F70,
	/* Array index: 0x0658 */ 0x968F,0x4540,
	/* Array index: 0x0658 */ 0x9696,0x4542,
	/* Array index: 0x0658 */ 0x969D,0x4A45,
	/* Array index: 0x0658 */ 0x969F,0x4A46,
	/* Array index: 0x0658 */ 0x96A0,0x4A44,
	/* Array index: 0x0658 */ 0x96A3,0x4F5A,
	/* Array index: 0x0658 */ 0x96A5,0x4F5B,
	/* Array index: 0x0658 */ 0x96AB,0x5348,
	/* Array index: 0x0658 */ 0x96AD,0x5670,
	/* Array index: 0x0658 */ 0x96AF,0x566F,
	/* Array index: 0x0658 */ 0x96B2,0x5671,
	/* Array index: 0x0658 */ 0x96BA,0x6325,
	/* Array index: 0x0658 */ 0x96BD,0x3439,
	/* Array index: 0x0658 */ 0x96D0,0x4A47,
	/* Array index: 0x0658 */ 0x96D1,0x4A49,
	/* Array index: 0x0658 */ 0x96E4,0x5E70,
	/* Array index: 0x0658 */ 0x96E6,0x6125,
	/* Array index: 0x0658 */ 0x96E7,0x622F,
	/* Array index: 0x0658 */ 0x96F4,0x4546,
	/* Array index: 0x0658 */ 0x96FC,0x4547,
	/* Array index: 0x0658 */ 0x970A,0x4F60,
	/* Array index: 0x0658 */ 0x970C,0x5350,
	/* Array index: 0x0658 */ 0x9714,0x534D,
	/* Array index: 0x0658 */ 0x9715,0x534C,
	/* Array index: 0x0658 */ 0x9717,0x5351,
	/* Array index: 0x0658 */ 0x971A,0x5674,
	/* Array index: 0x0658 */ 0x971B,0x5675,
	/* Array index: 0x0658 */ 0x9731,0x5D4F,
	/* Array index: 0x0658 */ 0x973B,0x666E,
	/* Array index: 0x0658 */ 0x973C,0x5F71,
	/* Array index: 0x0658 */ 0x9740,0x5F72,
	/* Array index: 0x0658 */ 0x9741,0x6059,
	/* Array index: 0x0658 */ 0x9745,0x6126,
	/* Array index: 0x0658 */ 0x974D,0x666F,
	/* Array index: 0x0658 */ 0x9750,0x6245,
	/* Array index: 0x0658 */ 0x9751,0x2A7E,
	/* Array index: 0x0658 */ 0x9755,0x4549,
	/* Array index: 0x0658 */ 0x9759,0x4A4C,
	/* Array index: 0x0658 */ 0x975D,0x594F,
	/* Array index: 0x0658 */ 0x975F,0x4021,
	/* Array index: 0x0658 */ 0x9763,0x2B21,
	/* Array index: 0x0658 */ 0x9764,0x4A4E,
	/* Array index: 0x0658 */ 0x9767,0x5E73,
	/* Array index: 0x0658 */ 0x976D,0x4024,
	/* Array index: 0x0658 */ 0x976F,0x4022,
	/* Array index: 0x0658 */ 0x9779,0x454B,
	/* Array index: 0x0658 */ 0x9787,0x4F62,
	/* Array index: 0x0658 */ 0x9789,0x4F63,
	/* Array index: 0x0658 */ 0x978C,0x4F64,
	/* Array index: 0x0658 */ 0x9793,0x5353,
	/* Array index: 0x0658 */ 0x979B,0x567A,
	/* Array index: 0x0658 */ 0x979F,0x567B,
	/* Array index: 0x0658 */ 0x97A9,0x5950,
	/* Array index: 0x0658 */ 0x97B8,0x5D50,
	/* Array index: 0x0658 */ 0x97BA,0x5D51,
	/* Array index: 0x0658 */ 0x97BD,0x5E75,
	/* Array index: 0x0658 */ 0x97BE,0x5E74,
	/* Array index: 0x0658 */ 0x97C0,0x5F73,
	/* Array index: 0x0658 */ 0x97C8,0x6128,
	/* Array index: 0x0658 */ 0x97CA,0x623B,
	/* Array index: 0x0658 */ 0x97D2,0x5355,
	/* Array index: 0x0658 */ 0x97DA,0x5951,
	/* Array index: 0x0658 */ 0x97E2,0x5E76,
	/* Array index: 0x0658 */ 0x97E4,0x6129,
	/* Array index: 0x0658 */ 0x97EE,0x454F,
	/* Array index: 0x0658 */ 0x97EF,0x4F65,
	/* Array index: 0x0658 */ 0x97F2,0x5B61,
	/* Array index: 0x0658 */ 0x97F5,0x4550,
	/* Array index: 0x0658 */ 0x97FC,0x5D53,
	/* Array index: 0x0658 */ 0x980B,0x4552,
	/* Array index: 0x0658 */ 0x9814,0x4A54,
	/* Array index: 0x0658 */ 0x9815,0x4A55,
	/* Array index: 0x0658 */ 0x9819,0x4A52,
	/* Array index: 0x0658 */ 0x981A,0x4A56,
	/* Array index: 0x0658 */ 0x981F,0x4F69,
	/* Array index: 0x0658 */ 0x9822,0x4F67,
	/* Array index: 0x0658 */ 0x9823,0x636E,
	/* Array index: 0x0658 */ 0x982A,0x4F66,
	/* Array index: 0x0658 */ 0x982E,0x535C,
	/* Array index: 0x0658 */ 0x9831,0x5359,
	/* Array index: 0x0658 */ 0x9833,0x535A,
	/* Array index: 0x0658 */ 0x9834,0x5228,
	/* Array index: 0x0658 */ 0x9836,0x5358,
	/* Array index: 0x0658 */ 0x983A,0x535D,
	/* Array index: 0x0658 */ 0x9847,0x5728,
	/* Array index: 0x0658 */ 0x984B,0x5959,
	/* Array index: 0x0658 */ 0x9861,0x642A,
	/* Array index: 0x0658 */ 0x9866,0x5E78,
	/* Array index: 0x0658 */ 0x9868,0x5E77,
	/* Array index: 0x0658 */ 0x98B5,0x5361,
	/* Array index: 0x0658 */ 0x98B7,0x572A,
	/* Array index: 0x0658 */ 0x98B9,0x595C,
	/* Array index: 0x0658 */ 0x98C3,0x5D56,
	/* Array index: 0x0658 */ 0x98C7,0x5E7B,
	/* Array index: 0x0658 */ 0x98C8,0x5E79,
	/* Array index: 0x0658 */ 0x98CA,0x5E7A,
	/* Array index: 0x0658 */ 0x98CD,0x6225,
	/* Array index: 0x0658 */ 0x98DC,0x5E7C,
	/* Array index: 0x0658 */ 0x98DD,0x6226,
	/* Array index: 0x0658 */ 0x98E1,0x3967,
	/* Array index: 0x0658 */ 0x98E4,0x6670,
	/* Array index: 0x0658 */ 0x98E6,0x3968,
	/* Array index: 0x0658 */ 0x98EC,0x455B,
	/* Array index: 0x0658 */ 0x98F0,0x402B,
	/* Array index: 0x0658 */ 0x98F1,0x455C,
	/* Array index: 0x0658 */ 0x98F3,0x4554,
	/* Array index: 0x0658 */ 0x98F5,0x4559,
	/* Array index: 0x0658 */ 0x98F7,0x4557,
	/* Array index: 0x0658 */ 0x98F8,0x4A5D,
	/* Array index: 0x0658 */ 0x98FF,0x4558,
	/* Array index: 0x0658 */ 0x9901,0x4A5E,
	/* Array index: 0x0658 */ 0x9906,0x4A5B,
	/* Array index: 0x0658 */ 0x990E,0x667A,
	/* Array index: 0x0658 */ 0x9919,0x4F6B,
	/* Array index: 0x0658 */ 0x991C,0x5365,
	/* Array index: 0x0658 */ 0x991D,0x5366,
	/* Array index: 0x0658 */ 0x9920,0x5364,
	/* Array index: 0x0658 */ 0x9922,0x5362,
	/* Array index: 0x0658 */ 0x9923,0x5363,
	/* Array index: 0x0658 */ 0x9936,0x5962,
	/* Array index: 0x0658 */ 0x9938,0x5961,
	/* Array index: 0x0658 */ 0x9939,0x5966,
	/* Array index: 0x0658 */ 0x993B,0x5965,
	/* Array index: 0x0658 */ 0x9940,0x5963,
	/* Array index: 0x0658 */ 0x9942,0x6671,
	/* Array index: 0x0658 */ 0x9944,0x5B69,
	/* Array index: 0x0658 */ 0x994D,0x5D58,
	/* Array index: 0x0658 */ 0x994F,0x5E7E,
	/* Array index: 0x0658 */ 0x995A,0x5F78,
	/* Array index: 0x0658 */ 0x995D,0x612A,
	/* Array index: 0x0658 */ 0x999A,0x455F,
	/* Array index: 0x0658 */ 0x99A0,0x5368,
	/* Array index: 0x0658 */ 0x99A2,0x5730,
	/* Array index: 0x0658 */ 0x99A4,0x5967,
	/* Array index: 0x0658 */ 0x99A9,0x5F21,
	/* Array index: 0x0658 */ 0x99AA,0x6060,
	/* Array index: 0x0658 */ 0x99C4,0x4A6A,
	/* Array index: 0x0658 */ 0x99C8,0x4F73,
	/* Array index: 0x0658 */ 0x99CA,0x4F6D,
	/* Array index: 0x0658 */ 0x99DA,0x4F6E,
	/* Array index: 0x0658 */ 0x99DE,0x4F72,
	/* Array index: 0x0658 */ 0x99E0,0x4F74,
	/* Array index: 0x0658 */ 0x99E1,0x5370,
	/* Array index: 0x0658 */ 0x99E6,0x5371,
	/* Array index: 0x0658 */ 0x99E8,0x536B,
	/* Array index: 0x0658 */ 0x99EB,0x536C,
	/* Array index: 0x0658 */ 0x99EF,0x536A,
	/* Array index: 0x0658 */ 0x99F3,0x5734,
	/* Array index: 0x0658 */ 0x99F5,0x5737,
	/* Array index: 0x0658 */ 0x9A00,0x5733,
	/* Array index: 0x0658 */ 0x9A08,0x596A,
	/* Array index: 0x0658 */ 0x9A0C,0x596B,
	/* Array index: 0x0658 */ 0x9A10,0x596D,
	/* Array index: 0x0658 */ 0x9A17,0x5B6F,
	/* Array index: 0x0658 */ 0x9A18,0x5B6D,
	/* Array index: 0x0658 */ 0x9A1F,0x5B6C,
	/* Array index: 0x0658 */ 0x9A21,0x5B6E,
	/* Array index: 0x0658 */ 0x9A26,0x5B6B,
	/* Array index: 0x0658 */ 0x9A2F,0x5D5A,
	/* Array index: 0x0658 */ 0x9A3B,0x5F24,
	/* Array index: 0x0658 */ 0x9A3C,0x5F22,
	/* Array index: 0x0658 */ 0x9A51,0x5F79,
	/* Array index: 0x0658 */ 0x9A58,0x6062,
	/* Array index: 0x0658 */ 0x9A5C,0x6061,
	/* Array index: 0x0658 */ 0x9A61,0x6172,
	/* Array index: 0x0658 */ 0x9A63,0x6173,
	/* Array index: 0x0658 */ 0x9AA9,0x402F,
	/* Array index: 0x0658 */ 0x9AB5,0x4F77,
	/* Array index: 0x0658 */ 0x9AB6,0x4F76,
	/* Array index: 0x0658 */ 0x9ABD,0x573A,
	/* Array index: 0x0658 */ 0x9AC4,0x5D61,
	/* Array index: 0x0658 */ 0x9AC8,0x5D60,
	/* Array index: 0x0658 */ 0x9AC9,0x5D5E,
	/* Array index: 0x0658 */ 0x9ACE,0x5F25,
	/* Array index: 0x0658 */ 0x9AD7,0x6174,
	/* Array index: 0x0658 */ 0x9AD9,0x6334,
	/* Array index: 0x0658 */ 0x9ADA,0x4A6E,
	/* Array index: 0x0658 */ 0x9ADD,0x5F7B,
	/* Array index: 0x0658 */ 0x9ADE,0x6063,
	/* Array index: 0x0658 */ 0x9AE0,0x4032,
	/* Array index: 0x0658 */ 0x9AE2,0x4561,
	/* Array index: 0x0658 */ 0x9AF4,0x4F79,
	/* Array index: 0x0658 */ 0x9AF5,0x5375,
	/* Array index: 0x0658 */ 0x9AF8,0x5374,
	/* Array index: 0x0658 */ 0x9B00,0x573B,
	/* Array index: 0x0658 */ 0x9B02,0x573C,
	/* Array index: 0x0658 */ 0x9B07,0x596F,
	/* Array index: 0x0658 */ 0x9B09,0x5B71,
	/* Array index: 0x0658 */ 0x9B14,0x5F27,
	/* Array index: 0x0658 */ 0x9B1D,0x6672,
	/* Array index: 0x0658 */ 0x9B21,0x612C,
	/* Array index: 0x0658 */ 0x9B26,0x4A75,
	/* Array index: 0x0658 */ 0x9B2A,0x5D62,
	/* Array index: 0x0658 */ 0x9B2C,0x612E,
	/* Array index: 0x0658 */ 0x9B2D,0x6433,
	/* Array index: 0x0658 */ 0x9B30,0x6228,
	/* Array index: 0x0658 */ 0x9B34,0x573E,
	/* Array index: 0x0658 */ 0x9B38,0x5D63,
	/* Array index: 0x0658 */ 0x9B3D,0x4562,
	/* Array index: 0x0658 */ 0x9B40,0x4A78,
	/* Array index: 0x0658 */ 0x9B49,0x573F,
	/* Array index: 0x0658 */ 0x9B50,0x5D64,
	/* Array index: 0x0658 */ 0x9B53,0x5F28,
	/* Array index: 0x0658 */ 0x9B57,0x612F,
	/* Array index: 0x0658 */ 0x9B5D,0x4563,
	/* Array index: 0x0658 */ 0x9B63,0x4F7E,
	/* Array index: 0x0658 */ 0x9B69,0x4F7C,
	/* Array index: 0x0658 */ 0x9B6B,0x4F7B,
	/* Array index: 0x0658 */ 0x9B72,0x4F7D,
	/* Array index: 0x0658 */ 0x9B83,0x5378,
	/* Array index: 0x0658 */ 0x9B84,0x537A,
	/* Array index: 0x0658 */ 0x9B89,0x5423,
	/* Array index: 0x0658 */ 0x9B94,0x5379,
	/* Array index: 0x0658 */ 0x9B98,0x6673,
	/* Array index: 0x0658 */ 0x9B99,0x5744,
	/* Array index: 0x0658 */ 0x9B9D,0x5745,
	/* Array index: 0x0658 */ 0x9B9F,0x5740,
	/* Array index: 0x0658 */ 0x9BA3,0x5742,
	/* Array index: 0x0658 */ 0x9BBB,0x5974,
	/* Array index: 0x0658 */ 0x9BC2,0x5973,
	/* Array index: 0x0658 */ 0x9BC5,0x5B74,
	/* Array index: 0x0658 */ 0x9BD2,0x5971,
	/* Array index: 0x0658 */ 0x9BDF,0x5B75,
	/* Array index: 0x0658 */ 0x9BE3,0x5B77,
	/* Array index: 0x0658 */ 0x9BE9,0x5B79,
	/* Array index: 0x0658 */ 0x9BEE,0x5B7A,
	/* Array index: 0x0658 */ 0x9BF6,0x5D6F,
	/* Array index: 0x0658 */ 0x9BF9,0x5D71,
	/* Array index: 0x0658 */ 0x9C0A,0x5D66,
	/* Array index: 0x0658 */ 0x9C0C,0x5D76,
	/* Array index: 0x0658 */ 0x9C10,0x5D73,
	/* Array index: 0x0658 */ 0x9C15,0x5D72,
	/* Array index: 0x0658 */ 0x9C1B,0x5D70,
	/* Array index: 0x0658 */ 0x9C1F,0x5F2C,
	/* Array index: 0x0658 */ 0x9C27,0x642F,
	/* Array index: 0x0658 */ 0x9C2E,0x5F2A,
	/* Array index: 0x0658 */ 0x9C35,0x6025,
	/* Array index: 0x0658 */ 0x9C38,0x5F7E,
	/* Array index: 0x0658 */ 0x9C3A,0x6023,
	/* Array index: 0x0658 */ 0x9C45,0x6022,
	/* Array index: 0x0658 */ 0x9C4F,0x6065,
	/* Array index: 0x0658 */ 0x9C51,0x6066,
	/* Array index: 0x0658 */ 0x9C53,0x6068,
	/* Array index: 0x0658 */ 0x9C5D,0x6132,
	/* Array index: 0x0658 */ 0x9C61,0x6133,
	/* Array index: 0x0658 */ 0x9C65,0x6134,
	/* Array index: 0x0658 */ 0x9C6C,0x6159,
	/* Array index: 0x0658 */ 0x9C6F,0x615B,
	/* Array index: 0x0658 */ 0x9C76,0x6175,
	/* Array index: 0x0658 */ 0x9C7B,0x6241,
	/* Array index: 0x0658 */ 0x9CE8,0x4564,
	/* Array index: 0x0658 */ 0x9CF8,0x5027,
	/* Array index: 0x0658 */ 0x9CFE,0x502A,
	/* Array index: 0x0658 */ 0x9D01,0x5024,
	/* Array index: 0x0658 */ 0x9D02,0x502B,
	/* Array index: 0x0658 */ 0x9D0A,0x542D,
	/* Array index: 0x0658 */ 0x9D0C,0x5025,
	/* Array index: 0x0658 */ 0x9D0D,0x5022,
	/* Array index: 0x0658 */ 0x9D0F,0x542A,
	/* Array index: 0x0658 */ 0x9D13,0x542C,
	/* Array index: 0x0658 */ 0x9D16,0x5436,
	/* Array index: 0x0658 */ 0x9D1A,0x5433,
	/* Array index: 0x0658 */ 0x9D21,0x5437,
	/* Array index: 0x0658 */ 0x9D24,0x5429,
	/* Array index: 0x0658 */ 0x9D27,0x5435,
	/* Array index: 0x0658 */ 0x9D2A,0x5432,
	/* Array index: 0x0658 */ 0x9D35,0x574D,
	/* Array index: 0x0658 */ 0x9D39,0x5748,
	/* Array index: 0x0658 */ 0x9D44,0x574A,
	/* Array index: 0x0658 */ 0x9D49,0x6675,
	/* Array index: 0x0658 */ 0x9D4D,0x5979,
	/* Array index: 0x0658 */ 0x9D4E,0x597B,
	/* Array index: 0x0658 */ 0x9D50,0x597D,
	/* Array index: 0x0658 */ 0x9D55,0x597E,
	/* Array index: 0x0658 */ 0x9D5E,0x5A22,
	/* Array index: 0x0658 */ 0x9D63,0x597A,
	/* Array index: 0x0658 */ 0x9D66,0x5B7D,
	/* Array index: 0x0658 */ 0x9D6D,0x5C27,
	/* Array index: 0x0658 */ 0x9D6E,0x5C22,
	/* Array index: 0x0658 */ 0x9D76,0x5C25,
	/* Array index: 0x0658 */ 0x9D7A,0x5B7E,
	/* Array index: 0x0658 */ 0x9D7E,0x5C26,
	/* Array index: 0x0658 */ 0x9D83,0x5C24,
	/* Array index: 0x0658 */ 0x9D91,0x5C23,
	/* Array index: 0x0658 */ 0x9D95,0x5D79,
	/* Array index: 0x0658 */ 0x9DA5,0x5D7A,
	/* Array index: 0x0658 */ 0x9DAE,0x5F35,
	/* Array index: 0x0658 */ 0x9DB0,0x5F31,
	/* Array index: 0x0658 */ 0x9DBD,0x5F32,
	/* Array index: 0x0658 */ 0x9DC4,0x5F36,
	/* Array index: 0x0658 */ 0x9DFC,0x606C,
	/* Array index: 0x0658 */ 0x9E08,0x6135,
	/* Array index: 0x0658 */ 0x9E0E,0x615D,
	/* Array index: 0x0658 */ 0x9E16,0x6229,
	/* Array index: 0x0658 */ 0x9E18,0x6230,
	/* Array index: 0x0658 */ 0x9E1C,0x6238,
	/* Array index: 0x0658 */ 0x9E7B,0x5F37,
	/* Array index: 0x0658 */ 0x9E81,0x4569,
	/* Array index: 0x0658 */ 0x9E84,0x502C,
	/* Array index: 0x0658 */ 0x9E85,0x5439,
	/* Array index: 0x0658 */ 0x9E90,0x5A24,
	/* Array index: 0x0658 */ 0x9E95,0x5C29,
	/* Array index: 0x0658 */ 0x9E98,0x5E21,
	/* Array index: 0x0658 */ 0x9E9E,0x5F3A,
	/* Array index: 0x0658 */ 0x9EA2,0x6231,
	/* Array index: 0x0658 */ 0x9EA3,0x623F,
	/* Array index: 0x0658 */ 0x9EA6,0x2547,
	/* Array index: 0x0658 */ 0x9EAF,0x5750,
	/* Array index: 0x0658 */ 0x9EB1,0x5A26,
	/* Array index: 0x0658 */ 0x9EB2,0x5A25,
	/* Array index: 0x0658 */ 0x9EBD,0x4B22,
	/* Array index: 0x0658 */ 0x9EC1,0x5E22,
	/* Array index: 0x0658 */ 0x9ECA,0x5A28,
	/* Array index: 0x0658 */ 0x9ED2,0x396B,
	/* Array index: 0x0658 */ 0x9ED7,0x543D,
	/* Array index: 0x0658 */ 0x9EE3,0x5C2A,
	/* Array index: 0x0658 */ 0x9EF1,0x6029,
	/* Array index: 0x0658 */ 0x9EFE,0x2772,
	/* Array index: 0x0658 */ 0x9F08,0x615F,
	/* Array index: 0x0658 */ 0x9F14,0x456A,
	/* Array index: 0x0658 */ 0x9F17,0x5C30,
	/* Array index: 0x0658 */ 0x9F1D,0x6160,
	/* Array index: 0x0658 */ 0x9F26,0x5A2B,
	/* Array index: 0x0658 */ 0x9F27,0x5A2A,
	/* Array index: 0x0658 */ 0x9F39,0x606D,
	/* Array index: 0x0658 */ 0x9F45,0x613B,
	/* Array index: 0x0658 */ 0x9F50,0x2370,
	/* Array index: 0x0658 */ 0x9F53,0x543E,
	/* Array index: 0x0658 */ 0x9F5A,0x5E24,
	/* Array index: 0x0658 */ 0x9F69,0x5F3E,
	/* Array index: 0x0658 */ 0x9F7C,0x6233,
	/* Array index: 0x0658 */ 0x9F8E,0x5755,
	/* Array index: 0x0658 */ 0x9F93,0x602B,
	/* Array index: 0x0658 */ 0x9F97,0x6242,
	/* Array index: 0x0658 */ 0x9F9D,0x5E47,
	/* Array index: 0x0658 */ 0x9FA1,0x5F3F,
	/* Array index: 0x0658 */ 0x9FA3,0x6161,
	/* Array index: 0x0658 */ 0x9FA5,0x6177,
};

#endif /* ICONV_FROM_UCS_CCS_CNS11643_PLANE14 && defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * cns11643_plane14 CCS description table.
 * ======================================================================
 */
const iconv_ccs_t
_iconv_ccs_cns11643_plane14 =
{
	TABLE_VERSION_1, /* Table version */
	ICONV_CCS_CNS11643_PLANE14, /* CCS name */
	TABLE_16BIT, /* Table bits */
#if defined (ICONV_FROM_UCS_CCS_CNS11643_PLANE14) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SIZE_OPTIMIZED,
	(__uint16_t *)&from_ucs_size_cns11643_plane14, /* UCS -> cns11643_plane14 table size-optimized table */
#elif defined (ICONV_FROM_UCS_CCS_CNS11643_PLANE14) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)&from_ucs_speed_cns11643_plane14, /* UCS -> cns11643_plane14 table speed-optimized table */
#else
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)NULL,
#endif
#if defined (ICONV_TO_UCS_CCS_CNS11643_PLANE14) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SIZE_OPTIMIZED,
	(__uint16_t *)&to_ucs_size_cns11643_plane14 /* cns11643_plane14 -> UCS table speed-optimized table */
#elif defined (ICONV_TO_UCS_CCS_CNS11643_PLANE14) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)&to_ucs_speed_cns11643_plane14 /* cns11643_plane14 -> UCS table speed-optimized table */
#else
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)NULL,
#endif
};

#endif /* ICONV_TO_UCS_CCS_CNS11643_PLANE14) || ... */

