/*
 * This file was generated automatically - don't edit it.
 * File contains iconv CCS tables for jis_x0212_1990 encoding.
 */

#include "ccsbi.h"

#if defined (ICONV_TO_UCS_CCS_JIS_X0212_1990) \
 || defined (ICONV_FROM_UCS_CCS_JIS_X0212_1990)

#include <_ansi.h>
#include <sys/types.h>
#include <sys/param.h>
#include "ccs.h"
#include "ccsnames.h"

/*
 * 16-bit jis_x0212_1990 -> UCS speed-optimized table (35328 bytes).
 * ======================================================================
 */
#if defined (ICONV_TO_UCS_CCS_JIS_X0212_1990) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
to_ucs_speed_jis_x0212_1990[] =
{
	/* Heading Block */
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,0x0100,INVBLK,INVBLK,INVBLK,0x0200,0x0300,
	INVBLK,0x0400,0x0500,0x0600,INVBLK,INVBLK,INVBLK,INVBLK,
	0x0700,0x0800,0x0900,0x0A00,0x0B00,0x0C00,0x0D00,0x0E00,
	0x0F00,0x1000,0x1100,0x1200,0x1300,0x1400,0x1500,0x1600,
	0x1700,0x1800,0x1900,0x1A00,0x1B00,0x1C00,0x1D00,0x1E00,
	0x1F00,0x2000,0x2100,0x2200,0x2300,0x2400,0x2500,0x2600,
	0x2700,0x2800,0x2900,0x2A00,0x2B00,0x2C00,0x2D00,0x2E00,
	0x2F00,0x3000,0x3100,0x3200,0x3300,0x3400,0x3500,0x3600,
	0x3700,0x3800,0x3900,0x3A00,0x3B00,0x3C00,0x3D00,0x3E00,
	0x3F00,0x4000,0x4100,0x4200,0x4300,0x4400,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	/* Block 35, Array index 0x0100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x02D8,
	0x02C7,0x00B8,0x02D9,0x02DD,0x00AF,0x02DB,0x02DA,0x007E,
	0x0384,0x0385,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x00A1,0x00A6,0x00BF,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x00BA,0x00AA,0x00A9,0x00AE,0x2122,
	0x00A4,0x2116,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 39, Array index 0x0200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x0386,0x0388,0x0389,0x038A,0x03AA,INVALC,0x038C,
	INVALC,0x038E,0x03AB,INVALC,0x038F,INVALC,INVALC,INVALC,
	INVALC,0x03AC,0x03AD,0x03AE,0x03AF,0x03CA,0x0390,0x03CC,
	0x03C2,0x03CD,0x03CB,0x03B0,0x03CE,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 40, Array index 0x0300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x0402,0x0403,0x0404,0x0405,0x0406,0x0407,
	0x0408,0x0409,0x040A,0x040B,0x040C,0x040E,0x040F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x0452,0x0453,0x0454,0x0455,0x0456,0x0457,
	0x0458,0x0459,0x045A,0x045B,0x045C,0x045E,0x045F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 42, Array index 0x0400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x00C6,0x0110,INVALC,0x0126,INVALC,0x0132,INVALC,
	0x0141,0x013F,INVALC,0x014A,0x00D8,0x0152,INVALC,0x0166,
	0x00DE,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x00E6,0x0111,0x00F0,0x0127,0x0131,0x0133,0x0138,
	0x0142,0x0140,0x0149,0x014B,0x00F8,0x0153,0x00DF,0x0167,
	0x00FE,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 43, Array index 0x0500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x00C1,0x00C0,0x00C4,0x00C2,0x0102,0x01CD,0x0100,
	0x0104,0x00C5,0x00C3,0x0106,0x0108,0x010C,0x00C7,0x010A,
	0x010E,0x00C9,0x00C8,0x00CB,0x00CA,0x011A,0x0116,0x0112,
	0x0118,INVALC,0x011C,0x011E,0x0122,0x0120,0x0124,0x00CD,
	0x00CC,0x00CF,0x00CE,0x01CF,0x0130,0x012A,0x012E,0x0128,
	0x0134,0x0136,0x0139,0x013D,0x013B,0x0143,0x0147,0x0145,
	0x00D1,0x00D3,0x00D2,0x00D6,0x00D4,0x01D1,0x0150,0x014C,
	0x00D5,0x0154,0x0158,0x0156,0x015A,0x015C,0x0160,0x015E,
	0x0164,0x0162,0x00DA,0x00D9,0x00DC,0x00DB,0x016C,0x01D3,
	0x0170,0x016A,0x0172,0x016E,0x0168,0x01D7,0x01DB,0x01D9,
	0x01D5,0x0174,0x00DD,0x0178,0x0176,0x0179,0x017D,0x017B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 44, Array index 0x0600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x00E1,0x00E0,0x00E4,0x00E2,0x0103,0x01CE,0x0101,
	0x0105,0x00E5,0x00E3,0x0107,0x0109,0x010D,0x00E7,0x010B,
	0x010F,0x00E9,0x00E8,0x00EB,0x00EA,0x011B,0x0117,0x0113,
	0x0119,0x01F5,0x011D,0x011F,INVALC,0x0121,0x0125,0x00ED,
	0x00EC,0x00EF,0x00EE,0x01D0,INVALC,0x012B,0x012F,0x0129,
	0x0135,0x0137,0x013A,0x013E,0x013C,0x0144,0x0148,0x0146,
	0x00F1,0x00F3,0x00F2,0x00F6,0x00F4,0x01D2,0x0151,0x014D,
	0x00F5,0x0155,0x0159,0x0157,0x015B,0x015D,0x0161,0x015F,
	0x0165,0x0163,0x00FA,0x00F9,0x00FC,0x00FB,0x016D,0x01D4,
	0x0171,0x016B,0x0173,0x016F,0x0169,0x01D8,0x01DC,0x01DA,
	0x01D6,0x0175,0x00FD,0x00FF,0x0177,0x017A,0x017E,0x017C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 49, Array index 0x0700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E02,0x4E04,0x4E05,0x4E0C,0x4E12,0x4E1F,0x4E23,
	0x4E24,0x4E28,0x4E2B,0x4E2E,0x4E2F,0x4E30,0x4E35,0x4E40,
	0x4E41,0x4E44,0x4E47,0x4E51,0x4E5A,0x4E5C,0x4E63,0x4E68,
	0x4E69,0x4E74,0x4E75,0x4E79,0x4E7F,0x4E8D,0x4E96,0x4E97,
	0x4E9D,0x4EAF,0x4EB9,0x4EC3,0x4ED0,0x4EDA,0x4EDB,0x4EE0,
	0x4EE1,0x4EE2,0x4EE8,0x4EEF,0x4EF1,0x4EF3,0x4EF5,0x4EFD,
	0x4EFE,0x4EFF,0x4F00,0x4F02,0x4F03,0x4F08,0x4F0B,0x4F0C,
	0x4F12,0x4F15,0x4F16,0x4F17,0x4F19,0x4F2E,0x4F31,0x4F60,
	0x4F33,0x4F35,0x4F37,0x4F39,0x4F3B,0x4F3E,0x4F40,0x4F42,
	0x4F48,0x4F49,0x4F4B,0x4F4C,0x4F52,0x4F54,0x4F56,0x4F58,
	0x4F5F,0x4F63,0x4F6A,0x4F6C,0x4F6E,0x4F71,0x4F77,0x4F78,
	0x4F79,0x4F7A,0x4F7D,0x4F7E,0x4F81,0x4F82,0x4F84,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 50, Array index 0x0800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F85,0x4F89,0x4F8A,0x4F8C,0x4F8E,0x4F90,0x4F92,
	0x4F93,0x4F94,0x4F97,0x4F99,0x4F9A,0x4F9E,0x4F9F,0x4FB2,
	0x4FB7,0x4FB9,0x4FBB,0x4FBC,0x4FBD,0x4FBE,0x4FC0,0x4FC1,
	0x4FC5,0x4FC6,0x4FC8,0x4FC9,0x4FCB,0x4FCC,0x4FCD,0x4FCF,
	0x4FD2,0x4FDC,0x4FE0,0x4FE2,0x4FF0,0x4FF2,0x4FFC,0x4FFD,
	0x4FFF,0x5000,0x5001,0x5004,0x5007,0x500A,0x500C,0x500E,
	0x5010,0x5013,0x5017,0x5018,0x501B,0x501C,0x501D,0x501E,
	0x5022,0x5027,0x502E,0x5030,0x5032,0x5033,0x5035,0x5040,
	0x5041,0x5042,0x5045,0x5046,0x504A,0x504C,0x504E,0x5051,
	0x5052,0x5053,0x5057,0x5059,0x505F,0x5060,0x5062,0x5063,
	0x5066,0x5067,0x506A,0x506D,0x5070,0x5071,0x503B,0x5081,
	0x5083,0x5084,0x5086,0x508A,0x508E,0x508F,0x5090,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 51, Array index 0x0900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5092,0x5093,0x5094,0x5096,0x509B,0x509C,0x509E,
	0x509F,0x50A0,0x50A1,0x50A2,0x50AA,0x50AF,0x50B0,0x50B9,
	0x50BA,0x50BD,0x50C0,0x50C3,0x50C4,0x50C7,0x50CC,0x50CE,
	0x50D0,0x50D3,0x50D4,0x50D8,0x50DC,0x50DD,0x50DF,0x50E2,
	0x50E4,0x50E6,0x50E8,0x50E9,0x50EF,0x50F1,0x50F6,0x50FA,
	0x50FE,0x5103,0x5106,0x5107,0x5108,0x510B,0x510C,0x510D,
	0x510E,0x50F2,0x5110,0x5117,0x5119,0x511B,0x511C,0x511D,
	0x511E,0x5123,0x5127,0x5128,0x512C,0x512D,0x512F,0x5131,
	0x5133,0x5134,0x5135,0x5138,0x5139,0x5142,0x514A,0x514F,
	0x5153,0x5155,0x5157,0x5158,0x515F,0x5164,0x5166,0x517E,
	0x5183,0x5184,0x518B,0x518E,0x5198,0x519D,0x51A1,0x51A3,
	0x51AD,0x51B8,0x51BA,0x51BC,0x51BE,0x51BF,0x51C2,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 52, Array index 0x0A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x51C8,0x51CF,0x51D1,0x51D2,0x51D3,0x51D5,0x51D8,
	0x51DE,0x51E2,0x51E5,0x51EE,0x51F2,0x51F3,0x51F4,0x51F7,
	0x5201,0x5202,0x5205,0x5212,0x5213,0x5215,0x5216,0x5218,
	0x5222,0x5228,0x5231,0x5232,0x5235,0x523C,0x5245,0x5249,
	0x5255,0x5257,0x5258,0x525A,0x525C,0x525F,0x5260,0x5261,
	0x5266,0x526E,0x5277,0x5278,0x5279,0x5280,0x5282,0x5285,
	0x528A,0x528C,0x5293,0x5295,0x5296,0x5297,0x5298,0x529A,
	0x529C,0x52A4,0x52A5,0x52A6,0x52A7,0x52AF,0x52B0,0x52B6,
	0x52B7,0x52B8,0x52BA,0x52BB,0x52BD,0x52C0,0x52C4,0x52C6,
	0x52C8,0x52CC,0x52CF,0x52D1,0x52D4,0x52D6,0x52DB,0x52DC,
	0x52E1,0x52E5,0x52E8,0x52E9,0x52EA,0x52EC,0x52F0,0x52F1,
	0x52F4,0x52F6,0x52F7,0x5300,0x5303,0x530A,0x530B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 53, Array index 0x0B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x530C,0x5311,0x5313,0x5318,0x531B,0x531C,0x531E,
	0x531F,0x5325,0x5327,0x5328,0x5329,0x532B,0x532C,0x532D,
	0x5330,0x5332,0x5335,0x533C,0x533D,0x533E,0x5342,0x534C,
	0x534B,0x5359,0x535B,0x5361,0x5363,0x5365,0x536C,0x536D,
	0x5372,0x5379,0x537E,0x5383,0x5387,0x5388,0x538E,0x5393,
	0x5394,0x5399,0x539D,0x53A1,0x53A4,0x53AA,0x53AB,0x53AF,
	0x53B2,0x53B4,0x53B5,0x53B7,0x53B8,0x53BA,0x53BD,0x53C0,
	0x53C5,0x53CF,0x53D2,0x53D3,0x53D5,0x53DA,0x53DD,0x53DE,
	0x53E0,0x53E6,0x53E7,0x53F5,0x5402,0x5413,0x541A,0x5421,
	0x5427,0x5428,0x542A,0x542F,0x5431,0x5434,0x5435,0x5443,
	0x5444,0x5447,0x544D,0x544F,0x545E,0x5462,0x5464,0x5466,
	0x5467,0x5469,0x546B,0x546D,0x546E,0x5474,0x547F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 54, Array index 0x0C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5481,0x5483,0x5485,0x5488,0x5489,0x548D,0x5491,
	0x5495,0x5496,0x549C,0x549F,0x54A1,0x54A6,0x54A7,0x54A9,
	0x54AA,0x54AD,0x54AE,0x54B1,0x54B7,0x54B9,0x54BA,0x54BB,
	0x54BF,0x54C6,0x54CA,0x54CD,0x54CE,0x54E0,0x54EA,0x54EC,
	0x54EF,0x54F6,0x54FC,0x54FE,0x54FF,0x5500,0x5501,0x5505,
	0x5508,0x5509,0x550C,0x550D,0x550E,0x5515,0x552A,0x552B,
	0x5532,0x5535,0x5536,0x553B,0x553C,0x553D,0x5541,0x5547,
	0x5549,0x554A,0x554D,0x5550,0x5551,0x5558,0x555A,0x555B,
	0x555E,0x5560,0x5561,0x5564,0x5566,0x557F,0x5581,0x5582,
	0x5586,0x5588,0x558E,0x558F,0x5591,0x5592,0x5593,0x5594,
	0x5597,0x55A3,0x55A4,0x55AD,0x55B2,0x55BF,0x55C1,0x55C3,
	0x55C6,0x55C9,0x55CB,0x55CC,0x55CE,0x55D1,0x55D2,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 55, Array index 0x0D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x55D3,0x55D7,0x55D8,0x55DB,0x55DE,0x55E2,0x55E9,
	0x55F6,0x55FF,0x5605,0x5608,0x560A,0x560D,0x560E,0x560F,
	0x5610,0x5611,0x5612,0x5619,0x562C,0x5630,0x5633,0x5635,
	0x5637,0x5639,0x563B,0x563C,0x563D,0x563F,0x5640,0x5641,
	0x5643,0x5644,0x5646,0x5649,0x564B,0x564D,0x564F,0x5654,
	0x565E,0x5660,0x5661,0x5662,0x5663,0x5666,0x5669,0x566D,
	0x566F,0x5671,0x5672,0x5675,0x5684,0x5685,0x5688,0x568B,
	0x568C,0x5695,0x5699,0x569A,0x569D,0x569E,0x569F,0x56A6,
	0x56A7,0x56A8,0x56A9,0x56AB,0x56AC,0x56AD,0x56B1,0x56B3,
	0x56B7,0x56BE,0x56C5,0x56C9,0x56CA,0x56CB,0x56CF,0x56D0,
	0x56CC,0x56CD,0x56D9,0x56DC,0x56DD,0x56DF,0x56E1,0x56E4,
	0x56E5,0x56E6,0x56E7,0x56E8,0x56F1,0x56EB,0x56ED,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 56, Array index 0x0E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x56F6,0x56F7,0x5701,0x5702,0x5707,0x570A,0x570C,
	0x5711,0x5715,0x571A,0x571B,0x571D,0x5720,0x5722,0x5723,
	0x5724,0x5725,0x5729,0x572A,0x572C,0x572E,0x572F,0x5733,
	0x5734,0x573D,0x573E,0x573F,0x5745,0x5746,0x574C,0x574D,
	0x5752,0x5762,0x5765,0x5767,0x5768,0x576B,0x576D,0x576E,
	0x576F,0x5770,0x5771,0x5773,0x5774,0x5775,0x5777,0x5779,
	0x577A,0x577B,0x577C,0x577E,0x5781,0x5783,0x578C,0x5794,
	0x5797,0x5799,0x579A,0x579C,0x579D,0x579E,0x579F,0x57A1,
	0x5795,0x57A7,0x57A8,0x57A9,0x57AC,0x57B8,0x57BD,0x57C7,
	0x57C8,0x57CC,0x57CF,0x57D5,0x57DD,0x57DE,0x57E4,0x57E6,
	0x57E7,0x57E9,0x57ED,0x57F0,0x57F5,0x57F6,0x57F8,0x57FD,
	0x57FE,0x57FF,0x5803,0x5804,0x5808,0x5809,0x57E1,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 57, Array index 0x0F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x580C,0x580D,0x581B,0x581E,0x581F,0x5820,0x5826,
	0x5827,0x582D,0x5832,0x5839,0x583F,0x5849,0x584C,0x584D,
	0x584F,0x5850,0x5855,0x585F,0x5861,0x5864,0x5867,0x5868,
	0x5878,0x587C,0x587F,0x5880,0x5881,0x5887,0x5888,0x5889,
	0x588A,0x588C,0x588D,0x588F,0x5890,0x5894,0x5896,0x589D,
	0x58A0,0x58A1,0x58A2,0x58A6,0x58A9,0x58B1,0x58B2,0x58C4,
	0x58BC,0x58C2,0x58C8,0x58CD,0x58CE,0x58D0,0x58D2,0x58D4,
	0x58D6,0x58DA,0x58DD,0x58E1,0x58E2,0x58E9,0x58F3,0x5905,
	0x5906,0x590B,0x590C,0x5912,0x5913,0x5914,0x8641,0x591D,
	0x5921,0x5923,0x5924,0x5928,0x592F,0x5930,0x5933,0x5935,
	0x5936,0x593F,0x5943,0x5946,0x5952,0x5953,0x5959,0x595B,
	0x595D,0x595E,0x595F,0x5961,0x5963,0x596B,0x596D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 58, Array index 0x1000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x596F,0x5972,0x5975,0x5976,0x5979,0x597B,0x597C,
	0x598B,0x598C,0x598E,0x5992,0x5995,0x5997,0x599F,0x59A4,
	0x59A7,0x59AD,0x59AE,0x59AF,0x59B0,0x59B3,0x59B7,0x59BA,
	0x59BC,0x59C1,0x59C3,0x59C4,0x59C8,0x59CA,0x59CD,0x59D2,
	0x59DD,0x59DE,0x59DF,0x59E3,0x59E4,0x59E7,0x59EE,0x59EF,
	0x59F1,0x59F2,0x59F4,0x59F7,0x5A00,0x5A04,0x5A0C,0x5A0D,
	0x5A0E,0x5A12,0x5A13,0x5A1E,0x5A23,0x5A24,0x5A27,0x5A28,
	0x5A2A,0x5A2D,0x5A30,0x5A44,0x5A45,0x5A47,0x5A48,0x5A4C,
	0x5A50,0x5A55,0x5A5E,0x5A63,0x5A65,0x5A67,0x5A6D,0x5A77,
	0x5A7A,0x5A7B,0x5A7E,0x5A8B,0x5A90,0x5A93,0x5A96,0x5A99,
	0x5A9C,0x5A9E,0x5A9F,0x5AA0,0x5AA2,0x5AA7,0x5AAC,0x5AB1,
	0x5AB2,0x5AB3,0x5AB5,0x5AB8,0x5ABA,0x5ABB,0x5ABF,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 59, Array index 0x1100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5AC4,0x5AC6,0x5AC8,0x5ACF,0x5ADA,0x5ADC,0x5AE0,
	0x5AE5,0x5AEA,0x5AEE,0x5AF5,0x5AF6,0x5AFD,0x5B00,0x5B01,
	0x5B08,0x5B17,0x5B34,0x5B19,0x5B1B,0x5B1D,0x5B21,0x5B25,
	0x5B2D,0x5B38,0x5B41,0x5B4B,0x5B4C,0x5B52,0x5B56,0x5B5E,
	0x5B68,0x5B6E,0x5B6F,0x5B7C,0x5B7D,0x5B7E,0x5B7F,0x5B81,
	0x5B84,0x5B86,0x5B8A,0x5B8E,0x5B90,0x5B91,0x5B93,0x5B94,
	0x5B96,0x5BA8,0x5BA9,0x5BAC,0x5BAD,0x5BAF,0x5BB1,0x5BB2,
	0x5BB7,0x5BBA,0x5BBC,0x5BC0,0x5BC1,0x5BCD,0x5BCF,0x5BD6,
	0x5BD7,0x5BD8,0x5BD9,0x5BDA,0x5BE0,0x5BEF,0x5BF1,0x5BF4,
	0x5BFD,0x5C0C,0x5C17,0x5C1E,0x5C1F,0x5C23,0x5C26,0x5C29,
	0x5C2B,0x5C2C,0x5C2E,0x5C30,0x5C32,0x5C35,0x5C36,0x5C59,
	0x5C5A,0x5C5C,0x5C62,0x5C63,0x5C67,0x5C68,0x5C69,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 60, Array index 0x1200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C6D,0x5C70,0x5C74,0x5C75,0x5C7A,0x5C7B,0x5C7C,
	0x5C7D,0x5C87,0x5C88,0x5C8A,0x5C8F,0x5C92,0x5C9D,0x5C9F,
	0x5CA0,0x5CA2,0x5CA3,0x5CA6,0x5CAA,0x5CB2,0x5CB4,0x5CB5,
	0x5CBA,0x5CC9,0x5CCB,0x5CD2,0x5CDD,0x5CD7,0x5CEE,0x5CF1,
	0x5CF2,0x5CF4,0x5D01,0x5D06,0x5D0D,0x5D12,0x5D2B,0x5D23,
	0x5D24,0x5D26,0x5D27,0x5D31,0x5D34,0x5D39,0x5D3D,0x5D3F,
	0x5D42,0x5D43,0x5D46,0x5D48,0x5D55,0x5D51,0x5D59,0x5D4A,
	0x5D5F,0x5D60,0x5D61,0x5D62,0x5D64,0x5D6A,0x5D6D,0x5D70,
	0x5D79,0x5D7A,0x5D7E,0x5D7F,0x5D81,0x5D83,0x5D88,0x5D8A,
	0x5D92,0x5D93,0x5D94,0x5D95,0x5D99,0x5D9B,0x5D9F,0x5DA0,
	0x5DA7,0x5DAB,0x5DB0,0x5DB4,0x5DB8,0x5DB9,0x5DC3,0x5DC7,
	0x5DCB,0x5DD0,0x5DCE,0x5DD8,0x5DD9,0x5DE0,0x5DE4,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 61, Array index 0x1300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5DE9,0x5DF8,0x5DF9,0x5E00,0x5E07,0x5E0D,0x5E12,
	0x5E14,0x5E15,0x5E18,0x5E1F,0x5E20,0x5E2E,0x5E28,0x5E32,
	0x5E35,0x5E3E,0x5E4B,0x5E50,0x5E49,0x5E51,0x5E56,0x5E58,
	0x5E5B,0x5E5C,0x5E5E,0x5E68,0x5E6A,0x5E6B,0x5E6C,0x5E6D,
	0x5E6E,0x5E70,0x5E80,0x5E8B,0x5E8E,0x5EA2,0x5EA4,0x5EA5,
	0x5EA8,0x5EAA,0x5EAC,0x5EB1,0x5EB3,0x5EBD,0x5EBE,0x5EBF,
	0x5EC6,0x5ECC,0x5ECB,0x5ECE,0x5ED1,0x5ED2,0x5ED4,0x5ED5,
	0x5EDC,0x5EDE,0x5EE5,0x5EEB,0x5F02,0x5F06,0x5F07,0x5F08,
	0x5F0E,0x5F19,0x5F1C,0x5F1D,0x5F21,0x5F22,0x5F23,0x5F24,
	0x5F28,0x5F2B,0x5F2C,0x5F2E,0x5F30,0x5F34,0x5F36,0x5F3B,
	0x5F3D,0x5F3F,0x5F40,0x5F44,0x5F45,0x5F47,0x5F4D,0x5F50,
	0x5F54,0x5F58,0x5F5B,0x5F60,0x5F63,0x5F64,0x5F67,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 62, Array index 0x1400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F6F,0x5F72,0x5F74,0x5F75,0x5F78,0x5F7A,0x5F7D,
	0x5F7E,0x5F89,0x5F8D,0x5F8F,0x5F96,0x5F9C,0x5F9D,0x5FA2,
	0x5FA7,0x5FAB,0x5FA4,0x5FAC,0x5FAF,0x5FB0,0x5FB1,0x5FB8,
	0x5FC4,0x5FC7,0x5FC8,0x5FC9,0x5FCB,0x5FD0,0x5FD1,0x5FD2,
	0x5FD3,0x5FD4,0x5FDE,0x5FE1,0x5FE2,0x5FE8,0x5FE9,0x5FEA,
	0x5FEC,0x5FED,0x5FEE,0x5FEF,0x5FF2,0x5FF3,0x5FF6,0x5FFA,
	0x5FFC,0x6007,0x600A,0x600D,0x6013,0x6014,0x6017,0x6018,
	0x601A,0x601F,0x6024,0x602D,0x6033,0x6035,0x6040,0x6047,
	0x6048,0x6049,0x604C,0x6051,0x6054,0x6056,0x6057,0x605D,
	0x6061,0x6067,0x6071,0x607E,0x607F,0x6082,0x6086,0x6088,
	0x608A,0x608E,0x6091,0x6093,0x6095,0x6098,0x609D,0x609E,
	0x60A2,0x60A4,0x60A5,0x60A8,0x60B0,0x60B1,0x60B7,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 63, Array index 0x1500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x60BB,0x60BE,0x60C2,0x60C4,0x60C8,0x60C9,0x60CA,
	0x60CB,0x60CE,0x60CF,0x60D4,0x60D5,0x60D9,0x60DB,0x60DD,
	0x60DE,0x60E2,0x60E5,0x60F2,0x60F5,0x60F8,0x60FC,0x60FD,
	0x6102,0x6107,0x610A,0x610C,0x6110,0x6111,0x6112,0x6113,
	0x6114,0x6116,0x6117,0x6119,0x611C,0x611E,0x6122,0x612A,
	0x612B,0x6130,0x6131,0x6135,0x6136,0x6137,0x6139,0x6141,
	0x6145,0x6146,0x6149,0x615E,0x6160,0x616C,0x6172,0x6178,
	0x617B,0x617C,0x617F,0x6180,0x6181,0x6183,0x6184,0x618B,
	0x618D,0x6192,0x6193,0x6197,0x6198,0x619C,0x619D,0x619F,
	0x61A0,0x61A5,0x61A8,0x61AA,0x61AD,0x61B8,0x61B9,0x61BC,
	0x61C0,0x61C1,0x61C2,0x61CE,0x61CF,0x61D5,0x61DC,0x61DD,
	0x61DE,0x61DF,0x61E1,0x61E2,0x61E7,0x61E9,0x61E5,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 64, Array index 0x1600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x61EC,0x61ED,0x61EF,0x6201,0x6203,0x6204,0x6207,
	0x6213,0x6215,0x621C,0x6220,0x6222,0x6223,0x6227,0x6229,
	0x622B,0x6239,0x623D,0x6242,0x6243,0x6244,0x6246,0x624C,
	0x6250,0x6251,0x6252,0x6254,0x6256,0x625A,0x625C,0x6264,
	0x626D,0x626F,0x6273,0x627A,0x627D,0x628D,0x628E,0x628F,
	0x6290,0x62A6,0x62A8,0x62B3,0x62B6,0x62B7,0x62BA,0x62BE,
	0x62BF,0x62C4,0x62CE,0x62D5,0x62D6,0x62DA,0x62EA,0x62F2,
	0x62F4,0x62FC,0x62FD,0x6303,0x6304,0x630A,0x630B,0x630D,
	0x6310,0x6313,0x6316,0x6318,0x6329,0x632A,0x632D,0x6335,
	0x6336,0x6339,0x633C,0x6341,0x6342,0x6343,0x6344,0x6346,
	0x634A,0x634B,0x634E,0x6352,0x6353,0x6354,0x6358,0x635B,
	0x6365,0x6366,0x636C,0x636D,0x6371,0x6374,0x6375,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 65, Array index 0x1700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6378,0x637C,0x637D,0x637F,0x6382,0x6384,0x6387,
	0x638A,0x6390,0x6394,0x6395,0x6399,0x639A,0x639E,0x63A4,
	0x63A6,0x63AD,0x63AE,0x63AF,0x63BD,0x63C1,0x63C5,0x63C8,
	0x63CE,0x63D1,0x63D3,0x63D4,0x63D5,0x63DC,0x63E0,0x63E5,
	0x63EA,0x63EC,0x63F2,0x63F3,0x63F5,0x63F8,0x63F9,0x6409,
	0x640A,0x6410,0x6412,0x6414,0x6418,0x641E,0x6420,0x6422,
	0x6424,0x6425,0x6429,0x642A,0x642F,0x6430,0x6435,0x643D,
	0x643F,0x644B,0x644F,0x6451,0x6452,0x6453,0x6454,0x645A,
	0x645B,0x645C,0x645D,0x645F,0x6460,0x6461,0x6463,0x646D,
	0x6473,0x6474,0x647B,0x647D,0x6485,0x6487,0x648F,0x6490,
	0x6491,0x6498,0x6499,0x649B,0x649D,0x649F,0x64A1,0x64A3,
	0x64A6,0x64A8,0x64AC,0x64B3,0x64BD,0x64BE,0x64BF,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 66, Array index 0x1800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x64C4,0x64C9,0x64CA,0x64CB,0x64CC,0x64CE,0x64D0,
	0x64D1,0x64D5,0x64D7,0x64E4,0x64E5,0x64E9,0x64EA,0x64ED,
	0x64F0,0x64F5,0x64F7,0x64FB,0x64FF,0x6501,0x6504,0x6508,
	0x6509,0x650A,0x650F,0x6513,0x6514,0x6516,0x6519,0x651B,
	0x651E,0x651F,0x6522,0x6526,0x6529,0x652E,0x6531,0x653A,
	0x653C,0x653D,0x6543,0x6547,0x6549,0x6550,0x6552,0x6554,
	0x655F,0x6560,0x6567,0x656B,0x657A,0x657D,0x6581,0x6585,
	0x658A,0x6592,0x6595,0x6598,0x659D,0x65A0,0x65A3,0x65A6,
	0x65AE,0x65B2,0x65B3,0x65B4,0x65BF,0x65C2,0x65C8,0x65C9,
	0x65CE,0x65D0,0x65D4,0x65D6,0x65D8,0x65DF,0x65F0,0x65F2,
	0x65F4,0x65F5,0x65F9,0x65FE,0x65FF,0x6600,0x6604,0x6608,
	0x6609,0x660D,0x6611,0x6612,0x6615,0x6616,0x661D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 67, Array index 0x1900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x661E,0x6621,0x6622,0x6623,0x6624,0x6626,0x6629,
	0x662A,0x662B,0x662C,0x662E,0x6630,0x6631,0x6633,0x6639,
	0x6637,0x6640,0x6645,0x6646,0x664A,0x664C,0x6651,0x664E,
	0x6657,0x6658,0x6659,0x665B,0x665C,0x6660,0x6661,0x66FB,
	0x666A,0x666B,0x666C,0x667E,0x6673,0x6675,0x667F,0x6677,
	0x6678,0x6679,0x667B,0x6680,0x667C,0x668B,0x668C,0x668D,
	0x6690,0x6692,0x6699,0x669A,0x669B,0x669C,0x669F,0x66A0,
	0x66A4,0x66AD,0x66B1,0x66B2,0x66B5,0x66BB,0x66BF,0x66C0,
	0x66C2,0x66C3,0x66C8,0x66CC,0x66CE,0x66CF,0x66D4,0x66DB,
	0x66DF,0x66E8,0x66EB,0x66EC,0x66EE,0x66FA,0x6705,0x6707,
	0x670E,0x6713,0x6719,0x671C,0x6720,0x6722,0x6733,0x673E,
	0x6745,0x6747,0x6748,0x674C,0x6754,0x6755,0x675D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 68, Array index 0x1A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6766,0x676C,0x676E,0x6774,0x6776,0x677B,0x6781,
	0x6784,0x678E,0x678F,0x6791,0x6793,0x6796,0x6798,0x6799,
	0x679B,0x67B0,0x67B1,0x67B2,0x67B5,0x67BB,0x67BC,0x67BD,
	0x67F9,0x67C0,0x67C2,0x67C3,0x67C5,0x67C8,0x67C9,0x67D2,
	0x67D7,0x67D9,0x67DC,0x67E1,0x67E6,0x67F0,0x67F2,0x67F6,
	0x67F7,0x6852,0x6814,0x6819,0x681D,0x681F,0x6828,0x6827,
	0x682C,0x682D,0x682F,0x6830,0x6831,0x6833,0x683B,0x683F,
	0x6844,0x6845,0x684A,0x684C,0x6855,0x6857,0x6858,0x685B,
	0x686B,0x686E,0x686F,0x6870,0x6871,0x6872,0x6875,0x6879,
	0x687A,0x687B,0x687C,0x6882,0x6884,0x6886,0x6888,0x6896,
	0x6898,0x689A,0x689C,0x68A1,0x68A3,0x68A5,0x68A9,0x68AA,
	0x68AE,0x68B2,0x68BB,0x68C5,0x68C8,0x68CC,0x68CF,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 69, Array index 0x1B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x68D0,0x68D1,0x68D3,0x68D6,0x68D9,0x68DC,0x68DD,
	0x68E5,0x68E8,0x68EA,0x68EB,0x68EC,0x68ED,0x68F0,0x68F1,
	0x68F5,0x68F6,0x68FB,0x68FC,0x68FD,0x6906,0x6909,0x690A,
	0x6910,0x6911,0x6913,0x6916,0x6917,0x6931,0x6933,0x6935,
	0x6938,0x693B,0x6942,0x6945,0x6949,0x694E,0x6957,0x695B,
	0x6963,0x6964,0x6965,0x6966,0x6968,0x6969,0x696C,0x6970,
	0x6971,0x6972,0x697A,0x697B,0x697F,0x6980,0x698D,0x6992,
	0x6996,0x6998,0x69A1,0x69A5,0x69A6,0x69A8,0x69AB,0x69AD,
	0x69AF,0x69B7,0x69B8,0x69BA,0x69BC,0x69C5,0x69C8,0x69D1,
	0x69D6,0x69D7,0x69E2,0x69E5,0x69EE,0x69EF,0x69F1,0x69F3,
	0x69F5,0x69FE,0x6A00,0x6A01,0x6A03,0x6A0F,0x6A11,0x6A15,
	0x6A1A,0x6A1D,0x6A20,0x6A24,0x6A28,0x6A30,0x6A32,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 70, Array index 0x1C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6A34,0x6A37,0x6A3B,0x6A3E,0x6A3F,0x6A45,0x6A46,
	0x6A49,0x6A4A,0x6A4E,0x6A50,0x6A51,0x6A52,0x6A55,0x6A56,
	0x6A5B,0x6A64,0x6A67,0x6A6A,0x6A71,0x6A73,0x6A7E,0x6A81,
	0x6A83,0x6A86,0x6A87,0x6A89,0x6A8B,0x6A91,0x6A9B,0x6A9D,
	0x6A9E,0x6A9F,0x6AA5,0x6AAB,0x6AAF,0x6AB0,0x6AB1,0x6AB4,
	0x6ABD,0x6ABE,0x6ABF,0x6AC6,0x6AC9,0x6AC8,0x6ACC,0x6AD0,
	0x6AD4,0x6AD5,0x6AD6,0x6ADC,0x6ADD,0x6AE4,0x6AE7,0x6AEC,
	0x6AF0,0x6AF1,0x6AF2,0x6AFC,0x6AFD,0x6B02,0x6B03,0x6B06,
	0x6B07,0x6B09,0x6B0F,0x6B10,0x6B11,0x6B17,0x6B1B,0x6B1E,
	0x6B24,0x6B28,0x6B2B,0x6B2C,0x6B2F,0x6B35,0x6B36,0x6B3B,
	0x6B3F,0x6B46,0x6B4A,0x6B4D,0x6B52,0x6B56,0x6B58,0x6B5D,
	0x6B60,0x6B67,0x6B6B,0x6B6E,0x6B70,0x6B75,0x6B7D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 71, Array index 0x1D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B7E,0x6B82,0x6B85,0x6B97,0x6B9B,0x6B9F,0x6BA0,
	0x6BA2,0x6BA3,0x6BA8,0x6BA9,0x6BAC,0x6BAD,0x6BAE,0x6BB0,
	0x6BB8,0x6BB9,0x6BBD,0x6BBE,0x6BC3,0x6BC4,0x6BC9,0x6BCC,
	0x6BD6,0x6BDA,0x6BE1,0x6BE3,0x6BE6,0x6BE7,0x6BEE,0x6BF1,
	0x6BF7,0x6BF9,0x6BFF,0x6C02,0x6C04,0x6C05,0x6C09,0x6C0D,
	0x6C0E,0x6C10,0x6C12,0x6C19,0x6C1F,0x6C26,0x6C27,0x6C28,
	0x6C2C,0x6C2E,0x6C33,0x6C35,0x6C36,0x6C3A,0x6C3B,0x6C3F,
	0x6C4A,0x6C4B,0x6C4D,0x6C4F,0x6C52,0x6C54,0x6C59,0x6C5B,
	0x6C5C,0x6C6B,0x6C6D,0x6C6F,0x6C74,0x6C76,0x6C78,0x6C79,
	0x6C7B,0x6C85,0x6C86,0x6C87,0x6C89,0x6C94,0x6C95,0x6C97,
	0x6C98,0x6C9C,0x6C9F,0x6CB0,0x6CB2,0x6CB4,0x6CC2,0x6CC6,
	0x6CCD,0x6CCF,0x6CD0,0x6CD1,0x6CD2,0x6CD4,0x6CD6,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 72, Array index 0x1E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6CDA,0x6CDC,0x6CE0,0x6CE7,0x6CE9,0x6CEB,0x6CEC,
	0x6CEE,0x6CF2,0x6CF4,0x6D04,0x6D07,0x6D0A,0x6D0E,0x6D0F,
	0x6D11,0x6D13,0x6D1A,0x6D26,0x6D27,0x6D28,0x6C67,0x6D2E,
	0x6D2F,0x6D31,0x6D39,0x6D3C,0x6D3F,0x6D57,0x6D5E,0x6D5F,
	0x6D61,0x6D65,0x6D67,0x6D6F,0x6D70,0x6D7C,0x6D82,0x6D87,
	0x6D91,0x6D92,0x6D94,0x6D96,0x6D97,0x6D98,0x6DAA,0x6DAC,
	0x6DB4,0x6DB7,0x6DB9,0x6DBD,0x6DBF,0x6DC4,0x6DC8,0x6DCA,
	0x6DCE,0x6DCF,0x6DD6,0x6DDB,0x6DDD,0x6DDF,0x6DE0,0x6DE2,
	0x6DE5,0x6DE9,0x6DEF,0x6DF0,0x6DF4,0x6DF6,0x6DFC,0x6E00,
	0x6E04,0x6E1E,0x6E22,0x6E27,0x6E32,0x6E36,0x6E39,0x6E3B,
	0x6E3C,0x6E44,0x6E45,0x6E48,0x6E49,0x6E4B,0x6E4F,0x6E51,
	0x6E52,0x6E53,0x6E54,0x6E57,0x6E5C,0x6E5D,0x6E5E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 73, Array index 0x1F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6E62,0x6E63,0x6E68,0x6E73,0x6E7B,0x6E7D,0x6E8D,
	0x6E93,0x6E99,0x6EA0,0x6EA7,0x6EAD,0x6EAE,0x6EB1,0x6EB3,
	0x6EBB,0x6EBF,0x6EC0,0x6EC1,0x6EC3,0x6EC7,0x6EC8,0x6ECA,
	0x6ECD,0x6ECE,0x6ECF,0x6EEB,0x6EED,0x6EEE,0x6EF9,0x6EFB,
	0x6EFD,0x6F04,0x6F08,0x6F0A,0x6F0C,0x6F0D,0x6F16,0x6F18,
	0x6F1A,0x6F1B,0x6F26,0x6F29,0x6F2A,0x6F2F,0x6F30,0x6F33,
	0x6F36,0x6F3B,0x6F3C,0x6F2D,0x6F4F,0x6F51,0x6F52,0x6F53,
	0x6F57,0x6F59,0x6F5A,0x6F5D,0x6F5E,0x6F61,0x6F62,0x6F68,
	0x6F6C,0x6F7D,0x6F7E,0x6F83,0x6F87,0x6F88,0x6F8B,0x6F8C,
	0x6F8D,0x6F90,0x6F92,0x6F93,0x6F94,0x6F96,0x6F9A,0x6F9F,
	0x6FA0,0x6FA5,0x6FA6,0x6FA7,0x6FA8,0x6FAE,0x6FAF,0x6FB0,
	0x6FB5,0x6FB6,0x6FBC,0x6FC5,0x6FC7,0x6FC8,0x6FCA,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 74, Array index 0x2000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6FDA,0x6FDE,0x6FE8,0x6FE9,0x6FF0,0x6FF5,0x6FF9,
	0x6FFC,0x6FFD,0x7000,0x7005,0x7006,0x7007,0x700D,0x7017,
	0x7020,0x7023,0x702F,0x7034,0x7037,0x7039,0x703C,0x7043,
	0x7044,0x7048,0x7049,0x704A,0x704B,0x7054,0x7055,0x705D,
	0x705E,0x704E,0x7064,0x7065,0x706C,0x706E,0x7075,0x7076,
	0x707E,0x7081,0x7085,0x7086,0x7094,0x7095,0x7096,0x7097,
	0x7098,0x709B,0x70A4,0x70AB,0x70B0,0x70B1,0x70B4,0x70B7,
	0x70CA,0x70D1,0x70D3,0x70D4,0x70D5,0x70D6,0x70D8,0x70DC,
	0x70E4,0x70FA,0x7103,0x7104,0x7105,0x7106,0x7107,0x710B,
	0x710C,0x710F,0x711E,0x7120,0x712B,0x712D,0x712F,0x7130,
	0x7131,0x7138,0x7141,0x7145,0x7146,0x7147,0x714A,0x714B,
	0x7150,0x7152,0x7157,0x715A,0x715C,0x715E,0x7160,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 75, Array index 0x2100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7168,0x7179,0x7180,0x7185,0x7187,0x718C,0x7192,
	0x719A,0x719B,0x71A0,0x71A2,0x71AF,0x71B0,0x71B2,0x71B3,
	0x71BA,0x71BF,0x71C0,0x71C1,0x71C4,0x71CB,0x71CC,0x71D3,
	0x71D6,0x71D9,0x71DA,0x71DC,0x71F8,0x71FE,0x7200,0x7207,
	0x7208,0x7209,0x7213,0x7217,0x721A,0x721D,0x721F,0x7224,
	0x722B,0x722F,0x7234,0x7238,0x7239,0x7241,0x7242,0x7243,
	0x7245,0x724E,0x724F,0x7250,0x7253,0x7255,0x7256,0x725A,
	0x725C,0x725E,0x7260,0x7263,0x7268,0x726B,0x726E,0x726F,
	0x7271,0x7277,0x7278,0x727B,0x727C,0x727F,0x7284,0x7289,
	0x728D,0x728E,0x7293,0x729B,0x72A8,0x72AD,0x72AE,0x72B1,
	0x72B4,0x72BE,0x72C1,0x72C7,0x72C9,0x72CC,0x72D5,0x72D6,
	0x72D8,0x72DF,0x72E5,0x72F3,0x72F4,0x72FA,0x72FB,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 76, Array index 0x2200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x72FE,0x7302,0x7304,0x7305,0x7307,0x730B,0x730D,
	0x7312,0x7313,0x7318,0x7319,0x731E,0x7322,0x7324,0x7327,
	0x7328,0x732C,0x7331,0x7332,0x7335,0x733A,0x733B,0x733D,
	0x7343,0x734D,0x7350,0x7352,0x7356,0x7358,0x735D,0x735E,
	0x735F,0x7360,0x7366,0x7367,0x7369,0x736B,0x736C,0x736E,
	0x736F,0x7371,0x7377,0x7379,0x737C,0x7380,0x7381,0x7383,
	0x7385,0x7386,0x738E,0x7390,0x7393,0x7395,0x7397,0x7398,
	0x739C,0x739E,0x739F,0x73A0,0x73A2,0x73A5,0x73A6,0x73AA,
	0x73AB,0x73AD,0x73B5,0x73B7,0x73B9,0x73BC,0x73BD,0x73BF,
	0x73C5,0x73C6,0x73C9,0x73CB,0x73CC,0x73CF,0x73D2,0x73D3,
	0x73D6,0x73D9,0x73DD,0x73E1,0x73E3,0x73E6,0x73E7,0x73E9,
	0x73F4,0x73F5,0x73F7,0x73F9,0x73FA,0x73FB,0x73FD,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 77, Array index 0x2300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x73FF,0x7400,0x7401,0x7404,0x7407,0x740A,0x7411,
	0x741A,0x741B,0x7424,0x7426,0x7428,0x7429,0x742A,0x742B,
	0x742C,0x742D,0x742E,0x742F,0x7430,0x7431,0x7439,0x7440,
	0x7443,0x7444,0x7446,0x7447,0x744B,0x744D,0x7451,0x7452,
	0x7457,0x745D,0x7462,0x7466,0x7467,0x7468,0x746B,0x746D,
	0x746E,0x7471,0x7472,0x7480,0x7481,0x7485,0x7486,0x7487,
	0x7489,0x748F,0x7490,0x7491,0x7492,0x7498,0x7499,0x749A,
	0x749C,0x749F,0x74A0,0x74A1,0x74A3,0x74A6,0x74A8,0x74A9,
	0x74AA,0x74AB,0x74AE,0x74AF,0x74B1,0x74B2,0x74B5,0x74B9,
	0x74BB,0x74BF,0x74C8,0x74C9,0x74CC,0x74D0,0x74D3,0x74D8,
	0x74DA,0x74DB,0x74DE,0x74DF,0x74E4,0x74E8,0x74EA,0x74EB,
	0x74EF,0x74F4,0x74FA,0x74FB,0x74FC,0x74FF,0x7506,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 78, Array index 0x2400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7512,0x7516,0x7517,0x7520,0x7521,0x7524,0x7527,
	0x7529,0x752A,0x752F,0x7536,0x7539,0x753D,0x753E,0x753F,
	0x7540,0x7543,0x7547,0x7548,0x754E,0x7550,0x7552,0x7557,
	0x755E,0x755F,0x7561,0x756F,0x7571,0x7579,0x757A,0x757B,
	0x757C,0x757D,0x757E,0x7581,0x7585,0x7590,0x7592,0x7593,
	0x7595,0x7599,0x759C,0x75A2,0x75A4,0x75B4,0x75BA,0x75BF,
	0x75C0,0x75C1,0x75C4,0x75C6,0x75CC,0x75CE,0x75CF,0x75D7,
	0x75DC,0x75DF,0x75E0,0x75E1,0x75E4,0x75E7,0x75EC,0x75EE,
	0x75EF,0x75F1,0x75F9,0x7600,0x7602,0x7603,0x7604,0x7607,
	0x7608,0x760A,0x760C,0x760F,0x7612,0x7613,0x7615,0x7616,
	0x7619,0x761B,0x761C,0x761D,0x761E,0x7623,0x7625,0x7626,
	0x7629,0x762D,0x7632,0x7633,0x7635,0x7638,0x7639,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 79, Array index 0x2500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x763A,0x763C,0x764A,0x7640,0x7641,0x7643,0x7644,
	0x7645,0x7649,0x764B,0x7655,0x7659,0x765F,0x7664,0x7665,
	0x766D,0x766E,0x766F,0x7671,0x7674,0x7681,0x7685,0x768C,
	0x768D,0x7695,0x769B,0x769C,0x769D,0x769F,0x76A0,0x76A2,
	0x76A3,0x76A4,0x76A5,0x76A6,0x76A7,0x76A8,0x76AA,0x76AD,
	0x76BD,0x76C1,0x76C5,0x76C9,0x76CB,0x76CC,0x76CE,0x76D4,
	0x76D9,0x76E0,0x76E6,0x76E8,0x76EC,0x76F0,0x76F1,0x76F6,
	0x76F9,0x76FC,0x7700,0x7706,0x770A,0x770E,0x7712,0x7714,
	0x7715,0x7717,0x7719,0x771A,0x771C,0x7722,0x7728,0x772D,
	0x772E,0x772F,0x7734,0x7735,0x7736,0x7739,0x773D,0x773E,
	0x7742,0x7745,0x7746,0x774A,0x774D,0x774E,0x774F,0x7752,
	0x7756,0x7757,0x775C,0x775E,0x775F,0x7760,0x7762,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 80, Array index 0x2600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7764,0x7767,0x776A,0x776C,0x7770,0x7772,0x7773,
	0x7774,0x777A,0x777D,0x7780,0x7784,0x778C,0x778D,0x7794,
	0x7795,0x7796,0x779A,0x779F,0x77A2,0x77A7,0x77AA,0x77AE,
	0x77AF,0x77B1,0x77B5,0x77BE,0x77C3,0x77C9,0x77D1,0x77D2,
	0x77D5,0x77D9,0x77DE,0x77DF,0x77E0,0x77E4,0x77E6,0x77EA,
	0x77EC,0x77F0,0x77F1,0x77F4,0x77F8,0x77FB,0x7805,0x7806,
	0x7809,0x780D,0x780E,0x7811,0x781D,0x7821,0x7822,0x7823,
	0x782D,0x782E,0x7830,0x7835,0x7837,0x7843,0x7844,0x7847,
	0x7848,0x784C,0x784E,0x7852,0x785C,0x785E,0x7860,0x7861,
	0x7863,0x7864,0x7868,0x786A,0x786E,0x787A,0x787E,0x788A,
	0x788F,0x7894,0x7898,0x78A1,0x789D,0x789E,0x789F,0x78A4,
	0x78A8,0x78AC,0x78AD,0x78B0,0x78B1,0x78B2,0x78B3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 81, Array index 0x2700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x78BB,0x78BD,0x78BF,0x78C7,0x78C8,0x78C9,0x78CC,
	0x78CE,0x78D2,0x78D3,0x78D5,0x78D6,0x78E4,0x78DB,0x78DF,
	0x78E0,0x78E1,0x78E6,0x78EA,0x78F2,0x78F3,0x7900,0x78F6,
	0x78F7,0x78FA,0x78FB,0x78FF,0x7906,0x790C,0x7910,0x791A,
	0x791C,0x791E,0x791F,0x7920,0x7925,0x7927,0x7929,0x792D,
	0x7931,0x7934,0x7935,0x793B,0x793D,0x793F,0x7944,0x7945,
	0x7946,0x794A,0x794B,0x794F,0x7951,0x7954,0x7958,0x795B,
	0x795C,0x7967,0x7969,0x796B,0x7972,0x7979,0x797B,0x797C,
	0x797E,0x798B,0x798C,0x7991,0x7993,0x7994,0x7995,0x7996,
	0x7998,0x799B,0x799C,0x79A1,0x79A8,0x79A9,0x79AB,0x79AF,
	0x79B1,0x79B4,0x79B8,0x79BB,0x79C2,0x79C4,0x79C7,0x79C8,
	0x79CA,0x79CF,0x79D4,0x79D6,0x79DA,0x79DD,0x79DE,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 82, Array index 0x2800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x79E0,0x79E2,0x79E5,0x79EA,0x79EB,0x79ED,0x79F1,
	0x79F8,0x79FC,0x7A02,0x7A03,0x7A07,0x7A09,0x7A0A,0x7A0C,
	0x7A11,0x7A15,0x7A1B,0x7A1E,0x7A21,0x7A27,0x7A2B,0x7A2D,
	0x7A2F,0x7A30,0x7A34,0x7A35,0x7A38,0x7A39,0x7A3A,0x7A44,
	0x7A45,0x7A47,0x7A48,0x7A4C,0x7A55,0x7A56,0x7A59,0x7A5C,
	0x7A5D,0x7A5F,0x7A60,0x7A65,0x7A67,0x7A6A,0x7A6D,0x7A75,
	0x7A78,0x7A7E,0x7A80,0x7A82,0x7A85,0x7A86,0x7A8A,0x7A8B,
	0x7A90,0x7A91,0x7A94,0x7A9E,0x7AA0,0x7AA3,0x7AAC,0x7AB3,
	0x7AB5,0x7AB9,0x7ABB,0x7ABC,0x7AC6,0x7AC9,0x7ACC,0x7ACE,
	0x7AD1,0x7ADB,0x7AE8,0x7AE9,0x7AEB,0x7AEC,0x7AF1,0x7AF4,
	0x7AFB,0x7AFD,0x7AFE,0x7B07,0x7B14,0x7B1F,0x7B23,0x7B27,
	0x7B29,0x7B2A,0x7B2B,0x7B2D,0x7B2E,0x7B2F,0x7B30,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 83, Array index 0x2900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7B31,0x7B34,0x7B3D,0x7B3F,0x7B40,0x7B41,0x7B47,
	0x7B4E,0x7B55,0x7B60,0x7B64,0x7B66,0x7B69,0x7B6A,0x7B6D,
	0x7B6F,0x7B72,0x7B73,0x7B77,0x7B84,0x7B89,0x7B8E,0x7B90,
	0x7B91,0x7B96,0x7B9B,0x7B9E,0x7BA0,0x7BA5,0x7BAC,0x7BAF,
	0x7BB0,0x7BB2,0x7BB5,0x7BB6,0x7BBA,0x7BBB,0x7BBC,0x7BBD,
	0x7BC2,0x7BC5,0x7BC8,0x7BCA,0x7BD4,0x7BD6,0x7BD7,0x7BD9,
	0x7BDA,0x7BDB,0x7BE8,0x7BEA,0x7BF2,0x7BF4,0x7BF5,0x7BF8,
	0x7BF9,0x7BFA,0x7BFC,0x7BFE,0x7C01,0x7C02,0x7C03,0x7C04,
	0x7C06,0x7C09,0x7C0B,0x7C0C,0x7C0E,0x7C0F,0x7C19,0x7C1B,
	0x7C20,0x7C25,0x7C26,0x7C28,0x7C2C,0x7C31,0x7C33,0x7C34,
	0x7C36,0x7C39,0x7C3A,0x7C46,0x7C4A,0x7C55,0x7C51,0x7C52,
	0x7C53,0x7C59,0x7C5A,0x7C5B,0x7C5C,0x7C5D,0x7C5E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 84, Array index 0x2A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7C61,0x7C63,0x7C67,0x7C69,0x7C6D,0x7C6E,0x7C70,
	0x7C72,0x7C79,0x7C7C,0x7C7D,0x7C86,0x7C87,0x7C8F,0x7C94,
	0x7C9E,0x7CA0,0x7CA6,0x7CB0,0x7CB6,0x7CB7,0x7CBA,0x7CBB,
	0x7CBC,0x7CBF,0x7CC4,0x7CC7,0x7CC8,0x7CC9,0x7CCD,0x7CCF,
	0x7CD3,0x7CD4,0x7CD5,0x7CD7,0x7CD9,0x7CDA,0x7CDD,0x7CE6,
	0x7CE9,0x7CEB,0x7CF5,0x7D03,0x7D07,0x7D08,0x7D09,0x7D0F,
	0x7D11,0x7D12,0x7D13,0x7D16,0x7D1D,0x7D1E,0x7D23,0x7D26,
	0x7D2A,0x7D2D,0x7D31,0x7D3C,0x7D3D,0x7D3E,0x7D40,0x7D41,
	0x7D47,0x7D48,0x7D4D,0x7D51,0x7D53,0x7D57,0x7D59,0x7D5A,
	0x7D5C,0x7D5D,0x7D65,0x7D67,0x7D6A,0x7D70,0x7D78,0x7D7A,
	0x7D7B,0x7D7F,0x7D81,0x7D82,0x7D83,0x7D85,0x7D86,0x7D88,
	0x7D8B,0x7D8C,0x7D8D,0x7D91,0x7D96,0x7D97,0x7D9D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 85, Array index 0x2B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7D9E,0x7DA6,0x7DA7,0x7DAA,0x7DB3,0x7DB6,0x7DB7,
	0x7DB9,0x7DC2,0x7DC3,0x7DC4,0x7DC5,0x7DC6,0x7DCC,0x7DCD,
	0x7DCE,0x7DD7,0x7DD9,0x7E00,0x7DE2,0x7DE5,0x7DE6,0x7DEA,
	0x7DEB,0x7DED,0x7DF1,0x7DF5,0x7DF6,0x7DF9,0x7DFA,0x7E08,
	0x7E10,0x7E11,0x7E15,0x7E17,0x7E1C,0x7E1D,0x7E20,0x7E27,
	0x7E28,0x7E2C,0x7E2D,0x7E2F,0x7E33,0x7E36,0x7E3F,0x7E44,
	0x7E45,0x7E47,0x7E4E,0x7E50,0x7E52,0x7E58,0x7E5F,0x7E61,
	0x7E62,0x7E65,0x7E6B,0x7E6E,0x7E6F,0x7E73,0x7E78,0x7E7E,
	0x7E81,0x7E86,0x7E87,0x7E8A,0x7E8D,0x7E91,0x7E95,0x7E98,
	0x7E9A,0x7E9D,0x7E9E,0x7F3C,0x7F3B,0x7F3D,0x7F3E,0x7F3F,
	0x7F43,0x7F44,0x7F47,0x7F4F,0x7F52,0x7F53,0x7F5B,0x7F5C,
	0x7F5D,0x7F61,0x7F63,0x7F64,0x7F65,0x7F66,0x7F6D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 86, Array index 0x2C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x7F71,0x7F7D,0x7F7E,0x7F7F,0x7F80,0x7F8B,0x7F8D,
	0x7F8F,0x7F90,0x7F91,0x7F96,0x7F97,0x7F9C,0x7FA1,0x7FA2,
	0x7FA6,0x7FAA,0x7FAD,0x7FB4,0x7FBC,0x7FBF,0x7FC0,0x7FC3,
	0x7FC8,0x7FCE,0x7FCF,0x7FDB,0x7FDF,0x7FE3,0x7FE5,0x7FE8,
	0x7FEC,0x7FEE,0x7FEF,0x7FF2,0x7FFA,0x7FFD,0x7FFE,0x7FFF,
	0x8007,0x8008,0x800A,0x800D,0x800E,0x800F,0x8011,0x8013,
	0x8014,0x8016,0x801D,0x801E,0x801F,0x8020,0x8024,0x8026,
	0x802C,0x802E,0x8030,0x8034,0x8035,0x8037,0x8039,0x803A,
	0x803C,0x803E,0x8040,0x8044,0x8060,0x8064,0x8066,0x806D,
	0x8071,0x8075,0x8081,0x8088,0x808E,0x809C,0x809E,0x80A6,
	0x80A7,0x80AB,0x80B8,0x80B9,0x80C8,0x80CD,0x80CF,0x80D2,
	0x80D4,0x80D5,0x80D7,0x80D8,0x80E0,0x80ED,0x80EE,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 87, Array index 0x2D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x80F0,0x80F2,0x80F3,0x80F6,0x80F9,0x80FA,0x80FE,
	0x8103,0x810B,0x8116,0x8117,0x8118,0x811C,0x811E,0x8120,
	0x8124,0x8127,0x812C,0x8130,0x8135,0x813A,0x813C,0x8145,
	0x8147,0x814A,0x814C,0x8152,0x8157,0x8160,0x8161,0x8167,
	0x8168,0x8169,0x816D,0x816F,0x8177,0x8181,0x8190,0x8184,
	0x8185,0x8186,0x818B,0x818E,0x8196,0x8198,0x819B,0x819E,
	0x81A2,0x81AE,0x81B2,0x81B4,0x81BB,0x81CB,0x81C3,0x81C5,
	0x81CA,0x81CE,0x81CF,0x81D5,0x81D7,0x81DB,0x81DD,0x81DE,
	0x81E1,0x81E4,0x81EB,0x81EC,0x81F0,0x81F1,0x81F2,0x81F5,
	0x81F6,0x81F8,0x81F9,0x81FD,0x81FF,0x8200,0x8203,0x820F,
	0x8213,0x8214,0x8219,0x821A,0x821D,0x8221,0x8222,0x8228,
	0x8232,0x8234,0x823A,0x8243,0x8244,0x8245,0x8246,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 88, Array index 0x2E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x824B,0x824E,0x824F,0x8251,0x8256,0x825C,0x8260,
	0x8263,0x8267,0x826D,0x8274,0x827B,0x827D,0x827F,0x8280,
	0x8281,0x8283,0x8284,0x8287,0x8289,0x828A,0x828E,0x8291,
	0x8294,0x8296,0x8298,0x829A,0x829B,0x82A0,0x82A1,0x82A3,
	0x82A4,0x82A7,0x82A8,0x82A9,0x82AA,0x82AE,0x82B0,0x82B2,
	0x82B4,0x82B7,0x82BA,0x82BC,0x82BE,0x82BF,0x82C6,0x82D0,
	0x82D5,0x82DA,0x82E0,0x82E2,0x82E4,0x82E8,0x82EA,0x82ED,
	0x82EF,0x82F6,0x82F7,0x82FD,0x82FE,0x8300,0x8301,0x8307,
	0x8308,0x830A,0x830B,0x8354,0x831B,0x831D,0x831E,0x831F,
	0x8321,0x8322,0x832C,0x832D,0x832E,0x8330,0x8333,0x8337,
	0x833A,0x833C,0x833D,0x8342,0x8343,0x8344,0x8347,0x834D,
	0x834E,0x8351,0x8355,0x8356,0x8357,0x8370,0x8378,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 89, Array index 0x2F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x837D,0x837F,0x8380,0x8382,0x8384,0x8386,0x838D,
	0x8392,0x8394,0x8395,0x8398,0x8399,0x839B,0x839C,0x839D,
	0x83A6,0x83A7,0x83A9,0x83AC,0x83BE,0x83BF,0x83C0,0x83C7,
	0x83C9,0x83CF,0x83D0,0x83D1,0x83D4,0x83DD,0x8353,0x83E8,
	0x83EA,0x83F6,0x83F8,0x83F9,0x83FC,0x8401,0x8406,0x840A,
	0x840F,0x8411,0x8415,0x8419,0x83AD,0x842F,0x8439,0x8445,
	0x8447,0x8448,0x844A,0x844D,0x844F,0x8451,0x8452,0x8456,
	0x8458,0x8459,0x845A,0x845C,0x8460,0x8464,0x8465,0x8467,
	0x846A,0x8470,0x8473,0x8474,0x8476,0x8478,0x847C,0x847D,
	0x8481,0x8485,0x8492,0x8493,0x8495,0x849E,0x84A6,0x84A8,
	0x84A9,0x84AA,0x84AF,0x84B1,0x84B4,0x84BA,0x84BD,0x84BE,
	0x84C0,0x84C2,0x84C7,0x84C8,0x84CC,0x84CF,0x84D3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 90, Array index 0x3000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x84DC,0x84E7,0x84EA,0x84EF,0x84F0,0x84F1,0x84F2,
	0x84F7,0x8532,0x84FA,0x84FB,0x84FD,0x8502,0x8503,0x8507,
	0x850C,0x850E,0x8510,0x851C,0x851E,0x8522,0x8523,0x8524,
	0x8525,0x8527,0x852A,0x852B,0x852F,0x8533,0x8534,0x8536,
	0x853F,0x8546,0x854F,0x8550,0x8551,0x8552,0x8553,0x8556,
	0x8559,0x855C,0x855D,0x855E,0x855F,0x8560,0x8561,0x8562,
	0x8564,0x856B,0x856F,0x8579,0x857A,0x857B,0x857D,0x857F,
	0x8581,0x8585,0x8586,0x8589,0x858B,0x858C,0x858F,0x8593,
	0x8598,0x859D,0x859F,0x85A0,0x85A2,0x85A5,0x85A7,0x85B4,
	0x85B6,0x85B7,0x85B8,0x85BC,0x85BD,0x85BE,0x85BF,0x85C2,
	0x85C7,0x85CA,0x85CB,0x85CE,0x85AD,0x85D8,0x85DA,0x85DF,
	0x85E0,0x85E6,0x85E8,0x85ED,0x85F3,0x85F6,0x85FC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 91, Array index 0x3100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x85FF,0x8600,0x8604,0x8605,0x860D,0x860E,0x8610,
	0x8611,0x8612,0x8618,0x8619,0x861B,0x861E,0x8621,0x8627,
	0x8629,0x8636,0x8638,0x863A,0x863C,0x863D,0x8640,0x8642,
	0x8646,0x8652,0x8653,0x8656,0x8657,0x8658,0x8659,0x865D,
	0x8660,0x8661,0x8662,0x8663,0x8664,0x8669,0x866C,0x866F,
	0x8675,0x8676,0x8677,0x867A,0x868D,0x8691,0x8696,0x8698,
	0x869A,0x869C,0x86A1,0x86A6,0x86A7,0x86A8,0x86AD,0x86B1,
	0x86B3,0x86B4,0x86B5,0x86B7,0x86B8,0x86B9,0x86BF,0x86C0,
	0x86C1,0x86C3,0x86C5,0x86D1,0x86D2,0x86D5,0x86D7,0x86DA,
	0x86DC,0x86E0,0x86E3,0x86E5,0x86E7,0x8688,0x86FA,0x86FC,
	0x86FD,0x8704,0x8705,0x8707,0x870B,0x870E,0x870F,0x8710,
	0x8713,0x8714,0x8719,0x871E,0x871F,0x8721,0x8723,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 92, Array index 0x3200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8728,0x872E,0x872F,0x8731,0x8732,0x8739,0x873A,
	0x873C,0x873D,0x873E,0x8740,0x8743,0x8745,0x874D,0x8758,
	0x875D,0x8761,0x8764,0x8765,0x876F,0x8771,0x8772,0x877B,
	0x8783,0x8784,0x8785,0x8786,0x8787,0x8788,0x8789,0x878B,
	0x878C,0x8790,0x8793,0x8795,0x8797,0x8798,0x8799,0x879E,
	0x87A0,0x87A3,0x87A7,0x87AC,0x87AD,0x87AE,0x87B1,0x87B5,
	0x87BE,0x87BF,0x87C1,0x87C8,0x87C9,0x87CA,0x87CE,0x87D5,
	0x87D6,0x87D9,0x87DA,0x87DC,0x87DF,0x87E2,0x87E3,0x87E4,
	0x87EA,0x87EB,0x87ED,0x87F1,0x87F3,0x87F8,0x87FA,0x87FF,
	0x8801,0x8803,0x8806,0x8809,0x880A,0x880B,0x8810,0x8819,
	0x8812,0x8813,0x8814,0x8818,0x881A,0x881B,0x881C,0x881E,
	0x881F,0x8828,0x882D,0x882E,0x8830,0x8832,0x8835,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 93, Array index 0x3300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x883A,0x883C,0x8841,0x8843,0x8845,0x8848,0x8849,
	0x884A,0x884B,0x884E,0x8851,0x8855,0x8856,0x8858,0x885A,
	0x885C,0x885F,0x8860,0x8864,0x8869,0x8871,0x8879,0x887B,
	0x8880,0x8898,0x889A,0x889B,0x889C,0x889F,0x88A0,0x88A8,
	0x88AA,0x88BA,0x88BD,0x88BE,0x88C0,0x88CA,0x88CB,0x88CC,
	0x88CD,0x88CE,0x88D1,0x88D2,0x88D3,0x88DB,0x88DE,0x88E7,
	0x88EF,0x88F0,0x88F1,0x88F5,0x88F7,0x8901,0x8906,0x890D,
	0x890E,0x890F,0x8915,0x8916,0x8918,0x8919,0x891A,0x891C,
	0x8920,0x8926,0x8927,0x8928,0x8930,0x8931,0x8932,0x8935,
	0x8939,0x893A,0x893E,0x8940,0x8942,0x8945,0x8946,0x8949,
	0x894F,0x8952,0x8957,0x895A,0x895B,0x895C,0x8961,0x8962,
	0x8963,0x896B,0x896E,0x8970,0x8973,0x8975,0x897A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 94, Array index 0x3400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x897B,0x897C,0x897D,0x8989,0x898D,0x8990,0x8994,
	0x8995,0x899B,0x899C,0x899F,0x89A0,0x89A5,0x89B0,0x89B4,
	0x89B5,0x89B6,0x89B7,0x89BC,0x89D4,0x89D5,0x89D6,0x89D7,
	0x89D8,0x89E5,0x89E9,0x89EB,0x89ED,0x89F1,0x89F3,0x89F6,
	0x89F9,0x89FD,0x89FF,0x8A04,0x8A05,0x8A07,0x8A0F,0x8A11,
	0x8A12,0x8A14,0x8A15,0x8A1E,0x8A20,0x8A22,0x8A24,0x8A26,
	0x8A2B,0x8A2C,0x8A2F,0x8A35,0x8A37,0x8A3D,0x8A3E,0x8A40,
	0x8A43,0x8A45,0x8A47,0x8A49,0x8A4D,0x8A4E,0x8A53,0x8A56,
	0x8A57,0x8A58,0x8A5C,0x8A5D,0x8A61,0x8A65,0x8A67,0x8A75,
	0x8A76,0x8A77,0x8A79,0x8A7A,0x8A7B,0x8A7E,0x8A7F,0x8A80,
	0x8A83,0x8A86,0x8A8B,0x8A8F,0x8A90,0x8A92,0x8A96,0x8A97,
	0x8A99,0x8A9F,0x8AA7,0x8AA9,0x8AAE,0x8AAF,0x8AB3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 95, Array index 0x3500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8AB6,0x8AB7,0x8ABB,0x8ABE,0x8AC3,0x8AC6,0x8AC8,
	0x8AC9,0x8ACA,0x8AD1,0x8AD3,0x8AD4,0x8AD5,0x8AD7,0x8ADD,
	0x8ADF,0x8AEC,0x8AF0,0x8AF4,0x8AF5,0x8AF6,0x8AFC,0x8AFF,
	0x8B05,0x8B06,0x8B0B,0x8B11,0x8B1C,0x8B1E,0x8B1F,0x8B0A,
	0x8B2D,0x8B30,0x8B37,0x8B3C,0x8B42,0x8B43,0x8B44,0x8B45,
	0x8B46,0x8B48,0x8B52,0x8B53,0x8B54,0x8B59,0x8B4D,0x8B5E,
	0x8B63,0x8B6D,0x8B76,0x8B78,0x8B79,0x8B7C,0x8B7E,0x8B81,
	0x8B84,0x8B85,0x8B8B,0x8B8D,0x8B8F,0x8B94,0x8B95,0x8B9C,
	0x8B9E,0x8B9F,0x8C38,0x8C39,0x8C3D,0x8C3E,0x8C45,0x8C47,
	0x8C49,0x8C4B,0x8C4F,0x8C51,0x8C53,0x8C54,0x8C57,0x8C58,
	0x8C5B,0x8C5D,0x8C59,0x8C63,0x8C64,0x8C66,0x8C68,0x8C69,
	0x8C6D,0x8C73,0x8C75,0x8C76,0x8C7B,0x8C7E,0x8C86,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 96, Array index 0x3600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8C87,0x8C8B,0x8C90,0x8C92,0x8C93,0x8C99,0x8C9B,
	0x8C9C,0x8CA4,0x8CB9,0x8CBA,0x8CC5,0x8CC6,0x8CC9,0x8CCB,
	0x8CCF,0x8CD6,0x8CD5,0x8CD9,0x8CDD,0x8CE1,0x8CE8,0x8CEC,
	0x8CEF,0x8CF0,0x8CF2,0x8CF5,0x8CF7,0x8CF8,0x8CFE,0x8CFF,
	0x8D01,0x8D03,0x8D09,0x8D12,0x8D17,0x8D1B,0x8D65,0x8D69,
	0x8D6C,0x8D6E,0x8D7F,0x8D82,0x8D84,0x8D88,0x8D8D,0x8D90,
	0x8D91,0x8D95,0x8D9E,0x8D9F,0x8DA0,0x8DA6,0x8DAB,0x8DAC,
	0x8DAF,0x8DB2,0x8DB5,0x8DB7,0x8DB9,0x8DBB,0x8DC0,0x8DC5,
	0x8DC6,0x8DC7,0x8DC8,0x8DCA,0x8DCE,0x8DD1,0x8DD4,0x8DD5,
	0x8DD7,0x8DD9,0x8DE4,0x8DE5,0x8DE7,0x8DEC,0x8DF0,0x8DBC,
	0x8DF1,0x8DF2,0x8DF4,0x8DFD,0x8E01,0x8E04,0x8E05,0x8E06,
	0x8E0B,0x8E11,0x8E14,0x8E16,0x8E20,0x8E21,0x8E22,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 97, Array index 0x3700 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8E23,0x8E26,0x8E27,0x8E31,0x8E33,0x8E36,0x8E37,
	0x8E38,0x8E39,0x8E3D,0x8E40,0x8E41,0x8E4B,0x8E4D,0x8E4E,
	0x8E4F,0x8E54,0x8E5B,0x8E5C,0x8E5D,0x8E5E,0x8E61,0x8E62,
	0x8E69,0x8E6C,0x8E6D,0x8E6F,0x8E70,0x8E71,0x8E79,0x8E7A,
	0x8E7B,0x8E82,0x8E83,0x8E89,0x8E90,0x8E92,0x8E95,0x8E9A,
	0x8E9B,0x8E9D,0x8E9E,0x8EA2,0x8EA7,0x8EA9,0x8EAD,0x8EAE,
	0x8EB3,0x8EB5,0x8EBA,0x8EBB,0x8EC0,0x8EC1,0x8EC3,0x8EC4,
	0x8EC7,0x8ECF,0x8ED1,0x8ED4,0x8EDC,0x8EE8,0x8EEE,0x8EF0,
	0x8EF1,0x8EF7,0x8EF9,0x8EFA,0x8EED,0x8F00,0x8F02,0x8F07,
	0x8F08,0x8F0F,0x8F10,0x8F16,0x8F17,0x8F18,0x8F1E,0x8F20,
	0x8F21,0x8F23,0x8F25,0x8F27,0x8F28,0x8F2C,0x8F2D,0x8F2E,
	0x8F34,0x8F35,0x8F36,0x8F37,0x8F3A,0x8F40,0x8F41,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 98, Array index 0x3800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x8F43,0x8F47,0x8F4F,0x8F51,0x8F52,0x8F53,0x8F54,
	0x8F55,0x8F58,0x8F5D,0x8F5E,0x8F65,0x8F9D,0x8FA0,0x8FA1,
	0x8FA4,0x8FA5,0x8FA6,0x8FB5,0x8FB6,0x8FB8,0x8FBE,0x8FC0,
	0x8FC1,0x8FC6,0x8FCA,0x8FCB,0x8FCD,0x8FD0,0x8FD2,0x8FD3,
	0x8FD5,0x8FE0,0x8FE3,0x8FE4,0x8FE8,0x8FEE,0x8FF1,0x8FF5,
	0x8FF6,0x8FFB,0x8FFE,0x9002,0x9004,0x9008,0x900C,0x9018,
	0x901B,0x9028,0x9029,0x902F,0x902A,0x902C,0x902D,0x9033,
	0x9034,0x9037,0x903F,0x9043,0x9044,0x904C,0x905B,0x905D,
	0x9062,0x9066,0x9067,0x906C,0x9070,0x9074,0x9079,0x9085,
	0x9088,0x908B,0x908C,0x908E,0x9090,0x9095,0x9097,0x9098,
	0x9099,0x909B,0x90A0,0x90A1,0x90A2,0x90A5,0x90B0,0x90B2,
	0x90B3,0x90B4,0x90B6,0x90BD,0x90CC,0x90BE,0x90C3,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 99, Array index 0x3900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x90C4,0x90C5,0x90C7,0x90C8,0x90D5,0x90D7,0x90D8,
	0x90D9,0x90DC,0x90DD,0x90DF,0x90E5,0x90D2,0x90F6,0x90EB,
	0x90EF,0x90F0,0x90F4,0x90FE,0x90FF,0x9100,0x9104,0x9105,
	0x9106,0x9108,0x910D,0x9110,0x9114,0x9116,0x9117,0x9118,
	0x911A,0x911C,0x911E,0x9120,0x9125,0x9122,0x9123,0x9127,
	0x9129,0x912E,0x912F,0x9131,0x9134,0x9136,0x9137,0x9139,
	0x913A,0x913C,0x913D,0x9143,0x9147,0x9148,0x914F,0x9153,
	0x9157,0x9159,0x915A,0x915B,0x9161,0x9164,0x9167,0x916D,
	0x9174,0x9179,0x917A,0x917B,0x9181,0x9183,0x9185,0x9186,
	0x918A,0x918E,0x9191,0x9193,0x9194,0x9195,0x9198,0x919E,
	0x91A1,0x91A6,0x91A8,0x91AC,0x91AD,0x91AE,0x91B0,0x91B1,
	0x91B2,0x91B3,0x91B6,0x91BB,0x91BC,0x91BD,0x91BF,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 100, Array index 0x3A00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x91C2,0x91C3,0x91C5,0x91D3,0x91D4,0x91D7,0x91D9,
	0x91DA,0x91DE,0x91E4,0x91E5,0x91E9,0x91EA,0x91EC,0x91ED,
	0x91EE,0x91EF,0x91F0,0x91F1,0x91F7,0x91F9,0x91FB,0x91FD,
	0x9200,0x9201,0x9204,0x9205,0x9206,0x9207,0x9209,0x920A,
	0x920C,0x9210,0x9212,0x9213,0x9216,0x9218,0x921C,0x921D,
	0x9223,0x9224,0x9225,0x9226,0x9228,0x922E,0x922F,0x9230,
	0x9233,0x9235,0x9236,0x9238,0x9239,0x923A,0x923C,0x923E,
	0x9240,0x9242,0x9243,0x9246,0x9247,0x924A,0x924D,0x924E,
	0x924F,0x9251,0x9258,0x9259,0x925C,0x925D,0x9260,0x9261,
	0x9265,0x9267,0x9268,0x9269,0x926E,0x926F,0x9270,0x9275,
	0x9276,0x9277,0x9278,0x9279,0x927B,0x927C,0x927D,0x927F,
	0x9288,0x9289,0x928A,0x928D,0x928E,0x9292,0x9297,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 101, Array index 0x3B00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9299,0x929F,0x92A0,0x92A4,0x92A5,0x92A7,0x92A8,
	0x92AB,0x92AF,0x92B2,0x92B6,0x92B8,0x92BA,0x92BB,0x92BC,
	0x92BD,0x92BF,0x92C0,0x92C1,0x92C2,0x92C3,0x92C5,0x92C6,
	0x92C7,0x92C8,0x92CB,0x92CC,0x92CD,0x92CE,0x92D0,0x92D3,
	0x92D5,0x92D7,0x92D8,0x92D9,0x92DC,0x92DD,0x92DF,0x92E0,
	0x92E1,0x92E3,0x92E5,0x92E7,0x92E8,0x92EC,0x92EE,0x92F0,
	0x92F9,0x92FB,0x92FF,0x9300,0x9302,0x9308,0x930D,0x9311,
	0x9314,0x9315,0x931C,0x931D,0x931E,0x931F,0x9321,0x9324,
	0x9325,0x9327,0x9329,0x932A,0x9333,0x9334,0x9336,0x9337,
	0x9347,0x9348,0x9349,0x9350,0x9351,0x9352,0x9355,0x9357,
	0x9358,0x935A,0x935E,0x9364,0x9365,0x9367,0x9369,0x936A,
	0x936D,0x936F,0x9370,0x9371,0x9373,0x9374,0x9376,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 102, Array index 0x3C00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x937A,0x937D,0x937F,0x9380,0x9381,0x9382,0x9388,
	0x938A,0x938B,0x938D,0x938F,0x9392,0x9395,0x9398,0x939B,
	0x939E,0x93A1,0x93A3,0x93A4,0x93A6,0x93A8,0x93AB,0x93B4,
	0x93B5,0x93B6,0x93BA,0x93A9,0x93C1,0x93C4,0x93C5,0x93C6,
	0x93C7,0x93C9,0x93CA,0x93CB,0x93CC,0x93CD,0x93D3,0x93D9,
	0x93DC,0x93DE,0x93DF,0x93E2,0x93E6,0x93E7,0x93F9,0x93F7,
	0x93F8,0x93FA,0x93FB,0x93FD,0x9401,0x9402,0x9404,0x9408,
	0x9409,0x940D,0x940E,0x940F,0x9415,0x9416,0x9417,0x941F,
	0x942E,0x942F,0x9431,0x9432,0x9433,0x9434,0x943B,0x943F,
	0x943D,0x9443,0x9445,0x9448,0x944A,0x944C,0x9455,0x9459,
	0x945C,0x945F,0x9461,0x9463,0x9468,0x946B,0x946D,0x946E,
	0x946F,0x9471,0x9472,0x9484,0x9483,0x9578,0x9579,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 103, Array index 0x3D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x957E,0x9584,0x9588,0x958C,0x958D,0x958E,0x959D,
	0x959E,0x959F,0x95A1,0x95A6,0x95A9,0x95AB,0x95AC,0x95B4,
	0x95B6,0x95BA,0x95BD,0x95BF,0x95C6,0x95C8,0x95C9,0x95CB,
	0x95D0,0x95D1,0x95D2,0x95D3,0x95D9,0x95DA,0x95DD,0x95DE,
	0x95DF,0x95E0,0x95E4,0x95E6,0x961D,0x961E,0x9622,0x9624,
	0x9625,0x9626,0x962C,0x9631,0x9633,0x9637,0x9638,0x9639,
	0x963A,0x963C,0x963D,0x9641,0x9652,0x9654,0x9656,0x9657,
	0x9658,0x9661,0x966E,0x9674,0x967B,0x967C,0x967E,0x967F,
	0x9681,0x9682,0x9683,0x9684,0x9689,0x9691,0x9696,0x969A,
	0x969D,0x969F,0x96A4,0x96A5,0x96A6,0x96A9,0x96AE,0x96AF,
	0x96B3,0x96BA,0x96CA,0x96D2,0x5DB2,0x96D8,0x96DA,0x96DD,
	0x96DE,0x96DF,0x96E9,0x96EF,0x96F1,0x96FA,0x9702,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 104, Array index 0x3E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9703,0x9705,0x9709,0x971A,0x971B,0x971D,0x9721,
	0x9722,0x9723,0x9728,0x9731,0x9733,0x9741,0x9743,0x974A,
	0x974E,0x974F,0x9755,0x9757,0x9758,0x975A,0x975B,0x9763,
	0x9767,0x976A,0x976E,0x9773,0x9776,0x9777,0x9778,0x977B,
	0x977D,0x977F,0x9780,0x9789,0x9795,0x9796,0x9797,0x9799,
	0x979A,0x979E,0x979F,0x97A2,0x97AC,0x97AE,0x97B1,0x97B2,
	0x97B5,0x97B6,0x97B8,0x97B9,0x97BA,0x97BC,0x97BE,0x97BF,
	0x97C1,0x97C4,0x97C5,0x97C7,0x97C9,0x97CA,0x97CC,0x97CD,
	0x97CE,0x97D0,0x97D1,0x97D4,0x97D7,0x97D8,0x97D9,0x97DD,
	0x97DE,0x97E0,0x97DB,0x97E1,0x97E4,0x97EF,0x97F1,0x97F4,
	0x97F7,0x97F8,0x97FA,0x9807,0x980A,0x9819,0x980D,0x980E,
	0x9814,0x9816,0x981C,0x981E,0x9820,0x9823,0x9826,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 105, Array index 0x3F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x982B,0x982E,0x982F,0x9830,0x9832,0x9833,0x9835,
	0x9825,0x983E,0x9844,0x9847,0x984A,0x9851,0x9852,0x9853,
	0x9856,0x9857,0x9859,0x985A,0x9862,0x9863,0x9865,0x9866,
	0x986A,0x986C,0x98AB,0x98AD,0x98AE,0x98B0,0x98B4,0x98B7,
	0x98B8,0x98BA,0x98BB,0x98BF,0x98C2,0x98C5,0x98C8,0x98CC,
	0x98E1,0x98E3,0x98E5,0x98E6,0x98E7,0x98EA,0x98F3,0x98F6,
	0x9902,0x9907,0x9908,0x9911,0x9915,0x9916,0x9917,0x991A,
	0x991B,0x991C,0x991F,0x9922,0x9926,0x9927,0x992B,0x9931,
	0x9932,0x9933,0x9934,0x9935,0x9939,0x993A,0x993B,0x993C,
	0x9940,0x9941,0x9946,0x9947,0x9948,0x994D,0x994E,0x9954,
	0x9958,0x9959,0x995B,0x995C,0x995E,0x995F,0x9960,0x999B,
	0x999D,0x999F,0x99A6,0x99B0,0x99B1,0x99B2,0x99B5,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 106, Array index 0x4000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x99B9,0x99BA,0x99BD,0x99BF,0x99C3,0x99C9,0x99D3,
	0x99D4,0x99D9,0x99DA,0x99DC,0x99DE,0x99E7,0x99EA,0x99EB,
	0x99EC,0x99F0,0x99F4,0x99F5,0x99F9,0x99FD,0x99FE,0x9A02,
	0x9A03,0x9A04,0x9A0B,0x9A0C,0x9A10,0x9A11,0x9A16,0x9A1E,
	0x9A20,0x9A22,0x9A23,0x9A24,0x9A27,0x9A2D,0x9A2E,0x9A33,
	0x9A35,0x9A36,0x9A38,0x9A47,0x9A41,0x9A44,0x9A4A,0x9A4B,
	0x9A4C,0x9A4E,0x9A51,0x9A54,0x9A56,0x9A5D,0x9AAA,0x9AAC,
	0x9AAE,0x9AAF,0x9AB2,0x9AB4,0x9AB5,0x9AB6,0x9AB9,0x9ABB,
	0x9ABE,0x9ABF,0x9AC1,0x9AC3,0x9AC6,0x9AC8,0x9ACE,0x9AD0,
	0x9AD2,0x9AD5,0x9AD6,0x9AD7,0x9ADB,0x9ADC,0x9AE0,0x9AE4,
	0x9AE5,0x9AE7,0x9AE9,0x9AEC,0x9AF2,0x9AF3,0x9AF5,0x9AF9,
	0x9AFA,0x9AFD,0x9AFF,0x9B00,0x9B01,0x9B02,0x9B03,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 107, Array index 0x4100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9B04,0x9B05,0x9B08,0x9B09,0x9B0B,0x9B0C,0x9B0D,
	0x9B0E,0x9B10,0x9B12,0x9B16,0x9B19,0x9B1B,0x9B1C,0x9B20,
	0x9B26,0x9B2B,0x9B2D,0x9B33,0x9B34,0x9B35,0x9B37,0x9B39,
	0x9B3A,0x9B3D,0x9B48,0x9B4B,0x9B4C,0x9B55,0x9B56,0x9B57,
	0x9B5B,0x9B5E,0x9B61,0x9B63,0x9B65,0x9B66,0x9B68,0x9B6A,
	0x9B6B,0x9B6C,0x9B6D,0x9B6E,0x9B73,0x9B75,0x9B77,0x9B78,
	0x9B79,0x9B7F,0x9B80,0x9B84,0x9B85,0x9B86,0x9B87,0x9B89,
	0x9B8A,0x9B8B,0x9B8D,0x9B8F,0x9B90,0x9B94,0x9B9A,0x9B9D,
	0x9B9E,0x9BA6,0x9BA7,0x9BA9,0x9BAC,0x9BB0,0x9BB1,0x9BB2,
	0x9BB7,0x9BB8,0x9BBB,0x9BBC,0x9BBE,0x9BBF,0x9BC1,0x9BC7,
	0x9BC8,0x9BCE,0x9BD0,0x9BD7,0x9BD8,0x9BDD,0x9BDF,0x9BE5,
	0x9BE7,0x9BEA,0x9BEB,0x9BEF,0x9BF3,0x9BF7,0x9BF8,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 108, Array index 0x4200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9BF9,0x9BFA,0x9BFD,0x9BFF,0x9C00,0x9C02,0x9C0B,
	0x9C0F,0x9C11,0x9C16,0x9C18,0x9C19,0x9C1A,0x9C1C,0x9C1E,
	0x9C22,0x9C23,0x9C26,0x9C27,0x9C28,0x9C29,0x9C2A,0x9C31,
	0x9C35,0x9C36,0x9C37,0x9C3D,0x9C41,0x9C43,0x9C44,0x9C45,
	0x9C49,0x9C4A,0x9C4E,0x9C4F,0x9C50,0x9C53,0x9C54,0x9C56,
	0x9C58,0x9C5B,0x9C5D,0x9C5E,0x9C5F,0x9C63,0x9C69,0x9C6A,
	0x9C5C,0x9C6B,0x9C68,0x9C6E,0x9C70,0x9C72,0x9C75,0x9C77,
	0x9C7B,0x9CE6,0x9CF2,0x9CF7,0x9CF9,0x9D0B,0x9D02,0x9D11,
	0x9D17,0x9D18,0x9D1C,0x9D1D,0x9D1E,0x9D2F,0x9D30,0x9D32,
	0x9D33,0x9D34,0x9D3A,0x9D3C,0x9D45,0x9D3D,0x9D42,0x9D43,
	0x9D47,0x9D4A,0x9D53,0x9D54,0x9D5F,0x9D63,0x9D62,0x9D65,
	0x9D69,0x9D6A,0x9D6B,0x9D70,0x9D76,0x9D77,0x9D7B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 109, Array index 0x4300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9D7C,0x9D7E,0x9D83,0x9D84,0x9D86,0x9D8A,0x9D8D,
	0x9D8E,0x9D92,0x9D93,0x9D95,0x9D96,0x9D97,0x9D98,0x9DA1,
	0x9DAA,0x9DAC,0x9DAE,0x9DB1,0x9DB5,0x9DB9,0x9DBC,0x9DBF,
	0x9DC3,0x9DC7,0x9DC9,0x9DCA,0x9DD4,0x9DD5,0x9DD6,0x9DD7,
	0x9DDA,0x9DDE,0x9DDF,0x9DE0,0x9DE5,0x9DE7,0x9DE9,0x9DEB,
	0x9DEE,0x9DF0,0x9DF3,0x9DF4,0x9DFE,0x9E0A,0x9E02,0x9E07,
	0x9E0E,0x9E10,0x9E11,0x9E12,0x9E15,0x9E16,0x9E19,0x9E1C,
	0x9E1D,0x9E7A,0x9E7B,0x9E7C,0x9E80,0x9E82,0x9E83,0x9E84,
	0x9E85,0x9E87,0x9E8E,0x9E8F,0x9E96,0x9E98,0x9E9B,0x9E9E,
	0x9EA4,0x9EA8,0x9EAC,0x9EAE,0x9EAF,0x9EB0,0x9EB3,0x9EB4,
	0x9EB5,0x9EC6,0x9EC8,0x9ECB,0x9ED5,0x9EDF,0x9EE4,0x9EE7,
	0x9EEC,0x9EED,0x9EEE,0x9EF0,0x9EF1,0x9EF2,0x9EF5,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 110, Array index 0x4400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x9EF8,0x9EFF,0x9F02,0x9F03,0x9F09,0x9F0F,0x9F10,
	0x9F11,0x9F12,0x9F14,0x9F16,0x9F17,0x9F19,0x9F1A,0x9F1B,
	0x9F1F,0x9F22,0x9F26,0x9F2A,0x9F2B,0x9F2F,0x9F31,0x9F32,
	0x9F34,0x9F37,0x9F39,0x9F3A,0x9F3C,0x9F3D,0x9F3F,0x9F41,
	0x9F43,0x9F44,0x9F45,0x9F46,0x9F47,0x9F53,0x9F55,0x9F56,
	0x9F57,0x9F58,0x9F5A,0x9F5D,0x9F5E,0x9F68,0x9F69,0x9F6D,
	0x9F6E,0x9F6F,0x9F70,0x9F71,0x9F73,0x9F75,0x9F7A,0x9F7D,
	0x9F8F,0x9F90,0x9F91,0x9F92,0x9F94,0x9F96,0x9F97,0x9F9E,
	0x9FA1,0x9FA2,0x9FA3,0x9FA5,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
};

#endif /* ICONV_TO_UCS_CCS_JIS_X0212_1990 && !defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit jis_x0212_1990 -> UCS size-optimized table (12602 bytes).
 * ======================================================================
 */
#if defined (ICONV_TO_UCS_CCS_JIS_X0212_1990) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
to_ucs_size_jis_x0212_1990[] =
{
	0x0049, /* Ranges number */
	0x0004, /* Unranged codes number */
	0x1895, /* First unranged code index */
	/* Ranges list: first code, last Code, array index. */
	/* Array index: 0x0003 */ 0x222F, 0x2239, 0x00DE,
	/* Array index: 0x0006 */ 0x2242, 0x2244, 0x00E9,
	/* Array index: 0x0009 */ 0x226B, 0x2271, 0x00EC,
	/* Array index: 0x000C */ 0x2661, 0x266C, 0x00F3,
	/* Array index: 0x000F */ 0x2671, 0x267C, 0x00FF,
	/* Array index: 0x0012 */ 0x2742, 0x274E, 0x010B,
	/* Array index: 0x0015 */ 0x2772, 0x277E, 0x0118,
	/* Array index: 0x0018 */ 0x2928, 0x2930, 0x0125,
	/* Array index: 0x001B */ 0x2941, 0x2950, 0x012E,
	/* Array index: 0x001E */ 0x2A21, 0x2A77, 0x013E,
	/* Array index: 0x0021 */ 0x2B21, 0x2B77, 0x0195,
	/* Array index: 0x0024 */ 0x3021, 0x307E, 0x01EC,
	/* Array index: 0x0027 */ 0x3121, 0x317E, 0x024A,
	/* Array index: 0x002A */ 0x3221, 0x327E, 0x02A8,
	/* Array index: 0x002D */ 0x3321, 0x337E, 0x0306,
	/* Array index: 0x0030 */ 0x3421, 0x347E, 0x0364,
	/* Array index: 0x0033 */ 0x3521, 0x357E, 0x03C2,
	/* Array index: 0x0036 */ 0x3621, 0x367E, 0x0420,
	/* Array index: 0x0039 */ 0x3721, 0x377E, 0x047E,
	/* Array index: 0x003C */ 0x3821, 0x387E, 0x04DC,
	/* Array index: 0x003F */ 0x3921, 0x397E, 0x053A,
	/* Array index: 0x0042 */ 0x3A21, 0x3A7E, 0x0598,
	/* Array index: 0x0045 */ 0x3B21, 0x3B7E, 0x05F6,
	/* Array index: 0x0048 */ 0x3C21, 0x3C7E, 0x0654,
	/* Array index: 0x004B */ 0x3D21, 0x3D7E, 0x06B2,
	/* Array index: 0x004E */ 0x3E21, 0x3E7E, 0x0710,
	/* Array index: 0x0051 */ 0x3F21, 0x3F7E, 0x076E,
	/* Array index: 0x0054 */ 0x4021, 0x407E, 0x07CC,
	/* Array index: 0x0057 */ 0x4121, 0x417E, 0x082A,
	/* Array index: 0x005A */ 0x4221, 0x427E, 0x0888,
	/* Array index: 0x005D */ 0x4321, 0x437E, 0x08E6,
	/* Array index: 0x0060 */ 0x4421, 0x447E, 0x0944,
	/* Array index: 0x0063 */ 0x4521, 0x457E, 0x09A2,
	/* Array index: 0x0066 */ 0x4621, 0x467E, 0x0A00,
	/* Array index: 0x0069 */ 0x4721, 0x477E, 0x0A5E,
	/* Array index: 0x006C */ 0x4821, 0x487E, 0x0ABC,
	/* Array index: 0x006F */ 0x4921, 0x497E, 0x0B1A,
	/* Array index: 0x0072 */ 0x4A21, 0x4A7E, 0x0B78,
	/* Array index: 0x0075 */ 0x4B21, 0x4B7E, 0x0BD6,
	/* Array index: 0x0078 */ 0x4C21, 0x4C7E, 0x0C34,
	/* Array index: 0x007B */ 0x4D21, 0x4D7E, 0x0C92,
	/* Array index: 0x007E */ 0x4E21, 0x4E7E, 0x0CF0,
	/* Array index: 0x0081 */ 0x4F21, 0x4F7E, 0x0D4E,
	/* Array index: 0x0084 */ 0x5021, 0x507E, 0x0DAC,
	/* Array index: 0x0087 */ 0x5121, 0x517E, 0x0E0A,
	/* Array index: 0x008A */ 0x5221, 0x527E, 0x0E68,
	/* Array index: 0x008D */ 0x5321, 0x537E, 0x0EC6,
	/* Array index: 0x0090 */ 0x5421, 0x547E, 0x0F24,
	/* Array index: 0x0093 */ 0x5521, 0x557E, 0x0F82,
	/* Array index: 0x0096 */ 0x5621, 0x567E, 0x0FE0,
	/* Array index: 0x0099 */ 0x5721, 0x577E, 0x103E,
	/* Array index: 0x009C */ 0x5821, 0x587E, 0x109C,
	/* Array index: 0x009F */ 0x5921, 0x597E, 0x10FA,
	/* Array index: 0x00A2 */ 0x5A21, 0x5A7E, 0x1158,
	/* Array index: 0x00A5 */ 0x5B21, 0x5B7E, 0x11B6,
	/* Array index: 0x00A8 */ 0x5C21, 0x5C7E, 0x1214,
	/* Array index: 0x00AB */ 0x5D21, 0x5D7E, 0x1272,
	/* Array index: 0x00AE */ 0x5E21, 0x5E7E, 0x12D0,
	/* Array index: 0x00B1 */ 0x5F21, 0x5F7E, 0x132E,
	/* Array index: 0x00B4 */ 0x6021, 0x607E, 0x138C,
	/* Array index: 0x00B7 */ 0x6121, 0x617E, 0x13EA,
	/* Array index: 0x00BA */ 0x6221, 0x627E, 0x1448,
	/* Array index: 0x00BD */ 0x6321, 0x637E, 0x14A6,
	/* Array index: 0x00C0 */ 0x6421, 0x647E, 0x1504,
	/* Array index: 0x00C3 */ 0x6521, 0x657E, 0x1562,
	/* Array index: 0x00C6 */ 0x6621, 0x667E, 0x15C0,
	/* Array index: 0x00C9 */ 0x6721, 0x677E, 0x161E,
	/* Array index: 0x00CC */ 0x6821, 0x687E, 0x167C,
	/* Array index: 0x00CF */ 0x6921, 0x697E, 0x16DA,
	/* Array index: 0x00D2 */ 0x6A21, 0x6A7E, 0x1738,
	/* Array index: 0x00D5 */ 0x6B21, 0x6B7E, 0x1796,
	/* Array index: 0x00D8 */ 0x6C21, 0x6C7E, 0x17F4,
	/* Array index: 0x00DB */ 0x6D21, 0x6D63, 0x1852,
	/* Ranges content */
	/* Range 0x222F - 0x2239, array index: 0x00DE */
	0x02D8,0x02C7,0x00B8,0x02D9,0x02DD,0x00AF,0x02DB,0x02DA,
	0x007E,0x0384,0x0385,
	/* Range 0x2242 - 0x2244, array index: 0x00E9 */
	0x00A1,0x00A6,0x00BF,
	/* Range 0x226B - 0x2271, array index: 0x00EC */
	0x00BA,0x00AA,0x00A9,0x00AE,0x2122,0x00A4,0x2116,
	/* Range 0x2661 - 0x266C, array index: 0x00F3 */
	0x0386,0x0388,0x0389,0x038A,0x03AA,INVALC,0x038C,INVALC,
	0x038E,0x03AB,INVALC,0x038F,
	/* Range 0x2671 - 0x267C, array index: 0x00FF */
	0x03AC,0x03AD,0x03AE,0x03AF,0x03CA,0x0390,0x03CC,0x03C2,
	0x03CD,0x03CB,0x03B0,0x03CE,
	/* Range 0x2742 - 0x274E, array index: 0x010B */
	0x0402,0x0403,0x0404,0x0405,0x0406,0x0407,0x0408,0x0409,
	0x040A,0x040B,0x040C,0x040E,0x040F,
	/* Range 0x2772 - 0x277E, array index: 0x0118 */
	0x0452,0x0453,0x0454,0x0455,0x0456,0x0457,0x0458,0x0459,
	0x045A,0x045B,0x045C,0x045E,0x045F,
	/* Range 0x2928 - 0x2930, array index: 0x0125 */
	0x0141,0x013F,INVALC,0x014A,0x00D8,0x0152,INVALC,0x0166,
	0x00DE,
	/* Range 0x2941 - 0x2950, array index: 0x012E */
	0x00E6,0x0111,0x00F0,0x0127,0x0131,0x0133,0x0138,0x0142,
	0x0140,0x0149,0x014B,0x00F8,0x0153,0x00DF,0x0167,0x00FE,
	/* Range 0x2A21 - 0x2A77, array index: 0x013E */
	0x00C1,0x00C0,0x00C4,0x00C2,0x0102,0x01CD,0x0100,0x0104,
	0x00C5,0x00C3,0x0106,0x0108,0x010C,0x00C7,0x010A,0x010E,
	0x00C9,0x00C8,0x00CB,0x00CA,0x011A,0x0116,0x0112,0x0118,
	INVALC,0x011C,0x011E,0x0122,0x0120,0x0124,0x00CD,0x00CC,
	0x00CF,0x00CE,0x01CF,0x0130,0x012A,0x012E,0x0128,0x0134,
	0x0136,0x0139,0x013D,0x013B,0x0143,0x0147,0x0145,0x00D1,
	0x00D3,0x00D2,0x00D6,0x00D4,0x01D1,0x0150,0x014C,0x00D5,
	0x0154,0x0158,0x0156,0x015A,0x015C,0x0160,0x015E,0x0164,
	0x0162,0x00DA,0x00D9,0x00DC,0x00DB,0x016C,0x01D3,0x0170,
	0x016A,0x0172,0x016E,0x0168,0x01D7,0x01DB,0x01D9,0x01D5,
	0x0174,0x00DD,0x0178,0x0176,0x0179,0x017D,0x017B,
	/* Range 0x2B21 - 0x2B77, array index: 0x0195 */
	0x00E1,0x00E0,0x00E4,0x00E2,0x0103,0x01CE,0x0101,0x0105,
	0x00E5,0x00E3,0x0107,0x0109,0x010D,0x00E7,0x010B,0x010F,
	0x00E9,0x00E8,0x00EB,0x00EA,0x011B,0x0117,0x0113,0x0119,
	0x01F5,0x011D,0x011F,INVALC,0x0121,0x0125,0x00ED,0x00EC,
	0x00EF,0x00EE,0x01D0,INVALC,0x012B,0x012F,0x0129,0x0135,
	0x0137,0x013A,0x013E,0x013C,0x0144,0x0148,0x0146,0x00F1,
	0x00F3,0x00F2,0x00F6,0x00F4,0x01D2,0x0151,0x014D,0x00F5,
	0x0155,0x0159,0x0157,0x015B,0x015D,0x0161,0x015F,0x0165,
	0x0163,0x00FA,0x00F9,0x00FC,0x00FB,0x016D,0x01D4,0x0171,
	0x016B,0x0173,0x016F,0x0169,0x01D8,0x01DC,0x01DA,0x01D6,
	0x0175,0x00FD,0x00FF,0x0177,0x017A,0x017E,0x017C,
	/* Range 0x3021 - 0x307E, array index: 0x01EC */
	0x4E02,0x4E04,0x4E05,0x4E0C,0x4E12,0x4E1F,0x4E23,0x4E24,
	0x4E28,0x4E2B,0x4E2E,0x4E2F,0x4E30,0x4E35,0x4E40,0x4E41,
	0x4E44,0x4E47,0x4E51,0x4E5A,0x4E5C,0x4E63,0x4E68,0x4E69,
	0x4E74,0x4E75,0x4E79,0x4E7F,0x4E8D,0x4E96,0x4E97,0x4E9D,
	0x4EAF,0x4EB9,0x4EC3,0x4ED0,0x4EDA,0x4EDB,0x4EE0,0x4EE1,
	0x4EE2,0x4EE8,0x4EEF,0x4EF1,0x4EF3,0x4EF5,0x4EFD,0x4EFE,
	0x4EFF,0x4F00,0x4F02,0x4F03,0x4F08,0x4F0B,0x4F0C,0x4F12,
	0x4F15,0x4F16,0x4F17,0x4F19,0x4F2E,0x4F31,0x4F60,0x4F33,
	0x4F35,0x4F37,0x4F39,0x4F3B,0x4F3E,0x4F40,0x4F42,0x4F48,
	0x4F49,0x4F4B,0x4F4C,0x4F52,0x4F54,0x4F56,0x4F58,0x4F5F,
	0x4F63,0x4F6A,0x4F6C,0x4F6E,0x4F71,0x4F77,0x4F78,0x4F79,
	0x4F7A,0x4F7D,0x4F7E,0x4F81,0x4F82,0x4F84,
	/* Range 0x3121 - 0x317E, array index: 0x024A */
	0x4F85,0x4F89,0x4F8A,0x4F8C,0x4F8E,0x4F90,0x4F92,0x4F93,
	0x4F94,0x4F97,0x4F99,0x4F9A,0x4F9E,0x4F9F,0x4FB2,0x4FB7,
	0x4FB9,0x4FBB,0x4FBC,0x4FBD,0x4FBE,0x4FC0,0x4FC1,0x4FC5,
	0x4FC6,0x4FC8,0x4FC9,0x4FCB,0x4FCC,0x4FCD,0x4FCF,0x4FD2,
	0x4FDC,0x4FE0,0x4FE2,0x4FF0,0x4FF2,0x4FFC,0x4FFD,0x4FFF,
	0x5000,0x5001,0x5004,0x5007,0x500A,0x500C,0x500E,0x5010,
	0x5013,0x5017,0x5018,0x501B,0x501C,0x501D,0x501E,0x5022,
	0x5027,0x502E,0x5030,0x5032,0x5033,0x5035,0x5040,0x5041,
	0x5042,0x5045,0x5046,0x504A,0x504C,0x504E,0x5051,0x5052,
	0x5053,0x5057,0x5059,0x505F,0x5060,0x5062,0x5063,0x5066,
	0x5067,0x506A,0x506D,0x5070,0x5071,0x503B,0x5081,0x5083,
	0x5084,0x5086,0x508A,0x508E,0x508F,0x5090,
	/* Range 0x3221 - 0x327E, array index: 0x02A8 */
	0x5092,0x5093,0x5094,0x5096,0x509B,0x509C,0x509E,0x509F,
	0x50A0,0x50A1,0x50A2,0x50AA,0x50AF,0x50B0,0x50B9,0x50BA,
	0x50BD,0x50C0,0x50C3,0x50C4,0x50C7,0x50CC,0x50CE,0x50D0,
	0x50D3,0x50D4,0x50D8,0x50DC,0x50DD,0x50DF,0x50E2,0x50E4,
	0x50E6,0x50E8,0x50E9,0x50EF,0x50F1,0x50F6,0x50FA,0x50FE,
	0x5103,0x5106,0x5107,0x5108,0x510B,0x510C,0x510D,0x510E,
	0x50F2,0x5110,0x5117,0x5119,0x511B,0x511C,0x511D,0x511E,
	0x5123,0x5127,0x5128,0x512C,0x512D,0x512F,0x5131,0x5133,
	0x5134,0x5135,0x5138,0x5139,0x5142,0x514A,0x514F,0x5153,
	0x5155,0x5157,0x5158,0x515F,0x5164,0x5166,0x517E,0x5183,
	0x5184,0x518B,0x518E,0x5198,0x519D,0x51A1,0x51A3,0x51AD,
	0x51B8,0x51BA,0x51BC,0x51BE,0x51BF,0x51C2,
	/* Range 0x3321 - 0x337E, array index: 0x0306 */
	0x51C8,0x51CF,0x51D1,0x51D2,0x51D3,0x51D5,0x51D8,0x51DE,
	0x51E2,0x51E5,0x51EE,0x51F2,0x51F3,0x51F4,0x51F7,0x5201,
	0x5202,0x5205,0x5212,0x5213,0x5215,0x5216,0x5218,0x5222,
	0x5228,0x5231,0x5232,0x5235,0x523C,0x5245,0x5249,0x5255,
	0x5257,0x5258,0x525A,0x525C,0x525F,0x5260,0x5261,0x5266,
	0x526E,0x5277,0x5278,0x5279,0x5280,0x5282,0x5285,0x528A,
	0x528C,0x5293,0x5295,0x5296,0x5297,0x5298,0x529A,0x529C,
	0x52A4,0x52A5,0x52A6,0x52A7,0x52AF,0x52B0,0x52B6,0x52B7,
	0x52B8,0x52BA,0x52BB,0x52BD,0x52C0,0x52C4,0x52C6,0x52C8,
	0x52CC,0x52CF,0x52D1,0x52D4,0x52D6,0x52DB,0x52DC,0x52E1,
	0x52E5,0x52E8,0x52E9,0x52EA,0x52EC,0x52F0,0x52F1,0x52F4,
	0x52F6,0x52F7,0x5300,0x5303,0x530A,0x530B,
	/* Range 0x3421 - 0x347E, array index: 0x0364 */
	0x530C,0x5311,0x5313,0x5318,0x531B,0x531C,0x531E,0x531F,
	0x5325,0x5327,0x5328,0x5329,0x532B,0x532C,0x532D,0x5330,
	0x5332,0x5335,0x533C,0x533D,0x533E,0x5342,0x534C,0x534B,
	0x5359,0x535B,0x5361,0x5363,0x5365,0x536C,0x536D,0x5372,
	0x5379,0x537E,0x5383,0x5387,0x5388,0x538E,0x5393,0x5394,
	0x5399,0x539D,0x53A1,0x53A4,0x53AA,0x53AB,0x53AF,0x53B2,
	0x53B4,0x53B5,0x53B7,0x53B8,0x53BA,0x53BD,0x53C0,0x53C5,
	0x53CF,0x53D2,0x53D3,0x53D5,0x53DA,0x53DD,0x53DE,0x53E0,
	0x53E6,0x53E7,0x53F5,0x5402,0x5413,0x541A,0x5421,0x5427,
	0x5428,0x542A,0x542F,0x5431,0x5434,0x5435,0x5443,0x5444,
	0x5447,0x544D,0x544F,0x545E,0x5462,0x5464,0x5466,0x5467,
	0x5469,0x546B,0x546D,0x546E,0x5474,0x547F,
	/* Range 0x3521 - 0x357E, array index: 0x03C2 */
	0x5481,0x5483,0x5485,0x5488,0x5489,0x548D,0x5491,0x5495,
	0x5496,0x549C,0x549F,0x54A1,0x54A6,0x54A7,0x54A9,0x54AA,
	0x54AD,0x54AE,0x54B1,0x54B7,0x54B9,0x54BA,0x54BB,0x54BF,
	0x54C6,0x54CA,0x54CD,0x54CE,0x54E0,0x54EA,0x54EC,0x54EF,
	0x54F6,0x54FC,0x54FE,0x54FF,0x5500,0x5501,0x5505,0x5508,
	0x5509,0x550C,0x550D,0x550E,0x5515,0x552A,0x552B,0x5532,
	0x5535,0x5536,0x553B,0x553C,0x553D,0x5541,0x5547,0x5549,
	0x554A,0x554D,0x5550,0x5551,0x5558,0x555A,0x555B,0x555E,
	0x5560,0x5561,0x5564,0x5566,0x557F,0x5581,0x5582,0x5586,
	0x5588,0x558E,0x558F,0x5591,0x5592,0x5593,0x5594,0x5597,
	0x55A3,0x55A4,0x55AD,0x55B2,0x55BF,0x55C1,0x55C3,0x55C6,
	0x55C9,0x55CB,0x55CC,0x55CE,0x55D1,0x55D2,
	/* Range 0x3621 - 0x367E, array index: 0x0420 */
	0x55D3,0x55D7,0x55D8,0x55DB,0x55DE,0x55E2,0x55E9,0x55F6,
	0x55FF,0x5605,0x5608,0x560A,0x560D,0x560E,0x560F,0x5610,
	0x5611,0x5612,0x5619,0x562C,0x5630,0x5633,0x5635,0x5637,
	0x5639,0x563B,0x563C,0x563D,0x563F,0x5640,0x5641,0x5643,
	0x5644,0x5646,0x5649,0x564B,0x564D,0x564F,0x5654,0x565E,
	0x5660,0x5661,0x5662,0x5663,0x5666,0x5669,0x566D,0x566F,
	0x5671,0x5672,0x5675,0x5684,0x5685,0x5688,0x568B,0x568C,
	0x5695,0x5699,0x569A,0x569D,0x569E,0x569F,0x56A6,0x56A7,
	0x56A8,0x56A9,0x56AB,0x56AC,0x56AD,0x56B1,0x56B3,0x56B7,
	0x56BE,0x56C5,0x56C9,0x56CA,0x56CB,0x56CF,0x56D0,0x56CC,
	0x56CD,0x56D9,0x56DC,0x56DD,0x56DF,0x56E1,0x56E4,0x56E5,
	0x56E6,0x56E7,0x56E8,0x56F1,0x56EB,0x56ED,
	/* Range 0x3721 - 0x377E, array index: 0x047E */
	0x56F6,0x56F7,0x5701,0x5702,0x5707,0x570A,0x570C,0x5711,
	0x5715,0x571A,0x571B,0x571D,0x5720,0x5722,0x5723,0x5724,
	0x5725,0x5729,0x572A,0x572C,0x572E,0x572F,0x5733,0x5734,
	0x573D,0x573E,0x573F,0x5745,0x5746,0x574C,0x574D,0x5752,
	0x5762,0x5765,0x5767,0x5768,0x576B,0x576D,0x576E,0x576F,
	0x5770,0x5771,0x5773,0x5774,0x5775,0x5777,0x5779,0x577A,
	0x577B,0x577C,0x577E,0x5781,0x5783,0x578C,0x5794,0x5797,
	0x5799,0x579A,0x579C,0x579D,0x579E,0x579F,0x57A1,0x5795,
	0x57A7,0x57A8,0x57A9,0x57AC,0x57B8,0x57BD,0x57C7,0x57C8,
	0x57CC,0x57CF,0x57D5,0x57DD,0x57DE,0x57E4,0x57E6,0x57E7,
	0x57E9,0x57ED,0x57F0,0x57F5,0x57F6,0x57F8,0x57FD,0x57FE,
	0x57FF,0x5803,0x5804,0x5808,0x5809,0x57E1,
	/* Range 0x3821 - 0x387E, array index: 0x04DC */
	0x580C,0x580D,0x581B,0x581E,0x581F,0x5820,0x5826,0x5827,
	0x582D,0x5832,0x5839,0x583F,0x5849,0x584C,0x584D,0x584F,
	0x5850,0x5855,0x585F,0x5861,0x5864,0x5867,0x5868,0x5878,
	0x587C,0x587F,0x5880,0x5881,0x5887,0x5888,0x5889,0x588A,
	0x588C,0x588D,0x588F,0x5890,0x5894,0x5896,0x589D,0x58A0,
	0x58A1,0x58A2,0x58A6,0x58A9,0x58B1,0x58B2,0x58C4,0x58BC,
	0x58C2,0x58C8,0x58CD,0x58CE,0x58D0,0x58D2,0x58D4,0x58D6,
	0x58DA,0x58DD,0x58E1,0x58E2,0x58E9,0x58F3,0x5905,0x5906,
	0x590B,0x590C,0x5912,0x5913,0x5914,0x8641,0x591D,0x5921,
	0x5923,0x5924,0x5928,0x592F,0x5930,0x5933,0x5935,0x5936,
	0x593F,0x5943,0x5946,0x5952,0x5953,0x5959,0x595B,0x595D,
	0x595E,0x595F,0x5961,0x5963,0x596B,0x596D,
	/* Range 0x3921 - 0x397E, array index: 0x053A */
	0x596F,0x5972,0x5975,0x5976,0x5979,0x597B,0x597C,0x598B,
	0x598C,0x598E,0x5992,0x5995,0x5997,0x599F,0x59A4,0x59A7,
	0x59AD,0x59AE,0x59AF,0x59B0,0x59B3,0x59B7,0x59BA,0x59BC,
	0x59C1,0x59C3,0x59C4,0x59C8,0x59CA,0x59CD,0x59D2,0x59DD,
	0x59DE,0x59DF,0x59E3,0x59E4,0x59E7,0x59EE,0x59EF,0x59F1,
	0x59F2,0x59F4,0x59F7,0x5A00,0x5A04,0x5A0C,0x5A0D,0x5A0E,
	0x5A12,0x5A13,0x5A1E,0x5A23,0x5A24,0x5A27,0x5A28,0x5A2A,
	0x5A2D,0x5A30,0x5A44,0x5A45,0x5A47,0x5A48,0x5A4C,0x5A50,
	0x5A55,0x5A5E,0x5A63,0x5A65,0x5A67,0x5A6D,0x5A77,0x5A7A,
	0x5A7B,0x5A7E,0x5A8B,0x5A90,0x5A93,0x5A96,0x5A99,0x5A9C,
	0x5A9E,0x5A9F,0x5AA0,0x5AA2,0x5AA7,0x5AAC,0x5AB1,0x5AB2,
	0x5AB3,0x5AB5,0x5AB8,0x5ABA,0x5ABB,0x5ABF,
	/* Range 0x3A21 - 0x3A7E, array index: 0x0598 */
	0x5AC4,0x5AC6,0x5AC8,0x5ACF,0x5ADA,0x5ADC,0x5AE0,0x5AE5,
	0x5AEA,0x5AEE,0x5AF5,0x5AF6,0x5AFD,0x5B00,0x5B01,0x5B08,
	0x5B17,0x5B34,0x5B19,0x5B1B,0x5B1D,0x5B21,0x5B25,0x5B2D,
	0x5B38,0x5B41,0x5B4B,0x5B4C,0x5B52,0x5B56,0x5B5E,0x5B68,
	0x5B6E,0x5B6F,0x5B7C,0x5B7D,0x5B7E,0x5B7F,0x5B81,0x5B84,
	0x5B86,0x5B8A,0x5B8E,0x5B90,0x5B91,0x5B93,0x5B94,0x5B96,
	0x5BA8,0x5BA9,0x5BAC,0x5BAD,0x5BAF,0x5BB1,0x5BB2,0x5BB7,
	0x5BBA,0x5BBC,0x5BC0,0x5BC1,0x5BCD,0x5BCF,0x5BD6,0x5BD7,
	0x5BD8,0x5BD9,0x5BDA,0x5BE0,0x5BEF,0x5BF1,0x5BF4,0x5BFD,
	0x5C0C,0x5C17,0x5C1E,0x5C1F,0x5C23,0x5C26,0x5C29,0x5C2B,
	0x5C2C,0x5C2E,0x5C30,0x5C32,0x5C35,0x5C36,0x5C59,0x5C5A,
	0x5C5C,0x5C62,0x5C63,0x5C67,0x5C68,0x5C69,
	/* Range 0x3B21 - 0x3B7E, array index: 0x05F6 */
	0x5C6D,0x5C70,0x5C74,0x5C75,0x5C7A,0x5C7B,0x5C7C,0x5C7D,
	0x5C87,0x5C88,0x5C8A,0x5C8F,0x5C92,0x5C9D,0x5C9F,0x5CA0,
	0x5CA2,0x5CA3,0x5CA6,0x5CAA,0x5CB2,0x5CB4,0x5CB5,0x5CBA,
	0x5CC9,0x5CCB,0x5CD2,0x5CDD,0x5CD7,0x5CEE,0x5CF1,0x5CF2,
	0x5CF4,0x5D01,0x5D06,0x5D0D,0x5D12,0x5D2B,0x5D23,0x5D24,
	0x5D26,0x5D27,0x5D31,0x5D34,0x5D39,0x5D3D,0x5D3F,0x5D42,
	0x5D43,0x5D46,0x5D48,0x5D55,0x5D51,0x5D59,0x5D4A,0x5D5F,
	0x5D60,0x5D61,0x5D62,0x5D64,0x5D6A,0x5D6D,0x5D70,0x5D79,
	0x5D7A,0x5D7E,0x5D7F,0x5D81,0x5D83,0x5D88,0x5D8A,0x5D92,
	0x5D93,0x5D94,0x5D95,0x5D99,0x5D9B,0x5D9F,0x5DA0,0x5DA7,
	0x5DAB,0x5DB0,0x5DB4,0x5DB8,0x5DB9,0x5DC3,0x5DC7,0x5DCB,
	0x5DD0,0x5DCE,0x5DD8,0x5DD9,0x5DE0,0x5DE4,
	/* Range 0x3C21 - 0x3C7E, array index: 0x0654 */
	0x5DE9,0x5DF8,0x5DF9,0x5E00,0x5E07,0x5E0D,0x5E12,0x5E14,
	0x5E15,0x5E18,0x5E1F,0x5E20,0x5E2E,0x5E28,0x5E32,0x5E35,
	0x5E3E,0x5E4B,0x5E50,0x5E49,0x5E51,0x5E56,0x5E58,0x5E5B,
	0x5E5C,0x5E5E,0x5E68,0x5E6A,0x5E6B,0x5E6C,0x5E6D,0x5E6E,
	0x5E70,0x5E80,0x5E8B,0x5E8E,0x5EA2,0x5EA4,0x5EA5,0x5EA8,
	0x5EAA,0x5EAC,0x5EB1,0x5EB3,0x5EBD,0x5EBE,0x5EBF,0x5EC6,
	0x5ECC,0x5ECB,0x5ECE,0x5ED1,0x5ED2,0x5ED4,0x5ED5,0x5EDC,
	0x5EDE,0x5EE5,0x5EEB,0x5F02,0x5F06,0x5F07,0x5F08,0x5F0E,
	0x5F19,0x5F1C,0x5F1D,0x5F21,0x5F22,0x5F23,0x5F24,0x5F28,
	0x5F2B,0x5F2C,0x5F2E,0x5F30,0x5F34,0x5F36,0x5F3B,0x5F3D,
	0x5F3F,0x5F40,0x5F44,0x5F45,0x5F47,0x5F4D,0x5F50,0x5F54,
	0x5F58,0x5F5B,0x5F60,0x5F63,0x5F64,0x5F67,
	/* Range 0x3D21 - 0x3D7E, array index: 0x06B2 */
	0x5F6F,0x5F72,0x5F74,0x5F75,0x5F78,0x5F7A,0x5F7D,0x5F7E,
	0x5F89,0x5F8D,0x5F8F,0x5F96,0x5F9C,0x5F9D,0x5FA2,0x5FA7,
	0x5FAB,0x5FA4,0x5FAC,0x5FAF,0x5FB0,0x5FB1,0x5FB8,0x5FC4,
	0x5FC7,0x5FC8,0x5FC9,0x5FCB,0x5FD0,0x5FD1,0x5FD2,0x5FD3,
	0x5FD4,0x5FDE,0x5FE1,0x5FE2,0x5FE8,0x5FE9,0x5FEA,0x5FEC,
	0x5FED,0x5FEE,0x5FEF,0x5FF2,0x5FF3,0x5FF6,0x5FFA,0x5FFC,
	0x6007,0x600A,0x600D,0x6013,0x6014,0x6017,0x6018,0x601A,
	0x601F,0x6024,0x602D,0x6033,0x6035,0x6040,0x6047,0x6048,
	0x6049,0x604C,0x6051,0x6054,0x6056,0x6057,0x605D,0x6061,
	0x6067,0x6071,0x607E,0x607F,0x6082,0x6086,0x6088,0x608A,
	0x608E,0x6091,0x6093,0x6095,0x6098,0x609D,0x609E,0x60A2,
	0x60A4,0x60A5,0x60A8,0x60B0,0x60B1,0x60B7,
	/* Range 0x3E21 - 0x3E7E, array index: 0x0710 */
	0x60BB,0x60BE,0x60C2,0x60C4,0x60C8,0x60C9,0x60CA,0x60CB,
	0x60CE,0x60CF,0x60D4,0x60D5,0x60D9,0x60DB,0x60DD,0x60DE,
	0x60E2,0x60E5,0x60F2,0x60F5,0x60F8,0x60FC,0x60FD,0x6102,
	0x6107,0x610A,0x610C,0x6110,0x6111,0x6112,0x6113,0x6114,
	0x6116,0x6117,0x6119,0x611C,0x611E,0x6122,0x612A,0x612B,
	0x6130,0x6131,0x6135,0x6136,0x6137,0x6139,0x6141,0x6145,
	0x6146,0x6149,0x615E,0x6160,0x616C,0x6172,0x6178,0x617B,
	0x617C,0x617F,0x6180,0x6181,0x6183,0x6184,0x618B,0x618D,
	0x6192,0x6193,0x6197,0x6198,0x619C,0x619D,0x619F,0x61A0,
	0x61A5,0x61A8,0x61AA,0x61AD,0x61B8,0x61B9,0x61BC,0x61C0,
	0x61C1,0x61C2,0x61CE,0x61CF,0x61D5,0x61DC,0x61DD,0x61DE,
	0x61DF,0x61E1,0x61E2,0x61E7,0x61E9,0x61E5,
	/* Range 0x3F21 - 0x3F7E, array index: 0x076E */
	0x61EC,0x61ED,0x61EF,0x6201,0x6203,0x6204,0x6207,0x6213,
	0x6215,0x621C,0x6220,0x6222,0x6223,0x6227,0x6229,0x622B,
	0x6239,0x623D,0x6242,0x6243,0x6244,0x6246,0x624C,0x6250,
	0x6251,0x6252,0x6254,0x6256,0x625A,0x625C,0x6264,0x626D,
	0x626F,0x6273,0x627A,0x627D,0x628D,0x628E,0x628F,0x6290,
	0x62A6,0x62A8,0x62B3,0x62B6,0x62B7,0x62BA,0x62BE,0x62BF,
	0x62C4,0x62CE,0x62D5,0x62D6,0x62DA,0x62EA,0x62F2,0x62F4,
	0x62FC,0x62FD,0x6303,0x6304,0x630A,0x630B,0x630D,0x6310,
	0x6313,0x6316,0x6318,0x6329,0x632A,0x632D,0x6335,0x6336,
	0x6339,0x633C,0x6341,0x6342,0x6343,0x6344,0x6346,0x634A,
	0x634B,0x634E,0x6352,0x6353,0x6354,0x6358,0x635B,0x6365,
	0x6366,0x636C,0x636D,0x6371,0x6374,0x6375,
	/* Range 0x4021 - 0x407E, array index: 0x07CC */
	0x6378,0x637C,0x637D,0x637F,0x6382,0x6384,0x6387,0x638A,
	0x6390,0x6394,0x6395,0x6399,0x639A,0x639E,0x63A4,0x63A6,
	0x63AD,0x63AE,0x63AF,0x63BD,0x63C1,0x63C5,0x63C8,0x63CE,
	0x63D1,0x63D3,0x63D4,0x63D5,0x63DC,0x63E0,0x63E5,0x63EA,
	0x63EC,0x63F2,0x63F3,0x63F5,0x63F8,0x63F9,0x6409,0x640A,
	0x6410,0x6412,0x6414,0x6418,0x641E,0x6420,0x6422,0x6424,
	0x6425,0x6429,0x642A,0x642F,0x6430,0x6435,0x643D,0x643F,
	0x644B,0x644F,0x6451,0x6452,0x6453,0x6454,0x645A,0x645B,
	0x645C,0x645D,0x645F,0x6460,0x6461,0x6463,0x646D,0x6473,
	0x6474,0x647B,0x647D,0x6485,0x6487,0x648F,0x6490,0x6491,
	0x6498,0x6499,0x649B,0x649D,0x649F,0x64A1,0x64A3,0x64A6,
	0x64A8,0x64AC,0x64B3,0x64BD,0x64BE,0x64BF,
	/* Range 0x4121 - 0x417E, array index: 0x082A */
	0x64C4,0x64C9,0x64CA,0x64CB,0x64CC,0x64CE,0x64D0,0x64D1,
	0x64D5,0x64D7,0x64E4,0x64E5,0x64E9,0x64EA,0x64ED,0x64F0,
	0x64F5,0x64F7,0x64FB,0x64FF,0x6501,0x6504,0x6508,0x6509,
	0x650A,0x650F,0x6513,0x6514,0x6516,0x6519,0x651B,0x651E,
	0x651F,0x6522,0x6526,0x6529,0x652E,0x6531,0x653A,0x653C,
	0x653D,0x6543,0x6547,0x6549,0x6550,0x6552,0x6554,0x655F,
	0x6560,0x6567,0x656B,0x657A,0x657D,0x6581,0x6585,0x658A,
	0x6592,0x6595,0x6598,0x659D,0x65A0,0x65A3,0x65A6,0x65AE,
	0x65B2,0x65B3,0x65B4,0x65BF,0x65C2,0x65C8,0x65C9,0x65CE,
	0x65D0,0x65D4,0x65D6,0x65D8,0x65DF,0x65F0,0x65F2,0x65F4,
	0x65F5,0x65F9,0x65FE,0x65FF,0x6600,0x6604,0x6608,0x6609,
	0x660D,0x6611,0x6612,0x6615,0x6616,0x661D,
	/* Range 0x4221 - 0x427E, array index: 0x0888 */
	0x661E,0x6621,0x6622,0x6623,0x6624,0x6626,0x6629,0x662A,
	0x662B,0x662C,0x662E,0x6630,0x6631,0x6633,0x6639,0x6637,
	0x6640,0x6645,0x6646,0x664A,0x664C,0x6651,0x664E,0x6657,
	0x6658,0x6659,0x665B,0x665C,0x6660,0x6661,0x66FB,0x666A,
	0x666B,0x666C,0x667E,0x6673,0x6675,0x667F,0x6677,0x6678,
	0x6679,0x667B,0x6680,0x667C,0x668B,0x668C,0x668D,0x6690,
	0x6692,0x6699,0x669A,0x669B,0x669C,0x669F,0x66A0,0x66A4,
	0x66AD,0x66B1,0x66B2,0x66B5,0x66BB,0x66BF,0x66C0,0x66C2,
	0x66C3,0x66C8,0x66CC,0x66CE,0x66CF,0x66D4,0x66DB,0x66DF,
	0x66E8,0x66EB,0x66EC,0x66EE,0x66FA,0x6705,0x6707,0x670E,
	0x6713,0x6719,0x671C,0x6720,0x6722,0x6733,0x673E,0x6745,
	0x6747,0x6748,0x674C,0x6754,0x6755,0x675D,
	/* Range 0x4321 - 0x437E, array index: 0x08E6 */
	0x6766,0x676C,0x676E,0x6774,0x6776,0x677B,0x6781,0x6784,
	0x678E,0x678F,0x6791,0x6793,0x6796,0x6798,0x6799,0x679B,
	0x67B0,0x67B1,0x67B2,0x67B5,0x67BB,0x67BC,0x67BD,0x67F9,
	0x67C0,0x67C2,0x67C3,0x67C5,0x67C8,0x67C9,0x67D2,0x67D7,
	0x67D9,0x67DC,0x67E1,0x67E6,0x67F0,0x67F2,0x67F6,0x67F7,
	0x6852,0x6814,0x6819,0x681D,0x681F,0x6828,0x6827,0x682C,
	0x682D,0x682F,0x6830,0x6831,0x6833,0x683B,0x683F,0x6844,
	0x6845,0x684A,0x684C,0x6855,0x6857,0x6858,0x685B,0x686B,
	0x686E,0x686F,0x6870,0x6871,0x6872,0x6875,0x6879,0x687A,
	0x687B,0x687C,0x6882,0x6884,0x6886,0x6888,0x6896,0x6898,
	0x689A,0x689C,0x68A1,0x68A3,0x68A5,0x68A9,0x68AA,0x68AE,
	0x68B2,0x68BB,0x68C5,0x68C8,0x68CC,0x68CF,
	/* Range 0x4421 - 0x447E, array index: 0x0944 */
	0x68D0,0x68D1,0x68D3,0x68D6,0x68D9,0x68DC,0x68DD,0x68E5,
	0x68E8,0x68EA,0x68EB,0x68EC,0x68ED,0x68F0,0x68F1,0x68F5,
	0x68F6,0x68FB,0x68FC,0x68FD,0x6906,0x6909,0x690A,0x6910,
	0x6911,0x6913,0x6916,0x6917,0x6931,0x6933,0x6935,0x6938,
	0x693B,0x6942,0x6945,0x6949,0x694E,0x6957,0x695B,0x6963,
	0x6964,0x6965,0x6966,0x6968,0x6969,0x696C,0x6970,0x6971,
	0x6972,0x697A,0x697B,0x697F,0x6980,0x698D,0x6992,0x6996,
	0x6998,0x69A1,0x69A5,0x69A6,0x69A8,0x69AB,0x69AD,0x69AF,
	0x69B7,0x69B8,0x69BA,0x69BC,0x69C5,0x69C8,0x69D1,0x69D6,
	0x69D7,0x69E2,0x69E5,0x69EE,0x69EF,0x69F1,0x69F3,0x69F5,
	0x69FE,0x6A00,0x6A01,0x6A03,0x6A0F,0x6A11,0x6A15,0x6A1A,
	0x6A1D,0x6A20,0x6A24,0x6A28,0x6A30,0x6A32,
	/* Range 0x4521 - 0x457E, array index: 0x09A2 */
	0x6A34,0x6A37,0x6A3B,0x6A3E,0x6A3F,0x6A45,0x6A46,0x6A49,
	0x6A4A,0x6A4E,0x6A50,0x6A51,0x6A52,0x6A55,0x6A56,0x6A5B,
	0x6A64,0x6A67,0x6A6A,0x6A71,0x6A73,0x6A7E,0x6A81,0x6A83,
	0x6A86,0x6A87,0x6A89,0x6A8B,0x6A91,0x6A9B,0x6A9D,0x6A9E,
	0x6A9F,0x6AA5,0x6AAB,0x6AAF,0x6AB0,0x6AB1,0x6AB4,0x6ABD,
	0x6ABE,0x6ABF,0x6AC6,0x6AC9,0x6AC8,0x6ACC,0x6AD0,0x6AD4,
	0x6AD5,0x6AD6,0x6ADC,0x6ADD,0x6AE4,0x6AE7,0x6AEC,0x6AF0,
	0x6AF1,0x6AF2,0x6AFC,0x6AFD,0x6B02,0x6B03,0x6B06,0x6B07,
	0x6B09,0x6B0F,0x6B10,0x6B11,0x6B17,0x6B1B,0x6B1E,0x6B24,
	0x6B28,0x6B2B,0x6B2C,0x6B2F,0x6B35,0x6B36,0x6B3B,0x6B3F,
	0x6B46,0x6B4A,0x6B4D,0x6B52,0x6B56,0x6B58,0x6B5D,0x6B60,
	0x6B67,0x6B6B,0x6B6E,0x6B70,0x6B75,0x6B7D,
	/* Range 0x4621 - 0x467E, array index: 0x0A00 */
	0x6B7E,0x6B82,0x6B85,0x6B97,0x6B9B,0x6B9F,0x6BA0,0x6BA2,
	0x6BA3,0x6BA8,0x6BA9,0x6BAC,0x6BAD,0x6BAE,0x6BB0,0x6BB8,
	0x6BB9,0x6BBD,0x6BBE,0x6BC3,0x6BC4,0x6BC9,0x6BCC,0x6BD6,
	0x6BDA,0x6BE1,0x6BE3,0x6BE6,0x6BE7,0x6BEE,0x6BF1,0x6BF7,
	0x6BF9,0x6BFF,0x6C02,0x6C04,0x6C05,0x6C09,0x6C0D,0x6C0E,
	0x6C10,0x6C12,0x6C19,0x6C1F,0x6C26,0x6C27,0x6C28,0x6C2C,
	0x6C2E,0x6C33,0x6C35,0x6C36,0x6C3A,0x6C3B,0x6C3F,0x6C4A,
	0x6C4B,0x6C4D,0x6C4F,0x6C52,0x6C54,0x6C59,0x6C5B,0x6C5C,
	0x6C6B,0x6C6D,0x6C6F,0x6C74,0x6C76,0x6C78,0x6C79,0x6C7B,
	0x6C85,0x6C86,0x6C87,0x6C89,0x6C94,0x6C95,0x6C97,0x6C98,
	0x6C9C,0x6C9F,0x6CB0,0x6CB2,0x6CB4,0x6CC2,0x6CC6,0x6CCD,
	0x6CCF,0x6CD0,0x6CD1,0x6CD2,0x6CD4,0x6CD6,
	/* Range 0x4721 - 0x477E, array index: 0x0A5E */
	0x6CDA,0x6CDC,0x6CE0,0x6CE7,0x6CE9,0x6CEB,0x6CEC,0x6CEE,
	0x6CF2,0x6CF4,0x6D04,0x6D07,0x6D0A,0x6D0E,0x6D0F,0x6D11,
	0x6D13,0x6D1A,0x6D26,0x6D27,0x6D28,0x6C67,0x6D2E,0x6D2F,
	0x6D31,0x6D39,0x6D3C,0x6D3F,0x6D57,0x6D5E,0x6D5F,0x6D61,
	0x6D65,0x6D67,0x6D6F,0x6D70,0x6D7C,0x6D82,0x6D87,0x6D91,
	0x6D92,0x6D94,0x6D96,0x6D97,0x6D98,0x6DAA,0x6DAC,0x6DB4,
	0x6DB7,0x6DB9,0x6DBD,0x6DBF,0x6DC4,0x6DC8,0x6DCA,0x6DCE,
	0x6DCF,0x6DD6,0x6DDB,0x6DDD,0x6DDF,0x6DE0,0x6DE2,0x6DE5,
	0x6DE9,0x6DEF,0x6DF0,0x6DF4,0x6DF6,0x6DFC,0x6E00,0x6E04,
	0x6E1E,0x6E22,0x6E27,0x6E32,0x6E36,0x6E39,0x6E3B,0x6E3C,
	0x6E44,0x6E45,0x6E48,0x6E49,0x6E4B,0x6E4F,0x6E51,0x6E52,
	0x6E53,0x6E54,0x6E57,0x6E5C,0x6E5D,0x6E5E,
	/* Range 0x4821 - 0x487E, array index: 0x0ABC */
	0x6E62,0x6E63,0x6E68,0x6E73,0x6E7B,0x6E7D,0x6E8D,0x6E93,
	0x6E99,0x6EA0,0x6EA7,0x6EAD,0x6EAE,0x6EB1,0x6EB3,0x6EBB,
	0x6EBF,0x6EC0,0x6EC1,0x6EC3,0x6EC7,0x6EC8,0x6ECA,0x6ECD,
	0x6ECE,0x6ECF,0x6EEB,0x6EED,0x6EEE,0x6EF9,0x6EFB,0x6EFD,
	0x6F04,0x6F08,0x6F0A,0x6F0C,0x6F0D,0x6F16,0x6F18,0x6F1A,
	0x6F1B,0x6F26,0x6F29,0x6F2A,0x6F2F,0x6F30,0x6F33,0x6F36,
	0x6F3B,0x6F3C,0x6F2D,0x6F4F,0x6F51,0x6F52,0x6F53,0x6F57,
	0x6F59,0x6F5A,0x6F5D,0x6F5E,0x6F61,0x6F62,0x6F68,0x6F6C,
	0x6F7D,0x6F7E,0x6F83,0x6F87,0x6F88,0x6F8B,0x6F8C,0x6F8D,
	0x6F90,0x6F92,0x6F93,0x6F94,0x6F96,0x6F9A,0x6F9F,0x6FA0,
	0x6FA5,0x6FA6,0x6FA7,0x6FA8,0x6FAE,0x6FAF,0x6FB0,0x6FB5,
	0x6FB6,0x6FBC,0x6FC5,0x6FC7,0x6FC8,0x6FCA,
	/* Range 0x4921 - 0x497E, array index: 0x0B1A */
	0x6FDA,0x6FDE,0x6FE8,0x6FE9,0x6FF0,0x6FF5,0x6FF9,0x6FFC,
	0x6FFD,0x7000,0x7005,0x7006,0x7007,0x700D,0x7017,0x7020,
	0x7023,0x702F,0x7034,0x7037,0x7039,0x703C,0x7043,0x7044,
	0x7048,0x7049,0x704A,0x704B,0x7054,0x7055,0x705D,0x705E,
	0x704E,0x7064,0x7065,0x706C,0x706E,0x7075,0x7076,0x707E,
	0x7081,0x7085,0x7086,0x7094,0x7095,0x7096,0x7097,0x7098,
	0x709B,0x70A4,0x70AB,0x70B0,0x70B1,0x70B4,0x70B7,0x70CA,
	0x70D1,0x70D3,0x70D4,0x70D5,0x70D6,0x70D8,0x70DC,0x70E4,
	0x70FA,0x7103,0x7104,0x7105,0x7106,0x7107,0x710B,0x710C,
	0x710F,0x711E,0x7120,0x712B,0x712D,0x712F,0x7130,0x7131,
	0x7138,0x7141,0x7145,0x7146,0x7147,0x714A,0x714B,0x7150,
	0x7152,0x7157,0x715A,0x715C,0x715E,0x7160,
	/* Range 0x4A21 - 0x4A7E, array index: 0x0B78 */
	0x7168,0x7179,0x7180,0x7185,0x7187,0x718C,0x7192,0x719A,
	0x719B,0x71A0,0x71A2,0x71AF,0x71B0,0x71B2,0x71B3,0x71BA,
	0x71BF,0x71C0,0x71C1,0x71C4,0x71CB,0x71CC,0x71D3,0x71D6,
	0x71D9,0x71DA,0x71DC,0x71F8,0x71FE,0x7200,0x7207,0x7208,
	0x7209,0x7213,0x7217,0x721A,0x721D,0x721F,0x7224,0x722B,
	0x722F,0x7234,0x7238,0x7239,0x7241,0x7242,0x7243,0x7245,
	0x724E,0x724F,0x7250,0x7253,0x7255,0x7256,0x725A,0x725C,
	0x725E,0x7260,0x7263,0x7268,0x726B,0x726E,0x726F,0x7271,
	0x7277,0x7278,0x727B,0x727C,0x727F,0x7284,0x7289,0x728D,
	0x728E,0x7293,0x729B,0x72A8,0x72AD,0x72AE,0x72B1,0x72B4,
	0x72BE,0x72C1,0x72C7,0x72C9,0x72CC,0x72D5,0x72D6,0x72D8,
	0x72DF,0x72E5,0x72F3,0x72F4,0x72FA,0x72FB,
	/* Range 0x4B21 - 0x4B7E, array index: 0x0BD6 */
	0x72FE,0x7302,0x7304,0x7305,0x7307,0x730B,0x730D,0x7312,
	0x7313,0x7318,0x7319,0x731E,0x7322,0x7324,0x7327,0x7328,
	0x732C,0x7331,0x7332,0x7335,0x733A,0x733B,0x733D,0x7343,
	0x734D,0x7350,0x7352,0x7356,0x7358,0x735D,0x735E,0x735F,
	0x7360,0x7366,0x7367,0x7369,0x736B,0x736C,0x736E,0x736F,
	0x7371,0x7377,0x7379,0x737C,0x7380,0x7381,0x7383,0x7385,
	0x7386,0x738E,0x7390,0x7393,0x7395,0x7397,0x7398,0x739C,
	0x739E,0x739F,0x73A0,0x73A2,0x73A5,0x73A6,0x73AA,0x73AB,
	0x73AD,0x73B5,0x73B7,0x73B9,0x73BC,0x73BD,0x73BF,0x73C5,
	0x73C6,0x73C9,0x73CB,0x73CC,0x73CF,0x73D2,0x73D3,0x73D6,
	0x73D9,0x73DD,0x73E1,0x73E3,0x73E6,0x73E7,0x73E9,0x73F4,
	0x73F5,0x73F7,0x73F9,0x73FA,0x73FB,0x73FD,
	/* Range 0x4C21 - 0x4C7E, array index: 0x0C34 */
	0x73FF,0x7400,0x7401,0x7404,0x7407,0x740A,0x7411,0x741A,
	0x741B,0x7424,0x7426,0x7428,0x7429,0x742A,0x742B,0x742C,
	0x742D,0x742E,0x742F,0x7430,0x7431,0x7439,0x7440,0x7443,
	0x7444,0x7446,0x7447,0x744B,0x744D,0x7451,0x7452,0x7457,
	0x745D,0x7462,0x7466,0x7467,0x7468,0x746B,0x746D,0x746E,
	0x7471,0x7472,0x7480,0x7481,0x7485,0x7486,0x7487,0x7489,
	0x748F,0x7490,0x7491,0x7492,0x7498,0x7499,0x749A,0x749C,
	0x749F,0x74A0,0x74A1,0x74A3,0x74A6,0x74A8,0x74A9,0x74AA,
	0x74AB,0x74AE,0x74AF,0x74B1,0x74B2,0x74B5,0x74B9,0x74BB,
	0x74BF,0x74C8,0x74C9,0x74CC,0x74D0,0x74D3,0x74D8,0x74DA,
	0x74DB,0x74DE,0x74DF,0x74E4,0x74E8,0x74EA,0x74EB,0x74EF,
	0x74F4,0x74FA,0x74FB,0x74FC,0x74FF,0x7506,
	/* Range 0x4D21 - 0x4D7E, array index: 0x0C92 */
	0x7512,0x7516,0x7517,0x7520,0x7521,0x7524,0x7527,0x7529,
	0x752A,0x752F,0x7536,0x7539,0x753D,0x753E,0x753F,0x7540,
	0x7543,0x7547,0x7548,0x754E,0x7550,0x7552,0x7557,0x755E,
	0x755F,0x7561,0x756F,0x7571,0x7579,0x757A,0x757B,0x757C,
	0x757D,0x757E,0x7581,0x7585,0x7590,0x7592,0x7593,0x7595,
	0x7599,0x759C,0x75A2,0x75A4,0x75B4,0x75BA,0x75BF,0x75C0,
	0x75C1,0x75C4,0x75C6,0x75CC,0x75CE,0x75CF,0x75D7,0x75DC,
	0x75DF,0x75E0,0x75E1,0x75E4,0x75E7,0x75EC,0x75EE,0x75EF,
	0x75F1,0x75F9,0x7600,0x7602,0x7603,0x7604,0x7607,0x7608,
	0x760A,0x760C,0x760F,0x7612,0x7613,0x7615,0x7616,0x7619,
	0x761B,0x761C,0x761D,0x761E,0x7623,0x7625,0x7626,0x7629,
	0x762D,0x7632,0x7633,0x7635,0x7638,0x7639,
	/* Range 0x4E21 - 0x4E7E, array index: 0x0CF0 */
	0x763A,0x763C,0x764A,0x7640,0x7641,0x7643,0x7644,0x7645,
	0x7649,0x764B,0x7655,0x7659,0x765F,0x7664,0x7665,0x766D,
	0x766E,0x766F,0x7671,0x7674,0x7681,0x7685,0x768C,0x768D,
	0x7695,0x769B,0x769C,0x769D,0x769F,0x76A0,0x76A2,0x76A3,
	0x76A4,0x76A5,0x76A6,0x76A7,0x76A8,0x76AA,0x76AD,0x76BD,
	0x76C1,0x76C5,0x76C9,0x76CB,0x76CC,0x76CE,0x76D4,0x76D9,
	0x76E0,0x76E6,0x76E8,0x76EC,0x76F0,0x76F1,0x76F6,0x76F9,
	0x76FC,0x7700,0x7706,0x770A,0x770E,0x7712,0x7714,0x7715,
	0x7717,0x7719,0x771A,0x771C,0x7722,0x7728,0x772D,0x772E,
	0x772F,0x7734,0x7735,0x7736,0x7739,0x773D,0x773E,0x7742,
	0x7745,0x7746,0x774A,0x774D,0x774E,0x774F,0x7752,0x7756,
	0x7757,0x775C,0x775E,0x775F,0x7760,0x7762,
	/* Range 0x4F21 - 0x4F7E, array index: 0x0D4E */
	0x7764,0x7767,0x776A,0x776C,0x7770,0x7772,0x7773,0x7774,
	0x777A,0x777D,0x7780,0x7784,0x778C,0x778D,0x7794,0x7795,
	0x7796,0x779A,0x779F,0x77A2,0x77A7,0x77AA,0x77AE,0x77AF,
	0x77B1,0x77B5,0x77BE,0x77C3,0x77C9,0x77D1,0x77D2,0x77D5,
	0x77D9,0x77DE,0x77DF,0x77E0,0x77E4,0x77E6,0x77EA,0x77EC,
	0x77F0,0x77F1,0x77F4,0x77F8,0x77FB,0x7805,0x7806,0x7809,
	0x780D,0x780E,0x7811,0x781D,0x7821,0x7822,0x7823,0x782D,
	0x782E,0x7830,0x7835,0x7837,0x7843,0x7844,0x7847,0x7848,
	0x784C,0x784E,0x7852,0x785C,0x785E,0x7860,0x7861,0x7863,
	0x7864,0x7868,0x786A,0x786E,0x787A,0x787E,0x788A,0x788F,
	0x7894,0x7898,0x78A1,0x789D,0x789E,0x789F,0x78A4,0x78A8,
	0x78AC,0x78AD,0x78B0,0x78B1,0x78B2,0x78B3,
	/* Range 0x5021 - 0x507E, array index: 0x0DAC */
	0x78BB,0x78BD,0x78BF,0x78C7,0x78C8,0x78C9,0x78CC,0x78CE,
	0x78D2,0x78D3,0x78D5,0x78D6,0x78E4,0x78DB,0x78DF,0x78E0,
	0x78E1,0x78E6,0x78EA,0x78F2,0x78F3,0x7900,0x78F6,0x78F7,
	0x78FA,0x78FB,0x78FF,0x7906,0x790C,0x7910,0x791A,0x791C,
	0x791E,0x791F,0x7920,0x7925,0x7927,0x7929,0x792D,0x7931,
	0x7934,0x7935,0x793B,0x793D,0x793F,0x7944,0x7945,0x7946,
	0x794A,0x794B,0x794F,0x7951,0x7954,0x7958,0x795B,0x795C,
	0x7967,0x7969,0x796B,0x7972,0x7979,0x797B,0x797C,0x797E,
	0x798B,0x798C,0x7991,0x7993,0x7994,0x7995,0x7996,0x7998,
	0x799B,0x799C,0x79A1,0x79A8,0x79A9,0x79AB,0x79AF,0x79B1,
	0x79B4,0x79B8,0x79BB,0x79C2,0x79C4,0x79C7,0x79C8,0x79CA,
	0x79CF,0x79D4,0x79D6,0x79DA,0x79DD,0x79DE,
	/* Range 0x5121 - 0x517E, array index: 0x0E0A */
	0x79E0,0x79E2,0x79E5,0x79EA,0x79EB,0x79ED,0x79F1,0x79F8,
	0x79FC,0x7A02,0x7A03,0x7A07,0x7A09,0x7A0A,0x7A0C,0x7A11,
	0x7A15,0x7A1B,0x7A1E,0x7A21,0x7A27,0x7A2B,0x7A2D,0x7A2F,
	0x7A30,0x7A34,0x7A35,0x7A38,0x7A39,0x7A3A,0x7A44,0x7A45,
	0x7A47,0x7A48,0x7A4C,0x7A55,0x7A56,0x7A59,0x7A5C,0x7A5D,
	0x7A5F,0x7A60,0x7A65,0x7A67,0x7A6A,0x7A6D,0x7A75,0x7A78,
	0x7A7E,0x7A80,0x7A82,0x7A85,0x7A86,0x7A8A,0x7A8B,0x7A90,
	0x7A91,0x7A94,0x7A9E,0x7AA0,0x7AA3,0x7AAC,0x7AB3,0x7AB5,
	0x7AB9,0x7ABB,0x7ABC,0x7AC6,0x7AC9,0x7ACC,0x7ACE,0x7AD1,
	0x7ADB,0x7AE8,0x7AE9,0x7AEB,0x7AEC,0x7AF1,0x7AF4,0x7AFB,
	0x7AFD,0x7AFE,0x7B07,0x7B14,0x7B1F,0x7B23,0x7B27,0x7B29,
	0x7B2A,0x7B2B,0x7B2D,0x7B2E,0x7B2F,0x7B30,
	/* Range 0x5221 - 0x527E, array index: 0x0E68 */
	0x7B31,0x7B34,0x7B3D,0x7B3F,0x7B40,0x7B41,0x7B47,0x7B4E,
	0x7B55,0x7B60,0x7B64,0x7B66,0x7B69,0x7B6A,0x7B6D,0x7B6F,
	0x7B72,0x7B73,0x7B77,0x7B84,0x7B89,0x7B8E,0x7B90,0x7B91,
	0x7B96,0x7B9B,0x7B9E,0x7BA0,0x7BA5,0x7BAC,0x7BAF,0x7BB0,
	0x7BB2,0x7BB5,0x7BB6,0x7BBA,0x7BBB,0x7BBC,0x7BBD,0x7BC2,
	0x7BC5,0x7BC8,0x7BCA,0x7BD4,0x7BD6,0x7BD7,0x7BD9,0x7BDA,
	0x7BDB,0x7BE8,0x7BEA,0x7BF2,0x7BF4,0x7BF5,0x7BF8,0x7BF9,
	0x7BFA,0x7BFC,0x7BFE,0x7C01,0x7C02,0x7C03,0x7C04,0x7C06,
	0x7C09,0x7C0B,0x7C0C,0x7C0E,0x7C0F,0x7C19,0x7C1B,0x7C20,
	0x7C25,0x7C26,0x7C28,0x7C2C,0x7C31,0x7C33,0x7C34,0x7C36,
	0x7C39,0x7C3A,0x7C46,0x7C4A,0x7C55,0x7C51,0x7C52,0x7C53,
	0x7C59,0x7C5A,0x7C5B,0x7C5C,0x7C5D,0x7C5E,
	/* Range 0x5321 - 0x537E, array index: 0x0EC6 */
	0x7C61,0x7C63,0x7C67,0x7C69,0x7C6D,0x7C6E,0x7C70,0x7C72,
	0x7C79,0x7C7C,0x7C7D,0x7C86,0x7C87,0x7C8F,0x7C94,0x7C9E,
	0x7CA0,0x7CA6,0x7CB0,0x7CB6,0x7CB7,0x7CBA,0x7CBB,0x7CBC,
	0x7CBF,0x7CC4,0x7CC7,0x7CC8,0x7CC9,0x7CCD,0x7CCF,0x7CD3,
	0x7CD4,0x7CD5,0x7CD7,0x7CD9,0x7CDA,0x7CDD,0x7CE6,0x7CE9,
	0x7CEB,0x7CF5,0x7D03,0x7D07,0x7D08,0x7D09,0x7D0F,0x7D11,
	0x7D12,0x7D13,0x7D16,0x7D1D,0x7D1E,0x7D23,0x7D26,0x7D2A,
	0x7D2D,0x7D31,0x7D3C,0x7D3D,0x7D3E,0x7D40,0x7D41,0x7D47,
	0x7D48,0x7D4D,0x7D51,0x7D53,0x7D57,0x7D59,0x7D5A,0x7D5C,
	0x7D5D,0x7D65,0x7D67,0x7D6A,0x7D70,0x7D78,0x7D7A,0x7D7B,
	0x7D7F,0x7D81,0x7D82,0x7D83,0x7D85,0x7D86,0x7D88,0x7D8B,
	0x7D8C,0x7D8D,0x7D91,0x7D96,0x7D97,0x7D9D,
	/* Range 0x5421 - 0x547E, array index: 0x0F24 */
	0x7D9E,0x7DA6,0x7DA7,0x7DAA,0x7DB3,0x7DB6,0x7DB7,0x7DB9,
	0x7DC2,0x7DC3,0x7DC4,0x7DC5,0x7DC6,0x7DCC,0x7DCD,0x7DCE,
	0x7DD7,0x7DD9,0x7E00,0x7DE2,0x7DE5,0x7DE6,0x7DEA,0x7DEB,
	0x7DED,0x7DF1,0x7DF5,0x7DF6,0x7DF9,0x7DFA,0x7E08,0x7E10,
	0x7E11,0x7E15,0x7E17,0x7E1C,0x7E1D,0x7E20,0x7E27,0x7E28,
	0x7E2C,0x7E2D,0x7E2F,0x7E33,0x7E36,0x7E3F,0x7E44,0x7E45,
	0x7E47,0x7E4E,0x7E50,0x7E52,0x7E58,0x7E5F,0x7E61,0x7E62,
	0x7E65,0x7E6B,0x7E6E,0x7E6F,0x7E73,0x7E78,0x7E7E,0x7E81,
	0x7E86,0x7E87,0x7E8A,0x7E8D,0x7E91,0x7E95,0x7E98,0x7E9A,
	0x7E9D,0x7E9E,0x7F3C,0x7F3B,0x7F3D,0x7F3E,0x7F3F,0x7F43,
	0x7F44,0x7F47,0x7F4F,0x7F52,0x7F53,0x7F5B,0x7F5C,0x7F5D,
	0x7F61,0x7F63,0x7F64,0x7F65,0x7F66,0x7F6D,
	/* Range 0x5521 - 0x557E, array index: 0x0F82 */
	0x7F71,0x7F7D,0x7F7E,0x7F7F,0x7F80,0x7F8B,0x7F8D,0x7F8F,
	0x7F90,0x7F91,0x7F96,0x7F97,0x7F9C,0x7FA1,0x7FA2,0x7FA6,
	0x7FAA,0x7FAD,0x7FB4,0x7FBC,0x7FBF,0x7FC0,0x7FC3,0x7FC8,
	0x7FCE,0x7FCF,0x7FDB,0x7FDF,0x7FE3,0x7FE5,0x7FE8,0x7FEC,
	0x7FEE,0x7FEF,0x7FF2,0x7FFA,0x7FFD,0x7FFE,0x7FFF,0x8007,
	0x8008,0x800A,0x800D,0x800E,0x800F,0x8011,0x8013,0x8014,
	0x8016,0x801D,0x801E,0x801F,0x8020,0x8024,0x8026,0x802C,
	0x802E,0x8030,0x8034,0x8035,0x8037,0x8039,0x803A,0x803C,
	0x803E,0x8040,0x8044,0x8060,0x8064,0x8066,0x806D,0x8071,
	0x8075,0x8081,0x8088,0x808E,0x809C,0x809E,0x80A6,0x80A7,
	0x80AB,0x80B8,0x80B9,0x80C8,0x80CD,0x80CF,0x80D2,0x80D4,
	0x80D5,0x80D7,0x80D8,0x80E0,0x80ED,0x80EE,
	/* Range 0x5621 - 0x567E, array index: 0x0FE0 */
	0x80F0,0x80F2,0x80F3,0x80F6,0x80F9,0x80FA,0x80FE,0x8103,
	0x810B,0x8116,0x8117,0x8118,0x811C,0x811E,0x8120,0x8124,
	0x8127,0x812C,0x8130,0x8135,0x813A,0x813C,0x8145,0x8147,
	0x814A,0x814C,0x8152,0x8157,0x8160,0x8161,0x8167,0x8168,
	0x8169,0x816D,0x816F,0x8177,0x8181,0x8190,0x8184,0x8185,
	0x8186,0x818B,0x818E,0x8196,0x8198,0x819B,0x819E,0x81A2,
	0x81AE,0x81B2,0x81B4,0x81BB,0x81CB,0x81C3,0x81C5,0x81CA,
	0x81CE,0x81CF,0x81D5,0x81D7,0x81DB,0x81DD,0x81DE,0x81E1,
	0x81E4,0x81EB,0x81EC,0x81F0,0x81F1,0x81F2,0x81F5,0x81F6,
	0x81F8,0x81F9,0x81FD,0x81FF,0x8200,0x8203,0x820F,0x8213,
	0x8214,0x8219,0x821A,0x821D,0x8221,0x8222,0x8228,0x8232,
	0x8234,0x823A,0x8243,0x8244,0x8245,0x8246,
	/* Range 0x5721 - 0x577E, array index: 0x103E */
	0x824B,0x824E,0x824F,0x8251,0x8256,0x825C,0x8260,0x8263,
	0x8267,0x826D,0x8274,0x827B,0x827D,0x827F,0x8280,0x8281,
	0x8283,0x8284,0x8287,0x8289,0x828A,0x828E,0x8291,0x8294,
	0x8296,0x8298,0x829A,0x829B,0x82A0,0x82A1,0x82A3,0x82A4,
	0x82A7,0x82A8,0x82A9,0x82AA,0x82AE,0x82B0,0x82B2,0x82B4,
	0x82B7,0x82BA,0x82BC,0x82BE,0x82BF,0x82C6,0x82D0,0x82D5,
	0x82DA,0x82E0,0x82E2,0x82E4,0x82E8,0x82EA,0x82ED,0x82EF,
	0x82F6,0x82F7,0x82FD,0x82FE,0x8300,0x8301,0x8307,0x8308,
	0x830A,0x830B,0x8354,0x831B,0x831D,0x831E,0x831F,0x8321,
	0x8322,0x832C,0x832D,0x832E,0x8330,0x8333,0x8337,0x833A,
	0x833C,0x833D,0x8342,0x8343,0x8344,0x8347,0x834D,0x834E,
	0x8351,0x8355,0x8356,0x8357,0x8370,0x8378,
	/* Range 0x5821 - 0x587E, array index: 0x109C */
	0x837D,0x837F,0x8380,0x8382,0x8384,0x8386,0x838D,0x8392,
	0x8394,0x8395,0x8398,0x8399,0x839B,0x839C,0x839D,0x83A6,
	0x83A7,0x83A9,0x83AC,0x83BE,0x83BF,0x83C0,0x83C7,0x83C9,
	0x83CF,0x83D0,0x83D1,0x83D4,0x83DD,0x8353,0x83E8,0x83EA,
	0x83F6,0x83F8,0x83F9,0x83FC,0x8401,0x8406,0x840A,0x840F,
	0x8411,0x8415,0x8419,0x83AD,0x842F,0x8439,0x8445,0x8447,
	0x8448,0x844A,0x844D,0x844F,0x8451,0x8452,0x8456,0x8458,
	0x8459,0x845A,0x845C,0x8460,0x8464,0x8465,0x8467,0x846A,
	0x8470,0x8473,0x8474,0x8476,0x8478,0x847C,0x847D,0x8481,
	0x8485,0x8492,0x8493,0x8495,0x849E,0x84A6,0x84A8,0x84A9,
	0x84AA,0x84AF,0x84B1,0x84B4,0x84BA,0x84BD,0x84BE,0x84C0,
	0x84C2,0x84C7,0x84C8,0x84CC,0x84CF,0x84D3,
	/* Range 0x5921 - 0x597E, array index: 0x10FA */
	0x84DC,0x84E7,0x84EA,0x84EF,0x84F0,0x84F1,0x84F2,0x84F7,
	0x8532,0x84FA,0x84FB,0x84FD,0x8502,0x8503,0x8507,0x850C,
	0x850E,0x8510,0x851C,0x851E,0x8522,0x8523,0x8524,0x8525,
	0x8527,0x852A,0x852B,0x852F,0x8533,0x8534,0x8536,0x853F,
	0x8546,0x854F,0x8550,0x8551,0x8552,0x8553,0x8556,0x8559,
	0x855C,0x855D,0x855E,0x855F,0x8560,0x8561,0x8562,0x8564,
	0x856B,0x856F,0x8579,0x857A,0x857B,0x857D,0x857F,0x8581,
	0x8585,0x8586,0x8589,0x858B,0x858C,0x858F,0x8593,0x8598,
	0x859D,0x859F,0x85A0,0x85A2,0x85A5,0x85A7,0x85B4,0x85B6,
	0x85B7,0x85B8,0x85BC,0x85BD,0x85BE,0x85BF,0x85C2,0x85C7,
	0x85CA,0x85CB,0x85CE,0x85AD,0x85D8,0x85DA,0x85DF,0x85E0,
	0x85E6,0x85E8,0x85ED,0x85F3,0x85F6,0x85FC,
	/* Range 0x5A21 - 0x5A7E, array index: 0x1158 */
	0x85FF,0x8600,0x8604,0x8605,0x860D,0x860E,0x8610,0x8611,
	0x8612,0x8618,0x8619,0x861B,0x861E,0x8621,0x8627,0x8629,
	0x8636,0x8638,0x863A,0x863C,0x863D,0x8640,0x8642,0x8646,
	0x8652,0x8653,0x8656,0x8657,0x8658,0x8659,0x865D,0x8660,
	0x8661,0x8662,0x8663,0x8664,0x8669,0x866C,0x866F,0x8675,
	0x8676,0x8677,0x867A,0x868D,0x8691,0x8696,0x8698,0x869A,
	0x869C,0x86A1,0x86A6,0x86A7,0x86A8,0x86AD,0x86B1,0x86B3,
	0x86B4,0x86B5,0x86B7,0x86B8,0x86B9,0x86BF,0x86C0,0x86C1,
	0x86C3,0x86C5,0x86D1,0x86D2,0x86D5,0x86D7,0x86DA,0x86DC,
	0x86E0,0x86E3,0x86E5,0x86E7,0x8688,0x86FA,0x86FC,0x86FD,
	0x8704,0x8705,0x8707,0x870B,0x870E,0x870F,0x8710,0x8713,
	0x8714,0x8719,0x871E,0x871F,0x8721,0x8723,
	/* Range 0x5B21 - 0x5B7E, array index: 0x11B6 */
	0x8728,0x872E,0x872F,0x8731,0x8732,0x8739,0x873A,0x873C,
	0x873D,0x873E,0x8740,0x8743,0x8745,0x874D,0x8758,0x875D,
	0x8761,0x8764,0x8765,0x876F,0x8771,0x8772,0x877B,0x8783,
	0x8784,0x8785,0x8786,0x8787,0x8788,0x8789,0x878B,0x878C,
	0x8790,0x8793,0x8795,0x8797,0x8798,0x8799,0x879E,0x87A0,
	0x87A3,0x87A7,0x87AC,0x87AD,0x87AE,0x87B1,0x87B5,0x87BE,
	0x87BF,0x87C1,0x87C8,0x87C9,0x87CA,0x87CE,0x87D5,0x87D6,
	0x87D9,0x87DA,0x87DC,0x87DF,0x87E2,0x87E3,0x87E4,0x87EA,
	0x87EB,0x87ED,0x87F1,0x87F3,0x87F8,0x87FA,0x87FF,0x8801,
	0x8803,0x8806,0x8809,0x880A,0x880B,0x8810,0x8819,0x8812,
	0x8813,0x8814,0x8818,0x881A,0x881B,0x881C,0x881E,0x881F,
	0x8828,0x882D,0x882E,0x8830,0x8832,0x8835,
	/* Range 0x5C21 - 0x5C7E, array index: 0x1214 */
	0x883A,0x883C,0x8841,0x8843,0x8845,0x8848,0x8849,0x884A,
	0x884B,0x884E,0x8851,0x8855,0x8856,0x8858,0x885A,0x885C,
	0x885F,0x8860,0x8864,0x8869,0x8871,0x8879,0x887B,0x8880,
	0x8898,0x889A,0x889B,0x889C,0x889F,0x88A0,0x88A8,0x88AA,
	0x88BA,0x88BD,0x88BE,0x88C0,0x88CA,0x88CB,0x88CC,0x88CD,
	0x88CE,0x88D1,0x88D2,0x88D3,0x88DB,0x88DE,0x88E7,0x88EF,
	0x88F0,0x88F1,0x88F5,0x88F7,0x8901,0x8906,0x890D,0x890E,
	0x890F,0x8915,0x8916,0x8918,0x8919,0x891A,0x891C,0x8920,
	0x8926,0x8927,0x8928,0x8930,0x8931,0x8932,0x8935,0x8939,
	0x893A,0x893E,0x8940,0x8942,0x8945,0x8946,0x8949,0x894F,
	0x8952,0x8957,0x895A,0x895B,0x895C,0x8961,0x8962,0x8963,
	0x896B,0x896E,0x8970,0x8973,0x8975,0x897A,
	/* Range 0x5D21 - 0x5D7E, array index: 0x1272 */
	0x897B,0x897C,0x897D,0x8989,0x898D,0x8990,0x8994,0x8995,
	0x899B,0x899C,0x899F,0x89A0,0x89A5,0x89B0,0x89B4,0x89B5,
	0x89B6,0x89B7,0x89BC,0x89D4,0x89D5,0x89D6,0x89D7,0x89D8,
	0x89E5,0x89E9,0x89EB,0x89ED,0x89F1,0x89F3,0x89F6,0x89F9,
	0x89FD,0x89FF,0x8A04,0x8A05,0x8A07,0x8A0F,0x8A11,0x8A12,
	0x8A14,0x8A15,0x8A1E,0x8A20,0x8A22,0x8A24,0x8A26,0x8A2B,
	0x8A2C,0x8A2F,0x8A35,0x8A37,0x8A3D,0x8A3E,0x8A40,0x8A43,
	0x8A45,0x8A47,0x8A49,0x8A4D,0x8A4E,0x8A53,0x8A56,0x8A57,
	0x8A58,0x8A5C,0x8A5D,0x8A61,0x8A65,0x8A67,0x8A75,0x8A76,
	0x8A77,0x8A79,0x8A7A,0x8A7B,0x8A7E,0x8A7F,0x8A80,0x8A83,
	0x8A86,0x8A8B,0x8A8F,0x8A90,0x8A92,0x8A96,0x8A97,0x8A99,
	0x8A9F,0x8AA7,0x8AA9,0x8AAE,0x8AAF,0x8AB3,
	/* Range 0x5E21 - 0x5E7E, array index: 0x12D0 */
	0x8AB6,0x8AB7,0x8ABB,0x8ABE,0x8AC3,0x8AC6,0x8AC8,0x8AC9,
	0x8ACA,0x8AD1,0x8AD3,0x8AD4,0x8AD5,0x8AD7,0x8ADD,0x8ADF,
	0x8AEC,0x8AF0,0x8AF4,0x8AF5,0x8AF6,0x8AFC,0x8AFF,0x8B05,
	0x8B06,0x8B0B,0x8B11,0x8B1C,0x8B1E,0x8B1F,0x8B0A,0x8B2D,
	0x8B30,0x8B37,0x8B3C,0x8B42,0x8B43,0x8B44,0x8B45,0x8B46,
	0x8B48,0x8B52,0x8B53,0x8B54,0x8B59,0x8B4D,0x8B5E,0x8B63,
	0x8B6D,0x8B76,0x8B78,0x8B79,0x8B7C,0x8B7E,0x8B81,0x8B84,
	0x8B85,0x8B8B,0x8B8D,0x8B8F,0x8B94,0x8B95,0x8B9C,0x8B9E,
	0x8B9F,0x8C38,0x8C39,0x8C3D,0x8C3E,0x8C45,0x8C47,0x8C49,
	0x8C4B,0x8C4F,0x8C51,0x8C53,0x8C54,0x8C57,0x8C58,0x8C5B,
	0x8C5D,0x8C59,0x8C63,0x8C64,0x8C66,0x8C68,0x8C69,0x8C6D,
	0x8C73,0x8C75,0x8C76,0x8C7B,0x8C7E,0x8C86,
	/* Range 0x5F21 - 0x5F7E, array index: 0x132E */
	0x8C87,0x8C8B,0x8C90,0x8C92,0x8C93,0x8C99,0x8C9B,0x8C9C,
	0x8CA4,0x8CB9,0x8CBA,0x8CC5,0x8CC6,0x8CC9,0x8CCB,0x8CCF,
	0x8CD6,0x8CD5,0x8CD9,0x8CDD,0x8CE1,0x8CE8,0x8CEC,0x8CEF,
	0x8CF0,0x8CF2,0x8CF5,0x8CF7,0x8CF8,0x8CFE,0x8CFF,0x8D01,
	0x8D03,0x8D09,0x8D12,0x8D17,0x8D1B,0x8D65,0x8D69,0x8D6C,
	0x8D6E,0x8D7F,0x8D82,0x8D84,0x8D88,0x8D8D,0x8D90,0x8D91,
	0x8D95,0x8D9E,0x8D9F,0x8DA0,0x8DA6,0x8DAB,0x8DAC,0x8DAF,
	0x8DB2,0x8DB5,0x8DB7,0x8DB9,0x8DBB,0x8DC0,0x8DC5,0x8DC6,
	0x8DC7,0x8DC8,0x8DCA,0x8DCE,0x8DD1,0x8DD4,0x8DD5,0x8DD7,
	0x8DD9,0x8DE4,0x8DE5,0x8DE7,0x8DEC,0x8DF0,0x8DBC,0x8DF1,
	0x8DF2,0x8DF4,0x8DFD,0x8E01,0x8E04,0x8E05,0x8E06,0x8E0B,
	0x8E11,0x8E14,0x8E16,0x8E20,0x8E21,0x8E22,
	/* Range 0x6021 - 0x607E, array index: 0x138C */
	0x8E23,0x8E26,0x8E27,0x8E31,0x8E33,0x8E36,0x8E37,0x8E38,
	0x8E39,0x8E3D,0x8E40,0x8E41,0x8E4B,0x8E4D,0x8E4E,0x8E4F,
	0x8E54,0x8E5B,0x8E5C,0x8E5D,0x8E5E,0x8E61,0x8E62,0x8E69,
	0x8E6C,0x8E6D,0x8E6F,0x8E70,0x8E71,0x8E79,0x8E7A,0x8E7B,
	0x8E82,0x8E83,0x8E89,0x8E90,0x8E92,0x8E95,0x8E9A,0x8E9B,
	0x8E9D,0x8E9E,0x8EA2,0x8EA7,0x8EA9,0x8EAD,0x8EAE,0x8EB3,
	0x8EB5,0x8EBA,0x8EBB,0x8EC0,0x8EC1,0x8EC3,0x8EC4,0x8EC7,
	0x8ECF,0x8ED1,0x8ED4,0x8EDC,0x8EE8,0x8EEE,0x8EF0,0x8EF1,
	0x8EF7,0x8EF9,0x8EFA,0x8EED,0x8F00,0x8F02,0x8F07,0x8F08,
	0x8F0F,0x8F10,0x8F16,0x8F17,0x8F18,0x8F1E,0x8F20,0x8F21,
	0x8F23,0x8F25,0x8F27,0x8F28,0x8F2C,0x8F2D,0x8F2E,0x8F34,
	0x8F35,0x8F36,0x8F37,0x8F3A,0x8F40,0x8F41,
	/* Range 0x6121 - 0x617E, array index: 0x13EA */
	0x8F43,0x8F47,0x8F4F,0x8F51,0x8F52,0x8F53,0x8F54,0x8F55,
	0x8F58,0x8F5D,0x8F5E,0x8F65,0x8F9D,0x8FA0,0x8FA1,0x8FA4,
	0x8FA5,0x8FA6,0x8FB5,0x8FB6,0x8FB8,0x8FBE,0x8FC0,0x8FC1,
	0x8FC6,0x8FCA,0x8FCB,0x8FCD,0x8FD0,0x8FD2,0x8FD3,0x8FD5,
	0x8FE0,0x8FE3,0x8FE4,0x8FE8,0x8FEE,0x8FF1,0x8FF5,0x8FF6,
	0x8FFB,0x8FFE,0x9002,0x9004,0x9008,0x900C,0x9018,0x901B,
	0x9028,0x9029,0x902F,0x902A,0x902C,0x902D,0x9033,0x9034,
	0x9037,0x903F,0x9043,0x9044,0x904C,0x905B,0x905D,0x9062,
	0x9066,0x9067,0x906C,0x9070,0x9074,0x9079,0x9085,0x9088,
	0x908B,0x908C,0x908E,0x9090,0x9095,0x9097,0x9098,0x9099,
	0x909B,0x90A0,0x90A1,0x90A2,0x90A5,0x90B0,0x90B2,0x90B3,
	0x90B4,0x90B6,0x90BD,0x90CC,0x90BE,0x90C3,
	/* Range 0x6221 - 0x627E, array index: 0x1448 */
	0x90C4,0x90C5,0x90C7,0x90C8,0x90D5,0x90D7,0x90D8,0x90D9,
	0x90DC,0x90DD,0x90DF,0x90E5,0x90D2,0x90F6,0x90EB,0x90EF,
	0x90F0,0x90F4,0x90FE,0x90FF,0x9100,0x9104,0x9105,0x9106,
	0x9108,0x910D,0x9110,0x9114,0x9116,0x9117,0x9118,0x911A,
	0x911C,0x911E,0x9120,0x9125,0x9122,0x9123,0x9127,0x9129,
	0x912E,0x912F,0x9131,0x9134,0x9136,0x9137,0x9139,0x913A,
	0x913C,0x913D,0x9143,0x9147,0x9148,0x914F,0x9153,0x9157,
	0x9159,0x915A,0x915B,0x9161,0x9164,0x9167,0x916D,0x9174,
	0x9179,0x917A,0x917B,0x9181,0x9183,0x9185,0x9186,0x918A,
	0x918E,0x9191,0x9193,0x9194,0x9195,0x9198,0x919E,0x91A1,
	0x91A6,0x91A8,0x91AC,0x91AD,0x91AE,0x91B0,0x91B1,0x91B2,
	0x91B3,0x91B6,0x91BB,0x91BC,0x91BD,0x91BF,
	/* Range 0x6321 - 0x637E, array index: 0x14A6 */
	0x91C2,0x91C3,0x91C5,0x91D3,0x91D4,0x91D7,0x91D9,0x91DA,
	0x91DE,0x91E4,0x91E5,0x91E9,0x91EA,0x91EC,0x91ED,0x91EE,
	0x91EF,0x91F0,0x91F1,0x91F7,0x91F9,0x91FB,0x91FD,0x9200,
	0x9201,0x9204,0x9205,0x9206,0x9207,0x9209,0x920A,0x920C,
	0x9210,0x9212,0x9213,0x9216,0x9218,0x921C,0x921D,0x9223,
	0x9224,0x9225,0x9226,0x9228,0x922E,0x922F,0x9230,0x9233,
	0x9235,0x9236,0x9238,0x9239,0x923A,0x923C,0x923E,0x9240,
	0x9242,0x9243,0x9246,0x9247,0x924A,0x924D,0x924E,0x924F,
	0x9251,0x9258,0x9259,0x925C,0x925D,0x9260,0x9261,0x9265,
	0x9267,0x9268,0x9269,0x926E,0x926F,0x9270,0x9275,0x9276,
	0x9277,0x9278,0x9279,0x927B,0x927C,0x927D,0x927F,0x9288,
	0x9289,0x928A,0x928D,0x928E,0x9292,0x9297,
	/* Range 0x6421 - 0x647E, array index: 0x1504 */
	0x9299,0x929F,0x92A0,0x92A4,0x92A5,0x92A7,0x92A8,0x92AB,
	0x92AF,0x92B2,0x92B6,0x92B8,0x92BA,0x92BB,0x92BC,0x92BD,
	0x92BF,0x92C0,0x92C1,0x92C2,0x92C3,0x92C5,0x92C6,0x92C7,
	0x92C8,0x92CB,0x92CC,0x92CD,0x92CE,0x92D0,0x92D3,0x92D5,
	0x92D7,0x92D8,0x92D9,0x92DC,0x92DD,0x92DF,0x92E0,0x92E1,
	0x92E3,0x92E5,0x92E7,0x92E8,0x92EC,0x92EE,0x92F0,0x92F9,
	0x92FB,0x92FF,0x9300,0x9302,0x9308,0x930D,0x9311,0x9314,
	0x9315,0x931C,0x931D,0x931E,0x931F,0x9321,0x9324,0x9325,
	0x9327,0x9329,0x932A,0x9333,0x9334,0x9336,0x9337,0x9347,
	0x9348,0x9349,0x9350,0x9351,0x9352,0x9355,0x9357,0x9358,
	0x935A,0x935E,0x9364,0x9365,0x9367,0x9369,0x936A,0x936D,
	0x936F,0x9370,0x9371,0x9373,0x9374,0x9376,
	/* Range 0x6521 - 0x657E, array index: 0x1562 */
	0x937A,0x937D,0x937F,0x9380,0x9381,0x9382,0x9388,0x938A,
	0x938B,0x938D,0x938F,0x9392,0x9395,0x9398,0x939B,0x939E,
	0x93A1,0x93A3,0x93A4,0x93A6,0x93A8,0x93AB,0x93B4,0x93B5,
	0x93B6,0x93BA,0x93A9,0x93C1,0x93C4,0x93C5,0x93C6,0x93C7,
	0x93C9,0x93CA,0x93CB,0x93CC,0x93CD,0x93D3,0x93D9,0x93DC,
	0x93DE,0x93DF,0x93E2,0x93E6,0x93E7,0x93F9,0x93F7,0x93F8,
	0x93FA,0x93FB,0x93FD,0x9401,0x9402,0x9404,0x9408,0x9409,
	0x940D,0x940E,0x940F,0x9415,0x9416,0x9417,0x941F,0x942E,
	0x942F,0x9431,0x9432,0x9433,0x9434,0x943B,0x943F,0x943D,
	0x9443,0x9445,0x9448,0x944A,0x944C,0x9455,0x9459,0x945C,
	0x945F,0x9461,0x9463,0x9468,0x946B,0x946D,0x946E,0x946F,
	0x9471,0x9472,0x9484,0x9483,0x9578,0x9579,
	/* Range 0x6621 - 0x667E, array index: 0x15C0 */
	0x957E,0x9584,0x9588,0x958C,0x958D,0x958E,0x959D,0x959E,
	0x959F,0x95A1,0x95A6,0x95A9,0x95AB,0x95AC,0x95B4,0x95B6,
	0x95BA,0x95BD,0x95BF,0x95C6,0x95C8,0x95C9,0x95CB,0x95D0,
	0x95D1,0x95D2,0x95D3,0x95D9,0x95DA,0x95DD,0x95DE,0x95DF,
	0x95E0,0x95E4,0x95E6,0x961D,0x961E,0x9622,0x9624,0x9625,
	0x9626,0x962C,0x9631,0x9633,0x9637,0x9638,0x9639,0x963A,
	0x963C,0x963D,0x9641,0x9652,0x9654,0x9656,0x9657,0x9658,
	0x9661,0x966E,0x9674,0x967B,0x967C,0x967E,0x967F,0x9681,
	0x9682,0x9683,0x9684,0x9689,0x9691,0x9696,0x969A,0x969D,
	0x969F,0x96A4,0x96A5,0x96A6,0x96A9,0x96AE,0x96AF,0x96B3,
	0x96BA,0x96CA,0x96D2,0x5DB2,0x96D8,0x96DA,0x96DD,0x96DE,
	0x96DF,0x96E9,0x96EF,0x96F1,0x96FA,0x9702,
	/* Range 0x6721 - 0x677E, array index: 0x161E */
	0x9703,0x9705,0x9709,0x971A,0x971B,0x971D,0x9721,0x9722,
	0x9723,0x9728,0x9731,0x9733,0x9741,0x9743,0x974A,0x974E,
	0x974F,0x9755,0x9757,0x9758,0x975A,0x975B,0x9763,0x9767,
	0x976A,0x976E,0x9773,0x9776,0x9777,0x9778,0x977B,0x977D,
	0x977F,0x9780,0x9789,0x9795,0x9796,0x9797,0x9799,0x979A,
	0x979E,0x979F,0x97A2,0x97AC,0x97AE,0x97B1,0x97B2,0x97B5,
	0x97B6,0x97B8,0x97B9,0x97BA,0x97BC,0x97BE,0x97BF,0x97C1,
	0x97C4,0x97C5,0x97C7,0x97C9,0x97CA,0x97CC,0x97CD,0x97CE,
	0x97D0,0x97D1,0x97D4,0x97D7,0x97D8,0x97D9,0x97DD,0x97DE,
	0x97E0,0x97DB,0x97E1,0x97E4,0x97EF,0x97F1,0x97F4,0x97F7,
	0x97F8,0x97FA,0x9807,0x980A,0x9819,0x980D,0x980E,0x9814,
	0x9816,0x981C,0x981E,0x9820,0x9823,0x9826,
	/* Range 0x6821 - 0x687E, array index: 0x167C */
	0x982B,0x982E,0x982F,0x9830,0x9832,0x9833,0x9835,0x9825,
	0x983E,0x9844,0x9847,0x984A,0x9851,0x9852,0x9853,0x9856,
	0x9857,0x9859,0x985A,0x9862,0x9863,0x9865,0x9866,0x986A,
	0x986C,0x98AB,0x98AD,0x98AE,0x98B0,0x98B4,0x98B7,0x98B8,
	0x98BA,0x98BB,0x98BF,0x98C2,0x98C5,0x98C8,0x98CC,0x98E1,
	0x98E3,0x98E5,0x98E6,0x98E7,0x98EA,0x98F3,0x98F6,0x9902,
	0x9907,0x9908,0x9911,0x9915,0x9916,0x9917,0x991A,0x991B,
	0x991C,0x991F,0x9922,0x9926,0x9927,0x992B,0x9931,0x9932,
	0x9933,0x9934,0x9935,0x9939,0x993A,0x993B,0x993C,0x9940,
	0x9941,0x9946,0x9947,0x9948,0x994D,0x994E,0x9954,0x9958,
	0x9959,0x995B,0x995C,0x995E,0x995F,0x9960,0x999B,0x999D,
	0x999F,0x99A6,0x99B0,0x99B1,0x99B2,0x99B5,
	/* Range 0x6921 - 0x697E, array index: 0x16DA */
	0x99B9,0x99BA,0x99BD,0x99BF,0x99C3,0x99C9,0x99D3,0x99D4,
	0x99D9,0x99DA,0x99DC,0x99DE,0x99E7,0x99EA,0x99EB,0x99EC,
	0x99F0,0x99F4,0x99F5,0x99F9,0x99FD,0x99FE,0x9A02,0x9A03,
	0x9A04,0x9A0B,0x9A0C,0x9A10,0x9A11,0x9A16,0x9A1E,0x9A20,
	0x9A22,0x9A23,0x9A24,0x9A27,0x9A2D,0x9A2E,0x9A33,0x9A35,
	0x9A36,0x9A38,0x9A47,0x9A41,0x9A44,0x9A4A,0x9A4B,0x9A4C,
	0x9A4E,0x9A51,0x9A54,0x9A56,0x9A5D,0x9AAA,0x9AAC,0x9AAE,
	0x9AAF,0x9AB2,0x9AB4,0x9AB5,0x9AB6,0x9AB9,0x9ABB,0x9ABE,
	0x9ABF,0x9AC1,0x9AC3,0x9AC6,0x9AC8,0x9ACE,0x9AD0,0x9AD2,
	0x9AD5,0x9AD6,0x9AD7,0x9ADB,0x9ADC,0x9AE0,0x9AE4,0x9AE5,
	0x9AE7,0x9AE9,0x9AEC,0x9AF2,0x9AF3,0x9AF5,0x9AF9,0x9AFA,
	0x9AFD,0x9AFF,0x9B00,0x9B01,0x9B02,0x9B03,
	/* Range 0x6A21 - 0x6A7E, array index: 0x1738 */
	0x9B04,0x9B05,0x9B08,0x9B09,0x9B0B,0x9B0C,0x9B0D,0x9B0E,
	0x9B10,0x9B12,0x9B16,0x9B19,0x9B1B,0x9B1C,0x9B20,0x9B26,
	0x9B2B,0x9B2D,0x9B33,0x9B34,0x9B35,0x9B37,0x9B39,0x9B3A,
	0x9B3D,0x9B48,0x9B4B,0x9B4C,0x9B55,0x9B56,0x9B57,0x9B5B,
	0x9B5E,0x9B61,0x9B63,0x9B65,0x9B66,0x9B68,0x9B6A,0x9B6B,
	0x9B6C,0x9B6D,0x9B6E,0x9B73,0x9B75,0x9B77,0x9B78,0x9B79,
	0x9B7F,0x9B80,0x9B84,0x9B85,0x9B86,0x9B87,0x9B89,0x9B8A,
	0x9B8B,0x9B8D,0x9B8F,0x9B90,0x9B94,0x9B9A,0x9B9D,0x9B9E,
	0x9BA6,0x9BA7,0x9BA9,0x9BAC,0x9BB0,0x9BB1,0x9BB2,0x9BB7,
	0x9BB8,0x9BBB,0x9BBC,0x9BBE,0x9BBF,0x9BC1,0x9BC7,0x9BC8,
	0x9BCE,0x9BD0,0x9BD7,0x9BD8,0x9BDD,0x9BDF,0x9BE5,0x9BE7,
	0x9BEA,0x9BEB,0x9BEF,0x9BF3,0x9BF7,0x9BF8,
	/* Range 0x6B21 - 0x6B7E, array index: 0x1796 */
	0x9BF9,0x9BFA,0x9BFD,0x9BFF,0x9C00,0x9C02,0x9C0B,0x9C0F,
	0x9C11,0x9C16,0x9C18,0x9C19,0x9C1A,0x9C1C,0x9C1E,0x9C22,
	0x9C23,0x9C26,0x9C27,0x9C28,0x9C29,0x9C2A,0x9C31,0x9C35,
	0x9C36,0x9C37,0x9C3D,0x9C41,0x9C43,0x9C44,0x9C45,0x9C49,
	0x9C4A,0x9C4E,0x9C4F,0x9C50,0x9C53,0x9C54,0x9C56,0x9C58,
	0x9C5B,0x9C5D,0x9C5E,0x9C5F,0x9C63,0x9C69,0x9C6A,0x9C5C,
	0x9C6B,0x9C68,0x9C6E,0x9C70,0x9C72,0x9C75,0x9C77,0x9C7B,
	0x9CE6,0x9CF2,0x9CF7,0x9CF9,0x9D0B,0x9D02,0x9D11,0x9D17,
	0x9D18,0x9D1C,0x9D1D,0x9D1E,0x9D2F,0x9D30,0x9D32,0x9D33,
	0x9D34,0x9D3A,0x9D3C,0x9D45,0x9D3D,0x9D42,0x9D43,0x9D47,
	0x9D4A,0x9D53,0x9D54,0x9D5F,0x9D63,0x9D62,0x9D65,0x9D69,
	0x9D6A,0x9D6B,0x9D70,0x9D76,0x9D77,0x9D7B,
	/* Range 0x6C21 - 0x6C7E, array index: 0x17F4 */
	0x9D7C,0x9D7E,0x9D83,0x9D84,0x9D86,0x9D8A,0x9D8D,0x9D8E,
	0x9D92,0x9D93,0x9D95,0x9D96,0x9D97,0x9D98,0x9DA1,0x9DAA,
	0x9DAC,0x9DAE,0x9DB1,0x9DB5,0x9DB9,0x9DBC,0x9DBF,0x9DC3,
	0x9DC7,0x9DC9,0x9DCA,0x9DD4,0x9DD5,0x9DD6,0x9DD7,0x9DDA,
	0x9DDE,0x9DDF,0x9DE0,0x9DE5,0x9DE7,0x9DE9,0x9DEB,0x9DEE,
	0x9DF0,0x9DF3,0x9DF4,0x9DFE,0x9E0A,0x9E02,0x9E07,0x9E0E,
	0x9E10,0x9E11,0x9E12,0x9E15,0x9E16,0x9E19,0x9E1C,0x9E1D,
	0x9E7A,0x9E7B,0x9E7C,0x9E80,0x9E82,0x9E83,0x9E84,0x9E85,
	0x9E87,0x9E8E,0x9E8F,0x9E96,0x9E98,0x9E9B,0x9E9E,0x9EA4,
	0x9EA8,0x9EAC,0x9EAE,0x9EAF,0x9EB0,0x9EB3,0x9EB4,0x9EB5,
	0x9EC6,0x9EC8,0x9ECB,0x9ED5,0x9EDF,0x9EE4,0x9EE7,0x9EEC,
	0x9EED,0x9EEE,0x9EF0,0x9EF1,0x9EF2,0x9EF5,
	/* Range 0x6D21 - 0x6D63, array index: 0x1852 */
	0x9EF8,0x9EFF,0x9F02,0x9F03,0x9F09,0x9F0F,0x9F10,0x9F11,
	0x9F12,0x9F14,0x9F16,0x9F17,0x9F19,0x9F1A,0x9F1B,0x9F1F,
	0x9F22,0x9F26,0x9F2A,0x9F2B,0x9F2F,0x9F31,0x9F32,0x9F34,
	0x9F37,0x9F39,0x9F3A,0x9F3C,0x9F3D,0x9F3F,0x9F41,0x9F43,
	0x9F44,0x9F45,0x9F46,0x9F47,0x9F53,0x9F55,0x9F56,0x9F57,
	0x9F58,0x9F5A,0x9F5D,0x9F5E,0x9F68,0x9F69,0x9F6D,0x9F6E,
	0x9F6F,0x9F70,0x9F71,0x9F73,0x9F75,0x9F7A,0x9F7D,0x9F8F,
	0x9F90,0x9F91,0x9F92,0x9F94,0x9F96,0x9F97,0x9F9E,0x9FA1,
	0x9FA2,0x9FA3,0x9FA5,
	/* Unranged codes (73 codes) */
	/* Array index: 0x1895 */ 0x2921,0x00C6,
	/* Array index: 0x1895 */ 0x2922,0x0110,
	/* Array index: 0x1895 */ 0x2924,0x0126,
	/* Array index: 0x1895 */ 0x2926,0x0132,
};

#endif /* ICONV_TO_UCS_CCS_JIS_X0212_1990 && defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit UCS -> jis_x0212_1990 speed-optimized table (45568 bytes).
 * ======================================================================
 */

#if defined (ICONV_FROM_UCS_CCS_JIS_X0212_1990) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
from_ucs_speed_jis_x0212_1990[] =
{
	/* Heading Block */
	0x0100,0x0200,0x0300,0x0400,0x0500,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,0x0600,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,0x0700,0x0800,
	0x0900,0x0A00,0x0B00,0x0C00,0x0D00,0x0E00,0x0F00,0x1000,
	0x1100,0x1200,0x1300,0x1400,0x1500,0x1600,0x1700,0x1800,
	0x1900,0x1A00,0x1B00,0x1C00,0x1D00,0x1E00,0x1F00,0x2000,
	0x2100,0x2200,0x2300,0x2400,0x2500,0x2600,0x2700,0x2800,
	0x2900,0x2A00,0x2B00,0x2C00,0x2D00,0x2E00,0x2F00,0x3000,
	0x3100,0x3200,0x3300,0x3400,0x3500,0x3600,0x3700,0x3800,
	0x3900,0x3A00,0x3B00,0x3C00,0x3D00,0x3E00,0x3F00,0x4000,
	0x4100,0x4200,0x4300,0x4400,0x4500,0x4600,0x4700,0x4800,
	0x4900,0x4A00,0x4B00,0x4C00,0x4D00,0x4E00,0x4F00,0x5000,
	0x5100,0x5200,0x5300,0x5400,0x5500,0x5600,0x5700,0x5800,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,INVBLK,
	/* Block 1, Array index 0x0100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2237,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x2242,INVALC,INVALC,0x2270,INVALC,0x2243,INVALC,
	INVALC,0x226D,0x226C,INVALC,INVALC,INVALC,0x226E,0x2234,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x2231,INVALC,0x226B,INVALC,INVALC,INVALC,INVALC,0x2244,
	0x2A22,0x2A21,0x2A24,0x2A2A,0x2A23,0x2A29,0x2921,0x2A2E,
	0x2A32,0x2A31,0x2A34,0x2A33,0x2A40,0x2A3F,0x2A42,0x2A41,
	INVALC,0x2A50,0x2A52,0x2A51,0x2A54,0x2A58,0x2A53,INVALC,
	0x292C,0x2A63,0x2A62,0x2A65,0x2A64,0x2A72,0x2930,0x294E,
	0x2B22,0x2B21,0x2B24,0x2B2A,0x2B23,0x2B29,0x2941,0x2B2E,
	0x2B32,0x2B31,0x2B34,0x2B33,0x2B40,0x2B3F,0x2B42,0x2B41,
	0x2943,0x2B50,0x2B52,0x2B51,0x2B54,0x2B58,0x2B53,INVALC,
	0x294C,0x2B63,0x2B62,0x2B65,0x2B64,0x2B72,0x2950,0x2B73,
	/* Block 2, Array index 0x0200 */
	0x2A27,0x2B27,0x2A25,0x2B25,0x2A28,0x2B28,0x2A2B,0x2B2B,
	0x2A2C,0x2B2C,0x2A2F,0x2B2F,0x2A2D,0x2B2D,0x2A30,0x2B30,
	0x2922,0x2942,0x2A37,0x2B37,INVALC,INVALC,0x2A36,0x2B36,
	0x2A38,0x2B38,0x2A35,0x2B35,0x2A3A,0x2B3A,0x2A3B,0x2B3B,
	0x2A3D,0x2B3D,0x2A3C,INVALC,0x2A3E,0x2B3E,0x2924,0x2944,
	0x2A47,0x2B47,0x2A45,0x2B45,INVALC,INVALC,0x2A46,0x2B46,
	0x2A44,0x2945,0x2926,0x2946,0x2A48,0x2B48,0x2A49,0x2B49,
	0x2947,0x2A4A,0x2B4A,0x2A4C,0x2B4C,0x2A4B,0x2B4B,0x2929,
	0x2949,0x2928,0x2948,0x2A4D,0x2B4D,0x2A4F,0x2B4F,0x2A4E,
	0x2B4E,0x294A,0x292B,0x294B,0x2A57,0x2B57,INVALC,INVALC,
	0x2A56,0x2B56,0x292D,0x294D,0x2A59,0x2B59,0x2A5B,0x2B5B,
	0x2A5A,0x2B5A,0x2A5C,0x2B5C,0x2A5D,0x2B5D,0x2A5F,0x2B5F,
	0x2A5E,0x2B5E,0x2A61,0x2B61,0x2A60,0x2B60,0x292F,0x294F,
	0x2A6C,0x2B6C,0x2A69,0x2B69,0x2A66,0x2B66,0x2A6B,0x2B6B,
	0x2A68,0x2B68,0x2A6A,0x2B6A,0x2A71,0x2B71,0x2A74,0x2B74,
	0x2A73,0x2A75,0x2B75,0x2A77,0x2B77,0x2A76,0x2B76,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2A26,0x2B26,0x2A43,
	0x2B43,0x2A55,0x2B55,0x2A67,0x2B67,0x2A70,0x2B70,0x2A6D,
	0x2B6D,0x2A6F,0x2B6F,0x2A6E,0x2B6E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x2B39,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 3, Array index 0x0300 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2230,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x222F,0x2232,0x2236,0x2235,INVALC,0x2233,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 4, Array index 0x0400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x2238,0x2239,0x2661,INVALC,
	0x2662,0x2663,0x2664,INVALC,0x2667,INVALC,0x2669,0x266C,
	0x2676,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2665,0x266A,0x2671,0x2672,0x2673,0x2674,
	0x267B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2678,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2675,0x267A,0x2677,0x2679,0x267C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 5, Array index 0x0500 */
	INVALC,INVALC,0x2742,0x2743,0x2744,0x2745,0x2746,0x2747,
	0x2748,0x2749,0x274A,0x274B,0x274C,INVALC,0x274D,0x274E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x2772,0x2773,0x2774,0x2775,0x2776,0x2777,
	0x2778,0x2779,0x277A,0x277B,0x277C,INVALC,0x277D,0x277E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 34, Array index 0x0600 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x2271,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x226F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 79, Array index 0x0700 */
	INVALC,INVALC,0x3021,INVALC,0x3022,0x3023,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3024,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3025,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3026,
	INVALC,INVALC,INVALC,0x3027,0x3028,INVALC,INVALC,INVALC,
	0x3029,INVALC,INVALC,0x302A,INVALC,INVALC,0x302B,0x302C,
	0x302D,INVALC,INVALC,INVALC,INVALC,0x302E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x302F,0x3030,INVALC,INVALC,0x3031,INVALC,INVALC,0x3032,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3033,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3034,INVALC,0x3035,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3036,INVALC,INVALC,INVALC,INVALC,
	0x3037,0x3038,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3039,0x303A,INVALC,INVALC,
	INVALC,0x303B,INVALC,INVALC,INVALC,INVALC,INVALC,0x303C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x303D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x303E,0x303F,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3040,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3041,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3042,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3043,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3044,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3045,0x3046,INVALC,INVALC,INVALC,INVALC,
	0x3047,0x3048,0x3049,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x304A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x304B,
	INVALC,0x304C,INVALC,0x304D,INVALC,0x304E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x304F,0x3050,0x3051,
	/* Block 80, Array index 0x0800 */
	0x3052,INVALC,0x3053,0x3054,INVALC,INVALC,INVALC,INVALC,
	0x3055,INVALC,INVALC,0x3056,0x3057,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3058,INVALC,INVALC,0x3059,0x305A,0x305B,
	INVALC,0x305C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x305D,INVALC,
	INVALC,0x305E,INVALC,0x3060,INVALC,0x3061,INVALC,0x3062,
	INVALC,0x3063,INVALC,0x3064,INVALC,INVALC,0x3065,INVALC,
	0x3066,INVALC,0x3067,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3068,0x3069,INVALC,0x306A,0x306B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x306C,INVALC,0x306D,INVALC,0x306E,INVALC,
	0x306F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3070,
	0x305F,INVALC,INVALC,0x3071,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3072,INVALC,0x3073,INVALC,0x3074,INVALC,
	INVALC,0x3075,INVALC,INVALC,INVALC,INVALC,INVALC,0x3076,
	0x3077,0x3078,0x3079,INVALC,INVALC,0x307A,0x307B,INVALC,
	INVALC,0x307C,0x307D,INVALC,0x307E,0x3121,INVALC,INVALC,
	INVALC,0x3122,0x3123,INVALC,0x3124,INVALC,0x3125,INVALC,
	0x3126,INVALC,0x3127,0x3128,0x3129,INVALC,INVALC,0x312A,
	INVALC,0x312B,0x312C,INVALC,INVALC,INVALC,0x312D,0x312E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x312F,INVALC,INVALC,INVALC,INVALC,0x3130,
	INVALC,0x3131,INVALC,0x3132,0x3133,0x3134,0x3135,INVALC,
	0x3136,0x3137,INVALC,INVALC,INVALC,0x3138,0x3139,INVALC,
	0x313A,0x313B,INVALC,0x313C,0x313D,0x313E,INVALC,0x313F,
	INVALC,INVALC,0x3140,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3141,INVALC,INVALC,INVALC,
	0x3142,INVALC,0x3143,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3144,INVALC,0x3145,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3146,0x3147,INVALC,0x3148,
	/* Block 81, Array index 0x0900 */
	0x3149,0x314A,INVALC,INVALC,0x314B,INVALC,INVALC,0x314C,
	INVALC,INVALC,0x314D,INVALC,0x314E,INVALC,0x314F,INVALC,
	0x3150,INVALC,INVALC,0x3151,INVALC,INVALC,INVALC,0x3152,
	0x3153,INVALC,INVALC,0x3154,0x3155,0x3156,0x3157,INVALC,
	INVALC,INVALC,0x3158,INVALC,INVALC,INVALC,INVALC,0x3159,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x315A,INVALC,
	0x315B,INVALC,0x315C,0x315D,INVALC,0x315E,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3176,INVALC,INVALC,INVALC,INVALC,
	0x315F,0x3160,0x3161,INVALC,INVALC,0x3162,0x3163,INVALC,
	INVALC,INVALC,0x3164,INVALC,0x3165,INVALC,0x3166,INVALC,
	INVALC,0x3167,0x3168,0x3169,INVALC,INVALC,INVALC,0x316A,
	INVALC,0x316B,INVALC,INVALC,INVALC,INVALC,INVALC,0x316C,
	0x316D,INVALC,0x316E,0x316F,INVALC,INVALC,0x3170,0x3171,
	INVALC,INVALC,0x3172,INVALC,INVALC,0x3173,INVALC,INVALC,
	0x3174,0x3175,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3177,INVALC,0x3178,0x3179,INVALC,0x317A,INVALC,
	INVALC,INVALC,0x317B,INVALC,INVALC,INVALC,0x317C,0x317D,
	0x317E,INVALC,0x3221,0x3222,0x3223,INVALC,0x3224,INVALC,
	INVALC,INVALC,INVALC,0x3225,0x3226,INVALC,0x3227,0x3228,
	0x3229,0x322A,0x322B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x322C,INVALC,INVALC,INVALC,INVALC,0x322D,
	0x322E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x322F,0x3230,INVALC,INVALC,0x3231,INVALC,INVALC,
	0x3232,INVALC,INVALC,0x3233,0x3234,INVALC,INVALC,0x3235,
	INVALC,INVALC,INVALC,INVALC,0x3236,INVALC,0x3237,INVALC,
	0x3238,INVALC,INVALC,0x3239,0x323A,INVALC,INVALC,INVALC,
	0x323B,INVALC,INVALC,INVALC,0x323C,0x323D,INVALC,0x323E,
	INVALC,INVALC,0x323F,INVALC,0x3240,INVALC,0x3241,INVALC,
	0x3242,0x3243,INVALC,INVALC,INVALC,INVALC,INVALC,0x3244,
	INVALC,0x3245,0x3251,INVALC,INVALC,INVALC,0x3246,INVALC,
	INVALC,INVALC,0x3247,INVALC,INVALC,INVALC,0x3248,INVALC,
	/* Block 82, Array index 0x0A00 */
	INVALC,INVALC,INVALC,0x3249,INVALC,INVALC,0x324A,0x324B,
	0x324C,INVALC,INVALC,0x324D,0x324E,0x324F,0x3250,INVALC,
	0x3252,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3253,
	INVALC,0x3254,INVALC,0x3255,0x3256,0x3257,0x3258,INVALC,
	INVALC,INVALC,INVALC,0x3259,INVALC,INVALC,INVALC,0x325A,
	0x325B,INVALC,INVALC,INVALC,0x325C,0x325D,INVALC,0x325E,
	INVALC,0x325F,INVALC,0x3260,0x3261,0x3262,INVALC,INVALC,
	0x3263,0x3264,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3265,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3266,INVALC,INVALC,INVALC,INVALC,0x3267,
	INVALC,INVALC,INVALC,0x3268,INVALC,0x3269,INVALC,0x326A,
	0x326B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x326C,
	INVALC,INVALC,INVALC,INVALC,0x326D,INVALC,0x326E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x326F,INVALC,
	INVALC,INVALC,INVALC,0x3270,0x3271,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3272,INVALC,INVALC,0x3273,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3274,INVALC,INVALC,INVALC,INVALC,0x3275,INVALC,INVALC,
	INVALC,0x3276,INVALC,0x3277,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3278,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3279,INVALC,0x327A,INVALC,0x327B,INVALC,0x327C,0x327D,
	INVALC,INVALC,0x327E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3321,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3322,
	INVALC,0x3323,0x3324,0x3325,INVALC,0x3326,INVALC,INVALC,
	0x3327,INVALC,INVALC,INVALC,INVALC,INVALC,0x3328,INVALC,
	INVALC,INVALC,0x3329,INVALC,INVALC,0x332A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x332B,INVALC,
	INVALC,INVALC,0x332C,0x332D,0x332E,INVALC,INVALC,0x332F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 83, Array index 0x0B00 */
	INVALC,0x3330,0x3331,INVALC,INVALC,0x3332,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3333,0x3334,INVALC,0x3335,0x3336,INVALC,
	0x3337,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3338,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3339,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x333A,0x333B,INVALC,INVALC,0x333C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x333D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x333E,INVALC,INVALC,
	INVALC,0x333F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3340,INVALC,0x3341,
	0x3342,INVALC,0x3343,INVALC,0x3344,INVALC,INVALC,0x3345,
	0x3346,0x3347,INVALC,INVALC,INVALC,INVALC,0x3348,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3349,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x334A,
	0x334B,0x334C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x334D,INVALC,0x334E,INVALC,INVALC,0x334F,INVALC,INVALC,
	INVALC,INVALC,0x3350,INVALC,0x3351,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3352,INVALC,0x3353,0x3354,0x3355,
	0x3356,INVALC,0x3357,INVALC,0x3358,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3359,0x335A,0x335B,0x335C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x335D,
	0x335E,INVALC,INVALC,INVALC,INVALC,INVALC,0x335F,0x3360,
	0x3361,INVALC,0x3362,0x3363,INVALC,0x3364,INVALC,INVALC,
	0x3365,INVALC,INVALC,INVALC,0x3366,INVALC,0x3367,INVALC,
	0x3368,INVALC,INVALC,INVALC,0x3369,INVALC,INVALC,0x336A,
	INVALC,0x336B,INVALC,INVALC,0x336C,INVALC,0x336D,INVALC,
	INVALC,INVALC,INVALC,0x336E,0x336F,INVALC,INVALC,INVALC,
	INVALC,0x3370,INVALC,INVALC,INVALC,0x3371,INVALC,INVALC,
	0x3372,0x3373,0x3374,INVALC,0x3375,INVALC,INVALC,INVALC,
	0x3376,0x3377,INVALC,INVALC,0x3378,INVALC,0x3379,0x337A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 84, Array index 0x0C00 */
	0x337B,INVALC,INVALC,0x337C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x337D,0x337E,0x3421,INVALC,INVALC,INVALC,
	INVALC,0x3422,INVALC,0x3423,INVALC,INVALC,INVALC,INVALC,
	0x3424,INVALC,INVALC,0x3425,0x3426,INVALC,0x3427,0x3428,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3429,INVALC,0x342A,
	0x342B,0x342C,INVALC,0x342D,0x342E,0x342F,INVALC,INVALC,
	0x3430,INVALC,0x3431,INVALC,INVALC,0x3432,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3433,0x3434,0x3435,INVALC,
	INVALC,INVALC,0x3436,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3438,0x3437,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3439,INVALC,0x343A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x343B,INVALC,0x343C,INVALC,0x343D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x343E,0x343F,INVALC,INVALC,
	INVALC,INVALC,0x3440,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3441,INVALC,INVALC,INVALC,INVALC,0x3442,INVALC,
	INVALC,INVALC,INVALC,0x3443,INVALC,INVALC,INVALC,0x3444,
	0x3445,INVALC,INVALC,INVALC,INVALC,INVALC,0x3446,INVALC,
	INVALC,INVALC,INVALC,0x3447,0x3448,INVALC,INVALC,INVALC,
	INVALC,0x3449,INVALC,INVALC,INVALC,0x344A,INVALC,INVALC,
	INVALC,0x344B,INVALC,INVALC,0x344C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x344D,0x344E,INVALC,INVALC,INVALC,0x344F,
	INVALC,INVALC,0x3450,INVALC,0x3451,0x3452,INVALC,0x3453,
	0x3454,INVALC,0x3455,INVALC,INVALC,0x3456,INVALC,INVALC,
	0x3457,INVALC,INVALC,INVALC,INVALC,0x3458,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3459,
	INVALC,INVALC,0x345A,0x345B,INVALC,0x345C,INVALC,INVALC,
	INVALC,INVALC,0x345D,INVALC,INVALC,0x345E,0x345F,INVALC,
	0x3460,INVALC,INVALC,INVALC,INVALC,INVALC,0x3461,0x3462,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3463,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 85, Array index 0x0D00 */
	INVALC,INVALC,0x3464,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3465,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3466,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3467,INVALC,INVALC,INVALC,INVALC,INVALC,0x3468,
	0x3469,INVALC,0x346A,INVALC,INVALC,INVALC,INVALC,0x346B,
	INVALC,0x346C,INVALC,INVALC,0x346D,0x346E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x346F,0x3470,INVALC,INVALC,0x3471,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3472,INVALC,0x3473,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3474,INVALC,
	INVALC,INVALC,0x3475,INVALC,0x3476,INVALC,0x3477,0x3478,
	INVALC,0x3479,INVALC,0x347A,INVALC,0x347B,0x347C,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x347D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x347E,
	INVALC,0x3521,INVALC,0x3522,INVALC,0x3523,INVALC,INVALC,
	0x3524,0x3525,INVALC,INVALC,INVALC,0x3526,INVALC,INVALC,
	INVALC,0x3527,INVALC,INVALC,INVALC,0x3528,0x3529,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x352A,INVALC,INVALC,0x352B,
	INVALC,0x352C,INVALC,INVALC,INVALC,INVALC,0x352D,0x352E,
	INVALC,0x352F,0x3530,INVALC,INVALC,0x3531,0x3532,INVALC,
	INVALC,0x3533,INVALC,INVALC,INVALC,INVALC,INVALC,0x3534,
	INVALC,0x3535,0x3536,0x3537,INVALC,INVALC,INVALC,0x3538,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3539,INVALC,
	INVALC,INVALC,0x353A,INVALC,INVALC,0x353B,0x353C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x353D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x353E,INVALC,0x353F,INVALC,INVALC,0x3540,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3541,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3542,INVALC,0x3543,0x3544,
	/* Block 86, Array index 0x0E00 */
	0x3545,0x3546,INVALC,INVALC,INVALC,0x3547,INVALC,INVALC,
	0x3548,0x3549,INVALC,INVALC,0x354A,0x354B,0x354C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x354D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x354E,0x354F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3550,INVALC,INVALC,0x3551,0x3552,INVALC,
	INVALC,INVALC,INVALC,0x3553,0x3554,0x3555,INVALC,INVALC,
	INVALC,0x3556,INVALC,INVALC,INVALC,INVALC,INVALC,0x3557,
	INVALC,0x3558,0x3559,INVALC,INVALC,0x355A,INVALC,INVALC,
	0x355B,0x355C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x355D,INVALC,0x355E,0x355F,INVALC,INVALC,0x3560,INVALC,
	0x3561,0x3562,INVALC,INVALC,0x3563,INVALC,0x3564,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3565,
	INVALC,0x3566,0x3567,INVALC,INVALC,INVALC,0x3568,INVALC,
	0x3569,INVALC,INVALC,INVALC,INVALC,INVALC,0x356A,0x356B,
	INVALC,0x356C,0x356D,0x356E,0x356F,INVALC,INVALC,0x3570,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3571,0x3572,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3573,INVALC,INVALC,
	INVALC,INVALC,0x3574,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3575,
	INVALC,0x3576,INVALC,0x3577,INVALC,INVALC,0x3578,INVALC,
	INVALC,0x3579,INVALC,0x357A,0x357B,INVALC,0x357C,INVALC,
	INVALC,0x357D,0x357E,0x3621,INVALC,INVALC,INVALC,0x3622,
	0x3623,INVALC,INVALC,0x3624,INVALC,INVALC,0x3625,INVALC,
	INVALC,INVALC,0x3626,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3627,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3628,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3629,
	/* Block 87, Array index 0x0F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x362A,INVALC,INVALC,
	0x362B,INVALC,0x362C,INVALC,INVALC,0x362D,0x362E,0x362F,
	0x3630,0x3631,0x3632,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3633,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3634,INVALC,INVALC,INVALC,
	0x3635,INVALC,INVALC,0x3636,INVALC,0x3637,INVALC,0x3638,
	INVALC,0x3639,INVALC,0x363A,0x363B,0x363C,INVALC,0x363D,
	0x363E,0x363F,INVALC,0x3640,0x3641,INVALC,0x3642,INVALC,
	INVALC,0x3643,INVALC,0x3644,INVALC,0x3645,INVALC,0x3646,
	INVALC,INVALC,INVALC,INVALC,0x3647,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3648,INVALC,
	0x3649,0x364A,0x364B,0x364C,INVALC,INVALC,0x364D,INVALC,
	INVALC,0x364E,INVALC,INVALC,INVALC,0x364F,INVALC,0x3650,
	INVALC,0x3651,0x3652,INVALC,INVALC,0x3653,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3654,0x3655,INVALC,INVALC,
	0x3656,INVALC,INVALC,0x3657,0x3658,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3659,INVALC,INVALC,
	INVALC,0x365A,0x365B,INVALC,INVALC,0x365C,0x365D,0x365E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x365F,0x3660,
	0x3661,0x3662,INVALC,0x3663,0x3664,0x3665,INVALC,INVALC,
	INVALC,0x3666,INVALC,0x3667,INVALC,INVALC,INVALC,0x3668,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3669,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x366A,INVALC,INVALC,
	INVALC,0x366B,0x366C,0x366D,0x3670,0x3671,INVALC,0x366E,
	0x366F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3672,INVALC,INVALC,0x3673,0x3674,INVALC,0x3675,
	INVALC,0x3676,INVALC,INVALC,0x3677,0x3678,0x3679,0x367A,
	0x367B,INVALC,INVALC,0x367D,INVALC,0x367E,INVALC,INVALC,
	INVALC,0x367C,INVALC,INVALC,INVALC,INVALC,0x3721,0x3722,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 88, Array index 0x1000 */
	INVALC,0x3723,0x3724,INVALC,INVALC,INVALC,INVALC,0x3725,
	INVALC,INVALC,0x3726,INVALC,0x3727,INVALC,INVALC,INVALC,
	INVALC,0x3728,INVALC,INVALC,INVALC,0x3729,INVALC,INVALC,
	INVALC,INVALC,0x372A,0x372B,INVALC,0x372C,INVALC,INVALC,
	0x372D,INVALC,0x372E,0x372F,0x3730,0x3731,INVALC,INVALC,
	INVALC,0x3732,0x3733,INVALC,0x3734,INVALC,0x3735,0x3736,
	INVALC,INVALC,INVALC,0x3737,0x3738,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3739,0x373A,0x373B,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x373C,0x373D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x373E,0x373F,INVALC,INVALC,
	INVALC,INVALC,0x3740,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3741,INVALC,INVALC,0x3742,INVALC,0x3743,
	0x3744,INVALC,INVALC,0x3745,INVALC,0x3746,0x3747,0x3748,
	0x3749,0x374A,INVALC,0x374B,0x374C,0x374D,INVALC,0x374E,
	INVALC,0x374F,0x3750,0x3751,0x3752,INVALC,0x3753,INVALC,
	INVALC,0x3754,INVALC,0x3755,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3756,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3757,0x3760,INVALC,0x3758,
	INVALC,0x3759,0x375A,INVALC,0x375B,0x375C,0x375D,0x375E,
	INVALC,0x375F,INVALC,INVALC,INVALC,INVALC,INVALC,0x3761,
	0x3762,0x3763,INVALC,INVALC,0x3764,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3765,INVALC,INVALC,INVALC,INVALC,0x3766,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3767,
	0x3768,INVALC,INVALC,INVALC,0x3769,INVALC,INVALC,0x376A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x376B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x376C,0x376D,INVALC,
	INVALC,0x377E,INVALC,INVALC,0x376E,INVALC,0x376F,0x3770,
	INVALC,0x3771,INVALC,INVALC,INVALC,0x3772,INVALC,INVALC,
	0x3773,INVALC,INVALC,INVALC,INVALC,0x3774,0x3775,INVALC,
	0x3776,INVALC,INVALC,INVALC,INVALC,0x3777,0x3778,0x3779,
	/* Block 89, Array index 0x1100 */
	INVALC,INVALC,INVALC,0x377A,0x377B,INVALC,INVALC,INVALC,
	0x377C,0x377D,INVALC,INVALC,0x3821,0x3822,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3823,INVALC,INVALC,0x3824,0x3825,
	0x3826,INVALC,INVALC,INVALC,INVALC,INVALC,0x3827,0x3828,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3829,INVALC,INVALC,
	INVALC,INVALC,0x382A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x382B,INVALC,INVALC,INVALC,INVALC,INVALC,0x382C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x382D,INVALC,INVALC,0x382E,0x382F,INVALC,0x3830,
	0x3831,INVALC,INVALC,INVALC,INVALC,0x3832,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3833,
	INVALC,0x3834,INVALC,INVALC,0x3835,INVALC,INVALC,0x3836,
	0x3837,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3838,INVALC,INVALC,INVALC,0x3839,INVALC,INVALC,0x383A,
	0x383B,0x383C,INVALC,INVALC,INVALC,INVALC,INVALC,0x383D,
	0x383E,0x383F,0x3840,INVALC,0x3841,0x3842,INVALC,0x3843,
	0x3844,INVALC,INVALC,INVALC,0x3845,INVALC,0x3846,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3847,INVALC,INVALC,
	0x3848,0x3849,0x384A,INVALC,INVALC,INVALC,0x384B,INVALC,
	INVALC,0x384C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x384D,0x384E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3850,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3851,INVALC,0x384F,INVALC,INVALC,INVALC,
	0x3852,INVALC,INVALC,INVALC,INVALC,0x3853,0x3854,INVALC,
	0x3855,INVALC,0x3856,INVALC,0x3857,INVALC,0x3858,INVALC,
	INVALC,INVALC,0x3859,INVALC,INVALC,0x385A,INVALC,INVALC,
	INVALC,0x385B,0x385C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x385D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x385E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 90, Array index 0x1200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x385F,0x3860,INVALC,
	INVALC,INVALC,INVALC,0x3861,0x3862,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3863,0x3864,0x3865,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3867,INVALC,INVALC,
	INVALC,0x3868,INVALC,0x3869,0x386A,INVALC,INVALC,INVALC,
	0x386B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x386C,
	0x386D,INVALC,INVALC,0x386E,INVALC,0x386F,0x3870,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3871,
	INVALC,INVALC,INVALC,0x3872,INVALC,INVALC,0x3873,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3874,0x3875,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3876,INVALC,0x3877,INVALC,0x3878,0x3879,0x387A,
	INVALC,0x387B,INVALC,0x387C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x387D,INVALC,0x387E,INVALC,0x3921,
	INVALC,INVALC,0x3922,INVALC,INVALC,0x3923,0x3924,INVALC,
	INVALC,0x3925,INVALC,0x3926,0x3927,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3928,0x3929,INVALC,0x392A,INVALC,
	INVALC,INVALC,0x392B,INVALC,INVALC,0x392C,INVALC,0x392D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x392E,
	INVALC,INVALC,INVALC,INVALC,0x392F,INVALC,INVALC,0x3930,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3931,0x3932,0x3933,
	0x3934,INVALC,INVALC,0x3935,INVALC,INVALC,INVALC,0x3936,
	INVALC,INVALC,0x3937,INVALC,0x3938,INVALC,INVALC,INVALC,
	INVALC,0x3939,INVALC,0x393A,0x393B,INVALC,INVALC,INVALC,
	0x393C,INVALC,0x393D,INVALC,INVALC,0x393E,INVALC,INVALC,
	INVALC,INVALC,0x393F,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3940,0x3941,0x3942,
	INVALC,INVALC,INVALC,0x3943,0x3944,INVALC,INVALC,0x3945,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3946,0x3947,
	INVALC,0x3948,0x3949,INVALC,0x394A,INVALC,INVALC,0x394B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 91, Array index 0x1300 */
	0x394C,INVALC,INVALC,INVALC,0x394D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x394E,0x394F,0x3950,INVALC,
	INVALC,INVALC,0x3951,0x3952,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3953,INVALC,
	INVALC,INVALC,INVALC,0x3954,0x3955,INVALC,INVALC,0x3956,
	0x3957,INVALC,0x3958,INVALC,INVALC,0x3959,INVALC,INVALC,
	0x395A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x395B,0x395C,INVALC,0x395D,
	0x395E,INVALC,INVALC,INVALC,0x395F,INVALC,INVALC,INVALC,
	0x3960,INVALC,INVALC,INVALC,INVALC,0x3961,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3962,INVALC,
	INVALC,INVALC,INVALC,0x3963,INVALC,0x3964,INVALC,0x3965,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3966,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3967,
	INVALC,INVALC,0x3968,0x3969,INVALC,INVALC,0x396A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x396B,INVALC,INVALC,INVALC,INVALC,
	0x396C,INVALC,INVALC,0x396D,INVALC,INVALC,0x396E,INVALC,
	INVALC,0x396F,INVALC,INVALC,0x3970,INVALC,0x3971,0x3972,
	0x3973,INVALC,0x3974,INVALC,INVALC,INVALC,INVALC,0x3975,
	INVALC,INVALC,INVALC,INVALC,0x3976,INVALC,INVALC,INVALC,
	INVALC,0x3977,0x3978,0x3979,INVALC,0x397A,INVALC,INVALC,
	0x397B,INVALC,0x397C,0x397D,INVALC,INVALC,INVALC,0x397E,
	INVALC,INVALC,INVALC,INVALC,0x3A21,INVALC,0x3A22,INVALC,
	0x3A23,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A24,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3A25,INVALC,0x3A26,INVALC,INVALC,INVALC,
	0x3A27,INVALC,INVALC,INVALC,INVALC,0x3A28,INVALC,INVALC,
	INVALC,INVALC,0x3A29,INVALC,INVALC,INVALC,0x3A2A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3A2B,0x3A2C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3A2D,INVALC,INVALC,
	/* Block 92, Array index 0x1400 */
	0x3A2E,0x3A2F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A30,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A31,
	INVALC,0x3A33,INVALC,0x3A34,INVALC,0x3A35,INVALC,INVALC,
	INVALC,0x3A36,INVALC,INVALC,INVALC,0x3A37,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3A38,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A32,INVALC,INVALC,INVALC,
	0x3A39,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3A3A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3A3B,0x3A3C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3A3D,INVALC,INVALC,INVALC,0x3A3E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A3F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A40,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A41,0x3A42,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A43,0x3A44,0x3A45,0x3A46,
	INVALC,0x3A47,INVALC,INVALC,0x3A48,INVALC,0x3A49,INVALC,
	INVALC,INVALC,0x3A4A,INVALC,INVALC,INVALC,0x3A4B,INVALC,
	0x3A4C,0x3A4D,INVALC,0x3A4E,0x3A4F,INVALC,0x3A50,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A51,0x3A52,INVALC,INVALC,0x3A53,0x3A54,INVALC,0x3A55,
	INVALC,0x3A56,0x3A57,INVALC,INVALC,INVALC,INVALC,0x3A58,
	INVALC,INVALC,0x3A59,INVALC,0x3A5A,INVALC,INVALC,INVALC,
	0x3A5B,0x3A5C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3A5D,INVALC,0x3A5E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A5F,0x3A60,
	0x3A61,0x3A62,0x3A63,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3A64,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A65,
	INVALC,0x3A66,INVALC,INVALC,0x3A67,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3A68,INVALC,INVALC,
	/* Block 93, Array index 0x1500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3A69,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A6A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3A6B,0x3A6C,
	INVALC,INVALC,INVALC,0x3A6D,INVALC,INVALC,0x3A6E,INVALC,
	INVALC,0x3A6F,INVALC,0x3A70,0x3A71,INVALC,0x3A72,INVALC,
	0x3A73,INVALC,0x3A74,INVALC,INVALC,0x3A75,0x3A76,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3A77,0x3A78,INVALC,0x3A79,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3A7A,0x3A7B,INVALC,INVALC,INVALC,0x3A7C,
	0x3A7D,0x3A7E,INVALC,INVALC,INVALC,0x3B21,INVALC,INVALC,
	0x3B22,INVALC,INVALC,INVALC,0x3B23,0x3B24,INVALC,INVALC,
	INVALC,INVALC,0x3B25,0x3B26,0x3B27,0x3B28,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B29,
	0x3B2A,INVALC,0x3B2B,INVALC,INVALC,INVALC,INVALC,0x3B2C,
	INVALC,INVALC,0x3B2D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3B2E,INVALC,0x3B2F,
	0x3B30,INVALC,0x3B31,0x3B32,INVALC,INVALC,0x3B33,INVALC,
	INVALC,INVALC,0x3B34,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3B35,INVALC,0x3B36,0x3B37,INVALC,INVALC,
	INVALC,INVALC,0x3B38,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3B39,INVALC,0x3B3A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3B3B,INVALC,INVALC,INVALC,INVALC,0x3B3D,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3B3C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B3E,INVALC,
	INVALC,0x3B3F,0x3B40,INVALC,0x3B41,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 94, Array index 0x1600 */
	INVALC,0x3B42,INVALC,INVALC,INVALC,INVALC,0x3B43,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3B44,INVALC,INVALC,
	INVALC,INVALC,0x3B45,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3B47,0x3B48,INVALC,0x3B49,0x3B4A,
	INVALC,INVALC,INVALC,0x3B46,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3B4B,INVALC,INVALC,0x3B4C,INVALC,INVALC,INVALC,
	INVALC,0x3B4D,INVALC,INVALC,INVALC,0x3B4E,INVALC,0x3B4F,
	INVALC,INVALC,0x3B50,0x3B51,INVALC,INVALC,0x3B52,INVALC,
	0x3B53,INVALC,0x3B57,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3B55,INVALC,INVALC,INVALC,0x3B54,INVALC,INVALC,
	INVALC,0x3B56,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B58,
	0x3B59,0x3B5A,0x3B5B,INVALC,0x3B5C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3B5D,INVALC,INVALC,0x3B5E,INVALC,INVALC,
	0x3B5F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3B60,0x3B61,INVALC,INVALC,INVALC,0x3B62,0x3B63,
	INVALC,0x3B64,INVALC,0x3B65,INVALC,INVALC,INVALC,INVALC,
	0x3B66,INVALC,0x3B67,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3B68,0x3B69,0x3B6A,0x3B6B,INVALC,INVALC,
	INVALC,0x3B6C,INVALC,0x3B6D,INVALC,INVALC,INVALC,0x3B6E,
	0x3B6F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3B70,
	INVALC,INVALC,INVALC,0x3B71,INVALC,INVALC,INVALC,INVALC,
	0x3B72,INVALC,0x6674,INVALC,0x3B73,INVALC,INVALC,INVALC,
	0x3B74,0x3B75,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3B76,INVALC,INVALC,INVALC,0x3B77,
	INVALC,INVALC,INVALC,0x3B78,INVALC,INVALC,0x3B7A,INVALC,
	0x3B79,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B7B,0x3B7C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3B7D,INVALC,INVALC,INVALC,0x3B7E,INVALC,INVALC,INVALC,
	INVALC,0x3C21,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C22,0x3C23,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 95, Array index 0x1700 */
	0x3C24,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C25,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C26,INVALC,INVALC,
	INVALC,INVALC,0x3C27,INVALC,0x3C28,0x3C29,INVALC,INVALC,
	0x3C2A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C2B,
	0x3C2C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C2E,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C2D,INVALC,
	INVALC,INVALC,0x3C2F,INVALC,INVALC,0x3C30,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C31,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3C34,INVALC,0x3C32,INVALC,INVALC,INVALC,INVALC,
	0x3C33,0x3C35,INVALC,INVALC,INVALC,INVALC,0x3C36,INVALC,
	0x3C37,INVALC,INVALC,0x3C38,0x3C39,INVALC,0x3C3A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C3B,INVALC,0x3C3C,0x3C3D,0x3C3E,0x3C3F,0x3C40,INVALC,
	0x3C41,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3C42,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3C43,INVALC,INVALC,0x3C44,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3C45,INVALC,0x3C46,0x3C47,INVALC,INVALC,
	0x3C48,INVALC,0x3C49,INVALC,0x3C4A,INVALC,INVALC,INVALC,
	INVALC,0x3C4B,INVALC,0x3C4C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C4D,0x3C4E,0x3C4F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C50,INVALC,
	INVALC,INVALC,INVALC,0x3C52,0x3C51,INVALC,0x3C53,INVALC,
	INVALC,0x3C54,0x3C55,INVALC,0x3C56,0x3C57,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3C58,INVALC,0x3C59,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C5A,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3C5B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 96, Array index 0x1800 */
	INVALC,INVALC,0x3C5C,INVALC,INVALC,INVALC,0x3C5D,0x3C5E,
	0x3C5F,INVALC,INVALC,INVALC,INVALC,INVALC,0x3C60,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3C61,INVALC,INVALC,0x3C62,0x3C63,INVALC,INVALC,
	INVALC,0x3C64,0x3C65,0x3C66,0x3C67,INVALC,INVALC,INVALC,
	0x3C68,INVALC,INVALC,0x3C69,0x3C6A,INVALC,0x3C6B,INVALC,
	0x3C6C,INVALC,INVALC,INVALC,0x3C6D,INVALC,0x3C6E,INVALC,
	INVALC,INVALC,INVALC,0x3C6F,INVALC,0x3C70,INVALC,0x3C71,
	0x3C72,INVALC,INVALC,INVALC,0x3C73,0x3C74,INVALC,0x3C75,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3C76,INVALC,INVALC,
	0x3C77,INVALC,INVALC,INVALC,0x3C78,INVALC,INVALC,INVALC,
	0x3C79,INVALC,INVALC,0x3C7A,INVALC,INVALC,INVALC,INVALC,
	0x3C7B,INVALC,INVALC,0x3C7C,0x3C7D,INVALC,INVALC,0x3C7E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D21,
	INVALC,INVALC,0x3D22,INVALC,0x3D23,0x3D24,INVALC,INVALC,
	0x3D25,INVALC,0x3D26,INVALC,INVALC,0x3D27,0x3D28,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3D29,INVALC,INVALC,INVALC,0x3D2A,INVALC,0x3D2B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D2C,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D2D,0x3D2E,INVALC,INVALC,
	INVALC,INVALC,0x3D2F,INVALC,0x3D32,INVALC,INVALC,0x3D30,
	INVALC,INVALC,INVALC,0x3D31,0x3D33,INVALC,INVALC,0x3D34,
	0x3D35,0x3D36,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D37,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3D38,INVALC,INVALC,0x3D39,
	0x3D3A,0x3D3B,INVALC,0x3D3C,INVALC,INVALC,INVALC,INVALC,
	0x3D3D,0x3D3E,0x3D3F,0x3D40,0x3D41,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D42,INVALC,
	INVALC,0x3D43,0x3D44,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D45,0x3D46,0x3D47,INVALC,0x3D48,0x3D49,0x3D4A,0x3D4B,
	INVALC,INVALC,0x3D4C,0x3D4D,INVALC,INVALC,0x3D4E,INVALC,
	INVALC,INVALC,0x3D4F,INVALC,0x3D50,INVALC,INVALC,INVALC,
	/* Block 97, Array index 0x1900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D51,
	INVALC,INVALC,0x3D52,INVALC,INVALC,0x3D53,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3D54,0x3D55,INVALC,INVALC,0x3D56,
	0x3D57,INVALC,0x3D58,INVALC,INVALC,INVALC,INVALC,0x3D59,
	INVALC,INVALC,INVALC,INVALC,0x3D5A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3D5B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3D5C,INVALC,0x3D5D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D5E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D5F,
	0x3D60,0x3D61,INVALC,INVALC,0x3D62,INVALC,INVALC,INVALC,
	INVALC,0x3D63,INVALC,INVALC,0x3D64,INVALC,0x3D65,0x3D66,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3D67,INVALC,INVALC,
	INVALC,0x3D68,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D69,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3D6A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D6B,0x3D6C,
	INVALC,INVALC,0x3D6D,INVALC,INVALC,INVALC,0x3D6E,INVALC,
	0x3D6F,INVALC,0x3D70,INVALC,INVALC,INVALC,0x3D71,INVALC,
	INVALC,0x3D72,INVALC,0x3D73,INVALC,0x3D74,INVALC,INVALC,
	0x3D75,INVALC,INVALC,INVALC,INVALC,0x3D76,0x3D77,INVALC,
	INVALC,INVALC,0x3D78,INVALC,0x3D79,0x3D7A,INVALC,INVALC,
	0x3D7B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3D7C,0x3D7D,INVALC,INVALC,INVALC,INVALC,INVALC,0x3D7E,
	INVALC,INVALC,INVALC,0x3E21,INVALC,INVALC,0x3E22,INVALC,
	INVALC,INVALC,0x3E23,INVALC,0x3E24,INVALC,INVALC,INVALC,
	0x3E25,0x3E26,0x3E27,0x3E28,INVALC,INVALC,0x3E29,0x3E2A,
	INVALC,INVALC,INVALC,INVALC,0x3E2B,0x3E2C,INVALC,INVALC,
	INVALC,0x3E2D,INVALC,0x3E2E,INVALC,0x3E2F,0x3E30,INVALC,
	INVALC,INVALC,0x3E31,INVALC,INVALC,0x3E32,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3E33,INVALC,INVALC,0x3E34,INVALC,INVALC,
	0x3E35,INVALC,INVALC,INVALC,0x3E36,0x3E37,INVALC,INVALC,
	/* Block 98, Array index 0x1A00 */
	INVALC,INVALC,0x3E38,INVALC,INVALC,INVALC,INVALC,0x3E39,
	INVALC,INVALC,0x3E3A,INVALC,0x3E3B,INVALC,INVALC,INVALC,
	0x3E3C,0x3E3D,0x3E3E,0x3E3F,0x3E40,INVALC,0x3E41,0x3E42,
	INVALC,0x3E43,INVALC,INVALC,0x3E44,INVALC,0x3E45,INVALC,
	INVALC,INVALC,0x3E46,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3E47,0x3E48,INVALC,INVALC,INVALC,INVALC,
	0x3E49,0x3E4A,INVALC,INVALC,INVALC,0x3E4B,0x3E4C,0x3E4D,
	INVALC,0x3E4E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3E4F,INVALC,INVALC,INVALC,0x3E50,0x3E51,INVALC,
	INVALC,0x3E52,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E53,INVALC,
	0x3E54,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E55,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3E56,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E57,INVALC,INVALC,0x3E58,0x3E59,INVALC,INVALC,0x3E5A,
	0x3E5B,0x3E5C,INVALC,0x3E5D,0x3E5E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3E5F,INVALC,0x3E60,INVALC,INVALC,
	INVALC,INVALC,0x3E61,0x3E62,INVALC,INVALC,INVALC,0x3E63,
	0x3E64,INVALC,INVALC,INVALC,0x3E65,0x3E66,INVALC,0x3E67,
	0x3E68,INVALC,INVALC,INVALC,INVALC,0x3E69,INVALC,INVALC,
	0x3E6A,INVALC,0x3E6B,INVALC,INVALC,0x3E6C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x3E6D,0x3E6E,INVALC,INVALC,0x3E6F,INVALC,INVALC,INVALC,
	0x3E70,0x3E71,0x3E72,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3E73,0x3E74,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3E75,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3E76,0x3E77,0x3E78,0x3E79,
	INVALC,0x3E7A,0x3E7B,INVALC,INVALC,0x3E7E,INVALC,0x3E7C,
	INVALC,0x3E7D,INVALC,INVALC,0x3F21,0x3F22,INVALC,0x3F23,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 99, Array index 0x1B00 */
	INVALC,0x3F24,INVALC,0x3F25,0x3F26,INVALC,INVALC,0x3F27,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3F28,INVALC,0x3F29,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F2A,INVALC,INVALC,INVALC,
	0x3F2B,INVALC,0x3F2C,0x3F2D,INVALC,INVALC,INVALC,0x3F2E,
	INVALC,0x3F2F,INVALC,0x3F30,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3F31,INVALC,INVALC,INVALC,0x3F32,INVALC,INVALC,
	INVALC,INVALC,0x3F33,0x3F34,0x3F35,INVALC,0x3F36,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F37,INVALC,INVALC,INVALC,
	0x3F38,0x3F39,0x3F3A,INVALC,0x3F3B,INVALC,0x3F3C,INVALC,
	INVALC,INVALC,0x3F3D,INVALC,0x3F3E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F3F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F40,INVALC,0x3F41,
	INVALC,INVALC,INVALC,0x3F42,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F43,INVALC,INVALC,0x3F44,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F45,0x3F46,0x3F47,
	0x3F48,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3F49,INVALC,
	0x3F4A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x3F4B,INVALC,INVALC,0x3F4C,0x3F4D,
	INVALC,INVALC,0x3F4E,INVALC,INVALC,INVALC,0x3F4F,0x3F50,
	INVALC,INVALC,INVALC,INVALC,0x3F51,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x3F52,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F53,0x3F54,INVALC,
	INVALC,INVALC,0x3F55,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F56,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F57,INVALC,0x3F58,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F59,0x3F5A,INVALC,INVALC,
	/* Block 100, Array index 0x1C00 */
	INVALC,INVALC,INVALC,0x3F5B,0x3F5C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x3F5D,0x3F5E,INVALC,0x3F5F,INVALC,INVALC,
	0x3F60,INVALC,INVALC,0x3F61,INVALC,INVALC,0x3F62,INVALC,
	0x3F63,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x3F64,0x3F65,INVALC,INVALC,0x3F66,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F67,0x3F68,INVALC,
	INVALC,0x3F69,INVALC,INVALC,0x3F6A,INVALC,INVALC,INVALC,
	INVALC,0x3F6B,0x3F6C,0x3F6D,0x3F6E,INVALC,0x3F6F,INVALC,
	INVALC,INVALC,0x3F70,0x3F71,INVALC,INVALC,0x3F72,INVALC,
	INVALC,INVALC,0x3F73,0x3F74,0x3F75,INVALC,INVALC,INVALC,
	0x3F76,INVALC,INVALC,0x3F77,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x3F78,0x3F79,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x3F7A,0x3F7B,INVALC,INVALC,
	INVALC,0x3F7C,INVALC,INVALC,0x3F7D,0x3F7E,INVALC,INVALC,
	0x4021,INVALC,INVALC,INVALC,0x4022,0x4023,INVALC,0x4024,
	INVALC,INVALC,0x4025,INVALC,0x4026,INVALC,INVALC,0x4027,
	INVALC,INVALC,0x4028,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4029,INVALC,INVALC,INVALC,0x402A,0x402B,INVALC,INVALC,
	INVALC,0x402C,0x402D,INVALC,INVALC,INVALC,0x402E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x402F,INVALC,0x4030,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4031,0x4032,0x4033,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4034,INVALC,INVALC,
	INVALC,0x4035,INVALC,INVALC,INVALC,0x4036,INVALC,INVALC,
	0x4037,INVALC,INVALC,INVALC,INVALC,INVALC,0x4038,INVALC,
	INVALC,0x4039,INVALC,0x403A,0x403B,0x403C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x403D,INVALC,INVALC,INVALC,
	0x403E,INVALC,INVALC,INVALC,INVALC,0x403F,INVALC,INVALC,
	INVALC,INVALC,0x4040,INVALC,0x4041,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4042,0x4043,INVALC,0x4044,INVALC,INVALC,
	0x4045,0x4046,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 101, Array index 0x1D00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4047,0x4048,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4049,INVALC,0x404A,INVALC,0x404B,INVALC,INVALC,INVALC,
	0x404C,INVALC,INVALC,INVALC,INVALC,INVALC,0x404D,INVALC,
	0x404E,INVALC,0x404F,INVALC,0x4050,0x4051,INVALC,INVALC,
	INVALC,0x4052,0x4053,INVALC,INVALC,INVALC,INVALC,0x4054,
	0x4055,INVALC,INVALC,INVALC,INVALC,0x4056,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4057,INVALC,0x4058,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4059,INVALC,INVALC,INVALC,0x405A,
	INVALC,0x405B,0x405C,0x405D,0x405E,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x405F,0x4060,0x4061,0x4062,INVALC,0x4063,
	0x4064,0x4065,INVALC,0x4066,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4067,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4068,0x4069,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x406A,INVALC,0x406B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x406C,INVALC,0x406D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x406E,
	0x406F,0x4070,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4071,0x4072,INVALC,0x4073,INVALC,0x4074,INVALC,0x4075,
	INVALC,0x4076,INVALC,0x4077,INVALC,INVALC,0x4078,INVALC,
	0x4079,INVALC,INVALC,INVALC,0x407A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x407B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x407C,0x407D,0x407E,
	INVALC,INVALC,INVALC,INVALC,0x4121,INVALC,INVALC,INVALC,
	INVALC,0x4122,0x4123,0x4124,0x4125,INVALC,0x4126,INVALC,
	0x4127,0x4128,INVALC,INVALC,INVALC,0x4129,INVALC,0x412A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x412B,0x412C,INVALC,INVALC,
	INVALC,0x412D,0x412E,INVALC,INVALC,0x412F,INVALC,INVALC,
	0x4130,INVALC,INVALC,INVALC,INVALC,0x4131,INVALC,0x4132,
	INVALC,INVALC,INVALC,0x4133,INVALC,INVALC,INVALC,0x4134,
	/* Block 102, Array index 0x1E00 */
	INVALC,0x4135,INVALC,INVALC,0x4136,INVALC,INVALC,INVALC,
	0x4137,0x4138,0x4139,INVALC,INVALC,INVALC,INVALC,0x413A,
	INVALC,INVALC,INVALC,0x413B,0x413C,INVALC,0x413D,INVALC,
	INVALC,0x413E,INVALC,0x413F,INVALC,INVALC,0x4140,0x4141,
	INVALC,INVALC,0x4142,INVALC,INVALC,INVALC,0x4143,INVALC,
	INVALC,0x4144,INVALC,INVALC,INVALC,INVALC,0x4145,INVALC,
	INVALC,0x4146,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4147,INVALC,0x4148,0x4149,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x414A,INVALC,INVALC,INVALC,0x414B,
	INVALC,0x414C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x414D,INVALC,0x414E,INVALC,0x414F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4150,
	0x4151,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4152,
	INVALC,INVALC,INVALC,0x4153,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4154,INVALC,INVALC,0x4155,INVALC,INVALC,
	INVALC,0x4156,INVALC,INVALC,INVALC,0x4157,INVALC,INVALC,
	INVALC,INVALC,0x4158,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4159,INVALC,INVALC,0x415A,INVALC,INVALC,
	0x415B,INVALC,INVALC,INVALC,INVALC,0x415C,INVALC,INVALC,
	0x415D,INVALC,INVALC,0x415E,INVALC,INVALC,0x415F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4160,INVALC,
	INVALC,INVALC,0x4161,0x4162,0x4163,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4164,
	INVALC,INVALC,0x4165,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4166,0x4167,INVALC,INVALC,INVALC,INVALC,0x4168,INVALC,
	0x4169,INVALC,INVALC,INVALC,0x416A,INVALC,0x416B,INVALC,
	0x416C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x416D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x416E,INVALC,0x416F,INVALC,0x4170,0x4171,INVALC,INVALC,
	INVALC,0x4172,INVALC,INVALC,INVALC,INVALC,0x4173,0x4174,
	/* Block 103, Array index 0x1F00 */
	0x4175,INVALC,INVALC,INVALC,0x4176,INVALC,INVALC,INVALC,
	0x4177,0x4178,INVALC,INVALC,INVALC,0x4179,INVALC,INVALC,
	INVALC,0x417A,0x417B,INVALC,INVALC,0x417C,0x417D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x417E,0x4221,INVALC,
	INVALC,0x4222,0x4223,0x4224,0x4225,INVALC,0x4226,INVALC,
	INVALC,0x4227,0x4228,0x4229,0x422A,INVALC,0x422B,INVALC,
	0x422C,0x422D,INVALC,0x422E,INVALC,INVALC,INVALC,0x4230,
	INVALC,0x422F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4231,INVALC,INVALC,INVALC,INVALC,0x4232,0x4233,INVALC,
	INVALC,INVALC,0x4234,INVALC,0x4235,INVALC,0x4237,INVALC,
	INVALC,0x4236,INVALC,INVALC,INVALC,INVALC,INVALC,0x4238,
	0x4239,0x423A,INVALC,0x423B,0x423C,INVALC,INVALC,INVALC,
	0x423D,0x423E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4240,0x4241,0x4242,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4244,INVALC,0x4245,INVALC,0x4247,
	0x4248,0x4249,INVALC,0x424A,0x424C,INVALC,0x4243,0x4246,
	0x424B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x424D,0x424E,0x424F,INVALC,INVALC,
	0x4250,INVALC,0x4251,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4252,0x4253,0x4254,0x4255,INVALC,INVALC,0x4256,
	0x4257,INVALC,INVALC,INVALC,0x4258,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4259,INVALC,INVALC,
	INVALC,0x425A,0x425B,INVALC,INVALC,0x425C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x425D,INVALC,INVALC,INVALC,0x425E,
	0x425F,INVALC,0x4260,0x4261,INVALC,INVALC,INVALC,INVALC,
	0x4262,INVALC,INVALC,INVALC,0x4263,INVALC,0x4264,0x4265,
	INVALC,INVALC,INVALC,INVALC,0x4266,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4267,INVALC,INVALC,INVALC,0x4268,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4269,INVALC,INVALC,0x426A,0x426B,INVALC,0x426C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x426D,0x423F,INVALC,INVALC,INVALC,INVALC,
	/* Block 104, Array index 0x2000 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x426E,INVALC,0x426F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4270,INVALC,
	INVALC,INVALC,INVALC,0x4271,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4272,INVALC,INVALC,0x4273,INVALC,INVALC,INVALC,
	0x4274,INVALC,0x4275,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4276,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4277,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4278,INVALC,0x4279,
	0x427A,INVALC,INVALC,INVALC,0x427B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x427C,0x427D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x427E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4321,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4322,INVALC,0x4323,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4324,INVALC,0x4325,INVALC,
	INVALC,INVALC,INVALC,0x4326,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4327,INVALC,INVALC,0x4328,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4329,0x432A,
	INVALC,0x432B,INVALC,0x432C,INVALC,INVALC,0x432D,INVALC,
	0x432E,0x432F,INVALC,0x4330,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4331,0x4332,0x4333,INVALC,INVALC,0x4334,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4335,0x4336,0x4337,INVALC,INVALC,
	0x4339,INVALC,0x433A,0x433B,INVALC,0x433C,INVALC,INVALC,
	0x433D,0x433E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x433F,INVALC,INVALC,INVALC,INVALC,0x4340,
	INVALC,0x4341,INVALC,INVALC,0x4342,INVALC,INVALC,INVALC,
	INVALC,0x4343,INVALC,INVALC,INVALC,INVALC,0x4344,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4345,INVALC,0x4346,INVALC,INVALC,INVALC,0x4347,0x4348,
	INVALC,0x4338,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 105, Array index 0x2100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x434A,INVALC,INVALC,INVALC,
	INVALC,0x434B,INVALC,INVALC,INVALC,0x434C,INVALC,0x434D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x434F,
	0x434E,INVALC,INVALC,INVALC,0x4350,0x4351,INVALC,0x4352,
	0x4353,0x4354,INVALC,0x4355,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4356,INVALC,INVALC,INVALC,0x4357,
	INVALC,INVALC,INVALC,INVALC,0x4358,0x4359,INVALC,INVALC,
	INVALC,INVALC,0x435A,INVALC,0x435B,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4349,INVALC,INVALC,0x435C,INVALC,0x435D,
	0x435E,INVALC,INVALC,0x435F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4360,INVALC,INVALC,0x4361,0x4362,
	0x4363,0x4364,0x4365,INVALC,INVALC,0x4366,INVALC,INVALC,
	INVALC,0x4367,0x4368,0x4369,0x436A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x436B,INVALC,0x436C,INVALC,0x436D,INVALC,
	0x436E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x436F,INVALC,
	0x4370,INVALC,0x4371,INVALC,0x4372,INVALC,INVALC,INVALC,
	INVALC,0x4373,INVALC,0x4374,INVALC,0x4375,INVALC,INVALC,
	INVALC,0x4376,0x4377,INVALC,INVALC,INVALC,0x4378,INVALC,
	INVALC,INVALC,0x4379,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x437A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x437B,INVALC,INVALC,
	0x437C,INVALC,INVALC,INVALC,0x437D,INVALC,INVALC,0x437E,
	0x4421,0x4422,INVALC,0x4423,INVALC,INVALC,0x4424,INVALC,
	INVALC,0x4425,INVALC,INVALC,0x4426,0x4427,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4428,INVALC,INVALC,
	0x4429,INVALC,0x442A,0x442B,0x442C,0x442D,INVALC,INVALC,
	0x442E,0x442F,INVALC,INVALC,INVALC,0x4430,0x4431,INVALC,
	INVALC,INVALC,INVALC,0x4432,0x4433,0x4434,INVALC,INVALC,
	/* Block 106, Array index 0x2200 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4435,INVALC,
	INVALC,0x4436,0x4437,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4438,0x4439,INVALC,0x443A,INVALC,INVALC,0x443B,0x443C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x443D,INVALC,0x443E,INVALC,0x443F,INVALC,INVALC,
	0x4440,INVALC,INVALC,0x4441,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4442,INVALC,INVALC,0x4443,INVALC,INVALC,
	INVALC,0x4444,INVALC,INVALC,INVALC,INVALC,0x4445,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4446,
	INVALC,INVALC,INVALC,0x4447,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4448,0x4449,0x444A,0x444B,INVALC,
	0x444C,0x444D,INVALC,INVALC,0x444E,INVALC,INVALC,INVALC,
	0x444F,0x4450,0x4451,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4452,0x4453,INVALC,INVALC,INVALC,0x4454,
	0x4455,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4456,INVALC,INVALC,
	INVALC,INVALC,0x4457,INVALC,INVALC,INVALC,0x4458,INVALC,
	0x4459,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x445A,INVALC,INVALC,INVALC,0x445B,0x445C,INVALC,
	0x445D,INVALC,INVALC,0x445E,INVALC,0x445F,INVALC,0x4460,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4461,
	0x4462,INVALC,0x4463,INVALC,0x4464,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4465,INVALC,INVALC,
	0x4466,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4467,INVALC,INVALC,INVALC,INVALC,0x4468,0x4469,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x446A,INVALC,INVALC,0x446B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x446C,0x446D,
	INVALC,0x446E,INVALC,0x446F,INVALC,0x4470,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4471,INVALC,
	/* Block 107, Array index 0x2300 */
	0x4472,0x4473,INVALC,0x4474,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4475,
	INVALC,0x4476,INVALC,INVALC,INVALC,0x4477,INVALC,INVALC,
	INVALC,INVALC,0x4478,INVALC,INVALC,0x4479,INVALC,INVALC,
	0x447A,INVALC,INVALC,INVALC,0x447B,INVALC,INVALC,INVALC,
	0x447C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x447D,INVALC,0x447E,INVALC,0x4521,INVALC,INVALC,0x4522,
	INVALC,INVALC,INVALC,0x4523,INVALC,INVALC,0x4524,0x4525,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4526,0x4527,INVALC,
	INVALC,0x4528,0x4529,INVALC,INVALC,INVALC,0x452A,INVALC,
	0x452B,0x452C,0x452D,INVALC,INVALC,0x452E,0x452F,INVALC,
	INVALC,INVALC,INVALC,0x4530,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4531,INVALC,INVALC,0x4532,
	INVALC,INVALC,0x4533,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4534,INVALC,0x4535,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4536,INVALC,
	INVALC,0x4537,INVALC,0x4538,INVALC,INVALC,0x4539,0x453A,
	INVALC,0x453B,INVALC,0x453C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x453D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x453E,INVALC,0x453F,0x4540,0x4541,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4542,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4543,INVALC,INVALC,INVALC,0x4544,
	0x4545,0x4546,INVALC,INVALC,0x4547,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4548,0x4549,0x454A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x454B,INVALC,
	0x454D,0x454C,INVALC,INVALC,0x454E,INVALC,INVALC,INVALC,
	0x454F,INVALC,INVALC,INVALC,0x4550,0x4551,0x4552,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4553,0x4554,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4555,INVALC,INVALC,0x4556,
	INVALC,INVALC,INVALC,INVALC,0x4557,INVALC,INVALC,INVALC,
	0x4558,0x4559,0x455A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x455B,0x455C,INVALC,INVALC,
	/* Block 108, Array index 0x2400 */
	INVALC,INVALC,0x455D,0x455E,INVALC,INVALC,0x455F,0x4560,
	INVALC,0x4561,INVALC,INVALC,INVALC,INVALC,INVALC,0x4562,
	0x4563,0x4564,INVALC,INVALC,INVALC,INVALC,INVALC,0x4565,
	INVALC,INVALC,INVALC,0x4566,INVALC,INVALC,0x4567,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4568,INVALC,INVALC,INVALC,
	0x4569,INVALC,INVALC,0x456A,0x456B,INVALC,INVALC,0x456C,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x456D,0x456E,INVALC,
	INVALC,INVALC,INVALC,0x456F,INVALC,INVALC,INVALC,0x4570,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4571,INVALC,
	INVALC,INVALC,0x4572,INVALC,INVALC,0x4573,INVALC,INVALC,
	INVALC,INVALC,0x4574,INVALC,INVALC,INVALC,0x4575,INVALC,
	0x4576,INVALC,INVALC,INVALC,INVALC,0x4577,INVALC,INVALC,
	0x4578,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4579,
	INVALC,INVALC,INVALC,0x457A,INVALC,INVALC,0x457B,INVALC,
	0x457C,INVALC,INVALC,INVALC,INVALC,0x457D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x457E,0x4621,INVALC,
	INVALC,INVALC,0x4622,INVALC,INVALC,0x4623,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4624,
	INVALC,INVALC,INVALC,0x4625,INVALC,INVALC,INVALC,0x4626,
	0x4627,INVALC,0x4628,0x4629,INVALC,INVALC,INVALC,INVALC,
	0x462A,0x462B,INVALC,INVALC,0x462C,0x462D,0x462E,INVALC,
	0x462F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4630,0x4631,INVALC,INVALC,INVALC,0x4632,0x4633,INVALC,
	INVALC,INVALC,INVALC,0x4634,0x4635,INVALC,INVALC,INVALC,
	INVALC,0x4636,INVALC,INVALC,0x4637,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4638,INVALC,
	INVALC,INVALC,0x4639,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x463A,INVALC,0x463B,INVALC,INVALC,0x463C,0x463D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x463E,INVALC,
	INVALC,0x463F,INVALC,INVALC,INVALC,INVALC,INVALC,0x4640,
	INVALC,0x4641,INVALC,INVALC,INVALC,INVALC,INVALC,0x4642,
	/* Block 109, Array index 0x2500 */
	INVALC,INVALC,0x4643,INVALC,0x4644,0x4645,INVALC,INVALC,
	INVALC,0x4646,INVALC,INVALC,INVALC,0x4647,0x4648,INVALC,
	0x4649,INVALC,0x464A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x464B,INVALC,INVALC,INVALC,INVALC,INVALC,0x464C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x464D,0x464E,
	0x464F,INVALC,INVALC,INVALC,0x4650,INVALC,0x4651,INVALC,
	INVALC,INVALC,INVALC,0x4652,INVALC,0x4653,0x4654,INVALC,
	INVALC,INVALC,0x4655,0x4656,INVALC,INVALC,INVALC,0x4657,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4658,0x4659,INVALC,0x465A,INVALC,0x465B,
	INVALC,INVALC,0x465C,INVALC,0x465D,INVALC,INVALC,INVALC,
	INVALC,0x465E,INVALC,0x465F,0x4660,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4736,
	INVALC,INVALC,INVALC,0x4661,INVALC,0x4662,INVALC,0x4663,
	INVALC,INVALC,INVALC,INVALC,0x4664,INVALC,0x4665,INVALC,
	0x4666,0x4667,INVALC,0x4668,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4669,0x466A,0x466B,
	INVALC,0x466C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x466D,0x466E,INVALC,0x466F,
	0x4670,INVALC,INVALC,INVALC,0x4671,INVALC,INVALC,0x4672,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4673,INVALC,0x4674,INVALC,0x4675,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4676,INVALC,INVALC,INVALC,0x4677,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4678,INVALC,0x4679,
	0x467A,0x467B,0x467C,INVALC,0x467D,INVALC,0x467E,INVALC,
	INVALC,INVALC,0x4721,INVALC,0x4722,INVALC,INVALC,INVALC,
	0x4723,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4724,
	INVALC,0x4725,INVALC,0x4726,0x4727,INVALC,0x4728,INVALC,
	INVALC,INVALC,0x4729,INVALC,0x472A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 110, Array index 0x2600 */
	INVALC,INVALC,INVALC,INVALC,0x472B,INVALC,INVALC,0x472C,
	INVALC,INVALC,0x472D,INVALC,INVALC,INVALC,0x472E,0x472F,
	INVALC,0x4730,INVALC,0x4731,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4732,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4733,0x4734,
	0x4735,INVALC,INVALC,INVALC,INVALC,INVALC,0x4737,0x4738,
	INVALC,0x4739,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x473A,INVALC,INVALC,0x473B,INVALC,INVALC,0x473C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x473D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x473E,0x473F,
	INVALC,0x4740,INVALC,INVALC,INVALC,0x4741,INVALC,0x4742,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4743,
	0x4744,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4745,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4746,INVALC,INVALC,INVALC,INVALC,0x4747,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4748,0x4749,INVALC,0x474A,INVALC,0x474B,0x474C,
	0x474D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x474E,INVALC,0x474F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4750,INVALC,INVALC,0x4751,
	INVALC,0x4752,INVALC,INVALC,INVALC,0x4753,INVALC,0x4754,
	INVALC,INVALC,INVALC,INVALC,0x4755,INVALC,INVALC,INVALC,
	0x4756,INVALC,0x4757,INVALC,INVALC,INVALC,0x4758,0x4759,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x475A,INVALC,
	INVALC,INVALC,INVALC,0x475B,INVALC,0x475C,INVALC,0x475D,
	0x475E,INVALC,0x475F,INVALC,INVALC,0x4760,INVALC,INVALC,
	INVALC,0x4761,INVALC,INVALC,INVALC,INVALC,INVALC,0x4762,
	0x4763,INVALC,INVALC,INVALC,0x4764,INVALC,0x4765,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4766,INVALC,INVALC,INVALC,
	/* Block 111, Array index 0x2700 */
	0x4767,INVALC,INVALC,INVALC,0x4768,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4769,INVALC,
	INVALC,INVALC,0x476A,INVALC,INVALC,INVALC,INVALC,0x476B,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x476C,INVALC,INVALC,INVALC,0x476D,INVALC,
	INVALC,0x476E,INVALC,0x476F,0x4770,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4771,0x4772,INVALC,INVALC,
	0x4773,0x4774,INVALC,0x4775,INVALC,INVALC,INVALC,0x4776,
	INVALC,0x4777,0x4778,0x4779,0x477A,INVALC,INVALC,0x477B,
	INVALC,INVALC,INVALC,INVALC,0x477C,0x477D,0x477E,INVALC,
	INVALC,INVALC,0x4821,0x4822,INVALC,INVALC,INVALC,INVALC,
	0x4823,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4824,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4825,INVALC,0x4826,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4827,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4828,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4829,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x482A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x482B,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x482C,0x482D,INVALC,
	INVALC,0x482E,INVALC,0x482F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4830,INVALC,INVALC,INVALC,0x4831,
	0x4832,0x4833,INVALC,0x4834,INVALC,INVALC,INVALC,0x4835,
	0x4836,INVALC,0x4837,INVALC,INVALC,0x4838,0x4839,0x483A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x483B,INVALC,0x483C,0x483D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x483E,INVALC,0x483F,INVALC,0x4840,INVALC,INVALC,
	/* Block 112, Array index 0x2800 */
	INVALC,INVALC,INVALC,INVALC,0x4841,INVALC,INVALC,INVALC,
	0x4842,INVALC,0x4843,INVALC,0x4844,0x4845,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4846,INVALC,
	0x4847,INVALC,0x4848,0x4849,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x484A,INVALC,
	INVALC,0x484B,0x484C,INVALC,INVALC,0x4853,INVALC,0x484D,
	0x484E,INVALC,INVALC,0x484F,INVALC,INVALC,0x4850,INVALC,
	INVALC,INVALC,INVALC,0x4851,0x4852,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4854,
	INVALC,0x4855,0x4856,0x4857,INVALC,INVALC,INVALC,0x4858,
	INVALC,0x4859,0x485A,INVALC,INVALC,0x485B,0x485C,INVALC,
	INVALC,0x485D,0x485E,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x485F,INVALC,INVALC,INVALC,0x4860,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4861,0x4862,INVALC,
	INVALC,INVALC,INVALC,0x4863,INVALC,INVALC,INVALC,0x4864,
	0x4865,INVALC,INVALC,0x4866,0x4867,0x4868,INVALC,INVALC,
	0x4869,INVALC,0x486A,0x486B,0x486C,INVALC,0x486D,INVALC,
	INVALC,INVALC,0x486E,INVALC,INVALC,INVALC,INVALC,0x486F,
	0x4870,INVALC,INVALC,INVALC,INVALC,0x4871,0x4872,0x4873,
	0x4874,INVALC,INVALC,INVALC,INVALC,INVALC,0x4875,0x4876,
	0x4877,INVALC,INVALC,INVALC,INVALC,0x4878,0x4879,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x487A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x487B,INVALC,0x487C,
	0x487D,INVALC,0x487E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4921,INVALC,INVALC,INVALC,0x4922,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4923,0x4924,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4925,INVALC,INVALC,INVALC,INVALC,0x4926,INVALC,INVALC,
	INVALC,0x4927,INVALC,INVALC,0x4928,0x4929,INVALC,INVALC,
	/* Block 113, Array index 0x2900 */
	0x492A,INVALC,INVALC,INVALC,INVALC,0x492B,0x492C,0x492D,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x492E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x492F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4930,INVALC,INVALC,0x4931,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4932,
	INVALC,INVALC,INVALC,INVALC,0x4933,INVALC,INVALC,0x4934,
	INVALC,0x4935,INVALC,INVALC,0x4936,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4937,0x4938,INVALC,INVALC,INVALC,
	0x4939,0x493A,0x493B,0x493C,INVALC,INVALC,0x4941,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x493D,0x493E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x493F,0x4940,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4942,0x4943,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4944,INVALC,0x4945,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4946,0x4947,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4948,INVALC,
	INVALC,0x4949,INVALC,INVALC,INVALC,0x494A,0x494B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x494C,0x494D,0x494E,0x494F,
	0x4950,INVALC,INVALC,0x4951,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4952,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4953,INVALC,INVALC,INVALC,INVALC,
	0x4954,0x4955,INVALC,INVALC,0x4956,INVALC,INVALC,0x4957,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4958,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4959,INVALC,0x495A,0x495B,0x495C,0x495D,INVALC,
	0x495E,INVALC,INVALC,INVALC,0x495F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4960,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4961,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 114, Array index 0x2A00 */
	INVALC,INVALC,INVALC,0x4962,0x4963,0x4964,0x4965,0x4966,
	INVALC,INVALC,INVALC,0x4967,0x4968,INVALC,INVALC,0x4969,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x496A,INVALC,
	0x496B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x496C,INVALC,0x496D,INVALC,0x496E,
	0x496F,0x4970,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4971,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4972,INVALC,INVALC,INVALC,0x4973,0x4974,0x4975,
	INVALC,INVALC,0x4976,0x4977,INVALC,INVALC,INVALC,INVALC,
	0x4978,INVALC,0x4979,INVALC,INVALC,INVALC,INVALC,0x497A,
	INVALC,INVALC,0x497B,INVALC,0x497C,INVALC,0x497D,INVALC,
	0x497E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A21,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A22,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A23,INVALC,INVALC,INVALC,INVALC,0x4A24,INVALC,0x4A25,
	INVALC,INVALC,INVALC,INVALC,0x4A26,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A27,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A28,0x4A29,INVALC,INVALC,INVALC,INVALC,
	0x4A2A,INVALC,0x4A2B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A2C,
	0x4A2D,INVALC,0x4A2E,0x4A2F,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A30,INVALC,INVALC,INVALC,INVALC,0x4A31,
	0x4A32,0x4A33,INVALC,INVALC,0x4A34,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A35,0x4A36,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A37,INVALC,INVALC,0x4A38,INVALC,
	INVALC,0x4A39,0x4A3A,INVALC,0x4A3B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A3C,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A3D,INVALC,
	/* Block 115, Array index 0x2B00 */
	0x4A3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A3F,
	0x4A40,0x4A41,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A42,INVALC,INVALC,INVALC,0x4A43,
	INVALC,INVALC,0x4A44,INVALC,INVALC,0x4A45,INVALC,0x4A46,
	INVALC,INVALC,INVALC,INVALC,0x4A47,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A48,INVALC,INVALC,INVALC,0x4A49,
	INVALC,INVALC,INVALC,INVALC,0x4A4A,INVALC,INVALC,INVALC,
	0x4A4B,0x4A4C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4A4D,0x4A4E,0x4A4F,INVALC,0x4A50,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A51,0x4A52,
	0x4A53,INVALC,INVALC,0x4A54,INVALC,0x4A55,0x4A56,INVALC,
	INVALC,INVALC,0x4A57,INVALC,0x4A58,INVALC,0x4A59,INVALC,
	0x4A5A,INVALC,INVALC,0x4A5B,INVALC,INVALC,INVALC,INVALC,
	0x4A5C,INVALC,INVALC,0x4A5D,INVALC,INVALC,0x4A5E,0x4A5F,
	INVALC,0x4A60,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A61,
	0x4A62,INVALC,INVALC,0x4A63,0x4A64,INVALC,INVALC,0x4A65,
	INVALC,INVALC,INVALC,INVALC,0x4A66,INVALC,INVALC,INVALC,
	INVALC,0x4A67,INVALC,INVALC,INVALC,0x4A68,0x4A69,INVALC,
	INVALC,INVALC,INVALC,0x4A6A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A6B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4A6C,INVALC,INVALC,INVALC,INVALC,0x4A6D,0x4A6E,INVALC,
	INVALC,0x4A6F,INVALC,INVALC,0x4A70,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A71,INVALC,
	INVALC,0x4A72,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A73,
	INVALC,0x4A74,INVALC,INVALC,0x4A75,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4A76,0x4A77,INVALC,
	0x4A78,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4A79,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4A7A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4A7B,0x4A7C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4A7D,0x4A7E,INVALC,INVALC,0x4B21,INVALC,
	/* Block 116, Array index 0x2C00 */
	INVALC,INVALC,0x4B22,INVALC,0x4B23,0x4B24,INVALC,0x4B25,
	INVALC,INVALC,INVALC,0x4B26,INVALC,0x4B27,INVALC,INVALC,
	INVALC,INVALC,0x4B28,0x4B29,INVALC,INVALC,INVALC,INVALC,
	0x4B2A,0x4B2B,INVALC,INVALC,INVALC,INVALC,0x4B2C,INVALC,
	INVALC,INVALC,0x4B2D,INVALC,0x4B2E,INVALC,INVALC,0x4B2F,
	0x4B30,INVALC,INVALC,INVALC,0x4B31,INVALC,INVALC,INVALC,
	INVALC,0x4B32,0x4B33,INVALC,INVALC,0x4B34,INVALC,INVALC,
	INVALC,INVALC,0x4B35,0x4B36,INVALC,0x4B37,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4B38,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B39,INVALC,INVALC,
	0x4B3A,INVALC,0x4B3B,INVALC,INVALC,INVALC,0x4B3C,INVALC,
	0x4B3D,INVALC,INVALC,INVALC,INVALC,0x4B3E,0x4B3F,0x4B40,
	0x4B41,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B42,0x4B43,
	INVALC,0x4B44,INVALC,0x4B45,0x4B46,INVALC,0x4B47,0x4B48,
	INVALC,0x4B49,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B4A,
	INVALC,0x4B4B,INVALC,INVALC,0x4B4C,INVALC,INVALC,INVALC,
	0x4B4D,0x4B4E,INVALC,0x4B4F,INVALC,0x4B50,0x4B51,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4B52,INVALC,
	0x4B53,INVALC,INVALC,0x4B54,INVALC,0x4B55,INVALC,0x4B56,
	0x4B57,INVALC,INVALC,INVALC,0x4B58,INVALC,0x4B59,0x4B5A,
	0x4B5B,INVALC,0x4B5C,INVALC,INVALC,0x4B5D,0x4B5E,INVALC,
	INVALC,INVALC,0x4B5F,0x4B60,INVALC,0x4B61,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B62,INVALC,0x4B63,
	INVALC,0x4B64,INVALC,INVALC,0x4B65,0x4B66,INVALC,0x4B67,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4B68,0x4B69,INVALC,
	INVALC,0x4B6A,INVALC,0x4B6B,0x4B6C,INVALC,INVALC,0x4B6D,
	INVALC,INVALC,0x4B6E,0x4B6F,INVALC,INVALC,0x4B70,INVALC,
	INVALC,0x4B71,INVALC,INVALC,INVALC,0x4B72,INVALC,INVALC,
	INVALC,0x4B73,INVALC,0x4B74,INVALC,INVALC,0x4B75,0x4B76,
	INVALC,0x4B77,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4B78,0x4B79,INVALC,0x4B7A,
	INVALC,0x4B7B,0x4B7C,0x4B7D,INVALC,0x4B7E,INVALC,0x4C21,
	/* Block 117, Array index 0x2D00 */
	0x4C22,0x4C23,INVALC,INVALC,0x4C24,INVALC,INVALC,0x4C25,
	INVALC,INVALC,0x4C26,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C27,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C28,0x4C29,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4C2A,INVALC,0x4C2B,INVALC,
	0x4C2C,0x4C2D,0x4C2E,0x4C2F,0x4C30,0x4C31,0x4C32,0x4C33,
	0x4C34,0x4C35,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4C36,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C37,INVALC,INVALC,0x4C38,0x4C39,INVALC,0x4C3A,0x4C3B,
	INVALC,INVALC,INVALC,0x4C3C,INVALC,0x4C3D,INVALC,INVALC,
	INVALC,0x4C3E,0x4C3F,INVALC,INVALC,INVALC,INVALC,0x4C40,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4C41,INVALC,INVALC,
	INVALC,INVALC,0x4C42,INVALC,INVALC,INVALC,0x4C43,0x4C44,
	0x4C45,INVALC,INVALC,0x4C46,INVALC,0x4C47,0x4C48,INVALC,
	INVALC,0x4C49,0x4C4A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C4B,0x4C4C,INVALC,INVALC,INVALC,0x4C4D,0x4C4E,0x4C4F,
	INVALC,0x4C50,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C51,
	0x4C52,0x4C53,0x4C54,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C55,0x4C56,0x4C57,INVALC,0x4C58,INVALC,INVALC,0x4C59,
	0x4C5A,0x4C5B,INVALC,0x4C5C,INVALC,INVALC,0x4C5D,INVALC,
	0x4C5E,0x4C5F,0x4C60,0x4C61,INVALC,INVALC,0x4C62,0x4C63,
	INVALC,0x4C64,0x4C65,INVALC,INVALC,0x4C66,INVALC,INVALC,
	INVALC,0x4C67,INVALC,0x4C68,INVALC,INVALC,INVALC,0x4C69,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4C6A,0x4C6B,INVALC,INVALC,0x4C6C,INVALC,INVALC,INVALC,
	0x4C6D,INVALC,INVALC,0x4C6E,INVALC,INVALC,INVALC,INVALC,
	0x4C6F,INVALC,0x4C70,0x4C71,INVALC,INVALC,0x4C72,0x4C73,
	INVALC,INVALC,INVALC,INVALC,0x4C74,INVALC,INVALC,INVALC,
	0x4C75,INVALC,0x4C76,0x4C77,INVALC,INVALC,INVALC,0x4C78,
	INVALC,INVALC,INVALC,INVALC,0x4C79,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4C7A,0x4C7B,0x4C7C,INVALC,INVALC,0x4C7D,
	/* Block 118, Array index 0x2E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4C7E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D21,INVALC,INVALC,INVALC,0x4D22,0x4D23,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D24,0x4D25,INVALC,INVALC,0x4D26,INVALC,INVALC,0x4D27,
	INVALC,0x4D28,0x4D29,INVALC,INVALC,INVALC,INVALC,0x4D2A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D2B,INVALC,
	INVALC,0x4D2C,INVALC,INVALC,INVALC,0x4D2D,0x4D2E,0x4D2F,
	0x4D30,INVALC,INVALC,0x4D31,INVALC,INVALC,INVALC,0x4D32,
	0x4D33,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D34,INVALC,
	0x4D35,INVALC,0x4D36,INVALC,INVALC,INVALC,INVALC,0x4D37,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D38,0x4D39,
	INVALC,0x4D3A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D3B,
	INVALC,0x4D3C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D3D,0x4D3E,0x4D3F,0x4D40,0x4D41,0x4D42,INVALC,
	INVALC,0x4D43,INVALC,INVALC,INVALC,0x4D44,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4D45,INVALC,0x4D46,0x4D47,INVALC,0x4D48,INVALC,INVALC,
	INVALC,0x4D49,INVALC,INVALC,0x4D4A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D4B,INVALC,0x4D4C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D4D,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4D4E,INVALC,INVALC,INVALC,INVALC,0x4D4F,
	0x4D50,0x4D51,INVALC,INVALC,0x4D52,INVALC,0x4D53,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4D54,INVALC,0x4D55,0x4D56,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4D57,
	INVALC,INVALC,INVALC,INVALC,0x4D58,INVALC,INVALC,0x4D59,
	0x4D5A,0x4D5B,INVALC,INVALC,0x4D5C,INVALC,INVALC,0x4D5D,
	INVALC,INVALC,INVALC,INVALC,0x4D5E,INVALC,0x4D5F,0x4D60,
	INVALC,0x4D61,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4D62,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 119, Array index 0x2F00 */
	0x4D63,INVALC,0x4D64,0x4D65,0x4D66,INVALC,INVALC,0x4D67,
	0x4D68,INVALC,0x4D69,INVALC,0x4D6A,INVALC,INVALC,0x4D6B,
	INVALC,INVALC,0x4D6C,0x4D6D,INVALC,0x4D6E,0x4D6F,INVALC,
	INVALC,0x4D70,INVALC,0x4D71,0x4D72,0x4D73,0x4D74,INVALC,
	INVALC,INVALC,INVALC,0x4D75,INVALC,0x4D76,0x4D77,INVALC,
	INVALC,0x4D78,INVALC,INVALC,INVALC,0x4D79,INVALC,INVALC,
	INVALC,INVALC,0x4D7A,0x4D7B,INVALC,0x4D7C,INVALC,INVALC,
	0x4D7D,0x4D7E,0x4E21,INVALC,0x4E22,INVALC,INVALC,INVALC,
	0x4E24,0x4E25,INVALC,0x4E26,0x4E27,0x4E28,INVALC,INVALC,
	INVALC,0x4E29,0x4E23,0x4E2A,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E2B,INVALC,INVALC,
	INVALC,0x4E2C,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E2D,
	INVALC,INVALC,INVALC,INVALC,0x4E2E,0x4E2F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E30,0x4E31,0x4E32,
	INVALC,0x4E33,INVALC,INVALC,0x4E34,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4E35,INVALC,INVALC,INVALC,0x4E36,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4E37,0x4E38,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E39,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4E3A,0x4E3B,0x4E3C,INVALC,0x4E3D,
	0x4E3E,INVALC,0x4E3F,0x4E40,0x4E41,0x4E42,0x4E43,0x4E44,
	0x4E45,INVALC,0x4E46,INVALC,INVALC,0x4E47,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4E48,INVALC,INVALC,
	INVALC,0x4E49,INVALC,INVALC,INVALC,0x4E4A,INVALC,INVALC,
	INVALC,0x4E4B,INVALC,0x4E4C,0x4E4D,INVALC,0x4E4E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4E4F,INVALC,INVALC,INVALC,
	INVALC,0x4E50,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4E51,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E52,INVALC,
	0x4E53,INVALC,INVALC,INVALC,0x4E54,INVALC,INVALC,INVALC,
	0x4E55,0x4E56,INVALC,INVALC,INVALC,INVALC,0x4E57,INVALC,
	INVALC,0x4E58,INVALC,INVALC,0x4E59,INVALC,INVALC,INVALC,
	/* Block 120, Array index 0x3000 */
	0x4E5A,INVALC,INVALC,INVALC,INVALC,INVALC,0x4E5B,INVALC,
	INVALC,INVALC,0x4E5C,INVALC,INVALC,INVALC,0x4E5D,INVALC,
	INVALC,INVALC,0x4E5E,INVALC,0x4E5F,0x4E60,INVALC,0x4E61,
	INVALC,0x4E62,0x4E63,INVALC,0x4E64,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4E65,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x4E66,INVALC,INVALC,INVALC,INVALC,0x4E67,0x4E68,0x4E69,
	INVALC,INVALC,INVALC,INVALC,0x4E6A,0x4E6B,0x4E6C,INVALC,
	INVALC,0x4E6D,INVALC,INVALC,INVALC,0x4E6E,0x4E6F,INVALC,
	INVALC,INVALC,0x4E70,INVALC,INVALC,0x4E71,0x4E72,INVALC,
	INVALC,INVALC,0x4E73,INVALC,INVALC,0x4E74,0x4E75,0x4E76,
	INVALC,INVALC,0x4E77,INVALC,INVALC,INVALC,0x4E78,0x4E79,
	INVALC,INVALC,INVALC,INVALC,0x4E7A,INVALC,0x4E7B,0x4E7C,
	0x4E7D,INVALC,0x4E7E,INVALC,0x4F21,INVALC,INVALC,0x4F22,
	INVALC,INVALC,0x4F23,INVALC,0x4F24,INVALC,INVALC,INVALC,
	0x4F25,INVALC,0x4F26,0x4F27,0x4F28,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F29,INVALC,INVALC,0x4F2A,INVALC,INVALC,
	0x4F2B,INVALC,INVALC,INVALC,0x4F2C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F2D,0x4F2E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F2F,0x4F30,0x4F31,INVALC,
	INVALC,INVALC,0x4F32,INVALC,INVALC,INVALC,INVALC,0x4F33,
	INVALC,INVALC,0x4F34,INVALC,INVALC,INVALC,INVALC,0x4F35,
	INVALC,INVALC,0x4F36,INVALC,INVALC,INVALC,0x4F37,0x4F38,
	INVALC,0x4F39,INVALC,INVALC,INVALC,0x4F3A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x4F3B,INVALC,
	INVALC,INVALC,INVALC,0x4F3C,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F3D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x4F3E,0x4F3F,INVALC,INVALC,0x4F40,INVALC,INVALC,
	INVALC,0x4F41,INVALC,INVALC,INVALC,INVALC,0x4F42,0x4F43,
	0x4F44,INVALC,INVALC,INVALC,0x4F45,INVALC,0x4F46,INVALC,
	INVALC,INVALC,0x4F47,INVALC,0x4F48,INVALC,INVALC,INVALC,
	0x4F49,0x4F4A,INVALC,INVALC,0x4F4B,INVALC,INVALC,INVALC,
	0x4F4C,INVALC,INVALC,0x4F4D,INVALC,INVALC,INVALC,INVALC,
	/* Block 121, Array index 0x3100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4F4E,0x4F4F,INVALC,
	INVALC,0x4F50,INVALC,INVALC,INVALC,0x4F51,0x4F52,INVALC,
	INVALC,0x4F53,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4F54,INVALC,INVALC,
	INVALC,0x4F55,0x4F56,0x4F57,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x4F58,0x4F59,INVALC,
	0x4F5A,INVALC,INVALC,INVALC,INVALC,0x4F5B,INVALC,0x4F5C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x4F5D,0x4F5E,INVALC,INVALC,0x4F5F,
	0x4F60,INVALC,INVALC,INVALC,0x4F61,INVALC,0x4F62,INVALC,
	INVALC,INVALC,0x4F63,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x4F64,INVALC,0x4F65,INVALC,
	0x4F66,0x4F67,INVALC,0x4F68,0x4F69,INVALC,INVALC,INVALC,
	0x4F6A,INVALC,0x4F6B,INVALC,INVALC,INVALC,0x4F6C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F6D,INVALC,INVALC,INVALC,0x4F6E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x4F6F,INVALC,INVALC,INVALC,INVALC,0x4F70,
	INVALC,INVALC,INVALC,INVALC,0x4F71,INVALC,INVALC,INVALC,
	0x4F72,INVALC,INVALC,INVALC,INVALC,0x4F74,0x4F75,0x4F76,
	INVALC,0x4F73,INVALC,INVALC,0x4F77,INVALC,INVALC,INVALC,
	0x4F78,INVALC,INVALC,INVALC,0x4F79,0x4F7A,INVALC,INVALC,
	0x4F7B,0x4F7C,0x4F7D,0x4F7E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5021,INVALC,0x5022,INVALC,0x5023,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5024,
	0x5025,0x5026,INVALC,INVALC,0x5027,INVALC,0x5028,INVALC,
	INVALC,INVALC,0x5029,0x502A,INVALC,0x502B,0x502C,INVALC,
	INVALC,INVALC,INVALC,0x502E,INVALC,INVALC,INVALC,0x502F,
	0x5030,0x5031,INVALC,INVALC,0x502D,INVALC,0x5032,INVALC,
	INVALC,INVALC,0x5033,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5034,0x5035,INVALC,INVALC,0x5037,0x5038,
	INVALC,INVALC,0x5039,0x503A,INVALC,INVALC,INVALC,0x503B,
	/* Block 122, Array index 0x3200 */
	0x5036,INVALC,INVALC,INVALC,INVALC,INVALC,0x503C,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x503D,INVALC,INVALC,INVALC,
	0x503E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x503F,INVALC,0x5040,INVALC,0x5041,0x5042,
	0x5043,INVALC,INVALC,INVALC,INVALC,0x5044,INVALC,0x5045,
	INVALC,0x5046,INVALC,INVALC,INVALC,0x5047,INVALC,INVALC,
	INVALC,0x5048,INVALC,INVALC,0x5049,0x504A,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x504B,INVALC,0x504C,INVALC,0x504D,
	INVALC,INVALC,INVALC,INVALC,0x504E,0x504F,0x5050,INVALC,
	INVALC,INVALC,0x5051,0x5052,INVALC,INVALC,INVALC,0x5053,
	INVALC,0x5054,INVALC,INVALC,0x5055,INVALC,INVALC,INVALC,
	0x5056,INVALC,INVALC,0x5057,0x5058,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5059,
	INVALC,0x505A,INVALC,0x505B,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x505C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x505D,INVALC,0x505E,0x505F,INVALC,0x5060,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5061,0x5062,INVALC,INVALC,INVALC,
	INVALC,0x5063,INVALC,0x5064,0x5065,0x5066,0x5067,INVALC,
	0x5068,INVALC,INVALC,0x5069,0x506A,INVALC,INVALC,INVALC,
	INVALC,0x506B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x506C,0x506D,INVALC,0x506E,INVALC,INVALC,INVALC,0x506F,
	INVALC,0x5070,INVALC,INVALC,0x5071,INVALC,INVALC,INVALC,
	0x5072,INVALC,INVALC,0x5073,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5074,INVALC,0x5075,INVALC,INVALC,0x5076,
	0x5077,INVALC,0x5078,INVALC,INVALC,INVALC,INVALC,0x5079,
	INVALC,INVALC,INVALC,INVALC,0x507A,INVALC,0x507B,INVALC,
	INVALC,INVALC,0x507C,INVALC,INVALC,0x507D,0x507E,INVALC,
	0x5121,INVALC,0x5122,INVALC,INVALC,0x5123,INVALC,INVALC,
	INVALC,INVALC,0x5124,0x5125,INVALC,0x5126,INVALC,INVALC,
	INVALC,0x5127,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5128,INVALC,INVALC,INVALC,0x5129,INVALC,INVALC,INVALC,
	/* Block 123, Array index 0x3300 */
	INVALC,INVALC,0x512A,0x512B,INVALC,INVALC,INVALC,0x512C,
	INVALC,0x512D,0x512E,INVALC,0x512F,INVALC,INVALC,INVALC,
	INVALC,0x5130,INVALC,INVALC,INVALC,0x5131,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5132,INVALC,INVALC,0x5133,INVALC,
	INVALC,0x5134,INVALC,INVALC,INVALC,INVALC,INVALC,0x5135,
	INVALC,INVALC,INVALC,0x5136,INVALC,0x5137,INVALC,0x5138,
	0x5139,INVALC,INVALC,INVALC,0x513A,0x513B,INVALC,INVALC,
	0x513C,0x513D,0x513E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x513F,0x5140,INVALC,0x5141,
	0x5142,INVALC,INVALC,INVALC,0x5143,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5144,0x5145,INVALC,
	INVALC,0x5146,INVALC,INVALC,0x5147,0x5148,INVALC,0x5149,
	0x514A,INVALC,INVALC,INVALC,INVALC,0x514B,INVALC,0x514C,
	INVALC,INVALC,0x514D,INVALC,INVALC,0x514E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x514F,INVALC,INVALC,
	0x5150,INVALC,INVALC,INVALC,INVALC,INVALC,0x5151,INVALC,
	0x5152,INVALC,0x5153,INVALC,INVALC,0x5154,0x5155,INVALC,
	INVALC,INVALC,0x5156,0x5157,INVALC,INVALC,INVALC,INVALC,
	0x5158,0x5159,INVALC,INVALC,0x515A,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x515B,INVALC,
	0x515C,INVALC,INVALC,0x515D,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x515E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x515F,INVALC,0x5160,INVALC,INVALC,
	INVALC,0x5161,INVALC,0x5162,0x5163,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5164,INVALC,
	INVALC,0x5165,INVALC,INVALC,0x5166,INVALC,0x5167,INVALC,
	INVALC,0x5168,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5169,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x516A,0x516B,INVALC,0x516C,0x516D,INVALC,INVALC,INVALC,
	INVALC,0x516E,INVALC,INVALC,0x516F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5170,INVALC,0x5171,0x5172,INVALC,
	/* Block 124, Array index 0x3400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5173,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5174,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5175,
	INVALC,INVALC,INVALC,0x5176,INVALC,INVALC,INVALC,0x5177,
	INVALC,0x5178,0x5179,0x517A,INVALC,0x517B,0x517C,0x517D,
	0x517E,0x5221,INVALC,INVALC,0x5222,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5223,INVALC,0x5224,
	0x5225,0x5226,INVALC,INVALC,INVALC,INVALC,INVALC,0x5227,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5228,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5229,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x522A,INVALC,INVALC,INVALC,0x522B,INVALC,0x522C,INVALC,
	INVALC,0x522D,0x522E,INVALC,INVALC,0x522F,INVALC,0x5230,
	INVALC,INVALC,0x5231,0x5232,INVALC,INVALC,INVALC,0x5233,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5234,INVALC,INVALC,INVALC,
	INVALC,0x5235,INVALC,INVALC,INVALC,INVALC,0x5236,INVALC,
	0x5237,0x5238,INVALC,INVALC,INVALC,INVALC,0x5239,INVALC,
	INVALC,INVALC,INVALC,0x523A,INVALC,INVALC,0x523B,INVALC,
	0x523C,INVALC,INVALC,INVALC,INVALC,0x523D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x523E,INVALC,INVALC,0x523F,
	0x5240,INVALC,0x5241,INVALC,INVALC,0x5242,0x5243,INVALC,
	INVALC,INVALC,0x5244,0x5245,0x5246,0x5247,INVALC,INVALC,
	INVALC,INVALC,0x5248,INVALC,INVALC,0x5249,INVALC,INVALC,
	0x524A,INVALC,0x524B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x524C,INVALC,0x524D,0x524E,
	INVALC,0x524F,0x5250,0x5251,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5252,INVALC,0x5253,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5254,INVALC,0x5255,0x5256,INVALC,INVALC,
	0x5257,0x5258,0x5259,INVALC,0x525A,INVALC,0x525B,INVALC,
	/* Block 125, Array index 0x3500 */
	INVALC,0x525C,0x525D,0x525E,0x525F,INVALC,0x5260,INVALC,
	INVALC,0x5261,INVALC,0x5262,0x5263,INVALC,0x5264,0x5265,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5266,INVALC,0x5267,INVALC,INVALC,INVALC,INVALC,
	0x5268,INVALC,INVALC,INVALC,INVALC,0x5269,0x526A,INVALC,
	0x526B,INVALC,INVALC,INVALC,0x526C,INVALC,INVALC,INVALC,
	INVALC,0x526D,INVALC,0x526E,0x526F,INVALC,0x5270,INVALC,
	INVALC,0x5271,0x5272,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5273,INVALC,
	INVALC,INVALC,0x5274,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5276,0x5277,0x5278,INVALC,0x5275,INVALC,INVALC,
	INVALC,0x5279,0x527A,0x527B,0x527C,0x527D,0x527E,INVALC,
	INVALC,0x5321,INVALC,0x5322,INVALC,INVALC,INVALC,0x5323,
	INVALC,0x5324,INVALC,INVALC,INVALC,0x5325,0x5326,INVALC,
	0x5327,INVALC,0x5328,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5329,INVALC,INVALC,0x532A,0x532B,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x532C,0x532D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x532E,
	INVALC,INVALC,INVALC,INVALC,0x532F,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5330,INVALC,
	0x5331,INVALC,INVALC,INVALC,INVALC,INVALC,0x5332,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5333,INVALC,INVALC,INVALC,INVALC,INVALC,0x5334,0x5335,
	INVALC,INVALC,0x5336,0x5337,0x5338,INVALC,INVALC,0x5339,
	INVALC,INVALC,INVALC,INVALC,0x533A,INVALC,INVALC,0x533B,
	0x533C,0x533D,INVALC,INVALC,INVALC,0x533E,INVALC,0x533F,
	INVALC,INVALC,INVALC,0x5340,0x5341,0x5342,INVALC,0x5343,
	INVALC,0x5344,0x5345,INVALC,INVALC,0x5346,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5347,INVALC,
	INVALC,0x5348,INVALC,0x5349,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x534A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 126, Array index 0x3600 */
	INVALC,INVALC,INVALC,0x534B,INVALC,INVALC,INVALC,0x534C,
	0x534D,0x534E,INVALC,INVALC,INVALC,INVALC,INVALC,0x534F,
	INVALC,0x5350,0x5351,0x5352,INVALC,INVALC,0x5353,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5354,0x5355,INVALC,
	INVALC,INVALC,INVALC,0x5356,INVALC,INVALC,0x5357,INVALC,
	INVALC,INVALC,0x5358,INVALC,INVALC,0x5359,INVALC,INVALC,
	INVALC,0x535A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x535B,0x535C,0x535D,INVALC,
	0x535E,0x535F,INVALC,INVALC,INVALC,INVALC,INVALC,0x5360,
	0x5361,INVALC,INVALC,INVALC,INVALC,0x5362,INVALC,INVALC,
	INVALC,0x5363,INVALC,0x5364,INVALC,INVALC,INVALC,0x5365,
	INVALC,0x5366,0x5367,INVALC,0x5368,0x5369,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x536A,INVALC,0x536B,
	INVALC,INVALC,0x536C,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x536D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x536E,INVALC,0x536F,0x5370,INVALC,INVALC,INVALC,0x5371,
	INVALC,0x5372,0x5373,0x5374,INVALC,0x5375,0x5376,INVALC,
	0x5377,INVALC,INVALC,0x5378,0x5379,0x537A,INVALC,INVALC,
	INVALC,0x537B,INVALC,INVALC,INVALC,INVALC,0x537C,0x537D,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x537E,0x5421,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5422,0x5423,
	INVALC,INVALC,0x5424,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5425,INVALC,INVALC,0x5426,0x5427,
	INVALC,0x5428,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5429,0x542A,0x542B,0x542C,0x542D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x542E,0x542F,0x5430,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5431,
	INVALC,0x5432,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5434,INVALC,INVALC,0x5435,0x5436,INVALC,
	INVALC,INVALC,0x5437,0x5438,INVALC,0x5439,INVALC,INVALC,
	INVALC,0x543A,INVALC,INVALC,INVALC,0x543B,0x543C,INVALC,
	INVALC,0x543D,0x543E,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 127, Array index 0x3700 */
	0x5433,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x543F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5440,0x5441,INVALC,INVALC,INVALC,0x5442,INVALC,0x5443,
	INVALC,INVALC,INVALC,INVALC,0x5444,0x5445,INVALC,INVALC,
	0x5446,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5447,
	0x5448,INVALC,INVALC,INVALC,0x5449,0x544A,INVALC,0x544B,
	INVALC,INVALC,INVALC,0x544C,INVALC,INVALC,0x544D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x544E,
	INVALC,INVALC,INVALC,INVALC,0x544F,0x5450,INVALC,0x5451,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5452,INVALC,
	0x5453,INVALC,0x5454,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5455,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5456,
	INVALC,0x5457,0x5458,INVALC,INVALC,0x5459,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x545A,INVALC,INVALC,0x545B,0x545C,
	INVALC,INVALC,INVALC,0x545D,INVALC,INVALC,INVALC,INVALC,
	0x545E,INVALC,INVALC,INVALC,INVALC,INVALC,0x545F,INVALC,
	INVALC,0x5460,INVALC,INVALC,INVALC,INVALC,0x5461,0x5462,
	INVALC,INVALC,0x5463,INVALC,INVALC,0x5464,INVALC,INVALC,
	INVALC,0x5465,INVALC,INVALC,INVALC,0x5466,INVALC,INVALC,
	0x5467,INVALC,0x5468,INVALC,INVALC,0x5469,0x546A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 128, Array index 0x3800 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x546C,0x546B,0x546D,0x546E,0x546F,
	INVALC,INVALC,INVALC,0x5470,0x5471,INVALC,INVALC,0x5472,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5473,
	INVALC,INVALC,0x5474,0x5475,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5476,0x5477,0x5478,INVALC,INVALC,
	INVALC,0x5479,INVALC,0x547A,0x547B,0x547C,0x547D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x547E,INVALC,INVALC,
	INVALC,0x5521,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5522,0x5523,0x5524,
	0x5525,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5526,INVALC,0x5527,INVALC,0x5528,
	0x5529,0x552A,INVALC,INVALC,INVALC,INVALC,0x552B,0x552C,
	INVALC,INVALC,INVALC,INVALC,0x552D,INVALC,INVALC,INVALC,
	INVALC,0x552E,0x552F,INVALC,INVALC,INVALC,0x5530,INVALC,
	INVALC,INVALC,0x5531,INVALC,INVALC,0x5532,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5533,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5534,INVALC,INVALC,0x5535,
	0x5536,INVALC,INVALC,0x5537,INVALC,INVALC,INVALC,INVALC,
	0x5538,INVALC,INVALC,INVALC,INVALC,INVALC,0x5539,0x553A,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x553B,INVALC,INVALC,INVALC,0x553C,
	INVALC,INVALC,INVALC,0x553D,INVALC,0x553E,INVALC,INVALC,
	0x553F,INVALC,INVALC,INVALC,0x5540,INVALC,0x5541,0x5542,
	INVALC,INVALC,0x5543,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5544,INVALC,INVALC,0x5545,0x5546,0x5547,
	/* Block 129, Array index 0x3900 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5548,
	0x5549,INVALC,0x554A,INVALC,INVALC,0x554B,0x554C,0x554D,
	INVALC,0x554E,INVALC,0x554F,0x5550,INVALC,0x5551,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5552,0x5553,0x5554,
	0x5555,INVALC,INVALC,INVALC,0x5556,INVALC,0x5557,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5558,INVALC,0x5559,INVALC,
	0x555A,INVALC,INVALC,INVALC,0x555B,0x555C,INVALC,0x555D,
	INVALC,0x555E,0x555F,INVALC,0x5560,INVALC,0x5561,INVALC,
	0x5562,INVALC,INVALC,INVALC,0x5563,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5564,INVALC,INVALC,INVALC,0x5565,INVALC,0x5566,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5567,INVALC,INVALC,
	INVALC,0x5568,INVALC,INVALC,INVALC,0x5569,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x556A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x556B,INVALC,INVALC,INVALC,INVALC,INVALC,0x556C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x556D,INVALC,0x556E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x556F,0x5570,
	INVALC,INVALC,INVALC,0x5571,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5572,0x5573,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5574,INVALC,INVALC,INVALC,INVALC,0x5575,INVALC,0x5576,
	INVALC,INVALC,0x5577,INVALC,0x5578,0x5579,INVALC,0x557A,
	0x557B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x557C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x557D,0x557E,INVALC,
	0x5621,INVALC,0x5622,0x5623,INVALC,INVALC,0x5624,INVALC,
	INVALC,0x5625,0x5626,INVALC,INVALC,INVALC,0x5627,INVALC,
	/* Block 130, Array index 0x3A00 */
	INVALC,INVALC,INVALC,0x5628,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5629,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x562A,0x562B,
	0x562C,INVALC,INVALC,INVALC,0x562D,INVALC,0x562E,INVALC,
	0x562F,INVALC,INVALC,INVALC,0x5630,INVALC,INVALC,0x5631,
	INVALC,INVALC,INVALC,INVALC,0x5632,INVALC,INVALC,INVALC,
	0x5633,INVALC,INVALC,INVALC,INVALC,0x5634,INVALC,INVALC,
	INVALC,INVALC,0x5635,INVALC,0x5636,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5637,INVALC,0x5638,
	INVALC,INVALC,0x5639,INVALC,0x563A,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x563B,INVALC,INVALC,INVALC,INVALC,0x563C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x563D,0x563E,INVALC,INVALC,INVALC,INVALC,INVALC,0x563F,
	0x5640,0x5641,INVALC,INVALC,INVALC,0x5642,INVALC,0x5643,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5644,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5645,INVALC,INVALC,0x5647,0x5648,0x5649,INVALC,
	INVALC,INVALC,INVALC,0x564A,INVALC,INVALC,0x564B,INVALC,
	0x5646,INVALC,INVALC,INVALC,INVALC,INVALC,0x564C,INVALC,
	0x564D,INVALC,INVALC,0x564E,INVALC,INVALC,0x564F,INVALC,
	INVALC,INVALC,0x5650,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5651,INVALC,
	INVALC,INVALC,0x5652,INVALC,0x5653,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5654,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5656,INVALC,0x5657,INVALC,INVALC,
	INVALC,INVALC,0x5658,0x5655,INVALC,INVALC,0x5659,0x565A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x565B,INVALC,0x565C,
	INVALC,INVALC,INVALC,0x565D,INVALC,0x565E,0x565F,INVALC,
	INVALC,0x5660,INVALC,INVALC,0x5661,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5662,0x5663,INVALC,INVALC,INVALC,
	0x5664,0x5665,0x5666,INVALC,INVALC,0x5667,0x5668,INVALC,
	0x5669,0x566A,INVALC,INVALC,INVALC,0x566B,INVALC,0x566C,
	/* Block 131, Array index 0x3B00 */
	0x566D,INVALC,INVALC,0x566E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x566F,
	INVALC,INVALC,INVALC,0x5670,0x5671,INVALC,INVALC,INVALC,
	INVALC,0x5672,0x5673,INVALC,INVALC,0x5674,INVALC,INVALC,
	INVALC,0x5675,0x5676,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5677,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5678,INVALC,0x5679,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x567A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x567B,0x567C,0x567D,0x567E,INVALC,
	INVALC,INVALC,INVALC,0x5721,INVALC,INVALC,0x5722,0x5723,
	INVALC,0x5724,INVALC,INVALC,INVALC,INVALC,0x5725,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5726,INVALC,INVALC,INVALC,
	0x5727,INVALC,INVALC,0x5728,INVALC,INVALC,INVALC,0x5729,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x572A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x572B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x572C,INVALC,0x572D,INVALC,0x572E,
	0x572F,0x5730,INVALC,0x5731,0x5732,INVALC,INVALC,0x5733,
	INVALC,0x5734,0x5735,INVALC,INVALC,INVALC,0x5736,INVALC,
	INVALC,0x5737,INVALC,INVALC,0x5738,INVALC,0x5739,INVALC,
	0x573A,INVALC,0x573B,0x573C,INVALC,INVALC,INVALC,INVALC,
	0x573D,0x573E,INVALC,0x573F,0x5740,INVALC,INVALC,0x5741,
	0x5742,0x5743,0x5744,INVALC,INVALC,INVALC,0x5745,INVALC,
	0x5746,INVALC,0x5747,INVALC,0x5748,INVALC,INVALC,0x5749,
	INVALC,INVALC,0x574A,INVALC,0x574B,INVALC,0x574C,0x574D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x574E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x574F,INVALC,INVALC,INVALC,INVALC,0x5750,INVALC,INVALC,
	INVALC,INVALC,0x5751,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5752,INVALC,0x5753,INVALC,0x5754,INVALC,INVALC,INVALC,
	0x5755,INVALC,0x5756,INVALC,INVALC,0x5757,INVALC,0x5758,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5759,0x575A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x575B,0x575C,INVALC,
	/* Block 132, Array index 0x3C00 */
	0x575D,0x575E,INVALC,INVALC,INVALC,INVALC,INVALC,0x575F,
	0x5760,INVALC,0x5761,0x5762,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5764,INVALC,0x5765,0x5766,0x5767,
	INVALC,0x5768,0x5769,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x576A,0x576B,0x576C,INVALC,
	0x576D,INVALC,INVALC,0x576E,INVALC,INVALC,INVALC,0x576F,
	INVALC,INVALC,0x5770,INVALC,0x5771,0x5772,INVALC,INVALC,
	INVALC,INVALC,0x5773,0x5774,0x5775,INVALC,INVALC,0x5776,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5777,0x5778,INVALC,
	INVALC,0x5779,INVALC,0x583E,0x5763,0x577A,0x577B,0x577C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x577D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x577E,INVALC,INVALC,INVALC,INVALC,0x5821,INVALC,0x5822,
	0x5823,INVALC,0x5824,INVALC,0x5825,INVALC,0x5826,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5827,INVALC,INVALC,
	INVALC,INVALC,0x5828,INVALC,0x5829,0x582A,INVALC,INVALC,
	0x582B,0x582C,INVALC,0x582D,0x582E,0x582F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5830,0x5831,
	INVALC,0x5832,INVALC,INVALC,0x5833,0x584C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5834,0x5835,
	0x5836,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5837,
	INVALC,0x5838,INVALC,INVALC,INVALC,INVALC,INVALC,0x5839,
	0x583A,0x583B,INVALC,INVALC,0x583C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x583D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x583F,INVALC,0x5840,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5841,INVALC,
	0x5842,0x5843,INVALC,INVALC,0x5844,INVALC,INVALC,INVALC,
	/* Block 133, Array index 0x3D00 */
	INVALC,0x5845,INVALC,INVALC,INVALC,INVALC,0x5846,INVALC,
	INVALC,INVALC,0x5847,INVALC,INVALC,INVALC,INVALC,0x5848,
	INVALC,0x5849,INVALC,INVALC,INVALC,0x584A,INVALC,INVALC,
	INVALC,0x584B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x584D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x584E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x584F,INVALC,0x5850,
	0x5851,INVALC,0x5852,INVALC,INVALC,0x5853,INVALC,0x5854,
	INVALC,0x5855,0x5856,INVALC,INVALC,INVALC,0x5857,INVALC,
	0x5858,0x5859,0x585A,INVALC,0x585B,INVALC,INVALC,INVALC,
	0x585C,INVALC,INVALC,INVALC,0x585D,0x585E,INVALC,0x585F,
	INVALC,INVALC,0x5860,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5861,INVALC,INVALC,0x5862,0x5863,INVALC,0x5864,INVALC,
	0x5865,INVALC,INVALC,INVALC,0x5866,0x5867,INVALC,INVALC,
	INVALC,0x5868,INVALC,INVALC,INVALC,0x5869,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x586A,0x586B,INVALC,0x586C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x586D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x586E,INVALC,
	0x586F,0x5870,0x5871,INVALC,INVALC,INVALC,INVALC,0x5872,
	INVALC,0x5873,INVALC,INVALC,0x5874,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5875,INVALC,INVALC,0x5876,0x5877,INVALC,
	0x5878,INVALC,0x5879,INVALC,INVALC,INVALC,INVALC,0x587A,
	0x587B,INVALC,INVALC,INVALC,0x587C,INVALC,INVALC,0x587D,
	INVALC,INVALC,INVALC,0x587E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5921,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5922,
	INVALC,INVALC,0x5923,INVALC,INVALC,INVALC,INVALC,0x5924,
	0x5925,0x5926,0x5927,INVALC,INVALC,INVALC,INVALC,0x5928,
	INVALC,INVALC,0x592A,0x592B,INVALC,0x592C,INVALC,INVALC,
	/* Block 134, Array index 0x3E00 */
	INVALC,INVALC,0x592D,0x592E,INVALC,INVALC,INVALC,0x592F,
	INVALC,INVALC,INVALC,INVALC,0x5930,INVALC,0x5931,INVALC,
	0x5932,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5933,INVALC,0x5934,INVALC,
	INVALC,INVALC,0x5935,0x5936,0x5937,0x5938,INVALC,0x5939,
	INVALC,INVALC,0x593A,0x593B,INVALC,INVALC,INVALC,0x593C,
	INVALC,INVALC,0x5929,0x593D,0x593E,INVALC,0x593F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5940,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5941,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5942,
	0x5943,0x5944,0x5945,0x5946,INVALC,INVALC,0x5947,INVALC,
	INVALC,0x5948,INVALC,INVALC,0x5949,0x594A,0x594B,0x594C,
	0x594D,0x594E,0x594F,INVALC,0x5950,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5951,INVALC,INVALC,INVALC,0x5952,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5953,0x5954,0x5955,INVALC,0x5956,INVALC,0x5957,
	INVALC,0x5958,INVALC,INVALC,INVALC,0x5959,0x595A,INVALC,
	INVALC,0x595B,INVALC,0x595C,0x595D,INVALC,INVALC,0x595E,
	INVALC,INVALC,INVALC,0x595F,INVALC,INVALC,INVALC,INVALC,
	0x5960,INVALC,INVALC,INVALC,INVALC,0x5961,INVALC,0x5962,
	0x5963,INVALC,0x5964,INVALC,INVALC,0x5965,INVALC,0x5966,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5974,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5967,INVALC,0x5968,0x5969,
	0x596A,INVALC,INVALC,INVALC,0x596B,0x596C,0x596D,0x596E,
	INVALC,INVALC,0x596F,INVALC,INVALC,INVALC,INVALC,0x5970,
	INVALC,INVALC,0x5971,0x5972,INVALC,INVALC,0x5973,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5975,INVALC,0x5976,INVALC,INVALC,INVALC,INVALC,0x5977,
	0x5978,INVALC,INVALC,INVALC,INVALC,INVALC,0x5979,INVALC,
	0x597A,INVALC,INVALC,INVALC,INVALC,0x597B,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x597C,INVALC,INVALC,0x597D,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x597E,INVALC,INVALC,0x5A21,
	/* Block 135, Array index 0x3F00 */
	0x5A22,INVALC,INVALC,INVALC,0x5A23,0x5A24,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5A25,0x5A26,INVALC,
	0x5A27,0x5A28,0x5A29,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A2A,0x5A2B,INVALC,0x5A2C,INVALC,INVALC,0x5A2D,INVALC,
	INVALC,0x5A2E,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A2F,
	INVALC,0x5A30,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A31,INVALC,
	0x5A32,INVALC,0x5A33,INVALC,0x5A34,0x5A35,INVALC,INVALC,
	0x5A36,0x3866,0x5A37,INVALC,INVALC,INVALC,0x5A38,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A39,0x5A3A,INVALC,INVALC,0x5A3B,0x5A3C,
	0x5A3D,0x5A3E,INVALC,INVALC,INVALC,0x5A3F,INVALC,INVALC,
	0x5A40,0x5A41,0x5A42,0x5A43,0x5A44,INVALC,INVALC,INVALC,
	INVALC,0x5A45,INVALC,INVALC,0x5A46,INVALC,INVALC,0x5A47,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5A48,0x5A49,0x5A4A,
	INVALC,INVALC,0x5A4B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5A6D,INVALC,INVALC,INVALC,INVALC,0x5A4C,INVALC,INVALC,
	INVALC,0x5A4D,INVALC,INVALC,INVALC,INVALC,0x5A4E,INVALC,
	0x5A4F,INVALC,0x5A50,INVALC,0x5A51,INVALC,INVALC,INVALC,
	INVALC,0x5A52,INVALC,INVALC,INVALC,INVALC,0x5A53,0x5A54,
	0x5A55,INVALC,INVALC,INVALC,INVALC,0x5A56,INVALC,INVALC,
	INVALC,0x5A57,INVALC,0x5A58,0x5A59,0x5A5A,INVALC,0x5A5B,
	0x5A5C,0x5A5D,INVALC,INVALC,INVALC,INVALC,INVALC,0x5A5E,
	0x5A5F,0x5A60,INVALC,0x5A61,INVALC,0x5A62,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5A63,0x5A64,INVALC,INVALC,0x5A65,INVALC,0x5A66,
	INVALC,INVALC,0x5A67,INVALC,0x5A68,INVALC,INVALC,INVALC,
	0x5A69,INVALC,INVALC,0x5A6A,INVALC,0x5A6B,INVALC,0x5A6C,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5A6E,INVALC,0x5A6F,0x5A70,INVALC,INVALC,
	/* Block 136, Array index 0x4000 */
	INVALC,INVALC,INVALC,INVALC,0x5A71,0x5A72,INVALC,0x5A73,
	INVALC,INVALC,INVALC,0x5A74,INVALC,INVALC,0x5A75,0x5A76,
	0x5A77,INVALC,INVALC,0x5A78,0x5A79,INVALC,INVALC,INVALC,
	INVALC,0x5A7A,INVALC,INVALC,INVALC,INVALC,0x5A7B,0x5A7C,
	INVALC,0x5A7D,INVALC,0x5A7E,INVALC,INVALC,INVALC,INVALC,
	0x5B21,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B22,0x5B23,
	INVALC,0x5B24,0x5B25,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5B26,0x5B27,INVALC,0x5B28,0x5B29,0x5B2A,INVALC,
	0x5B2B,INVALC,INVALC,0x5B2C,INVALC,0x5B2D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5B2E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5B2F,INVALC,INVALC,INVALC,INVALC,0x5B30,INVALC,INVALC,
	INVALC,0x5B31,INVALC,INVALC,0x5B32,0x5B33,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B34,
	INVALC,0x5B35,0x5B36,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5B37,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5B38,0x5B39,0x5B3A,0x5B3B,0x5B3C,
	0x5B3D,0x5B3E,INVALC,0x5B3F,0x5B40,INVALC,INVALC,INVALC,
	0x5B41,INVALC,INVALC,0x5B42,INVALC,0x5B43,INVALC,0x5B44,
	0x5B45,0x5B46,INVALC,INVALC,INVALC,INVALC,0x5B47,INVALC,
	0x5B48,INVALC,INVALC,0x5B49,INVALC,INVALC,INVALC,0x5B4A,
	INVALC,INVALC,INVALC,INVALC,0x5B4B,0x5B4C,0x5B4D,INVALC,
	INVALC,0x5B4E,INVALC,INVALC,INVALC,0x5B4F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5B50,0x5B51,
	INVALC,0x5B52,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5B53,0x5B54,0x5B55,INVALC,INVALC,INVALC,0x5B56,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5B57,0x5B58,INVALC,
	INVALC,0x5B59,0x5B5A,INVALC,0x5B5B,INVALC,INVALC,0x5B5C,
	INVALC,INVALC,0x5B5D,0x5B5E,0x5B5F,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5B60,0x5B61,INVALC,0x5B62,INVALC,INVALC,
	INVALC,0x5B63,INVALC,0x5B64,INVALC,INVALC,INVALC,INVALC,
	0x5B65,INVALC,0x5B66,INVALC,INVALC,INVALC,INVALC,0x5B67,
	/* Block 137, Array index 0x4100 */
	INVALC,0x5B68,INVALC,0x5B69,INVALC,INVALC,0x5B6A,INVALC,
	INVALC,0x5B6B,0x5B6C,0x5B6D,INVALC,INVALC,INVALC,INVALC,
	0x5B6E,INVALC,0x5B70,0x5B71,0x5B72,INVALC,INVALC,INVALC,
	0x5B73,0x5B6F,0x5B74,0x5B75,0x5B76,INVALC,0x5B77,0x5B78,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5B79,INVALC,INVALC,INVALC,INVALC,0x5B7A,0x5B7B,INVALC,
	0x5B7C,INVALC,0x5B7D,INVALC,INVALC,0x5B7E,INVALC,INVALC,
	INVALC,INVALC,0x5C21,INVALC,0x5C22,INVALC,INVALC,INVALC,
	INVALC,0x5C23,INVALC,0x5C24,INVALC,0x5C25,INVALC,INVALC,
	0x5C26,0x5C27,0x5C28,0x5C29,INVALC,INVALC,0x5C2A,INVALC,
	INVALC,0x5C2B,INVALC,INVALC,INVALC,0x5C2C,0x5C2D,INVALC,
	0x5C2E,INVALC,0x5C2F,INVALC,0x5C30,INVALC,INVALC,0x5C31,
	0x5C32,INVALC,INVALC,INVALC,0x5C33,INVALC,INVALC,INVALC,
	INVALC,0x5C34,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C35,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5C36,INVALC,0x5C37,INVALC,INVALC,INVALC,INVALC,
	0x5C38,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5C39,INVALC,0x5C3A,0x5C3B,0x5C3C,INVALC,INVALC,0x5C3D,
	0x5C3E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5C3F,INVALC,0x5C40,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C41,INVALC,INVALC,0x5C42,0x5C43,INVALC,
	0x5C44,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5C45,0x5C46,0x5C47,0x5C48,0x5C49,INVALC,
	INVALC,0x5C4A,0x5C4B,0x5C4C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5C4D,INVALC,INVALC,0x5C4E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C4F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C50,
	0x5C51,0x5C52,INVALC,INVALC,INVALC,0x5C53,INVALC,0x5C54,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 138, Array index 0x4200 */
	INVALC,0x5C55,INVALC,INVALC,INVALC,INVALC,0x5C56,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5C57,0x5C58,0x5C59,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5C5A,0x5C5B,INVALC,
	0x5C5C,0x5C5D,0x5C5E,INVALC,0x5C5F,INVALC,INVALC,INVALC,
	0x5C60,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C61,0x5C62,
	0x5C63,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5C64,0x5C65,0x5C66,INVALC,INVALC,0x5C67,INVALC,INVALC,
	INVALC,0x5C68,0x5C69,INVALC,INVALC,INVALC,0x5C6A,INVALC,
	0x5C6B,INVALC,0x5C6C,INVALC,INVALC,0x5C6D,0x5C6E,INVALC,
	INVALC,0x5C6F,INVALC,INVALC,INVALC,INVALC,INVALC,0x5C70,
	INVALC,INVALC,0x5C71,INVALC,INVALC,INVALC,INVALC,0x5C72,
	INVALC,INVALC,0x5C73,0x5C74,0x5C75,INVALC,INVALC,INVALC,
	INVALC,0x5C76,0x5C77,0x5C78,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5C79,INVALC,INVALC,0x5C7A,INVALC,
	0x5C7B,INVALC,INVALC,0x5C7C,INVALC,0x5C7D,INVALC,INVALC,
	INVALC,INVALC,0x5C7E,0x5D21,0x5D22,0x5D23,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5D24,INVALC,INVALC,INVALC,0x5D25,INVALC,INVALC,
	0x5D26,INVALC,INVALC,INVALC,0x5D27,0x5D28,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5D29,0x5D2A,INVALC,INVALC,0x5D2B,
	0x5D2C,INVALC,INVALC,INVALC,INVALC,0x5D2D,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5D2E,INVALC,INVALC,INVALC,0x5D2F,0x5D30,0x5D31,0x5D32,
	INVALC,INVALC,INVALC,INVALC,0x5D33,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5D34,0x5D35,0x5D36,0x5D37,
	0x5D38,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5D39,INVALC,INVALC,
	INVALC,0x5D3A,INVALC,0x5D3B,INVALC,0x5D3C,INVALC,INVALC,
	INVALC,0x5D3D,INVALC,0x5D3E,INVALC,INVALC,0x5D3F,INVALC,
	INVALC,0x5D40,INVALC,INVALC,INVALC,0x5D41,INVALC,0x5D42,
	/* Block 139, Array index 0x4300 */
	INVALC,INVALC,INVALC,INVALC,0x5D43,0x5D44,INVALC,0x5D45,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D46,
	INVALC,0x5D47,0x5D48,INVALC,0x5D49,0x5D4A,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D4B,INVALC,
	0x5D4C,INVALC,0x5D4D,INVALC,0x5D4E,INVALC,0x5D4F,INVALC,
	INVALC,INVALC,INVALC,0x5D50,0x5D51,INVALC,INVALC,0x5D52,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5D53,INVALC,0x5D54,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5D55,0x5D56,INVALC,
	0x5D57,INVALC,INVALC,0x5D58,INVALC,0x5D59,INVALC,0x5D5A,
	INVALC,0x5D5B,INVALC,INVALC,INVALC,0x5D5C,0x5D5D,INVALC,
	INVALC,INVALC,INVALC,0x5D5E,INVALC,INVALC,0x5D5F,0x5D60,
	0x5D61,INVALC,INVALC,INVALC,0x5D62,0x5D63,INVALC,INVALC,
	INVALC,0x5D64,INVALC,INVALC,INVALC,0x5D65,INVALC,0x5D66,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5D67,0x5D68,0x5D69,
	INVALC,0x5D6A,0x5D6B,0x5D6C,INVALC,INVALC,0x5D6D,0x5D6E,
	0x5D6F,INVALC,INVALC,0x5D70,INVALC,INVALC,0x5D71,INVALC,
	INVALC,INVALC,INVALC,0x5D72,INVALC,INVALC,INVALC,0x5D73,
	0x5D74,INVALC,0x5D75,INVALC,INVALC,INVALC,0x5D76,0x5D77,
	INVALC,0x5D78,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D79,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5D7A,
	INVALC,0x5D7B,INVALC,INVALC,INVALC,INVALC,0x5D7C,0x5D7D,
	INVALC,INVALC,INVALC,0x5D7E,INVALC,INVALC,0x5E21,0x5E22,
	INVALC,INVALC,INVALC,0x5E23,INVALC,INVALC,0x5E24,INVALC,
	INVALC,INVALC,INVALC,0x5E25,INVALC,INVALC,0x5E26,INVALC,
	0x5E27,0x5E28,0x5E29,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5E2A,INVALC,0x5E2B,0x5E2C,0x5E2D,INVALC,0x5E2E,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E2F,INVALC,0x5E30,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E31,INVALC,INVALC,INVALC,
	0x5E32,INVALC,INVALC,INVALC,0x5E33,0x5E34,0x5E35,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E36,INVALC,INVALC,0x5E37,
	/* Block 140, Array index 0x4400 */
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E38,0x5E39,INVALC,
	INVALC,INVALC,0x5E3F,0x5E3A,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5E3B,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E3C,INVALC,0x5E3D,0x5E3E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E40,INVALC,INVALC,
	0x5E41,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E42,
	INVALC,INVALC,INVALC,INVALC,0x5E43,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5E44,0x5E45,0x5E46,0x5E47,0x5E48,INVALC,
	0x5E49,INVALC,INVALC,INVALC,INVALC,0x5E4E,INVALC,INVALC,
	INVALC,INVALC,0x5E4A,0x5E4B,0x5E4C,INVALC,INVALC,INVALC,
	INVALC,0x5E4D,INVALC,INVALC,INVALC,INVALC,0x5E4F,INVALC,
	INVALC,INVALC,INVALC,0x5E50,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E51,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E52,INVALC,
	0x5E53,0x5E54,INVALC,INVALC,0x5E55,INVALC,0x5E56,INVALC,
	INVALC,0x5E57,INVALC,INVALC,0x5E58,0x5E59,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E5A,INVALC,0x5E5B,INVALC,0x5E5C,
	INVALC,INVALC,INVALC,INVALC,0x5E5D,0x5E5E,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5E5F,INVALC,0x5E60,0x5E61,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 141, Array index 0x4500 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5E62,0x5E63,INVALC,INVALC,INVALC,0x5E64,0x5E65,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5E66,INVALC,0x5E67,
	INVALC,0x5E68,INVALC,0x5E69,INVALC,INVALC,INVALC,0x5E6A,
	INVALC,0x5E6B,INVALC,0x5E6C,0x5E6D,INVALC,INVALC,0x5E6E,
	0x5E6F,0x5E72,INVALC,0x5E70,INVALC,0x5E71,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E73,0x5E74,INVALC,0x5E75,INVALC,
	0x5E76,0x5E77,INVALC,INVALC,INVALC,0x5E78,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x5E79,INVALC,0x5E7A,0x5E7B,INVALC,
	INVALC,INVALC,INVALC,0x5E7C,INVALC,INVALC,0x5E7D,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5E7E,0x5F21,
	INVALC,INVALC,INVALC,0x5F22,INVALC,INVALC,INVALC,INVALC,
	0x5F23,INVALC,0x5F24,0x5F25,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F26,INVALC,0x5F27,0x5F28,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F29,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F2A,0x5F2B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F2C,0x5F2D,INVALC,
	INVALC,0x5F2E,INVALC,0x5F2F,INVALC,INVALC,INVALC,0x5F30,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F32,0x5F31,INVALC,
	INVALC,0x5F33,INVALC,INVALC,INVALC,0x5F34,INVALC,INVALC,
	INVALC,0x5F35,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5F36,INVALC,INVALC,INVALC,0x5F37,INVALC,INVALC,0x5F38,
	0x5F39,INVALC,0x5F3A,INVALC,INVALC,0x5F3B,INVALC,0x5F3C,
	0x5F3D,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F3E,0x5F3F,
	/* Block 142, Array index 0x4600 */
	INVALC,0x5F40,INVALC,0x5F41,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F42,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x5F43,INVALC,INVALC,INVALC,INVALC,0x5F44,
	INVALC,INVALC,INVALC,0x5F45,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F46,INVALC,INVALC,
	INVALC,0x5F47,INVALC,INVALC,0x5F48,INVALC,0x5F49,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F4A,
	INVALC,INVALC,0x5F4B,INVALC,0x5F4C,INVALC,INVALC,INVALC,
	0x5F4D,INVALC,INVALC,INVALC,INVALC,0x5F4E,INVALC,INVALC,
	0x5F4F,0x5F50,INVALC,INVALC,INVALC,0x5F51,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F52,0x5F53,
	0x5F54,INVALC,INVALC,INVALC,INVALC,INVALC,0x5F55,INVALC,
	INVALC,INVALC,INVALC,0x5F56,0x5F57,INVALC,INVALC,0x5F58,
	INVALC,INVALC,0x5F59,INVALC,INVALC,0x5F5A,INVALC,0x5F5B,
	INVALC,0x5F5C,INVALC,0x5F5D,0x5F6F,INVALC,INVALC,INVALC,
	0x5F5E,INVALC,INVALC,INVALC,INVALC,0x5F5F,0x5F60,0x5F61,
	0x5F62,INVALC,0x5F63,INVALC,INVALC,INVALC,0x5F64,INVALC,
	INVALC,0x5F65,INVALC,INVALC,0x5F66,0x5F67,INVALC,0x5F68,
	INVALC,0x5F69,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x5F6A,0x5F6B,INVALC,0x5F6C,
	INVALC,INVALC,INVALC,INVALC,0x5F6D,INVALC,INVALC,INVALC,
	0x5F6E,0x5F70,0x5F71,INVALC,0x5F72,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x5F73,INVALC,INVALC,
	/* Block 143, Array index 0x4700 */
	INVALC,0x5F74,INVALC,INVALC,0x5F75,0x5F76,0x5F77,INVALC,
	INVALC,INVALC,INVALC,0x5F78,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x5F79,INVALC,INVALC,0x5F7A,INVALC,0x5F7B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x5F7C,0x5F7D,0x5F7E,0x6021,INVALC,INVALC,0x6022,0x6023,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6024,INVALC,0x6025,INVALC,INVALC,0x6026,0x6027,
	0x6028,0x6029,INVALC,INVALC,INVALC,0x602A,INVALC,INVALC,
	0x602B,0x602C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x602D,INVALC,0x602E,0x602F,0x6030,
	INVALC,INVALC,INVALC,INVALC,0x6031,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6032,0x6033,0x6034,0x6035,INVALC,
	INVALC,0x6036,0x6037,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6038,INVALC,INVALC,0x6039,0x603A,INVALC,0x603B,
	0x603C,0x603D,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x603E,0x603F,0x6040,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6041,0x6042,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6043,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6044,INVALC,0x6045,INVALC,INVALC,0x6046,INVALC,INVALC,
	INVALC,INVALC,0x6047,0x6048,INVALC,0x6049,0x604A,INVALC,
	INVALC,INVALC,0x604B,INVALC,INVALC,INVALC,INVALC,0x604C,
	INVALC,0x604D,INVALC,INVALC,INVALC,0x604E,0x604F,INVALC,
	INVALC,INVALC,INVALC,0x6050,INVALC,0x6051,INVALC,INVALC,
	INVALC,INVALC,0x6052,0x6053,INVALC,INVALC,INVALC,INVALC,
	0x6054,0x6055,INVALC,0x6056,0x6057,INVALC,INVALC,0x6058,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6059,
	INVALC,0x605A,INVALC,INVALC,0x605B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x605C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x605D,INVALC,INVALC,INVALC,INVALC,0x6064,0x605E,INVALC,
	0x605F,0x6060,INVALC,INVALC,INVALC,INVALC,INVALC,0x6061,
	INVALC,0x6062,0x6063,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 144, Array index 0x4800 */
	0x6065,INVALC,0x6066,INVALC,INVALC,INVALC,INVALC,0x6067,
	0x6068,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6069,
	0x606A,INVALC,INVALC,INVALC,INVALC,INVALC,0x606B,0x606C,
	0x606D,INVALC,INVALC,INVALC,INVALC,INVALC,0x606E,INVALC,
	0x606F,0x6070,INVALC,0x6071,INVALC,0x6072,INVALC,0x6073,
	0x6074,INVALC,INVALC,INVALC,0x6075,0x6076,0x6077,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6078,0x6079,0x607A,0x607B,
	INVALC,INVALC,0x607C,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x607D,0x607E,INVALC,0x6121,INVALC,INVALC,INVALC,0x6122,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6123,
	INVALC,0x6124,0x6125,0x6126,0x6127,0x6128,INVALC,INVALC,
	0x6129,INVALC,INVALC,INVALC,INVALC,0x612A,0x612B,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x612C,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x612D,INVALC,INVALC,
	0x612E,0x612F,INVALC,INVALC,0x6130,0x6131,0x6132,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6133,0x6134,INVALC,
	0x6135,INVALC,INVALC,INVALC,INVALC,INVALC,0x6136,INVALC,
	0x6137,0x6138,INVALC,INVALC,INVALC,INVALC,0x6139,INVALC,
	INVALC,INVALC,0x613A,0x613B,INVALC,0x613C,INVALC,INVALC,
	0x613D,INVALC,0x613E,0x613F,INVALC,0x6140,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6141,INVALC,INVALC,0x6142,0x6143,INVALC,INVALC,INVALC,
	0x6144,INVALC,INVALC,INVALC,INVALC,INVALC,0x6145,INVALC,
	INVALC,0x6146,INVALC,INVALC,INVALC,0x6147,0x6148,INVALC,
	INVALC,INVALC,INVALC,0x6149,INVALC,INVALC,0x614A,INVALC,
	/* Block 145, Array index 0x4900 */
	INVALC,INVALC,0x614B,INVALC,0x614C,INVALC,INVALC,INVALC,
	0x614D,INVALC,INVALC,INVALC,0x614E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x614F,INVALC,INVALC,0x6150,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6151,0x6152,0x6154,INVALC,0x6155,0x6156,INVALC,0x6153,
	INVALC,INVALC,INVALC,0x6157,0x6158,INVALC,INVALC,0x6159,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x615A,
	INVALC,INVALC,INVALC,0x615B,0x615C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x615D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x615E,INVALC,0x615F,INVALC,INVALC,
	INVALC,INVALC,0x6160,INVALC,INVALC,INVALC,0x6161,0x6162,
	INVALC,INVALC,INVALC,INVALC,0x6163,INVALC,INVALC,INVALC,
	0x6164,INVALC,INVALC,INVALC,0x6165,INVALC,INVALC,INVALC,
	INVALC,0x6166,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6167,INVALC,INVALC,
	0x6168,INVALC,INVALC,0x6169,0x616A,INVALC,0x616B,INVALC,
	0x616C,INVALC,INVALC,INVALC,INVALC,0x616D,INVALC,0x616E,
	0x616F,0x6170,INVALC,0x6171,INVALC,INVALC,INVALC,INVALC,
	0x6172,0x6173,0x6174,INVALC,INVALC,0x6175,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6176,INVALC,0x6177,0x6178,0x6179,INVALC,0x617A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x617B,0x617D,INVALC,
	INVALC,INVALC,INVALC,0x617E,0x6221,0x6222,INVALC,0x6223,
	0x6224,INVALC,INVALC,INVALC,0x617C,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x622D,INVALC,INVALC,0x6225,INVALC,0x6226,
	0x6227,0x6228,INVALC,INVALC,0x6229,0x622A,INVALC,0x622B,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x622C,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x622F,INVALC,INVALC,INVALC,0x6230,
	0x6231,INVALC,INVALC,INVALC,0x6232,INVALC,0x622E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6233,0x6234,
	/* Block 146, Array index 0x4A00 */
	0x6235,INVALC,INVALC,INVALC,0x6236,0x6237,0x6238,INVALC,
	0x6239,INVALC,INVALC,INVALC,INVALC,0x623A,INVALC,INVALC,
	0x623B,INVALC,INVALC,INVALC,0x623C,INVALC,0x623D,0x623E,
	0x623F,INVALC,0x6240,INVALC,0x6241,INVALC,0x6242,INVALC,
	0x6243,INVALC,0x6245,0x6246,INVALC,0x6244,INVALC,0x6247,
	INVALC,0x6248,INVALC,INVALC,INVALC,INVALC,0x6249,0x624A,
	INVALC,0x624B,INVALC,INVALC,0x624C,INVALC,0x624D,0x624E,
	INVALC,0x624F,0x6250,INVALC,0x6251,0x6252,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6253,INVALC,INVALC,INVALC,0x6254,
	0x6255,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6256,
	INVALC,INVALC,INVALC,0x6257,INVALC,INVALC,INVALC,0x6258,
	INVALC,0x6259,0x625A,0x625B,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x625C,INVALC,INVALC,0x625D,INVALC,INVALC,0x625E,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x625F,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6260,INVALC,INVALC,INVALC,
	INVALC,0x6261,0x6262,0x6263,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6264,INVALC,0x6265,INVALC,0x6266,0x6267,INVALC,
	INVALC,INVALC,0x6268,INVALC,INVALC,INVALC,0x6269,INVALC,
	INVALC,0x626A,INVALC,0x626B,0x626C,0x626D,INVALC,INVALC,
	0x626E,INVALC,INVALC,INVALC,INVALC,INVALC,0x626F,INVALC,
	INVALC,0x6270,INVALC,INVALC,INVALC,INVALC,0x6271,INVALC,
	0x6272,INVALC,INVALC,INVALC,0x6273,0x6274,0x6275,INVALC,
	0x6276,0x6277,0x6278,0x6279,INVALC,INVALC,0x627A,INVALC,
	INVALC,INVALC,INVALC,0x627B,0x627C,0x627D,INVALC,0x627E,
	INVALC,INVALC,0x6321,0x6322,INVALC,0x6323,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6324,0x6325,INVALC,INVALC,0x6326,
	INVALC,0x6327,0x6328,INVALC,INVALC,INVALC,0x6329,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x632A,0x632B,INVALC,INVALC,
	INVALC,0x632C,0x632D,INVALC,0x632E,0x632F,0x6330,0x6331,
	0x6332,0x6333,INVALC,INVALC,INVALC,INVALC,INVALC,0x6334,
	INVALC,0x6335,INVALC,0x6336,INVALC,0x6337,INVALC,INVALC,
	/* Block 147, Array index 0x4B00 */
	0x6338,0x6339,INVALC,INVALC,0x633A,0x633B,0x633C,0x633D,
	INVALC,0x633E,0x633F,INVALC,0x6340,INVALC,INVALC,INVALC,
	0x6341,INVALC,0x6342,0x6343,INVALC,INVALC,0x6344,INVALC,
	0x6345,INVALC,INVALC,INVALC,0x6346,0x6347,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6348,0x6349,0x634A,0x634B,INVALC,
	0x634C,INVALC,INVALC,INVALC,INVALC,INVALC,0x634D,0x634E,
	0x634F,INVALC,INVALC,0x6350,INVALC,0x6351,0x6352,INVALC,
	0x6353,0x6354,0x6355,INVALC,0x6356,INVALC,0x6357,INVALC,
	0x6358,INVALC,0x6359,0x635A,INVALC,INVALC,0x635B,0x635C,
	INVALC,INVALC,0x635D,INVALC,INVALC,0x635E,0x635F,0x6360,
	INVALC,0x6361,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6362,0x6363,INVALC,INVALC,0x6364,0x6365,INVALC,INVALC,
	0x6366,0x6367,INVALC,INVALC,INVALC,0x6368,INVALC,0x6369,
	0x636A,0x636B,INVALC,INVALC,INVALC,INVALC,0x636C,0x636D,
	0x636E,INVALC,INVALC,INVALC,INVALC,0x636F,0x6370,0x6371,
	0x6372,0x6373,INVALC,0x6374,0x6375,0x6376,INVALC,0x6377,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6378,0x6379,0x637A,INVALC,INVALC,0x637B,0x637C,INVALC,
	INVALC,INVALC,0x637D,INVALC,INVALC,INVALC,INVALC,0x637E,
	INVALC,0x6421,INVALC,INVALC,INVALC,INVALC,INVALC,0x6422,
	0x6423,INVALC,INVALC,INVALC,0x6424,0x6425,INVALC,0x6426,
	0x6427,INVALC,INVALC,0x6428,INVALC,INVALC,INVALC,0x6429,
	INVALC,INVALC,0x642A,INVALC,INVALC,INVALC,0x642B,INVALC,
	0x642C,INVALC,0x642D,0x642E,0x642F,0x6430,INVALC,0x6431,
	0x6432,0x6433,0x6434,0x6435,INVALC,0x6436,0x6437,0x6438,
	0x6439,INVALC,INVALC,0x643A,0x643B,0x643C,0x643D,INVALC,
	0x643E,INVALC,INVALC,0x643F,INVALC,0x6440,INVALC,0x6441,
	0x6442,0x6443,INVALC,INVALC,0x6444,0x6445,INVALC,0x6446,
	0x6447,0x6448,INVALC,0x6449,INVALC,0x644A,INVALC,0x644B,
	0x644C,INVALC,INVALC,INVALC,0x644D,INVALC,0x644E,INVALC,
	0x644F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6450,INVALC,0x6451,INVALC,INVALC,INVALC,0x6452,
	/* Block 148, Array index 0x4C00 */
	0x6453,INVALC,0x6454,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6455,INVALC,INVALC,INVALC,INVALC,0x6456,INVALC,INVALC,
	INVALC,0x6457,INVALC,INVALC,0x6458,0x6459,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x645A,0x645B,0x645C,0x645D,
	INVALC,0x645E,INVALC,INVALC,0x645F,0x6460,INVALC,0x6461,
	INVALC,0x6462,0x6463,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6464,0x6465,INVALC,0x6466,0x6467,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6468,
	0x6469,0x646A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x646B,0x646C,0x646D,INVALC,INVALC,0x646E,INVALC,0x646F,
	0x6470,INVALC,0x6471,INVALC,INVALC,INVALC,0x6472,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6473,0x6474,INVALC,0x6475,
	INVALC,0x6476,0x6477,INVALC,INVALC,0x6478,INVALC,0x6479,
	0x647A,0x647B,INVALC,0x647C,0x647D,INVALC,0x647E,INVALC,
	INVALC,INVALC,0x6521,INVALC,INVALC,0x6522,INVALC,0x6523,
	0x6524,0x6525,0x6526,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6527,INVALC,0x6528,0x6529,INVALC,0x652A,INVALC,0x652B,
	INVALC,INVALC,0x652C,INVALC,INVALC,0x652D,INVALC,INVALC,
	0x652E,INVALC,INVALC,0x652F,INVALC,INVALC,0x6530,INVALC,
	INVALC,0x6531,INVALC,0x6532,0x6533,INVALC,0x6534,INVALC,
	0x6535,0x653B,INVALC,0x6536,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6537,0x6538,0x6539,INVALC,
	INVALC,INVALC,0x653A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x653C,INVALC,INVALC,0x653D,0x653E,0x653F,0x6540,
	INVALC,0x6541,0x6542,0x6543,0x6544,0x6545,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6546,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6547,INVALC,INVALC,0x6548,INVALC,0x6549,0x654A,
	INVALC,INVALC,0x654B,INVALC,INVALC,INVALC,0x654C,0x654D,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x654F,
	0x6550,0x654E,0x6551,0x6552,INVALC,0x6553,INVALC,INVALC,
	/* Block 149, Array index 0x4D00 */
	INVALC,0x6554,0x6555,INVALC,0x6556,INVALC,INVALC,INVALC,
	0x6557,0x6558,INVALC,INVALC,INVALC,0x6559,0x655A,0x655B,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x655C,0x655D,0x655E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x655F,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6560,0x6561,
	INVALC,0x6562,0x6563,0x6564,0x6565,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6566,INVALC,0x6568,INVALC,0x6567,
	INVALC,INVALC,INVALC,0x6569,INVALC,0x656A,INVALC,INVALC,
	0x656B,INVALC,0x656C,INVALC,0x656D,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x656E,INVALC,INVALC,
	INVALC,0x656F,INVALC,INVALC,0x6570,INVALC,INVALC,0x6571,
	INVALC,0x6572,INVALC,0x6573,INVALC,INVALC,INVALC,INVALC,
	0x6574,INVALC,INVALC,0x6575,INVALC,0x6576,0x6577,0x6578,
	INVALC,0x6579,0x657A,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x657C,0x657B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 150, Array index 0x4E00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x657D,0x657E,INVALC,INVALC,INVALC,INVALC,0x6621,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6622,INVALC,INVALC,INVALC,
	0x6623,INVALC,INVALC,INVALC,0x6624,0x6625,0x6626,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6627,0x6628,0x6629,
	INVALC,0x662A,INVALC,INVALC,INVALC,INVALC,0x662B,INVALC,
	INVALC,0x662C,INVALC,0x662D,0x662E,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x662F,INVALC,0x6630,INVALC,
	INVALC,INVALC,0x6631,INVALC,INVALC,0x6632,INVALC,0x6633,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6634,INVALC,
	0x6635,0x6636,INVALC,0x6637,INVALC,INVALC,INVALC,INVALC,
	0x6638,0x6639,0x663A,0x663B,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x663C,0x663D,INVALC,INVALC,0x663E,0x663F,0x6640,
	0x6641,INVALC,INVALC,INVALC,0x6642,INVALC,0x6643,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 151, Array index 0x4F00 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6644,0x6645,INVALC,
	INVALC,INVALC,0x6646,INVALC,0x6647,0x6648,0x6649,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x664A,INVALC,INVALC,INVALC,
	INVALC,0x664B,INVALC,0x664C,INVALC,INVALC,INVALC,0x664D,
	0x664E,0x664F,0x6650,INVALC,0x6651,0x6652,INVALC,INVALC,
	INVALC,0x6653,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6654,INVALC,0x6655,INVALC,0x6656,0x6657,
	0x6658,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6659,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x665A,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x665B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x665C,0x665D,INVALC,0x665E,0x665F,
	INVALC,0x6660,0x6661,0x6662,0x6663,INVALC,INVALC,INVALC,
	INVALC,0x6664,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6665,INVALC,INVALC,INVALC,INVALC,0x6666,INVALC,
	INVALC,INVALC,0x6667,INVALC,INVALC,0x6668,INVALC,0x6669,
	INVALC,INVALC,INVALC,INVALC,0x666A,0x666B,0x666C,INVALC,
	INVALC,0x666D,INVALC,INVALC,INVALC,INVALC,0x666E,0x666F,
	INVALC,INVALC,INVALC,0x6670,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6671,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6672,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6673,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6675,INVALC,0x6676,INVALC,INVALC,0x6677,0x6678,0x6679,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x667A,INVALC,INVALC,INVALC,INVALC,INVALC,0x667B,
	INVALC,0x667C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x667D,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 152, Array index 0x5000 */
	INVALC,INVALC,0x667E,0x6721,INVALC,0x6722,INVALC,INVALC,
	INVALC,0x6723,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6724,0x6725,INVALC,0x6726,INVALC,INVALC,
	INVALC,0x6727,0x6728,0x6729,INVALC,INVALC,INVALC,INVALC,
	0x672A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x672B,INVALC,0x672C,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x672D,INVALC,0x672E,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x672F,INVALC,INVALC,INVALC,0x6730,0x6731,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6732,INVALC,0x6733,
	0x6734,INVALC,0x6735,0x6736,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6737,INVALC,INVALC,INVALC,0x6738,
	INVALC,INVALC,0x6739,INVALC,INVALC,INVALC,0x673A,INVALC,
	INVALC,INVALC,INVALC,0x673B,INVALC,INVALC,0x673C,0x673D,
	0x673E,INVALC,INVALC,0x673F,INVALC,0x6740,INVALC,0x6741,
	0x6742,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6743,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6744,0x6745,0x6746,
	INVALC,0x6747,0x6748,INVALC,INVALC,INVALC,0x6749,0x674A,
	INVALC,INVALC,0x674B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x674C,INVALC,0x674D,INVALC,
	INVALC,0x674E,0x674F,INVALC,INVALC,0x6750,0x6751,INVALC,
	0x6752,0x6753,0x6754,INVALC,0x6755,INVALC,0x6756,0x6757,
	INVALC,0x6758,INVALC,INVALC,0x6759,0x675A,INVALC,0x675B,
	INVALC,0x675C,0x675D,INVALC,0x675E,0x675F,0x6760,INVALC,
	0x6761,0x6762,INVALC,INVALC,0x6763,INVALC,INVALC,0x6764,
	0x6765,0x6766,INVALC,0x676A,INVALC,0x6767,0x6768,INVALC,
	0x6769,0x676B,INVALC,INVALC,0x676C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x676D,
	INVALC,0x676E,INVALC,INVALC,0x676F,INVALC,INVALC,0x6770,
	0x6771,INVALC,0x6772,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 153, Array index 0x5100 */
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6773,
	INVALC,INVALC,0x6774,INVALC,INVALC,0x6776,0x6777,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6778,INVALC,0x6779,INVALC,
	INVALC,0x6775,INVALC,INVALC,0x677A,INVALC,0x677B,INVALC,
	0x677C,INVALC,INVALC,0x677D,INVALC,0x6828,0x677E,INVALC,
	INVALC,INVALC,INVALC,0x6821,INVALC,INVALC,0x6822,0x6823,
	0x6824,INVALC,0x6825,0x6826,INVALC,0x6827,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6829,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x682A,INVALC,INVALC,0x682B,
	INVALC,INVALC,0x682C,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x682D,0x682E,0x682F,INVALC,INVALC,0x6830,0x6831,
	INVALC,0x6832,0x6833,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6834,0x6835,INVALC,0x6836,0x6837,INVALC,
	INVALC,INVALC,0x6838,INVALC,0x6839,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x683A,INVALC,0x683B,0x683C,INVALC,
	0x683D,INVALC,INVALC,INVALC,0x683E,INVALC,INVALC,0x683F,
	0x6840,INVALC,0x6841,0x6842,INVALC,INVALC,INVALC,0x6843,
	INVALC,INVALC,0x6844,INVALC,INVALC,0x6845,INVALC,INVALC,
	0x6846,INVALC,INVALC,INVALC,0x6847,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6848,INVALC,0x6849,INVALC,0x684A,0x684B,0x684C,
	INVALC,INVALC,0x684D,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x684E,INVALC,INVALC,0x684F,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 154, Array index 0x5200 */
	INVALC,INVALC,0x6850,INVALC,INVALC,INVALC,INVALC,0x6851,
	0x6852,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6853,INVALC,INVALC,INVALC,0x6854,0x6855,0x6856,
	INVALC,INVALC,0x6857,0x6858,0x6859,INVALC,INVALC,0x685A,
	INVALC,INVALC,0x685B,INVALC,INVALC,INVALC,0x685C,0x685D,
	INVALC,INVALC,INVALC,0x685E,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x685F,0x6860,0x6861,0x6862,0x6863,INVALC,INVALC,
	INVALC,0x6864,0x6865,0x6866,0x6867,INVALC,INVALC,INVALC,
	0x6868,0x6869,INVALC,INVALC,INVALC,INVALC,0x686A,0x686B,
	0x686C,INVALC,INVALC,INVALC,INVALC,0x686D,0x686E,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x686F,INVALC,INVALC,INVALC,
	0x6870,0x6871,INVALC,0x6872,0x6873,INVALC,0x6874,0x6875,
	0x6876,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6877,INVALC,0x6878,INVALC,0x6879,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x687A,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x687B,0x687C,0x687D,INVALC,INVALC,0x687E,INVALC,INVALC,
	INVALC,0x6921,0x6922,INVALC,INVALC,0x6923,INVALC,0x6924,
	INVALC,INVALC,INVALC,0x6925,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6926,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6927,0x6928,INVALC,INVALC,INVALC,
	INVALC,0x6929,0x692A,INVALC,0x692B,INVALC,0x692C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x692D,
	INVALC,INVALC,0x692E,0x692F,0x6930,INVALC,INVALC,INVALC,
	0x6931,INVALC,INVALC,INVALC,0x6932,0x6933,INVALC,INVALC,
	INVALC,0x6934,INVALC,INVALC,INVALC,0x6935,0x6936,INVALC,
	/* Block 155, Array index 0x5300 */
	INVALC,INVALC,0x6937,0x6938,0x6939,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x693A,0x693B,INVALC,INVALC,INVALC,
	0x693C,0x693D,INVALC,INVALC,INVALC,INVALC,0x693E,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x693F,INVALC,
	0x6940,INVALC,0x6941,0x6942,0x6943,INVALC,INVALC,0x6944,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6945,0x6946,INVALC,
	INVALC,INVALC,INVALC,0x6947,INVALC,0x6948,0x6949,INVALC,
	0x694A,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x694C,INVALC,INVALC,0x694D,INVALC,INVALC,0x694B,
	INVALC,INVALC,0x694E,0x694F,0x6950,INVALC,0x6951,INVALC,
	INVALC,0x6952,INVALC,INVALC,0x6953,INVALC,0x6954,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6955,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6956,INVALC,0x6957,INVALC,0x6958,0x6959,
	INVALC,INVALC,0x695A,INVALC,0x695B,0x695C,0x695D,INVALC,
	INVALC,0x695E,INVALC,0x695F,INVALC,INVALC,0x6960,0x6961,
	INVALC,0x6962,INVALC,0x6963,INVALC,INVALC,0x6964,INVALC,
	0x6965,INVALC,INVALC,INVALC,INVALC,INVALC,0x6966,INVALC,
	0x6967,INVALC,0x6968,INVALC,INVALC,0x6969,0x696A,0x696B,
	INVALC,INVALC,INVALC,0x696C,0x696D,INVALC,INVALC,INVALC,
	0x696E,INVALC,INVALC,INVALC,0x696F,0x6970,INVALC,0x6971,
	INVALC,0x6972,INVALC,INVALC,0x6973,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6974,0x6975,INVALC,0x6976,INVALC,INVALC,
	INVALC,0x6977,0x6978,INVALC,INVALC,0x6979,INVALC,0x697A,
	/* Block 156, Array index 0x5400 */
	0x697B,0x697C,0x697D,0x697E,0x6A21,0x6A22,INVALC,INVALC,
	0x6A23,0x6A24,INVALC,0x6A25,0x6A26,0x6A27,0x6A28,INVALC,
	0x6A29,INVALC,0x6A2A,INVALC,INVALC,INVALC,0x6A2B,INVALC,
	INVALC,0x6A2C,INVALC,0x6A2D,0x6A2E,INVALC,INVALC,INVALC,
	0x6A2F,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A30,INVALC,
	INVALC,INVALC,INVALC,0x6A31,INVALC,0x6A32,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6A33,0x6A34,0x6A35,INVALC,0x6A36,
	INVALC,0x6A37,0x6A38,INVALC,INVALC,0x6A39,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6A3A,INVALC,INVALC,0x6A3B,0x6A3C,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6A3D,0x6A3E,0x6A3F,
	INVALC,INVALC,INVALC,0x6A40,INVALC,INVALC,0x6A41,INVALC,
	INVALC,0x6A42,INVALC,0x6A43,INVALC,0x6A44,0x6A45,INVALC,
	0x6A46,INVALC,0x6A47,0x6A48,0x6A49,0x6A4A,0x6A4B,INVALC,
	INVALC,INVALC,INVALC,0x6A4C,INVALC,0x6A4D,INVALC,0x6A4E,
	0x6A4F,0x6A50,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A51,
	0x6A52,INVALC,INVALC,INVALC,0x6A53,0x6A54,0x6A55,0x6A56,
	INVALC,0x6A57,0x6A58,0x6A59,INVALC,0x6A5A,INVALC,0x6A5B,
	0x6A5C,INVALC,INVALC,INVALC,0x6A5D,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6A5E,INVALC,INVALC,0x6A5F,0x6A60,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A61,0x6A62,
	INVALC,0x6A63,INVALC,INVALC,0x6A64,INVALC,INVALC,INVALC,
	0x6A65,0x6A66,0x6A67,INVALC,INVALC,INVALC,INVALC,0x6A68,
	0x6A69,INVALC,INVALC,0x6A6A,0x6A6B,INVALC,0x6A6C,0x6A6D,
	INVALC,0x6A6E,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A6F,
	0x6A70,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A71,INVALC,
	0x6A72,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6A73,
	0x6A74,INVALC,INVALC,INVALC,INVALC,0x6A75,INVALC,0x6A76,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6A77,INVALC,0x6A78,
	INVALC,INVALC,0x6A79,0x6A7A,INVALC,INVALC,INVALC,0x6A7B,
	INVALC,INVALC,INVALC,0x6A7C,INVALC,INVALC,INVALC,0x6A7D,
	0x6A7E,0x6B21,0x6B22,INVALC,INVALC,0x6B23,INVALC,0x6B24,
	/* Block 157, Array index 0x5500 */
	0x6B25,INVALC,0x6B26,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6B27,INVALC,INVALC,INVALC,0x6B28,
	INVALC,0x6B29,INVALC,INVALC,INVALC,INVALC,0x6B2A,INVALC,
	0x6B2B,0x6B2C,0x6B2D,INVALC,0x6B2E,INVALC,0x6B2F,INVALC,
	INVALC,INVALC,0x6B30,0x6B31,INVALC,INVALC,0x6B32,0x6B33,
	0x6B34,0x6B35,0x6B36,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B37,INVALC,INVALC,INVALC,0x6B38,0x6B39,0x6B3A,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6B3B,INVALC,INVALC,
	INVALC,0x6B3C,INVALC,0x6B3D,0x6B3E,0x6B3F,INVALC,INVALC,
	INVALC,0x6B40,0x6B41,INVALC,INVALC,INVALC,0x6B42,0x6B43,
	0x6B44,INVALC,INVALC,0x6B45,0x6B46,INVALC,0x6B47,INVALC,
	0x6B48,INVALC,INVALC,0x6B49,0x6B50,0x6B4A,0x6B4B,0x6B4C,
	INVALC,INVALC,INVALC,0x6B4D,INVALC,INVALC,INVALC,INVALC,
	0x6B52,0x6B4E,0x6B4F,0x6B51,INVALC,INVALC,0x6B53,INVALC,
	0x6B54,INVALC,0x6B55,INVALC,INVALC,0x6B56,INVALC,0x6B57,
	INVALC,INVALC,INVALC,0x6B58,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B59,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6B5A,INVALC,INVALC,INVALC,INVALC,0x6B5B,
	INVALC,0x6B5C,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	/* Block 158, Array index 0x5600 */
	INVALC,INVALC,0x6B5E,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6B5D,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6B5F,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B60,
	0x6B61,INVALC,INVALC,INVALC,0x6B62,0x6B63,0x6B64,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B65,
	0x6B66,INVALC,0x6B67,0x6B68,0x6B69,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6B6A,INVALC,0x6B6B,0x6B6D,INVALC,INVALC,
	INVALC,INVALC,0x6B6E,0x6B6F,INVALC,0x6B6C,INVALC,0x6B70,
	INVALC,INVALC,0x6B71,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6B72,0x6B73,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B74,
	INVALC,INVALC,0x6B76,0x6B75,INVALC,0x6B77,INVALC,INVALC,
	INVALC,0x6B78,0x6B79,0x6B7A,INVALC,INVALC,INVALC,INVALC,
	0x6B7B,INVALC,INVALC,INVALC,INVALC,INVALC,0x6B7C,0x6B7D,
	INVALC,INVALC,INVALC,0x6B7E,0x6C21,INVALC,0x6C22,INVALC,
	INVALC,INVALC,INVALC,0x6C23,0x6C24,INVALC,0x6C25,INVALC,
	INVALC,INVALC,0x6C26,INVALC,INVALC,0x6C27,0x6C28,INVALC,
	INVALC,INVALC,0x6C29,0x6C2A,INVALC,0x6C2B,0x6C2C,0x6C2D,
	0x6C2E,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6C2F,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6C30,INVALC,0x6C31,INVALC,0x6C32,INVALC,
	INVALC,0x6C33,INVALC,INVALC,INVALC,0x6C34,INVALC,INVALC,
	INVALC,0x6C35,INVALC,INVALC,0x6C36,INVALC,INVALC,0x6C37,
	INVALC,INVALC,INVALC,0x6C38,INVALC,INVALC,INVALC,0x6C39,
	INVALC,0x6C3A,0x6C3B,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6C3C,0x6C3D,0x6C3E,0x6C3F,
	INVALC,INVALC,0x6C40,INVALC,INVALC,INVALC,0x6C41,0x6C42,
	0x6C43,INVALC,INVALC,INVALC,INVALC,0x6C44,INVALC,0x6C45,
	INVALC,0x6C46,INVALC,0x6C47,INVALC,INVALC,0x6C48,INVALC,
	0x6C49,INVALC,INVALC,0x6C4A,0x6C4B,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6C4C,INVALC,
	/* Block 159, Array index 0x5700 */
	INVALC,INVALC,0x6C4E,INVALC,INVALC,INVALC,INVALC,0x6C4F,
	INVALC,INVALC,0x6C4D,INVALC,INVALC,INVALC,0x6C50,INVALC,
	0x6C51,0x6C52,0x6C53,INVALC,INVALC,0x6C54,0x6C55,INVALC,
	INVALC,0x6C56,INVALC,INVALC,0x6C57,0x6C58,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,0x6C59,0x6C5A,0x6C5B,INVALC,INVALC,INVALC,
	0x6C5C,INVALC,0x6C5D,0x6C5E,0x6C5F,0x6C60,INVALC,0x6C61,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6C62,0x6C63,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6C64,INVALC,
	0x6C65,INVALC,INVALC,0x6C66,INVALC,INVALC,0x6C67,INVALC,
	INVALC,INVALC,INVALC,INVALC,0x6C68,INVALC,INVALC,INVALC,
	0x6C69,INVALC,INVALC,INVALC,0x6C6A,INVALC,0x6C6B,0x6C6C,
	0x6C6D,INVALC,INVALC,0x6C6E,0x6C6F,0x6C70,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6C71,INVALC,
	0x6C72,INVALC,INVALC,0x6C73,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,0x6C74,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6C75,
	INVALC,INVALC,INVALC,INVALC,0x6C76,INVALC,INVALC,0x6C77,
	INVALC,INVALC,INVALC,INVALC,0x6C78,0x6C79,0x6C7A,INVALC,
	0x6C7B,0x6C7C,0x6C7D,INVALC,INVALC,0x6C7E,INVALC,INVALC,
	0x6D21,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D22,
	/* Block 160, Array index 0x5800 */
	INVALC,INVALC,0x6D23,0x6D24,INVALC,INVALC,INVALC,INVALC,
	INVALC,0x6D25,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D26,
	0x6D27,0x6D28,0x6D29,INVALC,0x6D2A,INVALC,0x6D2B,0x6D2C,
	INVALC,0x6D2D,0x6D2E,0x6D2F,INVALC,INVALC,INVALC,0x6D30,
	INVALC,INVALC,0x6D31,INVALC,INVALC,INVALC,0x6D32,INVALC,
	INVALC,INVALC,0x6D33,0x6D34,INVALC,INVALC,INVALC,0x6D35,
	INVALC,0x6D36,0x6D37,INVALC,0x6D38,INVALC,INVALC,0x6D39,
	INVALC,0x6D3A,0x6D3B,INVALC,0x6D3C,0x6D3D,INVALC,0x6D3E,
	INVALC,0x6D3F,INVALC,0x6D40,0x6D41,0x6D42,0x6D43,0x6D44,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,0x6D45,INVALC,0x6D46,0x6D47,0x6D48,
	0x6D49,INVALC,0x6D4A,INVALC,INVALC,0x6D4B,0x6D4C,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	0x6D4D,0x6D4E,INVALC,INVALC,INVALC,0x6D4F,0x6D50,0x6D51,
	0x6D52,0x6D53,INVALC,0x6D54,INVALC,0x6D55,INVALC,INVALC,
	INVALC,INVALC,0x6D56,INVALC,INVALC,0x6D57,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D58,
	0x6D59,0x6D5A,0x6D5B,INVALC,0x6D5C,INVALC,0x6D5D,0x6D5E,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,0x6D5F,INVALC,
	INVALC,0x6D60,0x6D61,0x6D62,INVALC,0x6D63,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
	INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,INVALC,
};

#endif /* ICONV_FROM_UCS_CCS_JIS_X0212_1990 && !defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * 16-bit UCS -> jis_x0212_1990 size-optimized table (23194 bytes).
 * ======================================================================
 */

#if defined (ICONV_FROM_UCS_CCS_JIS_X0212_1990) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))

static const __uint16_t
from_ucs_size_jis_x0212_1990[] =
{
	0x01AA, /* Ranges number */
	0x0E83, /* Unranged codes number */
	0x1047, /* First unranged code index */
	/* Ranges list: first code, last Code, array index. */
	/* Array index: 0x0003 */ 0x00BF, 0x017E, 0x0501,
	/* Array index: 0x0006 */ 0x01CD, 0x01DC, 0x05C1,
	/* Array index: 0x0009 */ 0x02D8, 0x02DD, 0x05D1,
	/* Array index: 0x000C */ 0x0384, 0x0390, 0x05D7,
	/* Array index: 0x000F */ 0x03AA, 0x03B0, 0x05E4,
	/* Array index: 0x0012 */ 0x03CA, 0x03CE, 0x05EB,
	/* Array index: 0x0015 */ 0x0402, 0x040F, 0x05F0,
	/* Array index: 0x0018 */ 0x0452, 0x045F, 0x05FE,
	/* Array index: 0x001B */ 0x4E2E, 0x4E30, 0x060C,
	/* Array index: 0x001E */ 0x4EE0, 0x4EE2, 0x060F,
	/* Array index: 0x0021 */ 0x4EFD, 0x4F03, 0x0612,
	/* Array index: 0x0024 */ 0x4F15, 0x4F19, 0x0619,
	/* Array index: 0x0027 */ 0x4F48, 0x4F4C, 0x061E,
	/* Array index: 0x002A */ 0x4F77, 0x4F85, 0x0623,
	/* Array index: 0x002D */ 0x4F90, 0x4F94, 0x0632,
	/* Array index: 0x0030 */ 0x4FB9, 0x4FC1, 0x0637,
	/* Array index: 0x0033 */ 0x4FC5, 0x4FCF, 0x0640,
	/* Array index: 0x0036 */ 0x4FFC, 0x5001, 0x064B,
	/* Array index: 0x0039 */ 0x5017, 0x501E, 0x0651,
	/* Array index: 0x003C */ 0x5040, 0x5046, 0x0659,
	/* Array index: 0x003F */ 0x5051, 0x5053, 0x0660,
	/* Array index: 0x0042 */ 0x505F, 0x5067, 0x0663,
	/* Array index: 0x0045 */ 0x508E, 0x5096, 0x066C,
	/* Array index: 0x0048 */ 0x509B, 0x50A2, 0x0675,
	/* Array index: 0x004B */ 0x5106, 0x5110, 0x067D,
	/* Array index: 0x004E */ 0x5119, 0x511E, 0x0688,
	/* Array index: 0x0051 */ 0x5131, 0x5139, 0x068E,
	/* Array index: 0x0054 */ 0x51CF, 0x51D5, 0x0697,
	/* Array index: 0x0057 */ 0x51F2, 0x51F4, 0x069E,
	/* Array index: 0x005A */ 0x5212, 0x5218, 0x06A1,
	/* Array index: 0x005D */ 0x525F, 0x5261, 0x06A8,
	/* Array index: 0x0060 */ 0x5277, 0x5279, 0x06AB,
	/* Array index: 0x0063 */ 0x5293, 0x529C, 0x06AE,
	/* Array index: 0x0066 */ 0x52A4, 0x52A7, 0x06B8,
	/* Array index: 0x0069 */ 0x52B6, 0x52BD, 0x06BC,
	/* Array index: 0x006C */ 0x52E8, 0x52EC, 0x06C4,
	/* Array index: 0x006F */ 0x530A, 0x530C, 0x06C9,
	/* Array index: 0x0072 */ 0x531B, 0x531F, 0x06CC,
	/* Array index: 0x0075 */ 0x5325, 0x532D, 0x06D1,
	/* Array index: 0x0078 */ 0x533C, 0x533E, 0x06DA,
	/* Array index: 0x007B */ 0x53B4, 0x53BA, 0x06DD,
	/* Array index: 0x007E */ 0x54A6, 0x54AE, 0x06E4,
	/* Array index: 0x0081 */ 0x54B7, 0x54BB, 0x06ED,
	/* Array index: 0x0084 */ 0x54FC, 0x5501, 0x06F2,
	/* Array index: 0x0087 */ 0x5508, 0x550E, 0x06F8,
	/* Array index: 0x008A */ 0x553B, 0x553D, 0x06FF,
	/* Array index: 0x008D */ 0x558E, 0x5594, 0x0702,
	/* Array index: 0x0090 */ 0x55D1, 0x55D3, 0x0709,
	/* Array index: 0x0093 */ 0x560D, 0x5612, 0x070C,
	/* Array index: 0x0096 */ 0x5639, 0x5646, 0x0712,
	/* Array index: 0x0099 */ 0x565E, 0x5663, 0x0720,
	/* Array index: 0x009C */ 0x5699, 0x569F, 0x0726,
	/* Array index: 0x009F */ 0x56A6, 0x56AD, 0x072D,
	/* Array index: 0x00A2 */ 0x56C9, 0x56D0, 0x0735,
	/* Array index: 0x00A5 */ 0x56E4, 0x56E8, 0x073D,
	/* Array index: 0x00A8 */ 0x5720, 0x5725, 0x0742,
	/* Array index: 0x00AB */ 0x573D, 0x573F, 0x0748,
	/* Array index: 0x00AE */ 0x576B, 0x577E, 0x074B,
	/* Array index: 0x00B1 */ 0x5799, 0x57A1, 0x075F,
	/* Array index: 0x00B4 */ 0x57A7, 0x57A9, 0x0768,
	/* Array index: 0x00B7 */ 0x57FD, 0x57FF, 0x076B,
	/* Array index: 0x00BA */ 0x581E, 0x5820, 0x076E,
	/* Array index: 0x00BD */ 0x584C, 0x5850, 0x0771,
	/* Array index: 0x00C0 */ 0x587F, 0x5881, 0x0776,
	/* Array index: 0x00C3 */ 0x5887, 0x5890, 0x0779,
	/* Array index: 0x00C6 */ 0x58A0, 0x58A2, 0x0783,
	/* Array index: 0x00C9 */ 0x5912, 0x5914, 0x0786,
	/* Array index: 0x00CC */ 0x595B, 0x5963, 0x0789,
	/* Array index: 0x00CF */ 0x59AD, 0x59B0, 0x0792,
	/* Array index: 0x00D2 */ 0x59DD, 0x59DF, 0x0796,
	/* Array index: 0x00D5 */ 0x59EE, 0x59F4, 0x0799,
	/* Array index: 0x00D8 */ 0x5A0C, 0x5A0E, 0x07A0,
	/* Array index: 0x00DB */ 0x5A44, 0x5A48, 0x07A3,
	/* Array index: 0x00DE */ 0x5A9C, 0x5AA2, 0x07A8,
	/* Array index: 0x00E1 */ 0x5AB1, 0x5AB5, 0x07AF,
	/* Array index: 0x00E4 */ 0x5B7C, 0x5B81, 0x07B4,
	/* Array index: 0x00E7 */ 0x5B90, 0x5B96, 0x07BA,
	/* Array index: 0x00EA */ 0x5BD6, 0x5BDA, 0x07C1,
	/* Array index: 0x00ED */ 0x5C67, 0x5C69, 0x07C6,
	/* Array index: 0x00F0 */ 0x5C7A, 0x5C7D, 0x07C9,
	/* Array index: 0x00F3 */ 0x5C9F, 0x5CA3, 0x07CD,
	/* Array index: 0x00F6 */ 0x5D23, 0x5D27, 0x07D2,
	/* Array index: 0x00F9 */ 0x5D5F, 0x5D64, 0x07D7,
	/* Array index: 0x00FC */ 0x5D92, 0x5D95, 0x07DD,
	/* Array index: 0x00FF */ 0x5E68, 0x5E70, 0x07E1,
	/* Array index: 0x0102 */ 0x5EBD, 0x5EBF, 0x07EA,
	/* Array index: 0x0105 */ 0x5ED1, 0x5ED5, 0x07ED,
	/* Array index: 0x0108 */ 0x5F06, 0x5F08, 0x07F2,
	/* Array index: 0x010B */ 0x5F21, 0x5F24, 0x07F5,
	/* Array index: 0x010E */ 0x5FAB, 0x5FB1, 0x07F9,
	/* Array index: 0x0111 */ 0x5FC7, 0x5FCB, 0x0800,
	/* Array index: 0x0114 */ 0x5FD0, 0x5FD4, 0x0805,
	/* Array index: 0x0117 */ 0x5FE8, 0x5FF3, 0x080A,
	/* Array index: 0x011A */ 0x6047, 0x6049, 0x0816,
	/* Array index: 0x011D */ 0x60C8, 0x60CF, 0x0819,
	/* Array index: 0x0120 */ 0x6110, 0x6119, 0x0821,
	/* Array index: 0x0123 */ 0x6135, 0x6139, 0x082B,
	/* Array index: 0x0126 */ 0x617B, 0x6184, 0x0830,
	/* Array index: 0x0129 */ 0x619C, 0x61A0, 0x083A,
	/* Array index: 0x012C */ 0x61C0, 0x61C2, 0x083F,
	/* Array index: 0x012F */ 0x61DC, 0x61E2, 0x0842,
	/* Array index: 0x0132 */ 0x6242, 0x6246, 0x0849,
	/* Array index: 0x0135 */ 0x6250, 0x6256, 0x084E,
	/* Array index: 0x0138 */ 0x628D, 0x6290, 0x0855,
	/* Array index: 0x013B */ 0x6341, 0x6346, 0x0859,
	/* Array index: 0x013E */ 0x6352, 0x6354, 0x085F,
	/* Array index: 0x0141 */ 0x63AD, 0x63AF, 0x0862,
	/* Array index: 0x0144 */ 0x63D1, 0x63D5, 0x0865,
	/* Array index: 0x0147 */ 0x644F, 0x6454, 0x086A,
	/* Array index: 0x014A */ 0x645A, 0x6463, 0x0870,
	/* Array index: 0x014D */ 0x648F, 0x6491, 0x087A,
	/* Array index: 0x0150 */ 0x64BD, 0x64BF, 0x087D,
	/* Array index: 0x0153 */ 0x64C9, 0x64D1, 0x0880,
	/* Array index: 0x0156 */ 0x6508, 0x650A, 0x0889,
	/* Array index: 0x0159 */ 0x65B2, 0x65B4, 0x088C,
	/* Array index: 0x015C */ 0x65FE, 0x6600, 0x088F,
	/* Array index: 0x015F */ 0x661D, 0x6633, 0x0892,
	/* Array index: 0x0162 */ 0x6657, 0x665C, 0x08A9,
	/* Array index: 0x0165 */ 0x666A, 0x666C, 0x08AF,
	/* Array index: 0x0168 */ 0x6675, 0x6680, 0x08B2,
	/* Array index: 0x016B */ 0x668B, 0x668D, 0x08BE,
	/* Array index: 0x016E */ 0x6699, 0x66A0, 0x08C1,
	/* Array index: 0x0171 */ 0x66BF, 0x66C3, 0x08C9,
	/* Array index: 0x0174 */ 0x67B0, 0x67B2, 0x08CE,
	/* Array index: 0x0177 */ 0x67BB, 0x67BD, 0x08D1,
	/* Array index: 0x017A */ 0x682C, 0x6833, 0x08D4,
	/* Array index: 0x017D */ 0x686E, 0x6872, 0x08DC,
	/* Array index: 0x0180 */ 0x6879, 0x687C, 0x08E1,
	/* Array index: 0x0183 */ 0x68CF, 0x68D3, 0x08E5,
	/* Array index: 0x0186 */ 0x68E8, 0x68F1, 0x08EA,
	/* Array index: 0x0189 */ 0x68FB, 0x68FD, 0x08F4,
	/* Array index: 0x018C */ 0x6963, 0x6969, 0x08F7,
	/* Array index: 0x018F */ 0x6970, 0x6972, 0x08FE,
	/* Array index: 0x0192 */ 0x6A4E, 0x6A56, 0x0901,
	/* Array index: 0x0195 */ 0x6A9B, 0x6A9F, 0x090A,
	/* Array index: 0x0198 */ 0x6AAF, 0x6AB1, 0x090F,
	/* Array index: 0x019B */ 0x6ABD, 0x6ABF, 0x0912,
	/* Array index: 0x019E */ 0x6AD4, 0x6AD6, 0x0915,
	/* Array index: 0x01A1 */ 0x6AF0, 0x6AF2, 0x0918,
	/* Array index: 0x01A4 */ 0x6B0F, 0x6B11, 0x091B,
	/* Array index: 0x01A7 */ 0x6B9F, 0x6BA3, 0x091E,
	/* Array index: 0x01AA */ 0x6BA8, 0x6BB0, 0x0923,
	/* Array index: 0x01AD */ 0x6C26, 0x6C28, 0x092C,
	/* Array index: 0x01B0 */ 0x6C85, 0x6C89, 0x092F,
	/* Array index: 0x01B3 */ 0x6C94, 0x6C98, 0x0934,
	/* Array index: 0x01B6 */ 0x6CCD, 0x6CD6, 0x0939,
	/* Array index: 0x01B9 */ 0x6D26, 0x6D28, 0x0943,
	/* Array index: 0x01BC */ 0x6D94, 0x6D98, 0x0946,
	/* Array index: 0x01BF */ 0x6E4F, 0x6E54, 0x094B,
	/* Array index: 0x01C2 */ 0x6E5C, 0x6E5E, 0x0951,
	/* Array index: 0x01C5 */ 0x6EBF, 0x6EC3, 0x0954,
	/* Array index: 0x01C8 */ 0x6ECD, 0x6ECF, 0x0959,
	/* Array index: 0x01CB */ 0x6F4F, 0x6F53, 0x095C,
	/* Array index: 0x01CE */ 0x6F87, 0x6F8D, 0x0961,
	/* Array index: 0x01D1 */ 0x6F90, 0x6F96, 0x0968,
	/* Array index: 0x01D4 */ 0x6FA5, 0x6FA8, 0x096F,
	/* Array index: 0x01D7 */ 0x6FAE, 0x6FB0, 0x0973,
	/* Array index: 0x01DA */ 0x7005, 0x7007, 0x0976,
	/* Array index: 0x01DD */ 0x7048, 0x704B, 0x0979,
	/* Array index: 0x01E0 */ 0x7094, 0x7098, 0x097D,
	/* Array index: 0x01E3 */ 0x70D1, 0x70D8, 0x0982,
	/* Array index: 0x01E6 */ 0x7103, 0x7107, 0x098A,
	/* Array index: 0x01E9 */ 0x712D, 0x7131, 0x098F,
	/* Array index: 0x01EC */ 0x7145, 0x714B, 0x0994,
	/* Array index: 0x01EF */ 0x71AF, 0x71B3, 0x099B,
	/* Array index: 0x01F2 */ 0x71BF, 0x71C1, 0x09A0,
	/* Array index: 0x01F5 */ 0x7207, 0x7209, 0x09A3,
	/* Array index: 0x01F8 */ 0x7241, 0x7245, 0x09A6,
	/* Array index: 0x01FB */ 0x724E, 0x7250, 0x09AB,
	/* Array index: 0x01FE */ 0x735D, 0x7360, 0x09AE,
	/* Array index: 0x0201 */ 0x736B, 0x7371, 0x09B2,
	/* Array index: 0x0204 */ 0x739C, 0x73A6, 0x09B9,
	/* Array index: 0x0207 */ 0x73F7, 0x7401, 0x09C4,
	/* Array index: 0x020A */ 0x7426, 0x7431, 0x09CF,
	/* Array index: 0x020D */ 0x7443, 0x7447, 0x09DB,
	/* Array index: 0x0210 */ 0x7466, 0x7468, 0x09E0,
	/* Array index: 0x0213 */ 0x7485, 0x7489, 0x09E3,
	/* Array index: 0x0216 */ 0x748F, 0x7492, 0x09E8,
	/* Array index: 0x0219 */ 0x7498, 0x74A3, 0x09EC,
	/* Array index: 0x021C */ 0x74A6, 0x74B2, 0x09F8,
	/* Array index: 0x021F */ 0x74FA, 0x74FC, 0x0A05,
	/* Array index: 0x0222 */ 0x753D, 0x7540, 0x0A08,
	/* Array index: 0x0225 */ 0x7579, 0x757E, 0x0A0C,
	/* Array index: 0x0228 */ 0x75BF, 0x75C1, 0x0A12,
	/* Array index: 0x022B */ 0x75DF, 0x75E1, 0x0A15,
	/* Array index: 0x022E */ 0x7600, 0x760C, 0x0A18,
	/* Array index: 0x0231 */ 0x7612, 0x7616, 0x0A25,
	/* Array index: 0x0234 */ 0x7619, 0x761E, 0x0A2A,
	/* Array index: 0x0237 */ 0x7638, 0x763C, 0x0A30,
	/* Array index: 0x023A */ 0x7640, 0x764B, 0x0A35,
	/* Array index: 0x023D */ 0x766D, 0x7671, 0x0A41,
	/* Array index: 0x0240 */ 0x769B, 0x76AA, 0x0A46,
	/* Array index: 0x0243 */ 0x772D, 0x772F, 0x0A56,
	/* Array index: 0x0246 */ 0x7734, 0x7736, 0x0A59,
	/* Array index: 0x0249 */ 0x774D, 0x774F, 0x0A5C,
	/* Array index: 0x024C */ 0x775C, 0x7764, 0x0A5F,
	/* Array index: 0x024F */ 0x7770, 0x7774, 0x0A68,
	/* Array index: 0x0252 */ 0x7794, 0x7796, 0x0A6D,
	/* Array index: 0x0255 */ 0x77DE, 0x77E0, 0x0A70,
	/* Array index: 0x0258 */ 0x7821, 0x7823, 0x0A73,
	/* Array index: 0x025B */ 0x7860, 0x7864, 0x0A76,
	/* Array index: 0x025E */ 0x789D, 0x78A1, 0x0A7B,
	/* Array index: 0x0261 */ 0x78AC, 0x78B3, 0x0A80,
	/* Array index: 0x0264 */ 0x78C7, 0x78C9, 0x0A88,
	/* Array index: 0x0267 */ 0x78D2, 0x78D6, 0x0A8B,
	/* Array index: 0x026A */ 0x78DF, 0x78E1, 0x0A90,
	/* Array index: 0x026D */ 0x791C, 0x7920, 0x0A93,
	/* Array index: 0x0270 */ 0x7944, 0x7946, 0x0A98,
	/* Array index: 0x0273 */ 0x7991, 0x799C, 0x0A9B,
	/* Array index: 0x0276 */ 0x7A34, 0x7A3A, 0x0AA7,
	/* Array index: 0x0279 */ 0x7A44, 0x7A48, 0x0AAE,
	/* Array index: 0x027C */ 0x7A5C, 0x7A60, 0x0AB3,
	/* Array index: 0x027F */ 0x7AE8, 0x7AEC, 0x0AB8,
	/* Array index: 0x0282 */ 0x7B27, 0x7B31, 0x0ABD,
	/* Array index: 0x0285 */ 0x7B3D, 0x7B41, 0x0AC8,
	/* Array index: 0x0288 */ 0x7BBA, 0x7BBD, 0x0ACD,
	/* Array index: 0x028B */ 0x7BD6, 0x7BDB, 0x0AD1,
	/* Array index: 0x028E */ 0x7BF4, 0x7C06, 0x0AD7,
	/* Array index: 0x0291 */ 0x7C0B, 0x7C0F, 0x0AEA,
	/* Array index: 0x0294 */ 0x7C51, 0x7C5E, 0x0AEF,
	/* Array index: 0x0297 */ 0x7CB6, 0x7CBC, 0x0AFD,
	/* Array index: 0x029A */ 0x7CC7, 0x7CC9, 0x0B04,
	/* Array index: 0x029D */ 0x7CD3, 0x7CDA, 0x0B07,
	/* Array index: 0x02A0 */ 0x7D07, 0x7D09, 0x0B0F,
	/* Array index: 0x02A3 */ 0x7D0F, 0x7D13, 0x0B12,
	/* Array index: 0x02A6 */ 0x7D3C, 0x7D41, 0x0B17,
	/* Array index: 0x02A9 */ 0x7D59, 0x7D5D, 0x0B1D,
	/* Array index: 0x02AC */ 0x7D7F, 0x7D8D, 0x0B22,
	/* Array index: 0x02AF */ 0x7DC2, 0x7DC6, 0x0B31,
	/* Array index: 0x02B2 */ 0x7DCC, 0x7DCE, 0x0B36,
	/* Array index: 0x02B5 */ 0x7F3B, 0x7F3F, 0x0B39,
	/* Array index: 0x02B8 */ 0x7F5B, 0x7F5D, 0x0B3E,
	/* Array index: 0x02BB */ 0x7F61, 0x7F66, 0x0B41,
	/* Array index: 0x02BE */ 0x7F7D, 0x7F80, 0x0B47,
	/* Array index: 0x02C1 */ 0x7F8D, 0x7F91, 0x0B4B,
	/* Array index: 0x02C4 */ 0x7FFD, 0x7FFF, 0x0B50,
	/* Array index: 0x02C7 */ 0x800D, 0x8016, 0x0B53,
	/* Array index: 0x02CA */ 0x801D, 0x8020, 0x0B5D,
	/* Array index: 0x02CD */ 0x80D4, 0x80D8, 0x0B61,
	/* Array index: 0x02D0 */ 0x8116, 0x8118, 0x0B66,
	/* Array index: 0x02D3 */ 0x8167, 0x8169, 0x0B69,
	/* Array index: 0x02D6 */ 0x8184, 0x8186, 0x0B6C,
	/* Array index: 0x02D9 */ 0x81F0, 0x81F9, 0x0B6F,
	/* Array index: 0x02DC */ 0x8243, 0x8246, 0x0B79,
	/* Array index: 0x02DF */ 0x827D, 0x8284, 0x0B7D,
	/* Array index: 0x02E2 */ 0x82A0, 0x82AA, 0x0B85,
	/* Array index: 0x02E5 */ 0x82FD, 0x8301, 0x0B90,
	/* Array index: 0x02E8 */ 0x8307, 0x830B, 0x0B95,
	/* Array index: 0x02EB */ 0x831B, 0x8322, 0x0B9A,
	/* Array index: 0x02EE */ 0x832C, 0x8330, 0x0BA2,
	/* Array index: 0x02F1 */ 0x8342, 0x8344, 0x0BA7,
	/* Array index: 0x02F4 */ 0x8351, 0x8357, 0x0BAA,
	/* Array index: 0x02F7 */ 0x8398, 0x839D, 0x0BB1,
	/* Array index: 0x02FA */ 0x83BE, 0x83C0, 0x0BB7,
	/* Array index: 0x02FD */ 0x83CF, 0x83D1, 0x0BBA,
	/* Array index: 0x0300 */ 0x8456, 0x845C, 0x0BBD,
	/* Array index: 0x0303 */ 0x84A6, 0x84AA, 0x0BC4,
	/* Array index: 0x0306 */ 0x84EF, 0x84F2, 0x0BC9,
	/* Array index: 0x0309 */ 0x8522, 0x852B, 0x0BCD,
	/* Array index: 0x030C */ 0x8532, 0x8536, 0x0BD7,
	/* Array index: 0x030F */ 0x854F, 0x8553, 0x0BDC,
	/* Array index: 0x0312 */ 0x855C, 0x8564, 0x0BE1,
	/* Array index: 0x0315 */ 0x8579, 0x8581, 0x0BEA,
	/* Array index: 0x0318 */ 0x85B4, 0x85BF, 0x0BF3,
	/* Array index: 0x031B */ 0x860D, 0x8612, 0x0BFF,
	/* Array index: 0x031E */ 0x863C, 0x8642, 0x0C05,
	/* Array index: 0x0321 */ 0x8652, 0x8659, 0x0C0C,
	/* Array index: 0x0324 */ 0x8660, 0x8664, 0x0C14,
	/* Array index: 0x0327 */ 0x8675, 0x8677, 0x0C19,
	/* Array index: 0x032A */ 0x86A6, 0x86A8, 0x0C1C,
	/* Array index: 0x032D */ 0x86B1, 0x86B9, 0x0C1F,
	/* Array index: 0x0330 */ 0x86BF, 0x86C5, 0x0C28,
	/* Array index: 0x0333 */ 0x870E, 0x8714, 0x0C2F,
	/* Array index: 0x0336 */ 0x872E, 0x8732, 0x0C36,
	/* Array index: 0x0339 */ 0x8739, 0x8740, 0x0C3B,
	/* Array index: 0x033C */ 0x8783, 0x878C, 0x0C43,
	/* Array index: 0x033F */ 0x8795, 0x8799, 0x0C4D,
	/* Array index: 0x0342 */ 0x87AC, 0x87AE, 0x0C52,
	/* Array index: 0x0345 */ 0x87C8, 0x87CA, 0x0C55,
	/* Array index: 0x0348 */ 0x87E2, 0x87E4, 0x0C58,
	/* Array index: 0x034B */ 0x8809, 0x880B, 0x0C5B,
	/* Array index: 0x034E */ 0x8810, 0x881F, 0x0C5E,
	/* Array index: 0x0351 */ 0x8848, 0x884B, 0x0C6E,
	/* Array index: 0x0354 */ 0x8898, 0x88A0, 0x0C72,
	/* Array index: 0x0357 */ 0x88CA, 0x88D3, 0x0C7B,
	/* Array index: 0x035A */ 0x88EF, 0x88F1, 0x0C85,
	/* Array index: 0x035D */ 0x890D, 0x890F, 0x0C88,
	/* Array index: 0x0360 */ 0x8915, 0x891C, 0x0C8B,
	/* Array index: 0x0363 */ 0x8926, 0x8928, 0x0C93,
	/* Array index: 0x0366 */ 0x8930, 0x8932, 0x0C96,
	/* Array index: 0x0369 */ 0x895A, 0x895C, 0x0C99,
	/* Array index: 0x036C */ 0x8961, 0x8963, 0x0C9C,
	/* Array index: 0x036F */ 0x897A, 0x897D, 0x0C9F,
	/* Array index: 0x0372 */ 0x89B4, 0x89B7, 0x0CA3,
	/* Array index: 0x0375 */ 0x89D4, 0x89D8, 0x0CA7,
	/* Array index: 0x0378 */ 0x8A11, 0x8A15, 0x0CAC,
	/* Array index: 0x037B */ 0x8A56, 0x8A58, 0x0CB1,
	/* Array index: 0x037E */ 0x8A75, 0x8A80, 0x0CB4,
	/* Array index: 0x0381 */ 0x8AC6, 0x8ACA, 0x0CC0,
	/* Array index: 0x0384 */ 0x8AD1, 0x8AD7, 0x0CC5,
	/* Array index: 0x0387 */ 0x8AF4, 0x8AF6, 0x0CCC,
	/* Array index: 0x038A */ 0x8B42, 0x8B48, 0x0CCF,
	/* Array index: 0x038D */ 0x8B52, 0x8B54, 0x0CD6,
	/* Array index: 0x0390 */ 0x8C53, 0x8C5D, 0x0CD9,
	/* Array index: 0x0393 */ 0x8D9E, 0x8DA0, 0x0CE4,
	/* Array index: 0x0396 */ 0x8DC5, 0x8DCA, 0x0CE7,
	/* Array index: 0x0399 */ 0x8DF0, 0x8DF4, 0x0CED,
	/* Array index: 0x039C */ 0x8E04, 0x8E06, 0x0CF2,
	/* Array index: 0x039F */ 0x8E20, 0x8E27, 0x0CF5,
	/* Array index: 0x03A2 */ 0x8E36, 0x8E39, 0x0CFD,
	/* Array index: 0x03A5 */ 0x8E4B, 0x8E4F, 0x0D01,
	/* Array index: 0x03A8 */ 0x8E5B, 0x8E62, 0x0D06,
	/* Array index: 0x03AB */ 0x8E6C, 0x8E71, 0x0D0E,
	/* Array index: 0x03AE */ 0x8E79, 0x8E7B, 0x0D14,
	/* Array index: 0x03B1 */ 0x8E9A, 0x8E9E, 0x0D17,
	/* Array index: 0x03B4 */ 0x8EC0, 0x8EC4, 0x0D1C,
	/* Array index: 0x03B7 */ 0x8EED, 0x8EF1, 0x0D21,
	/* Array index: 0x03BA */ 0x8F16, 0x8F18, 0x0D26,
	/* Array index: 0x03BD */ 0x8F2C, 0x8F2E, 0x0D29,
	/* Array index: 0x03C0 */ 0x8F34, 0x8F37, 0x0D2C,
	/* Array index: 0x03C3 */ 0x8F4F, 0x8F55, 0x0D30,
	/* Array index: 0x03C6 */ 0x8FA0, 0x8FA6, 0x0D37,
	/* Array index: 0x03C9 */ 0x9028, 0x902F, 0x0D3E,
	/* Array index: 0x03CC */ 0x9095, 0x909B, 0x0D46,
	/* Array index: 0x03CF */ 0x90A0, 0x90A2, 0x0D4D,
	/* Array index: 0x03D2 */ 0x90B0, 0x90B6, 0x0D50,
	/* Array index: 0x03D5 */ 0x90C3, 0x90C8, 0x0D57,
	/* Array index: 0x03D8 */ 0x90D5, 0x90DF, 0x0D5D,
	/* Array index: 0x03DB */ 0x90FE, 0x9108, 0x0D68,
	/* Array index: 0x03DE */ 0x9114, 0x9129, 0x0D73,
	/* Array index: 0x03E1 */ 0x9136, 0x913D, 0x0D89,
	/* Array index: 0x03E4 */ 0x9157, 0x915B, 0x0D91,
	/* Array index: 0x03E7 */ 0x9179, 0x917B, 0x0D96,
	/* Array index: 0x03EA */ 0x9191, 0x9195, 0x0D99,
	/* Array index: 0x03ED */ 0x91AC, 0x91B3, 0x0D9E,
	/* Array index: 0x03F0 */ 0x91BB, 0x91C5, 0x0DA6,
	/* Array index: 0x03F3 */ 0x91E9, 0x91F1, 0x0DB1,
	/* Array index: 0x03F6 */ 0x9200, 0x920C, 0x0DBA,
	/* Array index: 0x03F9 */ 0x9223, 0x9228, 0x0DC7,
	/* Array index: 0x03FC */ 0x922E, 0x9230, 0x0DCD,
	/* Array index: 0x03FF */ 0x9235, 0x9247, 0x0DD0,
	/* Array index: 0x0402 */ 0x924D, 0x9251, 0x0DE3,
	/* Array index: 0x0405 */ 0x9265, 0x9269, 0x0DE8,
	/* Array index: 0x0408 */ 0x926E, 0x9270, 0x0DED,
	/* Array index: 0x040B */ 0x9275, 0x927F, 0x0DF0,
	/* Array index: 0x040E */ 0x9288, 0x928E, 0x0DFB,
	/* Array index: 0x0411 */ 0x92A4, 0x92A8, 0x0E02,
	/* Array index: 0x0414 */ 0x92B8, 0x92D0, 0x0E07,
	/* Array index: 0x0417 */ 0x92D5, 0x92E8, 0x0E20,
	/* Array index: 0x041A */ 0x931C, 0x932A, 0x0E34,
	/* Array index: 0x041D */ 0x9333, 0x9337, 0x0E43,
	/* Array index: 0x0420 */ 0x9347, 0x9349, 0x0E48,
	/* Array index: 0x0423 */ 0x9350, 0x9352, 0x0E4B,
	/* Array index: 0x0426 */ 0x936D, 0x9376, 0x0E4E,
	/* Array index: 0x0429 */ 0x937D, 0x9382, 0x0E58,
	/* Array index: 0x042C */ 0x93B4, 0x93B6, 0x0E5E,
	/* Array index: 0x042F */ 0x93C4, 0x93CD, 0x0E61,
	/* Array index: 0x0432 */ 0x93F7, 0x93FD, 0x0E6B,
	/* Array index: 0x0435 */ 0x940D, 0x940F, 0x0E72,
	/* Array index: 0x0438 */ 0x9415, 0x9417, 0x0E75,
	/* Array index: 0x043B */ 0x942E, 0x9434, 0x0E78,
	/* Array index: 0x043E */ 0x946B, 0x9472, 0x0E7F,
	/* Array index: 0x0441 */ 0x958C, 0x958E, 0x0E87,
	/* Array index: 0x0444 */ 0x959D, 0x95A1, 0x0E8A,
	/* Array index: 0x0447 */ 0x95D0, 0x95D3, 0x0E8F,
	/* Array index: 0x044A */ 0x95D9, 0x95E0, 0x0E93,
	/* Array index: 0x044D */ 0x9622, 0x9626, 0x0E9B,
	/* Array index: 0x0450 */ 0x9637, 0x963D, 0x0EA0,
	/* Array index: 0x0453 */ 0x9654, 0x9658, 0x0EA7,
	/* Array index: 0x0456 */ 0x967B, 0x9684, 0x0EAC,
	/* Array index: 0x0459 */ 0x96A4, 0x96A6, 0x0EB6,
	/* Array index: 0x045C */ 0x96DD, 0x96DF, 0x0EB9,
	/* Array index: 0x045F */ 0x9721, 0x9723, 0x0EBC,
	/* Array index: 0x0462 */ 0x9757, 0x975B, 0x0EBF,
	/* Array index: 0x0465 */ 0x9776, 0x9778, 0x0EC4,
	/* Array index: 0x0468 */ 0x9795, 0x979A, 0x0EC7,
	/* Array index: 0x046B */ 0x97B5, 0x97D1, 0x0ECD,
	/* Array index: 0x046E */ 0x97D7, 0x97E1, 0x0EEA,
	/* Array index: 0x0471 */ 0x982E, 0x9835, 0x0EF5,
	/* Array index: 0x0474 */ 0x9851, 0x985A, 0x0EFD,
	/* Array index: 0x0477 */ 0x9862, 0x9866, 0x0F07,
	/* Array index: 0x047A */ 0x98B7, 0x98BB, 0x0F0C,
	/* Array index: 0x047D */ 0x98E3, 0x98E7, 0x0F11,
	/* Array index: 0x0480 */ 0x9915, 0x991C, 0x0F16,
	/* Array index: 0x0483 */ 0x9931, 0x993C, 0x0F1E,
	/* Array index: 0x0486 */ 0x9946, 0x9948, 0x0F2A,
	/* Array index: 0x0489 */ 0x9958, 0x9960, 0x0F2D,
	/* Array index: 0x048C */ 0x99B0, 0x99B2, 0x0F36,
	/* Array index: 0x048F */ 0x99EA, 0x99EC, 0x0F39,
	/* Array index: 0x0492 */ 0x9A02, 0x9A04, 0x0F3C,
	/* Array index: 0x0495 */ 0x9A20, 0x9A24, 0x0F3F,
	/* Array index: 0x0498 */ 0x9A4A, 0x9A4E, 0x0F44,
	/* Array index: 0x049B */ 0x9AB2, 0x9AB6, 0x0F49,
	/* Array index: 0x049E */ 0x9AD5, 0x9AD7, 0x0F4E,
	/* Array index: 0x04A1 */ 0x9AFD, 0x9B12, 0x0F51,
	/* Array index: 0x04A4 */ 0x9B33, 0x9B3A, 0x0F67,
	/* Array index: 0x04A7 */ 0x9B55, 0x9B57, 0x0F6F,
	/* Array index: 0x04AA */ 0x9B68, 0x9B6E, 0x0F72,
	/* Array index: 0x04AD */ 0x9B75, 0x9B79, 0x0F79,
	/* Array index: 0x04B0 */ 0x9B84, 0x9B90, 0x0F7E,
	/* Array index: 0x04B3 */ 0x9BB0, 0x9BB2, 0x0F8B,
	/* Array index: 0x04B6 */ 0x9BBB, 0x9BC1, 0x0F8E,
	/* Array index: 0x04B9 */ 0x9BF7, 0x9BFA, 0x0F95,
	/* Array index: 0x04BC */ 0x9C16, 0x9C1E, 0x0F99,
	/* Array index: 0x04BF */ 0x9C22, 0x9C2A, 0x0FA2,
	/* Array index: 0x04C2 */ 0x9C35, 0x9C37, 0x0FAB,
	/* Array index: 0x04C5 */ 0x9C41, 0x9C45, 0x0FAE,
	/* Array index: 0x04C8 */ 0x9C4E, 0x9C5F, 0x0FB3,
	/* Array index: 0x04CB */ 0x9C68, 0x9C6B, 0x0FC5,
	/* Array index: 0x04CE */ 0x9D1C, 0x9D1E, 0x0FC9,
	/* Array index: 0x04D1 */ 0x9D2F, 0x9D34, 0x0FCC,
	/* Array index: 0x04D4 */ 0x9D69, 0x9D6B, 0x0FD2,
	/* Array index: 0x04D7 */ 0x9D92, 0x9D98, 0x0FD5,
	/* Array index: 0x04DA */ 0x9DD4, 0x9DD7, 0x0FDC,
	/* Array index: 0x04DD */ 0x9DDE, 0x9DE0, 0x0FE0,
	/* Array index: 0x04E0 */ 0x9E0E, 0x9E16, 0x0FE3,
	/* Array index: 0x04E3 */ 0x9E7A, 0x9E7C, 0x0FEC,
	/* Array index: 0x04E6 */ 0x9E80, 0x9E87, 0x0FEF,
	/* Array index: 0x04E9 */ 0x9EAC, 0x9EB5, 0x0FF7,
	/* Array index: 0x04EC */ 0x9EEC, 0x9EF2, 0x1001,
	/* Array index: 0x04EF */ 0x9F0F, 0x9F1B, 0x1008,
	/* Array index: 0x04F2 */ 0x9F39, 0x9F47, 0x1015,
	/* Array index: 0x04F5 */ 0x9F53, 0x9F5E, 0x1024,
	/* Array index: 0x04F8 */ 0x9F6D, 0x9F75, 0x1030,
	/* Array index: 0x04FB */ 0x9F8F, 0x9F97, 0x1039,
	/* Array index: 0x04FE */ 0x9FA1, 0x9FA5, 0x1042,
	/* Ranges content */
	/* Range 0x00BF - 0x017E, array index: 0x0501 */
	0x2244,0x2A22,0x2A21,0x2A24,0x2A2A,0x2A23,0x2A29,0x2921,
	0x2A2E,0x2A32,0x2A31,0x2A34,0x2A33,0x2A40,0x2A3F,0x2A42,
	0x2A41,INVALC,0x2A50,0x2A52,0x2A51,0x2A54,0x2A58,0x2A53,
	INVALC,0x292C,0x2A63,0x2A62,0x2A65,0x2A64,0x2A72,0x2930,
	0x294E,0x2B22,0x2B21,0x2B24,0x2B2A,0x2B23,0x2B29,0x2941,
	0x2B2E,0x2B32,0x2B31,0x2B34,0x2B33,0x2B40,0x2B3F,0x2B42,
	0x2B41,0x2943,0x2B50,0x2B52,0x2B51,0x2B54,0x2B58,0x2B53,
	INVALC,0x294C,0x2B63,0x2B62,0x2B65,0x2B64,0x2B72,0x2950,
	0x2B73,0x2A27,0x2B27,0x2A25,0x2B25,0x2A28,0x2B28,0x2A2B,
	0x2B2B,0x2A2C,0x2B2C,0x2A2F,0x2B2F,0x2A2D,0x2B2D,0x2A30,
	0x2B30,0x2922,0x2942,0x2A37,0x2B37,INVALC,INVALC,0x2A36,
	0x2B36,0x2A38,0x2B38,0x2A35,0x2B35,0x2A3A,0x2B3A,0x2A3B,
	0x2B3B,0x2A3D,0x2B3D,0x2A3C,INVALC,0x2A3E,0x2B3E,0x2924,
	0x2944,0x2A47,0x2B47,0x2A45,0x2B45,INVALC,INVALC,0x2A46,
	0x2B46,0x2A44,0x2945,0x2926,0x2946,0x2A48,0x2B48,0x2A49,
	0x2B49,0x2947,0x2A4A,0x2B4A,0x2A4C,0x2B4C,0x2A4B,0x2B4B,
	0x2929,0x2949,0x2928,0x2948,0x2A4D,0x2B4D,0x2A4F,0x2B4F,
	0x2A4E,0x2B4E,0x294A,0x292B,0x294B,0x2A57,0x2B57,INVALC,
	INVALC,0x2A56,0x2B56,0x292D,0x294D,0x2A59,0x2B59,0x2A5B,
	0x2B5B,0x2A5A,0x2B5A,0x2A5C,0x2B5C,0x2A5D,0x2B5D,0x2A5F,
	0x2B5F,0x2A5E,0x2B5E,0x2A61,0x2B61,0x2A60,0x2B60,0x292F,
	0x294F,0x2A6C,0x2B6C,0x2A69,0x2B69,0x2A66,0x2B66,0x2A6B,
	0x2B6B,0x2A68,0x2B68,0x2A6A,0x2B6A,0x2A71,0x2B71,0x2A74,
	0x2B74,0x2A73,0x2A75,0x2B75,0x2A77,0x2B77,0x2A76,0x2B76,
	/* Range 0x01CD - 0x01DC, array index: 0x05C1 */
	0x2A26,0x2B26,0x2A43,0x2B43,0x2A55,0x2B55,0x2A67,0x2B67,
	0x2A70,0x2B70,0x2A6D,0x2B6D,0x2A6F,0x2B6F,0x2A6E,0x2B6E,
	/* Range 0x02D8 - 0x02DD, array index: 0x05D1 */
	0x222F,0x2232,0x2236,0x2235,INVALC,0x2233,
	/* Range 0x0384 - 0x0390, array index: 0x05D7 */
	0x2238,0x2239,0x2661,INVALC,0x2662,0x2663,0x2664,INVALC,
	0x2667,INVALC,0x2669,0x266C,0x2676,
	/* Range 0x03AA - 0x03B0, array index: 0x05E4 */
	0x2665,0x266A,0x2671,0x2672,0x2673,0x2674,0x267B,
	/* Range 0x03CA - 0x03CE, array index: 0x05EB */
	0x2675,0x267A,0x2677,0x2679,0x267C,
	/* Range 0x0402 - 0x040F, array index: 0x05F0 */
	0x2742,0x2743,0x2744,0x2745,0x2746,0x2747,0x2748,0x2749,
	0x274A,0x274B,0x274C,INVALC,0x274D,0x274E,
	/* Range 0x0452 - 0x045F, array index: 0x05FE */
	0x2772,0x2773,0x2774,0x2775,0x2776,0x2777,0x2778,0x2779,
	0x277A,0x277B,0x277C,INVALC,0x277D,0x277E,
	/* Range 0x4E2E - 0x4E30, array index: 0x060C */
	0x302B,0x302C,0x302D,
	/* Range 0x4EE0 - 0x4EE2, array index: 0x060F */
	0x3047,0x3048,0x3049,
	/* Range 0x4EFD - 0x4F03, array index: 0x0612 */
	0x304F,0x3050,0x3051,0x3052,INVALC,0x3053,0x3054,
	/* Range 0x4F15 - 0x4F19, array index: 0x0619 */
	0x3059,0x305A,0x305B,INVALC,0x305C,
	/* Range 0x4F48 - 0x4F4C, array index: 0x061E */
	0x3068,0x3069,INVALC,0x306A,0x306B,
	/* Range 0x4F77 - 0x4F85, array index: 0x0623 */
	0x3076,0x3077,0x3078,0x3079,INVALC,INVALC,0x307A,0x307B,
	INVALC,INVALC,0x307C,0x307D,INVALC,0x307E,0x3121,
	/* Range 0x4F90 - 0x4F94, array index: 0x0632 */
	0x3126,INVALC,0x3127,0x3128,0x3129,
	/* Range 0x4FB9 - 0x4FC1, array index: 0x0637 */
	0x3131,INVALC,0x3132,0x3133,0x3134,0x3135,INVALC,0x3136,
	0x3137,
	/* Range 0x4FC5 - 0x4FCF, array index: 0x0640 */
	0x3138,0x3139,INVALC,0x313A,0x313B,INVALC,0x313C,0x313D,
	0x313E,INVALC,0x313F,
	/* Range 0x4FFC - 0x5001, array index: 0x064B */
	0x3146,0x3147,INVALC,0x3148,0x3149,0x314A,
	/* Range 0x5017 - 0x501E, array index: 0x0651 */
	0x3152,0x3153,INVALC,INVALC,0x3154,0x3155,0x3156,0x3157,
	/* Range 0x5040 - 0x5046, array index: 0x0659 */
	0x315F,0x3160,0x3161,INVALC,INVALC,0x3162,0x3163,
	/* Range 0x5051 - 0x5053, array index: 0x0660 */
	0x3167,0x3168,0x3169,
	/* Range 0x505F - 0x5067, array index: 0x0663 */
	0x316C,0x316D,INVALC,0x316E,0x316F,INVALC,INVALC,0x3170,
	0x3171,
	/* Range 0x508E - 0x5096, array index: 0x066C */
	0x317C,0x317D,0x317E,INVALC,0x3221,0x3222,0x3223,INVALC,
	0x3224,
	/* Range 0x509B - 0x50A2, array index: 0x0675 */
	0x3225,0x3226,INVALC,0x3227,0x3228,0x3229,0x322A,0x322B,
	/* Range 0x5106 - 0x5110, array index: 0x067D */
	0x324A,0x324B,0x324C,INVALC,INVALC,0x324D,0x324E,0x324F,
	0x3250,INVALC,0x3252,
	/* Range 0x5119 - 0x511E, array index: 0x0688 */
	0x3254,INVALC,0x3255,0x3256,0x3257,0x3258,
	/* Range 0x5131 - 0x5139, array index: 0x068E */
	0x325F,INVALC,0x3260,0x3261,0x3262,INVALC,INVALC,0x3263,
	0x3264,
	/* Range 0x51CF - 0x51D5, array index: 0x0697 */
	0x3322,INVALC,0x3323,0x3324,0x3325,INVALC,0x3326,
	/* Range 0x51F2 - 0x51F4, array index: 0x069E */
	0x332C,0x332D,0x332E,
	/* Range 0x5212 - 0x5218, array index: 0x06A1 */
	0x3333,0x3334,INVALC,0x3335,0x3336,INVALC,0x3337,
	/* Range 0x525F - 0x5261, array index: 0x06A8 */
	0x3345,0x3346,0x3347,
	/* Range 0x5277 - 0x5279, array index: 0x06AB */
	0x334A,0x334B,0x334C,
	/* Range 0x5293 - 0x529C, array index: 0x06AE */
	0x3352,INVALC,0x3353,0x3354,0x3355,0x3356,INVALC,0x3357,
	INVALC,0x3358,
	/* Range 0x52A4 - 0x52A7, array index: 0x06B8 */
	0x3359,0x335A,0x335B,0x335C,
	/* Range 0x52B6 - 0x52BD, array index: 0x06BC */
	0x335F,0x3360,0x3361,INVALC,0x3362,0x3363,INVALC,0x3364,
	/* Range 0x52E8 - 0x52EC, array index: 0x06C4 */
	0x3372,0x3373,0x3374,INVALC,0x3375,
	/* Range 0x530A - 0x530C, array index: 0x06C9 */
	0x337D,0x337E,0x3421,
	/* Range 0x531B - 0x531F, array index: 0x06CC */
	0x3425,0x3426,INVALC,0x3427,0x3428,
	/* Range 0x5325 - 0x532D, array index: 0x06D1 */
	0x3429,INVALC,0x342A,0x342B,0x342C,INVALC,0x342D,0x342E,
	0x342F,
	/* Range 0x533C - 0x533E, array index: 0x06DA */
	0x3433,0x3434,0x3435,
	/* Range 0x53B4 - 0x53BA, array index: 0x06DD */
	0x3451,0x3452,INVALC,0x3453,0x3454,INVALC,0x3455,
	/* Range 0x54A6 - 0x54AE, array index: 0x06E4 */
	0x352D,0x352E,INVALC,0x352F,0x3530,INVALC,INVALC,0x3531,
	0x3532,
	/* Range 0x54B7 - 0x54BB, array index: 0x06ED */
	0x3534,INVALC,0x3535,0x3536,0x3537,
	/* Range 0x54FC - 0x5501, array index: 0x06F2 */
	0x3542,INVALC,0x3543,0x3544,0x3545,0x3546,
	/* Range 0x5508 - 0x550E, array index: 0x06F8 */
	0x3548,0x3549,INVALC,INVALC,0x354A,0x354B,0x354C,
	/* Range 0x553B - 0x553D, array index: 0x06FF */
	0x3553,0x3554,0x3555,
	/* Range 0x558E - 0x5594, array index: 0x0702 */
	0x356A,0x356B,INVALC,0x356C,0x356D,0x356E,0x356F,
	/* Range 0x55D1 - 0x55D3, array index: 0x0709 */
	0x357D,0x357E,0x3621,
	/* Range 0x560D - 0x5612, array index: 0x070C */
	0x362D,0x362E,0x362F,0x3630,0x3631,0x3632,
	/* Range 0x5639 - 0x5646, array index: 0x0712 */
	0x3639,INVALC,0x363A,0x363B,0x363C,INVALC,0x363D,0x363E,
	0x363F,INVALC,0x3640,0x3641,INVALC,0x3642,
	/* Range 0x565E - 0x5663, array index: 0x0720 */
	0x3648,INVALC,0x3649,0x364A,0x364B,0x364C,
	/* Range 0x5699 - 0x569F, array index: 0x0726 */
	0x365A,0x365B,INVALC,INVALC,0x365C,0x365D,0x365E,
	/* Range 0x56A6 - 0x56AD, array index: 0x072D */
	0x365F,0x3660,0x3661,0x3662,INVALC,0x3663,0x3664,0x3665,
	/* Range 0x56C9 - 0x56D0, array index: 0x0735 */
	0x366B,0x366C,0x366D,0x3670,0x3671,INVALC,0x366E,0x366F,
	/* Range 0x56E4 - 0x56E8, array index: 0x073D */
	0x3677,0x3678,0x3679,0x367A,0x367B,
	/* Range 0x5720 - 0x5725, array index: 0x0742 */
	0x372D,INVALC,0x372E,0x372F,0x3730,0x3731,
	/* Range 0x573D - 0x573F, array index: 0x0748 */
	0x3739,0x373A,0x373B,
	/* Range 0x576B - 0x577E, array index: 0x074B */
	0x3745,INVALC,0x3746,0x3747,0x3748,0x3749,0x374A,INVALC,
	0x374B,0x374C,0x374D,INVALC,0x374E,INVALC,0x374F,0x3750,
	0x3751,0x3752,INVALC,0x3753,
	/* Range 0x5799 - 0x57A1, array index: 0x075F */
	0x3759,0x375A,INVALC,0x375B,0x375C,0x375D,0x375E,INVALC,
	0x375F,
	/* Range 0x57A7 - 0x57A9, array index: 0x0768 */
	0x3761,0x3762,0x3763,
	/* Range 0x57FD - 0x57FF, array index: 0x076B */
	0x3777,0x3778,0x3779,
	/* Range 0x581E - 0x5820, array index: 0x076E */
	0x3824,0x3825,0x3826,
	/* Range 0x584C - 0x5850, array index: 0x0771 */
	0x382E,0x382F,INVALC,0x3830,0x3831,
	/* Range 0x587F - 0x5881, array index: 0x0776 */
	0x383A,0x383B,0x383C,
	/* Range 0x5887 - 0x5890, array index: 0x0779 */
	0x383D,0x383E,0x383F,0x3840,INVALC,0x3841,0x3842,INVALC,
	0x3843,0x3844,
	/* Range 0x58A0 - 0x58A2, array index: 0x0783 */
	0x3848,0x3849,0x384A,
	/* Range 0x5912 - 0x5914, array index: 0x0786 */
	0x3863,0x3864,0x3865,
	/* Range 0x595B - 0x5963, array index: 0x0789 */
	0x3877,INVALC,0x3878,0x3879,0x387A,INVALC,0x387B,INVALC,
	0x387C,
	/* Range 0x59AD - 0x59B0, array index: 0x0792 */
	0x3931,0x3932,0x3933,0x3934,
	/* Range 0x59DD - 0x59DF, array index: 0x0796 */
	0x3940,0x3941,0x3942,
	/* Range 0x59EE - 0x59F4, array index: 0x0799 */
	0x3946,0x3947,INVALC,0x3948,0x3949,INVALC,0x394A,
	/* Range 0x5A0C - 0x5A0E, array index: 0x07A0 */
	0x394E,0x394F,0x3950,
	/* Range 0x5A44 - 0x5A48, array index: 0x07A3 */
	0x395B,0x395C,INVALC,0x395D,0x395E,
	/* Range 0x5A9C - 0x5AA2, array index: 0x07A8 */
	0x3970,INVALC,0x3971,0x3972,0x3973,INVALC,0x3974,
	/* Range 0x5AB1 - 0x5AB5, array index: 0x07AF */
	0x3977,0x3978,0x3979,INVALC,0x397A,
	/* Range 0x5B7C - 0x5B81, array index: 0x07B4 */
	0x3A43,0x3A44,0x3A45,0x3A46,INVALC,0x3A47,
	/* Range 0x5B90 - 0x5B96, array index: 0x07BA */
	0x3A4C,0x3A4D,INVALC,0x3A4E,0x3A4F,INVALC,0x3A50,
	/* Range 0x5BD6 - 0x5BDA, array index: 0x07C1 */
	0x3A5F,0x3A60,0x3A61,0x3A62,0x3A63,
	/* Range 0x5C67 - 0x5C69, array index: 0x07C6 */
	0x3A7C,0x3A7D,0x3A7E,
	/* Range 0x5C7A - 0x5C7D, array index: 0x07C9 */
	0x3B25,0x3B26,0x3B27,0x3B28,
	/* Range 0x5C9F - 0x5CA3, array index: 0x07CD */
	0x3B2F,0x3B30,INVALC,0x3B31,0x3B32,
	/* Range 0x5D23 - 0x5D27, array index: 0x07D2 */
	0x3B47,0x3B48,INVALC,0x3B49,0x3B4A,
	/* Range 0x5D5F - 0x5D64, array index: 0x07D7 */
	0x3B58,0x3B59,0x3B5A,0x3B5B,INVALC,0x3B5C,
	/* Range 0x5D92 - 0x5D95, array index: 0x07DD */
	0x3B68,0x3B69,0x3B6A,0x3B6B,
	/* Range 0x5E68 - 0x5E70, array index: 0x07E1 */
	0x3C3B,INVALC,0x3C3C,0x3C3D,0x3C3E,0x3C3F,0x3C40,INVALC,
	0x3C41,
	/* Range 0x5EBD - 0x5EBF, array index: 0x07EA */
	0x3C4D,0x3C4E,0x3C4F,
	/* Range 0x5ED1 - 0x5ED5, array index: 0x07ED */
	0x3C54,0x3C55,INVALC,0x3C56,0x3C57,
	/* Range 0x5F06 - 0x5F08, array index: 0x07F2 */
	0x3C5D,0x3C5E,0x3C5F,
	/* Range 0x5F21 - 0x5F24, array index: 0x07F5 */
	0x3C64,0x3C65,0x3C66,0x3C67,
	/* Range 0x5FAB - 0x5FB1, array index: 0x07F9 */
	0x3D31,0x3D33,INVALC,INVALC,0x3D34,0x3D35,0x3D36,
	/* Range 0x5FC7 - 0x5FCB, array index: 0x0800 */
	0x3D39,0x3D3A,0x3D3B,INVALC,0x3D3C,
	/* Range 0x5FD0 - 0x5FD4, array index: 0x0805 */
	0x3D3D,0x3D3E,0x3D3F,0x3D40,0x3D41,
	/* Range 0x5FE8 - 0x5FF3, array index: 0x080A */
	0x3D45,0x3D46,0x3D47,INVALC,0x3D48,0x3D49,0x3D4A,0x3D4B,
	INVALC,INVALC,0x3D4C,0x3D4D,
	/* Range 0x6047 - 0x6049, array index: 0x0816 */
	0x3D5F,0x3D60,0x3D61,
	/* Range 0x60C8 - 0x60CF, array index: 0x0819 */
	0x3E25,0x3E26,0x3E27,0x3E28,INVALC,INVALC,0x3E29,0x3E2A,
	/* Range 0x6110 - 0x6119, array index: 0x0821 */
	0x3E3C,0x3E3D,0x3E3E,0x3E3F,0x3E40,INVALC,0x3E41,0x3E42,
	INVALC,0x3E43,
	/* Range 0x6135 - 0x6139, array index: 0x082B */
	0x3E4B,0x3E4C,0x3E4D,INVALC,0x3E4E,
	/* Range 0x617B - 0x6184, array index: 0x0830 */
	0x3E58,0x3E59,INVALC,INVALC,0x3E5A,0x3E5B,0x3E5C,INVALC,
	0x3E5D,0x3E5E,
	/* Range 0x619C - 0x61A0, array index: 0x083A */
	0x3E65,0x3E66,INVALC,0x3E67,0x3E68,
	/* Range 0x61C0 - 0x61C2, array index: 0x083F */
	0x3E70,0x3E71,0x3E72,
	/* Range 0x61DC - 0x61E2, array index: 0x0842 */
	0x3E76,0x3E77,0x3E78,0x3E79,INVALC,0x3E7A,0x3E7B,
	/* Range 0x6242 - 0x6246, array index: 0x0849 */
	0x3F33,0x3F34,0x3F35,INVALC,0x3F36,
	/* Range 0x6250 - 0x6256, array index: 0x084E */
	0x3F38,0x3F39,0x3F3A,INVALC,0x3F3B,INVALC,0x3F3C,
	/* Range 0x628D - 0x6290, array index: 0x0855 */
	0x3F45,0x3F46,0x3F47,0x3F48,
	/* Range 0x6341 - 0x6346, array index: 0x0859 */
	0x3F6B,0x3F6C,0x3F6D,0x3F6E,INVALC,0x3F6F,
	/* Range 0x6352 - 0x6354, array index: 0x085F */
	0x3F73,0x3F74,0x3F75,
	/* Range 0x63AD - 0x63AF, array index: 0x0862 */
	0x4031,0x4032,0x4033,
	/* Range 0x63D1 - 0x63D5, array index: 0x0865 */
	0x4039,INVALC,0x403A,0x403B,0x403C,
	/* Range 0x644F - 0x6454, array index: 0x086A */
	0x405A,INVALC,0x405B,0x405C,0x405D,0x405E,
	/* Range 0x645A - 0x6463, array index: 0x0870 */
	0x405F,0x4060,0x4061,0x4062,INVALC,0x4063,0x4064,0x4065,
	INVALC,0x4066,
	/* Range 0x648F - 0x6491, array index: 0x087A */
	0x406E,0x406F,0x4070,
	/* Range 0x64BD - 0x64BF, array index: 0x087D */
	0x407C,0x407D,0x407E,
	/* Range 0x64C9 - 0x64D1, array index: 0x0880 */
	0x4122,0x4123,0x4124,0x4125,INVALC,0x4126,INVALC,0x4127,
	0x4128,
	/* Range 0x6508 - 0x650A, array index: 0x0889 */
	0x4137,0x4138,0x4139,
	/* Range 0x65B2 - 0x65B4, array index: 0x088C */
	0x4161,0x4162,0x4163,
	/* Range 0x65FE - 0x6600, array index: 0x088F */
	0x4173,0x4174,0x4175,
	/* Range 0x661D - 0x6633, array index: 0x0892 */
	0x417E,0x4221,INVALC,INVALC,0x4222,0x4223,0x4224,0x4225,
	INVALC,0x4226,INVALC,INVALC,0x4227,0x4228,0x4229,0x422A,
	INVALC,0x422B,INVALC,0x422C,0x422D,INVALC,0x422E,
	/* Range 0x6657 - 0x665C, array index: 0x08A9 */
	0x4238,0x4239,0x423A,INVALC,0x423B,0x423C,
	/* Range 0x666A - 0x666C, array index: 0x08AF */
	0x4240,0x4241,0x4242,
	/* Range 0x6675 - 0x6680, array index: 0x08B2 */
	0x4245,INVALC,0x4247,0x4248,0x4249,INVALC,0x424A,0x424C,
	INVALC,0x4243,0x4246,0x424B,
	/* Range 0x668B - 0x668D, array index: 0x08BE */
	0x424D,0x424E,0x424F,
	/* Range 0x6699 - 0x66A0, array index: 0x08C1 */
	0x4252,0x4253,0x4254,0x4255,INVALC,INVALC,0x4256,0x4257,
	/* Range 0x66BF - 0x66C3, array index: 0x08C9 */
	0x425E,0x425F,INVALC,0x4260,0x4261,
	/* Range 0x67B0 - 0x67B2, array index: 0x08CE */
	0x4331,0x4332,0x4333,
	/* Range 0x67BB - 0x67BD, array index: 0x08D1 */
	0x4335,0x4336,0x4337,
	/* Range 0x682C - 0x6833, array index: 0x08D4 */
	0x4350,0x4351,INVALC,0x4352,0x4353,0x4354,INVALC,0x4355,
	/* Range 0x686E - 0x6872, array index: 0x08DC */
	0x4361,0x4362,0x4363,0x4364,0x4365,
	/* Range 0x6879 - 0x687C, array index: 0x08E1 */
	0x4367,0x4368,0x4369,0x436A,
	/* Range 0x68CF - 0x68D3, array index: 0x08E5 */
	0x437E,0x4421,0x4422,INVALC,0x4423,
	/* Range 0x68E8 - 0x68F1, array index: 0x08EA */
	0x4429,INVALC,0x442A,0x442B,0x442C,0x442D,INVALC,INVALC,
	0x442E,0x442F,
	/* Range 0x68FB - 0x68FD, array index: 0x08F4 */
	0x4432,0x4433,0x4434,
	/* Range 0x6963 - 0x6969, array index: 0x08F7 */
	0x4448,0x4449,0x444A,0x444B,INVALC,0x444C,0x444D,
	/* Range 0x6970 - 0x6972, array index: 0x08FE */
	0x444F,0x4450,0x4451,
	/* Range 0x6A4E - 0x6A56, array index: 0x0901 */
	0x452A,INVALC,0x452B,0x452C,0x452D,INVALC,INVALC,0x452E,
	0x452F,
	/* Range 0x6A9B - 0x6A9F, array index: 0x090A */
	0x453E,INVALC,0x453F,0x4540,0x4541,
	/* Range 0x6AAF - 0x6AB1, array index: 0x090F */
	0x4544,0x4545,0x4546,
	/* Range 0x6ABD - 0x6ABF, array index: 0x0912 */
	0x4548,0x4549,0x454A,
	/* Range 0x6AD4 - 0x6AD6, array index: 0x0915 */
	0x4550,0x4551,0x4552,
	/* Range 0x6AF0 - 0x6AF2, array index: 0x0918 */
	0x4558,0x4559,0x455A,
	/* Range 0x6B0F - 0x6B11, array index: 0x091B */
	0x4562,0x4563,0x4564,
	/* Range 0x6B9F - 0x6BA3, array index: 0x091E */
	0x4626,0x4627,INVALC,0x4628,0x4629,
	/* Range 0x6BA8 - 0x6BB0, array index: 0x0923 */
	0x462A,0x462B,INVALC,INVALC,0x462C,0x462D,0x462E,INVALC,
	0x462F,
	/* Range 0x6C26 - 0x6C28, array index: 0x092C */
	0x464D,0x464E,0x464F,
	/* Range 0x6C85 - 0x6C89, array index: 0x092F */
	0x4669,0x466A,0x466B,INVALC,0x466C,
	/* Range 0x6C94 - 0x6C98, array index: 0x0934 */
	0x466D,0x466E,INVALC,0x466F,0x4670,
	/* Range 0x6CCD - 0x6CD6, array index: 0x0939 */
	0x4678,INVALC,0x4679,0x467A,0x467B,0x467C,INVALC,0x467D,
	INVALC,0x467E,
	/* Range 0x6D26 - 0x6D28, array index: 0x0943 */
	0x4733,0x4734,0x4735,
	/* Range 0x6D94 - 0x6D98, array index: 0x0946 */
	0x474A,INVALC,0x474B,0x474C,0x474D,
	/* Range 0x6E4F - 0x6E54, array index: 0x094B */
	0x4776,INVALC,0x4777,0x4778,0x4779,0x477A,
	/* Range 0x6E5C - 0x6E5E, array index: 0x0951 */
	0x477C,0x477D,0x477E,
	/* Range 0x6EBF - 0x6EC3, array index: 0x0954 */
	0x4831,0x4832,0x4833,INVALC,0x4834,
	/* Range 0x6ECD - 0x6ECF, array index: 0x0959 */
	0x4838,0x4839,0x483A,
	/* Range 0x6F4F - 0x6F53, array index: 0x095C */
	0x4854,INVALC,0x4855,0x4856,0x4857,
	/* Range 0x6F87 - 0x6F8D, array index: 0x0961 */
	0x4864,0x4865,INVALC,INVALC,0x4866,0x4867,0x4868,
	/* Range 0x6F90 - 0x6F96, array index: 0x0968 */
	0x4869,INVALC,0x486A,0x486B,0x486C,INVALC,0x486D,
	/* Range 0x6FA5 - 0x6FA8, array index: 0x096F */
	0x4871,0x4872,0x4873,0x4874,
	/* Range 0x6FAE - 0x6FB0, array index: 0x0973 */
	0x4875,0x4876,0x4877,
	/* Range 0x7005 - 0x7007, array index: 0x0976 */
	0x492B,0x492C,0x492D,
	/* Range 0x7048 - 0x704B, array index: 0x0979 */
	0x4939,0x493A,0x493B,0x493C,
	/* Range 0x7094 - 0x7098, array index: 0x097D */
	0x494C,0x494D,0x494E,0x494F,0x4950,
	/* Range 0x70D1 - 0x70D8, array index: 0x0982 */
	0x4959,INVALC,0x495A,0x495B,0x495C,0x495D,INVALC,0x495E,
	/* Range 0x7103 - 0x7107, array index: 0x098A */
	0x4962,0x4963,0x4964,0x4965,0x4966,
	/* Range 0x712D - 0x7131, array index: 0x098F */
	0x496D,INVALC,0x496E,0x496F,0x4970,
	/* Range 0x7145 - 0x714B, array index: 0x0994 */
	0x4973,0x4974,0x4975,INVALC,INVALC,0x4976,0x4977,
	/* Range 0x71AF - 0x71B3, array index: 0x099B */
	0x4A2C,0x4A2D,INVALC,0x4A2E,0x4A2F,
	/* Range 0x71BF - 0x71C1, array index: 0x09A0 */
	0x4A31,0x4A32,0x4A33,
	/* Range 0x7207 - 0x7209, array index: 0x09A3 */
	0x4A3F,0x4A40,0x4A41,
	/* Range 0x7241 - 0x7245, array index: 0x09A6 */
	0x4A4D,0x4A4E,0x4A4F,INVALC,0x4A50,
	/* Range 0x724E - 0x7250, array index: 0x09AB */
	0x4A51,0x4A52,0x4A53,
	/* Range 0x735D - 0x7360, array index: 0x09AE */
	0x4B3E,0x4B3F,0x4B40,0x4B41,
	/* Range 0x736B - 0x7371, array index: 0x09B2 */
	0x4B45,0x4B46,INVALC,0x4B47,0x4B48,INVALC,0x4B49,
	/* Range 0x739C - 0x73A6, array index: 0x09B9 */
	0x4B58,INVALC,0x4B59,0x4B5A,0x4B5B,INVALC,0x4B5C,INVALC,
	INVALC,0x4B5D,0x4B5E,
	/* Range 0x73F7 - 0x7401, array index: 0x09C4 */
	0x4B7A,INVALC,0x4B7B,0x4B7C,0x4B7D,INVALC,0x4B7E,INVALC,
	0x4C21,0x4C22,0x4C23,
	/* Range 0x7426 - 0x7431, array index: 0x09CF */
	0x4C2B,INVALC,0x4C2C,0x4C2D,0x4C2E,0x4C2F,0x4C30,0x4C31,
	0x4C32,0x4C33,0x4C34,0x4C35,
	/* Range 0x7443 - 0x7447, array index: 0x09DB */
	0x4C38,0x4C39,INVALC,0x4C3A,0x4C3B,
	/* Range 0x7466 - 0x7468, array index: 0x09E0 */
	0x4C43,0x4C44,0x4C45,
	/* Range 0x7485 - 0x7489, array index: 0x09E3 */
	0x4C4D,0x4C4E,0x4C4F,INVALC,0x4C50,
	/* Range 0x748F - 0x7492, array index: 0x09E8 */
	0x4C51,0x4C52,0x4C53,0x4C54,
	/* Range 0x7498 - 0x74A3, array index: 0x09EC */
	0x4C55,0x4C56,0x4C57,INVALC,0x4C58,INVALC,INVALC,0x4C59,
	0x4C5A,0x4C5B,INVALC,0x4C5C,
	/* Range 0x74A6 - 0x74B2, array index: 0x09F8 */
	0x4C5D,INVALC,0x4C5E,0x4C5F,0x4C60,0x4C61,INVALC,INVALC,
	0x4C62,0x4C63,INVALC,0x4C64,0x4C65,
	/* Range 0x74FA - 0x74FC, array index: 0x0A05 */
	0x4C7A,0x4C7B,0x4C7C,
	/* Range 0x753D - 0x7540, array index: 0x0A08 */
	0x4D2D,0x4D2E,0x4D2F,0x4D30,
	/* Range 0x7579 - 0x757E, array index: 0x0A0C */
	0x4D3D,0x4D3E,0x4D3F,0x4D40,0x4D41,0x4D42,
	/* Range 0x75BF - 0x75C1, array index: 0x0A12 */
	0x4D4F,0x4D50,0x4D51,
	/* Range 0x75DF - 0x75E1, array index: 0x0A15 */
	0x4D59,0x4D5A,0x4D5B,
	/* Range 0x7600 - 0x760C, array index: 0x0A18 */
	0x4D63,INVALC,0x4D64,0x4D65,0x4D66,INVALC,INVALC,0x4D67,
	0x4D68,INVALC,0x4D69,INVALC,0x4D6A,
	/* Range 0x7612 - 0x7616, array index: 0x0A25 */
	0x4D6C,0x4D6D,INVALC,0x4D6E,0x4D6F,
	/* Range 0x7619 - 0x761E, array index: 0x0A2A */
	0x4D70,INVALC,0x4D71,0x4D72,0x4D73,0x4D74,
	/* Range 0x7638 - 0x763C, array index: 0x0A30 */
	0x4D7D,0x4D7E,0x4E21,INVALC,0x4E22,
	/* Range 0x7640 - 0x764B, array index: 0x0A35 */
	0x4E24,0x4E25,INVALC,0x4E26,0x4E27,0x4E28,INVALC,INVALC,
	INVALC,0x4E29,0x4E23,0x4E2A,
	/* Range 0x766D - 0x7671, array index: 0x0A41 */
	0x4E30,0x4E31,0x4E32,INVALC,0x4E33,
	/* Range 0x769B - 0x76AA, array index: 0x0A46 */
	0x4E3A,0x4E3B,0x4E3C,INVALC,0x4E3D,0x4E3E,INVALC,0x4E3F,
	0x4E40,0x4E41,0x4E42,0x4E43,0x4E44,0x4E45,INVALC,0x4E46,
	/* Range 0x772D - 0x772F, array index: 0x0A56 */
	0x4E67,0x4E68,0x4E69,
	/* Range 0x7734 - 0x7736, array index: 0x0A59 */
	0x4E6A,0x4E6B,0x4E6C,
	/* Range 0x774D - 0x774F, array index: 0x0A5C */
	0x4E74,0x4E75,0x4E76,
	/* Range 0x775C - 0x7764, array index: 0x0A5F */
	0x4E7A,INVALC,0x4E7B,0x4E7C,0x4E7D,INVALC,0x4E7E,INVALC,
	0x4F21,
	/* Range 0x7770 - 0x7774, array index: 0x0A68 */
	0x4F25,INVALC,0x4F26,0x4F27,0x4F28,
	/* Range 0x7794 - 0x7796, array index: 0x0A6D */
	0x4F2F,0x4F30,0x4F31,
	/* Range 0x77DE - 0x77E0, array index: 0x0A70 */
	0x4F42,0x4F43,0x4F44,
	/* Range 0x7821 - 0x7823, array index: 0x0A73 */
	0x4F55,0x4F56,0x4F57,
	/* Range 0x7860 - 0x7864, array index: 0x0A76 */
	0x4F66,0x4F67,INVALC,0x4F68,0x4F69,
	/* Range 0x789D - 0x78A1, array index: 0x0A7B */
	0x4F74,0x4F75,0x4F76,INVALC,0x4F73,
	/* Range 0x78AC - 0x78B3, array index: 0x0A80 */
	0x4F79,0x4F7A,INVALC,INVALC,0x4F7B,0x4F7C,0x4F7D,0x4F7E,
	/* Range 0x78C7 - 0x78C9, array index: 0x0A88 */
	0x5024,0x5025,0x5026,
	/* Range 0x78D2 - 0x78D6, array index: 0x0A8B */
	0x5029,0x502A,INVALC,0x502B,0x502C,
	/* Range 0x78DF - 0x78E1, array index: 0x0A90 */
	0x502F,0x5030,0x5031,
	/* Range 0x791C - 0x7920, array index: 0x0A93 */
	0x5040,INVALC,0x5041,0x5042,0x5043,
	/* Range 0x7944 - 0x7946, array index: 0x0A98 */
	0x504E,0x504F,0x5050,
	/* Range 0x7991 - 0x799C, array index: 0x0A9B */
	0x5063,INVALC,0x5064,0x5065,0x5066,0x5067,INVALC,0x5068,
	INVALC,INVALC,0x5069,0x506A,
	/* Range 0x7A34 - 0x7A3A, array index: 0x0AA7 */
	0x513A,0x513B,INVALC,INVALC,0x513C,0x513D,0x513E,
	/* Range 0x7A44 - 0x7A48, array index: 0x0AAE */
	0x513F,0x5140,INVALC,0x5141,0x5142,
	/* Range 0x7A5C - 0x7A60, array index: 0x0AB3 */
	0x5147,0x5148,INVALC,0x5149,0x514A,
	/* Range 0x7AE8 - 0x7AEC, array index: 0x0AB8 */
	0x516A,0x516B,INVALC,0x516C,0x516D,
	/* Range 0x7B27 - 0x7B31, array index: 0x0ABD */
	0x5177,INVALC,0x5178,0x5179,0x517A,INVALC,0x517B,0x517C,
	0x517D,0x517E,0x5221,
	/* Range 0x7B3D - 0x7B41, array index: 0x0AC8 */
	0x5223,INVALC,0x5224,0x5225,0x5226,
	/* Range 0x7BBA - 0x7BBD, array index: 0x0ACD */
	0x5244,0x5245,0x5246,0x5247,
	/* Range 0x7BD6 - 0x7BDB, array index: 0x0AD1 */
	0x524D,0x524E,INVALC,0x524F,0x5250,0x5251,
	/* Range 0x7BF4 - 0x7C06, array index: 0x0AD7 */
	0x5255,0x5256,INVALC,INVALC,0x5257,0x5258,0x5259,INVALC,
	0x525A,INVALC,0x525B,INVALC,INVALC,0x525C,0x525D,0x525E,
	0x525F,INVALC,0x5260,
	/* Range 0x7C0B - 0x7C0F, array index: 0x0AEA */
	0x5262,0x5263,INVALC,0x5264,0x5265,
	/* Range 0x7C51 - 0x7C5E, array index: 0x0AEF */
	0x5276,0x5277,0x5278,INVALC,0x5275,INVALC,INVALC,INVALC,
	0x5279,0x527A,0x527B,0x527C,0x527D,0x527E,
	/* Range 0x7CB6 - 0x7CBC, array index: 0x0AFD */
	0x5334,0x5335,INVALC,INVALC,0x5336,0x5337,0x5338,
	/* Range 0x7CC7 - 0x7CC9, array index: 0x0B04 */
	0x533B,0x533C,0x533D,
	/* Range 0x7CD3 - 0x7CDA, array index: 0x0B07 */
	0x5340,0x5341,0x5342,INVALC,0x5343,INVALC,0x5344,0x5345,
	/* Range 0x7D07 - 0x7D09, array index: 0x0B0F */
	0x534C,0x534D,0x534E,
	/* Range 0x7D0F - 0x7D13, array index: 0x0B12 */
	0x534F,INVALC,0x5350,0x5351,0x5352,
	/* Range 0x7D3C - 0x7D41, array index: 0x0B17 */
	0x535B,0x535C,0x535D,INVALC,0x535E,0x535F,
	/* Range 0x7D59 - 0x7D5D, array index: 0x0B1D */
	0x5366,0x5367,INVALC,0x5368,0x5369,
	/* Range 0x7D7F - 0x7D8D, array index: 0x0B22 */
	0x5371,INVALC,0x5372,0x5373,0x5374,INVALC,0x5375,0x5376,
	INVALC,0x5377,INVALC,INVALC,0x5378,0x5379,0x537A,
	/* Range 0x7DC2 - 0x7DC6, array index: 0x0B31 */
	0x5429,0x542A,0x542B,0x542C,0x542D,
	/* Range 0x7DCC - 0x7DCE, array index: 0x0B36 */
	0x542E,0x542F,0x5430,
	/* Range 0x7F3B - 0x7F3F, array index: 0x0B39 */
	0x546C,0x546B,0x546D,0x546E,0x546F,
	/* Range 0x7F5B - 0x7F5D, array index: 0x0B3E */
	0x5476,0x5477,0x5478,
	/* Range 0x7F61 - 0x7F66, array index: 0x0B41 */
	0x5479,INVALC,0x547A,0x547B,0x547C,0x547D,
	/* Range 0x7F7D - 0x7F80, array index: 0x0B47 */
	0x5522,0x5523,0x5524,0x5525,
	/* Range 0x7F8D - 0x7F91, array index: 0x0B4B */
	0x5527,INVALC,0x5528,0x5529,0x552A,
	/* Range 0x7FFD - 0x7FFF, array index: 0x0B50 */
	0x5545,0x5546,0x5547,
	/* Range 0x800D - 0x8016, array index: 0x0B53 */
	0x554B,0x554C,0x554D,INVALC,0x554E,INVALC,0x554F,0x5550,
	INVALC,0x5551,
	/* Range 0x801D - 0x8020, array index: 0x0B5D */
	0x5552,0x5553,0x5554,0x5555,
	/* Range 0x80D4 - 0x80D8, array index: 0x0B61 */
	0x5578,0x5579,INVALC,0x557A,0x557B,
	/* Range 0x8116 - 0x8118, array index: 0x0B66 */
	0x562A,0x562B,0x562C,
	/* Range 0x8167 - 0x8169, array index: 0x0B69 */
	0x563F,0x5640,0x5641,
	/* Range 0x8184 - 0x8186, array index: 0x0B6C */
	0x5647,0x5648,0x5649,
	/* Range 0x81F0 - 0x81F9, array index: 0x0B6F */
	0x5664,0x5665,0x5666,INVALC,INVALC,0x5667,0x5668,INVALC,
	0x5669,0x566A,
	/* Range 0x8243 - 0x8246, array index: 0x0B79 */
	0x567B,0x567C,0x567D,0x567E,
	/* Range 0x827D - 0x8284, array index: 0x0B7D */
	0x572D,INVALC,0x572E,0x572F,0x5730,INVALC,0x5731,0x5732,
	/* Range 0x82A0 - 0x82AA, array index: 0x0B85 */
	0x573D,0x573E,INVALC,0x573F,0x5740,INVALC,INVALC,0x5741,
	0x5742,0x5743,0x5744,
	/* Range 0x82FD - 0x8301, array index: 0x0B90 */
	0x575B,0x575C,INVALC,0x575D,0x575E,
	/* Range 0x8307 - 0x830B, array index: 0x0B95 */
	0x575F,0x5760,INVALC,0x5761,0x5762,
	/* Range 0x831B - 0x8322, array index: 0x0B9A */
	0x5764,INVALC,0x5765,0x5766,0x5767,INVALC,0x5768,0x5769,
	/* Range 0x832C - 0x8330, array index: 0x0BA2 */
	0x576A,0x576B,0x576C,INVALC,0x576D,
	/* Range 0x8342 - 0x8344, array index: 0x0BA7 */
	0x5773,0x5774,0x5775,
	/* Range 0x8351 - 0x8357, array index: 0x0BAA */
	0x5779,INVALC,0x583E,0x5763,0x577A,0x577B,0x577C,
	/* Range 0x8398 - 0x839D, array index: 0x0BB1 */
	0x582B,0x582C,INVALC,0x582D,0x582E,0x582F,
	/* Range 0x83BE - 0x83C0, array index: 0x0BB7 */
	0x5834,0x5835,0x5836,
	/* Range 0x83CF - 0x83D1, array index: 0x0BBA */
	0x5839,0x583A,0x583B,
	/* Range 0x8456 - 0x845C, array index: 0x0BBD */
	0x5857,INVALC,0x5858,0x5859,0x585A,INVALC,0x585B,
	/* Range 0x84A6 - 0x84AA, array index: 0x0BC4 */
	0x586E,INVALC,0x586F,0x5870,0x5871,
	/* Range 0x84EF - 0x84F2, array index: 0x0BC9 */
	0x5924,0x5925,0x5926,0x5927,
	/* Range 0x8522 - 0x852B, array index: 0x0BCD */
	0x5935,0x5936,0x5937,0x5938,INVALC,0x5939,INVALC,INVALC,
	0x593A,0x593B,
	/* Range 0x8532 - 0x8536, array index: 0x0BD7 */
	0x5929,0x593D,0x593E,INVALC,0x593F,
	/* Range 0x854F - 0x8553, array index: 0x0BDC */
	0x5942,0x5943,0x5944,0x5945,0x5946,
	/* Range 0x855C - 0x8564, array index: 0x0BE1 */
	0x5949,0x594A,0x594B,0x594C,0x594D,0x594E,0x594F,INVALC,
	0x5950,
	/* Range 0x8579 - 0x8581, array index: 0x0BEA */
	0x5953,0x5954,0x5955,INVALC,0x5956,INVALC,0x5957,INVALC,
	0x5958,
	/* Range 0x85B4 - 0x85BF, array index: 0x0BF3 */
	0x5967,INVALC,0x5968,0x5969,0x596A,INVALC,INVALC,INVALC,
	0x596B,0x596C,0x596D,0x596E,
	/* Range 0x860D - 0x8612, array index: 0x0BFF */
	0x5A25,0x5A26,INVALC,0x5A27,0x5A28,0x5A29,
	/* Range 0x863C - 0x8642, array index: 0x0C05 */
	0x5A34,0x5A35,INVALC,INVALC,0x5A36,0x3866,0x5A37,
	/* Range 0x8652 - 0x8659, array index: 0x0C0C */
	0x5A39,0x5A3A,INVALC,INVALC,0x5A3B,0x5A3C,0x5A3D,0x5A3E,
	/* Range 0x8660 - 0x8664, array index: 0x0C14 */
	0x5A40,0x5A41,0x5A42,0x5A43,0x5A44,
	/* Range 0x8675 - 0x8677, array index: 0x0C19 */
	0x5A48,0x5A49,0x5A4A,
	/* Range 0x86A6 - 0x86A8, array index: 0x0C1C */
	0x5A53,0x5A54,0x5A55,
	/* Range 0x86B1 - 0x86B9, array index: 0x0C1F */
	0x5A57,INVALC,0x5A58,0x5A59,0x5A5A,INVALC,0x5A5B,0x5A5C,
	0x5A5D,
	/* Range 0x86BF - 0x86C5, array index: 0x0C28 */
	0x5A5E,0x5A5F,0x5A60,INVALC,0x5A61,INVALC,0x5A62,
	/* Range 0x870E - 0x8714, array index: 0x0C2F */
	0x5A75,0x5A76,0x5A77,INVALC,INVALC,0x5A78,0x5A79,
	/* Range 0x872E - 0x8732, array index: 0x0C36 */
	0x5B22,0x5B23,INVALC,0x5B24,0x5B25,
	/* Range 0x8739 - 0x8740, array index: 0x0C3B */
	0x5B26,0x5B27,INVALC,0x5B28,0x5B29,0x5B2A,INVALC,0x5B2B,
	/* Range 0x8783 - 0x878C, array index: 0x0C43 */
	0x5B38,0x5B39,0x5B3A,0x5B3B,0x5B3C,0x5B3D,0x5B3E,INVALC,
	0x5B3F,0x5B40,
	/* Range 0x8795 - 0x8799, array index: 0x0C4D */
	0x5B43,INVALC,0x5B44,0x5B45,0x5B46,
	/* Range 0x87AC - 0x87AE, array index: 0x0C52 */
	0x5B4B,0x5B4C,0x5B4D,
	/* Range 0x87C8 - 0x87CA, array index: 0x0C55 */
	0x5B53,0x5B54,0x5B55,
	/* Range 0x87E2 - 0x87E4, array index: 0x0C58 */
	0x5B5D,0x5B5E,0x5B5F,
	/* Range 0x8809 - 0x880B, array index: 0x0C5B */
	0x5B6B,0x5B6C,0x5B6D,
	/* Range 0x8810 - 0x881F, array index: 0x0C5E */
	0x5B6E,INVALC,0x5B70,0x5B71,0x5B72,INVALC,INVALC,INVALC,
	0x5B73,0x5B6F,0x5B74,0x5B75,0x5B76,INVALC,0x5B77,0x5B78,
	/* Range 0x8848 - 0x884B, array index: 0x0C6E */
	0x5C26,0x5C27,0x5C28,0x5C29,
	/* Range 0x8898 - 0x88A0, array index: 0x0C72 */
	0x5C39,INVALC,0x5C3A,0x5C3B,0x5C3C,INVALC,INVALC,0x5C3D,
	0x5C3E,
	/* Range 0x88CA - 0x88D3, array index: 0x0C7B */
	0x5C45,0x5C46,0x5C47,0x5C48,0x5C49,INVALC,INVALC,0x5C4A,
	0x5C4B,0x5C4C,
	/* Range 0x88EF - 0x88F1, array index: 0x0C85 */
	0x5C50,0x5C51,0x5C52,
	/* Range 0x890D - 0x890F, array index: 0x0C88 */
	0x5C57,0x5C58,0x5C59,
	/* Range 0x8915 - 0x891C, array index: 0x0C8B */
	0x5C5A,0x5C5B,INVALC,0x5C5C,0x5C5D,0x5C5E,INVALC,0x5C5F,
	/* Range 0x8926 - 0x8928, array index: 0x0C93 */
	0x5C61,0x5C62,0x5C63,
	/* Range 0x8930 - 0x8932, array index: 0x0C96 */
	0x5C64,0x5C65,0x5C66,
	/* Range 0x895A - 0x895C, array index: 0x0C99 */
	0x5C73,0x5C74,0x5C75,
	/* Range 0x8961 - 0x8963, array index: 0x0C9C */
	0x5C76,0x5C77,0x5C78,
	/* Range 0x897A - 0x897D, array index: 0x0C9F */
	0x5C7E,0x5D21,0x5D22,0x5D23,
	/* Range 0x89B4 - 0x89B7, array index: 0x0CA3 */
	0x5D2F,0x5D30,0x5D31,0x5D32,
	/* Range 0x89D4 - 0x89D8, array index: 0x0CA7 */
	0x5D34,0x5D35,0x5D36,0x5D37,0x5D38,
	/* Range 0x8A11 - 0x8A15, array index: 0x0CAC */
	0x5D47,0x5D48,INVALC,0x5D49,0x5D4A,
	/* Range 0x8A56 - 0x8A58, array index: 0x0CB1 */
	0x5D5F,0x5D60,0x5D61,
	/* Range 0x8A75 - 0x8A80, array index: 0x0CB4 */
	0x5D67,0x5D68,0x5D69,INVALC,0x5D6A,0x5D6B,0x5D6C,INVALC,
	INVALC,0x5D6D,0x5D6E,0x5D6F,
	/* Range 0x8AC6 - 0x8ACA, array index: 0x0CC0 */
	0x5E26,INVALC,0x5E27,0x5E28,0x5E29,
	/* Range 0x8AD1 - 0x8AD7, array index: 0x0CC5 */
	0x5E2A,INVALC,0x5E2B,0x5E2C,0x5E2D,INVALC,0x5E2E,
	/* Range 0x8AF4 - 0x8AF6, array index: 0x0CCC */
	0x5E33,0x5E34,0x5E35,
	/* Range 0x8B42 - 0x8B48, array index: 0x0CCF */
	0x5E44,0x5E45,0x5E46,0x5E47,0x5E48,INVALC,0x5E49,
	/* Range 0x8B52 - 0x8B54, array index: 0x0CD6 */
	0x5E4A,0x5E4B,0x5E4C,
	/* Range 0x8C53 - 0x8C5D, array index: 0x0CD9 */
	0x5E6C,0x5E6D,INVALC,INVALC,0x5E6E,0x5E6F,0x5E72,INVALC,
	0x5E70,INVALC,0x5E71,
	/* Range 0x8D9E - 0x8DA0, array index: 0x0CE4 */
	0x5F52,0x5F53,0x5F54,
	/* Range 0x8DC5 - 0x8DCA, array index: 0x0CE7 */
	0x5F5F,0x5F60,0x5F61,0x5F62,INVALC,0x5F63,
	/* Range 0x8DF0 - 0x8DF4, array index: 0x0CED */
	0x5F6E,0x5F70,0x5F71,INVALC,0x5F72,
	/* Range 0x8E04 - 0x8E06, array index: 0x0CF2 */
	0x5F75,0x5F76,0x5F77,
	/* Range 0x8E20 - 0x8E27, array index: 0x0CF5 */
	0x5F7C,0x5F7D,0x5F7E,0x6021,INVALC,INVALC,0x6022,0x6023,
	/* Range 0x8E36 - 0x8E39, array index: 0x0CFD */
	0x6026,0x6027,0x6028,0x6029,
	/* Range 0x8E4B - 0x8E4F, array index: 0x0D01 */
	0x602D,INVALC,0x602E,0x602F,0x6030,
	/* Range 0x8E5B - 0x8E62, array index: 0x0D06 */
	0x6032,0x6033,0x6034,0x6035,INVALC,INVALC,0x6036,0x6037,
	/* Range 0x8E6C - 0x8E71, array index: 0x0D0E */
	0x6039,0x603A,INVALC,0x603B,0x603C,0x603D,
	/* Range 0x8E79 - 0x8E7B, array index: 0x0D14 */
	0x603E,0x603F,0x6040,
	/* Range 0x8E9A - 0x8E9E, array index: 0x0D17 */
	0x6047,0x6048,INVALC,0x6049,0x604A,
	/* Range 0x8EC0 - 0x8EC4, array index: 0x0D1C */
	0x6054,0x6055,INVALC,0x6056,0x6057,
	/* Range 0x8EED - 0x8EF1, array index: 0x0D21 */
	0x6064,0x605E,INVALC,0x605F,0x6060,
	/* Range 0x8F16 - 0x8F18, array index: 0x0D26 */
	0x606B,0x606C,0x606D,
	/* Range 0x8F2C - 0x8F2E, array index: 0x0D29 */
	0x6075,0x6076,0x6077,
	/* Range 0x8F34 - 0x8F37, array index: 0x0D2C */
	0x6078,0x6079,0x607A,0x607B,
	/* Range 0x8F4F - 0x8F55, array index: 0x0D30 */
	0x6123,INVALC,0x6124,0x6125,0x6126,0x6127,0x6128,
	/* Range 0x8FA0 - 0x8FA6, array index: 0x0D37 */
	0x612E,0x612F,INVALC,INVALC,0x6130,0x6131,0x6132,
	/* Range 0x9028 - 0x902F, array index: 0x0D3E */
	0x6151,0x6152,0x6154,INVALC,0x6155,0x6156,INVALC,0x6153,
	/* Range 0x9095 - 0x909B, array index: 0x0D46 */
	0x616D,INVALC,0x616E,0x616F,0x6170,INVALC,0x6171,
	/* Range 0x90A0 - 0x90A2, array index: 0x0D4D */
	0x6172,0x6173,0x6174,
	/* Range 0x90B0 - 0x90B6, array index: 0x0D50 */
	0x6176,INVALC,0x6177,0x6178,0x6179,INVALC,0x617A,
	/* Range 0x90C3 - 0x90C8, array index: 0x0D57 */
	0x617E,0x6221,0x6222,INVALC,0x6223,0x6224,
	/* Range 0x90D5 - 0x90DF, array index: 0x0D5D */
	0x6225,INVALC,0x6226,0x6227,0x6228,INVALC,INVALC,0x6229,
	0x622A,INVALC,0x622B,
	/* Range 0x90FE - 0x9108, array index: 0x0D68 */
	0x6233,0x6234,0x6235,INVALC,INVALC,INVALC,0x6236,0x6237,
	0x6238,INVALC,0x6239,
	/* Range 0x9114 - 0x9129, array index: 0x0D73 */
	0x623C,INVALC,0x623D,0x623E,0x623F,INVALC,0x6240,INVALC,
	0x6241,INVALC,0x6242,INVALC,0x6243,INVALC,0x6245,0x6246,
	INVALC,0x6244,INVALC,0x6247,INVALC,0x6248,
	/* Range 0x9136 - 0x913D, array index: 0x0D89 */
	0x624D,0x624E,INVALC,0x624F,0x6250,INVALC,0x6251,0x6252,
	/* Range 0x9157 - 0x915B, array index: 0x0D91 */
	0x6258,INVALC,0x6259,0x625A,0x625B,
	/* Range 0x9179 - 0x917B, array index: 0x0D96 */
	0x6261,0x6262,0x6263,
	/* Range 0x9191 - 0x9195, array index: 0x0D99 */
	0x626A,INVALC,0x626B,0x626C,0x626D,
	/* Range 0x91AC - 0x91B3, array index: 0x0D9E */
	0x6273,0x6274,0x6275,INVALC,0x6276,0x6277,0x6278,0x6279,
	/* Range 0x91BB - 0x91C5, array index: 0x0DA6 */
	0x627B,0x627C,0x627D,INVALC,0x627E,INVALC,INVALC,0x6321,
	0x6322,INVALC,0x6323,
	/* Range 0x91E9 - 0x91F1, array index: 0x0DB1 */
	0x632C,0x632D,INVALC,0x632E,0x632F,0x6330,0x6331,0x6332,
	0x6333,
	/* Range 0x9200 - 0x920C, array index: 0x0DBA */
	0x6338,0x6339,INVALC,INVALC,0x633A,0x633B,0x633C,0x633D,
	INVALC,0x633E,0x633F,INVALC,0x6340,
	/* Range 0x9223 - 0x9228, array index: 0x0DC7 */
	0x6348,0x6349,0x634A,0x634B,INVALC,0x634C,
	/* Range 0x922E - 0x9230, array index: 0x0DCD */
	0x634D,0x634E,0x634F,
	/* Range 0x9235 - 0x9247, array index: 0x0DD0 */
	0x6351,0x6352,INVALC,0x6353,0x6354,0x6355,INVALC,0x6356,
	INVALC,0x6357,INVALC,0x6358,INVALC,0x6359,0x635A,INVALC,
	INVALC,0x635B,0x635C,
	/* Range 0x924D - 0x9251, array index: 0x0DE3 */
	0x635E,0x635F,0x6360,INVALC,0x6361,
	/* Range 0x9265 - 0x9269, array index: 0x0DE8 */
	0x6368,INVALC,0x6369,0x636A,0x636B,
	/* Range 0x926E - 0x9270, array index: 0x0DED */
	0x636C,0x636D,0x636E,
	/* Range 0x9275 - 0x927F, array index: 0x0DF0 */
	0x636F,0x6370,0x6371,0x6372,0x6373,INVALC,0x6374,0x6375,
	0x6376,INVALC,0x6377,
	/* Range 0x9288 - 0x928E, array index: 0x0DFB */
	0x6378,0x6379,0x637A,INVALC,INVALC,0x637B,0x637C,
	/* Range 0x92A4 - 0x92A8, array index: 0x0E02 */
	0x6424,0x6425,INVALC,0x6426,0x6427,
	/* Range 0x92B8 - 0x92D0, array index: 0x0E07 */
	0x642C,INVALC,0x642D,0x642E,0x642F,0x6430,INVALC,0x6431,
	0x6432,0x6433,0x6434,0x6435,INVALC,0x6436,0x6437,0x6438,
	0x6439,INVALC,INVALC,0x643A,0x643B,0x643C,0x643D,INVALC,
	0x643E,
	/* Range 0x92D5 - 0x92E8, array index: 0x0E20 */
	0x6440,INVALC,0x6441,0x6442,0x6443,INVALC,INVALC,0x6444,
	0x6445,INVALC,0x6446,0x6447,0x6448,INVALC,0x6449,INVALC,
	0x644A,INVALC,0x644B,0x644C,
	/* Range 0x931C - 0x932A, array index: 0x0E34 */
	0x645A,0x645B,0x645C,0x645D,INVALC,0x645E,INVALC,INVALC,
	0x645F,0x6460,INVALC,0x6461,INVALC,0x6462,0x6463,
	/* Range 0x9333 - 0x9337, array index: 0x0E43 */
	0x6464,0x6465,INVALC,0x6466,0x6467,
	/* Range 0x9347 - 0x9349, array index: 0x0E48 */
	0x6468,0x6469,0x646A,
	/* Range 0x9350 - 0x9352, array index: 0x0E4B */
	0x646B,0x646C,0x646D,
	/* Range 0x936D - 0x9376, array index: 0x0E4E */
	0x6478,INVALC,0x6479,0x647A,0x647B,INVALC,0x647C,0x647D,
	INVALC,0x647E,
	/* Range 0x937D - 0x9382, array index: 0x0E58 */
	0x6522,INVALC,0x6523,0x6524,0x6525,0x6526,
	/* Range 0x93B4 - 0x93B6, array index: 0x0E5E */
	0x6537,0x6538,0x6539,
	/* Range 0x93C4 - 0x93CD, array index: 0x0E61 */
	0x653D,0x653E,0x653F,0x6540,INVALC,0x6541,0x6542,0x6543,
	0x6544,0x6545,
	/* Range 0x93F7 - 0x93FD, array index: 0x0E6B */
	0x654F,0x6550,0x654E,0x6551,0x6552,INVALC,0x6553,
	/* Range 0x940D - 0x940F, array index: 0x0E72 */
	0x6559,0x655A,0x655B,
	/* Range 0x9415 - 0x9417, array index: 0x0E75 */
	0x655C,0x655D,0x655E,
	/* Range 0x942E - 0x9434, array index: 0x0E78 */
	0x6560,0x6561,INVALC,0x6562,0x6563,0x6564,0x6565,
	/* Range 0x946B - 0x9472, array index: 0x0E7F */
	0x6575,INVALC,0x6576,0x6577,0x6578,INVALC,0x6579,0x657A,
	/* Range 0x958C - 0x958E, array index: 0x0E87 */
	0x6624,0x6625,0x6626,
	/* Range 0x959D - 0x95A1, array index: 0x0E8A */
	0x6627,0x6628,0x6629,INVALC,0x662A,
	/* Range 0x95D0 - 0x95D3, array index: 0x0E8F */
	0x6638,0x6639,0x663A,0x663B,
	/* Range 0x95D9 - 0x95E0, array index: 0x0E93 */
	0x663C,0x663D,INVALC,INVALC,0x663E,0x663F,0x6640,0x6641,
	/* Range 0x9622 - 0x9626, array index: 0x0E9B */
	0x6646,INVALC,0x6647,0x6648,0x6649,
	/* Range 0x9637 - 0x963D, array index: 0x0EA0 */
	0x664D,0x664E,0x664F,0x6650,INVALC,0x6651,0x6652,
	/* Range 0x9654 - 0x9658, array index: 0x0EA7 */
	0x6655,INVALC,0x6656,0x6657,0x6658,
	/* Range 0x967B - 0x9684, array index: 0x0EAC */
	0x665C,0x665D,INVALC,0x665E,0x665F,INVALC,0x6660,0x6661,
	0x6662,0x6663,
	/* Range 0x96A4 - 0x96A6, array index: 0x0EB6 */
	0x666A,0x666B,0x666C,
	/* Range 0x96DD - 0x96DF, array index: 0x0EB9 */
	0x6677,0x6678,0x6679,
	/* Range 0x9721 - 0x9723, array index: 0x0EBC */
	0x6727,0x6728,0x6729,
	/* Range 0x9757 - 0x975B, array index: 0x0EBF */
	0x6733,0x6734,INVALC,0x6735,0x6736,
	/* Range 0x9776 - 0x9778, array index: 0x0EC4 */
	0x673C,0x673D,0x673E,
	/* Range 0x9795 - 0x979A, array index: 0x0EC7 */
	0x6744,0x6745,0x6746,INVALC,0x6747,0x6748,
	/* Range 0x97B5 - 0x97D1, array index: 0x0ECD */
	0x6750,0x6751,INVALC,0x6752,0x6753,0x6754,INVALC,0x6755,
	INVALC,0x6756,0x6757,INVALC,0x6758,INVALC,INVALC,0x6759,
	0x675A,INVALC,0x675B,INVALC,0x675C,0x675D,INVALC,0x675E,
	0x675F,0x6760,INVALC,0x6761,0x6762,
	/* Range 0x97D7 - 0x97E1, array index: 0x0EEA */
	0x6764,0x6765,0x6766,INVALC,0x676A,INVALC,0x6767,0x6768,
	INVALC,0x6769,0x676B,
	/* Range 0x982E - 0x9835, array index: 0x0EF5 */
	0x6822,0x6823,0x6824,INVALC,0x6825,0x6826,INVALC,0x6827,
	/* Range 0x9851 - 0x985A, array index: 0x0EFD */
	0x682D,0x682E,0x682F,INVALC,INVALC,0x6830,0x6831,INVALC,
	0x6832,0x6833,
	/* Range 0x9862 - 0x9866, array index: 0x0F07 */
	0x6834,0x6835,INVALC,0x6836,0x6837,
	/* Range 0x98B7 - 0x98BB, array index: 0x0F0C */
	0x683F,0x6840,INVALC,0x6841,0x6842,
	/* Range 0x98E3 - 0x98E7, array index: 0x0F11 */
	0x6849,INVALC,0x684A,0x684B,0x684C,
	/* Range 0x9915 - 0x991C, array index: 0x0F16 */
	0x6854,0x6855,0x6856,INVALC,INVALC,0x6857,0x6858,0x6859,
	/* Range 0x9931 - 0x993C, array index: 0x0F1E */
	0x685F,0x6860,0x6861,0x6862,0x6863,INVALC,INVALC,INVALC,
	0x6864,0x6865,0x6866,0x6867,
	/* Range 0x9946 - 0x9948, array index: 0x0F2A */
	0x686A,0x686B,0x686C,
	/* Range 0x9958 - 0x9960, array index: 0x0F2D */
	0x6870,0x6871,INVALC,0x6872,0x6873,INVALC,0x6874,0x6875,
	0x6876,
	/* Range 0x99B0 - 0x99B2, array index: 0x0F36 */
	0x687B,0x687C,0x687D,
	/* Range 0x99EA - 0x99EC, array index: 0x0F39 */
	0x692E,0x692F,0x6930,
	/* Range 0x9A02 - 0x9A04, array index: 0x0F3C */
	0x6937,0x6938,0x6939,
	/* Range 0x9A20 - 0x9A24, array index: 0x0F3F */
	0x6940,INVALC,0x6941,0x6942,0x6943,
	/* Range 0x9A4A - 0x9A4E, array index: 0x0F44 */
	0x694E,0x694F,0x6950,INVALC,0x6951,
	/* Range 0x9AB2 - 0x9AB6, array index: 0x0F49 */
	0x695A,INVALC,0x695B,0x695C,0x695D,
	/* Range 0x9AD5 - 0x9AD7, array index: 0x0F4E */
	0x6969,0x696A,0x696B,
	/* Range 0x9AFD - 0x9B12, array index: 0x0F51 */
	0x6979,INVALC,0x697A,0x697B,0x697C,0x697D,0x697E,0x6A21,
	0x6A22,INVALC,INVALC,0x6A23,0x6A24,INVALC,0x6A25,0x6A26,
	0x6A27,0x6A28,INVALC,0x6A29,INVALC,0x6A2A,
	/* Range 0x9B33 - 0x9B3A, array index: 0x0F67 */
	0x6A33,0x6A34,0x6A35,INVALC,0x6A36,INVALC,0x6A37,0x6A38,
	/* Range 0x9B55 - 0x9B57, array index: 0x0F6F */
	0x6A3D,0x6A3E,0x6A3F,
	/* Range 0x9B68 - 0x9B6E, array index: 0x0F72 */
	0x6A46,INVALC,0x6A47,0x6A48,0x6A49,0x6A4A,0x6A4B,
	/* Range 0x9B75 - 0x9B79, array index: 0x0F79 */
	0x6A4D,INVALC,0x6A4E,0x6A4F,0x6A50,
	/* Range 0x9B84 - 0x9B90, array index: 0x0F7E */
	0x6A53,0x6A54,0x6A55,0x6A56,INVALC,0x6A57,0x6A58,0x6A59,
	INVALC,0x6A5A,INVALC,0x6A5B,0x6A5C,
	/* Range 0x9BB0 - 0x9BB2, array index: 0x0F8B */
	0x6A65,0x6A66,0x6A67,
	/* Range 0x9BBB - 0x9BC1, array index: 0x0F8E */
	0x6A6A,0x6A6B,INVALC,0x6A6C,0x6A6D,INVALC,0x6A6E,
	/* Range 0x9BF7 - 0x9BFA, array index: 0x0F95 */
	0x6A7D,0x6A7E,0x6B21,0x6B22,
	/* Range 0x9C16 - 0x9C1E, array index: 0x0F99 */
	0x6B2A,INVALC,0x6B2B,0x6B2C,0x6B2D,INVALC,0x6B2E,INVALC,
	0x6B2F,
	/* Range 0x9C22 - 0x9C2A, array index: 0x0FA2 */
	0x6B30,0x6B31,INVALC,INVALC,0x6B32,0x6B33,0x6B34,0x6B35,
	0x6B36,
	/* Range 0x9C35 - 0x9C37, array index: 0x0FAB */
	0x6B38,0x6B39,0x6B3A,
	/* Range 0x9C41 - 0x9C45, array index: 0x0FAE */
	0x6B3C,INVALC,0x6B3D,0x6B3E,0x6B3F,
	/* Range 0x9C4E - 0x9C5F, array index: 0x0FB3 */
	0x6B42,0x6B43,0x6B44,INVALC,INVALC,0x6B45,0x6B46,INVALC,
	0x6B47,INVALC,0x6B48,INVALC,INVALC,0x6B49,0x6B50,0x6B4A,
	0x6B4B,0x6B4C,
	/* Range 0x9C68 - 0x9C6B, array index: 0x0FC5 */
	0x6B52,0x6B4E,0x6B4F,0x6B51,
	/* Range 0x9D1C - 0x9D1E, array index: 0x0FC9 */
	0x6B62,0x6B63,0x6B64,
	/* Range 0x9D2F - 0x9D34, array index: 0x0FCC */
	0x6B65,0x6B66,INVALC,0x6B67,0x6B68,0x6B69,
	/* Range 0x9D69 - 0x9D6B, array index: 0x0FD2 */
	0x6B78,0x6B79,0x6B7A,
	/* Range 0x9D92 - 0x9D98, array index: 0x0FD5 */
	0x6C29,0x6C2A,INVALC,0x6C2B,0x6C2C,0x6C2D,0x6C2E,
	/* Range 0x9DD4 - 0x9DD7, array index: 0x0FDC */
	0x6C3C,0x6C3D,0x6C3E,0x6C3F,
	/* Range 0x9DDE - 0x9DE0, array index: 0x0FE0 */
	0x6C41,0x6C42,0x6C43,
	/* Range 0x9E0E - 0x9E16, array index: 0x0FE3 */
	0x6C50,INVALC,0x6C51,0x6C52,0x6C53,INVALC,INVALC,0x6C54,
	0x6C55,
	/* Range 0x9E7A - 0x9E7C, array index: 0x0FEC */
	0x6C59,0x6C5A,0x6C5B,
	/* Range 0x9E80 - 0x9E87, array index: 0x0FEF */
	0x6C5C,INVALC,0x6C5D,0x6C5E,0x6C5F,0x6C60,INVALC,0x6C61,
	/* Range 0x9EAC - 0x9EB5, array index: 0x0FF7 */
	0x6C6A,INVALC,0x6C6B,0x6C6C,0x6C6D,INVALC,INVALC,0x6C6E,
	0x6C6F,0x6C70,
	/* Range 0x9EEC - 0x9EF2, array index: 0x1001 */
	0x6C78,0x6C79,0x6C7A,INVALC,0x6C7B,0x6C7C,0x6C7D,
	/* Range 0x9F0F - 0x9F1B, array index: 0x1008 */
	0x6D26,0x6D27,0x6D28,0x6D29,INVALC,0x6D2A,INVALC,0x6D2B,
	0x6D2C,INVALC,0x6D2D,0x6D2E,0x6D2F,
	/* Range 0x9F39 - 0x9F47, array index: 0x1015 */
	0x6D3A,0x6D3B,INVALC,0x6D3C,0x6D3D,INVALC,0x6D3E,INVALC,
	0x6D3F,INVALC,0x6D40,0x6D41,0x6D42,0x6D43,0x6D44,
	/* Range 0x9F53 - 0x9F5E, array index: 0x1024 */
	0x6D45,INVALC,0x6D46,0x6D47,0x6D48,0x6D49,INVALC,0x6D4A,
	INVALC,INVALC,0x6D4B,0x6D4C,
	/* Range 0x9F6D - 0x9F75, array index: 0x1030 */
	0x6D4F,0x6D50,0x6D51,0x6D52,0x6D53,INVALC,0x6D54,INVALC,
	0x6D55,
	/* Range 0x9F8F - 0x9F97, array index: 0x1039 */
	0x6D58,0x6D59,0x6D5A,0x6D5B,INVALC,0x6D5C,INVALC,0x6D5D,
	0x6D5E,
	/* Range 0x9FA1 - 0x9FA5, array index: 0x1042 */
	0x6D60,0x6D61,0x6D62,INVALC,0x6D63,
	/* Unranged codes (426 codes) */
	/* Array index: 0x1047 */ 0x007E,0x2237,
	/* Array index: 0x1047 */ 0x00A1,0x2242,
	/* Array index: 0x1047 */ 0x00A4,0x2270,
	/* Array index: 0x1047 */ 0x00A6,0x2243,
	/* Array index: 0x1047 */ 0x00A9,0x226D,
	/* Array index: 0x1047 */ 0x00AA,0x226C,
	/* Array index: 0x1047 */ 0x00AE,0x226E,
	/* Array index: 0x1047 */ 0x00AF,0x2234,
	/* Array index: 0x1047 */ 0x00B8,0x2231,
	/* Array index: 0x1047 */ 0x00BA,0x226B,
	/* Array index: 0x1047 */ 0x01F5,0x2B39,
	/* Array index: 0x1047 */ 0x02C7,0x2230,
	/* Array index: 0x1047 */ 0x03C2,0x2678,
	/* Array index: 0x1047 */ 0x2116,0x2271,
	/* Array index: 0x1047 */ 0x2122,0x226F,
	/* Array index: 0x1047 */ 0x4E02,0x3021,
	/* Array index: 0x1047 */ 0x4E04,0x3022,
	/* Array index: 0x1047 */ 0x4E05,0x3023,
	/* Array index: 0x1047 */ 0x4E0C,0x3024,
	/* Array index: 0x1047 */ 0x4E12,0x3025,
	/* Array index: 0x1047 */ 0x4E1F,0x3026,
	/* Array index: 0x1047 */ 0x4E23,0x3027,
	/* Array index: 0x1047 */ 0x4E24,0x3028,
	/* Array index: 0x1047 */ 0x4E28,0x3029,
	/* Array index: 0x1047 */ 0x4E2B,0x302A,
	/* Array index: 0x1047 */ 0x4E35,0x302E,
	/* Array index: 0x1047 */ 0x4E40,0x302F,
	/* Array index: 0x1047 */ 0x4E41,0x3030,
	/* Array index: 0x1047 */ 0x4E44,0x3031,
	/* Array index: 0x1047 */ 0x4E47,0x3032,
	/* Array index: 0x1047 */ 0x4E51,0x3033,
	/* Array index: 0x1047 */ 0x4E5A,0x3034,
	/* Array index: 0x1047 */ 0x4E5C,0x3035,
	/* Array index: 0x1047 */ 0x4E63,0x3036,
	/* Array index: 0x1047 */ 0x4E68,0x3037,
	/* Array index: 0x1047 */ 0x4E69,0x3038,
	/* Array index: 0x1047 */ 0x4E74,0x3039,
	/* Array index: 0x1047 */ 0x4E75,0x303A,
	/* Array index: 0x1047 */ 0x4E79,0x303B,
	/* Array index: 0x1047 */ 0x4E7F,0x303C,
	/* Array index: 0x1047 */ 0x4E8D,0x303D,
	/* Array index: 0x1047 */ 0x4E96,0x303E,
	/* Array index: 0x1047 */ 0x4E97,0x303F,
	/* Array index: 0x1047 */ 0x4E9D,0x3040,
	/* Array index: 0x1047 */ 0x4EAF,0x3041,
	/* Array index: 0x1047 */ 0x4EB9,0x3042,
	/* Array index: 0x1047 */ 0x4EC3,0x3043,
	/* Array index: 0x1047 */ 0x4ED0,0x3044,
	/* Array index: 0x1047 */ 0x4EDA,0x3045,
	/* Array index: 0x1047 */ 0x4EDB,0x3046,
	/* Array index: 0x1047 */ 0x4EE8,0x304A,
	/* Array index: 0x1047 */ 0x4EEF,0x304B,
	/* Array index: 0x1047 */ 0x4EF1,0x304C,
	/* Array index: 0x1047 */ 0x4EF3,0x304D,
	/* Array index: 0x1047 */ 0x4EF5,0x304E,
	/* Array index: 0x1047 */ 0x4F08,0x3055,
	/* Array index: 0x1047 */ 0x4F0B,0x3056,
	/* Array index: 0x1047 */ 0x4F0C,0x3057,
	/* Array index: 0x1047 */ 0x4F12,0x3058,
	/* Array index: 0x1047 */ 0x4F2E,0x305D,
	/* Array index: 0x1047 */ 0x4F31,0x305E,
	/* Array index: 0x1047 */ 0x4F33,0x3060,
	/* Array index: 0x1047 */ 0x4F35,0x3061,
	/* Array index: 0x1047 */ 0x4F37,0x3062,
	/* Array index: 0x1047 */ 0x4F39,0x3063,
	/* Array index: 0x1047 */ 0x4F3B,0x3064,
	/* Array index: 0x1047 */ 0x4F3E,0x3065,
	/* Array index: 0x1047 */ 0x4F40,0x3066,
	/* Array index: 0x1047 */ 0x4F42,0x3067,
	/* Array index: 0x1047 */ 0x4F52,0x306C,
	/* Array index: 0x1047 */ 0x4F54,0x306D,
	/* Array index: 0x1047 */ 0x4F56,0x306E,
	/* Array index: 0x1047 */ 0x4F58,0x306F,
	/* Array index: 0x1047 */ 0x4F5F,0x3070,
	/* Array index: 0x1047 */ 0x4F60,0x305F,
	/* Array index: 0x1047 */ 0x4F63,0x3071,
	/* Array index: 0x1047 */ 0x4F6A,0x3072,
	/* Array index: 0x1047 */ 0x4F6C,0x3073,
	/* Array index: 0x1047 */ 0x4F6E,0x3074,
	/* Array index: 0x1047 */ 0x4F71,0x3075,
	/* Array index: 0x1047 */ 0x4F89,0x3122,
	/* Array index: 0x1047 */ 0x4F8A,0x3123,
	/* Array index: 0x1047 */ 0x4F8C,0x3124,
	/* Array index: 0x1047 */ 0x4F8E,0x3125,
	/* Array index: 0x1047 */ 0x4F97,0x312A,
	/* Array index: 0x1047 */ 0x4F99,0x312B,
	/* Array index: 0x1047 */ 0x4F9A,0x312C,
	/* Array index: 0x1047 */ 0x4F9E,0x312D,
	/* Array index: 0x1047 */ 0x4F9F,0x312E,
	/* Array index: 0x1047 */ 0x4FB2,0x312F,
	/* Array index: 0x1047 */ 0x4FB7,0x3130,
	/* Array index: 0x1047 */ 0x4FD2,0x3140,
	/* Array index: 0x1047 */ 0x4FDC,0x3141,
	/* Array index: 0x1047 */ 0x4FE0,0x3142,
	/* Array index: 0x1047 */ 0x4FE2,0x3143,
	/* Array index: 0x1047 */ 0x4FF0,0x3144,
	/* Array index: 0x1047 */ 0x4FF2,0x3145,
	/* Array index: 0x1047 */ 0x5004,0x314B,
	/* Array index: 0x1047 */ 0x5007,0x314C,
	/* Array index: 0x1047 */ 0x500A,0x314D,
	/* Array index: 0x1047 */ 0x500C,0x314E,
	/* Array index: 0x1047 */ 0x500E,0x314F,
	/* Array index: 0x1047 */ 0x5010,0x3150,
	/* Array index: 0x1047 */ 0x5013,0x3151,
	/* Array index: 0x1047 */ 0x5022,0x3158,
	/* Array index: 0x1047 */ 0x5027,0x3159,
	/* Array index: 0x1047 */ 0x502E,0x315A,
	/* Array index: 0x1047 */ 0x5030,0x315B,
	/* Array index: 0x1047 */ 0x5032,0x315C,
	/* Array index: 0x1047 */ 0x5033,0x315D,
	/* Array index: 0x1047 */ 0x5035,0x315E,
	/* Array index: 0x1047 */ 0x503B,0x3176,
	/* Array index: 0x1047 */ 0x504A,0x3164,
	/* Array index: 0x1047 */ 0x504C,0x3165,
	/* Array index: 0x1047 */ 0x504E,0x3166,
	/* Array index: 0x1047 */ 0x5057,0x316A,
	/* Array index: 0x1047 */ 0x5059,0x316B,
	/* Array index: 0x1047 */ 0x506A,0x3172,
	/* Array index: 0x1047 */ 0x506D,0x3173,
	/* Array index: 0x1047 */ 0x5070,0x3174,
	/* Array index: 0x1047 */ 0x5071,0x3175,
	/* Array index: 0x1047 */ 0x5081,0x3177,
	/* Array index: 0x1047 */ 0x5083,0x3178,
	/* Array index: 0x1047 */ 0x5084,0x3179,
	/* Array index: 0x1047 */ 0x5086,0x317A,
	/* Array index: 0x1047 */ 0x508A,0x317B,
	/* Array index: 0x1047 */ 0x50AA,0x322C,
	/* Array index: 0x1047 */ 0x50AF,0x322D,
	/* Array index: 0x1047 */ 0x50B0,0x322E,
	/* Array index: 0x1047 */ 0x50B9,0x322F,
	/* Array index: 0x1047 */ 0x50BA,0x3230,
	/* Array index: 0x1047 */ 0x50BD,0x3231,
	/* Array index: 0x1047 */ 0x50C0,0x3232,
	/* Array index: 0x1047 */ 0x50C3,0x3233,
	/* Array index: 0x1047 */ 0x50C4,0x3234,
	/* Array index: 0x1047 */ 0x50C7,0x3235,
	/* Array index: 0x1047 */ 0x50CC,0x3236,
	/* Array index: 0x1047 */ 0x50CE,0x3237,
	/* Array index: 0x1047 */ 0x50D0,0x3238,
	/* Array index: 0x1047 */ 0x50D3,0x3239,
	/* Array index: 0x1047 */ 0x50D4,0x323A,
	/* Array index: 0x1047 */ 0x50D8,0x323B,
	/* Array index: 0x1047 */ 0x50DC,0x323C,
	/* Array index: 0x1047 */ 0x50DD,0x323D,
	/* Array index: 0x1047 */ 0x50DF,0x323E,
	/* Array index: 0x1047 */ 0x50E2,0x323F,
	/* Array index: 0x1047 */ 0x50E4,0x3240,
	/* Array index: 0x1047 */ 0x50E6,0x3241,
	/* Array index: 0x1047 */ 0x50E8,0x3242,
	/* Array index: 0x1047 */ 0x50E9,0x3243,
	/* Array index: 0x1047 */ 0x50EF,0x3244,
	/* Array index: 0x1047 */ 0x50F1,0x3245,
	/* Array index: 0x1047 */ 0x50F2,0x3251,
	/* Array index: 0x1047 */ 0x50F6,0x3246,
	/* Array index: 0x1047 */ 0x50FA,0x3247,
	/* Array index: 0x1047 */ 0x50FE,0x3248,
	/* Array index: 0x1047 */ 0x5103,0x3249,
	/* Array index: 0x1047 */ 0x5117,0x3253,
	/* Array index: 0x1047 */ 0x5123,0x3259,
	/* Array index: 0x1047 */ 0x5127,0x325A,
	/* Array index: 0x1047 */ 0x5128,0x325B,
	/* Array index: 0x1047 */ 0x512C,0x325C,
	/* Array index: 0x1047 */ 0x512D,0x325D,
	/* Array index: 0x1047 */ 0x512F,0x325E,
	/* Array index: 0x1047 */ 0x5142,0x3265,
	/* Array index: 0x1047 */ 0x514A,0x3266,
	/* Array index: 0x1047 */ 0x514F,0x3267,
	/* Array index: 0x1047 */ 0x5153,0x3268,
	/* Array index: 0x1047 */ 0x5155,0x3269,
	/* Array index: 0x1047 */ 0x5157,0x326A,
	/* Array index: 0x1047 */ 0x5158,0x326B,
	/* Array index: 0x1047 */ 0x515F,0x326C,
	/* Array index: 0x1047 */ 0x5164,0x326D,
	/* Array index: 0x1047 */ 0x5166,0x326E,
	/* Array index: 0x1047 */ 0x517E,0x326F,
	/* Array index: 0x1047 */ 0x5183,0x3270,
	/* Array index: 0x1047 */ 0x5184,0x3271,
	/* Array index: 0x1047 */ 0x518B,0x3272,
	/* Array index: 0x1047 */ 0x518E,0x3273,
	/* Array index: 0x1047 */ 0x5198,0x3274,
	/* Array index: 0x1047 */ 0x519D,0x3275,
	/* Array index: 0x1047 */ 0x51A1,0x3276,
	/* Array index: 0x1047 */ 0x51A3,0x3277,
	/* Array index: 0x1047 */ 0x51AD,0x3278,
	/* Array index: 0x1047 */ 0x51B8,0x3279,
	/* Array index: 0x1047 */ 0x51BA,0x327A,
	/* Array index: 0x1047 */ 0x51BC,0x327B,
	/* Array index: 0x1047 */ 0x51BE,0x327C,
	/* Array index: 0x1047 */ 0x51BF,0x327D,
	/* Array index: 0x1047 */ 0x51C2,0x327E,
	/* Array index: 0x1047 */ 0x51C8,0x3321,
	/* Array index: 0x1047 */ 0x51D8,0x3327,
	/* Array index: 0x1047 */ 0x51DE,0x3328,
	/* Array index: 0x1047 */ 0x51E2,0x3329,
	/* Array index: 0x1047 */ 0x51E5,0x332A,
	/* Array index: 0x1047 */ 0x51EE,0x332B,
	/* Array index: 0x1047 */ 0x51F7,0x332F,
	/* Array index: 0x1047 */ 0x5201,0x3330,
	/* Array index: 0x1047 */ 0x5202,0x3331,
	/* Array index: 0x1047 */ 0x5205,0x3332,
	/* Array index: 0x1047 */ 0x5222,0x3338,
	/* Array index: 0x1047 */ 0x5228,0x3339,
	/* Array index: 0x1047 */ 0x5231,0x333A,
	/* Array index: 0x1047 */ 0x5232,0x333B,
	/* Array index: 0x1047 */ 0x5235,0x333C,
	/* Array index: 0x1047 */ 0x523C,0x333D,
	/* Array index: 0x1047 */ 0x5245,0x333E,
	/* Array index: 0x1047 */ 0x5249,0x333F,
	/* Array index: 0x1047 */ 0x5255,0x3340,
	/* Array index: 0x1047 */ 0x5257,0x3341,
	/* Array index: 0x1047 */ 0x5258,0x3342,
	/* Array index: 0x1047 */ 0x525A,0x3343,
	/* Array index: 0x1047 */ 0x525C,0x3344,
	/* Array index: 0x1047 */ 0x5266,0x3348,
	/* Array index: 0x1047 */ 0x526E,0x3349,
	/* Array index: 0x1047 */ 0x5280,0x334D,
	/* Array index: 0x1047 */ 0x5282,0x334E,
	/* Array index: 0x1047 */ 0x5285,0x334F,
	/* Array index: 0x1047 */ 0x528A,0x3350,
	/* Array index: 0x1047 */ 0x528C,0x3351,
	/* Array index: 0x1047 */ 0x52AF,0x335D,
	/* Array index: 0x1047 */ 0x52B0,0x335E,
	/* Array index: 0x1047 */ 0x52C0,0x3365,
	/* Array index: 0x1047 */ 0x52C4,0x3366,
	/* Array index: 0x1047 */ 0x52C6,0x3367,
	/* Array index: 0x1047 */ 0x52C8,0x3368,
	/* Array index: 0x1047 */ 0x52CC,0x3369,
	/* Array index: 0x1047 */ 0x52CF,0x336A,
	/* Array index: 0x1047 */ 0x52D1,0x336B,
	/* Array index: 0x1047 */ 0x52D4,0x336C,
	/* Array index: 0x1047 */ 0x52D6,0x336D,
	/* Array index: 0x1047 */ 0x52DB,0x336E,
	/* Array index: 0x1047 */ 0x52DC,0x336F,
	/* Array index: 0x1047 */ 0x52E1,0x3370,
	/* Array index: 0x1047 */ 0x52E5,0x3371,
	/* Array index: 0x1047 */ 0x52F0,0x3376,
	/* Array index: 0x1047 */ 0x52F1,0x3377,
	/* Array index: 0x1047 */ 0x52F4,0x3378,
	/* Array index: 0x1047 */ 0x52F6,0x3379,
	/* Array index: 0x1047 */ 0x52F7,0x337A,
	/* Array index: 0x1047 */ 0x5300,0x337B,
	/* Array index: 0x1047 */ 0x5303,0x337C,
	/* Array index: 0x1047 */ 0x5311,0x3422,
	/* Array index: 0x1047 */ 0x5313,0x3423,
	/* Array index: 0x1047 */ 0x5318,0x3424,
	/* Array index: 0x1047 */ 0x5330,0x3430,
	/* Array index: 0x1047 */ 0x5332,0x3431,
	/* Array index: 0x1047 */ 0x5335,0x3432,
	/* Array index: 0x1047 */ 0x5342,0x3436,
	/* Array index: 0x1047 */ 0x534B,0x3438,
	/* Array index: 0x1047 */ 0x534C,0x3437,
	/* Array index: 0x1047 */ 0x5359,0x3439,
	/* Array index: 0x1047 */ 0x535B,0x343A,
	/* Array index: 0x1047 */ 0x5361,0x343B,
	/* Array index: 0x1047 */ 0x5363,0x343C,
	/* Array index: 0x1047 */ 0x5365,0x343D,
	/* Array index: 0x1047 */ 0x536C,0x343E,
	/* Array index: 0x1047 */ 0x536D,0x343F,
	/* Array index: 0x1047 */ 0x5372,0x3440,
	/* Array index: 0x1047 */ 0x5379,0x3441,
	/* Array index: 0x1047 */ 0x537E,0x3442,
	/* Array index: 0x1047 */ 0x5383,0x3443,
	/* Array index: 0x1047 */ 0x5387,0x3444,
	/* Array index: 0x1047 */ 0x5388,0x3445,
	/* Array index: 0x1047 */ 0x538E,0x3446,
	/* Array index: 0x1047 */ 0x5393,0x3447,
	/* Array index: 0x1047 */ 0x5394,0x3448,
	/* Array index: 0x1047 */ 0x5399,0x3449,
	/* Array index: 0x1047 */ 0x539D,0x344A,
	/* Array index: 0x1047 */ 0x53A1,0x344B,
	/* Array index: 0x1047 */ 0x53A4,0x344C,
	/* Array index: 0x1047 */ 0x53AA,0x344D,
	/* Array index: 0x1047 */ 0x53AB,0x344E,
	/* Array index: 0x1047 */ 0x53AF,0x344F,
	/* Array index: 0x1047 */ 0x53B2,0x3450,
	/* Array index: 0x1047 */ 0x53BD,0x3456,
	/* Array index: 0x1047 */ 0x53C0,0x3457,
	/* Array index: 0x1047 */ 0x53C5,0x3458,
	/* Array index: 0x1047 */ 0x53CF,0x3459,
	/* Array index: 0x1047 */ 0x53D2,0x345A,
	/* Array index: 0x1047 */ 0x53D3,0x345B,
	/* Array index: 0x1047 */ 0x53D5,0x345C,
	/* Array index: 0x1047 */ 0x53DA,0x345D,
	/* Array index: 0x1047 */ 0x53DD,0x345E,
	/* Array index: 0x1047 */ 0x53DE,0x345F,
	/* Array index: 0x1047 */ 0x53E0,0x3460,
	/* Array index: 0x1047 */ 0x53E6,0x3461,
	/* Array index: 0x1047 */ 0x53E7,0x3462,
	/* Array index: 0x1047 */ 0x53F5,0x3463,
	/* Array index: 0x1047 */ 0x5402,0x3464,
	/* Array index: 0x1047 */ 0x5413,0x3465,
	/* Array index: 0x1047 */ 0x541A,0x3466,
	/* Array index: 0x1047 */ 0x5421,0x3467,
	/* Array index: 0x1047 */ 0x5427,0x3468,
	/* Array index: 0x1047 */ 0x5428,0x3469,
	/* Array index: 0x1047 */ 0x542A,0x346A,
	/* Array index: 0x1047 */ 0x542F,0x346B,
	/* Array index: 0x1047 */ 0x5431,0x346C,
	/* Array index: 0x1047 */ 0x5434,0x346D,
	/* Array index: 0x1047 */ 0x5435,0x346E,
	/* Array index: 0x1047 */ 0x5443,0x346F,
	/* Array index: 0x1047 */ 0x5444,0x3470,
	/* Array index: 0x1047 */ 0x5447,0x3471,
	/* Array index: 0x1047 */ 0x544D,0x3472,
	/* Array index: 0x1047 */ 0x544F,0x3473,
	/* Array index: 0x1047 */ 0x545E,0x3474,
	/* Array index: 0x1047 */ 0x5462,0x3475,
	/* Array index: 0x1047 */ 0x5464,0x3476,
	/* Array index: 0x1047 */ 0x5466,0x3477,
	/* Array index: 0x1047 */ 0x5467,0x3478,
	/* Array index: 0x1047 */ 0x5469,0x3479,
	/* Array index: 0x1047 */ 0x546B,0x347A,
	/* Array index: 0x1047 */ 0x546D,0x347B,
	/* Array index: 0x1047 */ 0x546E,0x347C,
	/* Array index: 0x1047 */ 0x5474,0x347D,
	/* Array index: 0x1047 */ 0x547F,0x347E,
	/* Array index: 0x1047 */ 0x5481,0x3521,
	/* Array index: 0x1047 */ 0x5483,0x3522,
	/* Array index: 0x1047 */ 0x5485,0x3523,
	/* Array index: 0x1047 */ 0x5488,0x3524,
	/* Array index: 0x1047 */ 0x5489,0x3525,
	/* Array index: 0x1047 */ 0x548D,0x3526,
	/* Array index: 0x1047 */ 0x5491,0x3527,
	/* Array index: 0x1047 */ 0x5495,0x3528,
	/* Array index: 0x1047 */ 0x5496,0x3529,
	/* Array index: 0x1047 */ 0x549C,0x352A,
	/* Array index: 0x1047 */ 0x549F,0x352B,
	/* Array index: 0x1047 */ 0x54A1,0x352C,
	/* Array index: 0x1047 */ 0x54B1,0x3533,
	/* Array index: 0x1047 */ 0x54BF,0x3538,
	/* Array index: 0x1047 */ 0x54C6,0x3539,
	/* Array index: 0x1047 */ 0x54CA,0x353A,
	/* Array index: 0x1047 */ 0x54CD,0x353B,
	/* Array index: 0x1047 */ 0x54CE,0x353C,
	/* Array index: 0x1047 */ 0x54E0,0x353D,
	/* Array index: 0x1047 */ 0x54EA,0x353E,
	/* Array index: 0x1047 */ 0x54EC,0x353F,
	/* Array index: 0x1047 */ 0x54EF,0x3540,
	/* Array index: 0x1047 */ 0x54F6,0x3541,
	/* Array index: 0x1047 */ 0x5505,0x3547,
	/* Array index: 0x1047 */ 0x5515,0x354D,
	/* Array index: 0x1047 */ 0x552A,0x354E,
	/* Array index: 0x1047 */ 0x552B,0x354F,
	/* Array index: 0x1047 */ 0x5532,0x3550,
	/* Array index: 0x1047 */ 0x5535,0x3551,
	/* Array index: 0x1047 */ 0x5536,0x3552,
	/* Array index: 0x1047 */ 0x5541,0x3556,
	/* Array index: 0x1047 */ 0x5547,0x3557,
	/* Array index: 0x1047 */ 0x5549,0x3558,
	/* Array index: 0x1047 */ 0x554A,0x3559,
	/* Array index: 0x1047 */ 0x554D,0x355A,
	/* Array index: 0x1047 */ 0x5550,0x355B,
	/* Array index: 0x1047 */ 0x5551,0x355C,
	/* Array index: 0x1047 */ 0x5558,0x355D,
	/* Array index: 0x1047 */ 0x555A,0x355E,
	/* Array index: 0x1047 */ 0x555B,0x355F,
	/* Array index: 0x1047 */ 0x555E,0x3560,
	/* Array index: 0x1047 */ 0x5560,0x3561,
	/* Array index: 0x1047 */ 0x5561,0x3562,
	/* Array index: 0x1047 */ 0x5564,0x3563,
	/* Array index: 0x1047 */ 0x5566,0x3564,
	/* Array index: 0x1047 */ 0x557F,0x3565,
	/* Array index: 0x1047 */ 0x5581,0x3566,
	/* Array index: 0x1047 */ 0x5582,0x3567,
	/* Array index: 0x1047 */ 0x5586,0x3568,
	/* Array index: 0x1047 */ 0x5588,0x3569,
	/* Array index: 0x1047 */ 0x5597,0x3570,
	/* Array index: 0x1047 */ 0x55A3,0x3571,
	/* Array index: 0x1047 */ 0x55A4,0x3572,
	/* Array index: 0x1047 */ 0x55AD,0x3573,
	/* Array index: 0x1047 */ 0x55B2,0x3574,
	/* Array index: 0x1047 */ 0x55BF,0x3575,
	/* Array index: 0x1047 */ 0x55C1,0x3576,
	/* Array index: 0x1047 */ 0x55C3,0x3577,
	/* Array index: 0x1047 */ 0x55C6,0x3578,
	/* Array index: 0x1047 */ 0x55C9,0x3579,
	/* Array index: 0x1047 */ 0x55CB,0x357A,
	/* Array index: 0x1047 */ 0x55CC,0x357B,
	/* Array index: 0x1047 */ 0x55CE,0x357C,
	/* Array index: 0x1047 */ 0x55D7,0x3622,
	/* Array index: 0x1047 */ 0x55D8,0x3623,
	/* Array index: 0x1047 */ 0x55DB,0x3624,
	/* Array index: 0x1047 */ 0x55DE,0x3625,
	/* Array index: 0x1047 */ 0x55E2,0x3626,
	/* Array index: 0x1047 */ 0x55E9,0x3627,
	/* Array index: 0x1047 */ 0x55F6,0x3628,
	/* Array index: 0x1047 */ 0x55FF,0x3629,
	/* Array index: 0x1047 */ 0x5605,0x362A,
	/* Array index: 0x1047 */ 0x5608,0x362B,
	/* Array index: 0x1047 */ 0x560A,0x362C,
	/* Array index: 0x1047 */ 0x5619,0x3633,
	/* Array index: 0x1047 */ 0x562C,0x3634,
	/* Array index: 0x1047 */ 0x5630,0x3635,
	/* Array index: 0x1047 */ 0x5633,0x3636,
	/* Array index: 0x1047 */ 0x5635,0x3637,
	/* Array index: 0x1047 */ 0x5637,0x3638,
	/* Array index: 0x1047 */ 0x5649,0x3643,
	/* Array index: 0x1047 */ 0x564B,0x3644,
	/* Array index: 0x1047 */ 0x564D,0x3645,
	/* Array index: 0x1047 */ 0x564F,0x3646,
	/* Array index: 0x1047 */ 0x5654,0x3647,
	/* Array index: 0x1047 */ 0x5666,0x364D,
	/* Array index: 0x1047 */ 0x5669,0x364E,
	/* Array index: 0x1047 */ 0x566D,0x364F,
	/* Array index: 0x1047 */ 0x566F,0x3650,
	/* Array index: 0x1047 */ 0x5671,0x3651,
	/* Array index: 0x1047 */ 0x5672,0x3652,
	/* Array index: 0x1047 */ 0x5675,0x3653,
	/* Array index: 0x1047 */ 0x5684,0x3654,
	/* Array index: 0x1047 */ 0x5685,0x3655,
	/* Array index: 0x1047 */ 0x5688,0x3656,
	/* Array index: 0x1047 */ 0x568B,0x3657,
	/* Array index: 0x1047 */ 0x568C,0x3658,
	/* Array index: 0x1047 */ 0x5695,0x3659,
	/* Array index: 0x1047 */ 0x56B1,0x3666,
	/* Array index: 0x1047 */ 0x56B3,0x3667,
	/* Array index: 0x1047 */ 0x56B7,0x3668,
	/* Array index: 0x1047 */ 0x56BE,0x3669,
	/* Array index: 0x1047 */ 0x56C5,0x366A,
	/* Array index: 0x1047 */ 0x56D9,0x3672,
	/* Array index: 0x1047 */ 0x56DC,0x3673,
	/* Array index: 0x1047 */ 0x56DD,0x3674,
	/* Array index: 0x1047 */ 0x56DF,0x3675,
	/* Array index: 0x1047 */ 0x56E1,0x3676,
	/* Array index: 0x1047 */ 0x56EB,0x367D,
	/* Array index: 0x1047 */ 0x56ED,0x367E,
	/* Array index: 0x1047 */ 0x56F1,0x367C,
	/* Array index: 0x1047 */ 0x56F6,0x3721,
	/* Array index: 0x1047 */ 0x56F7,0x3722,
	/* Array index: 0x1047 */ 0x5701,0x3723,
	/* Array index: 0x1047 */ 0x5702,0x3724,
	/* Array index: 0x1047 */ 0x5707,0x3725,
	/* Array index: 0x1047 */ 0x570A,0x3726,
	/* Array index: 0x1047 */ 0x570C,0x3727,
	/* Array index: 0x1047 */ 0x5711,0x3728,
	/* Array index: 0x1047 */ 0x5715,0x3729,
	/* Array index: 0x1047 */ 0x571A,0x372A,
	/* Array index: 0x1047 */ 0x571B,0x372B,
	/* Array index: 0x1047 */ 0x571D,0x372C,
	/* Array index: 0x1047 */ 0x5729,0x3732,
	/* Array index: 0x1047 */ 0x572A,0x3733,
	/* Array index: 0x1047 */ 0x572C,0x3734,
	/* Array index: 0x1047 */ 0x572E,0x3735,
	/* Array index: 0x1047 */ 0x572F,0x3736,
	/* Array index: 0x1047 */ 0x5733,0x3737,
	/* Array index: 0x1047 */ 0x5734,0x3738,
	/* Array index: 0x1047 */ 0x5745,0x373C,
	/* Array index: 0x1047 */ 0x5746,0x373D,
	/* Array index: 0x1047 */ 0x574C,0x373E,
	/* Array index: 0x1047 */ 0x574D,0x373F,
	/* Array index: 0x1047 */ 0x5752,0x3740,
	/* Array index: 0x1047 */ 0x5762,0x3741,
	/* Array index: 0x1047 */ 0x5765,0x3742,
	/* Array index: 0x1047 */ 0x5767,0x3743,
	/* Array index: 0x1047 */ 0x5768,0x3744,
	/* Array index: 0x1047 */ 0x5781,0x3754,
	/* Array index: 0x1047 */ 0x5783,0x3755,
	/* Array index: 0x1047 */ 0x578C,0x3756,
	/* Array index: 0x1047 */ 0x5794,0x3757,
	/* Array index: 0x1047 */ 0x5795,0x3760,
	/* Array index: 0x1047 */ 0x5797,0x3758,
	/* Array index: 0x1047 */ 0x57AC,0x3764,
	/* Array index: 0x1047 */ 0x57B8,0x3765,
	/* Array index: 0x1047 */ 0x57BD,0x3766,
	/* Array index: 0x1047 */ 0x57C7,0x3767,
	/* Array index: 0x1047 */ 0x57C8,0x3768,
	/* Array index: 0x1047 */ 0x57CC,0x3769,
	/* Array index: 0x1047 */ 0x57CF,0x376A,
	/* Array index: 0x1047 */ 0x57D5,0x376B,
	/* Array index: 0x1047 */ 0x57DD,0x376C,
	/* Array index: 0x1047 */ 0x57DE,0x376D,
	/* Array index: 0x1047 */ 0x57E1,0x377E,
	/* Array index: 0x1047 */ 0x57E4,0x376E,
	/* Array index: 0x1047 */ 0x57E6,0x376F,
	/* Array index: 0x1047 */ 0x57E7,0x3770,
	/* Array index: 0x1047 */ 0x57E9,0x3771,
	/* Array index: 0x1047 */ 0x57ED,0x3772,
	/* Array index: 0x1047 */ 0x57F0,0x3773,
	/* Array index: 0x1047 */ 0x57F5,0x3774,
	/* Array index: 0x1047 */ 0x57F6,0x3775,
	/* Array index: 0x1047 */ 0x57F8,0x3776,
	/* Array index: 0x1047 */ 0x5803,0x377A,
	/* Array index: 0x1047 */ 0x5804,0x377B,
	/* Array index: 0x1047 */ 0x5808,0x377C,
	/* Array index: 0x1047 */ 0x5809,0x377D,
	/* Array index: 0x1047 */ 0x580C,0x3821,
	/* Array index: 0x1047 */ 0x580D,0x3822,
	/* Array index: 0x1047 */ 0x581B,0x3823,
	/* Array index: 0x1047 */ 0x5826,0x3827,
	/* Array index: 0x1047 */ 0x5827,0x3828,
	/* Array index: 0x1047 */ 0x582D,0x3829,
	/* Array index: 0x1047 */ 0x5832,0x382A,
	/* Array index: 0x1047 */ 0x5839,0x382B,
	/* Array index: 0x1047 */ 0x583F,0x382C,
	/* Array index: 0x1047 */ 0x5849,0x382D,
	/* Array index: 0x1047 */ 0x5855,0x3832,
	/* Array index: 0x1047 */ 0x585F,0x3833,
	/* Array index: 0x1047 */ 0x5861,0x3834,
	/* Array index: 0x1047 */ 0x5864,0x3835,
	/* Array index: 0x1047 */ 0x5867,0x3836,
	/* Array index: 0x1047 */ 0x5868,0x3837,
	/* Array index: 0x1047 */ 0x5878,0x3838,
	/* Array index: 0x1047 */ 0x587C,0x3839,
	/* Array index: 0x1047 */ 0x5894,0x3845,
	/* Array index: 0x1047 */ 0x5896,0x3846,
	/* Array index: 0x1047 */ 0x589D,0x3847,
	/* Array index: 0x1047 */ 0x58A6,0x384B,
	/* Array index: 0x1047 */ 0x58A9,0x384C,
	/* Array index: 0x1047 */ 0x58B1,0x384D,
	/* Array index: 0x1047 */ 0x58B2,0x384E,
	/* Array index: 0x1047 */ 0x58BC,0x3850,
	/* Array index: 0x1047 */ 0x58C2,0x3851,
	/* Array index: 0x1047 */ 0x58C4,0x384F,
	/* Array index: 0x1047 */ 0x58C8,0x3852,
	/* Array index: 0x1047 */ 0x58CD,0x3853,
	/* Array index: 0x1047 */ 0x58CE,0x3854,
	/* Array index: 0x1047 */ 0x58D0,0x3855,
	/* Array index: 0x1047 */ 0x58D2,0x3856,
	/* Array index: 0x1047 */ 0x58D4,0x3857,
	/* Array index: 0x1047 */ 0x58D6,0x3858,
	/* Array index: 0x1047 */ 0x58DA,0x3859,
	/* Array index: 0x1047 */ 0x58DD,0x385A,
	/* Array index: 0x1047 */ 0x58E1,0x385B,
	/* Array index: 0x1047 */ 0x58E2,0x385C,
	/* Array index: 0x1047 */ 0x58E9,0x385D,
	/* Array index: 0x1047 */ 0x58F3,0x385E,
	/* Array index: 0x1047 */ 0x5905,0x385F,
	/* Array index: 0x1047 */ 0x5906,0x3860,
	/* Array index: 0x1047 */ 0x590B,0x3861,
	/* Array index: 0x1047 */ 0x590C,0x3862,
	/* Array index: 0x1047 */ 0x591D,0x3867,
	/* Array index: 0x1047 */ 0x5921,0x3868,
	/* Array index: 0x1047 */ 0x5923,0x3869,
	/* Array index: 0x1047 */ 0x5924,0x386A,
	/* Array index: 0x1047 */ 0x5928,0x386B,
	/* Array index: 0x1047 */ 0x592F,0x386C,
	/* Array index: 0x1047 */ 0x5930,0x386D,
	/* Array index: 0x1047 */ 0x5933,0x386E,
	/* Array index: 0x1047 */ 0x5935,0x386F,
	/* Array index: 0x1047 */ 0x5936,0x3870,
	/* Array index: 0x1047 */ 0x593F,0x3871,
	/* Array index: 0x1047 */ 0x5943,0x3872,
	/* Array index: 0x1047 */ 0x5946,0x3873,
	/* Array index: 0x1047 */ 0x5952,0x3874,
	/* Array index: 0x1047 */ 0x5953,0x3875,
	/* Array index: 0x1047 */ 0x5959,0x3876,
	/* Array index: 0x1047 */ 0x596B,0x387D,
	/* Array index: 0x1047 */ 0x596D,0x387E,
	/* Array index: 0x1047 */ 0x596F,0x3921,
	/* Array index: 0x1047 */ 0x5972,0x3922,
	/* Array index: 0x1047 */ 0x5975,0x3923,
	/* Array index: 0x1047 */ 0x5976,0x3924,
	/* Array index: 0x1047 */ 0x5979,0x3925,
	/* Array index: 0x1047 */ 0x597B,0x3926,
	/* Array index: 0x1047 */ 0x597C,0x3927,
	/* Array index: 0x1047 */ 0x598B,0x3928,
	/* Array index: 0x1047 */ 0x598C,0x3929,
	/* Array index: 0x1047 */ 0x598E,0x392A,
	/* Array index: 0x1047 */ 0x5992,0x392B,
	/* Array index: 0x1047 */ 0x5995,0x392C,
	/* Array index: 0x1047 */ 0x5997,0x392D,
	/* Array index: 0x1047 */ 0x599F,0x392E,
	/* Array index: 0x1047 */ 0x59A4,0x392F,
	/* Array index: 0x1047 */ 0x59A7,0x3930,
	/* Array index: 0x1047 */ 0x59B3,0x3935,
	/* Array index: 0x1047 */ 0x59B7,0x3936,
	/* Array index: 0x1047 */ 0x59BA,0x3937,
	/* Array index: 0x1047 */ 0x59BC,0x3938,
	/* Array index: 0x1047 */ 0x59C1,0x3939,
	/* Array index: 0x1047 */ 0x59C3,0x393A,
	/* Array index: 0x1047 */ 0x59C4,0x393B,
	/* Array index: 0x1047 */ 0x59C8,0x393C,
	/* Array index: 0x1047 */ 0x59CA,0x393D,
	/* Array index: 0x1047 */ 0x59CD,0x393E,
	/* Array index: 0x1047 */ 0x59D2,0x393F,
	/* Array index: 0x1047 */ 0x59E3,0x3943,
	/* Array index: 0x1047 */ 0x59E4,0x3944,
	/* Array index: 0x1047 */ 0x59E7,0x3945,
	/* Array index: 0x1047 */ 0x59F7,0x394B,
	/* Array index: 0x1047 */ 0x5A00,0x394C,
	/* Array index: 0x1047 */ 0x5A04,0x394D,
	/* Array index: 0x1047 */ 0x5A12,0x3951,
	/* Array index: 0x1047 */ 0x5A13,0x3952,
	/* Array index: 0x1047 */ 0x5A1E,0x3953,
	/* Array index: 0x1047 */ 0x5A23,0x3954,
	/* Array index: 0x1047 */ 0x5A24,0x3955,
	/* Array index: 0x1047 */ 0x5A27,0x3956,
	/* Array index: 0x1047 */ 0x5A28,0x3957,
	/* Array index: 0x1047 */ 0x5A2A,0x3958,
	/* Array index: 0x1047 */ 0x5A2D,0x3959,
	/* Array index: 0x1047 */ 0x5A30,0x395A,
	/* Array index: 0x1047 */ 0x5A4C,0x395F,
	/* Array index: 0x1047 */ 0x5A50,0x3960,
	/* Array index: 0x1047 */ 0x5A55,0x3961,
	/* Array index: 0x1047 */ 0x5A5E,0x3962,
	/* Array index: 0x1047 */ 0x5A63,0x3963,
	/* Array index: 0x1047 */ 0x5A65,0x3964,
	/* Array index: 0x1047 */ 0x5A67,0x3965,
	/* Array index: 0x1047 */ 0x5A6D,0x3966,
	/* Array index: 0x1047 */ 0x5A77,0x3967,
	/* Array index: 0x1047 */ 0x5A7A,0x3968,
	/* Array index: 0x1047 */ 0x5A7B,0x3969,
	/* Array index: 0x1047 */ 0x5A7E,0x396A,
	/* Array index: 0x1047 */ 0x5A8B,0x396B,
	/* Array index: 0x1047 */ 0x5A90,0x396C,
	/* Array index: 0x1047 */ 0x5A93,0x396D,
	/* Array index: 0x1047 */ 0x5A96,0x396E,
	/* Array index: 0x1047 */ 0x5A99,0x396F,
	/* Array index: 0x1047 */ 0x5AA7,0x3975,
	/* Array index: 0x1047 */ 0x5AAC,0x3976,
	/* Array index: 0x1047 */ 0x5AB8,0x397B,
	/* Array index: 0x1047 */ 0x5ABA,0x397C,
	/* Array index: 0x1047 */ 0x5ABB,0x397D,
	/* Array index: 0x1047 */ 0x5ABF,0x397E,
	/* Array index: 0x1047 */ 0x5AC4,0x3A21,
	/* Array index: 0x1047 */ 0x5AC6,0x3A22,
	/* Array index: 0x1047 */ 0x5AC8,0x3A23,
	/* Array index: 0x1047 */ 0x5ACF,0x3A24,
	/* Array index: 0x1047 */ 0x5ADA,0x3A25,
	/* Array index: 0x1047 */ 0x5ADC,0x3A26,
	/* Array index: 0x1047 */ 0x5AE0,0x3A27,
	/* Array index: 0x1047 */ 0x5AE5,0x3A28,
	/* Array index: 0x1047 */ 0x5AEA,0x3A29,
	/* Array index: 0x1047 */ 0x5AEE,0x3A2A,
	/* Array index: 0x1047 */ 0x5AF5,0x3A2B,
	/* Array index: 0x1047 */ 0x5AF6,0x3A2C,
	/* Array index: 0x1047 */ 0x5AFD,0x3A2D,
	/* Array index: 0x1047 */ 0x5B00,0x3A2E,
	/* Array index: 0x1047 */ 0x5B01,0x3A2F,
	/* Array index: 0x1047 */ 0x5B08,0x3A30,
	/* Array index: 0x1047 */ 0x5B17,0x3A31,
	/* Array index: 0x1047 */ 0x5B19,0x3A33,
	/* Array index: 0x1047 */ 0x5B1B,0x3A34,
	/* Array index: 0x1047 */ 0x5B1D,0x3A35,
	/* Array index: 0x1047 */ 0x5B21,0x3A36,
	/* Array index: 0x1047 */ 0x5B25,0x3A37,
	/* Array index: 0x1047 */ 0x5B2D,0x3A38,
	/* Array index: 0x1047 */ 0x5B34,0x3A32,
	/* Array index: 0x1047 */ 0x5B38,0x3A39,
	/* Array index: 0x1047 */ 0x5B41,0x3A3A,
	/* Array index: 0x1047 */ 0x5B4B,0x3A3B,
	/* Array index: 0x1047 */ 0x5B4C,0x3A3C,
	/* Array index: 0x1047 */ 0x5B52,0x3A3D,
	/* Array index: 0x1047 */ 0x5B56,0x3A3E,
	/* Array index: 0x1047 */ 0x5B5E,0x3A3F,
	/* Array index: 0x1047 */ 0x5B68,0x3A40,
	/* Array index: 0x1047 */ 0x5B6E,0x3A41,
	/* Array index: 0x1047 */ 0x5B6F,0x3A42,
	/* Array index: 0x1047 */ 0x5B84,0x3A48,
	/* Array index: 0x1047 */ 0x5B86,0x3A49,
	/* Array index: 0x1047 */ 0x5B8A,0x3A4A,
	/* Array index: 0x1047 */ 0x5B8E,0x3A4B,
	/* Array index: 0x1047 */ 0x5BA8,0x3A51,
	/* Array index: 0x1047 */ 0x5BA9,0x3A52,
	/* Array index: 0x1047 */ 0x5BAC,0x3A53,
	/* Array index: 0x1047 */ 0x5BAD,0x3A54,
	/* Array index: 0x1047 */ 0x5BAF,0x3A55,
	/* Array index: 0x1047 */ 0x5BB1,0x3A56,
	/* Array index: 0x1047 */ 0x5BB2,0x3A57,
	/* Array index: 0x1047 */ 0x5BB7,0x3A58,
	/* Array index: 0x1047 */ 0x5BBA,0x3A59,
	/* Array index: 0x1047 */ 0x5BBC,0x3A5A,
	/* Array index: 0x1047 */ 0x5BC0,0x3A5B,
	/* Array index: 0x1047 */ 0x5BC1,0x3A5C,
	/* Array index: 0x1047 */ 0x5BCD,0x3A5D,
	/* Array index: 0x1047 */ 0x5BCF,0x3A5E,
	/* Array index: 0x1047 */ 0x5BE0,0x3A64,
	/* Array index: 0x1047 */ 0x5BEF,0x3A65,
	/* Array index: 0x1047 */ 0x5BF1,0x3A66,
	/* Array index: 0x1047 */ 0x5BF4,0x3A67,
	/* Array index: 0x1047 */ 0x5BFD,0x3A68,
	/* Array index: 0x1047 */ 0x5C0C,0x3A69,
	/* Array index: 0x1047 */ 0x5C17,0x3A6A,
	/* Array index: 0x1047 */ 0x5C1E,0x3A6B,
	/* Array index: 0x1047 */ 0x5C1F,0x3A6C,
	/* Array index: 0x1047 */ 0x5C23,0x3A6D,
	/* Array index: 0x1047 */ 0x5C26,0x3A6E,
	/* Array index: 0x1047 */ 0x5C29,0x3A6F,
	/* Array index: 0x1047 */ 0x5C2B,0x3A70,
	/* Array index: 0x1047 */ 0x5C2C,0x3A71,
	/* Array index: 0x1047 */ 0x5C2E,0x3A72,
	/* Array index: 0x1047 */ 0x5C30,0x3A73,
	/* Array index: 0x1047 */ 0x5C32,0x3A74,
	/* Array index: 0x1047 */ 0x5C35,0x3A75,
	/* Array index: 0x1047 */ 0x5C36,0x3A76,
	/* Array index: 0x1047 */ 0x5C59,0x3A77,
	/* Array index: 0x1047 */ 0x5C5A,0x3A78,
	/* Array index: 0x1047 */ 0x5C5C,0x3A79,
	/* Array index: 0x1047 */ 0x5C62,0x3A7A,
	/* Array index: 0x1047 */ 0x5C63,0x3A7B,
	/* Array index: 0x1047 */ 0x5C6D,0x3B21,
	/* Array index: 0x1047 */ 0x5C70,0x3B22,
	/* Array index: 0x1047 */ 0x5C74,0x3B23,
	/* Array index: 0x1047 */ 0x5C75,0x3B24,
	/* Array index: 0x1047 */ 0x5C87,0x3B29,
	/* Array index: 0x1047 */ 0x5C88,0x3B2A,
	/* Array index: 0x1047 */ 0x5C8A,0x3B2B,
	/* Array index: 0x1047 */ 0x5C8F,0x3B2C,
	/* Array index: 0x1047 */ 0x5C92,0x3B2D,
	/* Array index: 0x1047 */ 0x5C9D,0x3B2E,
	/* Array index: 0x1047 */ 0x5CA6,0x3B33,
	/* Array index: 0x1047 */ 0x5CAA,0x3B34,
	/* Array index: 0x1047 */ 0x5CB2,0x3B35,
	/* Array index: 0x1047 */ 0x5CB4,0x3B36,
	/* Array index: 0x1047 */ 0x5CB5,0x3B37,
	/* Array index: 0x1047 */ 0x5CBA,0x3B38,
	/* Array index: 0x1047 */ 0x5CC9,0x3B39,
	/* Array index: 0x1047 */ 0x5CCB,0x3B3A,
	/* Array index: 0x1047 */ 0x5CD2,0x3B3B,
	/* Array index: 0x1047 */ 0x5CD7,0x3B3D,
	/* Array index: 0x1047 */ 0x5CDD,0x3B3C,
	/* Array index: 0x1047 */ 0x5CEE,0x3B3E,
	/* Array index: 0x1047 */ 0x5CF1,0x3B3F,
	/* Array index: 0x1047 */ 0x5CF2,0x3B40,
	/* Array index: 0x1047 */ 0x5CF4,0x3B41,
	/* Array index: 0x1047 */ 0x5D01,0x3B42,
	/* Array index: 0x1047 */ 0x5D06,0x3B43,
	/* Array index: 0x1047 */ 0x5D0D,0x3B44,
	/* Array index: 0x1047 */ 0x5D12,0x3B45,
	/* Array index: 0x1047 */ 0x5D2B,0x3B46,
	/* Array index: 0x1047 */ 0x5D31,0x3B4B,
	/* Array index: 0x1047 */ 0x5D34,0x3B4C,
	/* Array index: 0x1047 */ 0x5D39,0x3B4D,
	/* Array index: 0x1047 */ 0x5D3D,0x3B4E,
	/* Array index: 0x1047 */ 0x5D3F,0x3B4F,
	/* Array index: 0x1047 */ 0x5D42,0x3B50,
	/* Array index: 0x1047 */ 0x5D43,0x3B51,
	/* Array index: 0x1047 */ 0x5D46,0x3B52,
	/* Array index: 0x1047 */ 0x5D48,0x3B53,
	/* Array index: 0x1047 */ 0x5D4A,0x3B57,
	/* Array index: 0x1047 */ 0x5D51,0x3B55,
	/* Array index: 0x1047 */ 0x5D55,0x3B54,
	/* Array index: 0x1047 */ 0x5D59,0x3B56,
	/* Array index: 0x1047 */ 0x5D6A,0x3B5D,
	/* Array index: 0x1047 */ 0x5D6D,0x3B5E,
	/* Array index: 0x1047 */ 0x5D70,0x3B5F,
	/* Array index: 0x1047 */ 0x5D79,0x3B60,
	/* Array index: 0x1047 */ 0x5D7A,0x3B61,
	/* Array index: 0x1047 */ 0x5D7E,0x3B62,
	/* Array index: 0x1047 */ 0x5D7F,0x3B63,
	/* Array index: 0x1047 */ 0x5D81,0x3B64,
	/* Array index: 0x1047 */ 0x5D83,0x3B65,
	/* Array index: 0x1047 */ 0x5D88,0x3B66,
	/* Array index: 0x1047 */ 0x5D8A,0x3B67,
	/* Array index: 0x1047 */ 0x5D99,0x3B6C,
	/* Array index: 0x1047 */ 0x5D9B,0x3B6D,
	/* Array index: 0x1047 */ 0x5D9F,0x3B6E,
	/* Array index: 0x1047 */ 0x5DA0,0x3B6F,
	/* Array index: 0x1047 */ 0x5DA7,0x3B70,
	/* Array index: 0x1047 */ 0x5DAB,0x3B71,
	/* Array index: 0x1047 */ 0x5DB0,0x3B72,
	/* Array index: 0x1047 */ 0x5DB2,0x6674,
	/* Array index: 0x1047 */ 0x5DB4,0x3B73,
	/* Array index: 0x1047 */ 0x5DB8,0x3B74,
	/* Array index: 0x1047 */ 0x5DB9,0x3B75,
	/* Array index: 0x1047 */ 0x5DC3,0x3B76,
	/* Array index: 0x1047 */ 0x5DC7,0x3B77,
	/* Array index: 0x1047 */ 0x5DCB,0x3B78,
	/* Array index: 0x1047 */ 0x5DCE,0x3B7A,
	/* Array index: 0x1047 */ 0x5DD0,0x3B79,
	/* Array index: 0x1047 */ 0x5DD8,0x3B7B,
	/* Array index: 0x1047 */ 0x5DD9,0x3B7C,
	/* Array index: 0x1047 */ 0x5DE0,0x3B7D,
	/* Array index: 0x1047 */ 0x5DE4,0x3B7E,
	/* Array index: 0x1047 */ 0x5DE9,0x3C21,
	/* Array index: 0x1047 */ 0x5DF8,0x3C22,
	/* Array index: 0x1047 */ 0x5DF9,0x3C23,
	/* Array index: 0x1047 */ 0x5E00,0x3C24,
	/* Array index: 0x1047 */ 0x5E07,0x3C25,
	/* Array index: 0x1047 */ 0x5E0D,0x3C26,
	/* Array index: 0x1047 */ 0x5E12,0x3C27,
	/* Array index: 0x1047 */ 0x5E14,0x3C28,
	/* Array index: 0x1047 */ 0x5E15,0x3C29,
	/* Array index: 0x1047 */ 0x5E18,0x3C2A,
	/* Array index: 0x1047 */ 0x5E1F,0x3C2B,
	/* Array index: 0x1047 */ 0x5E20,0x3C2C,
	/* Array index: 0x1047 */ 0x5E28,0x3C2E,
	/* Array index: 0x1047 */ 0x5E2E,0x3C2D,
	/* Array index: 0x1047 */ 0x5E32,0x3C2F,
	/* Array index: 0x1047 */ 0x5E35,0x3C30,
	/* Array index: 0x1047 */ 0x5E3E,0x3C31,
	/* Array index: 0x1047 */ 0x5E49,0x3C34,
	/* Array index: 0x1047 */ 0x5E4B,0x3C32,
	/* Array index: 0x1047 */ 0x5E50,0x3C33,
	/* Array index: 0x1047 */ 0x5E51,0x3C35,
	/* Array index: 0x1047 */ 0x5E56,0x3C36,
	/* Array index: 0x1047 */ 0x5E58,0x3C37,
	/* Array index: 0x1047 */ 0x5E5B,0x3C38,
	/* Array index: 0x1047 */ 0x5E5C,0x3C39,
	/* Array index: 0x1047 */ 0x5E5E,0x3C3A,
	/* Array index: 0x1047 */ 0x5E80,0x3C42,
	/* Array index: 0x1047 */ 0x5E8B,0x3C43,
	/* Array index: 0x1047 */ 0x5E8E,0x3C44,
	/* Array index: 0x1047 */ 0x5EA2,0x3C45,
	/* Array index: 0x1047 */ 0x5EA4,0x3C46,
	/* Array index: 0x1047 */ 0x5EA5,0x3C47,
	/* Array index: 0x1047 */ 0x5EA8,0x3C48,
	/* Array index: 0x1047 */ 0x5EAA,0x3C49,
	/* Array index: 0x1047 */ 0x5EAC,0x3C4A,
	/* Array index: 0x1047 */ 0x5EB1,0x3C4B,
	/* Array index: 0x1047 */ 0x5EB3,0x3C4C,
	/* Array index: 0x1047 */ 0x5EC6,0x3C50,
	/* Array index: 0x1047 */ 0x5ECB,0x3C52,
	/* Array index: 0x1047 */ 0x5ECC,0x3C51,
	/* Array index: 0x1047 */ 0x5ECE,0x3C53,
	/* Array index: 0x1047 */ 0x5EDC,0x3C58,
	/* Array index: 0x1047 */ 0x5EDE,0x3C59,
	/* Array index: 0x1047 */ 0x5EE5,0x3C5A,
	/* Array index: 0x1047 */ 0x5EEB,0x3C5B,
	/* Array index: 0x1047 */ 0x5F02,0x3C5C,
	/* Array index: 0x1047 */ 0x5F0E,0x3C60,
	/* Array index: 0x1047 */ 0x5F19,0x3C61,
	/* Array index: 0x1047 */ 0x5F1C,0x3C62,
	/* Array index: 0x1047 */ 0x5F1D,0x3C63,
	/* Array index: 0x1047 */ 0x5F28,0x3C68,
	/* Array index: 0x1047 */ 0x5F2B,0x3C69,
	/* Array index: 0x1047 */ 0x5F2C,0x3C6A,
	/* Array index: 0x1047 */ 0x5F2E,0x3C6B,
	/* Array index: 0x1047 */ 0x5F30,0x3C6C,
	/* Array index: 0x1047 */ 0x5F34,0x3C6D,
	/* Array index: 0x1047 */ 0x5F36,0x3C6E,
	/* Array index: 0x1047 */ 0x5F3B,0x3C6F,
	/* Array index: 0x1047 */ 0x5F3D,0x3C70,
	/* Array index: 0x1047 */ 0x5F3F,0x3C71,
	/* Array index: 0x1047 */ 0x5F40,0x3C72,
	/* Array index: 0x1047 */ 0x5F44,0x3C73,
	/* Array index: 0x1047 */ 0x5F45,0x3C74,
	/* Array index: 0x1047 */ 0x5F47,0x3C75,
	/* Array index: 0x1047 */ 0x5F4D,0x3C76,
	/* Array index: 0x1047 */ 0x5F50,0x3C77,
	/* Array index: 0x1047 */ 0x5F54,0x3C78,
	/* Array index: 0x1047 */ 0x5F58,0x3C79,
	/* Array index: 0x1047 */ 0x5F5B,0x3C7A,
	/* Array index: 0x1047 */ 0x5F60,0x3C7B,
	/* Array index: 0x1047 */ 0x5F63,0x3C7C,
	/* Array index: 0x1047 */ 0x5F64,0x3C7D,
	/* Array index: 0x1047 */ 0x5F67,0x3C7E,
	/* Array index: 0x1047 */ 0x5F6F,0x3D21,
	/* Array index: 0x1047 */ 0x5F72,0x3D22,
	/* Array index: 0x1047 */ 0x5F74,0x3D23,
	/* Array index: 0x1047 */ 0x5F75,0x3D24,
	/* Array index: 0x1047 */ 0x5F78,0x3D25,
	/* Array index: 0x1047 */ 0x5F7A,0x3D26,
	/* Array index: 0x1047 */ 0x5F7D,0x3D27,
	/* Array index: 0x1047 */ 0x5F7E,0x3D28,
	/* Array index: 0x1047 */ 0x5F89,0x3D29,
	/* Array index: 0x1047 */ 0x5F8D,0x3D2A,
	/* Array index: 0x1047 */ 0x5F8F,0x3D2B,
	/* Array index: 0x1047 */ 0x5F96,0x3D2C,
	/* Array index: 0x1047 */ 0x5F9C,0x3D2D,
	/* Array index: 0x1047 */ 0x5F9D,0x3D2E,
	/* Array index: 0x1047 */ 0x5FA2,0x3D2F,
	/* Array index: 0x1047 */ 0x5FA4,0x3D32,
	/* Array index: 0x1047 */ 0x5FA7,0x3D30,
	/* Array index: 0x1047 */ 0x5FB8,0x3D37,
	/* Array index: 0x1047 */ 0x5FC4,0x3D38,
	/* Array index: 0x1047 */ 0x5FDE,0x3D42,
	/* Array index: 0x1047 */ 0x5FE1,0x3D43,
	/* Array index: 0x1047 */ 0x5FE2,0x3D44,
	/* Array index: 0x1047 */ 0x5FF6,0x3D4E,
	/* Array index: 0x1047 */ 0x5FFA,0x3D4F,
	/* Array index: 0x1047 */ 0x5FFC,0x3D50,
	/* Array index: 0x1047 */ 0x6007,0x3D51,
	/* Array index: 0x1047 */ 0x600A,0x3D52,
	/* Array index: 0x1047 */ 0x600D,0x3D53,
	/* Array index: 0x1047 */ 0x6013,0x3D54,
	/* Array index: 0x1047 */ 0x6014,0x3D55,
	/* Array index: 0x1047 */ 0x6017,0x3D56,
	/* Array index: 0x1047 */ 0x6018,0x3D57,
	/* Array index: 0x1047 */ 0x601A,0x3D58,
	/* Array index: 0x1047 */ 0x601F,0x3D59,
	/* Array index: 0x1047 */ 0x6024,0x3D5A,
	/* Array index: 0x1047 */ 0x602D,0x3D5B,
	/* Array index: 0x1047 */ 0x6033,0x3D5C,
	/* Array index: 0x1047 */ 0x6035,0x3D5D,
	/* Array index: 0x1047 */ 0x6040,0x3D5E,
	/* Array index: 0x1047 */ 0x604C,0x3D62,
	/* Array index: 0x1047 */ 0x6051,0x3D63,
	/* Array index: 0x1047 */ 0x6054,0x3D64,
	/* Array index: 0x1047 */ 0x6056,0x3D65,
	/* Array index: 0x1047 */ 0x6057,0x3D66,
	/* Array index: 0x1047 */ 0x605D,0x3D67,
	/* Array index: 0x1047 */ 0x6061,0x3D68,
	/* Array index: 0x1047 */ 0x6067,0x3D69,
	/* Array index: 0x1047 */ 0x6071,0x3D6A,
	/* Array index: 0x1047 */ 0x607E,0x3D6B,
	/* Array index: 0x1047 */ 0x607F,0x3D6C,
	/* Array index: 0x1047 */ 0x6082,0x3D6D,
	/* Array index: 0x1047 */ 0x6086,0x3D6E,
	/* Array index: 0x1047 */ 0x6088,0x3D6F,
	/* Array index: 0x1047 */ 0x608A,0x3D70,
	/* Array index: 0x1047 */ 0x608E,0x3D71,
	/* Array index: 0x1047 */ 0x6091,0x3D72,
	/* Array index: 0x1047 */ 0x6093,0x3D73,
	/* Array index: 0x1047 */ 0x6095,0x3D74,
	/* Array index: 0x1047 */ 0x6098,0x3D75,
	/* Array index: 0x1047 */ 0x609D,0x3D76,
	/* Array index: 0x1047 */ 0x609E,0x3D77,
	/* Array index: 0x1047 */ 0x60A2,0x3D78,
	/* Array index: 0x1047 */ 0x60A4,0x3D79,
	/* Array index: 0x1047 */ 0x60A5,0x3D7A,
	/* Array index: 0x1047 */ 0x60A8,0x3D7B,
	/* Array index: 0x1047 */ 0x60B0,0x3D7C,
	/* Array index: 0x1047 */ 0x60B1,0x3D7D,
	/* Array index: 0x1047 */ 0x60B7,0x3D7E,
	/* Array index: 0x1047 */ 0x60BB,0x3E21,
	/* Array index: 0x1047 */ 0x60BE,0x3E22,
	/* Array index: 0x1047 */ 0x60C2,0x3E23,
	/* Array index: 0x1047 */ 0x60C4,0x3E24,
	/* Array index: 0x1047 */ 0x60D4,0x3E2B,
	/* Array index: 0x1047 */ 0x60D5,0x3E2C,
	/* Array index: 0x1047 */ 0x60D9,0x3E2D,
	/* Array index: 0x1047 */ 0x60DB,0x3E2E,
	/* Array index: 0x1047 */ 0x60DD,0x3E2F,
	/* Array index: 0x1047 */ 0x60DE,0x3E30,
	/* Array index: 0x1047 */ 0x60E2,0x3E31,
	/* Array index: 0x1047 */ 0x60E5,0x3E32,
	/* Array index: 0x1047 */ 0x60F2,0x3E33,
	/* Array index: 0x1047 */ 0x60F5,0x3E34,
	/* Array index: 0x1047 */ 0x60F8,0x3E35,
	/* Array index: 0x1047 */ 0x60FC,0x3E36,
	/* Array index: 0x1047 */ 0x60FD,0x3E37,
	/* Array index: 0x1047 */ 0x6102,0x3E38,
	/* Array index: 0x1047 */ 0x6107,0x3E39,
	/* Array index: 0x1047 */ 0x610A,0x3E3A,
	/* Array index: 0x1047 */ 0x610C,0x3E3B,
	/* Array index: 0x1047 */ 0x611C,0x3E44,
	/* Array index: 0x1047 */ 0x611E,0x3E45,
	/* Array index: 0x1047 */ 0x6122,0x3E46,
	/* Array index: 0x1047 */ 0x612A,0x3E47,
	/* Array index: 0x1047 */ 0x612B,0x3E48,
	/* Array index: 0x1047 */ 0x6130,0x3E49,
	/* Array index: 0x1047 */ 0x6131,0x3E4A,
	/* Array index: 0x1047 */ 0x6141,0x3E4F,
	/* Array index: 0x1047 */ 0x6145,0x3E50,
	/* Array index: 0x1047 */ 0x6146,0x3E51,
	/* Array index: 0x1047 */ 0x6149,0x3E52,
	/* Array index: 0x1047 */ 0x615E,0x3E53,
	/* Array index: 0x1047 */ 0x6160,0x3E54,
	/* Array index: 0x1047 */ 0x616C,0x3E55,
	/* Array index: 0x1047 */ 0x6172,0x3E56,
	/* Array index: 0x1047 */ 0x6178,0x3E57,
	/* Array index: 0x1047 */ 0x618B,0x3E5F,
	/* Array index: 0x1047 */ 0x618D,0x3E60,
	/* Array index: 0x1047 */ 0x6192,0x3E61,
	/* Array index: 0x1047 */ 0x6193,0x3E62,
	/* Array index: 0x1047 */ 0x6197,0x3E63,
	/* Array index: 0x1047 */ 0x6198,0x3E64,
	/* Array index: 0x1047 */ 0x61A5,0x3E69,
	/* Array index: 0x1047 */ 0x61A8,0x3E6A,
	/* Array index: 0x1047 */ 0x61AA,0x3E6B,
	/* Array index: 0x1047 */ 0x61AD,0x3E6C,
	/* Array index: 0x1047 */ 0x61B8,0x3E6D,
	/* Array index: 0x1047 */ 0x61B9,0x3E6E,
	/* Array index: 0x1047 */ 0x61BC,0x3E6F,
	/* Array index: 0x1047 */ 0x61CE,0x3E73,
	/* Array index: 0x1047 */ 0x61CF,0x3E74,
	/* Array index: 0x1047 */ 0x61D5,0x3E75,
	/* Array index: 0x1047 */ 0x61E5,0x3E7E,
	/* Array index: 0x1047 */ 0x61E7,0x3E7C,
	/* Array index: 0x1047 */ 0x61E9,0x3E7D,
	/* Array index: 0x1047 */ 0x61EC,0x3F21,
	/* Array index: 0x1047 */ 0x61ED,0x3F22,
	/* Array index: 0x1047 */ 0x61EF,0x3F23,
	/* Array index: 0x1047 */ 0x6201,0x3F24,
	/* Array index: 0x1047 */ 0x6203,0x3F25,
	/* Array index: 0x1047 */ 0x6204,0x3F26,
	/* Array index: 0x1047 */ 0x6207,0x3F27,
	/* Array index: 0x1047 */ 0x6213,0x3F28,
	/* Array index: 0x1047 */ 0x6215,0x3F29,
	/* Array index: 0x1047 */ 0x621C,0x3F2A,
	/* Array index: 0x1047 */ 0x6220,0x3F2B,
	/* Array index: 0x1047 */ 0x6222,0x3F2C,
	/* Array index: 0x1047 */ 0x6223,0x3F2D,
	/* Array index: 0x1047 */ 0x6227,0x3F2E,
	/* Array index: 0x1047 */ 0x6229,0x3F2F,
	/* Array index: 0x1047 */ 0x622B,0x3F30,
	/* Array index: 0x1047 */ 0x6239,0x3F31,
	/* Array index: 0x1047 */ 0x623D,0x3F32,
	/* Array index: 0x1047 */ 0x624C,0x3F37,
	/* Array index: 0x1047 */ 0x625A,0x3F3D,
	/* Array index: 0x1047 */ 0x625C,0x3F3E,
	/* Array index: 0x1047 */ 0x6264,0x3F3F,
	/* Array index: 0x1047 */ 0x626D,0x3F40,
	/* Array index: 0x1047 */ 0x626F,0x3F41,
	/* Array index: 0x1047 */ 0x6273,0x3F42,
	/* Array index: 0x1047 */ 0x627A,0x3F43,
	/* Array index: 0x1047 */ 0x627D,0x3F44,
	/* Array index: 0x1047 */ 0x62A6,0x3F49,
	/* Array index: 0x1047 */ 0x62A8,0x3F4A,
	/* Array index: 0x1047 */ 0x62B3,0x3F4B,
	/* Array index: 0x1047 */ 0x62B6,0x3F4C,
	/* Array index: 0x1047 */ 0x62B7,0x3F4D,
	/* Array index: 0x1047 */ 0x62BA,0x3F4E,
	/* Array index: 0x1047 */ 0x62BE,0x3F4F,
	/* Array index: 0x1047 */ 0x62BF,0x3F50,
	/* Array index: 0x1047 */ 0x62C4,0x3F51,
	/* Array index: 0x1047 */ 0x62CE,0x3F52,
	/* Array index: 0x1047 */ 0x62D5,0x3F53,
	/* Array index: 0x1047 */ 0x62D6,0x3F54,
	/* Array index: 0x1047 */ 0x62DA,0x3F55,
	/* Array index: 0x1047 */ 0x62EA,0x3F56,
	/* Array index: 0x1047 */ 0x62F2,0x3F57,
	/* Array index: 0x1047 */ 0x62F4,0x3F58,
	/* Array index: 0x1047 */ 0x62FC,0x3F59,
	/* Array index: 0x1047 */ 0x62FD,0x3F5A,
	/* Array index: 0x1047 */ 0x6303,0x3F5B,
	/* Array index: 0x1047 */ 0x6304,0x3F5C,
	/* Array index: 0x1047 */ 0x630A,0x3F5D,
	/* Array index: 0x1047 */ 0x630B,0x3F5E,
	/* Array index: 0x1047 */ 0x630D,0x3F5F,
	/* Array index: 0x1047 */ 0x6310,0x3F60,
	/* Array index: 0x1047 */ 0x6313,0x3F61,
	/* Array index: 0x1047 */ 0x6316,0x3F62,
	/* Array index: 0x1047 */ 0x6318,0x3F63,
	/* Array index: 0x1047 */ 0x6329,0x3F64,
	/* Array index: 0x1047 */ 0x632A,0x3F65,
	/* Array index: 0x1047 */ 0x632D,0x3F66,
	/* Array index: 0x1047 */ 0x6335,0x3F67,
	/* Array index: 0x1047 */ 0x6336,0x3F68,
	/* Array index: 0x1047 */ 0x6339,0x3F69,
	/* Array index: 0x1047 */ 0x633C,0x3F6A,
	/* Array index: 0x1047 */ 0x634A,0x3F70,
	/* Array index: 0x1047 */ 0x634B,0x3F71,
	/* Array index: 0x1047 */ 0x634E,0x3F72,
	/* Array index: 0x1047 */ 0x6358,0x3F76,
	/* Array index: 0x1047 */ 0x635B,0x3F77,
	/* Array index: 0x1047 */ 0x6365,0x3F78,
	/* Array index: 0x1047 */ 0x6366,0x3F79,
	/* Array index: 0x1047 */ 0x636C,0x3F7A,
	/* Array index: 0x1047 */ 0x636D,0x3F7B,
	/* Array index: 0x1047 */ 0x6371,0x3F7C,
	/* Array index: 0x1047 */ 0x6374,0x3F7D,
	/* Array index: 0x1047 */ 0x6375,0x3F7E,
	/* Array index: 0x1047 */ 0x6378,0x4021,
	/* Array index: 0x1047 */ 0x637C,0x4022,
	/* Array index: 0x1047 */ 0x637D,0x4023,
	/* Array index: 0x1047 */ 0x637F,0x4024,
	/* Array index: 0x1047 */ 0x6382,0x4025,
	/* Array index: 0x1047 */ 0x6384,0x4026,
	/* Array index: 0x1047 */ 0x6387,0x4027,
	/* Array index: 0x1047 */ 0x638A,0x4028,
	/* Array index: 0x1047 */ 0x6390,0x4029,
	/* Array index: 0x1047 */ 0x6394,0x402A,
	/* Array index: 0x1047 */ 0x6395,0x402B,
	/* Array index: 0x1047 */ 0x6399,0x402C,
	/* Array index: 0x1047 */ 0x639A,0x402D,
	/* Array index: 0x1047 */ 0x639E,0x402E,
	/* Array index: 0x1047 */ 0x63A4,0x402F,
	/* Array index: 0x1047 */ 0x63A6,0x4030,
	/* Array index: 0x1047 */ 0x63BD,0x4034,
	/* Array index: 0x1047 */ 0x63C1,0x4035,
	/* Array index: 0x1047 */ 0x63C5,0x4036,
	/* Array index: 0x1047 */ 0x63C8,0x4037,
	/* Array index: 0x1047 */ 0x63CE,0x4038,
	/* Array index: 0x1047 */ 0x63DC,0x403D,
	/* Array index: 0x1047 */ 0x63E0,0x403E,
	/* Array index: 0x1047 */ 0x63E5,0x403F,
	/* Array index: 0x1047 */ 0x63EA,0x4040,
	/* Array index: 0x1047 */ 0x63EC,0x4041,
	/* Array index: 0x1047 */ 0x63F2,0x4042,
	/* Array index: 0x1047 */ 0x63F3,0x4043,
	/* Array index: 0x1047 */ 0x63F5,0x4044,
	/* Array index: 0x1047 */ 0x63F8,0x4045,
	/* Array index: 0x1047 */ 0x63F9,0x4046,
	/* Array index: 0x1047 */ 0x6409,0x4047,
	/* Array index: 0x1047 */ 0x640A,0x4048,
	/* Array index: 0x1047 */ 0x6410,0x4049,
	/* Array index: 0x1047 */ 0x6412,0x404A,
	/* Array index: 0x1047 */ 0x6414,0x404B,
	/* Array index: 0x1047 */ 0x6418,0x404C,
	/* Array index: 0x1047 */ 0x641E,0x404D,
	/* Array index: 0x1047 */ 0x6420,0x404E,
	/* Array index: 0x1047 */ 0x6422,0x404F,
	/* Array index: 0x1047 */ 0x6424,0x4050,
	/* Array index: 0x1047 */ 0x6425,0x4051,
	/* Array index: 0x1047 */ 0x6429,0x4052,
	/* Array index: 0x1047 */ 0x642A,0x4053,
	/* Array index: 0x1047 */ 0x642F,0x4054,
	/* Array index: 0x1047 */ 0x6430,0x4055,
	/* Array index: 0x1047 */ 0x6435,0x4056,
	/* Array index: 0x1047 */ 0x643D,0x4057,
	/* Array index: 0x1047 */ 0x643F,0x4058,
	/* Array index: 0x1047 */ 0x644B,0x4059,
	/* Array index: 0x1047 */ 0x646D,0x4067,
	/* Array index: 0x1047 */ 0x6473,0x4068,
	/* Array index: 0x1047 */ 0x6474,0x4069,
	/* Array index: 0x1047 */ 0x647B,0x406A,
	/* Array index: 0x1047 */ 0x647D,0x406B,
	/* Array index: 0x1047 */ 0x6485,0x406C,
	/* Array index: 0x1047 */ 0x6487,0x406D,
	/* Array index: 0x1047 */ 0x6498,0x4071,
	/* Array index: 0x1047 */ 0x6499,0x4072,
	/* Array index: 0x1047 */ 0x649B,0x4073,
	/* Array index: 0x1047 */ 0x649D,0x4074,
	/* Array index: 0x1047 */ 0x649F,0x4075,
	/* Array index: 0x1047 */ 0x64A1,0x4076,
	/* Array index: 0x1047 */ 0x64A3,0x4077,
	/* Array index: 0x1047 */ 0x64A6,0x4078,
	/* Array index: 0x1047 */ 0x64A8,0x4079,
	/* Array index: 0x1047 */ 0x64AC,0x407A,
	/* Array index: 0x1047 */ 0x64B3,0x407B,
	/* Array index: 0x1047 */ 0x64C4,0x4121,
	/* Array index: 0x1047 */ 0x64D5,0x4129,
	/* Array index: 0x1047 */ 0x64D7,0x412A,
	/* Array index: 0x1047 */ 0x64E4,0x412B,
	/* Array index: 0x1047 */ 0x64E5,0x412C,
	/* Array index: 0x1047 */ 0x64E9,0x412D,
	/* Array index: 0x1047 */ 0x64EA,0x412E,
	/* Array index: 0x1047 */ 0x64ED,0x412F,
	/* Array index: 0x1047 */ 0x64F0,0x4130,
	/* Array index: 0x1047 */ 0x64F5,0x4131,
	/* Array index: 0x1047 */ 0x64F7,0x4132,
	/* Array index: 0x1047 */ 0x64FB,0x4133,
	/* Array index: 0x1047 */ 0x64FF,0x4134,
	/* Array index: 0x1047 */ 0x6501,0x4135,
	/* Array index: 0x1047 */ 0x6504,0x4136,
	/* Array index: 0x1047 */ 0x650F,0x413A,
	/* Array index: 0x1047 */ 0x6513,0x413B,
	/* Array index: 0x1047 */ 0x6514,0x413C,
	/* Array index: 0x1047 */ 0x6516,0x413D,
	/* Array index: 0x1047 */ 0x6519,0x413E,
	/* Array index: 0x1047 */ 0x651B,0x413F,
	/* Array index: 0x1047 */ 0x651E,0x4140,
	/* Array index: 0x1047 */ 0x651F,0x4141,
	/* Array index: 0x1047 */ 0x6522,0x4142,
	/* Array index: 0x1047 */ 0x6526,0x4143,
	/* Array index: 0x1047 */ 0x6529,0x4144,
	/* Array index: 0x1047 */ 0x652E,0x4145,
	/* Array index: 0x1047 */ 0x6531,0x4146,
	/* Array index: 0x1047 */ 0x653A,0x4147,
	/* Array index: 0x1047 */ 0x653C,0x4148,
	/* Array index: 0x1047 */ 0x653D,0x4149,
	/* Array index: 0x1047 */ 0x6543,0x414A,
	/* Array index: 0x1047 */ 0x6547,0x414B,
	/* Array index: 0x1047 */ 0x6549,0x414C,
	/* Array index: 0x1047 */ 0x6550,0x414D,
	/* Array index: 0x1047 */ 0x6552,0x414E,
	/* Array index: 0x1047 */ 0x6554,0x414F,
	/* Array index: 0x1047 */ 0x655F,0x4150,
	/* Array index: 0x1047 */ 0x6560,0x4151,
	/* Array index: 0x1047 */ 0x6567,0x4152,
	/* Array index: 0x1047 */ 0x656B,0x4153,
	/* Array index: 0x1047 */ 0x657A,0x4154,
	/* Array index: 0x1047 */ 0x657D,0x4155,
	/* Array index: 0x1047 */ 0x6581,0x4156,
	/* Array index: 0x1047 */ 0x6585,0x4157,
	/* Array index: 0x1047 */ 0x658A,0x4158,
	/* Array index: 0x1047 */ 0x6592,0x4159,
	/* Array index: 0x1047 */ 0x6595,0x415A,
	/* Array index: 0x1047 */ 0x6598,0x415B,
	/* Array index: 0x1047 */ 0x659D,0x415C,
	/* Array index: 0x1047 */ 0x65A0,0x415D,
	/* Array index: 0x1047 */ 0x65A3,0x415E,
	/* Array index: 0x1047 */ 0x65A6,0x415F,
	/* Array index: 0x1047 */ 0x65AE,0x4160,
	/* Array index: 0x1047 */ 0x65BF,0x4164,
	/* Array index: 0x1047 */ 0x65C2,0x4165,
	/* Array index: 0x1047 */ 0x65C8,0x4166,
	/* Array index: 0x1047 */ 0x65C9,0x4167,
	/* Array index: 0x1047 */ 0x65CE,0x4168,
	/* Array index: 0x1047 */ 0x65D0,0x4169,
	/* Array index: 0x1047 */ 0x65D4,0x416A,
	/* Array index: 0x1047 */ 0x65D6,0x416B,
	/* Array index: 0x1047 */ 0x65D8,0x416C,
	/* Array index: 0x1047 */ 0x65DF,0x416D,
	/* Array index: 0x1047 */ 0x65F0,0x416E,
	/* Array index: 0x1047 */ 0x65F2,0x416F,
	/* Array index: 0x1047 */ 0x65F4,0x4170,
	/* Array index: 0x1047 */ 0x65F5,0x4171,
	/* Array index: 0x1047 */ 0x65F9,0x4172,
	/* Array index: 0x1047 */ 0x6604,0x4176,
	/* Array index: 0x1047 */ 0x6608,0x4177,
	/* Array index: 0x1047 */ 0x6609,0x4178,
	/* Array index: 0x1047 */ 0x660D,0x4179,
	/* Array index: 0x1047 */ 0x6611,0x417A,
	/* Array index: 0x1047 */ 0x6612,0x417B,
	/* Array index: 0x1047 */ 0x6615,0x417C,
	/* Array index: 0x1047 */ 0x6616,0x417D,
	/* Array index: 0x1047 */ 0x6637,0x4230,
	/* Array index: 0x1047 */ 0x6639,0x422F,
	/* Array index: 0x1047 */ 0x6640,0x4231,
	/* Array index: 0x1047 */ 0x6645,0x4232,
	/* Array index: 0x1047 */ 0x6646,0x4233,
	/* Array index: 0x1047 */ 0x664A,0x4234,
	/* Array index: 0x1047 */ 0x664C,0x4235,
	/* Array index: 0x1047 */ 0x664E,0x4237,
	/* Array index: 0x1047 */ 0x6651,0x4236,
	/* Array index: 0x1047 */ 0x6660,0x423D,
	/* Array index: 0x1047 */ 0x6661,0x423E,
	/* Array index: 0x1047 */ 0x6673,0x4244,
	/* Array index: 0x1047 */ 0x6690,0x4250,
	/* Array index: 0x1047 */ 0x6692,0x4251,
	/* Array index: 0x1047 */ 0x66A4,0x4258,
	/* Array index: 0x1047 */ 0x66AD,0x4259,
	/* Array index: 0x1047 */ 0x66B1,0x425A,
	/* Array index: 0x1047 */ 0x66B2,0x425B,
	/* Array index: 0x1047 */ 0x66B5,0x425C,
	/* Array index: 0x1047 */ 0x66BB,0x425D,
	/* Array index: 0x1047 */ 0x66C8,0x4262,
	/* Array index: 0x1047 */ 0x66CC,0x4263,
	/* Array index: 0x1047 */ 0x66CE,0x4264,
	/* Array index: 0x1047 */ 0x66CF,0x4265,
	/* Array index: 0x1047 */ 0x66D4,0x4266,
	/* Array index: 0x1047 */ 0x66DB,0x4267,
	/* Array index: 0x1047 */ 0x66DF,0x4268,
	/* Array index: 0x1047 */ 0x66E8,0x4269,
	/* Array index: 0x1047 */ 0x66EB,0x426A,
	/* Array index: 0x1047 */ 0x66EC,0x426B,
	/* Array index: 0x1047 */ 0x66EE,0x426C,
	/* Array index: 0x1047 */ 0x66FA,0x426D,
	/* Array index: 0x1047 */ 0x66FB,0x423F,
	/* Array index: 0x1047 */ 0x6705,0x426E,
	/* Array index: 0x1047 */ 0x6707,0x426F,
	/* Array index: 0x1047 */ 0x670E,0x4270,
	/* Array index: 0x1047 */ 0x6713,0x4271,
	/* Array index: 0x1047 */ 0x6719,0x4272,
	/* Array index: 0x1047 */ 0x671C,0x4273,
	/* Array index: 0x1047 */ 0x6720,0x4274,
	/* Array index: 0x1047 */ 0x6722,0x4275,
	/* Array index: 0x1047 */ 0x6733,0x4276,
	/* Array index: 0x1047 */ 0x673E,0x4277,
	/* Array index: 0x1047 */ 0x6745,0x4278,
	/* Array index: 0x1047 */ 0x6747,0x4279,
	/* Array index: 0x1047 */ 0x6748,0x427A,
	/* Array index: 0x1047 */ 0x674C,0x427B,
	/* Array index: 0x1047 */ 0x6754,0x427C,
	/* Array index: 0x1047 */ 0x6755,0x427D,
	/* Array index: 0x1047 */ 0x675D,0x427E,
	/* Array index: 0x1047 */ 0x6766,0x4321,
	/* Array index: 0x1047 */ 0x676C,0x4322,
	/* Array index: 0x1047 */ 0x676E,0x4323,
	/* Array index: 0x1047 */ 0x6774,0x4324,
	/* Array index: 0x1047 */ 0x6776,0x4325,
	/* Array index: 0x1047 */ 0x677B,0x4326,
	/* Array index: 0x1047 */ 0x6781,0x4327,
	/* Array index: 0x1047 */ 0x6784,0x4328,
	/* Array index: 0x1047 */ 0x678E,0x4329,
	/* Array index: 0x1047 */ 0x678F,0x432A,
	/* Array index: 0x1047 */ 0x6791,0x432B,
	/* Array index: 0x1047 */ 0x6793,0x432C,
	/* Array index: 0x1047 */ 0x6796,0x432D,
	/* Array index: 0x1047 */ 0x6798,0x432E,
	/* Array index: 0x1047 */ 0x6799,0x432F,
	/* Array index: 0x1047 */ 0x679B,0x4330,
	/* Array index: 0x1047 */ 0x67B5,0x4334,
	/* Array index: 0x1047 */ 0x67C0,0x4339,
	/* Array index: 0x1047 */ 0x67C2,0x433A,
	/* Array index: 0x1047 */ 0x67C3,0x433B,
	/* Array index: 0x1047 */ 0x67C5,0x433C,
	/* Array index: 0x1047 */ 0x67C8,0x433D,
	/* Array index: 0x1047 */ 0x67C9,0x433E,
	/* Array index: 0x1047 */ 0x67D2,0x433F,
	/* Array index: 0x1047 */ 0x67D7,0x4340,
	/* Array index: 0x1047 */ 0x67D9,0x4341,
	/* Array index: 0x1047 */ 0x67DC,0x4342,
	/* Array index: 0x1047 */ 0x67E1,0x4343,
	/* Array index: 0x1047 */ 0x67E6,0x4344,
	/* Array index: 0x1047 */ 0x67F0,0x4345,
	/* Array index: 0x1047 */ 0x67F2,0x4346,
	/* Array index: 0x1047 */ 0x67F6,0x4347,
	/* Array index: 0x1047 */ 0x67F7,0x4348,
	/* Array index: 0x1047 */ 0x67F9,0x4338,
	/* Array index: 0x1047 */ 0x6814,0x434A,
	/* Array index: 0x1047 */ 0x6819,0x434B,
	/* Array index: 0x1047 */ 0x681D,0x434C,
	/* Array index: 0x1047 */ 0x681F,0x434D,
	/* Array index: 0x1047 */ 0x6827,0x434F,
	/* Array index: 0x1047 */ 0x6828,0x434E,
	/* Array index: 0x1047 */ 0x683B,0x4356,
	/* Array index: 0x1047 */ 0x683F,0x4357,
	/* Array index: 0x1047 */ 0x6844,0x4358,
	/* Array index: 0x1047 */ 0x6845,0x4359,
	/* Array index: 0x1047 */ 0x684A,0x435A,
	/* Array index: 0x1047 */ 0x684C,0x435B,
	/* Array index: 0x1047 */ 0x6852,0x4349,
	/* Array index: 0x1047 */ 0x6855,0x435C,
	/* Array index: 0x1047 */ 0x6857,0x435D,
	/* Array index: 0x1047 */ 0x6858,0x435E,
	/* Array index: 0x1047 */ 0x685B,0x435F,
	/* Array index: 0x1047 */ 0x686B,0x4360,
	/* Array index: 0x1047 */ 0x6875,0x4366,
	/* Array index: 0x1047 */ 0x6882,0x436B,
	/* Array index: 0x1047 */ 0x6884,0x436C,
	/* Array index: 0x1047 */ 0x6886,0x436D,
	/* Array index: 0x1047 */ 0x6888,0x436E,
	/* Array index: 0x1047 */ 0x6896,0x436F,
	/* Array index: 0x1047 */ 0x6898,0x4370,
	/* Array index: 0x1047 */ 0x689A,0x4371,
	/* Array index: 0x1047 */ 0x689C,0x4372,
	/* Array index: 0x1047 */ 0x68A1,0x4373,
	/* Array index: 0x1047 */ 0x68A3,0x4374,
	/* Array index: 0x1047 */ 0x68A5,0x4375,
	/* Array index: 0x1047 */ 0x68A9,0x4376,
	/* Array index: 0x1047 */ 0x68AA,0x4377,
	/* Array index: 0x1047 */ 0x68AE,0x4378,
	/* Array index: 0x1047 */ 0x68B2,0x4379,
	/* Array index: 0x1047 */ 0x68BB,0x437A,
	/* Array index: 0x1047 */ 0x68C5,0x437B,
	/* Array index: 0x1047 */ 0x68C8,0x437C,
	/* Array index: 0x1047 */ 0x68CC,0x437D,
	/* Array index: 0x1047 */ 0x68D6,0x4424,
	/* Array index: 0x1047 */ 0x68D9,0x4425,
	/* Array index: 0x1047 */ 0x68DC,0x4426,
	/* Array index: 0x1047 */ 0x68DD,0x4427,
	/* Array index: 0x1047 */ 0x68E5,0x4428,
	/* Array index: 0x1047 */ 0x68F5,0x4430,
	/* Array index: 0x1047 */ 0x68F6,0x4431,
	/* Array index: 0x1047 */ 0x6906,0x4435,
	/* Array index: 0x1047 */ 0x6909,0x4436,
	/* Array index: 0x1047 */ 0x690A,0x4437,
	/* Array index: 0x1047 */ 0x6910,0x4438,
	/* Array index: 0x1047 */ 0x6911,0x4439,
	/* Array index: 0x1047 */ 0x6913,0x443A,
	/* Array index: 0x1047 */ 0x6916,0x443B,
	/* Array index: 0x1047 */ 0x6917,0x443C,
	/* Array index: 0x1047 */ 0x6931,0x443D,
	/* Array index: 0x1047 */ 0x6933,0x443E,
	/* Array index: 0x1047 */ 0x6935,0x443F,
	/* Array index: 0x1047 */ 0x6938,0x4440,
	/* Array index: 0x1047 */ 0x693B,0x4441,
	/* Array index: 0x1047 */ 0x6942,0x4442,
	/* Array index: 0x1047 */ 0x6945,0x4443,
	/* Array index: 0x1047 */ 0x6949,0x4444,
	/* Array index: 0x1047 */ 0x694E,0x4445,
	/* Array index: 0x1047 */ 0x6957,0x4446,
	/* Array index: 0x1047 */ 0x695B,0x4447,
	/* Array index: 0x1047 */ 0x696C,0x444E,
	/* Array index: 0x1047 */ 0x697A,0x4452,
	/* Array index: 0x1047 */ 0x697B,0x4453,
	/* Array index: 0x1047 */ 0x697F,0x4454,
	/* Array index: 0x1047 */ 0x6980,0x4455,
	/* Array index: 0x1047 */ 0x698D,0x4456,
	/* Array index: 0x1047 */ 0x6992,0x4457,
	/* Array index: 0x1047 */ 0x6996,0x4458,
	/* Array index: 0x1047 */ 0x6998,0x4459,
	/* Array index: 0x1047 */ 0x69A1,0x445A,
	/* Array index: 0x1047 */ 0x69A5,0x445B,
	/* Array index: 0x1047 */ 0x69A6,0x445C,
	/* Array index: 0x1047 */ 0x69A8,0x445D,
	/* Array index: 0x1047 */ 0x69AB,0x445E,
	/* Array index: 0x1047 */ 0x69AD,0x445F,
	/* Array index: 0x1047 */ 0x69AF,0x4460,
	/* Array index: 0x1047 */ 0x69B7,0x4461,
	/* Array index: 0x1047 */ 0x69B8,0x4462,
	/* Array index: 0x1047 */ 0x69BA,0x4463,
	/* Array index: 0x1047 */ 0x69BC,0x4464,
	/* Array index: 0x1047 */ 0x69C5,0x4465,
	/* Array index: 0x1047 */ 0x69C8,0x4466,
	/* Array index: 0x1047 */ 0x69D1,0x4467,
	/* Array index: 0x1047 */ 0x69D6,0x4468,
	/* Array index: 0x1047 */ 0x69D7,0x4469,
	/* Array index: 0x1047 */ 0x69E2,0x446A,
	/* Array index: 0x1047 */ 0x69E5,0x446B,
	/* Array index: 0x1047 */ 0x69EE,0x446C,
	/* Array index: 0x1047 */ 0x69EF,0x446D,
	/* Array index: 0x1047 */ 0x69F1,0x446E,
	/* Array index: 0x1047 */ 0x69F3,0x446F,
	/* Array index: 0x1047 */ 0x69F5,0x4470,
	/* Array index: 0x1047 */ 0x69FE,0x4471,
	/* Array index: 0x1047 */ 0x6A00,0x4472,
	/* Array index: 0x1047 */ 0x6A01,0x4473,
	/* Array index: 0x1047 */ 0x6A03,0x4474,
	/* Array index: 0x1047 */ 0x6A0F,0x4475,
	/* Array index: 0x1047 */ 0x6A11,0x4476,
	/* Array index: 0x1047 */ 0x6A15,0x4477,
	/* Array index: 0x1047 */ 0x6A1A,0x4478,
	/* Array index: 0x1047 */ 0x6A1D,0x4479,
	/* Array index: 0x1047 */ 0x6A20,0x447A,
	/* Array index: 0x1047 */ 0x6A24,0x447B,
	/* Array index: 0x1047 */ 0x6A28,0x447C,
	/* Array index: 0x1047 */ 0x6A30,0x447D,
	/* Array index: 0x1047 */ 0x6A32,0x447E,
	/* Array index: 0x1047 */ 0x6A34,0x4521,
	/* Array index: 0x1047 */ 0x6A37,0x4522,
	/* Array index: 0x1047 */ 0x6A3B,0x4523,
	/* Array index: 0x1047 */ 0x6A3E,0x4524,
	/* Array index: 0x1047 */ 0x6A3F,0x4525,
	/* Array index: 0x1047 */ 0x6A45,0x4526,
	/* Array index: 0x1047 */ 0x6A46,0x4527,
	/* Array index: 0x1047 */ 0x6A49,0x4528,
	/* Array index: 0x1047 */ 0x6A4A,0x4529,
	/* Array index: 0x1047 */ 0x6A5B,0x4530,
	/* Array index: 0x1047 */ 0x6A64,0x4531,
	/* Array index: 0x1047 */ 0x6A67,0x4532,
	/* Array index: 0x1047 */ 0x6A6A,0x4533,
	/* Array index: 0x1047 */ 0x6A71,0x4534,
	/* Array index: 0x1047 */ 0x6A73,0x4535,
	/* Array index: 0x1047 */ 0x6A7E,0x4536,
	/* Array index: 0x1047 */ 0x6A81,0x4537,
	/* Array index: 0x1047 */ 0x6A83,0x4538,
	/* Array index: 0x1047 */ 0x6A86,0x4539,
	/* Array index: 0x1047 */ 0x6A87,0x453A,
	/* Array index: 0x1047 */ 0x6A89,0x453B,
	/* Array index: 0x1047 */ 0x6A8B,0x453C,
	/* Array index: 0x1047 */ 0x6A91,0x453D,
	/* Array index: 0x1047 */ 0x6AA5,0x4542,
	/* Array index: 0x1047 */ 0x6AAB,0x4543,
	/* Array index: 0x1047 */ 0x6AB4,0x4547,
	/* Array index: 0x1047 */ 0x6AC6,0x454B,
	/* Array index: 0x1047 */ 0x6AC8,0x454D,
	/* Array index: 0x1047 */ 0x6AC9,0x454C,
	/* Array index: 0x1047 */ 0x6ACC,0x454E,
	/* Array index: 0x1047 */ 0x6AD0,0x454F,
	/* Array index: 0x1047 */ 0x6ADC,0x4553,
	/* Array index: 0x1047 */ 0x6ADD,0x4554,
	/* Array index: 0x1047 */ 0x6AE4,0x4555,
	/* Array index: 0x1047 */ 0x6AE7,0x4556,
	/* Array index: 0x1047 */ 0x6AEC,0x4557,
	/* Array index: 0x1047 */ 0x6AFC,0x455B,
	/* Array index: 0x1047 */ 0x6AFD,0x455C,
	/* Array index: 0x1047 */ 0x6B02,0x455D,
	/* Array index: 0x1047 */ 0x6B03,0x455E,
	/* Array index: 0x1047 */ 0x6B06,0x455F,
	/* Array index: 0x1047 */ 0x6B07,0x4560,
	/* Array index: 0x1047 */ 0x6B09,0x4561,
	/* Array index: 0x1047 */ 0x6B17,0x4565,
	/* Array index: 0x1047 */ 0x6B1B,0x4566,
	/* Array index: 0x1047 */ 0x6B1E,0x4567,
	/* Array index: 0x1047 */ 0x6B24,0x4568,
	/* Array index: 0x1047 */ 0x6B28,0x4569,
	/* Array index: 0x1047 */ 0x6B2B,0x456A,
	/* Array index: 0x1047 */ 0x6B2C,0x456B,
	/* Array index: 0x1047 */ 0x6B2F,0x456C,
	/* Array index: 0x1047 */ 0x6B35,0x456D,
	/* Array index: 0x1047 */ 0x6B36,0x456E,
	/* Array index: 0x1047 */ 0x6B3B,0x456F,
	/* Array index: 0x1047 */ 0x6B3F,0x4570,
	/* Array index: 0x1047 */ 0x6B46,0x4571,
	/* Array index: 0x1047 */ 0x6B4A,0x4572,
	/* Array index: 0x1047 */ 0x6B4D,0x4573,
	/* Array index: 0x1047 */ 0x6B52,0x4574,
	/* Array index: 0x1047 */ 0x6B56,0x4575,
	/* Array index: 0x1047 */ 0x6B58,0x4576,
	/* Array index: 0x1047 */ 0x6B5D,0x4577,
	/* Array index: 0x1047 */ 0x6B60,0x4578,
	/* Array index: 0x1047 */ 0x6B67,0x4579,
	/* Array index: 0x1047 */ 0x6B6B,0x457A,
	/* Array index: 0x1047 */ 0x6B6E,0x457B,
	/* Array index: 0x1047 */ 0x6B70,0x457C,
	/* Array index: 0x1047 */ 0x6B75,0x457D,
	/* Array index: 0x1047 */ 0x6B7D,0x457E,
	/* Array index: 0x1047 */ 0x6B7E,0x4621,
	/* Array index: 0x1047 */ 0x6B82,0x4622,
	/* Array index: 0x1047 */ 0x6B85,0x4623,
	/* Array index: 0x1047 */ 0x6B97,0x4624,
	/* Array index: 0x1047 */ 0x6B9B,0x4625,
	/* Array index: 0x1047 */ 0x6BB8,0x4630,
	/* Array index: 0x1047 */ 0x6BB9,0x4631,
	/* Array index: 0x1047 */ 0x6BBD,0x4632,
	/* Array index: 0x1047 */ 0x6BBE,0x4633,
	/* Array index: 0x1047 */ 0x6BC3,0x4634,
	/* Array index: 0x1047 */ 0x6BC4,0x4635,
	/* Array index: 0x1047 */ 0x6BC9,0x4636,
	/* Array index: 0x1047 */ 0x6BCC,0x4637,
	/* Array index: 0x1047 */ 0x6BD6,0x4638,
	/* Array index: 0x1047 */ 0x6BDA,0x4639,
	/* Array index: 0x1047 */ 0x6BE1,0x463A,
	/* Array index: 0x1047 */ 0x6BE3,0x463B,
	/* Array index: 0x1047 */ 0x6BE6,0x463C,
	/* Array index: 0x1047 */ 0x6BE7,0x463D,
	/* Array index: 0x1047 */ 0x6BEE,0x463E,
	/* Array index: 0x1047 */ 0x6BF1,0x463F,
	/* Array index: 0x1047 */ 0x6BF7,0x4640,
	/* Array index: 0x1047 */ 0x6BF9,0x4641,
	/* Array index: 0x1047 */ 0x6BFF,0x4642,
	/* Array index: 0x1047 */ 0x6C02,0x4643,
	/* Array index: 0x1047 */ 0x6C04,0x4644,
	/* Array index: 0x1047 */ 0x6C05,0x4645,
	/* Array index: 0x1047 */ 0x6C09,0x4646,
	/* Array index: 0x1047 */ 0x6C0D,0x4647,
	/* Array index: 0x1047 */ 0x6C0E,0x4648,
	/* Array index: 0x1047 */ 0x6C10,0x4649,
	/* Array index: 0x1047 */ 0x6C12,0x464A,
	/* Array index: 0x1047 */ 0x6C19,0x464B,
	/* Array index: 0x1047 */ 0x6C1F,0x464C,
	/* Array index: 0x1047 */ 0x6C2C,0x4650,
	/* Array index: 0x1047 */ 0x6C2E,0x4651,
	/* Array index: 0x1047 */ 0x6C33,0x4652,
	/* Array index: 0x1047 */ 0x6C35,0x4653,
	/* Array index: 0x1047 */ 0x6C36,0x4654,
	/* Array index: 0x1047 */ 0x6C3A,0x4655,
	/* Array index: 0x1047 */ 0x6C3B,0x4656,
	/* Array index: 0x1047 */ 0x6C3F,0x4657,
	/* Array index: 0x1047 */ 0x6C4A,0x4658,
	/* Array index: 0x1047 */ 0x6C4B,0x4659,
	/* Array index: 0x1047 */ 0x6C4D,0x465A,
	/* Array index: 0x1047 */ 0x6C4F,0x465B,
	/* Array index: 0x1047 */ 0x6C52,0x465C,
	/* Array index: 0x1047 */ 0x6C54,0x465D,
	/* Array index: 0x1047 */ 0x6C59,0x465E,
	/* Array index: 0x1047 */ 0x6C5B,0x465F,
	/* Array index: 0x1047 */ 0x6C5C,0x4660,
	/* Array index: 0x1047 */ 0x6C67,0x4736,
	/* Array index: 0x1047 */ 0x6C6B,0x4661,
	/* Array index: 0x1047 */ 0x6C6D,0x4662,
	/* Array index: 0x1047 */ 0x6C6F,0x4663,
	/* Array index: 0x1047 */ 0x6C74,0x4664,
	/* Array index: 0x1047 */ 0x6C76,0x4665,
	/* Array index: 0x1047 */ 0x6C78,0x4666,
	/* Array index: 0x1047 */ 0x6C79,0x4667,
	/* Array index: 0x1047 */ 0x6C7B,0x4668,
	/* Array index: 0x1047 */ 0x6C9C,0x4671,
	/* Array index: 0x1047 */ 0x6C9F,0x4672,
	/* Array index: 0x1047 */ 0x6CB0,0x4673,
	/* Array index: 0x1047 */ 0x6CB2,0x4674,
	/* Array index: 0x1047 */ 0x6CB4,0x4675,
	/* Array index: 0x1047 */ 0x6CC2,0x4676,
	/* Array index: 0x1047 */ 0x6CC6,0x4677,
	/* Array index: 0x1047 */ 0x6CDA,0x4721,
	/* Array index: 0x1047 */ 0x6CDC,0x4722,
	/* Array index: 0x1047 */ 0x6CE0,0x4723,
	/* Array index: 0x1047 */ 0x6CE7,0x4724,
	/* Array index: 0x1047 */ 0x6CE9,0x4725,
	/* Array index: 0x1047 */ 0x6CEB,0x4726,
	/* Array index: 0x1047 */ 0x6CEC,0x4727,
	/* Array index: 0x1047 */ 0x6CEE,0x4728,
	/* Array index: 0x1047 */ 0x6CF2,0x4729,
	/* Array index: 0x1047 */ 0x6CF4,0x472A,
	/* Array index: 0x1047 */ 0x6D04,0x472B,
	/* Array index: 0x1047 */ 0x6D07,0x472C,
	/* Array index: 0x1047 */ 0x6D0A,0x472D,
	/* Array index: 0x1047 */ 0x6D0E,0x472E,
	/* Array index: 0x1047 */ 0x6D0F,0x472F,
	/* Array index: 0x1047 */ 0x6D11,0x4730,
	/* Array index: 0x1047 */ 0x6D13,0x4731,
	/* Array index: 0x1047 */ 0x6D1A,0x4732,
	/* Array index: 0x1047 */ 0x6D2E,0x4737,
	/* Array index: 0x1047 */ 0x6D2F,0x4738,
	/* Array index: 0x1047 */ 0x6D31,0x4739,
	/* Array index: 0x1047 */ 0x6D39,0x473A,
	/* Array index: 0x1047 */ 0x6D3C,0x473B,
	/* Array index: 0x1047 */ 0x6D3F,0x473C,
	/* Array index: 0x1047 */ 0x6D57,0x473D,
	/* Array index: 0x1047 */ 0x6D5E,0x473E,
	/* Array index: 0x1047 */ 0x6D5F,0x473F,
	/* Array index: 0x1047 */ 0x6D61,0x4740,
	/* Array index: 0x1047 */ 0x6D65,0x4741,
	/* Array index: 0x1047 */ 0x6D67,0x4742,
	/* Array index: 0x1047 */ 0x6D6F,0x4743,
	/* Array index: 0x1047 */ 0x6D70,0x4744,
	/* Array index: 0x1047 */ 0x6D7C,0x4745,
	/* Array index: 0x1047 */ 0x6D82,0x4746,
	/* Array index: 0x1047 */ 0x6D87,0x4747,
	/* Array index: 0x1047 */ 0x6D91,0x4748,
	/* Array index: 0x1047 */ 0x6D92,0x4749,
	/* Array index: 0x1047 */ 0x6DAA,0x474E,
	/* Array index: 0x1047 */ 0x6DAC,0x474F,
	/* Array index: 0x1047 */ 0x6DB4,0x4750,
	/* Array index: 0x1047 */ 0x6DB7,0x4751,
	/* Array index: 0x1047 */ 0x6DB9,0x4752,
	/* Array index: 0x1047 */ 0x6DBD,0x4753,
	/* Array index: 0x1047 */ 0x6DBF,0x4754,
	/* Array index: 0x1047 */ 0x6DC4,0x4755,
	/* Array index: 0x1047 */ 0x6DC8,0x4756,
	/* Array index: 0x1047 */ 0x6DCA,0x4757,
	/* Array index: 0x1047 */ 0x6DCE,0x4758,
	/* Array index: 0x1047 */ 0x6DCF,0x4759,
	/* Array index: 0x1047 */ 0x6DD6,0x475A,
	/* Array index: 0x1047 */ 0x6DDB,0x475B,
	/* Array index: 0x1047 */ 0x6DDD,0x475C,
	/* Array index: 0x1047 */ 0x6DDF,0x475D,
	/* Array index: 0x1047 */ 0x6DE0,0x475E,
	/* Array index: 0x1047 */ 0x6DE2,0x475F,
	/* Array index: 0x1047 */ 0x6DE5,0x4760,
	/* Array index: 0x1047 */ 0x6DE9,0x4761,
	/* Array index: 0x1047 */ 0x6DEF,0x4762,
	/* Array index: 0x1047 */ 0x6DF0,0x4763,
	/* Array index: 0x1047 */ 0x6DF4,0x4764,
	/* Array index: 0x1047 */ 0x6DF6,0x4765,
	/* Array index: 0x1047 */ 0x6DFC,0x4766,
	/* Array index: 0x1047 */ 0x6E00,0x4767,
	/* Array index: 0x1047 */ 0x6E04,0x4768,
	/* Array index: 0x1047 */ 0x6E1E,0x4769,
	/* Array index: 0x1047 */ 0x6E22,0x476A,
	/* Array index: 0x1047 */ 0x6E27,0x476B,
	/* Array index: 0x1047 */ 0x6E32,0x476C,
	/* Array index: 0x1047 */ 0x6E36,0x476D,
	/* Array index: 0x1047 */ 0x6E39,0x476E,
	/* Array index: 0x1047 */ 0x6E3B,0x476F,
	/* Array index: 0x1047 */ 0x6E3C,0x4770,
	/* Array index: 0x1047 */ 0x6E44,0x4771,
	/* Array index: 0x1047 */ 0x6E45,0x4772,
	/* Array index: 0x1047 */ 0x6E48,0x4773,
	/* Array index: 0x1047 */ 0x6E49,0x4774,
	/* Array index: 0x1047 */ 0x6E4B,0x4775,
	/* Array index: 0x1047 */ 0x6E57,0x477B,
	/* Array index: 0x1047 */ 0x6E62,0x4821,
	/* Array index: 0x1047 */ 0x6E63,0x4822,
	/* Array index: 0x1047 */ 0x6E68,0x4823,
	/* Array index: 0x1047 */ 0x6E73,0x4824,
	/* Array index: 0x1047 */ 0x6E7B,0x4825,
	/* Array index: 0x1047 */ 0x6E7D,0x4826,
	/* Array index: 0x1047 */ 0x6E8D,0x4827,
	/* Array index: 0x1047 */ 0x6E93,0x4828,
	/* Array index: 0x1047 */ 0x6E99,0x4829,
	/* Array index: 0x1047 */ 0x6EA0,0x482A,
	/* Array index: 0x1047 */ 0x6EA7,0x482B,
	/* Array index: 0x1047 */ 0x6EAD,0x482C,
	/* Array index: 0x1047 */ 0x6EAE,0x482D,
	/* Array index: 0x1047 */ 0x6EB1,0x482E,
	/* Array index: 0x1047 */ 0x6EB3,0x482F,
	/* Array index: 0x1047 */ 0x6EBB,0x4830,
	/* Array index: 0x1047 */ 0x6EC7,0x4835,
	/* Array index: 0x1047 */ 0x6EC8,0x4836,
	/* Array index: 0x1047 */ 0x6ECA,0x4837,
	/* Array index: 0x1047 */ 0x6EEB,0x483B,
	/* Array index: 0x1047 */ 0x6EED,0x483C,
	/* Array index: 0x1047 */ 0x6EEE,0x483D,
	/* Array index: 0x1047 */ 0x6EF9,0x483E,
	/* Array index: 0x1047 */ 0x6EFB,0x483F,
	/* Array index: 0x1047 */ 0x6EFD,0x4840,
	/* Array index: 0x1047 */ 0x6F04,0x4841,
	/* Array index: 0x1047 */ 0x6F08,0x4842,
	/* Array index: 0x1047 */ 0x6F0A,0x4843,
	/* Array index: 0x1047 */ 0x6F0C,0x4844,
	/* Array index: 0x1047 */ 0x6F0D,0x4845,
	/* Array index: 0x1047 */ 0x6F16,0x4846,
	/* Array index: 0x1047 */ 0x6F18,0x4847,
	/* Array index: 0x1047 */ 0x6F1A,0x4848,
	/* Array index: 0x1047 */ 0x6F1B,0x4849,
	/* Array index: 0x1047 */ 0x6F26,0x484A,
	/* Array index: 0x1047 */ 0x6F29,0x484B,
	/* Array index: 0x1047 */ 0x6F2A,0x484C,
	/* Array index: 0x1047 */ 0x6F2D,0x4853,
	/* Array index: 0x1047 */ 0x6F2F,0x484D,
	/* Array index: 0x1047 */ 0x6F30,0x484E,
	/* Array index: 0x1047 */ 0x6F33,0x484F,
	/* Array index: 0x1047 */ 0x6F36,0x4850,
	/* Array index: 0x1047 */ 0x6F3B,0x4851,
	/* Array index: 0x1047 */ 0x6F3C,0x4852,
	/* Array index: 0x1047 */ 0x6F57,0x4858,
	/* Array index: 0x1047 */ 0x6F59,0x4859,
	/* Array index: 0x1047 */ 0x6F5A,0x485A,
	/* Array index: 0x1047 */ 0x6F5D,0x485B,
	/* Array index: 0x1047 */ 0x6F5E,0x485C,
	/* Array index: 0x1047 */ 0x6F61,0x485D,
	/* Array index: 0x1047 */ 0x6F62,0x485E,
	/* Array index: 0x1047 */ 0x6F68,0x485F,
	/* Array index: 0x1047 */ 0x6F6C,0x4860,
	/* Array index: 0x1047 */ 0x6F7D,0x4861,
	/* Array index: 0x1047 */ 0x6F7E,0x4862,
	/* Array index: 0x1047 */ 0x6F83,0x4863,
	/* Array index: 0x1047 */ 0x6F9A,0x486E,
	/* Array index: 0x1047 */ 0x6F9F,0x486F,
	/* Array index: 0x1047 */ 0x6FA0,0x4870,
	/* Array index: 0x1047 */ 0x6FB5,0x4878,
	/* Array index: 0x1047 */ 0x6FB6,0x4879,
	/* Array index: 0x1047 */ 0x6FBC,0x487A,
	/* Array index: 0x1047 */ 0x6FC5,0x487B,
	/* Array index: 0x1047 */ 0x6FC7,0x487C,
	/* Array index: 0x1047 */ 0x6FC8,0x487D,
	/* Array index: 0x1047 */ 0x6FCA,0x487E,
	/* Array index: 0x1047 */ 0x6FDA,0x4921,
	/* Array index: 0x1047 */ 0x6FDE,0x4922,
	/* Array index: 0x1047 */ 0x6FE8,0x4923,
	/* Array index: 0x1047 */ 0x6FE9,0x4924,
	/* Array index: 0x1047 */ 0x6FF0,0x4925,
	/* Array index: 0x1047 */ 0x6FF5,0x4926,
	/* Array index: 0x1047 */ 0x6FF9,0x4927,
	/* Array index: 0x1047 */ 0x6FFC,0x4928,
	/* Array index: 0x1047 */ 0x6FFD,0x4929,
	/* Array index: 0x1047 */ 0x7000,0x492A,
	/* Array index: 0x1047 */ 0x700D,0x492E,
	/* Array index: 0x1047 */ 0x7017,0x492F,
	/* Array index: 0x1047 */ 0x7020,0x4930,
	/* Array index: 0x1047 */ 0x7023,0x4931,
	/* Array index: 0x1047 */ 0x702F,0x4932,
	/* Array index: 0x1047 */ 0x7034,0x4933,
	/* Array index: 0x1047 */ 0x7037,0x4934,
	/* Array index: 0x1047 */ 0x7039,0x4935,
	/* Array index: 0x1047 */ 0x703C,0x4936,
	/* Array index: 0x1047 */ 0x7043,0x4937,
	/* Array index: 0x1047 */ 0x7044,0x4938,
	/* Array index: 0x1047 */ 0x704E,0x4941,
	/* Array index: 0x1047 */ 0x7054,0x493D,
	/* Array index: 0x1047 */ 0x7055,0x493E,
	/* Array index: 0x1047 */ 0x705D,0x493F,
	/* Array index: 0x1047 */ 0x705E,0x4940,
	/* Array index: 0x1047 */ 0x7064,0x4942,
	/* Array index: 0x1047 */ 0x7065,0x4943,
	/* Array index: 0x1047 */ 0x706C,0x4944,
	/* Array index: 0x1047 */ 0x706E,0x4945,
	/* Array index: 0x1047 */ 0x7075,0x4946,
	/* Array index: 0x1047 */ 0x7076,0x4947,
	/* Array index: 0x1047 */ 0x707E,0x4948,
	/* Array index: 0x1047 */ 0x7081,0x4949,
	/* Array index: 0x1047 */ 0x7085,0x494A,
	/* Array index: 0x1047 */ 0x7086,0x494B,
	/* Array index: 0x1047 */ 0x709B,0x4951,
	/* Array index: 0x1047 */ 0x70A4,0x4952,
	/* Array index: 0x1047 */ 0x70AB,0x4953,
	/* Array index: 0x1047 */ 0x70B0,0x4954,
	/* Array index: 0x1047 */ 0x70B1,0x4955,
	/* Array index: 0x1047 */ 0x70B4,0x4956,
	/* Array index: 0x1047 */ 0x70B7,0x4957,
	/* Array index: 0x1047 */ 0x70CA,0x4958,
	/* Array index: 0x1047 */ 0x70DC,0x495F,
	/* Array index: 0x1047 */ 0x70E4,0x4960,
	/* Array index: 0x1047 */ 0x70FA,0x4961,
	/* Array index: 0x1047 */ 0x710B,0x4967,
	/* Array index: 0x1047 */ 0x710C,0x4968,
	/* Array index: 0x1047 */ 0x710F,0x4969,
	/* Array index: 0x1047 */ 0x711E,0x496A,
	/* Array index: 0x1047 */ 0x7120,0x496B,
	/* Array index: 0x1047 */ 0x712B,0x496C,
	/* Array index: 0x1047 */ 0x7138,0x4971,
	/* Array index: 0x1047 */ 0x7141,0x4972,
	/* Array index: 0x1047 */ 0x7150,0x4978,
	/* Array index: 0x1047 */ 0x7152,0x4979,
	/* Array index: 0x1047 */ 0x7157,0x497A,
	/* Array index: 0x1047 */ 0x715A,0x497B,
	/* Array index: 0x1047 */ 0x715C,0x497C,
	/* Array index: 0x1047 */ 0x715E,0x497D,
	/* Array index: 0x1047 */ 0x7160,0x497E,
	/* Array index: 0x1047 */ 0x7168,0x4A21,
	/* Array index: 0x1047 */ 0x7179,0x4A22,
	/* Array index: 0x1047 */ 0x7180,0x4A23,
	/* Array index: 0x1047 */ 0x7185,0x4A24,
	/* Array index: 0x1047 */ 0x7187,0x4A25,
	/* Array index: 0x1047 */ 0x718C,0x4A26,
	/* Array index: 0x1047 */ 0x7192,0x4A27,
	/* Array index: 0x1047 */ 0x719A,0x4A28,
	/* Array index: 0x1047 */ 0x719B,0x4A29,
	/* Array index: 0x1047 */ 0x71A0,0x4A2A,
	/* Array index: 0x1047 */ 0x71A2,0x4A2B,
	/* Array index: 0x1047 */ 0x71BA,0x4A30,
	/* Array index: 0x1047 */ 0x71C4,0x4A34,
	/* Array index: 0x1047 */ 0x71CB,0x4A35,
	/* Array index: 0x1047 */ 0x71CC,0x4A36,
	/* Array index: 0x1047 */ 0x71D3,0x4A37,
	/* Array index: 0x1047 */ 0x71D6,0x4A38,
	/* Array index: 0x1047 */ 0x71D9,0x4A39,
	/* Array index: 0x1047 */ 0x71DA,0x4A3A,
	/* Array index: 0x1047 */ 0x71DC,0x4A3B,
	/* Array index: 0x1047 */ 0x71F8,0x4A3C,
	/* Array index: 0x1047 */ 0x71FE,0x4A3D,
	/* Array index: 0x1047 */ 0x7200,0x4A3E,
	/* Array index: 0x1047 */ 0x7213,0x4A42,
	/* Array index: 0x1047 */ 0x7217,0x4A43,
	/* Array index: 0x1047 */ 0x721A,0x4A44,
	/* Array index: 0x1047 */ 0x721D,0x4A45,
	/* Array index: 0x1047 */ 0x721F,0x4A46,
	/* Array index: 0x1047 */ 0x7224,0x4A47,
	/* Array index: 0x1047 */ 0x722B,0x4A48,
	/* Array index: 0x1047 */ 0x722F,0x4A49,
	/* Array index: 0x1047 */ 0x7234,0x4A4A,
	/* Array index: 0x1047 */ 0x7238,0x4A4B,
	/* Array index: 0x1047 */ 0x7239,0x4A4C,
	/* Array index: 0x1047 */ 0x7253,0x4A54,
	/* Array index: 0x1047 */ 0x7255,0x4A55,
	/* Array index: 0x1047 */ 0x7256,0x4A56,
	/* Array index: 0x1047 */ 0x725A,0x4A57,
	/* Array index: 0x1047 */ 0x725C,0x4A58,
	/* Array index: 0x1047 */ 0x725E,0x4A59,
	/* Array index: 0x1047 */ 0x7260,0x4A5A,
	/* Array index: 0x1047 */ 0x7263,0x4A5B,
	/* Array index: 0x1047 */ 0x7268,0x4A5C,
	/* Array index: 0x1047 */ 0x726B,0x4A5D,
	/* Array index: 0x1047 */ 0x726E,0x4A5E,
	/* Array index: 0x1047 */ 0x726F,0x4A5F,
	/* Array index: 0x1047 */ 0x7271,0x4A60,
	/* Array index: 0x1047 */ 0x7277,0x4A61,
	/* Array index: 0x1047 */ 0x7278,0x4A62,
	/* Array index: 0x1047 */ 0x727B,0x4A63,
	/* Array index: 0x1047 */ 0x727C,0x4A64,
	/* Array index: 0x1047 */ 0x727F,0x4A65,
	/* Array index: 0x1047 */ 0x7284,0x4A66,
	/* Array index: 0x1047 */ 0x7289,0x4A67,
	/* Array index: 0x1047 */ 0x728D,0x4A68,
	/* Array index: 0x1047 */ 0x728E,0x4A69,
	/* Array index: 0x1047 */ 0x7293,0x4A6A,
	/* Array index: 0x1047 */ 0x729B,0x4A6B,
	/* Array index: 0x1047 */ 0x72A8,0x4A6C,
	/* Array index: 0x1047 */ 0x72AD,0x4A6D,
	/* Array index: 0x1047 */ 0x72AE,0x4A6E,
	/* Array index: 0x1047 */ 0x72B1,0x4A6F,
	/* Array index: 0x1047 */ 0x72B4,0x4A70,
	/* Array index: 0x1047 */ 0x72BE,0x4A71,
	/* Array index: 0x1047 */ 0x72C1,0x4A72,
	/* Array index: 0x1047 */ 0x72C7,0x4A73,
	/* Array index: 0x1047 */ 0x72C9,0x4A74,
	/* Array index: 0x1047 */ 0x72CC,0x4A75,
	/* Array index: 0x1047 */ 0x72D5,0x4A76,
	/* Array index: 0x1047 */ 0x72D6,0x4A77,
	/* Array index: 0x1047 */ 0x72D8,0x4A78,
	/* Array index: 0x1047 */ 0x72DF,0x4A79,
	/* Array index: 0x1047 */ 0x72E5,0x4A7A,
	/* Array index: 0x1047 */ 0x72F3,0x4A7B,
	/* Array index: 0x1047 */ 0x72F4,0x4A7C,
	/* Array index: 0x1047 */ 0x72FA,0x4A7D,
	/* Array index: 0x1047 */ 0x72FB,0x4A7E,
	/* Array index: 0x1047 */ 0x72FE,0x4B21,
	/* Array index: 0x1047 */ 0x7302,0x4B22,
	/* Array index: 0x1047 */ 0x7304,0x4B23,
	/* Array index: 0x1047 */ 0x7305,0x4B24,
	/* Array index: 0x1047 */ 0x7307,0x4B25,
	/* Array index: 0x1047 */ 0x730B,0x4B26,
	/* Array index: 0x1047 */ 0x730D,0x4B27,
	/* Array index: 0x1047 */ 0x7312,0x4B28,
	/* Array index: 0x1047 */ 0x7313,0x4B29,
	/* Array index: 0x1047 */ 0x7318,0x4B2A,
	/* Array index: 0x1047 */ 0x7319,0x4B2B,
	/* Array index: 0x1047 */ 0x731E,0x4B2C,
	/* Array index: 0x1047 */ 0x7322,0x4B2D,
	/* Array index: 0x1047 */ 0x7324,0x4B2E,
	/* Array index: 0x1047 */ 0x7327,0x4B2F,
	/* Array index: 0x1047 */ 0x7328,0x4B30,
	/* Array index: 0x1047 */ 0x732C,0x4B31,
	/* Array index: 0x1047 */ 0x7331,0x4B32,
	/* Array index: 0x1047 */ 0x7332,0x4B33,
	/* Array index: 0x1047 */ 0x7335,0x4B34,
	/* Array index: 0x1047 */ 0x733A,0x4B35,
	/* Array index: 0x1047 */ 0x733B,0x4B36,
	/* Array index: 0x1047 */ 0x733D,0x4B37,
	/* Array index: 0x1047 */ 0x7343,0x4B38,
	/* Array index: 0x1047 */ 0x734D,0x4B39,
	/* Array index: 0x1047 */ 0x7350,0x4B3A,
	/* Array index: 0x1047 */ 0x7352,0x4B3B,
	/* Array index: 0x1047 */ 0x7356,0x4B3C,
	/* Array index: 0x1047 */ 0x7358,0x4B3D,
	/* Array index: 0x1047 */ 0x7366,0x4B42,
	/* Array index: 0x1047 */ 0x7367,0x4B43,
	/* Array index: 0x1047 */ 0x7369,0x4B44,
	/* Array index: 0x1047 */ 0x7377,0x4B4A,
	/* Array index: 0x1047 */ 0x7379,0x4B4B,
	/* Array index: 0x1047 */ 0x737C,0x4B4C,
	/* Array index: 0x1047 */ 0x7380,0x4B4D,
	/* Array index: 0x1047 */ 0x7381,0x4B4E,
	/* Array index: 0x1047 */ 0x7383,0x4B4F,
	/* Array index: 0x1047 */ 0x7385,0x4B50,
	/* Array index: 0x1047 */ 0x7386,0x4B51,
	/* Array index: 0x1047 */ 0x738E,0x4B52,
	/* Array index: 0x1047 */ 0x7390,0x4B53,
	/* Array index: 0x1047 */ 0x7393,0x4B54,
	/* Array index: 0x1047 */ 0x7395,0x4B55,
	/* Array index: 0x1047 */ 0x7397,0x4B56,
	/* Array index: 0x1047 */ 0x7398,0x4B57,
	/* Array index: 0x1047 */ 0x73AA,0x4B5F,
	/* Array index: 0x1047 */ 0x73AB,0x4B60,
	/* Array index: 0x1047 */ 0x73AD,0x4B61,
	/* Array index: 0x1047 */ 0x73B5,0x4B62,
	/* Array index: 0x1047 */ 0x73B7,0x4B63,
	/* Array index: 0x1047 */ 0x73B9,0x4B64,
	/* Array index: 0x1047 */ 0x73BC,0x4B65,
	/* Array index: 0x1047 */ 0x73BD,0x4B66,
	/* Array index: 0x1047 */ 0x73BF,0x4B67,
	/* Array index: 0x1047 */ 0x73C5,0x4B68,
	/* Array index: 0x1047 */ 0x73C6,0x4B69,
	/* Array index: 0x1047 */ 0x73C9,0x4B6A,
	/* Array index: 0x1047 */ 0x73CB,0x4B6B,
	/* Array index: 0x1047 */ 0x73CC,0x4B6C,
	/* Array index: 0x1047 */ 0x73CF,0x4B6D,
	/* Array index: 0x1047 */ 0x73D2,0x4B6E,
	/* Array index: 0x1047 */ 0x73D3,0x4B6F,
	/* Array index: 0x1047 */ 0x73D6,0x4B70,
	/* Array index: 0x1047 */ 0x73D9,0x4B71,
	/* Array index: 0x1047 */ 0x73DD,0x4B72,
	/* Array index: 0x1047 */ 0x73E1,0x4B73,
	/* Array index: 0x1047 */ 0x73E3,0x4B74,
	/* Array index: 0x1047 */ 0x73E6,0x4B75,
	/* Array index: 0x1047 */ 0x73E7,0x4B76,
	/* Array index: 0x1047 */ 0x73E9,0x4B77,
	/* Array index: 0x1047 */ 0x73F4,0x4B78,
	/* Array index: 0x1047 */ 0x73F5,0x4B79,
	/* Array index: 0x1047 */ 0x7404,0x4C24,
	/* Array index: 0x1047 */ 0x7407,0x4C25,
	/* Array index: 0x1047 */ 0x740A,0x4C26,
	/* Array index: 0x1047 */ 0x7411,0x4C27,
	/* Array index: 0x1047 */ 0x741A,0x4C28,
	/* Array index: 0x1047 */ 0x741B,0x4C29,
	/* Array index: 0x1047 */ 0x7424,0x4C2A,
	/* Array index: 0x1047 */ 0x7439,0x4C36,
	/* Array index: 0x1047 */ 0x7440,0x4C37,
	/* Array index: 0x1047 */ 0x744B,0x4C3C,
	/* Array index: 0x1047 */ 0x744D,0x4C3D,
	/* Array index: 0x1047 */ 0x7451,0x4C3E,
	/* Array index: 0x1047 */ 0x7452,0x4C3F,
	/* Array index: 0x1047 */ 0x7457,0x4C40,
	/* Array index: 0x1047 */ 0x745D,0x4C41,
	/* Array index: 0x1047 */ 0x7462,0x4C42,
	/* Array index: 0x1047 */ 0x746B,0x4C46,
	/* Array index: 0x1047 */ 0x746D,0x4C47,
	/* Array index: 0x1047 */ 0x746E,0x4C48,
	/* Array index: 0x1047 */ 0x7471,0x4C49,
	/* Array index: 0x1047 */ 0x7472,0x4C4A,
	/* Array index: 0x1047 */ 0x7480,0x4C4B,
	/* Array index: 0x1047 */ 0x7481,0x4C4C,
	/* Array index: 0x1047 */ 0x74B5,0x4C66,
	/* Array index: 0x1047 */ 0x74B9,0x4C67,
	/* Array index: 0x1047 */ 0x74BB,0x4C68,
	/* Array index: 0x1047 */ 0x74BF,0x4C69,
	/* Array index: 0x1047 */ 0x74C8,0x4C6A,
	/* Array index: 0x1047 */ 0x74C9,0x4C6B,
	/* Array index: 0x1047 */ 0x74CC,0x4C6C,
	/* Array index: 0x1047 */ 0x74D0,0x4C6D,
	/* Array index: 0x1047 */ 0x74D3,0x4C6E,
	/* Array index: 0x1047 */ 0x74D8,0x4C6F,
	/* Array index: 0x1047 */ 0x74DA,0x4C70,
	/* Array index: 0x1047 */ 0x74DB,0x4C71,
	/* Array index: 0x1047 */ 0x74DE,0x4C72,
	/* Array index: 0x1047 */ 0x74DF,0x4C73,
	/* Array index: 0x1047 */ 0x74E4,0x4C74,
	/* Array index: 0x1047 */ 0x74E8,0x4C75,
	/* Array index: 0x1047 */ 0x74EA,0x4C76,
	/* Array index: 0x1047 */ 0x74EB,0x4C77,
	/* Array index: 0x1047 */ 0x74EF,0x4C78,
	/* Array index: 0x1047 */ 0x74F4,0x4C79,
	/* Array index: 0x1047 */ 0x74FF,0x4C7D,
	/* Array index: 0x1047 */ 0x7506,0x4C7E,
	/* Array index: 0x1047 */ 0x7512,0x4D21,
	/* Array index: 0x1047 */ 0x7516,0x4D22,
	/* Array index: 0x1047 */ 0x7517,0x4D23,
	/* Array index: 0x1047 */ 0x7520,0x4D24,
	/* Array index: 0x1047 */ 0x7521,0x4D25,
	/* Array index: 0x1047 */ 0x7524,0x4D26,
	/* Array index: 0x1047 */ 0x7527,0x4D27,
	/* Array index: 0x1047 */ 0x7529,0x4D28,
	/* Array index: 0x1047 */ 0x752A,0x4D29,
	/* Array index: 0x1047 */ 0x752F,0x4D2A,
	/* Array index: 0x1047 */ 0x7536,0x4D2B,
	/* Array index: 0x1047 */ 0x7539,0x4D2C,
	/* Array index: 0x1047 */ 0x7543,0x4D31,
	/* Array index: 0x1047 */ 0x7547,0x4D32,
	/* Array index: 0x1047 */ 0x7548,0x4D33,
	/* Array index: 0x1047 */ 0x754E,0x4D34,
	/* Array index: 0x1047 */ 0x7550,0x4D35,
	/* Array index: 0x1047 */ 0x7552,0x4D36,
	/* Array index: 0x1047 */ 0x7557,0x4D37,
	/* Array index: 0x1047 */ 0x755E,0x4D38,
	/* Array index: 0x1047 */ 0x755F,0x4D39,
	/* Array index: 0x1047 */ 0x7561,0x4D3A,
	/* Array index: 0x1047 */ 0x756F,0x4D3B,
	/* Array index: 0x1047 */ 0x7571,0x4D3C,
	/* Array index: 0x1047 */ 0x7581,0x4D43,
	/* Array index: 0x1047 */ 0x7585,0x4D44,
	/* Array index: 0x1047 */ 0x7590,0x4D45,
	/* Array index: 0x1047 */ 0x7592,0x4D46,
	/* Array index: 0x1047 */ 0x7593,0x4D47,
	/* Array index: 0x1047 */ 0x7595,0x4D48,
	/* Array index: 0x1047 */ 0x7599,0x4D49,
	/* Array index: 0x1047 */ 0x759C,0x4D4A,
	/* Array index: 0x1047 */ 0x75A2,0x4D4B,
	/* Array index: 0x1047 */ 0x75A4,0x4D4C,
	/* Array index: 0x1047 */ 0x75B4,0x4D4D,
	/* Array index: 0x1047 */ 0x75BA,0x4D4E,
	/* Array index: 0x1047 */ 0x75C4,0x4D52,
	/* Array index: 0x1047 */ 0x75C6,0x4D53,
	/* Array index: 0x1047 */ 0x75CC,0x4D54,
	/* Array index: 0x1047 */ 0x75CE,0x4D55,
	/* Array index: 0x1047 */ 0x75CF,0x4D56,
	/* Array index: 0x1047 */ 0x75D7,0x4D57,
	/* Array index: 0x1047 */ 0x75DC,0x4D58,
	/* Array index: 0x1047 */ 0x75E4,0x4D5C,
	/* Array index: 0x1047 */ 0x75E7,0x4D5D,
	/* Array index: 0x1047 */ 0x75EC,0x4D5E,
	/* Array index: 0x1047 */ 0x75EE,0x4D5F,
	/* Array index: 0x1047 */ 0x75EF,0x4D60,
	/* Array index: 0x1047 */ 0x75F1,0x4D61,
	/* Array index: 0x1047 */ 0x75F9,0x4D62,
	/* Array index: 0x1047 */ 0x760F,0x4D6B,
	/* Array index: 0x1047 */ 0x7623,0x4D75,
	/* Array index: 0x1047 */ 0x7625,0x4D76,
	/* Array index: 0x1047 */ 0x7626,0x4D77,
	/* Array index: 0x1047 */ 0x7629,0x4D78,
	/* Array index: 0x1047 */ 0x762D,0x4D79,
	/* Array index: 0x1047 */ 0x7632,0x4D7A,
	/* Array index: 0x1047 */ 0x7633,0x4D7B,
	/* Array index: 0x1047 */ 0x7635,0x4D7C,
	/* Array index: 0x1047 */ 0x7655,0x4E2B,
	/* Array index: 0x1047 */ 0x7659,0x4E2C,
	/* Array index: 0x1047 */ 0x765F,0x4E2D,
	/* Array index: 0x1047 */ 0x7664,0x4E2E,
	/* Array index: 0x1047 */ 0x7665,0x4E2F,
	/* Array index: 0x1047 */ 0x7674,0x4E34,
	/* Array index: 0x1047 */ 0x7681,0x4E35,
	/* Array index: 0x1047 */ 0x7685,0x4E36,
	/* Array index: 0x1047 */ 0x768C,0x4E37,
	/* Array index: 0x1047 */ 0x768D,0x4E38,
	/* Array index: 0x1047 */ 0x7695,0x4E39,
	/* Array index: 0x1047 */ 0x76AD,0x4E47,
	/* Array index: 0x1047 */ 0x76BD,0x4E48,
	/* Array index: 0x1047 */ 0x76C1,0x4E49,
	/* Array index: 0x1047 */ 0x76C5,0x4E4A,
	/* Array index: 0x1047 */ 0x76C9,0x4E4B,
	/* Array index: 0x1047 */ 0x76CB,0x4E4C,
	/* Array index: 0x1047 */ 0x76CC,0x4E4D,
	/* Array index: 0x1047 */ 0x76CE,0x4E4E,
	/* Array index: 0x1047 */ 0x76D4,0x4E4F,
	/* Array index: 0x1047 */ 0x76D9,0x4E50,
	/* Array index: 0x1047 */ 0x76E0,0x4E51,
	/* Array index: 0x1047 */ 0x76E6,0x4E52,
	/* Array index: 0x1047 */ 0x76E8,0x4E53,
	/* Array index: 0x1047 */ 0x76EC,0x4E54,
	/* Array index: 0x1047 */ 0x76F0,0x4E55,
	/* Array index: 0x1047 */ 0x76F1,0x4E56,
	/* Array index: 0x1047 */ 0x76F6,0x4E57,
	/* Array index: 0x1047 */ 0x76F9,0x4E58,
	/* Array index: 0x1047 */ 0x76FC,0x4E59,
	/* Array index: 0x1047 */ 0x7700,0x4E5A,
	/* Array index: 0x1047 */ 0x7706,0x4E5B,
	/* Array index: 0x1047 */ 0x770A,0x4E5C,
	/* Array index: 0x1047 */ 0x770E,0x4E5D,
	/* Array index: 0x1047 */ 0x7712,0x4E5E,
	/* Array index: 0x1047 */ 0x7714,0x4E5F,
	/* Array index: 0x1047 */ 0x7715,0x4E60,
	/* Array index: 0x1047 */ 0x7717,0x4E61,
	/* Array index: 0x1047 */ 0x7719,0x4E62,
	/* Array index: 0x1047 */ 0x771A,0x4E63,
	/* Array index: 0x1047 */ 0x771C,0x4E64,
	/* Array index: 0x1047 */ 0x7722,0x4E65,
	/* Array index: 0x1047 */ 0x7728,0x4E66,
	/* Array index: 0x1047 */ 0x7739,0x4E6D,
	/* Array index: 0x1047 */ 0x773D,0x4E6E,
	/* Array index: 0x1047 */ 0x773E,0x4E6F,
	/* Array index: 0x1047 */ 0x7742,0x4E70,
	/* Array index: 0x1047 */ 0x7745,0x4E71,
	/* Array index: 0x1047 */ 0x7746,0x4E72,
	/* Array index: 0x1047 */ 0x774A,0x4E73,
	/* Array index: 0x1047 */ 0x7752,0x4E77,
	/* Array index: 0x1047 */ 0x7756,0x4E78,
	/* Array index: 0x1047 */ 0x7757,0x4E79,
	/* Array index: 0x1047 */ 0x7767,0x4F22,
	/* Array index: 0x1047 */ 0x776A,0x4F23,
	/* Array index: 0x1047 */ 0x776C,0x4F24,
	/* Array index: 0x1047 */ 0x777A,0x4F29,
	/* Array index: 0x1047 */ 0x777D,0x4F2A,
	/* Array index: 0x1047 */ 0x7780,0x4F2B,
	/* Array index: 0x1047 */ 0x7784,0x4F2C,
	/* Array index: 0x1047 */ 0x778C,0x4F2D,
	/* Array index: 0x1047 */ 0x778D,0x4F2E,
	/* Array index: 0x1047 */ 0x779A,0x4F32,
	/* Array index: 0x1047 */ 0x779F,0x4F33,
	/* Array index: 0x1047 */ 0x77A2,0x4F34,
	/* Array index: 0x1047 */ 0x77A7,0x4F35,
	/* Array index: 0x1047 */ 0x77AA,0x4F36,
	/* Array index: 0x1047 */ 0x77AE,0x4F37,
	/* Array index: 0x1047 */ 0x77AF,0x4F38,
	/* Array index: 0x1047 */ 0x77B1,0x4F39,
	/* Array index: 0x1047 */ 0x77B5,0x4F3A,
	/* Array index: 0x1047 */ 0x77BE,0x4F3B,
	/* Array index: 0x1047 */ 0x77C3,0x4F3C,
	/* Array index: 0x1047 */ 0x77C9,0x4F3D,
	/* Array index: 0x1047 */ 0x77D1,0x4F3E,
	/* Array index: 0x1047 */ 0x77D2,0x4F3F,
	/* Array index: 0x1047 */ 0x77D5,0x4F40,
	/* Array index: 0x1047 */ 0x77D9,0x4F41,
	/* Array index: 0x1047 */ 0x77E4,0x4F45,
	/* Array index: 0x1047 */ 0x77E6,0x4F46,
	/* Array index: 0x1047 */ 0x77EA,0x4F47,
	/* Array index: 0x1047 */ 0x77EC,0x4F48,
	/* Array index: 0x1047 */ 0x77F0,0x4F49,
	/* Array index: 0x1047 */ 0x77F1,0x4F4A,
	/* Array index: 0x1047 */ 0x77F4,0x4F4B,
	/* Array index: 0x1047 */ 0x77F8,0x4F4C,
	/* Array index: 0x1047 */ 0x77FB,0x4F4D,
	/* Array index: 0x1047 */ 0x7805,0x4F4E,
	/* Array index: 0x1047 */ 0x7806,0x4F4F,
	/* Array index: 0x1047 */ 0x7809,0x4F50,
	/* Array index: 0x1047 */ 0x780D,0x4F51,
	/* Array index: 0x1047 */ 0x780E,0x4F52,
	/* Array index: 0x1047 */ 0x7811,0x4F53,
	/* Array index: 0x1047 */ 0x781D,0x4F54,
	/* Array index: 0x1047 */ 0x782D,0x4F58,
	/* Array index: 0x1047 */ 0x782E,0x4F59,
	/* Array index: 0x1047 */ 0x7830,0x4F5A,
	/* Array index: 0x1047 */ 0x7835,0x4F5B,
	/* Array index: 0x1047 */ 0x7837,0x4F5C,
	/* Array index: 0x1047 */ 0x7843,0x4F5D,
	/* Array index: 0x1047 */ 0x7844,0x4F5E,
	/* Array index: 0x1047 */ 0x7847,0x4F5F,
	/* Array index: 0x1047 */ 0x7848,0x4F60,
	/* Array index: 0x1047 */ 0x784C,0x4F61,
	/* Array index: 0x1047 */ 0x784E,0x4F62,
	/* Array index: 0x1047 */ 0x7852,0x4F63,
	/* Array index: 0x1047 */ 0x785C,0x4F64,
	/* Array index: 0x1047 */ 0x785E,0x4F65,
	/* Array index: 0x1047 */ 0x7868,0x4F6A,
	/* Array index: 0x1047 */ 0x786A,0x4F6B,
	/* Array index: 0x1047 */ 0x786E,0x4F6C,
	/* Array index: 0x1047 */ 0x787A,0x4F6D,
	/* Array index: 0x1047 */ 0x787E,0x4F6E,
	/* Array index: 0x1047 */ 0x788A,0x4F6F,
	/* Array index: 0x1047 */ 0x788F,0x4F70,
	/* Array index: 0x1047 */ 0x7894,0x4F71,
	/* Array index: 0x1047 */ 0x7898,0x4F72,
	/* Array index: 0x1047 */ 0x78A4,0x4F77,
	/* Array index: 0x1047 */ 0x78A8,0x4F78,
	/* Array index: 0x1047 */ 0x78BB,0x5021,
	/* Array index: 0x1047 */ 0x78BD,0x5022,
	/* Array index: 0x1047 */ 0x78BF,0x5023,
	/* Array index: 0x1047 */ 0x78CC,0x5027,
	/* Array index: 0x1047 */ 0x78CE,0x5028,
	/* Array index: 0x1047 */ 0x78DB,0x502E,
	/* Array index: 0x1047 */ 0x78E4,0x502D,
	/* Array index: 0x1047 */ 0x78E6,0x5032,
	/* Array index: 0x1047 */ 0x78EA,0x5033,
	/* Array index: 0x1047 */ 0x78F2,0x5034,
	/* Array index: 0x1047 */ 0x78F3,0x5035,
	/* Array index: 0x1047 */ 0x78F6,0x5037,
	/* Array index: 0x1047 */ 0x78F7,0x5038,
	/* Array index: 0x1047 */ 0x78FA,0x5039,
	/* Array index: 0x1047 */ 0x78FB,0x503A,
	/* Array index: 0x1047 */ 0x78FF,0x503B,
	/* Array index: 0x1047 */ 0x7900,0x5036,
	/* Array index: 0x1047 */ 0x7906,0x503C,
	/* Array index: 0x1047 */ 0x790C,0x503D,
	/* Array index: 0x1047 */ 0x7910,0x503E,
	/* Array index: 0x1047 */ 0x791A,0x503F,
	/* Array index: 0x1047 */ 0x7925,0x5044,
	/* Array index: 0x1047 */ 0x7927,0x5045,
	/* Array index: 0x1047 */ 0x7929,0x5046,
	/* Array index: 0x1047 */ 0x792D,0x5047,
	/* Array index: 0x1047 */ 0x7931,0x5048,
	/* Array index: 0x1047 */ 0x7934,0x5049,
	/* Array index: 0x1047 */ 0x7935,0x504A,
	/* Array index: 0x1047 */ 0x793B,0x504B,
	/* Array index: 0x1047 */ 0x793D,0x504C,
	/* Array index: 0x1047 */ 0x793F,0x504D,
	/* Array index: 0x1047 */ 0x794A,0x5051,
	/* Array index: 0x1047 */ 0x794B,0x5052,
	/* Array index: 0x1047 */ 0x794F,0x5053,
	/* Array index: 0x1047 */ 0x7951,0x5054,
	/* Array index: 0x1047 */ 0x7954,0x5055,
	/* Array index: 0x1047 */ 0x7958,0x5056,
	/* Array index: 0x1047 */ 0x795B,0x5057,
	/* Array index: 0x1047 */ 0x795C,0x5058,
	/* Array index: 0x1047 */ 0x7967,0x5059,
	/* Array index: 0x1047 */ 0x7969,0x505A,
	/* Array index: 0x1047 */ 0x796B,0x505B,
	/* Array index: 0x1047 */ 0x7972,0x505C,
	/* Array index: 0x1047 */ 0x7979,0x505D,
	/* Array index: 0x1047 */ 0x797B,0x505E,
	/* Array index: 0x1047 */ 0x797C,0x505F,
	/* Array index: 0x1047 */ 0x797E,0x5060,
	/* Array index: 0x1047 */ 0x798B,0x5061,
	/* Array index: 0x1047 */ 0x798C,0x5062,
	/* Array index: 0x1047 */ 0x79A1,0x506B,
	/* Array index: 0x1047 */ 0x79A8,0x506C,
	/* Array index: 0x1047 */ 0x79A9,0x506D,
	/* Array index: 0x1047 */ 0x79AB,0x506E,
	/* Array index: 0x1047 */ 0x79AF,0x506F,
	/* Array index: 0x1047 */ 0x79B1,0x5070,
	/* Array index: 0x1047 */ 0x79B4,0x5071,
	/* Array index: 0x1047 */ 0x79B8,0x5072,
	/* Array index: 0x1047 */ 0x79BB,0x5073,
	/* Array index: 0x1047 */ 0x79C2,0x5074,
	/* Array index: 0x1047 */ 0x79C4,0x5075,
	/* Array index: 0x1047 */ 0x79C7,0x5076,
	/* Array index: 0x1047 */ 0x79C8,0x5077,
	/* Array index: 0x1047 */ 0x79CA,0x5078,
	/* Array index: 0x1047 */ 0x79CF,0x5079,
	/* Array index: 0x1047 */ 0x79D4,0x507A,
	/* Array index: 0x1047 */ 0x79D6,0x507B,
	/* Array index: 0x1047 */ 0x79DA,0x507C,
	/* Array index: 0x1047 */ 0x79DD,0x507D,
	/* Array index: 0x1047 */ 0x79DE,0x507E,
	/* Array index: 0x1047 */ 0x79E0,0x5121,
	/* Array index: 0x1047 */ 0x79E2,0x5122,
	/* Array index: 0x1047 */ 0x79E5,0x5123,
	/* Array index: 0x1047 */ 0x79EA,0x5124,
	/* Array index: 0x1047 */ 0x79EB,0x5125,
	/* Array index: 0x1047 */ 0x79ED,0x5126,
	/* Array index: 0x1047 */ 0x79F1,0x5127,
	/* Array index: 0x1047 */ 0x79F8,0x5128,
	/* Array index: 0x1047 */ 0x79FC,0x5129,
	/* Array index: 0x1047 */ 0x7A02,0x512A,
	/* Array index: 0x1047 */ 0x7A03,0x512B,
	/* Array index: 0x1047 */ 0x7A07,0x512C,
	/* Array index: 0x1047 */ 0x7A09,0x512D,
	/* Array index: 0x1047 */ 0x7A0A,0x512E,
	/* Array index: 0x1047 */ 0x7A0C,0x512F,
	/* Array index: 0x1047 */ 0x7A11,0x5130,
	/* Array index: 0x1047 */ 0x7A15,0x5131,
	/* Array index: 0x1047 */ 0x7A1B,0x5132,
	/* Array index: 0x1047 */ 0x7A1E,0x5133,
	/* Array index: 0x1047 */ 0x7A21,0x5134,
	/* Array index: 0x1047 */ 0x7A27,0x5135,
	/* Array index: 0x1047 */ 0x7A2B,0x5136,
	/* Array index: 0x1047 */ 0x7A2D,0x5137,
	/* Array index: 0x1047 */ 0x7A2F,0x5138,
	/* Array index: 0x1047 */ 0x7A30,0x5139,
	/* Array index: 0x1047 */ 0x7A4C,0x5143,
	/* Array index: 0x1047 */ 0x7A55,0x5144,
	/* Array index: 0x1047 */ 0x7A56,0x5145,
	/* Array index: 0x1047 */ 0x7A59,0x5146,
	/* Array index: 0x1047 */ 0x7A65,0x514B,
	/* Array index: 0x1047 */ 0x7A67,0x514C,
	/* Array index: 0x1047 */ 0x7A6A,0x514D,
	/* Array index: 0x1047 */ 0x7A6D,0x514E,
	/* Array index: 0x1047 */ 0x7A75,0x514F,
	/* Array index: 0x1047 */ 0x7A78,0x5150,
	/* Array index: 0x1047 */ 0x7A7E,0x5151,
	/* Array index: 0x1047 */ 0x7A80,0x5152,
	/* Array index: 0x1047 */ 0x7A82,0x5153,
	/* Array index: 0x1047 */ 0x7A85,0x5154,
	/* Array index: 0x1047 */ 0x7A86,0x5155,
	/* Array index: 0x1047 */ 0x7A8A,0x5156,
	/* Array index: 0x1047 */ 0x7A8B,0x5157,
	/* Array index: 0x1047 */ 0x7A90,0x5158,
	/* Array index: 0x1047 */ 0x7A91,0x5159,
	/* Array index: 0x1047 */ 0x7A94,0x515A,
	/* Array index: 0x1047 */ 0x7A9E,0x515B,
	/* Array index: 0x1047 */ 0x7AA0,0x515C,
	/* Array index: 0x1047 */ 0x7AA3,0x515D,
	/* Array index: 0x1047 */ 0x7AAC,0x515E,
	/* Array index: 0x1047 */ 0x7AB3,0x515F,
	/* Array index: 0x1047 */ 0x7AB5,0x5160,
	/* Array index: 0x1047 */ 0x7AB9,0x5161,
	/* Array index: 0x1047 */ 0x7ABB,0x5162,
	/* Array index: 0x1047 */ 0x7ABC,0x5163,
	/* Array index: 0x1047 */ 0x7AC6,0x5164,
	/* Array index: 0x1047 */ 0x7AC9,0x5165,
	/* Array index: 0x1047 */ 0x7ACC,0x5166,
	/* Array index: 0x1047 */ 0x7ACE,0x5167,
	/* Array index: 0x1047 */ 0x7AD1,0x5168,
	/* Array index: 0x1047 */ 0x7ADB,0x5169,
	/* Array index: 0x1047 */ 0x7AF1,0x516E,
	/* Array index: 0x1047 */ 0x7AF4,0x516F,
	/* Array index: 0x1047 */ 0x7AFB,0x5170,
	/* Array index: 0x1047 */ 0x7AFD,0x5171,
	/* Array index: 0x1047 */ 0x7AFE,0x5172,
	/* Array index: 0x1047 */ 0x7B07,0x5173,
	/* Array index: 0x1047 */ 0x7B14,0x5174,
	/* Array index: 0x1047 */ 0x7B1F,0x5175,
	/* Array index: 0x1047 */ 0x7B23,0x5176,
	/* Array index: 0x1047 */ 0x7B34,0x5222,
	/* Array index: 0x1047 */ 0x7B47,0x5227,
	/* Array index: 0x1047 */ 0x7B4E,0x5228,
	/* Array index: 0x1047 */ 0x7B55,0x5229,
	/* Array index: 0x1047 */ 0x7B60,0x522A,
	/* Array index: 0x1047 */ 0x7B64,0x522B,
	/* Array index: 0x1047 */ 0x7B66,0x522C,
	/* Array index: 0x1047 */ 0x7B69,0x522D,
	/* Array index: 0x1047 */ 0x7B6A,0x522E,
	/* Array index: 0x1047 */ 0x7B6D,0x522F,
	/* Array index: 0x1047 */ 0x7B6F,0x5230,
	/* Array index: 0x1047 */ 0x7B72,0x5231,
	/* Array index: 0x1047 */ 0x7B73,0x5232,
	/* Array index: 0x1047 */ 0x7B77,0x5233,
	/* Array index: 0x1047 */ 0x7B84,0x5234,
	/* Array index: 0x1047 */ 0x7B89,0x5235,
	/* Array index: 0x1047 */ 0x7B8E,0x5236,
	/* Array index: 0x1047 */ 0x7B90,0x5237,
	/* Array index: 0x1047 */ 0x7B91,0x5238,
	/* Array index: 0x1047 */ 0x7B96,0x5239,
	/* Array index: 0x1047 */ 0x7B9B,0x523A,
	/* Array index: 0x1047 */ 0x7B9E,0x523B,
	/* Array index: 0x1047 */ 0x7BA0,0x523C,
	/* Array index: 0x1047 */ 0x7BA5,0x523D,
	/* Array index: 0x1047 */ 0x7BAC,0x523E,
	/* Array index: 0x1047 */ 0x7BAF,0x523F,
	/* Array index: 0x1047 */ 0x7BB0,0x5240,
	/* Array index: 0x1047 */ 0x7BB2,0x5241,
	/* Array index: 0x1047 */ 0x7BB5,0x5242,
	/* Array index: 0x1047 */ 0x7BB6,0x5243,
	/* Array index: 0x1047 */ 0x7BC2,0x5248,
	/* Array index: 0x1047 */ 0x7BC5,0x5249,
	/* Array index: 0x1047 */ 0x7BC8,0x524A,
	/* Array index: 0x1047 */ 0x7BCA,0x524B,
	/* Array index: 0x1047 */ 0x7BD4,0x524C,
	/* Array index: 0x1047 */ 0x7BE8,0x5252,
	/* Array index: 0x1047 */ 0x7BEA,0x5253,
	/* Array index: 0x1047 */ 0x7BF2,0x5254,
	/* Array index: 0x1047 */ 0x7C09,0x5261,
	/* Array index: 0x1047 */ 0x7C19,0x5266,
	/* Array index: 0x1047 */ 0x7C1B,0x5267,
	/* Array index: 0x1047 */ 0x7C20,0x5268,
	/* Array index: 0x1047 */ 0x7C25,0x5269,
	/* Array index: 0x1047 */ 0x7C26,0x526A,
	/* Array index: 0x1047 */ 0x7C28,0x526B,
	/* Array index: 0x1047 */ 0x7C2C,0x526C,
	/* Array index: 0x1047 */ 0x7C31,0x526D,
	/* Array index: 0x1047 */ 0x7C33,0x526E,
	/* Array index: 0x1047 */ 0x7C34,0x526F,
	/* Array index: 0x1047 */ 0x7C36,0x5270,
	/* Array index: 0x1047 */ 0x7C39,0x5271,
	/* Array index: 0x1047 */ 0x7C3A,0x5272,
	/* Array index: 0x1047 */ 0x7C46,0x5273,
	/* Array index: 0x1047 */ 0x7C4A,0x5274,
	/* Array index: 0x1047 */ 0x7C61,0x5321,
	/* Array index: 0x1047 */ 0x7C63,0x5322,
	/* Array index: 0x1047 */ 0x7C67,0x5323,
	/* Array index: 0x1047 */ 0x7C69,0x5324,
	/* Array index: 0x1047 */ 0x7C6D,0x5325,
	/* Array index: 0x1047 */ 0x7C6E,0x5326,
	/* Array index: 0x1047 */ 0x7C70,0x5327,
	/* Array index: 0x1047 */ 0x7C72,0x5328,
	/* Array index: 0x1047 */ 0x7C79,0x5329,
	/* Array index: 0x1047 */ 0x7C7C,0x532A,
	/* Array index: 0x1047 */ 0x7C7D,0x532B,
	/* Array index: 0x1047 */ 0x7C86,0x532C,
	/* Array index: 0x1047 */ 0x7C87,0x532D,
	/* Array index: 0x1047 */ 0x7C8F,0x532E,
	/* Array index: 0x1047 */ 0x7C94,0x532F,
	/* Array index: 0x1047 */ 0x7C9E,0x5330,
	/* Array index: 0x1047 */ 0x7CA0,0x5331,
	/* Array index: 0x1047 */ 0x7CA6,0x5332,
	/* Array index: 0x1047 */ 0x7CB0,0x5333,
	/* Array index: 0x1047 */ 0x7CBF,0x5339,
	/* Array index: 0x1047 */ 0x7CC4,0x533A,
	/* Array index: 0x1047 */ 0x7CCD,0x533E,
	/* Array index: 0x1047 */ 0x7CCF,0x533F,
	/* Array index: 0x1047 */ 0x7CDD,0x5346,
	/* Array index: 0x1047 */ 0x7CE6,0x5347,
	/* Array index: 0x1047 */ 0x7CE9,0x5348,
	/* Array index: 0x1047 */ 0x7CEB,0x5349,
	/* Array index: 0x1047 */ 0x7CF5,0x534A,
	/* Array index: 0x1047 */ 0x7D03,0x534B,
	/* Array index: 0x1047 */ 0x7D16,0x5353,
	/* Array index: 0x1047 */ 0x7D1D,0x5354,
	/* Array index: 0x1047 */ 0x7D1E,0x5355,
	/* Array index: 0x1047 */ 0x7D23,0x5356,
	/* Array index: 0x1047 */ 0x7D26,0x5357,
	/* Array index: 0x1047 */ 0x7D2A,0x5358,
	/* Array index: 0x1047 */ 0x7D2D,0x5359,
	/* Array index: 0x1047 */ 0x7D31,0x535A,
	/* Array index: 0x1047 */ 0x7D47,0x5360,
	/* Array index: 0x1047 */ 0x7D48,0x5361,
	/* Array index: 0x1047 */ 0x7D4D,0x5362,
	/* Array index: 0x1047 */ 0x7D51,0x5363,
	/* Array index: 0x1047 */ 0x7D53,0x5364,
	/* Array index: 0x1047 */ 0x7D57,0x5365,
	/* Array index: 0x1047 */ 0x7D65,0x536A,
	/* Array index: 0x1047 */ 0x7D67,0x536B,
	/* Array index: 0x1047 */ 0x7D6A,0x536C,
	/* Array index: 0x1047 */ 0x7D70,0x536D,
	/* Array index: 0x1047 */ 0x7D78,0x536E,
	/* Array index: 0x1047 */ 0x7D7A,0x536F,
	/* Array index: 0x1047 */ 0x7D7B,0x5370,
	/* Array index: 0x1047 */ 0x7D91,0x537B,
	/* Array index: 0x1047 */ 0x7D96,0x537C,
	/* Array index: 0x1047 */ 0x7D97,0x537D,
	/* Array index: 0x1047 */ 0x7D9D,0x537E,
	/* Array index: 0x1047 */ 0x7D9E,0x5421,
	/* Array index: 0x1047 */ 0x7DA6,0x5422,
	/* Array index: 0x1047 */ 0x7DA7,0x5423,
	/* Array index: 0x1047 */ 0x7DAA,0x5424,
	/* Array index: 0x1047 */ 0x7DB3,0x5425,
	/* Array index: 0x1047 */ 0x7DB6,0x5426,
	/* Array index: 0x1047 */ 0x7DB7,0x5427,
	/* Array index: 0x1047 */ 0x7DB9,0x5428,
	/* Array index: 0x1047 */ 0x7DD7,0x5431,
	/* Array index: 0x1047 */ 0x7DD9,0x5432,
	/* Array index: 0x1047 */ 0x7DE2,0x5434,
	/* Array index: 0x1047 */ 0x7DE5,0x5435,
	/* Array index: 0x1047 */ 0x7DE6,0x5436,
	/* Array index: 0x1047 */ 0x7DEA,0x5437,
	/* Array index: 0x1047 */ 0x7DEB,0x5438,
	/* Array index: 0x1047 */ 0x7DED,0x5439,
	/* Array index: 0x1047 */ 0x7DF1,0x543A,
	/* Array index: 0x1047 */ 0x7DF5,0x543B,
	/* Array index: 0x1047 */ 0x7DF6,0x543C,
	/* Array index: 0x1047 */ 0x7DF9,0x543D,
	/* Array index: 0x1047 */ 0x7DFA,0x543E,
	/* Array index: 0x1047 */ 0x7E00,0x5433,
	/* Array index: 0x1047 */ 0x7E08,0x543F,
	/* Array index: 0x1047 */ 0x7E10,0x5440,
	/* Array index: 0x1047 */ 0x7E11,0x5441,
	/* Array index: 0x1047 */ 0x7E15,0x5442,
	/* Array index: 0x1047 */ 0x7E17,0x5443,
	/* Array index: 0x1047 */ 0x7E1C,0x5444,
	/* Array index: 0x1047 */ 0x7E1D,0x5445,
	/* Array index: 0x1047 */ 0x7E20,0x5446,
	/* Array index: 0x1047 */ 0x7E27,0x5447,
	/* Array index: 0x1047 */ 0x7E28,0x5448,
	/* Array index: 0x1047 */ 0x7E2C,0x5449,
	/* Array index: 0x1047 */ 0x7E2D,0x544A,
	/* Array index: 0x1047 */ 0x7E2F,0x544B,
	/* Array index: 0x1047 */ 0x7E33,0x544C,
	/* Array index: 0x1047 */ 0x7E36,0x544D,
	/* Array index: 0x1047 */ 0x7E3F,0x544E,
	/* Array index: 0x1047 */ 0x7E44,0x544F,
	/* Array index: 0x1047 */ 0x7E45,0x5450,
	/* Array index: 0x1047 */ 0x7E47,0x5451,
	/* Array index: 0x1047 */ 0x7E4E,0x5452,
	/* Array index: 0x1047 */ 0x7E50,0x5453,
	/* Array index: 0x1047 */ 0x7E52,0x5454,
	/* Array index: 0x1047 */ 0x7E58,0x5455,
	/* Array index: 0x1047 */ 0x7E5F,0x5456,
	/* Array index: 0x1047 */ 0x7E61,0x5457,
	/* Array index: 0x1047 */ 0x7E62,0x5458,
	/* Array index: 0x1047 */ 0x7E65,0x5459,
	/* Array index: 0x1047 */ 0x7E6B,0x545A,
	/* Array index: 0x1047 */ 0x7E6E,0x545B,
	/* Array index: 0x1047 */ 0x7E6F,0x545C,
	/* Array index: 0x1047 */ 0x7E73,0x545D,
	/* Array index: 0x1047 */ 0x7E78,0x545E,
	/* Array index: 0x1047 */ 0x7E7E,0x545F,
	/* Array index: 0x1047 */ 0x7E81,0x5460,
	/* Array index: 0x1047 */ 0x7E86,0x5461,
	/* Array index: 0x1047 */ 0x7E87,0x5462,
	/* Array index: 0x1047 */ 0x7E8A,0x5463,
	/* Array index: 0x1047 */ 0x7E8D,0x5464,
	/* Array index: 0x1047 */ 0x7E91,0x5465,
	/* Array index: 0x1047 */ 0x7E95,0x5466,
	/* Array index: 0x1047 */ 0x7E98,0x5467,
	/* Array index: 0x1047 */ 0x7E9A,0x5468,
	/* Array index: 0x1047 */ 0x7E9D,0x5469,
	/* Array index: 0x1047 */ 0x7E9E,0x546A,
	/* Array index: 0x1047 */ 0x7F43,0x5470,
	/* Array index: 0x1047 */ 0x7F44,0x5471,
	/* Array index: 0x1047 */ 0x7F47,0x5472,
	/* Array index: 0x1047 */ 0x7F4F,0x5473,
	/* Array index: 0x1047 */ 0x7F52,0x5474,
	/* Array index: 0x1047 */ 0x7F53,0x5475,
	/* Array index: 0x1047 */ 0x7F6D,0x547E,
	/* Array index: 0x1047 */ 0x7F71,0x5521,
	/* Array index: 0x1047 */ 0x7F8B,0x5526,
	/* Array index: 0x1047 */ 0x7F96,0x552B,
	/* Array index: 0x1047 */ 0x7F97,0x552C,
	/* Array index: 0x1047 */ 0x7F9C,0x552D,
	/* Array index: 0x1047 */ 0x7FA1,0x552E,
	/* Array index: 0x1047 */ 0x7FA2,0x552F,
	/* Array index: 0x1047 */ 0x7FA6,0x5530,
	/* Array index: 0x1047 */ 0x7FAA,0x5531,
	/* Array index: 0x1047 */ 0x7FAD,0x5532,
	/* Array index: 0x1047 */ 0x7FB4,0x5533,
	/* Array index: 0x1047 */ 0x7FBC,0x5534,
	/* Array index: 0x1047 */ 0x7FBF,0x5535,
	/* Array index: 0x1047 */ 0x7FC0,0x5536,
	/* Array index: 0x1047 */ 0x7FC3,0x5537,
	/* Array index: 0x1047 */ 0x7FC8,0x5538,
	/* Array index: 0x1047 */ 0x7FCE,0x5539,
	/* Array index: 0x1047 */ 0x7FCF,0x553A,
	/* Array index: 0x1047 */ 0x7FDB,0x553B,
	/* Array index: 0x1047 */ 0x7FDF,0x553C,
	/* Array index: 0x1047 */ 0x7FE3,0x553D,
	/* Array index: 0x1047 */ 0x7FE5,0x553E,
	/* Array index: 0x1047 */ 0x7FE8,0x553F,
	/* Array index: 0x1047 */ 0x7FEC,0x5540,
	/* Array index: 0x1047 */ 0x7FEE,0x5541,
	/* Array index: 0x1047 */ 0x7FEF,0x5542,
	/* Array index: 0x1047 */ 0x7FF2,0x5543,
	/* Array index: 0x1047 */ 0x7FFA,0x5544,
	/* Array index: 0x1047 */ 0x8007,0x5548,
	/* Array index: 0x1047 */ 0x8008,0x5549,
	/* Array index: 0x1047 */ 0x800A,0x554A,
	/* Array index: 0x1047 */ 0x8024,0x5556,
	/* Array index: 0x1047 */ 0x8026,0x5557,
	/* Array index: 0x1047 */ 0x802C,0x5558,
	/* Array index: 0x1047 */ 0x802E,0x5559,
	/* Array index: 0x1047 */ 0x8030,0x555A,
	/* Array index: 0x1047 */ 0x8034,0x555B,
	/* Array index: 0x1047 */ 0x8035,0x555C,
	/* Array index: 0x1047 */ 0x8037,0x555D,
	/* Array index: 0x1047 */ 0x8039,0x555E,
	/* Array index: 0x1047 */ 0x803A,0x555F,
	/* Array index: 0x1047 */ 0x803C,0x5560,
	/* Array index: 0x1047 */ 0x803E,0x5561,
	/* Array index: 0x1047 */ 0x8040,0x5562,
	/* Array index: 0x1047 */ 0x8044,0x5563,
	/* Array index: 0x1047 */ 0x8060,0x5564,
	/* Array index: 0x1047 */ 0x8064,0x5565,
	/* Array index: 0x1047 */ 0x8066,0x5566,
	/* Array index: 0x1047 */ 0x806D,0x5567,
	/* Array index: 0x1047 */ 0x8071,0x5568,
	/* Array index: 0x1047 */ 0x8075,0x5569,
	/* Array index: 0x1047 */ 0x8081,0x556A,
	/* Array index: 0x1047 */ 0x8088,0x556B,
	/* Array index: 0x1047 */ 0x808E,0x556C,
	/* Array index: 0x1047 */ 0x809C,0x556D,
	/* Array index: 0x1047 */ 0x809E,0x556E,
	/* Array index: 0x1047 */ 0x80A6,0x556F,
	/* Array index: 0x1047 */ 0x80A7,0x5570,
	/* Array index: 0x1047 */ 0x80AB,0x5571,
	/* Array index: 0x1047 */ 0x80B8,0x5572,
	/* Array index: 0x1047 */ 0x80B9,0x5573,
	/* Array index: 0x1047 */ 0x80C8,0x5574,
	/* Array index: 0x1047 */ 0x80CD,0x5575,
	/* Array index: 0x1047 */ 0x80CF,0x5576,
	/* Array index: 0x1047 */ 0x80D2,0x5577,
	/* Array index: 0x1047 */ 0x80E0,0x557C,
	/* Array index: 0x1047 */ 0x80ED,0x557D,
	/* Array index: 0x1047 */ 0x80EE,0x557E,
	/* Array index: 0x1047 */ 0x80F0,0x5621,
	/* Array index: 0x1047 */ 0x80F2,0x5622,
	/* Array index: 0x1047 */ 0x80F3,0x5623,
	/* Array index: 0x1047 */ 0x80F6,0x5624,
	/* Array index: 0x1047 */ 0x80F9,0x5625,
	/* Array index: 0x1047 */ 0x80FA,0x5626,
	/* Array index: 0x1047 */ 0x80FE,0x5627,
	/* Array index: 0x1047 */ 0x8103,0x5628,
	/* Array index: 0x1047 */ 0x810B,0x5629,
	/* Array index: 0x1047 */ 0x811C,0x562D,
	/* Array index: 0x1047 */ 0x811E,0x562E,
	/* Array index: 0x1047 */ 0x8120,0x562F,
	/* Array index: 0x1047 */ 0x8124,0x5630,
	/* Array index: 0x1047 */ 0x8127,0x5631,
	/* Array index: 0x1047 */ 0x812C,0x5632,
	/* Array index: 0x1047 */ 0x8130,0x5633,
	/* Array index: 0x1047 */ 0x8135,0x5634,
	/* Array index: 0x1047 */ 0x813A,0x5635,
	/* Array index: 0x1047 */ 0x813C,0x5636,
	/* Array index: 0x1047 */ 0x8145,0x5637,
	/* Array index: 0x1047 */ 0x8147,0x5638,
	/* Array index: 0x1047 */ 0x814A,0x5639,
	/* Array index: 0x1047 */ 0x814C,0x563A,
	/* Array index: 0x1047 */ 0x8152,0x563B,
	/* Array index: 0x1047 */ 0x8157,0x563C,
	/* Array index: 0x1047 */ 0x8160,0x563D,
	/* Array index: 0x1047 */ 0x8161,0x563E,
	/* Array index: 0x1047 */ 0x816D,0x5642,
	/* Array index: 0x1047 */ 0x816F,0x5643,
	/* Array index: 0x1047 */ 0x8177,0x5644,
	/* Array index: 0x1047 */ 0x8181,0x5645,
	/* Array index: 0x1047 */ 0x818B,0x564A,
	/* Array index: 0x1047 */ 0x818E,0x564B,
	/* Array index: 0x1047 */ 0x8190,0x5646,
	/* Array index: 0x1047 */ 0x8196,0x564C,
	/* Array index: 0x1047 */ 0x8198,0x564D,
	/* Array index: 0x1047 */ 0x819B,0x564E,
	/* Array index: 0x1047 */ 0x819E,0x564F,
	/* Array index: 0x1047 */ 0x81A2,0x5650,
	/* Array index: 0x1047 */ 0x81AE,0x5651,
	/* Array index: 0x1047 */ 0x81B2,0x5652,
	/* Array index: 0x1047 */ 0x81B4,0x5653,
	/* Array index: 0x1047 */ 0x81BB,0x5654,
	/* Array index: 0x1047 */ 0x81C3,0x5656,
	/* Array index: 0x1047 */ 0x81C5,0x5657,
	/* Array index: 0x1047 */ 0x81CA,0x5658,
	/* Array index: 0x1047 */ 0x81CB,0x5655,
	/* Array index: 0x1047 */ 0x81CE,0x5659,
	/* Array index: 0x1047 */ 0x81CF,0x565A,
	/* Array index: 0x1047 */ 0x81D5,0x565B,
	/* Array index: 0x1047 */ 0x81D7,0x565C,
	/* Array index: 0x1047 */ 0x81DB,0x565D,
	/* Array index: 0x1047 */ 0x81DD,0x565E,
	/* Array index: 0x1047 */ 0x81DE,0x565F,
	/* Array index: 0x1047 */ 0x81E1,0x5660,
	/* Array index: 0x1047 */ 0x81E4,0x5661,
	/* Array index: 0x1047 */ 0x81EB,0x5662,
	/* Array index: 0x1047 */ 0x81EC,0x5663,
	/* Array index: 0x1047 */ 0x81FD,0x566B,
	/* Array index: 0x1047 */ 0x81FF,0x566C,
	/* Array index: 0x1047 */ 0x8200,0x566D,
	/* Array index: 0x1047 */ 0x8203,0x566E,
	/* Array index: 0x1047 */ 0x820F,0x566F,
	/* Array index: 0x1047 */ 0x8213,0x5670,
	/* Array index: 0x1047 */ 0x8214,0x5671,
	/* Array index: 0x1047 */ 0x8219,0x5672,
	/* Array index: 0x1047 */ 0x821A,0x5673,
	/* Array index: 0x1047 */ 0x821D,0x5674,
	/* Array index: 0x1047 */ 0x8221,0x5675,
	/* Array index: 0x1047 */ 0x8222,0x5676,
	/* Array index: 0x1047 */ 0x8228,0x5677,
	/* Array index: 0x1047 */ 0x8232,0x5678,
	/* Array index: 0x1047 */ 0x8234,0x5679,
	/* Array index: 0x1047 */ 0x823A,0x567A,
	/* Array index: 0x1047 */ 0x824B,0x5721,
	/* Array index: 0x1047 */ 0x824E,0x5722,
	/* Array index: 0x1047 */ 0x824F,0x5723,
	/* Array index: 0x1047 */ 0x8251,0x5724,
	/* Array index: 0x1047 */ 0x8256,0x5725,
	/* Array index: 0x1047 */ 0x825C,0x5726,
	/* Array index: 0x1047 */ 0x8260,0x5727,
	/* Array index: 0x1047 */ 0x8263,0x5728,
	/* Array index: 0x1047 */ 0x8267,0x5729,
	/* Array index: 0x1047 */ 0x826D,0x572A,
	/* Array index: 0x1047 */ 0x8274,0x572B,
	/* Array index: 0x1047 */ 0x827B,0x572C,
	/* Array index: 0x1047 */ 0x8287,0x5733,
	/* Array index: 0x1047 */ 0x8289,0x5734,
	/* Array index: 0x1047 */ 0x828A,0x5735,
	/* Array index: 0x1047 */ 0x828E,0x5736,
	/* Array index: 0x1047 */ 0x8291,0x5737,
	/* Array index: 0x1047 */ 0x8294,0x5738,
	/* Array index: 0x1047 */ 0x8296,0x5739,
	/* Array index: 0x1047 */ 0x8298,0x573A,
	/* Array index: 0x1047 */ 0x829A,0x573B,
	/* Array index: 0x1047 */ 0x829B,0x573C,
	/* Array index: 0x1047 */ 0x82AE,0x5745,
	/* Array index: 0x1047 */ 0x82B0,0x5746,
	/* Array index: 0x1047 */ 0x82B2,0x5747,
	/* Array index: 0x1047 */ 0x82B4,0x5748,
	/* Array index: 0x1047 */ 0x82B7,0x5749,
	/* Array index: 0x1047 */ 0x82BA,0x574A,
	/* Array index: 0x1047 */ 0x82BC,0x574B,
	/* Array index: 0x1047 */ 0x82BE,0x574C,
	/* Array index: 0x1047 */ 0x82BF,0x574D,
	/* Array index: 0x1047 */ 0x82C6,0x574E,
	/* Array index: 0x1047 */ 0x82D0,0x574F,
	/* Array index: 0x1047 */ 0x82D5,0x5750,
	/* Array index: 0x1047 */ 0x82DA,0x5751,
	/* Array index: 0x1047 */ 0x82E0,0x5752,
	/* Array index: 0x1047 */ 0x82E2,0x5753,
	/* Array index: 0x1047 */ 0x82E4,0x5754,
	/* Array index: 0x1047 */ 0x82E8,0x5755,
	/* Array index: 0x1047 */ 0x82EA,0x5756,
	/* Array index: 0x1047 */ 0x82ED,0x5757,
	/* Array index: 0x1047 */ 0x82EF,0x5758,
	/* Array index: 0x1047 */ 0x82F6,0x5759,
	/* Array index: 0x1047 */ 0x82F7,0x575A,
	/* Array index: 0x1047 */ 0x8333,0x576E,
	/* Array index: 0x1047 */ 0x8337,0x576F,
	/* Array index: 0x1047 */ 0x833A,0x5770,
	/* Array index: 0x1047 */ 0x833C,0x5771,
	/* Array index: 0x1047 */ 0x833D,0x5772,
	/* Array index: 0x1047 */ 0x8347,0x5776,
	/* Array index: 0x1047 */ 0x834D,0x5777,
	/* Array index: 0x1047 */ 0x834E,0x5778,
	/* Array index: 0x1047 */ 0x8370,0x577D,
	/* Array index: 0x1047 */ 0x8378,0x577E,
	/* Array index: 0x1047 */ 0x837D,0x5821,
	/* Array index: 0x1047 */ 0x837F,0x5822,
	/* Array index: 0x1047 */ 0x8380,0x5823,
	/* Array index: 0x1047 */ 0x8382,0x5824,
	/* Array index: 0x1047 */ 0x8384,0x5825,
	/* Array index: 0x1047 */ 0x8386,0x5826,
	/* Array index: 0x1047 */ 0x838D,0x5827,
	/* Array index: 0x1047 */ 0x8392,0x5828,
	/* Array index: 0x1047 */ 0x8394,0x5829,
	/* Array index: 0x1047 */ 0x8395,0x582A,
	/* Array index: 0x1047 */ 0x83A6,0x5830,
	/* Array index: 0x1047 */ 0x83A7,0x5831,
	/* Array index: 0x1047 */ 0x83A9,0x5832,
	/* Array index: 0x1047 */ 0x83AC,0x5833,
	/* Array index: 0x1047 */ 0x83AD,0x584C,
	/* Array index: 0x1047 */ 0x83C7,0x5837,
	/* Array index: 0x1047 */ 0x83C9,0x5838,
	/* Array index: 0x1047 */ 0x83D4,0x583C,
	/* Array index: 0x1047 */ 0x83DD,0x583D,
	/* Array index: 0x1047 */ 0x83E8,0x583F,
	/* Array index: 0x1047 */ 0x83EA,0x5840,
	/* Array index: 0x1047 */ 0x83F6,0x5841,
	/* Array index: 0x1047 */ 0x83F8,0x5842,
	/* Array index: 0x1047 */ 0x83F9,0x5843,
	/* Array index: 0x1047 */ 0x83FC,0x5844,
	/* Array index: 0x1047 */ 0x8401,0x5845,
	/* Array index: 0x1047 */ 0x8406,0x5846,
	/* Array index: 0x1047 */ 0x840A,0x5847,
	/* Array index: 0x1047 */ 0x840F,0x5848,
	/* Array index: 0x1047 */ 0x8411,0x5849,
	/* Array index: 0x1047 */ 0x8415,0x584A,
	/* Array index: 0x1047 */ 0x8419,0x584B,
	/* Array index: 0x1047 */ 0x842F,0x584D,
	/* Array index: 0x1047 */ 0x8439,0x584E,
	/* Array index: 0x1047 */ 0x8445,0x584F,
	/* Array index: 0x1047 */ 0x8447,0x5850,
	/* Array index: 0x1047 */ 0x8448,0x5851,
	/* Array index: 0x1047 */ 0x844A,0x5852,
	/* Array index: 0x1047 */ 0x844D,0x5853,
	/* Array index: 0x1047 */ 0x844F,0x5854,
	/* Array index: 0x1047 */ 0x8451,0x5855,
	/* Array index: 0x1047 */ 0x8452,0x5856,
	/* Array index: 0x1047 */ 0x8460,0x585C,
	/* Array index: 0x1047 */ 0x8464,0x585D,
	/* Array index: 0x1047 */ 0x8465,0x585E,
	/* Array index: 0x1047 */ 0x8467,0x585F,
	/* Array index: 0x1047 */ 0x846A,0x5860,
	/* Array index: 0x1047 */ 0x8470,0x5861,
	/* Array index: 0x1047 */ 0x8473,0x5862,
	/* Array index: 0x1047 */ 0x8474,0x5863,
	/* Array index: 0x1047 */ 0x8476,0x5864,
	/* Array index: 0x1047 */ 0x8478,0x5865,
	/* Array index: 0x1047 */ 0x847C,0x5866,
	/* Array index: 0x1047 */ 0x847D,0x5867,
	/* Array index: 0x1047 */ 0x8481,0x5868,
	/* Array index: 0x1047 */ 0x8485,0x5869,
	/* Array index: 0x1047 */ 0x8492,0x586A,
	/* Array index: 0x1047 */ 0x8493,0x586B,
	/* Array index: 0x1047 */ 0x8495,0x586C,
	/* Array index: 0x1047 */ 0x849E,0x586D,
	/* Array index: 0x1047 */ 0x84AF,0x5872,
	/* Array index: 0x1047 */ 0x84B1,0x5873,
	/* Array index: 0x1047 */ 0x84B4,0x5874,
	/* Array index: 0x1047 */ 0x84BA,0x5875,
	/* Array index: 0x1047 */ 0x84BD,0x5876,
	/* Array index: 0x1047 */ 0x84BE,0x5877,
	/* Array index: 0x1047 */ 0x84C0,0x5878,
	/* Array index: 0x1047 */ 0x84C2,0x5879,
	/* Array index: 0x1047 */ 0x84C7,0x587A,
	/* Array index: 0x1047 */ 0x84C8,0x587B,
	/* Array index: 0x1047 */ 0x84CC,0x587C,
	/* Array index: 0x1047 */ 0x84CF,0x587D,
	/* Array index: 0x1047 */ 0x84D3,0x587E,
	/* Array index: 0x1047 */ 0x84DC,0x5921,
	/* Array index: 0x1047 */ 0x84E7,0x5922,
	/* Array index: 0x1047 */ 0x84EA,0x5923,
	/* Array index: 0x1047 */ 0x84F7,0x5928,
	/* Array index: 0x1047 */ 0x84FA,0x592A,
	/* Array index: 0x1047 */ 0x84FB,0x592B,
	/* Array index: 0x1047 */ 0x84FD,0x592C,
	/* Array index: 0x1047 */ 0x8502,0x592D,
	/* Array index: 0x1047 */ 0x8503,0x592E,
	/* Array index: 0x1047 */ 0x8507,0x592F,
	/* Array index: 0x1047 */ 0x850C,0x5930,
	/* Array index: 0x1047 */ 0x850E,0x5931,
	/* Array index: 0x1047 */ 0x8510,0x5932,
	/* Array index: 0x1047 */ 0x851C,0x5933,
	/* Array index: 0x1047 */ 0x851E,0x5934,
	/* Array index: 0x1047 */ 0x852F,0x593C,
	/* Array index: 0x1047 */ 0x853F,0x5940,
	/* Array index: 0x1047 */ 0x8546,0x5941,
	/* Array index: 0x1047 */ 0x8556,0x5947,
	/* Array index: 0x1047 */ 0x8559,0x5948,
	/* Array index: 0x1047 */ 0x856B,0x5951,
	/* Array index: 0x1047 */ 0x856F,0x5952,
	/* Array index: 0x1047 */ 0x8585,0x5959,
	/* Array index: 0x1047 */ 0x8586,0x595A,
	/* Array index: 0x1047 */ 0x8589,0x595B,
	/* Array index: 0x1047 */ 0x858B,0x595C,
	/* Array index: 0x1047 */ 0x858C,0x595D,
	/* Array index: 0x1047 */ 0x858F,0x595E,
	/* Array index: 0x1047 */ 0x8593,0x595F,
	/* Array index: 0x1047 */ 0x8598,0x5960,
	/* Array index: 0x1047 */ 0x859D,0x5961,
	/* Array index: 0x1047 */ 0x859F,0x5962,
	/* Array index: 0x1047 */ 0x85A0,0x5963,
	/* Array index: 0x1047 */ 0x85A2,0x5964,
	/* Array index: 0x1047 */ 0x85A5,0x5965,
	/* Array index: 0x1047 */ 0x85A7,0x5966,
	/* Array index: 0x1047 */ 0x85AD,0x5974,
	/* Array index: 0x1047 */ 0x85C2,0x596F,
	/* Array index: 0x1047 */ 0x85C7,0x5970,
	/* Array index: 0x1047 */ 0x85CA,0x5971,
	/* Array index: 0x1047 */ 0x85CB,0x5972,
	/* Array index: 0x1047 */ 0x85CE,0x5973,
	/* Array index: 0x1047 */ 0x85D8,0x5975,
	/* Array index: 0x1047 */ 0x85DA,0x5976,
	/* Array index: 0x1047 */ 0x85DF,0x5977,
	/* Array index: 0x1047 */ 0x85E0,0x5978,
	/* Array index: 0x1047 */ 0x85E6,0x5979,
	/* Array index: 0x1047 */ 0x85E8,0x597A,
	/* Array index: 0x1047 */ 0x85ED,0x597B,
	/* Array index: 0x1047 */ 0x85F3,0x597C,
	/* Array index: 0x1047 */ 0x85F6,0x597D,
	/* Array index: 0x1047 */ 0x85FC,0x597E,
	/* Array index: 0x1047 */ 0x85FF,0x5A21,
	/* Array index: 0x1047 */ 0x8600,0x5A22,
	/* Array index: 0x1047 */ 0x8604,0x5A23,
	/* Array index: 0x1047 */ 0x8605,0x5A24,
	/* Array index: 0x1047 */ 0x8618,0x5A2A,
	/* Array index: 0x1047 */ 0x8619,0x5A2B,
	/* Array index: 0x1047 */ 0x861B,0x5A2C,
	/* Array index: 0x1047 */ 0x861E,0x5A2D,
	/* Array index: 0x1047 */ 0x8621,0x5A2E,
	/* Array index: 0x1047 */ 0x8627,0x5A2F,
	/* Array index: 0x1047 */ 0x8629,0x5A30,
	/* Array index: 0x1047 */ 0x8636,0x5A31,
	/* Array index: 0x1047 */ 0x8638,0x5A32,
	/* Array index: 0x1047 */ 0x863A,0x5A33,
	/* Array index: 0x1047 */ 0x8646,0x5A38,
	/* Array index: 0x1047 */ 0x865D,0x5A3F,
	/* Array index: 0x1047 */ 0x8669,0x5A45,
	/* Array index: 0x1047 */ 0x866C,0x5A46,
	/* Array index: 0x1047 */ 0x866F,0x5A47,
	/* Array index: 0x1047 */ 0x867A,0x5A4B,
	/* Array index: 0x1047 */ 0x8688,0x5A6D,
	/* Array index: 0x1047 */ 0x868D,0x5A4C,
	/* Array index: 0x1047 */ 0x8691,0x5A4D,
	/* Array index: 0x1047 */ 0x8696,0x5A4E,
	/* Array index: 0x1047 */ 0x8698,0x5A4F,
	/* Array index: 0x1047 */ 0x869A,0x5A50,
	/* Array index: 0x1047 */ 0x869C,0x5A51,
	/* Array index: 0x1047 */ 0x86A1,0x5A52,
	/* Array index: 0x1047 */ 0x86AD,0x5A56,
	/* Array index: 0x1047 */ 0x86D1,0x5A63,
	/* Array index: 0x1047 */ 0x86D2,0x5A64,
	/* Array index: 0x1047 */ 0x86D5,0x5A65,
	/* Array index: 0x1047 */ 0x86D7,0x5A66,
	/* Array index: 0x1047 */ 0x86DA,0x5A67,
	/* Array index: 0x1047 */ 0x86DC,0x5A68,
	/* Array index: 0x1047 */ 0x86E0,0x5A69,
	/* Array index: 0x1047 */ 0x86E3,0x5A6A,
	/* Array index: 0x1047 */ 0x86E5,0x5A6B,
	/* Array index: 0x1047 */ 0x86E7,0x5A6C,
	/* Array index: 0x1047 */ 0x86FA,0x5A6E,
	/* Array index: 0x1047 */ 0x86FC,0x5A6F,
	/* Array index: 0x1047 */ 0x86FD,0x5A70,
	/* Array index: 0x1047 */ 0x8704,0x5A71,
	/* Array index: 0x1047 */ 0x8705,0x5A72,
	/* Array index: 0x1047 */ 0x8707,0x5A73,
	/* Array index: 0x1047 */ 0x870B,0x5A74,
	/* Array index: 0x1047 */ 0x8719,0x5A7A,
	/* Array index: 0x1047 */ 0x871E,0x5A7B,
	/* Array index: 0x1047 */ 0x871F,0x5A7C,
	/* Array index: 0x1047 */ 0x8721,0x5A7D,
	/* Array index: 0x1047 */ 0x8723,0x5A7E,
	/* Array index: 0x1047 */ 0x8728,0x5B21,
	/* Array index: 0x1047 */ 0x8743,0x5B2C,
	/* Array index: 0x1047 */ 0x8745,0x5B2D,
	/* Array index: 0x1047 */ 0x874D,0x5B2E,
	/* Array index: 0x1047 */ 0x8758,0x5B2F,
	/* Array index: 0x1047 */ 0x875D,0x5B30,
	/* Array index: 0x1047 */ 0x8761,0x5B31,
	/* Array index: 0x1047 */ 0x8764,0x5B32,
	/* Array index: 0x1047 */ 0x8765,0x5B33,
	/* Array index: 0x1047 */ 0x876F,0x5B34,
	/* Array index: 0x1047 */ 0x8771,0x5B35,
	/* Array index: 0x1047 */ 0x8772,0x5B36,
	/* Array index: 0x1047 */ 0x877B,0x5B37,
	/* Array index: 0x1047 */ 0x8790,0x5B41,
	/* Array index: 0x1047 */ 0x8793,0x5B42,
	/* Array index: 0x1047 */ 0x879E,0x5B47,
	/* Array index: 0x1047 */ 0x87A0,0x5B48,
	/* Array index: 0x1047 */ 0x87A3,0x5B49,
	/* Array index: 0x1047 */ 0x87A7,0x5B4A,
	/* Array index: 0x1047 */ 0x87B1,0x5B4E,
	/* Array index: 0x1047 */ 0x87B5,0x5B4F,
	/* Array index: 0x1047 */ 0x87BE,0x5B50,
	/* Array index: 0x1047 */ 0x87BF,0x5B51,
	/* Array index: 0x1047 */ 0x87C1,0x5B52,
	/* Array index: 0x1047 */ 0x87CE,0x5B56,
	/* Array index: 0x1047 */ 0x87D5,0x5B57,
	/* Array index: 0x1047 */ 0x87D6,0x5B58,
	/* Array index: 0x1047 */ 0x87D9,0x5B59,
	/* Array index: 0x1047 */ 0x87DA,0x5B5A,
	/* Array index: 0x1047 */ 0x87DC,0x5B5B,
	/* Array index: 0x1047 */ 0x87DF,0x5B5C,
	/* Array index: 0x1047 */ 0x87EA,0x5B60,
	/* Array index: 0x1047 */ 0x87EB,0x5B61,
	/* Array index: 0x1047 */ 0x87ED,0x5B62,
	/* Array index: 0x1047 */ 0x87F1,0x5B63,
	/* Array index: 0x1047 */ 0x87F3,0x5B64,
	/* Array index: 0x1047 */ 0x87F8,0x5B65,
	/* Array index: 0x1047 */ 0x87FA,0x5B66,
	/* Array index: 0x1047 */ 0x87FF,0x5B67,
	/* Array index: 0x1047 */ 0x8801,0x5B68,
	/* Array index: 0x1047 */ 0x8803,0x5B69,
	/* Array index: 0x1047 */ 0x8806,0x5B6A,
	/* Array index: 0x1047 */ 0x8828,0x5B79,
	/* Array index: 0x1047 */ 0x882D,0x5B7A,
	/* Array index: 0x1047 */ 0x882E,0x5B7B,
	/* Array index: 0x1047 */ 0x8830,0x5B7C,
	/* Array index: 0x1047 */ 0x8832,0x5B7D,
	/* Array index: 0x1047 */ 0x8835,0x5B7E,
	/* Array index: 0x1047 */ 0x883A,0x5C21,
	/* Array index: 0x1047 */ 0x883C,0x5C22,
	/* Array index: 0x1047 */ 0x8841,0x5C23,
	/* Array index: 0x1047 */ 0x8843,0x5C24,
	/* Array index: 0x1047 */ 0x8845,0x5C25,
	/* Array index: 0x1047 */ 0x884E,0x5C2A,
	/* Array index: 0x1047 */ 0x8851,0x5C2B,
	/* Array index: 0x1047 */ 0x8855,0x5C2C,
	/* Array index: 0x1047 */ 0x8856,0x5C2D,
	/* Array index: 0x1047 */ 0x8858,0x5C2E,
	/* Array index: 0x1047 */ 0x885A,0x5C2F,
	/* Array index: 0x1047 */ 0x885C,0x5C30,
	/* Array index: 0x1047 */ 0x885F,0x5C31,
	/* Array index: 0x1047 */ 0x8860,0x5C32,
	/* Array index: 0x1047 */ 0x8864,0x5C33,
	/* Array index: 0x1047 */ 0x8869,0x5C34,
	/* Array index: 0x1047 */ 0x8871,0x5C35,
	/* Array index: 0x1047 */ 0x8879,0x5C36,
	/* Array index: 0x1047 */ 0x887B,0x5C37,
	/* Array index: 0x1047 */ 0x8880,0x5C38,
	/* Array index: 0x1047 */ 0x88A8,0x5C3F,
	/* Array index: 0x1047 */ 0x88AA,0x5C40,
	/* Array index: 0x1047 */ 0x88BA,0x5C41,
	/* Array index: 0x1047 */ 0x88BD,0x5C42,
	/* Array index: 0x1047 */ 0x88BE,0x5C43,
	/* Array index: 0x1047 */ 0x88C0,0x5C44,
	/* Array index: 0x1047 */ 0x88DB,0x5C4D,
	/* Array index: 0x1047 */ 0x88DE,0x5C4E,
	/* Array index: 0x1047 */ 0x88E7,0x5C4F,
	/* Array index: 0x1047 */ 0x88F5,0x5C53,
	/* Array index: 0x1047 */ 0x88F7,0x5C54,
	/* Array index: 0x1047 */ 0x8901,0x5C55,
	/* Array index: 0x1047 */ 0x8906,0x5C56,
	/* Array index: 0x1047 */ 0x8920,0x5C60,
	/* Array index: 0x1047 */ 0x8935,0x5C67,
	/* Array index: 0x1047 */ 0x8939,0x5C68,
	/* Array index: 0x1047 */ 0x893A,0x5C69,
	/* Array index: 0x1047 */ 0x893E,0x5C6A,
	/* Array index: 0x1047 */ 0x8940,0x5C6B,
	/* Array index: 0x1047 */ 0x8942,0x5C6C,
	/* Array index: 0x1047 */ 0x8945,0x5C6D,
	/* Array index: 0x1047 */ 0x8946,0x5C6E,
	/* Array index: 0x1047 */ 0x8949,0x5C6F,
	/* Array index: 0x1047 */ 0x894F,0x5C70,
	/* Array index: 0x1047 */ 0x8952,0x5C71,
	/* Array index: 0x1047 */ 0x8957,0x5C72,
	/* Array index: 0x1047 */ 0x896B,0x5C79,
	/* Array index: 0x1047 */ 0x896E,0x5C7A,
	/* Array index: 0x1047 */ 0x8970,0x5C7B,
	/* Array index: 0x1047 */ 0x8973,0x5C7C,
	/* Array index: 0x1047 */ 0x8975,0x5C7D,
	/* Array index: 0x1047 */ 0x8989,0x5D24,
	/* Array index: 0x1047 */ 0x898D,0x5D25,
	/* Array index: 0x1047 */ 0x8990,0x5D26,
	/* Array index: 0x1047 */ 0x8994,0x5D27,
	/* Array index: 0x1047 */ 0x8995,0x5D28,
	/* Array index: 0x1047 */ 0x899B,0x5D29,
	/* Array index: 0x1047 */ 0x899C,0x5D2A,
	/* Array index: 0x1047 */ 0x899F,0x5D2B,
	/* Array index: 0x1047 */ 0x89A0,0x5D2C,
	/* Array index: 0x1047 */ 0x89A5,0x5D2D,
	/* Array index: 0x1047 */ 0x89B0,0x5D2E,
	/* Array index: 0x1047 */ 0x89BC,0x5D33,
	/* Array index: 0x1047 */ 0x89E5,0x5D39,
	/* Array index: 0x1047 */ 0x89E9,0x5D3A,
	/* Array index: 0x1047 */ 0x89EB,0x5D3B,
	/* Array index: 0x1047 */ 0x89ED,0x5D3C,
	/* Array index: 0x1047 */ 0x89F1,0x5D3D,
	/* Array index: 0x1047 */ 0x89F3,0x5D3E,
	/* Array index: 0x1047 */ 0x89F6,0x5D3F,
	/* Array index: 0x1047 */ 0x89F9,0x5D40,
	/* Array index: 0x1047 */ 0x89FD,0x5D41,
	/* Array index: 0x1047 */ 0x89FF,0x5D42,
	/* Array index: 0x1047 */ 0x8A04,0x5D43,
	/* Array index: 0x1047 */ 0x8A05,0x5D44,
	/* Array index: 0x1047 */ 0x8A07,0x5D45,
	/* Array index: 0x1047 */ 0x8A0F,0x5D46,
	/* Array index: 0x1047 */ 0x8A1E,0x5D4B,
	/* Array index: 0x1047 */ 0x8A20,0x5D4C,
	/* Array index: 0x1047 */ 0x8A22,0x5D4D,
	/* Array index: 0x1047 */ 0x8A24,0x5D4E,
	/* Array index: 0x1047 */ 0x8A26,0x5D4F,
	/* Array index: 0x1047 */ 0x8A2B,0x5D50,
	/* Array index: 0x1047 */ 0x8A2C,0x5D51,
	/* Array index: 0x1047 */ 0x8A2F,0x5D52,
	/* Array index: 0x1047 */ 0x8A35,0x5D53,
	/* Array index: 0x1047 */ 0x8A37,0x5D54,
	/* Array index: 0x1047 */ 0x8A3D,0x5D55,
	/* Array index: 0x1047 */ 0x8A3E,0x5D56,
	/* Array index: 0x1047 */ 0x8A40,0x5D57,
	/* Array index: 0x1047 */ 0x8A43,0x5D58,
	/* Array index: 0x1047 */ 0x8A45,0x5D59,
	/* Array index: 0x1047 */ 0x8A47,0x5D5A,
	/* Array index: 0x1047 */ 0x8A49,0x5D5B,
	/* Array index: 0x1047 */ 0x8A4D,0x5D5C,
	/* Array index: 0x1047 */ 0x8A4E,0x5D5D,
	/* Array index: 0x1047 */ 0x8A53,0x5D5E,
	/* Array index: 0x1047 */ 0x8A5C,0x5D62,
	/* Array index: 0x1047 */ 0x8A5D,0x5D63,
	/* Array index: 0x1047 */ 0x8A61,0x5D64,
	/* Array index: 0x1047 */ 0x8A65,0x5D65,
	/* Array index: 0x1047 */ 0x8A67,0x5D66,
	/* Array index: 0x1047 */ 0x8A83,0x5D70,
	/* Array index: 0x1047 */ 0x8A86,0x5D71,
	/* Array index: 0x1047 */ 0x8A8B,0x5D72,
	/* Array index: 0x1047 */ 0x8A8F,0x5D73,
	/* Array index: 0x1047 */ 0x8A90,0x5D74,
	/* Array index: 0x1047 */ 0x8A92,0x5D75,
	/* Array index: 0x1047 */ 0x8A96,0x5D76,
	/* Array index: 0x1047 */ 0x8A97,0x5D77,
	/* Array index: 0x1047 */ 0x8A99,0x5D78,
	/* Array index: 0x1047 */ 0x8A9F,0x5D79,
	/* Array index: 0x1047 */ 0x8AA7,0x5D7A,
	/* Array index: 0x1047 */ 0x8AA9,0x5D7B,
	/* Array index: 0x1047 */ 0x8AAE,0x5D7C,
	/* Array index: 0x1047 */ 0x8AAF,0x5D7D,
	/* Array index: 0x1047 */ 0x8AB3,0x5D7E,
	/* Array index: 0x1047 */ 0x8AB6,0x5E21,
	/* Array index: 0x1047 */ 0x8AB7,0x5E22,
	/* Array index: 0x1047 */ 0x8ABB,0x5E23,
	/* Array index: 0x1047 */ 0x8ABE,0x5E24,
	/* Array index: 0x1047 */ 0x8AC3,0x5E25,
	/* Array index: 0x1047 */ 0x8ADD,0x5E2F,
	/* Array index: 0x1047 */ 0x8ADF,0x5E30,
	/* Array index: 0x1047 */ 0x8AEC,0x5E31,
	/* Array index: 0x1047 */ 0x8AF0,0x5E32,
	/* Array index: 0x1047 */ 0x8AFC,0x5E36,
	/* Array index: 0x1047 */ 0x8AFF,0x5E37,
	/* Array index: 0x1047 */ 0x8B05,0x5E38,
	/* Array index: 0x1047 */ 0x8B06,0x5E39,
	/* Array index: 0x1047 */ 0x8B0A,0x5E3F,
	/* Array index: 0x1047 */ 0x8B0B,0x5E3A,
	/* Array index: 0x1047 */ 0x8B11,0x5E3B,
	/* Array index: 0x1047 */ 0x8B1C,0x5E3C,
	/* Array index: 0x1047 */ 0x8B1E,0x5E3D,
	/* Array index: 0x1047 */ 0x8B1F,0x5E3E,
	/* Array index: 0x1047 */ 0x8B2D,0x5E40,
	/* Array index: 0x1047 */ 0x8B30,0x5E41,
	/* Array index: 0x1047 */ 0x8B37,0x5E42,
	/* Array index: 0x1047 */ 0x8B3C,0x5E43,
	/* Array index: 0x1047 */ 0x8B4D,0x5E4E,
	/* Array index: 0x1047 */ 0x8B59,0x5E4D,
	/* Array index: 0x1047 */ 0x8B5E,0x5E4F,
	/* Array index: 0x1047 */ 0x8B63,0x5E50,
	/* Array index: 0x1047 */ 0x8B6D,0x5E51,
	/* Array index: 0x1047 */ 0x8B76,0x5E52,
	/* Array index: 0x1047 */ 0x8B78,0x5E53,
	/* Array index: 0x1047 */ 0x8B79,0x5E54,
	/* Array index: 0x1047 */ 0x8B7C,0x5E55,
	/* Array index: 0x1047 */ 0x8B7E,0x5E56,
	/* Array index: 0x1047 */ 0x8B81,0x5E57,
	/* Array index: 0x1047 */ 0x8B84,0x5E58,
	/* Array index: 0x1047 */ 0x8B85,0x5E59,
	/* Array index: 0x1047 */ 0x8B8B,0x5E5A,
	/* Array index: 0x1047 */ 0x8B8D,0x5E5B,
	/* Array index: 0x1047 */ 0x8B8F,0x5E5C,
	/* Array index: 0x1047 */ 0x8B94,0x5E5D,
	/* Array index: 0x1047 */ 0x8B95,0x5E5E,
	/* Array index: 0x1047 */ 0x8B9C,0x5E5F,
	/* Array index: 0x1047 */ 0x8B9E,0x5E60,
	/* Array index: 0x1047 */ 0x8B9F,0x5E61,
	/* Array index: 0x1047 */ 0x8C38,0x5E62,
	/* Array index: 0x1047 */ 0x8C39,0x5E63,
	/* Array index: 0x1047 */ 0x8C3D,0x5E64,
	/* Array index: 0x1047 */ 0x8C3E,0x5E65,
	/* Array index: 0x1047 */ 0x8C45,0x5E66,
	/* Array index: 0x1047 */ 0x8C47,0x5E67,
	/* Array index: 0x1047 */ 0x8C49,0x5E68,
	/* Array index: 0x1047 */ 0x8C4B,0x5E69,
	/* Array index: 0x1047 */ 0x8C4F,0x5E6A,
	/* Array index: 0x1047 */ 0x8C51,0x5E6B,
	/* Array index: 0x1047 */ 0x8C63,0x5E73,
	/* Array index: 0x1047 */ 0x8C64,0x5E74,
	/* Array index: 0x1047 */ 0x8C66,0x5E75,
	/* Array index: 0x1047 */ 0x8C68,0x5E76,
	/* Array index: 0x1047 */ 0x8C69,0x5E77,
	/* Array index: 0x1047 */ 0x8C6D,0x5E78,
	/* Array index: 0x1047 */ 0x8C73,0x5E79,
	/* Array index: 0x1047 */ 0x8C75,0x5E7A,
	/* Array index: 0x1047 */ 0x8C76,0x5E7B,
	/* Array index: 0x1047 */ 0x8C7B,0x5E7C,
	/* Array index: 0x1047 */ 0x8C7E,0x5E7D,
	/* Array index: 0x1047 */ 0x8C86,0x5E7E,
	/* Array index: 0x1047 */ 0x8C87,0x5F21,
	/* Array index: 0x1047 */ 0x8C8B,0x5F22,
	/* Array index: 0x1047 */ 0x8C90,0x5F23,
	/* Array index: 0x1047 */ 0x8C92,0x5F24,
	/* Array index: 0x1047 */ 0x8C93,0x5F25,
	/* Array index: 0x1047 */ 0x8C99,0x5F26,
	/* Array index: 0x1047 */ 0x8C9B,0x5F27,
	/* Array index: 0x1047 */ 0x8C9C,0x5F28,
	/* Array index: 0x1047 */ 0x8CA4,0x5F29,
	/* Array index: 0x1047 */ 0x8CB9,0x5F2A,
	/* Array index: 0x1047 */ 0x8CBA,0x5F2B,
	/* Array index: 0x1047 */ 0x8CC5,0x5F2C,
	/* Array index: 0x1047 */ 0x8CC6,0x5F2D,
	/* Array index: 0x1047 */ 0x8CC9,0x5F2E,
	/* Array index: 0x1047 */ 0x8CCB,0x5F2F,
	/* Array index: 0x1047 */ 0x8CCF,0x5F30,
	/* Array index: 0x1047 */ 0x8CD5,0x5F32,
	/* Array index: 0x1047 */ 0x8CD6,0x5F31,
	/* Array index: 0x1047 */ 0x8CD9,0x5F33,
	/* Array index: 0x1047 */ 0x8CDD,0x5F34,
	/* Array index: 0x1047 */ 0x8CE1,0x5F35,
	/* Array index: 0x1047 */ 0x8CE8,0x5F36,
	/* Array index: 0x1047 */ 0x8CEC,0x5F37,
	/* Array index: 0x1047 */ 0x8CEF,0x5F38,
	/* Array index: 0x1047 */ 0x8CF0,0x5F39,
	/* Array index: 0x1047 */ 0x8CF2,0x5F3A,
	/* Array index: 0x1047 */ 0x8CF5,0x5F3B,
	/* Array index: 0x1047 */ 0x8CF7,0x5F3C,
	/* Array index: 0x1047 */ 0x8CF8,0x5F3D,
	/* Array index: 0x1047 */ 0x8CFE,0x5F3E,
	/* Array index: 0x1047 */ 0x8CFF,0x5F3F,
	/* Array index: 0x1047 */ 0x8D01,0x5F40,
	/* Array index: 0x1047 */ 0x8D03,0x5F41,
	/* Array index: 0x1047 */ 0x8D09,0x5F42,
	/* Array index: 0x1047 */ 0x8D12,0x5F43,
	/* Array index: 0x1047 */ 0x8D17,0x5F44,
	/* Array index: 0x1047 */ 0x8D1B,0x5F45,
	/* Array index: 0x1047 */ 0x8D65,0x5F46,
	/* Array index: 0x1047 */ 0x8D69,0x5F47,
	/* Array index: 0x1047 */ 0x8D6C,0x5F48,
	/* Array index: 0x1047 */ 0x8D6E,0x5F49,
	/* Array index: 0x1047 */ 0x8D7F,0x5F4A,
	/* Array index: 0x1047 */ 0x8D82,0x5F4B,
	/* Array index: 0x1047 */ 0x8D84,0x5F4C,
	/* Array index: 0x1047 */ 0x8D88,0x5F4D,
	/* Array index: 0x1047 */ 0x8D8D,0x5F4E,
	/* Array index: 0x1047 */ 0x8D90,0x5F4F,
	/* Array index: 0x1047 */ 0x8D91,0x5F50,
	/* Array index: 0x1047 */ 0x8D95,0x5F51,
	/* Array index: 0x1047 */ 0x8DA6,0x5F55,
	/* Array index: 0x1047 */ 0x8DAB,0x5F56,
	/* Array index: 0x1047 */ 0x8DAC,0x5F57,
	/* Array index: 0x1047 */ 0x8DAF,0x5F58,
	/* Array index: 0x1047 */ 0x8DB2,0x5F59,
	/* Array index: 0x1047 */ 0x8DB5,0x5F5A,
	/* Array index: 0x1047 */ 0x8DB7,0x5F5B,
	/* Array index: 0x1047 */ 0x8DB9,0x5F5C,
	/* Array index: 0x1047 */ 0x8DBB,0x5F5D,
	/* Array index: 0x1047 */ 0x8DBC,0x5F6F,
	/* Array index: 0x1047 */ 0x8DC0,0x5F5E,
	/* Array index: 0x1047 */ 0x8DCE,0x5F64,
	/* Array index: 0x1047 */ 0x8DD1,0x5F65,
	/* Array index: 0x1047 */ 0x8DD4,0x5F66,
	/* Array index: 0x1047 */ 0x8DD5,0x5F67,
	/* Array index: 0x1047 */ 0x8DD7,0x5F68,
	/* Array index: 0x1047 */ 0x8DD9,0x5F69,
	/* Array index: 0x1047 */ 0x8DE4,0x5F6A,
	/* Array index: 0x1047 */ 0x8DE5,0x5F6B,
	/* Array index: 0x1047 */ 0x8DE7,0x5F6C,
	/* Array index: 0x1047 */ 0x8DEC,0x5F6D,
	/* Array index: 0x1047 */ 0x8DFD,0x5F73,
	/* Array index: 0x1047 */ 0x8E01,0x5F74,
	/* Array index: 0x1047 */ 0x8E0B,0x5F78,
	/* Array index: 0x1047 */ 0x8E11,0x5F79,
	/* Array index: 0x1047 */ 0x8E14,0x5F7A,
	/* Array index: 0x1047 */ 0x8E16,0x5F7B,
	/* Array index: 0x1047 */ 0x8E31,0x6024,
	/* Array index: 0x1047 */ 0x8E33,0x6025,
	/* Array index: 0x1047 */ 0x8E3D,0x602A,
	/* Array index: 0x1047 */ 0x8E40,0x602B,
	/* Array index: 0x1047 */ 0x8E41,0x602C,
	/* Array index: 0x1047 */ 0x8E54,0x6031,
	/* Array index: 0x1047 */ 0x8E69,0x6038,
	/* Array index: 0x1047 */ 0x8E82,0x6041,
	/* Array index: 0x1047 */ 0x8E83,0x6042,
	/* Array index: 0x1047 */ 0x8E89,0x6043,
	/* Array index: 0x1047 */ 0x8E90,0x6044,
	/* Array index: 0x1047 */ 0x8E92,0x6045,
	/* Array index: 0x1047 */ 0x8E95,0x6046,
	/* Array index: 0x1047 */ 0x8EA2,0x604B,
	/* Array index: 0x1047 */ 0x8EA7,0x604C,
	/* Array index: 0x1047 */ 0x8EA9,0x604D,
	/* Array index: 0x1047 */ 0x8EAD,0x604E,
	/* Array index: 0x1047 */ 0x8EAE,0x604F,
	/* Array index: 0x1047 */ 0x8EB3,0x6050,
	/* Array index: 0x1047 */ 0x8EB5,0x6051,
	/* Array index: 0x1047 */ 0x8EBA,0x6052,
	/* Array index: 0x1047 */ 0x8EBB,0x6053,
	/* Array index: 0x1047 */ 0x8EC7,0x6058,
	/* Array index: 0x1047 */ 0x8ECF,0x6059,
	/* Array index: 0x1047 */ 0x8ED1,0x605A,
	/* Array index: 0x1047 */ 0x8ED4,0x605B,
	/* Array index: 0x1047 */ 0x8EDC,0x605C,
	/* Array index: 0x1047 */ 0x8EE8,0x605D,
	/* Array index: 0x1047 */ 0x8EF7,0x6061,
	/* Array index: 0x1047 */ 0x8EF9,0x6062,
	/* Array index: 0x1047 */ 0x8EFA,0x6063,
	/* Array index: 0x1047 */ 0x8F00,0x6065,
	/* Array index: 0x1047 */ 0x8F02,0x6066,
	/* Array index: 0x1047 */ 0x8F07,0x6067,
	/* Array index: 0x1047 */ 0x8F08,0x6068,
	/* Array index: 0x1047 */ 0x8F0F,0x6069,
	/* Array index: 0x1047 */ 0x8F10,0x606A,
	/* Array index: 0x1047 */ 0x8F1E,0x606E,
	/* Array index: 0x1047 */ 0x8F20,0x606F,
	/* Array index: 0x1047 */ 0x8F21,0x6070,
	/* Array index: 0x1047 */ 0x8F23,0x6071,
	/* Array index: 0x1047 */ 0x8F25,0x6072,
	/* Array index: 0x1047 */ 0x8F27,0x6073,
	/* Array index: 0x1047 */ 0x8F28,0x6074,
	/* Array index: 0x1047 */ 0x8F3A,0x607C,
	/* Array index: 0x1047 */ 0x8F40,0x607D,
	/* Array index: 0x1047 */ 0x8F41,0x607E,
	/* Array index: 0x1047 */ 0x8F43,0x6121,
	/* Array index: 0x1047 */ 0x8F47,0x6122,
	/* Array index: 0x1047 */ 0x8F58,0x6129,
	/* Array index: 0x1047 */ 0x8F5D,0x612A,
	/* Array index: 0x1047 */ 0x8F5E,0x612B,
	/* Array index: 0x1047 */ 0x8F65,0x612C,
	/* Array index: 0x1047 */ 0x8F9D,0x612D,
	/* Array index: 0x1047 */ 0x8FB5,0x6133,
	/* Array index: 0x1047 */ 0x8FB6,0x6134,
	/* Array index: 0x1047 */ 0x8FB8,0x6135,
	/* Array index: 0x1047 */ 0x8FBE,0x6136,
	/* Array index: 0x1047 */ 0x8FC0,0x6137,
	/* Array index: 0x1047 */ 0x8FC1,0x6138,
	/* Array index: 0x1047 */ 0x8FC6,0x6139,
	/* Array index: 0x1047 */ 0x8FCA,0x613A,
	/* Array index: 0x1047 */ 0x8FCB,0x613B,
	/* Array index: 0x1047 */ 0x8FCD,0x613C,
	/* Array index: 0x1047 */ 0x8FD0,0x613D,
	/* Array index: 0x1047 */ 0x8FD2,0x613E,
	/* Array index: 0x1047 */ 0x8FD3,0x613F,
	/* Array index: 0x1047 */ 0x8FD5,0x6140,
	/* Array index: 0x1047 */ 0x8FE0,0x6141,
	/* Array index: 0x1047 */ 0x8FE3,0x6142,
	/* Array index: 0x1047 */ 0x8FE4,0x6143,
	/* Array index: 0x1047 */ 0x8FE8,0x6144,
	/* Array index: 0x1047 */ 0x8FEE,0x6145,
	/* Array index: 0x1047 */ 0x8FF1,0x6146,
	/* Array index: 0x1047 */ 0x8FF5,0x6147,
	/* Array index: 0x1047 */ 0x8FF6,0x6148,
	/* Array index: 0x1047 */ 0x8FFB,0x6149,
	/* Array index: 0x1047 */ 0x8FFE,0x614A,
	/* Array index: 0x1047 */ 0x9002,0x614B,
	/* Array index: 0x1047 */ 0x9004,0x614C,
	/* Array index: 0x1047 */ 0x9008,0x614D,
	/* Array index: 0x1047 */ 0x900C,0x614E,
	/* Array index: 0x1047 */ 0x9018,0x614F,
	/* Array index: 0x1047 */ 0x901B,0x6150,
	/* Array index: 0x1047 */ 0x9033,0x6157,
	/* Array index: 0x1047 */ 0x9034,0x6158,
	/* Array index: 0x1047 */ 0x9037,0x6159,
	/* Array index: 0x1047 */ 0x903F,0x615A,
	/* Array index: 0x1047 */ 0x9043,0x615B,
	/* Array index: 0x1047 */ 0x9044,0x615C,
	/* Array index: 0x1047 */ 0x904C,0x615D,
	/* Array index: 0x1047 */ 0x905B,0x615E,
	/* Array index: 0x1047 */ 0x905D,0x615F,
	/* Array index: 0x1047 */ 0x9062,0x6160,
	/* Array index: 0x1047 */ 0x9066,0x6161,
	/* Array index: 0x1047 */ 0x9067,0x6162,
	/* Array index: 0x1047 */ 0x906C,0x6163,
	/* Array index: 0x1047 */ 0x9070,0x6164,
	/* Array index: 0x1047 */ 0x9074,0x6165,
	/* Array index: 0x1047 */ 0x9079,0x6166,
	/* Array index: 0x1047 */ 0x9085,0x6167,
	/* Array index: 0x1047 */ 0x9088,0x6168,
	/* Array index: 0x1047 */ 0x908B,0x6169,
	/* Array index: 0x1047 */ 0x908C,0x616A,
	/* Array index: 0x1047 */ 0x908E,0x616B,
	/* Array index: 0x1047 */ 0x9090,0x616C,
	/* Array index: 0x1047 */ 0x90A5,0x6175,
	/* Array index: 0x1047 */ 0x90BD,0x617B,
	/* Array index: 0x1047 */ 0x90BE,0x617D,
	/* Array index: 0x1047 */ 0x90CC,0x617C,
	/* Array index: 0x1047 */ 0x90D2,0x622D,
	/* Array index: 0x1047 */ 0x90E5,0x622C,
	/* Array index: 0x1047 */ 0x90EB,0x622F,
	/* Array index: 0x1047 */ 0x90EF,0x6230,
	/* Array index: 0x1047 */ 0x90F0,0x6231,
	/* Array index: 0x1047 */ 0x90F4,0x6232,
	/* Array index: 0x1047 */ 0x90F6,0x622E,
	/* Array index: 0x1047 */ 0x910D,0x623A,
	/* Array index: 0x1047 */ 0x9110,0x623B,
	/* Array index: 0x1047 */ 0x912E,0x6249,
	/* Array index: 0x1047 */ 0x912F,0x624A,
	/* Array index: 0x1047 */ 0x9131,0x624B,
	/* Array index: 0x1047 */ 0x9134,0x624C,
	/* Array index: 0x1047 */ 0x9143,0x6253,
	/* Array index: 0x1047 */ 0x9147,0x6254,
	/* Array index: 0x1047 */ 0x9148,0x6255,
	/* Array index: 0x1047 */ 0x914F,0x6256,
	/* Array index: 0x1047 */ 0x9153,0x6257,
	/* Array index: 0x1047 */ 0x9161,0x625C,
	/* Array index: 0x1047 */ 0x9164,0x625D,
	/* Array index: 0x1047 */ 0x9167,0x625E,
	/* Array index: 0x1047 */ 0x916D,0x625F,
	/* Array index: 0x1047 */ 0x9174,0x6260,
	/* Array index: 0x1047 */ 0x9181,0x6264,
	/* Array index: 0x1047 */ 0x9183,0x6265,
	/* Array index: 0x1047 */ 0x9185,0x6266,
	/* Array index: 0x1047 */ 0x9186,0x6267,
	/* Array index: 0x1047 */ 0x918A,0x6268,
	/* Array index: 0x1047 */ 0x918E,0x6269,
	/* Array index: 0x1047 */ 0x9198,0x626E,
	/* Array index: 0x1047 */ 0x919E,0x626F,
	/* Array index: 0x1047 */ 0x91A1,0x6270,
	/* Array index: 0x1047 */ 0x91A6,0x6271,
	/* Array index: 0x1047 */ 0x91A8,0x6272,
	/* Array index: 0x1047 */ 0x91B6,0x627A,
	/* Array index: 0x1047 */ 0x91D3,0x6324,
	/* Array index: 0x1047 */ 0x91D4,0x6325,
	/* Array index: 0x1047 */ 0x91D7,0x6326,
	/* Array index: 0x1047 */ 0x91D9,0x6327,
	/* Array index: 0x1047 */ 0x91DA,0x6328,
	/* Array index: 0x1047 */ 0x91DE,0x6329,
	/* Array index: 0x1047 */ 0x91E4,0x632A,
	/* Array index: 0x1047 */ 0x91E5,0x632B,
	/* Array index: 0x1047 */ 0x91F7,0x6334,
	/* Array index: 0x1047 */ 0x91F9,0x6335,
	/* Array index: 0x1047 */ 0x91FB,0x6336,
	/* Array index: 0x1047 */ 0x91FD,0x6337,
	/* Array index: 0x1047 */ 0x9210,0x6341,
	/* Array index: 0x1047 */ 0x9212,0x6342,
	/* Array index: 0x1047 */ 0x9213,0x6343,
	/* Array index: 0x1047 */ 0x9216,0x6344,
	/* Array index: 0x1047 */ 0x9218,0x6345,
	/* Array index: 0x1047 */ 0x921C,0x6346,
	/* Array index: 0x1047 */ 0x921D,0x6347,
	/* Array index: 0x1047 */ 0x9233,0x6350,
	/* Array index: 0x1047 */ 0x924A,0x635D,
	/* Array index: 0x1047 */ 0x9258,0x6362,
	/* Array index: 0x1047 */ 0x9259,0x6363,
	/* Array index: 0x1047 */ 0x925C,0x6364,
	/* Array index: 0x1047 */ 0x925D,0x6365,
	/* Array index: 0x1047 */ 0x9260,0x6366,
	/* Array index: 0x1047 */ 0x9261,0x6367,
	/* Array index: 0x1047 */ 0x9292,0x637D,
	/* Array index: 0x1047 */ 0x9297,0x637E,
	/* Array index: 0x1047 */ 0x9299,0x6421,
	/* Array index: 0x1047 */ 0x929F,0x6422,
	/* Array index: 0x1047 */ 0x92A0,0x6423,
	/* Array index: 0x1047 */ 0x92AB,0x6428,
	/* Array index: 0x1047 */ 0x92AF,0x6429,
	/* Array index: 0x1047 */ 0x92B2,0x642A,
	/* Array index: 0x1047 */ 0x92B6,0x642B,
	/* Array index: 0x1047 */ 0x92D3,0x643F,
	/* Array index: 0x1047 */ 0x92EC,0x644D,
	/* Array index: 0x1047 */ 0x92EE,0x644E,
	/* Array index: 0x1047 */ 0x92F0,0x644F,
	/* Array index: 0x1047 */ 0x92F9,0x6450,
	/* Array index: 0x1047 */ 0x92FB,0x6451,
	/* Array index: 0x1047 */ 0x92FF,0x6452,
	/* Array index: 0x1047 */ 0x9300,0x6453,
	/* Array index: 0x1047 */ 0x9302,0x6454,
	/* Array index: 0x1047 */ 0x9308,0x6455,
	/* Array index: 0x1047 */ 0x930D,0x6456,
	/* Array index: 0x1047 */ 0x9311,0x6457,
	/* Array index: 0x1047 */ 0x9314,0x6458,
	/* Array index: 0x1047 */ 0x9315,0x6459,
	/* Array index: 0x1047 */ 0x9355,0x646E,
	/* Array index: 0x1047 */ 0x9357,0x646F,
	/* Array index: 0x1047 */ 0x9358,0x6470,
	/* Array index: 0x1047 */ 0x935A,0x6471,
	/* Array index: 0x1047 */ 0x935E,0x6472,
	/* Array index: 0x1047 */ 0x9364,0x6473,
	/* Array index: 0x1047 */ 0x9365,0x6474,
	/* Array index: 0x1047 */ 0x9367,0x6475,
	/* Array index: 0x1047 */ 0x9369,0x6476,
	/* Array index: 0x1047 */ 0x936A,0x6477,
	/* Array index: 0x1047 */ 0x937A,0x6521,
	/* Array index: 0x1047 */ 0x9388,0x6527,
	/* Array index: 0x1047 */ 0x938A,0x6528,
	/* Array index: 0x1047 */ 0x938B,0x6529,
	/* Array index: 0x1047 */ 0x938D,0x652A,
	/* Array index: 0x1047 */ 0x938F,0x652B,
	/* Array index: 0x1047 */ 0x9392,0x652C,
	/* Array index: 0x1047 */ 0x9395,0x652D,
	/* Array index: 0x1047 */ 0x9398,0x652E,
	/* Array index: 0x1047 */ 0x939B,0x652F,
	/* Array index: 0x1047 */ 0x939E,0x6530,
	/* Array index: 0x1047 */ 0x93A1,0x6531,
	/* Array index: 0x1047 */ 0x93A3,0x6532,
	/* Array index: 0x1047 */ 0x93A4,0x6533,
	/* Array index: 0x1047 */ 0x93A6,0x6534,
	/* Array index: 0x1047 */ 0x93A8,0x6535,
	/* Array index: 0x1047 */ 0x93A9,0x653B,
	/* Array index: 0x1047 */ 0x93AB,0x6536,
	/* Array index: 0x1047 */ 0x93BA,0x653A,
	/* Array index: 0x1047 */ 0x93C1,0x653C,
	/* Array index: 0x1047 */ 0x93D3,0x6546,
	/* Array index: 0x1047 */ 0x93D9,0x6547,
	/* Array index: 0x1047 */ 0x93DC,0x6548,
	/* Array index: 0x1047 */ 0x93DE,0x6549,
	/* Array index: 0x1047 */ 0x93DF,0x654A,
	/* Array index: 0x1047 */ 0x93E2,0x654B,
	/* Array index: 0x1047 */ 0x93E6,0x654C,
	/* Array index: 0x1047 */ 0x93E7,0x654D,
	/* Array index: 0x1047 */ 0x9401,0x6554,
	/* Array index: 0x1047 */ 0x9402,0x6555,
	/* Array index: 0x1047 */ 0x9404,0x6556,
	/* Array index: 0x1047 */ 0x9408,0x6557,
	/* Array index: 0x1047 */ 0x9409,0x6558,
	/* Array index: 0x1047 */ 0x941F,0x655F,
	/* Array index: 0x1047 */ 0x943B,0x6566,
	/* Array index: 0x1047 */ 0x943D,0x6568,
	/* Array index: 0x1047 */ 0x943F,0x6567,
	/* Array index: 0x1047 */ 0x9443,0x6569,
	/* Array index: 0x1047 */ 0x9445,0x656A,
	/* Array index: 0x1047 */ 0x9448,0x656B,
	/* Array index: 0x1047 */ 0x944A,0x656C,
	/* Array index: 0x1047 */ 0x944C,0x656D,
	/* Array index: 0x1047 */ 0x9455,0x656E,
	/* Array index: 0x1047 */ 0x9459,0x656F,
	/* Array index: 0x1047 */ 0x945C,0x6570,
	/* Array index: 0x1047 */ 0x945F,0x6571,
	/* Array index: 0x1047 */ 0x9461,0x6572,
	/* Array index: 0x1047 */ 0x9463,0x6573,
	/* Array index: 0x1047 */ 0x9468,0x6574,
	/* Array index: 0x1047 */ 0x9483,0x657C,
	/* Array index: 0x1047 */ 0x9484,0x657B,
	/* Array index: 0x1047 */ 0x9578,0x657D,
	/* Array index: 0x1047 */ 0x9579,0x657E,
	/* Array index: 0x1047 */ 0x957E,0x6621,
	/* Array index: 0x1047 */ 0x9584,0x6622,
	/* Array index: 0x1047 */ 0x9588,0x6623,
	/* Array index: 0x1047 */ 0x95A6,0x662B,
	/* Array index: 0x1047 */ 0x95A9,0x662C,
	/* Array index: 0x1047 */ 0x95AB,0x662D,
	/* Array index: 0x1047 */ 0x95AC,0x662E,
	/* Array index: 0x1047 */ 0x95B4,0x662F,
	/* Array index: 0x1047 */ 0x95B6,0x6630,
	/* Array index: 0x1047 */ 0x95BA,0x6631,
	/* Array index: 0x1047 */ 0x95BD,0x6632,
	/* Array index: 0x1047 */ 0x95BF,0x6633,
	/* Array index: 0x1047 */ 0x95C6,0x6634,
	/* Array index: 0x1047 */ 0x95C8,0x6635,
	/* Array index: 0x1047 */ 0x95C9,0x6636,
	/* Array index: 0x1047 */ 0x95CB,0x6637,
	/* Array index: 0x1047 */ 0x95E4,0x6642,
	/* Array index: 0x1047 */ 0x95E6,0x6643,
	/* Array index: 0x1047 */ 0x961D,0x6644,
	/* Array index: 0x1047 */ 0x961E,0x6645,
	/* Array index: 0x1047 */ 0x962C,0x664A,
	/* Array index: 0x1047 */ 0x9631,0x664B,
	/* Array index: 0x1047 */ 0x9633,0x664C,
	/* Array index: 0x1047 */ 0x9641,0x6653,
	/* Array index: 0x1047 */ 0x9652,0x6654,
	/* Array index: 0x1047 */ 0x9661,0x6659,
	/* Array index: 0x1047 */ 0x966E,0x665A,
	/* Array index: 0x1047 */ 0x9674,0x665B,
	/* Array index: 0x1047 */ 0x9689,0x6664,
	/* Array index: 0x1047 */ 0x9691,0x6665,
	/* Array index: 0x1047 */ 0x9696,0x6666,
	/* Array index: 0x1047 */ 0x969A,0x6667,
	/* Array index: 0x1047 */ 0x969D,0x6668,
	/* Array index: 0x1047 */ 0x969F,0x6669,
	/* Array index: 0x1047 */ 0x96A9,0x666D,
	/* Array index: 0x1047 */ 0x96AE,0x666E,
	/* Array index: 0x1047 */ 0x96AF,0x666F,
	/* Array index: 0x1047 */ 0x96B3,0x6670,
	/* Array index: 0x1047 */ 0x96BA,0x6671,
	/* Array index: 0x1047 */ 0x96CA,0x6672,
	/* Array index: 0x1047 */ 0x96D2,0x6673,
	/* Array index: 0x1047 */ 0x96D8,0x6675,
	/* Array index: 0x1047 */ 0x96DA,0x6676,
	/* Array index: 0x1047 */ 0x96E9,0x667A,
	/* Array index: 0x1047 */ 0x96EF,0x667B,
	/* Array index: 0x1047 */ 0x96F1,0x667C,
	/* Array index: 0x1047 */ 0x96FA,0x667D,
	/* Array index: 0x1047 */ 0x9702,0x667E,
	/* Array index: 0x1047 */ 0x9703,0x6721,
	/* Array index: 0x1047 */ 0x9705,0x6722,
	/* Array index: 0x1047 */ 0x9709,0x6723,
	/* Array index: 0x1047 */ 0x971A,0x6724,
	/* Array index: 0x1047 */ 0x971B,0x6725,
	/* Array index: 0x1047 */ 0x971D,0x6726,
	/* Array index: 0x1047 */ 0x9728,0x672A,
	/* Array index: 0x1047 */ 0x9731,0x672B,
	/* Array index: 0x1047 */ 0x9733,0x672C,
	/* Array index: 0x1047 */ 0x9741,0x672D,
	/* Array index: 0x1047 */ 0x9743,0x672E,
	/* Array index: 0x1047 */ 0x974A,0x672F,
	/* Array index: 0x1047 */ 0x974E,0x6730,
	/* Array index: 0x1047 */ 0x974F,0x6731,
	/* Array index: 0x1047 */ 0x9755,0x6732,
	/* Array index: 0x1047 */ 0x9763,0x6737,
	/* Array index: 0x1047 */ 0x9767,0x6738,
	/* Array index: 0x1047 */ 0x976A,0x6739,
	/* Array index: 0x1047 */ 0x976E,0x673A,
	/* Array index: 0x1047 */ 0x9773,0x673B,
	/* Array index: 0x1047 */ 0x977B,0x673F,
	/* Array index: 0x1047 */ 0x977D,0x6740,
	/* Array index: 0x1047 */ 0x977F,0x6741,
	/* Array index: 0x1047 */ 0x9780,0x6742,
	/* Array index: 0x1047 */ 0x9789,0x6743,
	/* Array index: 0x1047 */ 0x979E,0x6749,
	/* Array index: 0x1047 */ 0x979F,0x674A,
	/* Array index: 0x1047 */ 0x97A2,0x674B,
	/* Array index: 0x1047 */ 0x97AC,0x674C,
	/* Array index: 0x1047 */ 0x97AE,0x674D,
	/* Array index: 0x1047 */ 0x97B1,0x674E,
	/* Array index: 0x1047 */ 0x97B2,0x674F,
	/* Array index: 0x1047 */ 0x97D4,0x6763,
	/* Array index: 0x1047 */ 0x97E4,0x676C,
	/* Array index: 0x1047 */ 0x97EF,0x676D,
	/* Array index: 0x1047 */ 0x97F1,0x676E,
	/* Array index: 0x1047 */ 0x97F4,0x676F,
	/* Array index: 0x1047 */ 0x97F7,0x6770,
	/* Array index: 0x1047 */ 0x97F8,0x6771,
	/* Array index: 0x1047 */ 0x97FA,0x6772,
	/* Array index: 0x1047 */ 0x9807,0x6773,
	/* Array index: 0x1047 */ 0x980A,0x6774,
	/* Array index: 0x1047 */ 0x980D,0x6776,
	/* Array index: 0x1047 */ 0x980E,0x6777,
	/* Array index: 0x1047 */ 0x9814,0x6778,
	/* Array index: 0x1047 */ 0x9816,0x6779,
	/* Array index: 0x1047 */ 0x9819,0x6775,
	/* Array index: 0x1047 */ 0x981C,0x677A,
	/* Array index: 0x1047 */ 0x981E,0x677B,
	/* Array index: 0x1047 */ 0x9820,0x677C,
	/* Array index: 0x1047 */ 0x9823,0x677D,
	/* Array index: 0x1047 */ 0x9825,0x6828,
	/* Array index: 0x1047 */ 0x9826,0x677E,
	/* Array index: 0x1047 */ 0x982B,0x6821,
	/* Array index: 0x1047 */ 0x983E,0x6829,
	/* Array index: 0x1047 */ 0x9844,0x682A,
	/* Array index: 0x1047 */ 0x9847,0x682B,
	/* Array index: 0x1047 */ 0x984A,0x682C,
	/* Array index: 0x1047 */ 0x986A,0x6838,
	/* Array index: 0x1047 */ 0x986C,0x6839,
	/* Array index: 0x1047 */ 0x98AB,0x683A,
	/* Array index: 0x1047 */ 0x98AD,0x683B,
	/* Array index: 0x1047 */ 0x98AE,0x683C,
	/* Array index: 0x1047 */ 0x98B0,0x683D,
	/* Array index: 0x1047 */ 0x98B4,0x683E,
	/* Array index: 0x1047 */ 0x98BF,0x6843,
	/* Array index: 0x1047 */ 0x98C2,0x6844,
	/* Array index: 0x1047 */ 0x98C5,0x6845,
	/* Array index: 0x1047 */ 0x98C8,0x6846,
	/* Array index: 0x1047 */ 0x98CC,0x6847,
	/* Array index: 0x1047 */ 0x98E1,0x6848,
	/* Array index: 0x1047 */ 0x98EA,0x684D,
	/* Array index: 0x1047 */ 0x98F3,0x684E,
	/* Array index: 0x1047 */ 0x98F6,0x684F,
	/* Array index: 0x1047 */ 0x9902,0x6850,
	/* Array index: 0x1047 */ 0x9907,0x6851,
	/* Array index: 0x1047 */ 0x9908,0x6852,
	/* Array index: 0x1047 */ 0x9911,0x6853,
	/* Array index: 0x1047 */ 0x991F,0x685A,
	/* Array index: 0x1047 */ 0x9922,0x685B,
	/* Array index: 0x1047 */ 0x9926,0x685C,
	/* Array index: 0x1047 */ 0x9927,0x685D,
	/* Array index: 0x1047 */ 0x992B,0x685E,
	/* Array index: 0x1047 */ 0x9940,0x6868,
	/* Array index: 0x1047 */ 0x9941,0x6869,
	/* Array index: 0x1047 */ 0x994D,0x686D,
	/* Array index: 0x1047 */ 0x994E,0x686E,
	/* Array index: 0x1047 */ 0x9954,0x686F,
	/* Array index: 0x1047 */ 0x999B,0x6877,
	/* Array index: 0x1047 */ 0x999D,0x6878,
	/* Array index: 0x1047 */ 0x999F,0x6879,
	/* Array index: 0x1047 */ 0x99A6,0x687A,
	/* Array index: 0x1047 */ 0x99B5,0x687E,
	/* Array index: 0x1047 */ 0x99B9,0x6921,
	/* Array index: 0x1047 */ 0x99BA,0x6922,
	/* Array index: 0x1047 */ 0x99BD,0x6923,
	/* Array index: 0x1047 */ 0x99BF,0x6924,
	/* Array index: 0x1047 */ 0x99C3,0x6925,
	/* Array index: 0x1047 */ 0x99C9,0x6926,
	/* Array index: 0x1047 */ 0x99D3,0x6927,
	/* Array index: 0x1047 */ 0x99D4,0x6928,
	/* Array index: 0x1047 */ 0x99D9,0x6929,
	/* Array index: 0x1047 */ 0x99DA,0x692A,
	/* Array index: 0x1047 */ 0x99DC,0x692B,
	/* Array index: 0x1047 */ 0x99DE,0x692C,
	/* Array index: 0x1047 */ 0x99E7,0x692D,
	/* Array index: 0x1047 */ 0x99F0,0x6931,
	/* Array index: 0x1047 */ 0x99F4,0x6932,
	/* Array index: 0x1047 */ 0x99F5,0x6933,
	/* Array index: 0x1047 */ 0x99F9,0x6934,
	/* Array index: 0x1047 */ 0x99FD,0x6935,
	/* Array index: 0x1047 */ 0x99FE,0x6936,
	/* Array index: 0x1047 */ 0x9A0B,0x693A,
	/* Array index: 0x1047 */ 0x9A0C,0x693B,
	/* Array index: 0x1047 */ 0x9A10,0x693C,
	/* Array index: 0x1047 */ 0x9A11,0x693D,
	/* Array index: 0x1047 */ 0x9A16,0x693E,
	/* Array index: 0x1047 */ 0x9A1E,0x693F,
	/* Array index: 0x1047 */ 0x9A27,0x6944,
	/* Array index: 0x1047 */ 0x9A2D,0x6945,
	/* Array index: 0x1047 */ 0x9A2E,0x6946,
	/* Array index: 0x1047 */ 0x9A33,0x6947,
	/* Array index: 0x1047 */ 0x9A35,0x6948,
	/* Array index: 0x1047 */ 0x9A36,0x6949,
	/* Array index: 0x1047 */ 0x9A38,0x694A,
	/* Array index: 0x1047 */ 0x9A41,0x694C,
	/* Array index: 0x1047 */ 0x9A44,0x694D,
	/* Array index: 0x1047 */ 0x9A47,0x694B,
	/* Array index: 0x1047 */ 0x9A51,0x6952,
	/* Array index: 0x1047 */ 0x9A54,0x6953,
	/* Array index: 0x1047 */ 0x9A56,0x6954,
	/* Array index: 0x1047 */ 0x9A5D,0x6955,
	/* Array index: 0x1047 */ 0x9AAA,0x6956,
	/* Array index: 0x1047 */ 0x9AAC,0x6957,
	/* Array index: 0x1047 */ 0x9AAE,0x6958,
	/* Array index: 0x1047 */ 0x9AAF,0x6959,
	/* Array index: 0x1047 */ 0x9AB9,0x695E,
	/* Array index: 0x1047 */ 0x9ABB,0x695F,
	/* Array index: 0x1047 */ 0x9ABE,0x6960,
	/* Array index: 0x1047 */ 0x9ABF,0x6961,
	/* Array index: 0x1047 */ 0x9AC1,0x6962,
	/* Array index: 0x1047 */ 0x9AC3,0x6963,
	/* Array index: 0x1047 */ 0x9AC6,0x6964,
	/* Array index: 0x1047 */ 0x9AC8,0x6965,
	/* Array index: 0x1047 */ 0x9ACE,0x6966,
	/* Array index: 0x1047 */ 0x9AD0,0x6967,
	/* Array index: 0x1047 */ 0x9AD2,0x6968,
	/* Array index: 0x1047 */ 0x9ADB,0x696C,
	/* Array index: 0x1047 */ 0x9ADC,0x696D,
	/* Array index: 0x1047 */ 0x9AE0,0x696E,
	/* Array index: 0x1047 */ 0x9AE4,0x696F,
	/* Array index: 0x1047 */ 0x9AE5,0x6970,
	/* Array index: 0x1047 */ 0x9AE7,0x6971,
	/* Array index: 0x1047 */ 0x9AE9,0x6972,
	/* Array index: 0x1047 */ 0x9AEC,0x6973,
	/* Array index: 0x1047 */ 0x9AF2,0x6974,
	/* Array index: 0x1047 */ 0x9AF3,0x6975,
	/* Array index: 0x1047 */ 0x9AF5,0x6976,
	/* Array index: 0x1047 */ 0x9AF9,0x6977,
	/* Array index: 0x1047 */ 0x9AFA,0x6978,
	/* Array index: 0x1047 */ 0x9B16,0x6A2B,
	/* Array index: 0x1047 */ 0x9B19,0x6A2C,
	/* Array index: 0x1047 */ 0x9B1B,0x6A2D,
	/* Array index: 0x1047 */ 0x9B1C,0x6A2E,
	/* Array index: 0x1047 */ 0x9B20,0x6A2F,
	/* Array index: 0x1047 */ 0x9B26,0x6A30,
	/* Array index: 0x1047 */ 0x9B2B,0x6A31,
	/* Array index: 0x1047 */ 0x9B2D,0x6A32,
	/* Array index: 0x1047 */ 0x9B3D,0x6A39,
	/* Array index: 0x1047 */ 0x9B48,0x6A3A,
	/* Array index: 0x1047 */ 0x9B4B,0x6A3B,
	/* Array index: 0x1047 */ 0x9B4C,0x6A3C,
	/* Array index: 0x1047 */ 0x9B5B,0x6A40,
	/* Array index: 0x1047 */ 0x9B5E,0x6A41,
	/* Array index: 0x1047 */ 0x9B61,0x6A42,
	/* Array index: 0x1047 */ 0x9B63,0x6A43,
	/* Array index: 0x1047 */ 0x9B65,0x6A44,
	/* Array index: 0x1047 */ 0x9B66,0x6A45,
	/* Array index: 0x1047 */ 0x9B73,0x6A4C,
	/* Array index: 0x1047 */ 0x9B7F,0x6A51,
	/* Array index: 0x1047 */ 0x9B80,0x6A52,
	/* Array index: 0x1047 */ 0x9B94,0x6A5D,
	/* Array index: 0x1047 */ 0x9B9A,0x6A5E,
	/* Array index: 0x1047 */ 0x9B9D,0x6A5F,
	/* Array index: 0x1047 */ 0x9B9E,0x6A60,
	/* Array index: 0x1047 */ 0x9BA6,0x6A61,
	/* Array index: 0x1047 */ 0x9BA7,0x6A62,
	/* Array index: 0x1047 */ 0x9BA9,0x6A63,
	/* Array index: 0x1047 */ 0x9BAC,0x6A64,
	/* Array index: 0x1047 */ 0x9BB7,0x6A68,
	/* Array index: 0x1047 */ 0x9BB8,0x6A69,
	/* Array index: 0x1047 */ 0x9BC7,0x6A6F,
	/* Array index: 0x1047 */ 0x9BC8,0x6A70,
	/* Array index: 0x1047 */ 0x9BCE,0x6A71,
	/* Array index: 0x1047 */ 0x9BD0,0x6A72,
	/* Array index: 0x1047 */ 0x9BD7,0x6A73,
	/* Array index: 0x1047 */ 0x9BD8,0x6A74,
	/* Array index: 0x1047 */ 0x9BDD,0x6A75,
	/* Array index: 0x1047 */ 0x9BDF,0x6A76,
	/* Array index: 0x1047 */ 0x9BE5,0x6A77,
	/* Array index: 0x1047 */ 0x9BE7,0x6A78,
	/* Array index: 0x1047 */ 0x9BEA,0x6A79,
	/* Array index: 0x1047 */ 0x9BEB,0x6A7A,
	/* Array index: 0x1047 */ 0x9BEF,0x6A7B,
	/* Array index: 0x1047 */ 0x9BF3,0x6A7C,
	/* Array index: 0x1047 */ 0x9BFD,0x6B23,
	/* Array index: 0x1047 */ 0x9BFF,0x6B24,
	/* Array index: 0x1047 */ 0x9C00,0x6B25,
	/* Array index: 0x1047 */ 0x9C02,0x6B26,
	/* Array index: 0x1047 */ 0x9C0B,0x6B27,
	/* Array index: 0x1047 */ 0x9C0F,0x6B28,
	/* Array index: 0x1047 */ 0x9C11,0x6B29,
	/* Array index: 0x1047 */ 0x9C31,0x6B37,
	/* Array index: 0x1047 */ 0x9C3D,0x6B3B,
	/* Array index: 0x1047 */ 0x9C49,0x6B40,
	/* Array index: 0x1047 */ 0x9C4A,0x6B41,
	/* Array index: 0x1047 */ 0x9C63,0x6B4D,
	/* Array index: 0x1047 */ 0x9C6E,0x6B53,
	/* Array index: 0x1047 */ 0x9C70,0x6B54,
	/* Array index: 0x1047 */ 0x9C72,0x6B55,
	/* Array index: 0x1047 */ 0x9C75,0x6B56,
	/* Array index: 0x1047 */ 0x9C77,0x6B57,
	/* Array index: 0x1047 */ 0x9C7B,0x6B58,
	/* Array index: 0x1047 */ 0x9CE6,0x6B59,
	/* Array index: 0x1047 */ 0x9CF2,0x6B5A,
	/* Array index: 0x1047 */ 0x9CF7,0x6B5B,
	/* Array index: 0x1047 */ 0x9CF9,0x6B5C,
	/* Array index: 0x1047 */ 0x9D02,0x6B5E,
	/* Array index: 0x1047 */ 0x9D0B,0x6B5D,
	/* Array index: 0x1047 */ 0x9D11,0x6B5F,
	/* Array index: 0x1047 */ 0x9D17,0x6B60,
	/* Array index: 0x1047 */ 0x9D18,0x6B61,
	/* Array index: 0x1047 */ 0x9D3A,0x6B6A,
	/* Array index: 0x1047 */ 0x9D3C,0x6B6B,
	/* Array index: 0x1047 */ 0x9D3D,0x6B6D,
	/* Array index: 0x1047 */ 0x9D42,0x6B6E,
	/* Array index: 0x1047 */ 0x9D43,0x6B6F,
	/* Array index: 0x1047 */ 0x9D45,0x6B6C,
	/* Array index: 0x1047 */ 0x9D47,0x6B70,
	/* Array index: 0x1047 */ 0x9D4A,0x6B71,
	/* Array index: 0x1047 */ 0x9D53,0x6B72,
	/* Array index: 0x1047 */ 0x9D54,0x6B73,
	/* Array index: 0x1047 */ 0x9D5F,0x6B74,
	/* Array index: 0x1047 */ 0x9D62,0x6B76,
	/* Array index: 0x1047 */ 0x9D63,0x6B75,
	/* Array index: 0x1047 */ 0x9D65,0x6B77,
	/* Array index: 0x1047 */ 0x9D70,0x6B7B,
	/* Array index: 0x1047 */ 0x9D76,0x6B7C,
	/* Array index: 0x1047 */ 0x9D77,0x6B7D,
	/* Array index: 0x1047 */ 0x9D7B,0x6B7E,
	/* Array index: 0x1047 */ 0x9D7C,0x6C21,
	/* Array index: 0x1047 */ 0x9D7E,0x6C22,
	/* Array index: 0x1047 */ 0x9D83,0x6C23,
	/* Array index: 0x1047 */ 0x9D84,0x6C24,
	/* Array index: 0x1047 */ 0x9D86,0x6C25,
	/* Array index: 0x1047 */ 0x9D8A,0x6C26,
	/* Array index: 0x1047 */ 0x9D8D,0x6C27,
	/* Array index: 0x1047 */ 0x9D8E,0x6C28,
	/* Array index: 0x1047 */ 0x9DA1,0x6C2F,
	/* Array index: 0x1047 */ 0x9DAA,0x6C30,
	/* Array index: 0x1047 */ 0x9DAC,0x6C31,
	/* Array index: 0x1047 */ 0x9DAE,0x6C32,
	/* Array index: 0x1047 */ 0x9DB1,0x6C33,
	/* Array index: 0x1047 */ 0x9DB5,0x6C34,
	/* Array index: 0x1047 */ 0x9DB9,0x6C35,
	/* Array index: 0x1047 */ 0x9DBC,0x6C36,
	/* Array index: 0x1047 */ 0x9DBF,0x6C37,
	/* Array index: 0x1047 */ 0x9DC3,0x6C38,
	/* Array index: 0x1047 */ 0x9DC7,0x6C39,
	/* Array index: 0x1047 */ 0x9DC9,0x6C3A,
	/* Array index: 0x1047 */ 0x9DCA,0x6C3B,
	/* Array index: 0x1047 */ 0x9DDA,0x6C40,
	/* Array index: 0x1047 */ 0x9DE5,0x6C44,
	/* Array index: 0x1047 */ 0x9DE7,0x6C45,
	/* Array index: 0x1047 */ 0x9DE9,0x6C46,
	/* Array index: 0x1047 */ 0x9DEB,0x6C47,
	/* Array index: 0x1047 */ 0x9DEE,0x6C48,
	/* Array index: 0x1047 */ 0x9DF0,0x6C49,
	/* Array index: 0x1047 */ 0x9DF3,0x6C4A,
	/* Array index: 0x1047 */ 0x9DF4,0x6C4B,
	/* Array index: 0x1047 */ 0x9DFE,0x6C4C,
	/* Array index: 0x1047 */ 0x9E02,0x6C4E,
	/* Array index: 0x1047 */ 0x9E07,0x6C4F,
	/* Array index: 0x1047 */ 0x9E0A,0x6C4D,
	/* Array index: 0x1047 */ 0x9E19,0x6C56,
	/* Array index: 0x1047 */ 0x9E1C,0x6C57,
	/* Array index: 0x1047 */ 0x9E1D,0x6C58,
	/* Array index: 0x1047 */ 0x9E8E,0x6C62,
	/* Array index: 0x1047 */ 0x9E8F,0x6C63,
	/* Array index: 0x1047 */ 0x9E96,0x6C64,
	/* Array index: 0x1047 */ 0x9E98,0x6C65,
	/* Array index: 0x1047 */ 0x9E9B,0x6C66,
	/* Array index: 0x1047 */ 0x9E9E,0x6C67,
	/* Array index: 0x1047 */ 0x9EA4,0x6C68,
	/* Array index: 0x1047 */ 0x9EA8,0x6C69,
	/* Array index: 0x1047 */ 0x9EC6,0x6C71,
	/* Array index: 0x1047 */ 0x9EC8,0x6C72,
	/* Array index: 0x1047 */ 0x9ECB,0x6C73,
	/* Array index: 0x1047 */ 0x9ED5,0x6C74,
	/* Array index: 0x1047 */ 0x9EDF,0x6C75,
	/* Array index: 0x1047 */ 0x9EE4,0x6C76,
	/* Array index: 0x1047 */ 0x9EE7,0x6C77,
	/* Array index: 0x1047 */ 0x9EF5,0x6C7E,
	/* Array index: 0x1047 */ 0x9EF8,0x6D21,
	/* Array index: 0x1047 */ 0x9EFF,0x6D22,
	/* Array index: 0x1047 */ 0x9F02,0x6D23,
	/* Array index: 0x1047 */ 0x9F03,0x6D24,
	/* Array index: 0x1047 */ 0x9F09,0x6D25,
	/* Array index: 0x1047 */ 0x9F1F,0x6D30,
	/* Array index: 0x1047 */ 0x9F22,0x6D31,
	/* Array index: 0x1047 */ 0x9F26,0x6D32,
	/* Array index: 0x1047 */ 0x9F2A,0x6D33,
	/* Array index: 0x1047 */ 0x9F2B,0x6D34,
	/* Array index: 0x1047 */ 0x9F2F,0x6D35,
	/* Array index: 0x1047 */ 0x9F31,0x6D36,
	/* Array index: 0x1047 */ 0x9F32,0x6D37,
	/* Array index: 0x1047 */ 0x9F34,0x6D38,
	/* Array index: 0x1047 */ 0x9F37,0x6D39,
	/* Array index: 0x1047 */ 0x9F68,0x6D4D,
	/* Array index: 0x1047 */ 0x9F69,0x6D4E,
	/* Array index: 0x1047 */ 0x9F7A,0x6D56,
	/* Array index: 0x1047 */ 0x9F7D,0x6D57,
	/* Array index: 0x1047 */ 0x9F9E,0x6D5F,
};

#endif /* ICONV_FROM_UCS_CCS_JIS_X0212_1990 && defined (TABLE_USE_SIZE_OPTIMIZATION) */

/*
 * jis_x0212_1990 CCS description table.
 * ======================================================================
 */
const iconv_ccs_t
_iconv_ccs_jis_x0212_1990 =
{
	TABLE_VERSION_1, /* Table version */
	ICONV_CCS_JIS_X0212_1990, /* CCS name */
	TABLE_16BIT, /* Table bits */
#if defined (ICONV_FROM_UCS_CCS_JIS_X0212_1990) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SIZE_OPTIMIZED,
	(__uint16_t *)&from_ucs_size_jis_x0212_1990, /* UCS -> jis_x0212_1990 table size-optimized table */
#elif defined (ICONV_FROM_UCS_CCS_JIS_X0212_1990) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)&from_ucs_speed_jis_x0212_1990, /* UCS -> jis_x0212_1990 table speed-optimized table */
#else
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)NULL,
#endif
#if defined (ICONV_TO_UCS_CCS_JIS_X0212_1990) \
 && (defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SIZE_OPTIMIZED,
	(__uint16_t *)&to_ucs_size_jis_x0212_1990 /* jis_x0212_1990 -> UCS table speed-optimized table */
#elif defined (ICONV_TO_UCS_CCS_JIS_X0212_1990) \
 && !(defined (TABLE_USE_SIZE_OPTIMIZATION))
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)&to_ucs_speed_jis_x0212_1990 /* jis_x0212_1990 -> UCS table speed-optimized table */
#else
	TABLE_SPEED_OPTIMIZED,
	(__uint16_t *)NULL,
#endif
};

#endif /* ICONV_TO_UCS_CCS_JIS_X0212_1990) || ... */

